package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorBehaviorAddReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorBehaviorGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorBehaviorQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorBehaviorDataVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorBehaviorVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorBehaviorService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/crm/behavior")
@Api(tags = "医生行为管理")
public class DoctorBehaviorController {
    
    @Autowired
    private DoctorBehaviorService doctorBehaviorService;

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DoctorBehaviorVO.class)
    public Result<DoctorBehaviorVO> add(@RequestBody DoctorBehaviorAddReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorBehaviorAddDTO addDTO = new DoctorBehaviorAddDTO();
        addDTO.setOperatedType(params.getOperatedType());
        addDTO.setRelOneType(params.getRelOneType());
        addDTO.setRelOneId(params.getRelOneId());
        addDTO.setRelTwoType(params.getRelTwoType());
        addDTO.setRelTwoId(params.getRelTwoId());
        addDTO.setOperatedContent(params.getOperatedContent());
        addDTO.setDoctorId(user.getId());
        addDTO.setIp(IpUtils.getIpAddr());
        DoctorBehaviorVO data = doctorBehaviorService.add(addDTO);
        return new Result<DoctorBehaviorVO>().ok(data);
    }

    @PostMapping("/query")
    @ApiOperation(value = "分页列表", response = DoctorBehaviorVO.class)
    public Result<PageData<DoctorBehaviorVO>> query(@RequestBody DoctorBehaviorQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorBehaviorQueryDTO queryDTO = new DoctorBehaviorQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setOperatedType(params.getOperatedType());
        queryDTO.setOperatedTypes(params.getOperatedTypes());
        queryDTO.setRelOneType(params.getRelOneType());
        queryDTO.setRelOneTypes(params.getOperatedTypes());
        queryDTO.setRelOneId(params.getRelOneId());
        queryDTO.setRelTwoType(params.getRelTwoType());
        queryDTO.setRelTwoTypes(params.getRelTwoTypes());
        queryDTO.setRelTwoId(params.getRelTwoId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        PageData<DoctorBehaviorVO> data = doctorBehaviorService.query(queryDTO);
        return new Result<PageData<DoctorBehaviorVO>>().ok(data);
    }

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DoctorBehaviorVO.class)
    public Result<List<DoctorBehaviorVO>> list(@RequestBody DoctorBehaviorQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorBehaviorQueryDTO queryDTO = new DoctorBehaviorQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setOperatedType(params.getOperatedType());
        queryDTO.setOperatedTypes(params.getOperatedTypes());
        queryDTO.setRelOneType(params.getRelOneType());
        queryDTO.setRelOneTypes(params.getOperatedTypes());
        queryDTO.setRelOneId(params.getRelOneId());
        queryDTO.setRelTwoType(params.getRelTwoType());
        queryDTO.setRelTwoTypes(params.getRelTwoTypes());
        queryDTO.setRelTwoId(params.getRelTwoId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setDoctorId(user.getId());
        List<DoctorBehaviorVO> data = doctorBehaviorService.list(queryDTO);
        return new Result<List<DoctorBehaviorVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorBehaviorVO.class)
    public Result<DoctorBehaviorVO> detail(@RequestBody DoctorBehaviorGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorBehaviorGetDTO getDTO = new DoctorBehaviorGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        DoctorBehaviorVO vo = doctorBehaviorService.detail(getDTO);
        return new Result<DoctorBehaviorVO>().ok(vo);
    }


    // 行为统计
    
    @PostMapping("/data/list")
    @ApiOperation(value = "医生行为数据列表", response = DoctorBehaviorDataVO.class)
    public Result<PageData<DoctorBehaviorDataVO>> doctorBehaviorDataList(@RequestBody DoctorBehaviorQueryReq params) {
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorBehaviorQueryDTO queryDTO = new DoctorBehaviorQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setOperatedType(params.getOperatedType());
        queryDTO.setOperatedTypes(params.getOperatedTypes());
        queryDTO.setRelOneType(params.getRelOneType());
        queryDTO.setRelOneTypes(params.getRelOneTypes());
        queryDTO.setRelOneId(params.getRelOneId());
        queryDTO.setRelTwoType(params.getRelTwoType());
        queryDTO.setRelTwoTypes(params.getRelTwoTypes());
        queryDTO.setRelTwoId(params.getRelTwoId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setOrder(params.getOrder());
        queryDTO.setOrderType(params.getOrderType());
        PageData<DoctorBehaviorDataVO> data = doctorBehaviorService.doctorBehaviorDataList(queryDTO);
        return new Result<PageData<DoctorBehaviorDataVO>>().ok(data);
    }

    @PostMapping("/manage/query")
    @ApiOperation(value = "管理端分页列表（可查看其他医生信息）", response = DoctorBehaviorVO.class)
    public Result<PageData<DoctorBehaviorVO>> doctorQuery(@RequestBody DoctorBehaviorQueryDTO params) {
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DoctorBehaviorVO> data = doctorBehaviorService.query(params);
        return new Result<PageData<DoctorBehaviorVO>>().ok(data);
    }

}
