package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkTargetCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateFocusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateStatusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTargetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTypeDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkHisGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkHisQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkTargetCountQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkUpdateTargetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientStatusMarkUpdateTypeReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkHisVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkTargetCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.ManageDoctorPatientStatusMarkHisVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.ManageDoctorPatientStatusMarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkHisService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/crm/v2/mark")
@Api(tags = "医生个案管理")
public class DoctorPatientStatusMarkController {
    
    @Autowired
    private DoctorPatientStatusMarkService doctorPatientStatusMarkService;
    @Autowired
    private DoctorPatientStatusMarkHisService doctorPatientStatusMarkHisService;

    @PostMapping("/manageQuery")
    @ApiOperation(value = "管理分页列表", response = ManageDoctorPatientStatusMarkVO.class)
    public Result<PageData<ManageDoctorPatientStatusMarkVO>> manageQuery(@RequestBody DoctorPatientStatusMarkQueryReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorPatientStatusMarkQueryDTO queryDTO = new DoctorPatientStatusMarkQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setIds(params.getIds());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientName(params.getPatientName());
        queryDTO.setStatus(StrUtil.isBlank(params.getStatus()) ? DoctorStatusMarkConstant.Status.Normal.getCode() : params.getStatus());
        queryDTO.setTargetId(params.getTargetId());
        queryDTO.setTargetIds(params.getTargetIds());
        queryDTO.setFocus(params.getFocus());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setType(params.getType());
        queryDTO.setDeptId(params.getDeptId());
        queryDTO.setDoctorId(params.getDoctorId());
        queryDTO.setTargetExeStatus(params.getTargetExeStatus());
        PageData<ManageDoctorPatientStatusMarkVO> vo = doctorPatientStatusMarkService.manageQuery(queryDTO);
        return new Result<PageData<ManageDoctorPatientStatusMarkVO>>().ok(vo);
    }

    @PostMapping("/manageDetail")
    @ApiOperation(value = "管理详情", response = ManageDoctorPatientStatusMarkVO.class)
    public Result<ManageDoctorPatientStatusMarkVO> manageDetail(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorPatientStatusMarkGetDTO queryDTO = new DoctorPatientStatusMarkGetDTO();
        queryDTO.setId(params.getId());
        ManageDoctorPatientStatusMarkVO vo = doctorPatientStatusMarkService.manageDetail(queryDTO);
        return new Result<ManageDoctorPatientStatusMarkVO>().ok(vo);
    }

    @PostMapping("/query")
    @ApiOperation(value = "分页列表", response = DoctorPatientStatusMarkVO.class)
    public Result<PageData<DoctorPatientStatusMarkVO>> query(@RequestBody DoctorPatientStatusMarkQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkQueryDTO queryDTO = new DoctorPatientStatusMarkQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setIds(params.getIds());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientName(params.getPatientName());
        queryDTO.setStatus(params.getStatus());
        queryDTO.setTargetId(params.getTargetId());
        queryDTO.setTargetIds(params.getTargetIds());
        queryDTO.setFocus(params.getFocus());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        queryDTO.setType(params.getType());
        PageData<DoctorPatientStatusMarkVO> vo = doctorPatientStatusMarkService.query(queryDTO);
        return new Result<PageData<DoctorPatientStatusMarkVO>>().ok(vo);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorPatientStatusMarkVO.class)
    public Result<DoctorPatientStatusMarkVO> detail(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkGetDTO getDTO = new DoctorPatientStatusMarkGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        DoctorPatientStatusMarkVO vo = doctorPatientStatusMarkService.detail(getDTO);
        return new Result<DoctorPatientStatusMarkVO>().ok(vo);
    }

    @PostMapping("/updateTarget")
    @ApiOperation(value = "更新目标", response = Void.class)
    public Result<Void> updateTarget(@RequestBody DoctorPatientStatusMarkUpdateTargetReq params) {
        ValidatorUtils.validateEntity(params);
        if (StrUtil.isBlank(params.getTargetId())) {
            throw new BusinessException("目标[targetId]不能为空");
        }
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkUpdateTargetDTO updateTargetDTO = new DoctorPatientStatusMarkUpdateTargetDTO();
        updateTargetDTO.setId(params.getId());
        updateTargetDTO.setTargetId(params.getTargetId());
        updateTargetDTO.setDoctorId(user.getId());
        updateTargetDTO.setOpUserId(user.getId());
        doctorPatientStatusMarkService.updateTarget(updateTargetDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/abandon")
    @ApiOperation(value = "放弃", response = Void.class)
    public Result<Void> abandon(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkUpdateStatusDTO updateStatusDTO = new DoctorPatientStatusMarkUpdateStatusDTO();
        updateStatusDTO.setId(params.getId());
        updateStatusDTO.setStatus(DoctorStatusMarkConstant.Status.Abandon.getCode());
        updateStatusDTO.setDoctorId(user.getId());
        updateStatusDTO.setOpUserId(user.getId());
        doctorPatientStatusMarkService.updateStatus(updateStatusDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/normal")
    @ApiOperation(value = "由放弃重新加入个案管理", response = Void.class)
    public Result<Void> normal(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkUpdateStatusDTO updateStatusDTO = new DoctorPatientStatusMarkUpdateStatusDTO();
        updateStatusDTO.setId(params.getId());
        updateStatusDTO.setStatus(DoctorStatusMarkConstant.Status.Normal.getCode());
        updateStatusDTO.setDoctorId(user.getId());
        updateStatusDTO.setOpUserId(user.getId());
        doctorPatientStatusMarkService.updateStatus(updateStatusDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/focus")
    @ApiOperation(value = "重点关注/取消关注", response = Void.class)
    public Result<Void> focus(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkUpdateFocusDTO updateFocusDTO = new DoctorPatientStatusMarkUpdateFocusDTO();
        updateFocusDTO.setId(params.getId());
        updateFocusDTO.setDoctorId(user.getId());
        updateFocusDTO.setOpUserId(user.getId());
        doctorPatientStatusMarkService.updateFocus(updateFocusDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/updateType")
    @ApiOperation(value = "更新个案类型", response = Void.class)
    public Result<Void> updateType(@RequestBody DoctorPatientStatusMarkUpdateTypeReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkUpdateTypeDTO updateTypeDTO = new DoctorPatientStatusMarkUpdateTypeDTO();
        updateTypeDTO.setId(params.getId());
        updateTypeDTO.setDoctorId(user.getId());
        updateTypeDTO.setOpUserId(user.getId());
        updateTypeDTO.setType(params.getType());
        doctorPatientStatusMarkService.updateType(updateTypeDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/remove")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> remove(@RequestBody DoctorPatientStatusMarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkGetDTO getDTO = new DoctorPatientStatusMarkGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        doctorPatientStatusMarkService.remove(getDTO);
        return new Result<Void>().success();
    }


    @PostMapping("/hisManagePage")
    @ApiOperation(value = "管理历史分页列表", response = ManageDoctorPatientStatusMarkHisVO.class)
    public Result<PageData<ManageDoctorPatientStatusMarkHisVO>> hisManagePage(@RequestBody DoctorPatientStatusMarkHisQueryReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorPatientStatusMarkHisQueryDTO queryDTO = new DoctorPatientStatusMarkHisQueryDTO();
        queryDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setIsDel(params.getIsDel());
        queryDTO.setType(params.getType());
        queryDTO.setOpType(OpTypeEnum.Create.getCode());
        queryDTO.setStatus(StrUtil.isBlank(params.getStatus()) ? DoctorStatusMarkConstant.Status.Normal.getCode() : params.getStatus());
        queryDTO.setTargetExeStatus(params.getTargetExeStatus());
        queryDTO.setDoctorId(params.getDoctorId());
        PageData<ManageDoctorPatientStatusMarkHisVO> vo = doctorPatientStatusMarkHisService.managePage(queryDTO);
        return new Result<PageData<ManageDoctorPatientStatusMarkHisVO>>().ok(vo);
    }

    @PostMapping("/hisManageDetail")
    @ApiOperation(value = "管理历史详情", response = ManageDoctorPatientStatusMarkHisVO.class)
    public Result<ManageDoctorPatientStatusMarkHisVO> hisManageDetail(@RequestBody DoctorPatientStatusMarkHisGetReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorPatientStatusMarkHisGetDTO queryDTO = new DoctorPatientStatusMarkHisGetDTO();
        queryDTO.setId(params.getId());
        ManageDoctorPatientStatusMarkHisVO vo = doctorPatientStatusMarkHisService.manageDetail(queryDTO);
        return new Result<ManageDoctorPatientStatusMarkHisVO>().ok(vo);
    }

    @PostMapping("/hisPage")
    @ApiOperation(value = "历史分页列表", response = DoctorPatientStatusMarkHisVO.class)
    public Result<PageData<DoctorPatientStatusMarkHisVO>> hisPage(@RequestBody DoctorPatientStatusMarkHisQueryReq params) {
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkHisQueryDTO queryDTO = new DoctorPatientStatusMarkHisQueryDTO();
        queryDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setIsDel(params.getIsDel());
        queryDTO.setDoctorId(user.getId());
        queryDTO.setType(params.getType());
        queryDTO.setOpType(OpTypeEnum.Create.getCode());
        PageData<DoctorPatientStatusMarkHisVO> vo = doctorPatientStatusMarkHisService.page(queryDTO);
        return new Result<PageData<DoctorPatientStatusMarkHisVO>>().ok(vo);
    }

    @PostMapping("/hisList")
    @ApiOperation(value = "历史列表", response = DoctorPatientStatusMarkHisVO.class)
    public Result<List<DoctorPatientStatusMarkHisVO>> hisQuery(@RequestBody DoctorPatientStatusMarkHisQueryReq params) {
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkHisQueryDTO queryDTO = new DoctorPatientStatusMarkHisQueryDTO();
        queryDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setIsDel(params.getIsDel());
        queryDTO.setDoctorId(user.getId());
        queryDTO.setType(params.getType());
        queryDTO.setOpType(OpTypeEnum.Create.getCode());
        List<DoctorPatientStatusMarkHisVO> vo = doctorPatientStatusMarkHisService.list(queryDTO);
        return new Result<List<DoctorPatientStatusMarkHisVO>>().ok(vo);
    }

    @PostMapping("/targetCount")
    @ApiOperation(value = "目标数量统计", response = Long.class)
    public Result<List<DoctorPatientStatusMarkTargetCountVO>> targetCount(@RequestBody DoctorPatientStatusMarkTargetCountQueryReq params) {
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        DoctorPatientStatusMarkTargetCountQueryDTO queryDTO = new DoctorPatientStatusMarkTargetCountQueryDTO();
        queryDTO.setStatus(params.getStatus());
        queryDTO.setStatuses(params.getStatuses());
        queryDTO.setTargetId(params.getTargetId());
        queryDTO.setTargetIds(params.getTargetIds());
        queryDTO.setDoctorId(user.getId());
        List<DoctorPatientStatusMarkTargetCountVO> data = doctorPatientStatusMarkService.getTargetCount(queryDTO);
        return new Result<List<DoctorPatientStatusMarkTargetCountVO>>().ok(data);
    }

}
