package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientTrackAddReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientTrackGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientTrackQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientTrackUpdateReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientTrackVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientTrackService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/crm/track")
@Api(tags = "医生个案备注管理")
public class DoctorPatientTrackController {
    
    @Autowired
    private DoctorPatientTrackService doctorPatientTrackService;

    @PostMapping("/query")
    @ApiOperation(value = "分页列表", response = DoctorPatientTrackVO.class)
    public Result<PageData<DoctorPatientTrackVO>> query(@RequestBody DoctorPatientTrackQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackQueryDTO queryDTO = new DoctorPatientTrackQueryDTO();
        queryDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        PageData<DoctorPatientTrackVO> data = doctorPatientTrackService.query(queryDTO);
        return new Result<PageData<DoctorPatientTrackVO>>().ok(data);
    }

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DoctorPatientTrackVO.class)
    public Result<List<DoctorPatientTrackVO>> list(@RequestBody DoctorPatientTrackQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackQueryDTO queryDTO = new DoctorPatientTrackQueryDTO();
        queryDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        List<DoctorPatientTrackVO> data = doctorPatientTrackService.list(queryDTO);
        return new Result<List<DoctorPatientTrackVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorPatientTrackVO.class)
    public Result<DoctorPatientTrackVO> detail(@RequestBody DoctorPatientTrackGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackGetDTO getDTO = new DoctorPatientTrackGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        DoctorPatientTrackVO vo = doctorPatientTrackService.detail(getDTO);
        return new Result<DoctorPatientTrackVO>().ok(vo);
    }

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DoctorPatientTrackVO.class)
    public Result<DoctorPatientTrackVO> add(@RequestBody DoctorPatientTrackAddReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackAddDTO addDTO = new DoctorPatientTrackAddDTO();
        addDTO.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        addDTO.setContent(params.getContent());
        addDTO.setDoctorId(user.getId());
        DoctorPatientTrackVO data = doctorPatientTrackService.add(addDTO);
        return new Result<DoctorPatientTrackVO>().ok(data);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> updateTarget(@RequestBody DoctorPatientTrackUpdateReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackUpdateDTO updateDTO = new DoctorPatientTrackUpdateDTO();
        updateDTO.setId(params.getId());
        updateDTO.setContent(params.getContent());
        updateDTO.setDoctorId(user.getId());
        doctorPatientTrackService.update(updateDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/remove")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> remove(@RequestBody DoctorPatientTrackGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientTrackGetDTO getDTO = new DoctorPatientTrackGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        doctorPatientTrackService.remove(getDTO);
        return new Result<Void>().success();
    }

}
