package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkCleanDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientRemarkAddReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientRemarkGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientRemarkQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorPatientRemarkUpdateReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientRemarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientRemarkService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/crm/remark")
@Api(tags = "医生备注管理")
public class DoctorRemarkController {
    
    @Autowired
    private DoctorPatientRemarkService doctorPatientRemarkService;

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DoctorPatientRemarkVO.class)
    public Result<DoctorPatientRemarkVO> add(@RequestBody DoctorPatientRemarkAddReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkAddDTO addDTO = new DoctorPatientRemarkAddDTO();
        addDTO.setPatientId(params.getPatientId());
        addDTO.setRelOneType(params.getRelOneType());
        addDTO.setRelOneId(params.getRelOneId());
        addDTO.setRemark(params.getRemark());
        addDTO.setDoctorId(user.getId());
        DoctorPatientRemarkVO data = doctorPatientRemarkService.add(addDTO);
        return new Result<DoctorPatientRemarkVO>().ok(data);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> updateTarget(@RequestBody DoctorPatientRemarkUpdateReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkUpdateDTO updateDTO = new DoctorPatientRemarkUpdateDTO();
        updateDTO.setId(params.getId());
        updateDTO.setRemark(params.getRemark());
        updateDTO.setDoctorId(user.getId());
        doctorPatientRemarkService.update(updateDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/remove")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> remove(@RequestBody DoctorPatientRemarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkGetDTO getDTO = new DoctorPatientRemarkGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        doctorPatientRemarkService.remove(getDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/clean")
    @ApiOperation(value = "清空", response = Void.class)
    public Result<Void> clean(@RequestBody DoctorPatientRemarkCleanDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorPatientRemarkService.clean(params);
        return new Result<Void>().success();
    }

    @PostMapping("/query")
    @ApiOperation(value = "分页列表", response = DoctorPatientRemarkVO.class)
    public Result<PageData<DoctorPatientRemarkVO>> query(@RequestBody DoctorPatientRemarkQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkQueryDTO queryDTO = new DoctorPatientRemarkQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setRelOneType(params.getRelOneType());
        queryDTO.setRelOneId(params.getRelOneId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        PageData<DoctorPatientRemarkVO> data = doctorPatientRemarkService.query(queryDTO);
        return new Result<PageData<DoctorPatientRemarkVO>>().ok(data);
    }

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DoctorPatientRemarkVO.class)
    public Result<List<DoctorPatientRemarkVO>> list(@RequestBody DoctorPatientRemarkQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkQueryDTO queryDTO = new DoctorPatientRemarkQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setRelOneType(params.getRelOneType());
        queryDTO.setRelOneId(params.getRelOneId());
        queryDTO.setStartDate(params.getStartDate());
        queryDTO.setEndDate(params.getEndDate());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        List<DoctorPatientRemarkVO> data = doctorPatientRemarkService.list(queryDTO);
        return new Result<List<DoctorPatientRemarkVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorPatientRemarkVO.class)
    public Result<DoctorPatientRemarkVO> detail(@RequestBody DoctorPatientRemarkGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorPatientRemarkGetDTO getDTO = new DoctorPatientRemarkGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        DoctorPatientRemarkVO vo = doctorPatientRemarkService.detail(getDTO);
        return new Result<DoctorPatientRemarkVO>().ok(vo);
    }
}
