package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetTotalCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetTotalCountTrendQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTaskTotalCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetCountQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetTotalCountQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetTotalCountTrendQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTaskTotalCountQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetCountTrendVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetCountWithDoctorVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetTotalCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTaskCountWithDoctorVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTaskTotalCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.ManageDoctorStatisticBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaCheckPermission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/manage/doctor/crm/statistic")
@Api(tags = "医生个案和任务统计管理")
public class ManageDoctorStatisticController {
    
    @Autowired
    private ManageDoctorStatisticBizService manageDoctorStatisticBizService;

    @SaCheckPermission("sys:crm:statistic:taskTotalCount")
    @PostMapping("/taskTotalCount")
    @ApiOperation(value = "统计医生任务各个状态数量（个案）", response = DoctorTaskTotalCountVO.class)
    public Result<DoctorTaskTotalCountVO> taskTotalCount(@RequestBody DoctorTaskTotalCountQueryReq req) {
        DoctorTaskTotalCountQueryDTO queryDTO = new DoctorTaskTotalCountQueryDTO();
        queryDTO.setDeptId(req.getDeptId());
        DoctorTaskTotalCountVO data = manageDoctorStatisticBizService.taskTotalCount(queryDTO);
        return new Result<DoctorTaskTotalCountVO>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:taskCountByDoctor")
    @PostMapping("/taskCountByDoctor")
    @ApiOperation(value = "根据医生统计医生任务各个状态数量（个案）", response = DoctorTaskCountWithDoctorVO.class)
    public Result<List<DoctorTaskCountWithDoctorVO>> taskCountByDoctor() {
        List<DoctorTaskCountWithDoctorVO> data = manageDoctorStatisticBizService.taskCountByDoctor();
        return new Result<List<DoctorTaskCountWithDoctorVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetTotalCount")
    @PostMapping("/targetTotalCount")
    @ApiOperation(value = "统计当前个案目标状态数量", response = DoctorTargetTotalCountVO.class)
    public Result<DoctorTargetTotalCountVO> targetTotalCount(@RequestBody DoctorTargetTotalCountQueryReq req) {
        DoctorTargetTotalCountQueryDTO queryDTO = new DoctorTargetTotalCountQueryDTO();
        queryDTO.setTargetCode(req.getTargetCode());
        queryDTO.setTargetCodes(req.getTargetCodes());
        queryDTO.setDateType(req.getDateType());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        queryDTO.setDeptId(req.getDeptId());
        queryDTO.setStageType(req.getStageType());
        DoctorTargetTotalCountVO data = manageDoctorStatisticBizService.targetTotalCount(queryDTO);
        return new Result<DoctorTargetTotalCountVO>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetHisTotalCount")
    @PostMapping("/targetHisTotalCount")
    @ApiOperation(value = "统计历史个案目标状态数量", response = DoctorTargetTotalCountVO.class)
    public Result<DoctorTargetTotalCountVO> targetHisTotalCount(@RequestBody DoctorTargetTotalCountQueryReq req) {
        DoctorTargetTotalCountQueryDTO queryDTO = new DoctorTargetTotalCountQueryDTO();
        queryDTO.setTargetCode(req.getTargetCode());
        queryDTO.setTargetCodes(req.getTargetCodes());
        queryDTO.setDateType(req.getDateType());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        queryDTO.setDeptId(req.getDeptId());
        queryDTO.setStageType(req.getStageType());
        DoctorTargetTotalCountVO data = manageDoctorStatisticBizService.targetHisTotalCount(queryDTO);
        return new Result<DoctorTargetTotalCountVO>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetCountByTarget")
    @PostMapping("/targetCountByTarget")
    @ApiOperation(value = "统计个案目标数量", response = DoctorTargetCountVO.class)
    public Result<List<DoctorTargetCountVO>> targetCount(@RequestBody(required = false) DoctorTargetCountQueryReq req) {
        DoctorTargetCountQueryDTO queryDTO = new DoctorTargetCountQueryDTO();
        if (null != req) {
            queryDTO.setStatus(req.getStatus());
            queryDTO.setTargetExeStatus(req.getTargetExeStatus());
        }
        List<DoctorTargetCountVO> data = manageDoctorStatisticBizService.targetCount(queryDTO);
        return new Result<List<DoctorTargetCountVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetHisCountByTarget")
    @PostMapping("/targetHisCountByTarget")
    @ApiOperation(value = "统计历史个案目标数量", response = DoctorTargetCountVO.class)
    public Result<List<DoctorTargetCountVO>> targetHisCount(@RequestBody(required = false) DoctorTargetCountQueryReq req) {
        DoctorTargetCountQueryDTO queryDTO = new DoctorTargetCountQueryDTO();
        if (null != req) {
            queryDTO.setStatus(req.getStatus());
            queryDTO.setTargetExeStatus(req.getTargetExeStatus());
        }
        List<DoctorTargetCountVO> data = manageDoctorStatisticBizService.targetHisCount(queryDTO);
        return new Result<List<DoctorTargetCountVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetCountByDoctor")
    @PostMapping("/targetCountByDoctor")
    @ApiOperation(value = "根据医生统计个案目标数量", response = DoctorTargetCountWithDoctorVO.class)
    public Result<List<DoctorTargetCountWithDoctorVO>> targetCountByDoctor() {
        List<DoctorTargetCountWithDoctorVO> data = manageDoctorStatisticBizService.targetCountByDoctor();
        return new Result<List<DoctorTargetCountWithDoctorVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetHisCountByDoctor")
    @PostMapping("/targetHisCountByDoctor")
    @ApiOperation(value = "根据医生统计历史个案目标数量", response = DoctorTargetCountWithDoctorVO.class)
    public Result<List<DoctorTargetCountWithDoctorVO>> targetHisCountByDoctor() {
        List<DoctorTargetCountWithDoctorVO> data = manageDoctorStatisticBizService.targetHisCountByDoctor();
        return new Result<List<DoctorTargetCountWithDoctorVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetCountTrend")
    @PostMapping("/targetCountTrend")
    @ApiOperation(value = "统计当前个案目标状态数量趋势", response = DoctorTargetCountTrendVO.class)
    public Result<List<DoctorTargetCountTrendVO>> targetCountTrend(@RequestBody DoctorTargetTotalCountTrendQueryReq req) {
        DoctorTargetTotalCountTrendQueryDTO queryDTO = new DoctorTargetTotalCountTrendQueryDTO();
        queryDTO.setTargetCode(req.getTargetCode());
        queryDTO.setTargetCodes(req.getTargetCodes());
        queryDTO.setDateType(req.getDateType());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        queryDTO.setStageType(req.getStageType());
        List<DoctorTargetCountTrendVO> data = manageDoctorStatisticBizService.targetCountTrend(queryDTO);
        return new Result<List<DoctorTargetCountTrendVO>>().ok(data);
    }
    
    @SaCheckPermission("sys:crm:statistic:targetHisCountTrend")
    @PostMapping("/targetHisCountTrend")
    @ApiOperation(value = "统计历史个案目标状态数量趋势", response = DoctorTargetCountTrendVO.class)
    public Result<List<DoctorTargetCountTrendVO>> targetHisCountTrend(@RequestBody DoctorTargetTotalCountTrendQueryReq req) {
        DoctorTargetTotalCountTrendQueryDTO queryDTO = new DoctorTargetTotalCountTrendQueryDTO();
        queryDTO.setTargetCode(req.getTargetCode());
        queryDTO.setTargetCodes(req.getTargetCodes());
        queryDTO.setDateType(req.getDateType());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        queryDTO.setStageType(req.getStageType());
        List<DoctorTargetCountTrendVO> data = manageDoctorStatisticBizService.targetHisCountTrend(queryDTO);
        return new Result<List<DoctorTargetCountTrendVO>>().ok(data);
    }
    
}
