package com.xxh.cloud.healthy.doctor.crm.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetConfigAddReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetConfigGetReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetConfigQueryReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.req.DoctorTargetConfigUpdateReq;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetConfigVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorTargetConfigService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/manage/doctor/crm/target")
@Api(tags = "医生目标配置管理")
public class ManageDoctorTargetConfigController {
    
    @Autowired
    private DoctorTargetConfigService doctorTargetConfigService;

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DoctorTargetConfigVO.class)
    public Result<DoctorTargetConfigVO> add(@RequestBody DoctorTargetConfigAddReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorTargetConfigAddDTO addDTO = new DoctorTargetConfigAddDTO();
        addDTO.setCode(params.getCode());
        addDTO.setName(params.getName());
        addDTO.setRemark(params.getRemark());
        addDTO.setDeptId(params.getDeptId());
        addDTO.setDiseaseId(params.getDiseaseId());
        addDTO.setExeDeptIds(params.getExeDeptIds());
        addDTO.setExeMaxDay(params.getExeMaxDay());
        addDTO.setType(params.getType());
        DoctorTargetConfigVO data = doctorTargetConfigService.add(addDTO);
        return new Result<DoctorTargetConfigVO>().ok(data);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> updateTarget(@RequestBody DoctorTargetConfigUpdateReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorTargetConfigUpdateDTO updateDTO = new DoctorTargetConfigUpdateDTO();
        updateDTO.setId(params.getId());
        // updateDTO.setCode(params.getCode()); // TODO 暂时不能更新code，更新code会导致无法与之前使用过的code相匹配，主要是查询的name的问题（name没有冗余）
        updateDTO.setName(params.getName());
        updateDTO.setRemark(params.getRemark());
        updateDTO.setDeptId(params.getDeptId());
        updateDTO.setDiseaseId(params.getDiseaseId());
        updateDTO.setExeDeptIds(params.getExeDeptIds());
        updateDTO.setExeMaxDay(params.getExeMaxDay());
        updateDTO.setType(params.getType());
        doctorTargetConfigService.update(updateDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/remove")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> remove(@RequestBody DoctorTargetConfigGetReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorTargetConfigGetDTO getDTO = new DoctorTargetConfigGetDTO();
        getDTO.setId(params.getId());
        doctorTargetConfigService.remove(getDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/query")
    @ApiOperation(value = "分页列表", response = DoctorTargetConfigVO.class)
    public Result<PageData<DoctorTargetConfigVO>> query(@RequestBody DoctorTargetConfigQueryReq params) {
        DoctorTargetConfigQueryDTO queryDTO = new DoctorTargetConfigQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setCode(params.getCode());
        queryDTO.setName(params.getName());
        queryDTO.setDeptId(params.getDeptId());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setType(params.getType());
        PageData<DoctorTargetConfigVO> data = doctorTargetConfigService.query(queryDTO);
        return new Result<PageData<DoctorTargetConfigVO>>().ok(data);
    }

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DoctorTargetConfigVO.class)
    public Result<List<DoctorTargetConfigVO>> list(@RequestBody DoctorTargetConfigQueryReq params) {
        DoctorTargetConfigQueryDTO queryDTO = new DoctorTargetConfigQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setCode(params.getCode());
        queryDTO.setName(params.getName());
        queryDTO.setDeptId(params.getDeptId());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setType(params.getType());
        List<DoctorTargetConfigVO> data = doctorTargetConfigService.list(queryDTO);
        return new Result<List<DoctorTargetConfigVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorTargetConfigVO.class)
    public Result<DoctorTargetConfigVO> detail(@RequestBody DoctorTargetConfigGetReq params) {
        ValidatorUtils.validateEntity(params);
        DoctorTargetConfigGetDTO getDTO = new DoctorTargetConfigGetDTO();
        getDTO.setId(params.getId());
        DoctorTargetConfigVO vo = doctorTargetConfigService.detail(getDTO);
        return new Result<DoctorTargetConfigVO>().ok(vo);
    }
}
