package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorBehaviorConstant;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorBehaviorInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorBehaviorInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorDataCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorBehaviorQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorBehaviorDataCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorBehaviorDataVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorBehaviorVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.mapper.DoctorBehaviorDataBizMapper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorBehaviorService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorBehaviorServiceImpl implements DoctorBehaviorService {

    @Autowired
    private DoctorBehaviorInfoService doctorBehaviorInfoService;
    @Autowired
    private DoctorBehaviorDataBizMapper doctorBehaviorDataBizMapper;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private DoctorPatientStatusMarkService doctorPatientStatusMarkService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorBehaviorVO add(DoctorBehaviorAddDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorBehaviorInfo entity = new DoctorBehaviorInfo();
        entity.setId(DoctorPatientHelper.getDoctorBehaviorInfo());
        entity.setDoctorId(params.getDoctorId());
        entity.setOperatedType(params.getOperatedType());
        entity.setOperatedContent(params.getOperatedContent());
        entity.setRelOneType(params.getRelOneType());
        entity.setRelOneId(params.getRelOneId());
        entity.setRelTwoType(params.getRelTwoType());
        entity.setRelTwoId(params.getRelTwoId());
        
        entity.setCreateDate(DateUtil.date());
        entity.setCreator(params.getDoctorId());
        entity.setIp(params.getIp());
        entity.setRowsStatus(YN.YES.getCode());

        doctorBehaviorInfoService.save(entity);

        return DoctorPatientHelper.getDoctorBehaviorVO(entity);
    }

    @Override
    public PageData<DoctorBehaviorVO> query(DoctorBehaviorQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DoctorBehaviorInfo> page = doctorBehaviorInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> doctorIds = new ArrayList<>();
        List<String> statusMarkIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());

            // 查询医生患者状态标记信息
            if (DoctorBehaviorConstant.RelOneType.Status_Mark.getCode().equals(i.getRelOneType())) {
                statusMarkIds.add(i.getRelOneId());
            }
        });

        // 查询医生信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(doctorIds)) {
            List<String> deptIds = new ArrayList<>();

            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        // 查询患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(statusMarkIds)) {
            DoctorPatientStatusMarkQueryDTO statusMarkQueryDTO = new DoctorPatientStatusMarkQueryDTO();
            statusMarkQueryDTO.setIds(statusMarkIds);
            List<DoctorPatientStatusMarkVO> statusMarkVOs = doctorPatientStatusMarkService.list(statusMarkQueryDTO);
            if (CollectionUtil.isNotEmpty(statusMarkVOs)) {
                statusMarkVOs.forEach(i -> {
                    PatientClientBO patient = i.getPatientInfo();
                    if (null != patient) {
                        patientMap.put(i.getId(), patient);
                    }
                });
            }
        }

        PageData<DoctorBehaviorVO> data = new PageConverter<DoctorBehaviorInfo, DoctorBehaviorVO>() {
            @Override
            public DoctorBehaviorVO converter(DoctorBehaviorInfo entity) {
                DoctorBehaviorVO vo = DoctorPatientHelper.getDoctorBehaviorVO(entity);

                // 医生信息
                AdminUserBO doctor = doctorMap.get(entity.getDoctorId());
                if (null != doctor) {
                    vo.setUserName(doctor.getUserName());
                    vo.setRealName(doctor.getRealName());
                    vo.setHeadUrl(doctor.getHeadUrl());
                    vo.setGender(doctor.getGender());
                    vo.setMobile(doctor.getMobile());

                    DeptBO dept = deptMap.get(doctor.getDeptId());
                    if (null != dept) {
                        vo.setDeptName(dept.getName());
                    }
                }

                PatientClientBO patient = patientMap.get(entity.getRelOneId());
                if (null != patient) {
                    vo.setPatientId(patient.getPatientId());
                    vo.setPatientName(patient.getPatientName());
                    vo.setPatientPhone(patient.getPhone());
                    vo.setPatientGender(patient.getPatientGenderStandard());
                    vo.setPatientAge(patient.getAge());
                }

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorBehaviorVO> list(DoctorBehaviorQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorBehaviorInfo> list = doctorBehaviorInfoService.list(new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .orderByDesc("create_date")
                .last(" limit 1000 ") // 限制查询数量
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> doctorIds = new ArrayList<>();
        List<String> statusMarkIds = new ArrayList<>();
        list.forEach(i -> {
            doctorIds.add(i.getDoctorId());

            // 查询医生患者状态标记信息
            if (DoctorBehaviorConstant.RelOneType.Status_Mark.getCode().equals(i.getRelOneType())) {
                statusMarkIds.add(i.getRelOneId());
            }
        });

        // 查询医生信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(doctorIds)) {
            List<String> deptIds = new ArrayList<>();

            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        // 查询患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(statusMarkIds)) {
            DoctorPatientStatusMarkQueryDTO statusMarkQueryDTO = new DoctorPatientStatusMarkQueryDTO();
            statusMarkQueryDTO.setIds(statusMarkIds);
            List<DoctorPatientStatusMarkVO> statusMarkVOs = doctorPatientStatusMarkService.list(statusMarkQueryDTO);
            if (CollectionUtil.isNotEmpty(statusMarkVOs)) {
                statusMarkVOs.forEach(i -> {
                    PatientClientBO patient = i.getPatientInfo();
                    if (null != patient) {
                        patientMap.put(i.getId(), patient);
                    }
                });
            }
        }

        return list.stream().map(i -> {
            DoctorBehaviorVO vo = DoctorPatientHelper.getDoctorBehaviorVO(i);

            // 医生信息
            AdminUserBO doctor = doctorMap.get(i.getDoctorId());
            if (null != doctor) {
                vo.setUserName(doctor.getUserName());
                vo.setRealName(doctor.getRealName());
                vo.setHeadUrl(doctor.getHeadUrl());
                vo.setGender(doctor.getGender());
                vo.setMobile(doctor.getMobile());

                DeptBO dept = deptMap.get(doctor.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }
            }

            PatientClientBO patient = patientMap.get(i.getRelOneId());
            if (null != patient) {
                vo.setPatientId(patient.getPatientId());
                vo.setPatientName(patient.getPatientName());
                vo.setPatientPhone(patient.getPhone());
                vo.setPatientGender(patient.getPatientGenderStandard());
                vo.setPatientAge(patient.getAge());
            }

            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public DoctorBehaviorVO detail(DoctorBehaviorGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorBehaviorInfo entity = doctorBehaviorInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return null;
        }
        return DoctorPatientHelper.getDoctorBehaviorVO(entity);
    }

    @Override
    public List<DoctorBehaviorDataCountVO> getBehaviorDataCount(DoctorBehaviorDataCountQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorBehaviorDataCountVO> dataList = doctorBehaviorDataBizMapper.getDoctorPatientBehaviorDataCount(new QueryWrapper<DoctorBehaviorInfo>()
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .in(CollectionUtil.isNotEmpty(params.getRelOneIds()), "rel_one_id", params.getRelOneIds())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoIds()), "rel_two_id", params.getRelTwoIds())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
        );
        return dataList;
    }

    @Override
    public PageData<DoctorBehaviorDataVO> doctorBehaviorDataList(DoctorBehaviorQueryDTO params) {
        // 排序方式
        String order = "DESC";
        if (YN.YES.getCode().equals(params.getOrder())) {
            order = "ASC";
        }
        // 排序字段
        String orderType = StrUtil.isBlank(params.getOrderType()) ? "" : params.getOrderType();

        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<Map<String, Object>> doctorPage = doctorBehaviorDataBizMapper.getDoctorPage(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
        , orderType, order);

        List<String> doctorIds = new ArrayList<>();
        doctorPage.getRecords().forEach(i -> {
            String doctorId = String.valueOf(i.get("doctorId"));
            doctorIds.add(doctorId);
        });

        if (CollectionUtil.isEmpty(doctorIds)) {
            return new PageData<>(null, 0);
        }

        // 查询医生行为数据信息
        List<DoctorBehaviorDataCountVO> dataList = doctorBehaviorDataBizMapper.getDoctorBehaviorDataCount(new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .in("doctor_id", doctorIds)
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
        );

        Map<String, List<DoctorBehaviorDataCountVO>> dataMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(dataList)) {
            dataList.forEach(i -> {
                List<DoctorBehaviorDataCountVO> data = dataMap.get(i.getDoctorId());
                if (null == data) {
                    data = new ArrayList<>();
                    dataMap.put(i.getDoctorId(), data);
                }
                data.add(i);
            });
        }

        // 查询医生与部门信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(doctorIds)) {
            List<String> deptIds = new ArrayList<>();

            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        PageData<DoctorBehaviorDataVO> data = new PageConverter<Map<String, Object>, DoctorBehaviorDataVO>() {
            @Override
            public DoctorBehaviorDataVO converter(Map<String, Object> map) {
                String doctorId = String.valueOf(map.get("doctorId"));

                DoctorBehaviorDataVO vo = new DoctorBehaviorDataVO();
                vo.setDoctorId(doctorId);
                vo.setDataCountList(dataMap.get(doctorId));

                AdminUserBO doctor = doctorMap.get(doctorId);
                if (null != doctor) {
                    vo.setUserName(doctor.getUserName());
                    vo.setRealName(doctor.getRealName());
                    vo.setHeadUrl(doctor.getHeadUrl());
                    vo.setGender(doctor.getGender());
                    vo.setMobile(doctor.getMobile());

                    DeptBO dept = deptMap.get(doctor.getDeptId());
                    if (null != dept) {
                        vo.setDeptName(dept.getName());
                    }
                }
                return vo;
            }
        }.getPageData(doctorPage);
        return data;
    }

    @Override
    public DoctorBehaviorVO firstSimple(DoctorBehaviorQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        DoctorBehaviorInfo entity = doctorBehaviorDataBizMapper.firstSimple(new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
        );

        return DoctorPatientHelper.getDoctorBehaviorVO(entity);
    }
    @Override
    public List<DoctorBehaviorVO> firstSimpleList(DoctorBehaviorQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorBehaviorInfo> list = doctorBehaviorDataBizMapper.firstSimpleList(new QueryWrapper<DoctorBehaviorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getOperatedType()), "operated_type", params.getOperatedType())
                .in(CollectionUtil.isNotEmpty(params.getOperatedTypes()), "operated_type", params.getOperatedTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .in(CollectionUtil.isNotEmpty(params.getRelOneTypes()), "rel_one_type", params.getRelOneTypes())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .eq(StrUtil.isNotBlank(params.getRelTwoType()), "rel_two_type", params.getRelTwoType())
                .in(CollectionUtil.isNotEmpty(params.getRelTwoTypes()), "rel_two_type", params.getRelTwoTypes())
                .eq(StrUtil.isNotBlank(params.getRelTwoId()), "rel_two_id", params.getRelTwoId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(i -> {
            DoctorBehaviorVO vo = DoctorPatientHelper.getDoctorBehaviorVO(i);
            return vo;
        }).collect(Collectors.toList());
    }
}
