package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientRemarkInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientRemarkInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkCleanDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientRemarkUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientRemarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientRemarkService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorPatientRemarkServiceImpl implements DoctorPatientRemarkService {

    @Autowired
    private DoctorPatientRemarkInfoService doctorPatientRemarkInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorPatientRemarkVO add(DoctorPatientRemarkAddDTO params) {
        ValidatorUtils.validateEntity(params);

        // zs20231229 有旧备注则删除备注，然后新增，保证最多有一条数据（旧数据可能查出多条所以直接删除而不更新）
        // doctorPatientRemarkInfoService.remove(new QueryWrapper<DoctorPatientRemarkInfo>()
        //     .eq("doctor_id", params.getDoctorId())
        //     .eq("patient_id", params.getPatientId())
        //     .eq("rel_one_type", params.getRelOneType())
        //     .eq("rel_one_id", params.getRelOneId())
        // );

        DoctorPatientRemarkInfo entity = new DoctorPatientRemarkInfo();
        entity.setId(DoctorPatientHelper.getDoctorPatientRemarkInfo());
        entity.setDoctorId(params.getDoctorId());
        entity.setPatientId(params.getPatientId());
        entity.setRemark(params.getRemark());
        entity.setRelOneType(params.getRelOneType());
        entity.setRelOneId(params.getRelOneId());
        entity.setRowsStatus(YN.YES.getCode());

        doctorPatientRemarkInfoService.save(entity);

        return DoctorPatientHelper.getDoctorPatientRemarkVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void update(DoctorPatientRemarkUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientRemarkInfo entity = doctorPatientRemarkInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }
        entity.setRemark(params.getRemark());
        doctorPatientRemarkInfoService.updateById(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(DoctorPatientRemarkGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientRemarkInfo entity = doctorPatientRemarkInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }
        entity.setRowsStatus(YN.NO.getCode());
        doctorPatientRemarkInfoService.updateById(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void clean(DoctorPatientRemarkCleanDTO params) {
        ValidatorUtils.validateEntity(params);

        List<DoctorPatientRemarkInfo> list = doctorPatientRemarkInfoService.list(new QueryWrapper<DoctorPatientRemarkInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("doctor_id", params.getDoctorId())
            .eq("rel_one_type", params.getRelOneType())
            .eq("rel_one_id", params.getRelOneId())
        );

        if (CollUtil.isNotEmpty(list)) {
            list.forEach(i -> i.setRowsStatus(YN.NO.getCode()));
            doctorPatientRemarkInfoService.updateBatchById(list);
        }
    }

    @Override
    public PageData<DoctorPatientRemarkVO> query(DoctorPatientRemarkQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DoctorPatientRemarkInfo> page = doctorPatientRemarkInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorPatientRemarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .orderByDesc("create_date")
        );

        PageData<DoctorPatientRemarkVO> data =  new PageConverter<DoctorPatientRemarkInfo, DoctorPatientRemarkVO>() {
            @Override
            public DoctorPatientRemarkVO converter(DoctorPatientRemarkInfo entity) {
                return DoctorPatientHelper.getDoctorPatientRemarkVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorPatientRemarkVO> list(DoctorPatientRemarkQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorPatientRemarkInfo> list = doctorPatientRemarkInfoService.list(new QueryWrapper<DoctorPatientRemarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getRelOneType()), "rel_one_type", params.getRelOneType())
                .eq(StrUtil.isNotBlank(params.getRelOneId()), "rel_one_id", params.getRelOneId())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .in(CollUtil.isNotEmpty(params.getQueryParams()), "CONCAT(patient_id,'_',rel_one_id)", params.getQueryParams())
                .orderByDesc("create_date")
                .last(" limit 1000 ") // 限制查询数量
        );
        return list.stream().map(DoctorPatientHelper::getDoctorPatientRemarkVO).collect(Collectors.toList());
    }

    @Override
    public DoctorPatientRemarkVO detail(DoctorPatientRemarkGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientRemarkInfo entity = doctorPatientRemarkInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return null;
        }
        return DoctorPatientHelper.getDoctorPatientRemarkVO(entity);
    }
}
