package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorTargetConstant;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientStatusMarkHisInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientStatusMarkHisInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTargetExeStatusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DwdDoctorComeQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkHisVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetConfigVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.ManageDoctorPatientStatusMarkHisVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkHisService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorTargetConfigService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DwdDoctorDataBizService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptBizClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorPatientStatusMarkHisServiceImpl implements DoctorPatientStatusMarkHisService {

    @Autowired
    private DoctorPatientStatusMarkHisInfoService doctorPatientStatusMarkHisInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private DeptBizClientService deptBizClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DoctorTargetConfigService doctorTargetConfigService;
    @Autowired
    private DwdDoctorDataBizService dwdDoctorDataBizService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DoctorPatientStatusMarkHisAddDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DateTime now = DateUtil.date();
        DoctorPatientStatusMarkHisInfo entity = new DoctorPatientStatusMarkHisInfo();
        entity.setId(DoctorPatientHelper.getDoctorPatientStatusMarkHisInfo());
        entity.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        entity.setPatientId(params.getPatientId());
        entity.setDoctorId(params.getDoctorId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setAddTime(now);
        entity.setOpTime(now);
        entity.setSourceType(params.getSourceType());
        entity.setSourceId(params.getSourceId());
        entity.setOpUserId(params.getOpUserId());
        entity.setStatus(params.getStatus());
        entity.setTargetId(params.getTargetId());
        entity.setTarget(params.getTarget());
        entity.setOpType(params.getOpType());
        entity.setType(params.getType());
        entity.setTargetCode(params.getTargetCode());
        entity.setTargetExeDeptIds(params.getTargetExeDeptIds());
        entity.setTargetExeMaxDay(params.getTargetExeMaxDay());
        entity.setTargetExeStatus(params.getTargetExeStatus());
        doctorPatientStatusMarkHisInfoService.save(entity);
    }

    @Override
    public void updateTargetExeStatusBatch(List<DoctorPatientStatusMarkUpdateTargetExeStatusDTO> params) {
        Map<String, DoctorPatientStatusMarkUpdateTargetExeStatusDTO> updateMap = new HashMap<>();
        List<String> ids = new ArrayList<>();
        params.forEach(i -> {
            ids.add(i.getId());
            updateMap.put(i.getId(), i);
        });

        log.info("1.查询当前数据 >>>>>> ");
        // 更新状态必须为normal才能更新目标
        List<DoctorPatientStatusMarkHisInfo> entityList = doctorPatientStatusMarkHisInfoService.list(new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
            .eq("status", DoctorStatusMarkConstant.Status.Normal.getCode())
            .in("id", ids)
        );

        entityList.forEach(i -> {
            DoctorPatientStatusMarkUpdateTargetExeStatusDTO updateDTO = updateMap.get(i.getId());
            if (null != updateDTO) {
                i.setTargetExeStatus(updateDTO.getTargetExeStatus());
                if (null != updateDTO.getOpTime()) {
                    i.setOpTime(updateDTO.getOpTime());
                }
            }
        });

        doctorPatientStatusMarkHisInfoService.updateBatchById(entityList);

    }
    
    @Override
    public PageData<DoctorPatientStatusMarkHisVO> page(DoctorPatientStatusMarkHisQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 按科室进行筛选
        List<String> queryDoctorIds = new ArrayList<>();
        if (StrUtil.isNotBlank(params.getDeptId())) {
            // 查询子科室
            List<DeptBO> deptBOs = deptBizClientService.getDeptTreeList(params.getDeptId());
            if (CollectionUtil.isEmpty(deptBOs)) {
                return new PageData<>(null, 0);
            }
            List<String> deptIds = new ArrayList<>();
            deptBOs.forEach(i -> deptIds.add(i.getId()));

            List<AdminUserBO> adminUserBOList = adminUserClientService.getUserListByDeptIds(deptIds);
            if (CollectionUtil.isEmpty(adminUserBOList)) {
                return new PageData<>(null, 0);
            }
            adminUserBOList.forEach(i -> queryDoctorIds.add(i.getId()));
        }

        Map<String, PatientClientBO> patientMap = new HashMap<>();
        List<String> patientIds = new ArrayList<>();
        // 若按照患者名称筛选则先查询患者信息
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }

        IPage<DoctorPatientStatusMarkHisInfo> page = doctorPatientStatusMarkHisInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getDoctorPatientMarkId()), "doctor_patient_mark_id", params.getDoctorPatientMarkId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .ne(StrUtil.isBlank(params.getIsDel()) || !YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getIsDel()) && YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getTargetExeStatus()), "target_exe_status", params.getTargetExeStatus())
                .orderByAsc("op_time")
        );
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询患者及病种信息
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        Map<String, DoctorTargetConfigVO> doctorTargetConfigVOMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> diseaseIds = new ArrayList<>();
            List<String> doctorIds = new ArrayList<>();
            List<String> targetIds = new ArrayList<>();
            page.getRecords().forEach(i -> {
                patientIds.add(i.getPatientId());
                diseaseIds.add(i.getDiseaseId());
                doctorIds.add(i.getDoctorId());
                targetIds.add(i.getTargetId());
            });

            // 医生
            List<String> deptIds = new ArrayList<>();
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }

            // 患者
            if (CollectionUtil.isEmpty(patientMap)) {
                PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
                patientQueryDTO.setPatientIds(patientIds);
                List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }

            // 病种
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));

            // 查询目标配置
            DoctorTargetConfigQueryDTO doctorTargetConfigQueryDTO = new DoctorTargetConfigQueryDTO();
            doctorTargetConfigQueryDTO.setIds(targetIds);
            List<DoctorTargetConfigVO> doctorTargetConfigVOList = doctorTargetConfigService.list(doctorTargetConfigQueryDTO);
            doctorTargetConfigVOList.forEach(i -> doctorTargetConfigVOMap.put(i.getId(), i));
        }

        PageData<DoctorPatientStatusMarkHisVO> data =  new PageConverter<DoctorPatientStatusMarkHisInfo, DoctorPatientStatusMarkHisVO>() {
            @Override
            public DoctorPatientStatusMarkHisVO converter(DoctorPatientStatusMarkHisInfo entity) {
                DoctorPatientStatusMarkHisVO vo = DoctorPatientHelper.getDoctorPatientStatusMarkHisVO(entity);

                vo.setPatientInfo(patientMap.get(vo.getPatientId()));
                vo.setDiseaseInfo(diseaseMap.get(vo.getDiseaseId()));

                AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorInfo(doctor);
                    vo.setDeptInfo(deptMap.get(doctor.getDeptId()));
                }
                
                DoctorTargetConfigVO doctorTargetConfigVO = doctorTargetConfigVOMap.get(vo.getTargetId());
                if (null != doctorTargetConfigVO) {
                    vo.setTargetName(doctorTargetConfigVO.getName());
                }

                return vo;
            }
        }.getPageData(page);
        return data;
    }
    
    @Override
    public List<DoctorPatientStatusMarkHisVO> list(DoctorPatientStatusMarkHisQueryDTO params) {
        List<DoctorPatientStatusMarkHisInfo> list = doctorPatientStatusMarkHisInfoService.list(new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
                .eq(StrUtil.isNotBlank(params.getDoctorPatientMarkId()), "doctor_patient_mark_id", params.getDoctorPatientMarkId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .ne(StrUtil.isBlank(params.getIsDel()) || !YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getIsDel()) && YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .orderByAsc("op_time")
                .last(" limit 1000 ") // 限制查询数量
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        return list.stream().map(DoctorPatientHelper::getDoctorPatientStatusMarkHisVO).collect(Collectors.toList());
    }

    @Override
    public PageData<ManageDoctorPatientStatusMarkHisVO> managePage(DoctorPatientStatusMarkHisQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 按科室进行筛选
        List<String> queryDoctorIds = new ArrayList<>();
        if (StrUtil.isNotBlank(params.getDeptId())) {
            // 查询子科室
            List<DeptBO> deptBOs = deptBizClientService.getDeptTreeList(params.getDeptId());
            if (CollectionUtil.isEmpty(deptBOs)) {
                return new PageData<>(null, 0);
            }
            List<String> deptIds = new ArrayList<>();
            deptBOs.forEach(i -> deptIds.add(i.getId()));

            List<AdminUserBO> adminUserBOList = adminUserClientService.getUserListByDeptIds(deptIds);
            if (CollectionUtil.isEmpty(adminUserBOList)) {
                return new PageData<>(null, 0);
            }
            adminUserBOList.forEach(i -> queryDoctorIds.add(i.getId()));
        }

        Map<String, PatientClientBO> patientMap = new HashMap<>();
        List<String> patientIds = new ArrayList<>();
        // 若按照患者名称筛选则先查询患者信息
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }

        IPage<DoctorPatientStatusMarkHisInfo> page = doctorPatientStatusMarkHisInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getDoctorPatientMarkId()), "doctor_patient_mark_id", params.getDoctorPatientMarkId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .ne(StrUtil.isBlank(params.getIsDel()) || !YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getIsDel()) && YN.YES.getCode().equals(params.getIsDel()), "op_type", OpTypeEnum.DEL.getCode())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getTargetExeStatus()), "target_exe_status", params.getTargetExeStatus())
                .orderByAsc("op_time")
        );
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询患者及病种信息
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        Map<String, DoctorTargetConfigVO> doctorTargetConfigVOMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> diseaseIds = new ArrayList<>();
            List<String> doctorIds = new ArrayList<>();
            List<String> targetIds = new ArrayList<>();
            page.getRecords().forEach(i -> {
                patientIds.add(i.getPatientId());
                diseaseIds.add(i.getDiseaseId());
                doctorIds.add(i.getDoctorId());
                targetIds.add(i.getTargetId());
            });

            // 医生
            List<String> deptIds = new ArrayList<>();
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }

            // 患者
            if (CollectionUtil.isEmpty(patientMap)) {
                PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
                patientQueryDTO.setPatientIds(patientIds);
                List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }

            // 病种
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            
            // 查询目标配置
            DoctorTargetConfigQueryDTO doctorTargetConfigQueryDTO = new DoctorTargetConfigQueryDTO();
            doctorTargetConfigQueryDTO.setIds(targetIds);
            List<DoctorTargetConfigVO> doctorTargetConfigVOList = doctorTargetConfigService.list(doctorTargetConfigQueryDTO);
            doctorTargetConfigVOList.forEach(i -> doctorTargetConfigVOMap.put(i.getId(), i));
        }

        PageData<ManageDoctorPatientStatusMarkHisVO> data =  new PageConverter<DoctorPatientStatusMarkHisInfo, ManageDoctorPatientStatusMarkHisVO>() {
            @Override
            public ManageDoctorPatientStatusMarkHisVO converter(DoctorPatientStatusMarkHisInfo entity) {
                ManageDoctorPatientStatusMarkHisVO vo = DoctorPatientHelper.getManageDoctorPatientStatusMarkHisVO(entity);

                PatientClientBO patient = patientMap.get(vo.getPatientId());
                if (null != patient) {
                    vo.setPatientName(patient.getPatientName());
                    vo.setPatientSex(patient.getPatientGenderStandard());
                    vo.setPatientAge(patient.getAge());
                }

                StandardDiseaseBO disease = diseaseMap.get(vo.getDiseaseId());
                if (null != disease) {
                    vo.setDiseaseName(disease.getName());
                }

                AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorName(doctor.getRealName());

                    DeptBO dept = deptMap.get(doctor.getDeptId());
                    if (null != dept) {
                        vo.setDeptName(dept.getName());
                    }
                }
                
                DoctorTargetConfigVO doctorTargetConfigVO = doctorTargetConfigVOMap.get(vo.getTargetId());
                if (null != doctorTargetConfigVO) {
                    vo.setTargetName(doctorTargetConfigVO.getName());
                }

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public ManageDoctorPatientStatusMarkHisVO manageDetail(DoctorPatientStatusMarkHisGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DoctorPatientStatusMarkHisInfo entity = doctorPatientStatusMarkHisInfoService.getOne(new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );

        if (null == entity) {
            return null;
        }
        ManageDoctorPatientStatusMarkHisVO vo = DoctorPatientHelper.getManageDoctorPatientStatusMarkHisVO(entity);

        
        // 医生
        AdminUserBO doctor = adminUserClientService.getUserById(vo.getDoctorId());
        if (null != doctor) {
            vo.setDoctorName(doctor.getRealName());

            // 查询部门信息
            DeptBO dept = deptClientService.getDeptById(doctor.getDeptId());
            if (null != dept) {
                vo.setDeptName(dept.getName());
            }
        }

        // 患者
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientId(vo.getPatientId());
        PatientClientBO patient = patientClientService.getOneSimple(patientQueryDTO);
        if (null != patient) {
            vo.setPatientName(patient.getPatientName());
            vo.setPatientSex(patient.getPatientGenderStandard());
            vo.setPatientAge(patient.getAge());
        }

        // 病种
        StandardDiseaseBO disease = standardDiseaseClientService.getById(vo.getDiseaseId());
        if (null != disease) {
            vo.setDiseaseName(disease.getName());
        }

        // 查询患者最新数据信息
        // List<DataDwsPatientDataLastDateBO> patientDataLastDateList = dwdDataPatientClientService.listPatientDataLastDateInfo(patientIds);
        // patientDataLastDateList.forEach(i -> patientDataLastDateMap.put(i.getPatientId(), i));

        // 查询目标配置
        DoctorTargetConfigGetDTO doctorTargetConfigGetDTO = new DoctorTargetConfigGetDTO();
        doctorTargetConfigGetDTO.setId(vo.getTargetId());
        DoctorTargetConfigVO doctorTargetConfigVO = doctorTargetConfigService.detail(doctorTargetConfigGetDTO);
        if (null != doctorTargetConfigVO) {
            vo.setTargetName(doctorTargetConfigVO.getName());
        }

        // 目标数据依据
        String targetCode = entity.getTargetCode();
        Integer maxDay = entity.getTargetExeMaxDay();
        if (null != maxDay) {
            Date startDate = entity.getAddTime(); // 目标开始时间
            Date endDate = DateUtil.offsetDay(startDate, maxDay); // 目标结束时间

            DwdDoctorComeQueryDTO dwdDoctorComeQueryDTO = new DwdDoctorComeQueryDTO();
            dwdDoctorComeQueryDTO.setStartDate(startDate);
            dwdDoctorComeQueryDTO.setEndDate(endDate);
            if (StrUtil.isNotBlank(entity.getTargetExeDeptIds())) {
                dwdDoctorComeQueryDTO.setDeptIds(StrJoinSplitUtils.split2List((entity.getTargetExeDeptIds())));
            }

            // 挂号
            if (DoctorTargetConstant.Target.TARGET_REG.getCode().equals(targetCode)) {
                List<Map<String, Object>> registerDataList = dwdDoctorDataBizService.registerDataList(dwdDoctorComeQueryDTO);
                vo.setRegisterDataList(registerDataList);
            }
            // 就诊
            else if (DoctorTargetConstant.Target.TARGET_VISIT.getCode().equals(targetCode)) {
                List<Map<String, Object>> visitDataList = dwdDoctorDataBizService.visitDataList(dwdDoctorComeQueryDTO);
                vo.setVisitDataList(visitDataList);
            }
            // 住院
            else if (DoctorTargetConstant.Target.TARGET_IN.getCode().equals(targetCode)) {
                List<Map<String, Object>> inHospitalDataList = dwdDoctorDataBizService.inHospitalDataList(dwdDoctorComeQueryDTO);
                vo.setInHospitalDataList(inHospitalDataList);
            }
            // 出院
            else if (DoctorTargetConstant.Target.TARGET_OUT.getCode().equals(targetCode)) {
                List<Map<String, Object>> outHospitalDataList = dwdDoctorDataBizService.outHospitalDataList(dwdDoctorComeQueryDTO);
                vo.setOutHospitalDataList(outHospitalDataList);
            }
            // 手术
            else if (DoctorTargetConstant.Target.TARGET_OPERATION.getCode().equals(targetCode)) {
                List<Map<String, Object>> operationDataList = dwdDoctorDataBizService.operationDataList(dwdDoctorComeQueryDTO);
                vo.setOperationDataList(operationDataList);
            }
            // 检查
            else if (DoctorTargetConstant.Target.TARGET_EXAMS.getCode().equals(targetCode)) {
                List<Map<String, Object>> examineDataList = dwdDoctorDataBizService.examineDataList(dwdDoctorComeQueryDTO);
                vo.setExamineDataList(examineDataList);
            }
            // 检验
            else if (DoctorTargetConstant.Target.TARGET_LABS.getCode().equals(targetCode)) {
                List<Map<String, Object>> laboratoryDataList = dwdDoctorDataBizService.laboratoryDataList(dwdDoctorComeQueryDTO);
                vo.setLaboratoryDataList(laboratoryDataList);
            }
        }

        return vo;
    }

}
