package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DataDwsPatientDataLastDateBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientStatusMarkInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientStatusMarkInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkHisAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkTargetCountQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateFocusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateStatusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTargetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTypeDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkTargetCountVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.mapper.DoctorStatusMarkDataBizMapper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkHisService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkV1Service;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorPatientStatusMarkV1ServiceImpl implements DoctorPatientStatusMarkV1Service {

    @Autowired
    private DoctorPatientStatusMarkInfoService doctorPatientStatusMarkInfoService;
    @Autowired
    private DoctorPatientStatusMarkHisService doctorPatientStatusMarkHisService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private DoctorStatusMarkDataBizMapper doctorStatusMarkDataBizMapper;
    
    /**
     * 保存个案管理，并记录历史
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorPatientStatusMarkVO add(DoctorPatientStatusMarkAddDTO params) {
        ValidatorUtils.validateEntity(params);
        // 根据患者id，医生id，病种id查询是否存在个案管理
        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getOne(new QueryWrapper<DoctorPatientStatusMarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("doctor_id", params.getDoctorId())
                .eq("patient_id", params.getPatientId())
                .eq("disease_id", params.getDiseaseId())
        );

        // 已存在并且不是忽略状态或是忽略状态但状态参数不是normal的数据直接返回
        if (null != entity && (!DoctorStatusMarkConstant.Status.Ignore.getCode().equals(entity.getStatus()) || !DoctorStatusMarkConstant.Status.Normal.getCode().equals(params.getStatus()))) {
            return DoctorPatientHelper.getDoctorPatientStatusMarkVO(entity);
        }

        DateTime now = DateUtil.date();
        if (null == entity) { // 如果不存在则新增并返回；如果存在不操作直接返回
            log.info("1.保存数据 >>>>>> ");
            entity = new DoctorPatientStatusMarkInfo();
            entity.setId(DoctorPatientHelper.getDoctorPatientStatusMarkInfo());
            entity.setPatientId(params.getPatientId());
            entity.setDoctorId(params.getDoctorId());
            entity.setDiseaseId(params.getDiseaseId());
            entity.setAddTime(now);
            entity.setOpTime(now);
            entity.setSourceType(params.getSourceType());
            entity.setSourceId(params.getSourceId());
            entity.setTarget("-1"); // 初始个案管理目标
            entity.setType(params.getType());

            entity.setStatus(params.getStatus());
            if (DoctorStatusMarkConstant.Status.Normal.getCode().equals(params.getStatus())) {
                entity.setStatus(DoctorStatusMarkConstant.Status.Normal.getCode()); // 加入个案管理
            } else {
                entity.setStatus(DoctorStatusMarkConstant.Status.Ignore.getCode()); // 忽略
            }
            entity.setFocus(YN.NO.getCode());
            entity.setRowsStatus(YN.YES.getCode());
            doctorPatientStatusMarkInfoService.save(entity);
        } else {
            log.info("1.更新数据 >>>>>> ");
            entity.setStatus(DoctorStatusMarkConstant.Status.Normal.getCode());
            entity.setAddTime(now);
            entity.setOpTime(now);
            entity.setSourceType(params.getSourceType());
            entity.setSourceId(params.getSourceId());
            doctorPatientStatusMarkInfoService.updateById(entity);
        }

        // 保存历史记录
        log.info("2.保存历史 >>>>>> ");
        DoctorPatientStatusMarkHisAddDTO hisAddDTO = new DoctorPatientStatusMarkHisAddDTO();
        hisAddDTO.setDoctorPatientMarkId(entity.getId());
        hisAddDTO.setDoctorId(entity.getDoctorId());
        hisAddDTO.setPatientId(entity.getPatientId());
        hisAddDTO.setDiseaseId(entity.getDiseaseId());
        hisAddDTO.setSourceType(entity.getSourceType());
        hisAddDTO.setSourceId(entity.getSourceId());
        hisAddDTO.setStatus(entity.getStatus());
        hisAddDTO.setAddTime(entity.getAddTime());
        hisAddDTO.setOpTime(entity.getOpTime());
        hisAddDTO.setTarget(entity.getTarget());
        hisAddDTO.setOpUserId(params.getOpUserId());
        hisAddDTO.setOpType(OpTypeEnum.Create.getCode());
        hisAddDTO.setType(entity.getType());
        doctorPatientStatusMarkHisService.add(hisAddDTO);

        return DoctorPatientHelper.getDoctorPatientStatusMarkVO(entity);
    }

    /**
     * 更新目标，同时保存历史
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTarget(DoctorPatientStatusMarkUpdateTargetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        // 更新状态必须为normal才能更新目标
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !DoctorStatusMarkConstant.Status.Normal.getCode().equals(entity.getStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }
        
        DateTime now = DateUtil.date();
        entity.setTarget(params.getTarget());
        entity.setOpTime(now);
        doctorPatientStatusMarkInfoService.updateById(entity);

        // 保存历史记录
        log.info("2.保存历史 >>>>>> ");
        DoctorPatientStatusMarkHisAddDTO hisAddDTO = new DoctorPatientStatusMarkHisAddDTO();
        hisAddDTO.setDoctorPatientMarkId(entity.getId());
        hisAddDTO.setDoctorId(entity.getDoctorId());
        hisAddDTO.setPatientId(entity.getPatientId());
        hisAddDTO.setDiseaseId(entity.getDiseaseId());
        hisAddDTO.setSourceType(entity.getSourceType());
        hisAddDTO.setSourceId(entity.getSourceId());
        hisAddDTO.setStatus(entity.getStatus());
        hisAddDTO.setAddTime(entity.getAddTime());
        hisAddDTO.setOpTime(entity.getOpTime());
        hisAddDTO.setTarget(entity.getTarget());
        hisAddDTO.setOpUserId(params.getOpUserId());
        hisAddDTO.setOpType(OpTypeEnum.Update.getCode());
        hisAddDTO.setType(entity.getType());
        doctorPatientStatusMarkHisService.add(hisAddDTO);
    }

    /**
     * 更新状态，同时保存历史
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateStatus(DoctorPatientStatusMarkUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        // 更新状态必须为normal或abandon才能更新目标
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || DoctorStatusMarkConstant.Status.Ignore.getCode().equals(entity.getStatus()) || !entity.getDoctorId().equals(params.getDoctorId()) || entity.getStatus().equals(params.getStatus())) {
            return;
        }
        
        DateTime now = DateUtil.date();
        entity.setStatus(params.getStatus());
        entity.setOpTime(now);
        doctorPatientStatusMarkInfoService.updateById(entity);

        // 保存历史记录
        log.info("2.保存历史 >>>>>> ");
        DoctorPatientStatusMarkHisAddDTO hisAddDTO = new DoctorPatientStatusMarkHisAddDTO();
        hisAddDTO.setDoctorPatientMarkId(entity.getId());
        hisAddDTO.setDoctorId(entity.getDoctorId());
        hisAddDTO.setPatientId(entity.getPatientId());
        hisAddDTO.setDiseaseId(entity.getDiseaseId());
        hisAddDTO.setSourceType(entity.getSourceType());
        hisAddDTO.setSourceId(entity.getSourceId());
        hisAddDTO.setStatus(entity.getStatus());
        hisAddDTO.setAddTime(entity.getAddTime());
        hisAddDTO.setOpTime(entity.getOpTime());
        hisAddDTO.setTarget(entity.getTarget());
        hisAddDTO.setOpUserId(params.getOpUserId());
        hisAddDTO.setOpType(OpTypeEnum.Update.getCode());
        hisAddDTO.setType(entity.getType());
        doctorPatientStatusMarkHisService.add(hisAddDTO);
    }

    /**
     * 关注/取消关注
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateFocus(DoctorPatientStatusMarkUpdateFocusDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        // 更新状态必须为normal才能更新目标
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !DoctorStatusMarkConstant.Status.Normal.getCode().equals(entity.getStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }

        if (YN.YES.getCode().equals(entity.getFocus())) {
            entity.setFocus(YN.NO.getCode());
        } else {
            entity.setFocus(YN.YES.getCode());
        }
        
        DateTime now = DateUtil.date();
        entity.setOpTime(now);
        doctorPatientStatusMarkInfoService.updateById(entity);
    }

    /**
     * 更新个案类型
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateType(DoctorPatientStatusMarkUpdateTypeDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        // 更新状态必须为normal才能更新目标
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !DoctorStatusMarkConstant.Status.Normal.getCode().equals(entity.getStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }

        DateTime now = DateUtil.date();
        entity.setType(params.getType());
        entity.setOpTime(now);
        doctorPatientStatusMarkInfoService.updateById(entity);

        // 保存历史记录
        log.info("2.保存历史 >>>>>> ");
        DoctorPatientStatusMarkHisAddDTO hisAddDTO = new DoctorPatientStatusMarkHisAddDTO();
        hisAddDTO.setDoctorPatientMarkId(entity.getId());
        hisAddDTO.setDoctorId(entity.getDoctorId());
        hisAddDTO.setPatientId(entity.getPatientId());
        hisAddDTO.setDiseaseId(entity.getDiseaseId());
        hisAddDTO.setSourceType(entity.getSourceType());
        hisAddDTO.setSourceId(entity.getSourceId());
        hisAddDTO.setStatus(entity.getStatus());
        hisAddDTO.setAddTime(entity.getAddTime());
        hisAddDTO.setOpTime(entity.getOpTime());
        hisAddDTO.setTarget(entity.getTarget());
        hisAddDTO.setOpUserId(params.getOpUserId());
        hisAddDTO.setOpType(OpTypeEnum.Update.getCode());
        hisAddDTO.setType(entity.getType());
        doctorPatientStatusMarkHisService.add(hisAddDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(DoctorPatientStatusMarkGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }

        DateTime now = DateUtil.date();
        entity.setRowsStatus(YN.NO.getCode());
        entity.setOpTime(now);
        doctorPatientStatusMarkInfoService.updateById(entity);

        // 保存历史记录
        log.info("1.保存历史 >>>>>> ");
        DoctorPatientStatusMarkHisAddDTO hisAddDTO = new DoctorPatientStatusMarkHisAddDTO();
        hisAddDTO.setDoctorPatientMarkId(entity.getId());
        hisAddDTO.setDoctorId(entity.getDoctorId());
        hisAddDTO.setPatientId(entity.getPatientId());
        hisAddDTO.setDiseaseId(entity.getDiseaseId());
        hisAddDTO.setSourceType(entity.getSourceType());
        hisAddDTO.setSourceId(entity.getSourceId());
        hisAddDTO.setStatus(entity.getStatus());
        hisAddDTO.setAddTime(entity.getAddTime());
        hisAddDTO.setOpTime(entity.getOpTime());
        hisAddDTO.setTarget(entity.getTarget());
        hisAddDTO.setOpUserId(params.getDoctorId());
        hisAddDTO.setOpType(OpTypeEnum.DEL.getCode());
        hisAddDTO.setType(entity.getType());
        doctorPatientStatusMarkHisService.add(hisAddDTO);
    }

    @Override
    public PageData<DoctorPatientStatusMarkVO> query(DoctorPatientStatusMarkQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        Map<String, PatientClientBO> patientMap = new HashMap<>();
        List<String> patientIds = new ArrayList<>();
        // 若按照患者名称筛选则先查询患者信息
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }

        IPage<DoctorPatientStatusMarkInfo> page = doctorPatientStatusMarkInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorPatientStatusMarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(patientIds), "patient_id", patientIds)
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getTarget()), "target", params.getTarget())
                .in(CollectionUtil.isNotEmpty(params.getTargets()), "target", params.getTargets())
                .eq(StrUtil.isNotBlank(params.getFocus()), "focus", YN.YES.getCode())
                .ge(null != params.getStartDate(), "op_time", params.getStartDate())
                .lt(null != params.getEndDate(), "op_time", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .orderByDesc("focus", "op_time")
        );
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询患者及病种信息
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        Map<String, DataDwsPatientDataLastDateBO> patientDataLastDateMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> diseaseIds = new ArrayList<>();
            List<String> doctorIds = new ArrayList<>();
            page.getRecords().forEach(i -> {
                patientIds.add(i.getPatientId());
                diseaseIds.add(i.getDiseaseId());
                doctorIds.add(i.getDoctorId());
            });

            // 医生
            List<String> deptIds = new ArrayList<>();
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            doctorList.forEach(i -> {
                doctorMap.put(i.getId(), i);
                deptIds.add(i.getDeptId());
            });

            // 查询部门信息
            if (CollectionUtil.isNotEmpty(deptIds)) {
                List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }

            // 患者
            if (CollectionUtil.isEmpty(patientMap)) {
                PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
                patientQueryDTO.setPatientIds(patientIds);
                List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }

            // 病种
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));

            // 查询患者最新数据信息
            List<DataDwsPatientDataLastDateBO> patientDataLastDateList = dwdDataPatientClientService.listPatientDataLastDateInfo(patientIds);
            patientDataLastDateList.forEach(i -> patientDataLastDateMap.put(i.getPatientId(), i));
        }

        PageData<DoctorPatientStatusMarkVO> data =  new PageConverter<DoctorPatientStatusMarkInfo, DoctorPatientStatusMarkVO>() {
            @Override
            public DoctorPatientStatusMarkVO converter(DoctorPatientStatusMarkInfo entity) {
                DoctorPatientStatusMarkVO vo = DoctorPatientHelper.getDoctorPatientStatusMarkVO(entity);
                vo.setPatientInfo(patientMap.get(vo.getPatientId()));
                vo.setDiseaseInfo(diseaseMap.get(vo.getDiseaseId()));
                vo.setPatientDataLastDateInfo(patientDataLastDateMap.get(vo.getPatientId()));

                AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorInfo(doctor);
                    vo.setDeptInfo(deptMap.get(doctor.getDeptId()));
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorPatientStatusMarkVO> list(DoctorPatientStatusMarkQueryDTO params) {
        List<DoctorPatientStatusMarkInfo> list = doctorPatientStatusMarkInfoService.list(new QueryWrapper<DoctorPatientStatusMarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getTarget()), "target", params.getTarget())
                .in(CollectionUtil.isNotEmpty(params.getTargets()), "target", params.getTargets())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .orderByDesc("op_time")
        );

        // 查询患者及病种信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> patientIds = new ArrayList<>();
            List<String> diseaseIds = new ArrayList<>();
            list.forEach(i -> {
                patientIds.add(i.getPatientId());
                diseaseIds.add(i.getDiseaseId());
            });

            // 患者
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(patientIds);
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            patientList.forEach(i -> patientMap.put(i.getPatientId(), i));

            // 病种
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
        }

        return list.stream().map(i -> {
            DoctorPatientStatusMarkVO vo = DoctorPatientHelper.getDoctorPatientStatusMarkVO(i);
            vo.setPatientInfo(patientMap.get(vo.getPatientId()));
            vo.setDiseaseInfo(diseaseMap.get(vo.getDiseaseId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public DoctorPatientStatusMarkVO detail(DoctorPatientStatusMarkGetDTO params) {
        DoctorPatientStatusMarkInfo entity = doctorPatientStatusMarkInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return null;
        }

        // 用户信息
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientId(entity.getPatientId());
        PatientClientBO patient = patientClientService.getOneSimple(patientQueryDTO);

        // 病种信息
        StandardDiseaseBO disease = standardDiseaseClientService.getById(entity.getDiseaseId());

        // 医生信息
        AdminUserBO doctor = adminUserClientService.getUserById(entity.getDoctorId());

        // 部门信息
        DeptBO dept = null;
        if (null != doctor) {
            dept = deptClientService.getDeptById(doctor.getDeptId());
        }

        DoctorPatientStatusMarkVO vo =  DoctorPatientHelper.getDoctorPatientStatusMarkVO(entity);
        vo.setPatientInfo(patient);
        vo.setDiseaseInfo(disease);
        vo.setDoctorInfo(doctor);
        vo.setDeptInfo(dept);

        return vo;
    }

    @Override
    public List<DoctorPatientStatusMarkTargetCountVO> getTargetCount(DoctorPatientStatusMarkTargetCountQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

         return doctorStatusMarkDataBizMapper.getTargetCountListV1(new QueryWrapper<DoctorPatientStatusMarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .in(CollectionUtil.isNotEmpty(params.getStatuses()), "status", params.getStatuses())
                .eq(StrUtil.isNotBlank(params.getTarget()), "target", params.getTarget())
                .in(CollectionUtil.isNotEmpty(params.getTargets()), "target", params.getTargets())
                .ge(null != params.getStartDate(), "op_time", params.getStartDate())
                .lt(null != params.getEndDate(), "op_time", params.getEndDate())
        );
    }

}