package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientTrackInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientTrackInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientTrackUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientStatusMarkVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorPatientTrackVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientTrackService;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorPatientTrackServiceImpl implements DoctorPatientTrackService {

    @Autowired
    private DoctorPatientTrackInfoService doctorPatientTrackInfoService;
    @Autowired
    private DoctorPatientStatusMarkService doctorPatientStatusMarkService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorPatientTrackVO add(DoctorPatientTrackAddDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientStatusMarkGetDTO doctorPatientStatusMarkGetDTO = new DoctorPatientStatusMarkGetDTO();
        doctorPatientStatusMarkGetDTO.setId(params.getDoctorPatientMarkId());
        doctorPatientStatusMarkGetDTO.setDoctorId(params.getDoctorId());
        DoctorPatientStatusMarkVO doctorPatientStatusMarkVO = doctorPatientStatusMarkService.detail(doctorPatientStatusMarkGetDTO);
        if (null == doctorPatientStatusMarkVO) {
            throw new BusinessException("医生患者状态标记数据不存在");
        }

        DoctorPatientTrackInfo entity = new DoctorPatientTrackInfo();
        entity.setId(DoctorPatientHelper.getDoctorPatientTrackInfo());
        entity.setDoctorPatientMarkId(params.getDoctorPatientMarkId());
        entity.setDoctorId(params.getDoctorId());
        entity.setContent(params.getContent());
        entity.setDiseaseId(doctorPatientStatusMarkVO.getDiseaseId());
        entity.setPatientId(doctorPatientStatusMarkVO.getPatientId());
        entity.setRowsStatus(YN.YES.getCode());

        doctorPatientTrackInfoService.save(entity);

        return DoctorPatientHelper.getDoctorPatientTrackVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void update(DoctorPatientTrackUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientTrackInfo entity = doctorPatientTrackInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }
        entity.setContent(params.getContent());
        doctorPatientTrackInfoService.updateById(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(DoctorPatientTrackGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientTrackInfo entity = doctorPatientTrackInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }
        entity.setRowsStatus(YN.NO.getCode());
        doctorPatientTrackInfoService.updateById(entity);
    }

    @Override
    public PageData<DoctorPatientTrackVO> query(DoctorPatientTrackQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DoctorPatientTrackInfo> page = doctorPatientTrackInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorPatientTrackInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getDoctorPatientMarkId()), "doctor_patient_mark_id", params.getDoctorPatientMarkId())
                .ge(null != params.getStartDate(), "update_date", params.getStartDate())
                .lt(null != params.getEndDate(), "update_date", params.getEndDate())
                .orderByDesc("update_date")
        );

        PageData<DoctorPatientTrackVO> data =  new PageConverter<DoctorPatientTrackInfo, DoctorPatientTrackVO>() {
            @Override
            public DoctorPatientTrackVO converter(DoctorPatientTrackInfo entity) {
                return DoctorPatientHelper.getDoctorPatientTrackVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorPatientTrackVO> list(DoctorPatientTrackQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorPatientTrackInfo> list = doctorPatientTrackInfoService.list(new QueryWrapper<DoctorPatientTrackInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getDoctorPatientMarkId()), "doctor_patient_mark_id", params.getDoctorPatientMarkId())
                .ge(null != params.getStartDate(), "update_date", params.getStartDate())
                .lt(null != params.getEndDate(), "update_date", params.getEndDate())
                .orderByDesc("update_date")
                .last(" limit 1000 ") // 限制查询数量
        );
        return list.stream().map(DoctorPatientHelper::getDoctorPatientTrackVO).collect(Collectors.toList());
    }

    @Override
    public DoctorPatientTrackVO detail(DoctorPatientTrackGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorPatientTrackInfo entity = doctorPatientTrackInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return null;
        }
        return DoctorPatientHelper.getDoctorPatientTrackVO(entity);
    }
}
