package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorTargetConfigInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorTargetConfigInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigAddDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigGetDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorTargetConfigUpdateDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.vos.DoctorTargetConfigVO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.helper.DoctorPatientHelper;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorTargetConfigService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorTargetConfigServiceImpl implements DoctorTargetConfigService {

    @Autowired
    private DoctorTargetConfigInfoService doctorTargetConfigInfoService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DeptClientService deptClientService;

    /**
     * code、dept_id、disease_id确定数据唯一
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorTargetConfigVO add(DoctorTargetConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);

        // 判断数据是否存在
        DoctorTargetConfigInfo entity = doctorTargetConfigInfoService.getOne(new QueryWrapper<DoctorTargetConfigInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("code", params.getCode())
            .eq("dept_id", params.getDeptId())
            .eq("disease_id", params.getDiseaseId())
        );
        if (null == entity) {
            entity = new DoctorTargetConfigInfo();
            entity.setId(DoctorPatientHelper.getDoctorTargetConfigInfo());
            entity.setCode(params.getCode());
            entity.setType(params.getType());
            entity.setName(params.getName());
            entity.setRemark(params.getRemark());
            entity.setDeptId(params.getDeptId());
            entity.setDiseaseId(params.getDiseaseId());
            entity.setExeMaxDay(params.getExeMaxDay());
            List<String> exeDeptIds = params.getExeDeptIds();
            if (CollectionUtil.isNotEmpty(exeDeptIds)) {
                entity.setExeDeptIds(StrJoinSplitUtils.joinPrefix(exeDeptIds));
            }
            entity.setRowsStatus(YN.YES.getCode());

            doctorTargetConfigInfoService.save(entity);
        }

        DoctorTargetConfigVO vo = DoctorPatientHelper.getDoctorTargetConfigVO(entity);

        List<Map<String, String>> deptInfoList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(params.getExeDeptIds())) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(params.getExeDeptIds());
            deptList.forEach(i -> {
                Map<String, String> deptMap = new HashMap<>();
                deptMap.put("deptId", i.getId());
                deptMap.put("deptName", i.getName());

                deptInfoList.add(deptMap);
            });
        }

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void update(DoctorTargetConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorTargetConfigInfo entity = doctorTargetConfigInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            throw new BusinessException("数据不存在");
        }

        // 若更新code、deptId、diseaseId则需要判断是否有数据存在，没有才能更新
        if (StrUtil.isNotBlank(params.getCode())) {
            entity.setCode(params.getCode());
        }
        if (StrUtil.isNotBlank(params.getType())) {
            entity.setType(params.getType());
        }
        if (StrUtil.isNotBlank(params.getDeptId())) {
            entity.setDeptId(params.getDeptId());
        }
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            entity.setDiseaseId(params.getDiseaseId());
        }
        DoctorTargetConfigInfo exist = doctorTargetConfigInfoService.getOne(new QueryWrapper<DoctorTargetConfigInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("code", params.getCode())
            .eq("dept_id", params.getDeptId())
            .eq("disease_id", params.getDiseaseId())
        );
        if (null != exist && !exist.getId().equals(entity.getId())) {
            throw new BusinessException("数据已存在：[" + exist.getName() + "]");
        }

        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setExeMaxDay(params.getExeMaxDay());
        List<String> exeDeptIds = params.getExeDeptIds();
        if (CollectionUtil.isNotEmpty(exeDeptIds)) {
            entity.setExeDeptIds(StrJoinSplitUtils.joinPrefix(exeDeptIds));
        }
        doctorTargetConfigInfoService.updateById(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(DoctorTargetConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorTargetConfigInfo entity = doctorTargetConfigInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return;
        }
        entity.setRowsStatus(YN.NO.getCode());
        doctorTargetConfigInfoService.updateById(entity);
    }

    @Override
    public PageData<DoctorTargetConfigVO> query(DoctorTargetConfigQueryDTO params) {
        IPage<DoctorTargetConfigInfo> page = doctorTargetConfigInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorTargetConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getCode()), "code", params.getCode())
                .like(StrUtil.isNotBlank(params.getName()), "name", params.getName())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }
        
        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        List<String> diseaseIds = new ArrayList<>();
        List<String> deptIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            diseaseIds.add(i.getDiseaseId());
            deptIds.add(i.getDeptId());
            if (StrUtil.isNotBlank(i.getExeDeptIds())) {
                deptIds.addAll(StrJoinSplitUtils.split2List(i.getExeDeptIds()));
            }
        });
        if (CollectionUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
        }
        // 查询部门信息
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        PageData<DoctorTargetConfigVO> data =  new PageConverter<DoctorTargetConfigInfo, DoctorTargetConfigVO>() {
            @Override
            public DoctorTargetConfigVO converter(DoctorTargetConfigInfo entity) {
                DoctorTargetConfigVO vo = DoctorPatientHelper.getDoctorTargetConfigVO(entity);
                
                StandardDiseaseBO disease = diseaseMap.get(vo.getDiseaseId());
                if (null != disease) {
                    vo.setDiseaseName(disease.getName());
                }

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                List<Map<String, String>> deptInfoList = new ArrayList<>();
                if (CollectionUtil.isNotEmpty(vo.getExeDeptIdList())) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(vo.getExeDeptIdList());
                    deptList.forEach(i -> {
                        Map<String, String> deptMap = new HashMap<>();
                        deptMap.put("deptId", i.getId());
                        deptMap.put("deptName", i.getName());
        
                        deptInfoList.add(deptMap);
                    });
                }
                vo.setExeDeptList(deptInfoList);
        
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorTargetConfigVO> list(DoctorTargetConfigQueryDTO params) {
        List<DoctorTargetConfigInfo> list = doctorTargetConfigInfoService.list(new QueryWrapper<DoctorTargetConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getCode()), "code", params.getCode())
                .like(StrUtil.isNotBlank(params.getName()), "name", params.getName())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .orderByDesc("update_date")
                .last(" limit 1000 ") // 限制查询数量
        );
        return list.stream().map(DoctorPatientHelper::getDoctorTargetConfigVO).collect(Collectors.toList());
    }

    @Override
    public DoctorTargetConfigVO detail(DoctorTargetConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorTargetConfigInfo entity = doctorTargetConfigInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return null;
        }

        DoctorTargetConfigVO vo = DoctorPatientHelper.getDoctorTargetConfigVO(entity);
        
        // 病种信息
        if (StrUtil.isNotBlank(entity.getDiseaseId())) {
            StandardDiseaseBO disease = standardDiseaseClientService.getById(entity.getDiseaseId());
            if (null != disease) {
                vo.setDiseaseName(disease.getName());
            }
        }
        // 部门信息
        if (StrUtil.isNotBlank(entity.getDeptId())) {
            DeptBO dept = deptClientService.getDeptById(entity.getDeptId());
            if (null != dept) {
                vo.setDeptName(dept.getName());
            }
        }

        List<Map<String, String>> deptInfoList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(vo.getExeDeptIdList())) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(vo.getExeDeptIdList());
            deptList.forEach(i -> {
                Map<String, String> deptMap = new HashMap<>();
                deptMap.put("deptId", i.getId());
                deptMap.put("deptName", i.getName());

                deptInfoList.add(deptMap);
            });
        }
        vo.setExeDeptList(deptInfoList);

        return vo;
    }
}
