package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.task.schedule;

import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.task.service.DoctorPatientStatusMarkTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class DoctorPatientStatusMarkTask {

    @Autowired
    private DoctorPatientStatusMarkTaskService doctorPatientStatusMarkTaskService;

    /**
     * 更新个案目标状态
     * 同步更新个案对应历史表中的状态
     */
    @XxlJob("updateTargetStatusJobHandler")
    public void updateTargetStatusJobHandler() throws Exception {
        log.info("更新个案目标状态开始 >>>>>> ");
        XxlJobHelper.log("更新个案目标状态开始 >>>>>> ");

        doctorPatientStatusMarkTaskService.updateTargetStatus();

        XxlJobHelper.log("更新个案目标状态结束 <<<<<< ");
        log.info("更新个案目标状态结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }

    /**
     * 更新个案历史目标状态
     * 由于某种情况（正在进行中的目标更改成其他目标那该目标无法完成或超时的情况）目标状态无法更新
     */
    @XxlJob("updateHisTargetStatusJobHandler")
    public void updateHisTargetStatusJobHandler() throws Exception {
        log.info("更新个案历史目标状态开始 >>>>>> ");
        XxlJobHelper.log("更新个案历史目标状态开始 >>>>>> ");

        doctorPatientStatusMarkTaskService.updateHisTargetStatus();

        XxlJobHelper.log("更新个案历史目标状态结束 <<<<<< ");
        log.info("更新个案历史目标状态结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }

}
