package com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorTargetConstant;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientStatusMarkHisInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.entity.DoctorPatientStatusMarkInfo;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientStatusMarkHisInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.base.service.DoctorPatientStatusMarkInfoService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DoctorPatientStatusMarkUpdateTargetExeStatusDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.bean.dto.DwdDoctorComeQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkHisService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DoctorPatientStatusMarkService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.service.DwdDoctorDataBizService;
import com.xxh.cloud.healthy.doctor.crm.framework.modules.biz.task.service.DoctorPatientStatusMarkTaskService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorPatientStatusMarkTaskServiceImpl implements DoctorPatientStatusMarkTaskService {
    
    @Autowired
    private DoctorPatientStatusMarkInfoService doctorPatientStatusMarkInfoService;
    @Autowired
    private DoctorPatientStatusMarkService doctorPatientStatusMarkService;
    @Autowired
    private DwdDoctorDataBizService dwdDoctorDataBizService;
    @Autowired
    private DoctorPatientStatusMarkHisInfoService doctorPatientStatusMarkHisInfoService;
    @Autowired
    private DoctorPatientStatusMarkHisService doctorPatientStatusMarkHisService;
    
    /**
     * 更新个案目标状态
     */
    @Override
    public void updateTargetStatus() {
        // 循环批处理个案信息
        String id = "0";
        while (true) {
            if (id == null) {
                return;
            }
            id = queryAndDealTargetStatus(id);
        }
    }

    // 分页查询个案信息，并循环个案信息处理目标状态
    private String queryAndDealTargetStatus(String id) {
        Date now = DateUtil.date(); // 当前时间

        // 分页查询
        List<DoctorPatientStatusMarkInfo> list = doctorPatientStatusMarkInfoService.list(new QueryWrapper<DoctorPatientStatusMarkInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("status", DoctorStatusMarkConstant.Status.Normal.getCode())
                .eq("target_exe_status", DoctorStatusMarkConstant.TargetExeStatus.DOING.getCode())
                .gt("id", id)
                .orderByAsc("id")
                .last(" limit 1000 ")
        );
        if (CollectionUtil.isEmpty(list)) {
            return null;
        }

        // 循环处理
        List<DoctorPatientStatusMarkUpdateTargetExeStatusDTO> targetList = new ArrayList<>(); // 需要更新目标状态的数据
        list.forEach(i -> {
            Integer maxDay = i.getTargetExeMaxDay();

            // 旧数据该字段没有值，暂不处理该字段
            if (null == maxDay) {
                return;
            }

            Date startDate = i.getAddTime(); // 目标开始时间
            Date endDate = DateUtil.offsetDay(startDate, maxDay); // 目标结束时间

            String targetExeStatus = this.targetExeStatus(i.getTargetCode(), startDate, endDate, StrJoinSplitUtils.split2List((i.getTargetExeDeptIds())));

            if (null != targetExeStatus) {
                DoctorPatientStatusMarkUpdateTargetExeStatusDTO updateTargetExeStatusDTO = new DoctorPatientStatusMarkUpdateTargetExeStatusDTO();
                updateTargetExeStatusDTO.setId(i.getId());
                updateTargetExeStatusDTO.setOpTime(now);
                updateTargetExeStatusDTO.setTargetExeStatus(targetExeStatus);

                targetList.add(updateTargetExeStatusDTO);
            }
        });

        if (CollectionUtil.isNotEmpty(targetList)) {
            // 更新当前目标状态，并同步更新历史表
            doctorPatientStatusMarkService.updateTargetExeStatusBatch(targetList);
        }

        return list.get(list.size() - 1).getId();
    }

    private String targetExeStatus(String targetCode, Date startDate, Date endDate, List<String> targetExeDeptIds) {
        String tartgetExeStatus = null;
        
        Date now = DateUtil.date(); // 当前时间

        // 查询dwd确定更新的目标状态
        Integer count = 0;

        DwdDoctorComeQueryDTO dwdDoctorComeQueryDTO = new DwdDoctorComeQueryDTO();
        dwdDoctorComeQueryDTO.setStartDate(startDate);
        dwdDoctorComeQueryDTO.setEndDate(endDate);
        dwdDoctorComeQueryDTO.setDeptIds(targetExeDeptIds);

        // 挂号
        if (DoctorTargetConstant.Target.TARGET_REG.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.registerCount(dwdDoctorComeQueryDTO);
        }
        // 就诊
        else if (DoctorTargetConstant.Target.TARGET_VISIT.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.visitCount(dwdDoctorComeQueryDTO);
        }
        // 住院
        else if (DoctorTargetConstant.Target.TARGET_IN.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.inHospitalCount(dwdDoctorComeQueryDTO);
        }
        // 出院
        else if (DoctorTargetConstant.Target.TARGET_OUT.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.outHospitalCount(dwdDoctorComeQueryDTO);
        }
        // 手术
        else if (DoctorTargetConstant.Target.TARGET_OPERATION.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.operationCount(dwdDoctorComeQueryDTO);
        }
        // 检查
        else if (DoctorTargetConstant.Target.TARGET_EXAMS.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.examineCount(dwdDoctorComeQueryDTO);
        }
        // 检验
        else if (DoctorTargetConstant.Target.TARGET_LABS.getCode().equals(targetCode)) {
            count = dwdDoctorDataBizService.laboratoryCount(dwdDoctorComeQueryDTO);
        }

        // 只要有数据则更新状态
        if (count > 0) {
            tartgetExeStatus = DoctorStatusMarkConstant.TargetExeStatus.DONE.getCode();
        } else if (now.compareTo(endDate) > 0) { // 没有数据则判断是否过期
            // 若当前时间大于目标结束时间则更新目标状态
            tartgetExeStatus = DoctorStatusMarkConstant.TargetExeStatus.EXPIRE.getCode();
        }

        return tartgetExeStatus;
    }

    @Override
    public void updateHisTargetStatus() {
        // 循环批处理个案信息
        String id = "0";
        while (true) {
            if (id == null) {
                return;
            }
            id = queryAndDealHisTargetStatus(id);
        }
    }

    // 分页查询个案信息，并循环个案信息处理目标状态
    private String queryAndDealHisTargetStatus(String id) {
        Date now = DateUtil.date(); // 当前时间

        // 分页查询
        List<DoctorPatientStatusMarkHisInfo> list = doctorPatientStatusMarkHisInfoService.list(new QueryWrapper<DoctorPatientStatusMarkHisInfo>()
                .eq("status", DoctorStatusMarkConstant.Status.Normal.getCode())
                .eq("target_exe_status", DoctorStatusMarkConstant.TargetExeStatus.DOING.getCode())
                .gt("id", id)
                .orderByAsc("id")
                .last(" limit 1000 ")
        );
        if (CollectionUtil.isEmpty(list)) {
            return null;
        }

        // 循环处理
        List<DoctorPatientStatusMarkUpdateTargetExeStatusDTO> targetList = new ArrayList<>(); // 需要更新目标状态的数据
        list.forEach(i -> {
            Integer maxDay = i.getTargetExeMaxDay();

            // 旧数据该字段没有值，暂不处理该字段
            if (null == maxDay) {
                return;
            }

            Date startDate = i.getAddTime(); // 目标开始时间
            Date endDate = DateUtil.offsetDay(startDate, maxDay); // 目标结束时间

            String targetExeStatus = this.targetExeStatus(i.getTargetCode(), startDate, endDate, StrJoinSplitUtils.split2List((i.getTargetExeDeptIds())));

            if (null != targetExeStatus) {
                DoctorPatientStatusMarkUpdateTargetExeStatusDTO updateTargetExeStatusDTO = new DoctorPatientStatusMarkUpdateTargetExeStatusDTO();
                updateTargetExeStatusDTO.setId(i.getId());
                updateTargetExeStatusDTO.setOpTime(now);
                updateTargetExeStatusDTO.setTargetExeStatus(targetExeStatus);

                targetList.add(updateTargetExeStatusDTO);
            }
        });

        if (CollectionUtil.isNotEmpty(targetList)) {
            // 更新历史表中的目标的状态
            doctorPatientStatusMarkHisService.updateTargetExeStatusBatch(targetList);
        }

        return list.get(list.size() - 1).getId();
    }

}
