package com.xxh.cloud.healthy.basic.framework.modules.article.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.article.client.api.bean.bos.ArticleCategoryBO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleVO;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.ArticleCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.ArticleInfo;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ArticleCategoryConvert {

    public static ArticleCategoryVO getArticleCategoryVO(ArticleCategoryInfo entity){
        ArticleCategoryVO data=new ArticleCategoryVO();
        BeanUtil.copyProperties(entity,data);
        return data;
    }



    public static ArticleVO getArticleVOList(ArticleInfo entity){
        ArticleVO data=new ArticleVO();
        BeanUtil.copyProperties(entity,data,"articleCategoryIds","deptIds",
                "diseaseIds","labelIds","content","articleCategoryList","deptList","labelList","diseaseList");
        if(StrUtil.isNotEmpty(entity.getArticleCategoryIds())){
            data.setArticleCategoryIds(StrJoinSplitUtils.split2List(entity.getArticleCategoryIds()));
        }
        if(StrUtil.isNotEmpty(entity.getDeptIds())){
            data.setDeptIds(StrJoinSplitUtils.split2List(entity.getDeptIds()));
        }
        if(StrUtil.isNotEmpty(entity.getLabelIds())){
            data.setLabelIds(StrJoinSplitUtils.split2List(entity.getLabelIds()));
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseIds())){
            data.setDiseaseIds(StrJoinSplitUtils.split2List(entity.getDiseaseIds()));
        }
        return data;
    }

    public static ArticleVO getArticleVOList(ArticleInfo entity,
                                             Map<String, ArticleCategoryBO> categoryMap,
                                             Map<String, DeptBO> deptMap,
                                             Map<String, LabelBO> labelMap,
                                             Map<String, AdminUserBO> userMap,
                                             Map<String, StandardDiseaseBO> diseaseMap){
        ArticleVO data=getArticleVOList(entity);

        List<String> categoryNameList=new ArrayList<>();
        getArticleVOList_Category(data,categoryMap,categoryNameList);

        List<String> deptNameList=new ArrayList<>();
        getArticleVOList_Dept(data,deptMap,deptNameList);

        List<String> labelNameList=new ArrayList<>();
        getArticleVOList_Label(data,labelMap,labelNameList);

        getArticleVOList_User(data,userMap);

        data.setArticleCategoryNames(CollUtil.join(categoryNameList,","));
        data.setDeptNames(CollUtil.join(deptNameList,","));
        data.setLabelNames(CollUtil.join(labelNameList,","));

        return data;
    }


    public static ArticleVO getArticleVOFull(ArticleInfo entity){
        ArticleVO data=getArticleVOList(entity);
        data.setContent(entity.getContent());
        return data;
    }

    public static ArticleVO getArticleVOFull(ArticleInfo entity,
                                         Map<String, ArticleCategoryBO> categoryMap,
                                         Map<String, DeptBO> deptMap,
                                         Map<String, LabelBO> labelMap,
                                         Map<String, AdminUserBO> userMap,
                                         Map<String, StandardDiseaseBO> diseaseMap){
        ArticleVO data=getArticleVOFull(entity);

        List<String> categoryNameList=new ArrayList<>();
        getArticleVOList_Category(data,categoryMap,categoryNameList);

        List<String> deptNameList=new ArrayList<>();
        getArticleVOList_Dept(data,deptMap,deptNameList);

        List<String> labelNameList=new ArrayList<>();
        getArticleVOList_Label(data,labelMap,labelNameList);

        List<String> diseaseNameList=new ArrayList<>();
        getArticleVOList_Disease(data,diseaseMap,diseaseNameList);

        getArticleVOList_User(data,userMap);

        data.setArticleCategoryNames(CollUtil.join(categoryNameList,","));
        data.setDeptNames(CollUtil.join(deptNameList,","));
        data.setLabelNames(CollUtil.join(labelNameList,","));
        data.setDiseaseNames(StrJoinSplitUtils.join(diseaseNameList));


        return data;
    }



    private static void getArticleVOList_Category(ArticleVO data,Map<String, ArticleCategoryBO> categoryMap,List<String> nameList){
         if(data==null || CollUtil.isEmpty(data.getArticleCategoryIds()) || MapUtil.isEmpty(categoryMap)){
             return;
         }
         data.getArticleCategoryIds().forEach(id->{
             ArticleCategoryBO articleCategory=categoryMap.get(id);
             data.addArticleCategory(articleCategory);
             if(nameList!=null && articleCategory!=null && StrUtil.isNotEmpty(articleCategory.getName())){
                 nameList.add(articleCategory.getName());
             }

         });
    }

    private static void getArticleVOList_Dept(ArticleVO data,Map<String, DeptBO> deptMap,List<String> nameList){
        if(data==null || CollUtil.isEmpty(data.getDeptIds()) || MapUtil.isEmpty(deptMap)){
            return;
        }
        data.getDeptIds().forEach(id->{
            DeptBO dept = deptMap.get(id);
            data.addDept(dept);
            if(nameList!=null && dept!=null && StrUtil.isNotEmpty(dept.getName())){
                nameList.add(dept.getName());
            }
        });
    }

    private static void getArticleVOList_Label(ArticleVO data,Map<String, LabelBO> labelMap,List<String> nameList){
        if(data==null || CollUtil.isEmpty(data.getLabelIds()) || MapUtil.isEmpty(labelMap)){
            return;
        }
        data.getLabelIds().forEach(id->{
            LabelBO label=labelMap.get(id);
            data.addLabel(label);
            if(nameList!=null && label!=null && StrUtil.isNotEmpty(label.getName())){
                nameList.add(label.getName());
            }
        });
    }

    private static void getArticleVOList_User(ArticleVO data, Map<String, AdminUserBO> userMap){
        if(data==null || MapUtil.isEmpty(userMap)){
            return;
        }
        if(StrUtil.isNotEmpty(data.getCreator())){
            AdminUserBO user=userMap.get(data.getCreator());
            data.setCreatorName(user==null?null:user.getUserName());
        }
        if(StrUtil.isNotEmpty(data.getUpdater())){
            AdminUserBO user=userMap.get(data.getUpdater());
            data.setUpdaterName(user==null?null:user.getUserName());
        }
    }

    private static void getArticleVOList_Disease(ArticleVO data,  Map<String, StandardDiseaseBO> diseaseMap,List<String> nameList){
        if(data==null || CollUtil.isEmpty(data.getDiseaseIds()) || MapUtil.isEmpty(diseaseMap)){
            return;
        }
        data.getDiseaseIds().forEach(id->{
            StandardDiseaseBO disease=diseaseMap.get(id);
            if(disease!=null){
                data.addDisease(disease);
                if(nameList!=null  && StrUtil.isNotEmpty(disease.getName())){
                    nameList.add(disease.getName());
                }
            }
        });

    }





}
