package com.xxh.cloud.healthy.basic.framework.modules.article.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;

import com.xxh.cloud.healthy.basic.article.client.api.ArticleCategoryClientService;
import com.xxh.cloud.healthy.basic.article.client.api.bean.bos.ArticleCategoryBO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleInfoQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleInfoSaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleInfoUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleInfoUpdateStatusDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleVO;

import com.xxh.cloud.healthy.basic.framework.modules.article.helper.ArticleCategoryConvert;
import com.xxh.cloud.healthy.basic.framework.modules.article.service.ArticleBizService;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.ArticleInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.ArticleInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.ArticleCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.constant.ArticleStatusEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@Service
public class ArticleBizServiceImpl implements ArticleBizService {

    @Autowired
    private ArticleInfoService articleInfoService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private ArticleCategoryClientService articleCategoryClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private ArticleCacheService articleCacheService;


    @Override
    public PageData<ArticleVO> query(ArticleInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<ArticleInfo> page = articleInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ArticleInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .like(StrUtil.isNotEmpty(params.getArticleCategoryId()),"article_category_ids",params.getArticleCategoryId())
                        .and(StrUtil.isNotEmpty(params.getKeywords()),
                                i->i.or().like("title",params.getKeywords())
                                        .or().like("messasge_conent",params.getKeywords())
                                        .or().like("summary",params.getKeywords())
                        )
                        .like(StrUtil.isNotEmpty(params.getAuthor()),"author",params.getAuthor())
                        .like(StrUtil.isNotEmpty(params.getSource()),"source",params.getSource())
                        .eq(StrUtil.isNotEmpty(params.getArticleBizType()),"article_biz_type",params.getArticleBizType())
                        .like(StrUtil.isNotEmpty(params.getDeptId()),"dept_ids",params.getDeptId())
                        .like(StrUtil.isNotEmpty(params.getLabelId()),"label_ids",params.getLabelId())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .orderByDesc("update_date")
        );


        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> articleCategoryIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> labelIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getArticleCategoryIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getArticleCategoryIds());
                articleCategoryIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getDeptIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getDeptIds());
                deptIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getLabelIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getLabelIds());
                labelIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getDiseaseIds());
                diseaseIds.addAll(list);
            }
        });


        Map<String, ArticleCategoryBO> categoryMap= articleCategoryClientService.getMapByIds(articleCategoryIds);
        Map<String, DeptBO> deptMap= deptClientService.getDeptMapByIds(deptIds);
        Map<String, LabelBO> labelMap= labelClientService.getLabelMapByIds(labelIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);

        PageData<ArticleVO> data= new PageConverter<ArticleInfo, ArticleVO>() {
            @Override
            public ArticleVO converter(ArticleInfo entity) {
                return ArticleCategoryConvert.getArticleVOList(entity,categoryMap,deptMap,labelMap,userMap,diseaseMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public ArticleVO save(ArticleInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleInfo entity=new ArticleInfo();
        entity.setId(IdUtil.getSnowflake(3,1).nextIdStr());
        entity.setTitle(params.getTitle());
        entity.setMessasgeConent(params.getMessasgeConent());
        entity.setSummary(params.getSummary());
        entity.setThumbnail(params.getThumbnail());
        entity.setArticleCategoryIds(StrJoinSplitUtils.join(params.getArticleCategoryIds()));
        entity.setDeptIds(StrJoinSplitUtils.join(params.getDeptIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));

        entity.setArticleBizType(params.getArticleBizType());
        entity.setAuthor(params.getAuthor());
        entity.setSource(params.getSource());
        entity.setContent(params.getContent());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        if(StrUtil.isNotEmpty(params.getStatus())){
            entity.setStatus(params.getStatus());
        }else {
            entity.setStatus(ArticleStatusEnum.ArticleStatus_NONE.getCode());
        }
        if(StrUtil.isNotEmpty(params.getCommentStatus())){
            entity.setCommentStatus(params.getCommentStatus());
        }else {
            entity.setCommentStatus(YN.NO.getCode());
        }
        entity.setDraftStatus(YN.NO.getCode());
        entity.setRelIdType(params.getRelIdType());
        entity.setRelId(params.getRelId());
        entity.setRowsStatus(YN.YES.getCode());
        articleInfoService.save(entity);
        return ArticleCategoryConvert.getArticleVOList(entity);
    }

    @Override
    public ArticleVO update(ArticleInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleInfo entity=articleInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setTitle(params.getTitle());
        entity.setMessasgeConent(params.getMessasgeConent());
        entity.setSummary(params.getSummary());
        entity.setThumbnail(params.getThumbnail());
        entity.setArticleCategoryIds(StrJoinSplitUtils.join(params.getArticleCategoryIds()));
        entity.setDeptIds(StrJoinSplitUtils.join(params.getDeptIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));

        entity.setArticleBizType(params.getArticleBizType());
        entity.setAuthor(params.getAuthor());
        entity.setSource(params.getSource());
        entity.setContent(params.getContent());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        entity.setStatus(ArticleStatusEnum.ArticleStatus_NONE.getCode());
        entity.setCommentStatus(params.getCommentStatus());
        entity.setRelIdType(params.getRelIdType());
        entity.setRelId(params.getRelId());
        articleInfoService.updateById(entity);
        articleCacheService.removeById(entity.getId());
        return ArticleCategoryConvert.getArticleVOList(entity);
    }

    @Override
    public void updateStatus(ArticleInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleInfo entity=articleInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return;
        }
        entity.setStatus(params.getStatus());
        articleInfoService.updateById(entity);
        articleCacheService.removeById(entity.getId());
    }

    @Override
    public ArticleVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ArticleInfo entity=articleInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> userIds=new HashSet<>();
        Set<String> articleCategoryIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> labelIds=new HashSet<>();


        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getArticleCategoryIds())){
            List<String> list= StrJoinSplitUtils.split2List(entity.getArticleCategoryIds());
            articleCategoryIds.addAll(list);
        }
        if(StrUtil.isNotEmpty(entity.getDeptIds())){
            List<String> list= StrJoinSplitUtils.split2List(entity.getDeptIds());
            deptIds.addAll(list);
        }
        if(StrUtil.isNotEmpty(entity.getLabelIds())){
            List<String> list= StrJoinSplitUtils.split2List(entity.getLabelIds());
            labelIds.addAll(list);
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseIds())){
            List<String> list= StrJoinSplitUtils.split2List(entity.getDiseaseIds());
            diseaseIds.addAll(list);
        }
        Map<String, ArticleCategoryBO> categoryMap= articleCategoryClientService.getMapByIds(articleCategoryIds);
        Map<String, DeptBO> deptMap= deptClientService.getDeptMapByIds(deptIds);
        System.out.println("+++++++++++++++++++++++++++++++++++ 1");
        System.out.println(JSON.toJSONString(labelIds));
        Map<String, LabelBO> labelMap= labelClientService.getLabelMapByIds(labelIds);
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        return ArticleCategoryConvert.getArticleVOFull(entity,categoryMap,deptMap,labelMap,userMap,diseaseMap);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ArticleInfo entity=articleInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        articleInfoService.updateById(entity);
        articleCacheService.removeById(entity.getId());

    }

    @Override
    public List<ArticleVO> listByIdsAndOrderByQueryIds(List<String> ids) {
        log.info("1.根据文章id列表查询跟请求id列表顺序一致的文章列表>>>>>> ids={}", JSON.toJSONString(ids));
        if (CollUtil.isEmpty(ids)) {
            return new ArrayList<>(0);
        }
        List<ArticleInfo> articleList = articleInfoService.listByIds(ids);
        log.info("2.查询文章列表", JSON.toJSONString(articleList));
        if(CollUtil.isEmpty(articleList)){
            return new ArrayList<>(0);
        }
    
        Set<String> userIds=new HashSet<>();
        Set<String> articleCategoryIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> labelIds=new HashSet<>();

        articleList.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getArticleCategoryIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getArticleCategoryIds());
                articleCategoryIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getDeptIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getDeptIds());
                deptIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getLabelIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getLabelIds());
                labelIds.addAll(list);
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                List<String> list= StrJoinSplitUtils.split2List(item.getDiseaseIds());
                diseaseIds.addAll(list);
            }
        });


        Map<String, ArticleCategoryBO> categoryMap= articleCategoryClientService.getMapByIds(articleCategoryIds);
        Map<String, DeptBO> deptMap= deptClientService.getDeptMapByIds(deptIds);
        Map<String, LabelBO> labelMap= labelClientService.getLabelMapByIds(labelIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);


        Map<String, ArticleInfo> articleMap = new HashMap<>();
        articleList.forEach(i -> articleMap.put(i.getId(), i));

        List<ArticleVO> resultList = new ArrayList<>();
        ids.forEach(i -> {
            ArticleInfo entity = articleMap.get(i);
            if (null != articleMap.get(i)) {
                ArticleVO vo = ArticleCategoryConvert.getArticleVOList(entity,categoryMap,deptMap,labelMap,userMap,diseaseMap);
                resultList.add(vo);
            }
        });
        log.info("3.文章列表排序", JSON.toJSONString(resultList));

        return resultList;
    }
}
