package com.xxh.cloud.healthy.basic.framework.modules.article.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.helper.ArticleCategoryConvert;
import com.xxh.cloud.healthy.basic.framework.modules.article.service.ArticleCategoryBizService;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.ArticleCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.ArticleCategoryInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.ArticleCategoryCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class ArticleCategoryBizServiceImpl implements ArticleCategoryBizService {

    @Autowired
    private ArticleCategoryInfoService articleCategoryInfoService;
    @Autowired
    private ArticleCategoryCacheService articleCategoryCacheService;



    @Override
    public List<ArticleCategoryVO> tree(ArticleCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ArticleCategoryInfo> list= articleCategoryInfoService.list(
                new QueryWrapper<ArticleCategoryInfo>()
                        .eq("rows_status",YN.YES.getCode())
        );
        List<ArticleCategoryVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(ArticleCategoryConvert.getArticleCategoryVO(item));
        });
        List<ArticleCategoryVO> data= TreeUtils.build(dataList);
        return data;
    }

    @Override
    public ArticleCategoryVO save(ArticleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleCategoryInfo entity=new ArticleCategoryInfo();
        entity.setId(IdUtil.getSnowflake(8,1).nextIdStr());
        if(StrUtil.isEmpty(params.getPid())){
            entity.setPid(Constant.RootParentNodeId);
        }else {
            entity.setPid(params.getPid());
        }
        entity.setName(params.getName());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        articleCategoryInfoService.save(entity);
        return ArticleCategoryConvert.getArticleCategoryVO(entity);
    }

    @Override
    public ArticleCategoryVO update(ArticleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleCategoryInfo entity=articleCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        articleCategoryInfoService.updateById(entity);
        articleCategoryCacheService.removeById(entity.getId());
        return ArticleCategoryConvert.getArticleCategoryVO(entity);
    }

    @Override
    public ArticleCategoryVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ArticleCategoryInfo entity=articleCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ArticleCategoryConvert.getArticleCategoryVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        long num= articleCategoryInfoService.count(
                new QueryWrapper<ArticleCategoryInfo>()
                        .eq("pid",id)
                        .eq("rows_status",YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"有关联的子分类");
        }
        ArticleCategoryInfo entity=articleCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        articleCategoryInfoService.updateById(entity);
        articleCategoryCacheService.removeById(entity.getId());
    }

}
