package com.xxh.cloud.healthy.basic.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.BasicCacheServiceImpl;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.helper.ArticleCategoryConvert;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.ArticleCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.ArticleCategoryInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.ArticleCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.ArticleCategoryCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
public class ArticleCategoryCacheServiceImpl extends BasicCacheServiceImpl<ArticleCategoryVO> implements ArticleCategoryCacheService {

    @Autowired
    private ArticleCategoryInfoService articleCategoryInfoService;

    @Override
    public String getCacheKey(String id) {
        return ArticleCacheKeysHelper.getArticleCategoryKey(id);
    }

    @Override
    public String getCacheKeyPattern() {
        return ArticleCacheKeysHelper.getArticleCategoryMainKey();
    }

    @Override
    public Class<ArticleCategoryVO> getJsonClazz() {
        return ArticleCategoryVO.class;
    }

    @Override
    public String getId(ArticleCategoryVO entity) {
        return entity.getId();
    }

    @Override
    public ArticleCategoryVO getById2DB(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        ArticleCategoryInfo entity=articleCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ArticleCategoryConvert.getArticleCategoryVO(entity);
    }

    @Override
    public List<ArticleCategoryVO> getListByIds2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<ArticleCategoryInfo> list= articleCategoryInfoService.list(
                new QueryWrapper<ArticleCategoryInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .in("id",ids)
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<ArticleCategoryVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(ArticleCategoryConvert.getArticleCategoryVO(item));
        });
        return data;
    }
}
