package com.xxh.cloud.healthy.basic.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.BasicCacheServiceImpl;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.HealthyServiceCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.HealthyServiceTargetCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dos.HealthyServiceInfo;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceTargetVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.helper.HealthyServiceHelper;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.service.HealthyServiceInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class HealthyServiceCacheServiceImpl  extends BasicCacheServiceImpl<HealthyServiceVO> implements HealthyServiceCacheService {

    @Autowired
    private HealthyServiceInfoService healthyServiceInfoService;
    @Autowired
    private HealthyServiceTargetCacheService healthyServiceTargetCacheService;
    @Autowired
    private AdminUserClientService adminUserClientService;

//    @Autowired
//    private DiseaseClientService diseaseClientService;


    @Override
    public String getCacheKey(String id) {
        return HealthyBasicCacheKeysHelper.getHealthyServiceKey(id);
    }

    @Override
    public String getCacheKeyPattern() {
        return HealthyBasicCacheKeysHelper.getHealthyServiceMainKey();
    }

    @Override
    public Class<HealthyServiceVO> getJsonClazz() {
        return HealthyServiceVO.class;
    }

    @Override
    public String getId(HealthyServiceVO entity) {
        return entity.getId();
    }


    @Override
    public HealthyServiceVO getById2DB(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        HealthyServiceInfo entity= healthyServiceInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        Set<String> serviceTargetIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getServiceTargetId())){
            serviceTargetIds.add(entity.getServiceTargetId());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, HealthyServiceTargetVO> serviceGroupMap= healthyServiceTargetCacheService.getMapByIds(serviceTargetIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        return HealthyServiceHelper.getHealthyServiceVO(entity,serviceGroupMap,userMap);
    }



    @Override
    public List<HealthyServiceVO> getListByIds2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<HealthyServiceInfo> list = healthyServiceInfoService.list(
                new QueryWrapper<HealthyServiceInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .in("id",ids)
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> serviceTargetIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getServiceTargetId())){
                serviceTargetIds.add(item.getServiceTargetId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, HealthyServiceTargetVO> serviceGroupMap= healthyServiceTargetCacheService.getMapByIds(serviceTargetIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        List<HealthyServiceVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(HealthyServiceHelper.getHealthyServiceVO(item,serviceGroupMap,userMap));
        });
        return data;
    }


}
