package com.xxh.cloud.healthy.basic.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.BasicCacheServiceImpl;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.HealthyServiceTargetCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dos.HealthyServiceTargetInfo;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceTargetVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.helper.HealthyServiceTargetHelper;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.service.HealthyServiceTargetInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class HealthyServiceTargetCacheServiceImpl  extends BasicCacheServiceImpl<HealthyServiceTargetVO>  implements HealthyServiceTargetCacheService {

    @Autowired
    private HealthyServiceTargetInfoService healthyServiceTargetInfoService;

    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public String getCacheKey(String id) {
        return HealthyBasicCacheKeysHelper.getHealthyServiceTargetKey(id);
    }

    @Override
    public String getCacheKeyPattern() {
        return HealthyBasicCacheKeysHelper.getHealthyServiceTargetMainKey();
    }

    @Override
    public Class<HealthyServiceTargetVO> getJsonClazz() {
        return HealthyServiceTargetVO.class;
    }

    @Override
    public String getId(HealthyServiceTargetVO entity) {
        return entity.getId();
    }




    @Override
    public HealthyServiceTargetVO getById2DB(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }



        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        HealthyServiceTargetVO data= HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity,userMap);
        return data;
    }

    @Override
    public List<HealthyServiceTargetVO> getListByIds2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<HealthyServiceTargetVO> data=new ArrayList<>();
        List<HealthyServiceTargetInfo> list =healthyServiceTargetInfoService.list(new QueryWrapper<HealthyServiceTargetInfo>()
                .eq("rows_status", YN.YES.getCode())
                .in("id",ids)
                .orderByDesc("orders")
        );
        if(CollUtil.isEmpty(list)){
            return data;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        list.forEach(item->{
            data.add(HealthyServiceTargetHelper.getHealthyServiceTargetVO(item,userMap));
        });

        return data;

    }




}
