package com.xxh.cloud.healthy.basic.framework.modules.disease.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.disease.base.entity.EntityDiseaseConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.disease.base.service.EntityDiseaseConfigInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.disease.biz.bean.dto.EntityDiseaseConfigListDTO;
import com.xxh.cloud.healthy.basic.framework.modules.disease.biz.bean.dto.EntityDiseaseConfigSaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.disease.biz.helper.EntityDiseaseHelper;
import com.xxh.cloud.healthy.basic.framework.modules.disease.biz.service.EntityDiseaseConfigBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class EntityDiseaseConfigBizServiceImpl implements EntityDiseaseConfigBizService {

    @Autowired
    private EntityDiseaseConfigInfoService entityDiseaseConfigInfoService;

    @Override
    public List<String> getDiseaseByEntityId(String entityId) {
        if (StrUtil.isBlank(entityId)) {
            return new ArrayList<>(0);
        }
        List<EntityDiseaseConfigInfo> list = entityDiseaseConfigInfoService.list(new QueryWrapper<EntityDiseaseConfigInfo>()
            .eq("entity_id", entityId)
        );
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        return list.stream().map(EntityDiseaseConfigInfo::getDiseaseId).collect(Collectors.toList());
    }

    @Override
    public Map<String, List<String>> listDiseaseMap(EntityDiseaseConfigListDTO params) {
        if (StrUtil.isBlank(params.getEntityId()) && CollUtil.isEmpty(params.getEntityIds())) {
            return new HashMap<>(0);
        }
        List<EntityDiseaseConfigInfo> list = entityDiseaseConfigInfoService.list(new QueryWrapper<EntityDiseaseConfigInfo>()
            .eq(StrUtil.isNotBlank(params.getEntityType()), "entity_type", params.getEntityType())
            .eq(StrUtil.isNotBlank(params.getEntityId()), "entity_id", params.getEntityId())
            .in(CollUtil.isNotEmpty(params.getEntityIds()), "entity_id", params.getEntityIds())
        );
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }

        Map<String, List<String>> map = new HashMap<>();
        list.forEach(i -> {
            List<String> l = map.get(i.getEntityType() + "-" + i.getEntityId());
            if (null == l) {
                l = new ArrayList<>();
                map.put(i.getEntityType() + "-" + i.getEntityId(), l);
            }
            l.add(i.getDiseaseId());
        });

        return map;
    }
    
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveDiseaseByEntityId(EntityDiseaseConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        
        // 保存新病种
        if (CollUtil.isEmpty(params.getDiseaseIds())) {
            return;
        }
        
        // 清空病种信息
        entityDiseaseConfigInfoService.remove(new QueryWrapper<EntityDiseaseConfigInfo>()
            .eq("entity_type", params.getEntityType())
            .eq("entity_id", params.getEntityId())
        );

        List<EntityDiseaseConfigInfo> list = new ArrayList<>();
        params.getDiseaseIds().forEach(i -> {
            EntityDiseaseConfigInfo entity = new EntityDiseaseConfigInfo();
            entity.setId(EntityDiseaseHelper.getEntityDiseaseId());
            entity.setEntityType(params.getEntityType());
            entity.setEntityId(params.getEntityId());
            entity.setDiseaseId(i);
            list.add(entity);
        });

        entityDiseaseConfigInfoService.saveBatch(list);
    }

    @Override
    public void saveDisease(List<EntityDiseaseConfigSaveDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> {
            this.saveDiseaseByEntityId(i);
        });
    }

}
