package com.xxh.cloud.healthy.basic.framework.modules.food.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.FoodCategoryCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.food.biz.FoodBizService;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dos.FoodInfo;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.vos.FoodCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.vos.FoodVO;
import com.xxh.cloud.healthy.basic.framework.modules.food.helper.FoodHelper;
import com.xxh.cloud.healthy.basic.framework.modules.food.service.FoodInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class FoodBizServiceImpl implements FoodBizService {


    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private FoodCategoryCacheService foodCategoryCacheService;
    @Autowired
    private FoodInfoService foodInfoService;


    @Override
    public PageData<FoodVO> query(FoodInfoQueryParam params) {
        ValidatorUtils.validateEntity(params);
        IPage<FoodInfo> page = foodInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<FoodInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())

                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())

                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())

                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())

        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        Map<String, FoodCategoryVO> foodCategoryMap= foodCategoryCacheService.getMapByIds(categoryIds);

        PageData<FoodVO> data= new PageConverter<FoodInfo, FoodVO>() {
            @Override
            public FoodVO converter(FoodInfo entity) {
                return FoodHelper.getFoodVO(entity,userMap,foodCategoryMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<FoodVO> list(FoodInfoListParam params) {
        ValidatorUtils.validateEntity(params);
        List<FoodVO> data=new ArrayList<>();
        List<FoodInfo> list = foodInfoService.list(new QueryWrapper<FoodInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())

        );
        if(CollUtil.isEmpty(list)){
            return data;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        Map<String, FoodCategoryVO> foodCategoryMap= foodCategoryCacheService.getMapByIds(categoryIds);

        list.forEach(item->{
            data.add(FoodHelper.getFoodVO(item,userMap,foodCategoryMap));
        });
        return data;
    }

    @Override
    public FoodVO save(FoodInfoSaveParam params) {
        ValidatorUtils.validateEntity(params);
        FoodInfo entity=new FoodInfo();
        entity.setId(FoodHelper.getFoodId());
        entity.setCategoryId(params.getCategoryId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setType(params.getType());
        entity.setName(params.getName());
        entity.setThumbnail(params.getThumbnail());
        entity.setWeight(params.getWeight());
        entity.setCalorie(params.getCalorie());
        entity.setUnit(params.getUnit());
        entity.setVideoPath(params.getVideoPath());
        if(CollUtil.isNotEmpty(params.getComponentRatio())){
            entity.setComponentRatio(JSON.toJSONString(params.getComponentRatio()));
        }
        if(CollUtil.isNotEmpty(params.getNutritionRatio())){
            entity.setNutritionRatio(JSON.toJSONString(params.getNutritionRatio()));
        }
        entity.setCookingMethod(params.getCookingMethod());
        if(CollUtil.isNotEmpty(params.getTaboo())){
            entity.setTaboo( JSON.toJSONString(params.getTaboo()));
        }
        entity.setRemark(params.getRemark());
        entity.setStatus(params.getStatus());
        entity.setRowsStatus(YN.YES.getCode());
        foodInfoService.save(entity);
        return FoodHelper.getFoodVO(entity);
    }

    @Override
    public FoodVO update(FoodInfoUpdateParam params) {
        ValidatorUtils.validateEntity(params);
        FoodInfo entity= foodInfoService.getById(params.getId());
        if(entity==null ||YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setCategoryId(params.getCategoryId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setType(params.getType());
        entity.setName(params.getName());
        entity.setThumbnail(params.getThumbnail());
        entity.setWeight(params.getWeight());
        entity.setCalorie(params.getCalorie());
        entity.setUnit(params.getUnit());
        entity.setVideoPath(params.getVideoPath());
        if(CollUtil.isNotEmpty(params.getComponentRatio())){
            entity.setComponentRatio(JSON.toJSONString(params.getComponentRatio()));
        }
        if(CollUtil.isNotEmpty(params.getNutritionRatio())){
            entity.setNutritionRatio(JSON.toJSONString(params.getNutritionRatio()));
        }
        entity.setCookingMethod(params.getCookingMethod());
        if(CollUtil.isNotEmpty(params.getTaboo())){
            entity.setTaboo( JSON.toJSONString(params.getTaboo()));
        }
        entity.setRemark(params.getRemark());
        entity.setStatus(params.getStatus());
        foodInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getFoodKey(entity.getId()));
        return FoodHelper.getFoodVO(entity);
    }

    @Override
    public FoodVO updateState(FoodInfoUpdateStatusParam params) {
        ValidatorUtils.validateEntity(params);
        FoodInfo entity= foodInfoService.getById(params.getId());
        if(entity==null ||YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(entity.getStatus().equals(params.getStatus())){
            return FoodHelper.getFoodVO(entity);
        }
        entity.setStatus(params.getStatus());
        foodInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getFoodKey(entity.getId()));
        return FoodHelper.getFoodVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        FoodInfo entity= foodInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        entity.setRowsStatus(YN.NO.getCode());
        foodInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getFoodKey(entity.getId()));
    }

    @Override
    public FoodVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        FoodInfo entity= foodInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return getByEntity(entity);
    }



    private FoodVO getByEntity(FoodInfo entity){

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCategoryId())){
            categoryIds.add(entity.getCategoryId());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        Map<String, FoodCategoryVO> foodCategoryMap= foodCategoryCacheService.getMapByIds(categoryIds);

        return FoodHelper.getFoodVO(entity,userMap,foodCategoryMap);
    }


}
