package com.xxh.cloud.healthy.basic.framework.modules.food.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.food.biz.FoodCategoryBizService;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dos.FoodCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dto.FoodCategoryListDTO;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dto.FoodCategoryQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dto.FoodCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.dto.FoodCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.food.entity.vos.FoodCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.food.helper.FoodCategoryHelper;
import com.xxh.cloud.healthy.basic.framework.modules.food.service.FoodCategoryInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class FoodCategoryBizServiceImpl implements FoodCategoryBizService {


    @Autowired
    private FoodCategoryInfoService foodCategoryInfoService;

    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;



    @Override
    public PageData<FoodCategoryVO> query(FoodCategoryQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<FoodCategoryInfo> page = foodCategoryInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<FoodCategoryInfo>()
                        .eq("rows_status", YN.YES.getCode())
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseVO>  diseaseMap= diseaseCacheService.getMapByIds(diseaseIds);
        PageData<FoodCategoryVO> data= new PageConverter<FoodCategoryInfo, FoodCategoryVO>() {
            @Override
            public FoodCategoryVO converter(FoodCategoryInfo entity) {
                return FoodCategoryHelper.getFoodCategoryVO(entity,userMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<FoodCategoryVO> list(FoodCategoryListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<FoodCategoryInfo> list= foodCategoryInfoService.list(new QueryWrapper<FoodCategoryInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
        );
        List<FoodCategoryVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);
//        Map<String,DiseaseVO>  diseaseMap= diseaseCacheService.getMapByIds(diseaseIds);
        list.forEach(item->{
            data.add(FoodCategoryHelper.getFoodCategoryVO(item,userMap));
        });
        return data;
    }

    @Override
    public FoodCategoryVO save(FoodCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        FoodCategoryInfo entity=new FoodCategoryInfo();
        entity.setId(params.getId());
        entity.setName(params.getName());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        foodCategoryInfoService.save(entity);
        return FoodCategoryHelper.getFoodCategoryVO(entity);
    }

    @Override
    public FoodCategoryVO update(FoodCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        FoodCategoryInfo entity= foodCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setName(params.getName());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        foodCategoryInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getFoodCategoryKey(entity.getId()));
        return FoodCategoryHelper.getFoodCategoryVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        FoodCategoryInfo entity= foodCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        foodCategoryInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getFoodCategoryKey(entity.getId()));
    }

    @Override
    public FoodCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        FoodCategoryInfo entity= foodCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.getByEntity(entity);
    }



    private FoodCategoryVO getByEntity(FoodCategoryInfo entity){
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);
//        Map<String,DiseaseVO>  diseaseMap= diseaseCacheService.getMapByIds(diseaseIds);

        return FoodCategoryHelper.getFoodCategoryVO(entity,userMap);
    }



}
