package com.xxh.cloud.healthy.basic.framework.modules.risk.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dos.RiskFactorInfo;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dos.RiskInfo;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dto.RiskFactorOpDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dto.RiskQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dto.RiskSaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.dto.RiskUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.entity.vos.RiskVO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.helper.RiskHelper;
import com.xxh.cloud.healthy.basic.framework.modules.risk.mapper.RiskInfoMapper;
import com.xxh.cloud.healthy.basic.framework.modules.risk.service.RiskFactorInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.risk.service.RiskInfoService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * <p>
 * 风险信息表 服务实现类
 * </p>
 */
@Service
public class RiskInfoServiceImpl extends ServiceImpl<RiskInfoMapper, RiskInfo> implements RiskInfoService {

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;

    @Autowired
    private AdminUserClientService adminUserClientService;


    @Autowired
    private RiskFactorInfoService riskFactorInfoService;

    @Override
    public PageData<RiskVO> query(RiskQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<RiskInfo> page = this.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<RiskInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseVO> diseaseMap= diseaseCacheService.getMapByIds(diseaseIds);

        PageData<RiskVO> data= new PageConverter<RiskInfo, RiskVO>() {
            @Override
            public RiskVO converter(RiskInfo entity) {
                return RiskHelper.getRiskVO(entity, userMap);
            }
        }.getPageData(page);
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RiskVO save(RiskSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RiskInfo entity=new RiskInfo();
        entity.setId(RiskHelper.getRiskId());
        entity.setName(params.getName());
        entity.setGrade(params.getGrade());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setHarm(params.getHarm());
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));
        entity.setRemark(params.getRemark());
        entity.setPrevention(params.getPrevention());
        entity.setFutureMemo(params.getFutureMemo());
        entity.setMemo(params.getMemo());
        entity.setSymptom(params.getSymptom());
        entity.setPathogeny(params.getPathogeny());
        entity.setComplication(params.getComplication());
        entity.setPreventionIdentification(params.getPreventionIdentification());
        entity.setTreatment(params.getTreatment());
        entity.setSeekMedicalAdvice(params.getSeekMedicalAdvice());
        entity.setInspect(params.getInspect());
        entity.setNursing(params.getNursing());
        entity.setDiet(params.getDiet());
        entity.setSource(params.getSource());
        entity.setStatus(YN.NO.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        this.save(entity);

        List<RiskFactorInfo> factorList=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getFactor())){
            params.getFactor().forEach(item->{
                factorList.add(this.getRiskFactorInfo(entity.getId(),item));
            });
        }
        if(factorList.size()>0){
            riskFactorInfoService.saveBatch(factorList);
        }
        //todo:add-log
        return RiskHelper.getRiskVO(entity);
    }

    private RiskFactorInfo getRiskFactorInfo(String riskId, RiskFactorOpDTO params){
        RiskFactorInfo entity=new RiskFactorInfo();
        entity.setId(RiskHelper.getRiskFactorId());
        entity.setRiskId(riskId);
        entity.setMetadataId(params.getMetadataId());
        entity.setOp(params.getOp());
        entity.setVal(params.getVal());
        entity.setTestingProectCode(params.getTestingProectCode());
        entity.setMedicalDept(params.getMedicalDept());
        return entity;
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public RiskVO update(RiskUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RiskInfo entity=this.getById(params.getId());
        if(entity==null ||YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setGrade(params.getGrade());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setHarm(params.getHarm());
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));
        entity.setRemark(params.getRemark());
        entity.setPrevention(params.getPrevention());
        entity.setFutureMemo(params.getFutureMemo());
        entity.setMemo(params.getMemo());
        entity.setSymptom(params.getSymptom());
        entity.setPathogeny(params.getPathogeny());
        entity.setComplication(params.getComplication());
        entity.setPreventionIdentification(params.getPreventionIdentification());
        entity.setTreatment(params.getTreatment());
        entity.setSeekMedicalAdvice(params.getSeekMedicalAdvice());
        entity.setInspect(params.getInspect());
        entity.setNursing(params.getNursing());
        entity.setDiet(params.getDiet());
        entity.setSource(params.getSource());
        this.updateById(entity);

        List<RiskFactorInfo> factorList=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getFactor())){
            params.getFactor().forEach(item->{
                factorList.add(this.getRiskFactorInfo(entity.getId(),item));
            });
        }
        if(factorList.size()>0){
            riskFactorInfoService.remove(new QueryWrapper<RiskFactorInfo>().eq("risk_id",entity.getId()));
            riskFactorInfoService.saveBatch(factorList);
        }
        redisUtils.delete(HealthyBasicCacheKeysHelper.getRiskKey(entity.getId()));
        //todo:add-log
        return RiskHelper.getRiskVO(entity);
    }

    @Override
    public RiskVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RiskInfo entity=this.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

    @Override
    public RiskVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        RiskInfo entity=this.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        return this.info(entity);
    }

    @Override
    public List<RiskVO> list2NullById(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<RiskInfo> list = this.list(
                new QueryWrapper<RiskInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .in("id",ids)
        );

        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> riskIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        list.forEach(item->{
            riskIds.add(item.getId());
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseVO> diseaseMap= diseaseCacheService.getMapByIds(diseaseIds);

        List<RiskFactorInfo> factorList= riskFactorInfoService.list(
                new QueryWrapper<RiskFactorInfo>()
                        .in("risk_id",riskIds)
        );

        List<RiskVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(RiskHelper.getRiskVO(item,factorList,userMap));
        });
        return data;
    }


    private RiskVO info(RiskInfo entity){
        List<RiskFactorInfo> factorList= riskFactorInfoService.list(
                new QueryWrapper<RiskFactorInfo>()
                        .eq("risk_id",entity.getId())
        );

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);

        return RiskHelper.getRiskVO(entity,factorList,userMap);
    }



    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RiskInfo entity=this.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        this.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getRiskKey(entity.getId()));
        //todo:add-log
    }


}
