package com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.HealthyServiceTargetCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.biz.HealthyServiceBizService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dos.HealthyServiceInfo;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceTargetVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.helper.HealthyServiceHelper;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.service.HealthyServiceInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class HealthyServiceBizServiceImpl implements HealthyServiceBizService {


    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private HealthyServiceInfoService healthyServiceInfoService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private HealthyServiceTargetCacheService healthyServiceTargetCacheService;




    @Override
    public PageData<HealthyServiceVO> query(HealthyServiceInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<HealthyServiceInfo> page = healthyServiceInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<HealthyServiceInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getServiceTargetId()),"service_target_id",params.getServiceTargetId())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())

                        .eq(StrUtil.isNotEmpty(params.getContentType()),"content_type",params.getContentType())
                        .eq(StrUtil.isNotEmpty(params.getContentId()),"content_id",params.getContentId())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> serviceTargetIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getServiceTargetId())){
                serviceTargetIds.add(item.getServiceTargetId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, HealthyServiceTargetVO> serviceGroupMap= healthyServiceTargetCacheService.getMapByIds(serviceTargetIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);


        PageData<HealthyServiceVO> data= new PageConverter<HealthyServiceInfo, HealthyServiceVO>() {
            @Override
            public HealthyServiceVO converter(HealthyServiceInfo entity) {
                return HealthyServiceHelper.getHealthyServiceVO(entity,serviceGroupMap,userMap);
            }
        }.getPageData(page);

        return data;
    }



    @Override
    public List<HealthyServiceVO> list(HealthyServiceInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<HealthyServiceInfo> list = healthyServiceInfoService.list(
                new QueryWrapper<HealthyServiceInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getServiceTargetId()),"service_target_id",params.getServiceTargetId())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getContentType()),"content_type",params.getContentType())
                        .eq(StrUtil.isNotEmpty(params.getContentId()),"content_id",params.getContentId())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> serviceTargetIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();


        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getServiceTargetId())){
                serviceTargetIds.add(item.getServiceTargetId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, HealthyServiceTargetVO> serviceGroupMap= healthyServiceTargetCacheService.getMapByIds(serviceTargetIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        List<HealthyServiceVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(HealthyServiceHelper.getHealthyServiceVO(item,serviceGroupMap,userMap));
        });
        return data;
    }



    @Override
    public HealthyServiceVO save(HealthyServiceInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceInfo entity=new HealthyServiceInfo();
        entity.setId(HealthyServiceHelper.getHealthyServiceId());
        entity.setName(params.getName());
        entity.setServiceTargetId(params.getServiceTargetId());
        if(params.getSendCondition()!=null){
            entity.setSendCondition(JSON.toJSONString(params.getSendCondition()));
        }
        if(params.getSendStrategy()!=null){
            entity.setSendStrategy(JSON.toJSONString(params.getSendStrategy()));
        }
        entity.setContentType(params.getContentType());
        if(CollUtil.isNotEmpty(params.getContent())){
            entity.setContent(JSON.toJSONString(params.getContent()));
        }
        entity.setContentSuggest(params.getContentSuggest());
        entity.setRemindTemplate(params.getRemindTemplate());
        entity.setStatus(YN.NO.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        healthyServiceInfoService.save(entity);
        return HealthyServiceHelper.getHealthyServiceVO(entity);
    }

    @Override
    public HealthyServiceVO update(HealthyServiceInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceInfo entity= healthyServiceInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }

        String opBeforeInfo=JSON.toJSONString(entity);
        entity.setServiceTargetId(params.getServiceTargetId());
        entity.setName(params.getName());
        if(params.getSendCondition()!=null){
            entity.setSendCondition(JSON.toJSONString(params.getSendCondition()));
        }
        if(params.getSendStrategy()!=null){
            entity.setSendStrategy(JSON.toJSONString(params.getSendStrategy()));
        }

        entity.setContentType(params.getContentType());
        if(CollUtil.isNotEmpty(params.getContent())){
            entity.setContent(JSON.toJSONString(params.getContent()));
        }
        entity.setContentSuggest(params.getContentSuggest());
        entity.setRemindTemplate(params.getRemindTemplate());
        entity.setStatus(params.getStatus());

        healthyServiceInfoService.updateById(entity);

        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceKey(entity.getId()));

        return HealthyServiceHelper.getHealthyServiceVO(entity);
    }

    @Override
    public HealthyServiceVO updateState(HealthyServiceInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceInfo entity= healthyServiceInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(entity.getStatus().equals(params.getStatus())){
            return HealthyServiceHelper.getHealthyServiceVO(entity);
        }

        String opBeforeInfo=JSON.toJSONString(entity);
        entity.setStatus(params.getStatus());
        healthyServiceInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceKey(entity.getId()));

        return HealthyServiceHelper.getHealthyServiceVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        HealthyServiceInfo entity= healthyServiceInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        healthyServiceInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceKey(entity.getId()));
    }

    @Override
    public HealthyServiceVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        HealthyServiceInfo entity= healthyServiceInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

    private HealthyServiceVO info(HealthyServiceInfo entity){

        Set<String> serviceTargetIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getServiceTargetId())){
            serviceTargetIds.add(entity.getServiceTargetId());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }

        Map<String, HealthyServiceTargetVO> serviceGroupMap= healthyServiceTargetCacheService.getMapByIds(serviceTargetIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        return HealthyServiceHelper.getHealthyServiceVO(entity,serviceGroupMap,userMap);
    }



}
