package com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.biz.HealthyServiceTargetBizService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dos.HealthyServiceInfo;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dos.HealthyServiceTargetInfo;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.entity.vos.HealthyServiceTargetVO;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.helper.HealthyServiceTargetHelper;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.service.HealthyServiceInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.serviceTarget.service.HealthyServiceTargetInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class HealthyServiceTargetBizServiceImpl implements HealthyServiceTargetBizService {



    @Autowired
    private HealthyServiceInfoService healthyServiceInfoService;
    @Autowired
    private HealthyServiceTargetInfoService healthyServiceTargetInfoService;
//    @Autowired
//    private DiseaseClientService diseaseClientService;

    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;






    @Override
    public PageData<HealthyServiceTargetVO> query(HealthyServiceGroupQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<HealthyServiceTargetInfo> page = healthyServiceTargetInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<HealthyServiceTargetInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .orderByDesc("orders")

        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        PageData<HealthyServiceTargetVO> data= new PageConverter<HealthyServiceTargetInfo, HealthyServiceTargetVO>() {
            @Override
            public HealthyServiceTargetVO converter(HealthyServiceTargetInfo entity) {
                return HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity,userMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<HealthyServiceTargetVO> list(HealthyServiceGroupListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<HealthyServiceTargetVO> data=new ArrayList<>();

        List<HealthyServiceTargetInfo> list =healthyServiceTargetInfoService.list(new QueryWrapper<HealthyServiceTargetInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                .orderByDesc("orders")
        );

        if(CollUtil.isEmpty(list)){
            return data;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);

        list.forEach(item->{
            data.add(HealthyServiceTargetHelper.getHealthyServiceTargetVO(item,userMap));
        });

        return data;
    }

    @Override
    public HealthyServiceTargetVO save(HealthyServiceGroupSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceTargetInfo entity=new HealthyServiceTargetInfo();
        entity.setId(HealthyServiceTargetHelper.getHealthyServiceTargetId());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setStatus(YN.NO.getCode());
        if(CollUtil.isNotEmpty(params.getTargetConfig())){
            entity.setTargetConfig(JSON.toJSONString(params.getTargetConfig()));
        }
        entity.setRowsStatus(YN.YES.getCode());
        healthyServiceTargetInfoService.save(entity);
        return HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity);
    }

    @Override
    public HealthyServiceTargetVO updateStatus(HealthyServiceGroupUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(entity.getStatus().equals(params.getStatus())){
            return HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity);
        }
        entity.setStatus(params.getStatus());
        healthyServiceTargetInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceTargetKey(entity.getId()));
        return HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity);
    }

    @Override
    public HealthyServiceTargetVO update(HealthyServiceGroupUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        if(CollUtil.isNotEmpty(params.getTargetConfig())){
            entity.setTargetConfig(JSON.toJSONString(params.getTargetConfig()));
        }
        entity.setDiseaseId(params.getDiseaseId());
        healthyServiceTargetInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceTargetKey(entity.getId()));
        return HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity);
    }

    @Override
    public HealthyServiceTargetVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

//    @Override
//    public HealthyServiceTargetVO info2NullById(String id) {
//        if(StrUtil.isEmpty(id)){
//            return null;
//        }
//        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(id);
//        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
//            return null;
//        }
//        return this.info(entity);
//    }

    private HealthyServiceTargetVO info(HealthyServiceTargetInfo entity){
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO> diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        HealthyServiceTargetVO data= HealthyServiceTargetHelper.getHealthyServiceTargetVO(entity,userMap);
        return data;
    }


    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        HealthyServiceTargetInfo entity= healthyServiceTargetInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        long num = healthyServiceInfoService.count(
                new QueryWrapper<HealthyServiceInfo>()
                        .eq("service_target_id",entity.getId())
                        .eq("rows_status",YN.YES.getCode())
        );

        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR,"有关联数据,无法操作");
        }
        entity.setRowsStatus(YN.NO.getCode());
        healthyServiceTargetInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getHealthyServiceTargetKey(entity.getId()));
    }



}
