package com.xxh.cloud.healthy.basic.framework.modules.sport.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.SportCategoryCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.sport.biz.SportBizService;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dos.SportInfo;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.vos.SportCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.vos.SportVO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.helper.SportHelper;
import com.xxh.cloud.healthy.basic.framework.modules.sport.service.SportInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SportBizServiceImpl implements SportBizService {


    @Autowired
    private SportInfoService sportInfoService;

    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SportCategoryCacheService sportCategoryCacheService;




    @Override
    public PageData<SportVO> query(SportInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<SportInfo> page = sportInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<SportInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .like(StrUtil.isNotEmpty(params.getMetVal()),"met_val",params.getMetVal())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())

                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())

                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .in(CollUtil.isNotEmpty(params.getCategoryIds()),"category_id",params.getCategoryIds())
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();


        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO>  diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        Map<String, SportCategoryVO> categoryMap= sportCategoryCacheService.getMapByIds(categoryIds);

        PageData<SportVO> data= new PageConverter<SportInfo, SportVO>() {
            @Override
            public SportVO converter(SportInfo entity) {
                return SportHelper.getSportVO(entity,userMap,categoryMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<SportVO> list(SportInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<SportVO> data=new ArrayList<>();
        List<SportInfo> list = sportInfoService.list(new QueryWrapper<SportInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
        );

        if(CollUtil.isEmpty(list)){
            return data;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, SportCategoryVO> categoryMap= sportCategoryCacheService.getMapByIds(categoryIds);

        list.forEach(item->{
            data.add(SportHelper.getSportVO(item,userMap,categoryMap));
        });

        return data;
    }

    @Override
    public SportVO save(SportInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        SportInfo entity=new SportInfo();
        entity.setId(SportHelper.getSportId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setCategoryId(params.getCategoryId());
        entity.setName(params.getName());
        entity.setThumbnail(params.getThumbnail());
        entity.setCalorie(params.getCalorie());
        entity.setUnit(params.getUnit());
        entity.setVideoPath(params.getVideoPath());
        if(CollUtil.isNotEmpty(params.getMemo())){
            entity.setMemo(JSON.toJSONString(params.getMemo()));
        }
        if(CollUtil.isNotEmpty(params.getTaboo())){
            entity.setTaboo(JSON.toJSONString(params.getTaboo()));
        }
        entity.setMetVal(params.getMetVal());
        entity.setRemark(params.getRemark());

        entity.setStatus(YN.NO.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        sportInfoService.save(entity);
        return SportHelper.getSportVO(entity);
    }

    @Override
    public SportVO update(SportInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        SportInfo entity= sportInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setCategoryId(params.getCategoryId());
        entity.setThumbnail(params.getThumbnail());
        entity.setCalorie(params.getCalorie());
        entity.setUnit(params.getUnit());
        entity.setVideoPath(params.getVideoPath());
        if(CollUtil.isNotEmpty(params.getMemo())){
            entity.setMemo(JSON.toJSONString(params.getMemo()));
        }
        if(CollUtil.isNotEmpty(params.getTaboo())){
            entity.setTaboo(JSON.toJSONString(params.getTaboo()));
        }
        entity.setMetVal(params.getMetVal());
        entity.setRemark(params.getRemark());
        entity.setStatus(params.getStatus());
        sportInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getSportKey(entity.getId()));
        return SportHelper.getSportVO(entity);
    }

    @Override
    public SportVO updateState(SportInfoUpdateStateDTO params) {
        ValidatorUtils.validateEntity(params);
        SportInfo entity= sportInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return SportHelper.getSportVO(entity);
        }
        entity.setStatus(params.getStatus());
        sportInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getSportKey(entity.getId()));
        return SportHelper.getSportVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        SportInfo entity= sportInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        sportInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getSportKey(entity.getId()));
    }

    @Override
    public SportVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        SportInfo entity= sportInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

//    @Override
//    public SportVO info2NullById(String id) {
//        if(StrUtil.isEmpty(id)){
//            return null;
//        }
//        SportInfo entity= sportInfoService.getById(id);
//        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
//            return null;
//        }
//        return this.info(entity);
//    }


    private SportVO info(SportInfo entity){
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCategoryId())){
            categoryIds.add(entity.getCategoryId());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO>  diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        Map<String, SportCategoryVO> categoryMap= sportCategoryCacheService.getMapByIds(categoryIds);

        return SportHelper.getSportVO(entity,userMap,categoryMap);
    }



}
