package com.xxh.cloud.healthy.basic.framework.modules.sport.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.cache.helper.HealthyBasicCacheKeysHelper;
import com.xxh.cloud.healthy.basic.framework.modules.sport.biz.SportCategoryBizService;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dos.SportCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dos.SportInfo;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dto.SportCategoryListDTO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dto.SportCategoryQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dto.SportCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.dto.SportCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.entity.vos.SportCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.sport.helper.SportCategoryHelper;
import com.xxh.cloud.healthy.basic.framework.modules.sport.service.SportCategoryInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.sport.service.SportInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SportCategoryBizServiceImpl implements SportCategoryBizService {


    @Autowired
    private SportCategoryInfoService sportCategoryInfoService;

    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;
//    @Autowired
//    private DiseaseClientService diseaseClientService;

    @Autowired
    private SportInfoService sportInfoService;

    @Override
    public PageData<SportCategoryVO> query(SportCategoryQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<SportCategoryInfo> page = sportCategoryInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<SportCategoryInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO>  diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);


        PageData<SportCategoryVO> data= new PageConverter<SportCategoryInfo, SportCategoryVO>() {
            @Override
            public SportCategoryVO converter(SportCategoryInfo entity) {
                return SportCategoryHelper.getSportCategoryVO(entity,userMap);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<SportCategoryVO> list(SportCategoryListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<SportCategoryVO> data=new ArrayList<>();
        List<SportCategoryInfo> list = sportCategoryInfoService.list(new QueryWrapper<SportCategoryInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
        );
        if(CollUtil.isEmpty(list)){
            return data;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO>  diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        list.forEach(item->{
            data.add(SportCategoryHelper.getSportCategoryVO(item,userMap));
        });
        return data;
    }

    @Override
    public SportCategoryVO save(SportCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        SportCategoryInfo entity=new SportCategoryInfo();
        if(StrUtil.isEmpty(params.getId())){
            entity.setId(params.getId());
        }else {
            entity.setId(SportCategoryHelper.getSportCategoryId());
        }
        entity.setName(params.getName());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        sportCategoryInfoService.save(entity);
        return SportCategoryHelper.getSportCategoryVO(entity);
    }

    @Override
    public SportCategoryVO update(SportCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        SportCategoryInfo entity= sportCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        sportCategoryInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getSportCategoryKey(entity.getId()));
        return SportCategoryHelper.getSportCategoryVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        SportCategoryInfo entity= sportCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        long num= sportInfoService.count(
                new QueryWrapper<SportInfo>()
                        .eq("category_id",entity.getId())
                        .eq("rows_status",YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR,"有关联数据,无法操作");
        }
        entity.setRowsStatus(YN.NO.getCode());
        sportCategoryInfoService.updateById(entity);
        redisUtils.delete(HealthyBasicCacheKeysHelper.getSportCategoryKey(entity.getId()));
    }

    @Override
    public SportCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        SportCategoryInfo entity= sportCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return getByEntity(entity);
    }


    private SportCategoryVO getByEntity(SportCategoryInfo entity){
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
//        Map<String, DiseaseBO>  diseaseMap= diseaseClientService.getDiseaseMapByIds(diseaseIds);
        return SportCategoryHelper.getSportCategoryVO(entity,userMap);
    }


}
