package com.xxh.cloud.healthy.basic.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.StandardLabelCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.client.helper.LabelBOConvert;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelInfoListParam;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelInfoQueryParam;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelInfoSaveParam;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelVO;
import com.xxh.cloud.healthy.basic.framework.modules.label.helper.StandardLabelHelper;
import com.xxh.cloud.healthy.basic.framework.modules.label.service.StandardLabelBizService;
import com.xxh.cloud.healthy.basic.label.client.api.LabelCategoryClientService;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelCategoryBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelSaveDTO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class LabelClientServiceImpl implements LabelClientService {

    @Autowired
    private StandardLabelCacheService standardLabelCacheService;
    @Autowired
    private StandardLabelBizService standardLabelBizService;
    @Autowired
    private LabelCategoryClientService labelCategoryClientService;

    @Override
    public LabelBO getLabelById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        StandardLabelVO entity = standardLabelCacheService.getById(id);
        return LabelBOConvert.getLabelBO(entity);
    }

    @Override
    public LabelBO copyAndGetLabelById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        StandardLabelVO vo = standardLabelCacheService.getById(id);
        if (null == vo) {
            return null;
        }

        StandardLabelInfoSaveParam standardLabelInfoSaveParam = new StandardLabelInfoSaveParam();
        standardLabelInfoSaveParam.setProjectCode(vo.getProjectCode());
        standardLabelInfoSaveParam.setName(vo.getName() + "_拷贝");
        standardLabelInfoSaveParam.setRemark(vo.getRemark());
        standardLabelInfoSaveParam.setCategoryId(vo.getCategoryId());
        standardLabelInfoSaveParam.setBizType(vo.getBizType());
        standardLabelInfoSaveParam.setBizTwoType(vo.getBizTwoType());
        standardLabelInfoSaveParam.setBizThreeType(vo.getBizThreeType());
        standardLabelInfoSaveParam.setOrders(vo.getOrders());
        standardLabelInfoSaveParam.setDiseaseIds(vo.getDiseaseIds());
        standardLabelInfoSaveParam.setStatDimensionType(vo.getStatDimensionType());

        standardLabelInfoSaveParam.setConsultationRemindStatus(vo.getConsultationRemindStatus());
        standardLabelInfoSaveParam.setConsultationDeptIds(vo.getConsultationDeptIds());
        standardLabelInfoSaveParam.setConsultationRemark(vo.getConsultationRemark());
        standardLabelInfoSaveParam.setConsultationSubjectName(vo.getConsultationSubjectName());

        standardLabelInfoSaveParam.setPotentialType(vo.getPotentialType());
        standardLabelInfoSaveParam.setIsPortrait(vo.getIsPortrait());

        StandardLabelVO resp = standardLabelBizService.save(standardLabelInfoSaveParam);
        return LabelBOConvert.getLabelBO(resp);
    }

    @Override
    public List<LabelBO> getLabelListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<StandardLabelVO> list = standardLabelCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        List<LabelBO> data = list.stream().map(LabelBOConvert::getLabelBO).collect(Collectors.toList());
        
        // zs20231228 查询标签分类
        if (CollectionUtil.isNotEmpty(data)) {
            List<String> categoryIds = new ArrayList<>();
            data.forEach(i -> categoryIds.add(i.getCategoryId()));

            if (CollectionUtil.isNotEmpty(categoryIds)) {
                Map<String, LabelCategoryBO> categoryMap = labelCategoryClientService.getMapByIds(categoryIds);
                if (CollectionUtil.isNotEmpty(categoryMap)) {
                    data.forEach(i -> i.setLabelCategoryInfo(categoryMap.get(i.getCategoryId())));
                }
            }
        }

        return data;
    }

    @Override
    public Map<String, LabelBO> getLabelMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<StandardLabelVO> list = standardLabelCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        Map<String, LabelBO> labelBoMap = new HashMap<>();
        list.forEach(i -> labelBoMap.put(i.getId(), LabelBOConvert.getLabelBO(i)));
        return labelBoMap;
    }

    @Override
    public List<LabelBO> queryList(LabelQueryPageDTO params) {
        StandardLabelInfoListParam standardLabelInfoListParam = new StandardLabelInfoListParam();
        standardLabelInfoListParam.setId(params.getId());
        standardLabelInfoListParam.setName(params.getName());
        standardLabelInfoListParam.setProjectCode(params.getProjectCode());
        standardLabelInfoListParam.setStatus(params.getStatus());
        standardLabelInfoListParam.setStatDimensionType(params.getStatDimensionType());
        standardLabelInfoListParam.setStatDimensionTypes(params.getStatDimensionTypes());
        // zs20231113
        standardLabelInfoListParam.setIds(params.getIds());
        standardLabelInfoListParam.setBizType(params.getBizType());
        standardLabelInfoListParam.setBizTypes(params.getBizTypes());
        standardLabelInfoListParam.setBizTwoType(params.getBizTwoType());
        standardLabelInfoListParam.setBizTwoTypes(params.getBizTwoTypes());
        standardLabelInfoListParam.setBizThreeType(params.getBizThreeType());
        standardLabelInfoListParam.setBizThreeTypes(params.getBizThreeTypes());
        standardLabelInfoListParam.setDiseaseId(params.getDiseaseId());
        standardLabelInfoListParam.setDiseaseIds(params.getDiseaseIds());
        standardLabelInfoListParam.setDiseaseQueryMode(params.getDiseaseQueryMode());
        
        standardLabelInfoListParam.setConsultationRemindStatus(params.getConsultationRemindStatus());
        standardLabelInfoListParam.setPotentialType(params.getPotentialType());
        standardLabelInfoListParam.setIsPotentialType(params.getIsPotentialType());
        standardLabelInfoListParam.setIsPortrait(params.getIsPortrait());
        standardLabelInfoListParam.setAdminLabel(params.getAdminLabel());
        
        List<StandardLabelVO> list = standardLabelBizService.list(standardLabelInfoListParam);
        
        return list.stream().map(LabelBOConvert::getLabelBO).collect(Collectors.toList());
    }

    @Override
    public List<LabelBO> addBatch(List<LabelSaveDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return new ArrayList<>(0);
        }
        List<StandardLabelInfoSaveParam> standardLabelInfoSaveParams = params.stream().map(i -> {
            StandardLabelInfoSaveParam standardLabelInfoSaveParam = new StandardLabelInfoSaveParam();
            standardLabelInfoSaveParam.setId(i.getId());
            standardLabelInfoSaveParam.setName(i.getName());
            standardLabelInfoSaveParam.setStatDimensionType(i.getStatDimensionType());
            standardLabelInfoSaveParam.setBizType(i.getBizType());
            standardLabelInfoSaveParam.setBizTwoType(i.getBizTwoType());
            standardLabelInfoSaveParam.setBizThreeType(i.getBizThreeType());
            standardLabelInfoSaveParam.setCategoryId(i.getCategoryId());
            standardLabelInfoSaveParam.setProjectCode(i.getProjectCode());
            standardLabelInfoSaveParam.setDiseaseIds(i.getDiseaseIds());
            standardLabelInfoSaveParam.setRemark(i.getRemark());
            standardLabelInfoSaveParam.setOrders(i.getOrders());

            standardLabelInfoSaveParam.setConsultationRemindStatus(i.getConsultationRemindStatus());
            standardLabelInfoSaveParam.setConsultationDeptIds(i.getConsultationDeptIds());
            standardLabelInfoSaveParam.setConsultationRemark(i.getConsultationRemark());
            standardLabelInfoSaveParam.setConsultationSubjectName(i.getConsultationSubjectName());
            standardLabelInfoSaveParam.setPotentialType(i.getPotentialType());
            standardLabelInfoSaveParam.setIsPortrait(i.getIsPortrait());

            return standardLabelInfoSaveParam;
        }).collect(Collectors.toList());
        List<StandardLabelVO> list = standardLabelBizService.saveBatch(standardLabelInfoSaveParams);

        return list.stream().map(LabelBOConvert::getLabelBO).collect(Collectors.toList());
    }

    @Override
    public PageData<LabelBO> queryPage(LabelQueryPageDTO params) {
        StandardLabelInfoQueryParam standardLabelInfoQueryParam = new StandardLabelInfoQueryParam();
        standardLabelInfoQueryParam.setId(params.getId());
        standardLabelInfoQueryParam.setIds(params.getIds());
        standardLabelInfoQueryParam.setBizType(params.getBizType());
        standardLabelInfoQueryParam.setBizTypes(params.getBizTypes());
        standardLabelInfoQueryParam.setBizTwoType(params.getBizTwoType());
        standardLabelInfoQueryParam.setBizTwoTypes(params.getBizTwoTypes());
        standardLabelInfoQueryParam.setBizThreeType(params.getBizThreeType());
        standardLabelInfoQueryParam.setBizThreeTypes(params.getBizThreeTypes());
        standardLabelInfoQueryParam.setCategoryId(params.getCategoryId());
        standardLabelInfoQueryParam.setCategoryIds(params.getCategoryIds());
        standardLabelInfoQueryParam.setDiseaseId(params.getDiseaseId());
        standardLabelInfoQueryParam.setDiseaseIds(params.getDiseaseIds());
        standardLabelInfoQueryParam.setStatDimensionType(params.getStatDimensionType());
        standardLabelInfoQueryParam.setStatDimensionTypes(params.getStatDimensionTypes());
        standardLabelInfoQueryParam.setDiseaseQueryMode(params.getDiseaseQueryMode());
        standardLabelInfoQueryParam.setStatus(params.getStatus());
        standardLabelInfoQueryParam.setName(params.getName());
        standardLabelInfoQueryParam.setProjectCode(params.getProjectCode());
        standardLabelInfoQueryParam.setPage(params.getPage());
        standardLabelInfoQueryParam.setRows(params.getRows());

        standardLabelInfoQueryParam.setConsultationRemindStatus(params.getConsultationRemindStatus());
        standardLabelInfoQueryParam.setPotentialType(params.getPotentialType());
        standardLabelInfoQueryParam.setIsPotentialType(params.getIsPotentialType());
        standardLabelInfoQueryParam.setIsPortrait(params.getIsPortrait());
        standardLabelInfoQueryParam.setAdminLabel(params.getAdminLabel());
        PageData<StandardLabelVO> page = standardLabelBizService.query(standardLabelInfoQueryParam);

        PageData<LabelBO> data = new PageConverter<StandardLabelVO, LabelBO>() {
            @Override
            public LabelBO converter(StandardLabelVO vo) {
                return StandardLabelHelper.getLabelBO(vo);
            }
        }.getPageData(page.getList(), page.getTotal(), page.getSize(), page.getPages());

        return data;
    }

}
