package com.xxh.cloud.healthy.basic.framework.modules.label.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.StandardLabelInfo;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelVO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelCategoryBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import java.util.Map;

public class StandardLabelHelper {

    /**
     * ID
     * */
    public static String getHealthyLabelId(){

        return StrUtil.format("L{}", IdUtil.getSnowflake(5,6).nextIdStr());
    }

    /**
     *
     * */
    public static String getHealthyLabelRelId(){
        return StrUtil.format("LR{}", IdUtil.getSnowflake(5,7).nextIdStr());
    }



    /**
     *  label
     * */
    public static StandardLabelVO getHealthyLabelVO(StandardLabelInfo entity){
        if(entity==null){
            return null;
        }
        StandardLabelVO data=new StandardLabelVO();
        BeanUtil.copyProperties(entity,data,"diseaseIds", "consultationDeptIds");
        data.setDiseaseIds(StrJoinSplitUtils.split2List(entity.getDiseaseIds()));
        if (StrUtil.isNotBlank(entity.getConsultationDeptIds())) {
            data.setConsultationDeptIds(StrJoinSplitUtils.split2List(entity.getConsultationDeptIds()));
        }
        return data;
    }

    public static LabelBO getLabelBO(StandardLabelVO vo){
        if(vo == null){
            return null;
        }
        LabelBO data = new LabelBO();
        BeanUtil.copyProperties(vo, data);
        return data;
    }


    /**
     *  label
     * */
    public static StandardLabelVO getHealthyLabelVO(StandardLabelInfo entity,
                                                    Map<String, AdminUserBO>  userMap,
                                                    Map<String, LabelCategoryBO> categoryMap,
                                                    Map<String, StandardDiseaseBO> diseaseMap){
        StandardLabelVO data= StandardLabelHelper.getHealthyLabelVO(entity);
        if(data==null){
            return null;
        }


        if(MapUtil.isNotEmpty(userMap)){
            if(StrUtil.isNotEmpty(data.getCreator())){
                AdminUserBO user= userMap.get(data.getCreator());
                if(user!=null){
                    data.setCreatorUserName(user.getUserName());
                    data.setCreatorRealName(user.getRealName());
                }
            }

            if(StrUtil.isNotEmpty(data.getUpdater())){
                AdminUserBO user= userMap.get(data.getUpdater());
                if(user!=null){
                    data.setUpdaterUserName(user.getUserName());
                    data.setUpdaterRealName(user.getRealName());
                }
            }
        }

        if(MapUtil.isNotEmpty(categoryMap) && StrUtil.isNotEmpty(data.getCategoryId())){
            LabelCategoryBO category =categoryMap.get(data.getCategoryId());
            if(category!=null){
                data.setCategoryName(category.getName());
            }
        }

        if(MapUtil.isNotEmpty(diseaseMap) && CollUtil.isNotEmpty(data.getDiseaseIds())){
            data.getDiseaseIds().forEach(id->{
                StandardDiseaseBO disease = diseaseMap.get(id);
                if(disease!=null){
                    data.addDisease(disease.getId(),disease.getName());
                }
            });
        }
        return data;
    }



}
