package com.xxh.cloud.healthy.basic.framework.modules.label.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.client.api.EntityDiseaseConfigClientService;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigListClientDTO;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigSaveClientDTO;
import com.xxh.cloud.healthy.basic.client.api.constant.EntityDiseaseTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.ModeEnum;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.StandardLabelInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.StandardLabelInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.StandardLabelCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelVO;
import com.xxh.cloud.healthy.basic.framework.modules.label.helper.StandardLabelHelper;
import com.xxh.cloud.healthy.basic.framework.modules.label.service.StandardLabelBizService;
import com.xxh.cloud.healthy.basic.label.client.api.LabelCategoryClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelCategoryBO;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class StandardLabelBizServiceImpl implements StandardLabelBizService {


    @Autowired
    private StandardLabelInfoService standardLabelInfoService;
    @Autowired
    private LabelCategoryClientService labelCategoryClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private StandardLabelCacheService standardLabelCacheService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private EntityDiseaseConfigClientService entityDiseaseConfigClientService;


    @Override
    public PageData<StandardLabelVO> query(StandardLabelInfoQueryParam params) {
        ValidatorUtils.validateEntity(params);
        IPage<StandardLabelInfo> page = standardLabelInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<StandardLabelInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())

                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .in(CollUtil.isNotEmpty(params.getCategoryIds()),"category_id",params.getCategoryIds())

                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())

                        .eq(StrUtil.isNotEmpty(params.getBizType()),"biz_type",params.getBizType())
                        .eq(StrUtil.isNotEmpty(params.getBizTwoType()),"biz_two_type",params.getBizTwoType())
                        .eq(StrUtil.isNotEmpty(params.getBizThreeType()),"biz_three_type",params.getBizThreeType())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .and(CollUtil.isNotEmpty(params.getDiseaseIds()), i -> {
                            params.getDiseaseIds().forEach(j -> {
                                if (ModeEnum.QueryModeEnum.OR.getCode().equals(params.getDiseaseQueryMode())) {  // or查询还是and查询
                                    i.or();
                                }
                                i.like("disease_ids", "," + j + ",");
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizTypes()), i -> {
                            params.getBizTypes().forEach(j -> {
                                i.or().eq("biz_type", j);
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizTwoTypes()), i -> {
                            params.getBizTwoTypes().forEach(j -> {
                                i.or().eq("biz_two_type", j);
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizThreeTypes()), i -> {
                            params.getBizThreeTypes().forEach(j -> {
                                i.or().eq("biz_three_type", j);
                            });
                        })

                        .eq(StrUtil.isNotEmpty(params.getStatDimensionType()),"stat_dimension_type",params.getStatDimensionType())
                        .in(CollUtil.isNotEmpty(params.getStatDimensionTypes()),"stat_dimension_type",params.getStatDimensionTypes())
                        .eq(StrUtil.isNotEmpty(params.getConsultationRemindStatus()),"consultation_remind_status",params.getConsultationRemindStatus())
                        .eq(StrUtil.isNotEmpty(params.getPotentialType()),"potential_type",params.getPotentialType())
                        .and(StrUtil.isNotBlank(params.getIsPotentialType()) && YN.YES.getCode().equals(params.getIsPotentialType()), i -> i.isNotNull("potential_type").ne("potential_type", ""))
                        .and(StrUtil.isNotBlank(params.getIsPotentialType()) && YN.NO.getCode().equals(params.getIsPotentialType()), i -> i.isNull("potential_type").or().eq("potential_type", ""))
                        .eq(StrUtil.isNotEmpty(params.getIsPortrait()),"is_portrait", params.getIsPortrait())
                        .eq(StrUtil.isNotEmpty(params.getConsultationFollowUpBaseDateType()),"consultation_follow_up_base_date_type", params.getConsultationFollowUpBaseDateType())
                        .eq(YN.NO.getCode().equals(params.getAdminLabel()),"admin_label", params.getAdminLabel())
                        .orderByDesc("orders")
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return new PageData<>(null, 0);
        }

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        page.getRecords().forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.LABEL.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        Set<String> userIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> deptIds = new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                Set<String> disIds = StrJoinSplitUtils.split2Set(item.getDiseaseIds());
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.LABEL.getCode() + "-" + item.getId());
                if (CollUtil.isNotEmpty(l)) {
                    disIds = new HashSet<>(l);
                }
                diseaseIds.addAll(disIds);
            }
            if (StrUtil.isNotEmpty(item.getConsultationDeptIds())) {
                Set<String> depIds = StrJoinSplitUtils.split2Set(item.getConsultationDeptIds());
                deptIds.addAll(depIds);
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, LabelCategoryBO> categoryMap= labelCategoryClientService.getMapByIds(categoryIds);
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);

        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            deptMap.putAll(deptClientService.getDeptMapByIds(deptIds));
        }

        PageData<StandardLabelVO> data= new PageConverter<StandardLabelInfo, StandardLabelVO>() {
            @Override
            public StandardLabelVO converter(StandardLabelInfo entity) {
                StandardLabelVO vo =  StandardLabelHelper.getHealthyLabelVO(entity,userMap,categoryMap,diseaseMap);
                List<DeptBO> depList = new ArrayList<>();
                if (null != vo) {
                    List<String> depIds = vo.getConsultationDeptIds();
                    if (CollUtil.isNotEmpty(depIds)) {
                        depIds.forEach(i -> {
                            DeptBO dep = deptMap.get(i);
                            if (null != dep) {
                                depList.add(dep);
                            }
                        });
                    }
                    if (CollUtil.isNotEmpty(depList)) {
                        vo.setConsultationDeptList(depList);
                    }
                }
                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<StandardLabelVO> list(StandardLabelInfoListParam params) {
        ValidatorUtils.validateEntity(params);
        List<StandardLabelInfo> list = standardLabelInfoService.list(
                new QueryWrapper<StandardLabelInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())

                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .in(CollUtil.isNotEmpty(params.getCategoryIds()),"category_id",params.getCategoryIds())

                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getBizType()),"biz_type",params.getBizType())
                        .eq(StrUtil.isNotEmpty(params.getBizTwoType()),"biz_two_type",params.getBizTwoType())
                        .eq(StrUtil.isNotEmpty(params.getBizThreeType()),"biz_three_type",params.getBizThreeType())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .and(CollUtil.isNotEmpty(params.getDiseaseIds()), i -> {
                            params.getDiseaseIds().forEach(j -> {
                                if (ModeEnum.QueryModeEnum.OR.getCode().equals(params.getDiseaseQueryMode())) {  // or查询还是and查询
                                    i.or();
                                }
                                i.like("disease_ids", "," + j + ",");
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizTypes()), i -> {
                            params.getBizTypes().forEach(j -> {
                                i.or().eq("biz_type", j);
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizTwoTypes()), i -> {
                            params.getBizTwoTypes().forEach(j -> {
                                i.or().eq("biz_two_type", j);
                            });
                        })
                        .and(CollUtil.isNotEmpty(params.getBizThreeTypes()), i -> {
                            params.getBizThreeTypes().forEach(j -> {
                                i.or().eq("biz_three_type", j);
                            });
                        })
                        .eq(StrUtil.isNotEmpty(params.getStatDimensionType()),"stat_dimension_type",params.getStatDimensionType())
                        .in(CollUtil.isNotEmpty(params.getStatDimensionTypes()),"stat_dimension_type",params.getStatDimensionTypes())
                        .eq(StrUtil.isNotEmpty(params.getConsultationRemindStatus()),"consultation_remind_status",params.getConsultationRemindStatus())
                        .eq(StrUtil.isNotEmpty(params.getPotentialType()),"potential_type",params.getPotentialType())
                        .and(StrUtil.isNotBlank(params.getIsPotentialType()) && YN.YES.getCode().equals(params.getIsPotentialType()), i -> i.isNotNull("potential_type").ne("potential_type", ""))
                        .and(StrUtil.isNotBlank(params.getIsPotentialType()) && YN.NO.getCode().equals(params.getIsPotentialType()), i -> i.isNull("potential_type").or().eq("potential_type", ""))
                        .eq(StrUtil.isNotEmpty(params.getIsPortrait()),"is_portrait", params.getIsPortrait())
                        .eq(StrUtil.isNotEmpty(params.getConsultationFollowUpBaseDateType()),"consultation_follow_up_base_date_type", params.getConsultationFollowUpBaseDateType())
                        .eq(YN.NO.getCode().equals(params.getAdminLabel()),"admin_label", params.getAdminLabel())
                        .orderByDesc("orders")

        );

        if(CollUtil.isEmpty(list)){
            return new ArrayList<>(0);
        }

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        list.forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.LABEL.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        Set<String> userIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> deptIds = new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                Set<String> disIds = StrJoinSplitUtils.split2Set(item.getDiseaseIds());
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.LABEL.getCode() + "-" + item.getId());
                if (CollUtil.isNotEmpty(l)) {
                    disIds = new HashSet<>(l);
                }
                diseaseIds.addAll(disIds);
            }
            if (StrUtil.isNotEmpty(item.getConsultationDeptIds())) {
                Set<String> depIds = StrJoinSplitUtils.split2Set(item.getConsultationDeptIds());
                deptIds.addAll(depIds);
            }
        });

        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, LabelCategoryBO> categoryMap= labelCategoryClientService.getMapByIds(categoryIds);
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);

        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            deptMap.putAll(deptClientService.getDeptMapByIds(deptIds));
        }

        List<StandardLabelVO> data=new ArrayList<>();

        list.forEach(item->{
            StandardLabelVO vo = StandardLabelHelper.getHealthyLabelVO(item,userMap,categoryMap,diseaseMap);

            List<DeptBO> depList = new ArrayList<>();
            if (null != vo) {
                List<String> depIds = vo.getConsultationDeptIds();
                if (CollUtil.isNotEmpty(depIds)) {
                    depIds.forEach(i -> {
                        DeptBO dep = deptMap.get(i);
                        if (null != dep) {
                            depList.add(dep);
                        }
                    });
                }
                if (CollUtil.isNotEmpty(depList)) {
                    vo.setConsultationDeptList(depList);
                }

            }
            data.add(vo);
        });
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StandardLabelVO save(StandardLabelInfoSaveParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelInfo entity=new StandardLabelInfo();
        entity.setId(StandardLabelHelper.getHealthyLabelId());
        entity.setProjectCode(params.getProjectCode());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setCategoryId(params.getCategoryId());
        entity.setBizType(params.getBizType());
        entity.setBizTwoType(params.getBizTwoType());
        entity.setBizThreeType(params.getBizThreeType());
        entity.setStatus(YN.YES.getCode());
        entity.setOrders(params.getOrders());
        entity.setRowsStatus(YN.YES.getCode());
        if (CollUtil.isNotEmpty(params.getDiseaseIds())) {
            entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        }
        entity.setStatDimensionType(params.getStatDimensionType());

        entity.setConsultationRemindStatus(params.getConsultationRemindStatus());
        entity.setConsultationRemark(params.getConsultationRemark());
        entity.setConsultationSubjectName(params.getConsultationSubjectName());
        if (CollUtil.isNotEmpty(params.getConsultationDeptIds())) {
            entity.setConsultationDeptIds(StrJoinSplitUtils.joinPrefix(params.getConsultationDeptIds()));
        }
        entity.setPotentialType(params.getPotentialType());
        entity.setIsPortrait(params.getIsPortrait());
        entity.setConsultationFollowUpBaseDateType(params.getConsultationFollowUpBaseDateType());
        entity.setConsultationManager(params.getConsultationManager());
        entity.setConsultationMessage(params.getConsultationMessage());
        entity.setLabelRemindStatus(params.getLabelRemindStatus());
        entity.setLabelManager(params.getLabelManager());
        entity.setLabelAssistant(params.getLabelAssistant());
        entity.setLabelDeptManager(params.getLabelDeptManager());
        entity.setAdminLabel(params.getAdminLabel());

        standardLabelInfoService.save(entity);

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.LABEL.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        return StandardLabelHelper.getHealthyLabelVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StandardLabelVO update(StandardLabelInfoUpdateParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelInfo entity= standardLabelInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setBizType(params.getBizType());
        entity.setBizTwoType(params.getBizTwoType());
        entity.setBizThreeType(params.getBizThreeType());
        entity.setCategoryId(params.getCategoryId());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setDiseaseIds("");
        if (CollUtil.isNotEmpty(params.getDiseaseIds())) {
            entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        }
        entity.setStatDimensionType(params.getStatDimensionType());

        entity.setConsultationRemindStatus(params.getConsultationRemindStatus());
        entity.setConsultationRemark(params.getConsultationRemark());
        entity.setConsultationSubjectName(params.getConsultationSubjectName());
        entity.setConsultationDeptIds(StrJoinSplitUtils.joinPrefix(params.getConsultationDeptIds()));
        entity.setPotentialType(params.getPotentialType());
        entity.setIsPortrait(params.getIsPortrait());
        entity.setConsultationFollowUpBaseDateType(params.getConsultationFollowUpBaseDateType());
        entity.setConsultationManager(params.getConsultationManager());
        entity.setConsultationMessage(params.getConsultationMessage());
        entity.setLabelRemindStatus(params.getLabelRemindStatus());
        entity.setLabelManager(params.getLabelManager());
        entity.setLabelAssistant(params.getLabelAssistant());
        entity.setLabelDeptManager(params.getLabelDeptManager());
        entity.setAdminLabel(params.getAdminLabel());
        standardLabelInfoService.updateById(entity);
        standardLabelCacheService.removeById(entity.getId());
        
        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.LABEL.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        return StandardLabelHelper.getHealthyLabelVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StandardLabelVO updateStatus(StandardLabelInfoUpdateStatusParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelInfo entity= standardLabelInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(entity.getStatus().equals(params.getStatus())){
            return StandardLabelHelper.getHealthyLabelVO(entity);
        }
        entity.setStatus(params.getStatus());
        standardLabelInfoService.updateById(entity);
        standardLabelCacheService.removeById(entity.getId());
        return StandardLabelHelper.getHealthyLabelVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        StandardLabelInfo entity= standardLabelInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }

        entity.setRowsStatus(YN.NO.getCode());
        standardLabelInfoService.updateById(entity);
        standardLabelCacheService.removeById(entity.getId());
    }

    @Override
    public StandardLabelVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        StandardLabelInfo entity= standardLabelInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        Set<String> userIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCategoryId())){
            categoryIds.add(entity.getCategoryId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }

        if(StrUtil.isNotEmpty(entity.getDiseaseIds())){
            diseaseIds=StrJoinSplitUtils.split2Set(entity.getDiseaseIds());
            // zs20240830 从新病种关系中获取，若不为空则以该数据为准
            List<String> l = entityDiseaseConfigClientService.getDiseaseByEntityId(id);
            if (CollUtil.isNotEmpty(l)) { // zs20240830 原来病种数据为空则从新病种关系中获取
                diseaseIds = new HashSet<>(l);
            }
        }

        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, LabelCategoryBO> categoryMap= labelCategoryClientService.getMapByIds(categoryIds);
        StandardLabelVO data= StandardLabelHelper.getHealthyLabelVO(entity,userMap,categoryMap,diseaseMap);
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<StandardLabelVO> saveBatch(List<StandardLabelInfoSaveParam> params) {
        log.info("StandardLabelBizServiceImpl:saveBatch:params:>>>>>> {}", JSON.toJSONString(params));
        params.forEach(i -> {
            ValidatorUtils.validateEntity(i);
        });

        // 若id不为空，则根据id去重
        Set<String> labelIds = new HashSet<>();
        params.forEach(i -> {
            if (StrUtil.isNotBlank(i.getId())) {
                labelIds.add(i.getId());
            }
        });

        // 查询已经存在的标签，包括rows_status为N的数据（未防止id冲突）
        List<StandardLabelInfo> existList = standardLabelInfoService.listByIds(labelIds);
        Set<String> existLabelIds = new HashSet<>(); // 已存在的需要的数据，不需要新增
        Set<String> updateRowsStatusLabelIds = new HashSet<>(); // 若需要的数据为删除状态则恢复可用状态
        existList.forEach(i -> {
            if (YN.NO.getCode().equals(i.getRowsStatus())) {
                updateRowsStatusLabelIds.add(i.getId());
            }
            existLabelIds.add(i.getId());
        });

        // 恢复删除状态的数据
        if (CollectionUtil.isNotEmpty(updateRowsStatusLabelIds)) {
            standardLabelInfoService.update(new UpdateWrapper<StandardLabelInfo>()
                    .set("rows_status", YN.YES.getCode())
                    .in("id", updateRowsStatusLabelIds)
            );
        }
        
        // 添加新标签
        List<StandardLabelInfo> list = new ArrayList<>();

        params.forEach(i -> {
            if (!existLabelIds.contains(i.getId())) {
                StandardLabelInfo entity=new StandardLabelInfo();
                if (StrUtil.isNotBlank(i.getId())) {
                    entity.setId(i.getId());
                } else {
                    entity.setId(StandardLabelHelper.getHealthyLabelId());
                }
                entity.setProjectCode(i.getProjectCode());
                entity.setName(i.getName());
                entity.setRemark(i.getRemark());
                entity.setCategoryId(i.getCategoryId());
                entity.setBizType(i.getBizType());
                entity.setBizTwoType(i.getBizTwoType());
                entity.setBizThreeType(i.getBizThreeType());
                entity.setStatus(YN.YES.getCode());
                entity.setOrders(i.getOrders());
                entity.setRowsStatus(YN.YES.getCode());
                if (CollUtil.isNotEmpty(i.getDiseaseIds())) {
                    entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(i.getDiseaseIds()));
                }
                entity.setStatDimensionType(i.getStatDimensionType());
                entity.setConsultationRemindStatus(i.getConsultationRemindStatus());
                entity.setPotentialType(i.getPotentialType());
                entity.setIsPortrait(i.getIsPortrait());
                entity.setConsultationManager(i.getConsultationManager());
                entity.setConsultationMessage(i.getConsultationMessage());
                entity.setLabelRemindStatus(i.getLabelRemindStatus());
                entity.setLabelManager(i.getLabelManager());
                entity.setLabelAssistant(i.getLabelAssistant());
                entity.setLabelDeptManager(i.getLabelDeptManager());
                list.add(entity);
            }
        });

        standardLabelInfoService.saveBatch(list);

        // zs20240830 保存病种
        if (CollUtil.isNotEmpty(list)) {
            List<EntityDiseaseConfigSaveClientDTO> entityDiseaseList = new ArrayList<>();
            list.forEach(i -> {
                EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
                entityDisease.setEntityType(EntityDiseaseTypeEnum.LABEL.getCode());
                entityDisease.setEntityId(i.getId());

                List<String> diseaseIds = null;
                if (StrUtil.isNotBlank(i.getDiseaseIds())) {
                    diseaseIds = StrJoinSplitUtils.split2List(i.getDiseaseIds());
                }
                entityDisease.setDiseaseIds(diseaseIds);
                entityDiseaseList.add(entityDisease);
            });
            entityDiseaseConfigClientService.saveDisease(entityDiseaseList);
        }

        existList.addAll(list); // 返回新增和已存在的数据
        return existList.stream().map(StandardLabelHelper::getHealthyLabelVO).collect(Collectors.toList());
    }




}
