package com.xxh.cloud.healthy.basic.framework.modules.label.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.StandardLabelCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.StandardLabelCategoryInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.cache.service.StandardLabelCategoryCacheService;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryListDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.label.helper.StandardLabelCategoryHelper;
import com.xxh.cloud.healthy.basic.framework.modules.label.service.StandardLabelCategoryBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
public class StandardLabelCategoryBizServiceImpl implements StandardLabelCategoryBizService {

    @Autowired
    private StandardLabelCategoryInfoService standardLabelCategoryInfoService;

    @Autowired
    private StandardLabelCategoryCacheService standardLabelCategoryCacheService;

    @Autowired
    private AdminUserClientService adminUserClientService;


    @Override
    public List<StandardLabelCategoryVO> tree(StandardLabelCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<StandardLabelCategoryInfo> list= standardLabelCategoryInfoService.list(
                new QueryWrapper<StandardLabelCategoryInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                .orderByDesc("orders")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);


        List<StandardLabelCategoryVO>  data=new ArrayList<>();

        list.forEach(item->{
            data.add(StandardLabelCategoryHelper.getHealthyLabelCategoryModel(item,userMap));
        });
        return TreeUtils.build(data);
    }

    @Override
    public List<StandardLabelCategoryVO> list(StandardLabelCategoryListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<StandardLabelCategoryInfo> list= standardLabelCategoryInfoService.list(new QueryWrapper<StandardLabelCategoryInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                .orderByDesc("orders")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);

        List<StandardLabelCategoryVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(StandardLabelCategoryHelper.getHealthyLabelCategoryModel(item,userMap));
        });
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StandardLabelCategoryVO save(StandardLabelCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelCategoryInfo entity=new StandardLabelCategoryInfo();
        entity.setProjectCode(params.getProjectCode());
        entity.setId(StandardLabelCategoryHelper.getHealthyLabelCategoryId());
        entity.setName(params.getName());
        entity.setPid(params.getPid());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        standardLabelCategoryInfoService.save(entity);
        return StandardLabelCategoryHelper.getHealthyLabelCategoryModel(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StandardLabelCategoryVO update(StandardLabelCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelCategoryInfo entity= standardLabelCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        standardLabelCategoryInfoService.updateById(entity);
        standardLabelCategoryCacheService.removeById(entity.getId());
        return StandardLabelCategoryHelper.getHealthyLabelCategoryModel(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        StandardLabelCategoryInfo entity= standardLabelCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        long num= standardLabelCategoryInfoService.count(new QueryWrapper<StandardLabelCategoryInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("pid",entity.getId())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"子分类，不能删除");
        }
        //todo:引用
        entity.setRowsStatus(YN.NO.getCode());
        standardLabelCategoryInfoService.updateById(entity);
        standardLabelCategoryCacheService.removeById(entity.getId());
    }

    @Override
    public StandardLabelCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        StandardLabelCategoryInfo entity= standardLabelCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> userIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO> userMap=adminUserClientService.getUserMapByIds(userIds);
        return StandardLabelCategoryHelper.getHealthyLabelCategoryModel(entity,userMap);
    }



}
