package com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushServeCategoryInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.PushServeCategoryInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.vos.PushServeCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.convert.PushServeCategoryConvert;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.PushServeCategoryBizService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class PushServeCategoryBizServiceImpl implements PushServeCategoryBizService {

    @Autowired
    private PushServeCategoryInfoService pushServeCategoryInfoService;

    @Override
    public List<PushServeCategoryVO> tree(PushServeCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushServeCategoryInfo> list= pushServeCategoryInfoService.list(
                new QueryWrapper<PushServeCategoryInfo>()
                        .eq("rows_status", YN.YES.getCode())
        );
        List<PushServeCategoryVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(PushServeCategoryConvert.getPushServeCategoryVO(item));
        });
        List<PushServeCategoryVO> data= TreeUtils.build(dataList);
        return data;
    }



    @Override
    public PushServeCategoryVO save(PushServeCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeCategoryInfo entity=new PushServeCategoryInfo();
        entity.setId(IdUtil.getSnowflake(5,1).nextIdStr());
        entity.setPid(params.getPid());
        entity.setName(params.getName());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        pushServeCategoryInfoService.save(entity);
        return PushServeCategoryConvert.getPushServeCategoryVO(entity);
    }

    @Override
    public PushServeCategoryVO update(PushServeCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeCategoryInfo entity=pushServeCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        pushServeCategoryInfoService.updateById(entity);
        return PushServeCategoryConvert.getPushServeCategoryVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        long num= pushServeCategoryInfoService.count(
                new QueryWrapper<PushServeCategoryInfo>()
                        .eq("pid",id)
                        .eq("rows_status",YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"有关联的子分类");
        }
        PushServeCategoryInfo entity=pushServeCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        pushServeCategoryInfoService.updateById(entity);
    }

    @Override
    public PushServeCategoryVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushServeCategoryInfo entity=pushServeCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return PushServeCategoryConvert.getPushServeCategoryVO(entity);
    }


}
