package com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.article.client.api.ArticleClientService;
import com.xxh.cloud.healthy.basic.client.api.EntityDiseaseConfigClientService;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigListClientDTO;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigSaveClientDTO;
import com.xxh.cloud.healthy.basic.client.api.constant.EntityDiseaseTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PuseServeMediaTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindEarlyDelayTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushServeConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.PushServeConfigInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.vos.PushServeConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.convert.PushServeConfigConvert;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.mapper.PushServeConfigQueryBizMapper;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.PushServeConfigBizService;
import com.xxh.cloud.healthy.basic.framework.modules.util.PushServeUtil;
import com.xxh.cloud.healthy.scale.framework.modules.client.api.ScaleConfigClientService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Slf4j
@Service
public class PushServeConfigBizServiceImpl implements PushServeConfigBizService {

    @Autowired
    private PushServeConfigInfoService pushServeConfigInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;

    @Autowired
    private PushServeConfigQueryBizMapper pushServeConfigQueryBizMapper;

    @Autowired
    private ArticleClientService articleClientService;
    @Autowired
    private ScaleConfigClientService scaleConfigClientService;
    @Autowired
    private EntityDiseaseConfigClientService entityDiseaseConfigClientService;




    @Override
    public PageData<PushServeConfigVO> query(PushServeConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<PushServeConfigInfo> page = pushServeConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PushServeConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getMediaId()),"media_id",params.getMediaId())
                        .like(StrUtil.isNotEmpty(params.getMediaName()),"media_name",params.getMediaName())
                        .eq(StrUtil.isNotEmpty(params.getMediaType()),"media_type",params.getMediaType())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getPushWay()),"push_way",params.getPushWay())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getLoopModel()),"loop_model",params.getLoopModel())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids", "," + params.getDiseaseId() + ",")
                        // zs20231021
                        .eq(StrUtil.isNotEmpty(params.getRemindWay()),"remind_way",params.getRemindWay())
                        // zs20240321
                        .eq(StrUtil.isNotEmpty(params.getExtend1()),"extend1",params.getExtend1())
                        .eq(StrUtil.isNotEmpty(params.getExtend2()),"extend2",params.getExtend2())
                        .eq(StrUtil.isNotEmpty(params.getExtend3()),"extend3",params.getExtend3())
                        .eq(StrUtil.isNotEmpty(params.getExtend4()),"extend4",params.getExtend4())
                        .eq(StrUtil.isNotEmpty(params.getExtend5()),"extend5",params.getExtend5())

                        .orderByDesc("create_date")
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        page.getRecords().forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.SERVICE.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                Set<String> _ids= StrJoinSplitUtils.split2Set(item.getDiseaseIds());
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.SERVICE.getCode() + "-" + item.getId());
                if (CollUtil.isNotEmpty(l)) {
                    _ids = new HashSet<>(l);
                }
                diseaseIds.addAll(_ids);
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        PageData<PushServeConfigVO> data= new PageConverter<PushServeConfigInfo, PushServeConfigVO>() {
            @Override
            public PushServeConfigVO converter(PushServeConfigInfo entity) {
                return PushServeConfigConvert.getPushServeConfigVO(entity,userMap,diseaseMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PushServeConfigVO> list(PushServeConfigListDTO params) {
        List<PushServeConfigInfo> list = pushServeConfigInfoService.list(new QueryWrapper<PushServeConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .in(CollUtil.isNotEmpty(params.getTypes()),"type",params.getTypes())
                        .eq(StrUtil.isNotEmpty(params.getMediaId()),"media_id",params.getMediaId())
                        .like(StrUtil.isNotEmpty(params.getMediaName()),"media_name",params.getMediaName())
                        .eq(StrUtil.isNotEmpty(params.getMediaType()),"media_type",params.getMediaType())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getPushWay()),"push_way",params.getPushWay())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getLoopModel()),"loop_model",params.getLoopModel())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids", "," + params.getDiseaseId() + ",")
                        // zs20231021
                        .eq(StrUtil.isNotEmpty(params.getRemindWay()),"remind_way",params.getRemindWay())
                        // zs20240321
                        .eq(StrUtil.isNotEmpty(params.getExtend1()),"extend1",params.getExtend1())
                        .eq(StrUtil.isNotEmpty(params.getExtend2()),"extend2",params.getExtend2())
                        .eq(StrUtil.isNotEmpty(params.getExtend3()),"extend3",params.getExtend3())
                        .eq(StrUtil.isNotEmpty(params.getExtend4()),"extend4",params.getExtend4())
                        .eq(StrUtil.isNotEmpty(params.getExtend5()),"extend5",params.getExtend5())
                        
                        .orderByDesc("create_date")
        );

        if(CollUtil.isEmpty(list)){
            return new ArrayList<>(0);
        }
        
        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        list.forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.SERVICE.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        List<PushServeConfigVO> data=new ArrayList<>();
        list.forEach(item->{
            PushServeConfigVO vo = PushServeConfigConvert.getPushServeConfigVO(item);
            // zs20240830 从新病种关系中获取，若不为空则以该数据为准
            List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.SERVICE.getCode() + "-" + item.getId());
            if (CollUtil.isNotEmpty(l)) {
                vo.setDiseaseIds(l);
            }
            data.add(vo);
        });
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PushServeConfigVO save(PushServeConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigInfo entity=new PushServeConfigInfo();
        entity.setId(IdUtil.getSnowflake(3,2).nextIdStr());
        entity.setName(params.getName());
        entity.setTitle(params.getTitle());
        entity.setType(params.getType());
        entity.setMediaType(params.getMediaType());
        entity.setMediaId(params.getMediaId());
        entity.setMediaName(params.getMediaName());
        entity.setRemark(params.getRemark());
        entity.setPushWay(params.getPushWay());
        if(params.getPushConfig()!=null){
            entity.setPushConfig(JSON.toJSONString(params.getPushConfig()));
        }
        entity.setBenchmarkDate(params.getBenchmarkDate());
        entity.setBenchmarkDateType(params.getBenchmarkDateType());
        entity.setEarlyDelayType(params.getEarlyDelayType());
        entity.setEarlyDelayNum(params.getEarlyDelayNum());
        entity.setRemindDayTime(params.getRemindDayTime());
        entity.setLoopModel(params.getLoopModel());
        entity.setLoopInterval(params.getLoopInterval());
        entity.setLoopMaxNum(params.getLoopMaxNum());
        entity.setStatus(YN.YES.getCode());
        entity.setRowsStatus(YN.YES.getCode());

        // zs:2023-09-18
        entity.setMediaImg(params.getMediaImg());
        entity.setMediaTitle(params.getMediaTitle());
        entity.setMediaRemark(params.getMediaRemark());
        entity.setRemindWay(params.getRemindWay());

        // zs20231116
        entity.setPushTarget(params.getPushTarget());

        // zs20231118 立即推送只提醒不加日程
        if (RemindEarlyDelayTypeEnum.Remind_Now.getCode().equals(params.getEarlyDelayType())) {
            entity.setRemindWay(RemindWayEnum.Remind.getCode());
        }

        // entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds())); // zs202312141517

        // zs20240321
        entity.setExtend1(params.getExtend1());
        entity.setExtend2(params.getExtend2());
        entity.setExtend3(params.getExtend3());
        entity.setExtend4(params.getExtend4());
        entity.setExtend5(params.getExtend5());

        // zs:20230919:推送服务快照key，md5摘要
        entity.setSnapshotKey(PushServeUtil.getPushServeSnapshotKey(entity));

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.SERVICE.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        pushServeConfigInfoService.save(entity);
        //todo:新增日志
        return PushServeConfigConvert.getPushServeConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PushServeConfigVO update(PushServeConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigInfo entity= pushServeConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }

        entity.setName(params.getName());
        entity.setMediaId(params.getMediaId());
        entity.setMediaName(params.getMediaName());
        entity.setMediaType(params.getMediaType());
        entity.setTitle(params.getTitle());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setPushWay(params.getPushWay());
        if(params.getPushConfig()!=null){
            entity.setPushConfig(JSON.toJSONString(params.getPushConfig()));
        }
        entity.setBenchmarkDate(params.getBenchmarkDate());
        entity.setBenchmarkDateType(params.getBenchmarkDateType());
        entity.setEarlyDelayType(params.getEarlyDelayType());
        entity.setEarlyDelayNum(params.getEarlyDelayNum());
        entity.setRemindDayTime(params.getRemindDayTime());
        entity.setLoopModel(params.getLoopModel());
        entity.setLoopInterval(params.getLoopInterval());
        entity.setLoopMaxNum(params.getLoopMaxNum());

        // zs:2023-09-18
        entity.setMediaImg(params.getMediaImg());
        entity.setMediaTitle(params.getMediaTitle());
        entity.setMediaRemark(params.getMediaRemark());
        entity.setRemindWay(params.getRemindWay());

        entity.setPushTarget(params.getPushTarget());

        if (RemindEarlyDelayTypeEnum.Remind_Now.getCode().equals(params.getEarlyDelayType())) {
            entity.setRemindWay(RemindWayEnum.Remind.getCode());
        }

        // entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds())); // zs202312141517

        // zs20240321
        entity.setExtend1(params.getExtend1());
        entity.setExtend2(params.getExtend2());
        entity.setExtend3(params.getExtend3());
        entity.setExtend4(params.getExtend4());
        entity.setExtend5(params.getExtend5());

        // zs:20230919:推送服务快照key, md5摘要
        entity.setSnapshotKey(PushServeUtil.getPushServeSnapshotKey(entity));

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.SERVICE.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        pushServeConfigInfoService.updateById(entity);
        //todo:修改日志
        return PushServeConfigConvert.getPushServeConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PushServeConfigVO updateStatus(PushServeConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigInfo entity= pushServeConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return PushServeConfigConvert.getPushServeConfigVO(entity);
        }
        entity.setStatus(params.getStatus());

        // zs:20230919:推送服务快照key, md5摘要
        entity.setSnapshotKey(PushServeUtil.getPushServeSnapshotKey(entity));

        pushServeConfigInfoService.updateById(entity);
        //todo:日志
        return PushServeConfigConvert.getPushServeConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushServeConfigInfo entity= pushServeConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        pushServeConfigInfoService.updateById(entity);
        //todo:日志
    }

    @Override
    public PushServeConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushServeConfigInfo entity= pushServeConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        Set<String> diseaseIds=StrJoinSplitUtils.split2Set(entity.getDiseaseIds());
        // zs20240830 从新病种关系中获取，若不为空则以该数据为准
        List<String> l = entityDiseaseConfigClientService.getDiseaseByEntityId(id);
        if (CollUtil.isNotEmpty(l)) { // zs20240830 原来病种数据为空则从新病种关系中获取
            diseaseIds = new HashSet<>(l);
        }
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        return PushServeConfigConvert.getPushServeConfigVO(entity,null,diseaseMap);
    }



    @Override
    public PageData<PushServeConfigVO> queryAndStatistic(PushServeConfigQueryDTO params) {
        IPage<PushServeConfigVO> page = pushServeConfigQueryBizMapper.queryAndStatistic(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PushServeConfigInfo>()
                        .eq("a.rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"a.name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"a.type",params.getType())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"a.title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"a.status",params.getStatus())
        );
        
        return new PageData<>(page.getRecords(), page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public List<PushServeConfigVO> listAndStatistic(PushServeConfigListDTO params) {
        List<PushServeConfigVO> list = pushServeConfigQueryBizMapper.listAndStatistic(new QueryWrapper<PushServeConfigInfo>()
                        .eq("a.rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"a.name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"a.type",params.getType())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"a.title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"a.status",params.getStatus())
        );

        return list;
    }

    @Override
    public PushServeConfigVO getFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushServeConfigInfo entity= pushServeConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        
        // 病种
        Set<String> diseaseIds=StrJoinSplitUtils.split2Set(entity.getDiseaseIds());
        // zs20240830 从新病种关系中获取，若不为空则以该数据为准
        List<String> l = entityDiseaseConfigClientService.getDiseaseByEntityId(id);
        if (CollUtil.isNotEmpty(l)) { // zs20240830 原来病种数据为空则从新病种关系中获取
            diseaseIds = new HashSet<>(l);
        }
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        PushServeConfigVO vo = PushServeConfigConvert.getPushServeConfigVO(entity,null,diseaseMap);

        // 媒体信息（文章/量表）
        if (PuseServeMediaTypeEnum.PuseServe_Article.getCode().equals(entity.getMediaType()) && StrUtil.isNotBlank(entity.getMediaId())) {
            vo.setArticleInfo(articleClientService.getById(entity.getMediaId()));
        } else if (PuseServeMediaTypeEnum.PuseServe_Scale.getCode().equals(entity.getMediaType()) && StrUtil.isNotBlank(entity.getMediaId())) {
            vo.setScaleInfo(scaleConfigClientService.getFullById(entity.getMediaId()));
        }

        return vo;
    }

}
