package com.xxh.cloud.healthy.basic.framework.modules.pushTask.convert;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.common.bean.ServPushConfigModel;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskActionConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskCustomTypeInfo;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskActionConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskCustomTypeVO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskActionConfigBO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskConfigBO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskMsgConfigBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class PushTaskConfigConvert {

    public static PushTaskConfigVO getPushTaskConfigVO(PushTaskConfigInfo entity){
        PushTaskConfigVO data=new PushTaskConfigVO();
        BeanUtil.copyProperties(entity,data,"handleButtons","taskPushConfig","actionList","diseaseIds");
        data.setHandleButtons(StrJoinSplitUtils.split2List(entity.getHandleButtons()));
        if(StrUtil.isNotEmpty(entity.getTaskPushConfig())){
            data.setTaskPushConfig(JSON.parseObject(entity.getTaskPushConfig(), ServPushConfigModel.class));
        }
        data.setDiseaseIds(StrJoinSplitUtils.split2List(entity.getDiseaseIds()));
        return data;
    }

    public static PushTaskConfigVO getPushTaskConfigVO(PushTaskConfigInfo entity,Map<String, StandardDiseaseBO> diseaseMap){
        PushTaskConfigVO data=getPushTaskConfigVO(entity);
        if(MapUtil.isNotEmpty(diseaseMap) && CollUtil.isNotEmpty(data.getDiseaseIds())){
            data.getDiseaseIds().forEach(id->{
                StandardDiseaseBO disease =diseaseMap.get(id);
                if(disease!=null){
                    data.addDisease(disease.getId(),disease.getName());
                }
            });
        }
        return data;
    }

    //
    public static PushTaskConfigVO getPushTaskConfigVO(PushTaskConfigInfo entity,Map<String, AdminUserBO> userMap,Map<String, StandardDiseaseBO> diseaseMap){
        PushTaskConfigVO data=getPushTaskConfigVO(entity);
        if(MapUtil.isNotEmpty(userMap)){
            if(StrUtil.isNotEmpty(entity.getCreator())){
                AdminUserBO user=userMap.get(entity.getCreator());
                if(user!=null){
                    data.setCreatorName(user.getUserName());
                }
            }

            if(StrUtil.isNotEmpty(entity.getUpdater())){
                AdminUserBO user=userMap.get(entity.getUpdater());
                if(user!=null){
                    data.setUpdaterName(user.getUserName());
                }
            }
        }

        if(MapUtil.isNotEmpty(diseaseMap) && CollUtil.isNotEmpty(data.getDiseaseIds())){
            data.getDiseaseIds().forEach(id->{
                StandardDiseaseBO disease =diseaseMap.get(id);
                if(disease!=null){
                    data.addDisease(disease.getId(),disease.getName());
                }
            });
        }

        return data;
    }


    public static PushTaskConfigVO getPushTaskConfigVO(PushTaskConfigInfo entity, List<PushTaskActionConfigInfo> actionList,Map<String, StandardDiseaseBO> diseaseMap){
        PushTaskConfigVO data=getPushTaskConfigVO(entity,diseaseMap);
        if(CollUtil.isNotEmpty(actionList)){
            actionList.forEach(item->{
                data.addActionList(getPushTaskActionConfigVO(item));
            });
        }
        return data;
    }



    public static PushTaskActionConfigVO getPushTaskActionConfigVO(PushTaskActionConfigInfo entity){
        PushTaskActionConfigVO data=new PushTaskActionConfigVO();
        data.setId(entity.getId());
        data.setName(entity.getName());
        data.setType(entity.getType());
        if(StrUtil.isNotEmpty(entity.getDataConfig())){
            // data.setDataConfig(JSON.parseObject(entity.getDataConfig(),Map.class));
            data.setDataConfig(JSON.parseObject(entity.getDataConfig(), new TypeReference<Map<String, Object>>(){}));
        }
        return data;
    }

    
    public static PushTaskConfigBO getPushTaskConfigBO(PushTaskConfigVO vo){
        if (null == vo) {
            return null;
        }
        PushTaskConfigBO data = new PushTaskConfigBO();
        BeanUtil.copyProperties(vo, data,"taskPushConfig", "actionList");
        if (null != vo.getTaskPushConfig()) {
            PushTaskMsgConfigBO bo = new PushTaskMsgConfigBO();
            BeanUtil.copyProperties(vo.getTaskPushConfig(), bo);
            data.setTaskPushConfig(bo);
        }
        if (null != vo.getActionList()) {
            List<PushTaskActionConfigBO> bos = new ArrayList<>();
            vo.getActionList().forEach(i -> {
                PushTaskActionConfigBO bo = new PushTaskActionConfigBO();
                BeanUtil.copyProperties(i, bo);
                bos.add(bo);
            });
            data.setActionList(bos);
        }
        return data;
    }

    public static PushTaskCustomTypeVO getPushTaskCustomTypeVO(PushTaskCustomTypeInfo entity){
        if (null == entity) {
            return null;
        }
        PushTaskCustomTypeVO data = new PushTaskCustomTypeVO();
        data.setId(entity.getId());
        data.setName(entity.getName());
        data.setType(entity.getType());
        data.setCreateDate(entity.getCreateDate());
        data.setCreator(entity.getCreator());
        data.setUpdateDate(entity.getUpdateDate());
        data.setUpdater(entity.getUpdater());
        return data;
    }

}
