package com.xxh.cloud.healthy.basic.framework.modules.pushTask.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.client.api.EntityDiseaseConfigClientService;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigListClientDTO;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigSaveClientDTO;
import com.xxh.cloud.healthy.basic.client.api.constant.EntityDiseaseTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PushWayEnum;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskActionConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.PushTaskActionConfigInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.PushTaskConfigInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskActionWxDataConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskCustomTypeVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.convert.PushTaskConfigConvert;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.service.PushTaskConfigBizService;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.service.PushTaskCustomTypeBizService;
import com.xxh.cloud.healthy.basic.pushserve.client.api.PushServeConfigClientService;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.dto.PushServeConfigClientQueryDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.DoctorTargetConfigClientService;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorTargetConfigBO;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class PushTaskConfigBizServiceImpl implements PushTaskConfigBizService {

    @Autowired
    private PushTaskConfigInfoService pushTaskConfigInfoService;
    @Autowired
    private PushTaskActionConfigInfoService pushTaskActionConfigInfoService;
    @Autowired
    private PushTaskCustomTypeBizService pushTaskCustomTypeBizService;

    @Autowired
    private AdminUserClientService adminUserClientService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;

    @Autowired
    private PushServeConfigClientService pushServeConfigClientService;
    @Autowired
    private DoctorTargetConfigClientService doctorTargetConfigClientService;
    @Autowired
    private EntityDiseaseConfigClientService entityDiseaseConfigClientService;


    @Override
    public PageData<PushTaskConfigVO> query(PushTaskConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<PushTaskConfigInfo> page = pushTaskConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PushTaskConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getUrgentLevel()),"urgent_level",params.getUrgentLevel())
                        .eq(StrUtil.isNotEmpty(params.getIsTaskPush()),"is_task_push",params.getIsTaskPush())
                        .eq(StrUtil.isNotEmpty(params.getIsLimitExeDate()),"is_limit_exe_date",params.getIsLimitExeDate())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                        .eq(StrUtil.isNotBlank(params.getTargetId()), "target_id", params.getTargetId())
                        .eq(StrUtil.isNotBlank(params.getManageType()), "manage_type", params.getManageType())
                        .eq(StrUtil.isNotBlank(params.getCustomType()), "custom_type", params.getCustomType())
                        .orderByDesc("create_date")
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        
        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        page.getRecords().forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.TASK.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> customTypes = new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                Set<String> _ids=StrJoinSplitUtils.split2Set(item.getDiseaseIds());
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.TASK.getCode() + "-" + item.getId());
                if (CollUtil.isNotEmpty(l)) {
                    _ids = new HashSet<>(l);
                }
                diseaseIds.addAll(_ids);
            }
            if (StrUtil.isNotBlank(item.getCustomType())) {
                customTypes.add(item.getCustomType());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        // 自定义类型
        Map<String, String> customTypeMap = new HashMap<>();
        PushTaskCustomTypeQueryDTO pushTaskCustomTypeQueryDTO = new PushTaskCustomTypeQueryDTO();
        pushTaskCustomTypeQueryDTO.setTypes(new ArrayList<>(customTypes));
        List<PushTaskCustomTypeVO> customTypeList = pushTaskCustomTypeBizService.list(pushTaskCustomTypeQueryDTO);
        if (CollectionUtil.isNotEmpty(customTypeList)) {
            customTypeList.forEach(i -> customTypeMap.put(i.getType(), i.getName()));
        }

        PageData<PushTaskConfigVO> data= new PageConverter<PushTaskConfigInfo, PushTaskConfigVO>() {
            @Override
            public PushTaskConfigVO converter(PushTaskConfigInfo entity) {
                PushTaskConfigVO vo = PushTaskConfigConvert.getPushTaskConfigVO(entity,userMap,diseaseMap);

                vo.setCustomTypeName(customTypeMap.get(vo.getCustomType()));

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PushTaskConfigVO> list(PushTaskConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushTaskConfigInfo> list = pushTaskConfigInfoService.list(new QueryWrapper<PushTaskConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getUrgentLevel()),"urgent_level",params.getUrgentLevel())
                        .eq(StrUtil.isNotEmpty(params.getIsTaskPush()),"is_task_push",params.getIsTaskPush())
                        .eq(StrUtil.isNotEmpty(params.getIsLimitExeDate()),"is_limit_exe_date",params.getIsLimitExeDate())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                        .eq(StrUtil.isNotBlank(params.getTargetId()), "target_id", params.getTargetId())
                        .eq(StrUtil.isNotBlank(params.getManageType()), "manage_type", params.getManageType())
                        .eq(StrUtil.isNotBlank(params.getCustomType()), "custom_type", params.getCustomType())
                        .orderByDesc("create_date")
                        .last(" limit 1000 ")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        list.forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.TASK.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        Set<String> diseaseIds=new HashSet<>();
        list.forEach(i -> {
            if(StrUtil.isNotEmpty(i.getDiseaseIds())){
                Set<String> _ids=StrJoinSplitUtils.split2Set(i.getDiseaseIds());
                diseaseIds.addAll(_ids);
            } else {
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.TASK.getCode() + "-" + i.getId());
                if (CollUtil.isNotEmpty(l)) {
                    diseaseIds.addAll(l);
                }
            }
        });
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Set<String> customTypes = new HashSet<>();
        list.forEach(i -> customTypes.add(i.getCustomType()));
        
        // 自定义类型
        Map<String, String> customTypeMap = new HashMap<>();
        PushTaskCustomTypeQueryDTO pushTaskCustomTypeQueryDTO = new PushTaskCustomTypeQueryDTO();
        pushTaskCustomTypeQueryDTO.setTypes(new ArrayList<>(customTypes));
        List<PushTaskCustomTypeVO> customTypeList = pushTaskCustomTypeBizService.list(pushTaskCustomTypeQueryDTO);
        if (CollectionUtil.isNotEmpty(customTypeList)) {
            customTypeList.forEach(i -> customTypeMap.put(i.getType(), i.getName()));
        }

        List<PushTaskConfigVO> data=new ArrayList<>();
        list.forEach(item->{
            PushTaskConfigVO vo = PushTaskConfigConvert.getPushTaskConfigVO(item);
            vo.getDiseaseIds().forEach(id->{
                StandardDiseaseBO disease =diseaseMap.get(id);
                if(disease!=null){
                    vo.addDisease(disease.getId(), disease.getName());
                }
            });

            vo.setCustomTypeName(customTypeMap.get(vo.getCustomType()));

            data.add(vo);
        });
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PushTaskConfigVO save(PushTaskConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        PushTaskConfigInfo entity=new PushTaskConfigInfo();
        entity.setId(IdUtil.getSnowflake(3,4).nextIdStr());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setUrgentLevel(params.getUrgentLevel());
        entity.setHandleButtons(StrJoinSplitUtils.join(params.getHandleButtons()));
        entity.setIsLimitExeDate(params.getIsLimitExeDate());
        entity.setLimitExeDateUnit(params.getLimitExeDateUnit());
        entity.setLimitExeDateNum(params.getLimitExeDateNum());
        entity.setIsTaskPush(params.getIsTaskPush());
        entity.setTaskPushWay(params.getTaskPushWay());
        entity.setTaskPushConfig(JSON.toJSONString(params.getTaskPushConfig()));
        entity.setRemindTitle(params.getRemindTitle());
        entity.setRemindDescribe(params.getRemindDescribe());
        entity.setStatus(StrUtil.isBlank(params.getStatus()) ? YN.YES.getCode() : params.getStatus());
        entity.setRowsStatus(YN.YES.getCode());
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        entity.setOpType(params.getOpType());
        entity.setTargetId(params.getTargetId());
        entity.setManageType(params.getManageType());
        entity.setCustomType(params.getCustomType());

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.TASK.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        List<PushTaskActionConfigInfo> list=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getActionList())){
            params.getActionList().forEach(item->{
                PushTaskActionConfigInfo entityItem=new PushTaskActionConfigInfo();
                entityItem.setId(IdUtil.getSnowflake(3,5).nextIdStr());
                entityItem.setPushTaskId(entity.getId());
                entityItem.setName(item.getName());
                entityItem.setType(item.getType());
                if(MapUtil.isNotEmpty(item.getDataConfig())){
                    entityItem.setDataConfig(JSON.toJSONString(item.getDataConfig()));
                }
                list.add(entityItem);
            });
        }
        pushTaskConfigInfoService.save(entity);
        if(list.size()>0){
            pushTaskActionConfigInfoService.saveBatch(list);
        }
        return  PushTaskConfigConvert.getPushTaskConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PushTaskConfigVO update(PushTaskConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        PushTaskConfigInfo entity= pushTaskConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setUrgentLevel(params.getUrgentLevel());
        entity.setHandleButtons(StrJoinSplitUtils.join(params.getHandleButtons()));
        entity.setIsLimitExeDate(params.getIsLimitExeDate());
        entity.setLimitExeDateUnit(params.getLimitExeDateUnit());
        entity.setLimitExeDateNum(params.getLimitExeDateNum());
        entity.setIsTaskPush(params.getIsTaskPush());
        entity.setTaskPushWay(params.getTaskPushWay());
        entity.setTaskPushConfig(JSON.toJSONString(params.getTaskPushConfig()));
        entity.setRemindTitle(params.getRemindTitle());
        entity.setRemindDescribe(params.getRemindDescribe());
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        entity.setOpType(params.getOpType());
        entity.setTargetId(params.getTargetId());
        entity.setManageType(params.getManageType());
        entity.setCustomType(params.getCustomType());

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.TASK.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        List<PushTaskActionConfigInfo> list=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getActionList())){
            params.getActionList().forEach(item->{
                PushTaskActionConfigInfo entityItem=new PushTaskActionConfigInfo();
                entityItem.setId(IdUtil.getSnowflake(3,5).nextIdStr());
                entityItem.setPushTaskId(entity.getId());
                entityItem.setName(item.getName());
                entityItem.setType(item.getType());
                if(MapUtil.isNotEmpty(item.getDataConfig())){
                    entityItem.setDataConfig(JSON.toJSONString(item.getDataConfig()));
                }
                list.add(entityItem);
            });
        }

        pushTaskConfigInfoService.updateById(entity);
        pushTaskActionConfigInfoService.remove(
                new QueryWrapper<PushTaskActionConfigInfo>()
                        .eq("push_task_id",entity.getId())
        );
        if(list.size()>0){
            pushTaskActionConfigInfoService.saveBatch(list);
        }
        return  PushTaskConfigConvert.getPushTaskConfigVO(entity);
    }

    @Override
    public PushTaskConfigVO updateStatus(PushTaskConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        PushTaskConfigInfo entity= pushTaskConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return PushTaskConfigConvert.getPushTaskConfigVO(entity);
        }
        entity.setStatus(params.getStatus());
        pushTaskConfigInfoService.updateById(entity);
        return  PushTaskConfigConvert.getPushTaskConfigVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushTaskConfigInfo entity= pushTaskConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        pushTaskConfigInfoService.updateById(entity);
    }

    @Override
    public PushTaskConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PushTaskConfigInfo entity= pushTaskConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        List<PushTaskActionConfigInfo> actionList= pushTaskActionConfigInfoService.list(
                new QueryWrapper<PushTaskActionConfigInfo>()
                        .eq("push_task_id",id)
        );

        if (CollectionUtil.isNotEmpty(actionList)) {
            List<String> wxPushServeIds = new ArrayList<>();
            actionList.forEach(i -> {
                // 微信
                if (PushWayEnum.PCW_WX.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getDataConfig())) {
                    PushTaskActionWxDataConfigVO wxconfig = JSONObject.parseObject(i.getDataConfig(), PushTaskActionWxDataConfigVO.class);
                    if (null != wxconfig.getPushId()) {
                        wxPushServeIds.add(wxconfig.getPushId());
                    }
                }
                // TODO 短信
                // TODO ai电话
            });

            if (CollectionUtil.isNotEmpty(wxPushServeIds)) {
                PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
                pushServeConfigClientQueryDTO.setIds(wxPushServeIds);
                List<PushServeConfigBO> pushServeConfigBOs = pushServeConfigClientService.list(pushServeConfigClientQueryDTO);
                if (CollectionUtil.isNotEmpty(pushServeConfigBOs)) {
                    Map<String, PushServeConfigBO> pushServeConfigBOMap = pushServeConfigBOs.stream().collect(Collectors.toMap(PushServeConfigBO::getId, i -> i));
                    actionList.forEach(i -> {
                        PushTaskActionWxDataConfigVO wxconfig = JSONObject.parseObject(i.getDataConfig(), PushTaskActionWxDataConfigVO.class);

                        PushServeConfigBO pushServeConfigBO = pushServeConfigBOMap.get(wxconfig.getPushId());
                        if (null != pushServeConfigBO) {
                            wxconfig.setPushServeConfig(pushServeConfigBO);
                            i.setDataConfig(JSON.toJSONString(wxconfig));
                        }
                    });
                }
            }
        }

        Set<String> diseaseIds  = StrJoinSplitUtils.split2Set(entity.getDiseaseIds());
        // zs20240830 从新病种关系中获取，若不为空则以该数据为准
        List<String> l = entityDiseaseConfigClientService.getDiseaseByEntityId(id);
        if (CollUtil.isNotEmpty(l)) { // zs20240830 原来病种数据为空则从新病种关系中获取
            diseaseIds = new HashSet<>(l);
        }
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        
        PushTaskConfigVO vo = PushTaskConfigConvert.getPushTaskConfigVO(entity,actionList,diseaseMap);

        if (StrUtil.isNotBlank(entity.getType())) {
            PushTaskCustomTypeQueryDTO pushTaskCustomTypeQueryDTO = new PushTaskCustomTypeQueryDTO();
            pushTaskCustomTypeQueryDTO.setType(entity.getType());
            PushTaskCustomTypeVO customTypeInfo = pushTaskCustomTypeBizService.getOne(pushTaskCustomTypeQueryDTO);
            if (null != customTypeInfo) {
                vo.setCustomTypeName(customTypeInfo.getName());
            }
        }

        if (null != vo && StrUtil.isNotBlank(vo.getTargetId())) {
            DoctorTargetConfigBO targetConfig = doctorTargetConfigClientService.getDetail(vo.getTargetId());
            if (null != targetConfig) {
                vo.setTargetCode(targetConfig.getCode());
                vo.setTargetName(targetConfig.getName());
            }
        }

        return vo;
    }

}
