package com.xxh.cloud.healthy.basic.framework.modules.pushTask.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.PushTaskCustomTypeInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.PushTaskCustomTypeInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.bean.vos.PushTaskCustomTypeVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.convert.PushTaskConfigConvert;
import com.xxh.cloud.healthy.basic.framework.modules.pushTask.service.PushTaskCustomTypeBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PushTaskCustomTypeBizServiceImpl implements PushTaskCustomTypeBizService {

    @Autowired
    private PushTaskCustomTypeInfoService pushTaskCustomTypeInfoService;

    @Override
    public List<PushTaskCustomTypeVO> list(PushTaskCustomTypeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushTaskCustomTypeInfo> list = pushTaskCustomTypeInfoService.list(new QueryWrapper<PushTaskCustomTypeInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .orderByDesc("create_date")
        );
        if(CollUtil.isEmpty(list)){
            return new ArrayList<>(0);
        }
        List<PushTaskCustomTypeVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(PushTaskConfigConvert.getPushTaskCustomTypeVO(item));
        });
        return data;
    }

    @Override
    public PushTaskCustomTypeVO save(PushTaskCustomTypeSaveDTO params) {
        ValidatorUtils.validateEntity(params);

        PushTaskCustomTypeInfo existEntity = pushTaskCustomTypeInfoService.getOne(new QueryWrapper<PushTaskCustomTypeInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("type", params.getType())
        );
        if (null != existEntity) {
            return PushTaskConfigConvert.getPushTaskCustomTypeVO(existEntity);
        }

        PushTaskCustomTypeInfo entity = new PushTaskCustomTypeInfo();
        entity.setId(IdUtil.getSnowflake(3,5).nextIdStr());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRowsStatus(YN.YES.getCode());

        pushTaskCustomTypeInfoService.save(entity);

        return PushTaskConfigConvert.getPushTaskCustomTypeVO(entity);
    }

    @Override
    public void update(PushTaskCustomTypeUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        
        PushTaskCustomTypeInfo entity = pushTaskCustomTypeInfoService.getOne(new QueryWrapper<PushTaskCustomTypeInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("id", params.getId())
        );

        if (null == entity) {
            throw new BusinessException("自定义类型数据不存在");
        }

        if (StrUtil.isNotBlank(params.getType())) {
            PushTaskCustomTypeInfo existEntity = pushTaskCustomTypeInfoService.getOne(new QueryWrapper<PushTaskCustomTypeInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("type", params.getType())
            );
            if (null != existEntity && !existEntity.getType().equals(params.getType())) {
                throw new BusinessException("类型已存在");
            }
        }

        entity.setType(params.getType());
        entity.setName(params.getName());

        pushTaskCustomTypeInfoService.updateById(entity);
    }

    @Override
    public void delById(String id) {
        PushTaskCustomTypeInfo entity = pushTaskCustomTypeInfoService.getOne(new QueryWrapper<PushTaskCustomTypeInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("id", id)
        );

        if (null == entity) {
            throw new BusinessException("自定义类型数据不存在");
        }

        entity.setRowsStatus(YN.NO.getCode());

        pushTaskCustomTypeInfoService.updateById(entity);
    }

    @Override
    public PushTaskCustomTypeVO getOne(PushTaskCustomTypeQueryDTO params) {
        if (StrUtil.isBlank(params.getId()) && StrUtil.isBlank(params.getType())) {
            return null;
        }
        PushTaskCustomTypeInfo entity = pushTaskCustomTypeInfoService.getOne(new QueryWrapper<PushTaskCustomTypeInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
        );

        return PushTaskConfigConvert.getPushTaskCustomTypeVO(entity);
    }

}
