package com.xxh.cloud.healthy.basic.framework.modules.remind.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.RemindConfig;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.RemindConfigService;
import com.xxh.cloud.healthy.basic.framework.modules.remind.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.remind.bean.vos.RemindConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.remind.convert.RemindConfigConvert;
import com.xxh.cloud.healthy.basic.framework.modules.remind.service.RemindConfigBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class RemindConfigBizServiceImpl implements RemindConfigBizService {

    @Autowired
    private RemindConfigService remindConfigService;

    @Autowired
    private AdminUserClientService adminUserClientService;


    @Override
    public PageData<RemindConfigVO> query(RemindConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<RemindConfig> page = remindConfigService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<RemindConfig>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getLevel()),"level",params.getLevel())
                        .eq(StrUtil.isNotEmpty(params.getTargetObjectType()),"target_object_type",params.getTargetObjectType())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getLoopModel()),"loop_model",params.getLoopModel())
                        .orderByDesc("create_date")
        );


        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> userIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);

        PageData<RemindConfigVO> data= new PageConverter<RemindConfig, RemindConfigVO>() {
            @Override
            public RemindConfigVO converter(RemindConfig entity) {
                return RemindConfigConvert.getRemindConfigVO(entity,userMap);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<RemindConfigVO> list(RemindConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RemindConfig> list = remindConfigService.list(new QueryWrapper<RemindConfig>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getLevel()),"level",params.getLevel())
                        .eq(StrUtil.isNotEmpty(params.getTargetObjectType()),"target_object_type",params.getTargetObjectType())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getLoopModel()),"loop_model",params.getLoopModel())
                        .orderByDesc("create_date")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<RemindConfigVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(RemindConfigConvert.getRemindConfigVO(item));
        });
        return data;
    }

    @Override
    public RemindConfigVO save(RemindConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfig entity=new RemindConfig();
        entity.setId(IdUtil.getSnowflake(3,1).nextIdStr());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setContent(params.getContent());
        entity.setLevel(params.getLevel());
        entity.setTargetObjectType(params.getTargetObjectType());
        entity.setBenchmarkDate(params.getBenchmarkDate());
        entity.setEarlyDelayType(params.getEarlyDelayType());
        entity.setEarlyDelayNum(params.getEarlyDelayNum());
        entity.setRemindDayTime(params.getRemindDayTime());
        entity.setLoopModel(params.getLoopModel());
        entity.setLoopInterval(params.getLoopInterval());
        entity.setLoopMaxNum(params.getLoopMaxNum());
        entity.setStatus(params.getStatus());
        entity.setRowsStatus(YN.YES.getCode());
        remindConfigService.save(entity);
        return RemindConfigConvert.getRemindConfigVO(entity);
    }

    @Override
    public RemindConfigVO update(RemindConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfig entity= remindConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setContent(params.getContent());
        entity.setLevel(params.getLevel());
        entity.setTargetObjectType(params.getTargetObjectType());
        entity.setBenchmarkDate(params.getBenchmarkDate());
        entity.setEarlyDelayType(params.getEarlyDelayType());
        entity.setEarlyDelayNum(params.getEarlyDelayNum());
        entity.setRemindDayTime(params.getRemindDayTime());
        entity.setLoopModel(params.getLoopModel());
        entity.setLoopInterval(params.getLoopInterval());
        entity.setLoopMaxNum(params.getLoopMaxNum());
        remindConfigService.updateById(entity);

        return RemindConfigConvert.getRemindConfigVO(entity);
    }

    @Override
    public RemindConfigVO updateStatus(RemindConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfig entity= remindConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return RemindConfigConvert.getRemindConfigVO(entity);
        }
        entity.setStatus(params.getStatus());
        remindConfigService.updateById(entity);
        return RemindConfigConvert.getRemindConfigVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RemindConfig entity= remindConfigService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        remindConfigService.updateById(entity);
    }

    @Override
    public RemindConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RemindConfig entity= remindConfigService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return RemindConfigConvert.getRemindConfigVO(entity);
    }
}
