package com.xxh.cloud.healthy.basic.framework.modules.risk.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.client.api.EntityDiseaseConfigClientService;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigListClientDTO;
import com.xxh.cloud.healthy.basic.client.api.bean.dto.EntityDiseaseConfigSaveClientDTO;
import com.xxh.cloud.healthy.basic.client.api.constant.EntityDiseaseTypeEnum;
import com.xxh.cloud.healthy.basic.framework.modules.base.entity.RiskConfigInfo;
import com.xxh.cloud.healthy.basic.framework.modules.base.service.RiskConfigInfoService;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigAddDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigDelDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigGetDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.vos.RiskConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.convert.RiskConfigConvert;
import com.xxh.cloud.healthy.basic.framework.modules.risk.service.RiskConfigBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class RiskConfigBizServiceImpl implements RiskConfigBizService {

    @Autowired
    private RiskConfigInfoService riskConfigInfoService;
    @Autowired
    private EntityDiseaseConfigClientService entityDiseaseConfigClientService;

    @Override
    public PageData<RiskConfigVO> page(RiskConfigQueryDTO params) {
        log.info("RiskConfigBizServiceImpl.page(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        IPage<RiskConfigInfo> page = riskConfigInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<RiskConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                .eq(StrUtil.isNotEmpty(params.getSysId()),"sys_id",params.getSysId())
                .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                .eq(StrUtil.isNotEmpty(params.getLevel()),"level",params.getLevel())
                .eq(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())
                .eq(StrUtil.isNotEmpty(params.getRelType()),"rel_type",params.getRelType())
                .orderByDesc("create_date")

        );

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        page.getRecords().forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.RISK.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        PageData<RiskConfigVO> data= new PageConverter<RiskConfigInfo, RiskConfigVO>() {
            @Override
            public RiskConfigVO converter(RiskConfigInfo entity) {
                RiskConfigVO vo = RiskConfigConvert.getRiskConfigVO(entity);
                // zs20240830 从新病种关系中获取，若不为空则以该数据为准
                List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.RISK.getCode() + "-" + vo.getId());
                if (CollUtil.isNotEmpty(l)) {
                    vo.setDiseaseIds(l);
                }
                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<RiskConfigVO> list(RiskConfigQueryDTO params) {
        log.info("RiskConfigBizServiceImpl.list(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        List<RiskConfigInfo> list = riskConfigInfoService.list(new QueryWrapper<RiskConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                .eq(StrUtil.isNotEmpty(params.getSysId()),"sys_id",params.getSysId())
                .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                .eq(StrUtil.isNotEmpty(params.getLevel()),"level",params.getLevel())
                .eq(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())
                .eq(StrUtil.isNotEmpty(params.getRelType()),"rel_type",params.getRelType())
                .orderByDesc("create_date")
        );

        // zs20240830 从新实体病种关系中获取病种信息
        Set<String> ids=new HashSet<>();
        list.forEach(item -> ids.add(item.getId()));
        EntityDiseaseConfigListClientDTO entityDiseaseQuery = new EntityDiseaseConfigListClientDTO();
        entityDiseaseQuery.setEntityType(EntityDiseaseTypeEnum.RISK.getCode());
        entityDiseaseQuery.setEntityIds(new ArrayList<>(ids));
        Map<String, List<String>> entityDiseaseMap = entityDiseaseConfigClientService.listDiseaseMap(entityDiseaseQuery);

        return list.stream().map(i -> {
            RiskConfigVO vo = RiskConfigConvert.getRiskConfigVO(i);
            // zs20240830 从新病种关系中获取，若不为空则以该数据为准
            List<String> l = entityDiseaseMap.get(EntityDiseaseTypeEnum.RISK.getCode() + "-" + vo.getId());
            if (CollUtil.isNotEmpty(l)) {
                vo.setDiseaseIds(l);
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public RiskConfigVO info(RiskConfigGetDTO params) {
        log.info("RiskConfigBizServiceImpl.info(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        
        RiskConfigInfo entity = riskConfigInfoService.getOne(new QueryWrapper<RiskConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .orderByDesc("create_date")
        );
        
        if(entity == null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        RiskConfigVO vo = RiskConfigConvert.getRiskConfigVO(entity);
        // zs20240830 从新病种关系中获取，若不为空则以该数据为准
        List<String> l = entityDiseaseConfigClientService.getDiseaseByEntityId(vo.getId());
        if (CollUtil.isNotEmpty(l)) {
            vo.setDiseaseIds(l);
        }
        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RiskConfigVO add(RiskConfigAddDTO params) {
        log.info("RiskConfigBizServiceImpl.add(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        RiskConfigInfo entity=new RiskConfigInfo();
        entity.setId(RiskConfigConvert.getRiskConfigInfoId());
        entity.setName(params.getName());
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        entity.setSysId(params.getSysId());
        entity.setRelType(params.getRelType());
        entity.setRelId(params.getRelId());
        entity.setType(params.getType());
        entity.setLevel(params.getLevel());
        if (StrUtil.isBlank(params.getStatus())) {
            entity.setStatus(YN.YES.getCode());
        } else {
            entity.setStatus(params.getStatus());
        }
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.RISK.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        riskConfigInfoService.save(entity);
        return RiskConfigConvert.getRiskConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RiskConfigVO edit(RiskConfigUpdateDTO params) {
        log.info("RiskConfigBizServiceImpl.edit(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        RiskConfigInfo entity= riskConfigInfoService.getById(params.getId());
        if(entity == null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        entity.setSysId(params.getSysId());
        entity.setType(params.getType());
        entity.setLevel(params.getLevel());
        entity.setStatus(params.getStatus());
        entity.setRelType(params.getRelType());
        entity.setRelId(params.getRelId());

        // zs20240830 保存病种
        EntityDiseaseConfigSaveClientDTO entityDisease = new EntityDiseaseConfigSaveClientDTO();
        entityDisease.setEntityType(EntityDiseaseTypeEnum.RISK.getCode());
        entityDisease.setEntityId(entity.getId());
        entityDisease.setDiseaseIds(params.getDiseaseIds());
        entityDiseaseConfigClientService.saveDiseaseByEntityId(entityDisease);

        riskConfigInfoService.updateById(entity);
        return RiskConfigConvert.getRiskConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(RiskConfigDelDTO params) {
        log.info("RiskConfigBizServiceImpl.remove(RiskConfigQueryDTO): >>>>>> {}", JSON.toJSONString(params));
        
        List<RiskConfigInfo> entity = riskConfigInfoService.list(new QueryWrapper<RiskConfigInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
        );
        if(CollectionUtil.isEmpty(entity)){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        
        riskConfigInfoService.update(new UpdateWrapper<RiskConfigInfo>()
                .set("rows_status", YN.NO.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
        );
    }

}
