package com.xxh.cloud.healthy.basic.sys.app.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.service.ArticleBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/article/info")
@Api(tags = "患者端,文章信息")
public class ArticleAppController {

    @Autowired
    private ArticleBizService articleBizService;

    @PostMapping("/query")
    @ApiOperation(value = "查询", response = ArticleVO.class)
    public Result<PageData<ArticleVO>> query(@RequestBody ArticleInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ArticleVO> data= articleBizService.query(params);
        return new Result<PageData<ArticleVO>>().ok(data);
    }

    @PostMapping("/info")
    @ApiOperation(value = "获取", response = ArticleVO.class)
    public  Result<ArticleVO>  info(@RequestBody ArticleInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleVO data= articleBizService.getById(params.getId());
        return new Result<ArticleVO>().ok(data);
    }

    @PostMapping("/listByIds")
    @ApiOperation(value = "列表查询并排序", response = ArticleVO.class)
    public Result<List<ArticleVO>> list(@RequestBody List<String> params) {
        ValidatorUtils.validateEntity(params);
        List<ArticleVO> data= articleBizService.listByIdsAndOrderByQueryIds(params);
        return new Result<List<ArticleVO>>().ok(data);
    }

}
