package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryGetDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.service.ArticleCategoryBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/healthy/article/category")
@Api(tags = "运营端,文章分类管理")
public class ArticleCategoryManagerController {

    @Autowired
    private ArticleCategoryBizService articleCategoryBizService;


    @SaCheckPermission("sys:healthy:article:category:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = ArticleCategoryVO.class)
    public Result<List<ArticleCategoryVO>> tree(@RequestBody ArticleCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ArticleCategoryVO> data= articleCategoryBizService.tree(params);
        return new Result<List<ArticleCategoryVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthy:article:category:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = ArticleCategoryVO.class)
    public Result<ArticleCategoryVO> save(@RequestBody ArticleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleCategoryVO data= articleCategoryBizService.save(params);
        return new Result<ArticleCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:healthy:article:category:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = ArticleCategoryVO.class)
    public Result<ArticleCategoryVO> update(@RequestBody ArticleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleCategoryVO data=articleCategoryBizService.update(params);
        return new Result<ArticleCategoryVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:article:category:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody ArticleCategoryGetDTO params) {
        ValidatorUtils.validateEntity(params);
        articleCategoryBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthy:article:category:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = ArticleCategoryVO.class)
    public  Result<ArticleCategoryVO>  info(@RequestBody ArticleCategoryGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleCategoryVO data= articleCategoryBizService.getById(params.getId());
        return new Result<ArticleCategoryVO>().ok(data);
    }




}
