package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleVO;
import com.xxh.cloud.healthy.basic.framework.modules.article.service.ArticleBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/article/info")
@Api(tags = "运营端,文章信息管理")
public class ArticleManagerController {

    @Autowired
    private ArticleBizService articleBizService;

    @SaCheckPermission("sys:healthy:article:info:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = ArticleVO.class)
    public Result<PageData<ArticleVO>> query(@RequestBody ArticleInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ArticleVO> data= articleBizService.query(params);
        return new Result<PageData<ArticleVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthy:article:info:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = ArticleVO.class)
    public Result<ArticleVO> save(@RequestBody ArticleInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleVO data= articleBizService.save(params);
        return new Result<ArticleVO>().ok(data);
    }

    @SaCheckPermission("sys:healthy:article:info:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = ArticleVO.class)
    public Result<ArticleVO> update(@RequestBody ArticleInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleVO data=articleBizService.update(params);
        return new Result<ArticleVO>().ok(data);
    }

    @SaCheckPermission("sys:healthy:article:info:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改文章状态", response = String.class)
    public Result<String> updateStatus(@RequestBody ArticleInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        articleBizService.updateStatus(params);
        return new Result<String>().ok("ok");
    }


    @SaCheckPermission("sys:healthy:article:info:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody ArticleInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        articleBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthy:article:info:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = ArticleVO.class)
    public  Result<ArticleVO>  info(@RequestBody ArticleInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ArticleVO data= articleBizService.getById(params.getId());
        return new Result<ArticleVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:article:info:query")
    @PostMapping("/listByIds")
    @ApiOperation(value = "列表查询并排序", response = ArticleVO.class)
    public Result<List<ArticleVO>> listByIds(@RequestBody List<String> params) {
        ValidatorUtils.validateEntity(params);
        List<ArticleVO> data= articleBizService.listByIdsAndOrderByQueryIds(params);
        return new Result<List<ArticleVO>>().ok(data);
    }



}
