package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryGetDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.dto.ArticleCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.article.bean.vos.ArticleCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategoryGetDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.PushServeCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.vos.PushServeCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.PushServeCategoryBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/healthybasic/pushServeCategory")
@Api(tags = "运营端,推送服务分类管理")
public class PushServeCategoryManagerController {

    @Autowired
    private PushServeCategoryBizService pushServeCategoryBizService;


    @SaCheckPermission("sys:healthybasic:pushServeCategory:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = PushServeCategoryVO.class)
    public Result<List<PushServeCategoryVO>> tree(@RequestBody PushServeCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushServeCategoryVO> data= pushServeCategoryBizService.tree(params);
        return new Result<List<PushServeCategoryVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthybasic:pushServeCategory:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = PushServeCategoryVO.class)
    public Result<PushServeCategoryVO> save(@RequestBody PushServeCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeCategoryVO data= pushServeCategoryBizService.save(params);
        return new Result<PushServeCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:healthybasic:pushServeCategory:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = PushServeCategoryVO.class)
    public Result<PushServeCategoryVO> update(@RequestBody PushServeCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeCategoryVO data=pushServeCategoryBizService.update(params);
        return new Result<PushServeCategoryVO>().ok(data);
    }


    @SaCheckPermission("sys:healthybasic:pushServeCategory:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody PushServeCategoryGetDTO params) {
        ValidatorUtils.validateEntity(params);
        pushServeCategoryBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthybasic:pushServeCategory:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = PushServeCategoryVO.class)
    public  Result<PushServeCategoryVO>  info(@RequestBody PushServeCategoryGetDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeCategoryVO data= pushServeCategoryBizService.getById(params.getId());
        return new Result<PushServeCategoryVO>().ok(data);
    }



}
