package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.bean.vos.PushServeConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.pushServe.service.PushServeConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/healthybasic/pushServeConfig")
@Api(tags = "运营端,推送服务配置管理")
public class PushServeConfigManagerController {

    @Autowired
    private PushServeConfigBizService pushServeConfigBizService;

    // @SaCheckPermission("sys:healthybasic:pushServeConfig:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = PushServeConfigVO.class)
    public Result<PageData<PushServeConfigVO>> query(@RequestBody PushServeConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<PushServeConfigVO> data= pushServeConfigBizService.query(params);
        return new Result<PageData<PushServeConfigVO>>().ok(data);
    }


    // @SaCheckPermission("sys:healthybasic:pushServeConfig:list")
    @PostMapping("/list")
    @ApiOperation(value = "查询", response = PushServeConfigVO.class)
    public Result<List<PushServeConfigVO>> list(@RequestBody PushServeConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushServeConfigVO> data= pushServeConfigBizService.list(params);
        return new Result<List<PushServeConfigVO>>().ok(data);
    }


    // @SaCheckPermission("sys:healthybasic:pushServeConfig:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = PushServeConfigVO.class)
    public  Result<PushServeConfigVO> save(@RequestBody PushServeConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigVO data= pushServeConfigBizService.save(params);
        return new Result<PushServeConfigVO>().ok(data);
    }

    // @SaCheckPermission("sys:healthybasic:pushServeConfig:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = PushServeConfigVO.class)
    public Result<PushServeConfigVO> update(@RequestBody PushServeConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigVO data= pushServeConfigBizService.update(params);
        return new Result<PushServeConfigVO>().ok(data);
    }

    // @SaCheckPermission("sys:healthybasic:pushServeConfig:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改状态", response = PushServeConfigVO.class)
    public Result<PushServeConfigVO> updateStatus(@RequestBody PushServeConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigVO data= pushServeConfigBizService.updateStatus(params);
        return new Result<PushServeConfigVO>().ok(data);
    }


    // @SaCheckPermission("sys:healthybasic:pushServeConfig:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody PushServeConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        pushServeConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    // @SaCheckPermission("sys:healthybasic:pushServeConfig:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = PushServeConfigVO.class)
    public Result<PushServeConfigVO> info(@RequestBody PushServeConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        PushServeConfigVO data= pushServeConfigBizService.getById(params.getId());
        return new Result<PushServeConfigVO>().ok(data);
    }




    // @SaCheckPermission("sys:healthybasic:pushServeConfig:query")
    @PostMapping("/queryAndStatistic")
    @ApiOperation(value = "查询并统计触发次数", response = PushServeConfigVO.class)
    public Result<PageData<PushServeConfigVO>> queryAndStatistic(@RequestBody PushServeConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<PushServeConfigVO> data= pushServeConfigBizService.queryAndStatistic(params);
        return new Result<PageData<PushServeConfigVO>>().ok(data);
    }


    // @SaCheckPermission("sys:healthybasic:pushServeConfig:list")
    @PostMapping("/listAndStatistic")
    @ApiOperation(value = "列表查询并统计触发次数", response = PushServeConfigVO.class)
    public Result<List<PushServeConfigVO>> listAndStatistic(@RequestBody PushServeConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PushServeConfigVO> data= pushServeConfigBizService.listAndStatistic(params);
        return new Result<List<PushServeConfigVO>>().ok(data);
    }


}
