package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.remind.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.remind.bean.vos.RemindConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.remind.service.RemindConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


//@SaCheckPermission("sys:healthy:basic:foodCategory")
@Log4j2
@RestController
@RequestMapping("/sys/healthybasic/remindConfig")
@Api(tags = "运营端,提醒配置管理")
public class RemindConfigManagerController {

    @Autowired
    private RemindConfigBizService remindConfigBizService;




    @SaCheckPermission("sys:healthybasic:remindConfig:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = RemindConfigVO.class)
    public Result<PageData<RemindConfigVO>> query(@RequestBody RemindConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<RemindConfigVO> data= remindConfigBizService.query(params);
        return new Result<PageData<RemindConfigVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthybasic:remindConfig:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = RemindConfigVO.class)
    public Result<List<RemindConfigVO>> list(@RequestBody RemindConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RemindConfigVO> data= remindConfigBizService.list(params);
        return new Result<List<RemindConfigVO>>().ok(data);
    }



    @SaCheckPermission("sys:healthybasic:remindConfig:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RemindConfigVO.class)
    public  Result<RemindConfigVO> save(@RequestBody RemindConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfigVO data= remindConfigBizService.save(params);
        return new Result<RemindConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:healthybasic:remindConfig:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RemindConfigVO.class)
    public Result<RemindConfigVO> update(@RequestBody RemindConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfigVO data= remindConfigBizService.update(params);
        return new Result<RemindConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:healthybasic:remindConfig:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改状态", response = RemindConfigVO.class)
    public Result<RemindConfigVO> updateStatus(@RequestBody RemindConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfigVO data= remindConfigBizService.updateStatus(params);
        return new Result<RemindConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:healthybasic:remindConfig:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RemindConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        remindConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthybasic:remindConfig:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RemindConfigVO.class)
    public Result<RemindConfigVO> info(@RequestBody RemindConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        RemindConfigVO data= remindConfigBizService.getById(params.getId());
        return new Result<RemindConfigVO>().ok(data);
    }



}
