package com.xxh.cloud.healthy.basic.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigAddDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigDelDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigGetDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigQueryDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.dto.RiskConfigUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.bean.vos.RiskConfigVO;
import com.xxh.cloud.healthy.basic.framework.modules.risk.service.RiskConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/basic/riskConfig")
@Api(tags = "运营端,风险配置管理")
public class RiskConfigManagerController {

    @Autowired
    private RiskConfigBizService riskConfigBizService;

    // @SaCheckPermission("sys:healthy:basic:risk:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = RiskConfigVO.class)
    public Result<PageData<RiskConfigVO>> query(@RequestBody RiskConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<RiskConfigVO> data= riskConfigBizService.page(params);
        return new Result<PageData<RiskConfigVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthy:basic:risk:query")
    @PostMapping("/list")
    @ApiOperation(value = "查询", response = RiskConfigVO.class)
    public Result<List<RiskConfigVO>> list(@RequestBody RiskConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RiskConfigVO> data= riskConfigBizService.list(params);
        return new Result<List<RiskConfigVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthy:basic:risk:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RiskConfigVO.class)
    public Result<RiskConfigVO> save(@RequestBody RiskConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);
        RiskConfigVO data= riskConfigBizService.add(params);
        return new Result<RiskConfigVO>().ok(data);
    }

    // @SaCheckPermission("sys:healthy:basic:risk:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RiskConfigVO.class)
    public Result<RiskConfigVO> update(@RequestBody RiskConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RiskConfigVO data= riskConfigBizService.edit(params);
        return new Result<RiskConfigVO>().ok(data);
    }

    // @SaCheckPermission("sys:healthy:basic:risk:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RiskConfigDelDTO params) {
        ValidatorUtils.validateEntity(params);
        riskConfigBizService.remove(params);
        return new Result<String>().ok("ok");
    }

    // @SaCheckPermission("sys:healthy:basic:risk:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RiskConfigVO.class)
    public Result<RiskConfigVO> info(@RequestBody RiskConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        RiskConfigVO data= riskConfigBizService.info(params);
        return new Result<RiskConfigVO>().ok(data);
    }



}
