package com.xxh.cloud.healthy.basic.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryByIdDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategorySaveDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryTreeDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.StandardLabelCategoryUpdateDTO;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelCategoryVO;
import com.xxh.cloud.healthy.basic.framework.modules.label.service.StandardLabelCategoryBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


//@SaCheckPermission("sys:healthy:basic:labelCategory")
@Log4j2
@RestController
@RequestMapping("/sys/standard/categoryLabel")
@Api(tags = "运营端,标准标签分类管理")
public class StandardLabelCategoryManagerController {
    
    @Autowired
    private StandardLabelCategoryBizService standardLabelCategoryBizService;

    @SaCheckPermission("sys:standard:categoryLabel:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = StandardLabelCategoryVO.class)
    public Result<List<StandardLabelCategoryVO>> tree(@RequestBody StandardLabelCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<StandardLabelCategoryVO> data= standardLabelCategoryBizService.tree(params);
        return new Result<List<StandardLabelCategoryVO>>().ok(data);
    }

    @SaCheckPermission("sys:standard:categoryLabel:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = StandardLabelCategoryVO.class)
    public Result<StandardLabelCategoryVO> save(@RequestBody StandardLabelCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelCategoryVO data= standardLabelCategoryBizService.save(params);
        return new Result<StandardLabelCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:standard:categoryLabel:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = StandardLabelCategoryVO.class)
    public Result<StandardLabelCategoryVO> update(@RequestBody StandardLabelCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelCategoryVO data= standardLabelCategoryBizService.update(params);
        return new Result<StandardLabelCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:standard:categoryLabel:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody StandardLabelCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        standardLabelCategoryBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:standard:categoryLabel:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = StandardLabelCategoryVO.class)
    public Result<StandardLabelCategoryVO> info(@RequestBody StandardLabelCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelCategoryVO data= standardLabelCategoryBizService.infoById(params.getId());
        return new Result<StandardLabelCategoryVO>().ok(data);
    }




}
