package com.xxh.cloud.healthy.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollectionUtil;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.ModeEnum;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.dto.*;
import com.xxh.cloud.healthy.basic.framework.modules.label.bean.vos.StandardLabelVO;
import com.xxh.cloud.healthy.basic.framework.modules.label.service.StandardLabelBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("sys:healthy:basic:labelInfo")
@Log4j2
@RestController
@RequestMapping("/sys/standard/labelInfo")
@Api(tags = "运营端,标准标签管理")
public class StandardLabelManagerController {

    @Autowired
    private StandardLabelBizService standardLabelBizService;

    @SaCheckPermission("sys:standard:labelInfo:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = StandardLabelVO.class)
    public Result<PageData<StandardLabelVO>> query(@RequestBody StandardLabelInfoQueryParam params) {
        ValidatorUtils.validateEntity(params);
        PageData<StandardLabelVO> data= standardLabelBizService.query(params);
        return new Result<PageData<StandardLabelVO>>().ok(data);
    }

    // @SaCheckPermission("sys:standard:labelInfo:list")  // TODO
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = StandardLabelVO.class)
    public Result<List<StandardLabelVO>> list(@RequestBody StandardLabelInfoListParam params) {
        ValidatorUtils.validateEntity(params);
        if (ModeEnum.RequestModeEnum.Doctor.getCode().equals(params.getRequestMode()) && CollectionUtil.isEmpty(params.getDiseaseIds())) {
            return new Result<List<StandardLabelVO>>().ok(new ArrayList<>(0));
        }
        List<StandardLabelVO> data= standardLabelBizService.list(params);
        return new Result<List<StandardLabelVO>>().ok(data);
    }

    @SaCheckPermission("sys:standard:labelInfo:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = StandardLabelVO.class)
    public  Result<StandardLabelVO> save(@RequestBody StandardLabelInfoSaveParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelVO data= standardLabelBizService.save(params);
        return new Result<StandardLabelVO>().ok(data);
    }

    @SaCheckPermission("sys:standard:labelInfo:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = StandardLabelVO.class)
    public Result<StandardLabelVO> update(@RequestBody StandardLabelInfoUpdateParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelVO data= standardLabelBizService.update(params);
        return new Result<StandardLabelVO>().ok(data);
    }

    @SaCheckPermission("sys:standard:labelInfo:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改状态", response = StandardLabelVO.class)
    public Result<StandardLabelVO> updateStatus(@RequestBody StandardLabelInfoUpdateStatusParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelVO data= standardLabelBizService.updateStatus(params);
        return new Result<StandardLabelVO>().ok(data);
    }


    @SaCheckPermission("sys:standard:labelInfo:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody StandardLabelInfoByIdParam params) {
        ValidatorUtils.validateEntity(params);
        standardLabelBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:standard:labelInfo:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = StandardLabelVO.class)
    public Result<StandardLabelVO> info(@RequestBody StandardLabelInfoByIdParam params) {
        ValidatorUtils.validateEntity(params);
        StandardLabelVO data= standardLabelBizService.infoById(params.getId());
        return new Result<StandardLabelVO>().ok(data);
    }





}
