package com.xxh.cloud.common.util;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

// 随访时间处理
public class FollowUpRegexUtil {
    
    /**
	 * 出院后1周
	 * 出院后2周
	 * 出院后2周。
	 * 出院后两周。
	 * 出院2周。
     * 1周
     * 一周后
     * 两周后
     * 半月后
     * 半月内。
     * 一月内
	 * 出院1个月
	 * 1个月。
	 * 一月
	 * 2024-6-10
	 * 2024.06.25
	 * 2024,6,25
     * 2024年6月30日
     * 
     * 每月底
     * 每月初
     * 每周末
     * 
     * 半年
     * 一年
     * 
	 * 产后一月
	 * 每月21-30日
	 */
    
    public static void main(String[] args) {
        // String str = "每月";
        // getFollowupDateStandard(DateUtil.date(), str);

        System.out.println(getFollowUpBaseDate("dgaghrh,，fadg出院dgraeg"));
    }

    public static String getFollowUpBaseDate(String content) {
        String result = "HZ";
        try {
            List<String> list = RegexUtil.listPatternStr(content, "出院");
            if (CollUtil.isNotEmpty(list)) {
                for (String item : list) {
                    System.out.println("匹配出的随访基准时间：" + item);
                    result = "CY";
                }
            }
        } catch(Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public static String getFollowupDateStandard(Date baseDate, String followupDateRaw) {
        if (null == baseDate || StrUtil.isBlank(followupDateRaw)) {
            return null;
        }

        // 基准时间设置到8点
        baseDate = DateUtil.offsetHour(DateUtil.parseDate(DateUtil.formatDate(baseDate)), 8);

        String result = null;
        try {
            List<String> list = RegexUtil.listPatternStr(followupDateRaw, "(([1-9]|[半|一|二|两|三|四|五|六|七|八|九|十])+(个){0,1}(日|天|周|月|年))|((20)[2-9]{2}(-|.|年)[0-9]{1,2}(-|.|月)[0-9]{1,2})|((每|下|本)(周|月)(初|末|底)?)");
            for (String item : list) {
                System.out.println("匹配出的随访时间：" + item);
                result = getDate(baseDate, item);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        System.out.println("解析出的随访时间：" + result);

        if (StrUtil.isNotBlank(result)) {
            Date resultDate = DateUtil.parseDateTime(result);
            if (DateUtil.offsetDay(DateUtil.date(), 2).before(resultDate)) {
                result = DateUtil.offsetDay(resultDate, -1).toString();
            }
        }
        return result;
    }

    private static String getDate(Date baseDate, String text) {

        // 解析：天
        if (text.endsWith("天")) {
            String value = text.substring(0, text.length() - 1);
            BigDecimal num = getDayNum(value);
            DateTime date = DateUtil.offsetDay(baseDate, num.intValue());
            return date.toString();
        }

        // 解析：本周末/下周末
        if (text.endsWith("周末")) {
            int offset = 0;
            if (text.contains("下")) {
                offset = 1;
            }
            DateTime date = DateUtil.endOfWeek(DateUtil.offsetWeek(baseDate, offset));
            return date.toString();
        }

        // 解析：本月末/下月末
        if (text.endsWith("月末") || text.endsWith("月底")) {
            int offset = 0;
            if (text.contains("下")) {
                offset = 1;
            }
            DateTime date = DateUtil.endOfMonth(DateUtil.offsetMonth(baseDate, offset));
            return date.toString();
        }

        // 解析：下月初
        if (text.endsWith("月初")) {
            DateTime date = DateUtil.beginOfMonth(DateUtil.offsetMonth(baseDate, 1));
            return date.toString();
        }

        // 解析：周
        if (text.endsWith("周")) {
            BigDecimal unit = new BigDecimal("7");
            String value = text.substring(0, text.length() - 1);
            if (value.endsWith("个")) {
                value = value.substring(0, value.length() - 1);
            }
            BigDecimal num = getDayNum(value);
            BigDecimal days = num.multiply(unit);

            DateTime date = DateUtil.offsetDay(baseDate, days.intValue());

            return date.toString();
        }
        
        // 解析：月
        if (text.endsWith("月")) {
            BigDecimal unit = new BigDecimal("28");
            String value = text.substring(0, text.length() - 1);
            if (value.endsWith("个")) {
                value = value.substring(0, value.length() - 1);
            }
            BigDecimal num = getDayNum(value);
            BigDecimal days = num.multiply(unit);

            DateTime date = DateUtil.offsetDay(baseDate, days.intValue());

            return date.toString();
        }

        // 解析：年
        if (text.endsWith("年")) {
            BigDecimal unit = new BigDecimal("365");
            String value = text.substring(0, text.length() - 1);
            BigDecimal num = getDayNum(value);
            BigDecimal days = num.multiply(unit);

            DateTime date = DateUtil.offsetDay(baseDate, days.intValue());

            return date.toString();
        }

        // 解析：2024-6-10  2024.06.25
        if (text.contains(",") || text.contains(".") || text.contains("-") || text.contains("年") || text.contains("月")) {
            text = text.replace(",", "-");
            text = text.replace(".", "-");
            text = text.replace("年", "-");
            text = text.replace("月", "-");

            List<String> result = new ArrayList<>();
            String[] ss = text.split("-");
            for (String s : ss) {
                if (s.length() == 1) {
                    result.add("0" + s);
                } else {
                    result.add(s);
                }
            }
            if (CollUtil.isNotEmpty(result) && result.size() == 3) {
                String date = result.get(0) + "-" + result.get(1) + "-" + result.get(2);
                return date + " 08:00:00";
            }
        }
        
        return null;
    }

    private static BigDecimal getDayNum(String value) {
        if (StrUtil.isNotBlank(value)) {
            if (value.contains("半")) {
                return new BigDecimal("0.5");
            }
            if (value.contains("一") || value.contains("每")) {
                value = value.replace("一", "1");
                value = value.replace("每", "1");
            }
            if (value.contains("二") || value.contains("两")) {
                value = value.replace("二", "2");
                value = value.replace("两", "2");
            }
            if (value.contains("三")) {
                value = value.replace("三", "3");
            }
            if (value.contains("四")) {
                value = value.replace("四", "4");
            }
            if (value.contains("五")) {
                value = value.replace("五", "5");
            }
            if (value.contains("六")) {
                value = value.replace("六", "6");
            }
            if (value.contains("七")) {
                value = value.replace("七", "7");
            }
            if (value.contains("八")) {
                value = value.replace("八", "8");
            }
            if (value.contains("九")) {
                value = value.replace("九", "9");
            }
            if ("十".equals(value)) {
                value = value.replace("十", "10");
            }
            if (value.startsWith("十")) {
                value = value.replace("十", "1");
            }
            if (value.endsWith("十")) {
                value = value.replace("十", "0");
            }
            if (value.contains("十")) {
                value = value.replace("十", "");
            }

            return new BigDecimal(value);
        }

        return null;
    }

}
