package com.xxh.cloud.common.util;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.xxh.cloud.common.bean.MaxMinValue;

public class RegexUtil {
    
    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 12--24
     * 23.4--54.1
     * 0.38 --4.34
     * 0.38 --  4.34
     *
     * 数字可以是整数也可以是小数；双横杠；横杠前后可以有空格（最多一个）
     * 
     * @param str 给定需要匹配的字符串
     * @return 返回匹配到的字符串集合
     */
    public static List<String> pattern1(String str) {
        return RegexUtil.listPatternStr(str, "[0-9]+\\.?[0-9]*\\s*--\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern1(String str) {
        return RegexUtil.pattern1(str).size() > 0 ? true : false;
    }

    public static int pattern1Count(String str) {
        return RegexUtil.pattern1(str).size();
    }


    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 滤泡期：0.4-2.5；排卵期：1.2-24.8黄体期：5.5-24.8； 绝经期：0.4-1.4滤泡期：0.4-2.5
     * 滤泡期：<0.1--0.6      黄体期：1.0--15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2
     * 卧位：25--60||立位：50--120||
     * 
     * 非数字只匹配二个字符,且不匹配|；
     * 冒号可以是中文也可以是英文；
     * 单横杠或双横杠；
     * 数字可以是整数也可以是小数；
     * 小于号或小于等于号最多有一个
     * 
     * @param str
     * @return
     */
    public static List<String> pattern2(String str) {
        return RegexUtil.listPatternStr(str, "[^|]\\D{1}\\s*(：|:)\\s*(<|≤)?[0-9]+\\.?[0-9]*\\s*(-|--)\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern2(String str) {
        return RegexUtil.pattern2(str).size() > 0 ? true : false;
    }

    public static int pattern2Count(String str) {
        return RegexUtil.pattern2(str).size();
    }

    
    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 滤泡期：0.4-2.5；排卵期：1.2-24.8黄体期：5.5-24.8； 绝经期：0.4-1.4滤泡期：0.4-2.5
     * 滤泡期：<0.1--0.6      黄体期：1.0--15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2
     * 卧位：25--60||立位：50--120||
     * 
     * 非数字只匹配三个字符,且不匹配|；
     * 冒号可以是中文也可以是英文；
     * 单横杠或双横杠；
     * 数字可以是整数也可以是小数；
     * 小于号或小于等于号最多有一个
     * 
     * @param str
     * @return
     */
    public static List<String> pattern3(String str) {
        return RegexUtil.listPatternStr(str, "[^|]\\D{2}\\s*(：|:)\\s*(<|≤)?[0-9]+\\.?[0-9]*\\s*(-|--)\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern3(String str) {
        return RegexUtil.pattern3(str).size() > 0 ? true : false;
    }

    public static int pattern3Count(String str) {
        return RegexUtil.pattern3(str).size();
    }

    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 滤泡期：0.4-2.5；排卵期：1.2-24.8黄体期：5.5-24.8； 绝经期：0.4-1.4滤泡期：0.4-2.5
     * 滤泡期：<0.1--0.6      黄体期：1.0--15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2
     * 卧位：25--60||立位：50--120||
     * 
     * 非数字只匹配四个字符,且不匹配|；
     * 冒号可以是中文也可以是英文；
     * 单横杠或双横杠；
     * 数字可以是整数也可以是小数；
     * 小于号或小于等于号最多有一个
     * 
     * @param str
     * @return
     */
    public static List<String> pattern4(String str) {
        return RegexUtil.listPatternStr(str, "[^|]\\D{3}\\s*(：|:)\\s*(<|≤)?[0-9]+\\.?[0-9]*\\s*(-|--)\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern4(String str) {
        return RegexUtil.pattern4(str).size() > 0 ? true : false;
    }

    public static int pattern4Count(String str) {
        return RegexUtil.pattern4(str).size();
    }

    
    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 滤泡期：0.4-2.5；排卵期：1.2-24.8黄体期：5.5-24.8； 绝经期：0.4-1.4滤泡期：0.4-2.5
     * 滤泡期：<0.1--0.6      黄体期：1.0--15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2
     * 卧位：25--60||立位：50--120||
     * 
     * 非数字只匹配五个字符,且不匹配|；
     * 冒号可以是中文也可以是英文；
     * 单横杠或双横杠；
     * 数字可以是整数也可以是小数；
     * 小于号或小于等于号最多有一个
     * 
     * @param str
     * @return
     */
    public static List<String> pattern5(String str) {
        return RegexUtil.listPatternStr(str, "[^|]\\D{4}\\s*(：|:)\\s*(<|≤)?[0-9]+\\.?[0-9]*\\s*(-|--)\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern5(String str) {
        return RegexUtil.pattern5(str).size() > 0 ? true : false;
    }

    public static int pattern5Count(String str) {
        return RegexUtil.pattern5(str).size();
    }


    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * <5.0E+001
     * 
     * 小于等于号；
     * 第一个数字可以是整数也可以是小数；
     * E+固定；
     * 第二个数字是整数；
     * 
     * @param str
     * @return
     */
    public static List<String> pattern6(String str) {
        return RegexUtil.listPatternStr(str, "(<|≤|>|≥)+[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistPattern6(String str) {
        return RegexUtil.pattern6(str).size() > 0 ? true : false;
    }

    public static int pattern6Count(String str) {
        return RegexUtil.pattern6(str).size();
    }

    
    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * ≤5.0
     * 
     * 小于等于号；
     * 数字可以是整数也可以是小数；
     * 
     * @param str
     * @return
     */
    public static List<String> pattern7(String str) {
        return RegexUtil.listPatternStr(str, "(<|≤|>|≥)+[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern7(String str) {
        return RegexUtil.pattern7(str).size() > 0 ? true : false;
    }

    public static int pattern7Count(String str) {
        return RegexUtil.pattern7(str).size();
    }


    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 5.0E+001--6.0E+001
     * 5.0E+001  --6.0E+001
     * 5.0E+001--  6.0E+001
     * 5.0E+001  --  6.0E+001
     * 
     * 小于等于号；
     * 第一个数字可以是整数也可以是小数；
     * E+固定；
     * 第二个数字是整数；
     * 
     * @param str
     * @return
     */
    public static List<String> pattern8(String str) {
        return RegexUtil.listPatternStr(str, "[0-9]+\\.?[0-9]*E\\+[0-9]+\\s*(-|--)\\s*[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistPattern8(String str) {
        return RegexUtil.pattern8(str).size() > 0 ? true : false;
    }

    public static int pattern8Count(String str) {
        return RegexUtil.pattern8(str).size();
    }


    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * 格式：
     * 例子：
     * 12--24
     * 23.4--54.1
     * 0.38 --4.34
     * 0.38 --  4.34
     *
     * 数字可以是整数也可以是小数；双横杠；横杠前后可以有空格（最多一个）
     * 
     * @param str 给定需要匹配的字符串
     * @return 返回匹配到的字符串集合
     */
    public static List<String> pattern9(String str) {
        return RegexUtil.listPatternStr(str, "[0-9]+\\.?[0-9]*\\s*×\\s*[0-9]+\\.?[0-9]*");
    }

    public static MaxMinValue pattern9value(String str) {
        MaxMinValue result = new MaxMinValue();

        try {
            List<String> l = RegexUtil.listPatternStr(str, "[0-9]+\\.?[0-9]*\\s*×\\s*[0-9]+\\.?[0-9]*");
            l.forEach(i -> {
                String[] s = i.split("×");
                if (s.length == 2) {
                    Integer m = Integer.valueOf(s[0].trim());
                    Integer n = Integer.valueOf(s[1].trim());
                    
                    result.setMax(m);
                    result.setMin(n);
                    if (n > m) {
                        result.setMax(n);
                        result.setMin(m);
                    }
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }



    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     * 
     * @param str  需要匹配的数据字符串
     * @param regex 正则表达式
     * @return
     */
    public static List<String> listPatternStr(String str, String regex) {
        List<String> list = new ArrayList<String>();
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(str);
        while(m.find()) {
            list.add(m.group());
        }
        return list;
    }



    public static void main(String[] args) {
        // List<String> list1 = pattern1("ghdashgk32.344 --12.56dsg12--24jk12  --24dh12--  24ag12  --  24k10-- 4grhnhkfns");
        // list1.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list2 = pattern2("ddbs滤泡期：0.4- 2.5；排卵期：1.2 -24.8黄体期：5.5 - 24.8； 绝经期：0.4-  1.4滤泡期：0.4-2.5ewtryg");
        // list2.forEach(i -> System.out.println(i));
        
        // System.out.println("\n-----\n");

        // List<String> list3 = pattern2("ddbshd滤泡期：<0.1  --0.6      黄体期：1.0-- 15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2ewtryg");
        // list3.forEach(i -> System.out.println(i));
        
        // System.out.println("\n-----\n");

        // List<String> list4 = pattern2("ddbsh卧位：25--60||立位：50--120||wtryg");
        // list4.forEach(i -> System.out.println(i));
        
        // System.out.println("\n-----\n");

        // List<String> list6 = pattern6("ddbshd<5.0E+001ewtryg");
        // list6.forEach(i -> System.out.println(i));
        
        // System.out.println("\n-----\n");

        // List<String> list7 = pattern7("ddbshd≤5.0tryg");
        // list7.forEach(i -> System.out.println(i));
        
        // List<String> list8 = pattern8("dd5.0E+001--6.0E+001bshd5.0E+001  --  6.0E+001tryg");
        // list8.forEach(i -> System.out.println(i));
        
        MaxMinValue mv = pattern9value("6 ×8mm");
        System.out.println(mv.getMax());
        System.out.println(mv.getMin());
    }
}
