package com.xxh.cloud.healthy.crm.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseTargetConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseTargetConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseTargetConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.DiseasePhaseConfigConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.DiseasePhaseTargetConfigConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.mapper.PatientComeDataBizMapper;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DiseasePhaseConfigBizService;
import com.xxh.cloud.healthy.crm.framework.modules.constant.DiseasePhaseConfigStateEnum;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
public class DiseasePhaseConfigBizServiceImpl implements DiseasePhaseConfigBizService {

    @Autowired
    private DiseasePhaseConfigInfoService diseasePhaseConfigInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;
    @Autowired
    private DiseasePhaseTargetConfigInfoService diseasePhaseTargetConfigInfoService;
    @Resource
    private PatientComeDataBizMapper patientComeDataBizMapper;

    @Override
    public PageData<DiseasePhaseConfigVO> query(DiseasePhaseConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<DiseasePhaseConfigInfo> page = diseasePhaseConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getIsDef()),"is_def",params.getIsDef())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .orderByAsc("orders")

        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return new PageData<>(null, 0);
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        PageData<DiseasePhaseConfigVO> data= new PageConverter<DiseasePhaseConfigInfo, DiseasePhaseConfigVO>() {
            @Override
            public DiseasePhaseConfigVO converter(DiseasePhaseConfigInfo entity) {
                return DiseasePhaseConfigConvert.getDiseasePhaseConfigVO(entity,userMap,diseaseMap);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<DiseasePhaseConfigVO> list(DiseasePhaseConfigListDTO params) {
        if (StrUtil.isEmpty(params.getDiseaseId()) && CollUtil.isEmpty(params.getDiseaseIds())) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR, "请确定病种");
        }
        List<DiseasePhaseConfigInfo> list= diseasePhaseConfigInfoService.list(
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .in(CollUtil.isNotEmpty(params.getDiseaseIds()),"disease_id",params.getDiseaseIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getIsDef()),"is_def",params.getIsDef())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .orderByAsc("orders")
        );
        List<DiseasePhaseConfigVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }
        Set<String> diseaseIds = new HashSet<>();
        Set<String> phaseIds = new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            phaseIds.add(item.getId());
        });
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        list.forEach(item->{
            data.add(DiseasePhaseConfigConvert.getDiseasePhaseConfigVO(item, new HashMap<>(), diseaseMap));
        });

        // 查询关联的目标
        List<DiseasePhaseTargetConfigInfo> targetList = diseasePhaseTargetConfigInfoService.list(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .in("phase_id", phaseIds)
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .orderByDesc("update_date"));
        if (CollUtil.isEmpty(targetList)) {
            return data;
        }
        // 设定阶段的目标
        Map<String, List<DiseasePhaseTargetConfigVO>> targetMap = new HashMap<>();
        targetList.forEach(target->{
            if (targetMap.containsKey(target.getPhaseId())) {
                targetMap.get(target.getPhaseId()).add(DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(target));
            } else {
                List<DiseasePhaseTargetConfigVO> vo = new ArrayList<>();
                vo.add(DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(target));
                targetMap.put(target.getPhaseId(), vo);
            }
        });
        data.forEach(p->{
            if (targetMap.containsKey(p.getId())) {
                p.addTargets(targetMap.get(p.getId()));
            }
        });
        return data;
    }

    @Override
    public void addDef(String diseaseId) {
        List<String> def = Arrays.asList("疾病筛查", "疾病会诊", "疾病治疗", "长期随访", "建议复诊");
        long num = diseasePhaseConfigInfoService.count(new QueryWrapper<DiseasePhaseConfigInfo>()
                .eq("disease_id", diseaseId));
        if (num > 0) {
            return;
        }
        List<DiseasePhaseConfigInfo> entityList = new ArrayList<>();
        for (int i = 0; i < def.size(); i++) {
            DiseasePhaseConfigInfo entity=new DiseasePhaseConfigInfo();
            entity.setId("P"+IdUtil.getSnowflake(1,1).nextIdStr());
            entity.setDiseaseId(diseaseId);
            entity.setName(def.get(i));
            entity.setOrders(i + 1);
            entity.setIsDef(YN.NO.getCode());
            entity.setState(DiseasePhaseConfigStateEnum.DPCS_Normal.getCode());
            entity.setRowsState(YN.YES.getCode());
            entityList.add(entity);
        }
        diseasePhaseConfigInfoService.saveBatch(entityList);



    }

    @Override
    public DiseasePhaseConfigVO add(DiseasePhaseConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);

        long num= diseasePhaseConfigInfoService.count(
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("disease_id",params.getDiseaseId())
                        .and(i -> i.eq("name",params.getName()).or().eq("orders", params.getOrders()))
                        .eq("rows_state",YN.YES.getCode())
        );

        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"名称或顺序已存在");
        }

        DiseasePhaseConfigInfo entity=new DiseasePhaseConfigInfo();
        entity.setId("P"+IdUtil.getSnowflake(1,1).nextIdStr());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setName(params.getName());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        if(StrUtil.isEmpty(params.getIsDef())){
            entity.setIsDef(YN.NO.getCode());
        }else {
            entity.setIsDef(params.getIsDef());
        }
        entity.setState(params.getState());
        entity.setRowsState(YN.YES.getCode());
        diseasePhaseConfigInfoService.save(entity);

        return DiseasePhaseConfigConvert.getDiseasePhaseConfigVO(entity);
    }

    @Override
    public DiseasePhaseConfigVO edit(DiseasePhaseConfigEditDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseConfigInfo entity=diseasePhaseConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 判断name是否重复
        long num = diseasePhaseConfigInfoService.count(new QueryWrapper<DiseasePhaseConfigInfo>()
                .eq("disease_id", entity.getDiseaseId())
                .and(i -> i.eq("name",params.getName()).or().eq("orders", params.getOrders()))
                .eq("rows_state",YN.YES.getCode())
                .ne("id", params.getId()));

        if (num > 0) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS, "名称或顺序已存在");
        }
        entity.setName(params.getName());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        entity.setIsDef(params.getIsDef());
        entity.setState(params.getState());
        diseasePhaseConfigInfoService.updateById(entity);

        return DiseasePhaseConfigConvert.getDiseasePhaseConfigVO(entity);
    }

    @Override
    public void updateState(DiseasePhaseConfigEditStateDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseConfigInfo entity=diseasePhaseConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(params.getState().equals(entity.getState())){
            return;
        }
        entity.setState(params.getState());
        diseasePhaseConfigInfoService.updateById(entity);
    }

    @Override
    public DiseasePhaseConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DiseasePhaseConfigInfo entity= diseasePhaseConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DiseasePhaseConfigConvert.getDiseasePhaseConfigVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DiseasePhaseConfigInfo entity= diseasePhaseConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 判断阶段是否，被医生患者服务关系表引用
        long relCount = patientDoctorRelationInfoService.count(new QueryWrapper<PatientDoctorRelationInfo>()
                .eq("phase_id", id)
                .eq("rows_state",YN.YES.getCode()));
        if (relCount > 0) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS, "阶段存在关联关系");
        }
        entity.setRowsState(YN.NO.getCode());
        diseasePhaseConfigInfoService.updateById(entity);
    }

    @Override
    public List<String> getDiseaseByDoctor(String doctorId) {
        List<String> list = patientComeDataBizMapper.getDiseaseByDoctor(doctorId);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        Set<String> set = new HashSet<>();
        list.forEach(i->{
            set.addAll(StrJoinSplitUtils.split2Set(i));
        });
        return new ArrayList<>(set);
    }

    @Override
    public List<String> getDeptByDiseaseId(String diseaseId) {
        List<String> list = patientComeDataBizMapper.getDeptByDiseaseId(new QueryWrapper<Void>()
                .like("disease_ids", diseaseId));
        return list;
    }
}
