package com.xxh.cloud.healthy.crm.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseTargetConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseTargetConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseTargetConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.DiseasePhaseTargetConfigConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DiseasePhaseTargetConfigBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DiseasePhaseTargetConfigBizServiceImpl implements DiseasePhaseTargetConfigBizService {

    @Autowired
    private DiseasePhaseTargetConfigInfoService diseasePhaseTargetConfigInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;


    @Override
    public PageData<DiseasePhaseTargetConfigVO> query(DiseaseTargetPhaseConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<DiseasePhaseTargetConfigInfo> page = diseasePhaseTargetConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getPhaseId()),"phase_id",params.getPhaseId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .eq(StrUtil.isNotEmpty(params.getConditionType()),"condition_type",params.getConditionType())
                        .orderByDesc("update_date")
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return new PageData<>(null, 0);
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }

        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);


        PageData<DiseasePhaseTargetConfigVO> data= new PageConverter<DiseasePhaseTargetConfigInfo, DiseasePhaseTargetConfigVO>() {
            @Override
            public DiseasePhaseTargetConfigVO converter(DiseasePhaseTargetConfigInfo entity) {
                return DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(entity,userMap);
            }
        }.getPageData(page);


        return data;
    }

    @Override
    public List<DiseasePhaseTargetConfigVO> list(DiseaseTargetPhaseConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DiseasePhaseTargetConfigInfo> list = diseasePhaseTargetConfigInfoService.list(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getPhaseId()),"phase_id",params.getPhaseId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .eq(StrUtil.isNotEmpty(params.getConditionType()),"condition_type",params.getConditionType())
                        .orderByDesc("update_date")
        );

        List<DiseasePhaseTargetConfigVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }

        list.forEach(item->{
            data.add(DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(item));
        });

        return data;
    }

    @Override
    public DiseasePhaseTargetConfigVO add(DiseaseTargetPhaseConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);

        // 判断名称是否存在
        long num= diseasePhaseTargetConfigInfoService.count(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("phase_id",params.getPhaseId())
                        .eq("name",params.getName())
                        .eq("rows_state",YN.YES.getCode()));
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"名称已存在");
        }
        DiseasePhaseTargetConfigInfo entity=new DiseasePhaseTargetConfigInfo();
        entity.setId("T"+ IdUtil.getSnowflake(1,2).nextIdStr());
        entity.setPhaseId(params.getPhaseId());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setState(params.getState());
        entity.setConditionType(params.getConditionType());
        entity.setCompletionLimit(params.getCompletionLimit());
        entity.setConditionContent(null);
        if (CollUtil.isNotEmpty(params.getTargetExeDeptIds())) {
            entity.setTargetExeDeptIds(StrJoinSplitUtils.joinPrefix(params.getTargetExeDeptIds()));
        }
        entity.setRowsState(YN.YES.getCode());
        diseasePhaseTargetConfigInfoService.save(entity);

        return DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(entity);
    }

    @Override
    public DiseasePhaseTargetConfigVO edit(DiseaseTargetPhaseConfigEditDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseTargetConfigInfo entity=diseasePhaseTargetConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 判断名称是否存在
        long num= diseasePhaseTargetConfigInfoService.count(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("phase_id", entity.getPhaseId())
                        .eq("name",params.getName())
                        .ne("id", entity.getId())
                        .eq("rows_state",YN.YES.getCode()));
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"名称已存在");
        }

        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setState(params.getState());
        entity.setConditionType(params.getConditionType());
        entity.setCompletionLimit(params.getCompletionLimit());
        if (CollUtil.isNotEmpty(params.getTargetExeDeptIds())) {
            entity.setTargetExeDeptIds(StrJoinSplitUtils.joinPrefix(params.getTargetExeDeptIds()));
        } else {
            entity.setTargetExeDeptIds("");
        }
        diseasePhaseTargetConfigInfoService.updateById(entity);
        return DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(entity);
    }

    @Override
    public void updateState(DiseaseTargetPhaseConfigUpdateStateDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseTargetConfigInfo entity=diseasePhaseTargetConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getState().equals(entity.getState())){
            return;
        }
        entity.setState(params.getState());
        diseasePhaseTargetConfigInfoService.updateById(entity);
    }

    @Override
    public DiseasePhaseTargetConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DiseasePhaseTargetConfigInfo entity=diseasePhaseTargetConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DiseasePhaseTargetConfigConvert.getDiseasePhaseTargetConfigVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DiseasePhaseTargetConfigInfo entity=diseasePhaseTargetConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 判断阶段是否，被医生患者服务关系表引用
        long relCount = patientDoctorRelationInfoService.count(new QueryWrapper<PatientDoctorRelationInfo>()
                .eq("phase_id", entity.getPhaseId())
                .eq("rows_state",YN.YES.getCode()));
        if (relCount > 0) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS, "阶段存在关联关系");
        }
        entity.setRowsState(YN.NO.getCode());
        diseasePhaseTargetConfigInfoService.updateById(entity);
    }
}
