package com.xxh.cloud.healthy.crm.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.*;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.rel.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.PatientDoctorRelationConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationBizService;
import com.xxh.cloud.healthy.crm.framework.modules.cache.service.DiseasePhaseTargetConfigCacheService;
import com.xxh.cloud.healthy.crm.framework.modules.constant.*;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorFollwUpClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorFollowUpClientAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorFollowUpClientCrmRelDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorFollowUpConstant;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import groovy.time.DatumDependentDuration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
public class PatientDoctorRelationBizServiceImpl implements PatientDoctorRelationBizService {

    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;
    @Autowired
    private PatientDoctorRelationTargetInfoService patientDoctorRelationTargetInfoService;
    @Autowired
    private PatientDoctorRelationHisInfoService patientDoctorRelationHisInfoService;
    @Autowired
    private PatientDoctorRelationChangeInfoService patientDoctorRelationChangeInfoService;
    @Autowired
    private DiseasePhaseTargetConfigCacheService diseasePhaseTargetConfigCacheService;
    @Autowired
    private DoctorFollwUpClientService doctorFollwUpClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    private void checkAdd2Transfer(String patientId ,String diseaseId,String doctorId){
        if(PatientDoctorRelationConfigConstant.PDR_Mult.equals(YN.YES.getCode())){
            long num =  patientDoctorRelationInfoService.count(
                    new QueryWrapper<PatientDoctorRelationInfo>()
                            .eq("rows_state",YN.YES.getCode())
                            .eq("state", PatientDoctorRelationStateEnum.PDRS_Normal.getCode())
                            .eq("patient_id",patientId)
                            .eq("disease_id",diseaseId)
                            .eq("doctor_id",doctorId)
            );
            if(num>0){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"此患者和病种已被关注,不能重复关注");
            }
        }else {
            long num =  patientDoctorRelationInfoService.count(
                    new QueryWrapper<PatientDoctorRelationInfo>()
                            .eq("rows_state",YN.YES.getCode())
                            .eq("state", PatientDoctorRelationStateEnum.PDRS_Normal.getCode())
                            .eq("patient_id",patientId)
                            .eq("disease_id",diseaseId)
            );
            if(num>0){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"此患者和病种已被关注,不能重复关注");
            }
        }
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO transferPatient(PatientDoctorRelationTransferDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 只能变更自己的个案 mj.li 240920
        if (!params.getOpUserId().equals(entity.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }

        if(!PatientDoctorRelationStateEnum.PDRS_Normal.getCode().equals(entity.getState())
                && !PatientDoctorRelationStateEnum.PDRS_Relation_Expire.getCode().equals(entity.getState())) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非正常状态，不能操作");
        }

        this.checkAdd2Transfer(entity.getPatientId(),entity.getDiseaseId(),params.getDoctorId());

        //原数据处理
        entity.setState(PatientDoctorRelationStateEnum.PDRS_Transfer.getCode());
        if(StrUtil.isNotEmpty(entity.getAddRemark())){
            entity.setAddRemark(entity.getAddRemark() + " , "+params.getRemark());
        }else {
            entity.setAddRemark(params.getRemark());
        }
        patientDoctorRelationInfoService.updateById(entity);


        PatientDoctorRelationHisInfo hisEntity= patientDoctorRelationHisInfoService.getOne(
                new QueryWrapper<PatientDoctorRelationHisInfo>()
                        .eq("relation_id",entity.getId())
        );
        if(hisEntity!=null){
            hisEntity.setEndDt(new Date());
            patientDoctorRelationHisInfoService.updateById(hisEntity);
        }

        PatientDoctorRelationChangeInfo changeEntity=new PatientDoctorRelationChangeInfo();
        changeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        changeEntity.setRelationId(entity.getId());
        changeEntity.setPatientId(entity.getPatientId());
        changeEntity.setDiseaseId(entity.getDiseaseId());
        changeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Transfer.getCode());
        changeEntity.setOldDoctorId(entity.getDoctorId());
        changeEntity.setNewDoctorId(params.getDoctorId());
        changeEntity.setOldPhaseId(entity.getPhaseId());
        changeEntity.setNewPhaseId(params.getPhaseId());
        changeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        changeEntity.setChangeUserId(params.getOpUserId());
        changeEntity.setChangeRemark(params.getRemark());
        changeEntity.setOldBatchNo(entity.getBatchNo());
        patientDoctorRelationChangeInfoService.save(changeEntity);

        patientDoctorRelationTargetInfoService.update(
                new UpdateWrapper<PatientDoctorRelationTargetInfo>()
                        .set("end_dt",new Date())
                        .set("update_date",new Date())
                        .set("updater",params.getOpUserId())
                        .eq("rows_state",YN.YES.getCode())
                        .eq("relation_id",entity.getId())
                        .isNull("end_dt")
        );


        //新关系建立
        PatientDoctorRelationInfo newEntity=new PatientDoctorRelationInfo();
        newEntity.setId("R"+IdUtil.getSnowflake(1,3).nextIdStr());
        newEntity.setPatientId(entity.getPatientId());
        newEntity.setDoctorId(params.getDoctorId());
        newEntity.setDiseaseId(params.getDiseaseId());
        newEntity.setAddRemark(params.getRemark());
        newEntity.setAddWay(params.getAddWay());
        newEntity.setAddUserId(params.getOpUserId());
        newEntity.setPhaseId(params.getPhaseId());
        newEntity.setSourceType(params.getSourceType());
        newEntity.setSourceObjId(params.getSourceObjId());
        newEntity.setRelType(entity.getRelType());
        newEntity.setState(PatientDoctorRelationStateEnum.PDRS_Normal.getCode());
        newEntity.setBatchNo(1);
        newEntity.setRowsState(YN.YES.getCode());
        patientDoctorRelationInfoService.save(newEntity);

        PatientDoctorRelationHisInfo newHisEntity=new PatientDoctorRelationHisInfo();
        newHisEntity.setId(IdUtil.getSnowflake(1,4).nextIdStr());
        newHisEntity.setRelationId(newEntity.getId());
        newHisEntity.setPatientId(newEntity.getPatientId());
        newHisEntity.setDoctorId(newEntity.getDoctorId());
        newHisEntity.setDiseaseId(newEntity.getDiseaseId());
        newHisEntity.setStartDt(new Date());
        newHisEntity.setRemark("新增");
        newHisEntity.setBatchNo(newEntity.getBatchNo());
        patientDoctorRelationHisInfoService.save(newHisEntity);

        PatientDoctorRelationChangeInfo newChangeEntity=new PatientDoctorRelationChangeInfo();
        newChangeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        newChangeEntity.setRelationId(newEntity.getId());
        newChangeEntity.setPatientId(newEntity.getPatientId());
        newChangeEntity.setDiseaseId(newEntity.getDiseaseId());
        newChangeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Add.getCode());
        newChangeEntity.setOldDoctorId(newEntity.getDoctorId());
        newChangeEntity.setNewDoctorId(newEntity.getDoctorId());
        newChangeEntity.setOldPhaseId(newEntity.getPhaseId());
        newChangeEntity.setNewPhaseId(newEntity.getPhaseId());
        newChangeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        newChangeEntity.setChangeUserId(params.getOpUserId());
        newChangeEntity.setChangeRemark("新增");
        newChangeEntity.setOldBatchNo(newEntity.getBatchNo());
        newChangeEntity.setNewBatchNo(newEntity.getBatchNo());
        patientDoctorRelationChangeInfoService.save(newChangeEntity);

        if(StrUtil.isNotEmpty(newEntity.getPhaseId()) && CollUtil.isNotEmpty(params.getPhaseTargetIds())){
            List<PatientDoctorRelationTargetInfo> targetList=new ArrayList<>();
            for(String targetId:params.getPhaseTargetIds()){
                DiseasePhaseTargetConfigInfo targetConfig = diseasePhaseTargetConfigCacheService.getTargetConfigById(targetId);
                if (targetConfig == null) {
                    continue;
                }
                PatientDoctorRelationTargetInfo targetEntity=new PatientDoctorRelationTargetInfo();
                targetEntity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
                targetEntity.setRelationId(newEntity.getId());
                targetEntity.setPhaseId(newEntity.getPhaseId());
                targetEntity.setTargetId(targetId);
                targetEntity.setStartDt(new Date());
                targetEntity.setResultState(PDRTargetResultStateEnum.PDRTRS_None.getCode());
                targetEntity.setRowsState(YN.YES.getCode());
                if (null == params.getLimitDt()) {
                    if (targetConfig.getCompletionLimit() != null) {
                        targetEntity.setLimitDt(DateUtil.offsetDay(targetEntity.getStartDt(), targetConfig.getCompletionLimit()));
                    }
                } else {
                    targetEntity.setLimitDt(params.getLimitDt());
                }
                targetList.add(targetEntity);
                // 加入随访计划
                if (YN.YES.getCode().equals(params.getAddFollowUp())) {
                    addFollowUp(entity, targetEntity, targetConfig);
                }
            }
            patientDoctorRelationTargetInfoService.saveBatch(targetList);
        }

        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(newEntity);
    }




    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO add(PatientDoctorRelationAddDTO params) {
        ValidatorUtils.validateEntity(params);

        this.checkAdd2Transfer(params.getPatientId(),params.getDiseaseId(),params.getDoctorId());

        PatientDoctorRelationInfo entity=new PatientDoctorRelationInfo();
        entity.setId("R"+IdUtil.getSnowflake(1,3).nextIdStr());
        entity.setPatientId(params.getPatientId());
        entity.setDoctorId(params.getDoctorId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setAddRemark(params.getAddRemark());
        entity.setAddWay(params.getAddWay());
        entity.setAddUserId(params.getAddUserId());
        entity.setPhaseId(params.getPhaseId());
        entity.setRelType(params.getRelType());
        entity.setSourceType(params.getSourceType());
        entity.setSourceObjId(params.getSourceObjId());
        entity.setState(PatientDoctorRelationStateEnum.PDRS_Normal.getCode());
        entity.setRowsState(YN.YES.getCode());
        entity.setBatchNo(1); // 新增批次=1
        patientDoctorRelationInfoService.save(entity);


        PatientDoctorRelationHisInfo hisEntity=new PatientDoctorRelationHisInfo();
        hisEntity.setId(IdUtil.getSnowflake(1,4).nextIdStr());
        hisEntity.setRelationId(entity.getId());
        hisEntity.setPatientId(entity.getPatientId());
        hisEntity.setDoctorId(entity.getDoctorId());
        hisEntity.setDiseaseId(entity.getDiseaseId());
        hisEntity.setPhaseId(entity.getPhaseId());
        hisEntity.setRelType(entity.getRelType());
        hisEntity.setBatchNo(entity.getBatchNo());
        hisEntity.setStartDt(new Date());
        hisEntity.setRemark("新增");
        patientDoctorRelationHisInfoService.save(hisEntity);


        PatientDoctorRelationChangeInfo changeEntity=new PatientDoctorRelationChangeInfo();
        changeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        changeEntity.setRelationId(entity.getId());
        changeEntity.setPatientId(entity.getPatientId());
        changeEntity.setDiseaseId(entity.getDiseaseId());
        changeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Add.getCode());
        changeEntity.setOldDoctorId(entity.getDoctorId());
        changeEntity.setNewDoctorId(entity.getDoctorId());
        changeEntity.setOldPhaseId(entity.getPhaseId());
        changeEntity.setNewPhaseId(entity.getPhaseId());
        changeEntity.setOldBatchNo(entity.getBatchNo());
        changeEntity.setNewBatchNo(entity.getBatchNo());
        changeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        changeEntity.setChangeUserId(params.getAddUserId());
        changeEntity.setChangeRemark("新增");
        patientDoctorRelationChangeInfoService.save(changeEntity);

        if(StrUtil.isNotEmpty(params.getPhaseId()) && CollUtil.isNotEmpty(params.getPhaseTargetIds())){
            List<PatientDoctorRelationTargetInfo> targetList=new ArrayList<>();
            for(String targetId:params.getPhaseTargetIds()){
                DiseasePhaseTargetConfigInfo targetConfig = diseasePhaseTargetConfigCacheService.getTargetConfigById(targetId);
                if (targetConfig == null) {
                    continue;
                }
                PatientDoctorRelationTargetInfo targetEntity=new PatientDoctorRelationTargetInfo();
                targetEntity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
                targetEntity.setRelationId(entity.getId());
                targetEntity.setPhaseId(entity.getPhaseId());
                targetEntity.setBatchNo(entity.getBatchNo());
                targetEntity.setTargetId(targetId);
                targetEntity.setStartDt(new Date());
                targetEntity.setResultState(PDRTargetResultStateEnum.PDRTRS_None.getCode());
                targetEntity.setRowsState(YN.YES.getCode());
                if (null == params.getLimitDt()) {
                    if (targetConfig.getCompletionLimit() != null) {
                        targetEntity.setLimitDt(DateUtil.offsetDay(targetEntity.getStartDt(), targetConfig.getCompletionLimit()));
                    }
                } else {
                    targetEntity.setLimitDt(params.getLimitDt());
                }
                targetList.add(targetEntity);
                // 加入随访计划
                if (YN.YES.getCode().equals(params.getAddFollowUp())) {
                    addFollowUp(entity, targetEntity, targetConfig);
                }
            }
            patientDoctorRelationTargetInfoService.saveBatch(targetList);
        }

        // 关联存在的随访任务
        relFollowUp(entity);

        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO updatePhase(PatientDoctorRelationUpdatePhaseDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(params.getId());
        // 只能变更自己的个案 mj.li 240920
        if (entity == null || !params.getOpUserId().equals(entity.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }
        if (params.getPhaseId().equals(entity.getPhaseId())) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR, "当前阶段相同");
        }
        int oldBatchNo = entity.getBatchNo() == null ? 1 : entity.getBatchNo();
        int newBatchNo = oldBatchNo + 1;
        if(entity.getBatchNo() == null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(PatientDoctorRelationStateEnum.PDRS_Abandon.getCode().equals(entity.getState()) ||
                PatientDoctorRelationStateEnum.PDRS_Transfer.getCode().equals(entity.getState())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非正常状态，不能操作");
        }

        String oldPhaseId=entity.getPhaseId();
        if(StrUtil.isNotEmpty(entity.getAddRemark())){
            entity.setAddRemark(entity.getAddRemark() + " , "+params.getRemark());
        }else {
            entity.setAddRemark(params.getRemark());
        }
        entity.setRelType(params.getRelType());
        entity.setPhaseId(params.getPhaseId());
        entity.setState(PatientDoctorRelationStateEnum.PDRS_Normal.getCode());
        entity.setBatchNo(newBatchNo);
        patientDoctorRelationInfoService.updateById(entity);



        if(!oldPhaseId.equals(params.getPhaseId())){
            patientDoctorRelationTargetInfoService.update(
                    new UpdateWrapper<PatientDoctorRelationTargetInfo>()
                            .set("end_dt",new Date())
                            .set("update_date",new Date())
                            .set("updater",params.getOpUserId())
                            .set("result_state", PDRTargetResultStateEnum.PDRTRS_Change.getCode())
                            .eq("rows_state",YN.YES.getCode())
                            .eq("relation_id",entity.getId())
                            .eq("phase_id",oldPhaseId)
                            .eq("batch_no",oldBatchNo)
                            .ne("result_state", PDRTargetResultStateEnum.PDRTRS_Completed.getCode())
            );
        }

        PatientDoctorRelationChangeInfo changeEntity=new PatientDoctorRelationChangeInfo();
        changeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        changeEntity.setRelationId(entity.getId());
        changeEntity.setPatientId(entity.getPatientId());
        changeEntity.setDiseaseId(entity.getDiseaseId());
        changeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Phase2Target.getCode());
        changeEntity.setOldDoctorId(entity.getDoctorId());
        changeEntity.setNewDoctorId(entity.getDoctorId());
        changeEntity.setOldBatchNo(oldBatchNo);
        changeEntity.setNewBatchNo(newBatchNo);
        changeEntity.setNewPhaseId(entity.getPhaseId());
        changeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        changeEntity.setChangeUserId(params.getOpUserId());
        changeEntity.setChangeRemark(params.getRemark());
        patientDoctorRelationChangeInfoService.save(changeEntity);

        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO updateTarget(PatientDoctorRelationUpdateTargetDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 只能变更自己的个案 mj.li 240920
        if (!params.getOpUserId().equals(entity.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }
        if(PatientDoctorRelationStateEnum.PDRS_Abandon.getCode().equals(entity.getState()) ||
                PatientDoctorRelationStateEnum.PDRS_Transfer.getCode().equals(entity.getState())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非正常状态，不能操作");
        }

        if(StrUtil.isEmpty(entity.getPhaseId())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"阶段不能为空,先设置阶段");
        }

        List<PatientDoctorRelationTargetInfo> targetList= patientDoctorRelationTargetInfoService.list(
                new QueryWrapper<PatientDoctorRelationTargetInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .eq("relation_id",entity.getId())
                        .eq("phase_id",entity.getPhaseId())
                        .eq("batch_no",entity.getBatchNo())
                        .eq("result_state", PDRTargetResultStateEnum.PDRTRS_None.getCode()) // 查询未执行的
        );

        Map<String,PatientDoctorRelationTargetInfo> targetMap=new HashMap<>();
        if(CollUtil.isNotEmpty(targetList)){
            targetList.forEach(item->{
                targetMap.put(item.getTargetId(),item);
            });
        }

        List<PatientDoctorRelationTargetInfo> saveList=new ArrayList<>();
        List<PatientDoctorRelationTargetInfo> updateList=new ArrayList<>();

        for(String targetId : params.getPhaseTargetIds()){
            PatientDoctorRelationTargetInfo targetEntity= targetMap.get(targetId);
            if(targetEntity!=null){
                targetMap.remove(targetId);
            }else {
                DiseasePhaseTargetConfigInfo targetConfig = diseasePhaseTargetConfigCacheService.getTargetConfigById(targetId);
                if (targetConfig == null) {
                    continue;
                }
                targetEntity=new PatientDoctorRelationTargetInfo();
                targetEntity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
                targetEntity.setRelationId(entity.getId());
                targetEntity.setPhaseId(entity.getPhaseId());
                targetEntity.setTargetId(targetId);
                targetEntity.setBatchNo(entity.getBatchNo());
                targetEntity.setStartDt(new Date());
                targetEntity.setResultState(PDRTargetResultStateEnum.PDRTRS_None.getCode());
                targetEntity.setRowsState(YN.YES.getCode());
                if (null == params.getLimitDt()) {
                    if (targetConfig.getCompletionLimit() != null) {
                        targetEntity.setLimitDt(DateUtil.offsetDay(targetEntity.getStartDt(), targetConfig.getCompletionLimit()));
                    }
                } else {
                    targetEntity.setLimitDt(params.getLimitDt());
                }
                saveList.add(targetEntity);
                // 加入随访计划
                if (YN.YES.getCode().equals(params.getAddFollowUp())) {
                    addFollowUp(entity, targetEntity, targetConfig);
                }
            }
        }

        if(MapUtil.isNotEmpty(targetMap)){
            targetMap.forEach((key,item)->{
                item.setRowsState(YN.NO.getCode());
                updateList.add(item);
            });
        }


        if(updateList.size()>0){
            patientDoctorRelationTargetInfoService.updateBatchById(updateList);
        }

        if(saveList.size()>0){
            patientDoctorRelationTargetInfoService.saveBatch(saveList);
        }
        if (!PatientDoctorRelationStateEnum.PDRS_Normal.getCode().equals(entity.getState())) {
            entity.setState(PatientDoctorRelationStateEnum.PDRS_Normal.getCode());
            patientDoctorRelationInfoService.updateById(entity); // 阶段更新，进行中
        }

        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO abandonPatient(PatientDoctorRelationAbandonDTO params) {
        ValidatorUtils.validateEntity(params);

        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(params.getId());

        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 只能变更自己的个案 mj.li 240920
        if (!params.getOpUserId().equals(entity.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }

        if(!PatientDoctorRelationStateEnum.PDRS_Normal.getCode().equals(entity.getState())
                && !PatientDoctorRelationStateEnum.PDRS_Relation_Expire.getCode().equals(entity.getState())) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非正常状态，不能操作");
        }

        entity.setState(PatientDoctorRelationStateEnum.PDRS_Abandon.getCode());

        if(StrUtil.isNotEmpty(entity.getAddRemark())){
            entity.setAddRemark(entity.getAddRemark() + " , "+params.getRemark());
        }else {
            entity.setAddRemark(params.getRemark());
        }
        patientDoctorRelationInfoService.updateById(entity);

        PatientDoctorRelationHisInfo hisEntity= patientDoctorRelationHisInfoService.getOne(
                new QueryWrapper<PatientDoctorRelationHisInfo>()
                        .eq("relation_id",entity.getId())
        );

        if(hisEntity!=null){
            hisEntity.setEndDt(new Date());
            patientDoctorRelationHisInfoService.updateById(hisEntity);
        }


        PatientDoctorRelationChangeInfo changeEntity=new PatientDoctorRelationChangeInfo();
        changeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        changeEntity.setRelationId(entity.getId());
        changeEntity.setPatientId(entity.getPatientId());
        changeEntity.setDiseaseId(entity.getDiseaseId());
        changeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Abandon.getCode());
        changeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        changeEntity.setChangeUserId(params.getOpUserId());
        changeEntity.setChangeRemark(params.getRemark());
        changeEntity.setOldDoctorId(entity.getDoctorId());
        changeEntity.setOldPhaseId(entity.getPhaseId());
        changeEntity.setOldBatchNo(entity.getBatchNo());
        patientDoctorRelationChangeInfoService.save(changeEntity);


        patientDoctorRelationTargetInfoService.update(
                new UpdateWrapper<PatientDoctorRelationTargetInfo>()
                        .set("end_dt",new Date())
                        .set("update_date",new Date())
                        .set("updater",params.getOpUserId())
                        .eq("rows_state",YN.YES.getCode())
                        .eq("relation_id",entity.getId())
                        .isNull("end_dt")
        );




        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientDoctorRelationVO completePatient(PatientDoctorRelationAbandonDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(!PatientDoctorRelationStateEnum.PDRS_Normal.getCode().equals(entity.getState())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非正常状态，不能操作");
        }

        entity.setState(PatientDoctorRelationStateEnum.PDRS_Relation_Complete.getCode());
        if(StrUtil.isNotEmpty(entity.getAddRemark())){
            entity.setAddRemark(entity.getAddRemark() + " , "+params.getRemark());
        }else {
            entity.setAddRemark(params.getRemark());
        }
        patientDoctorRelationInfoService.updateById(entity);

        PatientDoctorRelationHisInfo hisEntity= patientDoctorRelationHisInfoService.getOne(
                new QueryWrapper<PatientDoctorRelationHisInfo>()
                        .eq("relation_id",entity.getId())
        );

        if(hisEntity!=null){
            hisEntity.setEndDt(new Date());
            patientDoctorRelationHisInfoService.updateById(hisEntity);
        }

        PatientDoctorRelationChangeInfo changeEntity=new PatientDoctorRelationChangeInfo();
        changeEntity.setId(IdUtil.getSnowflake(1,5).nextIdStr());
        changeEntity.setRelationId(entity.getId());
        changeEntity.setPatientId(entity.getPatientId());
        changeEntity.setDiseaseId(entity.getDiseaseId());
        changeEntity.setChangeType(PdrChangeTypeEnum.PDRC_Type_Complete.getCode());
        changeEntity.setChangeWay(PdrChangeAddWayEnum.PDRC_Way_Human.getCode());
        changeEntity.setChangeUserId(params.getOpUserId());
        changeEntity.setChangeRemark(params.getRemark());
        patientDoctorRelationChangeInfoService.save(changeEntity);

        patientDoctorRelationTargetInfoService.update(
                new UpdateWrapper<PatientDoctorRelationTargetInfo>()
                        .set("end_dt",new Date())
                        .set("update_date",new Date())
                        .set("updater",params.getOpUserId())
                        .eq("rows_state",YN.YES.getCode())
                        .eq("relation_id",entity.getId())
                        .isNull("end_dt")
        );



        return PatientDoctorRelationConvert.getPatientDoctorRelationVO(entity);
    }


    private void addFollowUp(PatientDoctorRelationInfo relation,
                             PatientDoctorRelationTargetInfo relationTarget,
                             DiseasePhaseTargetConfigInfo targetConfig) {
        DoctorFollowUpClientAddDTO followUpClientAdd = new DoctorFollowUpClientAddDTO();
        followUpClientAdd.setType(DoctorFollowUpConstant.Type.Normal.getCode());
        followUpClientAdd.setPatientId(relation.getPatientId());
        followUpClientAdd.setDiseaseId(relation.getDiseaseId());
        followUpClientAdd.setDoctorId(relation.getDoctorId());
        followUpClientAdd.setContent(targetConfig.getName());
        followUpClientAdd.setFollowDate(relationTarget.getLimitDt());
        followUpClientAdd.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
        followUpClientAdd.setSourceDate(DateUtil.date());
        followUpClientAdd.setCrmRelationId(relation.getId());
        AdminUserBO doctor = adminUserClientService.getUserById(relation.getDoctorId());
        if (null != doctor) {
            followUpClientAdd.setSourceDeptId(doctor.getDeptId());
        }
        doctorFollwUpClientService.add(followUpClientAdd);
    }

    private void relFollowUp(PatientDoctorRelationInfo relation) {
        DoctorFollowUpClientCrmRelDTO crmRelDTO = new DoctorFollowUpClientCrmRelDTO();
        crmRelDTO.setCrmRelationId(relation.getId());
        crmRelDTO.setPatientId(relation.getPatientId());
        crmRelDTO.setDoctorId(relation.getDoctorId());
        crmRelDTO.setDiseaseId(relation.getDiseaseId());
        doctorFollwUpClientService.relCrm(crmRelDTO);
    }
}
