package com.xxh.cloud.healthy.crm.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.*;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationChangeInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationTargetInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.rel.PatientDoctorRelationChangeDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.rel.PatientDoctorRelationQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationChangeVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationFullVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.PatientDoctorRelationChangeConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.PatientDoctorRelationConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.mapper.PatientComeDataBizMapper;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationQueryService;
import com.xxh.cloud.healthy.crm.framework.modules.cache.service.DiseasePhaseConfigCacheService;
import com.xxh.cloud.healthy.crm.framework.modules.cache.service.DiseasePhaseTargetConfigCacheService;
import com.xxh.cloud.healthy.data.client.api.DoctorFollowPatientClientService;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DataDwsPatientDataLastDateBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorFollowPatientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.DoctorFollowPatientListClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class PatientDoctorRelationQueryServiceImpl implements PatientDoctorRelationQueryService {

    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;
    @Autowired
    private PatientDoctorRelationTargetInfoService patientDoctorRelationTargetInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DiseasePhaseConfigCacheService diseasePhaseConfigCacheService;
    @Autowired
    private DiseasePhaseTargetConfigCacheService diseasePhaseTargetConfigCacheService;
    @Autowired
    private PatientDoctorRelationChangeInfoService patientDoctorRelationChangeInfoService;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;
    @Autowired
    private DoctorFollowPatientClientService doctorFollowPatientClientService;
    @Autowired
    private PatientClientService patientClientService;

    @Override
    public PageData<PatientDoctorRelationFullVO> queryPdrFull(PatientDoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<PatientDoctorRelationInfo> page = patientDoctorRelationInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PatientDoctorRelationInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .and(StrUtil.isNotEmpty(params.getPatientId()),
                                i->i.or().eq("patient_id", params.getPatientId())
                                        .or().inSql("patient_id", StrUtil.format(
                                                "select patient_id from hd_task_patient_info where patient_name like '%{}%' or id_card_no = '{}' ", params.getPatientId(), params.getPatientId())))
                        .eq(StrUtil.isNotEmpty(params.getDoctorId()),"doctor_id",params.getDoctorId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getPhaseId()),"phase_id",params.getPhaseId())
                        .like(StrUtil.isNotEmpty(params.getPhaseTargetId()),"phase_target_ids",params.getPhaseTargetId())
                        .eq(StrUtil.isNotEmpty(params.getAddWay()),"add_way",params.getAddWay())
                        .eq(StrUtil.isNotEmpty(params.getSourceType()),"source_type",params.getSourceType())
                        .eq(StrUtil.isNotEmpty(params.getSourceObjId()),"source_obj_id",params.getSourceObjId())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .inSql(StrUtil.isNotEmpty(params.getPatientGender()),"patient_id",
                                StrUtil.format("select patient_id from hd_task_patient_info where patient_gender = '{}' ", params.getPatientGender()))
                        .ge(StrUtil.isNotEmpty(params.getStartDt()),"create_date",params.getStartDt())
                        .le(StrUtil.isNotEmpty(params.getEndDt()),"create_date",params.getEndDt())
                        .orderByDesc("create_date")

        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return new PageData<>(null, 0);
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> phaseIds=new HashSet<>();
        Set<String> patientIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getDoctorId())){
                userIds.add(item.getDoctorId());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }

            if(StrUtil.isNotEmpty(item.getPatientId())){
                patientIds.add(item.getPatientId());
            }

            if(StrUtil.isNotEmpty(item.getPhaseId())){
                phaseIds.add(item.getPhaseId());
            }

            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Map<String,DiseasePhaseConfigInfo> phaseMap= diseasePhaseConfigCacheService.getPhaseConfigMap(phaseIds);
        // 查询患者信息
        PatientQueryDTO patientQuery = new PatientQueryDTO();
        patientQuery.setPatientIds(new ArrayList<>(patientIds));
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQuery);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientList)) {
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }
        // 查询患者最新数据信息
        Map<String, DataDwsPatientDataLastDateBO> patientDataLastDateMap = dwdDataPatientClientService.mapPatientDataLastDateInfo(new ArrayList<>(patientIds));
        // 是否关注患者
        Map<String, String> followMap = new HashMap<>();
        DoctorFollowPatientListClientDTO followDTO = new DoctorFollowPatientListClientDTO();
        followDTO.setDoctorId(params.getDoctorId());
        followDTO.setPatientIds(new ArrayList<>(patientIds));
        List<DoctorFollowPatientBO> followList = doctorFollowPatientClientService.simpleList(followDTO);
        if (CollUtil.isNotEmpty(followList)) {
            followList.forEach(i -> followMap.put(i.getPatientId(), i.getStatus()));
        }



        PageData<PatientDoctorRelationFullVO> data = new PageConverter<PatientDoctorRelationInfo, PatientDoctorRelationFullVO>() {
            @Override
            public PatientDoctorRelationFullVO converter(PatientDoctorRelationInfo entity) {
                return PatientDoctorRelationConvert.getPatientDoctorRelationFullVO(entity, userMap, diseaseMap, phaseMap, patientMap, patientDataLastDateMap,followMap);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public PatientDoctorRelationFullVO getPdrFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PatientDoctorRelationInfo entity= patientDoctorRelationInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //
        List<PatientDoctorRelationTargetInfo> targetList= patientDoctorRelationTargetInfoService.list(
                new QueryWrapper<PatientDoctorRelationTargetInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .eq("relation_id",entity.getId())
//                        .eq("batch_no",entity.getBatchNo())
        );
        //
        List<DiseasePhaseConfigInfo> diseasePhaseConfigList= diseasePhaseConfigCacheService.getPhaseConfigListByDiseaseId(entity.getDiseaseId());
        //
        Map<String,DiseasePhaseTargetConfigInfo>  targetConfigMap= diseasePhaseTargetConfigCacheService.getTargetConfigMapByDiseaseId2DB(entity.getDiseaseId());
        //

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> phaseIds=new HashSet<>();
        Set<String> patientIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getDoctorId())){
            userIds.add(entity.getDoctorId());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }

        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getPhaseId())){
            phaseIds.add(entity.getPhaseId());
        }
        if(StrUtil.isNotEmpty(entity.getPatientId())){
            patientIds.add(entity.getPatientId());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Map<String,DiseasePhaseConfigInfo> phaseMap= diseasePhaseConfigCacheService.getPhaseConfigMap(phaseIds);

        // 查询患者信息
        PatientQueryDTO patientQuery = new PatientQueryDTO();
        patientQuery.setPatientIds(new ArrayList<>(patientIds));
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQuery);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientList)) {
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }
        // 查询患者最新数据信息
        Map<String, DataDwsPatientDataLastDateBO> patientDataLastDateMap = dwdDataPatientClientService.mapPatientDataLastDateInfo(new ArrayList<>(patientIds));
        // 是否关注患者
        Map<String, String> followMap = new HashMap<>();
        DoctorFollowPatientListClientDTO followDTO = new DoctorFollowPatientListClientDTO();
        followDTO.setDoctorId(entity.getDoctorId());
        followDTO.setPatientIds(new ArrayList<>(patientIds));
        List<DoctorFollowPatientBO> followList = doctorFollowPatientClientService.simpleList(followDTO);
        if (CollUtil.isNotEmpty(followList)) {
            followList.forEach(i -> followMap.put(i.getPatientId(), i.getStatus()));
        }



        return PatientDoctorRelationConvert.getPatientDoctorRelationFullVO(entity,diseasePhaseConfigList,targetList,targetConfigMap,userMap,diseaseMap,phaseMap, patientMap, patientDataLastDateMap,followMap);
    }

    @Override
    public List<StandardDiseaseBO> queryDisease(PatientDoctorRelationQueryDTO params) {

        List<PatientDoctorRelationInfo> relList = patientDoctorRelationInfoService.list(
                new QueryWrapper<PatientDoctorRelationInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getPatientId()),"patient_id",params.getPatientId())
                        .eq(StrUtil.isNotEmpty(params.getDoctorId()),"doctor_id",params.getDoctorId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getPhaseId()),"phase_id",params.getPhaseId())
                        .like(StrUtil.isNotEmpty(params.getPhaseTargetId()),"phase_target_ids",params.getPhaseTargetId())
                        .eq(StrUtil.isNotEmpty(params.getAddWay()),"add_way",params.getAddWay())
                        .eq(StrUtil.isNotEmpty(params.getSourceType()),"source_type",params.getSourceType())
                        .eq(StrUtil.isNotEmpty(params.getSourceObjId()),"source_obj_id",params.getSourceObjId())
                        .eq(StrUtil.isNotEmpty(params.getState()),"state",params.getState())
                        .ge(StrUtil.isNotEmpty(params.getStartDt()),"create_date",params.getStartDt())
                        .le(StrUtil.isNotEmpty(params.getEndDt()),"create_date",params.getEndDt())
                        .orderByDesc("create_date"));

        if(CollUtil.isEmpty(relList)) {
            return new ArrayList<>();
        }
        Set<String> diseaseIds=new HashSet<>();

        relList.forEach(item->{
            diseaseIds.add(item.getDiseaseId());
        });
        return standardDiseaseClientService.getList(diseaseIds);
    }

    @Override
    public List<PatientDoctorRelationChangeVO> relationChange(PatientDoctorRelationChangeDTO params) {
        List<PatientDoctorRelationChangeInfo> list = patientDoctorRelationChangeInfoService.list(
                new QueryWrapper<PatientDoctorRelationChangeInfo>()
                        .eq(StrUtil.isNotEmpty(params.getPatientId()),"patient_id", params.getPatientId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id", params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getDoctorId()),"doctor_id", params.getDiseaseId())
        );
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        Set<String> patientIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        Set<String> doctorIds = new HashSet<>();
        Set<String> phaseIds = new HashSet<>();
        Set<String> relationIds = new HashSet<>();
        Set<String> targetIds = new HashSet<>();
        list.forEach(item->{
            relationIds.add(item.getRelationId());
            if (StrUtil.isNotEmpty(item.getPatientId())) {
                patientIds.add(item.getPatientId());
            }
            if (StrUtil.isNotEmpty(item.getDiseaseId())) {
                diseaseIds.add(item.getDiseaseId());
            }
            if (StrUtil.isNotEmpty(item.getNewDoctorId())) {
                doctorIds.add(item.getNewDoctorId());
            }
            if (StrUtil.isNotEmpty(item.getOldDoctorId())) {
                doctorIds.add(item.getOldDoctorId());
            }
            if (StrUtil.isNotEmpty(item.getNewPhaseId())) {
                phaseIds.add(item.getNewPhaseId());
            }
            if (StrUtil.isNotEmpty(item.getOldPhaseId())) {
                phaseIds.add(item.getOldPhaseId());
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(doctorIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        Map<String,DiseasePhaseConfigInfo> phaseMap= diseasePhaseConfigCacheService.getPhaseConfigMap(phaseIds);
        // 查询患者信息
        PatientQueryDTO patientQuery = new PatientQueryDTO();
        patientQuery.setPatientIds(new ArrayList<>(patientIds));
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQuery);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientList)) {
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }

        // 关系所属目标列表
        List<PatientDoctorRelationTargetInfo> targetList= patientDoctorRelationTargetInfoService.list(
                new QueryWrapper<PatientDoctorRelationTargetInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .in("relation_id", relationIds));
        Map<String, List<PatientDoctorRelationTargetInfo>> targetMap = new HashMap<>();
        if (CollUtil.isNotEmpty(targetList)) {
            targetList.forEach(item->{
                targetIds.add(item.getTargetId());
                List<PatientDoctorRelationTargetInfo> targets;
                if (targetMap.containsKey(item.getRelationId())) {
                    targets = targetMap.get(item.getRelationId());
                } else {
                    targets = new ArrayList<>();
                }
                targets.add(item);
                targetMap.put(item.getRelationId(), targets);
            });
        }

        // 阶段所属目标列表
        Map<String, DiseasePhaseTargetConfigInfo> targetConfigMap
                = diseasePhaseTargetConfigCacheService.getTargetConfigMap2DB(targetIds);
        return list.stream().map(item->
                PatientDoctorRelationChangeConvert.getPatientDoctorRelationChangeVO(item, userMap,diseaseMap,phaseMap,patientMap,targetMap,targetConfigMap)).collect(Collectors.toList());
    }

    @Override
    public PatientDoctorRelationInfo getPatientDoctorRelationById(String relationId) {
        if(StrUtil.isEmpty(relationId)){
            return null;
        }

        PatientDoctorRelationInfo entity=patientDoctorRelationInfoService.getById(relationId);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            return null;
        }
        return entity;
    }


}
