package com.xxh.cloud.healthy.crm.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientServiceRecordInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientServiceRecordInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordAddDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordEditDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientServiceRecordVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.PatientServiceRecordConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationQueryService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientServiceRecordBizService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

@Service
public class PatientServiceRecordBizServiceImpl implements PatientServiceRecordBizService {


    @Autowired
    private PatientServiceRecordInfoService patientServiceRecordInfoService;

    @Autowired
    private PatientDoctorRelationQueryService patientDoctorRelationQueryService;

    @Autowired
    private AdminUserClientService adminUserClientService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DeptClientService deptClientService;


    @Override
    public PageData<PatientServiceRecordVO> query(PatientServiceRecordQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<PatientServiceRecordInfo> page = patientServiceRecordInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PatientServiceRecordInfo>()
                        .eq("rows_state", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getPatientId()),"patient_id",params.getPatientId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getDoctorId()),"doctor_id",params.getDoctorId())
                        .eq(StrUtil.isNotEmpty(params.getRelationId()),"relation_id",params.getRelationId())
                        .eq(StrUtil.isNotEmpty(params.getPhaseId()),"phase_id",params.getPhaseId())
                        .eq(StrUtil.isNotEmpty(params.getBizType()),"biz_type",params.getBizType())
                        .ge(StrUtil.isNotEmpty(params.getAddDtStart()),"add_dt",params.getAddDtStart())
                        .le(StrUtil.isNotEmpty(params.getAddDtEnd()),"add_dt",params.getAddDtEnd())
                        .orderByDesc("add_dt")

        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return new PageData<>(null, 0);
        }
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> patientIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getDoctorId())){
                userIds.add(item.getDoctorId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getPatientId())){
                patientIds.add(item.getPatientId());
            }
        });


        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        userMap.keySet().forEach(k-> deptIds.add(userMap.get(k).getDeptId()));
        // 查询部门
        Map<String, DeptBO> deptMaps =  deptClientService.getDeptMapByIds(deptIds);


        //todo: 患者
        PageData<PatientServiceRecordVO> data= new PageConverter<PatientServiceRecordInfo, PatientServiceRecordVO>() {
            @Override
            public PatientServiceRecordVO converter(PatientServiceRecordInfo entity) {
                return PatientServiceRecordConvert.getPatientServiceRecordVO(entity,userMap,diseaseMap, deptMaps);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public PatientServiceRecordVO add(PatientServiceRecordAddDTO params, LoginUser user) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationInfo patientDoctorRelation= patientDoctorRelationQueryService.getPatientDoctorRelationById(params.getRelationId());
        if(patientDoctorRelation==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"医生患者信息不存在");
        }
        // 只能变更自己的个案 mj.li 240920
        if (!user.getId().equals(patientDoctorRelation.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }
        PatientServiceRecordInfo entity=new PatientServiceRecordInfo();
        entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        entity.setRelationId(patientDoctorRelation.getId());
        entity.setDiseaseId(patientDoctorRelation.getDiseaseId());
        entity.setPatientId(patientDoctorRelation.getPatientId());
        entity.setDoctorId(patientDoctorRelation.getDoctorId());
        entity.setPhaseId(patientDoctorRelation.getPhaseId());
        entity.setBizType(params.getBizType());
        entity.setContent(params.getContent());
        entity.setAddDt(params.getAddDt() == null ? new Date() : params.getAddDt());
        entity.setRowsState(YN.YES.getCode());
        patientServiceRecordInfoService.save(entity);
        //todo:log
        return PatientServiceRecordConvert.getPatientServiceRecordVO(entity);
    }

    @Override
    public PatientServiceRecordVO edit(PatientServiceRecordEditDTO params, LoginUser user) {
        ValidatorUtils.validateEntity(params);
        PatientServiceRecordInfo entity= patientServiceRecordInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 只能变更自己的个案 mj.li 240920
        if (!user.getId().equals(entity.getDoctorId())) {
            throw new BusinessException("您无权限修改该病种");
        }
        entity.setBizType(params.getBizType());
        entity.setContent(params.getContent());
        entity.setAddDt(params.getAddDt() == null ? new Date() : params.getAddDt());
        patientServiceRecordInfoService.updateById(entity);
        //todo:log
        return PatientServiceRecordConvert.getPatientServiceRecordVO(entity);
    }

    @Override
    public PatientServiceRecordVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PatientServiceRecordInfo entity= patientServiceRecordInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        Set<String> patientIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getDoctorId())){
            userIds.add(entity.getDoctorId());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseId())){
            diseaseIds.add(entity.getDiseaseId());
        }
        if(StrUtil.isNotEmpty(entity.getPatientId())){
            patientIds.add(entity.getPatientId());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        userMap.keySet().forEach(k-> deptIds.add(userMap.get(k).getDeptId()));
        // 查询部门
        Map<String, DeptBO> deptMaps =  deptClientService.getDeptMapByIds(deptIds);

        //todo: 患者
        return PatientServiceRecordConvert.getPatientServiceRecordVO(entity,userMap,diseaseMap,deptMaps);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        PatientServiceRecordInfo entity= patientServiceRecordInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsState(YN.NO.getCode());
        patientServiceRecordInfoService.updateById(entity);
        //todo:log
    }


}
