package com.xxh.cloud.healthy.crm.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseTargetConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseTargetConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.cache.keys.CrmConfigCacheKeyUtil;
import com.xxh.cloud.healthy.crm.framework.modules.cache.service.DiseasePhaseConfigCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DiseasePhaseConfigCacheServiceImpl implements DiseasePhaseConfigCacheService {

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private DiseasePhaseConfigInfoService diseasePhaseConfigInfoService;


    @Override
    public DiseasePhaseConfigInfo getPhaseConfigById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        return this.getPhaseConfigById2DB(id);
    }


    @Override
    public List<DiseasePhaseConfigInfo> getPhaseConfigList(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        return this.getPhaseConfigList2DB(ids);
    }

    @Override
    public Map<String, DiseasePhaseConfigInfo> getPhaseConfigMap(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        return this.getPhaseConfigMap2DB(ids);
    }


    @Override
    public List<DiseasePhaseConfigInfo> getPhaseConfigListByDiseaseId(String diseaseId) {
        if(StrUtil.isEmpty(diseaseId)){
            return null;
        }
        List<DiseasePhaseConfigInfo> data= diseasePhaseConfigInfoService.list(
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .eq("disease_id",diseaseId)
                        .orderByAsc("orders")
        );
        return data;
    }

    @Override
    public Map<String, DiseasePhaseConfigInfo> getPhaseConfigMapByDiseaseId(String diseaseId) {
        if(StrUtil.isEmpty(diseaseId)){
            return null;
        }
        List<DiseasePhaseConfigInfo> list=this.getPhaseConfigListByDiseaseId(diseaseId);
        Map<String, DiseasePhaseConfigInfo> data=new HashMap<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(item.getId(),item);
            });
        }
        return data;
    }

    @Override
    public void removePhaseConfig(String id) {
        if(StrUtil.isEmpty(id)){
            return;
        }
        String key= CrmConfigCacheKeyUtil.getPhaseConfigKey(id);
        redisUtils.delete(key);
    }

    @Override
    public void removePhaseConfig(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return;
        }
        Set<String> keys=new HashSet<>();
        ids.forEach(id->{
            keys.add(CrmConfigCacheKeyUtil.getPhaseConfigKey(id));
        });
        redisUtils.delete(keys);
    }

    @Override
    public void removePhaseConfigAll() {
        Set<String> keys= redisUtils.keys(CrmConfigCacheKeyUtil.getPhaseConfigMainKey());
        if(CollUtil.isEmpty(keys)){
            return;
        }
        redisUtils.delete(keys);
    }



    @Override
    public DiseasePhaseConfigInfo getPhaseConfigById2DB(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        DiseasePhaseConfigInfo entity= diseasePhaseConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            return null;
        }
        return entity;
    }

    @Override
    public List<DiseasePhaseConfigInfo> getPhaseConfigList2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<DiseasePhaseConfigInfo> data= diseasePhaseConfigInfoService.list(
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .in("id",ids)
        );
        return data;
    }

    @Override
    public Map<String, DiseasePhaseConfigInfo> getPhaseConfigMap2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<DiseasePhaseConfigInfo> list= this.getPhaseConfigList2DB(ids);
        Map<String, DiseasePhaseConfigInfo> data=new HashMap<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(item.getId(),item);
            });
        }
        return data;
    }






}
