package com.xxh.cloud.healthy.crm.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseTargetConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseTargetConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.cache.keys.CrmConfigCacheKeyUtil;
import com.xxh.cloud.healthy.crm.framework.modules.cache.service.DiseasePhaseTargetConfigCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DiseasePhaseTargetConfigCacheServiceImpl implements DiseasePhaseTargetConfigCacheService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private DiseasePhaseTargetConfigInfoService diseasePhaseTargetConfigInfoService;


    @Autowired
    private DiseasePhaseConfigInfoService diseasePhaseConfigInfoService;



    @Override
    public DiseasePhaseTargetConfigInfo getTargetConfigById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        return this.getTargetConfigById2DB(id);
    }

    @Override
    public List<DiseasePhaseTargetConfigInfo> getTargetConfigList(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        return this.getTargetConfigList2DB(ids);
    }

    @Override
    public Map<String, DiseasePhaseTargetConfigInfo> getTargetConfigMap(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        return this.getTargetConfigMap2DB(ids);
    }

    @Override
    public DiseasePhaseTargetConfigInfo getTargetConfigById2DB(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        DiseasePhaseTargetConfigInfo entity= diseasePhaseTargetConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsState())){
            return null;
        }
        return entity;
    }

    @Override
    public List<DiseasePhaseTargetConfigInfo> getTargetConfigList2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<DiseasePhaseTargetConfigInfo> data= diseasePhaseTargetConfigInfoService.list(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .in("id",ids)
        );
        return data;
    }

    @Override
    public Map<String, DiseasePhaseTargetConfigInfo> getTargetConfigMap2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }
        List<DiseasePhaseTargetConfigInfo> list= this.getTargetConfigList2DB(ids);
        Map<String, DiseasePhaseTargetConfigInfo> data=new HashMap<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(item.getId(),item);
            });
        }
        return data;
    }

    @Override
    public List<DiseasePhaseTargetConfigInfo> getTargetConfigListByDiseaseId2DB(String diseaseId) {
        if(StrUtil.isEmpty(diseaseId)){
            return null;
        }

        List<DiseasePhaseConfigInfo> phaseList= diseasePhaseConfigInfoService.list(
                new QueryWrapper<DiseasePhaseConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .eq("disease_id",diseaseId)
                        .orderByAsc("orders")
        );


        if(CollUtil.isEmpty(phaseList)){
            return null;
        }

        List<String> phaseIds=new ArrayList<>();
        phaseList.forEach(item->{
            phaseIds.add(item.getId());
        });

        List<DiseasePhaseTargetConfigInfo> data= diseasePhaseTargetConfigInfoService.list(
                new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                        .eq("rows_state",YN.YES.getCode())
                        .in("phase_id",phaseIds)
                        .orderByAsc("phase_id")
        );

        return data;
    }

    @Override
    public Map<String, DiseasePhaseTargetConfigInfo> getTargetConfigMapByDiseaseId2DB(String diseaseId) {
        if(StrUtil.isEmpty(diseaseId)){
            return null;
        }
        List<DiseasePhaseTargetConfigInfo> targetConfigList= this.getTargetConfigListByDiseaseId2DB(diseaseId);
        if(CollUtil.isEmpty(targetConfigList)){
            return null;
        }
        Map<String, DiseasePhaseTargetConfigInfo> data=new HashMap<>();
        targetConfigList.forEach(item->{
            data.put(item.getId(),item);
        });
        return data;
    }

    @Override
    public void removeTargetConfig(String id) {
        if(StrUtil.isEmpty(id)){
            return;
        }
        String key= CrmConfigCacheKeyUtil.getTargetConfigKey(id);
        redisUtils.delete(key);
    }

    @Override
    public void removeTargetConfig(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return;
        }
        Set<String> keys=new HashSet<>();
        ids.forEach(id->{
            keys.add(CrmConfigCacheKeyUtil.getTargetConfigKey(id));
        });
        redisUtils.delete(keys);
    }

    @Override
    public void removeTargetConfigAll() {
        Set<String> keys= redisUtils.keys(CrmConfigCacheKeyUtil.getTargetConfigMainKey());
        if(CollUtil.isEmpty(keys)){
            return;
        }
        redisUtils.delete(keys);
    }

}
