package com.xxh.cloud.healthy.crm.framework.modules.client;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.crm.client.api.PatientDoctorRelationClientService;
import com.xxh.cloud.healthy.crm.client.api.bean.bo.PatientDoctorRelationBO;
import com.xxh.cloud.healthy.crm.client.api.bean.dto.PatientDoctorRelQueryClientDTO;
import com.xxh.cloud.healthy.crm.client.api.bean.dto.PatientDoctorRelSearchClientDTO;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationTargetInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.rel.PatientDoctorRelationSearchDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.convert.PatientDoctorRelationConvert;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationStatisticsBizService;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class PatientDoctorRelationClientServiceImpl implements PatientDoctorRelationClientService {

    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;
    @Autowired
    private PatientDoctorRelationStatisticsBizService patientDoctorRelationStatisticsBizService;
    @Autowired
    private PatientDoctorRelationTargetInfoService patientDoctorRelationTargetInfoService;


    @Override
    public List<PatientDoctorRelationBO> queryPatientDoctorRelList(PatientDoctorRelQueryClientDTO params) {
        List<PatientDoctorRelationInfo> entityList = patientDoctorRelationInfoService.list(new QueryWrapper<PatientDoctorRelationInfo>()
                .eq("rows_state", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotEmpty(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotEmpty(params.getSourceType()), "source_type", params.getSourceType())
                .in(CollUtil.isNotEmpty(params.getSourceObjIds()), "source_obj_id", params.getSourceObjIds())
                .in(CollUtil.isNotEmpty(params.getSourceObjIds()), "state", params.getSourceObjIds())
                .orderByDesc("create_date"));
        if (CollUtil.isEmpty(entityList)) {
            return new ArrayList<>();
        }

        return entityList.stream().map(PatientDoctorRelationConvert::getPatientDoctorRelationBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, PatientDoctorRelationBO> queryPatientDoctorRelMapLast(PatientDoctorRelQueryClientDTO params) {
        Map<String, PatientDoctorRelationBO> map = new HashMap<>();
        List<PatientDoctorRelationBO> list = queryPatientDoctorRelList(params);
        if (CollUtil.isEmpty(list)) {
            return map;
        }
        for (PatientDoctorRelationBO bo : list) {
            if (!map.containsKey(bo.getPatientId())) {
                map.put(bo.getPatientId(), bo);
            }

        }
        return map;
    }

    @Override
    public List<String> relationSearch(PatientDoctorRelSearchClientDTO params) {
        if (StrUtil.isEmpty(params.getDiseaseId()) && CollUtil.isEmpty(params.getDiseaseIds())) {
            return new ArrayList<>();
        }
        PatientDoctorRelationSearchDTO dto = new PatientDoctorRelationSearchDTO();
        dto.setPatientId(params.getPatientId());
        dto.setDoctorId(params.getDoctorId());
        dto.setDiseaseId(params.getDiseaseId());
        dto.setDiseaseIds(params.getDiseaseIds());
        return patientDoctorRelationStatisticsBizService.relationSearch(dto);
    }
}
