package com.xxh.cloud.healthy.crm.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.DiseasePhaseTargetConfigInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.entity.PatientDoctorRelationTargetInfo;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.DiseasePhaseTargetConfigInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.base.service.PatientDoctorRelationTargetInfoService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.bos.ConditionContentBO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.DwdPatientChangeQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.DwdPatientComeQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.mapper.PatientComeDataBizMapper;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DwdPatientComeDataBizService;
import com.xxh.cloud.healthy.crm.framework.modules.constant.DiseaseTargetPhaseTypeEnum;
import com.xxh.cloud.healthy.crm.framework.modules.constant.PDRTargetResultStateEnum;
import com.xxh.cloud.healthy.crm.framework.modules.constant.PatientDoctorRelationStateEnum;
import com.xxh.cloud.healthy.crm.framework.modules.task.service.PatientDoctorRelationTargetTaskService;
import com.xxl.job.core.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Slf4j
@Service
public class PatientDoctorRelationTargetTaskServiceImpl implements PatientDoctorRelationTargetTaskService {
    
    @Autowired
    private DwdPatientComeDataBizService dwdPatientComeDataBizService;
    @Autowired
    private PatientDoctorRelationTargetInfoService patientDoctorRelationTargetInfoService;
    @Autowired
    private DiseasePhaseTargetConfigInfoService diseasePhaseTargetConfigInfoService;
    @Resource
    private PatientComeDataBizMapper patientComeDataBizMapper;
    @Autowired
    private PatientDoctorRelationInfoService patientDoctorRelationInfoService;
    
    /**
     * 更新个案目标状态
     */
    @Override
    public void updateTargetStatusTask() {
        // 循环批处理个案目标状态
        String id = "0";
        while (true) {
            if (id == null) {
                return;
            }
            id = queryAndDealTargetStatus(id);
        }

    }

    // 查询未执行状态的目标数据，并循环处理目标状态
    private String queryAndDealTargetStatus(String id) {
        Date now = new Date(); // 当前时间

        // 查询未执行状态的目标数据
        List<PatientDoctorRelationTargetInfo> relationTargetList = patientDoctorRelationTargetInfoService.list(new QueryWrapper<PatientDoctorRelationTargetInfo>()
                .eq("rows_state", YN.YES.getCode())
                .eq("result_state", PDRTargetResultStateEnum.PDRTRS_None.getCode())
                .gt("id", id)
                .orderByAsc("id")
                .last(" limit 1000 ")
        );
        if (CollectionUtil.isEmpty(relationTargetList)) {
            return null;
        }

        Set<String> relationIds = new HashSet<>();
        Set<String> targetIds = new HashSet<>();
        relationTargetList.forEach(i -> {
            relationIds.add(i.getRelationId());
            targetIds.add(i.getTargetId());
        });

        // 查询医生患者关系表
        Map<String, PatientDoctorRelationInfo> relationMap = new HashMap<>();
        List<PatientDoctorRelationInfo> relationList = patientDoctorRelationInfoService.list(new QueryWrapper<PatientDoctorRelationInfo>()
            .in("id", relationIds)
        );
        if (CollUtil.isNotEmpty(relationList)) {
            relationList.forEach(i -> relationMap.put(i.getId(), i));
        }

        // 查询目标配置信息
        Map<String, DiseasePhaseTargetConfigInfo> targetConfigMap = new HashMap<>();
        List<DiseasePhaseTargetConfigInfo> targetConfigList = diseasePhaseTargetConfigInfoService.list(new QueryWrapper<DiseasePhaseTargetConfigInfo>()
                .in("id", targetIds)
        );
        if (CollUtil.isNotEmpty(targetConfigList)) {
            targetConfigList.forEach(i -> targetConfigMap.put(i.getId(), i));
        }

        // 循环处理
        List<PatientDoctorRelationTargetInfo> targetList = new ArrayList<>(); // 需要更新目标状态的数据
        relationTargetList.forEach(i -> {
            DiseasePhaseTargetConfigInfo targetConfig = targetConfigMap.get(i.getTargetId());
            if (null == targetConfig) {
                return;
            }

            PatientDoctorRelationInfo relation = relationMap.get(i.getRelationId());
            if (null == relation) {
                return;
            }

            Boolean boo = this.targetExeStatus(relation, targetConfig, i.getStartDt(), i.getEndDt());
            if (boo) {
                // i.setEndDt(now);
                i.setResultState(PDRTargetResultStateEnum.PDRTRS_Completed.getCode());
                i.setResultDt(now);
                targetList.add(i);
            }

            // 校验目标过期
            if (null != i.getLimitDt()) {
                if (now.compareTo(i.getLimitDt()) > 0) {
                    i.setResultState(PDRTargetResultStateEnum.PDRTRS_Expire.getCode());
                    i.setResultDt(now);
                    targetList.add(i);
                }
            } else if (null != targetConfig.getCompletionLimit() && targetConfig.getCompletionLimit() > 0) {
                Date endDate =  DateUtil.addDays(i.getStartDt(), targetConfig.getCompletionLimit());
                if (now.compareTo(endDate) > 0) {
                    i.setResultState(PDRTargetResultStateEnum.PDRTRS_Expire.getCode());
                    i.setResultDt(now);
                    targetList.add(i);
                }
            }
        });

        if (CollectionUtil.isNotEmpty(targetList)) {
            // 更新当前目标状态，并同步更新历史表
            patientDoctorRelationTargetInfoService.updateBatchById(targetList);
        }

        return relationTargetList.get(relationTargetList.size() - 1).getId();
    }

    private Boolean targetExeStatus(PatientDoctorRelationInfo relation, DiseasePhaseTargetConfigInfo targetConfig, Date startDate, Date endDate) {
        Boolean boo = false;
        String conditionType = targetConfig.getConditionType();
        ConditionContentBO bo = JSONObject.parseObject(targetConfig.getConditionContent(), ConditionContentBO.class);
        // 判断完成时间
        if (null == endDate && null != targetConfig.getCompletionLimit() && targetConfig.getCompletionLimit() > 0) {
            endDate =  DateUtil.addDays(startDate, targetConfig.getCompletionLimit());
        }

        // 查询dwd确定更新的目标状态
        Integer count = 0;

        // 就诊
        if (DiseaseTargetPhaseTypeEnum.DTPT_visit.getCode().equals(conditionType)) {
            DwdPatientComeQueryDTO dwdDoctorComeQueryDTO = new DwdPatientComeQueryDTO();
            dwdDoctorComeQueryDTO.setPatientId(relation.getPatientId());
            dwdDoctorComeQueryDTO.setStartDate(startDate);
            dwdDoctorComeQueryDTO.setEndDate(endDate);
            if (StrUtil.isNotEmpty(targetConfig.getTargetExeDeptIds())) {
                dwdDoctorComeQueryDTO.setDeptIds(StrJoinSplitUtils.split2List(targetConfig.getTargetExeDeptIds()));
            }
            count = dwdPatientComeDataBizService.visitCount(dwdDoctorComeQueryDTO);
        }
        // 住院
        else if (DiseaseTargetPhaseTypeEnum.DPCS_In_Hospital.getCode().equals(conditionType)) {
            DwdPatientComeQueryDTO dwdDoctorComeQueryDTO = new DwdPatientComeQueryDTO();
            dwdDoctorComeQueryDTO.setPatientId(relation.getPatientId());
            dwdDoctorComeQueryDTO.setStartDate(startDate);
            dwdDoctorComeQueryDTO.setEndDate(endDate);
            if (StrUtil.isNotEmpty(targetConfig.getTargetExeDeptIds())) {
                dwdDoctorComeQueryDTO.setDeptIds(StrJoinSplitUtils.split2List(targetConfig.getTargetExeDeptIds()));
            }
            count = dwdPatientComeDataBizService.inHospitalCount(dwdDoctorComeQueryDTO);
        }
        // 会诊
        else if (DiseaseTargetPhaseTypeEnum.DPCS_Multi_Treatment.getCode().equals(conditionType)) {
            count = patientComeDataBizMapper.consultationCount(new QueryWrapper<Void>()
                    .eq("a.patient_id", relation.getPatientId())
                    .in("b.mdt_dept_code", StrJoinSplitUtils.split2List(targetConfig.getTargetExeDeptIds()))
                    .ge("a.submit_time", startDate)
                    .le(null!=endDate,"a.submit_time", endDate)
            );
        }
        // 随访
        else if (DiseaseTargetPhaseTypeEnum.DPCS_Follow_Up.getCode().equals(conditionType)) {
            count = patientComeDataBizMapper.followUpCount(new QueryWrapper<Void>()
                    .eq("patient_id", relation.getPatientId())
                    .eq("doctor_id", relation.getDoctorId())
                    .eq("status", "done")
                    .ge("follow_time", startDate)
                    .le(null!=endDate,"follow_time", endDate)
            );
        }
        // 转科室
        else if (DiseaseTargetPhaseTypeEnum.DPCS_Change_Dept.getCode().equals(conditionType)) {
            DwdPatientChangeQueryDTO dwdDoctorComeQueryDTO = new DwdPatientChangeQueryDTO();
            dwdDoctorComeQueryDTO.setPatientId(relation.getPatientId());
            dwdDoctorComeQueryDTO.setChangeType("ZKZRKS");
            dwdDoctorComeQueryDTO.setStartDate(startDate);
            dwdDoctorComeQueryDTO.setEndDate(endDate);
            if (StrUtil.isNotEmpty(targetConfig.getTargetExeDeptIds())) {
                dwdDoctorComeQueryDTO.setDeptIds(StrJoinSplitUtils.split2List(targetConfig.getTargetExeDeptIds()));
            }
            count = dwdPatientComeDataBizService.inpatientChangeCount(dwdDoctorComeQueryDTO);
        }
        // 自定义条件
        else if (DiseaseTargetPhaseTypeEnum.DPCS_Cust_Condition.getCode().equals(conditionType)) {
            // TODO 
        }

        // 只要有数据则更新状态
        if (count > 0) {
            boo = true;
        }

        return boo;
    }

    @Override
    public void updatePhaseStatusTask() {
        // 循环批处理个案目标状态
        String id = "0";
        while (true) {
            if (id == null) {
                return;
            }
            id = queryAndDealPhaseStatus(id);
        }
    }

    // 查询正常状态的阶段数据，并循环处理阶段状态
    private String queryAndDealPhaseStatus(String id) {
        List<PatientDoctorRelationInfo> relationList = patientDoctorRelationInfoService.list(new QueryWrapper<PatientDoctorRelationInfo>()
                .eq("rows_state", YN.YES.getCode())
                .eq("state", PatientDoctorRelationStateEnum.PDRS_Normal.getCode())
                .gt("id", id)
                .orderByAsc("id")
                .last(" limit 1000 ")
        );
        if (CollectionUtil.isEmpty(relationList)) {
            return null;
        }

        Set<String> relationIds = new HashSet<>();
        relationList.forEach(i -> {
            relationIds.add(i.getId());
        });

        // // 查询未执行状态的目标数据，整理出对应的relationId，则该id对应的阶段未完成，没有的则为完成
        // Set<String> relationTargetSet = new HashSet<>();
        // List<PatientDoctorRelationTargetInfo> relationTargetList = patientDoctorRelationTargetInfoService.list(new QueryWrapper<PatientDoctorRelationTargetInfo>()
        //         .eq("rows_state", YN.YES.getCode())
        //         .eq("result_state", PDRTargetResultStateEnum.PDRTRS_None.getCode())
        //         .in("relation_id", relationIds)
        // );
        // if (CollUtil.isNotEmpty(relationTargetList)) {
        //     relationTargetList.forEach(i -> relationTargetSet.add(i.getRelationId())); //
        // }

        // // 循环处理
        // List<PatientDoctorRelationInfo> phaseList = new ArrayList<>(); // 需要更新目标状态的数据
        // relationList.forEach(i -> {
        //     if (!relationTargetSet.contains(i.getId())) { // 不存在则为完成，存在则说明还有目标没完成
        //         i.setState(PatientDoctorRelationStateEnum.PDRS_Relation_Complete.getCode());
        //         phaseList.add(i);
        //     }
        // });

        Map<String, List<String>> relationTargetMap = new HashMap<>(); // key: relationId  v: 各个目标的完成状态
        List<PatientDoctorRelationTargetInfo> relationTargetList = patientDoctorRelationTargetInfoService.list(new QueryWrapper<PatientDoctorRelationTargetInfo>()
                .eq("rows_state", YN.YES.getCode())
                .in("relation_id", relationIds)
        );
        if (CollUtil.isNotEmpty(relationTargetList)) {
            relationTargetList.forEach(i -> {
                List<String> li;
                if (relationTargetMap.containsKey(i.getRelationId())) {
                    li = relationTargetMap.get(i.getRelationId());
                } else {
                    li= new ArrayList<>();
                }
                li.add(i.getResultState());
                relationTargetMap.put(i.getRelationId(), li);
            });
        }

        // 循环处理
        List<PatientDoctorRelationInfo> phaseList = new ArrayList<>(); // 需要更新目标状态的数据
        relationList.forEach(i -> {
            if (!relationTargetMap.containsKey(i.getId())) { // 不存在说明该阶段不存在目标，则该阶段不变
                return;
            }
            List<String> targetStateList = relationTargetMap.get(i.getId());
            if (targetStateList.contains(PDRTargetResultStateEnum.PDRTRS_Expire.getCode())) {
                i.setState(PatientDoctorRelationStateEnum.PDRS_Relation_Expire.getCode()); // 目标过去，则阶段过期
                phaseList.add(i);
            } else if (!targetStateList.contains(PDRTargetResultStateEnum.PDRTRS_None.getCode())) {
                i.setState(PatientDoctorRelationStateEnum.PDRS_Relation_Complete.getCode()); // 没有未完成的目标
                phaseList.add(i);
            }
        });

        if (CollectionUtil.isNotEmpty(phaseList)) {
            // 更新当前阶段状态
            patientDoctorRelationInfoService.updateBatchById(phaseList);
        }

        return relationList.get(relationList.size() - 1).getId();
    }

}
