package com.xxh.cloud.healthy.crm.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DiseasePhaseConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/crm/diseasePhaseConfig")
@Api(tags = "病种阶段管理")
public class DiseasePhaseConfigMgrController {


    @Autowired
    private DiseasePhaseConfigBizService diseasePhaseConfigBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页查询", response = DiseasePhaseConfigVO.class)
    public Result<PageData<DiseasePhaseConfigVO>> query(@RequestBody DiseasePhaseConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DiseasePhaseConfigVO> data = diseasePhaseConfigBizService.query(params);
        return new Result<PageData<DiseasePhaseConfigVO>>().ok(data);
    }



    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DiseasePhaseConfigVO.class)
    public Result<DiseasePhaseConfigVO> add(@RequestBody DiseasePhaseConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseConfigVO data = diseasePhaseConfigBizService.add(params);
        return new Result<DiseasePhaseConfigVO>().ok(data);
    }

    @PostMapping("/edit")
    @ApiOperation(value = "修改", response = DiseasePhaseConfigVO.class)
    public Result<DiseasePhaseConfigVO> edit(@RequestBody DiseasePhaseConfigEditDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseConfigVO data = diseasePhaseConfigBizService.edit(params);
        return new Result<DiseasePhaseConfigVO>().ok(data);
    }

    @PostMapping("/updateState")
    @ApiOperation(value = "更新状态", response = String.class)
    public Result<String> updateState(@RequestBody DiseasePhaseConfigEditStateDTO params) {
        ValidatorUtils.validateEntity(params);
        diseasePhaseConfigBizService.updateState(params);
        return new Result<String>().ok("ok");
    }

    @PostMapping("/infoById")
    @ApiOperation(value = "获取", response = DiseasePhaseConfigVO.class)
    public Result<DiseasePhaseConfigVO> infoById(@RequestBody DiseasePhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseConfigVO data = diseasePhaseConfigBizService.getById(params.getId());
        return new Result<DiseasePhaseConfigVO>().ok(data);
    }

    @PostMapping("/delById")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> delById(@RequestBody DiseasePhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        diseasePhaseConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @PostMapping("/addDef")
    @ApiOperation(value = "新增默认数据", response = DiseasePhaseConfigVO.class)
    public Result<String> addDef(@RequestBody DiseasePhaseConfigAddDefDTO params) {
        ValidatorUtils.validateEntity(params);
        diseasePhaseConfigBizService.addDef(params.getDiseaseId());
        return new Result<String>().success();
    }

}
