package com.xxh.cloud.healthy.crm.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.DiseasePhaseConfigListDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.DiseaseTargetPhaseConfigGetDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.query.DiseasePhaseConfigDataListDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DiseasePhaseConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/crm/diseasePhaseConfigQuery")
@Api(tags = "病种阶段和目标查询服务")
public class DiseasePhaseQueryMgrController {

    @Autowired
    private DiseasePhaseConfigBizService diseasePhaseConfigBizService;

    @PostMapping("/phaseConfigList")
    @ApiOperation(value = "根据病种获取阶段", response = DiseasePhaseConfigVO.class)
    public Result<List<DiseasePhaseConfigVO>> phaseConfigList(@RequestBody DiseasePhaseConfigDataListDTO params) {
        ValidatorUtils.validateEntity(params);

        DiseasePhaseConfigListDTO queryParams=new DiseasePhaseConfigListDTO();
        queryParams.setDiseaseId(params.getDiseaseId());
        queryParams.setDiseaseIds(params.getDiseaseIds());
        queryParams.setState(params.getState());
        queryParams.setNotInIds(params.getNotInIds());
        List<DiseasePhaseConfigVO> data = diseasePhaseConfigBizService.list(queryParams);

        return new Result<List<DiseasePhaseConfigVO>>().ok(data);
    }

    @PostMapping("/diseaseByDoctor")
    @ApiOperation(value = "医生所在部门可管理的病种", response = List.class)
    public Result<List<String>> diseaseByDoctor(@RequestBody DiseaseTargetPhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        return new Result<List<String>>().ok(diseasePhaseConfigBizService.getDiseaseByDoctor(params.getId()));
    }

    @PostMapping("/deptByDiseaseId")
    @ApiOperation(value = "病种所属部门", response = List.class)
    public Result<List<String>> deptByDiseaseId(@RequestBody DiseaseTargetPhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        return new Result<List<String>>().ok(diseasePhaseConfigBizService.getDeptByDiseaseId(params.getId()));
    }
}
