package com.xxh.cloud.healthy.crm.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseTargetConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.DiseasePhaseTargetConfigBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/crm/diseaseTargetPhaseConfig")
@Api(tags = "病种阶段目标管理")
public class DiseasePhaseTargetConfigMgrController {

    @Autowired
    private DiseasePhaseTargetConfigBizService diseasePhaseTargetConfigBizService;


    @PostMapping("/query")
    @ApiOperation(value = "分页查询", response = DiseasePhaseTargetConfigVO.class)
    public Result<PageData<DiseasePhaseTargetConfigVO>> query(@RequestBody DiseaseTargetPhaseConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DiseasePhaseTargetConfigVO> data = diseasePhaseTargetConfigBizService.query(params);
        return new Result<PageData<DiseasePhaseTargetConfigVO>>().ok(data);
    }


    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DiseasePhaseTargetConfigVO.class)
    public Result<DiseasePhaseTargetConfigVO> add(@RequestBody DiseaseTargetPhaseConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseTargetConfigVO data = diseasePhaseTargetConfigBizService.add(params);
        return new Result<DiseasePhaseTargetConfigVO>().ok(data);
    }

    @PostMapping("/edit")
    @ApiOperation(value = "修改", response = DiseasePhaseTargetConfigVO.class)
    public Result<DiseasePhaseTargetConfigVO> edit(@RequestBody DiseaseTargetPhaseConfigEditDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseTargetConfigVO data = diseasePhaseTargetConfigBizService.edit(params);
        return new Result<DiseasePhaseTargetConfigVO>().ok(data);
    }

    @PostMapping("/updateState")
    @ApiOperation(value = "更新状态", response = String.class)
    public Result<String> updateState(@RequestBody DiseaseTargetPhaseConfigUpdateStateDTO params) {
        ValidatorUtils.validateEntity(params);
        diseasePhaseTargetConfigBizService.updateState(params);
        return new Result<String>().ok("ok");
    }

    @PostMapping("/infoById")
    @ApiOperation(value = "获取", response = DiseasePhaseTargetConfigVO.class)
    public Result<DiseasePhaseTargetConfigVO> infoById(@RequestBody DiseaseTargetPhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DiseasePhaseTargetConfigVO data = diseasePhaseTargetConfigBizService.getById(params.getId());
        return new Result<DiseasePhaseTargetConfigVO>().ok(data);
    }

    @PostMapping("/delById")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> delById(@RequestBody DiseaseTargetPhaseConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        diseasePhaseTargetConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }





}
