package com.xxh.cloud.healthy.crm.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.rel.*;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationChangeVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationFullVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientDoctorRelationVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientServiceRecordVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationBizService;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientDoctorRelationQueryService;
import com.xxh.cloud.healthy.crm.framework.modules.constant.PatientDoctorRelationAddWayEnum;
import com.xxh.cloud.healthy.crm.framework.modules.constant.PatientDoctorRelationSourceTypeEnum;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/crm/patientDoctorRel")
@Api(tags = "患者医生服务关系管理")
public class PatientDoctorRelationMgrController {


    @Autowired
    private PatientDoctorRelationBizService patientDoctorRelationBizService;

    @Autowired
    private PatientDoctorRelationQueryService patientDoctorRelationQueryService;


    @PostMapping("/query")
    @ApiOperation(value = "查询我的客户(我的)", response = PatientDoctorRelationFullVO.class)
    public Result<PageData<PatientDoctorRelationFullVO>> query(@RequestBody PatientDoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setDoctorId(user.getId());
        PageData<PatientDoctorRelationFullVO> data = patientDoctorRelationQueryService.queryPdrFull(params);
        return new Result<PageData<PatientDoctorRelationFullVO>>().ok(data);
    }


    @PostMapping("/queryAll")
    @ApiOperation(value = "查询我的客户(所有)", response = PatientDoctorRelationFullVO.class)
    public Result<PageData<PatientDoctorRelationFullVO>> queryAll(@RequestBody PatientDoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<PatientDoctorRelationFullVO> data = patientDoctorRelationQueryService.queryPdrFull(params);
        return new Result<PageData<PatientDoctorRelationFullVO>>().ok(data);
    }


    @PostMapping("/add")
    @ApiOperation(value = "关注(新增关系)", response = PatientDoctorRelationVO.class)
    public Result<PatientDoctorRelationVO> add(@RequestBody PatientDoctorRelationAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();

        params.setSourceType(PatientDoctorRelationSourceTypeEnum.PDRS_PatientLabel.getCode());
        params.setAddWay(PatientDoctorRelationAddWayEnum.PDR_Way_Human.getCode());
        params.setAddUserId(user.getId());

        PatientDoctorRelationVO data = patientDoctorRelationBizService.add(params);
        return new Result<PatientDoctorRelationVO>().ok(data);
    }


    @PostMapping("/updatePhase")
    @ApiOperation(value = "更改阶段", response = PatientDoctorRelationVO.class)
    public Result<PatientDoctorRelationVO> updatePhase(@RequestBody PatientDoctorRelationUpdatePhaseDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setOpUserId(user.getId());
        PatientDoctorRelationVO data = patientDoctorRelationBizService.updatePhase(params);
        return new Result<PatientDoctorRelationVO>().ok(data);
    }

    @PostMapping("/updateTarget")
    @ApiOperation(value = "更改目标", response = PatientDoctorRelationVO.class)
    public Result<PatientDoctorRelationVO> updateTarget(@RequestBody PatientDoctorRelationUpdateTargetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setOpUserId(user.getId());
        PatientDoctorRelationVO data = patientDoctorRelationBizService.updateTarget(params);
        return new Result<PatientDoctorRelationVO>().ok(data);
    }

    @PostMapping("/abandonPatient")
    @ApiOperation(value = "放弃患者", response = PatientDoctorRelationVO.class)
    public Result<PatientDoctorRelationVO> abandonPatient(@RequestBody PatientDoctorRelationAbandonDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setOpUserId(user.getId());
        PatientDoctorRelationVO data = patientDoctorRelationBizService.abandonPatient(params);
        return new Result<PatientDoctorRelationVO>().ok(data);
    }

    @PostMapping("/transferPatient")
    @ApiOperation(value = "移交患者", response = PatientDoctorRelationVO.class)
    public Result<PatientDoctorRelationVO> transferPatient(@RequestBody PatientDoctorRelationTransferDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setOpUserId(user.getId());
        PatientDoctorRelationVO data = patientDoctorRelationBizService.transferPatient(params);
        return new Result<PatientDoctorRelationVO>().ok(data);
    }


    @PostMapping("/infoById")
    @ApiOperation(value = "获取患者", response = PatientDoctorRelationFullVO.class)
    public Result<PatientDoctorRelationFullVO> infoById(@RequestBody PatientDoctorRelationGetDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientDoctorRelationFullVO data = patientDoctorRelationQueryService.getPdrFullById(params.getId());
        return new Result<PatientDoctorRelationFullVO>().ok(data);
    }

    @PostMapping("/relDiseaseList")
    @ApiOperation(value = "个案病种列表", response = PatientDoctorRelationFullVO.class)
    public Result<List<StandardDiseaseBO>> relDiseaseList(@RequestBody PatientDoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<StandardDiseaseBO> list = patientDoctorRelationQueryService.queryDisease(params);
        return new Result<List<StandardDiseaseBO>>().ok(list);
    }

    @PostMapping("/relationChange")
    @ApiOperation(value = "个案变更查询", response = PatientDoctorRelationFullVO.class)
    public Result<List<PatientDoctorRelationChangeVO>> relationChange(@RequestBody PatientDoctorRelationChangeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PatientDoctorRelationChangeVO> list = patientDoctorRelationQueryService.relationChange(params);
        return new Result<List<PatientDoctorRelationChangeVO>>().ok(list);
    }

}
