package com.xxh.cloud.healthy.crm.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.config.DiseasePhaseConfigQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordAddDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordEditDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordGetDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.dto.record.PatientServiceRecordQueryDTO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.DiseasePhaseConfigVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.bean.vos.PatientServiceRecordVO;
import com.xxh.cloud.healthy.crm.framework.modules.biz.service.PatientServiceRecordBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/crm/patientServiceRecord")
@Api(tags = "医生患者服务记录管理")
public class PatientServiceRecordMgrController {

    @Autowired
    private PatientServiceRecordBizService patientServiceRecordBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页查询", response = PatientServiceRecordVO.class)
    public Result<PageData<PatientServiceRecordVO>> query(@RequestBody PatientServiceRecordQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<PatientServiceRecordVO> data = patientServiceRecordBizService.query(params);
        return new Result<PageData<PatientServiceRecordVO>>().ok(data);
    }

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = PatientServiceRecordVO.class)
    public Result<PatientServiceRecordVO> add(@RequestBody PatientServiceRecordAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        PatientServiceRecordVO data = patientServiceRecordBizService.add(params, user);
        return new Result<PatientServiceRecordVO>().ok(data);
    }


    @PostMapping("/edit")
    @ApiOperation(value = "修改", response = PatientServiceRecordVO.class)
    public Result<PatientServiceRecordVO> edit(@RequestBody PatientServiceRecordEditDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        PatientServiceRecordVO data = patientServiceRecordBizService.edit(params, user);
        return new Result<PatientServiceRecordVO>().ok(data);
    }


    @PostMapping("/getById")
    @ApiOperation(value = "获取", response = PatientServiceRecordVO.class)
    public Result<PatientServiceRecordVO> getById(@RequestBody PatientServiceRecordGetDTO params) {
        ValidatorUtils.validateEntity(params);
        PatientServiceRecordVO data = patientServiceRecordBizService.getById(params.getId());
        return new Result<PatientServiceRecordVO>().ok(data);
    }


    @PostMapping("/delById")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> delById(@RequestBody PatientServiceRecordGetDTO params) {
        ValidatorUtils.validateEntity(params);
        patientServiceRecordBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


}
