package com.xxh.cloud.healthy.service.com.app.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientDataQueryDTO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataArticleExamineQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientArchivesVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeTargetVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientArchivesQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/data/patient")
@Api(tags = "app患者数据管理")
public class HealthyDataPatientAppController {
    
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private PatientArchivesQueryBizService patientArchivesQueryBizService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private VisitVerificationClientService visitVerificationClientService;

    private MemberFamilyBO getMemberFamily(String userId) {
        MemberFamilyBO memberFamilyBO = customerFamilyClientService.getChoose(userId);
        // if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getPatHisNo())) {
        //     return null;
        // }
        if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getMainPersonId())) {
            return null;
        }
        
        // 黑名单验证
        // VisitVerificationSearchDTO visitVerificationSearchDTO = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), memberFamilyBO.getPatHisNo());
        Map<String, List<VisitVerificationBO>> verificationMap = visitVerificationClientService.getVisitVerificationByPatient(Arrays.asList(memberFamilyBO.getPatHisNo()));
        if (CollUtil.isNotEmpty(verificationMap) && isBlack(verificationMap.get(memberFamilyBO.getPatHisNo()))) {
            return null;
        }

        return memberFamilyBO;
    }
    
    private Boolean isBlack(List<VisitVerificationBO> verificationList) {
        if (CollUtil.isNotEmpty(verificationList)) {
            for (VisitVerificationBO bo : verificationList) {
                if (BasicVisitConstant.VerType.BLACK_LIST.getCode().equals(bo.getVerType())) {
                    return true;
                }
            }
        }
        return false;
    }

    @PostMapping("/info")
    @ApiOperation(value = "查询患者详情", response = DoctorDwdPatientVO.class)
    public Result<DoctorDwdPatientVO> detail(@RequestBody TaskPatientGetDTO params) {
        log.info("/app/healthy/data/patient/info >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        MemberFamilyBO memberFamilyBO = this.getMemberFamily(user.getId());
        if (null == memberFamilyBO) {
            return new Result<DoctorDwdPatientVO>().success();
        }
        params.setPatientId(memberFamilyBO.getPatHisNo());
        // params.setPatientId(memberFamilyBO.getMainPersonId());
        TaskPatientVO data = taskPatientQueryBizService.getOneSimple(params);

        DoctorDwdPatientVO vo = new DoctorDwdPatientVO();
        if (null != data) {
            vo.setPatientId(data.getPatientId());
            vo.setPatientName(data.getPatientName());
            vo.setPatientGenderStandard(data.getPatientGenderStandard());
            vo.setBirthday(data.getBirthday());
            vo.setPhone(data.getPhone());
            vo.setPatientAge(data.getAge());
        }
        vo.setMemberFamilyBO(memberFamilyBO);

        return new Result<DoctorDwdPatientVO>().ok(vo);
    }

    @Deprecated
    // 查询用户档案及服务
    @PostMapping("/archives")
    @ApiOperation(value = "查询用户档案及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesData(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/app/healthy/data/patient/archives >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        MemberFamilyBO memberFamilyBO = this.getMemberFamily(user.getId());
        if (null == memberFamilyBO) {
            return new Result<PatientArchivesVO>().success();
        }
        // params.setPatientId(memberFamilyBO.getPatHisNo());
        params.setPatientId(memberFamilyBO.getMainPersonId());
        
        if (StrUtil.isBlank(params.getQueryDate()) && StrUtil.isNotBlank(params.getEndDate())) {
            params.setQueryDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }
        PatientArchivesVO data = patientArchivesQueryBizService.patientArchives(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    // 查询用户档案+体检及服务
    @PostMapping("/archives/v2")
    @ApiOperation(value = "查询用户档案+体检及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesDataV2(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/app/healthy/data/patient/archives/v2 >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        MemberFamilyBO memberFamilyBO = this.getMemberFamily(user.getId());
        if (null == memberFamilyBO) {
            return new Result<PatientArchivesVO>().success();
        }
        // params.setPatientId(memberFamilyBO.getMainPersonId());
        params.setMainPersonId(memberFamilyBO.getMainPersonId());
        
        if (StrUtil.isBlank(params.getQueryDate()) && StrUtil.isNotBlank(params.getEndDate())) {
            params.setQueryDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }
        PatientArchivesVO data = patientArchivesQueryBizService.patientMergeArchives(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    // 异步查询用户档案+体检及服务
    @PostMapping("/archives/v3")
    @ApiOperation(value = "异步查询用户档案+体检及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesDataV3(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/app/healthy/data/patient/archives/v3 >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        MemberFamilyBO memberFamilyBO = this.getMemberFamily(user.getId());
        if (null == memberFamilyBO) {
            return new Result<PatientArchivesVO>().success();
        }
        // params.setPatientId(memberFamilyBO.getMainPersonId());
        params.setMainPersonId(memberFamilyBO.getMainPersonId());

        if (StrUtil.isBlank(params.getQueryDate()) && StrUtil.isNotBlank(params.getEndDate())) {
            params.setQueryDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }
        PatientArchivesVO data = patientArchivesQueryBizService.patientMergeArchivesV3(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    @SaIgnore
    @PostMapping("/examineReport/list")
    @ApiOperation(value = "文章中的检查报告列表", response = Map.class)
    public Result<List<Map<String, String>>> examineReportList(@RequestBody DataArticleExamineQueryDTO params) {
        log.info("/app/healthy/data/patient/examineReport/list >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        List<Map<String, String>> data = patientArchivesQueryBizService.examineReportList(params);
        return new Result<List<Map<String, String>>>().ok(data);
    }

    @SaIgnore
    @PostMapping("/laboratoryReport/list")
    @ApiOperation(value = "文章中的检验报告列表", response = Map.class)
    public Result<List<Map<String, String>>> laboratoryReportList(@RequestBody DataArticleExamineQueryDTO params) {
        log.info("/app/healthy/data/patient/laboratoryReport/list >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        List<Map<String, String>> data = patientArchivesQueryBizService.laboratoryReportList(params);
        return new Result<List<Map<String, String>>>().ok(data);
    }

}
