package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.LabelPatientStatDimensionTypeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.LabelPatientStatisticClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientGetDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorPatientLabelQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientFullVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DiseaseLabelStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDataQueryService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/doctor/data")
@Api(tags = "医生数据[病种、标签、患者]")
public class DoctorDataQueryController {
    
    @Autowired
    private DoctorDataQueryService doctorDataQueryService;

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/disease/list")
    @ApiOperation(value = "医生病种列表", response = StandardDiseaseBO.class)
    public Result<List<StandardDiseaseBO>> diseaseList() {
        log.info("/sys/doctor/data/disease/list >>> ");
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<StandardDiseaseBO> data = doctorDataQueryService.diseaseList(user.getDeptId());
        return new Result<List<StandardDiseaseBO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/label/query")
    @ApiOperation(value = "医生标签分页", response = LabelBO.class)
    public Result<PageData<LabelBO>> labelPage(@RequestBody DoctorPatientLabelQueryDTO params) {
        log.info("/sys/doctor/data/label/query >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setIsAdmin(YN.NO.getCode());  // 初始化，不能前端传
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setIsAdmin(YN.YES.getCode());
        }
        PageData<LabelBO> data = doctorDataQueryService.labelPage(params, user.getDeptId());
        return new Result<PageData<LabelBO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/label/list")
    @ApiOperation(value = "医生标签列表", response = LabelBO.class)
    public Result<List<LabelBO>> labelList(@RequestBody DoctorPatientLabelQueryDTO params) {
        log.info("/sys/doctor/data/label/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setIsAdmin(YN.NO.getCode());  // 初始化，不能前端传
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setIsAdmin(YN.YES.getCode());
        }
        List<LabelBO> data = doctorDataQueryService.labelList(params, user.getDeptId());
        return new Result<List<LabelBO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/label/disease/list")
    @ApiOperation(value = "病种标签列表", response = DiseaseLabelStatisticVO.class)
    public Result<List<DiseaseLabelStatisticVO>> labelStatDimensionTypeList(@RequestBody DoctorPatientLabelQueryDTO params) {
        log.info("/sys/doctor/data/label/disease/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setIsAdmin(YN.NO.getCode());  // 初始化，不能前端传
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setIsAdmin(YN.YES.getCode());
        }
        List<DiseaseLabelStatisticVO> data = doctorDataQueryService.diseaseLabelStatisticList(params, user.getDeptId());
        return new Result<List<DiseaseLabelStatisticVO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/patient/query")
    @ApiOperation(value = "医生患者分页", response = DoctorLabelPatientFullVO.class)
    public Result<PageData<DoctorLabelPatientFullVO>> patientPage(@RequestBody PatientQueryDTO params) {
        log.info("/sys/doctor/data/patient/query >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        if (null == params.getPage()) {
            params.setPage(1);
        }
        if (null == params.getRows()) {
            params.setRows(10);
        }
        PageData<DoctorLabelPatientFullVO> data = doctorDataQueryService.patientPage(params, user.getDeptId());
        return new Result<PageData<DoctorLabelPatientFullVO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/patient/list")
    @ApiOperation(value = "医生患者列表", response = DoctorLabelPatientFullVO.class)
    public Result<List<DoctorLabelPatientFullVO>> patientList(@RequestBody PatientQueryDTO params) {
        log.info("/sys/doctor/data/patient/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<DoctorLabelPatientFullVO> data = doctorDataQueryService.patientList(params, user.getDeptId());
        return new Result<List<DoctorLabelPatientFullVO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/patient/detail")
    @ApiOperation(value = "医生患者详情", response = DoctorLabelPatientFullVO.class)
    public Result<TaskPatientVO> patientDetail(@RequestBody PatientGetDTO params) {
        log.info("/sys/doctor/data/patient/detail >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        TaskPatientVO data = doctorDataQueryService.patientDetail(params, user.getDeptId());
        return new Result<TaskPatientVO>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/label/patient/count")
    @ApiOperation(value = "医生标签患者数量列表", response = LabelPatientStatisticClientBO.class)
    public Result<List<LabelPatientStatisticClientBO>> labelPatientCountList(@RequestBody DoctorPatientLabelQueryDTO params) {
        log.info("/sys/doctor/data/label/patient/count >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setIsAdmin(YN.NO.getCode()); // 初始化，不能前端传
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setIsAdmin(YN.YES.getCode());
        }
        List<LabelPatientStatisticClientBO> data = doctorDataQueryService.labelPatientCountList(params, user.getDeptId());
        return new Result<List<LabelPatientStatisticClientBO>>().ok(data);
    }

    // @SaCheckPermission("sys:doctor:data:queryByDate")
    @PostMapping("/label/disease/patient/count")
    @ApiOperation(value = "医生潜在标签患者数量列表", response = LabelPatientStatDimensionTypeClientBO.class)
    public Result<List<LabelPatientStatDimensionTypeClientBO>> labelStatDimensionTypePatientCountList(@RequestBody DoctorPatientLabelQueryDTO params) {
        log.info("/sys/doctor/data/label/disease/patient/count >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setIsAdmin(YN.NO.getCode()); // 初始化，不能前端传
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setIsAdmin(YN.YES.getCode());
        }
        // params.setIsPotentialType(YN.YES.getCode()); // 只查潜在类型信息
        List<LabelPatientStatDimensionTypeClientBO> data = doctorDataQueryService.statLabelPatientCountList(params, user.getDeptId());
        return new Result<List<LabelPatientStatDimensionTypeClientBO>>().ok(data);
    }

}
