package com.xxh.cloud.healthy.service.com.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorFollowPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorFollowPatientBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/follow/patient")
@Api(tags = "医生关注患者管理")
public class DoctorFollowPatientController {
    
    @Autowired
    private DoctorFollowPatientBizService doctorFollowPatientBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DoctorFollowPatientVO.class)
    public Result<PageData<DoctorFollowPatientVO>> query(@RequestBody DoctorFollowPatientQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorFollowPatientVO> data = doctorFollowPatientBizService.page(params);
        return new Result<PageData<DoctorFollowPatientVO>>().ok(data);
    }

    @PostMapping("/simpleQuery")
    @ApiOperation(value = "简单分页", response = DoctorFollowPatientVO.class)
    public Result<PageData<DoctorFollowPatientVO>> simpleQuery(@RequestBody DoctorFollowPatientQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorFollowPatientVO> data = doctorFollowPatientBizService.simplePage(params);
        return new Result<PageData<DoctorFollowPatientVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorFollowPatientVO.class)
    public Result<DoctorFollowPatientVO> detail(@RequestBody DoctorFollowPatientGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorFollowPatientVO vo = doctorFollowPatientBizService.detail(params);
        return new Result<DoctorFollowPatientVO>().ok(vo);
    }

    @PostMapping("/followOrNot")
    @ApiOperation(value = "关注或取消关注", response = DoctorFollowPatientVO.class)
    public Result<DoctorFollowPatientVO> add(@RequestBody DoctorFollowPatientAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorFollowPatientVO vo = doctorFollowPatientBizService.followOrNot(params);
        return new Result<DoctorFollowPatientVO>().ok(vo);
    }

}
