package com.xxh.cloud.healthy.service.com.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientDetailDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientUpdateDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorManagePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorManagePatientBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/manage/patient")
@Api(tags = "医生管理患者管理")
public class DoctorManagePatientController {
    
    @Autowired
    private DoctorManagePatientBizService doctorManagePatientBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DoctorManagePatientVO.class)
    public Result<PageData<DoctorManagePatientVO>> query(@RequestBody DoctorManagePatientQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorManagePatientVO> data = doctorManagePatientBizService.page(params);
        return new Result<PageData<DoctorManagePatientVO>>().ok(data);
    }

    @PostMapping("/getById")
    @ApiOperation(value = "详情", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> getById(@RequestBody DoctorManagePatientGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorManagePatientVO vo = doctorManagePatientBizService.getById(params);
        return new Result<DoctorManagePatientVO>().ok(vo);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> detail(@RequestBody DoctorManagePatientDetailDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorManagePatientVO vo = doctorManagePatientBizService.detail(params);
        return new Result<DoctorManagePatientVO>().ok(vo);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存/更新", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> add(@RequestBody DoctorManagePatientAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorManagePatientVO vo = doctorManagePatientBizService.save(params);
        return new Result<DoctorManagePatientVO>().ok(vo);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> update(@RequestBody DoctorManagePatientUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorManagePatientBizService.update(params);
        return new Result<DoctorManagePatientVO>().success();
    }

    @PostMapping("/del")
    @ApiOperation(value = "删除", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> del(@RequestBody DoctorManagePatientGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorManagePatientBizService.del(params);
        return new Result<DoctorManagePatientVO>().success();
    }


    // 管理端查询

    @PostMapping("/manage/query")
    @ApiOperation(value = "管理分页", response = DoctorManagePatientVO.class)
    public Result<PageData<DoctorManagePatientVO>> manageQuery(@RequestBody DoctorManagePatientQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DoctorManagePatientVO> data = doctorManagePatientBizService.page(params);
        return new Result<PageData<DoctorManagePatientVO>>().ok(data);
    }

    @PostMapping("/manage/getById")
    @ApiOperation(value = "管理详情", response = DoctorManagePatientVO.class)
    public Result<DoctorManagePatientVO> manageQuery(@RequestBody DoctorManagePatientGetDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorManagePatientVO vo = doctorManagePatientBizService.getById(params);
        return new Result<DoctorManagePatientVO>().ok(vo);
    }

}
