package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorLabelPatientDataQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorLabelPatientExcludDataQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientExcludeStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorLabelPatientDataBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/manage/patient/data")
@Api(tags = "医生管理患者数据")
public class DoctorManagePatientDataController {
    
    @Autowired
    private DoctorLabelPatientDataBizService doctorLabelPatientDataBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DoctorLabelPatientDataVO.class)
    public Result<PageData<DoctorLabelPatientDataVO>> query(@RequestBody DoctorLabelPatientDataQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<DoctorLabelPatientDataVO> data = doctorLabelPatientDataBizService.query(params);
        return new Result<PageData<DoctorLabelPatientDataVO>>().ok(data);
    }

    @PostMapping("/queryStatistic")
    @ApiOperation(value = "按场景分页", response = DoctorLabelPatientStatisticVO.class)
    public Result<PageData<DoctorLabelPatientStatisticVO>> queryStatisticBySecene(@RequestBody DoctorLabelPatientDataQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<DoctorLabelPatientStatisticVO> data = doctorLabelPatientDataBizService.queryStatisticBySecene(params);
        return new Result<PageData<DoctorLabelPatientStatisticVO>>().ok(data);
    }

    @PostMapping("/queryStatisticByMergeSecene")
    @ApiOperation(value = "合并场景分页", response = DoctorLabelPatientStatisticVO.class)
    public Result<PageData<DoctorLabelPatientStatisticVO>> queryStatisticByNoSecene(@RequestBody DoctorLabelPatientDataQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<DoctorLabelPatientStatisticVO> data = doctorLabelPatientDataBizService.queryStatisticByNoSecene(params);
        return new Result<PageData<DoctorLabelPatientStatisticVO>>().ok(data);
    }

    @PostMapping("/excludeStatisticReason")
    @ApiOperation(value = "排除原因统计", response = DoctorLabelPatientExcludeStatisticVO.class)
    public Result<List<DoctorLabelPatientExcludeStatisticVO>> excludeStatisticReason(@RequestBody DoctorLabelPatientExcludDataQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        List<DoctorLabelPatientExcludeStatisticVO> data = doctorLabelPatientDataBizService.excludeStatisticReason(params);
        return new Result<List<DoctorLabelPatientExcludeStatisticVO>>().ok(data);
    }

}
