package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdLaboratoryQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorDwdPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientPushServeBehaviorQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientPushServeQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdLaboratoryReportItemVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientPushServeBehaviorVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PushServeDataQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/dept/patient")
@Api(tags = "部门（科室）患者")
public class DoctorPatientController {
    
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private PushServeDataQueryBizService pushServeDataQueryBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    protected RedisUtils redisUtils;

    @Value("${temporary.cache.expire:7200}")
    private Integer expireTime;
    
    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    @PostMapping("/query")
    @ApiOperation(value = "查询患者信息", response = TaskPatientVO.class)
    public Result<PageData<TaskPatientVO>> query(@RequestBody TaskPatientQueryDTO params) {
        log.info("/sys/healthy/dept/patient/query >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PageData<TaskPatientVO> data = taskPatientQueryBizService.query(params);
        return new Result<PageData<TaskPatientVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:dept:detail")
    @PostMapping("/detail")
    @ApiOperation(value = "查询患者详情", response = TaskPatientVO.class)
    public Result<TaskPatientVO> detail(@RequestBody TaskPatientGetDTO params) {
        log.info("/sys/healthy/dept/patient/detail >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        TaskPatientVO data = taskPatientQueryBizService.getById(params);
        return new Result<TaskPatientVO>().ok(data);
    }

    // 病种运营服务详情下单服务对象 // 将要弃用,替换：/sys/healthy/dept/pushserve/remindUser
    @Deprecated
    @PostMapping("/queryPatient")
    @ApiOperation(value = "查询某服务下患者数据", response = PushServePatientVO.class)
    public Result<PageData<PushServePatientVO>> queryPatient(@RequestBody PatientPushServeQueryDTO params) {
        log.info("/sys/healthy/dept/patient/queryPatient >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PageData<PushServePatientVO> data = pushServeDataQueryBizService.queryPatient(params);
        return new Result<PageData<PushServePatientVO>>().ok(data);
    }
    
    // 病种运营服务详情下的用户行为分析统计
    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    @PostMapping("/queryPatientPushServeBehavior")
    @ApiOperation(value = "服务用户行为分析", response = PatientPushServeBehaviorVO.class)
    public Result<List<PatientPushServeBehaviorVO>> queryPatientPushServeBehavior(@RequestBody PatientPushServeBehaviorQueryDTO params) {
        log.info("/sys/healthy/dept/patient/queryPatientPushServeBehavior >>> {}", JSON.toJSONString(params));
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDeptId(user.getDeptId());

        // TODO 注释默认查询时间
        // DateTime now = DateUtil.date();
        // if (StrUtil.isBlank(params.getStartDate())) {
        //     params.setStartDate(DateUtil.offsetDay(now, -7).toDateStr());
        // }
        // if (StrUtil.isBlank(params.getEndDate())) {
        //     params.setEndDate(now.toDateStr());
        // }

        String key = StrUtil.format("statistic:queryPatientPushServeBehavior:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<List<PatientPushServeBehaviorVO>>().ok(JSON.parseArray(cache.toString(), PatientPushServeBehaviorVO.class));
        }
        
        List<PatientPushServeBehaviorVO> data = pushServeDataQueryBizService.queryPatientPushServeBehavior(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<List<PatientPushServeBehaviorVO>>().ok(data);
    }
    
    // 患者检验趋势
    @PostMapping("/dwd/laboratoryTrend")
    @ApiOperation(value = "患者检验趋势", response = DataDwdLaboratoryReportItemVO.class)
    public Result<Map<String, List<DataDwdLaboratoryReportItemVO>>> laboratoryTrend(@RequestBody DataDwdLaboratoryQueryDTO params) {
        log.info("/sys/healthy/dept/patient/laboratoryTrend >>> {}", JSON.toJSONString(params));
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDeptId(user.getDeptId());
        
        // 默认30天的检验
        // if (null == params.getStartDate()) {
        //     params.setStartDate(DateUtil.offsetDay(DateUtil.date(), 30));
        // }

        Map<String, List<DataDwdLaboratoryReportItemVO>> data = doctorDwdPatientDataBizService.laboratoryTrend(params);
        return new Result<Map<String, List<DataDwdLaboratoryReportItemVO>>>().ok(data);
    }

    // dwd数仓
    // 数仓查询患者信息
    @PostMapping("/dwd/list")
    @ApiOperation(value = "dwd查询患者信息列表", response = TaskPatientVO.class)
    public Result<List<DoctorDwdPatientVO>> dwdList(@RequestBody DoctorDwdPatientQueryDTO params) {
        log.info("/sys/healthy/dept/patient/dwdList >>> {}", JSON.toJSONString(params));
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDeptId(user.getDeptId());
        List<DoctorDwdPatientVO> data = doctorDwdPatientDataBizService.queryPatientList(params);
        return new Result<List<DoctorDwdPatientVO>>().ok(data);
    }
    @PostMapping("/dwd/detail")
    @ApiOperation(value = "dwd查询患者信息", response = TaskPatientVO.class)
    public Result<DoctorDwdPatientVO> dwdDetail(@RequestBody DoctorDwdPatientGetDTO params) {
        log.info("/sys/healthy/dept/patient/dwdDetail >>> {}", JSON.toJSONString(params));
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDeptId(user.getDeptId());
        DoctorDwdPatientVO data = doctorDwdPatientDataBizService.getOnePatient(params);
        return new Result<DoctorDwdPatientVO>().ok(data);
    }

}
