package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientDataQueryDTO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.basic.common.constant.PushServeTypeEnum;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientPushServeQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PushServiceStatisticGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PushServiceStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientArchivesVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientNewestDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeTargetVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientArchivesQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PushServeDataQueryBizService;
import com.xxh.cloud.healthy.mdt.api.ConsultationClientService;
import com.xxh.cloud.healthy.mdt.api.bean.bo.ConsultationBO;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/dept/pushserve")
@Api(tags = "部门（科室）推送服务")
public class DoctorPushServiceController {
    
    @Autowired
    private PushServeDataQueryBizService pushServeDataQueryBizService;
    @Autowired
    private PatientArchivesQueryBizService patientArchivesQueryBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private ConsultationClientService consultationClientService;
    @Autowired
    protected RedisUtils redisUtils;

    @Value("${temporary.cache.expire:7200}")
    private Integer expireTime;
    
    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    // @PostMapping("/preQuery")
    // @ApiOperation(value = "统计患者诊前推送服务", response = PushServeDataOneDayVO.class)
    // public Result<List<PushServeDataOneDayVO>> preQuery(@RequestBody PushServiceStatisticQueryDTO params) {
    //     log.info("/sys/healthy/dept/pushserve/preQuery >>> {}", JSON.toJSONString(params));
    //     LoginUser user= AuthContextUtils.getCurrentLoginUser();
    //     params.setDeptId(user.getDeptId());
    //     params.setOneType(PushServeTypeEnum.Pre.getCode());
    //     List<PushServeDataOneDayVO> data = pushServeDataQueryBizService.query(params);
    //     return new Result<List<PushServeDataOneDayVO>>().ok(data);
    // }
    @PostMapping("/preQuery")
    @ApiOperation(value = "统计患者诊前推送服务", response = PushServeDataVO.class)
    public Result<List<PushServeDataVO>> preQuery(@RequestBody PushServiceStatisticQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/preQuery >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setOneType(PushServeTypeEnum.Pre.getCode());
        
        String key = StrUtil.format("statistic:preQuery:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<List<PushServeDataVO>>().ok(JSON.parseArray(cache.toString(), PushServeDataVO.class));
        }
        
        List<PushServeDataVO> data = pushServeDataQueryBizService.pushServeDataQuery(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<List<PushServeDataVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    // @PostMapping("/duringQuery")
    // @ApiOperation(value = "统计患者诊中推送服务", response = PushServeDataOneDayVO.class)
    // public Result<List<PushServeDataOneDayVO>> duringQuery(@RequestBody PushServiceStatisticQueryDTO params) {
    //     log.info("/sys/healthy/dept/pushserve/duringQuery >>> {}", JSON.toJSONString(params));
    //     LoginUser user= AuthContextUtils.getCurrentLoginUser();
    //     params.setDeptId(user.getDeptId());
    //     params.setOneType(PushServeTypeEnum.During.getCode());
    //     List<PushServeDataOneDayVO> data = pushServeDataQueryBizService.query(params);
    //     return new Result<List<PushServeDataOneDayVO>>().ok(data);
    // }
    @PostMapping("/duringQuery")
    @ApiOperation(value = "统计患者诊中推送服务", response = PushServeDataVO.class)
    public Result<List<PushServeDataVO>> duringQuery(@RequestBody PushServiceStatisticQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/duringQuery >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setOneType(PushServeTypeEnum.During.getCode());
        
        String key = StrUtil.format("statistic:duringQuery:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<List<PushServeDataVO>>().ok(JSON.parseArray(cache.toString(), PushServeDataVO.class));
        }
        
        List<PushServeDataVO> data = pushServeDataQueryBizService.pushServeDataQuery(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<List<PushServeDataVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    // @PostMapping("/afterQuery")
    // @ApiOperation(value = "统计患者诊后推送服务", response = PushServeDataOneDayVO.class)
    // public Result<List<PushServeDataOneDayVO>> afterQuery(@RequestBody PushServiceStatisticQueryDTO params) {
    //     log.info("/sys/healthy/dept/pushserve/afterQuery >>> {}", JSON.toJSONString(params));
    //     LoginUser user= AuthContextUtils.getCurrentLoginUser();
    //     params.setDeptId(user.getDeptId());
    //     params.setOneType(PushServeTypeEnum.After.getCode());
    //     List<PushServeDataOneDayVO> data = pushServeDataQueryBizService.query(params);
    //     return new Result<List<PushServeDataOneDayVO>>().ok(data);
    // }
    @PostMapping("/afterQuery")
    @ApiOperation(value = "统计患者诊后推送服务", response = PushServeDataVO.class)
    public Result<List<PushServeDataVO>> afterQuery(@RequestBody PushServiceStatisticQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/afterQuery >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setOneType(PushServeTypeEnum.After.getCode());
        
        String key = StrUtil.format("statistic:afterQuery:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<List<PushServeDataVO>>().ok(JSON.parseArray(cache.toString(), PushServeDataVO.class));
        }
        
        List<PushServeDataVO> data = pushServeDataQueryBizService.pushServeDataQuery(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<List<PushServeDataVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:dept:queryByDate")
    // @PostMapping("/detail")
    // @ApiOperation(value = "统计患者推送服务详情", response = PushServeDataOneDayVO.class)
    // public Result<PushServeDataOneDayVO> detail(@RequestBody PushServiceStatisticGetDTO params) {
    //     log.info("/sys/healthy/dept/pushserve/detail >>> {}", JSON.toJSONString(params));
    //     LoginUser user= AuthContextUtils.getCurrentLoginUser();
    //     params.setDeptId(user.getDeptId());
    //     PushServeDataOneDayVO data = pushServeDataQueryBizService.detail(params);
    //     return new Result<PushServeDataOneDayVO>().ok(data);
    // }
    @PostMapping("/detail")
    @ApiOperation(value = "统计患者推送服务详情", response = PushServeDataVO.class)
    public Result<PushServeDataVO> detail(@RequestBody PushServiceStatisticGetDTO params) {
        log.info("/sys/healthy/dept/pushserve/detail >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        
        String key = StrUtil.format("statistic:detail:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PushServeDataVO>().ok(JSON.parseObject(cache.toString(), PushServeDataVO.class));
        }
        
        PushServeDataVO data = pushServeDataQueryBizService.pushServeDataDetail(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PushServeDataVO>().ok(data);
    }

    // 病种运营服务详情下服务对象
    @PostMapping("/remindUser")
    @ApiOperation(value = "查询服务对象（提醒对象）", response = PushServeTargetVO.class)
    public Result<PageData<PushServeTargetVO>> remindUser(@RequestBody PatientPushServeQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/remindUser >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PageData<PushServeTargetVO> data = pushServeDataQueryBizService.remindUser(params);
        return new Result<PageData<PushServeTargetVO>>().ok(data);
    }

    // 查询用户档案及服务
    @PostMapping("/archives")
    @ApiOperation(value = "查询用户档案及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesData(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/archives >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        // PatientArchivesVO data = patientArchivesQueryBizService.patientArchives(params);
        PatientArchivesVO data = patientArchivesQueryBizService.patientArchivesV2(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    // 查询用户档案+体检及服务
    @PostMapping("/archives/v2")
    @ApiOperation(value = "查询用户档案+体检及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesDataV2(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/archives/v2 >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PatientArchivesVO data = patientArchivesQueryBizService.patientArchivesV2(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    // 异步查询用户档案+体检及服务
    @PostMapping("/archives/v3")
    @ApiOperation(value = "异步查询用户档案+体检及服务", response = PushServeTargetVO.class)
    public Result<PatientArchivesVO> archivesDataV3(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/sys/healthy/dept/pushserve/archives/v3 >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PatientArchivesVO data = patientArchivesQueryBizService.patientArchivesV3(params);
        return new Result<PatientArchivesVO>().ok(data);
    }

    // 患者最新就诊/住院/出院/会诊记录
    @PostMapping("/newest/visit")
    @ApiOperation(value = "患者最新就诊/住院/出院/会诊信息", response = PatientNewestDataVO.class)
    public Result<PatientNewestDataVO> newestVisit(@RequestBody DwdPatientDataQueryDTO params) {
        log.info("/app/healthy/data/patient/newest/visit >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        if (StrUtil.isBlank(params.getPatientId())) {
            return new Result<PatientNewestDataVO>().success();
        }

        PatientNewestDataVO vo = new PatientNewestDataVO();
        vo.setPatientVisitRecordInfo(doctorDwdPatientDataBizService.getOnePatientVisitLastDateInfo(params.getPatientId()));
        vo.setPatientInpatientRecordInfo(doctorDwdPatientDataBizService.getOnePatientInpatientLastDateInfo(params.getPatientId()));

        PatientDischargeRecordClientBO patientDischargeRecordClientBO = doctorDwdPatientDataBizService.getOnePatientOutpatientLastDateInfo(params.getPatientId());
        vo.setPatientDischargeRecordInfo(patientDischargeRecordClientBO);

        Map<String, ConsultationBO> consultationBO = consultationClientService.lastOrdinarySimpleMap(params.getPatientIds());
        vo.setConsultationInfo(consultationBO.get(params.getPatientId()));

        return new Result<PatientNewestDataVO>().ok(vo);
    }

}
