package com.xxh.cloud.healthy.service.com.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientDiseaseStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticUserQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientAfterStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDiseaseStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDuringStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientPreStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeUserVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiseaseDataQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/data")
@Api(tags = "医生看板数据统计")
public class PatientDataStatisticController {

    @Autowired
    private PatientDataQueryBizService patientDataQueryBizService;
    @Autowired
    private PatientDiseaseDataQueryBizService patientDiseaseDataQueryBizService;
    @Autowired
    protected RedisUtils redisUtils;

    @Value("${temporary.cache.expire:7200}")
    private Integer expireTime;
    
    // 看板统计
    // 所有统计
    @PostMapping("/statistic")
    @ApiOperation(value = "统计查询", response = PatientStatisticVO.class)
    public Result<PatientStatisticVO> query(@RequestBody PatientStatisticQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        
        String key = StrUtil.format("statistic:-:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PatientStatisticVO>().ok(JSON.parseObject(cache.toString(), PatientStatisticVO.class));
        }
        
        PatientStatisticVO data = patientDataQueryBizService.patientStatistic(params);
        log.info("/sys/healthy/doctor/data/statistic <<<<<< {}", JSON.toJSONString(data));
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PatientStatisticVO>().ok(data);
    }


    @PostMapping("/statistic/disease")
    @ApiOperation(value = "按病种统计查询", response = PatientDiseaseStatisticVO.class)
    public Result<PatientDiseaseStatisticVO> queryByDiseaseId(@RequestBody PatientDiseaseStatisticQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        
        String key = StrUtil.format("statistic:disease:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PatientDiseaseStatisticVO>().ok(JSON.parseObject(cache.toString(), PatientDiseaseStatisticVO.class));
        }

        PatientDiseaseStatisticVO data = patientDiseaseDataQueryBizService.patientStatistic(params);
        log.info("/sys/healthy/doctor/data/statistic/disease:data <<<<<< {}", JSON.toJSONString(data));
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PatientDiseaseStatisticVO>().ok(data);
    }



    ////// 实时统计

    // 看板/病种运营统计
    // 诊前统计
    @PostMapping("/statistic/pre")
    @ApiOperation(value = "看板诊前统计", response = PatientPreStatisticVO.class)
    public Result<PatientPreStatisticVO> queryPre(@RequestBody PatientStatisticQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        //todo:
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());

        String key = StrUtil.format("statistic:pre:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PatientPreStatisticVO>().ok(JSON.parseObject(cache.toString(), PatientPreStatisticVO.class));
        }

        PatientPreStatisticVO data = patientDataQueryBizService.patientPreStatisticByDeptIdV2(params);

        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PatientPreStatisticVO>().ok(data);
    }
    // 诊中统计
    @PostMapping("/statistic/during")
    @ApiOperation(value = "看板诊中统计", response = PatientDuringStatisticVO.class)
    public Result<PatientDuringStatisticVO> queryDuring(@RequestBody PatientStatisticQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        
        String key = StrUtil.format("statistic:during:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PatientDuringStatisticVO>().ok(JSON.parseObject(cache.toString(), PatientDuringStatisticVO.class));
        }

        PatientDuringStatisticVO data = patientDataQueryBizService.patientDuringStatisticByDeptIdV2(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PatientDuringStatisticVO>().ok(data);
    }
    // 诊后统计
    @PostMapping("/statistic/after")
    @ApiOperation(value = "看板诊后统计", response = PatientAfterStatisticVO.class)
    public Result<PatientAfterStatisticVO> queryAfter(@RequestBody PatientStatisticQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        
        String key = StrUtil.format("statistic:after:{}", params.md5());
        // 缓存
        Object cache = redisUtils.get(key);
        if (null != cache) {
            return new Result<PatientAfterStatisticVO>().ok(JSON.parseObject(cache.toString(), PatientAfterStatisticVO.class));
        }

        PatientAfterStatisticVO data = patientDataQueryBizService.patientAfterStatisticByDeptIdV2(params);
        
        redisUtils.set(key, JSON.toJSONString(data), expireTime);

        return new Result<PatientAfterStatisticVO>().ok(data);
    }

    // 统计数据对应的用户列表
    @PostMapping("/statistic/patient")
    @ApiOperation(value = "统计查询患者列表", response = TaskPatientVO.class)
    public Result<PageData<TaskPatientVO>> queryPatient(@RequestBody PatientStatisticPatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        //todo:
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PageData<TaskPatientVO> data = patientDataQueryBizService.patientLabelStatisticPatientList(params);
        return new Result<PageData<TaskPatientVO>>().ok(data);
    }

    // 病种运营下未关注/发送失败用户列表
    @PostMapping("/statistic/user")
    @ApiOperation(value = "统计查询用户列表", response = PushServeUserVO.class)
    public Result<PageData<PushServeUserVO>> queryUser(@RequestBody PatientStatisticUserQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        PageData<PushServeUserVO> data = patientDataQueryBizService.patientLabelStatisticUserList(params);
        return new Result<PageData<PushServeUserVO>>().ok(data);
    }

}
