package com.xxh.cloud.healthy.service.com.manager.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.PatientFeeRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.*;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiseaseDataQueryBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/fee/data")
@Api(tags = "患者收费记录")
public class PatientFeeDataController {

    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;

    @PostMapping("/query")
    @ApiOperation(value = "查询分页", response = PatientFeeRecordBO.class)
    public Result<PageData<PatientFeeRecordBO>> queryUser(@RequestBody PatientFeeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<PatientFeeRecordBO> data = doctorDwdPatientDataBizService.pagePatientFeeRecord(params);
        return new Result<PageData<PatientFeeRecordBO>>().ok(data);
    }

    @GetMapping("/export")
    @ApiOperation(value = "导出", response = void.class)
    public void applyPageExport(HttpServletResponse response, PatientFeeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<PatientFeeRecordBO> list = doctorDwdPatientDataBizService.listPatientFeeRecord(params);

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("patientId", "患者编号");
        writer.addHeaderAlias("patientName", "患者姓名");
        writer.addHeaderAlias("feeCodeStandard", "费用编号");
        writer.addHeaderAlias("feeNameStandard", "费用名称");
        writer.addHeaderAlias("itemCodeStandard", "收费项目编号");
        writer.addHeaderAlias("itemNameStandard", "收费项目名称");
        writer.addHeaderAlias("totalFee", "收费总金额");
        writer.addHeaderAlias("ownFee", "自费金额");
        writer.addHeaderAlias("ownPayFee", "自付金额");
        writer.addHeaderAlias("centerPayFee", "统筹金额");
        writer.addHeaderAlias("registerSn", "挂号流水号");
        writer.addHeaderAlias("departmentCodeStandard", "挂号科室编码");
        writer.addHeaderAlias("departmentNameStandard", "挂号科室名称");
        writer.addHeaderAlias("doctorCode", "医生编号");
        writer.addHeaderAlias("doctorName", "医生名称");
        writer.addHeaderAlias("regLevelCodeRaw", "挂号级别编号");
        writer.addHeaderAlias("regLevelNameRaw", "挂号级别名称");
        writer.addHeaderAlias("registerDate", "挂号时间");

        writer.write(list, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        response.setHeader("Content-Disposition","attachment;filename=PatientFee.xlsx");
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }

    }
}
