package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossRemarkDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossUpdateDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientLossVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientLossDataBizService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorRelationClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.bo.DoctorRelationBO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorRelationClientQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/patient/loss")
@Api(tags = "流失患者管理")
public class PatientLossController {
    
    @Autowired
    private PatientLossDataBizService patientLossDataBizService;
    @Autowired
    private DoctorRelationClientService doctorRelationClientService;

    @PostMapping("/query")
    @ApiOperation(value = "流失分页", response = PatientLossVO.class)
    public Result<PageData<PatientLossVO>> query(@RequestBody PatientLossQueryDTO params) {
        log.info("/sys/healthy/patient/loss/query >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        DoctorRelationClientQueryDTO relation = new DoctorRelationClientQueryDTO();
        relation.setDoctorId(user.getId());
        relation.setType(DoctorRelationConstant.Type.DEPT.getCode());
        List<DoctorRelationBO> deptList = doctorRelationClientService.list(relation);
        if (CollUtil.isEmpty(deptList)) {
            return new Result<PageData<PatientLossVO>>().success();
        }
        params.setDeptIds(deptList.stream().map(DoctorRelationBO::getRelationId).collect(Collectors.toList()));
        
        PageData<PatientLossVO> data = patientLossDataBizService.query(params);
        return new Result<PageData<PatientLossVO>>().ok(data);
    }
    
    @PostMapping("/list")
    @ApiOperation(value = "流失列表", response = PatientLossVO.class)
    public Result<List<PatientLossVO>> list(@RequestBody PatientLossQueryDTO params) {
        log.info("/sys/healthy/patient/loss/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        
        DoctorRelationClientQueryDTO relation = new DoctorRelationClientQueryDTO();
        relation.setDoctorId(user.getId());
        relation.setType(DoctorRelationConstant.Type.DEPT.getCode());
        List<DoctorRelationBO> deptList = doctorRelationClientService.list(relation);
        if (CollUtil.isEmpty(deptList)) {
            return new Result<List<PatientLossVO>>().success();
        }
        params.setDeptIds(deptList.stream().map(DoctorRelationBO::getRelationId).collect(Collectors.toList()));
        
        List<PatientLossVO> data = patientLossDataBizService.list(params);
        return new Result<List<PatientLossVO>>().ok(data);
    }

    @PostMapping("/manage/query")
    @ApiOperation(value = "流失分页(manage)", response = PatientLossVO.class)
    public Result<PageData<PatientLossVO>> manageQuery(@RequestBody PatientLossQueryDTO params) {
        log.info("/sys/healthy/patient/loss/query >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        PageData<PatientLossVO> data = patientLossDataBizService.query(params);
        return new Result<PageData<PatientLossVO>>().ok(data);
    }
    
    @PostMapping("/manage/list")
    @ApiOperation(value = "流失列表(manage)", response = PatientLossVO.class)
    public Result<List<PatientLossVO>> manageList(@RequestBody PatientLossQueryDTO params) {
        log.info("/sys/healthy/patient/loss/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        
        List<PatientLossVO> data = patientLossDataBizService.list(params);
        return new Result<List<PatientLossVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "流失详情", response = PatientLossVO.class)
    public Result<PatientLossVO> detail(@RequestBody PatientLossGetDTO params) {
        log.info("/sys/healthy/patient/loss/detail >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        
        PatientLossVO data = patientLossDataBizService.getById(params);
        return new Result<PatientLossVO>().ok(data);
    }

    @PostMapping("/remark")
    @ApiOperation(value = "备注", response = Void.class)
    public Result<Void> remark(@RequestBody PatientLossRemarkDTO params) {
        log.info("/sys/healthy/patient/loss/remark >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        
        patientLossDataBizService.remark(params);
        return new Result<Void>().success();
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody PatientLossUpdateDTO params) {
        log.info("/sys/healthy/patient/loss/update >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        
        patientLossDataBizService.update(params);
        return new Result<Void>().success();
    }

}
