package com.xxh.cloud.healthy.service.com.manager.controller;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PulmonaryNoduleMapper;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;

/**
 * 临时肺结节大小数据清洗接口
 */
@SaCheckLogin
@RestController
@RequestMapping("/demo/pnc")
public class PulmonaryNoduleController {

    @Autowired
    private PulmonaryNoduleMapper pulmonaryNoduleMapper;
    
    @PostMapping("/deal")
    public Result<Void> queryPre() {

        int pages = 1;
        while (true) {
            IPage<Map<String, String>> page = pulmonaryNoduleMapper.page(new Page<Void>(pages, 500), new QueryWrapper<Void>()
                .isNull("result")
                .orderByAsc("inspect_code")
            );

            if (CollUtil.isEmpty(page.getRecords())) {
                break;
            }

            List<Map<String, String>> updateList = new ArrayList<>();
            page.getRecords().forEach(item -> {
                Map<String, String> map = new HashMap<>();
                map.put("inspectCode", item.get("inspectCode"));

                List<String> patterList = this.listPatternStr(item.get("inspect"), "[0-9]+\\.?[0-9]*(C|M)M");
            
                List<String> resultList = new ArrayList<>();
                patterList.forEach(i -> {
                    BigDecimal bd = new BigDecimal(i.substring(0, i.length() - 2));
                    if (i.endsWith("CM")) {
                        resultList.add(bd.multiply(new BigDecimal("10")).toString());
                    } else {
                        resultList.add(bd.toString());
                    }
                });
                
                if (CollUtil.isNotEmpty(resultList)) {
                    String result = StrJoinSplitUtils.join(resultList);
                    map.put("result", result);

                    resultList.forEach(i -> {
                        if (null == map.get("resultMax") || new BigDecimal(map.get("resultMax")).compareTo(new BigDecimal(i)) < 0) {
                            map.put("resultMax", i);
                        }
                    });

                    updateList.add(map);
                }
            });

            if (CollUtil.isNotEmpty(updateList)) {
                updateList.forEach(i -> {
                    pulmonaryNoduleMapper.update(new UpdateWrapper<Void>()
                        .set("result", i.get("result"))
                        .set("result_max", i.get("resultMax"))
                        .eq("inspect_code", i.get("inspectCode"))
                    );
                });
            }

            pages = pages + 1;
        }

        return new Result<Void>().success();
    }

    private List<String> listPatternStr(String str, String regex) {
        List<String> list = new ArrayList<String>();
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(str);
        while(m.find()) {
            list.add(m.group());
        }
        return list;
    }

}
