package com.xxh.cloud.healthy.data.framework.modules.biz.mapper;

import java.util.List;
import java.util.Map;

import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDiagnosisReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeContentClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientExamineReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientChangeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientReserveClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientLaboratoryReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineItemClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInReserveBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.PatientFeeRecordBO;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdLaboratoryQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdLaboratoryReportItemVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientRegisterVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwsPatientDataLastDateVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdInpatientReserveVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdOperationApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdRegisterRecordVO;

/**
 * 20231030
 */
public interface DwdPatientDataBizMapper {

    /**
     * 患者信息
     */
    @Select(" SELECT patient_id AS patientId, main_person_id AS mainPersonId, patient_name AS patientName, card_no_type_standard AS cardNoTypeStandard, " + 
            "   card_no AS cardNo, patient_gender_standard AS patientGenderStandard, birthday, phone " + 
            " FROM dwd_patient_info " + 
            " ${ew.customSqlSegment} ")
    List<DataDwdPatientVO> patientList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者挂号信息
     */
    @Select(" SELECT a.id, b.patient_id patientId, a.department_code_standard, a.reg_date " + 
            " FROM dwd_register_record_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<DataDwdPatientRegisterVO> patientRegisterList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者就诊信息
     */
    @Select(" SELECT a.id, b.patient_id patientId, a.department_code_standard, a.visit_date " + 
            " FROM dwd_visit_record_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<PatientVisitRecordClientBO> patientVisitList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者住院信息
     */
    @Select(" SELECT a.id, b.patient_id patientId, a.admit_department_code_standard, a.inpatient_date " + 
            " FROM dwd_inpatient_record_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<DataDwdPatientInHospitalVO> patientInHospitalList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者住院信息
     */
    @Select(" SELECT a.id, b.patient_id patientId, a.admit_department_code_standard, a.inpatient_date " + 
            " FROM dwd_inpatient_record_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<PatientInpatientRecordClientBO> patientInHospitalClientList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者住院申请信息
     */
    @Select(" SELECT a.id, b.patient_id patientId, a.apply_department_code_standard " + 
            " FROM dwd_inpatient_reserve_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<DataDwdPatientInHospitalApplyVO> patientInHospitalApplyList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者转科信息
     */
    @Select(" SELECT a.change_id, b.patient_id, a.change_type, a.change_type_name, a.code, a.name, a.change_time " + 
            " FROM dwd_inpatient_change_info a " + 
            " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
            " ${ew.customSqlSegment} ")
    List<PatientInpatientChangeClientBO> patientInHospitalChangeList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 患者住院信息
     * todo: wjj
     */
//    @Deprecated
//    @Select(" SELECT patient_id, admit_department_code_standard, id, inpatient_date " +
//            " FROM dwd_inpatient_record_info " +
//            " ${ew.customSqlSegment} ")
//    List<DataDwdPatientInHospitalVO> patientInHospitalList(@Param(Constants.WRAPPER) Wrapper<DataDwdPatientQueryDTO> wrapper);

    /**
     * 查询患者信息列表
     * 20240311 修改查询数仓的患者表
     */
    @Select(" SELECT a.patient_id patientId, a.main_person_id mainPersonId, a.patient_name patientName, a.patient_gender_standard patientGenderStandard, " +
            " a.birthday, a.phone, a.card_no_type_standard cardNoTypeStandard, a.card_no cardNo, a.death_status deathStatus, a.death_time deathTime " + 
            " FROM dwd_patient_info a " + 
            " ${ew.customSqlSegment} ")
    List<DoctorDwdPatientVO> queryPatientList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdPatientVO> wrapper);

    /**
     * 查询患者信息
     * 20240311 修改查询数仓的患者表
     */
    @Select(" SELECT a.patient_id patientId, a.main_person_id mainPersonId, a.patient_name patientName, a.patient_gender_standard patientGenderStandard, " + 
            " a.birthday, a.phone, a.card_no_type_standard cardNoTypeStandard, a.card_no cardNo, a.death_status deathStatus, a.death_time deathTime " + 
            " FROM dwd_patient_info a " + 
            " ${ew.customSqlSegment} ")
    DoctorDwdPatientVO getOnePatient(@Param(Constants.WRAPPER) Wrapper<DoctorDwdPatientVO> wrapper);

    /**
     * 挂号患者  TODO 是否需要 LEFT JOIN dwd_visit_record_info
     */
    // @Select(" SELECT a.patient_id, a.department_code_standard, a.department_name_standard, a.reg_date, a.visit_status_standard " + 
    //         " FROM dwd_register_record_info a " + 
    //         " WHERE (a.patient_id, a.department_code_standard, a.reg_date) IN  ( " + 
    //         "   SELECT b.patient_id, b.department_code_standard, MAX(b.reg_date) reg_date  " + 
    //         "   FROM dwd_register_record_info b " + 
    //         "   ${ew.customSqlSegment} " + 
    //         "   GROUP BY b.patient_id, b.department_code_standard " + 
    //         " ) ")
    // List<DoctorDwdRegisterRecordVO> queryRegisterList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);
    // 查询范围内最新的数据
    @Select(" SELECT a.patient_id, a.department_code_standard, MAX(a.reg_date) reg_date  " + 
            "   FROM dwd_register_record_info a " + 
            "   ${ew.customSqlSegment} " + 
            "   GROUP BY a.patient_id, a.department_code_standard ")
    List<DoctorDwdRegisterRecordVO> queryLastRegisterList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);
//     @Select(" SELECT b.main_person_id patient_id, a.department_code_standard, MAX(a.reg_date) reg_date  " + 
//             "   FROM dwd_register_record_info a " + 
//             "   INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             "   ${ew.customSqlSegment} " + 
//             "   GROUP BY b.main_person_id, a.department_code_standard ")
//     List<DoctorDwdRegisterRecordVO> queryLastRegisterList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);

    @Select(" SELECT a.patient_id, a.department_code_standard, a.department_name_standard, a.reg_date, a.visit_status_standard " + 
            " FROM dwd_register_record_info a " + 
            " ${ew.customSqlSegment} ")
    List<DoctorDwdRegisterRecordVO> queryRegisterList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);
//     @Select(" SELECT b.main_person_id patient_id, a.department_code_standard, a.department_name_standard, a.reg_date, a.visit_status_standard " + 
//             " FROM dwd_register_record_info a " + 
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} ")
//     List<DoctorDwdRegisterRecordVO> queryRegisterList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);


    /**
     * wjj:
     * */
    @Select("SELECT a.patient_id as patientId, a.id as id, a.apply_department_code_standard as deptCode, a.apply_date as applyDate " +
            " FROM dwd_inpatient_reserve_info a " + 
            " ${ew.customSqlSegment} ")
    public List<DoctorDwdInReserveBO> queryInApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInReserveBO> wrapper);
//     @Select("SELECT b.main_person_id as patientId, a.id as id, a.apply_department_code_standard as deptCode, a.apply_date as applyDate " +
//             " FROM dwd_inpatient_reserve_info a " + 
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} ")
//     public List<DoctorDwdInReserveBO> queryInApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInReserveBO> wrapper);


    /**
     *
     * */
    @Select(" SELECT a.patient_id as patientId, a.id as id, a.visit_id as visitId, a.admit_department_code_standard as deptCode, a.inpatient_date as inDate" +
            "  FROM dwd_inpatient_record_info a " +
            "  ${ew.customSqlSegment}")
    public List<DoctorDwdInRecordBO>  queryInRecordList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInRecordBO> wrapper);
//     @Select(" SELECT b.main_person_id as patientId, a.id as id, a.visit_id as visitId, a.admit_department_code_standard as deptCode, a.inpatient_date as inDate" +
//             "  FROM dwd_inpatient_record_info a " +
//             "  INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             "  ${ew.customSqlSegment}")
//     public List<DoctorDwdInRecordBO>  queryInRecordList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInRecordBO> wrapper);

    /**
     * 最新住院申请患者
     */
    // @Select(" SELECT a.patient_id, a.apply_department_code_standard, a.apply_department_name_standard, a.apply_date, b.id in_hospital_id " + 
    //         " FROM dwd_inpatient_reserve_info a " + 
    //         " LEFT JOIN dwd_inpatient_record_info b ON a.visit_id = b.visit_id " + 
    //         " WHERE (a.patient_id, a.apply_department_code_standard, a.apply_date) IN  ( " + 
    //         "   SELECT c.patient_id, c.apply_department_code_standard, MAX(c.apply_date) apply_date  " + 
    //         "   FROM dwd_inpatient_reserve_info c " + 
    //         "   ${ew.customSqlSegment} " + 
    //         "   GROUP BY c.patient_id, c.apply_department_code_standard " + 
    //         " ) ")
    // List<DoctorDwdInpatientReserveVO> queryInHospitalApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);
    // @Select(" SELECT a.patient_id, a.apply_department_code_standard, a.apply_department_name_standard, a.apply_date " + 
    //         " FROM dwd_inpatient_reserve_info a " + 
    //         " WHERE (a.patient_id, a.apply_department_code_standard, a.apply_date) IN  ( " + 
    //         "   SELECT c.patient_id, c.apply_department_code_standard, MAX(c.apply_date) apply_date  " + 
    //         "   FROM dwd_inpatient_reserve_info c " + 
    //         "   ${ew.customSqlSegment} " + 
    //         "   GROUP BY c.patient_id, c.apply_department_code_standard " + 
    //         " ) ")
    // List<DoctorDwdInpatientReserveVO> queryInHospitalApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);



    // @Select(" SELECT a.patient_id, a.apply_department_code_standard, MAX(c.apply_date) apply_date  " + 
    //         "   FROM dwd_inpatient_reserve_info a " + 
    //         "   ${ew.customSqlSegment} ")
    // List<DoctorDwdInpatientReserveVO> queryLastInHospitalApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);



//    /**
//     * delete
//     * */
//    @Select(" SELECT a.patient_id, a.apply_department_code_standard, a.apply_department_name_standard, a.apply_date " +
//            " FROM dwd_inpatient_reserve_info a " +
//            " ${ew.customSqlSegment} ")
//    List<DoctorDwdInpatientReserveVO> queryInHospitalApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);



    /**
     * 手术申请患者
     */
    // @Select(" SELECT a.patient_id, a.department_code_standard, a.department_name_standard, a.apply_time, b.id operation_id " + 
    //         " FROM dwd_operation_apply_info a " + 
    //         " LEFT JOIN dwd_operation_record_info b ON a.visit_id = b.visit_id " + 
    //         " WHERE (a.patient_id, a.department_code_standard, a.apply_time) IN  ( " + 
    //         "   SELECT c.patient_id, c.department_code_standard, MAX(c.apply_time) apply_time  " + 
    //         "   FROM dwd_operation_apply_info c " + 
    //         "   ${ew.customSqlSegment} " + 
    //         "   GROUP BY c.patient_id, c.department_code_standard " + 
    //         " ) ")
    // List<DoctorDwdOperationApplyVO> queryOperationApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    @Deprecated
    @Select(" SELECT a.patient_id, a.department_code_standard, MAX(a.apply_time) apply_time  " + 
            "   FROM dwd_operation_apply_info a " + 
            "   ${ew.customSqlSegment} ")
    List<DoctorDwdOperationApplyVO> queryLastOperationApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);
//     @Select(" SELECT b.main_person_id patientId, a.department_code_standard, MAX(a.apply_time) apply_time  " + 
//             "   FROM dwd_operation_apply_info a " + 
//             "   INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             "   ${ew.customSqlSegment} ")
//     List<DoctorDwdOperationApplyVO> queryLastOperationApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    @Select(" SELECT a.patient_id, a.department_code_standard, a.department_name_standard, a.apply_time, b.id operation_id " + 
            " FROM dwd_operation_apply_info a " + 
            " LEFT JOIN dwd_operation_record_info b ON a.visit_id = b.visit_id " + 
            " ${ew.customSqlSegment} ")
    List<DoctorDwdOperationApplyVO> queryOperationApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);
//     @Select(" SELECT c.main_person_id patientId, a.department_code_standard, a.department_name_standard, a.apply_time, b.id operation_id " + 
//             " FROM dwd_operation_apply_info a " + 
//             " INNER JOIN dwd_patient_info c ON a.patient_id = c.patient_id " + 
//             " LEFT JOIN dwd_operation_record_info b ON a.visit_id = b.visit_id " + 
//             " ${ew.customSqlSegment} ")
//     List<DoctorDwdOperationApplyVO> queryOperationApplyList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    /**
     * 查询门诊/急诊患者id
     */
    @Select(" SELECT a.patient_id " + 
            " FROM dwd_visit_record_info a " + 
            " ${ew.customSqlSegment} " + 
            " GROUP BY a.patient_id ")
    List<String> queryVisitRecordPatientIdList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);
//     @Select(" SELECT b.main_person_id patientId " + 
//             " FROM dwd_visit_record_info a " + 
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} " + 
//             " GROUP BY a.patient_id ")
//     List<String> queryVisitRecordPatientIdList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    /**
     * 查询住院患者id
     */
    @Select(" SELECT a.patient_id " + 
            " FROM dwd_inpatient_record_info a " + 
            " ${ew.customSqlSegment} " + 
            " GROUP BY a.patient_id ")
    List<String> queryInHospitalRecordPatientIdList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);
//     @Select(" SELECT b.main_person_id patientId " + 
//             " FROM dwd_inpatient_record_info a " + 
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} " + 
//             " GROUP BY a.patient_id ")
//     List<String> queryInHospitalRecordPatientIdList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    /**
     * 查询挂号申请数量
     */
    @Select(" SELECT COUNT(1) " + 
            " FROM dwd_visit_record_info a " + 
            " ${ew.customSqlSegment} ")
    Long queryRegisterApplyCount(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    /**
     * 查询住院申请数量
     */
    @Select(" SELECT COUNT(1) count " + 
            " FROM dwd_inpatient_record_info a " + 
            " ${ew.customSqlSegment} ")
    Long queryInHospitalApplyCount(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    /**
     * 查询手术申请数量
     */
    @Select(" SELECT COUNT(1) count " + 
            " FROM dwd_operation_apply_info a " + 
            " ${ew.customSqlSegment} ")
    Long queryOperationApplyCount(@Param(Constants.WRAPPER) Wrapper<DoctorDwdOperationApplyVO> wrapper);

    // 检验报告详情
    @Select(" SELECT b.* " + 
            " FROM dwd_laboratory_report_info a " +
            " INNER JOIN dwd_laboratory_report_item_info b ON a.id = b.laboratory_report_id " +
            " ${ew.customSqlSegment} ")
    List<DataDwdLaboratoryReportItemVO> getLaboratoryReportItemList(@Param(Constants.WRAPPER) Wrapper<DataDwdLaboratoryQueryDTO> wrapper);
//     @Select(" SELECT b.main_person_id patientId, a.laboratory_report_id laboratoryReportId, a.id, a.sample_type_raw sampleTypeRaw, a.sample_type_standard sampleTypeStandard, a.project_group_code_raw projectGroupCodeRaw, a.project_group_code_standard projectGroupCodeStandard, " + 
//             " a.project_group_name_raw projectGroupNameRaw, a.project_group_name_standard projectGroupNameStandard, a.project_id_raw projectIdRaw, a.project_id_standard projectIdStandard, a.name_raw nameRaw, a.name_standard nameStandard, a.code_raw codeRaw, a.code_standard codeStandard, " +
//             " a.abnormal, a.abnormal_indicator abnormalIndicator, a.unit_raw unitRaw, a.unit_standard unitStandard, a.reference_raw referenceRaw, a.reference_standard referenceStandard, a.inspect_result_raw inspectResultRaw, a.result_type resultType, a.result_two_category_code resultTwoCategoryCode, " +
//             " a.result_two_category_name resultTwoCategoryName, a.result_narrate resultNarrate, a.result_value resultValue, a.result_value_flag resultValueFlag, a.result_value_rate resultValueRate, a.result_value_reference resultValueReference, a.data_update_date dataUpdateDate, a.op_data_user opDataUser, a.op_date opDate, a.reports_time reportsTime " +
//             " FROM dwd_laboratory_report_item_info a " +
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} ")
//     List<DataDwdLaboratoryReportItemVO> getLaboratoryReportItemList(@Param(Constants.WRAPPER) Wrapper<DataDwdLaboratoryQueryDTO> wrapper);



    // 查询上级部门id
    @Select(" SELECT id, pid " + 
            " FROM sys_dept " +
            " ${ew.customSqlSegment} ")
    List<Map<String, String>> getParentDeptIds(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);

    // 查询下级部门id列表
    @Select(" SELECT id, pid " + 
            " FROM sys_dept " +
            " ${ew.customSqlSegment} ")
    List<Map<String, String>> getSubDeptIds(@Param(Constants.WRAPPER) Wrapper<DoctorDwdInpatientReserveVO> wrapper);





    /**
     * 患者最新数据
     */
    @Select(" SELECT patient_id AS patientId, last_op_date AS lastOpDate, content " + 
            " FROM dws_patient_data_last_date_info " +
            " ${ew.customSqlSegment} ")
    DataDwsPatientDataLastDateVO getPatientDataLastDateInfo(@Param(Constants.WRAPPER) Wrapper<String> wrapper);
//     @Select(" SELECT b.main_person_id AS patientId, a.last_op_date AS lastOpDate, a.content " + 
//             " FROM dws_patient_data_last_date_info a " +
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} ")
//     DataDwsPatientDataLastDateVO getPatientDataLastDateInfo(@Param(Constants.WRAPPER) Wrapper<String> wrapper);

    @Select(" SELECT patient_id AS patientId, last_op_date AS lastOpDate, content " + 
            " FROM dws_patient_data_last_date_info " +
            " ${ew.customSqlSegment} ")
    List<DataDwsPatientDataLastDateVO> listPatientDataLastDateInfo(@Param(Constants.WRAPPER) Wrapper<String> wrapper);
//     @Select(" SELECT b.main_person_id AS patientId, a.last_op_date AS lastOpDate, a.content " + 
//             " FROM dws_patient_data_last_date_info a " +
//             " INNER JOIN dwd_patient_info b ON a.patient_id = b.patient_id " + 
//             " ${ew.customSqlSegment} ")
//     List<DataDwsPatientDataLastDateVO> listPatientDataLastDateInfo(@Param(Constants.WRAPPER) Wrapper<String> wrapper);



    // 检查报告
    @Select(" SELECT * " + 
            " FROM dwd_examine_report_info " +
            " ${ew.customSqlSegment} ")
    List<PatientExamineReportClientBO> listExamineReportInfo(@Param(Constants.WRAPPER) Wrapper<PatientExamineReportClientBO> wrapper);

    // 检验报告
    @Select(" SELECT * " + 
            " FROM dwd_laboratory_report_info " +
            " ${ew.customSqlSegment} ")
    List<PatientLaboratoryReportClientBO> listLaboratoryReportInfo(@Param(Constants.WRAPPER) Wrapper<PatientLaboratoryReportClientBO> wrapper);

    // 医嘱用药
    @Select(" SELECT * " + 
            " FROM dwd_recipe_medicine_info " +
            " ${ew.customSqlSegment} ")
    List<PatientRecipeMedicineClientBO> listRecipeMedicineInfo(@Param(Constants.WRAPPER) Wrapper<PatientRecipeMedicineClientBO> wrapper);

    // 医嘱用药详情
    @Select(" SELECT * " + 
            " FROM dwd_recipe_medicine_item_info " +
            " ${ew.customSqlSegment} ")
    List<PatientRecipeMedicineItemClientBO> listRecipeMedicineItemInfo(@Param(Constants.WRAPPER) Wrapper<PatientRecipeMedicineItemClientBO> wrapper);

    // 住院数据
    @Select(" SELECT * " + 
        " FROM dwd_inpatient_record_info " + 
        " ${ew.customSqlSegment} ")
    List<PatientInpatientRecordClientBO> listPatientInpatientInfo(@Param(Constants.WRAPPER) Wrapper<PatientInpatientRecordClientBO> wrapper);

    // 住院最新数据
    @Select(" SELECT * " + 
        " FROM dwd_inpatient_record_info " + 
        " WHERE CONCAT(patient_id, '-', inpatient_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.inpatient_date)) " + 
        "   FROM dwd_inpatient_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    List<PatientInpatientRecordClientBO> listPatientInpatientLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientInpatientRecordClientBO> wrapper);

    @Select(" SELECT * " + 
        " FROM dwd_inpatient_record_info " + 
        " WHERE CONCAT(patient_id, '-', inpatient_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.inpatient_date)) " + 
        "   FROM dwd_inpatient_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    PatientInpatientRecordClientBO getOnePatientInpatientLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientInpatientRecordClientBO> wrapper);

    // 最新住院但未出院数据
    @Select(" SELECT a.* " +
        " FROM dwd_inpatient_record_info a " + 
        " LEFT JOIN dwd_discharge_record_info b ON a.visit_id = b.visit_id " + 
        " WHERE CONCAT(a.patient_id, '-', a.inpatient_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.inpatient_date)) " + 
        "   FROM dwd_inpatient_record_info a " +
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) AND b.id IS NULL ")
    List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndNotOut(@Param(Constants.WRAPPER) Wrapper<PatientInpatientRecordClientBO> wrapper);

    // 最新住院且出院数据
    @Select(" SELECT a.*, b.is_out " + 
            " FROM dwd_inpatient_record_info a " + 
            " INNER JOIN ( " +
            "   SELECT CONCAT(a.patient_id, '-', MAX(a.inpatient_date)) as pi, IF(MAX(b.visit_id) IS NULL, 'N', 'Y') as is_out " + 
            "   FROM dwd_inpatient_record_info a  " + 
            "   LEFT JOIN dwd_discharge_record_info b ON a.visit_id = b.visit_id  " + 
            "   ${ew.customSqlSegment} " + 
            "   GROUP BY a.patient_id  " + 
            " ) b ON CONCAT(a.patient_id, '-', a.inpatient_date) = b.pi ")
    List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndOut(@Param(Constants.WRAPPER) Wrapper<PatientInpatientRecordClientBO> wrapper);

    // 住院变更最新数据
    @Select(" SELECT * " + 
        " FROM dwd_inpatient_change_info " + 
        " ${ew.customSqlSegment} ")
    List<PatientInpatientChangeClientBO> listPatientInpatientChangeLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientInpatientChangeClientBO> wrapper);
    
    // 出院最新数据
    @Select(" SELECT * " + 
        " FROM dwd_discharge_record_info " + 
        " WHERE CONCAT(patient_id, '-', leave_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.leave_date)) " + 
        "   FROM dwd_discharge_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientDischargeRecordClientBO> wrapper);

    @Select(" SELECT * " + 
        " FROM dwd_discharge_record_info " + 
        " WHERE CONCAT(patient_id, '-', leave_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.leave_date)) " + 
        "   FROM dwd_discharge_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    PatientDischargeRecordClientBO getOnePatientOutpatientLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientDischargeRecordClientBO> wrapper);

    @Select(" SELECT * " + 
        " FROM dwd_discharge_content_info " + 
        " ${ew.customSqlSegment} ")
    List<PatientDischargeContentClientBO> listPatientOutpatientContentInfo(@Param(Constants.WRAPPER) Wrapper<PatientDischargeContentClientBO> wrapper);

    @Select(" SELECT * " + 
        " FROM dwd_discharge_content_info " + 
        " ${ew.customSqlSegment} ")
    PatientDischargeContentClientBO getOnePatientOutpatientContentInfo(@Param(Constants.WRAPPER) Wrapper<PatientDischargeContentClientBO> wrapper);

    // 就诊最新数据
    @Select(" SELECT * " + 
        " FROM dwd_visit_record_info " + 
        " WHERE CONCAT(patient_id, '-', visit_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.visit_date)) " + 
        "   FROM dwd_visit_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    List<PatientVisitRecordClientBO> listPatientVisitLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientVisitRecordClientBO> wrapper);

    @Select(" SELECT * " + 
        " FROM dwd_visit_record_info " + 
        " WHERE CONCAT(patient_id, '-', visit_date) IN ( " + 
        "   SELECT CONCAT(a.patient_id, '-', MAX(a.visit_date)) " + 
        "   FROM dwd_visit_record_info a " + 
        "   ${ew.customSqlSegment} " + 
        "   GROUP BY a.patient_id " + 
        " ) ")
    PatientVisitRecordClientBO getOnePatientVisitLastDateInfo(@Param(Constants.WRAPPER) Wrapper<PatientVisitRecordClientBO> wrapper);

    // 查询患者每日患者数据变化
    @Select(" SELECT patient_id " + 
        " FROM dws_patient_data_day_info " + 
        " ${ew.customSqlSegment} " + 
        " GROUP BY patient_id " + 
        " ORDER BY patient_id ASC ")
    IPage<String> pagePatientChangeDateInfo(Page<Void> page, @Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    @Select(" SELECT patient_id as patientId, data_day as dataDay " + 
        " FROM dws_patient_data_day_info " + 
        " ${ew.customSqlSegment} " + 
        " ORDER BY patient_id ASC ")
    IPage<Map<String, String>> pagePatientChangeDateMapInfo(Page<Void> page, @Param(Constants.WRAPPER) Wrapper<Void> wrapper);


    // 诊断
    @Select(" SELECT * " + 
        " FROM dwd_diagnosis_report_info " + 
        " ${ew.customSqlSegment} ")
    List<PatientDiagnosisReportClientBO> listPatientDiagnosisReportInfo(@Param(Constants.WRAPPER) Wrapper<PatientDiagnosisReportClientBO> wrapper);

    // 诊断去重
    @Select(" SELECT DISTINCT patient_id,patient_name,diagnosis_code_standard " +
            " FROM dwd_diagnosis_report_info " +
            " ${ew.customSqlSegment} ")
    List<PatientDiagnosisReportClientBO> listPatientDiagnosisReportDistinct(@Param(Constants.WRAPPER) Wrapper<PatientDiagnosisReportClientBO> wrapper);


    // 查询费用明细
    @Select(" select fi.patient_id, fi.patient_name, fi.fee_code_standard, fi.fee_name_standard, fi.item_code_standard, fi.item_name_standard, fi.total_fee, fi.own_fee, fi.own_pay_fee, fi.center_pay_fee, " +
            " fi.register_sn, re.department_code_standard, re.department_name_standard, re.doctor_code, re.doctor_name, re.reg_level_code_raw, re.reg_level_name_raw, fi.register_date " +
            " from dwd_collect_fee_record_info fi left join dwd_register_record_info re on fi.register_sn = re.id " +
            " ${ew.customSqlSegment} ")
    IPage<PatientFeeRecordBO> pagePatientFeeRecord(Page<Void> page, @Param(Constants.WRAPPER) Wrapper<PatientFeeRecordBO> wrapper);

    // 查询费用明细
    @Select(" select fi.patient_id, fi.patient_name, fi.fee_code_standard, fi.fee_name_standard, fi.item_code_standard, fi.item_name_standard, fi.total_fee, fi.own_fee, fi.own_pay_fee, fi.center_pay_fee, " +
            " fi.register_sn, re.department_code_standard, re.department_name_standard, re.doctor_code, re.doctor_name, re.reg_level_code_raw, re.reg_level_name_raw, fi.register_date " +
            " from dwd_collect_fee_record_info fi left join dwd_register_record_info re on fi.register_sn = re.id " +
            " ${ew.customSqlSegment} ")
    List<PatientFeeRecordBO> listPatientFeeRecord(@Param(Constants.WRAPPER) Wrapper<PatientFeeRecordBO> wrapper);


    // 挂号流失
    @Select(" SELECT a.* " + 
            " FROM dwd_register_record_info a " + 
            " LEFT JOIN dwd_visit_record_info b ON a.id = b.register_id " + 
            " ${ew.customSqlSegment} ")
    List<DoctorDwdRegisterRecordVO> queryRegisterLossList(@Param(Constants.WRAPPER) Wrapper<DoctorDwdRegisterRecordVO> wrapper);
    
    // 就诊流失
    @Select(" SELECT a.* " + 
            " FROM dwd_visit_record_info a " + 
            " LEFT JOIN dwd_examine_report_info b ON a.register_id = b.visit_id " + 
            " LEFT JOIN dwd_laboratory_report_info c ON a.register_id = c.visit_id " + 
            " LEFT JOIN dwd_recipe_therapy_info d ON a.register_id = d.visit_id " + 
            " LEFT JOIN dwd_recipe_medicine_info e ON a.register_id = e.visit_id " + 
            " ${ew.customSqlSegment} ")
    List<PatientVisitRecordClientBO> queryVisitLossList(@Param(Constants.WRAPPER) Wrapper<PatientVisitRecordClientBO> wrapper);
    
    // 住院流失
//    @Select(" SELECT a.* " +
//            " FROM dwd_inpatient_reserve_info a " +
//            " LEFT JOIN dwd_inpatient_record_info b ON a.patient_id = b.patient_id " +
//            " ${ew.customSqlSegment} ")
    @Select(" SELECT a.* " +
            " FROM dwd_inpatient_reserve_info a " +
            " LEFT JOIN dwd_inpatient_record_info b on a.patient_id = b.patient_id " +
            " and (b.id is null or (b.id is not null and b.inpatient_date>=a.apply_date)) " +
            " ${ew.customSqlSegment} ")
    List<PatientInpatientReserveClientBO> queryInpatientLossList(@Param(Constants.WRAPPER) Wrapper<PatientInpatientReserveClientBO> wrapper);
    
}
