package com.xxh.cloud.healthy.data.framework.modules.biz.mapper;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientLabelDataVO;

public interface PatientDataQueryBizMapper {

    @Select(" SELECT DISTINCT a.patient_id " + 
            "    FROM hd_task_patient_info a " +
            "    INNER JOIN hd_task_dept_patient_relation_info e ON a.patient_id = e.patient_id " +
            "    ${ew.customSqlSegment} ")
    IPage<TaskPatientInfo> queryDeptAllPatient(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);

    /**
     * 统计当前标签用户数量
     * 本统计数据以患者当前标签为主进行统计
     */
    @Select(" SELECT COUNT(1) `num` " + 
            " FROM " +
            "  (SELECT DISTINCT a.patient_id, b.dept_id " +
            "     FROM h_patient_label_info a " +
            "     INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " + // 防止有标签却没有人的信息
            "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "     INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "     ${ew.customSqlSegment} " +
            "  ) e ")
    Long patientLabelCount(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);

    @Select(" SELECT COUNT(IF(stat_dimension_type = 'during_register', 1, NULL)) duringRegisterNum, COUNT(IF(stat_dimension_type = 'during_inHospital', 1, NULL)) duringInHospitalNum, COUNT(IF(stat_dimension_type = 'during_operation', 1, NULL)) duringOperationNum " + 
            " FROM " +
            "  (SELECT a.patient_id, b.dept_id, c.stat_dimension_type " +
            "     FROM h_patient_label_info a " +
            "     INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " + // 防止有标签却没有人的信息
            "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "     INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "     ${ew.customSqlSegment} " +
            "     GROUP BY a.patient_id, b.dept_id, c.stat_dimension_type " +
            "  ) e ")
    Map<String, Long> patientLabelCountV2(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);

    /**
     * 统计当前标签用户列表
     */
    // @Select(" SELECT DISTINCT d.* " +
    //         "    FROM h_patient_label_info a " +
    //         "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
    //         "    INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
    //         "    INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " +
    //         "    ${ew.customSqlSegment}") 
    // IPage<TaskPatientInfo> patientPreLabelStatisticPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);
    @Select(" SELECT DISTINCT a.patient_id " +
            "    FROM hd_task_patient_info a " +
            "    INNER JOIN h_patient_label_info d ON a.patient_id = d.patient_id " +
            "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "    INNER JOIN s_md_standard_label_info c ON d.label_id = c.id " +
            "    ${ew.customSqlSegment}") 
    IPage<TaskPatientInfo> patientLabelPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);

    /**
     * 统计当前标签用户数量
     * 本统计数据以患者当前标签为主进行统计
     */
    @Select(" SELECT COUNT(1) `num` " + 
            " FROM " +
            "  (SELECT DISTINCT a.patient_id, b.dept_id " +
            "     FROM h_patient_label_history_info a " +
            "     INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " + // 防止有标签却没有人的信息
            "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "     INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "     ${ew.customSqlSegment} " +
            "  ) e ")
    Long patientHistoryLabelCount(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);

    /**
     * 统计当前标签用户数量
     * 本统计数据以患者当前标签为主进行统计
     */
    @Select(" SELECT COUNT(IF(stat_dimension_type = 'during_register', 1, NULL)) duringRegisterNum, COUNT(IF(stat_dimension_type = 'during_inHospital', 1, NULL)) duringInHospitalNum, COUNT(IF(stat_dimension_type = 'during_operation', 1, NULL)) duringOperationNum " + 
            " FROM " +
            "  (SELECT a.patient_id, b.dept_id, c.stat_dimension_type " +
            "     FROM h_patient_label_history_info a " +
            "     INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " + // 防止有标签却没有人的信息
            "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "     INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "     ${ew.customSqlSegment} " +
            "     GROUP BY a.patient_id, b.dept_id, c.stat_dimension_type " +
            "  ) e ")
    Map<String, Long> patientHistoryLabelCountV2(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);

    /**
     * 统计当前标签用户列表
     * 本统计数据以患者当前标签为主进行统计
     */
    // @Select(" SELECT DISTINCT d.* " +
    //         "     FROM h_patient_label_history_info a " +
    //         "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
    //         "     INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
    //         "     INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " +
    //         "     ${ew.customSqlSegment} ")
    // IPage<TaskPatientInfo> patientHistoryLabelPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);
    @Select(" SELECT DISTINCT a.patient_id " +
            "     FROM hd_task_patient_info a " +
            "     INNER JOIN h_patient_label_history_info d ON a.patient_id = d.patient_id " +
            "     INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "     INNER JOIN s_md_standard_label_info c ON d.label_id = c.id " +
            "     ${ew.customSqlSegment} ")
    IPage<TaskPatientInfo> patientHistoryLabelPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);

    /**
     * 诊中
     * 统计流失标签用户数量
     * 本统计数据以患者历史标签为主进行统计
     * 注意：每个机构每种流失类型的标签只有一个，id为 deptId—流失类型：1686623479010824192-during_register
     */
    @Select(" SELECT COUNT(1) `num` " + 
            " FROM " +  
            " ( " +  
            "    SELECT DISTINCT a.patient_id " + 
            "    FROM h_patient_label_history_info a " + 
            "    WHERE (a.patient_id, a.label_id, a.op_date) IN " + 
            "    ( " + 
            "        SELECT a.patient_id, a.label_id, MAX(a.op_date) op_date " + 
            "        FROM h_patient_label_history_info a " + 
            "        ${ew.customSqlSegment} " + 
            "        GROUP BY a.patient_id, a.label_id " + 
            "    ) AND a.op_type = ${opType} " + 
            " ) e ")
    Long patientLostHistoryLabelCount(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper, @Param("opType") String opType);





    ////////////// 一下是定时任务统计所有部门的数据，并保存到数据库，上面是实时查询某个部门数据

    /**
     * 诊前
     * 统计标签用户数量
     * 本统计数据以患者历史标签为主进行统计
     */
    @Select(" SELECT e.dept_id deptId, COUNT(e.dept_id) `num` " + 
            " FROM " +
            "        (SELECT DISTINCT a.patient_id, b.dept_id " +
            "            FROM h_patient_label_history_info a " +
            "            INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "            INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "            ${ew.customSqlSegment} " +
            "        ) e " +
            " GROUP BY dept_id")
    List<PatientDataOneDayInfo> patientHistoryLabelStatistic(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);

    /**
     * 诊前
     * 查询诊前数据的用户列表
     */
    // @Select(" SELECT DISTINCT d.* " +
    //         "    FROM h_patient_label_history_info a " +
    //         "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
    //         "    INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
    //         "    INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " +
    //         "    ${ew.customSqlSegment}") 
    // IPage<TaskPatientInfo> patientPreHistoryLabelStatisticPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);
//     @Select(" SELECT DISTINCT a.patient_id " +
//             "    FROM h_patient_label_history_info a " +
//             "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
//             "    INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
//             "    ${ew.customSqlSegment}") 
//     IPage<TaskPatientInfo> patientPreHistoryLabelStatisticPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper);

    /**
     * 诊中
     * 统计流失标签用户数量
     * 本统计数据以患者历史标签为主进行统计
     */
    @Select(" SELECT dept_id, COUNT(1) `num` " + 
            " FROM " +  
            " ( " +  
            "    SELECT DISTINCT a.patient_id, c.dept_id " + 
            "    FROM h_patient_label_history_info a " + 
            "    INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
            "    INNER JOIN hd_task_dept_patient_relation_info c ON a.patient_id = c.patient_id " + 
            "    WHERE (a.patient_id, b.stat_dimension_type, a.op_date) IN " + 
            "    ( " + 
            "        SELECT a.patient_id, b.stat_dimension_type, MAX(a.op_date) op_date " + 
            "        FROM h_patient_label_history_info a " + 
            "        INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
            "        ${ew.customSqlSegment} " + 
            "        GROUP BY a.patient_id, b.stat_dimension_type " + 
            "    ) AND a.op_type = ${opType} " + 
            " ) e " + 
            " GROUP BY dept_id ")
    List<PatientDataOneDayInfo> patientLostHistoryLabelStatistic(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper, @Param("opType") String opType);

    /**
     * 诊中
     * 查询诊中数据的用户列表
     */
    // @Select(" SELECT DISTINCT d.* " + 
    //         "    FROM h_patient_label_history_info a " + 
    //         "    INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
    //         "    INNER JOIN hd_task_dept_patient_relation_info c ON a.patient_id = c.patient_id " + 
    //         "    INNER JOIN hd_task_patient_info d ON a.patient_id = d.patient_id " + 
    //         "    WHERE (a.patient_id, b.stat_dimension_type, a.op_date) IN " + 
    //         "    ( " + 
    //         "        SELECT a.patient_id, b.stat_dimension_type, MAX(a.op_date) op_date " + 
    //         "        FROM h_patient_label_history_info a " + 
    //         "        INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
    //         "        ${ew.customSqlSegment} " + 
    //         "        GROUP BY a.patient_id, b.stat_dimension_type " + 
    //         "    ) AND a.op_type = ${opType} ")
    // IPage<TaskPatientInfo> patientDuringHistoryLabelStatisticPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper, @Param("opType") String opType);
    @Select(" SELECT DISTINCT a.patient_id " + 
            "    FROM h_patient_label_history_info a " + 
            "    INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
            "    INNER JOIN hd_task_dept_patient_relation_info c ON a.patient_id = c.patient_id " + 
            "    WHERE (a.patient_id, b.stat_dimension_type, a.op_date) IN " + 
            "    ( " + 
            "        SELECT a.patient_id, b.stat_dimension_type, MAX(a.op_date) op_date " + 
            "        FROM h_patient_label_history_info a " + 
            "        INNER JOIN s_md_standard_label_info b ON a.label_id = b.id " + 
            "        ${ew.customSqlSegment} " + 
            "        GROUP BY a.patient_id, b.stat_dimension_type " + 
            "    ) AND a.op_type = ${opType} ")
    IPage<TaskPatientInfo> patientDuringHistoryLabelStatisticPatientList(Page<TaskPatientInfo> page, @Param(Constants.WRAPPER) Wrapper<TaskPatientInfo> wrapper, @Param("opType") String opType);

    /**
     * 诊后
     * 统计进行服务推送的次数
     * 本数据以患者日程为主进行统计
     */
//     @Select("select e.dept_id deptId, COUNT(e.dept_id) `num` " + 
//             " FROM " +
//             "        (SELECT DISTINCT a.id, b.dept_id " +
//             "            FROM h_patient_schedule_info a " +
//             "            INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
//             "            INNER JOIN h_patient_label_history_info c ON a.patient_id = c.patient_id AND c.op_type = '1'  " +
//             "            ${ew.customSqlSegment} " +
//             "        ) e " +
//             " GROUP BY dept_id ")
//     List<PatientDataOneDayInfo> patientPushServeStatistic(@Param(Constants.WRAPPER) Wrapper<PatientDataOneDayInfo> wrapper);


    /**
     * 查询用户当前标签数据
     */
    @Select(" SELECT DISTINCT a.patient_id AS patientId , a.label_id AS labelId " +
            "    FROM h_patient_label_info a " +
            "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "    INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "    ${ew.customSqlSegment}") 
    List<PatientLabelDataVO> patientLabelList(@Param(Constants.WRAPPER) Wrapper<PatientLabelDataVO> wrapper);

    /**
     * 查询用户当前标签数据
     */
    @Select(" SELECT DISTINCT a.patient_id AS patientId, a.label_id AS labelId " +
            "    FROM h_patient_label_history_info a " +
            "    INNER JOIN hd_task_dept_patient_relation_info b ON a.patient_id = b.patient_id " +
            "    INNER JOIN s_md_standard_label_info c ON a.label_id = c.id " +
            "    ${ew.customSqlSegment}") 
    List<PatientLabelDataVO> patientHistoryLabelList(@Param(Constants.WRAPPER) Wrapper<PatientLabelDataVO> wrapper);
}
