package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptLabelRelationQueryDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskDeptLabelRelationInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskDeptLabelRelationInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptLabelRelationAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptLabelRelationDelDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskDeptLabelRelationVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DeptLabelRelationBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;

@Service
public class DeptLabelRelationBizServiceImpl implements DeptLabelRelationBizService {

    @Autowired
    private TaskDeptLabelRelationInfoService taskDeptLabelRelationInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DeptLabelRelationAddDTO params) {
        ValidatorUtils.validateEntity(params);

        TaskDeptLabelRelationInfo entity = taskDeptLabelRelationInfoService.getOne(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("label_id", params.getLabelId())
                .last("limit 1")
        );

        if (null != entity) {
            return;
        }

        entity = new TaskDeptLabelRelationInfo();
        entity.setDeptId(params.getDeptId());
        entity.setLabelId(params.getLabelId());
        entity.setOpDt(DateUtil.date());
        taskDeptLabelRelationInfoService.save(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<DeptLabelRelationAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        List<String> deptLabelIds = new ArrayList<>();
        params.forEach(i -> deptLabelIds.add(i.getDeptId() + "-" + i.getLabelId()));
        
        List<TaskDeptLabelRelationInfo> entityList = taskDeptLabelRelationInfoService.list(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .in("CONCAT(dept_id, '-', label_id)", deptLabelIds)
        );

        Set<String> deptLabelIdSet = entityList.stream().map(i -> i.getDeptId() + "-" + i.getLabelId()).collect(Collectors.toSet());

        DateTime now = DateTime.now();
        List<TaskDeptLabelRelationInfo> list = new ArrayList<>();
        params.forEach(i -> {
            if (deptLabelIdSet.contains(i.getDeptId() + "-" + i.getLabelId())) {
                return;
            }

            TaskDeptLabelRelationInfo entity = new TaskDeptLabelRelationInfo();
            entity.setDeptId(i.getDeptId());
            entity.setLabelId(i.getLabelId());
            entity.setOpDt(now);

            list.add(entity);
        });

        taskDeptLabelRelationInfoService.saveBatch(list);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void del(DeptLabelRelationDelDTO params) {
        ValidatorUtils.validateEntity(params);

        taskDeptLabelRelationInfoService.remove(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("label_id", params.getLabelId())
        );
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delBatch(List<DeptLabelRelationDelDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        List<String> deptLabelIds = new ArrayList<>();
        params.forEach(i -> deptLabelIds.add(i.getDeptId() + "-" + i.getLabelId()));
        
        taskDeptLabelRelationInfoService.remove(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .in("CONCAT(dept_id, '-', label_id)", deptLabelIds)
        );
    }

    @Override
    public List<TaskDeptLabelRelationVO> getByDeptLabelIds(Set<String> deptLabelIds) {
        if (CollectionUtil.isEmpty(deptLabelIds)) {
            return new ArrayList<>(0);
        }
        
        List<TaskDeptLabelRelationInfo> entityList = taskDeptLabelRelationInfoService.list(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .in("CONCAT(dept_id, '-', label_id)", deptLabelIds)
        );
        if (CollUtil.isEmpty(entityList)) {
            return new ArrayList<>();
        }
        return entityList.stream().map(DoctorDataHelper::getTaskDeptLabelRelationVO).collect(Collectors.toList());
    }

    @Override
    public List<TaskDeptLabelRelationVO> getByDeptLabel(DeptLabelRelationQueryDTO params) {
        List<TaskDeptLabelRelationInfo> entityList = taskDeptLabelRelationInfoService.list(new QueryWrapper<TaskDeptLabelRelationInfo>()
                .eq(StrUtil.isNotEmpty(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotEmpty(params.getLabelId()),"label_id", params.getLabelId())
        );
        if (CollUtil.isEmpty(entityList)) {
            return new ArrayList<>();
        }
        return entityList.stream().map(DoctorDataHelper::getTaskDeptLabelRelationVO).collect(Collectors.toList());
    }

}
