package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskDeptPatientRelationInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskDeptPatientRelationInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationDelDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskDeptPatientRelationVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DeptPatientRelationBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DeptPatientRelationBizServiceImpl implements DeptPatientRelationBizService {

    @Autowired
    private TaskDeptPatientRelationInfoService taskDeptPatientRelationInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DeptPatientRelationAddDTO params) {
        ValidatorUtils.validateEntity(params);

        TaskDeptPatientRelationInfo entity = taskDeptPatientRelationInfoService.getOne(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("patient_id", params.getPatientId())
                .last("limit 1")
        );

        // 两种情况可以绑定：1. 没有绑定时可以绑定 2. 已绑定类型为Label并且将要绑定类型为Register时可以绑定
        if (null == entity) {
            entity = new TaskDeptPatientRelationInfo();
            entity.setDeptId(params.getDeptId());
            entity.setPatientId(params.getPatientId());
            entity.setType(params.getType());
            entity.setRegisterDt(DateUtil.date());
            taskDeptPatientRelationInfoService.save(entity);
        } else if (DeptPatientTypeEnum.Label.getCode().equals(entity.getType()) && DeptPatientTypeEnum.Register.getCode().equals(params.getType())) {
            taskDeptPatientRelationInfoService.update(new UpdateWrapper<TaskDeptPatientRelationInfo>()
                    .set("type", DeptPatientTypeEnum.Register.getCode())
                    .set("register_dt", DateUtil.date())
                    .eq("dept_id", entity.getDeptId())
                    .eq("patient_id", entity.getPatientId())
            );
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<DeptPatientRelationAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        List<String> deptPatientIds = new ArrayList<>();
        params.forEach(i -> deptPatientIds.add(i.getDeptId() + "-" + i.getPatientId()));
        
        List<TaskDeptPatientRelationInfo> entityList = taskDeptPatientRelationInfoService.list(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .in("CONCAT(dept_id, '-', patient_id)", deptPatientIds)
        );

        Map<String, TaskDeptPatientRelationInfo> entityMap = entityList.stream().collect(Collectors.toMap(i -> i.getDeptId() + "-" + i.getPatientId(), i -> i));

        DateTime now = DateTime.now();
        List<TaskDeptPatientRelationInfo> saveList = new ArrayList<>(); // 存储需要新增的数据
        List<String> updateList = new ArrayList<>(); // 存储需要更新的数据
        params.forEach(i -> {
            TaskDeptPatientRelationInfo entity = entityMap.get(i.getDeptId() + "-" + i.getPatientId());

            // 两种情况可以绑定：1. 没有绑定时可以绑定 2. 已绑定类型为Label并且将要绑定类型为Register时可以绑定
            if (null == entity) {
                entity = new TaskDeptPatientRelationInfo();
                entity.setDeptId(i.getDeptId());
                entity.setPatientId(i.getPatientId());
                entity.setRegisterDt(now);
                entity.setType(i.getType());

                saveList.add(entity);
            } else if (DeptPatientTypeEnum.Label.getCode().equals(entity.getType()) && DeptPatientTypeEnum.Register.getCode().equals(i.getType())) {
                updateList.add(entity.getDeptId() + "-" + entity.getPatientId());
            }
        });

        if (CollectionUtil.isNotEmpty(saveList)) {
            taskDeptPatientRelationInfoService.saveBatch(saveList);
        }
        if (CollectionUtil.isNotEmpty(updateList)) {
            taskDeptPatientRelationInfoService.update(new UpdateWrapper<TaskDeptPatientRelationInfo>()
                    .set("type", DeptPatientTypeEnum.Register.getCode())
                    .set("register_dt", DateUtil.date())
                    .in("CONCAT(dept_id, '-', patient_id)", updateList)
            );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void del(DeptPatientRelationDelDTO params) {
        ValidatorUtils.validateEntity(params);

        TaskDeptPatientRelationInfo entity = taskDeptPatientRelationInfoService.getOne(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("patient_id", params.getPatientId())
                .last("limit 1")
        );

        // zs20231020
        if (null == entity || DeptPatientTypeEnum.Register.getCode().equals(entity.getType())) {
            return;
        }

        taskDeptPatientRelationInfoService.remove(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("patient_id", params.getPatientId())
        );
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delBatch(List<DeptPatientRelationDelDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        List<String> deptPatientIds = new ArrayList<>();
        params.forEach(i -> deptPatientIds.add(i.getDeptId() + "-" + i.getPatientId()));

        List<TaskDeptPatientRelationInfo> entityList = taskDeptPatientRelationInfoService.list(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .in("CONCAT(dept_id, '-', patient_id)", deptPatientIds)
        );

        if (CollectionUtil.isNotEmpty(entityList)) {
            List<String> delList = new ArrayList<>(); // 存储需要删除的数据
            entityList.forEach(i -> {
                if (!DeptPatientTypeEnum.Register.getCode().equals(i.getType())) {
                    delList.add(i.getDeptId() + "-" + i.getPatientId());
                }
            });
            if (CollectionUtil.isNotEmpty(delList)) {
                taskDeptPatientRelationInfoService.remove(new QueryWrapper<TaskDeptPatientRelationInfo>()
                        .in("CONCAT(dept_id, '-', patient_id)", delList)
                );
            }
        }
        
    }

    @Override
    public List<TaskDeptPatientRelationVO> getByDeptPatientIds(Set<String> deptPatientIds) {
        if (CollectionUtil.isEmpty(deptPatientIds)) {
            return new ArrayList<>(0);
        }
        
        List<TaskDeptPatientRelationInfo> entityList = taskDeptPatientRelationInfoService.list(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .in("CONCAT(dept_id, '-', patient_id)", deptPatientIds)
        );

        return entityList.stream().map(DoctorDataHelper::getTaskDeptPatientRelationVO).collect(Collectors.toList());
    }

    @Override
    public List<TaskDeptPatientRelationVO> query(DeptPatientRelationQueryDTO params) {
        
        List<TaskDeptPatientRelationInfo> entityList = taskDeptPatientRelationInfoService.list(new QueryWrapper<TaskDeptPatientRelationInfo>()
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
        );

        return entityList.stream().map(DoctorDataHelper::getTaskDeptPatientRelationVO).collect(Collectors.toList());
    }
    
}
