package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.common.constant.ModeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.crm.client.api.PatientDoctorRelationClientService;
import com.xxh.cloud.healthy.crm.client.api.bean.bo.PatientDoctorRelationBO;
import com.xxh.cloud.healthy.crm.client.api.bean.dto.PatientDoctorRelQueryClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.LabelPatientStatDimensionTypeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.LabelPatientStatisticClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientGetDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorDwdPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorPatientLabelQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwsPatientDataLastDateVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorFollowPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientFullVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DiseaseLabelStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.LabelBizQueryMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDataQueryService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorFollowPatientBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.common.util.DataUtil;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DoctorDataQueryServiceImpl implements DoctorDataQueryService {

    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private LabelBizQueryMapper labelBizQueryMapper;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private DoctorFollowPatientBizService doctorFollowPatientBizService;
    @Autowired
    private PatientDoctorRelationClientService patientDoctorRelationClientService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;

    @Override
    public List<StandardDiseaseBO> diseaseList(String deptId) {
        if (StrUtil.isBlank(deptId)) {
            return new ArrayList<>(0);
        }
        DeptBO dept = deptClientService.getDeptById(deptId);
        if (null == dept || CollUtil.isEmpty(dept.getDiseaseIds())) {
            return new ArrayList<>(0);
        }
        return standardDiseaseClientService.getList(dept.getDiseaseIds());
    }

    @Override
    public PageData<LabelBO> labelPage(DoctorPatientLabelQueryDTO params, String deptId) {
        if (StrUtil.isBlank(deptId)) {
            return new PageData<>(null, 0);
        }

        // 非管理员需按病种查
        if (StrUtil.isBlank(params.getIsAdmin()) || !YN.YES.getCode().equals(params.getIsAdmin())) {
            DeptBO dept = deptClientService.getDeptById(deptId);
            if (null == dept || CollUtil.isEmpty(dept.getDiseaseIds())) {
                return new PageData<>(null, 0);
            }

            if (CollUtil.isEmpty(params.getDiseaseIds())) {
                params.setDiseaseIds(dept.getDiseaseIds());
            } else {
                Set<String> diseaseIds = new HashSet<>();
                dept.getDiseaseIds().forEach(i -> {
                    if (params.getDiseaseIds().contains(i)) {
                        diseaseIds.add(i);
                    }
                });
                if (CollUtil.isNotEmpty(diseaseIds)) {
                    return new PageData<>(null, 0);
                }
                params.setDiseaseIds(new ArrayList<>(diseaseIds));
            }
            params.setDiseaseQueryMode(ModeEnum.QueryModeEnum.OR.getCode()); // or 查询
        }

        LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
        BeanUtil.copyProperties(params, labelQueryPageDTO);
        labelQueryPageDTO.setStatus(YN.YES.getCode());
        return labelClientService.queryPage(labelQueryPageDTO);
    }

    @Override
    public List<LabelBO> labelList(DoctorPatientLabelQueryDTO params, String deptId) {
        if (StrUtil.isBlank(deptId)) {
            return new ArrayList<>(0);
        }

        if (StrUtil.isBlank(params.getIsAdmin()) || !YN.YES.getCode().equals(params.getIsAdmin())) {
            DeptBO dept = deptClientService.getDeptById(deptId);
            if (null == dept || CollUtil.isEmpty(dept.getDiseaseIds())) {
                return new ArrayList<>(0);
            }

            if (CollUtil.isEmpty(params.getDiseaseIds())) {
                params.setDiseaseIds(dept.getDiseaseIds());
            } else {
                Set<String> diseaseIds = new HashSet<>();
                dept.getDiseaseIds().forEach(i -> {
                    if (params.getDiseaseIds().contains(i)) {
                        diseaseIds.add(i);
                    }
                });
                if (CollUtil.isNotEmpty(diseaseIds)) {
                    return new ArrayList<>(0);
                }
                params.setDiseaseIds(new ArrayList<>(diseaseIds));
            }
            params.setDiseaseQueryMode(ModeEnum.QueryModeEnum.OR.getCode()); // or 查询
        }

        LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
        BeanUtil.copyProperties(params, labelQueryPageDTO);
        labelQueryPageDTO.setStatus(YN.YES.getCode());
        return labelClientService.queryList(labelQueryPageDTO);
    }

    @Override
    public List<DiseaseLabelStatisticVO> diseaseLabelStatisticList(DoctorPatientLabelQueryDTO params, String deptId) {
        List<LabelBO> labelList = this.labelList(params, deptId);
        if (CollUtil.isEmpty(labelList)) {
            return new ArrayList<>(0);
        }

        Map<String, List<LabelBO>> labelMap = new HashMap<>();
        labelList.forEach(i -> {
            Map<String, String> diseaseMap = i.getDiseaseList().get(0);
            String diseaseId = diseaseMap.get("id");
            String diseaseName = diseaseMap.get("name");
            if (StrUtil.isBlank(diseaseName)) {
                diseaseName = "其他";
            }
            List<LabelBO> l = labelMap.get(diseaseId + "::" + diseaseName);
            if (null == l) {
                l = new ArrayList<>();
                labelMap.put(diseaseId + "::" + diseaseName, l);
            }
            l.add(i);
        });

        List<DiseaseLabelStatisticVO> voList = new ArrayList<>();
        labelMap.forEach((k, v) -> {
            String[] disease = k.split("::");
            DiseaseLabelStatisticVO vo = new DiseaseLabelStatisticVO();
            vo.setDiseaseId(disease[0]);
            vo.setDiseaseName(disease[1]);
            vo.setLabelBOList(v);

            voList.add(vo);
        });
        return voList;
    }

    @Override
    public PageData<DoctorLabelPatientFullVO> patientPage(PatientQueryDTO params, String deptId) {
        if (StrUtil.isBlank(deptId)) {
            return new PageData<>(null, 0);
        }

        Set<String> labelIds = new HashSet<>();
        DoctorPatientLabelQueryDTO labelQueryPageDTO = new DoctorPatientLabelQueryDTO();
        labelQueryPageDTO.setLabelId(params.getLabelId());
        labelQueryPageDTO.setLabelIds(params.getLabelIds());
        List<LabelBO> labelList = this.labelList(labelQueryPageDTO, deptId);
        if (CollUtil.isEmpty(labelList)) {
            return new PageData<>(null, 0);
        }
        labelList.forEach(i -> labelIds.add(i.getId()));
        if (CollUtil.isEmpty(labelIds)) {
            return new PageData<>(null, 0);
        }

        IPage<DoctorLabelPatientFullVO> page = labelBizQueryMapper.pageLabelPatient(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorLabelPatientFullVO>()
            .in("b.label_id", labelIds)
            .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
            .in(CollUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
            .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
            .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "a.patient_gender_standard", params.getPatientGenderStandard())
            .eq(StrUtil.isNotBlank(params.getLabelId()), "b.label_id", params.getLabelId())
            .in(CollUtil.isNotEmpty(params.getLabelIds()), "b.label_id", params.getLabelIds())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> patientIds = new ArrayList<>();
        Set<String> patientLabelIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            patientIds.add(i.getPatientId());
            patientLabelIds.add(i.getPatientLabelId());
        });
        
        // 查询患者最新数据信息
        Map<String, DataDwsPatientDataLastDateVO> patientDataLastDateMap = doctorDwdPatientDataBizService.mapPatientDataLastDateInfo(patientIds);
        
        // 查询患者最新就诊数据
        Map<String, PatientVisitRecordClientBO> visitMap = new HashMap<>();
        List<PatientVisitRecordClientBO> visitList = doctorDwdPatientDataBizService.listPatientVisitLastDateInfo(patientIds);
        if (CollUtil.isNotEmpty(visitList)) {
            visitList.forEach(i -> visitMap.put(i.getPatientId(), i));
        }

        // 查询患者最新住院数据
        Map<String, PatientInpatientRecordClientBO> inPatientMap = new HashMap<>();
        PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
        patientLastDataQueryDTO.setPatientIds(patientIds);
        List<PatientInpatientRecordClientBO> inPatientList = doctorDwdPatientDataBizService.listPatientInpatientLastDateInfo(patientLastDataQueryDTO);
        if (CollUtil.isNotEmpty(inPatientList)) {
            inPatientList.forEach(i -> inPatientMap.put(i.getPatientId(), i));
        }

        // 查询患者最新出院数据
        Map<String, PatientDischargeRecordClientBO> outPatientMap = new HashMap<>();
        List<PatientDischargeRecordClientBO> outPatientList = doctorDwdPatientDataBizService.listPatientOutpatientLastDateInfo(patientIds);
        if (CollUtil.isNotEmpty(outPatientList)) {
            outPatientList.forEach(i -> outPatientMap.put(i.getPatientId(), i));
        }

        // 是否关注患者
        Map<String, DoctorFollowPatientVO> followMap = new HashMap<>();
        DoctorFollowPatientQueryDTO followDTO = new DoctorFollowPatientQueryDTO();
        followDTO.setDoctorId(params.getDoctorId());
        followDTO.setPatientIds(patientIds);
        List<DoctorFollowPatientVO> followList = doctorFollowPatientBizService.simpleList(followDTO);
        if (CollUtil.isNotEmpty(followList)) {
            followList.forEach(i -> followMap.put(i.getPatientId(), i));
        }

        // 个案管理
        Map<String, PatientDoctorRelationBO> crmMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientDoctorRelQueryClientDTO patientDoctorRelQueryClientDTO = new PatientDoctorRelQueryClientDTO();
            patientDoctorRelQueryClientDTO.setDoctorId(params.getDoctorId());
            patientDoctorRelQueryClientDTO.setSourceType("patient_label"); // PatientDoctorRelationSourceTypeEnum.PDRS_PatientLabel
            patientDoctorRelQueryClientDTO.setSourceObjIds(patientLabelIds);
            List<PatientDoctorRelationBO> crmList = patientDoctorRelationClientService.queryPatientDoctorRelList(patientDoctorRelQueryClientDTO);
            if (CollUtil.isNotEmpty(crmList)) {
                crmList.forEach(i -> crmMap.put(i.getPatientId() + "-" + i.getSourceObjId(), i));
            }
        }

        // family
        Map<String, List<MemberFamilyBO>> memberFamilyMap = new HashMap<>(); // 查询家庭成员
        if (CollUtil.isNotEmpty(patientIds)) {
            MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
            memberFamilyListClientDTO.setPatHisNos(patientIds);
            memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
            List<MemberFamilyBO> memberFamilyList = customerFamilyClientService.listByUser(memberFamilyListClientDTO);
            if (CollectionUtil.isNotEmpty(memberFamilyList)) {
                memberFamilyList.forEach(i -> {
                    List<MemberFamilyBO> members = memberFamilyMap.get(i.getPatHisNo());
                    if (null == members) {
                        members = new ArrayList<>();
                        memberFamilyMap.put(i.getPatHisNo(), members);
                    }
                    members.add(i);
                });
            }
        }

        Date now = DateUtil.date();
        page.getRecords().forEach(i -> {
            i.setAge(DataUtil.getAge(i.getIdCardNo(), i.getBirthday()));
            i.setIdCardNo(null);
            
            DataDwsPatientDataLastDateVO patientDataLastDate = patientDataLastDateMap.get(i.getPatientId());
            i.setLastDataInfo(patientDataLastDate);

            PatientVisitRecordClientBO visit = visitMap.get(i.getPatientId());
            i.setLastVisitInfo(visit);
            if (null != visit) {
                i.setVisitDate(visit.getVisitDate());
                i.setVisitDoctorId(visit.getDoctorCode());
                i.setVisitDoctorName(visit.getDoctorName());
                i.setVisitDeptId(visit.getDepartmentCodeStandard());
                i.setVisitDeptName(visit.getDepartmentNameStandard());
            }

            PatientInpatientRecordClientBO inPatient = inPatientMap.get(i.getPatientId());
            i.setLastInpatientInfo(inPatient);
            if (null != inPatient && (null == visit || visit.getVisitDate().before(inPatient.getInpatientDate()))) {
                i.setVisitDate(inPatient.getInpatientDate());
                i.setVisitDoctorId(inPatient.getDoctorCode());
                i.setVisitDoctorName(inPatient.getDoctorName());
                i.setVisitDeptId(inPatient.getAdmitDepartmentCodeStandard());
                i.setVisitDeptName(inPatient.getAdmitDepartmentNameStandard());
            }

            PatientDischargeRecordClientBO outPatient = outPatientMap.get(i.getPatientId());
            i.setLastDischargeInfo(outPatient);
            if (null != inPatient) {
                if (YN.YES.getCode().equals(inPatient.getIsOut()) 
                    || (null == outPatient && inPatient.getInpatientDate().after(DateUtil.offsetMonth(now, -1)))
                    || (null != outPatient && outPatient.getLeaveDate().before(inPatient.getInpatientDate()) && inPatient.getInpatientDate().after(DateUtil.offsetMonth(now, -1)))
                ) {
                    i.setInpatientStatus(YN.YES.getCode());
                }
            }

            i.setFollowStatus(YN.NO.getCode());
            DoctorFollowPatientVO follow = followMap.get(i.getPatientId());
            if (null != follow) {
                i.setFollowStatus(YN.YES.getCode());
            }
            
            i.setCrmStatus(YN.NO.getCode());
            PatientDoctorRelationBO crm = crmMap.get(i.getPatientId() + "-" + i.getPatientLabelId());
            if (null != crm) {
                i.setCrmStatus(YN.YES.getCode());
            }
            
            // family
            i.setMemberFamilyList(memberFamilyMap.get(i.getPatientId()));

            if (StrUtil.isBlank(i.getPhone())) {
                if (StrUtil.isNotBlank(i.getLinkPhone())) {
                    i.setPhone(i.getLinkPhone());
                } else if (StrUtil.isNotBlank(i.getHomePhone())) {
                    i.setPhone(i.getHomePhone());
                } else if (StrUtil.isNotBlank(i.getWorkPhone())) {
                    i.setPhone(i.getWorkPhone());
                }
            }
            if (StrUtil.isNotBlank(i.getPhone())) {
                if (StrUtil.isNotBlank(i.getLinkPhone()) && !i.getPhone().equals(i.getLinkPhone())) {
                    i.setSparePhone(i.getLinkPhone());
                } else if (StrUtil.isNotBlank(i.getHomePhone()) && !i.getPhone().equals(i.getHomePhone())) {
                    i.setSparePhone(i.getHomePhone());
                } else if (StrUtil.isNotBlank(i.getWorkPhone()) && !i.getPhone().equals(i.getWorkPhone())) {
                    i.setSparePhone(i.getWorkPhone());
                }
            }
        });

        return new PageData<>(page.getRecords(), page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public List<DoctorLabelPatientFullVO> patientList(PatientQueryDTO params, String deptId) {
        if (StrUtil.isBlank(deptId)) {
            return new ArrayList<>(0);
        }

        Set<String> labelIds = new HashSet<>();
        DoctorPatientLabelQueryDTO labelQueryPageDTO = new DoctorPatientLabelQueryDTO();
        labelQueryPageDTO.setLabelId(params.getLabelId());
        labelQueryPageDTO.setLabelIds(params.getLabelIds());
        List<LabelBO> labelList = this.labelList(labelQueryPageDTO, deptId);
        if (CollUtil.isEmpty(labelList)) {
            return new ArrayList<>(0);
        }
        labelList.forEach(i -> labelIds.add(i.getId()));
        if (CollUtil.isEmpty(labelIds)) {
            return new ArrayList<>(0);
        }

        List<DoctorLabelPatientFullVO> list = labelBizQueryMapper.listLabelPatient(new QueryWrapper<DoctorLabelPatientFullVO>()
            .in("b.label_id", labelIds)
            .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
            .in(CollUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
            .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
            .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "a.patient_gender_standard", params.getPatientGenderStandard())
            .eq(StrUtil.isNotBlank(params.getLabelId()), "b.label_id", params.getLabelId())
            .in(CollUtil.isNotEmpty(params.getLabelIds()), "b.label_id", params.getLabelIds())
        );

        return list;
    }

    @Override
    public TaskPatientVO patientDetail(PatientGetDTO params, String deptId) {
        // 患者信息
        TaskPatientGetDTO getDTO = new TaskPatientGetDTO();
        getDTO.setPatientId(params.getPatientId());
        TaskPatientVO patient = taskPatientQueryBizService.getOneSimple(getDTO);

        if (null == patient) {
            return null;
        }
        patient.setInpatientStatus(YN.NO.getCode());

        // 患者是否去世
        patient.setDeathStatus(YN.YES.getCode());
        DoctorDwdPatientGetDTO doctorDwdPatientGetDTO = new DoctorDwdPatientGetDTO();
        doctorDwdPatientGetDTO.setPatientId(patient.getPatientId());
        DoctorDwdPatientVO dwdPatient = doctorDwdPatientDataBizService.getOnePatient(doctorDwdPatientGetDTO);
        if (null != dwdPatient && YN.NO.getCode().equals(dwdPatient.getDeathStatus())) {
            patient.setDeathStatus(dwdPatient.getDeathStatus());
            patient.setDeathTime(dwdPatient.getDeathTime());
            if (StrUtil.isNotBlank(dwdPatient.getDeathTime())) {
                if (dwdPatient.getDeathTime().length() == 12) {
                    patient.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmm").toString());
                }
                if (dwdPatient.getDeathTime().length() == 14) {
                    patient.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmmss").toString());
                }
            }
        }
        
        // 查询患者最新数据信息
        DataDwsPatientDataLastDateVO patientDataLastDate = doctorDwdPatientDataBizService.getPatientDataLastDateInfo(params.getPatientId());
        patient.setLastDataInfo(patientDataLastDate);
        
        // 查询患者最新就诊数据
        PatientVisitRecordClientBO visit = doctorDwdPatientDataBizService.getOnePatientVisitLastDateInfo(params.getPatientId());
        patient.setLastVisitInfo(visit);
        if (null != visit) {
            patient.setVisitDate(visit.getVisitDate());
            patient.setVisitDoctorId(visit.getDoctorCode());
            patient.setVisitDoctorName(visit.getDoctorName());
            patient.setVisitDeptId(visit.getDepartmentCodeStandard());
            patient.setVisitDeptName(visit.getDepartmentNameStandard());
        }

        // 查询患者最新住院数据
        PatientInpatientRecordClientBO inPatient = doctorDwdPatientDataBizService.getOnePatientInpatientLastDateInfo(params.getPatientId());
        patient.setLastInpatientInfo(inPatient);
        if (null != inPatient && (null == visit || visit.getVisitDate().before(inPatient.getInpatientDate()))) {
            patient.setVisitDate(inPatient.getInpatientDate());
            patient.setVisitDoctorId(inPatient.getDoctorCode());
            patient.setVisitDoctorName(inPatient.getDoctorName());
            patient.setVisitDeptId(inPatient.getAdmitDepartmentCodeStandard());
            patient.setVisitDeptName(inPatient.getAdmitDepartmentNameStandard());
        }

        // 查询患者最新出院数据
        PatientDischargeRecordClientBO outPatient = doctorDwdPatientDataBizService.getOnePatientOutpatientLastDateInfo(params.getPatientId());
        patient.setLastDischargeInfo(outPatient);

        Date now = DateUtil.date();
        if (null != inPatient) {
            if (!YN.YES.getCode().equals(inPatient.getIsOut()) 
                || (null == outPatient && inPatient.getInpatientDate().after(DateUtil.offsetMonth(now, -1)))
                || (null != outPatient && outPatient.getLeaveDate().before(inPatient.getInpatientDate()) && inPatient.getInpatientDate().after(DateUtil.offsetMonth(now, -1)))
            ) {
                patient.setInpatientStatus(YN.YES.getCode());
            }
        }

        // 是否关注患者
        patient.setFollowStatus(YN.NO.getCode());
        DoctorFollowPatientQueryDTO followDTO = new DoctorFollowPatientQueryDTO();
        followDTO.setDoctorId(params.getDoctorId());
        followDTO.setPatientId(params.getPatientId());
        List<DoctorFollowPatientVO> followList = doctorFollowPatientBizService.simpleList(followDTO);
        if (CollUtil.isNotEmpty(followList)) {
            patient.setFollowStatus(YN.YES.getCode());
        }

        return patient;
    }
    
    @Override
    public List<LabelPatientStatisticClientBO> labelPatientCountList(DoctorPatientLabelQueryDTO params, String deptId) {
        List<LabelBO> labelList = this.labelList(params, deptId);
        if (CollUtil.isEmpty(labelList)) {
            return new ArrayList<>(0);
        }
        
        Set<String> labelIds = new HashSet<>();
        labelList.forEach(i -> labelIds.add(i.getId()));
        if (CollUtil.isEmpty(labelIds)) {
            return new ArrayList<>(0);
        }

        Map<String, Long> labelPatientNumMap = new HashMap<>();
        List<Map<String, Object>> labelPatientNumList = labelBizQueryMapper.listLabelPatientNum(new QueryWrapper<Void>()
            .in("a.label_id", labelIds)
            .ge(null != params.getStartDate(), "a.data_dt", params.getStartDate())
            .lt(null != params.getEndDate(), "a.data_dt", params.getEndDate())
        );
        if (CollUtil.isNotEmpty(labelPatientNumList)) {
            labelPatientNumList.forEach(i -> {
                String labelId = String.valueOf(i.get("labelId"));
                Long patientNum = Long.valueOf(String.valueOf(i.get("patientNum")));

                labelPatientNumMap.put(labelId, patientNum);
            });
        }

        List<LabelPatientStatisticClientBO> list = new ArrayList<>();
        labelList.forEach(i -> {
            LabelPatientStatisticClientBO bo = new LabelPatientStatisticClientBO();
            bo.setId(i.getId());
            bo.setName(i.getName());
            bo.setCategoryId(i.getCategoryId());
            bo.setStatus(i.getStatus());
            bo.setDiseaseIds(i.getDiseaseIds());
            bo.setDiseaseList(i.getDiseaseList());
            bo.setBizType(i.getBizType());

            Long patientNum = labelPatientNumMap.get(i.getId());
            if (null == patientNum) {
                patientNum = 0L;
            }
            bo.setPatientNum(patientNum);

            list.add(bo);
        });

        return list;
    }

    @Override
    public List<LabelPatientStatDimensionTypeClientBO> statLabelPatientCountList(DoctorPatientLabelQueryDTO params, String deptId) {
        List<LabelPatientStatisticClientBO> labelList = this.labelPatientCountList(params, deptId);
        if (CollUtil.isEmpty(labelList)) {
            return new ArrayList<>(0);
        }

        Map<String, List<LabelPatientStatisticClientBO>> labelMap = new HashMap<>();
        labelList.forEach(i -> {
            String key = "其他::其他";
            if (CollUtil.isNotEmpty(i.getDiseaseList())) {
                Map<String, String> diseaseMap = i.getDiseaseList().get(0);
                String diseaseId = diseaseMap.get("id");
                String diseaseName = diseaseMap.get("name");
                if (StrUtil.isBlank(diseaseName)) {
                    diseaseName = "其他";
                }
                key = diseaseId + "::" + diseaseName;
            }
            List<LabelPatientStatisticClientBO> l = labelMap.get(key);
            if (null == l) {
                l = new ArrayList<>();
                labelMap.put(key, l);
            }
            l.add(i);
        });

        List<LabelPatientStatDimensionTypeClientBO> voList = new ArrayList<>();
        labelMap.forEach((k, v) -> {
            String[] disease = k.split("::");

            LabelPatientStatDimensionTypeClientBO vo = new LabelPatientStatDimensionTypeClientBO();

            vo.setDiseaseId(disease[0]);
            vo.setDiseaseName(disease[1]);
            vo.setStatisticLabelList(v);

            voList.add(vo);
        });
        return voList;
    }

}
