package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollUtil;

import com.xxh.cloud.common.util.FollowUpRegexUtil;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdDischargeContentDetailVO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDiagnosisReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeContentClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientExamineReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientChangeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientReserveClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientLaboratoryReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineItemClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDiagnosisDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientExamineAndLaboratoryReportClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientRecipeMedicineClientDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInReserveBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.PatientFeeRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdLaboratoryReportItemVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientRegisterVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwsPatientDataLastDateVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdOperationApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdRegisterRecordVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DwdPatientDataBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.common.util.DataUtil;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@DS("wd")
@Slf4j
@Service
public class DoctorDwdPatientDataBizServiceImpl implements DoctorDwdPatientDataBizService {
    
    @Autowired
    private DwdPatientDataBizMapper dwdPatientDataBizMapper;

    // 查询患者信息
    @Override
    public List<DataDwdPatientVO> patientList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientList:params:>>>>>> {} ", JSON.toJSONString(params));
        
        List<DataDwdPatientVO> list = dwdPatientDataBizMapper.patientList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "gender", params.getPatientGenderStandard())
                .like(StrUtil.isNotBlank(params.getPatientName()), "name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPhone()), "phone", params.getPhone())
                .le(null != params.getStartAge(), "birthday", DataUtil.getBirthdayByAge(params.getStartAge())) // 年龄小的出生日期大
                .ge(null != params.getEndAge(), "birthday", DataUtil.getBirthdayByAge(params.getEndAge())) // 年龄大的出生日期小
                .eq(StrUtil.isNotBlank(params.getIdCardNo()), "card_no", params.getIdCardNo())
                .orderByAsc("patient_id")
        );
        return list;
    }

    @Override
    public List<DataDwdPatientRegisterVO> patientRegisterList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientRegisterList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientRegisterVO> list = dwdPatientDataBizMapper.patientRegisterList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .orderByAsc("a.id")
        );
        return list;
    }

    @Override
    public List<PatientVisitRecordClientBO> patientVisitList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientRegisterList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<PatientVisitRecordClientBO> list = dwdPatientDataBizMapper.patientVisitList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "a.department_code_standard", params.getDeptId())
                .in(CollectionUtil.isNotEmpty(params.getDeptIds()), "a.department_code_standard", params.getDeptIds())
                .ge(null != params.getStartDate(), "a.visit_date", params.getStartDate())
                .lt(null != params.getEndAge(), "a.visit_date", params.getEndDate())
                .orderByAsc("a.id")
        );
        return list;
    }

    @Override
    public List<DataDwdPatientInHospitalVO> patientInHospitalList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientInHospitalVO> list = dwdPatientDataBizMapper.patientInHospitalList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "a.admit_department_code_standard", params.getDeptId())
                .in(CollectionUtil.isNotEmpty(params.getDeptIds()), "a.admit_department_code_standard", params.getDeptIds())
                .ge(null != params.getStartDate(), "a.inpatient_date", params.getStartDate())
                .lt(null != params.getEndAge(), "a.inpatient_date", params.getEndDate())
                .orderByAsc("a.id")
        );
        return list;
    }

    @Override
    public List<PatientInpatientRecordClientBO> patientInHospitalClientList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.patientInHospitalClientList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "a.admit_department_code_standard", params.getDeptId())
                .in(CollectionUtil.isNotEmpty(params.getDeptIds()), "a.admit_department_code_standard", params.getDeptIds())
                .ge(null != params.getStartDate(), "a.inpatient_date", params.getStartDate())
                .lt(null != params.getEndAge(), "a.inpatient_date", params.getEndDate())
                .orderByAsc("a.id")
        );
        return list;
    }

    @Override
    public List<DataDwdPatientInHospitalApplyVO> patientInHospitalApplyList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalApplyList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientInHospitalApplyVO> list = dwdPatientDataBizMapper.patientInHospitalApplyList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .orderByAsc("a.id")
        );
        return list;
    }

    @Override
    public List<PatientInpatientChangeClientBO> patientInHospitalChangeList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<PatientInpatientChangeClientBO> list = dwdPatientDataBizMapper.patientInHospitalChangeList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "b.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "a.code", params.getDeptId())
                .in(CollectionUtil.isNotEmpty(params.getDeptIds()), "a.code", params.getDeptIds())
                .ge(null != params.getStartDate(), "a.change_time", params.getStartDate())
                .lt(null != params.getEndAge(), "a.change_time", params.getEndDate())
                .orderByAsc("a.inpatient_id")
        );
        return list;
    }


    // 同步患者列表
    @Override
    public List<DataDwdPatientVO> patientSyncList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientPage:params:>>>>>> {} ", JSON.toJSONString(params));
        
        List<DataDwdPatientVO> list = dwdPatientDataBizMapper.patientList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .gt(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .orderByAsc("patient_id")
                .last(" limit " + params.getRows())
        );
        return list;
    }

    // 同步挂号患者列表
    @Override
    public List<DataDwdPatientRegisterVO> patientRegisterSyncList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientRegisterList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientRegisterVO> list = dwdPatientDataBizMapper.patientRegisterList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .gt(StrUtil.isNotBlank(params.getId()), "a.id", params.getId())
                .orderByAsc("a.id")
                .last(" limit " + params.getRows())
        );
        return list;
    }

    // 同步住院患者列表
    @Override
    public List<DataDwdPatientInHospitalVO> patientInHospitalSyncList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientInHospitalVO> list = dwdPatientDataBizMapper.patientInHospitalList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .gt(StrUtil.isNotBlank(params.getId()), "a.id", params.getId())
                .orderByAsc("a.id")
                .last(" limit " + params.getRows())
        );
        return list;
    }

    // 同步住院申请患者列表
    @Override
    public List<DataDwdPatientInHospitalApplyVO> patientInHospitalApplySyncList(DataDwdPatientQueryDTO params) {
        log.info("PatientLostLabelBizServiceImpl.patientInHospitalApplyList:params:>>>>>> {} ", JSON.toJSONString(params));
        List<DataDwdPatientInHospitalApplyVO> list = dwdPatientDataBizMapper.patientInHospitalApplyList(new QueryWrapper<DataDwdPatientQueryDTO>()
                .gt(StrUtil.isNotBlank(params.getId()), "a.id", params.getId())
                .orderByAsc("a.id")
                .last(" limit " + params.getRows())
        );
        return list;
    }





    // 查询挂号的患者
    @Override
    public List<DoctorDwdRegisterRecordVO> queryRegisterList(Date startDate, Date endDate) {
        // 查询范围内最新的数据
        List<DoctorDwdRegisterRecordVO> lastList = dwdPatientDataBizMapper.queryLastRegisterList(new QueryWrapper<DoctorDwdRegisterRecordVO>()
                .ge("a.reg_date", DateUtil.date(startDate).toDateStr())
                .lt("a.reg_date", DateUtil.date(endDate))
        );
        if (CollectionUtil.isEmpty(lastList)) {
            return new ArrayList<>(0);
        }
        List<String> queryParams = lastList.stream().map(i -> i.getPatientId() + "-" + i.getDepartmentCodeStandard() + "-" + DateUtil.formatDateTime(i.getRegDate())).collect(Collectors.toList());
        // 查询最新的具体数据
        List<DoctorDwdRegisterRecordVO> list = dwdPatientDataBizMapper.queryRegisterList(new QueryWrapper<DoctorDwdRegisterRecordVO>()
                .in("CONCAT(a.patient_id, '-', a.department_code_standard, '-', a.reg_date)", queryParams)
        );
        // List<DoctorDwdRegisterRecordVO> list = dwdPatientDataBizMapper.queryRegisterList(new QueryWrapper<DoctorDwdRegisterRecordVO>()
        //         .in("CONCAT(b.main_person_id, '-', a.department_code_standard, '-', a.reg_date)", queryParams)
        // );
        log.info("PatientLostLabelBizServiceImpl.queryRegisterList:list: <<<<<< {} ", JSON.toJSONString(list));
        return list;
    }


    // 查询住院申请的患者
    @Override
    public List<DoctorDwdInReserveBO> queryInApplyList(String date) {
        // List<DoctorDwdInReserveBO> list= dwdPatientDataBizMapper.queryInApplyList(
        //         new QueryWrapper<DoctorDwdInReserveBO>()
        //                 .eq("DATE_FORMAT(apply_date,'%Y-%m-%d')",date)
        //                 //todo:状态
        // );
        List<DoctorDwdInReserveBO> list= dwdPatientDataBizMapper.queryInApplyList(
                new QueryWrapper<DoctorDwdInReserveBO>()
                        .eq("DATE_FORMAT(a.apply_date,'%Y-%m-%d')",date)
                        //todo:状态
        );
        return list;
    }



    @Override
    public List<DoctorDwdInRecordBO> queryInRecordList(Date startDate, Date endDate, Set<String> pids) {
        List<DoctorDwdInRecordBO> list = dwdPatientDataBizMapper.queryInRecordList(new QueryWrapper<DoctorDwdInRecordBO>()
                .in(CollUtil.isNotEmpty(pids),"a.patient_id", pids)
                .ge("a.inpatient_date",startDate)
                .le("a.inpatient_date", endDate)
                //todo:状态
        );
        // List<DoctorDwdInRecordBO> list = dwdPatientDataBizMapper.queryInRecordList(new QueryWrapper<DoctorDwdInRecordBO>()
        //         .in(CollUtil.isNotEmpty(pids),"b.main_person_id", pids)
        //         .ge("a.inpatient_date",startDate)
        //         .le("a.inpatient_date", endDate)
        //         //todo:状态
        // );
        return list;
    }






    //todo:del
//    @Override
//    public List<DoctorDwdInpatientReserveVO> queryInHospitalApplyList(Date startDate, Date endDate) {
//        // 查询住院申请信息
//        List<DoctorDwdInpatientReserveVO> lastList = dwdPatientDataBizMapper.queryInHospitalApplyList(new QueryWrapper<DoctorDwdInpatientReserveVO>()
//                .ge("a.apply_date", DateUtil.date(startDate).toDateStr())
//                .lt("a.apply_date", DateUtil.date(endDate))
//        );
//        if (CollectionUtil.isEmpty(lastList)) {
//            return new ArrayList<>(0);
//        }
//        List<String> queryParams = lastList.stream().map(i -> i.getPatientId() + "-" + i.getApplyDepartmentCodeStandard() + "-" + DateUtil.formatDateTime(i.getApplyDate())).collect(Collectors.toList());
//        // 查询最新的具体数据
//        List<DoctorDwdInpatientReserveVO> list = dwdPatientDataBizMapper.queryInHospitalApplyList(new QueryWrapper<DoctorDwdInpatientReserveVO>()
//                .in("CONCAT(a.patient_id, '-', a.apply_department_code_standard, '-', a.apply_date)", queryParams)
//        );
//        log.info("PatientLostLabelBizServiceImpl.queryInHospitalApplyList:list: <<<<<< {} ", JSON.toJSONString(list));
//        return list;
//    }


    //todo:del wjj
//    // 查询住院的患者
//    @Override
//    public List<DataDwdPatientInHospitalVO> queryInHospitalList(List<String> deptIds, Date startDate, Date endDate) {
//        // 查询住院申请信息
//        List<DataDwdPatientInHospitalVO> list = dwdPatientDataBizMapper.patientInHospitalList(new QueryWrapper<DataDwdPatientQueryDTO>()
//                .in("admit_department_code_standard", deptIds)
//                .ge("inpatient_date", DateUtil.date(startDate).toDateStr())
//                .lt("inpatient_date", DateUtil.date(endDate))
//        );
//        log.info("PatientLostLabelBizServiceImpl.queryInHospitalList:list: <<<<<< {} ", JSON.toJSONString(list));
//        return list;
//    }

    // 查询手术申请的患者
    @Override
    public List<DoctorDwdOperationApplyVO> queryOperationApplyList(Date startDate, Date endDate) {
        List<DoctorDwdOperationApplyVO> lastList = dwdPatientDataBizMapper.queryOperationApplyList(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .ge("a.apply_time", DateUtil.date(startDate).toDateStr())
                .lt("a.apply_time", DateUtil.date(endDate))
        );
        if (CollectionUtil.isEmpty(lastList)) {
            return new ArrayList<>(0);
        }
        List<String> queryParams = lastList.stream().map(i -> i.getPatientId() + "-" + i.getDepartmentCodeStandard() + "-" + DateUtil.formatDateTime(i.getApplyTime())).collect(Collectors.toList());
        List<DoctorDwdOperationApplyVO> list = dwdPatientDataBizMapper.queryOperationApplyList(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .in("CONCAT(a.patient_id, '-', a.department_code_standard, '-', a.apply_time)", queryParams)
        );
        // List<DoctorDwdOperationApplyVO> list = dwdPatientDataBizMapper.queryOperationApplyList(new QueryWrapper<DoctorDwdOperationApplyVO>()
        //         .in("CONCAT(c.main_person_id, '-', a.department_code_standard, '-', a.apply_time)", queryParams)
        // );
        log.info("PatientLostLabelBizServiceImpl.queryOperationApplyList:list: <<<<<< {} ", JSON.toJSONString(list));
        return list;
    }

    // 实时统计门诊/急诊/住院患者总数量
    @Override
    public List<String> queryVisitPatientCount(String deptId) {
        // 查询门诊/急诊患者id
        List<String> visitPatientIds = dwdPatientDataBizMapper.queryVisitRecordPatientIdList(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .eq("a.department_code_standard", deptId)
        );
        // 查询住院患者id
        List<String> inHospitalPatientIds = dwdPatientDataBizMapper.queryInHospitalRecordPatientIdList(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .eq("a.admit_department_code_standard", deptId)
        );

        visitPatientIds.addAll(inHospitalPatientIds);
        Set<String> patientIds = new HashSet<>(visitPatientIds);

        return new ArrayList<>(patientIds);
    }

    @Override
    public Long queryApplyCount(String deptId) {
        // 查询挂号数量
        Long registerCount = dwdPatientDataBizMapper.queryRegisterApplyCount(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .eq("a.department_code_standard", deptId)
                // .ge(StrUtil.isNotBlank(startDate), "a.reg_date", startDate)  // 统计某时间段内的流失比率
                // .lt(StrUtil.isNotBlank(endDate), "a.reg_date", endDate)
        );
        // 查询住院申请数量
        Long inHospitalApplyCount = dwdPatientDataBizMapper.queryInHospitalApplyCount(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .eq("a.admit_department_code_standard", deptId)
                // .ge(StrUtil.isNotBlank(startDate), "a.apply_date", startDate)
                // .lt(StrUtil.isNotBlank(endDate), "a.apply_date", endDate)
        );
        // 查询手术申请数量
        Long operationApplyCount = dwdPatientDataBizMapper.queryOperationApplyCount(new QueryWrapper<DoctorDwdOperationApplyVO>()
                .eq("a.department_code_standard", deptId)
                // .ge(StrUtil.isNotBlank(startDate), "a.apply_time", startDate)
                // .lt(StrUtil.isNotBlank(endDate), "a.apply_time", endDate)
        );

        return registerCount + inHospitalApplyCount + operationApplyCount;
    }

    @Override
    public List<DoctorDwdPatientVO> queryPatientList(DoctorDwdPatientQueryDTO params) {
        List<DoctorDwdPatientVO> list = dwdPatientDataBizMapper.queryPatientList(new QueryWrapper<DoctorDwdPatientVO>()
                // .eq(StrUtil.isNotBlank(params.getPatientId()), "main_person_id", params.getPatientId())
                // .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "main_person_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .like(StrUtil.isNotBlank(params.getPatientName()), "patient_name", params.getPatientName())
                .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "patient_gender_standard", params.getPatientGenderStandard())
                .like(StrUtil.isNotBlank(params.getPhone()), "phone", params.getPhone())
                .le(null != params.getStartAge(), "birthday", DataUtil.getBirthdayByAge(params.getStartAge())) // 年龄小的出生日期大
                .ge(null != params.getEndAge(), "birthday", DataUtil.getBirthdayByAge(params.getEndAge())) // 年龄大的出生日期小
                .eq(StrUtil.isNotBlank(params.getIdCardNo()), "card_no", params.getIdCardNo())
        );

        // 年龄计算
        if (CollectionUtil.isNotEmpty(list)) {
            list.forEach(i -> {
                i.setPatientAge(DataUtil.getAge(i.getCardNo(), i.getBirthday()));
                i.setCardNo(null);
            });
        }
        return list;
    }

    @Override
    public DoctorDwdPatientVO getOnePatient(DoctorDwdPatientGetDTO params) {
        DoctorDwdPatientVO vo = dwdPatientDataBizMapper.getOnePatient(new QueryWrapper<DoctorDwdPatientVO>()
        // .eq(StrUtil.isNotBlank(params.getPatientId()), "main_person_id", params.getPatientId())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
        );

        // 年龄计算
        if (null != vo) {
            vo.setPatientAge(DataUtil.getAge(vo.getCardNo(), vo.getBirthday()));
            vo.setCardNo(null);
        }

        return vo;
    }

    @Override
    public Map<String, List<DataDwdLaboratoryReportItemVO>> laboratoryTrend(DataDwdLaboratoryQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        ValidatorUtils.validateEntity(params);

        Map<String, List<DataDwdLaboratoryReportItemVO>> map = new HashMap<>();
        
        List<DataDwdLaboratoryReportItemVO> list = dwdPatientDataBizMapper.getLaboratoryReportItemList(new QueryWrapper<DataDwdLaboratoryQueryDTO>()
                .eq("a.patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getProjectIdRaw()), "b.project_id_raw", params.getProjectIdRaw())
                .in(CollectionUtil.isNotEmpty(params.getProjectIdRaws()), "b.project_id_raw", params.getProjectIdRaws())
                .eq(StrUtil.isNotBlank(params.getProjectIdStandard()), "b.project_id_standard", params.getProjectIdStandard())
                .in(CollectionUtil.isNotEmpty(params.getProjectIdStandards()), "b.project_id_standard", params.getProjectIdStandards())
                .ge(null != params.getStartDate(), "a.reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "a.reports_time", params.getEndDate())
                .orderByDesc("a.reports_time")
        );
        // List<DataDwdLaboratoryReportItemVO> list = dwdPatientDataBizMapper.getLaboratoryReportItemList(new QueryWrapper<DataDwdLaboratoryQueryDTO>()
        //         .eq("b.main_person_id", params.getPatientId())
        //         .eq(StrUtil.isNotBlank(params.getProjectIdStandard()), "a.project_id_standard", params.getProjectIdStandard())
        //         .in(CollectionUtil.isNotEmpty(params.getProjectIdStandards()), "a.project_id_standard", params.getProjectIdStandards())
        //         .ge(null != params.getStartDate(), "a.reports_time", params.getStartDate())
        //         .lt(null != params.getEndDate(), "a.reports_time", params.getEndDate())
        // );

        if (CollectionUtil.isNotEmpty(list)) {
            list.forEach(i -> {
                List<DataDwdLaboratoryReportItemVO> vos = map.get(i.getProjectIdStandard());
                if (null == vos) {
                    vos = new ArrayList<>();
                    // map.put(i.getProjectIdStandard(), vos);
                    map.put(i.getProjectIdRaw(), vos);
                }
                vos.add(i);
            });
        }
        return map;
    }

    @Override
    public DataDwsPatientDataLastDateVO getPatientDataLastDateInfo(String patientId) {
        DataDwsPatientDataLastDateVO entity = dwdPatientDataBizMapper.getPatientDataLastDateInfo(new QueryWrapper<String>()
                .eq("patient_id", patientId)
        );
        // DataDwsPatientDataLastDateVO entity = dwdPatientDataBizMapper.getPatientDataLastDateInfo(new QueryWrapper<String>()
        //         .eq("b.main_person_id", patientId)
        // );

        if (null == entity) {
            return entity;
        }

        String content = entity.getContent();
        if (StrUtil.isNotBlank(content)) {
            String[] contentSplit = content.split("__");
            entity.setContentDate(contentSplit[0]);
            entity.setContentMap(JSONObject.parseObject(contentSplit[1], new TypeReference<Map<String, String>>(){}));
            entity.setContent(null);
        }
        return entity;
    }

    @Override
    public List<DataDwsPatientDataLastDateVO> listPatientDataLastDateInfo(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        List<DataDwsPatientDataLastDateVO> list = dwdPatientDataBizMapper.listPatientDataLastDateInfo(new QueryWrapper<String>()
                .in("patient_id", patientIds)
        );
        // List<DataDwsPatientDataLastDateVO> list = dwdPatientDataBizMapper.listPatientDataLastDateInfo(new QueryWrapper<String>()
        //         .in("b.main_person_id", patientIds)
        // );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        list.forEach(entity -> {
            String content = entity.getContent();
            if (StrUtil.isNotBlank(content)) {
                String[] contentSplit = content.split("__");
                entity.setContentDate(contentSplit[0]);
                entity.setContentMap(JSONObject.parseObject(contentSplit[1], new TypeReference<Map<String, String>>(){}));
                entity.setContent(null);
            }
        });

        return list;
    }

    @Override
    public Map<String, DataDwsPatientDataLastDateVO> mapPatientDataLastDateInfo(List<String> patientIds) {
        List<DataDwsPatientDataLastDateVO> list = listPatientDataLastDateInfo(patientIds);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }

        Map<String, DataDwsPatientDataLastDateVO> map = new HashMap<>();
        list.forEach(i -> {
            DataDwsPatientDataLastDateVO bo = map.get(i.getPatientId());
            if (null == bo || bo.getLastOpDate().compareTo(i.getLastOpDate()) < 0) {
                map.put(i.getPatientId(), i);
            }
        });

        return map;
    }

    @Override
    public List<PatientExamineReportClientBO> listExamineReportInfo(PatientExamineAndLaboratoryReportClientDTO params) {
        return dwdPatientDataBizMapper.listExamineReportInfo(new QueryWrapper<PatientExamineReportClientBO>()
                .ge(StrUtil.isNotBlank(params.getStartDate()), "reports_time", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "reports_time", params.getEndDate())
        );
    }

    @Override
    public List<PatientLaboratoryReportClientBO> listLaboratoryReportInfo(PatientExamineAndLaboratoryReportClientDTO params) {
        return dwdPatientDataBizMapper.listLaboratoryReportInfo(new QueryWrapper<PatientLaboratoryReportClientBO>()
                .ge(StrUtil.isNotBlank(params.getStartDate()), "reports_time", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "reports_time", params.getEndDate())
        );
    }

    @Override
    public List<PatientRecipeMedicineClientBO> listRecipeMedicineInfo(PatientRecipeMedicineClientDTO params) {
        List<PatientRecipeMedicineClientBO> list = dwdPatientDataBizMapper.listRecipeMedicineInfo(new QueryWrapper<PatientRecipeMedicineClientBO>()
                .eq(StrUtil.isNotBlank(params.getScene()), "scene_type_standard", params.getScene())
                .in(CollectionUtil.isNotEmpty(params.getScenes()), "scene_type_standard", params.getScenes())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "pay_time", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "pay_time", params.getEndDate())
        );

        if (CollectionUtil.isNotEmpty(list)) {
            List<String> recipeMedicineIds = list.stream().map(PatientRecipeMedicineClientBO::getId).collect(Collectors.toList());
            List<PatientRecipeMedicineItemClientBO> voList = dwdPatientDataBizMapper.listRecipeMedicineItemInfo(new QueryWrapper<PatientRecipeMedicineItemClientBO>()
                    .in("recipe_medicine_id", recipeMedicineIds)
            );
            Map<String, List<PatientRecipeMedicineItemClientBO>> voMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(voList)) {
                voList.forEach(i -> {
                    List<PatientRecipeMedicineItemClientBO> vos = voMap.get(i.getRecipeMedicineId());
                    if (null == vos) {
                        vos = new ArrayList<>();
                        voMap.put(i.getRecipeMedicineId(), vos);
                    }
                    vos.add(i);
                });
            }
            if (CollectionUtil.isNotEmpty(voMap)) {
                list.forEach(i -> i.setPatientRecipeMedicineItemList(voMap.get(i.getId())));
            }
        }
        return list;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatient(PatientDataClientDTO params) {
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.listPatientInpatientInfo(new QueryWrapper<PatientInpatientRecordClientBO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "inpatient_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "inpatient_date", params.getEndDate())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> inpatientIds = new ArrayList<>();
        list.forEach(i -> inpatientIds.add(i.getId()));

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .in("inpatient_id", inpatientIds)
        );
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
            if (CollUtil.isNotEmpty(changes)) {
                PatientInpatientChangeClientBO dept = null; // 最后变动科室
                PatientInpatientChangeClientBO bed = null; // 最后变动床位
                for(PatientInpatientChangeClientBO change : changes) {
                    if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        dept = change;
                    }
                    if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        bed = change;
                    }
                }
                
                String inpatientDate = DateUtil.format(i.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
                // 最后变动科室
                if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setAdmitDepartmentNameRaw(dept.getName());
                    i.setAdmitDepartmentNameStandard(dept.getName());
                    i.setAdmitDepartmentCodeRaw(dept.getCode());
                    i.setAdmitDepartmentCodeStandard(dept.getCode());
                }
                // 最后变动床位
                if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setBedNumber(bed.getCode());
                }
            }
        });
        return list;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientInfo(PatientDataClientDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new ArrayList<>(0);
        }
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.listPatientInpatientLastDateInfo(new QueryWrapper<PatientInpatientRecordClientBO>()
                .in("patient_id", params.getPatientIds())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "inpatient_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "inpatient_date", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "admit_department_code_standard", params.getDeptId())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> inpatientIds = new ArrayList<>();
        list.forEach(i -> inpatientIds.add(i.getId()));

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .in("inpatient_id", inpatientIds)
        );
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
            if (CollUtil.isNotEmpty(changes)) {
                PatientInpatientChangeClientBO dept = null; // 最后变动科室
                PatientInpatientChangeClientBO bed = null; // 最后变动床位
                for(PatientInpatientChangeClientBO change : changes) {
                    if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        dept = change;
                    }
                    if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        bed = change;
                    }
                }
                
                String inpatientDate = DateUtil.format(i.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
                // 最后变动科室
                if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setAdmitDepartmentNameRaw(dept.getName());
                    i.setAdmitDepartmentNameStandard(dept.getName());
                    i.setAdmitDepartmentCodeRaw(dept.getCode());
                    i.setAdmitDepartmentCodeStandard(dept.getCode());
                }
                // 最后变动床位
                if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setBedNumber(bed.getCode());
                }
            }
        });
        return list;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateInfo(PatientLastDataQueryDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new ArrayList<>(0);
        }
        if (null == params.getStartDate()) {
            params.setStartDate(DateUtil.offsetDay(DateUtil.date(), -90));
        }
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.listPatientInpatientLastDateInfo(new QueryWrapper<PatientInpatientRecordClientBO>()
                .in("patient_id", params.getPatientIds())
                .ge(null != params.getStartDate(), "inpatient_date", params.getStartDate()) // 只查询最近三个月的数据
                .lt(null != params.getEndDate(), "inpatient_date", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "admit_department_code_standard", params.getDeptId())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> inpatientIds = new ArrayList<>();
        list.forEach(i -> inpatientIds.add(i.getId()));

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .in("inpatient_id", inpatientIds)
        );
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
            if (CollUtil.isNotEmpty(changes)) {
                PatientInpatientChangeClientBO dept = null; // 最后变动科室
                PatientInpatientChangeClientBO bed = null; // 最后变动床位
                for(PatientInpatientChangeClientBO change : changes) {
                    if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        dept = change;
                    }
                    if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        bed = change;
                    }
                }
                
                String inpatientDate = DateUtil.format(i.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
                // 最后变动科室
                if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setAdmitDepartmentNameRaw(dept.getName());
                    i.setAdmitDepartmentNameStandard(dept.getName());
                    i.setAdmitDepartmentCodeRaw(dept.getCode());
                    i.setAdmitDepartmentCodeStandard(dept.getCode());
                }
                // 最后变动床位
                if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setBedNumber(bed.getCode());
                }
            }
        });
        return list;
    }

    @Override
    public PatientInpatientRecordClientBO getOnePatientInpatientLastDateInfo(String patientId) {
        if (StrUtil.isBlank(patientId)) {
            return null;
        }
        PatientInpatientRecordClientBO inpatientRecord = dwdPatientDataBizMapper.getOnePatientInpatientLastDateInfo(new QueryWrapper<PatientInpatientRecordClientBO>()
                .eq("patient_id", patientId)
                .gt("inpatient_date", DateUtil.offsetDay(DateUtil.date(), -90).toDateStr()) // 只查询最近三个月的数据
        );

        if (null == inpatientRecord) {
            return null;
        }

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .eq("inpatient_id", inpatientRecord.getId())
        );

        if (CollUtil.isNotEmpty(changeList)) {
            PatientInpatientChangeClientBO dept = null; // 最后变动科室
            PatientInpatientChangeClientBO bed = null; // 最后变动床位
            for(PatientInpatientChangeClientBO change : changeList) {
                if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                    dept = change;
                }
                if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                    bed = change;
                }

                if (StrUtil.isNotBlank(change.getChangeType()) && change.getChangeType().contains("CY")) {
                    inpatientRecord.setIsOut(YN.YES.getCode());
                }
            }
            
            String inpatientDate = DateUtil.format(inpatientRecord.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
            // 最后变动科室
            if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                inpatientRecord.setAdmitDepartmentNameRaw(dept.getName());
                inpatientRecord.setAdmitDepartmentNameStandard(dept.getName());
                inpatientRecord.setAdmitDepartmentCodeRaw(dept.getCode());
                inpatientRecord.setAdmitDepartmentCodeStandard(dept.getCode());
            }
            // 最后变动床位
            if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                inpatientRecord.setBedNumber(bed.getCode());
            }
        }
        return inpatientRecord;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndNotOut(PatientLastDataQueryDTO params) {
        if (CollUtil.isEmpty(params.getPatientIds()) && StrUtil.isEmpty(params.getPatientId())) {
            return new ArrayList<>(0);
        }
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.listPatientInpatientLastDateAndNotOut(new QueryWrapper<PatientInpatientRecordClientBO>()
                .eq(StrUtil.isNotEmpty(params.getPatientId()),"a.patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()),"a.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "a.visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "a.visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "a.inpatient_date", params.getStartDate()) // 只查询最近三个月的数据
                .lt(null != params.getEndDate(), "a.inpatient_date", params.getEndDate())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> inpatientIds = new ArrayList<>();
        list.forEach(i -> inpatientIds.add(i.getId()));

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .in("inpatient_id", inpatientIds)
        );
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
            if (CollUtil.isNotEmpty(changes)) {
                PatientInpatientChangeClientBO dept = null; // 最后变动科室
                PatientInpatientChangeClientBO bed = null; // 最后变动床位
                for(PatientInpatientChangeClientBO change : changes) {
                    if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        dept = change;
                    }
                    if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        bed = change;
                    }

                    if (StrUtil.isNotBlank(change.getChangeType()) && change.getChangeType().contains("CY")) {
                        i.setIsOut(YN.YES.getCode());
                    }
                }
                
                String inpatientDate = DateUtil.format(i.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
                // 最后变动科室
                if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setAdmitDepartmentNameRaw(dept.getName());
                    i.setAdmitDepartmentNameStandard(dept.getName());
                    i.setAdmitDepartmentCodeRaw(dept.getCode());
                    i.setAdmitDepartmentCodeStandard(dept.getCode());
                }
                // 最后变动床位
                if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setBedNumber(bed.getCode());
                }
            }
        });
        return list;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndOut(PatientLastDataQueryDTO params) {
        if (CollUtil.isEmpty(params.getPatientIds()) && StrUtil.isEmpty(params.getPatientId())) {
            return new ArrayList<>(0);
        }
        List<PatientInpatientRecordClientBO> list = dwdPatientDataBizMapper.listPatientInpatientLastDateAndOut(new QueryWrapper<PatientInpatientRecordClientBO>()
                .eq(StrUtil.isNotEmpty(params.getPatientId()),"a.patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()),"a.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "a.visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "a.visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "a.inpatient_date", params.getStartDate())
                .lt(null != params.getEndDate(), "a.inpatient_date", params.getEndDate())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> inpatientIds = new ArrayList<>();
        list.forEach(i -> inpatientIds.add(i.getId()));

        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
                .in("inpatient_id", inpatientIds)
        );
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
            if (CollUtil.isNotEmpty(changes)) {
                PatientInpatientChangeClientBO dept = null; // 最后变动科室
                PatientInpatientChangeClientBO bed = null; // 最后变动床位
                for(PatientInpatientChangeClientBO change : changes) {
                    if ("ZKZRKS".equals(change.getChangeType()) && (dept == null || dept.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        dept = change;
                    }
                    if ("ZKZRCW".equals(change.getChangeType()) && (bed == null || bed.getChangeTime().compareTo(change.getChangeTime()) < 0)) {
                        bed = change;
                    }

                    if (StrUtil.isNotBlank(change.getChangeType()) && change.getChangeType().contains("CY")) {
                        i.setIsOut(YN.YES.getCode());
                        i.setOutDate(DateUtil.parseDateTime(change.getChangeTime()));
                    }
                }
                
                String inpatientDate = DateUtil.format(i.getInpatientDate(), "yyyy-MM-dd HH:mm:ss").toString(); // 变动时间大于住院记录中的时间才以变动数据为准
                // 最后变动科室
                if (null != dept && dept.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setAdmitDepartmentNameRaw(dept.getName());
                    i.setAdmitDepartmentNameStandard(dept.getName());
                    i.setAdmitDepartmentCodeRaw(dept.getCode());
                    i.setAdmitDepartmentCodeStandard(dept.getCode());
                }
                // 最后变动床位
                if (null != bed && bed.getChangeTime().compareTo(inpatientDate) > 0) {
                    i.setBedNumber(bed.getCode());
                }
            }
        });
        return list;
    }

    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientInfo(PatientDataClientDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new ArrayList<>(0);
        }
        List<PatientDischargeRecordClientBO> list = dwdPatientDataBizMapper.listPatientOutpatientLastDateInfo(new QueryWrapper<PatientDischargeRecordClientBO>()
                .in("patient_id", params.getPatientIds())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "leave_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "leave_date", params.getEndDate())
        );

        return list;
    }

    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateInfo(PatientLastDataQueryDTO params) {
        List<PatientDischargeRecordClientBO> list = dwdPatientDataBizMapper.listPatientOutpatientLastDateInfo(new QueryWrapper<PatientDischargeRecordClientBO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .ge(null != params.getStartDate(), "leave_date", params.getStartDate())
                .lt(null != params.getEndDate(), "leave_date", params.getEndDate())
        );

        if (CollUtil.isNotEmpty(list)) {
            List<String> ids = new ArrayList<>();
            list.forEach(i -> ids.add(i.getId()));

            List<PatientDischargeContentClientBO> contentList = dwdPatientDataBizMapper.listPatientOutpatientContentInfo(new QueryWrapper<PatientDischargeContentClientBO>()
                    .in("id", ids)
            );
            if (CollUtil.isNotEmpty(contentList)) {
                Map<String, PatientDischargeContentClientBO> contentMap = new HashMap<>();
                contentList.forEach(i -> contentMap.put(i.getId(), i));

                list.forEach(i -> {
                    PatientDischargeContentClientBO content = contentMap.get(i.getId());
                    if (null != content) {
                        i.setContent(content);

                        if (null != content && StrUtil.isNotBlank(content.getContentStandard())) {
                            DwdDischargeContentDetailVO contentDetailVO = JSONObject.parseObject(content.getContentStandard(), DwdDischargeContentDetailVO.class);
                            if (null != contentDetailVO && CollUtil.isNotEmpty(contentDetailVO.getValueList())) {
                                contentDetailVO.getValueList().forEach(j -> {
                                    // if ("SFSJ".equals(j.getName())) { // 随访时间
                                    //     content.setFollowupDateRaw(j.getContent());
                                    // }
                                    
                                    // 5.0数据结构
                                    String idValue = j.get("id");
                                    String tablenameValue = j.get("tablename");
                                    String valueValue = j.get("value");
                                    if ("SFSJ".equals(idValue)) { // 随访时间
                                        content.setFollowupDateRaw(tablenameValue);
                                        if (StrUtil.isBlank(tablenameValue) && StrUtil.isNotBlank(valueValue)) {
                                            content.setFollowupDateRaw(valueValue);
                                        }
                                    }

                                    // 3.0数据结构
                                    String nameValue = j.get("name");
                                    String contentValue = j.get("content");
                                    if ("SFSJ".equals(nameValue)) { // 随访时间
                                        content.setFollowupDateRaw(contentValue);
                                    }
                                });
                                
                                if (StrUtil.isNotBlank(content.getFollowupDateRaw())) {
                                    log.info("出院随访>>>>：{}，{}", i.getVisitId(), content.getFollowupDateRaw());
                                    content.setFollowupDateStandard(FollowUpRegexUtil.getFollowupDateStandard(i.getLeaveDate(), content.getFollowupDateRaw()));
                                }
                            }
                        }
                    }
                });
            }
        }

        return list;
    }

    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateInfo(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        List<PatientDischargeRecordClientBO> list = dwdPatientDataBizMapper.listPatientOutpatientLastDateInfo(new QueryWrapper<PatientDischargeRecordClientBO>()
                .in("patient_id", patientIds)
                .gt("leave_date", DateUtil.offsetDay(DateUtil.date(), -90).toDateStr()) // 只查询最近三个月的数据
        );

        if (CollUtil.isNotEmpty(list)) {
            List<String> ids = new ArrayList<>();
            list.forEach(i -> ids.add(i.getId()));

            List<PatientDischargeContentClientBO> contentList = dwdPatientDataBizMapper.listPatientOutpatientContentInfo(new QueryWrapper<PatientDischargeContentClientBO>()
                    .in("id", ids)
            );
            if (CollUtil.isNotEmpty(contentList)) {
                Map<String, PatientDischargeContentClientBO> contentMap = new HashMap<>();
                contentList.forEach(i -> contentMap.put(i.getId(), i));

                list.forEach(i -> {
                    PatientDischargeContentClientBO content = contentMap.get(i.getId());
                    if (null != content) {
                        i.setContent(content);

                        if (null != content && StrUtil.isNotBlank(content.getContentStandard())) {
                            DwdDischargeContentDetailVO contentDetailVO = JSONObject.parseObject(content.getContentStandard(), DwdDischargeContentDetailVO.class);
                            if (null != contentDetailVO && CollUtil.isNotEmpty(contentDetailVO.getValueList())) {
                                contentDetailVO.getValueList().forEach(j -> {
                                    // if ("SFSJ".equals(j.getName())) { // 随访时间
                                    //     content.setFollowupDateRaw(j.getContent());
                                    // }

                                    // 5.0数据结构
                                    String idValue = j.get("id");
                                    String tablenameValue = j.get("tablename");
                                    String valueValue = j.get("value");
                                    if ("SFSJ".equals(idValue)) { // 随访时间
                                        content.setFollowupDateRaw(tablenameValue);
                                        if (StrUtil.isBlank(tablenameValue) && StrUtil.isNotBlank(valueValue)) {
                                            content.setFollowupDateRaw(valueValue);
                                        }
                                    }

                                    // 3.0数据结构
                                    String nameValue = j.get("name");
                                    String contentValue = j.get("content");
                                    if ("SFSJ".equals(nameValue)) { // 随访时间
                                        content.setFollowupDateRaw(contentValue);
                                    }
                                });
                                
                                if (StrUtil.isNotBlank(content.getFollowupDateRaw())) {
                                    content.setFollowupDateStandard(FollowUpRegexUtil.getFollowupDateStandard(i.getLeaveDate(), content.getFollowupDateRaw()));
                                }
                            }
                        }
                    }
                });
            }
        }

        return list;
    }

    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateWitiSFSJInfo(PatientLastDataQueryDTO params) {
        List<PatientDischargeRecordClientBO> list = dwdPatientDataBizMapper.listPatientOutpatientLastDateInfo(new QueryWrapper<PatientDischargeRecordClientBO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "leave_date", params.getStartDate())
                .lt(null != params.getEndDate(), "leave_date", params.getEndDate())
        );

        List<PatientDischargeRecordClientBO> resultList = new ArrayList<>(); // 只返回有随访时间的数据
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> ids = new ArrayList<>();
        list.forEach(i -> ids.add(i.getId()));

        // 出院小结
        Map<String, PatientDischargeContentClientBO> contentMap = new HashMap<>();
        List<PatientDischargeContentClientBO> contentList = dwdPatientDataBizMapper.listPatientOutpatientContentInfo(new QueryWrapper<PatientDischargeContentClientBO>()
                .in("id", ids)
        );
        if (CollUtil.isNotEmpty(contentList)) {
            contentList.forEach(i -> contentMap.put(i.getId(), i));
        }
        
        // 科室变动
        Map<String, List<PatientInpatientChangeClientBO>> changeMap = new HashMap<>();
        List<PatientInpatientChangeClientBO> changeList = dwdPatientDataBizMapper.listPatientInpatientChangeLastDateInfo(new QueryWrapper<PatientInpatientChangeClientBO>()
            .in("inpatient_id", ids)
        );
        changeList.forEach(i -> {
            List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getInpatientId());
            if (null == changes) {
                changes = new ArrayList<>();
                changeMap.put(i.getInpatientId(), changes);
            }
            changes.add(i);
        });

        list.forEach(i -> {
            PatientDischargeContentClientBO content = contentMap.get(i.getId());
            if (null == content) {
                return;
            }

            i.setContent(content);

            if (null != content && StrUtil.isNotBlank(content.getContentStandard())) {
                DwdDischargeContentDetailVO contentDetailVO = JSONObject.parseObject(content.getContentStandard(), DwdDischargeContentDetailVO.class);
                if (null != contentDetailVO && CollUtil.isNotEmpty(contentDetailVO.getValueList())) {
                    contentDetailVO.getValueList().forEach(j -> {
                        // if ("SFSJ".equals(j.getName())) { // 随访时间
                        //     content.setFollowupDateRaw(j.getContent());

                        //     resultList.add(i);
                        // }

                        // 5.0数据结构
                        String idValue = j.get("id");
                        String tablenameValue = j.get("tablename");
                        String valueValue = j.get("value");
                        if ("SFSJ".equals(idValue)) { // 随访时间
                            content.setFollowupDateRaw(tablenameValue);
                            if (StrUtil.isBlank(tablenameValue) && StrUtil.isNotBlank(valueValue)) {
                                content.setFollowupDateRaw(valueValue);
                            }

                            resultList.add(i);
                        }

                        // 3.0数据结构
                        String nameValue = j.get("name");
                        String contentValue = j.get("content");
                        if ("SFSJ".equals(nameValue)) { // 随访时间
                            content.setFollowupDateRaw(contentValue);

                            resultList.add(i);
                        }
                    });
                    
                    if (StrUtil.isNotBlank(content.getFollowupDateRaw())) {
                        log.info("出院随访>>>>：{}，{}", i.getVisitId(), content.getFollowupDateRaw());
                        content.setFollowupDateStandard(FollowUpRegexUtil.getFollowupDateStandard(i.getLeaveDate(), content.getFollowupDateRaw()));
                    }
                }
            }

            if (StrUtil.isBlank(i.getOutDepartmentCodeStandard())) {
                List<PatientInpatientChangeClientBO> changes = changeMap.get(i.getId());
                if (CollUtil.isNotEmpty(changes)) {
                    for(PatientInpatientChangeClientBO change : changes) {
                        if ("CYKS".equals(change.getChangeType())) {
                            i.setOutDepartmentCodeStandard(change.getCode());
                        }
                    }
                }
            }
        });

        return resultList;
    }

    @Override
    public PatientDischargeRecordClientBO getOnePatientOutpatientLastDateInfo(String patientId) {
        if (StrUtil.isBlank(patientId)) {
            return null;
        }
        PatientDischargeRecordClientBO outpatient = dwdPatientDataBizMapper.getOnePatientOutpatientLastDateInfo(new QueryWrapper<PatientDischargeRecordClientBO>()
                .eq("patient_id", patientId)
                .gt("leave_date", DateUtil.offsetDay(DateUtil.date(), -90).toDateStr()) // 只查询最近三个月的数据
        );

        if (null != outpatient) {
            PatientDischargeContentClientBO outpatientContent = dwdPatientDataBizMapper.getOnePatientOutpatientContentInfo(new QueryWrapper<PatientDischargeContentClientBO>()
                    .eq("id", outpatient.getId())
            );

            if (null != outpatientContent && StrUtil.isNotBlank(outpatientContent.getContentStandard())) {
                outpatient.setContent(outpatientContent);

                DwdDischargeContentDetailVO contentDetailVO = JSONObject.parseObject(outpatientContent.getContentStandard(), DwdDischargeContentDetailVO.class);
                if (null != contentDetailVO && CollUtil.isNotEmpty(contentDetailVO.getValueList())) {
                    contentDetailVO.getValueList().forEach(i -> {
                        // if ("SFSJ".equals(i.getName())) { // 随访时间
                        //     outpatientContent.setFollowupDateRaw(i.getContent());
                        // }

                        // 5.0数据结构
                        String idValue = i.get("id");
                        String tablenameValue = i.get("tablename");
                        String valueValue = i.get("value");
                        if ("SFSJ".equals(idValue)) { // 随访时间
                            outpatientContent.setFollowupDateRaw(tablenameValue);
                            if (StrUtil.isBlank(tablenameValue) && StrUtil.isNotBlank(valueValue)) {
                                outpatientContent.setFollowupDateRaw(valueValue);
                            }
                        }

                        // 3.0数据结构
                        String nameValue = i.get("name");
                        String contentValue = i.get("content");
                        if ("SFSJ".equals(nameValue)) { // 随访时间
                            outpatientContent.setFollowupDateRaw(contentValue);
                        }
                    });
                    
                    if (StrUtil.isNotBlank(outpatientContent.getFollowupDateRaw())) {
                        outpatientContent.setFollowupDateStandard(FollowUpRegexUtil.getFollowupDateStandard(outpatient.getLeaveDate(), outpatientContent.getFollowupDateRaw()));
                    }
                }
            }
        }

        return outpatient;
    }

    @Override
    public List<PatientVisitRecordClientBO> listPatientVisitInfo(PatientDataClientDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new ArrayList<>(0);
        }
        List<PatientVisitRecordClientBO> list = dwdPatientDataBizMapper.listPatientVisitLastDateInfo(new QueryWrapper<PatientVisitRecordClientBO>()
                .in("patient_id", params.getPatientIds())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "visit_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "visit_date", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "department_code_standard", params.getDeptId())
        );

        return list;
    }

    @Override
    public List<PatientVisitRecordClientBO> listPatientVisitLastDateInfo(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        List<PatientVisitRecordClientBO> list = dwdPatientDataBizMapper.listPatientVisitLastDateInfo(new QueryWrapper<PatientVisitRecordClientBO>()
                .in("patient_id", patientIds)
                .gt("visit_date", DateUtil.offsetDay(DateUtil.date(), -90).toDateStr()) // 只查询最近三个月的数据
        );

        return list;
    }

    @Override
    public PatientVisitRecordClientBO getOnePatientVisitLastDateInfo(String patientId) {
        if (StrUtil.isBlank(patientId)) {
            return null;
        }
        PatientVisitRecordClientBO visit = dwdPatientDataBizMapper.getOnePatientVisitLastDateInfo(new QueryWrapper<PatientVisitRecordClientBO>()
                .eq("patient_id", patientId)
                .gt("visit_date", DateUtil.offsetDay(DateUtil.date(), -90).toDateStr()) // 只查询最近三个月的数据
        );

        return visit;
    }

    @Override
    public IPage<String> pagePatientChangeDateInfo(Page<Void> page, String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return null;
        }
        return dwdPatientDataBizMapper.pagePatientChangeDateInfo(page, new QueryWrapper<Void>()
            .ge("data_day", startDate)
            .lt(StrUtil.isNotBlank(endDate), "data_day", endDate)
        );
    }
    @Override
    public IPage<Map<String, String>> pagePatientChangeDateMapInfo(Page<Void> page, String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return null;
        }
        return dwdPatientDataBizMapper.pagePatientChangeDateMapInfo(page, new QueryWrapper<Void>()
            .ge("data_day", startDate)
            .lt(StrUtil.isNotBlank(endDate), "data_day", endDate)
        );
    }

    @Override
    public List<PatientDiagnosisReportClientBO> listPatientDiagnosisReportInfo(PatientDiagnosisDataClientDTO params) {
        List<PatientDiagnosisReportClientBO> list = dwdPatientDataBizMapper.listPatientDiagnosisReportInfo(new QueryWrapper<PatientDiagnosisReportClientBO>()
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
            .in(CollUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
            .eq(StrUtil.isNotBlank(params.getSceneType()), "scene_type_standard", params.getSceneType())
            .eq(StrUtil.isNotBlank(params.getIndex()), "sort_idx", params.getIndex())
            .in(CollUtil.isNotEmpty(params.getDiseaseIds()), "diagnosis_code_standard", params.getDiseaseIds())
            .ge(null != params.getStartDate(), "diagnosis_time", params.getStartDate())
            .lt(null != params.getEndDate(), "diagnosis_time", params.getEndDate())
            .orderByDesc(StrUtil.isNotBlank(params.getOrderBy()), params.getOrderBy())
        );

        return list;
    }

    @Override
    public List<PatientDiagnosisReportClientBO> listPatientDiagnosisReportDistinct(PatientDiagnosisDataClientDTO params) {
        List<PatientDiagnosisReportClientBO> list = dwdPatientDataBizMapper.listPatientDiagnosisReportDistinct(new QueryWrapper<PatientDiagnosisReportClientBO>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getSceneType()), "scene_type_standard", params.getSceneType())
                .eq(StrUtil.isNotBlank(params.getIndex()), "sort_idx", params.getIndex())
                .in(CollUtil.isNotEmpty(params.getDiseaseIds()), "diagnosis_code_standard", params.getDiseaseIds())
                .ge(null != params.getStartDate(), "diagnosis_time", params.getStartDate())
                .lt(null != params.getEndDate(), "diagnosis_time", params.getEndDate())
                .orderByDesc(StrUtil.isNotBlank(params.getOrderBy()), params.getOrderBy())
        );

        return list;
    }

    @Override
    public PageData<PatientFeeRecordBO> pagePatientFeeRecord(PatientFeeQueryDTO params) {
        IPage<PatientFeeRecordBO> page = dwdPatientDataBizMapper.pagePatientFeeRecord(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientFeeRecordBO>()
                .eq(StrUtil.isNotEmpty(params.getDeptId()),"re.department_code_standard", params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()),"re.department_code_standard", params.getDeptIds())
                .like(StrUtil.isNotEmpty(params.getItemNameStandard()),"fi.item_name_standard", params.getItemNameStandard())
                .eq(StrUtil.isNotEmpty(params.getPatientId()),"fi.patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()),"fi.patient_id", params.getPatientIds())
                .like(StrUtil.isNotEmpty(params.getPatientName()),"fi.patient_name", params.getPatientName())
                .eq(StrUtil.isNotEmpty(params.getDoctorId()),"re.doctor_code", params.getDoctorId())
                .ge(null != params.getStartDate(), "re.visit_date", params.getStartDate())
                .lt(null != params.getEndDate(), "re.visit_date", params.getEndDate())
                .orderByDesc("re.visit_date")
        );

        PageData<PatientFeeRecordBO> data = new PageConverter<PatientFeeRecordBO, PatientFeeRecordBO>() {
            @Override
            public PatientFeeRecordBO converter(PatientFeeRecordBO bo) {
                return bo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<PatientFeeRecordBO> listPatientFeeRecord(PatientFeeQueryDTO params) {
        List<PatientFeeRecordBO> bos = dwdPatientDataBizMapper.listPatientFeeRecord(new QueryWrapper<PatientFeeRecordBO>()
                .eq(StrUtil.isNotEmpty(params.getDeptId()),"re.department_code_standard", params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()),"re.department_code_standard", params.getDeptIds())
                .like(StrUtil.isNotEmpty(params.getItemNameStandard()),"fi.item_name_standard", params.getItemNameStandard())
                .eq(StrUtil.isNotEmpty(params.getPatientId()),"fi.patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()),"fi.patient_id", params.getPatientIds())
                .like(StrUtil.isNotEmpty(params.getPatientName()),"fi.patient_name", params.getPatientName())
                .eq(StrUtil.isNotEmpty(params.getDoctorId()),"re.doctor_code", params.getDoctorId())
                .ge(null != params.getStartDate(), "re.visit_date", params.getStartDate())
                .lt(null != params.getEndDate(), "re.visit_date", params.getEndDate())
                .orderByDesc("re.visit_date")
                .last(" limit 50000 ") // 限制查询数量
        );
        return bos;
    }

    @Override
    public List<DoctorDwdRegisterRecordVO> queryRegisterLossList(Date startDate, Date endDate) {
        // 查询范围内最新的数据
        List<DoctorDwdRegisterRecordVO> list = dwdPatientDataBizMapper.queryRegisterLossList(new QueryWrapper<DoctorDwdRegisterRecordVO>()
                .isNull("b.id")
                .ge("a.reg_date", DateUtil.date(startDate).toDateStr())
                .lt("a.reg_date", DateUtil.date(endDate).toDateStr())
                .eq("a.status_raw", "1")
        ); // 当日退号的不算流失
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        
        return list;
    }

    @Override
    public List<DoctorDwdRegisterRecordVO> queryRegisterVisitList(Set<String> patientIds, Date startDate, Date endDate) {
        // 查询范围内最新的数据
        List<DoctorDwdRegisterRecordVO> list = dwdPatientDataBizMapper.queryRegisterLossList(new QueryWrapper<DoctorDwdRegisterRecordVO>()
                .isNotNull("b.id")
                .in("a.patient_Id", patientIds)
                .ge("a.reg_date", DateUtil.date(startDate).toDateStr())
                .lt("a.reg_date", DateUtil.date(endDate).toDateStr())

        ); // 当日退号的不算流失
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list;
    }

    @Override
    public List<PatientVisitRecordClientBO> queryVisitLossList(Date startDate, Date endDate) {
        // 查询范围内最新的数据
        List<PatientVisitRecordClientBO> list = dwdPatientDataBizMapper.queryVisitLossList(new QueryWrapper<PatientVisitRecordClientBO>()
                .isNull("b.id")
                .isNull("c.id")
                .isNull("d.id")
                .isNull("e.id")
                .ge("a.visit_date", DateUtil.date(startDate).toDateStr())
                .lt("a.visit_date", DateUtil.date(endDate).toDateStr())
        );
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        
        return list;
    }

    @Override
    public List<PatientInpatientReserveClientBO> queryInpatientLossList(Date startDate, Date endDate, Integer inHospitalLostDay) {
        // 查询范围内最新的数据
//        List<PatientInpatientReserveClientBO> list = dwdPatientDataBizMapper.queryInpatientLossList(new QueryWrapper<PatientInpatientReserveClientBO>()
//                .isNull("b.id")
//                .ge("a.apply_date", DateUtil.date(startDate).toDateStr())
//                .lt("a.apply_date", DateUtil.date(endDate).toDateStr())
//                .ge("b.inpatient_date", DateUtil.date(startDate).toDateStr())
//                .ge("b.inpatient_date", DateUtil.date(DateUtil.offsetDay(startDate, inHospitalLostDay)).toDateStr())
//        );
        List<PatientInpatientReserveClientBO> list = dwdPatientDataBizMapper.queryInpatientLossList(new QueryWrapper<PatientInpatientReserveClientBO>()
                        .and(wrapper -> wrapper
                                .or(w -> w // 对应 OR 的第一个条件
                                        .isNull("b.id")
                                        .ge("a.apply_date", DateUtil.date(startDate).toDateStr())
                                        .lt("a.apply_date", DateUtil.date(endDate).toDateStr())
                                )
                                .or(w -> w // 对应 OR 的第二个条件
                                        .isNotNull("b.id")
                                        .ge("a.apply_date", DateUtil.date(startDate).toDateStr())
                                        .lt("a.apply_date", DateUtil.date(endDate).toDateStr())
                                        .ge("b.inpatient_date", DateUtil.date(DateUtil.offsetDay(startDate, inHospitalLostDay)).toDateStr())
                                )
                        )
                /*.isNull("b.id")
                .ge("a.apply_date", DateUtil.date(startDate).toDateStr())
                .lt("a.apply_date", DateUtil.date(endDate).toDateStr())*/
//                .ge("b.inpatient_date", DateUtil.date(startDate).toDateStr())
//                .ge("b.inpatient_date", DateUtil.date(DateUtil.offsetDay(startDate, inHospitalLostDay)).toDateStr())
        );
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        
        return list;
    }

}
