package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataUpdateTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorFollowPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorFollowPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorFollowPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorFollowPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorFollowPatientBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DoctorFollowPatientBizServiceImpl implements DoctorFollowPatientBizService {

    @Autowired
    private DoctorFollowPatientInfoService doctorFollowPatientInfoService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;

    @Override
    public PageData<DoctorFollowPatientVO> page(DoctorFollowPatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 若按照患者名称筛选则先查询患者信息
        Set<String> patientIds = new HashSet<>();
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }
        
        IPage<DoctorFollowPatientInfo> page = doctorFollowPatientInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorFollowPatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(patientIds), "patient_id", patientIds)
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .ge(null != params.getStartDate(), "data_update_date", params.getStartDate())
                .lt(null != params.getEndDate(), "data_update_date", params.getEndDate())
                .orderByDesc("data_update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // Set<String> patientIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            if (StrUtil.isNotBlank(i.getPatientId())) {
                patientIds.add(i.getPatientId());
            }
        });
        
        // 患者信息
        // Map<String, PatientClientBO> patientMap = new HashMap<>();
        Map<String, PatientVisitRecordClientBO> visitRecordMap = new HashMap<>();
        Map<String, PatientInpatientRecordClientBO> inpatientRecordMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {

            if (CollUtil.isEmpty(patientMap)) {
                PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
                patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
                List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
                if (CollUtil.isNotEmpty(patientList)) {
                    patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
                }
            }

            // 查询患者最新就诊信息
            List<PatientVisitRecordClientBO> visitRecords = doctorDwdPatientDataBizService.listPatientVisitLastDateInfo(new ArrayList<>(patientIds));
            if (CollUtil.isNotEmpty(visitRecords)) {
                visitRecords.forEach(i -> visitRecordMap.put(i.getPatientId(), i));
            }
            
            // 查询患者最新住院信息
            PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
            patientLastDataQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientInpatientRecordClientBO> inpatientRecords = doctorDwdPatientDataBizService.listPatientInpatientLastDateInfo(patientLastDataQueryDTO);
            if (CollUtil.isNotEmpty(inpatientRecords)) {
                inpatientRecords.forEach(i -> inpatientRecordMap.put(i.getPatientId(), i));
            }
        }

        PageData<DoctorFollowPatientVO> data = new PageConverter<DoctorFollowPatientInfo, DoctorFollowPatientVO>() {
            @Override
            public DoctorFollowPatientVO converter(DoctorFollowPatientInfo entity) {
                DoctorFollowPatientVO vo = DoctorDataHelper.getDoctorFollowPatientVO(entity);
                vo.setPatientInfo(patientMap.get(vo.getPatientId()));
                vo.setInpatientRecordInfo(inpatientRecordMap.get(vo.getPatientId()));
                vo.setVisitRecordInfo(visitRecordMap.get(vo.getPatientId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public PageData<DoctorFollowPatientVO> simplePage(DoctorFollowPatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DoctorFollowPatientInfo> page = doctorFollowPatientInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorFollowPatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .ge(null != params.getStartDate(), "data_update_date", params.getStartDate())
                .lt(null != params.getEndDate(), "data_update_date", params.getEndDate())
                .orderByDesc("data_update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> patientIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            if (StrUtil.isNotBlank(i.getPatientId())) {
                patientIds.add(i.getPatientId());
            }
        });
        
        // 患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        PageData<DoctorFollowPatientVO> data = new PageConverter<DoctorFollowPatientInfo, DoctorFollowPatientVO>() {
            @Override
            public DoctorFollowPatientVO converter(DoctorFollowPatientInfo entity) {
                DoctorFollowPatientVO vo = DoctorDataHelper.getDoctorFollowPatientVO(entity);
                vo.setPatientInfo(patientMap.get(vo.getPatientId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorFollowPatientVO> simpleList(DoctorFollowPatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DoctorFollowPatientInfo> list = doctorFollowPatientInfoService.list(new QueryWrapper<DoctorFollowPatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .ge(null != params.getStartDate(), "data_update_date", params.getStartDate())
                .lt(null != params.getEndDate(), "data_update_date", params.getEndDate())
                .orderByDesc("data_update_date")
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Set<String> patientIds = new HashSet<>();
        list.forEach(i -> {
            if (StrUtil.isNotBlank(i.getPatientId())) {
                patientIds.add(i.getPatientId());
            }
        });
        
        // 患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        return list.stream().map(i -> {
            DoctorFollowPatientVO vo = DoctorDataHelper.getDoctorFollowPatientVO(i);
            vo.setPatientInfo(patientMap.get(vo.getPatientId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public DoctorFollowPatientVO detail(DoctorFollowPatientGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DoctorFollowPatientInfo entity = doctorFollowPatientInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return null;
        }
        
        DoctorFollowPatientVO vo = DoctorDataHelper.getDoctorFollowPatientVO(entity);

        if (StrUtil.isNotBlank(vo.getPatientId())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientId(vo.getPatientId());
            vo.setPatientInfo(patientClientService.getOneSimple(patientQueryDTO));

            // 查询患者最新就诊信息
            vo.setVisitRecordInfo(doctorDwdPatientDataBizService.getOnePatientVisitLastDateInfo(vo.getPatientId()));
            // 查询患者最新住院信息
            vo.setInpatientRecordInfo(doctorDwdPatientDataBizService.getOnePatientInpatientLastDateInfo(vo.getPatientId()));
        }

        return vo;
    }

    @Override
    public DoctorFollowPatientVO followOrNot(DoctorFollowPatientAddDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorFollowPatientInfo entity = doctorFollowPatientInfoService.getOne(new QueryWrapper<DoctorFollowPatientInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("doctor_id", params.getDoctorId())
            .eq("patient_id", params.getPatientId())
        );

        if (null != entity && YN.YES.getCode().equals(entity.getStatus())) {
            entity.setStatus(YN.NO.getCode());
            doctorFollowPatientInfoService.updateById(entity);
        } else {
            Date dataUpdateDate = null;
            String dataUpdateType = null;
            // 查询患者最新就诊信息
            PatientVisitRecordClientBO visitRecord = doctorDwdPatientDataBizService.getOnePatientVisitLastDateInfo(params.getPatientId());
            // 查询患者最新住院信息
            PatientInpatientRecordClientBO inpatientRecord = doctorDwdPatientDataBizService.getOnePatientInpatientLastDateInfo(params.getPatientId());

            // 比较就诊与住院哪个时间最新
            if (null != visitRecord && null != inpatientRecord) {
                dataUpdateDate = visitRecord.getVisitDate();
                dataUpdateType = PatientDataUpdateTypeEnum.Visit.getCode();
                if (inpatientRecord.getInpatientDate().after(visitRecord.getVisitDate())) {
                    dataUpdateDate = inpatientRecord.getInpatientDate();
                    dataUpdateType = PatientDataUpdateTypeEnum.Inpatient.getCode();
                }
            } else if (null != visitRecord) {
                dataUpdateDate = visitRecord.getVisitDate();
                dataUpdateType = PatientDataUpdateTypeEnum.Visit.getCode();
            } else if (null != inpatientRecord) {
                dataUpdateDate = inpatientRecord.getInpatientDate();
                dataUpdateType = PatientDataUpdateTypeEnum.Inpatient.getCode();
            }

            if (null == entity) {
                entity = new DoctorFollowPatientInfo();
                entity.setId(DoctorDataHelper.getDoctorFollowPatientId());
                entity.setDoctorId(params.getDoctorId());
                entity.setPatientId(params.getPatientId());
                entity.setRowsStatus(YN.YES.getCode());

                entity.setDataUpdateDate(dataUpdateDate);
                entity.setDataUpdateType(dataUpdateType);
                entity.setStatus(YN.YES.getCode());
                doctorFollowPatientInfoService.save(entity);
            } else {
                entity.setDataUpdateDate(dataUpdateDate);
                entity.setDataUpdateType(dataUpdateType);
                entity.setStatus(YN.YES.getCode());
                doctorFollowPatientInfoService.updateById(entity);
            }
        }

        return DoctorDataHelper.getDoctorFollowPatientVO(entity);
    }

}
