package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.constant.DoctorManagePatientConstant;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorLabelPatientDataInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorLabelPatientDataInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorLabelPatientDataQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorLabelPatientExcludDataQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorConsultationLaunchApplyDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientDiseaseDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientExcludeStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorManagePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DoctorLabelPatientStatisticBizQueryMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorLabelPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorManagePatientBizService;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorLabelPatientDataBizServiceImpl implements DoctorLabelPatientDataBizService {

    @Autowired
    private DoctorLabelPatientDataInfoService doctorLabelPatientDataInfoService;
    @Autowired
    private DoctorManagePatientBizService doctorManagePatientBizService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private DoctorLabelPatientStatisticBizQueryMapper doctorLabelPatientStatisticBizQueryMapper;

    @Override
    public PageData<DoctorLabelPatientDataVO> query(DoctorLabelPatientDataQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        String startDate = null;
        String endDate = null;
        if (null != params.getStartDate()) {
            startDate = DateUtil.formatDate(params.getStartDate());
        }
        if (null != params.getEndDate()) {
            endDate = DateUtil.formatDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 按医生姓名查询
        if (StrUtil.isNotBlank(params.getDoctorName())) {
            List<String> doctorIds = doctorLabelPatientStatisticBizQueryMapper.queryDoctorList(new QueryWrapper<String>()
                    .eq("rows_status", YN.YES.getCode())
                    .like("real_name", params.getDoctorName())
            );
            if (CollUtil.isEmpty(doctorIds)) {
                return new PageData<>(null, 0);
            }
            params.setDoctorIds(doctorIds);
        }

        IPage<DoctorLabelPatientDataInfo> page = doctorLabelPatientDataInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorLabelPatientDataInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getScene()), "scene", params.getScene())
                .ge(null != startDate, "biz_day", startDate)
                .lt(null != endDate, "biz_day", endDate)
                .orderByDesc("biz_day")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<DoctorLabelPatientDataVO> list = new ArrayList<>();
        Set<String> doctorIds = new HashSet<>();
        Set<String> labelIds = new HashSet<>();
        Set<String> managePatientIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());
            labelIds.add(i.getLabelId());

            DoctorLabelPatientDataVO vo = DoctorDataHelper.getDoctorLabelPatientDataVO(i);

            if (CollUtil.isNotEmpty(vo.getExcludePatientIdList())) {
                managePatientIds.addAll(vo.getExcludePatientIdList());
            }
            if (CollUtil.isNotEmpty(vo.getJoinPatientIdList())) {
                managePatientIds.addAll(vo.getJoinPatientIdList());
            }

            list.add(vo);
        });
        
        // 运营助理信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        // 标签
        Map<String, DoctorLabelPatientDiseaseDataVO> labelMap = new HashMap<>();
        if (CollUtil.isNotEmpty(labelIds)) {
            List<DoctorLabelPatientDiseaseDataVO> labelList = doctorLabelPatientStatisticBizQueryMapper.queryLabelList(new QueryWrapper<DoctorLabelPatientDiseaseDataVO>()
                    .in("id", labelIds)
            );
            if (CollUtil.isNotEmpty(labelList)) {
                labelList.forEach(i -> labelMap.put(i.getLabelId(), i));
            }
        }

        // 标签下患者管理(排除/纳入)
        Map<String, DoctorManagePatientVO> excludeMap = new HashMap<>();
        Map<String, DoctorManagePatientVO> joinMap = new HashMap<>();
        Map<String, List<DoctorConsultationLaunchApplyDataVO>> consultationMap = new HashMap<>();
        if (CollUtil.isNotEmpty(managePatientIds)) {
            DoctorManagePatientQueryDTO doctorManagePatientQueryDTO = new DoctorManagePatientQueryDTO();
            doctorManagePatientQueryDTO.setIds(new ArrayList<>(managePatientIds));
            List<DoctorManagePatientVO> doctorManagePatientVOs = doctorManagePatientBizService.list(doctorManagePatientQueryDTO);
            if (CollUtil.isNotEmpty(doctorManagePatientVOs)) {
                doctorManagePatientVOs.forEach(i -> {
                    if (DoctorManagePatientConstant.OpType.EXCLUDE.getCode().equals(i.getOpType())) {
                        excludeMap.put(i.getId(), i);
                    }
                    if (DoctorManagePatientConstant.OpType.JOIN.getCode().equals(i.getOpType())) {
                        joinMap.put(i.getId(), i);
                    }
                });
            }
            
            // 会诊数据
            List<DoctorConsultationLaunchApplyDataVO> consultationList = doctorLabelPatientStatisticBizQueryMapper.queryDoctorConsultatioLaunchApplyList(new QueryWrapper<DoctorConsultationLaunchApplyDataVO>()
                    .in("manage_id", managePatientIds)
            );
            if (CollUtil.isNotEmpty(consultationList)) {
                consultationList.forEach(i -> {
                    List<DoctorConsultationLaunchApplyDataVO> l = consultationMap.get(i.getManageId());
                    if (null == l) {
                        l = new ArrayList<>();
                        consultationMap.put(i.getManageId(), l);
                    }
                    l.add(i);
                });
            }

            // TODO 随访数据

        }

        list.forEach(i -> {
            AdminUserBO doctor = doctorMap.get(i.getDoctorId());
            if (null != doctor) {
                i.setDoctorInfo(doctor);
                i.setDeptInfo(deptMap.get(doctor.getDeptId()));
            }

            DoctorLabelPatientDiseaseDataVO label = labelMap.get(i.getLabelId());
            if (null != label) {
                i.setLabelName(label.getLabelName());
            }

            List<DoctorManagePatientVO> excludeList = new ArrayList<>();
            List<String> excludePatientIdList = i.getExcludePatientIdList();
            if (CollUtil.isNotEmpty(excludePatientIdList)) {
                excludePatientIdList.forEach(j -> {
                    excludeList.add(excludeMap.get(j));
                });
            }
            i.setExcludeInfoList(excludeList);
            
            List<DoctorManagePatientVO> joinList = new ArrayList<>();
            List<DoctorConsultationLaunchApplyDataVO> consultationList = new ArrayList<>();
            List<String> joinPatientIdList = i.getJoinPatientIdList();
            if (CollUtil.isNotEmpty(joinPatientIdList)) {
                joinPatientIdList.forEach(j -> {
                    joinList.add(joinMap.get(j));

                    List<DoctorConsultationLaunchApplyDataVO> l = consultationMap.get(j);
                    if (CollUtil.isNotEmpty(l)) {
                        consultationList.addAll(l);
                    }
                });
            }
            i.setJoinInfoList(joinList);
            i.setConsultationList(consultationList);
        });

        return new PageData<>(list, page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public PageData<DoctorLabelPatientStatisticVO> queryStatisticBySecene(DoctorLabelPatientDataQueryDTO params) {
        PageData<DoctorLabelPatientDataVO> page = this.query(params);

        List<DoctorLabelPatientStatisticVO> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(page.getList())) {
            page.getList().forEach(i -> {
                DoctorLabelPatientStatisticVO vo = new DoctorLabelPatientStatisticVO();
                vo.setStatisticDate(i.getBizDay());
                vo.setLabelId(i.getLabelId());
                vo.setLabelName(i.getLabelName());
                
                vo.setLabelNum(0);
                if (CollUtil.isNotEmpty(i.getPatientLabelIdList())) {
                    vo.setLabelNum(i.getPatientLabelIdList().size());
                }
                
                DeptBO dept = i.getDeptInfo();
                if (null != dept) {
                    vo.setDeptId(dept.getId());
                    vo.setDeptName(dept.getName());
                }

                vo.setDoctorId(i.getDoctorId());
                AdminUserBO doctor = i.getDoctorInfo();
                if (null != doctor) {
                    vo.setDoctorName(doctor.getRealName());
                }

                vo.setScene(i.getScene());
                
                // 排除
                vo.setExcludeCount(0);
                if (CollUtil.isNotEmpty(i.getExcludePatientIdList())) {
                    vo.setExcludeCount(i.getExcludePatientIdList().size());
                }
                vo.setExcludeList(i.getExcludeInfoList());
                // 排除原因统计
                if (CollUtil.isNotEmpty(vo.getExcludeList())) {
                    Map<String, Integer> excludeStatisticMap = new HashMap<>();
                    vo.getExcludeList().forEach(j -> {
                        Integer count = excludeStatisticMap.get(j.getRemark());
                        if (null == count) {
                            excludeStatisticMap.put(j.getRemark(), 1);
                        } else {
                            excludeStatisticMap.put(j.getRemark(), excludeStatisticMap.get(j.getRemark()) + 1);
                        }
                    });
                    if (CollUtil.isNotEmpty(excludeStatisticMap)) {
                        List<DoctorLabelPatientExcludeStatisticVO> excludeStatisticList = new ArrayList<>();
                        excludeStatisticMap.forEach((k, v) -> {
                            DoctorLabelPatientExcludeStatisticVO excludeStatisticVO = new DoctorLabelPatientExcludeStatisticVO();
                            excludeStatisticVO.setExcludeReason(k);
                            excludeStatisticVO.setExcludeCount(v);
                            excludeStatisticList.add(excludeStatisticVO);
                        });
                        vo.setExcludeStatisticVOList(excludeStatisticList);
                    }
                }

                // 纳入
                vo.setJoinCount(0);
                if (CollUtil.isNotEmpty(i.getJoinPatientIdList())) {
                    vo.setJoinCount(i.getJoinPatientIdList().size());
                }
                vo.setJoinList(i.getJoinInfoList());

                // 会诊
                vo.setConsultationCount(0);
                if (CollUtil.isNotEmpty(i.getConsultationIdList())) {
                    vo.setConsultationCount(i.getConsultationIdList().size());
                }
                vo.setConsultationList(i.getConsultationList());
                // 会诊统计
                vo.setConsultationCount(0);
                vo.setTodoConsultationCount(0);
                vo.setDoneConsultationCount(0);
                if (CollUtil.isNotEmpty(vo.getConsultationList())) {
                    Map<String, Integer> consultationStatisticMap = new HashMap<>();
                    vo.getConsultationList().forEach(j -> {
                        Integer count = consultationStatisticMap.get(j.getStatus());
                        if (null == count) {
                            consultationStatisticMap.put(j.getStatus(), 1);
                        } else {
                            consultationStatisticMap.put(j.getStatus(), consultationStatisticMap.get(j.getStatus()) + 1);
                        }
                    });
                    if (CollUtil.isNotEmpty(consultationStatisticMap)) {
                        consultationStatisticMap.forEach((k, v) -> {
                            vo.setConsultationCount(vo.getConsultationCount() + v);
                            if (ConsultationLaunchApplyConstant.ApplyStatus.START.getCode().equals(k) || ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(k)) {
                                vo.setTodoConsultationCount(vo.getTodoConsultationCount() + v);
                            }
                            if (ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(k)) {
                                vo.setDoneConsultationCount(vo.getTodoConsultationCount() + v);
                            }
                        });
                    }
                }

                list.add(vo);
            });

        }

        return new PageData<>(list, page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public PageData<DoctorLabelPatientStatisticVO> queryStatisticByNoSecene(DoctorLabelPatientDataQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        String startDate = null;
        String endDate = null;
        if (null != params.getStartDate()) {
            startDate = DateUtil.formatDate(params.getStartDate());
        }
        if (null != params.getEndDate()) {
            endDate = DateUtil.formatDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 按医生姓名查询
        if (StrUtil.isNotBlank(params.getDoctorName())) {
            List<String> doctorIds = doctorLabelPatientStatisticBizQueryMapper.queryDoctorList(new QueryWrapper<String>()
                    .eq("rows_status", YN.YES.getCode())
                    .like("real_name", params.getDoctorName())
            );
            if (CollUtil.isEmpty(doctorIds)) {
                return new PageData<>(null, 0);
            }
            params.setDoctorIds(doctorIds);
        }

        // 聚合查询
        IPage<DoctorLabelPatientDataInfo> page = doctorLabelPatientStatisticBizQueryMapper.queryDoctorDataStatisticList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorLabelPatientDataInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .ge(null != startDate, "biz_day", startDate)
                .lt(null != endDate, "biz_day", endDate)
        );
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 根据聚合查询的结果查询详细的数据
        List<String> queryParams = new ArrayList<>();
        List<String> doctorIds = new ArrayList<>();
        List<String> labelIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            queryParams.add(i.getBizDay() + "_" + i.getDoctorId() + "_" + i.getLabelId());
            doctorIds.add(i.getDoctorId());
            labelIds.add(i.getLabelId());
        });

        Set<String> manageIds = new HashSet<>();
        Set<String> consultationIds = new HashSet<>();
        Map<String, List<DoctorLabelPatientDataInfo>> dataMap = new HashMap<>();
        List<DoctorLabelPatientDataInfo> dataList = doctorLabelPatientDataInfoService.list(new QueryWrapper<DoctorLabelPatientDataInfo>()
                .in("CONCAT(biz_day,'_',doctor_id,'_',label_id)", queryParams)
                .ge(null != startDate, "biz_day", startDate)
                .lt(null != endDate, "biz_day", endDate)
        );
        dataList.forEach(i -> {
            List<DoctorLabelPatientDataInfo> l = dataMap.get(i.getBizDay() + "_" + i.getDoctorId() + "_" + i.getLabelId());
            if (null == l) {
                l = new ArrayList<>();
                dataMap.put(i.getBizDay() + "_" + i.getDoctorId() + "_" + i.getLabelId(), l);
            }
            l.add(i);

            if (StrUtil.isNotBlank(i.getExcludePatientIds())) {
                manageIds.addAll(StrJoinSplitUtils.split2List(i.getExcludePatientIds()));
            }
            if (StrUtil.isNotBlank(i.getJoinPatientIds())) {
                manageIds.addAll(StrJoinSplitUtils.split2List(i.getJoinPatientIds()));
            }
            if (StrUtil.isNotBlank(i.getConsultationIds())) {
                consultationIds.addAll(StrJoinSplitUtils.split2List(i.getConsultationIds()));
            }
        });

        
        // 运营助理信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        // 标签信息
        Map<String, DoctorLabelPatientDiseaseDataVO> labelMap = new HashMap<>();
        if (CollUtil.isNotEmpty(labelIds)) {
            List<DoctorLabelPatientDiseaseDataVO> labelList = doctorLabelPatientStatisticBizQueryMapper.queryLabelList(new QueryWrapper<DoctorLabelPatientDiseaseDataVO>()
                    .in("id", labelIds)
            );
            if (CollUtil.isNotEmpty(labelList)) {
                labelList.forEach(i -> labelMap.put(i.getLabelId(), i));
            }
        }

        // 标签下患者管理(排除/纳入)
        Map<String, DoctorManagePatientVO> excludeMap = new HashMap<>(); // 排除
        Map<String, DoctorManagePatientVO> joinMap = new HashMap<>(); // 纳入
        Map<String, List<DoctorConsultationLaunchApplyDataVO>> consultationMap = new HashMap<>(); // 会诊

        if (CollUtil.isNotEmpty(manageIds)) {
            DoctorManagePatientQueryDTO doctorManagePatientQueryDTO = new DoctorManagePatientQueryDTO();
            doctorManagePatientQueryDTO.setIds(new ArrayList<>(manageIds));
            List<DoctorManagePatientVO> doctorManagePatientVOs = doctorManagePatientBizService.list(doctorManagePatientQueryDTO);
            if (CollUtil.isNotEmpty(doctorManagePatientVOs)) {
                doctorManagePatientVOs.forEach(i -> {
                    if (DoctorManagePatientConstant.OpType.EXCLUDE.getCode().equals(i.getOpType())) {
                        excludeMap.put(i.getId(), i);
                    }
                    if (DoctorManagePatientConstant.OpType.JOIN.getCode().equals(i.getOpType())) {
                        joinMap.put(i.getId(), i);
                    }
                });
            }
            
            // 会诊数据
            List<DoctorConsultationLaunchApplyDataVO> consultationList = doctorLabelPatientStatisticBizQueryMapper.queryDoctorConsultatioLaunchApplyList(new QueryWrapper<DoctorConsultationLaunchApplyDataVO>()
                    .in("manage_id", manageIds)
            );
            if (CollUtil.isNotEmpty(consultationList)) {
                consultationList.forEach(i -> {
                    List<DoctorConsultationLaunchApplyDataVO> l = consultationMap.get(i.getManageId());
                    if (null == l) {
                        l = new ArrayList<>();
                        consultationMap.put(i.getManageId(), l);
                    }
                    l.add(i);
                });
            }
        }

        List<DoctorLabelPatientStatisticVO> list = new ArrayList<>();

        page.getRecords().forEach(i -> {

            DoctorLabelPatientStatisticVO vo = new DoctorLabelPatientStatisticVO();
            vo.setScene("所有场景");
            vo.setStatisticDate(i.getBizDay());
            vo.setLabelId(i.getLabelId());
            vo.setDoctorId(i.getDoctorId());

            DoctorLabelPatientDiseaseDataVO label = labelMap.get(i.getLabelId());
            if (null != label) {
                vo.setLabelName(label.getLabelName());
            }

            AdminUserBO doctor = doctorMap.get(i.getDoctorId());
            if (null != doctor) {
                vo.setDoctorName(doctor.getRealName());
                vo.setDeptId(doctor.getDeptId());
                DeptBO dept = deptMap.get(doctor.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }
            }

            vo.setExcludeCount(0);
            vo.setExcludeList(new ArrayList<>());
            vo.setJoinCount(0);
            vo.setJoinList(new ArrayList<>());
            vo.setConsultationCount(0);
            vo.setConsultationList(new ArrayList<>());
            vo.setConsultationCount(0);
            vo.setTodoConsultationCount(0);
            vo.setDoneConsultationCount(0);

            List<DoctorLabelPatientDataInfo> l = dataMap.get(i.getBizDay() + "_" + i.getDoctorId() + "_" + i.getLabelId());
            if (CollUtil.isNotEmpty(l)) {
                l.forEach(j -> {
                    // 排除
                    if (StrUtil.isNotBlank(j.getExcludePatientIds())) {
                        List<String> excludePatientIds = StrJoinSplitUtils.split2List(j.getExcludePatientIds());
                        vo.setExcludeCount(vo.getExcludeCount() + excludePatientIds.size());
                        excludePatientIds.forEach(k -> {
                            vo.getExcludeList().add(excludeMap.get(k));
                        });
                    }
                    // 纳入
                    if (StrUtil.isNotBlank(j.getJoinPatientIds())) {
                        List<String> joinPatientIds = StrJoinSplitUtils.split2List(j.getJoinPatientIds());
                        vo.setJoinCount(vo.getJoinCount() + joinPatientIds.size());
                        joinPatientIds.forEach(k -> {
                            vo.getJoinList().add(joinMap.get(k));

                            // 会诊
                            List<DoctorConsultationLaunchApplyDataVO> consultations = consultationMap.get(k);
                            if (CollUtil.isNotEmpty(consultations)) {
                                vo.getConsultationList().addAll(consultations);
                            }
                        });
                    }
                    // 会诊
                    // if (StrUtil.isNotBlank(j.getConsultationIds())) {
                    //     List<String> ids = StrJoinSplitUtils.split2List(j.getConsultationIds());
                    //     vo.setConsultationCount(vo.getConsultationCount() + ids.size());
                    //     ids.forEach(k -> {
                    //         List<DoctorConsultationLaunchApplyDataVO> consultations = consultationMap.get(k);
                    //         if (CollUtil.isNotEmpty(consultations)) {
                    //             vo.getConsultationList().addAll(consultations);
                    //         }
                    //     });
                    // }
                });

                // 排除原因
                if (CollUtil.isNotEmpty(vo.getExcludeList())) {
                    Map<String, Integer> excludeStatisticMap = new HashMap<>();
                    vo.getExcludeList().forEach(j -> {
                        Integer count = excludeStatisticMap.get(j.getRemark());
                        if (null == count) {
                            excludeStatisticMap.put(j.getRemark(), 1);
                        } else {
                            excludeStatisticMap.put(j.getRemark(), excludeStatisticMap.get(j.getRemark()) + 1);
                        }
                    });
                    if (CollUtil.isNotEmpty(excludeStatisticMap)) {
                        List<DoctorLabelPatientExcludeStatisticVO> excludeStatisticList = new ArrayList<>();
                        excludeStatisticMap.forEach((k, v) -> {
                            DoctorLabelPatientExcludeStatisticVO excludeStatisticVO = new DoctorLabelPatientExcludeStatisticVO();
                            excludeStatisticVO.setExcludeReason(k);
                            excludeStatisticVO.setExcludeCount(v);
                            excludeStatisticList.add(excludeStatisticVO);
                        });
                        vo.setExcludeStatisticVOList(excludeStatisticList);
                    }
                }

                // 会诊统计
                if (CollUtil.isNotEmpty(vo.getConsultationList())) {
                    Map<String, Integer> consultationStatisticMap = new HashMap<>();
                    vo.getConsultationList().forEach(j -> {
                        Integer count = consultationStatisticMap.get(j.getStatus());
                        if (null == count) {
                            consultationStatisticMap.put(j.getStatus(), 1);
                        } else {
                            consultationStatisticMap.put(j.getStatus(), consultationStatisticMap.get(j.getStatus()) + 1);
                        }
                    });
                    if (CollUtil.isNotEmpty(consultationStatisticMap)) {
                        consultationStatisticMap.forEach((k, v) -> {
                            vo.setConsultationCount(vo.getConsultationCount() + v);
                            if (ConsultationLaunchApplyConstant.ApplyStatus.START.getCode().equals(k) || ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(k)) {
                                vo.setTodoConsultationCount(vo.getTodoConsultationCount() + v);
                            }
                            if (ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(k)) {
                                vo.setDoneConsultationCount(vo.getTodoConsultationCount() + v);
                            }
                        });
                    }
                }
            }
            list.add(vo);
        });

        return new PageData<>(list, page.getTotal(), page.getSize(), page.getPages());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveBatch(List<DoctorLabelPatientDataInfo> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        // 查询需要保存的统计是否已存在，删除存在的统计数据
        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> queryParams.add(i.getBizDay() + "_" + i.getDoctorId() + "_" + i.getLabelId()));
        if (CollUtil.isNotEmpty(queryParams)) {
            List<DoctorLabelPatientDataInfo> list = doctorLabelPatientDataInfoService.list(new QueryWrapper<DoctorLabelPatientDataInfo>()
                    .in("CONCAT(biz_day,'_',doctor_id,'_',label_id)", queryParams)
            );
            if (CollUtil.isNotEmpty(list)) {
                doctorLabelPatientDataInfoService.removeBatchByIds(list.stream().map(i -> i.getId()).collect(Collectors.toList()));
            }
        }

        // 保存统计
        doctorLabelPatientDataInfoService.saveBatch(params);
    }

    @Override
    public List<DoctorLabelPatientExcludeStatisticVO> excludeStatisticReason(DoctorLabelPatientExcludDataQueryDTO params) {
        // 按医生姓名查询
        if (StrUtil.isNotBlank(params.getDoctorName())) {
            List<String> doctorIds = doctorLabelPatientStatisticBizQueryMapper.queryDoctorList(new QueryWrapper<String>()
                    .eq("rows_status", YN.YES.getCode())
                    .like("real_name", params.getDoctorName())
            );
            if (CollUtil.isEmpty(doctorIds)) {
                return new ArrayList<>(0);
            }
            params.setDoctorIds(doctorIds);
        }

        DoctorManagePatientQueryDTO doctorManagePatientQueryDTO = new DoctorManagePatientQueryDTO();
        doctorManagePatientQueryDTO.setDoctorId(params.getDoctorId());
        doctorManagePatientQueryDTO.setDoctorIds(params.getDoctorIds());
        doctorManagePatientQueryDTO.setStartDate(params.getStartDate());
        doctorManagePatientQueryDTO.setEndDate(params.getEndDate());
        List<DoctorManagePatientVO> list = doctorManagePatientBizService.list(doctorManagePatientQueryDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Map<String, List<DoctorManagePatientVO>> map = new HashMap<>();
        list.forEach(i -> {
            List<DoctorManagePatientVO> l = map.get(i.getRemark());
            if (null == l) {
                l = new ArrayList<>();
                map.put(i.getRemark(), l);
            }
            l.add(i);
        });

        List<DoctorLabelPatientExcludeStatisticVO> data = new ArrayList<>();
        map.forEach((k, v) -> {
            DoctorLabelPatientExcludeStatisticVO vo = new DoctorLabelPatientExcludeStatisticVO();
            vo.setExcludeReason(k);
            vo.setExcludeCount(v.size());
            vo.setExcludeList(v);

            data.add(vo);
        });

        return data;
    }

}
