package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorManagePatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorManagePatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientDetailDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorManagePatientUpdateDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorManagePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorManagePatientBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.healthy.doctor.crm.client.api.DoctorBehaviorClientService;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorBehaviorBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.dto.DoctorBehaviorQueryClientDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorBehaviorConstant;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelQueryClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelQueryDTO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorManagePatientBizServiceImpl implements DoctorManagePatientBizService {

    @Autowired
    private DoctorManagePatientInfoService doctorManagePatientInfoService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private PatientLabelQueryClientService patientLabelQueryClientService; // 注意循环引用
    @Autowired
    private DoctorBehaviorClientService doctorBehaviorClientService;
    @Autowired
    private LabelClientService labelClientService;

    @Override
    public PageData<DoctorManagePatientVO> page(DoctorManagePatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DoctorManagePatientInfo> page = doctorManagePatientInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorManagePatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getOpSubType()), "op_sub_type", params.getOpSubType())
                .ge(null != params.getStartDate(), "op_date", params.getStartDate())
                .lt(null != params.getEndDate(), "op_date", params.getEndDate())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> doctorIds = new HashSet<>();
        Set<String> patientIds = new HashSet<>();
        Set<String> labelIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());
            patientIds.add(i.getPatientId());
            labelIds.add(i.getLabelId());
        });
        
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        Map<String, DoctorBehaviorBO> behaviorDataMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            // 患者信息
            TaskPatientQueryDTO patientQueryDTO = new TaskPatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
            
            // 查询医生操作数据
            DoctorBehaviorQueryClientDTO behaviorQuery = new DoctorBehaviorQueryClientDTO();
            behaviorQuery.setDoctorId(params.getDoctorId());
            behaviorQuery.setRelOneType(DoctorBehaviorConstant.RelOneType.Label_Patient.getCode());
            behaviorQuery.setRelOneIds(new ArrayList<>(patientIds));
            behaviorQuery.setRelTwoType(DoctorBehaviorConstant.RelTwoType.Label.getCode());
            behaviorQuery.setRelTwoId(params.getLabelId());
            List<DoctorBehaviorBO> firstBehaviorList = doctorBehaviorClientService.listFirstBehaviorData(behaviorQuery);
            firstBehaviorList.forEach(i -> behaviorDataMap.put(i.getDoctorId() + "-" + i.getRelOneId() + "-" + i.getRelTwoId(), i));
        }

        // 标签数据
        Map<String, LabelBO> labelMap = labelClientService.getLabelMapByIds(labelIds);

        PageData<DoctorManagePatientVO> data = new PageConverter<DoctorManagePatientInfo, DoctorManagePatientVO>() {
            @Override
            public DoctorManagePatientVO converter(DoctorManagePatientInfo entity) {
                DoctorManagePatientVO vo = DoctorDataHelper.getDoctorManagePatientVO(entity);

                AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
                vo.setDoctorInfo(doctor);
                if (StrUtil.isNotBlank(doctor.getDeptId())) {
                    vo.setDeptInfo(deptMap.get(doctor.getDeptId()));
                }

                vo.setPatientInfo(patientMap.get(vo.getPatientId()));

                DoctorBehaviorBO behavior = behaviorDataMap.get(vo.getDoctorId() + "-" + vo.getPatientId() + "-" + vo.getLabelId());
                if (null != behavior) {
                    vo.setFirstVisitDate(behavior.getCreateDate());
                }

                if (StrUtil.isNotBlank(vo.getLabelId())) {
                    vo.setLabelBO(labelMap.get(vo.getLabelId()));
                }

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorManagePatientVO> list(DoctorManagePatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        List<DoctorManagePatientInfo> list = doctorManagePatientInfoService.list(new QueryWrapper<DoctorManagePatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getOpSubType()), "op_sub_type", params.getOpSubType())
                .ge(null != params.getStartDate(), "op_date", params.getStartDate())
                .lt(null != params.getEndDate(), "op_date", params.getEndDate())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Set<String> doctorIds = new HashSet<>();
        Set<String> patientIds = new HashSet<>();
        list.forEach(i -> {
            doctorIds.add(i.getDoctorId());
            patientIds.add(i.getPatientId());
        });
        
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        // 患者信息
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            TaskPatientQueryDTO patientQueryDTO = new TaskPatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        return list.stream().map(i -> {
            DoctorManagePatientVO vo = DoctorDataHelper.getDoctorManagePatientVO(i);

            AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
            vo.setDoctorInfo(doctor);
            if (StrUtil.isNotBlank(doctor.getDeptId())) {
                vo.setDeptInfo(deptMap.get(doctor.getDeptId()));
            }

            vo.setPatientInfo(patientMap.get(vo.getPatientId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public List<DoctorManagePatientVO> simpleList(DoctorManagePatientQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        List<DoctorManagePatientInfo> list = doctorManagePatientInfoService.list(new QueryWrapper<DoctorManagePatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getOpSubType()), "op_sub_type", params.getOpSubType())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(DoctorDataHelper::getDoctorManagePatientVO).collect(Collectors.toList());
    }

    @Override
    public DoctorManagePatientVO getById(DoctorManagePatientGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DoctorManagePatientInfo entity = doctorManagePatientInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return null;
        }

        DoctorManagePatientVO vo = DoctorDataHelper.getDoctorManagePatientVO(entity);

        if (null != vo) {
            // 医生信息
            vo.setDoctorInfo(adminUserClientService.getUserById(vo.getDoctorId()));
            // 患者信息
            TaskPatientGetDTO patientGetDTO = new TaskPatientGetDTO();
            patientGetDTO.setPatientId(vo.getPatientId());
            vo.setPatientInfo(taskPatientQueryBizService.getOneSimple(patientGetDTO));
            // 第一次查看
            DoctorBehaviorQueryClientDTO behaviorQuery = new DoctorBehaviorQueryClientDTO();
            behaviorQuery.setDoctorId(vo.getDoctorId());
            behaviorQuery.setRelOneType(DoctorBehaviorConstant.RelOneType.Label_Patient.getCode());
            behaviorQuery.setRelOneId(vo.getPatientId());
            behaviorQuery.setRelTwoType(DoctorBehaviorConstant.RelTwoType.Label.getCode());
            behaviorQuery.setRelTwoId(vo.getLabelId());
            DoctorBehaviorBO firstBehavior = doctorBehaviorClientService.getFirstBehaviorData(behaviorQuery);
            if (null != firstBehavior) {
                vo.setFirstVisitDate(firstBehavior.getCreateDate());
            }
        }

        return vo;
    }

    @Override
    public DoctorManagePatientVO detail(DoctorManagePatientDetailDTO params) {
        ValidatorUtils.validateEntity(params);
        
        // 查询患者标签信息
        LabelQueryDTO labelQueryDTO = new LabelQueryDTO();
        labelQueryDTO.setPatientId(params.getPatientId());
        labelQueryDTO.setLabelId(params.getLabelId());
        List<PatientLabelBO> patientLabelBOs = patientLabelQueryClientService.queryList(labelQueryDTO);
        if (CollUtil.isEmpty(patientLabelBOs)) {
            return null;
        }
        Date patientLabelDate = patientLabelBOs.get(0).getDataDt();
        
        DoctorManagePatientInfo entity = doctorManagePatientInfoService.getOne(new QueryWrapper<DoctorManagePatientInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .eq(StrUtil.isNotBlank(params.getLabelId()), "label_id", params.getLabelId())
            .eq("label_date", patientLabelDate)
        );

        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return null;
        }

        DoctorManagePatientVO vo = DoctorDataHelper.getDoctorManagePatientVO(entity);

        if (null != vo) {
            // 医生信息
            vo.setDoctorInfo(adminUserClientService.getUserById(vo.getDoctorId()));
            // 患者信息
            TaskPatientGetDTO patientGetDTO = new TaskPatientGetDTO();
            patientGetDTO.setPatientId(vo.getPatientId());
            vo.setPatientInfo(taskPatientQueryBizService.getOneSimple(patientGetDTO));
            // 第一次查看
            DoctorBehaviorQueryClientDTO behaviorQuery = new DoctorBehaviorQueryClientDTO();
            behaviorQuery.setDoctorId(vo.getDoctorId());
            behaviorQuery.setRelOneType(DoctorBehaviorConstant.RelOneType.Label_Patient.getCode());
            behaviorQuery.setRelOneId(vo.getPatientId());
            behaviorQuery.setRelTwoType(DoctorBehaviorConstant.RelTwoType.Label.getCode());
            behaviorQuery.setRelTwoId(vo.getLabelId());
            DoctorBehaviorBO firstBehavior = doctorBehaviorClientService.getFirstBehaviorData(behaviorQuery);
            if (null != firstBehavior) {
                vo.setFirstVisitDate(firstBehavior.getCreateDate());
            }
        }

        return vo;
    }

    @Override
    public DoctorManagePatientVO save(DoctorManagePatientAddDTO params) {
        ValidatorUtils.validateEntity(params);

        // 查询患者标签信息
        LabelQueryDTO labelQueryDTO = new LabelQueryDTO();
        labelQueryDTO.setPatientId(params.getPatientId());
        labelQueryDTO.setLabelId(params.getLabelId());
        List<PatientLabelBO> patientLabelBOs = patientLabelQueryClientService.queryList(labelQueryDTO);
        if (CollUtil.isEmpty(patientLabelBOs)) {
            return null;
        }
        Date patientLabelDate = patientLabelBOs.get(0).getDataDt();
        
        DoctorManagePatientInfo entity = doctorManagePatientInfoService.getOne(new QueryWrapper<DoctorManagePatientInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("doctor_id", params.getDoctorId())
            .eq("patient_id", params.getPatientId())
            .eq("label_date", patientLabelDate)
        );
        if (null != entity) {
            entity.setDoctorId(params.getDoctorId());
            entity.setPatientId(params.getPatientId());
            entity.setLabelId(params.getLabelId());
            entity.setLabelDate(patientLabelDate);
            entity.setOpType(params.getOpType());
            entity.setOpSubType(params.getOpSubType());
            entity.setOpScene(params.getOpScene());
            entity.setOpContent(params.getOpContent());
            entity.setRemark(params.getRemark());
            entity.setOpDate(DateUtil.date());
            doctorManagePatientInfoService.updateById(entity);
        } else {   
            entity = new DoctorManagePatientInfo();
            entity.setId(DoctorDataHelper.getDoctorManagePatientId());
            entity.setDoctorId(params.getDoctorId());
            entity.setPatientId(params.getPatientId());
            entity.setLabelId(params.getLabelId());
            entity.setLabelDate(patientLabelDate);
            entity.setOpType(params.getOpType());
            entity.setOpSubType(params.getOpSubType());
            entity.setOpScene(params.getOpScene());
            entity.setOpContent(params.getOpContent());
            entity.setRemark(params.getRemark());
            entity.setOpDate(DateUtil.date());
            entity.setRowsStatus(YN.YES.getCode());
            doctorManagePatientInfoService.save(entity);
        }

        return DoctorDataHelper.getDoctorManagePatientVO(entity);
    }

    @Override
    public void update(DoctorManagePatientUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DoctorManagePatientInfo entity = doctorManagePatientInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }

        entity.setOpType(params.getOpType());
        entity.setOpSubType(params.getOpSubType());
        entity.setOpContent(params.getOpContent());
        entity.setRemark(params.getRemark());

        doctorManagePatientInfoService.updateById(entity);
    }

    @Override
    public void del(DoctorManagePatientGetDTO params) {
        DoctorManagePatientInfo entity = doctorManagePatientInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            return;
        }

        entity.setRowsStatus(YN.NO.getCode());

        doctorManagePatientInfoService.updateById(entity);
    }

}
