package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientRegisterVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientRelationMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DwdPatientDataInitBizService;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DwdPatientDataInitBizServiceImpl implements DwdPatientDataInitBizService {
    
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private DeptPatientRelationMapper deptPatientRelationMapper;
    
    ////////////// 以下为同步到中间表，然后通过sql去重插入目标表

    @Override
    public void initPatientInfo() {
        // 从dwd患者信息表中同步数据到中间表
        initPatientMid();

        // 从中间表同步到目标表
        // deptPatientRelationMapper.savePatientBatch();  // TODO 为不影响业务暂时只同步到中间表，后面直接通过sql同步到目标表
    }

    private void initPatientMid() {
        // 从dwd患者信息表中同步数据到中间表
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPage(1);
        dataDwdPatientQueryDTO.setRows(10000);
        // this.savePatientMid(dataDwdPatientQueryDTO);
        this.savePatientMid(dataDwdPatientQueryDTO);

    }

    private void savePatientMid(DataDwdPatientQueryDTO dataDwdPatientQueryDTO) {
        log.info("dwd同步患者信息>>>>>> {}", JSON.toJSONString(dataDwdPatientQueryDTO));
        while (true) {
            // dwd查询患者信息
            List<DataDwdPatientVO> list = doctorDwdPatientDataBizService.patientSyncList(dataDwdPatientQueryDTO);
            if (CollectionUtil.isEmpty(list)) {
                return;
            }

            // 保存查询的患者信息
            List<TaskPatientAddDTO> patientAddDTOs = new ArrayList<>();
            list.forEach(i -> {
                TaskPatientAddDTO patientAddDTO = new TaskPatientAddDTO();
                patientAddDTO.setPatientId(i.getPatientId());
                patientAddDTO.setPatientName(i.getPatientName());
                patientAddDTO.setBirthday(i.getBirthday());
                patientAddDTO.setPatientGenderStandard(i.getPatientGenderStandard());
                patientAddDTO.setIdCardNo(i.getCardNo());
                patientAddDTO.setPhone(i.getPhone());
                patientAddDTO.setPatientGender(i.getPatientGenderRaw());
                patientAddDTO.setMainPersonId(i.getMainPersonId());

                patientAddDTOs.add(patientAddDTO);
            });
            // 保存同步信息
            deptPatientRelationMapper.savePatientMidBatch(patientAddDTOs);

            dataDwdPatientQueryDTO.setPatientId(list.get(list.size() - 1).getPatientId());
        }
    }

    @Override
    public void initDeptPatientInfo() {
        // 从dwd挂号信息表中同步患者信息到中间表
        initPatientRegisterMid();

        // 从dwd住院信息表中同步患者信息到中间表
        initPatientInHospitalMid();

        // 从dwd住院申请表中同步患者信息到中间表
        initPatientInHospitalApplyMid();

        // 从中间表同步到目标表
        // deptPatientRelationMapper.saveDeptPatientBatch(); // TODO 为不影响业务暂时只同步到中间表，后面直接通过sql同步到目标表
    }

    private void initPatientRegisterMid() {
        // 从dwd挂号信息表中同步患者信息到中间表
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPage(1); // 从第一页开始
        dataDwdPatientQueryDTO.setRows(10000); // 每次查询10000条
        // this.savePatientRegisterMid(dataDwdPatientQueryDTO);
        this.savePatientRegisterMid(dataDwdPatientQueryDTO);
    }

    private void savePatientRegisterMid(DataDwdPatientQueryDTO dataDwdPatientQueryDTO) {
        log.info("dwd同步患者挂号信息以绑定部门>>>>>> {}", JSON.toJSONString(dataDwdPatientQueryDTO));
        while (true) {
            // dwd查询患者信息
            List<DataDwdPatientRegisterVO> list = doctorDwdPatientDataBizService.patientRegisterSyncList(dataDwdPatientQueryDTO);
            if (CollectionUtil.isEmpty(list)) {
                break;
            }

            // 保存查询的患者信息
            List<DeptPatientRelationAddDTO> patientAddDTOs = new ArrayList<>();
            list.forEach(i -> {
                DeptPatientRelationAddDTO deptPatientAddDTO = new DeptPatientRelationAddDTO();
                deptPatientAddDTO.setPatientId(i.getPatientId());
                deptPatientAddDTO.setDeptId(i.getDepartmentCodeStandard());
                deptPatientAddDTO.setRegisterDt(i.getRegDate());
                deptPatientAddDTO.setType(DeptPatientTypeEnum.Register.getCode());

                patientAddDTOs.add(deptPatientAddDTO);
            });
            // 保存同步信息
            deptPatientRelationMapper.saveDeptPatientMidBatch(patientAddDTOs);

            dataDwdPatientQueryDTO.setId(list.get(list.size() - 1).getId());
        }
    }

    private void initPatientInHospitalMid() {
        // 从dwd住院信息表中同步患者信息到中间表
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPage(1); // 从第一页开始
        dataDwdPatientQueryDTO.setRows(10000); // 每次查询10000条
        // this.savePatientInHospitalMid(dataDwdPatientQueryDTO);
        this.savePatientInHospitalMid(dataDwdPatientQueryDTO);
    }

    private void savePatientInHospitalMid(DataDwdPatientQueryDTO dataDwdPatientQueryDTO) {
        log.info("dwd同步患者住院信息以绑定部门>>>>>> {}", JSON.toJSONString(dataDwdPatientQueryDTO));
        String id = null;
        while (true) {
            dataDwdPatientQueryDTO.setId(id);
            // dwd查询患者信息
            List<DataDwdPatientInHospitalVO> list = doctorDwdPatientDataBizService.patientInHospitalSyncList(dataDwdPatientQueryDTO);
            if (CollectionUtil.isEmpty(list)) {
                return;
            }

            // 保存查询的患者信息
            List<DeptPatientRelationAddDTO> patientAddDTOs = new ArrayList<>();
            list.forEach(i -> {
                DeptPatientRelationAddDTO deptPatientAddDTO = new DeptPatientRelationAddDTO();
                deptPatientAddDTO.setPatientId(i.getPatientId());
                deptPatientAddDTO.setDeptId(i.getAdmitDepartmentCodeStandard());
                deptPatientAddDTO.setRegisterDt(i.getInpatientDate());
                deptPatientAddDTO.setType(DeptPatientTypeEnum.Register.getCode()); // 住院也是挂号类型。

                patientAddDTOs.add(deptPatientAddDTO);
            });
            // 保存同步信息
            deptPatientRelationMapper.saveDeptPatientMidBatch(patientAddDTOs);

            id = list.get(list.size() - 1).getId();
        }
    }

    private void initPatientInHospitalApplyMid() {
        // 从dwd住院申请表中同步患者信息到中间表
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPage(1); // 从第一页开始
        dataDwdPatientQueryDTO.setRows(10000); // 每次查询10000条
        // this.savePatientInHospitalApplyMid(dataDwdPatientQueryDTO);
        this.savePatientInHospitalApplyMid(dataDwdPatientQueryDTO);
    }

    private void savePatientInHospitalApplyMid(DataDwdPatientQueryDTO dataDwdPatientQueryDTO) {
        log.info("dwd同步患者住院申请以绑定部门>>>>>> {}", JSON.toJSONString(dataDwdPatientQueryDTO));
        while (true) {
            // dwd查询患者信息
            List<DataDwdPatientInHospitalApplyVO> list = doctorDwdPatientDataBizService.patientInHospitalApplySyncList(dataDwdPatientQueryDTO);
            if (CollectionUtil.isEmpty(list)) {
                return;
            }
            
            // 保存查询的患者信息
            List<DeptPatientRelationAddDTO> patientAddDTOs = new ArrayList<>();
            list.forEach(i -> {
                DeptPatientRelationAddDTO deptPatientAddDTO = new DeptPatientRelationAddDTO();
                deptPatientAddDTO.setPatientId(i.getPatientId());
                deptPatientAddDTO.setDeptId(i.getApplyDepartmentCodeStandard());
                deptPatientAddDTO.setRegisterDt(i.getApplyDate());
                deptPatientAddDTO.setType(DeptPatientTypeEnum.Register.getCode()); // 住院申请也是挂号类型。

                patientAddDTOs.add(deptPatientAddDTO);
            });
            // 保存同步信息
            deptPatientRelationMapper.saveDeptPatientMidBatch(patientAddDTOs);

            dataDwdPatientQueryDTO.setId(list.get(list.size() - 1).getId());
        }
    }

}
