package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DeptPushServeVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientPushServeDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataQueryAsyncBizService;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
@Async("taskExecutor")
public class PatientDataQueryAsyncBizServiceImpl implements PatientDataQueryAsyncBizService {

    @Autowired
    private PatientDataQueryBizMapper patientDataQueryBizMapper;
    @Autowired
    private DeptPatientQueryBizMapper deptPatientQueryBizMapper;
    @Autowired
    private PatientPushServeDataQueryBizMapper patientPushServeDataQueryBizMapper;

    
    @Override
    public Future<Long> deptAllPatientCount(QueryWrapper<TaskPatientInfo> queryWrapper) {
        Long count = deptPatientQueryBizMapper.deptAllPatientCount(queryWrapper);
        return new AsyncResult<>(count);
    }
    
    @Override
    public Future<Long> patientLabelCount(QueryWrapper<PatientDataOneDayInfo> queryWrapper) {
        Long count = patientDataQueryBizMapper.patientLabelCount(queryWrapper);
        return new AsyncResult<>(count);
    }

    @Override
    public Future<Long> patientHistoryLabelCount(QueryWrapper<PatientDataOneDayInfo> queryWrapper) {
        Long count = patientDataQueryBizMapper.patientHistoryLabelCount(queryWrapper);
        return new AsyncResult<>(count);
    }

    @Override
    public Future<Map<String, Long>> patientHistoryLabelCountV2(QueryWrapper<PatientDataOneDayInfo> queryWrapper) {
        Map<String, Long> historyMap = patientDataQueryBizMapper.patientHistoryLabelCountV2(queryWrapper);
        return new AsyncResult<>(historyMap);
    }

    @Override
    public Future<Map<String, Long>> patientLabelCountV2(QueryWrapper<PatientDataOneDayInfo> queryWrapper) {
        Map<String, Long> historyMap = patientDataQueryBizMapper.patientLabelCountV2(queryWrapper);
        return new AsyncResult<>(historyMap);
    }

    @Override
    public Future<List<Map<String, Object>>> deptPushServeRemindCountMap(QueryWrapper<DeptPushServeVO> queryWrapper) {
        List<Map<String, Object>> list = patientPushServeDataQueryBizMapper.deptPushServeRemindCountMap(queryWrapper);
        return new AsyncResult<>(list);
    }

    @Override
    public Future<List<Map<String, Object>>> deptPushServeScheduleCountMap(QueryWrapper<DeptPushServeVO> queryWrapper) {
        List<Map<String, Object>> list = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(queryWrapper);
        return new AsyncResult<>(list);
    }

    @Override
    public Future<List<Map<String, Object>>> deptPushServeBehaviorCountMap(QueryWrapper<DeptPushServeVO> queryWrapper) {
        List<Map<String, Object>> list = patientPushServeDataQueryBizMapper.deptPushServeBehaviorCountMap(queryWrapper);
        return new AsyncResult<>(list);
    }

}
