package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.common.util.DataUtil;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PushServeTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.constant.LabelStatDimensionTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.PushServeConfigClientService;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.dto.PushServeConfigClientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataDimensionTypeEnum;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataOperatedTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientDataOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientStatisticUserQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DeptPushServeVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientAfterStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDuringStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientLabelDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientPreStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeUserVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientPushServeDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataQueryAsyncBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientDataQueryBizServiceImpl implements PatientDataQueryBizService {

    @Autowired
    private PatientDataOneDayInfoService patientDataOneDayInfoService;
    @Autowired
    private PatientDataQueryBizMapper patientDataQueryBizMapper;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private PatientPushServeDataQueryBizMapper patientPushServeDataQueryBizMapper;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private PushServeConfigClientService pushServeConfigClientService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private DeptPatientQueryBizMapper deptPatientQueryBizMapper;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private PatientDataQueryAsyncBizService patientDataQueryAsyncBizService;

    @Override
    public PatientStatisticVO patientStatistic(PatientStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:params >>>>>> {}", JSON.toJSONString(params));

        PatientStatisticVO vo = new PatientStatisticVO();

        // 就诊患者总数量
        List<String> patientTotalCount = doctorDwdPatientDataBizService.queryVisitPatientCount(params.getDeptId());
        vo.setPatientTotalCount(patientTotalCount.size());

        // 当前潜在患者数量
        Long potentialPatientCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode(), LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
        );
        vo.setPotentialPatientCount(potentialPatientCount.intValue());

        // 发现潜在患者/未就诊
        PatientDataOneDayInfo noVisitCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
        vo.setNotVisitCount(noVisitCount.getNum());
        vo.setNotVisitCountYesterday(noVisitCount.getAddNum());

        // 发现潜在患者/已就诊
        PatientDataOneDayInfo visitedCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
        vo.setVisitedCount(visitedCount.getNum());
        vo.setVisitedCountYesterday(visitedCount.getAddNum());

        // 发现潜在患者/已确诊
        PatientDataOneDayInfo confirmedCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_Pre_Confirmed);
        vo.setConfirmedCount(confirmedCount.getNum());
        vo.setConfirmedCountYesterday(confirmedCount.getAddNum());

        // 就诊转化率: (发现潜在患者/已就诊) / (发现潜在患者/未就诊) * 100%
        String transformRatio = vo.getVisitedCount() == 0 ? "0%" : new BigDecimal(vo.getVisitedCount() + "").divide(new BigDecimal(vo.getNotVisitCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:transformRatio >>>>>> {}", JSON.toJSONString(transformRatio));
        vo.setTransformRatio(transformRatio);
        // 30天平均转化率
        BigDecimal notVisitedCount30 = this.getTransformRatio30(params, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
        BigDecimal visitedCount30 = this.getTransformRatio30(params, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
        String transformRatio30 = notVisitedCount30.intValue() == 0 ? "0%" : visitedCount30.divide(notVisitedCount30, 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientStatistic:transformRatio30 >>>>>> {}", JSON.toJSONString(transformRatio30));
        vo.setTransformRatio30(transformRatio30);

        // 诊中
        // 挂号流失患者
        // 统计本科室对应所有疾病对应的标签分组为“挂号流失患者”所有标签的患者总数量。根据userId去重
        // 挂号流失患者的数据定义为“预约挂号”且“7天内没有该科室就诊记录”的患者
        PatientDataOneDayInfo registerCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Register);
        vo.setRegisterCount(registerCount.getNum());
        vo.setRegisterCountYesterday(registerCount.getAddNum());
        // 住院流失患者
        // 统计本科室对应所有疾病对应的标签分组为“住院流失患者”所有标签的患者总数量。根据userId去重
        // 住院流失患者的数据定义为“提交住院申请单”且“7天内住院就诊记录”的患者
        PatientDataOneDayInfo inHospitalCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_InHospital);
        vo.setInHospitalCount(inHospitalCount.getNum());
        vo.setInHospitalCountYesterday(inHospitalCount.getAddNum());
        // 手术流失患者
        // 统计本科室对应所有疾病对应的标签分组为“手术流失患者”所有标签的患者总数量。根据userId去重
        // 手术流失患者的数据定义为“申请手术”且“7天内没该手术记录”的患者
        PatientDataOneDayInfo operationCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Operation);
        vo.setOperationCount(operationCount.getNum());
        vo.setOperationCountYesterday(operationCount.getAddNum());
        // 流失比例
        // 分子：所有申请单；分母：标签人数
        // 累计人数在此处为流失比例，
        PatientDataOneDayInfo lostRatio = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Lost);
        vo.setLostRatio(lostRatio.getNum() + "%");
        vo.setLostRatioYesterday(lostRatio.getAddNum());

        // 诊后
        // 疾病管理服务次数
        PatientDataOneDayInfo serviceCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Service);
        vo.setServiceCount(serviceCount.getNum());
        vo.setServiceCountYesterday(serviceCount.getAddNum());
        // 服务接受率
        PatientDataOneDayInfo serviceRatio = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Accept);
        vo.setServiceRatio(serviceRatio.getNum() + "%");
        vo.setServiceRatioYesterday(serviceRatio.getAddNum() + "%");
        // 复诊率
        PatientDataOneDayInfo consultationRatio = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Consultation);
        vo.setConsultationRatio(consultationRatio.getNum() + "%");
        vo.setConsultationRatioYesterday(consultationRatio.getAddNum() + "%");
        // 失访人数
        PatientDataOneDayInfo lostCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_After_Lost);
        vo.setLostCount(lostCount.getNum());
        vo.setLostCountYesterday(lostCount.getAddNum());

        return vo;
    }
    // 统计查询
    private PatientDataOneDayInfo getPatientDataOneDayInfo(PatientStatisticQueryDTO params, PatientDataDimensionTypeEnum type) {
        log.info("PatientDataQueryBizServiceImpl.getPatientDataOneDayInfo:params:type >>>>>> {} : {}", JSON.toJSONString(params), type.getCode());
        PatientDataOneDayInfo entity = patientDataOneDayInfoService.getOne(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("data_dimension", type.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
                .last("limit 1")
        );
        if (null == entity) {
            entity = new PatientDataOneDayInfo();
            entity.setNum(0);
            entity.setAddNum(0);
        } else {
            if (null == entity.getNum()) {
                entity.setNum(0);
            }
            if (null == entity.getAddNum()) {
                entity.setAddNum(0);
            }
        }
        log.info("PatientDataQueryBizServiceImpl.getPatientDataOneDayInfo:entity <<<<<< {}", JSON.toJSONString(entity));
        return entity;
    }
    // 30天平均转化率
    private BigDecimal getTransformRatio30(PatientStatisticQueryDTO params, PatientDataDimensionTypeEnum type) {
        List<PatientDataOneDayInfo> list = patientDataOneDayInfoService.list(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("data_dimension", type.getCode())
                .orderByDesc("biz_day")
                .last("limit 30")
        );

        BigDecimal count = new BigDecimal(0);
        list.forEach(i -> count.add(new BigDecimal(i.getNum() + "")));

        return count;
    }

    @Override
    public PageData<TaskPatientVO> patientLabelStatisticPatientList(PatientStatisticPatientQueryDTO params) {
        log.info("1.PatientDataQueryBizServiceImpl.patientLabelStatisticPatientList:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        IPage<TaskPatientInfo> page = null;
        
        // 部门患者总数量（门诊/急诊/住院患者总数量）
        if (PatientDataDimensionTypeEnum.Total_Patient.getCode().equals(params.getLabelStatDimensionType())) {
            page = this.totalPatientQuery(params);
        }
        // 潜在患者（当前标签为未就诊+已就诊）
        if (PatientDataDimensionTypeEnum.Potential_Patient.getCode().equals(params.getLabelStatDimensionType())) {
            page = this.potentialPatientQuery(params);
        }
        // 诊前
        if (
            PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit.getCode().equals(params.getLabelStatDimensionType()) 
            || PatientDataDimensionTypeEnum.LSD_Pre_Visited.getCode().equals(params.getLabelStatDimensionType())
            || PatientDataDimensionTypeEnum.LSD_Pre_Confirmed.getCode().equals(params.getLabelStatDimensionType())
        ) {
            page = this.prePatientQuery(params);
        }
        // 诊中 TODO 暂时没有按病种查询
        if (
            PatientDataDimensionTypeEnum.LSD_During_Register.getCode().equals(params.getLabelStatDimensionType()) 
            || PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode().equals(params.getLabelStatDimensionType())
            || PatientDataDimensionTypeEnum.LSD_During_Operation.getCode().equals(params.getLabelStatDimensionType())
        ) {
            page = this.duringPatientQuery(params);
        }
        // TODO 诊后
        // TODO 疾病管理服务

        // TODO 失访
        if (PatientDataDimensionTypeEnum.LSD_After_Lost.getCode().equals(params.getLabelStatDimensionType())) {
            page = this.afterLostPatientQuery(params);
        }


        if (page == null) {
            return null;
        }

        List<TaskPatientInfo> list = page.getRecords();
        // Map<String, DoctorDwdPatientVO> dwdPatientMap = new HashMap<>(); // dwd获取患者
        Map<String, TaskPatientVO> patientMap = new HashMap<>(); // 获取患者
        Map<String, List<PatientLabelDataVO>> patientLabelMap = new HashMap<>(); // 患者标签
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> patientIds = list.stream().map(TaskPatientInfo::getPatientId).collect(Collectors.toList());

            // 查询患者信息
            TaskPatientQueryDTO taskPatientQueryDTO = new TaskPatientQueryDTO();
            taskPatientQueryDTO.setPatientIds(patientIds);
            List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(taskPatientQueryDTO);
            if (CollectionUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }

            // 查询患者标签
            if (!PatientDataDimensionTypeEnum.Total_Patient.getCode().equals(params.getLabelStatDimensionType())) {
                QueryWrapper<PatientLabelDataVO> queryWrapper = new QueryWrapper<PatientLabelDataVO>()
                        .in("a.patient_id", patientIds)
                        .like(StrUtil.isNotBlank(params.getDiseaseId()), "c.disease_ids", "," + params.getDiseaseId() + ",");
                if (PatientDataDimensionTypeEnum.Potential_Patient.getCode().equals(params.getLabelStatDimensionType())) {
                    queryWrapper.in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit.getCode(), PatientDataDimensionTypeEnum.LSD_Pre_Visited.getCode());
                } else {
                    queryWrapper.eq("c.stat_dimension_type", params.getLabelStatDimensionType());
                }
                
                List<PatientLabelDataVO> patientLabelDataList = null;
                if (YN.YES.getCode().equals(params.getIsNowLabel())) { // 当前标签
                    patientLabelDataList = patientDataQueryBizMapper.patientLabelList(queryWrapper);
                } else {
                    patientLabelDataList = patientDataQueryBizMapper.patientHistoryLabelList(queryWrapper);
                }
                if (CollectionUtil.isNotEmpty(patientLabelDataList)) {
                    List<String> labelIds = new ArrayList<>();
                    patientLabelDataList.forEach(i -> labelIds.add(i.getLabelId()));

                    LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
                    labelQueryPageDTO.setIds(labelIds);
                    List<LabelBO> labelBOList = labelClientService.queryList(labelQueryPageDTO);
                    if (CollectionUtil.isNotEmpty(labelBOList)) {
                        Map<String, LabelBO> labelBOMap = new HashMap<>();
                        labelBOList.forEach(i -> labelBOMap.put(i.getId(), i));

                        patientLabelDataList.forEach(i -> {
                            i.setLabelBO(labelBOMap.get(i.getLabelId()));

                            List<PatientLabelDataVO> patientLabelDataVOs = patientLabelMap.get(i.getPatientId());
                            if (null == patientLabelDataVOs) {
                                patientLabelDataVOs = new ArrayList<>();
                                patientLabelMap.put(i.getPatientId(), patientLabelDataVOs);
                            }
                            patientLabelDataVOs.add(i);
                        });
                    }
                }
            }
        }

        PageData<TaskPatientVO> data = new PageConverter<TaskPatientInfo, TaskPatientVO>() {
            @Override
            public TaskPatientVO converter(TaskPatientInfo entity) {
                TaskPatientVO vo = DoctorDataHelper.getTaskPatientVO(entity);
                
                // 20240307 不从dwd数仓获取用户信息
                // DoctorDwdPatientVO dwdPatient = dwdPatientMap.get(vo.getPatientId());
                // if (null != dwdPatient) {
                //     vo.setBirthday(dwdPatient.getBirthday());
                //     vo.setPatientGenderStandard(dwdPatient.getPatientGenderStandard());
                //     vo.setPatientName(dwdPatient.getPatientName());
                //     vo.setPhone(dwdPatient.getPhone());
                //     vo.setAge(dwdPatient.getPatientAge());
                // }
                TaskPatientVO patient = patientMap.get(vo.getPatientId());
                if (null != patient) {
                    vo.setBirthday(patient.getBirthday());
                    vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                    vo.setPatientName(patient.getPatientName());
                    vo.setPhone(patient.getPhone());
                    vo.setSparePhone(patient.getSparePhone());
                    vo.setLinkPhone(patient.getLinkPhone());
                    vo.setHomePhone(patient.getHomePhone());
                    vo.setWorkPhone(patient.getWorkPhone());
                    vo.setAge(patient.getAge());
                }

                vo.setPatientLabelInfoList(patientLabelMap.get(vo.getPatientId()));

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    /**
     * 部门总患者
     * 从mysql的部门患者关联表查询
     * 门诊/急诊/住院患者
     */
    private IPage<TaskPatientInfo> totalPatientQuery(PatientStatisticPatientQueryDTO params) {
        return patientDataQueryBizMapper.queryDeptAllPatient(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                .eq("e.dept_id", params.getDeptId())
                .eq("e.type", DeptPatientTypeEnum.Register.getCode()) // 只查询挂号住院手术等患者，不包括打标签关联的患者
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
        );
    }

    /**
     * 部门当前总潜在患者
     * 从当前标签中查询
     * 未就诊/已就诊标签患者
     */
    private IPage<TaskPatientInfo> potentialPatientQuery(PatientStatisticPatientQueryDTO params) {
        return patientDataQueryBizMapper.patientLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit.getCode(), PatientDataDimensionTypeEnum.LSD_Pre_Visited.getCode())
                .eq("b.dept_id", params.getDeptId())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
        );
    }

    /**
     * 部门诊前患者
     * 未就诊/已就诊/已确诊标签患者
     */
    private IPage<TaskPatientInfo> prePatientQuery(PatientStatisticPatientQueryDTO params) {
        // 查询当前标签
        if (YN.YES.getCode().equals(params.getIsNowLabel())) {
            return patientDataQueryBizMapper.patientLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                    .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                    .eq("b.dept_id", params.getDeptId())
                    .like(StrUtil.isNotBlank(params.getDiseaseId()), "c.disease_ids", "," + params.getDiseaseId() + ",")
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "d.create_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "d.create_date", params.getEndDate())
                    .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                    .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
            );
        }

        // 查询历史标签
        return patientDataQueryBizMapper.patientHistoryLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                .eq("d.op_type", OpTypeEnum.Create.getCode())
                .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                .eq("b.dept_id", params.getDeptId())
                .like(StrUtil.isNotBlank(params.getDiseaseId()), "c.disease_ids", "," + params.getDiseaseId() + ",")
                .ge(StrUtil.isNotBlank(params.getStartDate()), "d.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "d.op_date", params.getEndDate())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
        );
    }
    
    /**
     * 部门诊中患者
     * 挂号流失/住院流失/手术流失标签患者
     */
    private IPage<TaskPatientInfo> duringPatientQuery(PatientStatisticPatientQueryDTO params) {
        // return patientDataQueryBizMapper.patientDuringHistoryLabelStatisticPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
        //         .eq("a.op_type", OpTypeEnum.Create.getCode())
        //         .eq("b.stat_dimension_type", params.getLabelStatDimensionType())
        //         .eq("c.dept_id", params.getDeptId())
        //         .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
        //         .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate()),
        //         OpTypeEnum.Create.getCode()
        // );
        // 查询当前标签
        if (YN.YES.getCode().equals(params.getIsNowLabel())) {
            return patientDataQueryBizMapper.patientLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                    .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                    .eq("b.dept_id", params.getDeptId())
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "d.create_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "d.create_date", params.getEndDate())
                    .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                    .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
            );
        }

        // 查询历史标签
        return patientDataQueryBizMapper.patientHistoryLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                .eq("d.op_type", OpTypeEnum.Create.getCode())
                .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "d.op_date", params.getStartDate())
                .ge(StrUtil.isNotBlank(params.getEndDate()), "d.op_date", params.getEndDate())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
        );
    }
    
    /**
     * 部门诊后患者
     * 失访标签患者
     */
    private IPage<TaskPatientInfo> afterLostPatientQuery(PatientStatisticPatientQueryDTO params) {
        // 查询当前标签
        if (YN.YES.getCode().equals(params.getIsNowLabel())) {
            return patientDataQueryBizMapper.patientLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                    .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                    .eq("b.dept_id", params.getDeptId())
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "d.create_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "d.create_date", params.getEndDate())
                    .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                    .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
            );
        }

        // 查询历史标签
        return patientDataQueryBizMapper.patientHistoryLabelPatientList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                .eq("d.op_type", OpTypeEnum.Create.getCode())
                .eq("c.stat_dimension_type", params.getLabelStatDimensionType())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "d.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "d.op_date", params.getEndDate())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPatientPhone()), "a.phone", params.getPatientPhone())
        );
    }
    

    @Override
    public PageData<PushServeUserVO> patientLabelStatisticUserList(PatientStatisticUserQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        // 查询与服务是否匹配
        DeptBO deptBO = deptClientService.getDeptById(params.getDeptId());
        log.info("1.查询部门信息<<<<<< deptBO={}", JSON.toJSONString(deptBO));
        if (null == deptBO || CollectionUtil.isEmpty(deptBO.getDiseaseIds())) {
            return null;
        }
        PushServeConfigBO pushServeConfigBO = pushServeConfigClientService.getById(params.getPushServeId());
        log.info("1.查询推送服务配置信息<<<<<< pushServeConfigBO={}", JSON.toJSONString(pushServeConfigBO));
        if (null == pushServeConfigBO || CollectionUtil.isEmpty(pushServeConfigBO.getDiseaseIds())) {
            return null;
        }
        boolean isValidity = false;
        for(String diseaseId : deptBO.getDiseaseIds()) {
            if (pushServeConfigBO.getDiseaseIds().contains(diseaseId)) {
                isValidity = true;
            }
        }
        log.info("3.部门是否和推送服务匹配<<<<<< isValidity={}", isValidity);
        if (!isValidity) {
            return null;
        }

        
        IPage<PushServeUserVO>  page = null;
        // 未关注推送失败用户列表
        if (PatientDataDimensionTypeEnum.Not_Following_Push_Fail.getCode().equals(params.getLabelStatDimensionType())) {
            page = patientPushServeDataQueryBizMapper.notFollowingPushFailUserList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DeptPushServeVO>()
                    .eq("a.rows_status", YN.YES.getCode())
                    .eq("b.op_type", OpTypeEnum.Create.getCode())
                    .eq(StrUtil.isNotBlank(params.getPushServeId()), "b.push_serve_id", params.getPushServeId())
                    .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.NotFollow.getCode())
            );
        }
        // 发送失败用户列表
        if (PatientDataDimensionTypeEnum.Push_Fail.getCode().equals(params.getLabelStatDimensionType())) {
            page = patientPushServeDataQueryBizMapper.pushFailUserList(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DeptPushServeVO>()
                    .eq("a.rows_status", YN.YES.getCode())
                    .eq("b.op_type", OpTypeEnum.Create.getCode())
                    .eq(StrUtil.isNotBlank(params.getPushServeId()), "b.push_serve_id", params.getPushServeId())
                    .in("a.push_status", PatientPushServeConstant.RemindPushStatus.Fail.getCode(), PatientPushServeConstant.RemindPushStatus.Invalid.getCode(), PatientPushServeConstant.RemindPushStatus.NotFollow.getCode(), PatientPushServeConstant.RemindPushStatus.Expire.getCode())
            );
        }

        log.info("4.查询结果<<<<<< ");
        if (page == null || CollectionUtil.isEmpty(page.getRecords())) {
            return null;
        }

        PageData<PushServeUserVO> data = new PageConverter<PushServeUserVO, PushServeUserVO>() {
            @Override
            public PushServeUserVO converter(PushServeUserVO entity) {
                entity.setPatientAge(DataUtil.getAge(entity.getIdCardNo(), null));
                entity.setIdCardNo(null);
                return entity;
            }
        }.getPageData(page);
        return data;
    }
    



    ////////////////// 以下实时统计



    @Override
    public PatientPreStatisticVO patientPreStatisticByDeptId(PatientStatisticQueryDTO params) {
        PatientPreStatisticVO vo = new PatientPreStatisticVO();

        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr());
        }

        // dwd就诊患者总数量，弃用
        // List<String> patientTotalCount = doctorDwdPatientDataBizService.queryVisitPatientCount(params.getDeptId());
        // vo.setPatientTotalCount(patientTotalCount.size());

        // 就诊患者总数量，中部门患者关联表查
        Long patientTotalCount = deptPatientQueryBizMapper.deptAllPatientCount(new QueryWrapper<TaskPatientInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("type", DeptPatientTypeEnum.Register.getCode())
        );
        vo.setPatientTotalCount(patientTotalCount);

        // 当前潜在患者数量
        Long potentialPatientCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit.getCode(), PatientDataDimensionTypeEnum.LSD_Pre_Visited.getCode())
                .eq("b.dept_id", params.getDeptId())
        );
        vo.setPotentialPatientCount(potentialPatientCount);
        

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        // TODO 标签数据量庞大，直接查影响性能，可根据部门标签来查询来限定标签的数据量
        List<String> notVisitLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/未就诊标签
        List<String> visitedLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/已就诊标签
        List<String> confirmedLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/已确诊标签
        // 有病种参数则根据病种筛选出相关标签
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(params.getDiseaseId())) {
                        if (LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode().equals(i.getStatDimensionType())) {
                            notVisitLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode().equals(i.getStatDimensionType())) {
                            visitedLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode().equals(i.getStatDimensionType())) {
                            confirmedLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }
        
        // 诊前
        // 发现潜在患者/未就诊
        Long historyNotVisitCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            historyNotVisitCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        vo.setHistoryNotVisitCount(historyNotVisitCount); // 历史上所有该标签的患者
        Long nowNotVisitCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            nowNotVisitCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }
        vo.setNowNotVisitCount(nowNotVisitCount); // 当前所有该标签的患者
        Long newNotVisitCountYesterday = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            newNotVisitCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }
        vo.setNewNotVisitCountYesterday(newNotVisitCountYesterday); // 昨天被打上该标签的患者

        // 发现潜在患者/已就诊
        Long historyVisitedCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            historyVisitedCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        vo.setHistoryVisitedCount(historyVisitedCount); // 历史上所有该标签的患者
        Long nowVisitedCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            nowVisitedCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }
        vo.setNowVisitedCount(nowVisitedCount); // 当前所有该标签的患者
        Long newVisitedCountYesterday = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            newVisitedCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }
        vo.setNewVisitedCountYesterday(newVisitedCountYesterday); // 昨天被打上该标签的患者

        // 发现潜在患者/已确诊
        Long historyConfirmedCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            historyConfirmedCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        vo.setHistoryConfirmedCount(historyConfirmedCount); // 历史上所有该标签的患者
        Long nowConfirmedCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            nowConfirmedCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
            );
        }
        vo.setNowConfirmedCount(nowConfirmedCount); // 当前所有该标签的患者
        Long newConfirmedCountYesterday = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            newConfirmedCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
            );
        }
        vo.setNewConfirmedCountYesterday(newConfirmedCountYesterday); // 昨天被打上该标签的患者

        // 就诊转化率: (发现潜在患者/已就诊) / (发现潜在患者/未就诊) * 100%
        String transformRatio = vo.getHistoryNotVisitCount() == 0 ? "0%" : new BigDecimal(vo.getHistoryVisitedCount() + "").divide(new BigDecimal(vo.getHistoryNotVisitCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientPreStatisticByDeptId:transformRatio >>>>>> {}", JSON.toJSONString(transformRatio));
        vo.setTransformRatio(transformRatio);

        // 30天平均转化率
        // 30天内未就诊
        Long notVisitedCount30 = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            notVisitedCount30 = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", DateUtil.offsetDay(now, -30).toDateStr())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }
        // 30天内已就诊
        Long visitedCount30 = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            visitedCount30 = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", DateUtil.offsetDay(now, -30).toDateStr())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }
        String transformRatio30 = null == notVisitedCount30 || notVisitedCount30 == 0 ? "0%" : new BigDecimal(visitedCount30 + "").divide(new BigDecimal(notVisitedCount30 + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientPreStatisticByDeptId:transformRatio30 >>>>>> {}", JSON.toJSONString(transformRatio30));
        vo.setTransformRatio30(transformRatio30);

        return vo;
    }

    // @Override
    // public PatientDuringStatisticVO patientDuringStatisticByDeptId(PatientStatisticQueryDTO params) {
    //     log.info("PatientDataQueryBizServiceImpl.patientDuringStatisticByDeptId:params >>>>>> {}", JSON.toJSONString(params));

    //     PatientDuringStatisticVO vo = new PatientDuringStatisticVO();

    //     DateTime now = DateUtil.date();
    //     String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
    //     String nowDate = now.toDateStr();

    //     if (StrUtil.isNotBlank(params.getEndDate())) {
    //         params.setEndDate(DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr());
    //     }

    //     // 诊中
    //     // 挂号流失患者；标签id为部门id+统计维度分类
    //     // 统计本科室对应所有疾病对应的标签分组为“挂号流失患者”所有标签的患者总数量。根据userId去重
    //     // 挂号流失患者的数据定义为“预约挂号”且“7天内没有该科室就诊记录”的患者
    //     // PatientDataOneDayInfo registerCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Register);
    //     Long historyRegisterCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode()) // 挂号流失的标签id
    //             .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
    //             .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setHistoryRegisterCount(historyRegisterCount);
    //     Long nowNotRegisterCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("b.dept_id", params.getDeptId())
    //             .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
    //     );
    //     vo.setNowRegisterCount(nowNotRegisterCount); // 当前所有该标签的患者
    //     Long newRegisterCountYesterday = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode())
    //             .ge("a.op_date", yesterdayDate)
    //             .lt("a.op_date", nowDate), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setNewRegisterCountYesterday(newRegisterCountYesterday);

    //     // 住院流失患者
    //     // 统计本科室对应所有疾病对应的标签分组为“住院流失患者”所有标签的患者总数量。根据userId去重
    //     // 住院流失患者的数据定义为“提交住院申请单”且“7天内住院就诊记录”的患者
    //     // PatientDataOneDayInfo inHospitalCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_InHospital);
    //     Long historyInHospitalCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
    //             .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
    //             .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setHistoryInHospitalCount(historyInHospitalCount);
    //     Long nowInHospitalCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("b.dept_id", params.getDeptId())
    //             .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
    //     );
    //     vo.setNowInHospitalCount(nowInHospitalCount);
    //     Long newInHospitalCountYesterday = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
    //             .ge("a.op_date", yesterdayDate)
    //             .lt("a.op_date", nowDate), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setNewInHospitalCountYesterday(newInHospitalCountYesterday);

    //     // 手术流失患者
    //     // 统计本科室对应所有疾病对应的标签分组为“手术流失患者”所有标签的患者总数量。根据userId去重
    //     // 手术流失患者的数据定义为“申请手术”且“7天内没该手术记录”的患者
    //     // PatientDataOneDayInfo operationCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Operation);
    //     Long historyOperationCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
    //             .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
    //             .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setHistoryOperationCount(historyOperationCount);
    //     Long nowOperationCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("b.dept_id", params.getDeptId())
    //             .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
    //     );
    //     vo.setNowOperationCount(nowOperationCount);
    //     Long newOperationCountYesterday = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
    //             .ge("a.op_date", yesterdayDate)
    //             .lt("a.op_date", nowDate), 
    //             OpTypeEnum.Create.getCode()
    //     );
    //     vo.setNewOperationCountYesterday(newOperationCountYesterday);

    //     // 流失比例
    //     // 分子：所有申请单；分母：标签人数
    //     // 申请单从数仓中获取：挂号申请/住院申请/手术申请；流失标签从所有历史标签中获取
    //     // 申请单总数
    //     Long allApplyCount = doctorDwdPatientDataBizService.queryApplyCount(params.getDeptId());
    //     // 历史流失标签人数
    //     Long historyLostCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.op_type", OpTypeEnum.Create.getCode())
    //             .eq("b.dept_id", params.getDeptId())
    //             .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
    //             .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
    //             .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
    //     );
    //     vo.setHistoryLostRatio(null == allApplyCount || 0 == allApplyCount ? "0%" : new BigDecimal(historyLostCount).divide(new BigDecimal(allApplyCount), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)) + "%");

    //     // 昨日流失标签人数
    //     Long newNotVisitCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
    //             .eq("a.op_type", OpTypeEnum.Create.getCode())
    //             .ge("a.op_date", yesterdayDate)
    //             .lt("a.op_date", nowDate)
    //             .eq("b.dept_id", params.getDeptId())
    //             .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
    //     );
    //     vo.setNewLostYesterday(newNotVisitCountYesterday);

    //     return vo;
    // }
    @Override
    public PatientDuringStatisticVO patientDuringStatisticByDeptId(PatientStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientDuringStatisticByDeptId:params >>>>>> {}", JSON.toJSONString(params));

        PatientDuringStatisticVO vo = new PatientDuringStatisticVO();

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr());
        }

        List<String> registerLabelIds = new ArrayList<>(); // 筛选出挂号流失患者
        List<String> inhospitalLabelIds = new ArrayList<>(); // 筛选出住院流失患者
        List<String> operationLabelIds = new ArrayList<>(); // 筛选出手术流失患者
        // 根据病种筛选出相关标签
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(params.getDiseaseId())) {
                        if (LabelStatDimensionTypeEnum.LSD_During_Register.getCode().equals(i.getStatDimensionType())) {
                            registerLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode().equals(i.getStatDimensionType())) {
                            inhospitalLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_During_Operation.getCode().equals(i.getStatDimensionType())) {
                            operationLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }

        log.info("1.挂号流失标签>>>>>> {}", JSON.toJSONString(registerLabelIds));
        log.info("2.住院流失标签>>>>>> {}", JSON.toJSONString(inhospitalLabelIds));
        log.info("3.手术流失标签>>>>>> {}", JSON.toJSONString(operationLabelIds));
        
        // 诊中
        // 挂号流失患者；标签id为部门id+统计维度分类
        // 统计本科室对应所有疾病对应的标签分组为“挂号流失患者”所有标签的患者总数量。根据userId去重
        // 挂号流失患者的数据定义为“预约挂号”且“7天内没有该科室就诊记录”的患者
        // PatientDataOneDayInfo registerCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Register);
        // Long historyRegisterCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        //         .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode()) // 挂号流失的标签id
        //         .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
        //         .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
        //         OpTypeEnum.Create.getCode()
        // );
        // 历史流失标签患者
        Long historyRegisterCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode()) // 挂号流失的标签id
                .eq("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_During_Register.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );
        log.info("4.历史流失标签患者数量>>>>>> {}", historyRegisterCount);
        vo.setHistoryRegisterCount(historyRegisterCount);
        // 当前流失标签患者
        Long nowNotRegisterCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.dept_id", params.getDeptId())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
        );
        log.info("5.当前流失标签患者数量>>>>>> {}", nowNotRegisterCount);
        vo.setNowRegisterCount(nowNotRegisterCount);
        // 昨日流失标签患者
        // Long newRegisterCountYesterday = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        //         .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode())
        //         .ge("a.op_date", yesterdayDate)
        //         .lt("a.op_date", nowDate), 
        //         OpTypeEnum.Create.getCode()
        // );
        Long newRegisterCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
        );
        log.info("6.昨日流失标签患者数量>>>>>> {}", newRegisterCountYesterday);
        vo.setNewRegisterCountYesterday(newRegisterCountYesterday);

        // 住院流失患者
        // 统计本科室对应所有疾病对应的标签分组为“住院流失患者”所有标签的患者总数量。根据userId去重
        // 住院流失患者的数据定义为“提交住院申请单”且“7天内住院就诊记录”的患者
        // PatientDataOneDayInfo inHospitalCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_InHospital);
        // Long historyInHospitalCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        //         .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
        //         .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
        //         .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
        //         OpTypeEnum.Create.getCode()
        // );
        Long historyInHospitalCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );
        log.info("7.历史住院流失患者数量>>>>>> {}", historyInHospitalCount);
        vo.setHistoryInHospitalCount(historyInHospitalCount);
        Long nowInHospitalCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.dept_id", params.getDeptId())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
        );
        log.info("8.当前住院流失患者数量>>>>>> {}", nowInHospitalCount);
        vo.setNowInHospitalCount(nowInHospitalCount);
        Long newInHospitalCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
        );
        log.info("9.昨日新增住院流失患者数量>>>>>> {}", newInHospitalCountYesterday);
        vo.setNewInHospitalCountYesterday(newInHospitalCountYesterday);


        // 住院流失（直接打标签的流失）
        Long otherHistoryInHospitalCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherHistoryInHospitalCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", "")) // 自动打的流失标签此字段没有值，依此区分
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        log.info("10.历史住院流失患者标签数量>>>>>> {}", otherHistoryInHospitalCount);
        vo.setOtherHistoryInHospitalCount(otherHistoryInHospitalCount); // 历史上所有该标签的患者
        Long otherNowNotVisitCount = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherNowNotVisitCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", ""))
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
            );
        }
        log.info("11.当前住院流失患者标签数量>>>>>> {}", otherNowNotVisitCount);
        vo.setOtherNowInHospitalCount(otherNowNotVisitCount); // 当前所有该标签的患者
        Long otherNewNotVisitCountYesterday = 0L;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherNewNotVisitCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", ""))
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
            );
        }
        log.info("12.昨日新增住院流失患者标签数量>>>>>> {}", otherNewNotVisitCountYesterday);
        vo.setOtherNewInHospitalCountYesterday(otherNewNotVisitCountYesterday); // 昨天被打上该标签的患者




        
        // 手术流失患者
        // 统计本科室对应所有疾病对应的标签分组为“手术流失患者”所有标签的患者总数量。根据userId去重
        // 手术流失患者的数据定义为“申请手术”且“7天内没该手术记录”的患者
        // PatientDataOneDayInfo operationCount = this.getPatientDataOneDayInfo(params, PatientDataDimensionTypeEnum.LSD_During_Operation);
        // Long historyOperationCount = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        //         .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
        //         .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
        //         .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate()), 
        //         OpTypeEnum.Create.getCode()
        // );
        Long historyOperationCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );
        log.info("13.历史手术流失患者数量>>>>>> {}", historyOperationCount);
        vo.setHistoryOperationCount(historyOperationCount);
        Long nowOperationCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.dept_id", params.getDeptId())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
        );
        log.info("14.当前手术流失患者数量>>>>>> {}", nowOperationCount);
        vo.setNowOperationCount(nowOperationCount);
        // Long newOperationCountYesterday = patientDataQueryBizMapper.patientLostHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        //         .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
        //         .ge("a.op_date", yesterdayDate)
        //         .lt("a.op_date", nowDate), 
        //         OpTypeEnum.Create.getCode()
        // );
        Long newOperationCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode())
                .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
        );
        log.info("15.昨日新增手术流失患者数量>>>>>> {}", newOperationCountYesterday);
        vo.setNewOperationCountYesterday(newOperationCountYesterday);

        // 流失比例
        // 分子：所有申请单；分母：标签人数
        // 申请单从数仓中获取：挂号申请/住院申请/手术申请；流失标签从所有历史标签中获取
        // 申请单总数
        Long allApplyCount = doctorDwdPatientDataBizService.queryApplyCount(params.getDeptId());
        // 历史流失标签人数
        Long historyLostCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                // .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())  // 在某个时间段内的流失数量与当前历史的流失
                // .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );
        log.info("16.流失比例>>>>>> allApplyCount={} : historyLostCount={}", allApplyCount);
        vo.setHistoryLostRatio((null == allApplyCount || 0 == allApplyCount) ? "0%" : new BigDecimal(historyLostCount).divide(new BigDecimal(allApplyCount), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)) + "%");

        // 昨日流失标签人数
        Long newNotVisitCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
        );
        vo.setNewLostYesterday(newNotVisitCountYesterday);

        return vo;
    }

    @Override
    public PatientAfterStatisticVO patientAfterStatisticByDeptId(PatientStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientAfterStatisticByDeptId:params >>>>>> {}", JSON.toJSONString(params));

        PatientAfterStatisticVO vo = new PatientAfterStatisticVO();
        vo.setConsultationRatio("-");
        vo.setConsultationRatioYesterday("-");
        vo.setLostCount(0L);
        vo.setLostCountYesterday(0L);
        vo.setServiceRatio("-");
        vo.setServiceRatioYesterday("-");
        vo.setServiceCount(0);
        vo.setServiceCountYesterday(0);

        List<String> pushServeTypeList = new ArrayList<>();
        pushServeTypeList.add(PushServeTypeEnum.Push_article.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visited.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visit_push.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Medicine.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Get_lost.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Satisfaction.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Dr_service.getCode());
        PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
        pushServeConfigClientQueryDTO.setTypes(pushServeTypeList);
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            pushServeConfigClientQueryDTO.setDiseaseId(params.getDiseaseId());
        }
        // 查询部门下可管理的推送服务
        List<DeptPushServeVO> deptPushServeVOs = this.getDeptPushServeList(params.getDeptId(), pushServeConfigClientQueryDTO);
        
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());
        // 诊后
        if (CollectionUtil.isNotEmpty(pushServeIds)) {
            // 疾病管理服务次数
            this.getServiceCount(pushServeIds, vo, params.getStartDate(), params.getEndDate());

            // TODO 服务接受率：服务阅读次数/服务推送次数
            this.getServiceRatio(pushServeIds, vo, params.getStartDate(), params.getEndDate());
        }

        
        // TODO 复诊率
        
        // 失访人数
        this.getLostVisitCount(vo, params.getDeptId(), params.getDiseaseId(), params.getStartDate(), params.getEndDate());

        return vo;
    }

    // 疾病管理服务次数
    private void getServiceCount(List<String> pushServeIds, PatientAfterStatisticVO vo, String startDate, String endDate) {
        DateTime now = DateUtil.date();
        // 所有诊后服务数据
        Map<String, Integer> allData = this.patientPushServeDataMap(pushServeIds, now, startDate, endDate);
        // 昨日新增诊后服务数据
        Map<String, Integer> yesterdayNewData = this.patientPushServeDataMap(pushServeIds, now, DateUtil.offsetDay(now, -1).toDateStr(), now.toDateStr());

        pushServeIds.forEach(i -> {
            Integer num = allData.get(i);
            vo.setServiceCount(null == num ? vo.getServiceCount() : num + vo.getServiceCount());
            
            Integer yesterdayNum = yesterdayNewData.get(i);
            vo.setServiceCountYesterday(null == yesterdayNum ? vo.getServiceCountYesterday() : yesterdayNum + vo.getServiceCountYesterday());
        });
    }

    // 服务接受率
    private void getServiceRatio(List<String> pushServeIds, PatientAfterStatisticVO vo, String startDate, String endDate) {
        DateTime now = DateUtil.date();
        // 诊后服务阅读次数 // TODO 一个服务每个用户可以阅读多次，比率就会很大，是否相同的用户阅读记为一次
        Map<String, Integer> allData = this.patientPushServeReadDataMap(pushServeIds, startDate, endDate);
        // 昨日诊后服务阅读次数
        Map<String, Integer> yesterdayNewData = this.patientPushServeReadDataMap(pushServeIds, DateUtil.offsetDay(now, -1).toDateStr(), now.toDateStr());

        Integer totalNum = 0;
        Integer totalYesterdayNum = 0;
        for(String pushServeId : pushServeIds) {
            Integer num = allData.get(pushServeId);
            if (null == num) {
                num = 0;
            }
            Integer yesterdayNum = yesterdayNewData.get(pushServeId);
            if (null == yesterdayNum) {
                yesterdayNum = 0;
            }
            totalNum = totalNum + num;
            totalYesterdayNum = totalYesterdayNum + yesterdayNum;
        }
        vo.setServiceRatio(null == vo.getServiceCount() || 0== vo.getServiceCount() ? "0%" : new BigDecimal(totalNum + "").divide(new BigDecimal(vo.getServiceCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%");
        vo.setServiceRatioYesterday(null == vo.getServiceCountYesterday() || 0 == vo.getServiceCountYesterday() ? "0%" : new BigDecimal(totalYesterdayNum + "").divide(new BigDecimal(vo.getServiceCountYesterday() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%");
    }

    // 失访人数
    private void getLostVisitCount(PatientAfterStatisticVO vo, String deptId, String diseaseId, String startDate, String endDate) {

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        List<String> lostLabelIds = new ArrayList<>(); // 筛选出失访患者
        // 根据病种筛选出相关标签
        if (StrUtil.isNotBlank(diseaseId)) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_After_Lost.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(diseaseId)) {
                        if (LabelStatDimensionTypeEnum.LSD_After_Lost.getCode().equals(i.getStatDimensionType())) {
                            lostLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }
        
        Long historyLostCount = 0L;
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            historyLostCount = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
                    .ge(StrUtil.isNotBlank(startDate), "a.op_date", startDate)
                    .lt(StrUtil.isNotBlank(endDate), "a.op_date", endDate)
            );
        }
        vo.setLostCount(historyLostCount); // 历史上所有该标签的患者
        Long nowLostCount = 0L;
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            nowLostCount = patientDataQueryBizMapper.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
            );
        }
        vo.setNowLostCount(nowLostCount); // 当前所有该标签的患者
        Long newLostCountYesterday = 0L;
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            newLostCountYesterday = patientDataQueryBizMapper.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
            );
        }
        vo.setLostCountYesterday(newLostCountYesterday); // 昨天被打上该标签的患者
    }

    // 处理部门与推送服务关联关系
    private List<DeptPushServeVO> getDeptPushServeList(String deptId, PushServeConfigClientQueryDTO pushServeConfigParams) {
        DeptBO deptBO = deptClientService.getDeptById(deptId);
        log.info("1>>>>>>>> deptBO");
        List<PushServeConfigBO> pushServeList = pushServeConfigClientService.list(pushServeConfigParams);
        log.info("2>>>>>>>> List<PushServeConfigBO>={}", pushServeList.size());
        
        if (null == deptBO || CollectionUtil.isEmpty(pushServeList)) {
            return new ArrayList<>(0);
        }

        List<DeptPushServeVO> list = new ArrayList<>();

        List<String> deptDiseaseIds = deptBO.getDiseaseIds();
        if (CollectionUtil.isNotEmpty(deptDiseaseIds)) {
            pushServeList.forEach(i -> {
                List<String> pushServeDiseaseIds = i.getDiseaseIds();
                if (CollectionUtil.isNotEmpty(pushServeDiseaseIds)) {
                    for (String id : pushServeDiseaseIds) {
                        if (deptDiseaseIds.contains(id)) {
                            DeptPushServeVO vo = new DeptPushServeVO();
                            vo.setDeptId(deptBO.getId());
                            vo.setDiseaseIds(pushServeDiseaseIds);
                            vo.setMediaId(i.getMediaId());
                            vo.setMediaName(i.getMediaName());
                            vo.setMediaType(i.getMediaType());
                            vo.setName(i.getName());
                            vo.setPushServeId(i.getId());
                            vo.setStatus(i.getStatus());
                            vo.setTitle(i.getTitle());
                            vo.setType(i.getType());
                            list.add(vo);
                            break;
                        }
                    }
                }
                
            });
        }
        return list;
    }

    // 诊后服务数据
    private Map<String, Integer> patientPushServeDataMap(List<String> pushServeIds, DateTime now, String startDate, String endDate) {
        log.info("1.查询诊后患者服务>>>>>> {} : {} : {} : {}", JSON.toJSONString(pushServeIds), now, startDate, endDate);

        Map<String, Integer> pushServeDataMap = new HashMap<>();
        // 1.有提醒从提醒中查询发送成功的数据
        List<Map<String, Object>> pushServeRemindData = patientPushServeDataQueryBizMapper.deptPushServeRemindCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.Success.getCode())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.push_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.push_date", endDate)
        );
        log.info("2.查询成功提醒数据<<<<<< {}", JSON.toJSONString(pushServeRemindData));
        if (CollectionUtil.isNotEmpty(pushServeRemindData)) {
            pushServeRemindData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());

                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 2.从日程中查询当前时间之前的数据
        List<Map<String, Object>> pushServeScheduleData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .in("a.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .le("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.schedule_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.schedule_date", endDate)
                // .isNotNull("a.user_id")  // zs202311121435 过滤掉没有user_id的数据（没有user_id其实是无法推送服务的）
                // .ne("a.user_id", "")
        );
        log.info("3.查询已推送当前时间之前的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleData)) {
            pushServeScheduleData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 3.从行为中查询用户查看的未来时间的日程
        // List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleVisitCountMap(new QueryWrapper<DeptPushServeVO>()
        //         .eq("a.operated_type", "browse")
        //         .gt("b.schedule_date", now.toString())
        //         .in("a.source_three_id", pushServeIds)
        // );
        // 3.从日程中查询未来时间的已读日程的数据
        List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.status", PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .gt("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.schedule_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.schedule_date", endDate)
        );
        log.info("4.查询当前时间之后查看过的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleVisitData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleVisitData)) {
            pushServeScheduleVisitData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }

        return pushServeDataMap;
    }

    private Map<String, Integer> patientPushServeReadDataMap(List<String> pushServeIds, String startDate, String endDate) {
        Map<String, Integer> pushServeReadDataMap = new HashMap<>();

        List<Map<String, Object>> patientPushServeReadData = patientPushServeDataQueryBizMapper.deptPushServeBehaviorCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .ge(StrUtil.isNotBlank(startDate), "a.operated_start_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.operated_start_date", endDate)
                .in("a.source_three_id", pushServeIds)
        );
        if (CollectionUtil.isNotEmpty(patientPushServeReadData)) {
            patientPushServeReadData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeReadDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeReadDataMap.put(pushServeId, count);
                } else {
                    pushServeReadDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        return pushServeReadDataMap;
    }


    ////// v2

    @Override
    public PatientPreStatisticVO patientPreStatisticByDeptIdV2(PatientStatisticQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr());
        }

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        // TODO 标签数据量庞大，直接查影响性能，可根据部门标签来查询来限定标签的数据量
        List<String> notVisitLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/未就诊标签
        List<String> visitedLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/已就诊标签
        List<String> confirmedLabelIds = new ArrayList<>(); // 筛选出发现潜在患者/已确诊标签
        // 有病种参数则根据病种筛选出相关标签
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(params.getDiseaseId())) {
                        if (LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode().equals(i.getStatDimensionType())) {
                            notVisitLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode().equals(i.getStatDimensionType())) {
                            visitedLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode().equals(i.getStatDimensionType())) {
                            confirmedLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }
        
        // 就诊患者总数量，中部门患者关联表查
        Future<Long> patientTotalCount = patientDataQueryAsyncBizService.deptAllPatientCount(new QueryWrapper<TaskPatientInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("type", DeptPatientTypeEnum.Register.getCode())
        );

        // 当前潜在患者数量
        Future<Long> potentialPatientCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit.getCode(), PatientDataDimensionTypeEnum.LSD_Pre_Visited.getCode())
                .eq("b.dept_id", params.getDeptId())
        );
        
        // 诊前
        // 发现潜在患者/未就诊
        Future<Long> historyNotVisitCount = null;
        Future<Long> nowNotVisitCount = null;
        Future<Long> newNotVisitCountYesterday = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            historyNotVisitCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // historyNotVisitCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            nowNotVisitCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // nowNotVisitCount = patientDataQueryAsyncBizService.patientLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            newNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // newNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }

        // 发现潜在患者/已就诊
        Future<Long> historyVisitedCount = null;
        Future<Long> nowVisitedCount = null;
        Future<Long> newVisitedCountYesterday = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            historyVisitedCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // historyVisitedCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            nowVisitedCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // nowVisitedCount = patientDataQueryAsyncBizService.patientLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            newVisitedCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // newVisitedCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }

        // 发现潜在患者/已确诊
        Future<Long> historyConfirmedCount = null;
        Future<Long> nowConfirmedCount = null;
        Future<Long> newConfirmedCountYesterday = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            historyConfirmedCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // historyConfirmedCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            nowConfirmedCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // nowConfirmedCount = patientDataQueryAsyncBizService.patientLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
            );
        }
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(confirmedLabelIds))) {
            newConfirmedCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // newConfirmedCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Confirmed.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(confirmedLabelIds), "a.label_id", confirmedLabelIds)
            );
        }

        // 30天平均转化率
        // 30天内未就诊
        Future<Long> notVisitedCount30 = null;
        Future<Long> visitedCount30 = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(notVisitLabelIds))) {
            notVisitedCount30 = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // notVisitedCount30 = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", DateUtil.offsetDay(now, -30).toDateStr())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(notVisitLabelIds), "a.label_id", notVisitLabelIds)
            );
        }
        // 30天内已就诊
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(visitedLabelIds))) {
            visitedCount30 = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // visitedCount30 = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", DateUtil.offsetDay(now, -30).toDateStr())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_Pre_Visited.getCode())
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(visitedLabelIds), "a.label_id", visitedLabelIds)
            );
        }


        // 等待异步返回结果
        
        PatientPreStatisticVO vo = new PatientPreStatisticVO();
        vo.setPatientTotalCount(0L);
        vo.setPotentialPatientCount(0L);
        vo.setHistoryNotVisitCount(0L); // 历史上所有该标签的患者
        vo.setNowNotVisitCount(0L); // 当前所有该标签的患者
        vo.setNewNotVisitCountYesterday(0L); // 昨天被打上该标签的患者
        vo.setHistoryVisitedCount(0L); // 历史上所有该标签的患者
        vo.setNowVisitedCount(0L); // 当前所有该标签的患者
        vo.setNewVisitedCountYesterday(0L); // 昨天被打上该标签的患者
        vo.setHistoryConfirmedCount(0L); // 历史上所有该标签的患者
        vo.setNowConfirmedCount(0L); // 当前所有该标签的患者
        vo.setNewConfirmedCountYesterday(0L); // 昨天被打上该标签的患者
        vo.setTransformRatio("0%");
        vo.setTransformRatio30("0%");

        try {
            vo.setPatientTotalCount(patientTotalCount.get());

            vo.setPotentialPatientCount(potentialPatientCount.get());

            if (null != historyNotVisitCount) {
                vo.setHistoryNotVisitCount(historyNotVisitCount.get()); // 历史上所有该标签的患者
            }
            if (null != nowNotVisitCount) {
                vo.setNowNotVisitCount(nowNotVisitCount.get()); // 当前所有该标签的患者
            }
            if (null != newNotVisitCountYesterday) {
                vo.setNewNotVisitCountYesterday(newNotVisitCountYesterday.get()); // 昨天被打上该标签的患者
            }
            if (null != historyVisitedCount) {
                vo.setHistoryVisitedCount(historyVisitedCount.get()); // 历史上所有该标签的患者
            }
            if (null != nowVisitedCount) {
                vo.setNowVisitedCount(nowVisitedCount.get()); // 当前所有该标签的患者
            }
            if (null != newVisitedCountYesterday) {
                vo.setNewVisitedCountYesterday(newVisitedCountYesterday.get()); // 昨天被打上该标签的患者
            }
            if (null != historyConfirmedCount) {
                vo.setHistoryConfirmedCount(historyConfirmedCount.get()); // 历史上所有该标签的患者
            }
            if (null != nowConfirmedCount) {
                vo.setNowConfirmedCount(nowConfirmedCount.get()); // 当前所有该标签的患者
            }
            if (null != newConfirmedCountYesterday) {
                vo.setNewConfirmedCountYesterday(newConfirmedCountYesterday.get()); // 昨天被打上该标签的患者
            }

            Long notVisitedCount30Long = 0L;
            Long visitedCount30Long = 0L;
            if (null != notVisitedCount30) {
                notVisitedCount30Long = notVisitedCount30.get();
            }
            if (null != visitedCount30) {
                visitedCount30Long = visitedCount30.get();
            }
            String transformRatio30 = null == notVisitedCount30Long || notVisitedCount30Long == 0 ? "0%" : new BigDecimal(visitedCount30Long + "").divide(new BigDecimal(notVisitedCount30Long + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
            log.info("PatientDataQueryBizServiceImpl.patientPreStatisticByDeptId:transformRatio30 >>>>>> {}", JSON.toJSONString(transformRatio30));
            vo.setTransformRatio30(transformRatio30);

        } catch (Exception e) {
            e.printStackTrace();
        }

        // 就诊转化率: (发现潜在患者/已就诊) / (发现潜在患者/未就诊) * 100%
        String transformRatio = vo.getHistoryNotVisitCount() == 0 ? "0%" : new BigDecimal(vo.getHistoryVisitedCount() + "").divide(new BigDecimal(vo.getHistoryNotVisitCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%";
        log.info("PatientDataQueryBizServiceImpl.patientPreStatisticByDeptId:transformRatio >>>>>> {}", JSON.toJSONString(transformRatio));
        vo.setTransformRatio(transformRatio);

        return vo;
    }

    @Override
    public PatientDuringStatisticVO patientDuringStatisticByDeptIdV2(PatientStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientDuringStatisticByDeptId:params >>>>>> {}", JSON.toJSONString(params));

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr());
        }

        List<String> registerLabelIds = new ArrayList<>(); // 筛选出挂号流失患者
        List<String> inhospitalLabelIds = new ArrayList<>(); // 筛选出住院流失患者
        List<String> operationLabelIds = new ArrayList<>(); // 筛选出手术流失患者
        // 根据病种筛选出相关标签
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(params.getDiseaseId())) {
                        if (LabelStatDimensionTypeEnum.LSD_During_Register.getCode().equals(i.getStatDimensionType())) {
                            registerLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode().equals(i.getStatDimensionType())) {
                            inhospitalLabelIds.add(i.getId());
                        } else if (LabelStatDimensionTypeEnum.LSD_During_Operation.getCode().equals(i.getStatDimensionType())) {
                            operationLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }

        log.info("1.挂号流失标签>>>>>> {}", JSON.toJSONString(registerLabelIds));
        log.info("2.住院流失标签>>>>>> {}", JSON.toJSONString(inhospitalLabelIds));
        log.info("3.手术流失标签>>>>>> {}", JSON.toJSONString(operationLabelIds));
        
        // 诊中
        // 历史流失标签患者
        Future<Map<String, Long>> historyCount = patientDataQueryAsyncBizService.patientHistoryLabelCountV2(new QueryWrapper<PatientDataOneDayInfo>()
        // Future<Map<String, Long>> historyCount = patientDataQueryAsyncBizService.patientHistoryLabelCountV3(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_During_Register.getCode(), LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );
        // 当前流失标签患者
        Future<Map<String, Long>> nowCount = patientDataQueryAsyncBizService.patientLabelCountV2(new QueryWrapper<PatientDataOneDayInfo>()
        // Future<Map<String, Long>> nowCount = patientDataQueryAsyncBizService.patientLabelCountV3(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_During_Register.getCode(), LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.create_date", params.getStartDate())
        );
        Future<Map<String, Long>> newCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCountV2(new QueryWrapper<PatientDataOneDayInfo>()
        // Future<Map<String, Long>> newCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCountV3(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                // .eq("a.label_id", params.getDeptId() + "-" + PatientDataDimensionTypeEnum.LSD_During_Register.getCode())
                .in("c.stat_dimension_type", PatientDataDimensionTypeEnum.LSD_During_Register.getCode(), LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                .eq("b.dept_id", params.getDeptId())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
        );

        // 住院流失（直接打标签的流失）
        Future<Long> otherHistoryInHospitalCount = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherHistoryInHospitalCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // otherHistoryInHospitalCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", "")) // 自动打的流失标签此字段没有值，依此区分
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
                    .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())
                    .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
            );
        }
        Future<Long> otherNowNotVisitCount = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherNowNotVisitCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // otherNowNotVisitCount = patientDataQueryAsyncBizService.patientLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", ""))
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
            );
        }
        Future<Long> otherNewNotVisitCountYesterday = null;
        if (!(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isEmpty(inhospitalLabelIds))) {
            otherNewNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // otherNewNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", params.getDeptId())
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
                    .and(i -> i.isNull("c.biz_type").or().eq("c.biz_type", ""))
                    .in(StrUtil.isNotBlank(params.getDiseaseId()) && CollectionUtil.isNotEmpty(inhospitalLabelIds), "a.label_id", inhospitalLabelIds)
            );
        }

        // 流失比例
        // 分子：所有申请单；分母：标签人数
        // 申请单从数仓中获取：挂号申请/住院申请/手术申请；流失标签从所有历史标签中获取
        // 申请单总数
        Long allApplyCount = doctorDwdPatientDataBizService.queryApplyCount(params.getDeptId());
        // 历史流失标签人数
        Future<Long> historyLostCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        // Future<Long> historyLostCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                // .ge(StrUtil.isNotBlank(params.getStartDate()), "a.op_date", params.getStartDate())  // 在某个时间段内的流失数量与当前历史的流失
                // .lt(StrUtil.isNotBlank(params.getEndDate()), "a.op_date", params.getEndDate())
        );

        // 昨日流失标签人数
        Future<Long> newNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
        // Future<Long> newNotVisitCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .ge("a.op_date", yesterdayDate)
                .lt("a.op_date", nowDate)
                .eq("b.dept_id", params.getDeptId())
                .in("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode(), PatientDataDimensionTypeEnum.LSD_During_InHospital.getCode(), LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
        );


        // 等待异步请求结果
        PatientDuringStatisticVO vo = new PatientDuringStatisticVO();
        vo.setHistoryRegisterCount(0L);
        vo.setHistoryInHospitalCount(0L);
        vo.setHistoryOperationCount(0L);
        vo.setNowRegisterCount(0L);
        vo.setNowInHospitalCount(0L);
        vo.setNowOperationCount(0L);
        vo.setNewRegisterCountYesterday(0L);
        vo.setNewInHospitalCountYesterday(0L);
        vo.setNewOperationCountYesterday(0L);
        vo.setOtherHistoryInHospitalCount(0L); // 历史上所有该标签的患者
        vo.setOtherNowInHospitalCount(0L); // 当前所有该标签的患者
        vo.setOtherNewInHospitalCountYesterday(0L); // 昨天被打上该标签的患者
        vo.setHistoryLostRatio("0%");
        vo.setNewLostYesterday(0L);


        try {
            Map<String, Long> historyMap = historyCount.get();
            log.info("4.历史流失标签患者数量>>>>>> {}", JSON.toJSONString(historyMap));
            vo.setHistoryRegisterCount(historyMap.get("duringRegisterNum"));
            vo.setHistoryInHospitalCount(historyMap.get("duringInHospitalNum"));
            vo.setHistoryOperationCount(historyMap.get("duringOperationNum"));

            Map<String, Long> nowMap = nowCount.get();
            log.info("5.当前流失标签患者数量>>>>>> {}", JSON.toJSONString(nowMap));
            vo.setNowRegisterCount(nowMap.get("duringRegisterNum"));
            vo.setNowInHospitalCount(nowMap.get("duringInHospitalNum"));
            vo.setNowOperationCount(nowMap.get("duringOperationNum"));

            Map<String, Long> newCountYesterdayMap = newCountYesterday.get();
            log.info("6.昨日流失标签患者数量>>>>>> {}", JSON.toJSONString(newCountYesterdayMap));
            vo.setNewRegisterCountYesterday(newCountYesterdayMap.get("duringRegisterNum"));
            vo.setNewInHospitalCountYesterday(newCountYesterdayMap.get("duringInHospitalNum"));
            vo.setNewOperationCountYesterday(newCountYesterdayMap.get("duringOperationNum"));

            if (null != otherHistoryInHospitalCount) {
                Long otherHistoryInHospitalCountLong = otherHistoryInHospitalCount.get();
                log.info("10.历史住院流失患者标签数量>>>>>> {}", otherHistoryInHospitalCountLong);
                vo.setOtherHistoryInHospitalCount(otherHistoryInHospitalCountLong); // 历史上所有该标签的患者
            }

            if (null != otherNowNotVisitCount) {
                Long otherNowNotVisitCountLong = otherNowNotVisitCount.get();
                log.info("11.当前住院流失患者标签数量>>>>>> {}", otherNowNotVisitCountLong);
                vo.setOtherNowInHospitalCount(otherNowNotVisitCountLong); // 当前所有该标签的患者
            }

            if (null != otherNewNotVisitCountYesterday) {
                Long otherNewNotVisitCountYesterdayLong = otherNewNotVisitCountYesterday.get();
                log.info("12.昨日新增住院流失患者标签数量>>>>>> {}", otherNewNotVisitCountYesterdayLong);
                vo.setOtherNewInHospitalCountYesterday(otherNewNotVisitCountYesterdayLong); // 昨天被打上该标签的患者
            }

            Long historyLostCountLong = historyLostCount.get();
            log.info("16.流失比例>>>>>> allApplyCount={} : historyLostCount={}", allApplyCount);
            vo.setHistoryLostRatio((null == allApplyCount || 0 == allApplyCount) ? "0%" : new BigDecimal(historyLostCountLong).divide(new BigDecimal(allApplyCount), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)) + "%");

            Long newNotVisitCountYesterdayLong = newNotVisitCountYesterday.get();
            vo.setNewLostYesterday(newNotVisitCountYesterdayLong);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return vo;
    }

    @Override
    public PatientAfterStatisticVO patientAfterStatisticByDeptIdV2(PatientStatisticQueryDTO params) {
        log.info("PatientDataQueryBizServiceImpl.patientAfterStatisticByDeptId:params >>>>>> {}", JSON.toJSONString(params));

        PatientAfterStatisticVO vo = new PatientAfterStatisticVO();
        vo.setConsultationRatio("-");
        vo.setConsultationRatioYesterday("-");
        vo.setLostCount(0L);
        vo.setLostCountYesterday(0L);
        vo.setServiceRatio("-");
        vo.setServiceRatioYesterday("-");
        vo.setServiceCount(0);
        vo.setServiceCountYesterday(0);

        List<String> pushServeTypeList = new ArrayList<>();
        pushServeTypeList.add(PushServeTypeEnum.Push_article.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visited.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visit_push.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Medicine.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Get_lost.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Satisfaction.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Dr_service.getCode());
        PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
        pushServeConfigClientQueryDTO.setTypes(pushServeTypeList);
        if (StrUtil.isNotBlank(params.getDiseaseId())) {
            pushServeConfigClientQueryDTO.setDiseaseId(params.getDiseaseId());
        }
        // 查询部门下可管理的推送服务
        List<DeptPushServeVO> deptPushServeVOs = this.getDeptPushServeList(params.getDeptId(), pushServeConfigClientQueryDTO);
        
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());
        // 诊后
        
        // 失访人数
        Map<String, Future<Long>> lostVisitCountMap = this.getLostVisitCountV2(params.getDeptId(), params.getDiseaseId(), params.getStartDate(), params.getEndDate());

        if (CollectionUtil.isNotEmpty(pushServeIds)) {
            DateTime now = DateUtil.date();
            // 疾病管理服务次数
            // this.getServiceCountV2(pushServeIds, vo, params.getStartDate(), params.getEndDate());
            Map<String, Future<List<Map<String, Object>>>> pushServiceCountMap = this.patientPushServeDataMapV2(pushServeIds, now, params.getStartDate(), params.getEndDate());
            Map<String, Future<List<Map<String, Object>>>> yesterdayPushServiceCountMap = this.patientPushServeDataMapV2(pushServeIds, now, DateUtil.offsetDay(now, -1).toDateStr(), now.toDateStr());


            // TODO 服务接受率：服务阅读次数/服务推送次数
            // this.getServiceRatioV2(pushServeIds, vo, params.getStartDate(), params.getEndDate());
            Future<List<Map<String, Object>>> patientPushServeReadData = this.patientPushServeReadDataMapV2(pushServeIds, params.getStartDate(), params.getEndDate());
            Future<List<Map<String, Object>>> yesterdayPatientPushServeReadData = this.patientPushServeReadDataMapV2(pushServeIds, DateUtil.offsetDay(now, -1).toDateStr(), now.toDateStr());

            

            // 异步数据处理
            try {
                // 所有诊后服务数据
                Map<String, Integer> allData = new HashMap<>();
                List<Map<String, Object>> pushServeRemindData = pushServiceCountMap.get("pushServeRemindData").get();
                log.info("2.查询成功提醒数据<<<<<< {}", JSON.toJSONString(pushServeRemindData));
                List<Map<String, Object>> pushServeScheduleData = pushServiceCountMap.get("pushServeScheduleData").get();
                log.info("3.查询已推送当前时间之前的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleData));
                List<Map<String, Object>> pushServeScheduleVisitData = pushServiceCountMap.get("pushServeScheduleVisitData").get();
                log.info("4.查询当前时间之后查看过的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleVisitData));
                if (CollectionUtil.isNotEmpty(pushServeRemindData)) {
                    pushServeRemindData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());

                        Integer dataCount = allData.get(pushServeId);
                        if (null == dataCount) {
                            allData.put(pushServeId, count);
                        } else {
                            allData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                if (CollectionUtil.isNotEmpty(pushServeScheduleData)) {
                    pushServeScheduleData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = allData.get(pushServeId);
                        if (null == dataCount) {
                            allData.put(pushServeId, count);
                        } else {
                            allData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                if (CollectionUtil.isNotEmpty(pushServeScheduleVisitData)) {
                    pushServeScheduleVisitData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = allData.get(pushServeId);
                        if (null == dataCount) {
                            allData.put(pushServeId, count);
                        } else {
                            allData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                // 昨日新增诊后服务数据
                Map<String, Integer> yesterdayNewData = new HashMap<>();
                List<Map<String, Object>> yesterdayPushServeRemindData = yesterdayPushServiceCountMap.get("pushServeRemindData").get();
                log.info("2.查询成功提醒数据<<<<<< {}", JSON.toJSONString(yesterdayPushServeRemindData));
                List<Map<String, Object>> yesterdayPushServeScheduleData = yesterdayPushServiceCountMap.get("pushServeScheduleData").get();
                log.info("3.查询已推送当前时间之前的日程数据<<<<<< {}", JSON.toJSONString(yesterdayPushServeScheduleData));
                List<Map<String, Object>> yesterdayPushServeScheduleVisitData = yesterdayPushServiceCountMap.get("pushServeScheduleVisitData").get();
                log.info("4.查询当前时间之后查看过的日程数据<<<<<< {}", JSON.toJSONString(yesterdayPushServeScheduleVisitData));
                if (CollectionUtil.isNotEmpty(yesterdayPushServeRemindData)) {
                    yesterdayPushServeRemindData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());

                        Integer dataCount = yesterdayNewData.get(pushServeId);
                        if (null == dataCount) {
                            yesterdayNewData.put(pushServeId, count);
                        } else {
                            yesterdayNewData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                if (CollectionUtil.isNotEmpty(yesterdayPushServeScheduleData)) {
                    yesterdayPushServeScheduleData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = yesterdayNewData.get(pushServeId);
                        if (null == dataCount) {
                            yesterdayNewData.put(pushServeId, count);
                        } else {
                            yesterdayNewData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                if (CollectionUtil.isNotEmpty(yesterdayPushServeScheduleVisitData)) {
                    yesterdayPushServeScheduleVisitData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = yesterdayNewData.get(pushServeId);
                        if (null == dataCount) {
                            yesterdayNewData.put(pushServeId, count);
                        } else {
                            yesterdayNewData.put(pushServeId, count + dataCount);
                        }
                    });
                }
                
                pushServeIds.forEach(i -> {
                    Integer num = allData.get(i);
                    vo.setServiceCount(null == num ? vo.getServiceCount() : num + vo.getServiceCount());
                    
                    Integer yesterdayNum = yesterdayNewData.get(i);
                    vo.setServiceCountYesterday(null == yesterdayNum ? vo.getServiceCountYesterday() : yesterdayNum + vo.getServiceCountYesterday());
                });

                
                
                // 服务接受率
                List<Map<String, Object>> allPushServeReadData = patientPushServeReadData.get();
                List<Map<String, Object>> yesterdayPushServeReadData = yesterdayPatientPushServeReadData.get();
                Map<String, Integer> allPushServeReadDataMap = new HashMap<>();
                Map<String, Integer> yesterdayPushServeReadDataMap = new HashMap<>();
                if (CollectionUtil.isNotEmpty(allPushServeReadData)) {
                    allPushServeReadData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = allPushServeReadDataMap.get(pushServeId);
                        if (null == dataCount) {
                            allPushServeReadDataMap.put(pushServeId, count);
                        } else {
                            allPushServeReadDataMap.put(pushServeId, count + dataCount);
                        }
                    });
                }
                if (CollectionUtil.isNotEmpty(yesterdayPushServeReadData)) {
                    yesterdayPushServeReadData.forEach(i -> {
                        String pushServeId = i.get("pushServeId").toString();
                        Integer count = Integer.parseInt(i.get("count").toString());
                        
                        Integer dataCount = yesterdayPushServeReadDataMap.get(pushServeId);
                        if (null == dataCount) {
                            yesterdayPushServeReadDataMap.put(pushServeId, count);
                        } else {
                            yesterdayPushServeReadDataMap.put(pushServeId, count + dataCount);
                        }
                    });
                }
                Integer totalNum = 0;
                Integer totalYesterdayNum = 0;
                for(String pushServeId : pushServeIds) {
                    Integer num = allPushServeReadDataMap.get(pushServeId);
                    if (null == num) {
                        num = 0;
                    }
                    Integer yesterdayNum = yesterdayPushServeReadDataMap.get(pushServeId);
                    if (null == yesterdayNum) {
                        yesterdayNum = 0;
                    }
                    totalNum = totalNum + num;
                    totalYesterdayNum = totalYesterdayNum + yesterdayNum;
                }
                vo.setServiceRatio(null == vo.getServiceCount() || 0== vo.getServiceCount() ? "0%" : new BigDecimal(totalNum + "").divide(new BigDecimal(vo.getServiceCount() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%");
                vo.setServiceRatioYesterday(null == vo.getServiceCountYesterday() || 0 == vo.getServiceCountYesterday() ? "0%" : new BigDecimal(totalYesterdayNum + "").divide(new BigDecimal(vo.getServiceCountYesterday() + ""), 4, BigDecimal.ROUND_DOWN).multiply(new BigDecimal("100")) + "%");
            
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        
        // TODO 复诊率
        


        // 异步数据处理
        try {
            if (null != lostVisitCountMap.get("historyLostCount")) {
                vo.setLostCount(lostVisitCountMap.get("historyLostCount").get()); // 历史上所有该标签的患者
            }
            if (null != lostVisitCountMap.get("nowLostCount")) {
                vo.setNowLostCount(lostVisitCountMap.get("nowLostCount").get()); // 当前所有该标签的患者
            }
            if (null != lostVisitCountMap.get("newLostCountYesterday")) {
                vo.setLostCountYesterday(lostVisitCountMap.get("newLostCountYesterday").get()); // 昨天被打上该标签的患者
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        
        return vo;
    }

    // 失访人数
    private Map<String, Future<Long>> getLostVisitCountV2(String deptId, String diseaseId, String startDate, String endDate) {
        Map<String, Future<Long>> map = new HashMap<>();

        DateTime now = DateUtil.date();
        String yesterdayDate = DateUtil.offsetDay(now, -1).toDateStr();
        String nowDate = now.toDateStr();

        List<String> lostLabelIds = new ArrayList<>(); // 筛选出失访患者
        // 根据病种筛选出相关标签
        if (StrUtil.isNotBlank(diseaseId)) {
            List<String> statDimensionTypes = new ArrayList<>();
            statDimensionTypes.add(LabelStatDimensionTypeEnum.LSD_After_Lost.getCode());
            // 查询标签数据
            LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
            labelQueryPageDTO.setStatDimensionTypes(statDimensionTypes);
            List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
            if (CollectionUtil.isNotEmpty(labelBOs)) {
                labelBOs.forEach(i -> {
                    if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(diseaseId)) {
                        if (LabelStatDimensionTypeEnum.LSD_After_Lost.getCode().equals(i.getStatDimensionType())) {
                            lostLabelIds.add(i.getId());
                        }
                    }
                });
            }
        }
        
        Future<Long> historyLostCount = null;
        Future<Long> nowLostCount = null;
        Future<Long> newLostCountYesterday = null;
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            historyLostCount = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // historyLostCount = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
                    .ge(StrUtil.isNotBlank(startDate), "a.op_date", startDate)
                    .lt(StrUtil.isNotBlank(endDate), "a.op_date", endDate)
            );
        }
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            nowLostCount = patientDataQueryAsyncBizService.patientLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // nowLostCount = patientDataQueryAsyncBizService.patientLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
            );
        }
        if (!(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isEmpty(lostLabelIds))) {
            newLostCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelCount(new QueryWrapper<PatientDataOneDayInfo>()
            // newLostCountYesterday = patientDataQueryAsyncBizService.patientHistoryLabelV2Count(new QueryWrapper<PatientDataOneDayInfo>()
                    .eq("a.op_type", OpTypeEnum.Create.getCode())
                    .ge("a.op_date", yesterdayDate)
                    .lt("a.op_date", nowDate)
                    .eq("b.dept_id", deptId)
                    .eq("c.stat_dimension_type", LabelStatDimensionTypeEnum.LSD_After_Lost.getCode())
                    .in(StrUtil.isNotBlank(diseaseId) && CollectionUtil.isNotEmpty(lostLabelIds), "a.label_id", lostLabelIds)
            );
        }

        map.put("historyLostCount", historyLostCount);
        map.put("nowLostCount", nowLostCount);
        map.put("newLostCountYesterday", newLostCountYesterday);

        return map;
    }

    // 诊后服务数据
    private Map<String, Future<List<Map<String, Object>>>> patientPushServeDataMapV2(List<String> pushServeIds, DateTime now, String startDate, String endDate) {
        log.info("1.查询诊后患者服务>>>>>> {} : {} : {} : {}", JSON.toJSONString(pushServeIds), now, startDate, endDate);

        Map<String, Future<List<Map<String, Object>>>> map = new HashMap<>();
        // 1.有提醒从提醒中查询发送成功的数据
        Future<List<Map<String, Object>>> pushServeRemindData = patientDataQueryAsyncBizService.deptPushServeRemindCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.Success.getCode())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.push_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.push_date", endDate)
        );
        
        // 2.从日程中查询当前时间之前的数据
        Future<List<Map<String, Object>>> pushServeScheduleData = patientDataQueryAsyncBizService.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .in("a.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .le("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.schedule_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.schedule_date", endDate)
                // .isNotNull("a.user_id")  // zs202311121435 过滤掉没有user_id的数据（没有user_id其实是无法推送服务的）
                // .ne("a.user_id", "")
        );

        // 3.从日程中查询未来时间的已读日程的数据
        Future<List<Map<String, Object>>> pushServeScheduleVisitData = patientDataQueryAsyncBizService.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.status", PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .gt("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .ge(StrUtil.isNotBlank(startDate), "a.schedule_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.schedule_date", endDate)
        );


        map.put("pushServeRemindData", pushServeRemindData);
        map.put("pushServeScheduleData", pushServeScheduleData);
        map.put("pushServeScheduleVisitData", pushServeScheduleVisitData);

        return map;
    }

    private Future<List<Map<String, Object>>> patientPushServeReadDataMapV2(List<String> pushServeIds, String startDate, String endDate) {
        Future<List<Map<String, Object>>> patientPushServeReadData = patientDataQueryAsyncBizService.deptPushServeBehaviorCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .ge(StrUtil.isNotBlank(startDate), "a.operated_start_date", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.operated_start_date", endDate)
                .in("a.source_three_id", pushServeIds)
        );
        return patientPushServeReadData;
    }

}
