package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDiseaseDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientDiseaseDataOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiseaseDataBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientDiseaseDataBizServiceImpl implements PatientDiseaseDataBizService {
    
    @Autowired
    private PatientDiseaseDataOneDayInfoService patientDataOneDayInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PatientDiseaseDataOneDayInfo params) {
        log.info("PatientDiseaseDataBizServiceImpl.add:params >>>>>> {}", JSON.toJSONString(params));
        if (null == params) {
            return;
        }

        if (null == params.getNum()) {
            params.setNum(0);
        }
        if (null == params.getAddNum()) {
            params.setAddNum(0);
        }

        DateTime now = DateUtil.date();

        // 查询当天是否已经存在数据，不存在则新增，存在则更新
        PatientDiseaseDataOneDayInfo entity = patientDataOneDayInfoService.getOne(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("disease_id", params.getDiseaseId())
                .eq("data_dimension", params.getDataDimension())
                .eq("biz_day", now.toDateStr())
        );

        if (null != entity) {
            entity.setNum(params.getNum());
            entity.setAddNum(params.getAddNum());
            entity.setOpDate(now);

            patientDataOneDayInfoService.updateById(entity);
            return;
        }

        params.setId(DoctorDataHelper.getPatientDataOneDayInfoId());
        params.setBizDay(now.toDateStr());
        params.setOpDate(now);

        patientDataOneDayInfoService.save(params);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<PatientDiseaseDataOneDayInfo> params) {
        log.info("PatientDiseaseDataBizServiceImpl.addBatch:params >>>>>> {}", JSON.toJSONString(params));
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        DateTime now = DateUtil.date();

        // 整理查询参数
        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> queryParams.add(i.getDeptId() + "-" + i.getDiseaseId() + "-" + i.getDataDimension() + "-" + now.toDateStr()));
        // 查询当天已存在的数据
        List<PatientDiseaseDataOneDayInfo> entityList = patientDataOneDayInfoService.list(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .in("CONCAT(dept_id, '-', disease_id, '-', data_dimension, '-', biz_day)", queryParams)
        );
        // 更新已存在的数据
        Map<String, PatientDiseaseDataOneDayInfo> paramsMap = params.stream().collect(Collectors.toMap(i -> i.getDeptId() + "-" + i.getDiseaseId() + "-" + i.getDataDimension() + "-" + now.toDateStr(), i -> i));
        if (CollectionUtil.isNotEmpty(entityList)) {
            List<PatientDiseaseDataOneDayInfo> updateList = new ArrayList<>();
            entityList.forEach(i -> {
                PatientDiseaseDataOneDayInfo param = paramsMap.get(i.getDeptId() + "-" + i.getDiseaseId() + "-" + i.getDataDimension() + "-" + i.getBizDay());
                if (null != param) {
                    i.setNum(param.getNum());
                    i.setAddNum(param.getAddNum());
                    // i.setBizDay(now.toDateStr());
                    i.setOpDate(now);

                    updateList.add(i);
                    paramsMap.remove(i.getDeptId() + "-" + i.getDiseaseId() + "-" + i.getDataDimension() + "-" + i.getBizDay()); // 把更新的数据移除，留下需要新增的数据
                }
            });
            // 更新
            if (CollectionUtil.isNotEmpty(updateList)) {
                log.info("PatientDiseaseDataBizServiceImpl.addBatch:updateList >>>>>> {}", JSON.toJSONString(updateList));
                patientDataOneDayInfoService.updateBatchById(updateList);
            }
        }
        
        if (CollectionUtil.isNotEmpty(paramsMap)) {
            // 处理需要新增的数据
            List<PatientDiseaseDataOneDayInfo> saveList = new ArrayList<>();
            paramsMap.entrySet().forEach(i -> {
                PatientDiseaseDataOneDayInfo entity = i.getValue();

                if (null == entity.getNum()) {
                    entity.setNum(0);
                }
                if (null == entity.getAddNum()) {
                    entity.setAddNum(0);
                }

                entity.setId(DoctorDataHelper.getPatientDataOneDayInfoId());
                entity.setBizDay(now.toDateStr());
                entity.setOpDate(now);

                saveList.add(entity);
            });
            // 新增
            log.info("PatientDiseaseDataBizServiceImpl.addBatch:saveList >>>>>> {}", JSON.toJSONString(saveList));
            patientDataOneDayInfoService.saveBatch(saveList);
        }
    }

}
