package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDiagnosisReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDiagnosisDataClientDTO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DiseaseDiagnosisRelBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DiseaseSystemBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientEstimateDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDiagnosisDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.TaskPatientHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DiseaseDiagnosisQueryMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DwdPatientDataBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientEstimateDateBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class PatientEstimateDateBizServiceImpl implements PatientEstimateDateBizService {

    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private DiseaseDiagnosisQueryMapper diseaseDiagnosisQueryMapper;

    @Override
    public List<PatientDiagnosisDataVO> getPatientDiagnosis(String patientId) {
        PatientDiagnosisDataClientDTO clientDTO = new PatientDiagnosisDataClientDTO();
        clientDTO.setPatientId(patientId);
        clientDTO.setIndex("1");
        List<PatientDiagnosisReportClientBO> list = doctorDwdPatientDataBizService.listPatientDiagnosisReportDistinct(clientDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        Set<String> diseaseIds = new HashSet<>();
        list.forEach(bo-> diseaseIds.add(bo.getDiagnosisCodeStandard()));

        // 查询关联系统
        List<DiseaseSystemBO> diseaseSystemBOList = diseaseDiagnosisQueryMapper.queryDiseaseSystemBO(
                new QueryWrapper<DiseaseSystemBO>()
                .in("a.id", diseaseIds));
        Map<String, DiseaseSystemBO> diseaseSystemBOMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseSystemBOList)) {
            diseaseSystemBOMap.putAll(diseaseSystemBOList.stream().collect(Collectors.toMap(DiseaseSystemBO::getDiseaseId, item -> item)));
        }
        // 查询病种关联诊断依据
        List<DiseaseDiagnosisRelBO> diseaseDiagnosisRelList = diseaseDiagnosisQueryMapper.queryDiseaseDiagnosisRelBO(
                new QueryWrapper<DiseaseDiagnosisRelBO>()
                .in("disease_id", diseaseIds));
        Map<String, List<DiseaseDiagnosisRelBO>> diseaseDiagnosisRelMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseDiagnosisRelList)) {
            diseaseDiagnosisRelList.forEach(rel->{
                List<DiseaseDiagnosisRelBO> bos;
                if (diseaseDiagnosisRelMap.containsKey(rel.getDiseaseId())) {
                    bos = diseaseDiagnosisRelMap.get(rel.getDiseaseId());
                } else {
                    bos = new ArrayList<>();
                }
                bos.add(rel);
                diseaseDiagnosisRelMap.put(rel.getDiseaseId(), bos);
            });
        }

        return list.stream().map(item-> TaskPatientHelper
                .getPatientClientBO(item, diseaseSystemBOMap, diseaseDiagnosisRelMap)).collect(Collectors.toList());
    }

    @Override
    public List<PatientDiagnosisDataVO> getPatientDiagnosis2(PatientEstimateDTO params) {
        PatientDiagnosisDataClientDTO clientDTO = new PatientDiagnosisDataClientDTO();
        clientDTO.setPatientId(params.getPatientId());
        clientDTO.setIndex("1");
        List<PatientDiagnosisReportClientBO> list = doctorDwdPatientDataBizService.listPatientDiagnosisReportDistinct(clientDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        Set<String> icdCodes = new HashSet<>();
        list.forEach(bo-> icdCodes.add(bo.getDiagnosisCodeStandard()));

        // 关联icd_rel表
        List<String> diseaseIds = diseaseDiagnosisQueryMapper.queryIcdRel(new QueryWrapper<DiseaseSystemBO>()
                .in("icd10_code", icdCodes)
                .eq("project_code", params.getProjectCode())
        );
        if (CollUtil.isEmpty(diseaseIds)) {
            return new ArrayList<>();
        }
        // 查询关联系统
        List<DiseaseSystemBO> diseaseSystemBOList = diseaseDiagnosisQueryMapper.queryDiseaseSystemBO(
                new QueryWrapper<DiseaseSystemBO>()
                        .in("a.id", diseaseIds));
        // 查询病种关联诊断依据
        List<DiseaseDiagnosisRelBO> diseaseDiagnosisRelList = diseaseDiagnosisQueryMapper.queryDiseaseDiagnosisRelBO(
                new QueryWrapper<DiseaseDiagnosisRelBO>()
                        .in("disease_id", diseaseIds));
        Map<String, List<DiseaseDiagnosisRelBO>> diseaseDiagnosisRelMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseDiagnosisRelList)) {
            diseaseDiagnosisRelList.forEach(rel->{
                List<DiseaseDiagnosisRelBO> bos;
                if (diseaseDiagnosisRelMap.containsKey(rel.getDiseaseId())) {
                    bos = diseaseDiagnosisRelMap.get(rel.getDiseaseId());
                } else {
                    bos = new ArrayList<>();
                }
                bos.add(rel);
                diseaseDiagnosisRelMap.put(rel.getDiseaseId(), bos);
            });
        }
        return diseaseSystemBOList.stream().map(item-> TaskPatientHelper
                .getPatientClientBO(item, diseaseDiagnosisRelMap)).collect(Collectors.toList());
    }
}
