package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.data.client.api.constant.PatientLossConstant;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientLossInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientLossInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossRemarkDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossUpdateDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientLossVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientLossDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientLossDataBizServiceImpl implements PatientLossDataBizService {
    
    @Autowired
    private PatientLossInfoService patientLossInfoService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;
    @Autowired
    private DeptClientService deptClientService;

    @Override
    public PageData<PatientLossVO> query(PatientLossQueryDTO params) {
        if (null != params.getEndTime()) {
            params.setEndTime(DateUtil.offsetDay(params.getEndTime(), 1));
        }

        IPage<PatientLossInfo> page = patientLossInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientLossInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .in(CollUtil.isNotEmpty(params.getDeptIds()), "dept_id", params.getDeptIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
            .ne(StrUtil.isBlank(params.getStatus()), "status", PatientLossConstant.LossStatus.Exclude.getCode())
            .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
            .ge(null != params.getStartTime(), "data_source_date", params.getStartTime())
            .lt(null != params.getEndTime(), "data_source_date", params.getEndTime())
            .orderByDesc("data_source_date")
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> patientIds = new ArrayList<>();
        List<String> deptIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            patientIds.add(i.getPatientId());
            deptIds.add(i.getDeptId());
        });
        
        // 查询患者信息
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        TaskPatientQueryDTO patientQuery = new TaskPatientQueryDTO();
        patientQuery.setPatientIds(patientIds);
        List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(patientQuery);
        if (CollUtil.isNotEmpty(patientList)) {
            patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
        }

        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        PageData<PatientLossVO> data= new PageConverter<PatientLossInfo, PatientLossVO>() {
            @Override
            public PatientLossVO converter(PatientLossInfo entity) {
                PatientLossVO vo = DoctorDataHelper.getPatientLossVO(entity);

                TaskPatientVO patient = patientMap.get(vo.getPatientId());
                if (null != patient) {
                    vo.setPatientName(patient.getPatientName());
                    vo.setPatientGender(patient.getPatientGender());
                    vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                    vo.setAge(patient.getAge());
                    vo.setBirthday(patient.getBirthday());
                    vo.setPhone(patient.getPhone());
                    vo.setSparePhone(patient.getSparePhone());
                    vo.setHomePhone(patient.getHomePhone());
                    vo.setLinkPhone(patient.getLinkPhone());
                    vo.setWorkPhone(patient.getWorkPhone());
                }

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<PatientLossVO> list(PatientLossQueryDTO params) {
        if (null != params.getEndTime()) {
            params.setEndTime(DateUtil.offsetDay(params.getEndTime(), 1));
        }

        List<PatientLossInfo> list = patientLossInfoService.list(new QueryWrapper<PatientLossInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .in(CollUtil.isNotEmpty(params.getDeptIds()), "dept_id", params.getDeptIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
            .ge(null != params.getStartTime(), "data_source_date", params.getStartTime())
            .lt(null != params.getEndTime(), "data_source_date", params.getEndTime())
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        
        List<String> patientIds = new ArrayList<>();
        List<String> deptIds = new ArrayList<>();
        list.forEach(i -> {
            patientIds.add(i.getPatientId());
            deptIds.add(i.getDeptId());
        });
        
        // 查询患者信息
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        TaskPatientQueryDTO patientQuery = new TaskPatientQueryDTO();
        patientQuery.setPatientIds(patientIds);
        List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(patientQuery);
        if (CollUtil.isNotEmpty(patientList)) {
            patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
        }

        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        return list.stream().map(i -> {
            PatientLossVO vo = DoctorDataHelper.getPatientLossVO(i);
            
            TaskPatientVO patient = patientMap.get(vo.getPatientId());
            if (null != patient) {
                vo.setPatientName(patient.getPatientName());
                vo.setPatientGender(patient.getPatientGender());
                vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                vo.setAge(patient.getAge());
                vo.setBirthday(patient.getBirthday());
                vo.setPhone(patient.getPhone());
                vo.setSparePhone(patient.getSparePhone());
                vo.setHomePhone(patient.getHomePhone());
                vo.setLinkPhone(patient.getLinkPhone());
                vo.setWorkPhone(patient.getWorkPhone());
            }
            
            DeptBO dept = deptMap.get(vo.getDeptId());
            if (null != dept) {
                vo.setDeptName(dept.getName());
            }

            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public PatientLossVO getById(PatientLossGetDTO params) {
        PatientLossInfo entity = patientLossInfoService.getById(params.getId());
        if (null == entity) {
            return null;
        }

        PatientLossVO vo = DoctorDataHelper.getPatientLossVO(entity);

        if (null == vo) {
            return null;
        }

        TaskPatientGetDTO patientGet = new TaskPatientGetDTO();
        patientGet.setPatientId(vo.getPatientId());
        TaskPatientVO patient = taskPatientQueryBizService.getById(patientGet);

        DeptBO dept = deptClientService.getDeptById(vo.getDeptId());

        if (null != patient) {
            vo.setPatientName(patient.getPatientName());
            vo.setPatientGender(patient.getPatientGender());
            vo.setPatientGenderStandard(patient.getPatientGenderStandard());
            vo.setAge(patient.getAge());
            vo.setBirthday(patient.getBirthday());
            vo.setPhone(patient.getPhone());
            vo.setSparePhone(patient.getSparePhone());
            vo.setHomePhone(patient.getHomePhone());
            vo.setLinkPhone(patient.getLinkPhone());
            vo.setWorkPhone(patient.getWorkPhone());
        }

        if (null != dept) {
            vo.setDeptName(dept.getName());
        }

        return vo;
    }

    @Override
    public void saveBatch(List<PatientLossAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        // 存在的数据不更新
        List<String> dataSourceIds = new ArrayList<>();
        params.forEach(i -> dataSourceIds.add(i.getDataSourceId()));

        Set<String> existSet = new HashSet<>();
        List<PatientLossInfo> existList = patientLossInfoService.list(new QueryWrapper<PatientLossInfo>()
            .in("data_source_id", dataSourceIds)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existSet.add(i.getDataSourceId()));
        }
        
        List<PatientLossInfo> saveList = new ArrayList<>();

        params.forEach(i -> {
            if (existSet.contains(i.getDataSourceId())) {
                return;
            }

            PatientLossInfo loss = new PatientLossInfo();
            loss.setId(StrUtil.format("PLI{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            loss.setDeptId(i.getDeptId());
            loss.setSourceDeptId(i.getSourceDeptId());
            loss.setPatientId(i.getPatientId());
            loss.setType(i.getType());
            loss.setStatus(PatientLossConstant.LossStatus.Loss.getCode());
            loss.setDataSourceId(i.getDataSourceId());
            loss.setDataSourceDate(i.getDataSourceDate());
            loss.setDataUpdateDate(DateUtil.date());
            saveList.add(loss);
        });

        if (CollUtil.isNotEmpty(saveList)) {
            patientLossInfoService.saveBatch(saveList);
        }
    }

    @Override
    public void remark(PatientLossRemarkDTO params) {
        PatientLossInfo loss = new PatientLossInfo();
        loss.setId(params.getId());
        loss.setRemark(params.getRemark());
        loss.setRemarkDate(DateUtil.date());
        patientLossInfoService.updateById(loss);
    }

    @Override
    public void update(PatientLossUpdateDTO params) {
        PatientLossInfo loss = new PatientLossInfo();
        loss.setId(params.getId());
        loss.setStatus(params.getStatus());
        loss.setRemark(params.getRemark());
        loss.setRemarkDate(DateUtil.date());
        patientLossInfoService.updateById(loss);
    }

}
