package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.constant.LabelStatDimensionTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientLostLabelLog;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientLostLabelLogService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdInpatientReserveVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdOperationApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdRegisterRecordVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientLostLabelBizService;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelQueryClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAndPatientAddDTO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelQueryDTO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.PatientRemoveLabelDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientLostLabelBizServiceImpl implements PatientLostLabelBizService {

    @Autowired
    private PatientLostLabelLogService patientLostLabelLogService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private PatientLabelQueryClientService patientLabelQueryClientService;
    @Autowired
    private PatientLabelClientService patientLabelClientService;

    /**
     * 确定患者是否流失：
     * 1. 查询已被打上挂号流失标签的患者和打标签时间（挂号时间）
     * 2. 查询待流失患者和待流失标记时间（挂号时间）
     * 3. 根据前两步的最早时间，查询到现在的最近时间的挂号患者
     * 4. 如果期间已就诊，流失患者移除流失标签，待流失患者移除流失标记，新患者不操作；
     * 5. 如果期间未就诊，流失患者不操作，待流失患者根据标记时间到当前时间是否超过流失时间来确定是否打流失标签或不操作，新患者根据挂号时间是否超过流失时间标记待流失或流失
     * 6. 打的流失标签的id和name格式为 id: deptId+"-"+labelId    name: deptName+"-"+labelName
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void registerLost(List<DoctorDwdRegisterRecordVO> registerList, Date minDate) {
        List<String> patientIds = new ArrayList<>(); // 要处理的已流失或待流失的患者id

        // 查询挂号流失患者
        List<PatientLabelBO> lostPatientList = this.getLabelList(LabelStatDimensionTypeEnum.LSD_During_Register);
        Map<String, List<PatientLabelBO>> lostPatientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(lostPatientList)) {
            for (PatientLabelBO i : lostPatientList) {
                patientIds.add(i.getPatientId());

                String[] labelIds = i.getLabelId().split("-");
                List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(i.getPatientId() + "-" + labelIds[0]);
                if (CollectionUtil.isEmpty(patientLabelBOs)) {
                    patientLabelBOs = new ArrayList<>();
                    lostPatientMap.put(i.getPatientId() + "-" + labelIds[0], patientLabelBOs);
                }
                patientLabelBOs.add(i);
            }
        }
        // 待流失患者
        List<PatientLostLabelLog> patientLostLabelLogList = patientLostLabelLogService.list(new QueryWrapper<PatientLostLabelLog>()
                .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
        );
        log.info("PatientLostLabelBizServiceImpl.registerLost:patientLostLabelLogList: <<<<<< {} ", JSON.toJSONString(patientLostLabelLogList));
        Map<String, PatientLostLabelLog> patientLostLabelLogMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(patientLostLabelLogList)) {
            for (PatientLostLabelLog i : patientLostLabelLogList) {
                patientIds.add(i.getPatientId());

                patientLostLabelLogMap.put(i.getPatientId() + "-" + i.getDeptId(), i);
            }
        }

        Map<String, PatientLostLabelLog> lostLabelLogDoneMap = new HashMap<>(); // 已经处理的待流失数据

        List<PatientRemoveLabelDTO> removeLostLabelList = new ArrayList<>(); // 将要移除的挂号流失患者标签
        List<String> removeLostLabelLogList = new ArrayList<>(); // 将要移除的待流失患者记录
        List<PatientLostLabelLog> addLostLabelLogList = new ArrayList<>(); // 将要添加的挂号流失患者记录
        List<LabelAndPatientAddDTO> addLostLabelList = new ArrayList<>(); // 将要添加的挂号流失患者标签

        // 挂号患者
        if (CollectionUtil.isNotEmpty(registerList)) {
            registerList.forEach(i -> {
                String key = i.getPatientId() + "-" + i.getDepartmentCodeStandard();
                PatientLostLabelLog patientLostLabelLog = patientLostLabelLogMap.get(key);

                if (null != patientLostLabelLog) {
                    lostLabelLogDoneMap.put(key, patientLostLabelLog);
                }
                
                // 就诊
                if ("Doing".equals(i.getVisitStatusStandard()) || "Finish".equals(i.getVisitStatusStandard())) { // TODO 枚举: Doing  Finish
                    List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(key);
                    if (CollectionUtil.isNotEmpty(patientLabelBOs)) {
                        // 删除流失标签，并记录标签历史
                        List<PatientRemoveLabelDTO> patientRemoveLabelDTOList = patientLabelBOs.stream().map(item -> {
                            PatientRemoveLabelDTO patientRemoveLabelDTO = new PatientRemoveLabelDTO();
                            patientRemoveLabelDTO.setPatientId(item.getPatientId());
                            patientRemoveLabelDTO.setLabelId(item.getLabelId());
                            return patientRemoveLabelDTO;
                        }).collect(Collectors.toList());
                        removeLostLabelList.addAll(patientRemoveLabelDTOList);
                    }
                    if (null != patientLostLabelLog) {
                        // 删除患者流失标记
                        removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());
                    }
                } else { // 未就诊
                    if (null != patientLostLabelLog) {
                        if (DateUtil.date(patientLostLabelLog.getDataDt()).before(minDate)) {
                            // 判断流失标记的挂号时间是否超过流失时间，超过则删除流失标记并打标签
                            removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());

                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                            lostLabel.setLabelId(i.getDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                            lostLabel.setLabelName(i.getDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getMsg());
                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                            lostLabel.setPatientId(i.getPatientId());
                            lostLabel.setDataDt(i.getRegDate());
                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Register_Lost.getCode());
                            addLostLabelList.add(lostLabel);
                        }
                    } else {
                        if (DateUtil.date(i.getRegDate()).before(minDate)) {
                            // 判断流失标记的挂号时间是否超过流失时间，超过则删除流失标记并打标签
                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                            lostLabel.setLabelId(i.getDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                            lostLabel.setLabelName(i.getDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getMsg());
                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                            lostLabel.setPatientId(i.getPatientId());
                            lostLabel.setDataDt(i.getRegDate());
                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Register_Lost.getCode());
                            addLostLabelList.add(lostLabel);
                        } else {
                            PatientLostLabelLog lostLabelLog = new PatientLostLabelLog();
                            lostLabelLog.setId(DoctorDataHelper.getPatientLostLabelLogId());
                            lostLabelLog.setDeptId(i.getDepartmentCodeStandard());
                            lostLabelLog.setDeptName(i.getDepartmentNameStandard());
                            lostLabelLog.setPatientId(i.getPatientId());
                            lostLabelLog.setLostType(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                            lostLabelLog.setDataDt(i.getRegDate());
                            addLostLabelLogList.add(lostLabelLog);
                        }
                    }
                }
            });
        }

        // 处理未处理的待流失标记患者
        patientLostLabelLogList.forEach(i -> {
            PatientLostLabelLog patientLostLabelLog = lostLabelLogDoneMap.get(i.getPatientId() + "-" + i.getDeptId());
            if (null == patientLostLabelLog && DateUtil.date(i.getDataDt()).before(minDate)) { // 不存在说明没处理；超过流失时间处理该数据
                removeLostLabelLogList.add(i.getPatientId() + "-" + i.getDeptId());

                LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                lostLabel.setLabelId(i.getDeptId() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                lostLabel.setLabelName(i.getDeptName() + "-" + LabelStatDimensionTypeEnum.LSD_During_Register.getMsg());
                // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Register.getCode());
                lostLabel.setPatientId(i.getPatientId());
                lostLabel.setDataDt(i.getDataDt());
                addLostLabelList.add(lostLabel);
            }
        });

        // 添加待流失标记
        log.info("PatientLostLabelBizServiceImpl.registerLost:addLostLabelLogList: >>>>>> {} ", JSON.toJSONString(addLostLabelLogList));
        if (CollectionUtil.isNotEmpty(addLostLabelLogList)) {
            patientLostLabelLogService.saveBatch(addLostLabelLogList);
        }
        // 移除待流失标记
        log.info("PatientLostLabelBizServiceImpl.registerLost:removeLostLabelLogList: >>>>>> {} ", JSON.toJSONString(removeLostLabelLogList));
        if (CollectionUtil.isNotEmpty(removeLostLabelLogList)) {
            patientLostLabelLogService.remove(new QueryWrapper<PatientLostLabelLog>()
                    .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_Register.getCode())
                    .in("CONCAT(patient_id, '-', dept_id)", removeLostLabelLogList)
            );
        }
        // 移除流失标签
        log.info("PatientLostLabelBizServiceImpl.registerLost:removeLostLabelList: >>>>>> {} ", JSON.toJSONString(removeLostLabelList));
        if (CollectionUtil.isNotEmpty(removeLostLabelList)) {
            patientLabelClientService.patientRemoveLabelBatch(removeLostLabelList);
        }
        // 打流失标签
        log.info("PatientLostLabelBizServiceImpl.registerLost:addLostLabelList: >>>>>> {} ", JSON.toJSONString(addLostLabelList));
        if (CollectionUtil.isNotEmpty(addLostLabelList)) {
            patientLabelClientService.createLabelAndBindPatientBatch(addLostLabelList);
        }

    }




    //todo:
//    @Transactional(rollbackFor = Exception.class)
//    @Override
//    public void inHospitalLost(List<DoctorDwdInpatientReserveVO> inHospitalList, Date minDate) {
//        List<String> patientIds = new ArrayList<>(); // 要处理的已流失或待流失的患者id
//        // 查询住院流失患者
//        List<PatientLabelBO> lostPatientList = this.getLabelList(LabelStatDimensionTypeEnum.LSD_During_InHospital);
//        Map<String, List<PatientLabelBO>> lostPatientMap = new HashMap<>();
//        if (CollectionUtil.isNotEmpty(lostPatientList)) {
//            for (PatientLabelBO i : lostPatientList) {
//                patientIds.add(i.getPatientId());
//
//                String[] labelIds = i.getLabelId().split("-");
//                List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(i.getPatientId() + "-" + labelIds[0]);
//                if (CollectionUtil.isEmpty(patientLabelBOs)) {
//                    patientLabelBOs = new ArrayList<>();
//                    lostPatientMap.put(i.getPatientId() + "-" + labelIds[0], patientLabelBOs);
//                }
//                patientLabelBOs.add(i);
//            }
//        }
//        // 待流失患者
//        List<PatientLostLabelLog> patientLostLabelLogList = patientLostLabelLogService.list(new QueryWrapper<PatientLostLabelLog>()
//                .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
//        );
//        log.info("PatientLostLabelBizServiceImpl.inHospitalLost:patientLostLabelLogList: >>>>>> {} ", JSON.toJSONString(patientLostLabelLogList));
//        Map<String, PatientLostLabelLog> patientLostLabelLogMap = new HashMap<>();
//        if (CollectionUtil.isNotEmpty(patientLostLabelLogList)) {
//            for (PatientLostLabelLog i : patientLostLabelLogList) {
//                patientIds.add(i.getPatientId());
//
//                patientLostLabelLogMap.put(i.getPatientId() + "-" + i.getDeptId(), i);
//            }
//        }
//
//        Map<String, PatientLostLabelLog> lostLabelLogDoneMap = new HashMap<>(); // 已经处理的待流失数据
//
//        List<PatientRemoveLabelDTO> removeLostLabelList = new ArrayList<>(); // 将要移除的流失患者标签
//        List<String> removeLostLabelLogList = new ArrayList<>(); // 将要移除的待流失患者记录
//        List<PatientLostLabelLog> addLostLabelLogList = new ArrayList<>(); // 将要添加的流失患者记录
//        List<LabelAndPatientAddDTO> addLostLabelList = new ArrayList<>(); // 将要添加的流失患者标签
//
//        // 住院患者
//        if (CollectionUtil.isNotEmpty(inHospitalList)) {
//            inHospitalList.forEach(i -> {
//                String key = i.getPatientId() + "-" + i.getApplyDepartmentCodeStandard();
//                PatientLostLabelLog patientLostLabelLog = patientLostLabelLogMap.get(key);
//
//                if (null != patientLostLabelLog) {
//                    lostLabelLogDoneMap.put(key, patientLostLabelLog);
//                }
//
//                // 住院
//                if (StrUtil.isNotBlank(i.getInHospitalId())) {
//                    List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(key);
//                    if (CollectionUtil.isNotEmpty(patientLabelBOs)) {
//                        // 删除流失标签，并记录标签历史
//                        List<PatientRemoveLabelDTO> patientRemoveLabelDTOList = patientLabelBOs.stream().map(item -> {
//                            PatientRemoveLabelDTO patientRemoveLabelDTO = new PatientRemoveLabelDTO();
//                            patientRemoveLabelDTO.setPatientId(item.getPatientId());
//                            patientRemoveLabelDTO.setLabelId(item.getLabelId());
//                            return patientRemoveLabelDTO;
//                        }).collect(Collectors.toList());
//                        removeLostLabelList.addAll(patientRemoveLabelDTOList);
//                    }
//                    if (null != patientLostLabelLog) {
//                        // 删除患者流失标记
//                        removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());
//                    }
//                } else { // 未住院
//                    if (null != patientLostLabelLog) {
//                        if (DateUtil.date(patientLostLabelLog.getDataDt()).before(minDate)) {
//                            // 判断流失标记的时间是否超过流失时间，超过则删除流失标记并打标签
//                            removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());
//
//                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
//                            lostLabel.setLabelId(i.getApplyDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                            lostLabel.setLabelName(i.getApplyDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getMsg());
//                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
//                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                            lostLabel.setPatientId(i.getPatientId());
//                            lostLabel.setDataDt(i.getApplyDate());
//                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Inhospital_Lost.getCode());
//                            addLostLabelList.add(lostLabel);
//                        }
//                    } else {
//                        if (DateUtil.date(i.getApplyDate()).before(minDate)) {
//                            // 判断流失标记的时间是否超过流失时间，超过则删除流失标记并打标签
//                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
//                            lostLabel.setLabelId(i.getApplyDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                            lostLabel.setLabelName(i.getApplyDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getMsg());
//                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
//                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                            lostLabel.setPatientId(i.getPatientId());
//                            lostLabel.setDataDt(i.getApplyDate());
//                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Inhospital_Lost.getCode());
//                            addLostLabelList.add(lostLabel);
//                        } else {
//                            PatientLostLabelLog lostLabelLog = new PatientLostLabelLog();
//                            lostLabelLog.setId(DoctorDataHelper.getPatientLostLabelLogId());
//                            lostLabelLog.setDeptId(i.getApplyDepartmentCodeStandard());
//                            lostLabelLog.setDeptName(i.getApplyDepartmentNameStandard());
//                            lostLabelLog.setPatientId(i.getPatientId());
//                            lostLabelLog.setLostType(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                            lostLabelLog.setDataDt(i.getApplyDate());
//                            addLostLabelLogList.add(lostLabelLog);
//                        }
//                    }
//                }
//            });
//        }
//
//        // 处理未处理的待流失标记患者
//        patientLostLabelLogList.forEach(i -> {
//            PatientLostLabelLog patientLostLabelLog = lostLabelLogDoneMap.get(i.getPatientId() + "-" + i.getDeptId());
//            if (null == patientLostLabelLog && DateUtil.date(i.getDataDt()).before(minDate)) { // 不存在说明没处理；超过流失时间处理该数据
//                removeLostLabelLogList.add(i.getPatientId() + "-" + i.getDeptId());
//
//                LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
//                lostLabel.setLabelId(i.getDeptId() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                lostLabel.setLabelName(i.getDeptName() + "-" + LabelStatDimensionTypeEnum.LSD_During_InHospital.getMsg());
//                // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
//                lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode());
//                lostLabel.setPatientId(i.getPatientId());
//                lostLabel.setDataDt(i.getDataDt());
//                addLostLabelList.add(lostLabel);
//            }
//        });
//
//        // 添加待流失标记
//        log.info("PatientLostLabelBizServiceImpl.inHospitalLost:addLostLabelLogList: >>>>>> {} ", JSON.toJSONString(addLostLabelLogList));
//        if (CollectionUtil.isNotEmpty(addLostLabelLogList)) {
//            patientLostLabelLogService.saveBatch(addLostLabelLogList);
//        }
//        // 移除待流失标记
//        log.info("PatientLostLabelBizServiceImpl.inHospitalLost:removeLostLabelLogList: >>>>>> {} ", JSON.toJSONString(removeLostLabelLogList));
//        if (CollectionUtil.isNotEmpty(removeLostLabelLogList)) {
//            patientLostLabelLogService.remove(new QueryWrapper<PatientLostLabelLog>()
//                    .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_InHospital.getCode())
//                    .in("CONCAT(patient_id, '-', dept_id)", removeLostLabelLogList)
//            );
//        }
//        // 移除流失标签
//        log.info("PatientLostLabelBizServiceImpl.inHospitalLost:removeLostLabelList: >>>>>> {} ", JSON.toJSONString(removeLostLabelList));
//        if (CollectionUtil.isNotEmpty(removeLostLabelList)) {
//            patientLabelClientService.patientRemoveLabelBatch(removeLostLabelList);
//        }
//        // 打流失标签
//        log.info("PatientLostLabelBizServiceImpl.inHospitalLost:addLostLabelList: >>>>>> {} ", JSON.toJSONString(addLostLabelList));
//        if (CollectionUtil.isNotEmpty(addLostLabelList)) {
//            patientLabelClientService.createLabelAndBindPatientBatch(addLostLabelList);
//        }
//
//    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void operationLost(List<DoctorDwdOperationApplyVO> operationList, Date minDate) {
        List<String> patientIds = new ArrayList<>(); // 要处理的已流失或待流失的患者id
        // 查询流失患者
        List<PatientLabelBO> lostPatientList = this.getLabelList(LabelStatDimensionTypeEnum.LSD_During_Operation); 
        Map<String, List<PatientLabelBO>> lostPatientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(lostPatientList)) {
            for (PatientLabelBO i : lostPatientList) {
                patientIds.add(i.getPatientId());

                String[] labelIds = i.getLabelId().split("-");
                List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(i.getPatientId() + "-" + labelIds[0]);
                if (CollectionUtil.isEmpty(patientLabelBOs)) {
                    patientLabelBOs = new ArrayList<>();
                    lostPatientMap.put(i.getPatientId() + "-" + labelIds[0], patientLabelBOs);
                }
                patientLabelBOs.add(i);
            }
        }
        // 待流失患者
        List<PatientLostLabelLog> patientLostLabelLogList = patientLostLabelLogService.list(new QueryWrapper<PatientLostLabelLog>()
                .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
        );
        log.info("PatientLostLabelBizServiceImpl.operationLost:patientLostLabelLogList: >>>>>> {} ", JSON.toJSONString(patientLostLabelLogList));
        Map<String, PatientLostLabelLog> patientLostLabelLogMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(patientLostLabelLogList)) {
            for (PatientLostLabelLog i : patientLostLabelLogList) {
                patientIds.add(i.getPatientId());

                patientLostLabelLogMap.put(i.getPatientId() + "-" + i.getDeptId(), i);
            }
        }

        Map<String, PatientLostLabelLog> lostLabelLogDoneMap = new HashMap<>(); // 已经处理的待流失数据

        List<PatientRemoveLabelDTO> removeLostLabelList = new ArrayList<>(); // 将要移除的流失患者标签
        List<String> removeLostLabelLogList = new ArrayList<>(); // 将要移除的待流失患者记录
        List<PatientLostLabelLog> addLostLabelLogList = new ArrayList<>(); // 将要添加的流失患者记录
        List<LabelAndPatientAddDTO> addLostLabelList = new ArrayList<>(); // 将要添加的流失患者标签

        // 手术患者
        if (CollectionUtil.isNotEmpty(operationList)) {
            operationList.forEach(i -> {
                String key = i.getPatientId() + "-" + i.getDepartmentCodeStandard();
                PatientLostLabelLog patientLostLabelLog = patientLostLabelLogMap.get(key);
                
                if (null != patientLostLabelLog) {
                    lostLabelLogDoneMap.put(key, patientLostLabelLog);
                }
                
                // 手术
                if (StrUtil.isNotBlank(i.getOperationId())) { 
                    List<PatientLabelBO> patientLabelBOs = lostPatientMap.get(key);
                    if (CollectionUtil.isNotEmpty(patientLabelBOs)) {
                        // 删除流失标签，并记录标签历史
                        List<PatientRemoveLabelDTO> patientRemoveLabelDTOList = patientLabelBOs.stream().map(item -> {
                            PatientRemoveLabelDTO patientRemoveLabelDTO = new PatientRemoveLabelDTO();
                            patientRemoveLabelDTO.setPatientId(item.getPatientId());
                            patientRemoveLabelDTO.setLabelId(item.getLabelId());
                            return patientRemoveLabelDTO;
                        }).collect(Collectors.toList());
                        removeLostLabelList.addAll(patientRemoveLabelDTOList);
                    }
                    if (null != patientLostLabelLog) {
                        // 删除患者流失标记
                        removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());
                    }
                } else { // 未手术
                    if (null != patientLostLabelLog) {
                        if (DateUtil.date(patientLostLabelLog.getDataDt()).before(minDate)) {
                            // 判断流失标记的时间是否超过流失时间，超过则删除流失标记并打标签
                            removeLostLabelLogList.add(patientLostLabelLog.getPatientId() + "-" + patientLostLabelLog.getDeptId());

                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                            lostLabel.setLabelId(i.getDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                            lostLabel.setLabelName(i.getDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getMsg());
                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                            lostLabel.setPatientId(i.getPatientId());
                            lostLabel.setDataDt(i.getApplyTime());
                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Operation_Lost.getCode());
                            addLostLabelList.add(lostLabel);
                        }
                    } else {
                        if (DateUtil.date(i.getApplyTime()).before(minDate)) {
                            // 判断流失标记的时间是否超过流失时间，超过则删除流失标记并打标签
                            LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                            lostLabel.setLabelId(i.getDepartmentCodeStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                            lostLabel.setLabelName(i.getDepartmentNameStandard() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getMsg());
                            // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                            lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                            lostLabel.setPatientId(i.getPatientId());
                            lostLabel.setDataDt(i.getApplyTime());
                            lostLabel.setSourceType(ObjectTypeEnum.OT_Label_Operation_Lost.getCode());
                            addLostLabelList.add(lostLabel);
                        } else {
                            PatientLostLabelLog lostLabelLog = new PatientLostLabelLog();
                            lostLabelLog.setId(DoctorDataHelper.getPatientLostLabelLogId());
                            lostLabelLog.setDeptId(i.getDepartmentCodeStandard());
                            lostLabelLog.setDeptName(i.getDepartmentNameStandard());
                            lostLabelLog.setPatientId(i.getPatientId());
                            lostLabelLog.setLostType(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                            lostLabelLog.setDataDt(i.getApplyTime());
                            addLostLabelLogList.add(lostLabelLog);
                        }
                    }
                }
            });
        }

        // 处理未处理的待流失标记患者
        patientLostLabelLogList.forEach(i -> {
            PatientLostLabelLog patientLostLabelLog = lostLabelLogDoneMap.get(i.getPatientId() + "-" + i.getDeptId());
            if (null == patientLostLabelLog && DateUtil.date(i.getDataDt()).before(minDate)) { // 不存在说明没处理；超过流失时间处理该数据
                removeLostLabelLogList.add(i.getPatientId() + "-" + i.getDeptId());

                LabelAndPatientAddDTO lostLabel = new LabelAndPatientAddDTO();
                lostLabel.setLabelId(i.getDeptId() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                lostLabel.setLabelName(i.getDeptName() + "-" + LabelStatDimensionTypeEnum.LSD_During_Operation.getMsg());
                // lostLabel.setBizType(LabelTypeEnum.Tags_Type_Statistics.getCode()); // 不要赋值，用于区分标签的来源，统计中用此字段统计流失时使用
                lostLabel.setStatDimensionType(LabelStatDimensionTypeEnum.LSD_During_Operation.getCode());
                lostLabel.setPatientId(i.getPatientId());
                lostLabel.setDataDt(i.getDataDt());
                addLostLabelList.add(lostLabel);
            }
        });

        // 添加待流失标记
        log.info("PatientLostLabelBizServiceImpl.operationLost:addLostLabelLogList: >>>>>> {} ", JSON.toJSONString(addLostLabelLogList));
        if (CollectionUtil.isNotEmpty(addLostLabelLogList)) {
            patientLostLabelLogService.saveBatch(addLostLabelLogList);
        }
        // 移除待流失标记
        log.info("PatientLostLabelBizServiceImpl.operationLost:removeLostLabelLogList: >>>>>> {} ", JSON.toJSONString(removeLostLabelLogList));
        if (CollectionUtil.isNotEmpty(removeLostLabelLogList)) {
            patientLostLabelLogService.remove(new QueryWrapper<PatientLostLabelLog>()
                    .eq("lost_type", LabelStatDimensionTypeEnum.LSD_During_Operation.getCode())
                    .in("CONCAT(patient_id, '-', dept_id)", removeLostLabelLogList)
            );
        }
        // 移除流失标签
        log.info("PatientLostLabelBizServiceImpl.operationLost:removeLostLabelList: >>>>>> {} ", JSON.toJSONString(removeLostLabelList));
        if (CollectionUtil.isNotEmpty(removeLostLabelList)) {
            patientLabelClientService.patientRemoveLabelBatch(removeLostLabelList);
        }
        // 打流失标签
        log.info("PatientLostLabelBizServiceImpl.operationLost:addLostLabelList: >>>>>> {} ", JSON.toJSONString(addLostLabelList));
        if (CollectionUtil.isNotEmpty(addLostLabelList)) {
            patientLabelClientService.createLabelAndBindPatientBatch(addLostLabelList);
        }

    }

    // 查询流失患者
    private List<PatientLabelBO> getLabelList(LabelStatDimensionTypeEnum statDimensionType) {
        LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
        labelQueryPageDTO.setStatDimensionType(statDimensionType.getCode());
        List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
        if (CollectionUtil.isNotEmpty(labelBOs)) {
            new ArrayList<>(0);
        }
        List<String> labelIds = labelBOs.stream().map(LabelBO::getId).collect(Collectors.toList());

        if (CollectionUtil.isEmpty(labelIds)) {
            return new ArrayList<>();
        }

        LabelQueryDTO labelQueryDTO = new LabelQueryDTO();
        labelQueryDTO.setLabelIds(labelIds);
        List<PatientLabelBO> list = patientLabelQueryClientService.queryList(labelQueryDTO);
        log.info("PatientLostLabelBizServiceImpl.getLabelList:list: <<<<<< {} ", JSON.toJSONString(list));
        
        return list;
    }

}
