package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientManageDetailInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientManageDetailInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageDetailAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageDetailQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientManageDetailVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageDetailBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorRelationClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.bo.DoctorRelationBO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorRelationClientQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class PatientManageDetailBizServiceImpl implements PatientManageDetailBizService {

    @Autowired
    private PatientManageDetailInfoService patientManageDetailInfoService;
    @Autowired
    private DoctorRelationClientService doctorRelationClientService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;

    @Override
    public PageData<PatientManageDetailVO> query(PatientManageDetailQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<PatientManageDetailInfo> page = patientManageDetailInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PatientManageDetailInfo>()
                    .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                    .ge(null != params.getStartDate(), "label_date", params.getStartDate())
                    .lt(null != params.getEndDate(), "label_date", params.getEndDate())
        );
        
        if(page==null || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> doctorIds = new ArrayList<>();
        List<String> patientIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());
            patientIds.add(i.getPatientId());
        });

        // 医生信息
        Map<String, AdminUserBO> doctorMap = adminUserClientService.getUserMapByIds(doctorIds);

        // 患者信息
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            TaskPatientQueryDTO patientQueryDTO = new TaskPatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<TaskPatientVO> patients = taskPatientQueryBizService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patients)) {
                patients.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        PageData<PatientManageDetailVO> data= new PageConverter<PatientManageDetailInfo, PatientManageDetailVO>() {
            @Override
            public PatientManageDetailVO converter(PatientManageDetailInfo entity) {
                PatientManageDetailVO vo = DoctorDataHelper.getPatientManageDetailVO(entity);

                AdminUserBO doctor = doctorMap.get(vo.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorName(doctor.getRealName());
                }
                
                TaskPatientVO patient = patientMap.get(vo.getPatientId());
                if (null != patient) {
                    vo.setPatientName(patient.getPatientName());
                    vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                    vo.setPhone(patient.getPhone());
                    vo.setSparePhone(patient.getSparePhone());
                    vo.setAge(patient.getAge());
                }
                return vo;
            }
        }.getPageData(page);

        return data;
    }
    
    @Override
    public List<PatientManageDetailVO> diversionQuery(PatientManageDetailQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        // 查询运营助理管理的患者
        List<Map<String, Object>> doctorPatientList = patientManageDetailInfoService.listMaps(new QueryWrapper<PatientManageDetailInfo>()
            .select("patient_id as patientId")
            .eq("doctor_id", params.getDoctorId())
            // .ge("label_date", startDate)
            // .lt("label_date", queryEndDate)
            .groupBy("patient_id")
        );

        // 查询运营助理管理的科室列表
        DoctorRelationClientQueryDTO doctorRelationClientQuery = new DoctorRelationClientQueryDTO();
        doctorRelationClientQuery.setType(DoctorRelationConstant.Type.DEPT.getCode());
        doctorRelationClientQuery.setDoctorId(params.getDoctorId());
        List<DoctorRelationBO> doctorDeptList = doctorRelationClientService.list(doctorRelationClientQuery);
        if (CollUtil.isEmpty(doctorDeptList)) {
            return new ArrayList<>(0);
        }
        Set<String> deptIds = new HashSet<>();
        doctorDeptList.forEach(i -> {
            deptIds.add(i.getRelationId());
        });

        // 查询指定科室的就诊/住院信息，整理出map：key:deptId value:patientId
        // 就诊
        Map<String, Set<String>> deptPatientVisitMap = new HashMap<>();
        DataDwdPatientQueryDTO visitQuery = new DataDwdPatientQueryDTO();
        visitQuery.setDeptIds(new ArrayList<>(deptIds));
        visitQuery.setStartDate(params.getStartDate());
        visitQuery.setEndDate(params.getEndDate());
        List<PatientVisitRecordClientBO> visitList = doctorDwdPatientDataBizService.patientVisitList(visitQuery);
        if (CollUtil.isNotEmpty(visitList)) {
            visitList.forEach(i -> {
                Set<String> s = deptPatientVisitMap.get(i.getDepartmentCodeStandard());
                if (null == s) {
                    s = new HashSet<>();
                    deptPatientVisitMap.put(i.getDepartmentCodeStandard(), s);
                }
                s.add(i.getPatientId());
            });
        }
        // 住院
        Map<String, Set<String>> deptPatientInpatientMap = new HashMap<>();
        DataDwdPatientQueryDTO inpatientQuery = new DataDwdPatientQueryDTO();
        inpatientQuery.setDeptIds(new ArrayList<>(deptIds));
        inpatientQuery.setStartDate(params.getStartDate());
        inpatientQuery.setEndDate(params.getEndDate());
        List<DataDwdPatientInHospitalVO> inpatientList = doctorDwdPatientDataBizService.patientInHospitalList(visitQuery);
        if (CollUtil.isNotEmpty(inpatientList)) {
            inpatientList.forEach(i -> {
                Set<String> s = deptPatientInpatientMap.get(i.getAdmitDepartmentCodeStandard());
                if (null == s) {
                    s = new HashSet<>();
                    deptPatientInpatientMap.put(i.getAdmitDepartmentCodeStandard(), s);
                }
                s.add(i.getPatientId());
            });
        }

        List<PatientManageDetailVO> list = new ArrayList<>();
        doctorPatientList.forEach(i -> {
            String patientId = String.valueOf(i.get("patientId"));
            PatientManageDetailVO entity = new PatientManageDetailVO();
            entity.setDoctorId(params.getDoctorId());
            entity.setPatientId(patientId);
        });

        List<String> patientIds = new ArrayList<>();
        list.forEach(i -> {
            patientIds.add(i.getPatientId());
        });

        // 医生信息
        AdminUserBO doctor = adminUserClientService.getUserById(params.getDoctorId());

        // 患者信息
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            TaskPatientQueryDTO patientQueryDTO = new TaskPatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<TaskPatientVO> patients = taskPatientQueryBizService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patients)) {
                patients.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        list.forEach(i -> {
            if (null != doctor) {
                i.setDoctorName(doctor.getRealName());
            }
            
            TaskPatientVO patient = patientMap.get(i.getPatientId());
            if (null != patient) {
                i.setPatientName(patient.getPatientName());
                i.setPatientGenderStandard(patient.getPatientGenderStandard());
                i.setPhone(patient.getPhone());
                i.setSparePhone(patient.getSparePhone());
                i.setAge(patient.getAge());
            }
        });

        return list;
    }

    @Override
    public void saveBatch(List<PatientManageDetailAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        List<String> existParams = new ArrayList<>();
        params.forEach(i -> {
            existParams.add(i.getDoctorId() + "_" + i.getPatientId() + "_" + i.getLabelId() + "_" + DateUtil.formatDateTime(i.getLabelDate()));
        });

        Map<String, PatientManageDetailInfo> existMap = new HashMap<>();
        List<PatientManageDetailInfo> existList = patientManageDetailInfoService.list(new QueryWrapper<PatientManageDetailInfo>()
            .in("CONCAT(doctor_id,'_',patient_id,'_',label_id,'_',DATE_FORMAT(label_date, '%Y-%m-%d %H:%i:%s'))", existParams)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> {
                existMap.put(i.getDoctorId() + "_" + i.getPatientId() + "_" + i.getLabelId() + "_" + DateUtil.formatDateTime(i.getLabelDate()), i);
            });
        }

        List<PatientManageDetailInfo> saveList = new ArrayList<>();
        List<PatientManageDetailInfo> updateList = new ArrayList<>();

        params.forEach(i -> {
            PatientManageDetailInfo entity = existMap.get(i.getDoctorId() + "_" + i.getPatientId() + "_" + i.getLabelId() + "_" + DateUtil.formatDateTime(i.getLabelDate()));

            if (null == entity) {
                entity = new PatientManageDetailInfo();
                entity.setId(StrUtil.format("PMD{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }

            entity.setDoctorId(i.getDoctorId());
            entity.setPatientId(i.getPatientId());
            entity.setLabelId(i.getLabelId());
            entity.setLabelDate(i.getLabelDate());
            entity.setSourceType(i.getSourceType());
            entity.setSourceId(i.getSourceId());
            entity.setSourceDate(i.getSourceDate());
            entity.setRemarkStatus(i.getRemarkStatus());
            entity.setRemarkDate(i.getRemarkDate());
            entity.setDataUpdateDate(DateUtil.date());
        });
        
        if (CollUtil.isNotEmpty(saveList)) {
            patientManageDetailInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            patientManageDetailInfoService.updateBatchById(updateList);
        }
    }
}
