package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientManageOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientManageOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageOneDayAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageOneDayQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientManageOneDayVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageOneDayBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class PatientManageOneDayBizServiceImpl implements PatientManageOneDayBizService {

    @Autowired
    private PatientManageOneDayInfoService patientManageOneDayInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public PageData<PatientManageOneDayVO> query(PatientManageOneDayQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<PatientManageOneDayInfo> page = patientManageOneDayInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientManageOneDayInfo>()
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .ge(null != params.getStartDate(), "biz_day", params.getStartDate())
                .le(null != params.getEndDate(), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> doctorIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());
        });

        // 医生信息
        Map<String, AdminUserBO> doctorMap = adminUserClientService.getUserMapByIds(doctorIds);

        PageData<PatientManageOneDayVO> data = new PageConverter<PatientManageOneDayInfo, PatientManageOneDayVO>() {
            @Override
            public PatientManageOneDayVO converter(PatientManageOneDayInfo entity) {
                PatientManageOneDayVO vo = new PatientManageOneDayVO();
                vo.setId(entity.getId());
                vo.setBizDay(entity.getBizDay());
                vo.setDoctorId(entity.getDoctorId());
                vo.setTotalNum(entity.getTotalNum());
                vo.setExcludeNum(entity.getExcludeNum());
                vo.setRemarkNum(entity.getRemarkNum());
                vo.setJoinNum(entity.getJoinNum());
                vo.setOpDate(entity.getOpDate());

                AdminUserBO doctor = doctorMap.get(entity.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorName(doctor.getRealName());
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }
    
    @Override
    public void saveBatch(List<PatientManageOneDayAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        List<String> existParams = new ArrayList<>();
        params.forEach(i -> {
            existParams.add(i.getBizDay() + "_" + i.getDoctorId());
        });

        Map<String, PatientManageOneDayInfo> existMap = new HashMap<>();
        List<PatientManageOneDayInfo> existList = patientManageOneDayInfoService.list(new QueryWrapper<PatientManageOneDayInfo>()
            .in("CONCAT(biz_day,'_',doctor_id)", existParams)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> {
                existMap.put(i.getBizDay() + "_" + i.getDoctorId(), i);
            });
        }

        List<PatientManageOneDayInfo> saveList = new ArrayList<>();
        List<PatientManageOneDayInfo> updateList = new ArrayList<>();

        params.forEach(i -> {
            PatientManageOneDayInfo entity = existMap.get(i.getBizDay() + "_" + i.getDoctorId());

            if (null == entity) {
                entity = new PatientManageOneDayInfo();
                entity.setId(StrUtil.format("PMOD{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }

            entity.setBizDay(i.getBizDay());
            entity.setDoctorId(i.getDoctorId());
            entity.setTotalNum(i.getTotalNum());
            entity.setExcludeNum(i.getExcludeNum());
            entity.setJoinNum(i.getJoinNum());
            entity.setRemarkNum(i.getRemarkNum());
            entity.setOpDate(DateUtil.date());
        });
        
        if (CollUtil.isNotEmpty(saveList)) {
            patientManageOneDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            patientManageOneDayInfoService.updateBatchById(updateList);
        }
    }

}
