package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientPushServeDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientPushServeDataOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PushServeDataBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PushServeDataBizServiceImpl implements PushServeDataBizService {

    @Autowired
    private PatientPushServeDataOneDayInfoService patientPushServeDataOneDayInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PatientPushServeDataOneDayInfo params) {
        log.info("PushServeDataBizServiceImpl.add:params >>>>>> {}", JSON.toJSONString(params));
        if (null == params) {
            return;
        }

        if (null == params.getServeNum()) {
            params.setServeNum(0);
        }
        if (null == params.getComeNum()) {
            params.setComeNum(0);
        }
        if (null == params.getReadNum()) {
            params.setReadNum(0);
        }
        if (null == params.getReadUserNum()) {
            params.setReadUserNum(0);
        }
        if (null == params.getClickNum()) {
            params.setClickNum(0);
        }
        if (null == params.getPushFailNum()) {
            params.setPushFailNum(0);
        }
        if (null == params.getNotFollowingPushFailNum()) {
            params.setNotFollowingPushFailNum(0);
        }

        DateTime now = DateUtil.date();

        // 查询当天是否已经存在数据，不存在则新增，存在则更新
        PatientPushServeDataOneDayInfo entity = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .eq("dept_id", params.getDeptId())
                .eq("push_serve_id", params.getPushServeId())
                .eq("biz_day", now.toDateStr())
        );

        if (null != entity) {
            entity.setServeNum(params.getServeNum());
            entity.setComeNum(params.getComeNum());
            entity.setReadNum(params.getReadNum());
            entity.setReadUserNum(params.getReadUserNum());
            entity.setClickNum(params.getClickNum());
            entity.setPushFailNum(params.getPushFailNum());
            entity.setNotFollowingPushFailNum(params.getNotFollowingPushFailNum());
            entity.setOpDate(now);

            patientPushServeDataOneDayInfoService.updateById(entity);
            return;
        }

        params.setId(DoctorDataHelper.getPatientPushServeDataOneDayInfoId());
        params.setBizDay(now.toDateStr());
        params.setOpDate(now);

        patientPushServeDataOneDayInfoService.save(params);
    }

    // @Transactional(rollbackFor = Exception.class)
    // @Override
    // public void addBatch(List<PatientPushServeDataOneDayInfo> params) {
    //     log.info("PushServeDataBizServiceImpl.addBatch:params >>>>>> {}", JSON.toJSONString(params));
    //     if (CollectionUtil.isEmpty(params)) {
    //         return;
    //     }
    //     DateTime now = DateUtil.date();

    //     // 整理查询参数
    //     List<String> queryParams = new ArrayList<>();
    //     params.forEach(i -> queryParams.add(i.getDeptId() + "-" + i.getPushServeId() + "-" + now.toDateStr()));
    //     // 查询当天已存在的数据
    //     List<PatientPushServeDataOneDayInfo> entityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
    //             .in("CONCAT(dept_id, '-', push_serve_id, '-', biz_day)", queryParams)
    //     );
    //     // 更新已存在的数据
    //     Map<String, PatientPushServeDataOneDayInfo> paramsMap = params.stream().collect(Collectors.toMap(i -> i.getDeptId() + "-" + i.getPushServeId() + "-" + now.toDateStr(), i -> i));
    //     if (CollectionUtil.isNotEmpty(entityList)) {
    //         List<PatientPushServeDataOneDayInfo> updateList = new ArrayList<>();
    //         entityList.forEach(i -> {
    //             PatientPushServeDataOneDayInfo param = paramsMap.get(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay());
    //             if (null != param) {
    //                 i.setServeNum(param.getServeNum());
    //                 i.setComeNum(param.getComeNum());
    //                 i.setReadNum(param.getReadNum());
    //                 i.setClickNum(param.getClickNum());
    //                 i.setPushFailNum(param.getPushFailNum());
    //                 i.setNotFollowingPushFailNum(param.getNotFollowingPushFailNum());
    //                 // i.setBizDay(now.toDateStr());
    //                 i.setOpDate(now);

    //                 updateList.add(i);
    //                 paramsMap.remove(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay()); // 把更新的数据移除，留下需要新增的数据
    //             }
    //         });
    //         // 更新
    //         if (CollectionUtil.isNotEmpty(updateList)) {
    //             log.info("PushServeDataBizServiceImpl.addBatch:updateList >>>>>> {}", JSON.toJSONString(updateList));
    //             patientPushServeDataOneDayInfoService.updateBatchById(updateList);
    //         }
    //     }
        
    //     if (CollectionUtil.isNotEmpty(paramsMap)) {
    //         // 处理需要新增的数据
    //         List<PatientPushServeDataOneDayInfo> saveList = new ArrayList<>();
    //         paramsMap.entrySet().forEach(i -> {
    //             PatientPushServeDataOneDayInfo entity = i.getValue();

    //             if (null == entity.getServeNum()) {
    //                 entity.setServeNum(0);
    //             }
    //             if (null == entity.getComeNum()) {
    //                 entity.setComeNum(0);
    //             }
    //             if (null == entity.getReadNum()) {
    //                 entity.setReadNum(0);
    //             }
    //             if (null == entity.getClickNum()) {
    //                 entity.setClickNum(0);
    //             }
    //             if (null == entity.getPushFailNum()) {
    //                 entity.setPushFailNum(0);
    //             }
    //             if (null == entity.getNotFollowingPushFailNum()) {
    //                 entity.setNotFollowingPushFailNum(0);
    //             }
    //             if (null == entity.getReadTotalTime()) {
    //                 entity.setReadTotalTime(0);
    //             }

    //             entity.setId(DoctorDataHelper.getPatientPushServeDataOneDayInfoId());
    //             entity.setBizDay(now.toDateStr());
    //             entity.setOpDate(now);

    //             saveList.add(entity);
    //         });
    //         // 新增
    //         log.info("PatientDiseaseDataBizServiceImpl.addBatch:saveList >>>>>> {}", JSON.toJSONString(saveList));
    //         patientPushServeDataOneDayInfoService.saveBatch(saveList);
    //     }
    // }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<PatientPushServeDataOneDayInfo> params) {
        log.info("PushServeDataBizServiceImpl.addBatch:params >>>>>> {}", JSON.toJSONString(params));
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        
        DateTime now = DateUtil.date();

        // 整理查询参数
        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> queryParams.add(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay()));
        // 查询当天已存在的数据
        List<PatientPushServeDataOneDayInfo> entityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .in("CONCAT(dept_id, '-', push_serve_id, '-', biz_day)", queryParams)
        );
        // 更新已存在的数据
        Map<String, PatientPushServeDataOneDayInfo> paramsMap = new HashMap<>();
        params.forEach(i -> {
            paramsMap.put(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay(), i);
        });
        
        if (CollectionUtil.isNotEmpty(entityList)) {
            List<PatientPushServeDataOneDayInfo> updateList = new ArrayList<>();
            entityList.forEach(i -> {
                PatientPushServeDataOneDayInfo param = paramsMap.get(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay());
                if (null != param) {
                    i.setServeNum(param.getServeNum());
                    i.setComeNum(param.getComeNum());
                    i.setReadNum(param.getReadNum());
                    i.setReadUserNum(param.getReadUserNum());
                    i.setClickNum(param.getClickNum());
                    i.setPushFailNum(param.getPushFailNum());
                    i.setNotFollowingPushFailNum(param.getNotFollowingPushFailNum());
                    i.setReadTotalTime(param.getReadTotalTime());
                    // i.setBizDay(now.toDateStr());
                    i.setOpDate(now);

                    updateList.add(i);
                    paramsMap.remove(i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getBizDay()); // 把更新的数据移除，留下需要新增的数据
                }
            });
            // 更新
            if (CollectionUtil.isNotEmpty(updateList)) {
                log.info("PushServeDataBizServiceImpl.addBatch:updateList >>>>>> {}", JSON.toJSONString(updateList));
                patientPushServeDataOneDayInfoService.updateBatchById(updateList);
            }
        }
        
        if (CollectionUtil.isNotEmpty(paramsMap)) {
            // 处理需要新增的数据
            List<PatientPushServeDataOneDayInfo> saveList = new ArrayList<>();
            paramsMap.entrySet().forEach(i -> {
                PatientPushServeDataOneDayInfo entity = i.getValue();

                if (null == entity.getServeNum()) {
                    entity.setServeNum(0);
                }
                if (null == entity.getComeNum()) {
                    entity.setComeNum(0);
                }
                if (null == entity.getReadNum()) {
                    entity.setReadNum(0);
                }
                if (null == entity.getReadUserNum()) {
                    entity.setReadUserNum(0);
                }
                if (null == entity.getClickNum()) {
                    entity.setClickNum(0);
                }
                if (null == entity.getPushFailNum()) {
                    entity.setPushFailNum(0);
                }
                if (null == entity.getNotFollowingPushFailNum()) {
                    entity.setNotFollowingPushFailNum(0);
                }
                if (null == entity.getReadTotalTime()) {
                    entity.setReadTotalTime(0);
                }

                entity.setId(DoctorDataHelper.getPatientPushServeDataOneDayInfoId());
                entity.setBizDay(entity.getBizDay());
                entity.setOpDate(now);

                saveList.add(entity);
            });
            // 新增
            log.info("PatientDiseaseDataBizServiceImpl.addBatch:saveList >>>>>> {}", JSON.toJSONString(saveList));
            patientPushServeDataOneDayInfoService.saveBatch(saveList);
        }
    }

}
