package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PushServeTypeEnum;
import com.xxh.cloud.healthy.basic.pushserve.client.api.PushServeConfigClientService;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.dto.PushServeConfigClientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataOperatedTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientPushServeDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientPushServeDataOneDayInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientPushServeBehaviorQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientPushServeQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PushServiceStatisticGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PushServiceStatisticQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeTargetVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DeptPushServeVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientBehaviorDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientPushServeBehaviorVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeDataOneDayVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PushServeDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientPushServeDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PushServeDataQueryBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientPushServeClientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientRemindBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientRemindQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PushServeDataQueryBizServiceImpl implements PushServeDataQueryBizService {

    @Autowired
    private PatientPushServeDataOneDayInfoService patientPushServeDataOneDayInfoService;
    @Autowired
    private DeptPatientQueryBizMapper deptPatientQueryBizMapper;
    @Autowired
    private PushServeConfigClientService pushServeConfigClientService;
    @Autowired
    private PatientPushServeDataQueryBizMapper patientPushServeDataQueryBizMapper;
    @Autowired
    private PatientPushServeClientService patientPushServeClientService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private TaskPatientQueryBizService taskPatientQueryBizService;

    @Override
    public List<PushServeDataOneDayVO> query(PushServiceStatisticQueryDTO params) {
        log.info("PushServeQueryBizServiceImpl.query:params >>>>>> {}", JSON.toJSONString(params));
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getEndDate())) {
            params.setEndDate(now);
        }
        // 查询结束时间的数据（默认当前数据）
        List<PatientPushServeDataOneDayInfo> endDateEntityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getOneType()), "one_type", params.getOneType())
                .like(StrUtil.isNotBlank(params.getDiseaseId()), "disease_ids", "," + params.getDiseaseId() + ",")
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "push_serve_id", params.getPushServeId())
                .eq(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
        );
        if (CollectionUtil.isEmpty(endDateEntityList)) {
            return new ArrayList<>();
        }

        // 查询开始时间的数据
        if (StrUtil.isNotBlank(params.getStartDate()) && params.getStartDate().compareTo(params.getEndDate()) < 0) {
            List<PatientPushServeDataOneDayInfo> startDateEntityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                    .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                    .eq(StrUtil.isNotBlank(params.getOneType()), "one_type", params.getOneType())
                    .like(StrUtil.isNotBlank(params.getDiseaseId()), "disease_ids", "," + params.getDiseaseId() + ",")
                    .eq(StrUtil.isNotBlank(params.getPushServeId()), "push_serve_id", params.getPushServeId())
                    .eq(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                    .orderByDesc("biz_day")
            );
            if (CollectionUtil.isNotEmpty(startDateEntityList)) {
                Map<String, PatientPushServeDataOneDayInfo> startDateEntityMap = startDateEntityList.stream().collect(Collectors.toMap(i -> i.getPushServeId(), i -> i));

                endDateEntityList.forEach(i -> {
                    PatientPushServeDataOneDayInfo info = startDateEntityMap.get(i.getPushServeId());
                    if (null != info) {
                        i.setServeNum(i.getServeNum() - info.getServeNum());
                        i.setClickNum(i.getClickNum() - info.getClickNum());
                        i.setReadNum(i.getReadNum() - info.getReadNum());
                        i.setComeNum(i.getComeNum() - info.getComeNum());
                        i.setReadTotalTime(i.getReadTotalTime() - info.getReadTotalTime());
                        i.setNotFollowingPushFailNum(i.getNotFollowingPushFailNum() - info.getNotFollowingPushFailNum());
                        i.setPushFailNum(i.getPushFailNum() - info.getPushFailNum());
                    }
                });
            }
        }

        Map<String, PushServeConfigBO> map = new HashMap<>();
        if (CollectionUtil.isNotEmpty(endDateEntityList)) {
            List<String> pushServeIds = new ArrayList<>();
            endDateEntityList.forEach(i -> pushServeIds.add(i.getPushServeId()));

            PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
            pushServeConfigClientQueryDTO.setIds(pushServeIds);
            List<PushServeConfigBO> pushServeConfigBOs = pushServeConfigClientService.list(pushServeConfigClientQueryDTO);
            if (CollectionUtil.isNotEmpty(pushServeConfigBOs)) {
                pushServeConfigBOs.forEach(i -> {
                    PushServeConfigBO bo = map.get(i.getId());
                    if (null == bo) {
                        map.put(i.getId(), i);
                    }
                });
            }
        }

        return endDateEntityList.stream().map(i -> {
            PushServeDataOneDayVO vo = DoctorDataHelper.getPushServiceDataOneDayVO(i);
            vo.setPushServeConfigBO(map.get(i.getPushServeId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public PushServeDataOneDayVO detail(PushServiceStatisticGetDTO params) {
        log.info("PushServeQueryBizServiceImpl.detail:params >>>>>> {}", JSON.toJSONString(params));
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getEndDate())) {
            params.setEndDate(now);
        }
        // 查询结束时间数据（默认当前时间）
        PatientPushServeDataOneDayInfo endDateEntity = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .eq("push_serve_id", params.getPushServiceId())
                .eq("dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
                .last("limit 1")
        );
        if (null == endDateEntity) {
            return new PushServeDataOneDayVO();
        }

        // 查询开始时间的数据
        if (StrUtil.isNotBlank(params.getStartDate()) && params.getStartDate().compareTo(params.getEndDate()) < 0) {
            PatientPushServeDataOneDayInfo startDateEntity = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                    .eq("push_serve_id", params.getPushServiceId())
                    .eq("dept_id", params.getDeptId())
                    .eq(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                    .orderByDesc("biz_day")
                    .last("limit 1")
            );
            if (null != startDateEntity) {
                endDateEntity.setServeNum(endDateEntity.getServeNum() - startDateEntity.getServeNum());
                endDateEntity.setClickNum(endDateEntity.getClickNum() - startDateEntity.getClickNum());
                endDateEntity.setReadNum(endDateEntity.getReadNum() - startDateEntity.getReadNum());
                endDateEntity.setComeNum(endDateEntity.getComeNum() - startDateEntity.getComeNum());
                endDateEntity.setReadTotalTime(endDateEntity.getReadTotalTime() - startDateEntity.getReadTotalTime());
                endDateEntity.setNotFollowingPushFailNum(endDateEntity.getNotFollowingPushFailNum() - startDateEntity.getNotFollowingPushFailNum());
                endDateEntity.setPushFailNum(endDateEntity.getPushFailNum() - startDateEntity.getPushFailNum());
            }
        }

        PushServeDataOneDayVO vo = DoctorDataHelper.getPushServiceDataOneDayVO(endDateEntity);

        PushServeConfigBO pushServeConfigBO = pushServeConfigClientService.getById(vo.getPushServeId());
        vo.setPushServeConfigBO(pushServeConfigBO);

        return vo;
    }

    @Deprecated
    @Override
    public PageData<PushServePatientVO> queryPatient(PatientPushServeQueryDTO params) {
        log.info("PushServeQueryBizServiceImpl:queryPatient:params:>>>>>> {}", JSON.toJSONString(params));

        // 20240307 不从dwd数仓获取用户信息
        // 查询参数中包含患者信息参数则先从dwd数仓中筛选
        // List<DoctorDwdPatientVO> dwdPatientList = null;
        // List<String> patientIds = new ArrayList<>();
        // if (StrUtil.isNotBlank(params.getPatientName()) || StrUtil.isNotBlank(params.getPatientGenderStandard()) 
        //         || StrUtil.isNotBlank(params.getPhone()) || null != params.getStartAge() || null != params.getEndAge()) {
        //     DoctorDwdPatientQueryDTO doctorDwdPatientQueryDTO = new DoctorDwdPatientQueryDTO();
        //     doctorDwdPatientQueryDTO.setPatientName(params.getPatientName());
        //     doctorDwdPatientQueryDTO.setPhone(params.getPhone());
        //     doctorDwdPatientQueryDTO.setIdCardNo(params.getIdCardNo());
        //     doctorDwdPatientQueryDTO.setPatientGenderStandard(params.getPatientGenderStandard());
        //     doctorDwdPatientQueryDTO.setStartAge(params.getStartAge());
        //     doctorDwdPatientQueryDTO.setEndAge(params.getEndAge());
        //     dwdPatientList = doctorDwdPatientDataBizService.queryPatientList(doctorDwdPatientQueryDTO);
        //     if (CollectionUtil.isEmpty(dwdPatientList)) {
        //         return new PageData<>(null, 0);
        //     }
        //     patientIds = dwdPatientList.stream().map(DoctorDwdPatientVO::getPatientId).collect(Collectors.toList());
        // }
        // 查询患者信息
        List<TaskPatientVO> patientList = null;
        List<String> patientIds = new ArrayList<>();
        if (StrUtil.isNotBlank(params.getPatientName()) || StrUtil.isNotBlank(params.getPatientGenderStandard()) 
                || StrUtil.isNotBlank(params.getPhone()) || null != params.getStartAge() || null != params.getEndAge()) {
            TaskPatientQueryDTO taskPatientQueryDTO = new TaskPatientQueryDTO();
            taskPatientQueryDTO.setPatientName(params.getPatientName());
            taskPatientQueryDTO.setPhone(params.getPhone());
            taskPatientQueryDTO.setIdCardNo(params.getIdCardNo());
            taskPatientQueryDTO.setPatientGenderStandard(params.getPatientGenderStandard());
            taskPatientQueryDTO.setStartAge(params.getStartAge());
            taskPatientQueryDTO.setEndAge(params.getEndAge());
            patientList = taskPatientQueryBizService.simpleList(taskPatientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientIds = patientList.stream().map(TaskPatientVO::getPatientId).collect(Collectors.toList());
        }

        // 查询服务
        IPage<PushServePatientVO> page = deptPatientQueryBizMapper.queryDeptPatientByPushServeId(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                // .eq(StrUtil.isNotBlank(params.getDeptId()), "d.dept_id", params.getDeptId())
                .in(CollectionUtil.isNotEmpty(patientIds), "a.patient_id", patientIds)
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .eq("b.push_serve_id", params.getPushServeId())
                .orderByAsc("a.patient_name")
        );

        List<PushServePatientVO> list = page.getRecords();
        if (CollectionUtil.isNotEmpty(list) && StrUtil.isNotBlank(params.getPatientId())) {
            List<String> pushServeIds = list.stream().map(PushServePatientVO::getPushServeId).collect(Collectors.toList());
            // 查询用户行为表获取行为信息
            // 阅读时间/阅读状态（状态跟时间是一致的）
            List<Map<String, Object>> patientBrowseList = patientPushServeDataQueryBizMapper.patientBrowseMap(new QueryWrapper<DeptPushServeVO>()
                    .in(CollectionUtil.isNotEmpty(pushServeIds), "b.source_three_id", pushServeIds)
                    .eq(StrUtil.isNotBlank(params.getPatientId()), "b.patient_id", params.getPatientId())
                    .eq("b.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
            );
            Map<String, String> patientBrowseMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(patientBrowseList)) {
                patientBrowseList.forEach(i -> {
                    String patientPushServeId = i.get("patientId").toString() + "-" + i.get("pushServeId").toString();

                    String operatedDate = null;
                    Object date = i.get("operatedDate");
                    if (null != date) {
                        if (date instanceof LocalDateTime) {
                            LocalDateTime ldt = (LocalDateTime) date;
                            operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                        } else if (date instanceof LocalDate) {
                            LocalDate ldt = (LocalDate) date;
                            operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                        } else {
                            operatedDate = i.get("operatedDate").toString();
                        }
                    }

                    patientBrowseMap.put(patientPushServeId, operatedDate);
                });
            }

            // 感兴趣状态
            List<Map<String, Object>> interestedList = patientPushServeDataQueryBizMapper.patientInterestedMap(new QueryWrapper<DeptPushServeVO>()
                    .in(CollectionUtil.isNotEmpty(pushServeIds), "a.source_three_id", pushServeIds)
                    .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                    .eq("a.operated_type", PatientDataOperatedTypeEnum.Interested.getCode())
            );
            Map<String, Boolean> interestedMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(interestedList)) {
                interestedList.forEach(i -> {
                    String patientPushServeId = i.get("patientId").toString() + "-" + i.get("pushServeId").toString();
                    interestedMap.put(patientPushServeId, true);
                });
            }

            // 20240307 不从dwd数仓获取用户信息
            // 若之前未查询dwd信息则查询，查询过就不需要查了
            // Map<String, DoctorDwdPatientVO> patientMap = new HashMap<>(); // 从dwd查询患者信息
            // if (null == dwdPatientList) {
            //     patientIds = list.stream().map(PushServePatientVO::getPatientId).collect(Collectors.toList());
            //     // dwd
            //     DoctorDwdPatientQueryDTO doctorDwdPatientQueryDTO = new DoctorDwdPatientQueryDTO();
            //     doctorDwdPatientQueryDTO.setPatientIds(patientIds);
            //     dwdPatientList = doctorDwdPatientDataBizService.queryPatientList(doctorDwdPatientQueryDTO);
            // }
            // if (CollectionUtil.isNotEmpty(dwdPatientList)) {
            //     dwdPatientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            // }
            Map<String, TaskPatientVO> patientMap = new HashMap<>(); // 查询患者信息
            if (null == patientList) {
                patientIds = list.stream().map(PushServePatientVO::getPatientId).collect(Collectors.toList());
                
                TaskPatientQueryDTO taskPatientQueryDTO = new TaskPatientQueryDTO();
                taskPatientQueryDTO.setPatientIds(patientIds);
                patientList = taskPatientQueryBizService.simpleList(taskPatientQueryDTO);
            }
            if (CollectionUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }


            list.forEach(i -> {
                String operatedDate = patientBrowseMap.get(i.getPatientId() + "-" + i.getPushServeId());
                if (StrUtil.isNotBlank(operatedDate)) {
                    i.setReadDate(DateUtil.parseDateTime(operatedDate));
                    i.setRead(YN.YES.getCode());
                }

                Boolean interested = interestedMap.get(i.getPatientId() + "-" + i.getPushServeId());
                if (null != interested && interested) {
                    i.setInterested(YN.YES.getCode());
                }

                // 20240307 不从dwd数仓获取用户信息
                // DoctorDwdPatientVO dwdPatientVO = patientMap.get(i.getPatientId());
                // if (null != dwdPatientVO) {
                //     i.setPatientName(dwdPatientVO.getPatientName());
                //     i.setPatientGenderStandard(dwdPatientVO.getPatientGenderStandard());
                //     i.setBirthday(dwdPatientVO.getBirthday());
                //     i.setPhone(dwdPatientVO.getPhone());
                //     i.setPatientAge(dwdPatientVO.getPatientAge());
                // }
                TaskPatientVO patientVO = patientMap.get(i.getPatientId());
                if (null != patientVO) {
                    i.setPatientName(patientVO.getPatientName());
                    i.setPatientGenderStandard(patientVO.getPatientGenderStandard());
                    i.setBirthday(patientVO.getBirthday());
                    i.setPhone(patientVO.getPhone());
                    i.setPatientAge(patientVO.getAge());
                }
            });
        }

        PageData<PushServePatientVO> data = new PageConverter<PushServePatientVO, PushServePatientVO>() {
            @Override
            public PushServePatientVO converter(PushServePatientVO entity) {
                return entity;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PatientPushServeBehaviorVO> queryPatientPushServeBehavior(PatientPushServeBehaviorQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }
        // 浏览
        List<PatientBehaviorDataVO> browseCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> browseCountMap = browseCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> browseSet = browseCountMap.keySet();

        // 感兴趣
        List<PatientBehaviorDataVO> interestedCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Interested.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> interestedCountMap = interestedCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> interestedSet = interestedCountMap.keySet();

        // 已知晓
        List<PatientBehaviorDataVO> knewCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Knew.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> knewCountMap = knewCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> knewSet = knewCountMap.keySet();

        // 衍生阅读
        List<PatientBehaviorDataVO> furtherReadingCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Further_Reading.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> furtherReadingCountMap = furtherReadingCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> furtherReadingSet = furtherReadingCountMap.keySet();

        // 挂号
        List<PatientBehaviorDataVO> registerCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Register.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> registerCountMap = registerCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> registerSet = registerCountMap.keySet();

        // 在线问诊
        List<PatientBehaviorDataVO> consultationOnlineCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Consultation_Online.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> consultationOnlineCountMap = consultationOnlineCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> consultationOnlineSet = consultationOnlineCountMap.keySet();

        // 就诊
        List<PatientBehaviorDataVO> consultationCountList = patientPushServeDataQueryBizMapper.behaviorCountByDayMap(new QueryWrapper<DeptPushServeVO>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "a.source_three_id", params.getPushServeId())
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Consultation.getCode())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.operated_start_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "a.operated_start_date", params.getEndDate())
        );
        Map<String, Long> consultationCountMap = consultationCountList.stream().collect(Collectors.toMap(PatientBehaviorDataVO::getDate, PatientBehaviorDataVO::getCount));
        Set<String> consultationSet = consultationCountMap.keySet();

        Set<String> set = new HashSet<>();
        set.addAll(browseSet);
        set.addAll(interestedSet);
        set.addAll(knewSet);
        set.addAll(furtherReadingSet);
        set.addAll(registerSet);
        set.addAll(consultationOnlineSet);
        set.addAll(consultationSet);

        List<PatientPushServeBehaviorVO> list = new ArrayList<>();
        for (String date : set) {
            PatientPushServeBehaviorVO vo = new PatientPushServeBehaviorVO();
            vo.setDate(date);

            vo.setBrowseCount(null == browseCountMap.get(date) ? 0 : browseCountMap.get(date));
            vo.setInterestedCount(null == interestedCountMap.get(date) ? 0 : interestedCountMap.get(date));
            vo.setKnewCount(null == knewCountMap.get(date) ? 0 : knewCountMap.get(date));
            vo.setFurtherReadingCount(null == furtherReadingCountMap.get(date) ? 0 : furtherReadingCountMap.get(date));
            vo.setRegisterCount(null == registerCountMap.get(date) ? 0 : registerCountMap.get(date));
            vo.setConsultationOnlineCount(null == consultationOnlineCountMap.get(date) ? 0 : consultationOnlineCountMap.get(date));
            vo.setConsultationCount(null == consultationCountMap.get(date) ? 0 : consultationCountMap.get(date));
            
            list.add(vo);
        }

        Comparator<PatientPushServeBehaviorVO> dateComparator = Comparator.comparing(PatientPushServeBehaviorVO::getDate);
        list.sort(dateComparator);

        return list;
    }

    /**
     * 患者推送服务数据统计
     * 可根据病种查询
     * 1. 查询满足需求的服务信息：无需验证病种，统计的部门查询条件可约束
     * 2. 根据查询的服务查询对应的统计信息
     * 3. 若有时间查询，则时间范围内的数据为边界时间数据差
     */
    @Override
    public List<PushServeDataVO> pushServeDataQuery(PushServiceStatisticQueryDTO params) {
        log.info("PushServeQueryBizServiceImpl.pushServeDataQuery(PushServiceStatisticQueryDTO) >>>>>> {}", JSON.toJSONString(params));
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getEndDate())) {
            params.setEndDate(now);
        }

        // 查询诊前/中/后服务
        List<String> statDimensionTypes = new ArrayList<>();
        if (PushServeTypeEnum.Pre.getCode().equals(params.getOneType())) {
            statDimensionTypes.add(PushServeTypeEnum.Risk.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Alert.getCode());
            statDimensionTypes.add(PushServeTypeEnum.No_visit.getCode());
        } else if (PushServeTypeEnum.During.getCode().equals(params.getOneType())) {
            statDimensionTypes.add(PushServeTypeEnum.Push.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Ntro.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Get_visit.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Article.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Opt_risk.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Service.getCode());
        } else if (PushServeTypeEnum.After.getCode().equals(params.getOneType())) {
            statDimensionTypes.add(PushServeTypeEnum.Push_article.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Visited.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Visit_push.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Medicine.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Get_lost.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Satisfaction.getCode());
            statDimensionTypes.add(PushServeTypeEnum.Dr_service.getCode());
        }
        // 查询推送服务配置信息
        PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
        pushServeConfigClientQueryDTO.setDiseaseId(params.getDiseaseId()); // 根据病种查询
        pushServeConfigClientQueryDTO.setTypes(statDimensionTypes);
        pushServeConfigClientQueryDTO.setStatus(YN.YES.getCode());
        log.info("查询推送服务配置信息>>>>>>", JSON.toJSONString(pushServeConfigClientQueryDTO));
        List<PushServeConfigBO> pushServeConfigBOs = pushServeConfigClientService.list(pushServeConfigClientQueryDTO);

        if (CollectionUtil.isEmpty(pushServeConfigBOs)) {
            return new ArrayList<>(0);
        }

        List<String> pushServeIds = pushServeConfigBOs.stream().map(PushServeConfigBO::getId).collect(Collectors.toList());

        log.info("1>>>>>>>>>>>>>>> {}", JSON.toJSONString(pushServeIds));

        // 查询结束时间的数据（默认最新一条数据）
        List<PatientPushServeDataOneDayInfo> endDateDataList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .select("dept_id", "push_serve_id", "one_type", "MAX(biz_day) biz_day")
                .in(CollectionUtil.isNotEmpty(pushServeIds), "push_serve_id", pushServeIds)
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getOneType()), "one_type", params.getOneType())
                .like(StrUtil.isNotBlank(params.getDiseaseId()), "disease_ids", "," + params.getDiseaseId() + ",")
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "push_serve_id", params.getPushServeId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .groupBy("dept_id, push_serve_id, one_type")
        );

        log.info("2>>>>>>>>>>>>>>> {}", JSON.toJSONString(endDateDataList));

        Map<String, PushServeDataOneDayVO> entityMap = new HashMap<>();

        if (CollectionUtil.isNotEmpty(endDateDataList)) {
            List<String> lastQueryParams = endDateDataList.stream().map(i -> i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getOneType() + "-" + i.getBizDay()).collect(Collectors.toList());

            log.info("3>>>>>>>>>>>>>>> {}", JSON.toJSONString(lastQueryParams));

            List<PatientPushServeDataOneDayInfo> endDateEntityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                    .in("CONCAT(dept_id, '-', push_serve_id, '-', one_type, '-', biz_day)", lastQueryParams)
            );

            log.info("4<<<<<<<<<<<< {}", JSON.toJSONString(endDateEntityList));

            if (StrUtil.isNotBlank(params.getStartDate()) && params.getStartDate().compareTo(params.getEndDate()) < 0) {
                // 查询开始时间的数据（在时间范围外的最新一条数据）
                List<PatientPushServeDataOneDayInfo> startDateDataList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                        .select("dept_id", "push_serve_id", "one_type", "MAX(biz_day) biz_day")
                        .in(CollectionUtil.isNotEmpty(pushServeIds), "push_serve_id", pushServeIds)
                        .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                        .eq(StrUtil.isNotBlank(params.getOneType()), "one_type", params.getOneType())
                        .like(StrUtil.isNotBlank(params.getDiseaseId()), "disease_ids", "," + params.getDiseaseId() + ",")
                        .eq(StrUtil.isNotBlank(params.getPushServeId()), "push_serve_id", params.getPushServeId())
                        .le(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                        .groupBy("dept_id, push_serve_id, one_type")
                );

                log.info("5>>>>>>>>>>>>>>> {}", JSON.toJSONString(startDateDataList));
                
                List<String> oldestDateDataParams = startDateDataList.stream().map(i -> {
                    return i.getDeptId() + "-" + i.getPushServeId() + "-" + i.getOneType() + "-" + i.getBizDay();
                }).collect(Collectors.toList());

                List<PatientPushServeDataOneDayInfo> startDateEntityList = patientPushServeDataOneDayInfoService.list(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                        .in("CONCAT(dept_id, '-', push_serve_id, '-', one_type, '-', biz_day)", oldestDateDataParams)
                );
                
                Map<String, PatientPushServeDataOneDayInfo> startDateEntityMap = startDateEntityList.stream().collect(Collectors.toMap(i -> i.getPushServeId(), i -> i));

                log.info("6>>>>>>>>>>>>>>> {}", JSON.toJSONString(startDateEntityMap));

                endDateEntityList.forEach(i -> {
                    PatientPushServeDataOneDayInfo info = startDateEntityMap.get(i.getPushServeId());
                    if (null != info && i.getBizDay().compareTo(info.getBizDay()) > 0) {
                        i.setServeNum(i.getServeNum() - info.getServeNum());
                        i.setClickNum(i.getClickNum() - info.getClickNum());
                        i.setReadNum(i.getReadNum() - info.getReadNum());
                        i.setReadUserNum(i.getReadUserNum() - info.getReadUserNum());
                        i.setComeNum(i.getComeNum() - info.getComeNum());
                        i.setReadTotalTime(i.getReadTotalTime() - info.getReadTotalTime());
                        i.setNotFollowingPushFailNum(i.getNotFollowingPushFailNum() - info.getNotFollowingPushFailNum());
                        i.setPushFailNum(i.getPushFailNum() - info.getPushFailNum());
                    }
                    
                });
            }

            if (CollectionUtil.isNotEmpty(endDateEntityList)) {
                endDateEntityList.forEach(i -> entityMap.put(i.getPushServeId(), DoctorDataHelper.getPushServiceDataOneDayVO(i)));
            }
        }

        log.info("7>>>>>>>>>>>>>>> {}", JSON.toJSONString(entityMap));

        // 服务触发次数(引擎给患者推送服务的次数)
        Map<String, Integer> pushServePushNumMap = new HashMap<>();
        PatientPushServeQueryClientDTO patientPushServeQueryClientDTO = new PatientPushServeQueryClientDTO();
        patientPushServeQueryClientDTO.setOpType(OpTypeEnum.Create.getCode());
        patientPushServeQueryClientDTO.setPushServeIds(pushServeIds);
        if (StrUtil.isNotBlank(params.getStartDate())) {
            patientPushServeQueryClientDTO.setStartDate(DateUtil.parseDate(params.getStartDate()));
        }
        if (StrUtil.isNotBlank(params.getEndDate())) {
            patientPushServeQueryClientDTO.setEndDate(DateUtil.parseDate(params.getEndDate()));
        }
        List<PatientPushServeClientBO> patientPushServeClientBOList = patientPushServeClientService.getPatientPushServeList(patientPushServeQueryClientDTO);
        if (CollUtil.isNotEmpty(patientPushServeClientBOList)) {
            patientPushServeClientBOList.forEach(i -> {
                Integer num = pushServePushNumMap.get(i.getPushServeId());
                if (null == num) {
                    num = 0;
                }
                pushServePushNumMap.put(i.getPushServeId(), num + 1);
            });
        }

        List<PushServeDataVO> pushServeConfigBOList = pushServeConfigBOs.stream().map(i -> {

            Integer num = pushServePushNumMap.get(i.getId());
            if (null == num) {
                num = 0;
            }

            PushServeDataVO pushServeDataVO = new PushServeDataVO();
            pushServeDataVO.setPushServeDataOneDayVO(entityMap.get(i.getId()));
            pushServeDataVO.setPushServeConfigBO(i);
            pushServeDataVO.setPushServeNum(num);

            return pushServeDataVO;
        }).collect(Collectors.toList());

        // Comparator<PushServeDataVO> dateComparator = Comparator.comparing(PushServeDataVO::getPushServeNum);
        Comparator<PushServeDataVO> dateComparator = new Comparator<PushServeDataVO>() {
            @Override
            public int compare(PushServeDataVO o1, PushServeDataVO o2) {
                Integer p1 = null == o1.getPushServeDataOneDayVO() || null == o1.getPushServeDataOneDayVO().getServeNum() ? 0 : o1.getPushServeDataOneDayVO().getServeNum();
                Integer p2 = null == o2.getPushServeDataOneDayVO() || null == o2.getPushServeDataOneDayVO().getServeNum() ? 0 : o2.getPushServeDataOneDayVO().getServeNum();
                return p1 - p2;
            }
        };
        pushServeConfigBOList.sort(dateComparator.reversed());

        return pushServeConfigBOList;
    }

    /**
     * 查询服务统计详情
     * 1. 根据部门和服务查询服务统计数据
     */
    @Override
    public PushServeDataVO pushServeDataDetail(PushServiceStatisticGetDTO params) {
        log.info("PushServeQueryBizServiceImpl.pushServeDataDetail:params >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        PushServeDataVO vo = new PushServeDataVO();
        
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getEndDate())) {
            params.setEndDate(now);
        }

        // 查询服务信息
        PushServeConfigBO pushServeConfigBO = pushServeConfigClientService.getById(params.getPushServiceId());
        log.info("查询推送服务配置信息>>>>>>", JSON.toJSONString(pushServeConfigBO));
        vo.setPushServeConfigBO(pushServeConfigBO);

        // 服务触发次数(引擎给患者推送服务的次数)
        PatientPushServeQueryClientDTO patientPushServeQueryClientDTO = new PatientPushServeQueryClientDTO();
        patientPushServeQueryClientDTO.setOpType(OpTypeEnum.Create.getCode());
        patientPushServeQueryClientDTO.setPushServeId(pushServeConfigBO.getId());
        if (StrUtil.isNotBlank(params.getStartDate())) {
            patientPushServeQueryClientDTO.setStartDate(DateUtil.parseDate(params.getStartDate()));
        }
        if (StrUtil.isNotBlank(params.getEndDate())) {
            patientPushServeQueryClientDTO.setEndDate(DateUtil.parseDate(params.getEndDate()));
        }
        List<PatientPushServeClientBO> patientPushServeClientBOList = patientPushServeClientService.getPatientPushServeList(patientPushServeQueryClientDTO);
        vo.setPushServeNum(patientPushServeClientBOList.size());

        // 查询结束时间的数据（默认最新一条数据）
        PatientPushServeDataOneDayInfo lastDateData = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .select("dept_id", "push_serve_id", "MAX(biz_day) biz_day")
                .eq("push_serve_id", params.getPushServiceId())
                .eq("dept_id", params.getDeptId())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
                .last(" limit 1 ")
        );

        log.info("1.查询最新的一条统计数据>>>>>>>>>>>>>>> {}", JSON.toJSONString(lastDateData));

        if (null == lastDateData) {
            return vo;
        }

        // 查询结束时间数据（默认当前时间）
        PatientPushServeDataOneDayInfo endDateEntity = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                .eq("push_serve_id", lastDateData.getPushServeId())
                .eq("dept_id", lastDateData.getDeptId())
                .eq("biz_day", lastDateData.getBizDay())
        );
        if (null == endDateEntity) {
            return vo;
        }

        log.info("2.查询最新的一条统计数据>>>>>>>>>>>>>>> {}", JSON.toJSONString(endDateEntity));

        // 查询开始时间的数据
        if (StrUtil.isNotBlank(params.getStartDate()) && params.getStartDate().compareTo(params.getEndDate()) < 0) {
            // 查询开始时间的数据（在时间范围外的最新一条数据）
            PatientPushServeDataOneDayInfo startDateData = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                    .select("dept_id", "push_serve_id", "MAX(biz_day) biz_day")
                    .eq("push_serve_id", params.getPushServiceId())
                    .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                    .le(StrUtil.isNotBlank(params.getStartDate()), "biz_day", params.getStartDate())
                    .orderByAsc("biz_day")
                    .last(" limit 1 ")
            );

            log.info("3.查询距离开始时间最新的一条统计数据>>>>>>>>>>>>>>> {}", JSON.toJSONString(startDateData));

            if (null != startDateData) {
                PatientPushServeDataOneDayInfo startDateEntity = patientPushServeDataOneDayInfoService.getOne(new QueryWrapper<PatientPushServeDataOneDayInfo>()
                        .eq("push_serve_id", startDateData.getPushServeId())
                        .eq("dept_id", startDateData.getDeptId())
                        .eq("biz_day", startDateData.getBizDay())
                        .orderByDesc("biz_day")
                        .last("limit 1")
                );

                log.info("4.查询距离开始时间最新的一条统计数据>>>>>>>>>>>>>>> {}", JSON.toJSONString(startDateEntity));

                if (null != startDateEntity && endDateEntity.getBizDay().compareTo(startDateData.getBizDay()) > 0) {
                    endDateEntity.setServeNum(endDateEntity.getServeNum() - startDateEntity.getServeNum());
                    endDateEntity.setClickNum(endDateEntity.getClickNum() - startDateEntity.getClickNum());
                    endDateEntity.setReadNum(endDateEntity.getReadNum() - startDateEntity.getReadNum());
                    endDateEntity.setComeNum(endDateEntity.getComeNum() - startDateEntity.getComeNum());
                    endDateEntity.setReadTotalTime(endDateEntity.getReadTotalTime() - startDateEntity.getReadTotalTime());
                    endDateEntity.setNotFollowingPushFailNum(endDateEntity.getNotFollowingPushFailNum() - startDateEntity.getNotFollowingPushFailNum());
                    endDateEntity.setPushFailNum(endDateEntity.getPushFailNum() - startDateEntity.getPushFailNum());
                }
            }
            
        }

        PushServeDataOneDayVO pushServeDataOneDayVO = DoctorDataHelper.getPushServiceDataOneDayVO(endDateEntity);
        vo.setPushServeDataOneDayVO(pushServeDataOneDayVO);

        return vo;
    }

    /**
     * 服务对象
     * 查询服务的对象数据
     */
    @Override
    public PageData<PushServeTargetVO> remindUser(PatientPushServeQueryDTO params) {
        log.info("1.PushServeQueryBizServiceImpl:remindUser:params:>>>>>> {}", JSON.toJSONString(params));
        // 查询患者推送服务推送的用户信息
        // TODO 未验证部门
        IPage<PushServeTargetVO> page = deptPatientQueryBizMapper.queryUsersByPushServeId(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<TaskPatientInfo>()
                // .eq(StrUtil.isNotBlank(params.getDeptId()), "d.dept_id", params.getDeptId())
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .eq("a.push_serve_id", params.getPushServeId())
                .in("b.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .like(StrUtil.isNotBlank(params.getUserName()), "c.name", params.getUserName())
                .like(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .orderByDesc("b.schedule_date")
        );
        log.info("1.服务对象数据:<<<<<< {}", JSON.toJSONString(page));

        List<PushServeTargetVO> list = page.getRecords();
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> patientIds = new ArrayList<>();
            List<String> shcheduleIds = new ArrayList<>(); 
            List<String> remindQueryParams = new ArrayList<>();
            list.forEach(i -> {
                patientIds.add(i.getPatientId());
                // 日程不一定有，当生成日程时没有家属则不会生成日程，也不会有提醒
                if (StrUtil.isNotBlank(i.getScheduleId())) {
                    shcheduleIds.add(i.getScheduleId());
                    remindQueryParams.add(i.getPatientPushServeId() + "-" + i.getUserId() + "-" + i.getScheduleDate());
                }
            });
            
            // 20240307 不从dwd数仓获取用户信息
            // 查询dwd患者信息
            // DoctorDwdPatientQueryDTO doctorDwdPatientQueryDTO = new DoctorDwdPatientQueryDTO();
            // doctorDwdPatientQueryDTO.setPatientIds(patientIds);
            // log.info("2.查询dwd患者信息:>>>>>> {}", JSON.toJSONString(doctorDwdPatientQueryDTO));
            // List<DoctorDwdPatientVO> dwdPatientList = doctorDwdPatientDataBizService.queryPatientList(doctorDwdPatientQueryDTO);
            // Map<String, DoctorDwdPatientVO> dwdPatientMap = new HashMap<>();
            // if (CollectionUtil.isNotEmpty(dwdPatientList)) {
            //     dwdPatientList.forEach(i -> dwdPatientMap.put(i.getPatientId(), i));
            // }
            // 患者信息
            TaskPatientQueryDTO taskPatientQueryDTO = new TaskPatientQueryDTO();
            taskPatientQueryDTO.setPatientIds(patientIds);
            log.info("2.查询dwd患者信息:>>>>>> {}", JSON.toJSONString(taskPatientQueryDTO));
            List<TaskPatientVO> patientList = taskPatientQueryBizService.simpleList(taskPatientQueryDTO);
            Map<String, TaskPatientVO> patientMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }

            // 日程行为
            Map<String, String> userScheduleBrowseMap = new HashMap<>();
            Map<String, Boolean> scheduleInterestedMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(shcheduleIds)) {
                // 阅读
                List<Map<String, Object>> userScheduleBrowseList = patientPushServeDataQueryBizMapper.userBrowseMap(new QueryWrapper<DeptPushServeVO>()
                        .eq("a.source_one_type", "schedule")
                        .in(CollectionUtil.isNotEmpty(shcheduleIds), "a.source_one_id", shcheduleIds)
                        .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                );
                if (CollectionUtil.isNotEmpty(userScheduleBrowseList)) {
                    userScheduleBrowseList.forEach(i -> {
                        String operatedDate = null;
                        Object date = i.get("operatedDate");
                        if (null != date) {
                            if (date instanceof LocalDateTime) {
                                LocalDateTime ldt = (LocalDateTime) date;
                                operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                            } else if (date instanceof LocalDate) {
                                LocalDate ldt = (LocalDate) date;
                                operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                            } else {
                                operatedDate = String.valueOf(date);
                            }
                        }

                        userScheduleBrowseMap.put(String.valueOf(i.get("sourceOneId")), operatedDate);
                    });
                }
                // 感兴趣
                List<Map<String, Object>> scheduleInterestedList = patientPushServeDataQueryBizMapper.userInterestedMap(new QueryWrapper<DeptPushServeVO>()
                        .eq("a.source_one_type", "schedule")
                        .in(CollectionUtil.isNotEmpty(shcheduleIds), "a.source_one_id", shcheduleIds)
                        .eq("a.operated_type", PatientDataOperatedTypeEnum.Interested.getCode())
                );
                if (CollectionUtil.isNotEmpty(scheduleInterestedList)) {
                    scheduleInterestedList.forEach(i -> scheduleInterestedMap.put(String.valueOf(i.get("sourceOneId")), true));
                }
            }

            // 查询提醒表
            Map<String, PatientRemindBO> remindMap = new HashMap<>();
            Map<String, String> userRemindBrowseMap = new HashMap<>();
            Map<String, Boolean> remindInterestedMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(remindQueryParams)) {
                List<String> remindIds = new ArrayList<>();
                PatientRemindQueryClientDTO patientRemindQueryClientDTO = new PatientRemindQueryClientDTO();
                patientRemindQueryClientDTO.setQuerParams(remindQueryParams);
                List<PatientRemindBO> remindList = patientPushServeClientService.patientRemindBOList(patientRemindQueryClientDTO);
                if (CollectionUtil.isNotEmpty(remindList)) {
                    remindList.forEach(i -> {
                        remindIds.add(i.getId());
                        remindMap.put(i.getBizId() + "-" + i.getUserId() + "-" + DateUtil.formatDateTime(i.getPushDate()), i);
                    });
                }

                // 提醒行为
                if (CollectionUtil.isNotEmpty(remindIds)) {
                    // 阅读
                    List<Map<String, Object>> userRemindBrowseList = patientPushServeDataQueryBizMapper.userBrowseMap(new QueryWrapper<DeptPushServeVO>()
                            .eq("a.source_one_type", "remind")
                            .in(CollectionUtil.isNotEmpty(remindIds), "a.source_one_id", remindIds)
                            .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                    );
                    if (CollectionUtil.isNotEmpty(userRemindBrowseList)) {
                        userRemindBrowseList.forEach(i -> {
                            String operatedDate = null;
                            Object date = i.get("operatedDate");
                            if (null != date) {
                                if (date instanceof LocalDateTime) {
                                    LocalDateTime ldt = (LocalDateTime) date;
                                    operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                                } else if (date instanceof LocalDate) {
                                    LocalDate ldt = (LocalDate) date;
                                    operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                                } else {
                                    operatedDate = String.valueOf(date);
                                }
                            }

                            userRemindBrowseMap.put(String.valueOf(i.get("sourceOneId")), operatedDate);
                        });
                    }
                    // 感兴趣
                    List<Map<String, Object>> remindInterestedList = patientPushServeDataQueryBizMapper.userInterestedMap(new QueryWrapper<DeptPushServeVO>()
                            .eq("a.source_one_type", "remind")
                            .in(CollectionUtil.isNotEmpty(remindIds), "a.source_one_id", remindIds)
                            .eq("a.operated_type", PatientDataOperatedTypeEnum.Interested.getCode())
                    );
                    if (CollectionUtil.isNotEmpty(remindInterestedList)) {
                        remindInterestedList.forEach(i -> remindInterestedMap.put(String.valueOf(i.get("sourceOneId")), true));
                    }
                }
            }

            list.forEach(i -> {
                // 20240307 不从dwd数仓获取用户信息
                // dwd患者信息
                // DoctorDwdPatientVO dwdPatient = dwdPatientMap.get(i.getPatientId());
                // if (null != dwdPatient) {
                //     i.setPatientName(dwdPatient.getPatientName());
                //     i.setPatientPhone(dwdPatient.getPhone());
                //     i.setPatientBirthday(dwdPatient.getBirthday());
                //     i.setPatientGenderStandard(dwdPatient.getPatientGenderStandard());
                //     i.setPatientAge(dwdPatient.getPatientAge());
                // }
                // 患者信息
                TaskPatientVO patient = patientMap.get(i.getPatientId());
                if (null != patient) {
                    i.setPatientName(patient.getPatientName());
                    i.setPatientPhone(patient.getPhone());
                    i.setPatientBirthday(patient.getBirthday());
                    i.setPatientGenderStandard(patient.getPatientGenderStandard());
                    i.setPatientAge(patient.getAge());
                }

                // 日程
                String scheduleBrowseDate = userScheduleBrowseMap.get(i.getScheduleId());
                if (StrUtil.isNotBlank(scheduleBrowseDate)) {
                    i.setReadDate(scheduleBrowseDate);
                    i.setRead(YN.YES.getCode());
                }
                Boolean scheduleInterested = scheduleInterestedMap.get(i.getScheduleId());
                if (null != scheduleInterested && scheduleInterested) {
                    i.setInterested(YN.YES.getCode());
                }
                // zs202311121446 若userId为空则直接返回未关注
                if (StrUtil.isBlank(i.getUserId())) {
                    i.setPushServeStatus(PatientPushServeConstant.RemindPushStatus.NotFollow.getCode());
                }

                // 提醒
                PatientRemindBO remindBO = remindMap.get(i.getPatientPushServeId() + "-" + i.getUserId() + "-" + i.getScheduleDate());
                if (null != remindBO) {
                    i.setPushServeStatus(remindBO.getPushStatus());
                    i.setPushServeErrorCode(remindBO.getErrorCode());
                    i.setPushServeErrorMsg(remindBO.getErrorMsg());

                    String remindBrowseDate = userRemindBrowseMap.get(remindBO.getId());
                    if (StrUtil.isNotBlank(remindBrowseDate)) {
                        i.setReadDate(remindBrowseDate);
                        i.setRead(YN.YES.getCode());
                    }
                    
                    Boolean remindInterested = remindInterestedMap.get(remindBO.getId());
                    if (null != remindInterested && remindInterested) {
                        i.setInterested(YN.YES.getCode());
                    }
                }
            });
        }

        PageData<PushServeTargetVO> data = new PageConverter<PushServeTargetVO, PushServeTargetVO>() {
            @Override
            public PushServeTargetVO converter(PushServeTargetVO entity) {
                return entity;
            }
        }.getPageData(page);
        return data;
    }

}
