package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import com.xxh.cloud.healthy.data.framework.modules.biz.helper.TaskPatientHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientBizService;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class TaskPatientBizServiceImpl implements TaskPatientBizService {

    @Autowired
    private TaskPatientInfoService taskPatientInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(TaskPatientAddDTO params) {
        log.info("1.患者信息>>>>>> params={}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        
        // 查询患者信息是否存在
        TaskPatientInfo entity = taskPatientInfoService.getOne(new QueryWrapper<TaskPatientInfo>()
                .eq("patient_id", params.getPatientId())
        );
        log.info("2.查询是否存在患者信息<<<<<< entity={}", JSON.toJSONString(entity));
        if (entity == null) {
            entity = TaskPatientHelper.getTaskPatientInfo(params);
            log.info("4.保存患者信息>>>>>> entity={}", JSON.toJSONString(entity));
            taskPatientInfoService.save(entity);
        } else {
            TaskPatientHelper.getTaskPatientInfo(entity, params);
            log.info("3.更新患者信息>>>>>> entity={}", JSON.toJSONString(entity));
            taskPatientInfoService.updateById(entity);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<TaskPatientAddDTO> params) {
        log.info("1.要保存的患者信息>>>>>> params.size={}", params.size());
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        // 查询要保存的数据是否存在
        Set<String> patientIds = params.stream().map(TaskPatientAddDTO::getPatientId).collect(Collectors.toSet());
        List<TaskPatientInfo> existList = taskPatientInfoService.listByIds(patientIds);
        Map<String, TaskPatientInfo> existMap = existList.stream().collect(Collectors.toMap(TaskPatientInfo::getPatientId, i -> i));
        
        List<TaskPatientInfo> addList = new ArrayList<>(); // 待新增的用户信息
        List<TaskPatientInfo> updateList = new ArrayList<>(); // 待更新的用户信息
        params.forEach(i -> {
            if (existMap.containsKey(i.getPatientId())) {
                TaskPatientInfo entity = existMap.get(i.getPatientId());
                TaskPatientHelper.getTaskPatientInfo(entity, i);
                updateList.add(entity);
            } else {
                TaskPatientInfo entity =  TaskPatientHelper.getTaskPatientInfo(i);
                addList.add(entity);
            }
        });

        if (CollectionUtil.isNotEmpty(addList)) {
            taskPatientInfoService.saveBatch(addList);
        }
        if (CollectionUtil.isNotEmpty(updateList)) {
            taskPatientInfoService.updateBatchById(updateList);
        }
    }
}
