package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorDwdPatientBO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientQueryBizService;
import com.xxh.cloud.common.util.DataUtil;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class TaskPatientQueryBizServiceImpl implements TaskPatientQueryBizService {

    @Autowired
    private DeptPatientQueryBizMapper deptPatientQueryBizMapper;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private TaskPatientInfoService taskPatientInfoService;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;

    @Override
    public PageData<TaskPatientVO> query(TaskPatientQueryDTO params) {
        log.info("TaskPatientQueryBizServiceImpl:query:params:>>>>>> {}", JSON.toJSONString(params));

        IPage<TaskPatientInfo> entityPage = null;

        QueryWrapper<TaskPatientInfo> queryWrapper = new QueryWrapper<TaskPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getDeptId()), "e.dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .le(null != params.getStartAge(), "a.birthday", DataUtil.getBirthdayByAge(params.getStartAge())) // 年龄小的出生日期大
                .ge(null != params.getEndAge(), "a.birthday", DataUtil.getBirthdayByAge(params.getEndAge())) // 年龄大的出生日期小
                .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "a.patient_gender_standard", params.getPatientGenderStandard())
                .eq(StrUtil.isNotBlank(params.getRelationType()), "e.type", params.getRelationType()) // 用户与部门的关系类型
        ;

        if (CollectionUtil.isNotEmpty(params.getLabelIds()) || CollectionUtil.isNotEmpty(params.getDiseaseIds()) || CollectionUtil.isNotEmpty(params.getStatDimensionType())) {
            queryWrapper.in(CollectionUtil.isNotEmpty(params.getStatDimensionType()), "c.stat_dimension_type", params.getStatDimensionType())
                .in(CollectionUtil.isNotEmpty(params.getLabelIds()), "c.id", params.getLabelIds())
                .and(CollectionUtil.isNotEmpty(params.getDiseaseIds()), i -> {
                    params.getDiseaseIds().forEach(item -> {
                        i.or().like("c.disease_ids", "," +item + ",");
                    });
                });
            entityPage = deptPatientQueryBizMapper.queryDeptPatient(new Page<>(params.getPage(), params.getRows()), queryWrapper, new QueryWrapper<TaskPatientInfo>().orderByDesc("data_update_date"));
        } else {
            entityPage = deptPatientQueryBizMapper.queryDeptAllPatient(new Page<>(params.getPage(), params.getRows()), queryWrapper, new QueryWrapper<TaskPatientInfo>().orderByDesc("data_update_date"));
        }

        List<TaskPatientInfo> entityList = entityPage.getRecords();

        Map<String, List<MemberFamilyBO>> memberFamilyMap = new HashMap<>(); // 查询家庭成员
        Map<String, DoctorDwdPatientBO> patientClientBOMap = new HashMap<>(); // 死亡状态
        if (CollectionUtil.isNotEmpty(entityList)) {

            List<String> patientIds = entityList.stream().map(TaskPatientInfo::getPatientId).collect(Collectors.toList());

            // family
            MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
            memberFamilyListClientDTO.setPatHisNos(patientIds); // zs20231115 家属表中patientId来自海鹚和本平台的patientId不一致，一致的是patHisNos（就诊人门诊id）
            memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
            List<MemberFamilyBO> memberFamilyList = customerFamilyClientService.listByUser(memberFamilyListClientDTO);
            if (CollectionUtil.isNotEmpty(memberFamilyList)) {
                memberFamilyList.forEach(i -> {
                    List<MemberFamilyBO> members = memberFamilyMap.get(i.getPatHisNo());
                    if (null == members) {
                        members = new ArrayList<>();
                        memberFamilyMap.put(i.getPatHisNo(), members);
                    }
                    members.add(i);
                });
            }
            // MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
            // memberFamilyListClientDTO.setMainPersonIds(patientIds); // zs20240318 替换为主患者id
            // memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
            // List<MemberFamilyBO> memberFamilyList = customerFamilyClientService.listByUser(memberFamilyListClientDTO);
            // if (CollectionUtil.isNotEmpty(memberFamilyList)) {
            //     memberFamilyList.forEach(i -> {
            //         List<MemberFamilyBO> members = memberFamilyMap.get(i.getMainPersonId());
            //         if (null == members) {
            //             members = new ArrayList<>();
            //             memberFamilyMap.put(i.getMainPersonId(), members);
            //         }
            //         members.add(i);
            //     });
            // }

            // 患者是否去世
            List<DoctorDwdPatientBO> patientClientBOList = dwdDataPatientClientService.queryPatientList(patientIds);
            if (CollectionUtil.isNotEmpty(patientClientBOList)) {
                patientClientBOList.forEach(i -> patientClientBOMap.put(i.getPatientId(), i));
            }

        }

        PageData<TaskPatientVO> data = new PageConverter<TaskPatientInfo, TaskPatientVO>() {
            @Override
            public TaskPatientVO converter(TaskPatientInfo entity) {
                TaskPatientVO vo = DoctorDataHelper.getTaskPatientVO(entity);

                // family
                vo.setMemberFamilyList(memberFamilyMap.get(vo.getPatientId()));

                // 患者是否去世
                vo.setDeathStatus(YN.YES.getCode());
                DoctorDwdPatientBO dwdPatient = patientClientBOMap.get(entity.getPatientId());
                if (null != dwdPatient && YN.NO.getCode().equals(dwdPatient.getDeathStatus())) {
                    vo.setDeathStatus(dwdPatient.getDeathStatus());
                    vo.setDeathTime(dwdPatient.getDeathTime());
                    if (StrUtil.isNotBlank(dwdPatient.getDeathTime())) {
                        if (dwdPatient.getDeathTime().length() == 12) {
                            vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmm").toString());
                        }
                        if (dwdPatient.getDeathTime().length() == 14) {
                            vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmmss").toString());
                        }
                    }
                }

                return vo;
            }
        }.getPageData(entityPage);
        return data;
    }

    @Override
    public List<TaskPatientVO> list(TaskPatientQueryDTO params) {
        log.info("TaskPatientQueryBizServiceImpl:query:params:>>>>>> {}", JSON.toJSONString(params));

        List<TaskPatientInfo> entityList = null;

        QueryWrapper<TaskPatientInfo> queryWrapper = new QueryWrapper<TaskPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getDeptId()), "e.dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
                .like(StrUtil.isNotBlank(params.getPatientName()), "a.patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPhone()), "a.phone", params.getPhone())
                .eq(StrUtil.isNotBlank(params.getIdCardNo()), "a.id_card_no", params.getIdCardNo())
                .le(null != params.getStartAge(), "a.birthday", DataUtil.getBirthdayByAge(params.getStartAge())) // 年龄小的出生日期大
                .ge(null != params.getEndAge(), "a.birthday", DataUtil.getBirthdayByAge(params.getEndAge())) // 年龄大的出生日期小
                .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "a.patient_gender_standard", params.getPatientGenderStandard());

        if (CollectionUtil.isNotEmpty(params.getLabelIds()) || CollectionUtil.isNotEmpty(params.getDiseaseIds()) || CollectionUtil.isNotEmpty(params.getStatDimensionType())) {
            entityList = deptPatientQueryBizMapper.listDeptPatient(queryWrapper
                    .in(CollectionUtil.isNotEmpty(params.getStatDimensionType()), "c.stat_dimension_type", params.getStatDimensionType())
                    .in(CollectionUtil.isNotEmpty(params.getLabelIds()), "c.id", params.getLabelIds())
                    .and(CollectionUtil.isNotEmpty(params.getDiseaseIds()), i -> {
                        params.getDiseaseIds().forEach(item -> {
                            i.or().like("c.disease_ids", "," +item + ",");
                        });
                    })
            );
        } else {
            entityList = deptPatientQueryBizMapper.listDeptAllPatient(queryWrapper);
        }

        // Map<String, DoctorDwdPatientVO> patientMap = new HashMap<>();
        Map<String, TaskPatientVO> patientMap = new HashMap<>();
        Map<String, DoctorDwdPatientBO> patientClientBOMap = new HashMap<>(); // 死亡状态
        if (CollectionUtil.isNotEmpty(entityList)) {
            // 若之前未查询dwd信息则查询，查询过就不需要查了
            List<String> patientIds = entityList.stream().map(TaskPatientInfo::getPatientId).collect(Collectors.toList());

            // dwd获取患者信息
            // DoctorDwdPatientQueryDTO doctorDwdPatientQueryDTO = new DoctorDwdPatientQueryDTO();
            // doctorDwdPatientQueryDTO.setPatientIds(patientIds);
            // List<DoctorDwdPatientVO> patientList = doctorDwdPatientDataBizService.queryPatientList(doctorDwdPatientQueryDTO);

            TaskPatientQueryDTO taskPatientQueryDTO = new TaskPatientQueryDTO();
            taskPatientQueryDTO.setPatientIds(patientIds);
            List<TaskPatientVO> patientList = this.simpleList(taskPatientQueryDTO);
            
            
            if (CollectionUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
            
            // 患者是否去世
            List<DoctorDwdPatientBO> patientClientBOList = dwdDataPatientClientService.queryPatientList(patientIds);
            if (CollectionUtil.isNotEmpty(patientClientBOList)) {
                patientClientBOList.forEach(i -> patientClientBOMap.put(i.getPatientId(), i));
            }

        }

        return entityList.stream().map(i -> {
            TaskPatientVO vo = DoctorDataHelper.getTaskPatientVO(i);
            // DoctorDwdPatientVO dwdPatient = patientMap.get(vo.getPatientId());
            TaskPatientVO patient = patientMap.get(vo.getPatientId());
            if (null != patient) {
                vo.setPatientName(patient.getPatientName());
                vo.setBirthday(patient.getBirthday());
                vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                vo.setPhone(patient.getPhone());
                vo.setAge(patient.getAge());
            }

            // 患者是否去世
            vo.setDeathStatus(YN.YES.getCode());
            DoctorDwdPatientBO dwdPatient = patientClientBOMap.get(vo.getPatientId());
            if (null != dwdPatient && YN.NO.getCode().equals(dwdPatient.getDeathStatus())) {
                vo.setDeathStatus(dwdPatient.getDeathStatus());
                vo.setDeathTime(dwdPatient.getDeathTime());
                if (StrUtil.isNotBlank(dwdPatient.getDeathTime())) {
                    if (dwdPatient.getDeathTime().length() == 12) {
                        vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmm").toString());
                    }
                    if (dwdPatient.getDeathTime().length() == 14) {
                        vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmmss").toString());
                    }
                }
            }

            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public TaskPatientVO getById(TaskPatientGetDTO params) {
        log.info("TaskPatientQueryBizServiceImpl:getById:params:>>>>>> {}", JSON.toJSONString(params));
        if (StrUtil.isBlank(params.getDeptId()) || StrUtil.isBlank(params.getPatientId())) {
            return null;
        }
        
        TaskPatientInfo entity = deptPatientQueryBizMapper.getDeptPatient(new QueryWrapper<TaskPatientInfo>()
                .eq("a.dept_id", params.getDeptId())
                .eq("a.patient_id", params.getPatientId())
        );

        TaskPatientVO vo = DoctorDataHelper.getTaskPatientVO(entity);

        if (null != vo) {
            // dwd
            // DoctorDwdPatientGetDTO doctorDwdPatientGetDTO = new DoctorDwdPatientGetDTO();
            // doctorDwdPatientGetDTO.setPatientId(entity.getPatientId());
            // DoctorDwdPatientVO patient = doctorDwdPatientDataBizService.getOnePatient(doctorDwdPatientGetDTO);

            // 患者是否去世
            vo.setDeathStatus(YN.YES.getCode());
            DoctorDwdPatientBO dwdPatient = dwdDataPatientClientService.getOnePatient(vo.getPatientId());
            if (null != dwdPatient && YN.NO.getCode().equals(dwdPatient.getDeathStatus())) {
                vo.setDeathStatus(dwdPatient.getDeathStatus());
                vo.setDeathTime(dwdPatient.getDeathTime());
                if (StrUtil.isNotBlank(dwdPatient.getDeathTime())) {
                    if (dwdPatient.getDeathTime().length() == 12) {
                        vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmm").toString());
                    }
                    if (dwdPatient.getDeathTime().length() == 14) {
                        vo.setDeathTime(DateUtil.parse(dwdPatient.getDeathTime(), "yyyyMMddHHmmss").toString());
                    }
                }
            }

            TaskPatientGetDTO taskPatientGetDTO = new TaskPatientGetDTO();
            taskPatientGetDTO.setPatientId(entity.getPatientId());
            TaskPatientVO patient = this.getOneSimple(taskPatientGetDTO);

            if (null != patient) {
                vo.setPatientName(patient.getPatientName());
                vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                vo.setBirthday(patient.getBirthday());
                vo.setPhone(patient.getPhone());
                vo.setAge(patient.getAge());
            }
        }

        return vo;
    }

    @Override
    public List<TaskPatientVO> simpleList(TaskPatientQueryDTO params) {
        List<TaskPatientInfo> list = taskPatientInfoService.list(new QueryWrapper<TaskPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getMainPersonId()), "main_person_id", params.getMainPersonId())
                .like(StrUtil.isNotBlank(params.getPatientName()), "patient_name", params.getPatientName())
                .like(StrUtil.isNotBlank(params.getPhone()), "phone", params.getPhone())
                .eq(StrUtil.isNotBlank(params.getIdCardNo()), "id_card_no", params.getIdCardNo())
                .eq(StrUtil.isNotBlank(params.getPatientGenderStandard()), "patient_gender_standard", params.getPatientGenderStandard())
                .le(null != params.getStartAge(), "birthday", DataUtil.getBirthdayByAge(params.getStartAge())) // 年龄小的出生日期大
                .ge(null != params.getEndAge(), "birthday", DataUtil.getBirthdayByAge(params.getEndAge())) // 年龄大的出生日期小
                // .last(" limit 1000 ") // 限制查询数量 // zs20240902导出excel所以取消限制
        );

        return list.stream().map(DoctorDataHelper::getTaskPatientVO).collect(Collectors.toList());
    }

    @Override
    public TaskPatientVO getOneSimple(TaskPatientGetDTO params) {
        if (StrUtil.isBlank(params.getPatientId())) {
            return null;
        }
        TaskPatientInfo entity = taskPatientInfoService.getById(params.getPatientId());

        return DoctorDataHelper.getTaskPatientVO(entity);
    }

}
