package com.xxh.cloud.healthy.data.framework.modules.biz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.StandardExamLabItem;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.StandardExamLabItemRel;
import com.xxh.cloud.healthy.data.framework.modules.base.service.StandardExamLabItemRelService;
import com.xxh.cloud.healthy.data.framework.modules.base.service.StandardExamLabItemService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.ViewUndrugztinfoBizService;
import com.xxh.cloud.healthy.data.framework.modules.third.ViewUndrugztinfoApiHelper;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.ViewUndrugztinfoInfo;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.ViewUndrugztinfoResp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class ViewUndrugztinfoBizServiceImpl implements ViewUndrugztinfoBizService {

    @Autowired
    private StandardExamLabItemService standardExamLabItemService;
    @Autowired
    private StandardExamLabItemRelService standardExamLabItemRelService;

    @Override
    public void sync(String startTime, String endTime) {
        List<ViewUndrugztinfoResp> resps = ViewUndrugztinfoApiHelper.viewUndrugztinfo(startTime, endTime);
        if (CollUtil.isEmpty(resps)) {
            return;
        }
        log.info(StrUtil.format("同步组套数量:{}", resps.size()));
        List<StandardExamLabItem> entityList = new ArrayList<>();
        List<StandardExamLabItemRel> relList = new ArrayList<>();
        List<String> groupItemCodes = new ArrayList<>();
        for (ViewUndrugztinfoResp resp : resps) {
            StandardExamLabItem groupEntity = setStandardExamLabItem(resp.getPackageInfo());
            entityList.add(groupEntity);
            if (CollUtil.isNotEmpty(resp.getInfo())) {
                groupItemCodes.add(groupEntity.getItemCode());
                for (ViewUndrugztinfoInfo info : resp.getInfo()) {
                    StandardExamLabItem infoEntity = setStandardExamLabItem(info);
                    entityList.add(infoEntity);
                    // 组套关系的entity
                    StandardExamLabItemRel rel = new StandardExamLabItemRel();
                    rel.setId(IdUtil.getSnowflake(3,1).nextIdStr());
                    rel.setGroupItem(groupEntity.getItemCode());
                    rel.setInfoItem(info.getItemCode());
                    rel.setRowsStatus(YN.YES.getCode());
                    relList.add(rel);
                }
            }
        }

        if (CollUtil.isNotEmpty(entityList)) {
            standardExamLabItemService.saveOrUpdateBatch(entityList);
        }
        if (CollUtil.isNotEmpty(relList)) {
            standardExamLabItemRelService.remove(new QueryWrapper<StandardExamLabItemRel>().in("group_item", groupItemCodes));
            standardExamLabItemRelService.saveBatch(relList);
        }
    }

    public StandardExamLabItem setStandardExamLabItem(ViewUndrugztinfoInfo info) {
        StandardExamLabItem entity = new StandardExamLabItem();
        BeanUtil.copyProperties(info, entity);
        entity.setRowsStatus(YN.YES.getCode());
        return entity;
    }
}
