package com.xxh.cloud.healthy.data.framework.modules.client.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DataDwsPatientDataLastDateBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorDwdPatientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDiagnosisReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientExamineReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientChangeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientLaboratoryReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.DataDwdPatientQueryClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDiagnosisDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientExamineAndLaboratoryReportClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientRecipeMedicineClientDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorDwdPatientGetDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DoctorDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwsPatientDataLastDateVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DoctorDwdPatientDataClientServiceImpl implements DwdDataPatientClientService {

    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;

    @Override
    public DataDwsPatientDataLastDateBO getPatientDataLastDateInfo(String patientId) {
        DataDwsPatientDataLastDateVO vo = doctorDwdPatientDataBizService.getPatientDataLastDateInfo(patientId);
        if (null == vo) {
            return null;
        }

        DataDwsPatientDataLastDateBO bo = new DataDwsPatientDataLastDateBO();
        bo.setPatientId(vo.getPatientId());
        bo.setLastOpDate(vo.getLastOpDate());
        bo.setContentDate(vo.getContentDate());
        bo.setContentMap(vo.getContentMap());

        return bo;
    }

    @Override
    public List<DataDwsPatientDataLastDateBO> listPatientDataLastDateInfo(List<String> patientIds) {
        List<DataDwsPatientDataLastDateVO> list = doctorDwdPatientDataBizService.listPatientDataLastDateInfo(patientIds);
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(vo -> {
            DataDwsPatientDataLastDateBO bo = new DataDwsPatientDataLastDateBO();
            bo.setPatientId(vo.getPatientId());
            bo.setLastOpDate(vo.getLastOpDate());
            bo.setContentDate(vo.getContentDate());
            bo.setContentMap(vo.getContentMap());
            return bo;
        }).collect(Collectors.toList());
    }

    @Override
    public Map<String, DataDwsPatientDataLastDateBO> mapPatientDataLastDateInfo(List<String> patientIds) {
        List<DataDwsPatientDataLastDateBO> list = listPatientDataLastDateInfo(patientIds);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }

        Map<String, DataDwsPatientDataLastDateBO> map = new HashMap<>();
        list.forEach(i -> {
            DataDwsPatientDataLastDateBO bo = map.get(i.getPatientId());
            if (null == bo || bo.getLastOpDate().compareTo(i.getLastOpDate()) < 0) {
                map.put(i.getPatientId(), i);
            }
        });

        return map;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatient(PatientDataClientDTO params) {
        return doctorDwdPatientDataBizService.listPatientInpatient(params);
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateInfo(PatientLastDataQueryDTO params) {
        return doctorDwdPatientDataBizService.listPatientInpatientLastDateInfo(params);
    }

    @Override
    public Map<String, PatientInpatientRecordClientBO> mapPatientInpatientLastDateInfo(PatientLastDataQueryDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new HashMap<>(0);
        }

        List<PatientInpatientRecordClientBO> list = doctorDwdPatientDataBizService.listPatientInpatientLastDateInfo(params);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        
        Map<String, PatientInpatientRecordClientBO> map = new HashMap<>();
        list.forEach(i -> map.put(i.getPatientId(), i));

        return map;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndNotOut(PatientLastDataQueryDTO params) {
        return doctorDwdPatientDataBizService.listPatientInpatientLastDateAndNotOut(params);
    }

    @Override
    public Map<String, PatientInpatientRecordClientBO> mapPatientInpatientLastDateAndNotOut(PatientLastDataQueryDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new HashMap<>(0);
        }

        List<PatientInpatientRecordClientBO> list = doctorDwdPatientDataBizService.listPatientInpatientLastDateAndNotOut(params);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        
        Map<String, PatientInpatientRecordClientBO> map = new HashMap<>();
        list.forEach(i -> map.put(i.getPatientId(), i));

        return map;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientLastDateAndOut(PatientLastDataQueryDTO params) {
        return doctorDwdPatientDataBizService.listPatientInpatientLastDateAndOut(params);
    }

    @Override
    public Map<String, PatientInpatientRecordClientBO> mapPatientInpatientLastDateAndOut(PatientLastDataQueryDTO params) {
        if (CollectionUtil.isEmpty(params.getPatientIds())) {
            return new HashMap<>(0);
        }

        List<PatientInpatientRecordClientBO> list = doctorDwdPatientDataBizService.listPatientInpatientLastDateAndOut(params);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        
        Map<String, PatientInpatientRecordClientBO> map = new HashMap<>();
        list.forEach(i -> map.put(i.getPatientId(), i));

        return map;
    }

    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateInfo(PatientLastDataQueryDTO params) {
        return doctorDwdPatientDataBizService.listPatientOutpatientLastDateInfo(params);
    }
    
    @Override
    public Map<String, PatientDischargeRecordClientBO> mapPatientOutpatientLastDateInfo(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new HashMap<>(0);
        }
        List<PatientDischargeRecordClientBO> list = doctorDwdPatientDataBizService.listPatientOutpatientLastDateInfo(patientIds);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        
        Map<String, PatientDischargeRecordClientBO> map = new HashMap<>();
        list.forEach(i -> map.put(i.getPatientId(), i));

        return map;
    }
    
    @Override
    public List<PatientVisitRecordClientBO> listPatientVisitLastDateInfo(PatientDataClientDTO params) {
        return doctorDwdPatientDataBizService.listPatientVisitInfo(params);
    }
    
    @Override
    public List<PatientDischargeRecordClientBO> listPatientOutpatientLastDateWitiSFSJInfo(PatientLastDataQueryDTO params) {
        return doctorDwdPatientDataBizService.listPatientOutpatientLastDateWitiSFSJInfo(params);
    }
    
    @Override
    public Map<String, PatientVisitRecordClientBO> mapPatientVisitLastDateInfo(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new HashMap<>(0);
        }
        List<PatientVisitRecordClientBO> list = doctorDwdPatientDataBizService.listPatientVisitLastDateInfo(patientIds);
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        
        Map<String, PatientVisitRecordClientBO> map = new HashMap<>();
        list.forEach(i -> map.put(i.getPatientId(), i));

        return map;
    }

    @Override
    public List<PatientExamineReportClientBO> listExamineReportInfo(PatientExamineAndLaboratoryReportClientDTO params) {
        return doctorDwdPatientDataBizService.listExamineReportInfo(params);
    }

    @Override
    public List<PatientLaboratoryReportClientBO> listLaboratoryReportInfo(PatientExamineAndLaboratoryReportClientDTO params) {
        return doctorDwdPatientDataBizService.listLaboratoryReportInfo(params);
    }

    @Override
    public List<PatientRecipeMedicineClientBO> listRecipeMedicineInfo(PatientRecipeMedicineClientDTO params) {
        return doctorDwdPatientDataBizService.listRecipeMedicineInfo(params);
    }

    @Override
    public List<DoctorDwdPatientBO> queryPatientList(List<String> patientIds) {
        DoctorDwdPatientQueryDTO getDTO = new DoctorDwdPatientQueryDTO();
        getDTO.setPatientIds(patientIds);
        List<DoctorDwdPatientVO> vos = doctorDwdPatientDataBizService.queryPatientList(getDTO);
        
        if (CollectionUtil.isEmpty(vos)) {
            return new ArrayList<>(0);
        }

        List<DoctorDwdPatientBO> bos = new ArrayList<>();

        vos.forEach(vo -> {
            DoctorDwdPatientBO bo = new DoctorDwdPatientBO();
            bo.setPatientId(vo.getPatientId());
            bo.setPatientName(vo.getPatientName());
            bo.setPatientGenderStandard(vo.getPatientGenderStandard());
            bo.setPatientAge(vo.getPatientAge());
            bo.setBirthday(vo.getBirthday());
            bo.setCardNo(vo.getCardNo());
            bo.setCardNoTypeStandard(vo.getCardNoTypeStandard());
            bo.setPhone(vo.getPhone());
            bo.setDeathStatus(vo.getDeathStatus());
            bo.setDeathTime(vo.getDeathTime());
            
            bos.add(bo);
        });
        return bos;
    }

    @Override
    public DoctorDwdPatientBO getOnePatient(String patientId) {
        if (StrUtil.isBlank(patientId)) {
            return null;
        }
        DoctorDwdPatientGetDTO getDTO = new DoctorDwdPatientGetDTO();
        getDTO.setPatientId(patientId);
        DoctorDwdPatientVO vo = doctorDwdPatientDataBizService.getOnePatient(getDTO);
        
        if (null == vo) {
            return null;
        }
        DoctorDwdPatientBO bo = new DoctorDwdPatientBO();
        bo.setPatientId(vo.getPatientId());
        bo.setPatientName(vo.getPatientName());
        bo.setPatientGenderStandard(vo.getPatientGenderStandard());
        bo.setPatientAge(vo.getPatientAge());
        bo.setBirthday(vo.getBirthday());
        bo.setCardNo(vo.getCardNo());
        bo.setCardNoTypeStandard(vo.getCardNoTypeStandard());
        bo.setPhone(vo.getPhone());
        bo.setDeathStatus(vo.getDeathStatus());
        bo.setDeathTime(vo.getDeathTime());
        return bo;
    }

    @Override
    public List<PatientDiagnosisReportClientBO> listPatientDiagnosisReportInfo(PatientDiagnosisDataClientDTO params) {
        return doctorDwdPatientDataBizService.listPatientDiagnosisReportInfo(params);
    }

    @Override
    public List<PatientVisitRecordClientBO> patientVisitList(DataDwdPatientQueryClientDTO params) {
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPatientId(params.getPatientId());
        dataDwdPatientQueryDTO.setPatientIds(params.getPatientIds());
        dataDwdPatientQueryDTO.setDeptId(params.getDeptId());
        dataDwdPatientQueryDTO.setDeptIds(params.getDeptIds());
        dataDwdPatientQueryDTO.setStartDate(params.getStartDate());
        dataDwdPatientQueryDTO.setEndDate(params.getEndDate());

        return doctorDwdPatientDataBizService.patientVisitList(dataDwdPatientQueryDTO);
    }

    @Override
    public List<PatientInpatientRecordClientBO> patientInHospitalList(DataDwdPatientQueryClientDTO params) {
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPatientId(params.getPatientId());
        dataDwdPatientQueryDTO.setPatientIds(params.getPatientIds());
        dataDwdPatientQueryDTO.setDeptId(params.getDeptId());
        dataDwdPatientQueryDTO.setDeptIds(params.getDeptIds());
        dataDwdPatientQueryDTO.setStartDate(params.getStartDate());
        dataDwdPatientQueryDTO.setEndDate(params.getEndDate());

        return doctorDwdPatientDataBizService.patientInHospitalClientList(dataDwdPatientQueryDTO);
    }

    @Override
    public List<PatientInpatientChangeClientBO> patientInHospitalChangeList(DataDwdPatientQueryClientDTO params) {
        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPatientId(params.getPatientId());
        dataDwdPatientQueryDTO.setPatientIds(params.getPatientIds());
        dataDwdPatientQueryDTO.setDeptId(params.getDeptId());
        dataDwdPatientQueryDTO.setDeptIds(params.getDeptIds());
        dataDwdPatientQueryDTO.setStartDate(params.getStartDate());
        dataDwdPatientQueryDTO.setEndDate(params.getEndDate());

        return doctorDwdPatientDataBizService.patientInHospitalChangeList(dataDwdPatientQueryDTO);
    }

}
