package com.xxh.cloud.healthy.data.framework.modules.client.service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.DoctorManagePatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorManagePatientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.DoctorManagePatientQueryClientDTO;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorManagePatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorManagePatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorManagePatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DoctorManagePatientClientServiceImpl implements DoctorManagePatientClientService {

    @Autowired
    private DoctorManagePatientInfoService doctorManagePatientInfoService;

    @Override
    public List<DoctorManagePatientBO> simpleList(DoctorManagePatientQueryClientDTO params) {
        ValidatorUtils.validateEntity(params);

        List<DoctorManagePatientInfo> list = doctorManagePatientInfoService.list(new QueryWrapper<DoctorManagePatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
                .eq(StrUtil.isNotBlank(params.getOpSubType()), "op_sub_type", params.getOpSubType())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<DoctorManagePatientVO> voList = list.stream().map(DoctorDataHelper::getDoctorManagePatientVO).collect(Collectors.toList());

        return voList.stream().map(DoctorDataHelper::getDoctorManagePatientBO).collect(Collectors.toList());
    }
    
}
