package com.xxh.cloud.healthy.data.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.xxh.cloud.healthy.data.client.api.TaskDeptLabelClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.TaskDeptLabelRelationBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.*;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskDeptLabelRelationVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DeptLabelRelationBizService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class TaskDeptLabelClientServiceImpl implements TaskDeptLabelClientService {

    @Autowired
    private DeptLabelRelationBizService deptLabelRelationBizService;

    @Override
    public void add(DeptLabelAddDTO params) {
        if (null == params) {
            return;
        }
        DeptLabelRelationAddDTO addDTO = new DeptLabelRelationAddDTO();
        addDTO.setDeptId(params.getDeptId());
        addDTO.setLabelId(params.getLabelId());
        deptLabelRelationBizService.add(addDTO);
    }

    @Override
    public void addBatch(List<DeptLabelAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        List<DeptLabelRelationAddDTO> addDTOs = params.stream().map(i -> {
            DeptLabelRelationAddDTO addDTO = new DeptLabelRelationAddDTO();
            addDTO.setDeptId(i.getDeptId());
            addDTO.setLabelId(i.getLabelId());
            return addDTO;
        }).collect(Collectors.toList());
        deptLabelRelationBizService.addBatch(addDTOs);
    }

    @Override
    public void del(DeptLabelDelDTO params) {
        if (null == params) {
            return;
        }
        DeptLabelRelationDelDTO delDTO = new DeptLabelRelationDelDTO();
        delDTO.setDeptId(params.getDeptId());
        delDTO.setLabelId(params.getLabelId());
        deptLabelRelationBizService.del(delDTO);
    }

    @Override
    public void delBatch(List<DeptLabelDelDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        List<DeptLabelRelationDelDTO> delDTOs = params.stream().map(i -> {
            DeptLabelRelationDelDTO delDTO = new DeptLabelRelationDelDTO();
            delDTO.setDeptId(i.getDeptId());
            delDTO.setLabelId(i.getLabelId());
            return delDTO;
        }).collect(Collectors.toList());
        deptLabelRelationBizService.delBatch(delDTOs);
    }

    @Override
    public List<TaskDeptLabelRelationBO> getByDeptLabelIds(Set<String> deptLabelIds) {
        List<TaskDeptLabelRelationVO> list = deptLabelRelationBizService.getByDeptLabelIds(deptLabelIds);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(DoctorDataHelper::getTaskDeptLabelRelationBO).collect(Collectors.toList());
    }

    @Override
    public List<TaskDeptLabelRelationBO> getByDeptLabel(DeptLabelQueryDTO params) {
        DeptLabelRelationQueryDTO dto = new DeptLabelRelationQueryDTO();
        dto.setDeptId(params.getDeptId());
        dto.setLabelId(params.getLabelId());
        List<TaskDeptLabelRelationVO> list = deptLabelRelationBizService.getByDeptLabel(dto);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(DoctorDataHelper::getTaskDeptLabelRelationBO).collect(Collectors.toList());
    }

}
