package com.xxh.cloud.healthy.data.framework.modules.mq.consumer;

import lombok.extern.log4j.Log4j2;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.xxh.cloud.common.mq.bean.MQPatientDeptAddBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DeptPatientRelationBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 患者挂号数据消费者
 * */
@Log4j2
@Component
@ConditionalOnProperty(name = "consumer.patientRegister.enable", havingValue = "true")
public class PatientRegisterConsumer {

    @Autowired
    private DeptPatientRelationBizService deptPatientRelationBizService;

    @JmsListener(destination = HMQQueueName.Que_PatientRegister)
    public void queueRegisterConsumer(String message) {
        log.info("1.患者消费者消费消息开始>>>>>> message={}", message);

        if (StrUtil.isBlank(message)) {
            return;
        }
        List<MQPatientDeptAddBean> mqMessageList = JSONArray.parseArray(message, MQPatientDeptAddBean.class);

        if (CollectionUtil.isEmpty(mqMessageList)) {
            return;
        }

        Map<String, DeptPatientRelationAddDTO> addMap = new HashMap<>();  // 用于本批次数据去重，如果数据库中没有数据，并且当前批次有重复数据，插入会失败
        mqMessageList.forEach(i -> {
            DeptPatientRelationAddDTO deptPatientRelationAddDTO = new DeptPatientRelationAddDTO();
            deptPatientRelationAddDTO.setPatientId(i.getPatientId());
            deptPatientRelationAddDTO.setDeptId(i.getDeptCode());
            deptPatientRelationAddDTO.setType(DeptPatientTypeEnum.Register.getCode());

            addMap.put(i.getDeptCode() + "-" +  i.getPatientId(), deptPatientRelationAddDTO);
        });

        log.info("2.患者挂号消息信息保存>>>>>> {}", JSON.toJSONString(addMap));
        if (CollectionUtil.isNotEmpty(addMap)) {
            deptPatientRelationBizService.addBatch(addMap.values().stream().map(i -> i).collect(Collectors.toList()));
        }
        
        log.info("3.患者消费者消费消息结束<<<<<<");
    }

}
