package com.xxh.cloud.healthy.data.framework.modules.mq.consumer;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.xxh.cloud.common.mq.bean.MQPatientAddBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.TaskPatientAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.TaskPatientBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
@ConditionalOnProperty(name = "consumer.patient.enable", havingValue = "true")
public class TaskPatientInfoConsumer {

    @Autowired
    private TaskPatientBizService taskPatientBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;

    // 保存患者信息
    @JmsListener(destination = HMQQueueName.Que_PatientInfo)
    public void queuePatientConsumer(String message) {
        log.info("1.患者消费者消费消息开始>>>>>> message={}", message);

        if (StrUtil.isBlank(message)) {
            return;
        }
        List<MQPatientAddBean> mqMessageList = JSONArray.parseArray(message, MQPatientAddBean.class);

        if (CollectionUtil.isEmpty(mqMessageList)) {
            return;
        }

        // 注意：mq中不会传过来患者主id，直接去查询也可能查询不多，需要另起定时任务去同步主id
        // // 查询患者主id
        // List<String> patientIds = mqMessageList.stream().map(MQPatientAddBean::getId).collect(Collectors.toList());
        // DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        // dataDwdPatientQueryDTO.setPatientIds(patientIds);
        // List<DataDwdPatientVO> patientList = doctorDwdPatientDataBizService.patientList(dataDwdPatientQueryDTO);
        // if (CollectionUtil.isEmpty(patientList)) {
        //     return;
        // }
        // Map<String, String> patientMap = new HashMap<>();
        // patientList.forEach(i -> patientMap.put(i.getPatientId(), i.getMainPersonId()));

        // 保存患者信息
        Map<String, TaskPatientAddDTO> addMap = new HashMap<>();  // 用于本批次数据去重，如果数据库中没有数据，并且当前批次有重复数据，插入会失败
        mqMessageList.forEach(i -> {
            TaskPatientAddDTO taskPatientAddDTO = new TaskPatientAddDTO();
            taskPatientAddDTO.setPatientId(i.getId());
            taskPatientAddDTO.setPatientName(i.getName());
            taskPatientAddDTO.setIdCardNo(i.getCardNo());
            taskPatientAddDTO.setBirthday(i.getBirthday());
            taskPatientAddDTO.setPatientGender(i.getGender());
            taskPatientAddDTO.setPatientGenderStandard(i.getGender());
            taskPatientAddDTO.setPhone(i.getPhone());
            taskPatientAddDTO.setHomePhone(i.getHomePhone());
            taskPatientAddDTO.setWorkPhone(i.getWorkPhone());
            taskPatientAddDTO.setLinkPhone(i.getLinkPhone());
            taskPatientAddDTO.setDataUpdateDate(DateUtil.date());
            // taskPatientAddDTO.setMainPersonId(patientMap.get(i.getId()));
            
            addMap.put(i.getId(), taskPatientAddDTO);
        });

        log.info("2.患者消息信息保存>>>>>> {}", JSON.toJSONString(addMap));
        if (CollectionUtil.isNotEmpty(addMap)) {
            taskPatientBizService.addBatch(addMap.values().stream().map(i -> i).collect(Collectors.toList()));
        }

        log.info("3.患者消费者消费消息结束<<<<<<");
    }

}
