package com.xxh.cloud.healthy.data.framework.modules.task.schedule;

import java.util.Date;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.healthy.data.framework.modules.task.service.DrugTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class DrugTask {

    @Autowired
    private DrugTaskService drugTaskService;
    
    // 同步药品任务
    @XxlJob("syncDrugTask")
    public void syncDrugTask() throws Exception {
        log.info("同步药品任务数据开始 >>>>>> ");
        XxlJobHelper.log("同步药品任务数据开始 >>>>>> ");

        // dayNum与startTime、endTime二选一生效，指定时间要按照最多10天的时间进行拆分执行
        // String jobParam = "{\"dayNum\": \"10\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetDay(now, -10); // 默认10天
        Date endTime = DateUtil.date();

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.parseDateTime(end);  // 不能单独设置结束时间
                        }

                    }
                }
            }
        }

        drugTaskService.syncDrugTask(startTime, endTime);

        XxlJobHelper.log("同步药品任务数据结束 <<<<<< ");
        log.info("同步药品任务数据结束 <<<<<< ");
    }
    
    // 同步药品说明任务
    @XxlJob("syncDrugDescriptionTask")
    public void syncDrugDescriptionTask() throws Exception {
        log.info("同步药品说明任务数据开始 >>>>>> ");
        XxlJobHelper.log("同步药品说明任务数据开始 >>>>>> ");

        drugTaskService.syncDrugDescriptionTask();

        XxlJobHelper.log("同步药品说明任务数据结束 <<<<<< ");
        log.info("同步药品说明任务数据结束 <<<<<< ");
    }

}
