package com.xxh.cloud.healthy.data.framework.modules.task.schedule;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientLossDataStatisticService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
// @ConditionalOnProperty(name = "task.enable", havingValue = "true")
public class PatientLossDataStatisticTask {

    @Autowired
    private PatientLossDataStatisticService patientLossDataStatisticService;
    
    // 挂号流失患者
    @XxlJob("patientRegisterLossJob")
    public void patientRegisterLossJob() throws Exception {
        log.info("统计挂号流失患者数据开始 >>>>>> ");
        XxlJobHelper.log("统计挂号流失患者数据开始 >>>>>> ");

        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        // 日统计
        patientLossDataStatisticService.patientRegisterLossJob(startDate, endDate);

        XxlJobHelper.log("统计挂号流失患者数据结束 <<<<<< ");
        log.info("统计挂号流失患者数据结束 <<<<<< ");
    }

    // 就诊流失患者
    @XxlJob("patientVisitLossJob")
    public void patientVisitLossJob() throws Exception {
        log.info("统计就诊流失患者数据开始 >>>>>> ");
        XxlJobHelper.log("统计就诊流失患者数据开始 >>>>>> ");

        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        // 日统计
        patientLossDataStatisticService.patientVisitLossJob(startDate, endDate);

        XxlJobHelper.log("统计就诊流失患者数据结束 <<<<<< ");
        log.info("统计就诊流失患者数据结束 <<<<<< ");
    }

    // 住院流失患者
    @XxlJob("patientInpatientLossJob")
    public void patientInpatientLossJob() throws Exception {
        log.info("查询住院流失患者数据开始 >>>>>> ");
        XxlJobHelper.log("查询住院流失患者数据开始 >>>>>> ");

        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        // 日统计
        patientLossDataStatisticService.patientInpatientLossJob(startDate, endDate);

        XxlJobHelper.log("查询住院流失患者数据结束 <<<<<< ");
        log.info("查询住院流失患者数据结束 <<<<<< ");
    }

}
