package com.xxh.cloud.healthy.data.framework.modules.task.schedule;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientRuleTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 患者待执行规则定时任务
 */
@Slf4j
@Component
// @ConditionalOnProperty(name = "task.enable", havingValue = "true")
public class PatientRuleTask {

    @Autowired
    private PatientRuleTaskService patientRuleTaskService;
    
    // 清除两天前的所有已执行规则数据
    // @Scheduled(cron = "0 0/60 * * * ?") // 每天凌晨1点执行一次
    @XxlJob("removeXePatientRuleIngTask")
    public void removeXePatientRuleIngTaskHandler() throws Exception {
        log.info("清除两天前的所有已执行规则数据开始 >>>>>> ");
        XxlJobHelper.log("清除两天前的所有已执行规则数据开始 >>>>>> ");

        patientRuleTaskService.removeXePatientRuleIngTaskExecute();

        XxlJobHelper.log("清除两天前的所有已执行规则数据结束 <<<<<< ");
        log.info("清除两天前的所有已执行规则数据结束 <<<<<< ");
    }

    // 可以指定时间段执行
    // 查询近几天所有变化的患者数据，插入执行规则表中
    // @Scheduled(cron = "0 0/60 * * * ?") // 每天凌晨1点执行一次
    @XxlJob("exeXePatientRuleIngTask")
    public void exeXePatientRuleIngTaskHandler() throws Exception {
        log.info("查询近几天数据变化的患者执行规则开始 >>>>>> ");
        XxlJobHelper.log("查询近几天数据变化的患者执行规则开始 >>>>>> ");
        
        String jobParam = XxlJobHelper.getJobParam();

        int day = 7; // 默认7天
        String startDate = DateUtil.format(DateUtil.offsetDay(DateUtil.date(), -(day - 1)), "yyyyMMdd");
        String endDate = null;

        if (StrUtil.isNotBlank(jobParam)) {
            if (jobParam.matches("\\d+")) { // 参数为数字则表示天数
                day = Integer.parseInt(jobParam);
                startDate = DateUtil.format(DateUtil.offsetDay(DateUtil.date(), -(day - 1)), "yyyyMMdd");
            } else {
                String[] dateArray = jobParam.split(",");
                
                // 注意时间格式为：yyyMMdd
                String start = dateArray[0].trim();
                String end = null;
                if (dateArray.length == 2) {
                    end = dateArray[1].trim();
                }

                startDate = DateUtil.format(DateUtil.parseDate(start), "yyyyMMdd");
                endDate = DateUtil.format(DateUtil.offsetDay(DateUtil.parseDate(end), 1), "yyyyMMdd");

            }
        }

        patientRuleTaskService.exeXePatientRuleIngTaskExecute(startDate, endDate);

        XxlJobHelper.log("查询近几天数据变化的患者执行规则结束 <<<<<< ");
        log.info("查询近几天数据变化的患者执行规则结束 <<<<<< ");
    }

    // 根据需要去触发执行
    // 查询指定时间段所有变化的患者数据，插入执行规则表中
    @Deprecated // 合并到 exeXePatientRuleIngTask
    @XxlJob("exeXePatientRuleIngTaskWithDate")
    public void exeXePatientRuleIngTaskWithDateHandler() throws Exception {
        log.info("查询指定时间段数据变化的患者执行规则开始 >>>>>> ");
        XxlJobHelper.log("查询指定时间段数据变化的患者执行规则开始 >>>>>> ");
        
        // String jobParam = "2024-05-02,2024-05-21";
        String jobParam = XxlJobHelper.getJobParam();
        if (StrUtil.isBlank(jobParam)) {
            return;
        }

        String[] dateArray = jobParam.split(",");

        // 注意时间格式为：yyyMMdd
        String start = dateArray[0].trim();
        String end = null;
        if (dateArray.length == 2) {
            end = dateArray[1].trim();
        }

        String startDate = DateUtil.format(DateUtil.parseDate(start), "yyyyMMdd");
        String endDate = DateUtil.format(DateUtil.offsetDay(DateUtil.parseDate(end), 1), "yyyyMMdd");

        patientRuleTaskService.exeXePatientRuleIngTaskExecute(startDate, endDate);

        XxlJobHelper.log("查询指定时间段数据变化的患者执行规则结束 <<<<<< ");
        log.info("查询指定时间段数据变化的患者执行规则结束 <<<<<< ");
    }

}
