package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataUpdateTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorFollowPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorFollowPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.DoctorFollowPatientTaskService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorRemindClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorRemindClientAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;

@Service
public class DoctorFollowPatientTaskServiceImpl implements DoctorFollowPatientTaskService {

    @Autowired
    private DoctorFollowPatientInfoService doctorFollowPatientInfoService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private DoctorRemindClientService doctorRemindClientService;
    @Autowired
    private PatientClientService patientClientService;

    @Override
    public void updateFollowPatientDataDateTaskExecute() {
        // 查询所有关注的患者

        int page = 1;
        while (true) {
            IPage<DoctorFollowPatientInfo> pageData = doctorFollowPatientInfoService.page(new Page<>(page, 1000), new QueryWrapper<DoctorFollowPatientInfo>()
                    .eq("rows_status", YN.YES.getCode())
                    .orderByAsc("id")
            );
            if (null == pageData || CollUtil.isEmpty(pageData.getRecords())) {
                break;
            }

            Set<String> patientIds = new HashSet<>();
            pageData.getRecords().forEach(i -> patientIds.add(i.getPatientId()));
            
            Map<String, PatientClientBO> patientMap = new HashMap<>();
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
  
                // 根据关注的患者查询当天的就诊/住院信息
            PatientDataClientDTO patientDataClientDTO = new PatientDataClientDTO();
            patientDataClientDTO.setPatientIds(new ArrayList<>(patientIds));
            patientDataClientDTO.setStartDate(DateUtil.date().toDateStr());

            Map<String, Date> visitMap = new HashMap<>();
            List<PatientVisitRecordClientBO> visitList = doctorDwdPatientDataBizService.listPatientVisitInfo(patientDataClientDTO);
            if (CollUtil.isNotEmpty(visitList)) {
                visitList.forEach(i -> visitMap.put(i.getPatientId(), i.getVisitDate()));
            }

            Map<String, Date> inpatientMap = new HashMap<>();
            List<PatientInpatientRecordClientBO> inpatientList = doctorDwdPatientDataBizService.listPatientInpatientInfo(patientDataClientDTO);
            if (CollUtil.isNotEmpty(inpatientList)) {
                inpatientList.forEach(i -> inpatientMap.put(i.getPatientId(), i.getInpatientDate()));
            }

            // 更新查询出来的信息的患者对应的数据的时间
            List<DoctorFollowPatientInfo> updateList = new ArrayList<>();
            pageData.getRecords().forEach(i -> {
                boolean flag = false;
                Date visitDate = visitMap.get(i.getPatientId());
                if (null != visitDate) {
                    if (null == i.getDataUpdateDate() || i.getDataUpdateDate().before(visitDate)) {
                        i.setDataUpdateDate(visitDate);
                        i.setDataUpdateType(PatientDataUpdateTypeEnum.Visit.getCode());
                        flag = true;
                    }
                }

                Date inpatientDate = inpatientMap.get(i.getPatientId());
                if (null != inpatientDate) {
                    if (null == i.getDataUpdateDate() || i.getDataUpdateDate().before(inpatientDate)) {
                        i.setDataUpdateDate(inpatientDate);
                        i.setDataUpdateType(PatientDataUpdateTypeEnum.Inpatient.getCode());
                        flag = true;
                    }
                }

                if (flag) {
                    updateList.add(i);
                }
            });

            if (CollUtil.isNotEmpty(updateList)) {
                doctorFollowPatientInfoService.updateBatchById(updateList);
                
                this.saveRemind(updateList, patientMap);
            }
            
            page = page + 1;
        }
    }

    private void saveRemind(List<DoctorFollowPatientInfo> followList, Map<String, PatientClientBO> patientMap) {
        List<DoctorRemindClientAddDTO> remindList = new ArrayList<>();
        followList.forEach(i -> {
            PatientClientBO patient = patientMap.get(i.getPatientId());
            if (null == patient) {
                return;
            }

            String dataType = "就诊";
            if (PatientDataUpdateTypeEnum.Inpatient.getCode().equals(i.getDataUpdateType())) {
                dataType = "住院";
            }
            Map<String, String> contentMap = new HashMap<>();
            contentMap.put("patientName", patient.getPatientName());
            contentMap.put("content", dataType);
    
            DoctorRemindClientAddDTO remindDoctor = new DoctorRemindClientAddDTO();
            remindDoctor.setDoctorId(i.getDoctorId());
            remindDoctor.setBizId(i.getId());
            remindDoctor.setBizOneType(PushTaskConstant.OpType.Follow.getCode());
            remindDoctor.setBizTwoType(PushTaskConstant.OpType.Follow.getCode());
            remindDoctor.setTitle("关注用户" + dataType + "消息提醒");
            remindDoctor.setContent(JSON.toJSONString(contentMap));
            remindDoctor.setRemark("关注用户" + dataType + "消息提醒"); // TODO FIXME: 使用html或markdown进行消息的配置
            remindDoctor.setPushDate(DateUtil.date());
            remindDoctor.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());

            remindList.add(remindDoctor);
        });
        
        doctorRemindClientService.addbatch(remindList);
    }
    
}
