package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.data.client.api.constant.DoctorManagePatientConstant;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorLabelPatientDataInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorManagePatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorManagePatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorConsultationLaunchApplyDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorLabelPatientDiseaseDataVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorRelationStatisticVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DoctorLabelPatientStatisticBizQueryMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorLabelPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.DoctorLabelPatientDataTaskService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorLabelPatientDataTaskServiceImpl implements DoctorLabelPatientDataTaskService {

    @Autowired
    private DoctorLabelPatientDataBizService doctorLabelPatientDataBizService;
    @Autowired
    private DoctorManagePatientInfoService doctorManagePatientInfoService;
    @Autowired
    private DoctorLabelPatientStatisticBizQueryMapper doctorLabelPatientStatisticBizQueryMapper;

    @Override
    public void doctorManagePatientStatisticTaskExecute(String startDate, String endDate) {
        Date now = DateUtil.date();

        // 查询标签患者信息获取这些标签对应的病种id，生成以病种id为key，标签为value的map1
        log.info("查询指定时间的患者标签数据 >>>>>> ");
        List<DoctorLabelPatientDiseaseDataVO> patientLabelDiseaseList =  doctorLabelPatientStatisticBizQueryMapper.queryPatientLabelDiseaseList(new QueryWrapper<Void>()
                .eq("b.rows_status", YN.YES.getCode())
                .eq("b.status", YN.YES.getCode())
                .eq("b.biz_type", "tags_type_data")
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .ge("a.data_dt", startDate)
                .lt(StrUtil.isNotBlank(endDate), "a.data_dt", endDate)
        );
        if (CollUtil.isEmpty(patientLabelDiseaseList)) {
            return;
        }
        Map<String, List<DoctorLabelPatientDiseaseDataVO>> doctorLabelPatientDiseaseDataMap = new HashMap<>();
        patientLabelDiseaseList.forEach(i -> {
            if (StrUtil.isNotBlank(i.getDiseaseIds())) {
                List<String> diseaseIds = StrJoinSplitUtils.split2List(i.getDiseaseIds());
                diseaseIds.forEach(j -> {
                    List<DoctorLabelPatientDiseaseDataVO> l = doctorLabelPatientDiseaseDataMap.get(j);
                    if (null == l) {
                        l = new ArrayList<>();
                        doctorLabelPatientDiseaseDataMap.put(j, l);
                    }
                    l.add(i);
                });
            }
        });

        // 查询运营助理关联的科室与病种
        log.info("查询运营助理关联的科室与病种 >>>>>> ");
        List<DoctorRelationStatisticVO> doctorRelationList = doctorLabelPatientStatisticBizQueryMapper.queryDoctorRelationList(new QueryWrapper<DoctorRelationStatisticVO>()
                .eq("a.rows_status", YN.YES.getCode())
                // .eq("a.status", YN.YES.getCode())
                .eq("a.status", "normal")
        );
        if (CollUtil.isEmpty(doctorRelationList)) {
            return;
        }

        // 循环运营助理把所有科室与病种关系都替换为标签关系
        log.info("循环运营助理把所有科室与病种关系都替换为标签关系 >>>>>> ");
        // 1、整理出用来查询标签下患者管理(排除/纳入)的参数（医生、患者、标签、标签时间）
        Map<String, List<DoctorLabelPatientDiseaseDataVO>> doctorLabelMap = new HashMap<>(); // key为doctorId
        Set<String> doctorManagePatientParams = new HashSet<>();
        doctorRelationList.forEach(i -> {
            List<String> diseaseIds = null;

            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                diseaseIds = StrJoinSplitUtils.split2List(i.getRelationId());
            } else if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getDiseaseIds())) {
                diseaseIds = StrJoinSplitUtils.split2List(i.getDiseaseIds());
            }

            if (CollUtil.isNotEmpty(diseaseIds)) {
                diseaseIds.forEach(j -> {
                    List<DoctorLabelPatientDiseaseDataVO> labels = doctorLabelPatientDiseaseDataMap.get(j);
                    if (CollUtil.isNotEmpty(labels)) {
                        List<DoctorLabelPatientDiseaseDataVO> labelList = doctorLabelMap.get(i.getDoctorId());
                        if (null == labelList) {
                            labelList = new ArrayList<>();
                            doctorLabelMap.put(i.getDoctorId(), labelList);
                        }
                        labelList.addAll(labels);

                        labels.forEach(k -> {
                            doctorManagePatientParams.add(i.getDoctorId()+ "_" + k.getPatientId() + "_" + k.getLabelId() + "_" + DateUtil.formatDateTime(k.getDataDt()));
                        });
                    }

                });
            }
        });

        // 排除纳入数据
        log.info("排除纳入数据 >>>>>> ");
        List<String> managePatientIdList = new ArrayList<>();
        Map<String, List<DoctorManagePatientInfo>> managePatientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorManagePatientParams)) {
            List<DoctorManagePatientInfo> managePatientList = doctorManagePatientInfoService.list(new QueryWrapper<DoctorManagePatientInfo>()
                    .eq("rows_status", YN.YES.getCode())
                    .in("CONCAT(doctor_id, '_', patient_id, '_', label_id, '_', label_date)", doctorManagePatientParams)
            );

            if (CollUtil.isNotEmpty(managePatientList)) {
                managePatientList.forEach(i -> {
                    managePatientIdList.add(i.getId());

                    List<DoctorManagePatientInfo> l = managePatientMap.get(i.getDoctorId() + "_" + i.getLabelId() + "_" + DateUtil.formatDate(i.getLabelDate()));
                    if (null == l) {
                        l = new ArrayList<>();
                        managePatientMap.put(i.getDoctorId() + "_" + i.getLabelId() + "_" + DateUtil.formatDate(i.getLabelDate()), l);
                    }

                    l.add(i);
                });
            }
        }

        // 会诊数据
        log.info("会诊数据 >>>>>> ");
        Map<String, List<DoctorConsultationLaunchApplyDataVO>> consultationMap = new HashMap<>(); // 可以为排除纳入数据的id
        if (CollUtil.isNotEmpty(managePatientIdList)) {
            List<DoctorConsultationLaunchApplyDataVO> consultationList = doctorLabelPatientStatisticBizQueryMapper.queryDoctorConsultatioLaunchApplyList(new QueryWrapper<DoctorConsultationLaunchApplyDataVO>()
                    .in("manage_id", managePatientIdList)
            );
            if (CollUtil.isNotEmpty(consultationList)) {
                consultationList.forEach(i -> {
                    List<DoctorConsultationLaunchApplyDataVO> l = consultationMap.get(i.getManageId());
                    if (null == l) {
                        l = new ArrayList<>();
                        consultationMap.put(i.getManageId(), l);
                    }
                    l.add(i);
                });
            }
        }

        // TODO 随访

        log.info("整理出基本统计数据 >>>>>> ");
        List<DoctorLabelPatientDataInfo> dataList = new ArrayList<>();
        doctorLabelMap.forEach((k, v) -> {
            if (CollUtil.isNotEmpty(v)) {
                Map<String, DoctorLabelPatientDataInfo> map = new HashMap<>(); // key由dockerId和labelId、统计日期yyyy-MM-dd拼接
                v.forEach(i -> {
                    String bizDay = DateUtil.formatDate(i.getDataDt());
                    DoctorLabelPatientDataInfo entity = map.get(k + "_" + i.getLabelId() + "_" + bizDay);
                    
                    if (null == entity) {
                        entity = new DoctorLabelPatientDataInfo();
                        entity.setId(DoctorDataHelper.getDoctorLabelPatientDataId());
                        entity.setBizDay(bizDay);
                        entity.setDoctorId(k);
                        entity.setLabelId(i.getLabelId());
                        entity.setOpDate(now);
                        entity.setOpUser("task");

                        map.put(k + "_" + i.getLabelId() + "_" + bizDay, entity);

                        dataList.add(entity);
                    }
                    
                    List<String> patientLabelIds = new ArrayList<>();
                    if (StrUtil.isNotBlank(entity.getPatientLabelIds())) {
                        patientLabelIds = StrJoinSplitUtils.split2List(entity.getPatientLabelIds());
                    }
                    patientLabelIds.add(i.getId());

                    entity.setPatientLabelIds(StrJoinSplitUtils.joinPrefix(patientLabelIds));
                });
            }
        });
        if (CollUtil.isEmpty(dataList)) {
            return;
        }

        
        log.info("整理出需要保存的数据 >>>>>> ");
        List<DoctorLabelPatientDataInfo> list = new ArrayList<>();
        
        dataList.forEach(i -> {
            // 1.排除/纳入数据
            // 没有排除/纳入数据则没有场景，没有场景直接保存
            List<DoctorManagePatientInfo> l = managePatientMap.get(i.getDoctorId() + "_" + i.getLabelId() + "_" + i.getBizDay());
            if (CollUtil.isEmpty(l)) {
                list.add(i);
                return;
            }
            Set<String> scenes = new HashSet<>();
            Map<String, List<String>> excludeMap = new HashMap<>(); // key为场景
            Map<String, List<String>> joinMap = new HashMap<>(); // key为场景
            l.forEach(j -> {
                String scene = j.getOpScene();
                if (StrUtil.isBlank(scene)) {
                    scene = "other";
                }
                scenes.add(scene);

                if (DoctorManagePatientConstant.OpType.EXCLUDE.getCode().equals(j.getOpType())) {
                    List<String> excludes = excludeMap.get(scene);
                    if (null == excludes) {
                        excludes = new ArrayList<>();
                        excludeMap.put(scene, excludes);
                    }
                    excludes.add(j.getId());
                } else if (DoctorManagePatientConstant.OpType.JOIN.getCode().equals(j.getOpType())) {
                    List<String> joins = joinMap.get(scene);
                    if (null == joins) {
                        joins = new ArrayList<>();
                        joinMap.put(scene, joins);
                    }
                    joins.add(j.getId());
                }
            });
            if (CollUtil.isNotEmpty(scenes)) {
                scenes.forEach(j -> {
                    List<String> excludes = excludeMap.get(j);
                    List<String> joins = joinMap.get(j);

                    DoctorLabelPatientDataInfo entity = new DoctorLabelPatientDataInfo();
                    BeanUtil.copyProperties(i, entity);
                    entity.setId(DoctorDataHelper.getDoctorLabelPatientDataId());
                    entity.setScene(j);
                    if ("other".equals(j)) {
                        entity.setScene(null);
                    }

                    if (CollUtil.isNotEmpty(excludes)) {
                        entity.setExcludePatientIds(StrJoinSplitUtils.joinPrefix(excludes));
                    }
                    if (CollUtil.isNotEmpty(joins)) {
                        entity.setJoinPatientIds(StrJoinSplitUtils.joinPrefix(joins));
                        
                        List<String> consultationIds = new ArrayList<>();
                        joins.forEach(k -> {
                            // 会诊
                            List<DoctorConsultationLaunchApplyDataVO> consultations = consultationMap.get(k);
                            if (CollUtil.isNotEmpty(consultations)) {
                                if (CollUtil.isNotEmpty(consultations)) {
                                    consultations.forEach(consultation -> consultationIds.add(consultation.getId()));
                                }
                            }

                            // TODO 随访
                        });
                        if (CollUtil.isNotEmpty(consultationIds)) {
                            entity.setConsultationIds(StrJoinSplitUtils.joinPrefix(consultationIds));
                        }
                    }

                    list.add(entity);
                });
            }

        });

        // 更新或新增统计数据
        log.info("更新或新增统计数据 >>>>>> ");
        doctorLabelPatientDataBizService.saveBatch(list);
    }

}
