package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.common.constant.ModeEnum;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.constant.DoctorManagePatientConstant;
import com.xxh.cloud.healthy.data.client.api.constant.PatientManageDetailConstant;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DoctorManagePatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientManageDetailInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DoctorManagePatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.base.service.PatientManageDetailInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientDiversionOneDayAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageDetailAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageOneDayAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientInHospitalVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DeptPatientQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiversionOneDayBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageDetailBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageOneDayBizService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.DoctorManagePatientTaskService;
import com.xxh.cloud.healthy.doctor.crm.client.api.DoctorPatientRemarkClientService;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorPatientRemarkBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.dto.DoctorPatientRemarkQueryClientDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorRemarkConstant;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelQueryClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelHistoryBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorRelationClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.bo.DoctorRelationBO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorRelationClientQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorManagePatientTaskServiceImpl implements DoctorManagePatientTaskService {

    @Autowired
    private DoctorManagePatientInfoService doctorManagePatientInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private DeptPatientQueryBizMapper deptPatientQueryBizMapper;
    @Autowired
    private PatientLabelQueryClientService patientLabelQueryClientService;
    @Autowired
    private PatientManageDetailInfoService patientManageDetailInfoService;
    @Autowired
    private DoctorPatientRemarkClientService doctorPatientRemarkClientService;
    @Autowired
    private PatientManageDetailBizService patientManageDetailBizService;
    @Autowired
    private PatientManageOneDayBizService patientManageOneDayBizService;
    @Autowired
    private PatientDiversionOneDayBizService patientDiversionOneDayBizService;
    @Autowired
    private DoctorRelationClientService doctorRelationClientService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;

    @Override
    public void doctorManagePatientDetailTask(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }
        
        // 整理出运营助理与其管理标签关系
        log.info("运营助理");
        List<String> doctorIds = deptPatientQueryBizMapper.listAssistantId(); 
        if (CollUtil.isEmpty(doctorIds)) {
            return;
        }
        log.info("运营助理与科室关系");
        List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
        if (CollUtil.isEmpty(doctorList)) {
            return;
        }
        List<String> deptIds = new ArrayList<>();
        Map<String, List<String>> deptDoctorMap = new HashMap<>(); // key:deptId  value:doctorId
        doctorList.forEach(i -> {
            deptIds.add(i.getDeptId());

            List<String> l = deptDoctorMap.get(i.getDeptId());
            if (null == l) {
                l = new ArrayList<>();
                deptDoctorMap.put(i.getDeptId(), l);
            }
            l.add(i.getId());
        });

        log.info("科室");
        List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
        if (CollUtil.isEmpty(deptList)) {
            return;
        }
        log.info("标签与运营助理关系");
        Map<String, Set<String>> labelDoctorMap = new HashMap<>(); // key:labelId  value:doctorId
        deptList.forEach(i -> {
            if (CollUtil.isNotEmpty(i.getDiseaseIds())) {
                LabelQueryPageDTO labelQuery = new LabelQueryPageDTO();
                labelQuery.setDiseaseIds(i.getDiseaseIds());
                labelQuery.setDiseaseQueryMode(ModeEnum.QueryModeEnum.OR.getCode());
                List<LabelBO> labelList = labelClientService.queryList(labelQuery);
                if (CollUtil.isNotEmpty(labelList)) {
                    List<String> labelIds = labelList.stream().map(LabelBO::getId).collect(Collectors.toList());

                    List<String> doctors = deptDoctorMap.get(i.getId());
                    if (CollUtil.isNotEmpty(doctors)) {
                        labelIds.forEach(j -> {
                            Set<String> l = labelDoctorMap.get(j);
                            if (null == l) {
                                l = new HashSet<>();
                                labelDoctorMap.put(j, l);
                            }
                            l.addAll(doctors);
                        });
                    }
                }
            }
        });
        if (CollUtil.isEmpty(labelDoctorMap)) {
            return;
        }

        log.info("日循环统计");
        Set<String> labelIds = labelDoctorMap.keySet();
        while (!startDate.after(endDate)) {
            log.info("循环处理");
            Date queryEndDate = DateUtil.offsetDay(startDate, 1);
            
            // 应管理的患者数：查询每天每个运营助理标签抓到的患者数据
            log.info("患者标签历史");
            LabelQueryDTO label = new LabelQueryDTO();
            label.setOpType(OpTypeEnum.Create.getCode());
            label.setLabelIds(new ArrayList<>(labelIds));
            label.setStartDate(startDate);
            label.setEndDate(queryEndDate);
            List<PatientLabelHistoryBO> doctorPatientLabelList = patientLabelQueryClientService.queryHistory(label);

            List<String> queryParams = new ArrayList<>();
            if (CollUtil.isNotEmpty(doctorPatientLabelList)) {
                doctorPatientLabelList.forEach(i -> {
                    queryParams.add(i.getPatientId() + "_" + i.getLabelId());
                });
            }
            
            // 查询管理操作的患者数
            log.info("运营助理管理患者操作");
            Map<String, DoctorManagePatientInfo> manageMap = new HashMap<>(); // key: doctorId
            List<DoctorManagePatientInfo> manageList = doctorManagePatientInfoService.list(new QueryWrapper<DoctorManagePatientInfo>()
                .eq("rows_status", YN.YES.getCode())
                .in("CONCAT(patient_id,'_',label_id)", queryParams)
            );
            if (CollUtil.isNotEmpty(manageList)) {
                manageList.forEach(i -> {
                    manageMap.put(i.getDoctorId() + "_" + i.getLabelId() + "_" + DateUtil.formatDateTime(i.getLabelDate()), i);
                });
            }

            // 查询备注数据
            log.info("运营助理备注操作");
            Map<String, List<DoctorPatientRemarkBO>> remarkMap = new HashMap<>(); // key:doctorId_labelId
            DoctorPatientRemarkQueryClientDTO remarkQuery = new DoctorPatientRemarkQueryClientDTO();
            remarkQuery.setRelOneType(DoctorRemarkConstant.RelOneType.Label_Patient.getCode());
            remarkQuery.setStartDate(startDate);
            remarkQuery.setQueryParams(queryParams);
            List<DoctorPatientRemarkBO> remarkList = doctorPatientRemarkClientService.list(remarkQuery);
            if (CollUtil.isNotEmpty(remarkList)) {
                remarkList.forEach(i -> {
                    List<DoctorPatientRemarkBO> l = remarkMap.get(i.getDoctorId() + "_" + i.getPatientId() + "_" + i.getRelOneId());
                    if (null == l) {
                        l = new ArrayList<>();
                        remarkMap.put(i.getDoctorId() + "_" + i.getPatientId() + "_" + i.getRelOneId(), l);
                    }
                    l.add(i);
                });
            }

            // 处理
            log.info("处理并保存");
            List<PatientManageDetailAddDTO> saveDetailList = new ArrayList<>();
            doctorPatientLabelList.forEach(i -> {
                Set<String> doctors = labelDoctorMap.get(i.getLabelId());
                if (CollUtil.isNotEmpty(doctors)) {
                    doctors.forEach(j -> {
                        PatientManageDetailAddDTO detail = new PatientManageDetailAddDTO();
                        detail.setDoctorId(j);
                        detail.setPatientId(i.getPatientId());
                        detail.setLabelId(i.getLabelId());
                        detail.setLabelDate(i.getDataDt());
                        detail.setRemarkStatus(YN.NO.getCode());
                        saveDetailList.add(detail);

                        DoctorManagePatientInfo dmp = manageMap.get(j + "_" + i.getLabelId() + "_" + DateUtil.formatDateTime(i.getDataDt()));
                        if (null != dmp) {
                            detail.setSourceType(PatientManageDetailConstant.SourceType.EXCLUDE.getCode());
                            if (DoctorManagePatientConstant.OpType.JOIN.getCode().equals(dmp.getOpType())) {
                                detail.setSourceType(PatientManageDetailConstant.SourceType.JOIN.getCode());
                            }
                            detail.setSourceId(dmp.getId());
                            detail.setSourceDate(dmp.getOpDate());
                        }

                        List<DoctorPatientRemarkBO> dprs = remarkMap.get(j + "_" + i.getPatientId() + "_" + i.getLabelId());
                        if (CollUtil.isNotEmpty(dprs)) {
                            dprs.forEach(k -> {
                                if (k.getUpdateDate().after(i.getDataDt()) && k.getUpdateDate().before(DateUtil.parseDateTime(i.getValidityValue()))) {
                                    detail.setRemarkStatus(YN.YES.getCode());
                                    if (null == detail.getRemarkDate() || detail.getRemarkDate().after(k.getUpdateDate())) {
                                        detail.setRemarkDate(k.getUpdateDate());
                                    }
                                }
                            });
                        }
                    });
                }
            });

            if (CollUtil.isNotEmpty(saveDetailList)) {
                patientManageDetailBizService.saveBatch(saveDetailList);
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

    @Override
    public void doctorManagePatientDayStatTask(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }
        
        log.info("日循环统计");
        while (!startDate.after(endDate)) {
            String bizDay = DateUtil.formatDate(startDate);
            Date queryEndDate = DateUtil.offsetDay(startDate, 1);

            List<Map<String, Object>> pmdList = patientManageDetailInfoService.listMaps(new QueryWrapper<PatientManageDetailInfo>()
                .select("doctor_id as doctorId", "COUNT(1) as totalNum", "COUNT(IF(source_type = 'exclude',1,NULL)) as excludeNum", "COUNT(IF(source_type = 'join',1,NULL)) as joinNum", "COUNT(IF(remark_status = 'Y',1,NULL)) as remarkNum")
                .ge("label_date", startDate)
                .lt("label_date", queryEndDate)
                .groupBy("doctor_id")
            );

            List<PatientManageOneDayAddDTO> saveList = new ArrayList<>();
            if (CollUtil.isNotEmpty(pmdList)) {
                pmdList.forEach(i -> {
                    String doctorId = String.valueOf(i.get("doctorId"));
                    Integer totalNum = Integer.valueOf(String.valueOf(i.get("totalNum")));
                    Integer excludeNum = Integer.valueOf(String.valueOf(i.get("excludeNum")));
                    Integer joinNum = Integer.valueOf(String.valueOf(i.get("joinNum")));
                    Integer remarkNum = Integer.valueOf(String.valueOf(i.get("remarkNum")));

                    PatientManageOneDayAddDTO entity = new PatientManageOneDayAddDTO();
                    entity.setBizDay(bizDay);
                    entity.setDoctorId(doctorId);
                    entity.setTotalNum(totalNum);
                    entity.setExcludeNum(excludeNum);
                    entity.setJoinNum(joinNum);
                    entity.setRemarkNum(remarkNum);
                    
                    saveList.add(entity);
                });
            }

            if (CollUtil.isNotEmpty(saveList)) {
                patientManageOneDayBizService.saveBatch(saveList);
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

    @Override
    public void doctorManagePatientDiversionDayStatTask(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }
        
        log.info("日循环统计");
        while (!startDate.after(endDate)) {
            String bizDay = DateUtil.formatDate(startDate);
            Date queryEndDate = DateUtil.offsetDay(startDate, 1);

            // 查询运营助理管理的患者
            List<Map<String, Object>> doctorPatientList = patientManageDetailInfoService.listMaps(new QueryWrapper<PatientManageDetailInfo>()
                .select("doctor_id as doctorId", "patient_id as patientId")
                // .ge("label_date", startDate)
                // .lt("label_date", queryEndDate)
                .groupBy("doctor_id", "patient_id")
            );
            if (CollUtil.isEmpty(doctorPatientList)) {
                break;
            }

            // 整理出运营助理-患者关系map   key:doctorId  value:patientId
            Map<String, Set<String>> doctorPatientMap = new HashMap<>();
            doctorPatientList.forEach(i -> {
                String doctorId = String.valueOf(i.get("doctorId"));
                String patientId = String.valueOf(i.get("patientId"));

                Set<String> s = doctorPatientMap.get(doctorId);
                if (null == s) {
                    s = new HashSet<>();
                    doctorPatientMap.put(doctorId, s);
                }
                s.add(patientId);
            });

            // 查询运营助理管理的科室列表
            DoctorRelationClientQueryDTO doctorRelationClientQuery = new DoctorRelationClientQueryDTO();
            doctorRelationClientQuery.setType(DoctorRelationConstant.Type.DEPT.getCode());
            List<DoctorRelationBO> doctorDeptList = doctorRelationClientService.list(doctorRelationClientQuery);
            if (CollUtil.isEmpty(doctorDeptList)) {
                break;
            }
            Map<String, Set<String>> doctorDeptMap = new HashMap<>(); // key: doctorId  value: deptId
            Set<String> deptIds = new HashSet<>();
            doctorDeptList.forEach(i -> {
                Set<String> s = doctorDeptMap.get(i.getDoctorId());
                if (null == s) {
                    s = new HashSet<>();
                    doctorDeptMap.put(i.getDoctorId(), s);
                }
                s.add(i.getRelationId());

                deptIds.add(i.getRelationId());
            });

            // 查询指定科室的就诊/住院信息，整理出map：key:deptId value:patientId
            // 就诊
            Map<String, Set<String>> deptPatientVisitMap = new HashMap<>();
            DataDwdPatientQueryDTO visitQuery = new DataDwdPatientQueryDTO();
            visitQuery.setDeptIds(new ArrayList<>(deptIds));
            visitQuery.setStartDate(startDate);
            visitQuery.setEndDate(queryEndDate);
            List<PatientVisitRecordClientBO> visitList = doctorDwdPatientDataBizService.patientVisitList(visitQuery);
            if (CollUtil.isNotEmpty(visitList)) {
                visitList.forEach(i -> {
                    Set<String> s = deptPatientVisitMap.get(i.getDepartmentCodeStandard());
                    if (null == s) {
                        s = new HashSet<>();
                        deptPatientVisitMap.put(i.getDepartmentCodeStandard(), s);
                    }
                    s.add(i.getPatientId());
                });
            }
            // 住院
            Map<String, Set<String>> deptPatientInpatientMap = new HashMap<>();
            DataDwdPatientQueryDTO inpatientQuery = new DataDwdPatientQueryDTO();
            inpatientQuery.setDeptIds(new ArrayList<>(deptIds));
            inpatientQuery.setStartDate(startDate);
            inpatientQuery.setEndDate(queryEndDate);
            List<DataDwdPatientInHospitalVO> inpatientList = doctorDwdPatientDataBizService.patientInHospitalList(visitQuery);
            if (CollUtil.isNotEmpty(inpatientList)) {
                inpatientList.forEach(i -> {
                    Set<String> s = deptPatientInpatientMap.get(i.getAdmitDepartmentCodeStandard());
                    if (null == s) {
                        s = new HashSet<>();
                        deptPatientInpatientMap.put(i.getAdmitDepartmentCodeStandard(), s);
                    }
                    s.add(i.getPatientId());
                });
            }

            List<PatientDiversionOneDayAddDTO> saveList = new ArrayList<>();
            doctorPatientMap.forEach((k, v) -> {
                PatientDiversionOneDayAddDTO entity = new PatientDiversionOneDayAddDTO();
                entity.setBizDay(bizDay);
                entity.setDoctorId(k);
                entity.setTotalNum(v.size());
                entity.setDiversionNum(0);

                Set<String> depIds = doctorDeptMap.get(k);
                depIds.forEach(i -> {
                    Set<String> pflag = new HashSet<>(); // 就诊和住院重复则只取就诊计算
                    Set<String> pvs = deptPatientVisitMap.get(i);
                    if (CollUtil.isNotEmpty(pvs)) {
                        pvs.forEach(j -> {
                            if (v.contains(j)) {
                                entity.setDiversionNum(entity.getDiversionNum() + 1);
                                pflag.add(j);
                            }
                        });
                    }

                    Set<String> pis = deptPatientInpatientMap.get(i);
                    if (CollUtil.isNotEmpty(pis)) {
                        pis.forEach(j -> {
                            if (v.contains(j) && !pflag.contains(j)) {
                                entity.setDiversionNum(entity.getDiversionNum() + 1);
                            }
                        });
                    }
                });
                
                saveList.add(entity);
            });

            if (CollUtil.isNotEmpty(saveList)) {
                patientDiversionOneDayBizService.saveBatch(saveList);
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

}
