package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DrugInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.DrugItemInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DrugInfoService;
import com.xxh.cloud.healthy.data.framework.modules.base.service.DrugItemInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.helper.DoctorDataHelper;
import com.xxh.cloud.healthy.data.framework.modules.task.service.DrugTaskService;
import com.xxh.cloud.healthy.data.framework.modules.third.DrugThirdHttpUtil;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.DrugApiReq;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.DrugApiResp;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.DrugApiResultResp;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.DrugDetailApiResp;
import com.xxh.cloud.healthy.data.framework.modules.third.bean.DrugDetailApiResultResp;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DrugTaskServiceImpl implements DrugTaskService {

    @Autowired
    private DrugInfoService drugInfoService;
    @Autowired
    private DrugItemInfoService drugItemInfoService;

    @Override
    public void syncDrugTask(Date startDate, Date endDate) {
        if (!startDate.before(endDate)) {
            return;
        }

        Date limitStartDate = startDate;
        Date limitEndDate = DateUtil.offsetDay(limitStartDate, 10);
        if (limitEndDate.after(endDate)) {
            limitEndDate = endDate;
        }

        while (true) {
            if (!limitStartDate.before(endDate)) {
                return;
            }
            this.syncDrugTaskExe(DateUtil.formatDateTime(limitStartDate), DateUtil.formatDateTime(limitEndDate));
            limitStartDate = limitEndDate;
            limitEndDate = DateUtil.offsetDay(limitStartDate, 10);
        }
    }

    private void syncDrugTaskExe(String startDate, String endDate) {
        DrugApiReq req = new DrugApiReq();
        req.setStartTime(startDate);
        req.setEndTime(endDate);
        DrugApiResp resp = DrugThirdHttpUtil.getDrugDataHttpApi(req);
        if (null == resp || CollUtil.isEmpty(resp.getResult())) {
            return;
        }

        List<DrugApiResultResp> respDrugList = resp.getResult();

        List<String> drugCodeList = new ArrayList<>();
        respDrugList.forEach(i -> {
            drugCodeList.add(i.getDrugCode());
        });

        // 查询是否存在药品信息
        Map<String, DrugInfo> existMap = new HashMap<>();
        List<DrugInfo> existList = drugInfoService.list(new QueryWrapper<DrugInfo>()
            .eq("rows_status", YN.YES.getCode())
            .in("drug_code", drugCodeList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDrugCode(), i));
        }

        Date now = DateUtil.date();
        List<DrugInfo> drugSaveList = new ArrayList<>();
        List<DrugInfo> drugUpdateList = new ArrayList<>();
        respDrugList.forEach(i -> {
            DrugInfo exist = existMap.get(i.getDrugCode());
            if (null != exist) {
                exist.setDrugName(i.getDrugName());
                exist.setFormalName(i.getFormalName());
                exist.setRegularName(i.getRegularName());
                exist.setApproveInfo(i.getApproveInfo());
                exist.setCaution(i.getCaution());
                exist.setValidState(i.getValidState());
                exist.setValidState(i.getValidState());
                exist.setSyncFlag(YN.YES.getCode());
                exist.setSyncDate(now);
                exist.setStatus(YN.YES.getCode());
                exist.setRowsStatus(YN.YES.getCode());
                drugUpdateList.add(exist);
                return;
            }

            DrugInfo drugInfo = new DrugInfo();
            drugInfo.setId(DoctorDataHelper.getDrugInfoId());
            drugInfo.setDrugCode(i.getDrugCode());
            drugInfo.setDrugName(i.getDrugName());
            drugInfo.setFormalName(i.getFormalName());
            drugInfo.setRegularName(i.getRegularName());
            drugInfo.setApproveInfo(i.getApproveInfo());
            drugInfo.setCaution(i.getCaution());
            drugInfo.setValidState(i.getValidState());
            drugInfo.setValidState(i.getValidState());
            drugInfo.setSyncFlag(YN.YES.getCode());
            drugInfo.setSyncDate(now);
            drugInfo.setStatus(YN.YES.getCode());
            drugInfo.setRowsStatus(YN.YES.getCode());

            drugSaveList.add(drugInfo);
        });

        // 保存
        if (CollUtil.isNotEmpty(drugSaveList)) {
            drugInfoService.saveBatch(drugSaveList);
        }
        // 更新
        if (CollUtil.isNotEmpty(drugUpdateList)) {
            drugInfoService.updateBatchById(drugUpdateList);
        }
    }

    @Override
    public void syncDrugDescriptionTask() {
        boolean b = true;
        while (b) {
            b = syncDrugDescriptionTaskExe();
        }
    }

    public boolean syncDrugDescriptionTaskExe() {

        List<DrugInfo> existList = drugInfoService.list(new QueryWrapper<DrugInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("sync_flag", YN.YES.getCode())
            .last(" limit 100 ")
        );

        if (CollUtil.isEmpty(existList)) {
            return false;
        }

        Map<String, String> drugIdAndCodeMap = new HashMap<>();
        List<String> drugCodeList = new ArrayList<>();
        existList.forEach(i -> {
            drugCodeList.add(i.getDrugCode());
            drugIdAndCodeMap.put(i.getDrugCode(), i.getId());
        });

        Map<String, List<DrugItemInfo>> drugItemMap = new HashMap<>();
        List<DrugItemInfo> drugItemList = drugItemInfoService.list(new QueryWrapper<DrugItemInfo>()
            .in("drug_code", drugCodeList)
        );
        if (CollUtil.isNotEmpty(drugItemList)) {
            drugItemList.forEach(i -> {
                List<DrugItemInfo> l = drugItemMap.get(i.getDrugCode());
                if (null == l) {
                    l = new ArrayList<>();
                    drugItemMap.put(i.getDrugCode(), l);
                }
                l.add(i);
            });
        }

        Map<String, List<DrugDetailApiResultResp>> drugItemRespMap = new HashMap<>();
        existList.forEach(i -> {
            DrugDetailApiResp drugItemResp = DrugThirdHttpUtil.getDrugDetailDataHttpApi(i.getDrugCode());
            log.info("同步药品说明 <<<<<< {}", drugItemResp.getName());
            if (null != drugCodeList && CollUtil.isNotEmpty(drugItemResp.getSections())) {
                drugItemRespMap.put(i.getDrugCode(), drugItemResp.getSections());
            }
        });

        if (CollUtil.isEmpty(drugItemRespMap)) {
            return false;
        }

        Date now = DateUtil.date();

        List<DrugItemInfo> saveList = new ArrayList<>();
        List<DrugItemInfo> updateList = new ArrayList<>();
        // List<String> updateDrugSyncFlagList = new ArrayList<>();
        drugItemRespMap.forEach((k, v) -> {
            // updateDrugSyncFlagList.add(k);

            List<DrugItemInfo> itemList = drugItemMap.get(k);
            if (CollUtil.isNotEmpty(itemList)) {
                itemList.forEach(i -> i.setRowsStatus(YN.NO.getCode()));

                updateList.addAll(itemList); // 需要更新的数据预先保存到待更新数组中

                if (CollUtil.isNotEmpty(v)) {
                    v.forEach(i -> {
                        boolean b = true;
                        // 存在则更新
                        for (DrugItemInfo item : itemList) {
                            if (i.getIndex() == item.getIdx()) {
                                item.setSection(i.getSection());
                                item.setName(i.getName());
                                item.setTitle(i.getTitle());
                                item.setContent(i.getContent());
                                item.setSyncDate(now);
                                item.setStatus(YN.YES.getCode());
                                item.setRowsStatus(YN.YES.getCode());

                                b = false;
                            }
                        }

                        // 不存在则新增
                        if (b) {
                            DrugItemInfo dii = new DrugItemInfo();
                            dii.setId(DoctorDataHelper.getDrugItemInfoId());
                            dii.setDrugCode(k);
                            dii.setDrugId(drugIdAndCodeMap.get(k));
                            dii.setIdx(i.getIndex());
                            dii.setSection(i.getSection());
                            dii.setName(i.getName());
                            dii.setTitle(i.getTitle());
                            dii.setContent(i.getContent());
                            dii.setSyncDate(now);
                            dii.setStatus(YN.YES.getCode());
                            dii.setRowsStatus(YN.NO.getCode());
                            
                            saveList.add(dii);
                        }

                    });
                }

                return;
            }

            if (CollUtil.isNotEmpty(v)) {
                v.forEach(i -> {
                    DrugItemInfo dii = new DrugItemInfo();
                    dii.setId(DoctorDataHelper.getDrugItemInfoId());
                    dii.setDrugCode(k);
                    dii.setDrugId(drugIdAndCodeMap.get(k));
                    dii.setIdx(i.getIndex());
                    dii.setSection(i.getSection());
                    dii.setName(i.getName());
                    dii.setTitle(i.getTitle());
                    dii.setContent(i.getContent());
                    dii.setSyncDate(now);
                    dii.setStatus(YN.YES.getCode());
                    dii.setRowsStatus(YN.NO.getCode());

                    saveList.add(dii);
                });
            }
        });

        if (CollUtil.isNotEmpty(saveList)) {
            drugItemInfoService.saveBatch(saveList);
        }

        if (CollUtil.isNotEmpty(updateList)) {
            drugItemInfoService.updateBatchById(updateList);
        }

        if (CollUtil.isNotEmpty(drugCodeList)) {
            drugInfoService.update(new UpdateWrapper<DrugInfo>()
                .set("sync_flag", YN.NO.getCode())
                .in("drug_code", drugCodeList)
            );
        }

        return true;
    }

}
