package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PushServeTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.constant.LabelStatDimensionTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.PushServeConfigClientService;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.dto.PushServeConfigClientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataDimensionTypeEnum;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataOperatedTypeEnum;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientDiseaseDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.PatientPushServeDataOneDayInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.entity.TaskPatientInfo;
import com.xxh.cloud.healthy.data.framework.modules.base.service.TaskPatientInfoService;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DataDwdPatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DataDwdPatientVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DeptPushServeVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdOperationApplyVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdRegisterRecordVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientDiseaseDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.PatientPushServeDataQueryBizMapper;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiseaseDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientLostLabelBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PushServeDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientDataStatisticService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientInpatientLossService;
import com.xxh.cloud.healthy.push.framework.modules.biz.constant.WechatErrorCodeEnum;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.dto.DeptQueryDTO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class PatientDataStatisticServiceImpl implements PatientDataStatisticService {

    @Autowired
    private PatientDataQueryBizMapper patientDataQueryBizMapper;
    @Autowired
    private PatientDataBizService patientDataBizService;
    @Autowired
    private PatientDiseaseDataQueryBizMapper patientDiseaseDataQueryBizMapper;
    @Autowired
    private PatientDiseaseDataBizService patientDiseaseDataBizService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private PatientPushServeDataQueryBizMapper patientPushServeDataQueryBizMapper;
    @Autowired
    private PushServeDataBizService pushServeDataBizService;
    @Autowired
    private PatientLostLabelBizService patientLostLabelBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private PushServeConfigClientService pushServeConfigClientService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private PatientInpatientLossService patientInpatientLossService;
    @Autowired
    private TaskPatientInfoService taskPatientInfoService;


    // 挂号流失最短时间
    @Value("${xx.loss.register.day:1}")
    private Integer registerLostDay;
    // 住院流失最短时间
    @Value("${xx.loss.inHospital.day:7}")
    private Integer inHospitalLostDay;
    // 手术流失最短时间
    @Value("${xx.loss.operation.day:7}")
    private Integer operationLostDay;
    // // 住院申请与住院的最大间隔时间
    // @Value("${xx.loss.inHospital.reserve:7}")
    // private Integer reserveDay;

    @Override
    public void patientDataStatisticTaskExecute() {
        // 诊前：发现潜在患者
        // 统计：发现潜在患者/未就诊
        this.savePatientData(LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
        // 统计：发现潜在患者/已就诊
        this.savePatientData(LabelStatDimensionTypeEnum.LSD_Pre_Visited, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
        // 统计：发现潜在患者/已确诊
        this.savePatientData(LabelStatDimensionTypeEnum.LSD_Pre_Confirmed, PatientDataDimensionTypeEnum.LSD_Pre_Confirmed);

        // 诊中：优化诊疗过程
        // 统计：挂号流失患者
        this.saveLostPatientData(LabelStatDimensionTypeEnum.LSD_During_Register, PatientDataDimensionTypeEnum.LSD_During_Register);
        // 住院流失患者
        this.saveLostPatientData(LabelStatDimensionTypeEnum.LSD_During_InHospital, PatientDataDimensionTypeEnum.LSD_During_InHospital);
        // 手术流失患者
        this.saveLostPatientData(LabelStatDimensionTypeEnum.LSD_During_Operation, PatientDataDimensionTypeEnum.LSD_During_Operation);

        // 诊后：服务确诊患者 TODO 暂时统计疾病附案例服务次数
        // 疾病管理服务次数
        this.savePatientPushServeData();
    }

    // 诊前：潜在患者
    private void savePatientData(LabelStatDimensionTypeEnum labelStatDimensionTypeEnum, PatientDataDimensionTypeEnum patientDataDimensionTypeEnum) {
        log.info("1.诊前：潜在患者>>>>>> {}, {}", labelStatDimensionTypeEnum.getCode(), patientDataDimensionTypeEnum.getCode());
        DateTime now = DateUtil.date();
        // 所有数据
        List<PatientDataOneDayInfo> allList = patientDataQueryBizMapper.patientHistoryLabelStatistic(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .eq("c.stat_dimension_type", labelStatDimensionTypeEnum.getCode())
        );
        log.info("2.潜在患者所有数据>>>>>> allList={}", JSON.toJSONString(allList));
        if (CollectionUtil.isEmpty(allList)) {
            return;
        }

        // 昨日数据
        List<PatientDataOneDayInfo> yesterdayList = patientDataQueryBizMapper.patientHistoryLabelStatistic(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .eq("c.stat_dimension_type", labelStatDimensionTypeEnum.getCode())
                .lt("a.op_date", now.toDateStr())
        );
        log.info("3.潜在患者昨日数据>>>>>> allList={}", JSON.toJSONString(yesterdayList));
        Map<String, PatientDataOneDayInfo> yesterdayMap = yesterdayList.stream().collect(Collectors.toMap(PatientDataOneDayInfo::getDeptId, i -> i));

        // 处理
        allList.forEach(i -> {
            i.setDataDimension(patientDataDimensionTypeEnum.getCode());
            PatientDataOneDayInfo yesterday = yesterdayMap.get(i.getDeptId());
            if (null != yesterday) {
                i.setAddNum(yesterday.getNum());
            }
        });
        // 保存
        patientDataBizService.addBatch(allList);
    }

    // 诊中：流失患者
    // 逻辑复杂：流失但是查询时间内未流失则未流失
    private void saveLostPatientData(LabelStatDimensionTypeEnum labelStatDimensionTypeEnum, PatientDataDimensionTypeEnum patientDataDimensionTypeEnum) {
        log.info("1.诊中：流失患者>>>>>> {}, {}", labelStatDimensionTypeEnum.getCode(), patientDataDimensionTypeEnum.getCode());
        DateTime now = DateUtil.date();
        // 所有数据
        List<PatientDataOneDayInfo> allList = patientDataQueryBizMapper.patientLostHistoryLabelStatistic(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.stat_dimension_type", labelStatDimensionTypeEnum.getCode()), OpTypeEnum.Create.getCode());

        log.info("2.流失患者所有数据>>>>>> allList={}", JSON.toJSONString(allList));

        if (CollectionUtil.isEmpty(allList)) {
            return;
        }

        // 昨日数据
        List<PatientDataOneDayInfo> yesterdayList = patientDataQueryBizMapper.patientLostHistoryLabelStatistic(new QueryWrapper<PatientDataOneDayInfo>()
                .eq("b.stat_dimension_type", labelStatDimensionTypeEnum.getCode())
                .ge("a.op_date", DateUtil.offsetDay(now, -1).toDateStr())
                .lt("a.op_date", now.toDateStr()), 
                OpTypeEnum.Create.getCode());

        log.info("3.流失患者昨日数据>>>>>> yesterdayList={}", JSON.toJSONString(yesterdayList));

        Map<String, PatientDataOneDayInfo> yesterdayMap = yesterdayList.stream().collect(Collectors.toMap(PatientDataOneDayInfo::getDeptId, i -> i));
        // 处理
        allList.forEach(i -> {
            i.setDataDimension(patientDataDimensionTypeEnum.getCode());
            PatientDataOneDayInfo yesterday = yesterdayMap.get(i.getDeptId());
            if (null != yesterday) {
                i.setAddNum(yesterday.getNum());
            }
        });
        // 保存
        patientDataBizService.addBatch(allList);
    }

    // 诊后：服务确诊患者
    // 疾病管理服务次数
    // private void savePatientPushServeData() {
    //     DateTime now = DateUtil.date();
    //     // 所有数据
    //     List<PatientDataOneDayInfo> allList = patientDataQueryBizMapper.patientPushServeStatistic(new QueryWrapper<PatientDataOneDayInfo>());
    //     // 昨日数据
    //     List<PatientDataOneDayInfo> yesterdayList = patientDataQueryBizMapper.patientPushServeStatistic(new QueryWrapper<PatientDataOneDayInfo>()
    //             .lt("a.op_date", now.toDateStr())
    //     );

    //     Map<String, PatientDataOneDayInfo> yesterdayMap = yesterdayList.stream().collect(Collectors.toMap(PatientDataOneDayInfo::getDeptId, i -> i));
    //     // 处理
    //     allList.forEach(i -> {
    //         i.setDataDimension(PatientDataDimensionTypeEnum.LSD_After_Service.getCode());
    //         PatientDataOneDayInfo yesterday = yesterdayMap.get(i.getDeptId());
    //         if (null != yesterday) {
    //             i.setAddNum(yesterday.getNum());
    //         }
    //     });
    //     // 保存
    //     patientDataBizService.addBatch(allList);
    // }
    private void savePatientPushServeData() {
        // 查询诊后服务列表
        List<String> pushServeTypeList = new ArrayList<>();
        pushServeTypeList.add(PushServeTypeEnum.Push_article.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visited.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visit_push.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Medicine.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Get_lost.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Satisfaction.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Dr_service.getCode());

        PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
        pushServeConfigClientQueryDTO.setTypes(pushServeTypeList);
        List<DeptPushServeVO> deptPushServeVOs = this.getList(new DeptQueryDTO(), pushServeConfigClientQueryDTO);
        log.info("1.PatientDataStatisticServiceImpl.savePatientPushServeData:deptPushServeVOs >>>>>> {}", JSON.toJSONString(deptPushServeVOs));
        if (CollectionUtil.isEmpty(deptPushServeVOs)) {
            return;
        }
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());

        DateTime now = DateUtil.date();

        // 所有诊后服务数据
        Map<String, Integer> allData = this.patientPushServeData(pushServeIds, now, false);
        // 所有昨日诊后服务数据
        Map<String, Integer> yesterdayData = this.patientPushServeData(pushServeIds, now, true);

        List<PatientDataOneDayInfo> list = new ArrayList<>();
        // 循环服务进行服务统计
        deptPushServeVOs.forEach(i -> {
            PatientDataOneDayInfo info = new PatientDataOneDayInfo();
            info.setBizDay(now.toDateStr());
            info.setDeptId(i.getDeptId());
            info.setDataDimension(PatientDataDimensionTypeEnum.LSD_After_Service.getCode());

            Integer num = allData.get(i.getPushServeId());
            info.setNum(null == num ? 0 : num);
            
            Integer yesterdayNum = yesterdayData.get(i.getPushServeId());
            info.setNum(null == yesterdayNum ? 0 : yesterdayNum);
            

            log.info("PatientDataStatisticServiceImpl.patientPushServeData:info >>>>>> {}", JSON.toJSONString(info));
            list.add(info);
        });
        // 保存统计数据
        patientDataBizService.addBatch(list);
    }
    // 诊后服务数据
    private Map<String, Integer> patientPushServeData(List<String> pushServeIds, DateTime now, boolean yesterday) {
        log.info("1.查询诊后患者服务>>>>>> {} : {} : {}", JSON.toJSONString(pushServeIds), now, yesterday);
        Map<String, Integer> pushServeDataMap = new HashMap<>();
        // 1.有提醒从提醒中查询发送成功的数据
        List<Map<String, Object>> pushServeRemindData = patientPushServeDataQueryBizMapper.deptPushServeRemindCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.Success.getCode())
                .in("b.push_serve_id", pushServeIds)
                .lt(yesterday, "a.push_date", now.toDateStr())
        );
        log.info("2.查询成功提醒数据<<<<<< {}", JSON.toJSONString(pushServeRemindData));
        if (CollectionUtil.isNotEmpty(pushServeRemindData)) {
            pushServeRemindData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());

                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 2.从日程中查询当前时间之前的数据
        List<Map<String, Object>> pushServeScheduleData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .in("a.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .le("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .lt(yesterday, "a.schedule_date", now.toDateStr())
        );
        log.info("3.查询已推送当前时间之前的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleData)) {
            pushServeScheduleData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 3.从行为中查询用户查看的未来时间的日程
        // List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleVisitCountMap(new QueryWrapper<DeptPushServeVO>()
        //         .eq("a.operated_type", "browse")
        //         .gt("b.schedule_date", now.toString())
        //         .in("a.source_three_id", pushServeIds)
        // );
        List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.status", PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .gt("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                .lt(yesterday, "a.schedule_date", now.toDateStr())
        );
        log.info("4.查询当前时间之后查看过的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleVisitData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleVisitData)) {
            pushServeScheduleVisitData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }

        return pushServeDataMap;
    }

    @Override
    public void patientDiseaseDataStatisticTaskExecute() {
        // 查询部门信息及部门下病种id
        List<DeptBO> deptList = deptClientService.getDeptList(new DeptQueryDTO());
        log.info("PatientDataStatisticServiceImpl.patientDiseaseDataStatisticTaskExecute:deptList >>>>>> {}", JSON.toJSONString(deptList));
        // 循环部门与部门下病种，统计部门与病种对应的服务和历史标签数据
        deptList.forEach(dept -> {
            List<String> diseaseIds = dept.getDiseaseIds();
            if (CollectionUtil.isNotEmpty(diseaseIds)) {
                diseaseIds.forEach(diseaseId -> {
                    // 诊前
                    // 统计：发现潜在患者/未就诊
                    this.savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_Pre_Not_Visit, PatientDataDimensionTypeEnum.LSD_Pre_Not_Visit);
                    // 统计：发现潜在患者/已就诊
                    this.savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_Pre_Visited, PatientDataDimensionTypeEnum.LSD_Pre_Visited);
                    // 统计：发现潜在患者/已确诊
                    this.savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_Pre_Confirmed, PatientDataDimensionTypeEnum.LSD_Pre_Confirmed);

                    // TODO 诊中，要从数仓中获取流失患者的病种，暂时使用看板中的数据
                    // 统计：挂号流失患者
                    // savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_During_Register, PatientDataDimensionTypeEnum.LSD_During_Register);
                    // 统计：住院流失患者
                    // savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_During_InHospital, PatientDataDimensionTypeEnum.LSD_During_InHospital);
                    // 统计：手术流失患者
                    // savePatientDiseaseData(dept.getId(), diseaseId, LabelStatDimensionTypeEnum.LSD_During_Operation, PatientDataDimensionTypeEnum.LSD_During_Operation);

                    // TODO 诊后
                    // 统计：疾病管理服务次数
                    this.savePatientDiseasePushServeData(dept.getId(), diseaseId);
                });
            }
        });
    }

    // 按部门病种进行数据统计
    // private void savePatientDiseaseData(String deptId, String diseaseId, LabelStatDimensionTypeEnum labelStatDimensionTypeEnum, PatientDataDimensionTypeEnum patientDataDimensionTypeEnum) {
    //     DateTime now = DateUtil.date();
    //     // 所有信息
    //     List<PatientDiseaseDataOneDayInfo> allList = patientDiseaseDataQueryBizMapper.patientDiseaseStatistic(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
    //             .eq("b.dept_id", deptId)
    //             .eq("a.op_type", OpTypeEnum.Create.getCode())
    //             .eq("a.is_validity", YN.YES.getCode())
    //             .like("d.disease_ids", "," + diseaseId + ",")
    //             .eq("d.stat_dimension_type", labelStatDimensionTypeEnum.getCode())
    //             .eq("f.id", diseaseId)
    //     );
    //     // 昨日信息
    //     List<PatientDiseaseDataOneDayInfo> yesterdayList = patientDiseaseDataQueryBizMapper.patientDiseaseStatistic(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
    //             .eq("b.dept_id", deptId)
    //             .eq("a.op_type", OpTypeEnum.Create.getCode())
    //             .eq("a.is_validity", YN.YES.getCode())
    //             .like("d.disease_ids", "," + diseaseId + ",")
    //             .lt("a.op_date", now.toDateStr())
    //             .eq("f.id", diseaseId)
    //     );
    //     Map<String, PatientDiseaseDataOneDayInfo> yesterdayMap = yesterdayList.stream().collect(Collectors.toMap(i -> i.getDeptId() + "-" + i.getDiseaseId(), i -> i));
        
    //     // 整理数据，按deptId和diseaseId统计数量，按deptId和标签维度统计数量
    //     allList.forEach(i -> {
    //         i.setDataDimension(patientDataDimensionTypeEnum.getCode());
    //         PatientDiseaseDataOneDayInfo yesterday = yesterdayMap.get(i.getDeptId());
    //         if (null != yesterday) {
    //             i.setAddNum(yesterday.getNum());
    //         }
    //     });
        
    //     patientDiseaseDataBizService.addBatch(allList);
    // }
    // 诊前：
    // 按部门病种进行数据统计
    private void savePatientDiseaseData(String deptId, String diseaseId, LabelStatDimensionTypeEnum labelStatDimensionTypeEnum, PatientDataDimensionTypeEnum patientDataDimensionTypeEnum) {
        DateTime now = DateUtil.date();
        // 查询标签数据
        LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
        labelQueryPageDTO.setStatDimensionType(labelStatDimensionTypeEnum.getCode());
        List<LabelBO> labelBOs = labelClientService.queryList(labelQueryPageDTO);
        if (CollectionUtil.isEmpty(labelBOs)) {
            return;
        }
        // 过滤出需要的标签数据
        List<String> labelIdList = new ArrayList<>();
        labelBOs.forEach(i -> {
            if (CollectionUtil.isNotEmpty(i.getDiseaseIds()) && i.getDiseaseIds().contains(diseaseId)) {
                labelIdList.add(i.getId());
            }
        });
        if (CollectionUtil.isEmpty(labelIdList)) {
            return;
        }
        log.info("1.需要的标签数据>>>>>>>> List<LabelBO>={}", JSON.toJSONString(labelIdList));
        // 根据标签统计所有人数
        PatientDiseaseDataOneDayInfo allData = patientDiseaseDataQueryBizMapper.patientLabelStatistic(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .in("a.label_id", labelIdList)
        );
        log.info("2.根据标签统计所有人数<<<<<< List<PatientDiseaseDataOneDayInfo>={}", JSON.toJSONString(allData));
        if (null == allData) {
            return;
        }
        // 根据标签统计昨日人数
        PatientDiseaseDataOneDayInfo yesterdayData = patientDiseaseDataQueryBizMapper.patientLabelStatistic(new QueryWrapper<PatientDiseaseDataOneDayInfo>()
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .in("a.label_id", labelIdList)
                .lt("a.op_date", now.toDateStr())
        );
        log.info("3.根据标签统计所有人数<<<<<< List<PatientDiseaseDataOneDayInfo>={}", JSON.toJSONString(yesterdayData));
        
        allData.setAddNum(0);
        if (null != yesterdayData) {
            allData.setAddNum(yesterdayData.getNum());
        }
        
        allData.setDeptId(deptId);
        allData.setDiseaseId(diseaseId);
        allData.setDataDimension(patientDataDimensionTypeEnum.getCode());

        patientDiseaseDataBizService.add(allData);
    }
    // TODO 诊中：
    
    // 诊后：
    private void savePatientDiseasePushServeData(String deptId, String diseaseId) {
        // 查询诊后服务列表
        List<String> pushServeTypeList = new ArrayList<>();
        pushServeTypeList.add(PushServeTypeEnum.Push_article.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visited.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Visit_push.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Medicine.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Get_lost.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Satisfaction.getCode());
        pushServeTypeList.add(PushServeTypeEnum.Dr_service.getCode());

        PushServeConfigClientQueryDTO pushServeConfigClientQueryDTO = new PushServeConfigClientQueryDTO();
        pushServeConfigClientQueryDTO.setTypes(pushServeTypeList);
        pushServeConfigClientQueryDTO.setDiseaseId(diseaseId);

        DeptQueryDTO deptQueryDTO = new DeptQueryDTO();
        deptQueryDTO.setId(deptId);

        List<DeptPushServeVO> deptPushServeVOs = this.getList(deptQueryDTO, pushServeConfigClientQueryDTO);
        log.info("1.PatientDataStatisticServiceImpl.savePatientDiseasePushServeData:deptPushServeVOs >>>>>> {}", JSON.toJSONString(deptPushServeVOs));
        if (CollectionUtil.isEmpty(deptPushServeVOs)) {
            return;
        }
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());

        DateTime now = DateUtil.date();

        // 所有诊后服务数据
        Map<String, Integer> allData = this.patientPushServeData(pushServeIds, now, false);
        // 所有昨日诊后服务数据
        Map<String, Integer> yesterdayData = this.patientPushServeData(pushServeIds, now, true);

        PatientDiseaseDataOneDayInfo data = new PatientDiseaseDataOneDayInfo();
        data.setDeptId(deptId);
        data.setDiseaseId(diseaseId);
        data.setDataDimension(PatientDataDimensionTypeEnum.LSD_After_Service.getCode());
        data.setBizDay(now.toDateStr());
        data.setNum(0);
        data.setAddNum(0);
        // 循环服务进行服务统计
        deptPushServeVOs.forEach(i -> {
            Integer num = allData.get(i.getPushServeId());
            if (null != num) {
                data.setNum(data.getNum() + num);
            }
            Integer yesterdayNum = yesterdayData.get(i.getPushServeId());
            if (null != yesterdayNum) {
                data.setAddNum(data.getAddNum() + yesterdayNum);
            }
        });
        log.info("2.PatientDataStatisticServiceImpl.savePatientDiseasePushServeData:data >>>>>> {}", JSON.toJSONString(data));
        // 保存统计数据
        patientDiseaseDataBizService.add(data);
    }

    // 处理部门与推送服务关联关系
    private List<DeptPushServeVO> getList(DeptQueryDTO deptQueryParams, PushServeConfigClientQueryDTO pushServeConfigParams) {
        List<DeptBO> deptList = deptClientService.getDeptList(deptQueryParams);
        log.info("1>>>>>>>> List<DeptBO>={}", deptList.size());
        List<PushServeConfigBO> pushServeList = pushServeConfigClientService.list(pushServeConfigParams);
        log.info("2>>>>>>>> List<PushServeConfigBO>={}", pushServeList.size());
        
        if (CollectionUtil.isEmpty(deptList) || CollectionUtil.isEmpty(pushServeList)) {
            return new ArrayList<>(0);
        }

        List<DeptPushServeVO> list = new ArrayList<>();

        deptList.forEach(dept -> {
            List<String> deptDiseaseIds = dept.getDiseaseIds();
            if (CollectionUtil.isNotEmpty(deptDiseaseIds)) {
                pushServeList.forEach(i -> {
                    List<String> pushServeDiseaseIds = i.getDiseaseIds();
                    if (CollectionUtil.isNotEmpty(pushServeDiseaseIds)) {
                        for (String id : pushServeDiseaseIds) {
                            if (deptDiseaseIds.contains(id)) {
                                DeptPushServeVO vo = new DeptPushServeVO();
                                vo.setDeptId(dept.getId());
                                vo.setDiseaseIds(pushServeDiseaseIds);
                                vo.setMediaId(i.getMediaId());
                                vo.setMediaName(i.getMediaName());
                                vo.setMediaType(i.getMediaType());
                                vo.setName(i.getName());
                                vo.setPushServeId(i.getId());
                                vo.setStatus(i.getStatus());
                                vo.setTitle(i.getTitle());
                                vo.setType(i.getType());
                                list.add(vo);
                                break;
                            }
                        }
                    }
                    
                });
            }
        });
        return list;
    }

    // 按部门、病种、推送服务进行数据统计
    @Override
    public void patientPushServeDataStatisticTask() {
        // 查询部门服务
        List<DeptPushServeVO> deptPushServeVOs = this.getList(new DeptQueryDTO(), new PushServeConfigClientQueryDTO());
        if (CollectionUtil.isEmpty(deptPushServeVOs)) {
            return;
        }
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());
        log.info("1.部门服务id >>>>>> {}", JSON.toJSONString(pushServeIds));

        List<PatientPushServeDataOneDayInfo> dataList = new ArrayList<>();

        DateTime now = DateTime.now();
        // 查询当前部门最新统计时间
        String lastDate = patientPushServeDataQueryBizMapper.lastPatientPushServiceDataOneDayDate();

        // 
        DateTime bizDay = DateUtil.date();
        if (null != lastDate) {
            bizDay = DateUtil.parseDate(lastDate);
        }
        bizDay = DateUtil.offsetDay(bizDay, -2);

        this.patientPushServeDataStatisticTaskExecute(deptPushServeVOs, dataList, pushServeIds, now, bizDay);
        
        // 保存统计数据
        pushServeDataBizService.addBatch(dataList);
    }

    // 循环执行
    private void patientPushServeDataStatisticTaskExecute(List<DeptPushServeVO> deptPushServeVOs, List<PatientPushServeDataOneDayInfo> dataList, List<String> pushServeIds, DateTime now, DateTime bizDay) {
        if (bizDay.toDateStr().compareTo(now.toDateStr()) > 0) {
            return;
        }
        String endDate = DateUtil.offsetDay(bizDay, 1).toDateStr();
        // 查询服务人次
        Map<String, Integer> pushServeDataMap = new HashMap<>();
        // 1.有提醒从提醒中查询发送成功的数据
        log.info("2.有提醒从提醒中查询发送成功的数据 >>>>>> {} {}", endDate, JSON.toJSONString(pushServeIds));
        List<Map<String, Object>> pushServeRemindData = patientPushServeDataQueryBizMapper.deptPushServeRemindCountMap(new QueryWrapper<DeptPushServeVO>()
                // .eq("a.rows_status", YN.YES.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.Success.getCode())
                .in("b.push_serve_id", pushServeIds)
                .lt("a.push_date", endDate)
        );
        if (CollectionUtil.isNotEmpty(pushServeRemindData)) {
            pushServeRemindData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());

                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 2.从日程中查询当前时间之前的数据
        log.info("3.查询已推送当前时间之前的日程数据 >>>>>> ");
        List<Map<String, Object>> pushServeScheduleData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                // .eq("a.rows_status", YN.YES.getCode())
                .in("a.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .lt("a.schedule_date", endDate)
                .in("b.push_serve_id", pushServeIds)
                // .isNotNull("a.user_id")  // zs202311121435 过滤掉没有user_id的数据（没有user_id其实是无法推送服务的）
                // .ne("a.user_id", "")
        );
        if (CollectionUtil.isNotEmpty(pushServeScheduleData)) {
            pushServeScheduleData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 3.从行为中查询用户查看的未来时间的日程
        // List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleVisitCountMap(new QueryWrapper<DeptPushServeVO>()
        //         .eq("a.operated_type", "browse")
        //         .ge("b.schedule_date", endDate)
        //         .in("a.source_three_id", pushServeIds)
        // );
        // 3.从日程中查询未来时间的已读日程的数据
        log.info("4.查询已推送当前时间之前的日程数据 >>>>>> ");
        List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                // .eq("a.rows_status", YN.YES.getCode())
                .eq("a.status", PatientPushServeConstant.ScheduleStatus.DONE.getCode()) 
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .ge("a.schedule_date", endDate)
                .in("b.push_serve_id", pushServeIds)
        );
        if (CollectionUtil.isNotEmpty(pushServeScheduleVisitData)) {
            pushServeScheduleVisitData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }



        
        // 未关注人数发送失败次数
        log.info("5.未关注人数发送失败次数 >>>>>> ");
        List<Map<String, Object>> notFollowingPushFailNumList = patientPushServeDataQueryBizMapper.notFollowingPushFailNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("b.op_type",  OpTypeEnum.Create.getCode())
                // .eq("a.error_code", WechatErrorCodeEnum.F43004.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.NotFollow.getCode())
                .in("b.push_serve_id", pushServeIds)
                .lt("a.push_date", endDate)
        );
        Map<String, Integer> notFollowingPushFailNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(notFollowingPushFailNumList)) {
            notFollowingPushFailNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                notFollowingPushFailNumMap.put(pushServeId, count);
            });
        }




        // 发送失败次数
        log.info("6.发送失败次数>>>>>> ");
        List<Map<String, Object>> pushFailNumList = patientPushServeDataQueryBizMapper.pushFailNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("b.op_type",  OpTypeEnum.Create.getCode())
                .in("a.push_status", PatientPushServeConstant.RemindPushStatus.Fail.getCode(), PatientPushServeConstant.RemindPushStatus.Invalid.getCode(), PatientPushServeConstant.RemindPushStatus.NotFollow.getCode())
                .in("b.push_serve_id", pushServeIds)
                .lt("a.push_date", endDate)
        );
        Map<String, Integer> pushFailNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(pushFailNumList)) {
            pushFailNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                pushFailNumMap.put(pushServeId, count);
            });
        }




        // 点击行动埋点次数
        log.info("7.点击行动埋点次数>>>>>> ");
        List<Map<String, Object>> behaviorNumList = patientPushServeDataQueryBizMapper.behaviorNumMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Further_Reading.getCode(), PatientDataOperatedTypeEnum.Knew.getCode(), PatientDataOperatedTypeEnum.Register.getCode(), PatientDataOperatedTypeEnum.Consultation.getCode(), PatientDataOperatedTypeEnum.Consultation_Online.getCode())
                .in("a.source_three_id", pushServeIds)
                .lt("a.operated_start_date", endDate)
        );
        Map<String, Integer> behaviorNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(behaviorNumList)) {
            behaviorNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                behaviorNumMap.put(pushServeId, count);
            });
        }




        // TODO 服务后就诊人数





        // 阅读次数
        log.info("8.阅读次数>>>>>> ");
        List<Map<String, Object>> browseNumList = patientPushServeDataQueryBizMapper.behaviorNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .in("a.source_three_id", pushServeIds)
                .lt("a.operated_start_date", endDate)
        );
        Map<String, Integer> browseNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseNumList)) {
            browseNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                browseNumMap.put(pushServeId, count);
            });
        }

        // 阅读人数
        log.info("8.阅读人数>>>>>> ");
        List<Map<String, Object>> browseUserNumList = patientPushServeDataQueryBizMapper.behaviorUserNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .in("a.source_three_id", pushServeIds)
                .lt("a.operated_start_date", endDate)
        );
        Map<String, Integer> browseUserNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseUserNumList)) {
            browseUserNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                browseUserNumMap.put(pushServeId, count);
            });
        }



        // 总阅读时间
        log.info("9.总阅读时间>>>>>> ");
        List<Map<String, Object>> browseDurationList = patientPushServeDataQueryBizMapper.browseDurationMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Browse_Duration.getCode())
                .in("a.source_three_id", pushServeIds)
                .lt("a.operated_start_date", endDate)
        );
        Map<String, Integer> browseDurationMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseDurationList)) {
            browseDurationList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer sum = Integer.valueOf(i.get("sum").toString());
                browseDurationMap.put(pushServeId, sum);
            });
        }



        // 循环服务进行服务统计
        deptPushServeVOs.forEach(i -> {
            PatientPushServeDataOneDayInfo info = new PatientPushServeDataOneDayInfo();
            info.setBizDay(bizDay.toDateStr());
            info.setDeptId(i.getDeptId());
            info.setDiseaseIds(StrJoinSplitUtils.joinPrefix(i.getDiseaseIds()));
            info.setPushServeId(i.getPushServeId());
            info.setTwoType(i.getType());

            if (PushServeTypeEnum.Risk.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Alert.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.No_visit.getCode().equals(i.getType())
                ) { // 诊前
                info.setOneType(PushServeTypeEnum.Pre.getCode());
            } else if (PushServeTypeEnum.Push.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Ntro.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Get_visit.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Article.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Opt_risk.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Service.getCode().equals(i.getType())
                ) { // 诊中
                info.setOneType(PushServeTypeEnum.During.getCode());
            } else if (PushServeTypeEnum.Push_article.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Visited.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Visit_push.getCode().equals(i.getType())
                    || PushServeTypeEnum.Medicine.getCode().equals(i.getType())
                    || PushServeTypeEnum.Get_lost.getCode().equals(i.getType())
                    || PushServeTypeEnum.Satisfaction.getCode().equals(i.getType())
                    || PushServeTypeEnum.Dr_service.getCode().equals(i.getType())
                ) { // 诊后
                info.setOneType(PushServeTypeEnum.After.getCode());
            }

            // 服务人次
            Integer serveNum = pushServeDataMap.get(i.getPushServeId());
            info.setServeNum(serveNum);

            // 未关注人数发送失败
            Integer notFollowingPushFailNum = notFollowingPushFailNumMap.get(i.getPushServeId());
            info.setNotFollowingPushFailNum(notFollowingPushFailNum);

            // 发送失败次数
            Integer pushFailNum = pushFailNumMap.get(i.getPushServeId());
            info.setPushFailNum(pushFailNum);

            // 点击行动埋点次数
            Integer behaviorNum = behaviorNumMap.get(i.getPushServeId());
            info.setClickNum(behaviorNum);

            // TODO 服务后就诊人数

            // 阅读次数
            Integer browseNum = browseNumMap.get(i.getPushServeId());
            info.setReadNum(browseNum);

            // 阅读人数
            Integer browseUserNum = browseUserNumMap.get(i.getPushServeId());
            info.setReadUserNum(browseUserNum);

            // 总阅读时间
            Integer browseDuration = browseDurationMap.get(i.getPushServeId());
            info.setReadTotalTime(browseDuration);

            dataList.add(info);
        });
        patientPushServeDataStatisticTaskExecute(deptPushServeVOs, dataList, pushServeIds, now, DateUtil.parseDate(endDate));
    }

    /**
     * 1. 查询部门服务
     * 2. 根据服务ids统计服务患者数据
     */
    @Override
    public void patientPushServeDataStatisticTaskExecute() {
        // 查询部门服务
        // List<DeptPushServeVO> deptPushServeVOs = patientPushServeDataQueryBizMapper.deptPushServeList(new QueryWrapper<DeptPushServeVO>());
        List<DeptPushServeVO> deptPushServeVOs = this.getList(new DeptQueryDTO(), new PushServeConfigClientQueryDTO());
        log.info("1.PatientDataStatisticServiceImpl.patientServiceDataStatisticTaskExecute:deptPushServeVOs >>>>>> {}", JSON.toJSONString(deptPushServeVOs));
        if (CollectionUtil.isEmpty(deptPushServeVOs)) {
            return;
        }
        List<String> pushServeIds = deptPushServeVOs.stream().map(DeptPushServeVO::getPushServeId).collect(Collectors.toList());


        DateTime now = DateTime.now();
        List<PatientPushServeDataOneDayInfo> list = new ArrayList<>();



        // 查询服务人次
        Map<String, Integer> pushServeDataMap = new HashMap<>();
        // 1.有提醒从提醒中查询发送成功的数据
        List<Map<String, Object>> pushServeRemindData = patientPushServeDataQueryBizMapper.deptPushServeRemindCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .eq("a.push_status", PatientPushServeConstant.RemindPushStatus.Success.getCode())
                .in("b.push_serve_id", pushServeIds)
        );
        log.info("2.查询成功提醒数据<<<<<< {}", JSON.toJSONString(pushServeRemindData));
        if (CollectionUtil.isNotEmpty(pushServeRemindData)) {
            pushServeRemindData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());

                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 2.从日程中查询当前时间之前的数据
        List<Map<String, Object>> pushServeScheduleData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .in("a.status", PatientPushServeConstant.ScheduleStatus.TODO.getCode(), PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .le("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
                // .isNotNull("a.user_id")  // zs202311121435 过滤掉没有user_id的数据（没有user_id其实是无法推送服务的）
                // .ne("a.user_id", "")
        );
        log.info("3.查询已推送当前时间之前的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleData)) {
            pushServeScheduleData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }
        // 3.从行为中查询用户查看的未来时间的日程
        // List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleVisitCountMap(new QueryWrapper<DeptPushServeVO>()
        //         .eq("a.operated_type", "browse")
        //         .gt("b.schedule_date", now.toString())
        //         .in("a.source_three_id", pushServeIds)
        // );
        List<Map<String, Object>> pushServeScheduleVisitData = patientPushServeDataQueryBizMapper.deptPushServeScheduleCountMap(new QueryWrapper<DeptPushServeVO>()
                .eq("a.rows_status", YN.YES.getCode())
                .eq("a.status", PatientPushServeConstant.ScheduleStatus.DONE.getCode())
                .eq("a.biz_one_type", ObjectTypeEnum.OT_OneType_PatientPushServe.getCode())
                .in("a.schedule_type", RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode())
                .gt("a.schedule_date", now.toString())
                .in("b.push_serve_id", pushServeIds)
        );
        log.info("4.查询当前时间之后查看过的日程数据<<<<<< {}", JSON.toJSONString(pushServeScheduleVisitData));
        if (CollectionUtil.isNotEmpty(pushServeScheduleVisitData)) {
            pushServeScheduleVisitData.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.parseInt(i.get("count").toString());
                
                Integer dataCount = pushServeDataMap.get(pushServeId);
                if (null == dataCount) {
                    pushServeDataMap.put(pushServeId, count);
                } else {
                    pushServeDataMap.put(pushServeId, count + dataCount);
                }
            });
        }



        
        // 未关注人数发送失败次数
        List<Map<String, Object>> notFollowingPushFailNumList = patientPushServeDataQueryBizMapper.notFollowingPushFailNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("b.op_type",  OpTypeEnum.Create.getCode())
                .eq("a.error_code", WechatErrorCodeEnum.F43004.getCode())
                .in("b.push_serve_id", pushServeIds)
        );
        log.info("4.查询未关注的推送失败次数>>>>>> {}", JSON.toJSONString(notFollowingPushFailNumList));
        Map<String, Integer> notFollowingPushFailNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(notFollowingPushFailNumList)) {
            notFollowingPushFailNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                notFollowingPushFailNumMap.put(pushServeId, count);
            });
        }




        // 发送失败次数
        List<Map<String, Object>> pushFailNumList = patientPushServeDataQueryBizMapper.pushFailNumMap(new QueryWrapper<DeptPushServeVO>()
                .eq("b.op_type",  OpTypeEnum.Create.getCode())
                .in("a.push_status", PatientPushServeConstant.RemindPushStatus.Fail.getCode(), PatientPushServeConstant.RemindPushStatus.Invalid.getCode(), PatientPushServeConstant.RemindPushStatus.NotFollow.getCode())
                .in("b.push_serve_id", pushServeIds)
        );
        log.info("5.查询发送失败次数>>>>>> {}", JSON.toJSONString(pushFailNumList));
        Map<String, Integer> pushFailNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(pushFailNumList)) {
            pushFailNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                pushFailNumMap.put(pushServeId, count);
            });
        }




        // 点击行动埋点次数
        List<Map<String, Object>> behaviorNumList = patientPushServeDataQueryBizMapper.behaviorNumMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Further_Reading.getCode(), PatientDataOperatedTypeEnum.Knew.getCode(), PatientDataOperatedTypeEnum.Register.getCode(), PatientDataOperatedTypeEnum.Consultation.getCode(), PatientDataOperatedTypeEnum.Consultation_Online.getCode())
                .in("a.source_three_id", pushServeIds)
        );
        log.info("6.服务点击行动埋点次数 >>>>>> {}", JSON.toJSONString(behaviorNumList));
        Map<String, Integer> behaviorNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(behaviorNumList)) {
            behaviorNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                behaviorNumMap.put(pushServeId, count);
            });
        }




        // TODO 服务后就诊人数





        // 阅读次数
        List<Map<String, Object>> browseNumList = patientPushServeDataQueryBizMapper.behaviorNumMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .in("a.source_three_id", pushServeIds)
        );
        log.info("7.服务阅读次数>>>>>> {}", JSON.toJSONString(browseNumList));
        Map<String, Integer> browseNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseNumList)) {
            browseNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                browseNumMap.put(pushServeId, count);
            });
        }

        // 阅读人数
        List<Map<String, Object>> browseUserNumList = patientPushServeDataQueryBizMapper.behaviorUserNumMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .in("a.source_three_id", pushServeIds)
        );
        log.info("7.服务阅读次数>>>>>> {}", JSON.toJSONString(browseUserNumList));
        Map<String, Integer> browseUserNumMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseUserNumList)) {
            browseUserNumList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer count = Integer.valueOf(i.get("count").toString());
                browseUserNumMap.put(pushServeId, count);
            });
        }



        // 总阅读时间
        List<Map<String, Object>> browseDurationList = patientPushServeDataQueryBizMapper.browseDurationMap(new QueryWrapper<DeptPushServeVO>()
                .in("a.operated_type", PatientDataOperatedTypeEnum.Browse_Duration.getCode())
                .in("a.source_three_id", pushServeIds)
        );
        log.info("8.服务总阅读时间>>>>>> {}", JSON.toJSONString(browseDurationList));
        Map<String, Integer> browseDurationMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(browseDurationList)) {
            browseDurationList.forEach(i -> {
                String pushServeId = i.get("pushServeId").toString();
                Integer sum = Integer.valueOf(i.get("sum").toString());
                browseDurationMap.put(pushServeId, sum);
            });
        }



        // 循环服务进行服务统计
        deptPushServeVOs.forEach(i -> {
            PatientPushServeDataOneDayInfo info = new PatientPushServeDataOneDayInfo();
            info.setBizDay(now.toDateStr());
            info.setDeptId(i.getDeptId());
            info.setDiseaseIds(StrJoinSplitUtils.joinPrefix(i.getDiseaseIds()));
            info.setPushServeId(i.getPushServeId());
            info.setTwoType(i.getType());

            if (PushServeTypeEnum.Risk.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Alert.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.No_visit.getCode().equals(i.getType())
                ) { // 诊前
                info.setOneType(PushServeTypeEnum.Pre.getCode());
            } else if (PushServeTypeEnum.Push.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Ntro.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Get_visit.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Article.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Opt_risk.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Service.getCode().equals(i.getType())
                ) { // 诊中
                info.setOneType(PushServeTypeEnum.During.getCode());
            } else if (PushServeTypeEnum.Push_article.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Visited.getCode().equals(i.getType()) 
                    || PushServeTypeEnum.Visit_push.getCode().equals(i.getType())
                    || PushServeTypeEnum.Medicine.getCode().equals(i.getType())
                    || PushServeTypeEnum.Get_lost.getCode().equals(i.getType())
                    || PushServeTypeEnum.Satisfaction.getCode().equals(i.getType())
                    || PushServeTypeEnum.Dr_service.getCode().equals(i.getType())
                ) { // 诊后
                info.setOneType(PushServeTypeEnum.After.getCode());
            }

            // 服务人次
            Integer serveNum = pushServeDataMap.get(i.getPushServeId());
            info.setServeNum(serveNum);

            // 未关注人数发送失败
            Integer notFollowingPushFailNum = notFollowingPushFailNumMap.get(i.getPushServeId());
            info.setNotFollowingPushFailNum(notFollowingPushFailNum);

            // 发送失败次数
            Integer pushFailNum = pushFailNumMap.get(i.getPushServeId());
            info.setPushFailNum(pushFailNum);

            // 点击行动埋点次数
            Integer behaviorNum = behaviorNumMap.get(i.getPushServeId());
            info.setClickNum(behaviorNum);

            // TODO 服务后就诊人数

            // 阅读次数
            Integer browseNum = browseNumMap.get(i.getPushServeId());
            info.setReadNum(browseNum);

            // 阅读人数
            Integer browseUserNum = browseUserNumMap.get(i.getPushServeId());
            info.setReadUserNum(browseUserNum);

            // 总阅读时间
            Integer browseDuration = browseDurationMap.get(i.getPushServeId());
            info.setReadTotalTime(browseDuration);

            log.info("PatientDataStatisticServiceImpl.patientServiceDataStatisticTaskExecute:info >>>>>> {}", JSON.toJSONString(info));
            list.add(info);
        });
        // 保存统计数据
        pushServeDataBizService.addBatch(list);
    }



    @Override
    public void patientLabelingTaskExecute() {
        DateTime now = DateUtil.date();
        
        // 打标签：挂号流失患者
        log.info("1.挂号流失患者开始>>>>>>");
        List<DoctorDwdRegisterRecordVO> registerRecordVOs = doctorDwdPatientDataBizService.queryRegisterList(DateUtil.offsetDay(now, -registerLostDay), now);
        log.info("PatientDataStatisticServiceImpl.patientLabelingTaskExecute:registerRecordVOs: >>>>>> {} ", JSON.toJSONString(registerRecordVOs));
        patientLostLabelBizService.registerLost(registerRecordVOs, DateUtil.offsetDay(now, -registerLostDay));
        log.info("1.挂号流失患者结束<<<<<<");




//        // 查询住院申请
//        log.info("2.住院流失患者开始>>>>>>");
//        List<DoctorDwdInpatientReserveVO> inpatientReserveVOs = doctorDwdPatientDataBizService.queryInHospitalApplyList(DateUtil.offsetDay(now, -inHospitalLostDay), now);
//        // 处理住院申请的住院状态
//        this.queryInHospitalApplyList(inpatientReserveVOs, DateUtil.offsetDay(now, -inHospitalLostDay), now);
//        log.info("PatientDataStatisticServiceImpl.patientLabelingTaskExecute:inpatientReserveVOs: >>>>>> {} ", JSON.toJSONString(inpatientReserveVOs));
//        patientLostLabelBizService.inHospitalLost(inpatientReserveVOs, DateUtil.offsetDay(now, -inHospitalLostDay));
//        log.info("2.住院流失患者结束<<<<<<");



        // 打标签：手术流失患者
        log.info("3.手术流失患者开始>>>>>>");
        List<DoctorDwdOperationApplyVO> operationApplyVOs = doctorDwdPatientDataBizService.queryOperationApplyList(DateUtil.offsetDay(now, -operationLostDay), now);
        log.info("PatientDataStatisticServiceImpl.patientLabelingTaskExecute:operationApplyVOs: >>>>>> {} ", JSON.toJSONString(operationApplyVOs));
        patientLostLabelBizService.operationLost(operationApplyVOs, DateUtil.offsetDay(now, -operationLostDay));
        log.info("3.手术流失患者结束<<<<<<");
    }

    @Override
    public void patientInpatientLabelingTaskExecute() {
        // 打标签：住院流失患者
        patientInpatientLossService.processByEnd(DateUtil.date(), inHospitalLostDay);
    }

    @Override
    public void patientMainPersonIdTaskExecute() {
        // 查询没有主患者id的患者信息列表
        List<TaskPatientInfo> patientList = taskPatientInfoService.list(new QueryWrapper<TaskPatientInfo>()
            .and(i -> i.isNull("main_person_id").or().eq("main_person_id", ""))
            .last(" limit 1000 ")
        );
        if (CollectionUtil.isEmpty(patientList)) {
            return;
        }

        DataDwdPatientQueryDTO dataDwdPatientQueryDTO = new DataDwdPatientQueryDTO();
        dataDwdPatientQueryDTO.setPatientIds(patientList.stream().map(TaskPatientInfo::getPatientId).collect(Collectors.toList()));
        List<DataDwdPatientVO> dwdPatientList = doctorDwdPatientDataBizService.patientList(dataDwdPatientQueryDTO);
        if (CollectionUtil.isEmpty(dwdPatientList)) {
            return;
        }

        // 整理数据
        Map<String, String> map = new HashMap<>();
        dwdPatientList.forEach(i -> {
            if (StrUtil.isNotBlank(i.getMainPersonId())) {
                map.put(i.getPatientId(), i.getMainPersonId());
            }
        });

        List<TaskPatientInfo> list = new ArrayList<>();
        patientList.forEach(i -> {
            String mainPersonId = map.get(i.getPatientId());
            if (StrUtil.isNotBlank(mainPersonId)) {
                i.setMainPersonId(mainPersonId);
                list.add(i);
            }
        });

        // 更新主患者id
        if (CollectionUtil.isNotEmpty(list)) {
            taskPatientInfoService.updateBatchById(list);
        }
    }

    // 查询住院科室信息
//    private List<DoctorDwdInpatientReserveVO> queryInHospitalApplyList(List<DoctorDwdInpatientReserveVO> inpatientReserveVOs, Date startDate, Date endDate) {
//        if (CollectionUtil.isEmpty(inpatientReserveVOs)) {
//            return new ArrayList<>(0);
//        }
//        List<String> inpatientReserveDeptIds = new ArrayList<>();
//        inpatientReserveVOs.forEach(i -> {
//            inpatientReserveDeptIds.add(i.getApplyDepartmentCodeStandard());
//        });
//        // 根据科室查询对应的科室：key为住院申请科室id，value为住院科室id列表
//        log.info("1.根据科室查询对应的科室>>>>>> {}", JSON.toJSONString(inpatientReserveDeptIds));
//        Map<String, List<String>> deptIdMap = this.listByDeptIds(inpatientReserveDeptIds);
//        log.info("2.根据科室查询对应的科室<<<<<< {}", JSON.toJSONString(deptIdMap));
//        if (CollectionUtil.isEmpty(deptIdMap)) {
//            return new ArrayList<>(0);
//        }
//        List<String> deptIds = new ArrayList<>();
//        deptIdMap.forEach((k, v) -> deptIds.addAll(v));
//        // 住院预约信息
//        log.info("3.住院信息>>>>>> deptId:{}", JSON.toJSONString(deptIds));
//        List<DataDwdPatientInHospitalVO> voList = doctorDwdPatientDataBizService.queryInHospitalList(deptIds, startDate, endDate);
//        log.info("4.住院信息<<<<<< voList:{}", JSON.toJSONString(voList));
//        // 整理成：key为住院科室id，value为住院信息列表
//        Map<String, List<DataDwdPatientInHospitalVO>> voMap = new HashMap<>();
//        if (CollectionUtil.isNotEmpty(voList)) {
//            voList.forEach(i -> {
//                List<DataDwdPatientInHospitalVO> l = voMap.get(i.getAdmitDepartmentCodeStandard());
//                if (null == l) {
//                    l = new ArrayList<>();
//                    voMap.put(i.getAdmitDepartmentCodeStandard(), l);
//                }
//                l.add(i);
//            });
//        }
//        log.info("5.住院信息>>>>>> {}", JSON.toJSONString(voMap));
//
//        // 整理成：key为住院申请科室id，value为住院信息列表
//        Map<String, List<DataDwdPatientInHospitalVO>> recordVoMap = new HashMap<>();
//        deptIdMap.forEach((k, v) -> {
//            List<DataDwdPatientInHospitalVO> reserveVOList = new ArrayList<>();
//            if (CollectionUtil.isNotEmpty(v)) {
//                v.forEach(i -> {
//                    List<DataDwdPatientInHospitalVO> l = voMap.get(i);
//                    if (null != l) {
//                        reserveVOList.addAll(l);
//                    }
//                });
//            }
//            recordVoMap.put(k, reserveVOList);
//        });
//
//        log.info("6.住院信息>>>>>> {}", JSON.toJSONString(recordVoMap));
//
//        inpatientReserveVOs.forEach(i -> {
//            List<DataDwdPatientInHospitalVO> inHospitalVOs = recordVoMap.get(i.getApplyDepartmentCodeStandard());
//            if (CollectionUtil.isNotEmpty(inHospitalVOs)) {
//                inHospitalVOs.forEach(j -> {
//                    // 在时间范围内匹配，默认7天
//                    if (j.getPatientId().equals(i.getPatientId()) && j.getInpatientDate().after(i.getApplyDate()) && j.getInpatientDate().before(DateUtil.offsetDay(i.getApplyDate(), reserveDay))) {
//                        i.setInHospitalId(j.getId());
//                    }
//                });
//            }
//        });
//        return inpatientReserveVOs;
//    }

    // 整理住院科室id与住院申请科室id的关系
//    private Map<String, List<String>> listByDeptIds(List<String> deptIds) {
//        Map<String, List<String>> deptIdMap = new HashMap<>(); // key为住院申请科室id，value为住院科室的id
//
//        // 通过住院申请科室id查询pid
//        List<Map<String, String>> parentDepts = dwdPatientDataBizMapper.getParentDeptIds(new QueryWrapper<DoctorDwdInpatientReserveVO>()
//                .in("id", deptIds)
//        );
//        if (CollectionUtil.isEmpty(parentDepts)) {
//            return deptIdMap;
//        }
//        log.info("1.住院申请科室信息>>>>>> {}", JSON.toJSONString(parentDepts));
//
//        Map<String, String> idPidMap = new HashMap<>(); // key为住院申请科室id，value为key对应的pid
//
//        List<String> pids = new ArrayList<>();
//        parentDepts.forEach(i -> {
//            String id = String.valueOf(i.get("id"));
//            String pid = String.valueOf(i.get("pid"));
//            if ("0".equals(pid)) {
//                pids.add(id);
//                idPidMap.put(id, id);
//            } else {
//                pids.add(pid);
//                idPidMap.put(id, pid);
//            }
//        });
//        log.info("2.住院申请科室与其上级科室 >>>>>> {}", JSON.toJSONString(idPidMap));
//
//        Map<String, List<String>> subDeptMap = new HashMap<>(); // key为住院申请科室对应的pid，value为住院科室的id
//
//        List<Map<String, String>> subDepts = dwdPatientDataBizMapper.getSubDeptIds(new QueryWrapper<DoctorDwdInpatientReserveVO>()
//                .in("pid", pids)
//        );
//        if (CollectionUtil.isNotEmpty(subDepts)) {
//            subDepts.forEach(i -> {
//                String id = String.valueOf(i.get("id"));
//                String pid = String.valueOf(i.get("pid"));
//
//                List<String> subDeptIds = subDeptMap.get(pid);
//                if (null == subDeptIds) {
//                    subDeptIds = new ArrayList<>();
//                    subDeptMap.put(pid, subDeptIds);
//                }
//                subDeptIds.add(id);
//            });
//        }
//        log.info("3.住院申请科室的上级科室与其下级科室 >>>>>> {}", JSON.toJSONString(subDeptMap));
//
//        idPidMap.forEach((k, v) -> {
//            List<String> l = subDeptMap.get(v);
//            if (null == l) {
//                l = new ArrayList<>();
//            }
//            l.add(v);
//            deptIdMap.put(k, l);
//        });
//        log.info("4.住院申请科室与其上级科室及同级科室 >>>>>> {}", JSON.toJSONString(deptIdMap));
//
//        return deptIdMap;
//    }


}
