package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.DoctorDwdInReserveBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.LabelRecordBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.bo.PatientInLossUnitedRO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.LabelBizQueryService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientInpatientLossService;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAddOneDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PatientInpatientLossServiceImpl implements PatientInpatientLossService {


    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private LabelBizQueryService labelBizQueryService;
    @Autowired
    private PatientLabelClientService patientLabelClientService;

    // 住院申请与住院的最大间隔时间
    @Value("${xx.loss.inHospital.losslabel:inhospitallosslabel}")
    private String inhospitalLossLabel;
    @Value("${xx.loss.inHospital.labelValidityNum:9999}")
    private Integer inhospitalLabelValidityNum;

    @Override
    public void processByStart(Date startDate, int day) {
        Date endDate= DateUtil.offsetDay(startDate,day);
        this.process(startDate,endDate);
    }

    @Override
    public void processByEnd(Date endDate, int day) {
        Date startDate= DateUtil.offsetDay(endDate,-day);
        this.process(startDate,endDate);
    }

    @Override
    public void process(Date startDate, Date endDate) {
        String startDay= DateUtil.formatDate(startDate);
        ConsoleUtils.log("+++++执行住院流失统计，参数 开始时间={} 结束时间={}",DateUtil.formatDateTime(startDate),DateUtil.formatDateTime(endDate));
        ConsoleUtils.log("+++++执行住院流失统计，统计日期={}",startDay);
        List<PatientInLossUnitedRO> list= this.getInLossPatientByDay(startDay,startDate,endDate);
        if(CollUtil.isEmpty(list)){
            return;
        }
        this.addInLossLabel(list);
    }





    /**
     * 流失标签
     * */
    private void addInLossLabel(List<PatientInLossUnitedRO> list){
        List<LabelAddOneDTO> labelParamList=new ArrayList<>();
        list.forEach(item->{
            LabelAddOneDTO dataItem=this.getLabelAddOneDTO(item);
            labelParamList.add(dataItem);
        });
        patientLabelClientService.addBatch2(labelParamList);
    }

    private  LabelAddOneDTO getLabelAddOneDTO(PatientInLossUnitedRO item){
        LabelAddOneDTO data=new LabelAddOneDTO();
        data.setPatientId(item.getPatientId());
        //todo:
        data.setLabelId(inhospitalLossLabel); // 固定住院流失标签id
        data.setValidityNum(inhospitalLabelValidityNum);
        if(StrUtil.isNotEmpty(item.getApplyId())){
            data.setSourceType(ObjectTypeEnum.OT_DWD_In_Apply.getCode());
            data.setSourceRelId(item.getApplyId());
            data.setSourceRelTwoId(item.getApplyDeptCode());
            data.setDataDt(item.getOpDate());
            data.setRemark("住院申请单流失");
        }else {
            data.setSourceType(ObjectTypeEnum.OT_Label_Info.getCode());
            data.setSourceRelId(item.getLabelId());
            data.setDataDt(item.getOpDate());
            data.setRemark("标签住院流失");
        }
        return data;
    }


    /**
     * 获取流失客户
     * */
    private List<PatientInLossUnitedRO> getInLossPatientByDay(String startDay,Date startDate, Date endDate){
        List<PatientInLossUnitedRO> data=new ArrayList<>();

        //todo:状态判断待处理
        List<DoctorDwdInReserveBO> inApply= doctorDwdPatientDataBizService.queryInApplyList(startDay);
        List<LabelRecordBO> labelApply= labelBizQueryService.queryLabel(startDay);

        Map<String, PatientInLossUnitedRO> inLossApplyMap=new HashMap<>();

        if(CollUtil.isNotEmpty(labelApply)){
            labelApply.forEach(item->{
                inLossApplyMap.put(item.getPatientId(),new PatientInLossUnitedRO(item.getPatientId(),item.getLabelId(),item.getLabelName(),item.getOpDate()));
            });
        }

        if(CollUtil.isNotEmpty(inApply)){
            inApply.forEach(item->{
                inLossApplyMap.put(item.getPatientId(),new PatientInLossUnitedRO(item.getPatientId(),item.getId(),item.getDeptCode(),null,item.getApplyDate()));
            });
        }

        if(MapUtil.isEmpty(inLossApplyMap)){
            ConsoleUtils.log("+++++未发现未住院的患者");
            return data;
        }

        DateTime start = DateUtil.parse(StrUtil.format("{} 00:00:00",startDay),"yyyy-MM-dd HH:mm:ss");
        DateTime end = DateUtil.parse(StrUtil.format("{} 23:59:59",DateUtil.formatDate(endDate)),"yyyy-MM-dd HH:mm:ss");

        //todo:状态判断待处理
        List<DoctorDwdInRecordBO> inRecordList= doctorDwdPatientDataBizService.queryInRecordList(start,end,inLossApplyMap.keySet());
        if(CollUtil.isNotEmpty(inRecordList)){
            inRecordList.forEach(item->{
                inLossApplyMap.remove(item.getPatientId());
            });
        }

        inLossApplyMap.forEach((key,item)->{
            data.add(item);
        });

        return data;
    }

}
