package com.xxh.cloud.healthy.data.framework.modules.task.service.impl;

import cn.hutool.core.text.StrFormatter;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientReserveClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientLossConstant;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientLossAddDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.DoctorDwdRegisterRecordVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientLossDataBizService;
import com.xxh.cloud.healthy.data.framework.modules.task.service.PatientLossDataStatisticService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class PatientLossDataStatisticServiceImpl implements PatientLossDataStatisticService {

    @Autowired
    private PatientLossDataBizService patientLossDataBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;


    // 挂号流失最短时间
    @Value("${xx.loss.register.day:1}")
    private Integer registerLossDay;
    // 就诊流失最短时间
    @Value("${xx.loss.visit.day:7}")
    private Integer visitLossDay;
    // 住院流失最短时间
    @Value("${xx.loss.inHospital.day:15}")
    private Integer inHospitalLostDay;

    @Override
    public void patientRegisterLossJob(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }

        while (!startDate.after(endDate)) {
            // 查询挂号流失信息
            List<DoctorDwdRegisterRecordVO> registerRecordVOs = doctorDwdPatientDataBizService.queryRegisterLossList(DateUtil.offsetDay(startDate, -registerLossDay), DateUtil.offsetDay(startDate, -registerLossDay + 1));
            // 保存流失数据
            if (CollUtil.isNotEmpty(registerRecordVOs)) {
                // 校验当天同部门已就诊
                Map<String, List<DoctorDwdRegisterRecordVO>> regVisMap = new HashMap<>();
                Set<String> patientIds = new HashSet<>();
                registerRecordVOs.forEach(vo -> patientIds.add(vo.getPatientId()));
                List<DoctorDwdRegisterRecordVO> regVisList = doctorDwdPatientDataBizService.queryRegisterVisitList(patientIds, DateUtil.offsetDay(startDate, -registerLossDay), DateUtil.offsetDay(startDate, -registerLossDay + 1));
                if (CollUtil.isNotEmpty(regVisList)) {
                    regVisList.forEach(vo -> {
                        String mapKey = StrUtil.format("{}_{}", vo.getPatientId(), vo.getDepartmentCodeStandard());
                        if (regVisMap.containsKey(mapKey)) {
                            List<DoctorDwdRegisterRecordVO> mapList = regVisMap.get(mapKey);
                            mapList.add(vo);
                            regVisMap.put(mapKey, mapList);
                        } else {
                            List<DoctorDwdRegisterRecordVO> mapList = new ArrayList<>();
                            mapList.add(vo);
                            regVisMap.put(mapKey, mapList);
                        }
                    });
                }

                // 保存流失数据
                List<PatientLossAddDTO> saveList = new ArrayList<>();
                for (DoctorDwdRegisterRecordVO i : registerRecordVOs) {
                    // 校验当前就诊
                    String mapKey = StrUtil.format("{}_{}", i.getPatientId(), i.getDepartmentCodeStandard());
                    if (!regVisMap.containsKey(mapKey)) { // 当日同部门无就诊
                        PatientLossAddDTO loss = new PatientLossAddDTO();
                        loss.setDeptId(i.getDepartmentCodeStandard());
                        loss.setSourceDeptId(i.getDepartmentCodeRaw());
                        loss.setPatientId(i.getPatientId());
                        loss.setType(PatientLossConstant.LossType.Register.getCode());
                        loss.setDataSourceId(i.getId());
                        loss.setDataSourceDate(i.getRegDate());
                        saveList.add(loss);
                    }
                }

                if (CollUtil.isNotEmpty(saveList)) {
                    patientLossDataBizService.saveBatch(saveList);
                }
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

    @Override
    public void patientVisitLossJob(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }

        while (!startDate.after(endDate)) {
            // 查询就诊流失信息
            List<PatientVisitRecordClientBO> visitRecordVOs = doctorDwdPatientDataBizService.queryVisitLossList(DateUtil.offsetDay(startDate, -visitLossDay), DateUtil.offsetDay(startDate, -visitLossDay + 1));

            // 保存流失数据
            if (CollUtil.isNotEmpty(visitRecordVOs)) {
                List<PatientLossAddDTO> saveList = new ArrayList<>();

                visitRecordVOs.forEach(i -> {
                    PatientLossAddDTO loss = new PatientLossAddDTO();
                    loss.setDeptId(i.getDepartmentCodeStandard());
                    loss.setSourceDeptId(i.getDepartmentCodeRaw());
                    loss.setPatientId(i.getPatientId());
                    loss.setType(PatientLossConstant.LossType.Visit.getCode());
                    loss.setDataSourceId(i.getId());
                    loss.setDataSourceDate(i.getVisitDate());

                    saveList.add(loss);
                });

                if (CollUtil.isNotEmpty(saveList)) {
                    patientLossDataBizService.saveBatch(saveList);
                }
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }

    }

    @Override
    public void patientInpatientLossJob(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }

        while (!startDate.after(endDate)) {
            DateTime dateTime = DateUtil.offsetDay(startDate, -inHospitalLostDay);
            DateTime dateTime1 = DateUtil.offsetDay(startDate, -inHospitalLostDay + 1);
            // 查询住院流失信息
            List<PatientInpatientReserveClientBO> inpatientRecordVOs = doctorDwdPatientDataBizService.queryInpatientLossList(dateTime, dateTime1, inHospitalLostDay);


            // 保存流失数据
            if (CollUtil.isNotEmpty(inpatientRecordVOs)) {
                List<PatientLossAddDTO> saveList = new ArrayList<>();

                inpatientRecordVOs.forEach(i -> {
                    PatientLossAddDTO loss = new PatientLossAddDTO();
                    loss.setDeptId(i.getAdmitDepartmentCodeStandard());
                    loss.setSourceDeptId(i.getAdmitDepartmentCodeRaw());
                    loss.setPatientId(i.getPatientId());
                    loss.setType(PatientLossConstant.LossType.Inpatient.getCode());
                    loss.setDataSourceId(i.getId());
                    loss.setDataSourceDate(i.getApplyDate());

                    saveList.add(loss);
                });

                if (CollUtil.isNotEmpty(saveList)) {
                    List<PatientLossAddDTO> patientLossAddDTOS = duplicateRemoval(saveList);
                    patientLossDataBizService.saveBatch(patientLossAddDTOS);
                }
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

    /**
     * 去重
     *
     * @param saveList
     */
    private List<PatientLossAddDTO> duplicateRemoval(List<PatientLossAddDTO> saveList) {
        List<PatientLossAddDTO> collect = saveList.stream().collect(Collectors.toMap(PatientLossAddDTO::getPatientId,
                        Function.identity(),
                        (existing, replacement) -> existing,
                        LinkedHashMap::new)).values()
                .stream()
                .collect(Collectors.toList());
        return collect;
    }

}
