package com.xxh.cloud.healthy.drug.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.drug.framework.modules.base.entity.DrugNotityConfigInfo;
import com.xxh.cloud.healthy.drug.framework.modules.base.service.DrugNotityConfigInfoService;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigDelDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigListDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigLogAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigQueryDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigSnapshotAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigUpdateDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigUpdateStatusDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigSnapshotVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.helper.DrugNotityHelper;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigBizService;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigLogBizService;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigSnapshotBizService;
import com.xxh.cloud.metadata.client.api.StandardMedicantClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardMedicantBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DrugNotityConfigBizServiceImpl implements DrugNotityConfigBizService {

    @Autowired
    private DrugNotityConfigInfoService drugNotityConfigInfoService;
    @Autowired
    private DrugNotityConfigSnapshotBizService drugNotityConfigSnapshotBizService;
    @Autowired
    private DrugNotityConfigLogBizService drugNotityConfigLogBizService;
    @Autowired
    private StandardMedicantClientService standardMedicantClientService;

    @Override
    public PageData<DrugNotityConfigVO> query(DrugNotityConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        IPage<DrugNotityConfigInfo> page = drugNotityConfigInfoService.page(new Page<>(params.getPage(), params.getRows()), 
                new QueryWrapper<DrugNotityConfigInfo>()
                    .eq("rows_status", YN.YES.getCode())
                    .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                    .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                    .eq(StrUtil.isNotBlank(params.getMedicantId()), "medicant_id", params.getMedicantId())
                    .eq(StrUtil.isNotBlank(params.getMedicantCode()), "medicant_code", params.getMedicantCode())
                    .in(CollUtil.isNotEmpty(params.getMedicantCodes()), "medicant_code", params.getMedicantCodes())
                    .eq(StrUtil.isNotBlank(params.getProjectCode()), "project_code", params.getProjectCode())
                    .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> medicantIds = new ArrayList<>();
        page.getRecords().forEach(item -> {
            medicantIds.add(item.getMedicantId());
        });
        // 查询药品信息
        Map<String, StandardMedicantBO> medicantMap = standardMedicantClientService.getMap(medicantIds);
        
        PageData<DrugNotityConfigVO> data = new PageConverter<DrugNotityConfigInfo, DrugNotityConfigVO>() {
            @Override
            public DrugNotityConfigVO converter(DrugNotityConfigInfo entity) {
                DrugNotityConfigVO vo = DrugNotityHelper.getDrugNotityConfigVO(entity);
                vo.setStandardMedicantInfo(medicantMap.get(vo.getMedicantId()));
                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<DrugNotityConfigVO> list(DrugNotityConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        
        List<DrugNotityConfigInfo> list = drugNotityConfigInfoService.list(new QueryWrapper<DrugNotityConfigInfo>()
                    .eq("rows_status", YN.YES.getCode())
                    .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                    .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                    .eq(StrUtil.isNotBlank(params.getMedicantId()), "medicant_id", params.getMedicantId())
                    .eq(StrUtil.isNotBlank(params.getMedicantCode()), "medicant_code", params.getMedicantCode())
                    .in(CollUtil.isNotEmpty(params.getMedicantCodes()), "medicant_code", params.getMedicantCodes())
                    .eq(StrUtil.isNotBlank(params.getProjectCode()), "project_code", params.getProjectCode())
                    .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> medicantIds = new ArrayList<>();
        list.forEach(item -> {
            medicantIds.add(item.getMedicantId());
        });
        // 查询药品信息
        Map<String, StandardMedicantBO> medicantMap = standardMedicantClientService.getMap(medicantIds);
        
        return list.stream().map(i -> {
            DrugNotityConfigVO vo = DrugNotityHelper.getDrugNotityConfigVO(i);
            vo.setStandardMedicantInfo(medicantMap.get(vo.getMedicantId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public DrugNotityConfigVO getById(String id) {
        if (StrUtil.isBlank(id)) {
            return null;
        }
        
        DrugNotityConfigInfo entity = drugNotityConfigInfoService.getById(id);
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return null;
        }

        DrugNotityConfigVO vo = DrugNotityHelper.getDrugNotityConfigVO(entity);

        // 查询药品信息
        StandardMedicantBO medicant = standardMedicantClientService.getById(entity.getMedicantId());
        vo.setStandardMedicantInfo(medicant);

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DrugNotityConfigVO add(DrugNotityConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DrugNotityConfigInfo entity = new DrugNotityConfigInfo();
        entity.setId(DrugNotityHelper.getDrugNotityConfigInfoId());
        entity.setMedicantId(params.getMedicantId());
        entity.setMedicantCode(params.getMedicantCode());
        entity.setProjectCode(params.getProjectCode());
        entity.setOpUser(params.getOpUser());
        entity.setOpTime(DateUtil.date());
        entity.setOpRemark(params.getOpRemark());
        entity.setEffectiveTime(params.getEffectiveTime());
        entity.setRowsStatus(YN.YES.getCode());

        entity.setStatus(YN.NO.getCode());
        if (YN.YES.getCode().equals(params.getStatus())) {
            entity.setStatus(YN.YES.getCode());

            // 若是上架状态则生成快照，并保存快照id
            DrugNotityConfigSnapshotAddDTO snapshotAddDTO = new DrugNotityConfigSnapshotAddDTO();
            snapshotAddDTO.setConfigId(entity.getId());
            snapshotAddDTO.setEffectiveTime(entity.getEffectiveTime());
            snapshotAddDTO.setMedicantId(entity.getMedicantId());
            snapshotAddDTO.setMedicantCode(entity.getMedicantCode());
            snapshotAddDTO.setProjectCode(entity.getProjectCode());
            snapshotAddDTO.setStatus(entity.getStatus());
            snapshotAddDTO.setOpUser(entity.getOpUser());
            snapshotAddDTO.setOpRemark(entity.getOpRemark());
            
            DrugNotityConfigSnapshotVO snapshot = drugNotityConfigSnapshotBizService.add(snapshotAddDTO);

            entity.setSnapshotId(snapshot.getId());
        }
        drugNotityConfigInfoService.save(entity);

        // 记录日志
        DrugNotityConfigLogAddDTO logAddDTO = new DrugNotityConfigLogAddDTO();
        logAddDTO.setConfigId(entity.getId());
        logAddDTO.setMedicantId(entity.getMedicantId());
        logAddDTO.setMedicantCode(entity.getMedicantCode());
        logAddDTO.setProjectCode(entity.getProjectCode());
        logAddDTO.setOpActType(OpTypeEnum.Create.getCode());
        logAddDTO.setOpRemark(entity.getOpRemark());
        logAddDTO.setOpStatus(entity.getStatus());
        logAddDTO.setOpEffectiveTime(entity.getEffectiveTime());
        logAddDTO.setOpUser(entity.getOpUser());
        drugNotityConfigLogBizService.add(logAddDTO);

        return DrugNotityHelper.getDrugNotityConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void update(DrugNotityConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        DrugNotityConfigInfo entity = drugNotityConfigInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return;
        }

        // entity.setProjectCode(params.getProjectCode());
        entity.setOpUser(params.getOpUser());
        entity.setOpTime(DateUtil.date());
        entity.setOpRemark(params.getOpRemark());
        // 若是下架状态则删除快照id
        if (YN.NO.getCode().equals(params.getStatus())) { // 不论状态原先是什么状态，下架都清空快照与提醒生效时间
            entity.setStatus(YN.NO.getCode());
            entity.setSnapshotId("");
            entity.setEffectiveTime("");
            if (StrUtil.isBlank(params.getOpRemark())) {
                entity.setOpRemark("下架");
            }
        } else if (YN.NO.getCode().equals(entity.getStatus())) { // 原先下架状态上架，生成快照
            entity.setStatus(YN.YES.getCode());
            entity.setEffectiveTime(params.getEffectiveTime());
            if (StrUtil.isBlank(params.getOpRemark())) {
                entity.setOpRemark("上架");
            }
            
            // 若是上架状态则生成快照，并保存快照id
            DrugNotityConfigSnapshotAddDTO snapshotAddDTO = new DrugNotityConfigSnapshotAddDTO();
            snapshotAddDTO.setConfigId(entity.getId());
            snapshotAddDTO.setEffectiveTime(entity.getEffectiveTime());
            snapshotAddDTO.setMedicantId(entity.getMedicantId());
            snapshotAddDTO.setMedicantCode(entity.getMedicantCode());
            snapshotAddDTO.setProjectCode(entity.getProjectCode());
            snapshotAddDTO.setStatus(entity.getStatus());
            snapshotAddDTO.setOpUser(entity.getOpUser());
            snapshotAddDTO.setOpRemark(entity.getOpRemark());
            
            DrugNotityConfigSnapshotVO snapshot = drugNotityConfigSnapshotBizService.add(snapshotAddDTO);

            entity.setSnapshotId(snapshot.getId());
        }
        drugNotityConfigInfoService.updateById(entity);

        // 记录日志
        DrugNotityConfigLogAddDTO logAddDTO = new DrugNotityConfigLogAddDTO();
        logAddDTO.setConfigId(entity.getId());
        logAddDTO.setMedicantId(entity.getMedicantId());
        logAddDTO.setMedicantCode(entity.getMedicantCode());
        logAddDTO.setProjectCode(entity.getProjectCode());
        logAddDTO.setOpActType(OpTypeEnum.Update.getCode());
        logAddDTO.setOpRemark(entity.getOpRemark());
        logAddDTO.setOpStatus(entity.getStatus());
        logAddDTO.setOpEffectiveTime(entity.getEffectiveTime());
        logAddDTO.setOpUser(entity.getOpUser());
        drugNotityConfigLogBizService.add(logAddDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateStatus(DrugNotityConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);

        DrugNotityConfigInfo entity = drugNotityConfigInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return;
        }

        entity.setOpUser(params.getOpUser());
        entity.setOpTime(DateUtil.date());
        entity.setOpRemark(params.getOpRemark());
        
        if (YN.YES.getCode().equals(entity.getStatus())) { // 若是上架则下架，清空快照和提醒生效时间
            entity.setStatus(YN.NO.getCode());
            entity.setSnapshotId("");
            entity.setEffectiveTime("");
            if (StrUtil.isBlank(params.getOpRemark())) {
                entity.setOpRemark("下架");
            }
        } else { // 若是下架状态则上架，生成新的快照信息
            entity.setStatus(YN.YES.getCode());
            entity.setEffectiveTime(params.getEffectiveTime());
            if (StrUtil.isBlank(params.getOpRemark())) {
                entity.setOpRemark("上架");
            }
            
            // 若是上架状态则生成快照，并保存快照id
            DrugNotityConfigSnapshotAddDTO snapshotAddDTO = new DrugNotityConfigSnapshotAddDTO();
            snapshotAddDTO.setConfigId(entity.getId());
            snapshotAddDTO.setEffectiveTime(entity.getEffectiveTime());
            snapshotAddDTO.setMedicantId(entity.getMedicantId());
            snapshotAddDTO.setMedicantCode(entity.getMedicantCode());
            snapshotAddDTO.setProjectCode(entity.getProjectCode());
            snapshotAddDTO.setStatus(entity.getStatus());
            snapshotAddDTO.setOpUser(entity.getOpUser());
            snapshotAddDTO.setOpRemark(entity.getOpRemark());
            
            DrugNotityConfigSnapshotVO snapshot = drugNotityConfigSnapshotBizService.add(snapshotAddDTO);

            entity.setSnapshotId(snapshot.getId());
        }

        drugNotityConfigInfoService.updateById(entity);

        // 记录日志
        DrugNotityConfigLogAddDTO logAddDTO = new DrugNotityConfigLogAddDTO();
        logAddDTO.setConfigId(entity.getId());
        logAddDTO.setMedicantId(entity.getMedicantId());
        logAddDTO.setMedicantCode(entity.getMedicantCode());
        logAddDTO.setProjectCode(entity.getProjectCode());
        logAddDTO.setOpActType(OpTypeEnum.Update.getCode());
        logAddDTO.setOpRemark(entity.getOpRemark());
        logAddDTO.setOpStatus(entity.getStatus());
        logAddDTO.setOpEffectiveTime(entity.getEffectiveTime());
        logAddDTO.setOpUser(entity.getOpUser());
        drugNotityConfigLogBizService.add(logAddDTO);
    }
    
    @Override
    public void del(DrugNotityConfigDelDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DrugNotityConfigInfo entity = drugNotityConfigInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return;
        }
        entity.setSnapshotId("");
        entity.setEffectiveTime("");
        entity.setOpUser(params.getOpUser());
        entity.setOpTime(DateUtil.date());
        entity.setOpRemark(params.getOpRemark());
        entity.setStatus(YN.NO.getCode());
        entity.setRowsStatus(YN.NO.getCode());
        drugNotityConfigInfoService.updateById(entity);
        
        // 记录日志
        DrugNotityConfigLogAddDTO logAddDTO = new DrugNotityConfigLogAddDTO();
        logAddDTO.setConfigId(entity.getId());
        logAddDTO.setMedicantId(entity.getMedicantId());
        logAddDTO.setMedicantCode(entity.getMedicantCode());
        logAddDTO.setProjectCode(entity.getProjectCode());
        logAddDTO.setOpActType(OpTypeEnum.DEL.getCode());
        logAddDTO.setOpRemark(entity.getOpRemark());
        logAddDTO.setOpStatus(entity.getStatus());
        logAddDTO.setOpEffectiveTime(entity.getEffectiveTime());
        logAddDTO.setOpUser(entity.getOpUser());
        drugNotityConfigLogBizService.add(logAddDTO);
    }

}
