package com.xxh.cloud.healthy.drug.framework.modules.biz.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.drug.framework.modules.base.entity.DrugNotityConfigLogInfo;
import com.xxh.cloud.healthy.drug.framework.modules.base.service.DrugNotityConfigLogInfoService;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigLogAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigLogQueryDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigLogVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.helper.DrugNotityHelper;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigLogBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DrugNotityConfigLogBizServiceImpl implements DrugNotityConfigLogBizService {

    @Autowired
    private DrugNotityConfigLogInfoService drugNotityConfigLogInfoBizService;

    @Override
    public PageData<DrugNotityConfigLogVO> query(DrugNotityConfigLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        IPage<DrugNotityConfigLogInfo> page = drugNotityConfigLogInfoBizService.page(new Page<>(params.getPage(), params.getRows()), 
                new QueryWrapper<DrugNotityConfigLogInfo>()
                    .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                    .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                    .eq(StrUtil.isNotBlank(params.getConfigId()), "config_id", params.getConfigId())
                    .eq(StrUtil.isNotBlank(params.getMedicantId()), "medicant_id", params.getMedicantId())
                    .eq(StrUtil.isNotBlank(params.getMedicantCode()), "medicant_code", params.getMedicantCode())
                    .eq(StrUtil.isNotBlank(params.getProjectCode()), "project_code", params.getProjectCode())
        );
        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        PageData<DrugNotityConfigLogVO> data = new PageConverter<DrugNotityConfigLogInfo, DrugNotityConfigLogVO>() {
            @Override
            public DrugNotityConfigLogVO converter(DrugNotityConfigLogInfo entity) {
                return DrugNotityHelper.getDrugNotityConfigLogVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public DrugNotityConfigLogVO getById(String id) {
        if (StrUtil.isBlank(id)) {
            return null;
        }
        
        DrugNotityConfigLogInfo entity = drugNotityConfigLogInfoBizService.getById(id);
        if (null == entity) {
            return null;
        }
        return DrugNotityHelper.getDrugNotityConfigLogVO(entity);
    }

    @Override
    public void add(DrugNotityConfigLogAddDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DrugNotityConfigLogInfo entity = new DrugNotityConfigLogInfo();
        entity.setId(DrugNotityHelper.getDrugNotityConfigLogInfoId());
        entity.setConfigId(params.getConfigId());
        entity.setMedicantId(params.getMedicantId());
        entity.setMedicantCode(params.getMedicantCode());
        entity.setProjectCode(params.getProjectCode());
        entity.setOpActType(params.getOpActType());
        entity.setOpEffectiveTime(params.getOpEffectiveTime());
        entity.setOpStatus(params.getOpStatus());
        entity.setOpUser(params.getOpUser());
        entity.setOpTime(DateUtil.date());
        entity.setOpRemark(params.getOpRemark());

        drugNotityConfigLogInfoBizService.save(entity);
    }
    
}
