package com.xxh.cloud.healthy.drug.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.drug.framework.modules.base.entity.DrugNotityConfigSnapshotInfo;
import com.xxh.cloud.healthy.drug.framework.modules.base.service.DrugNotityConfigSnapshotInfoService;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigSnapshotAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigSnapshotListDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigSnapshotVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.helper.DrugNotityHelper;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigSnapshotBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DrugNotityConfigSnapshotBizServiceImpl implements DrugNotityConfigSnapshotBizService {

    @Autowired
    private DrugNotityConfigSnapshotInfoService drugNotityConfigSnapshotInfoService;

    @Override
    public List<DrugNotityConfigSnapshotVO> list(DrugNotityConfigSnapshotListDTO params) {
        List<DrugNotityConfigSnapshotInfo> list = drugNotityConfigSnapshotInfoService.list(new QueryWrapper<DrugNotityConfigSnapshotInfo>()
                    .eq("rows_status", YN.YES.getCode())
                    .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                    .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                    .eq(StrUtil.isNotBlank(params.getMedicantId()), "medicant_id", params.getMedicantId())
                    .eq(StrUtil.isNotBlank(params.getMedicantCode()), "medicant_code", params.getMedicantCode())
                    .eq(StrUtil.isNotBlank(params.getProjectCode()), "project_code", params.getProjectCode())
                    .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
        );
        
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        // TODO 查询药品信息
        
        return list.stream().map(i -> {
            DrugNotityConfigSnapshotVO vo = DrugNotityHelper.getDrugNotityConfigSnapshotVO(i);
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public DrugNotityConfigSnapshotVO getById(String id) {
        if (StrUtil.isBlank(id)) {
            return null;
        }
        
        DrugNotityConfigSnapshotInfo entity = drugNotityConfigSnapshotInfoService.getById(id);
        if (null == entity) {
            return null;
        }
        
        // TODO 查询药品信息
        
        return DrugNotityHelper.getDrugNotityConfigSnapshotVO(entity);
    }

    @Override
    public DrugNotityConfigSnapshotVO add(DrugNotityConfigSnapshotAddDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DateTime now = DateUtil.date();

        DrugNotityConfigSnapshotInfo entity = new DrugNotityConfigSnapshotInfo();
        entity.setId(DrugNotityHelper.getDrugNotityConfigSnapshotInfoId());
        entity.setConfigId(params.getConfigId());
        entity.setMedicantId(params.getMedicantId());
        entity.setMedicantCode(params.getMedicantCode());
        entity.setProjectCode(params.getProjectCode());
        entity.setStatus(params.getStatus());
        entity.setEffectiveTime(params.getEffectiveTime());
        entity.setOpUser(params.getOpUser());
        entity.setOpTime(now);
        entity.setOpRemark(params.getOpRemark());
        entity.setCreator(params.getOpUser());
        entity.setCreateDate(now);

        drugNotityConfigSnapshotInfoService.save(entity);
        
        return DrugNotityHelper.getDrugNotityConfigSnapshotVO(entity);
    }
    
}
