package com.xxh.cloud.healthy.drug.framework.modules.task.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.common.constant.PushWayEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineItemClientBO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigListDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.mapper.DrugNotitySyncFlagMapper;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigBizService;
import com.xxh.cloud.healthy.drug.framework.modules.task.service.DrugNotityTaskHandlerService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientRemindBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientScheduleBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientRemindAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DrugNotityTaskHandlerServiceImpl implements DrugNotityTaskHandlerService {

    @Autowired
    private DrugNotitySyncFlagMapper drugNotitySyncFlagMapper;
    @Autowired
    private PatientPushServeClientService patientPushServeClientService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private DrugNotityConfigBizService drugNotityConfigBizService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveRecipeMedicineSchedule(List<PatientRecipeMedicineClientBO> recipeMedicineList, String flagTime) {

        this.saveSchedule(recipeMedicineList);

        // 更新同步标记
        log.info("更新同步标记时间 >>>>>> flagTime={}", flagTime);
        drugNotitySyncFlagMapper.updateSyncFlag(flagTime, "Recipe_Medicine_Schedule");
    }

    // 处理患者服药
    private void saveSchedule(List<PatientRecipeMedicineClientBO> recipeMedicineList) {
        if (CollectionUtil.isEmpty(recipeMedicineList)) {
            return;
        }

        // 本次提醒的所有药物id
        Map<String, Boolean> drugNotityMap = new HashMap<>();
        Set<String> medicineIds = new HashSet<>();
        recipeMedicineList.forEach(i -> {
            if (CollectionUtil.isNotEmpty(i.getPatientRecipeMedicineItemList())) {
                i.getPatientRecipeMedicineItemList().forEach(j -> {
                    medicineIds.add(j.getCodeStandard());
                    drugNotityMap.put(j.getCodeStandard(), true); // 初始化时默认都提醒
                });
            }
        });
        if (CollectionUtil.isEmpty(medicineIds)) {
            return;
        }

        // 获取药物配置信息确定药物是否推送
        DrugNotityConfigListDTO drugNotityConfigListDTO = new DrugNotityConfigListDTO();
        drugNotityConfigListDTO.setMedicantCodes(new ArrayList<>(medicineIds));
        List<DrugNotityConfigVO> drugNotityConfigVOs = drugNotityConfigBizService.list(drugNotityConfigListDTO);
        if (CollectionUtil.isNotEmpty(drugNotityConfigVOs)) {
            drugNotityConfigVOs.forEach(i -> {
                if (YN.NO.getCode().equals(i.getStatus())) {
                    drugNotityMap.put(i.getMedicantCode(),  false);
                }
            });
        }

        List<PatientScheduleAddClientDTO> scheduleList = new ArrayList<>();

        Set<String> patientIds = new HashSet<>(); // 患者id，用户查询对应的用户信息（user_id, open_id），只查本人

        for (PatientRecipeMedicineClientBO item : recipeMedicineList) {
            patientIds.add(item.getPatientId());

            // Map<String, PatientScheduleAddClientDTO> scheduleMap = new HashMap<>(); // 提醒信息
            Map<String, List<Map<String, String>>> contentMap = new HashMap<>(); // 提醒的药物信息 key: 日程时间  value: 用药详情列表

            String date = DateUtil.format(item.getPayTime(), "yyyy-MM-dd HH:mm").toString() + ":00";

            for (PatientRecipeMedicineItemClientBO i : item.getPatientRecipeMedicineItemList()) {
                // 判断本药物是否需要提醒
                if (!drugNotityMap.get(i.getCodeStandard())) {
                    continue;
                }

                String scheduleDate = date;

                Integer frequency = 0;
                Integer totalNum = 1;
                if (StrUtil.isNotBlank(i.getMedicationAmount())) {
                    // 服药总药量
                    BigDecimal totalAmount = this.getTotal(new BigDecimal(i.getEachQuantity()), i.getDoseUnitStandard(), new BigDecimal(i.getMedicationAmount()), i.getMedicationAmountUnit(), i.getMedicineSpec());

                    // 计算频次与总天数，根据频次与总天数计算每个日程时间
                    frequency = this.getFrequency(i.getFrequencyCodeStandard()); // 每个日程间隔数
                    totalNum = this.getTotalNum(i.getFrequencyCodeStandard(), totalAmount, i.getEachQuantity()); // 总日程数
                }

                for (int num = 0; num < totalNum; num++) {
                    if (num == 0) {
                        // TODO 有的药物不需要开药时直接吃，第一次吃药可能需要延后几天再按照一定频次吃药
                        // if () {
                            // TODO FIXME
                            // Integer numDay = 0;
                            // scheduleDate = DateUtil.offsetDay(DateUtil.parseDate(scheduleDate), numDay).toDateStr() + " 07:00:00";
                        // } 

                        // TODO 领药当天服药的总药量根据领药时间有所不同，晚上领药可能只需要吃一顿，中午领药可能吃两顿
                    } else {
                        scheduleDate = DateUtil.offsetDay(DateUtil.parseDate(scheduleDate), frequency).toDateStr() + " 07:00:00";
                    }

                    // 医嘱服药内容
                    List<Map<String, String>> contentList = contentMap.get(scheduleDate);
                    if (null == contentList) {
                        contentList = new ArrayList<>();
                        contentMap.put(scheduleDate, contentList);
                    }

                    Map<String, String> itemContentMap = this.medicineContent(i);
                    contentList.add(itemContentMap);

                    // 药品即将用尽提醒
                    if (totalNum > 2 && totalNum - 2 <= num) {
                        itemContentMap.put("continued", YN.YES.getCode());
                    }
                }
            }

            if (CollectionUtil.isNotEmpty(contentMap)) {
                String continuedDate = null;

                for (Entry<String, List<Map<String, String>>> entry : contentMap.entrySet()) {
                    String k = entry.getKey();
                    List<Map<String, String>> v = entry.getValue();

                    PatientScheduleAddClientDTO schedule = new PatientScheduleAddClientDTO();
                    schedule.setScheduleDate(k);
                    schedule.setPatientId(item.getPatientId());
                    // schedule.setUserId(); // 另外查询
                    schedule.setBizId(item.getId());
                    schedule.setBizOneType("recipe_medicine");
                    // patientScheduleInfo.setBizTwoType(null);
                    schedule.setTitle("【服药提醒】请遵医嘱服药");
                    schedule.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
                    schedule.setScheduleType(RemindWayEnum.ALL.getCode()); // 只提醒

                    Map<String, Object> content = new HashMap<>();
                    content.put("patientName", item.getPatientName());
                    content.put("item", v);
                    schedule.setContent(JSON.toJSONString(content));

                    // 判断是否续药
                    for (Map<String, String> m : v) {
                        if (YN.YES.getCode().equals(m.get("continued"))) {
                            if (null == continuedDate || continuedDate.compareTo(k) > 0) {
                                continuedDate = k;
                            }
                        }
                    }

                    scheduleList.add(schedule);
                }

                if (null != continuedDate) {
                    PatientScheduleAddClientDTO schedule = new PatientScheduleAddClientDTO();
                    schedule.setScheduleDate(continuedDate);
                    schedule.setPatientId(item.getPatientId());
                    // schedule.setUserId(); // 另外查询
                    schedule.setBizId(item.getId());
                    schedule.setBizOneType("recipe_medicine_continued");
                    // patientScheduleInfo.setBizTwoType(null);
                    schedule.setTitle("【续药提醒】请遵医嘱续药");
                    schedule.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
                    schedule.setScheduleType(RemindWayEnum.ALL.getCode()); // 只提醒

                    Map<String, Object> content = new HashMap<>();
                    content.put("patientName", item.getPatientName());
                    schedule.setContent(JSON.toJSONString(content));
                    
                    scheduleList.add(schedule);
                }
            }
        }

        if (CollectionUtil.isNotEmpty(patientIds)) {
            // TODO 根据患者查询家属（暂时是本人）
            List<PatientScheduleAddClientDTO> schedules = new ArrayList<>();

            Map<String, List<MemberFamilyBO>> memberMap = this.getMemberMapByPatientIds(new ArrayList<>(patientIds));
            scheduleList.forEach(i -> {
                List<MemberFamilyBO> memberList = memberMap.get(i.getPatientId());
                if (CollectionUtil.isNotEmpty(memberList)) {
                    if (memberList.size() == 1) {
                        PatientScheduleAddClientDTO schedule = new PatientScheduleAddClientDTO();
                        BeanUtil.copyProperties(i, schedule, false);
                        schedule.setUserId(memberList.get(0).getUserId());
                        schedules.add(schedule);
                    } else {
                        memberList.forEach(member -> {
                            if ("1".equals(member.getRelationType()) && YN.YES.getCode().equals(member.getIsAuth())) {
                                // TODO 设置要发送给家属的家属id，若家属有多个那需要copy实体并分别设置家属id
                                PatientScheduleAddClientDTO schedule = new PatientScheduleAddClientDTO();
                                BeanUtil.copyProperties(i, schedule, false);
                                schedule.setUserId(member.getUserId());
                                schedules.add(schedule);
                            }
                        });
                    }
                }
            });

            // 保存到日程
            patientPushServeClientService.addPatientScheduleBatch(schedules);
        }
    }

    private Map<String, String> medicineContent(PatientRecipeMedicineItemClientBO medicine) {
        Map<String, String> map = new HashMap<>();
        map.put("name", medicine.getNameStandard());
        map.put("code", medicine.getCodeStandard());
        map.put("quantity", medicine.getEachQuantity());
        map.put("unit", medicine.getDoseUnitStandard());
        map.put("frequencyCode", medicine.getFrequencyCodeStandard());
        map.put("frequencyName", medicine.getFrequencyNameStandard());
        return map;
    }

    public static void main(String[] args) {
        DrugNotityTaskHandlerServiceImpl impl = new DrugNotityTaskHandlerServiceImpl();

        BigDecimal totalAmount = impl.getTotal(new BigDecimal("10"), "mg", new BigDecimal("6"), "片", "10mg*100片/瓶");//impl.getTotal(new BigDecimal("10"), "mg", new BigDecimal("1"), "盒", "10mg*20片/盒");          // 预期：6*10     = 60
        System.out.println("totalAmount: " + totalAmount);

        Integer frequency = impl.getFrequency("Q8H"); // 每个日程间隔数
        System.out.println("frequency: " + frequency);
        Integer totalNum = impl.getTotalNum("Q8H", totalAmount, "10"); // 总日程数
        System.out.println("totalNum: " + totalNum);

        // BigDecimal a = impl.getTotal(new BigDecimal("10"), "mg", new BigDecimal("6"), "片", "10mg*100片/瓶");          // 预期：6*10     = 60
        // System.out.println("a: " + a);

        // BigDecimal b = impl.getTotal(new BigDecimal("30"), "mg", new BigDecimal("1"), "盒", "30mg*14片/盒");           // 预期：1*14*30  = 420
        // System.out.println("b: " + b);
        
        // BigDecimal c = impl.getTotal(new BigDecimal("250"), "ml", new BigDecimal("4"), "瓶", "250ml:12.5g*1袋/袋");    // 预期：4*250    = 1000
        // System.out.println("c: " + c);
        
        // BigDecimal d = impl.getTotal(new BigDecimal("1"), "g", new BigDecimal("1"), "支", "5ml:1g*1支/支");            // 预期：1*1*1    = 1
        // System.out.println("d: " + d);
        
        // BigDecimal e = impl.getTotal(new BigDecimal("80"), "mg", new BigDecimal("1"), "片", "(80mg+12.5mg)*7片/盒");   // 预期：1*80     = 80
        // System.out.println("e: " + e);
        
        // BigDecimal f = impl.getTotal(new BigDecimal("10"), "mg", new BigDecimal("1"), "片", "10mg：5mg*10片/盒");      // 预期：1*10     = 10
        // System.out.println("f: " + f);
        
        // BigDecimal g = impl.getTotal(new BigDecimal("2"), "IU", new BigDecimal("1"), "支", "10ml:400iu*1支/支");       // 预期：1*400    = 400
        // System.out.println("g: " + g);
        
        // BigDecimal h = impl.getTotal(new BigDecimal("4250"), "IU", new BigDecimal("1"), "支", "4250IUaXa*1支/支");     // 预期：1*4250   = 4250
        // System.out.println("h: " + h);
        
        // BigDecimal i = impl.getTotal(new BigDecimal("7"), "ml", null, "瓶", "60ml:1.2g*1瓶/瓶");                     // 预期：0
        // System.out.println("i: " + i);
        
        // BigDecimal j = impl.getTotal(new BigDecimal("45"), "ml", null, "盒", "15mg*10袋/盒");                        // 预期：0
        // System.out.println("j: " + j);
        
        // BigDecimal k = impl.getTotal(new BigDecimal("1"), "份", new BigDecimal("24"), "袋", "(A剂+B剂)*6袋/盒");       // 预期：24
        // System.out.println("k: " + k);
        
        // BigDecimal l = impl.getTotal(new BigDecimal("2.5"), "g", new BigDecimal("2"), "支", "2.5g*1支/支");            // 预期：2*2.5    = 5
        // System.out.println("l: " + l);
    }

    /**
     * 
     * 药品规格格式：([num3][unit3]+[num4][unit4][+...]):[?]3*[num2][unit2]/unit1
     * 10mg*100片/瓶
     * 30mg*14片/盒
     * (0.5mg+10mg)*18粒/盒    // 0.5mg和10mg只有一个有效，根据单次用药剂量来确定（整数倍且药量最少），非10.5mg
     * 10mg：5mg*10片/盒       // 把中文的冒号替换成英文的冒号
     * 60ml:1.2g*1瓶/瓶        // 根据单次用药的单位确定（若为ml则使用60ml，若为g则使用1.2g）
     * 10ml:400iu*1支/支       // 
     * 4250IUaXa*1支/支        // 
     * (A剂+B剂)*6袋/盒        // 若单次用药单位（例如：份）不在规格中则星号前的所有数据算一个单位的该单位（即：一份）
     * 
     * @param eachQuantity 单次剂量
     * @param doseUnit 单次剂量单位 注意：该单位为药品规格中的最小单位
     * @param total 总剂量
     * @param totalUnit 总剂量单位 注意：本方法总剂量单位的范围大于等于单次剂量单位
     * @param medicineSpec 药品规格
     * @return 服药总药量（单位为单次用药单位：doseUnit）
     */
    private BigDecimal getTotal(BigDecimal eachQuantity, String doseUnit, BigDecimal total, String totalUnit, String medicineSpec) {
        if (null == eachQuantity || eachQuantity.compareTo(new BigDecimal("0")) <= 0 || StrUtil.isBlank(doseUnit) || null == total || total.compareTo(new BigDecimal("0")) <= 0 || StrUtil.isBlank(totalUnit) || StrUtil.isBlank(medicineSpec)) {
            return new BigDecimal("0");
        }

        // 小写转大写
        doseUnit = doseUnit.toUpperCase();
        totalUnit = totalUnit.toUpperCase();
        medicineSpec = medicineSpec.toUpperCase();
        
        // 总单位与单次单位一致则直接返回总剂量即为总次数
        if (doseUnit.equals(totalUnit)) {
            return total;
        }

        // 把所有中文冒号改为英文冒号
        if (medicineSpec.indexOf("：") >= 0) {
            medicineSpec = medicineSpec.replace('：', ':');
        }

        // 解析药品规格：
        String[] split1 = medicineSpec.split("/");
        String[] split2 = split1[0].split("\\*");
        
        String unit1 = split1[1]; // 斜杠后的单位
        log.info("单位1：{}", unit1);

        BigDecimal num2 = getNumber(split2[1]); // 斜杠前*号后的数值
        String unit2 = getUnit(split2[1], num2.intValue()); // 斜杠前*号后的数值单位
        log.info("数值单位2：{}{}", num2, unit2);

        BigDecimal num3 = null; // *号前的数值
        // String unit3 = null; // *号前的数值单位
        if (!split2[0].contains(doseUnit)) { // 单次剂量单位不在规格中，则规格中剂量为一个单位
            num3 = new BigDecimal("1");
        } else if (split2[0].indexOf(":") != -1 || split2[0].indexOf("(") != -1) {
            String[] split3 = null;
            if (split2[0].indexOf(":") != -1) {
                split3 = split2[0].split(":");
            }
            if (split2[0].indexOf("(") != -1) {
                split3 = split2[0].substring(split2[0].indexOf("(") + 1, split2[0].indexOf(")")).split("\\+");
            }

            BigDecimal num4 = null;
            for (String data : split3) {
                BigDecimal num = getNumber(data);
                String unit = getUnit(data, num.intValue());

                if (doseUnit.equals(unit)) { // 单次用药剂量单位与药品规格单位一致
                    if (eachQuantity.compareTo(num) >= 0) { // 根据单次用药剂量来确定（整数倍且药量最少）
                        BigDecimal[] bigDecimalArray = eachQuantity.divideAndRemainder(num);
                        if (bigDecimalArray[1].equals(new BigDecimal(0)) && (null == num4 || num4.compareTo(bigDecimalArray[0]) > 0)) {
                            num4 = bigDecimalArray[0];
                            num3 = num;
                        }
                    } else { // 根据单次用药剂量来确定（整数倍且药量最多）
                        BigDecimal[] bigDecimalArray = num.divideAndRemainder(eachQuantity);
                        if (bigDecimalArray[1].equals(new BigDecimal(0)) && (null == num4 || num4.compareTo(bigDecimalArray[0]) < 0)) {
                            num4 = bigDecimalArray[0];
                            num3 = num;
                        }
                    }
                }
            }
        } else {
            num3 = getNumber(split2[0]);
            // unit3 = getUnit(split2[0], num3.intValue());
        }
        // log.info("数值单位3：{}{}", num3, unit3);
        log.info("数值3：{}", num3);
        
        if (null == num3) {
            return new BigDecimal("0");
        }

        // 根据总剂量单位换算成单次剂量单位后的总剂量
        if (unit1.equals(totalUnit)) {
            return new BigDecimal(total + "").multiply(num2).multiply(num3);
        }

        // if (unit2.equals(totalUnit)) {
        //     return new BigDecimal(total + "").multiply(num3);
        // }
        // return total;
        return new BigDecimal(total + "").multiply(num3);
    }

    // 获取数值
    private BigDecimal getNumber(String data) {
        String number = "0";
        Pattern pattern = Pattern.compile("\\d+\\.?\\d*");
        Matcher matcher = pattern.matcher(data);

        if (matcher.find()) {
            number = matcher.group();
        }
        return new BigDecimal(number);
    }

    // 获取单位
    private String getUnit(String data, Integer number) {
        String numString = String.valueOf(number);
        int startIndex = data.indexOf(number) + numString.length();
        return data.substring(startIndex + 1).trim();
    }

    /**
     * 返回频次的天数
     * 一天多次返回 1
     * 多天一次返回 天数
     */
    private Integer getFrequency(String frequencyCode) {
        frequencyCode = frequencyCode.toUpperCase();
        if ("ST".equals(frequencyCode)) return 0; // 立即
        if ("QD".equals(frequencyCode)) return 1; // 每日一次
        if ("BID".equals(frequencyCode)) return 1; // 每日两次
        if ("TID".equals(frequencyCode)) return 1; // 每日三次
        if ("QID".equals(frequencyCode)) return 1; // 每日四次
        if ("QOD".equals(frequencyCode)) return 2; // 隔日一次，每两天一次
        if ("QW".equals(frequencyCode)) return 7; // 每周一次，每7天一次
        if ("BIW".equals(frequencyCode)) return 3; // 每周两次，每3天一次
        if ("TIW".equals(frequencyCode)) return 2; // 每周三次， 每两天一次
        if ("QOW".equals(frequencyCode)) return 14; // 隔周一次，每14天一次
        if ("2W".equals(frequencyCode)) return 14; // 每两周一次，每14天一次
        if ("3W".equals(frequencyCode)) return 21; // 每三周一次，每21天一次
        if ("4W".equals(frequencyCode)) return 28; // 每四周一次，每28天一次
        return 1;
    }

    /**
     * 返回每天的服药次数
     * @param frequencyCode
     * @return
     */
    private Integer getEachDayQuantity(String frequencyCode) {
        frequencyCode = frequencyCode.toUpperCase();
        if ("ST".equals(frequencyCode)) return 1; // 立即
        if ("BID".equals(frequencyCode)) return 2; // 每日两次
        if ("TID".equals(frequencyCode)) return 3; // 每日三次
        if ("QID".equals(frequencyCode)) return 4; // 每日四次
        if ("Q1/2H".equals(frequencyCode)) return 48; // 30分钟一次
        if ("QH".equals(frequencyCode)) return 24; // 每小时一次
        if ("Q2H".equals(frequencyCode)) return 12; // 二小时一次
        if ("Q3H".equals(frequencyCode)) return 8; // 三小时一次
        if ("Q4H".equals(frequencyCode)) return 6; // 四小时一次
        if ("Q6H".equals(frequencyCode)) return 4; // 六小时一次
        if ("Q8H".equals(frequencyCode)) return 3; // 八小时一次
        if ("Q12H".equals(frequencyCode)) return 2; // 12小时一次
        return 1;
    }

    /**
     * 返回总共提醒的天数
     * 总提醒次数=总药量/(每次药量*每天次数)
     */
    private Integer getTotalNum(String frequencyCode, BigDecimal total, String eachQuantity) {
        if (total.equals(new BigDecimal("0"))) {
            return 1;
        }
        BigDecimal each = new BigDecimal(eachQuantity);

        if ("ST".equals(frequencyCode)) return 1;

        Integer eachDayQuantity = getEachDayQuantity(frequencyCode);
        return total.divide(each.multiply(new BigDecimal(eachDayQuantity)), 0, BigDecimal.ROUND_CEILING).intValue();
    }

    // TODO 暂时只推送给本人，后续需要区分推送给本人和家属
    private Map<String, List<MemberFamilyBO>> getMemberMapByPatientIds(List<String> patientIds) {
        // 1:本人 2:配偶 3:父母 4:子女 5:他人
        MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
        // memberFamilyListClientDTO.setRelationType("1"); // 只查询本人
        memberFamilyListClientDTO.setPatHisNos(patientIds); // zs20231115
                                                            // 家属表中patientId来自海鹚和本平台的patientId不一致，一致的是patHisNos（就诊人门诊id）
        // memberFamilyListClientDTO.setMainPersonId(patientId); // zs20240318
        // memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
        List<MemberFamilyBO> memberFamilyBOs = customerFamilyClientService.listByUser(memberFamilyListClientDTO);

        Map<String, List<MemberFamilyBO>> memberFamilyMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(memberFamilyBOs)) {
            memberFamilyBOs.forEach(i -> {
                List<MemberFamilyBO> list = memberFamilyMap.get(i.getPatHisNo());
                if (null == list) {
                    list = new ArrayList<>();
                    memberFamilyMap.put(i.getPatHisNo(), list);
                }
                list.add(i);
            });
        }
        return memberFamilyMap;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveRecipeMedicineRemind(List<PatientScheduleBO> scheduleList, String flagTime) {
        this.saveRemind(scheduleList);

        // 更新同步标记
        log.info("更新同步标记时间 >>>>>> flagTime={}", flagTime);
        drugNotitySyncFlagMapper.updateSyncFlag(flagTime, "Recipe_Medicine_Remind");
    }

    private void saveRemind(List<PatientScheduleBO> scheduleList) {
        if (CollectionUtil.isEmpty(scheduleList)) {
            return;
        }

        List<PatientRemindAddClientDTO> remindList = new ArrayList<>();
        Set<String> patientIds = new HashSet<>();
        for (PatientScheduleBO schedule : scheduleList) {
            PatientRemindAddClientDTO remind = new PatientRemindAddClientDTO();
            remind.setPatientId(schedule.getPatientId());
            remind.setUserId(schedule.getUserId());
            remind.setPushDate(schedule.getScheduleDate());
            remind.setBizId(schedule.getBizId());
            remind.setBizOneType(schedule.getBizOneType());
            remind.setBizTwoType(schedule.getBizTwoType());
            remind.setTitle(schedule.getTitle());
            remind.setContent(schedule.getContent());
            remind.setPushChannel(PushWayEnum.PCW_WX.getCode());
            remind.setPushStatus(PatientPushServeConstant.RemindPushStatus.Todo.getCode());
            // remind.setPushAccount(null);

            remindList.add(remind);

            patientIds.add(schedule.getPatientId());
        }

        Map<String, MemberFamilyBO> memberMap = getMemberMapByPatientIds2(new ArrayList<>(patientIds));
        remindList.forEach(i -> {
            MemberFamilyBO member = memberMap.get(i.getPatientId() + "-" + i.getUserId());
            if (null != member) {
                i.setPushAccount(member.getUserOpenId());
            }
        });

        // 保存到提醒
        List<PatientRemindBO> remindBOList = patientPushServeClientService.addPatientRemindBatch(remindList);

        // 发送提醒 TODO 暂时通过其他定时发送提醒任务发送
    }
    
    // TODO 暂时只推送给本人，后续需要区分推送给本人和家属
    private Map<String, MemberFamilyBO> getMemberMapByPatientIds2(List<String> patientIds) {
        // 1:本人 2:配偶 3:父母 4:子女 5:他人
        MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
        memberFamilyListClientDTO.setRelationType("1"); // TODO 只查询本人
        memberFamilyListClientDTO.setPatHisNos(patientIds); // zs20231115
                                                            // 家属表中patientId来自海鹚和本平台的patientId不一致，一致的是patHisNos（就诊人门诊id）
        // memberFamilyListClientDTO.setMainPersonId(patientId); // zs20240318
        memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
        List<MemberFamilyBO> memberFamilyBOs = customerFamilyClientService.listByUser(memberFamilyListClientDTO);

        Map<String, MemberFamilyBO> memberFamilyMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(memberFamilyBOs)) {
            memberFamilyBOs.forEach(i -> {
                MemberFamilyBO member = memberFamilyMap.get(i.getPatHisNo() + "-" + i.getUserId());
                if (null == member) {
                    memberFamilyMap.put(i.getPatHisNo() + "-" + i.getUserId(), member);
                }
            });
        }
        return memberFamilyMap;
    }

}
