package com.xxh.cloud.healthy.drug.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientRecipeMedicineClientDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.DrugNotitySyncFlagModel;
import com.xxh.cloud.healthy.drug.framework.modules.biz.mapper.DrugNotitySyncFlagMapper;
import com.xxh.cloud.healthy.drug.framework.modules.task.service.DrugNotityTaskHandlerService;
import com.xxh.cloud.healthy.drug.framework.modules.task.service.DrugNotityTaskService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientScheduleBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleQueryClientDTO;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DrugNotityTaskServiceImpl implements DrugNotityTaskService {

    @Autowired
    private DrugNotitySyncFlagMapper drugNotitySyncFlagMapper;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;
    @Autowired
    private DrugNotityTaskHandlerService drugNotityTaskHandlerService;
    @Autowired
    private PatientPushServeClientService patientPushServeClientService;

    @Override
    public void recipeMedicineScheduleJobHandler() {
        // 获取同步标记
        DrugNotitySyncFlagModel syncFlag = drugNotitySyncFlagMapper.getSyncFlag(new QueryWrapper<DrugNotitySyncFlagModel>()
            .eq("code", "Recipe_Medicine_Schedule")
        );
 
        if (null == syncFlag) {
            syncFlag = new DrugNotitySyncFlagModel();
            syncFlag.setCode("Recipe_Medicine_Schedule");
            syncFlag.setName("医嘱用药提醒");
            syncFlag.setSyncFlag(DateUtil.offsetMinute(DateUtil.date(), -5).toString("yyyy-MM-dd HH:mm:ss"));
            syncFlag.setMaxMin(5);
            drugNotitySyncFlagMapper.insertSyncFlag(syncFlag);
        }

        if (StrUtil.isBlank(syncFlag.getSyncFlag()) || syncFlag.getMaxMin() < 0) {
            throw new BusinessException("同步标记配置异常");
        }

        // 处理边界时间
        String now = DateUtil.now();
        String startTime = syncFlag.getSyncFlag();
        // 若开始时间大于当前时间则停止同步
        if (now.compareTo(startTime) < 0) {
            return;
        }
        String endTime = DateUtil.offsetMinute(DateUtil.parseDateTime(syncFlag.getSyncFlag()), syncFlag.getMaxMin()).toString();
        // 若结束时间大于当前时间则停止同步
        if (now.compareTo(endTime) < 0) {
            return;
        }

        // 查询未推送的医嘱用药
        PatientRecipeMedicineClientDTO patientRecipeMedicineParams = new PatientRecipeMedicineClientDTO();
        patientRecipeMedicineParams.setStartDate(startTime);
        patientRecipeMedicineParams.setEndDate(endTime);

        List<String> scenes = new ArrayList<>();
        scenes.add(DwdConstant.DataSceneEnum.MZ.getCode());
        scenes.add(DwdConstant.DataSceneEnum.JZ.getCode());
        // scenes.add(DwdConstant.DataSceneEnum.ZY.getCode()); // TODO 不提醒住院场景用药
        patientRecipeMedicineParams.setScenes(scenes);

        List<PatientRecipeMedicineClientBO> recipeMedicineList = dwdDataPatientClientService.listRecipeMedicineInfo(patientRecipeMedicineParams);

        // 保存医嘱用药到日程中
        drugNotityTaskHandlerService.saveRecipeMedicineSchedule(recipeMedicineList, endTime);

    }

    @Override
    public void recipeMedicineRemindJobHandler() {
        // 查询当前时间附近的服药日程数据，并根据日程时间生成提醒
        // 获取同步标记
        DrugNotitySyncFlagModel syncFlag = drugNotitySyncFlagMapper.getSyncFlag(new QueryWrapper<DrugNotitySyncFlagModel>()
            .eq("code", "Recipe_Medicine_Remind")
        );

        if (null == syncFlag) {
            syncFlag = new DrugNotitySyncFlagModel();
            syncFlag.setCode("Recipe_Medicine_Remind");
            syncFlag.setName("医嘱用药提醒");
            syncFlag.setSyncFlag(DateUtil.offsetMinute(DateUtil.date(), -5).toString("yyyy-MM-dd HH:mm:ss"));
            syncFlag.setMaxMin(5);
            drugNotitySyncFlagMapper.insertSyncFlag(syncFlag);
        }

        if (StrUtil.isBlank(syncFlag.getSyncFlag()) || syncFlag.getMaxMin() < 0) {
            throw new BusinessException("同步标记配置异常");
        }

        // 处理边界时间 // 为防止先运行提醒再生成日程导致提醒没有日程数据生成不了，所以处理时间需要比日程处理时间晚
        String date = DateUtil.offsetMinute(DateUtil.date(), -5).toString();
        // String date = DateUtil.offsetMinute(DateUtil.parse("2024-04-29 18:21:00"), -5).toString();
        String startTime = syncFlag.getSyncFlag();
        // 若开始时间大于当前时间则停止同步
        if (date.compareTo(startTime) < 0) {
            return;
        }
        String endTime = DateUtil.offsetMinute(DateUtil.parseDateTime(syncFlag.getSyncFlag()), syncFlag.getMaxMin()).toString();
        // 若结束时间大于当前时间则停止同步
        if (date.compareTo(endTime) < 0) {
            return;
        }

        // 查询未生成提醒的医嘱用药日程
        PatientScheduleQueryClientDTO scheduleParams = new PatientScheduleQueryClientDTO();
        scheduleParams.setStartDate(startTime);
        scheduleParams.setEndDate(endTime);
        List<PatientScheduleBO> scheduleList = patientPushServeClientService.patientScheduleBOList(scheduleParams);

        // 保存医嘱用药日程到提醒中
        drugNotityTaskHandlerService.saveRecipeMedicineRemind(scheduleList, endTime);
    }
    
}
