package com.xxh.cloud.healthy.drug.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigAddDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigDelDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigGetDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigListDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigQueryDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigUpdateDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigUpdateStatusDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/drug/config")
@Api(tags = "医生药物管理")
public class DrugNotityConfigBizController {
    
    @Autowired
    private DrugNotityConfigBizService drugNotityConfigBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DrugNotityConfigVO.class)
    public Result<PageData<DrugNotityConfigVO>> query(@RequestBody DrugNotityConfigQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DrugNotityConfigVO> data = drugNotityConfigBizService.query(params);
        return new Result<PageData<DrugNotityConfigVO>>().ok(data);
    }

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DrugNotityConfigVO.class)
    public Result<List<DrugNotityConfigVO>> detail(@RequestBody DrugNotityConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<DrugNotityConfigVO> list = drugNotityConfigBizService.list(params);
        return new Result<List<DrugNotityConfigVO>>().ok(list);
    }

    @PostMapping("/info")
    @ApiOperation(value = "详情", response = DrugNotityConfigVO.class)
    public Result<DrugNotityConfigVO> detail(@RequestBody DrugNotityConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DrugNotityConfigVO vo = drugNotityConfigBizService.getById(params.getId());
        return new Result<DrugNotityConfigVO>().ok(vo);
    }

    @PostMapping("/add")
    @ApiOperation(value = "新增", response = DrugNotityConfigVO.class)
    public Result<DrugNotityConfigVO> add(@RequestBody DrugNotityConfigAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUser(user.getId());
        if (StrUtil.isBlank(params.getProjectCode())) {
            params.setProjectCode("WBMD");
        }
        if (StrUtil.isBlank(params.getOpRemark())) {
            params.setOpRemark("新增");
        }
        DrugNotityConfigVO vo = drugNotityConfigBizService.add(params);
        return new Result<DrugNotityConfigVO>().ok(vo);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody DrugNotityConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUser(user.getId());
        drugNotityConfigBizService.update(params);
        return new Result<Void>().success();
    }

    @PostMapping("/updateStatus")
    @ApiOperation(value = "上下架", response = Void.class)
    public Result<Void> updateStatus(@RequestBody DrugNotityConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUser(user.getId());
        drugNotityConfigBizService.updateStatus(params);
        return new Result<Void>().success();
    }

    @PostMapping("/del")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> add(@RequestBody DrugNotityConfigDelDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUser(user.getId());
        if (StrUtil.isBlank(params.getOpRemark())) {
            params.setOpRemark("删除");
        }
        drugNotityConfigBizService.del(params);
        return new Result<Void>().success();
    }

}
