package com.xxh.cloud.healthy.drug.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigLogGetDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.dto.DrugNotityConfigLogQueryDTO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.bean.vo.DrugNotityConfigLogVO;
import com.xxh.cloud.healthy.drug.framework.modules.biz.service.DrugNotityConfigLogBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/drug/config/log")
@Api(tags = "医生药物日志管理")
public class DrugNotityConfigLogBizController {
    
    @Autowired
    private DrugNotityConfigLogBizService drugNotityConfigLogBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DrugNotityConfigLogVO.class)
    public Result<PageData<DrugNotityConfigLogVO>> query(@RequestBody DrugNotityConfigLogQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DrugNotityConfigLogVO> data = drugNotityConfigLogBizService.query(params);
        return new Result<PageData<DrugNotityConfigLogVO>>().ok(data);
    }

    @PostMapping("/info")
    @ApiOperation(value = "详情", response = DrugNotityConfigLogVO.class)
    public Result<DrugNotityConfigLogVO> detail(@RequestBody DrugNotityConfigLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DrugNotityConfigLogVO vo = drugNotityConfigLogBizService.getById(params.getId());
        return new Result<DrugNotityConfigLogVO>().ok(vo);
    }

}
