package com.xxh.cloud.healthy.mdt.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant.ConsultationType;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationAdviceDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationAdviceQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorUpdateDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationFinishDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationSummaryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationGetDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationJoinOrRefuseDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationSignDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationSignQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationStartOrCancelDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationUpdateDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationAdviceQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationAdviceReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationDoctorQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationFinishReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationSummaryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationGetReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationJoinOrRefuseReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationSignQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationSignReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationStartOrCancelReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationAdviceVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationDoctorVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationSignVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationStatusCountStatisticVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt")
@Api(tags = "医生mdt管理")
public class ConsultationBizController {
    
    @Autowired
    private ConsultationBizService consultationBizService;

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody ConsultationUpdateDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setSummaryUserId(user.getId());
        consultationBizService.update(params);
        return new Result<Void>().success();
    }

    @PostMapping("/doctor/update")
    @ApiOperation(value = "更新会诊医生信息", response = Void.class)
    public Result<Void> updateDoctor(@RequestBody ConsultationDoctorUpdateDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setSummaryUserId(user.getId());
        consultationBizService.updateDoctor(params);
        return new Result<Void>().success();
    }

    @PostMapping("/query")
    @ApiOperation(value = "会诊分页列表", response = ConsultationVO.class)
    public Result<PageData<ConsultationVO>> query(@RequestBody ConsultationQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationQueryDTO queryDTO = new ConsultationQueryDTO();
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        queryDTO.setDoctorId(user.getId());
        queryDTO.setPatientName(params.getPatientName());
        queryDTO.setStatus(params.getStatus());
        queryDTO.setJoinStatus(params.getJoinStatus());

        queryDTO.setApplyType(ConsultationType.MDT.getCode()); // 只查询mdt会诊
        PageData<ConsultationVO> data = consultationBizService.page(queryDTO);
        return new Result<PageData<ConsultationVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "会诊详情", response = ConsultationVO.class)
    public Result<ConsultationVO> detail(@RequestBody ConsultationGetReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationGetDTO getDTO = new ConsultationGetDTO();
        getDTO.setId(params.getId());
        getDTO.setDoctorId(user.getId());
        ConsultationVO vo = consultationBizService.detail(getDTO);
        return new Result<ConsultationVO>().ok(vo);
    }

    @PostMapping("/trendsList")
    @ApiOperation(value = "会诊动态列表", response = ConsultationDoctorVO.class)
    public Result<List<ConsultationDoctorVO>> trendsList(@RequestBody ConsultationDoctorQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationDoctorQueryDTO queryDTO = new ConsultationDoctorQueryDTO();
        queryDTO.setConsultationId(params.getConsultationId());
        queryDTO.setDoctorId(user.getId());
        List<ConsultationDoctorVO> data = consultationBizService.trendsList(queryDTO);
        return new Result<List<ConsultationDoctorVO>>().ok(data);
    }

    @PostMapping("/signList")
    @ApiOperation(value = "会诊签到列表", response = ConsultationSignVO.class)
    public Result<List<ConsultationSignVO>> signList(@RequestBody ConsultationSignQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationSignQueryDTO queryDTO = new ConsultationSignQueryDTO();
        queryDTO.setConsultationId(params.getConsultationId());
        queryDTO.setDoctorId(user.getId());
        List<ConsultationSignVO> data = consultationBizService.signList(queryDTO);
        return new Result<List<ConsultationSignVO>>().ok(data);
    }

    @PostMapping("/ownerAdviceDetail")
    @ApiOperation(value = "自己的会诊意见详情", response = ConsultationAdviceVO.class)
    public Result<ConsultationAdviceVO> ownerAdviceDetail(@RequestBody ConsultationAdviceQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationAdviceQueryDTO queryDTO = new ConsultationAdviceQueryDTO();
        // queryDTO.setId(params.getId());
        queryDTO.setConsultationId(params.getConsultationId());
        queryDTO.setDoctorId(user.getId());
        ConsultationAdviceVO data = consultationBizService.ownerAdviceDetail(queryDTO);
        return new Result<ConsultationAdviceVO>().ok(data);
    }

    @PostMapping("/adviceList")
    @ApiOperation(value = "会诊意见列表", response = ConsultationAdviceVO.class)
    public Result<List<ConsultationAdviceVO>> adviceList(@RequestBody ConsultationAdviceQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationAdviceQueryDTO queryDTO = new ConsultationAdviceQueryDTO();
        queryDTO.setConsultationId(params.getConsultationId());
        queryDTO.setDoctorId(user.getId());
        List<ConsultationAdviceVO> data = consultationBizService.adviceList(queryDTO);
        return new Result<List<ConsultationAdviceVO>>().ok(data);
    }

    @PostMapping("/adviceDetail")
    @ApiOperation(value = "会诊意见详情", response = ConsultationAdviceVO.class)
    public Result<ConsultationAdviceVO> adviceDetail(@RequestBody ConsultationAdviceQueryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationAdviceQueryDTO queryDTO = new ConsultationAdviceQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setConsultationId(params.getConsultationId());
        queryDTO.setDoctorId(user.getId());
        ConsultationAdviceVO data = consultationBizService.adviceDetail(queryDTO);
        return new Result<ConsultationAdviceVO>().ok(data);
    }

    @PostMapping("/join")
    @ApiOperation(value = "加入会诊", response = Void.class)
    public Result<Void> join(@RequestBody ConsultationJoinOrRefuseReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationJoinOrRefuseDTO queryDTO = new ConsultationJoinOrRefuseDTO();
        queryDTO.setId(params.getId());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.join(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/refuse")
    @ApiOperation(value = "拒绝会诊", response = Void.class)
    public Result<Void> refuse(@RequestBody ConsultationJoinOrRefuseReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationJoinOrRefuseDTO queryDTO = new ConsultationJoinOrRefuseDTO();
        queryDTO.setId(params.getId());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.refuse(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/refuseAndAppoint")
    @ApiOperation(value = "拒绝会诊并指定会诊医生", response = Void.class)
    public Result<Void> refuseAndAppoint(@RequestBody ConsultationJoinOrRefuseReq params) {
        ValidatorUtils.validateEntity(params);
        if (StrUtil.isBlank(params.getAppointDoctorId())) {
            throw new BusinessException("未指定替补会诊医生");
        }
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationJoinOrRefuseDTO queryDTO = new ConsultationJoinOrRefuseDTO();
        queryDTO.setId(params.getId());
        queryDTO.setAppointDoctorId(params.getAppointDoctorId());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.refuseAndAppoint(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/sign")
    @ApiOperation(value = "会诊签到", response = Void.class)
    public Result<Void> sign(@RequestBody ConsultationSignReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationSignDTO queryDTO = new ConsultationSignDTO();
        queryDTO.setConsultationId(params.getId());
        queryDTO.setSignDoctorId(params.getSignDoctorId());
        queryDTO.setSignWay(params.getSignWay());
        queryDTO.setSignLatitude(params.getSignLatitude());
        queryDTO.setSignLongitude(params.getSignLongitude());
        queryDTO.setSignIp(IpUtils.getIpAddr());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.sign(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/unsign")
    @ApiOperation(value = "取消会诊签到", response = Void.class)
    public Result<Void> unsign(@RequestBody ConsultationSignReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationSignDTO queryDTO = new ConsultationSignDTO();
        queryDTO.setConsultationId(params.getId());
        queryDTO.setSignDoctorId(params.getSignDoctorId());
        queryDTO.setSignWay(params.getSignWay());
        queryDTO.setSignLatitude(params.getSignLatitude());
        queryDTO.setSignLongitude(params.getSignLongitude());
        queryDTO.setSignIp(IpUtils.getIpAddr());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.unsign(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/start")
    @ApiOperation(value = "会诊开始（也可以签到并开始）", response = Void.class)
    public Result<Void> start(@RequestBody ConsultationStartOrCancelReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationStartOrCancelDTO queryDTO = new ConsultationStartOrCancelDTO();
        queryDTO.setId(params.getId());
        queryDTO.setSignDoctorList(params.getSignDoctorList());
        queryDTO.setImageList(params.getImageList());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.start(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/cancel")
    @ApiOperation(value = "会诊取消", response = Void.class)
    public Result<Void> cancel(@RequestBody ConsultationStartOrCancelReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationStartOrCancelDTO queryDTO = new ConsultationStartOrCancelDTO();
        queryDTO.setId(params.getId());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.cancel(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/advice")
    @ApiOperation(value = "会诊意见", response = Void.class)
    public Result<Void> advice(@RequestBody ConsultationAdviceReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationAdviceDTO queryDTO = new ConsultationAdviceDTO();
        queryDTO.setConsultationId(params.getId());
        queryDTO.setAdvice(params.getAdvice());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.advice(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/summary")
    @ApiOperation(value = "会诊总结", response = Void.class)
    public Result<Void> summary(@RequestBody ConsultationSummaryReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationSummaryDTO queryDTO = new ConsultationSummaryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setSummary(params.getSummary());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.summary(queryDTO);
        return new Result<Void>().success();
    }

    @PostMapping("/finish")
    @ApiOperation(value = "会诊完结", response = Void.class)
    public Result<Void> finish(@RequestBody ConsultationFinishReq params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationFinishDTO queryDTO = new ConsultationFinishDTO();
        queryDTO.setId(params.getId());
        queryDTO.setDoctorId(user.getId());
        consultationBizService.finish(queryDTO);
        return new Result<Void>().success();
    }


    @PostMapping("/statusCountStatistic")
    @ApiOperation(value = "状态数量统计", response = ConsultationStatusCountStatisticVO.class)
    public Result<List<ConsultationStatusCountStatisticVO>> statusCountStatistic() {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<ConsultationStatusCountStatisticVO> data = consultationBizService.statusCountStatistic(user.getId());
        return new Result<List<ConsultationStatusCountStatisticVO>>().ok(data);
    }

}
