package com.xxh.cloud.healthy.mdt.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationApplyTimeOutStat;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.cache.ConsultationLaunchApplyCacheService;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationApplyTaskBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt/launch/apply")
@Api(tags = "医生发起会诊申请管理")
public class ConsultationLaunchApplyBizController {
    
    @Autowired
    private ConsultationLaunchApplyBizService consultationLaunchApplyBizService;
    @Autowired
    private ConsultationApplyTaskBizService consultationApplyTaskBizService;
    @Autowired
    private ConsultationLaunchApplyCacheService consultationLaunchApplyCacheService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = ConsultationLaunchApplyVO.class)
    public Result<PageData<ConsultationLaunchApplyVO>> query(@RequestBody ConsultationLaunchApplyQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyVO> data = consultationLaunchApplyBizService.page(params);
        return new Result<PageData<ConsultationLaunchApplyVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = ConsultationLaunchApplyVO.class)
    public Result<ConsultationLaunchApplyVO> detail(@RequestBody ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);
//        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        ConsultationLaunchApplyVO vo = consultationLaunchApplyBizService.detail(params);
        return new Result<ConsultationLaunchApplyVO>().ok(vo);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存", response = ConsultationLaunchApplyVO.class)
    public Result<ConsultationLaunchApplyVO> add(@RequestBody ConsultationLaunchApplyAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setApplyDoctorId(user.getId());
        ConsultationLaunchApplyVO vo = consultationLaunchApplyBizService.add(params);
        return new Result<ConsultationLaunchApplyVO>().ok(vo);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody ConsultationLaunchApplyUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setApplyDoctorId(user.getId());
        consultationLaunchApplyBizService.update(params);
        return new Result<Void>().success();
    }

    @PostMapping("/launch")
    @ApiOperation(value = "发起提醒", response = Void.class)
    public Result<Void> launch(@RequestBody ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setApplyDoctorId(user.getId());
        params.setOpUserId(user.getId());
        consultationLaunchApplyBizService.launch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/saveAndLaunch")
    @ApiOperation(value = "保存并发起提醒", response = Void.class)
    public Result<Void> launch(@RequestBody ConsultationLaunchApplyAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setApplyDoctorId(user.getId());
        params.setOpUserId(user.getId());
        consultationLaunchApplyBizService.saveAndLaunch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/resendLaunch")
    @ApiOperation(value = "重新发起提醒", response = Void.class)
    public Result<Void> resendLaunch(@RequestBody ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setResendFlg(YN.YES.getCode());
        params.setOpUserId(user.getId());
        consultationLaunchApplyBizService.resendLaunch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/getUrlKey")
    @ApiOperation(value = "刷新urlKey", response = ConsultationApplyTimeOutStat.class)
    public Result<String> consultationApplyTimeOutStat(@RequestBody Map<String, String> params) {
        return new Result<String>().ok(consultationLaunchApplyCacheService.getUrlKey());
    }

    @PostMapping("/createExamine")
    @ApiOperation(value = "发起申诉", response = ConsultationApplyTimeOutStat.class)
    public Result<String> createExamine(@RequestBody ConsultationApplyExamineCreateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUserId(user.getId());
        consultationLaunchApplyBizService.createExamine(params);
        return new Result<String>().success();
    }

    @PostMapping("/auditExamine")
    @ApiOperation(value = "审批申诉", response = ConsultationApplyTimeOutStat.class)
    public Result<String> auditExamine(@RequestBody ConsultationApplyExamineAuditDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setOpUserId(user.getId());
        consultationLaunchApplyBizService.auditExamine(params);
        return new Result<String>().success();
    }

}
