package com.xxh.cloud.healthy.mdt.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyChangeStatDetailVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyChangeStatExportVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyChangeStatVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyStatExportVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyStatVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyChangeStatBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyStatBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt/launch/apply/stat")
@Api(tags = "会诊申请统计管理")
public class ConsultationLaunchApplyStatBizController {
    
    @Autowired
    private ConsultationLaunchApplyStatBizService consultationLaunchApplyStatBizService;
    @Autowired
    private ConsultationLaunchApplyChangeStatBizService consultationLaunchApplyChangeStatBizService;

    @PostMapping("/statQuery")
    @ApiOperation(value = "统计分页", response = ConsultationLaunchApplyStatVO.class)
    public Result<PageData<ConsultationLaunchApplyStatVO>> statQuery(@RequestBody ConsultationLaunchApplyStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyStatVO> data = consultationLaunchApplyStatBizService.query(params);
        return new Result<PageData<ConsultationLaunchApplyStatVO>>().ok(data);
    }

    @PostMapping("/dept/statQuery")
    @ApiOperation(value = "根据科室统计分页", response = ConsultationLaunchApplyStatVO.class)
    public Result<PageData<ConsultationLaunchApplyStatVO>> statQueryByDept(@RequestBody ConsultationLaunchApplyStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyStatVO> data = consultationLaunchApplyStatBizService.queryByDept(params);
        return new Result<PageData<ConsultationLaunchApplyStatVO>>().ok(data);
    }

    @GetMapping("/dept/stat/export")
    @ApiOperation(value = "根据科室统计导出", response = Void.class)
    public void statExport(HttpServletResponse response, ConsultationLaunchApplyStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyStatVO> data = consultationLaunchApplyStatBizService.queryByDept(params);
        if (null == data || CollUtil.isEmpty(data.getList())) {
            return;
        }
        
        List<ConsultationLaunchApplyStatExportVO> list = new ArrayList<>();
        data.getList().forEach(i -> {
            ConsultationLaunchApplyStatExportVO export = new ConsultationLaunchApplyStatExportVO();
            export.setDeptId(i.getDeptId());
            export.setStartNum(i.getStartNum());
            export.setApplyNum(i.getApplyNum());
            export.setNotApplyNum(i.getNotApplyNum());
            export.setExamineNum(i.getExamineNum());
            export.setFinishNum(i.getFinishNum());
            export.setChangeNum(i.getChangeNum());
            export.setChangeNowNum(i.getChangeNowNum());
            export.setOutChangeNum(i.getOutChangeNum());
            export.setTherapyChangeNum(i.getTherapyChangeNum());
            export.setRealChangeNum(i.getRealChangeNum());
            export.setDeptName(i.getDeptName());

            list.add(export);
        });

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("deptId", "科室编号");
        writer.addHeaderAlias("deptName", "科室名称");
        writer.addHeaderAlias("startNum", "会诊提醒数量");
        writer.addHeaderAlias("applyNum", "已发起会诊数量");
        writer.addHeaderAlias("notApplyNum", "未发起会诊数量");
        writer.addHeaderAlias("examineNum", "申诉数量");
        writer.addHeaderAlias("finishNum", "已完成会诊数量");
        writer.addHeaderAlias("changeNum", "会诊建议转科数量");
        writer.addHeaderAlias("changeNowNum", "立即转科数量");
        writer.addHeaderAlias("outChangeNum", "出院后再入院数量");
        writer.addHeaderAlias("therapyChangeNum", "治疗后转科数量");
        writer.addHeaderAlias("realChangeNum", "实际转科数量");

        writer.write(list, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
        response.setHeader("Content-Disposition","attachment;filename=consultationApplyStat.xlsx"); 
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }
    }

    // 详情查询：/sys/doctor/mdt/manage/apply/query
    
    @PostMapping("/statTotal")
    @ApiOperation(value = "总量统计", response = ConsultationLaunchApplyStatVO.class)
    public Result<ConsultationLaunchApplyStatVO> statTotal(@RequestBody ConsultationLaunchApplyStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        ConsultationLaunchApplyStatVO data = consultationLaunchApplyStatBizService.statTotal(params);
        return new Result<ConsultationLaunchApplyStatVO>().ok(data);
    }


    @PostMapping("/changeStatQuery")
    @ApiOperation(value = "变更统计分页", response = ConsultationLaunchApplyChangeStatVO.class)
    public Result<PageData<ConsultationLaunchApplyChangeStatVO>> changeStatQuery(@RequestBody ConsultationLaunchApplyChangeStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyChangeStatVO> data = consultationLaunchApplyChangeStatBizService.query(params);
        return new Result<PageData<ConsultationLaunchApplyChangeStatVO>>().ok(data);
    }

    @PostMapping("/dept/changeStatQuery")
    @ApiOperation(value = "根据科室变更统计分页", response = ConsultationLaunchApplyChangeStatVO.class)
    public Result<PageData<ConsultationLaunchApplyChangeStatVO>> changeStatQueryByDept(@RequestBody ConsultationLaunchApplyChangeStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyChangeStatVO> data = consultationLaunchApplyChangeStatBizService.queryByDept(params);
        return new Result<PageData<ConsultationLaunchApplyChangeStatVO>>().ok(data);
    }

    @GetMapping("/dept/changeStat/export")
    @ApiOperation(value = "根据科室变更统计导出", response = Void.class)
    public void changeStatQueryByDept(HttpServletResponse response, ConsultationLaunchApplyChangeStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyChangeStatVO> data = consultationLaunchApplyChangeStatBizService.queryByDept(params);
        if (null == data || CollUtil.isEmpty(data.getList())) {
            return;
        }
        
        List<ConsultationLaunchApplyChangeStatExportVO> list = new ArrayList<>();
        data.getList().forEach(i -> {
            ConsultationLaunchApplyChangeStatExportVO export = new ConsultationLaunchApplyChangeStatExportVO();
            export.setDeptId(i.getDeptId());
            export.setDiseaseId(i.getDiseaseId());
            export.setFinishNum(i.getFinishNum());
            export.setChangeNum(i.getChangeNum());
            export.setChangeNowNum(i.getChangeNowNum());
            export.setOutChangeNum(i.getOutChangeNum());
            export.setTherapyChangeNum(i.getTherapyChangeNum());
            export.setRealChangeNum(i.getRealChangeNum());
            export.setDeptName(i.getDeptName());
            export.setDiseaseName(i.getDiseaseName());

            list.add(export);
        });

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("deptId", "科室编号");
        writer.addHeaderAlias("deptName", "科室名称");
        writer.addHeaderAlias("diseaseName", "病种名称");
        writer.addHeaderAlias("diseaseId", "病种ID");
        writer.addHeaderAlias("finishNum", "已完成会诊数量");
        writer.addHeaderAlias("changeNum", "会诊后建议转科数量");
        writer.addHeaderAlias("changeNowNum", "会诊后立即转科数量");
        writer.addHeaderAlias("outChangeNum", "出院后再入院数量");
        writer.addHeaderAlias("therapyChangeNum", "治疗后转科数量");
        writer.addHeaderAlias("realChangeNum", "总转科数量");

        writer.write(list, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
        response.setHeader("Content-Disposition","attachment;filename=consultationApplyChangeStat.xlsx"); 
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }
    }

    @PostMapping("/dept/changeStatDetail")
    @ApiOperation(value = "根据变更统计查询详情分页", response = ConsultationLaunchApplyChangeStatDetailVO.class)
    public Result<PageData<ConsultationLaunchApplyChangeStatDetailVO>> changeStatQueryByDetail(@RequestBody ConsultationLaunchApplyChangeStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        PageData<ConsultationLaunchApplyChangeStatDetailVO> data = consultationLaunchApplyChangeStatBizService.queryDetailByDept(params);
        return new Result<PageData<ConsultationLaunchApplyChangeStatDetailVO>>().ok(data);
    }

    @PostMapping("/changeStatTotal")
    @ApiOperation(value = "变更总量统计", response = ConsultationLaunchApplyChangeStatVO.class)
    public Result<ConsultationLaunchApplyChangeStatVO> changeStatTotal(@RequestBody ConsultationLaunchApplyChangeStatQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
//        params.setApplyDoctorId(user.getId());
        ConsultationLaunchApplyChangeStatVO data = consultationLaunchApplyChangeStatBizService.statTotal(params);
        return new Result<ConsultationLaunchApplyChangeStatVO>().ok(data);
    }

}
