package com.xxh.cloud.healthy.mdt.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationApplyStatGetDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationApplyStatSearchDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyListDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationApplyTimeOutStat;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationStatusStatDeptVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.cache.ConsultationLaunchApplyCacheService;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationApplyTaskBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@SaIgnore
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt/launch/stat")
@Api(tags = "会诊申请超时统计")
public class ConsultationLaunchApplyToStatBizController {
    
    @Autowired
    private ConsultationLaunchApplyBizService consultationLaunchApplyBizService;
    @Autowired
    private ConsultationLaunchApplyCacheService consultationLaunchApplyCacheService;

    @PostMapping("/listUndoing48H")
    @ApiOperation(value = "查询48小时未发起", response = ConsultationLaunchApplyVO.class)
    public Result<List<ConsultationLaunchApplyVO>> listUndoing48H(@RequestBody ConsultationApplyStatSearchDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationLaunchApplyVO>>().ok(consultationLaunchApplyBizService.listUndoing48H(params.getSearchDate()));
    }

    @PostMapping("/listUndone48H")
    @ApiOperation(value = "查询48小时未完成", response = ConsultationLaunchApplyVO.class)
    public Result<List<ConsultationLaunchApplyVO>> listUndone48H(@RequestBody ConsultationApplyStatSearchDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationLaunchApplyVO>>().ok(consultationLaunchApplyBizService.listUndone48H(params.getSearchDate()));
    }

    @PostMapping("/listUndone24H")
    @ApiOperation(value = "查询发起后24小时未完成", response = ConsultationLaunchApplyVO.class)
    public Result<List<ConsultationLaunchApplyVO>> listUndone24H(@RequestBody ConsultationApplyStatSearchDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationLaunchApplyVO>>().ok(consultationLaunchApplyBizService.listUndone24H(params.getSearchDate()));
    }

    @PostMapping("/searchTimeoutStat")
    @ApiOperation(value = "查询会诊超时统计数据", response = ConsultationApplyTimeOutStat.class)
    public Result<List<ConsultationApplyTimeOutStat>> searchTimeoutStat(@RequestBody ConsultationApplyStatGetDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationApplyTimeOutStat>>().ok(consultationLaunchApplyBizService.searchTimeoutStat(params.getDays()));
    }

    @PostMapping("/listApply48H")
    @ApiOperation(value = "查询48小时的会诊申请", response = ConsultationApplyTimeOutStat.class)
    public Result<List<ConsultationLaunchApplyVO>> listApply48H(@RequestBody ConsultationLaunchApplyListDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationLaunchApplyVO>>().ok(consultationLaunchApplyBizService.listApply48H(params));
    }

    @PostMapping("/statByDeptRelation")
    @ApiOperation(value = "查询48小时的会诊申请,按照部门分组", response = ConsultationApplyTimeOutStat.class)
    public Result<List<ConsultationStatusStatDeptVO>> statByDeptRelation(@RequestBody ConsultationApplyStatSearchDTO params) {
        ValidatorUtils.validateEntity(params);
        consultationLaunchApplyCacheService.checkUrlKey(params.getKey());
        return new Result<List<ConsultationStatusStatDeptVO>>().ok(consultationLaunchApplyBizService.statByDeptRelation(params.getSearchDate()));
    }

//    @Autowired
//    private ConsultationApplyTaskBizService consultationApplyTaskBizService;
//
//    @Deprecated
//    @PostMapping("/consultationApplyTimeOutStat")
//    @ApiOperation(value = "每日统计接口(测试)", response = ConsultationApplyTimeOutStat.class)
//    public Result<String> consultationApplyTimeOutStat(@RequestBody Map<String, String> params) {
//        ValidatorUtils.validateEntity(params);
//        return new Result<String>().ok(consultationApplyTaskBizService.consultationApplyTimeOutStat(params.get("date"), "N"));
//    }
}
