package com.xxh.cloud.healthy.mdt.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationGetDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationManageCountStatisticQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationGetReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationManageCountStatisticQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.req.ConsultationQueryReq;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyBizService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxl.job.core.util.DateUtil;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt/manage")
@Api(tags = "医生mdt管理")
public class ConsultationManageBizController {
    
    @Autowired
    private ConsultationBizService consultationBizService;
    @Autowired
    private ConsultationLaunchApplyBizService consultationLaunchApplyBizService;

    @PostMapping("/doctor/add")
    @ApiOperation(value = "添加会诊医生", response = ConsultationDoctorVO.class)
    public Result<ConsultationDoctorVO> addDoctorAndSendMsg(@RequestBody ConsultationDoctorAddDTO params) {
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        // params.setApplyDoctorId(user.getId());
        ConsultationDoctorVO vo = consultationBizService.addDoctorAndSendMsg(params);
        return new Result<ConsultationDoctorVO>().ok(vo);
    }

    @PostMapping("/query")
    @ApiOperation(value = "会诊分页列表", response = ConsultationVO.class)
    public Result<PageData<ConsultationVO>> query(@RequestBody ConsultationQueryReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationQueryDTO queryDTO = new ConsultationQueryDTO();
        queryDTO.setId(params.getId());
        queryDTO.setIds(params.getIds());
        queryDTO.setDiseaseId(params.getDiseaseId());
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());
        // queryDTO.setDoctorId(user.getId());
        queryDTO.setPatientName(params.getPatientName());
        queryDTO.setStatus(params.getStatus());
        queryDTO.setJoinStatus(params.getJoinStatus());
        queryDTO.setConsultationName(params.getConsultationName());
        
        queryDTO.setApplyType(params.getApplyType());
        // queryDTO.setApplyType(ConsultationType.MDT.getCode()); // 只查询mdt会诊
        PageData<ConsultationVO> data = consultationBizService.simplePage(queryDTO);
        return new Result<PageData<ConsultationVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "会诊详情", response = ConsultationVO.class)
    public Result<ConsultationVO> detail(@RequestBody ConsultationGetReq params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationGetDTO queryDTO = new ConsultationGetDTO();
        queryDTO.setId(params.getId());
        ConsultationVO data = consultationBizService.simpleDetail(queryDTO);
        return new Result<ConsultationVO>().ok(data);
    }



    // 对从海泰同步过来的会诊进行同步

    @PostMapping("/statistic/count")
    @ApiOperation(value = "聚合统计mdt数量", response = ConsultationManageCountStatisticVO.class)
    public Result<PageData<ConsultationManageCountStatisticVO>> statisticCountByGroup(@RequestBody ConsultationManageCountStatisticQueryReq req) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationManageCountStatisticQueryDTO queryDTO = new ConsultationManageCountStatisticQueryDTO();
        queryDTO.setPage(req.getPage());
        queryDTO.setRows(req.getRows());
        queryDTO.setConsultationName(req.getConsultationName());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        PageData<ConsultationManageCountStatisticVO> data = consultationBizService.statisticCountByGroup(queryDTO);
        return new Result<PageData<ConsultationManageCountStatisticVO>>().ok(data);
    }

    @PostMapping("/statistic/doctor")
    @ApiOperation(value = "聚合统计mdt医生", response = ConsultationManageCountStatisticVO.class)
    public Result<PageData<ConsultationManageCountStatisticVO>> statisticDoctorByGroup(@RequestBody ConsultationManageCountStatisticQueryReq req) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        ConsultationManageCountStatisticQueryDTO queryDTO = new ConsultationManageCountStatisticQueryDTO();
        queryDTO.setPage(req.getPage());
        queryDTO.setRows(req.getRows());
        queryDTO.setConsultationName(req.getConsultationName());
        queryDTO.setStartDate(req.getStartDate());
        queryDTO.setEndDate(req.getEndDate());
        PageData<ConsultationManageCountStatisticVO> data = consultationBizService.statisticDoctorByGroup(queryDTO);
        return new Result<PageData<ConsultationManageCountStatisticVO>>().ok(data);
    }



    // 对需要在海泰发起会诊的申请进行统计
    
    @PostMapping("/apply/query")
    @ApiOperation(value = "会诊申请分页查询", response = ConsultationLaunchApplyVO.class)
    public Result<PageData<ConsultationLaunchApplyVO>> applyPage(@RequestBody ConsultationLaunchApplyQueryDTO req) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // req.setStartDate(req.getStartDate());
        // req.setEndDate(req.getEndDate());
        req.setManageFlag(YN.YES.getCode());
        PageData<ConsultationLaunchApplyVO> data = consultationLaunchApplyBizService.page(req);
        return new Result<PageData<ConsultationLaunchApplyVO>>().ok(data);
    }

    @PostMapping("/apply/statistic")
    @ApiOperation(value = "会诊申请统计", response = ConsultationLaunchApplyStatisticVO.class)
    public Result<ConsultationLaunchApplyStatisticVO> applyStatistic(@RequestBody ConsultationLaunchApplyQueryDTO req) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // req.setStartDate(req.getStartDate());
        // req.setEndDate(req.getEndDate());
        req.setManageFlag(YN.YES.getCode());
        ConsultationLaunchApplyStatisticVO data = consultationLaunchApplyBizService.statistic(req);
        return new Result<ConsultationLaunchApplyStatisticVO>().ok(data);
    }

    @GetMapping("/apply/query/export")
    @ApiOperation(value = "会诊申请分页查询导出", response = void.class)
    public void applyPageExport(HttpServletResponse response, ConsultationLaunchApplyQueryDTO req) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // req.setStartDate(req.getStartDate());
        // req.setEndDate(req.getEndDate());
        req.setManageFlag(YN.YES.getCode());
        PageData<ConsultationLaunchApplyVO> data = consultationLaunchApplyBizService.page(req);
        if (null == data || CollUtil.isEmpty(data.getList())) {
            return;
        }

        List<ConsultationLaunchApplyExportVO> exportList = new ArrayList<>();
        data.getList().forEach(i -> {
            ConsultationLaunchApplyExportVO export = new ConsultationLaunchApplyExportVO();

            export.setRealStatus(i.getRealStatus());
            export.setStatus(i.getStatus());
            if (ConsultationLaunchApplyConstant.ApplyStatus.START.getCode().equals(i.getStatus())) {
                export.setStatus("等待发起会诊申请");
            } else if (ConsultationLaunchApplyConstant.ApplyStatus.START_EXPIRE.getCode().equals(i.getStatus()) || ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDOING.getCode().equals(i.getStatus())) {
                export.setStatus("超时未发起会诊申请");
            } else if (ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(i.getStatus())) {
                export.setStatus("已发起会诊申请，会诊未完成");
            } else if (ConsultationLaunchApplyConstant.ApplyStatus.EXPIRE.getCode().equals(i.getStatus()) || ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode().equals(i.getStatus())) {
                export.setStatus("超时未完成会诊");
            } else if (ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(i.getStatus())) {
                export.setStatus("已完成会诊");
            }

            export.setInpatientId(i.getInpatientId());
            PatientClientBO patient = i.getPatientInfo();
            if (null != patient) {
                export.setPatientId(patient.getPatientId());
                export.setPatientName(patient.getPatientName());
                export.setPatientGender(patient.getPatientGenderStandard());
                if ("W".equals(patient.getPatientGenderStandard())) {
                    export.setPatientGender("女");
                } else if ("M".equals(patient.getPatientGenderStandard())) {
                    export.setPatientGender("男");
                }
                export.setPatientAge(patient.getAge());
            }
            export.setContent(i.getContent());
            
            export.setType(i.getType());
            if (ConsultationLaunchApplyConstant.Type.Task.getCode().equals(i.getType())) {
                export.setType("AI自动提醒会诊");
            } else {
                export.setType("运营助理申请会诊");
            }

            AdminUserBO applyDoctor = i.getApplyDoctorInfo();
            if (null != applyDoctor) {
                export.setApplyDoctorName(applyDoctor.getRealName());
            }
            
            DeptBO applyDept = i.getApplyDeptInfo();
            if (null != applyDept) {
                export.setApplyDeptName(applyDept.getName());
            }

            Date applyTime = i.getApplyTime();
            if (null != applyTime) {
                export.setApplyTime(DateUtil.formatDateTime(applyTime));
            }

            AdminUserBO launchDoctor = i.getLaunchDoctorInfo();
            if (null != launchDoctor) {
                export.setLaunchDoctorName(launchDoctor.getRealName());
            }

            DeptBO launchDept = i.getLaunchDeptInfo();
            if (null != launchDept) {
                export.setLaunchDeptName(launchDept.getName());
            }

            StandardDiseaseBO disease = i.getDiseaseInfo();
            if (null != disease) {
                export.setDiseaseName(disease.getName());
            }

            List<ConsultationDoctorVO> consultationDoctorList = i.getConsultationDoctorList();
            if (CollUtil.isNotEmpty(consultationDoctorList)) {
                String consultationDeptName = "";
                for (ConsultationDoctorVO consultationDoctor : consultationDoctorList) {
                    consultationDeptName = consultationDeptName + consultationDoctor.getDeptName() + consultationDoctor.getDoctorName() + ",";
                }
                export.setConsultationDeptName(consultationDeptName.substring(0, consultationDeptName.length() - 1));
            }

            Date submitTime = i.getSubmitTime();
            if (null != submitTime) {
                export.setSubmitTime(DateUtil.formatDateTime(submitTime));
            }

            Date finishTime = i.getFinishTime();
            if (null != finishTime) {
                export.setFinishTime(DateUtil.formatDateTime(finishTime));
            }

            ConsultationVO consultation = i.getConsultationInfo();
            if (null != consultation) {
                export.setConsultationContent(consultation.getRemark());

                List<HisConsultationAdviceVO> consultationAdviceList = consultation.getHisConsultationAdviceList();
                if (CollUtil.isNotEmpty(consultationAdviceList)) {
                    String advice = "";
                    for (HisConsultationAdviceVO consultationAdvice : consultationAdviceList) {
                        advice = advice + consultationAdvice.getSubmitContent() + ",";
                    }
                    export.setConsultationAdvice(advice.substring(0, advice.length() - 1));
                }
            }

            // 督办信息
            export.setResendFlg(YN.YES.getCode().equals(i.getResendFlg()) ? "是" : "否");
            if (CollUtil.isNotEmpty(i.getApplyLogs())) {
                for (int index = 0; index < i.getApplyLogs().size(); index++) {
                    ConsultationLaunchApplyLogVO applyLogs = i.getApplyLogs().get(index);
                    if (index == 0) {
                        export.setResendDate1(DateUtil.formatDateTime(applyLogs.getOpDate()));
                        export.setResendUser1(applyLogs.getOpUserName());
                    }
                    if (index == 1) {
                        export.setResendDate2(DateUtil.formatDateTime(applyLogs.getOpDate()));
                        export.setResendUser2(applyLogs.getOpUserName());
                    }
                }
            }
            exportList.add(export);
        });

        if (CollUtil.isEmpty(exportList)) {
            return;
        }

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("realStatus", "实际状态");
        writer.addHeaderAlias("status", "状态");
        writer.addHeaderAlias("inpatientId", "住院/就诊号");
        writer.addHeaderAlias("patientId", "患者编号");
        writer.addHeaderAlias("patientName", "患者姓名");
        writer.addHeaderAlias("patientGender", "患者性别");
        writer.addHeaderAlias("patientAge", "患者年龄");
        writer.addHeaderAlias("diseaseName", "病种");
        writer.addHeaderAlias("content", "申请内容");
        writer.addHeaderAlias("type", "会诊发起来源");
        writer.addHeaderAlias("launchDoctorName", "发起会诊申请医生");
        writer.addHeaderAlias("launchDeptName", "发起会诊申请科室");
        writer.addHeaderAlias("consultationDeptName", "参与协助会诊科室");
        writer.addHeaderAlias("applyDoctorName", "运营助理");
        writer.addHeaderAlias("applyDeptName", "运营助理科室");
        writer.addHeaderAlias("applyTime", "助理申请时间");
        writer.addHeaderAlias("submitTime", "海泰会诊发起时间");
        writer.addHeaderAlias("finishTime", "会诊完成时间");
        writer.addHeaderAlias("consultationContent", "会诊申请内容");
        writer.addHeaderAlias("consultationAdvice", "会诊结论");

        // 增加督办相关的内容
        writer.addHeaderAlias("resendFlg", "是否督办");
        writer.addHeaderAlias("resendDate1", "第一次督办时间");
        writer.addHeaderAlias("resendUser1", "第二次督办人员");
        writer.addHeaderAlias("resendDate2", "第一次督办时间");
        writer.addHeaderAlias("resendUser2", "第二次督办人员");

        writer.write(exportList, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
        response.setHeader("Content-Disposition","attachment;filename=consultation.xlsx"); 
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }
    }

    @PostMapping("/advice/query")
    @ApiOperation(value = "会诊结论分页列表", response = ConsultationVO.class)
    public Result<PageData<ConsultationVO>> adviceQuery(@RequestBody HisConsultationAdviceQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<ConsultationVO> data = consultationBizService.pageFull(params);
        return new Result<PageData<ConsultationVO>>().ok(data);
    }

    @GetMapping("/advice/query/export")
    @ApiOperation(value = "会诊结论分页列表导出excel", response = Void.class)
    public void labelPatientPageExport(HttpServletResponse response, HisConsultationAdviceQueryDTO params) {
        log.info("/sys/doctor/mdt/manage/advice/query/export >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();

        PageData<ConsultationVO> data = consultationBizService.pageFull(params);

        List<ConsultationExportVO> exportList = new ArrayList<>();
        data.getList().forEach(i -> {
            ConsultationExportVO exportVO = new ConsultationExportVO();

            // 患者基本信息
            exportVO.setPatientName(i.getPatientName());
            exportVO.setPatientPhone(i.getPatientPhone());
            exportVO.setPatientAge( i.getPatientAge());
            // exportVO.setDeathStatus("N".equals(i.getDeathStatus()) ? "已死亡" : "");
            exportVO.setPatientSex(i.getPatientSex());
            if ("M".equals(i.getPatientSex())) {
                exportVO.setPatientSex("男");
            } else if ("W".equals(i.getPatientSex())) {
                exportVO.setPatientSex("女");
            }

            exportVO.setPatientId(i.getPatientId());

            // exportVO.setApplyType(i.getApplyType());
            // for (ConsultationConstant.ConsultationType applyType : ConsultationConstant.ConsultationType.values()) {
            //     if (applyType.getCode().equals(i.getApplyType())) {
            //         exportVO.setApplyType(applyType.getMsg());
            //     }
            // }
            
            // exportVO.setId(i.getId());
            // exportVO.setName(i.getName());
            exportVO.setRemark(i.getRemark());
            exportVO.setInpatientId(i.getInpatientId());
            exportVO.setVisitId(i.getVisitId());
            exportVO.setApplyTime(i.getApplyTime());
            // exportVO.setApplyDoctorId(i.getApplyDoctorId());
            exportVO.setApplyDoctorName(i.getApplyDoctorName());
            // exportVO.setApplyDeptId(i.getApplyDeptId());
            exportVO.setApplyDeptName(i.getApplyDeptName());

            List<HisConsultationAdviceVO> adviceVOs = i.getHisConsultationAdviceList();
            if (CollUtil.isNotEmpty(adviceVOs)) {
                HisConsultationAdviceVO adviceVO = adviceVOs.get(0);
                // exportVO.setMdtDeptCode(adviceVO.getMdtDeptCode());
                exportVO.setMdtDeptName(adviceVO.getMdtDeptName());
                // exportVO.setSubmitDoctorId(adviceVO.getSubmitDoctorId());
                exportVO.setSubmitDoctorName(adviceVO.getSubmitDoctorName());
                exportVO.setSubmitTime(adviceVO.getSubmitTime());
                exportVO.setSubmitContent(adviceVO.getSubmitContent());

                for (ConsultationConstant.ConsultationTargetType targetType : ConsultationConstant.ConsultationTargetType.values()) {
                    if (targetType.getCode().equals(adviceVO.getTargetType())) {
                        exportVO.setTargetType(targetType.getMsg());
                    }
                }

                // exportVO.setTarget(adviceVO.getTarget());
                exportVO.setTargetStatus(adviceVO.getTargetStatus());
                exportVO.setTargetFinishTime(adviceVO.getTargetFinishTime());
            }
            exportList.add(exportVO);
        });

        if (CollUtil.isEmpty(exportList)) {
            return;
        }

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        // writer.addHeaderAlias("id", "会诊编号");
        // writer.addHeaderAlias("applyType", "会诊类型");
        writer.addHeaderAlias("patientPhone", "患者手机号");
        writer.addHeaderAlias("patientName", "患者姓名");
        writer.addHeaderAlias("patientAge", "患者年龄");
        // writer.addHeaderAlias("deathStatus", "是否死亡");
        writer.addHeaderAlias("patientSex", "患者性别");
        writer.addHeaderAlias("patientId", "患者编号");
        writer.addHeaderAlias("inpatientId", "患者住院号");
        writer.addHeaderAlias("visitId", "患者就诊号");

        // writer.addHeaderAlias("name", "会诊名称");
        writer.addHeaderAlias("remark", "会诊申请内容");
        writer.addHeaderAlias("applyTime", "会诊申请时间");
        // writer.addHeaderAlias("applyDoctorId", "会诊申请医生编号");
        writer.addHeaderAlias("applyDoctorName", "会诊申请医生姓名");
        // writer.addHeaderAlias("applyDeptId", "申请科室编号");
        writer.addHeaderAlias("applyDeptName", "申请科室名称");

        // writer.addHeaderAlias("mdtDeptCode", "会诊科室编号");
        writer.addHeaderAlias("mdtDeptName", "会诊科室名称");
        // writer.addHeaderAlias("submitDoctorId", "会诊医生编号");
        writer.addHeaderAlias("submitDoctorName", "会诊医生姓名");

        writer.addHeaderAlias("submitTime", "会诊结论时间");
        writer.addHeaderAlias("submitContent", "会诊结论内容");
        writer.addHeaderAlias("targetType", "会诊目标类型");
        // writer.addHeaderAlias("target", "会诊目标解析数据");
        writer.addHeaderAlias("targetStatus", "会诊目标状态");
        writer.addHeaderAlias("targetFinishTime", "会诊目标完成时间");

        writer.write(exportList, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
        response.setHeader("Content-Disposition","attachment;filename=consultationTarget.xlsx"); 
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }
    }

}
