package com.xxh.cloud.healthy.mdt.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceSyncDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationSyncDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.HisConsultationAdviceVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.HisConsultationAdviceBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/mdt/his")
@Api(tags = "医生同步会诊意见管理")
public class HisConsultationBizController {
    
    @Autowired
    private HisConsultationAdviceBizService hisConsultationAdviceBizService;
    @Autowired
    private ConsultationBizService consultationBizService;
    @Autowired
    private ConsultationMapper consultationMapper;

    @PostMapping("/list")
    @ApiOperation(value = "列表", response = HisConsultationAdviceVO.class)
    public Result<List<HisConsultationAdviceVO>> query(@RequestBody HisConsultationAdviceQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<HisConsultationAdviceVO> data = hisConsultationAdviceBizService.list(params);
        return new Result<List<HisConsultationAdviceVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = HisConsultationAdviceVO.class)
    public Result<HisConsultationAdviceVO> detail(@RequestBody HisConsultationAdviceQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        HisConsultationAdviceVO vo = hisConsultationAdviceBizService.detail(params);
        return new Result<HisConsultationAdviceVO>().ok(vo);
    }

    @PostMapping("/syncConsultation")
    @ApiOperation(value = "同步会诊", response = Void.class)
    public Result<Void> syncConsultation(@RequestBody HisConsultationSyncDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        consultationBizService.syncConsultation(params);
        return new Result<Void>().success();
    }

    @PostMapping("/syncConsultation/multi")
    @ApiOperation(value = "大量同步会诊记录", response = Void.class)
    public Result<Void> syncHistoryConsultation(@RequestBody HisConsultationSyncDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();

        Date finishDate = params.getEndDate();
        Integer during = params.getDuring();
        if (null == during || during <= 0) {
            during = 12;
        }
        
        // 每次循时间
        Date startDate = params.getStartDate();
        Date endDate = startDate;
        while (true) {
            endDate = DateUtil.offsetHour(endDate, during);
            if (endDate.equals(finishDate)) {
                break;
            }
            if (endDate.after(finishDate)) {
                endDate = finishDate;
            }

            final Date start = startDate; // 为防止startDate或endDate变化导致数据出错
            final Date end = endDate;
            HisConsultationSyncDTO sync = new HisConsultationSyncDTO();
            sync.setApplyType(params.getApplyType());
            sync.setStartDate(start);
            sync.setEndDate(end);
            log.info(JSON.toJSONString(sync));
            consultationBizService.syncConsultation(sync);

            startDate = DateUtil.offsetHour(startDate, during);
            if (startDate.after(finishDate)) {
                break;
            }
        }

        return new Result<Void>().success();
    }

    @PostMapping("/syncConsultationAdvice")
    @ApiOperation(value = "同步会诊意见", response = Void.class)
    public Result<Void> syncConsultationAdvice(@RequestBody HisConsultationAdviceSyncDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        hisConsultationAdviceBizService.syncConsultationAdvice(params);
        return new Result<Void>().success();
    }

    @PostMapping("/syncConsultationAdvice/multi")
    @ApiOperation(value = "大量同步会诊意见", response = Void.class)
    public Result<Void> syncHistoryConsultationAdvice(@RequestBody HisConsultationAdviceSyncDTO params) {
        Integer pages = 1;
        while (true) {
            IPage<ConsultationInfo> page = consultationMapper.hisConsultationAdvicePage(new Page<>(pages, 5000), new QueryWrapper<ConsultationInfo>()
                    .eq("a.rows_status", YN.YES.getCode())
                    .isNull("b.id")
                    .ge(null != params.getStartDate(), "a.apply_time", params.getStartDate())
                    .lt(null != params.getEndDate(), "a.apply_time", params.getEndDate())
                    .orderByDesc("a.apply_time")
            );

            if (CollectionUtil.isEmpty(page.getRecords())) {
                break;
            }

            List<String> consultationIds = new ArrayList<>();
            page.getRecords().forEach(i -> consultationIds.add(i.getId()));

            HisConsultationAdviceSyncDTO sync = new HisConsultationAdviceSyncDTO();
            sync.setConsultationIds(consultationIds);
            log.info(JSON.toJSONString(sync));
            hisConsultationAdviceBizService.syncConsultationAdvice(sync);

            pages = pages + 1;
        }

        return new Result<Void>().success();
    }

}
