package com.xxh.cloud.healthy.mdt.framework.modules.analysis;


import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;

import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Map;

import org.springframework.stereotype.Component;

// 会诊结论目标解析
@Slf4j
@Component
public class HAnalysisApiHelper {

    // 会诊结论目标解析URL
    private static String analysisUrl = "http://172.18.84.102:8081/api/llm/chat/chat"; // TODO

    // 获取会诊结论目标解析数据
    public static HAnalysisDataModel getAnalysisData(String content) {
        Map<String, String> reqMap = new HashMap<>();
        reqMap.put("conversation_id", "");
        reqMap.put("history_len", "-1");
        reqMap.put("stream", "false");
        reqMap.put("model_name", "Qwen-14B-Chat");
        reqMap.put("temperature", "0.7");
        reqMap.put("prompt_name", "consultation_result");
        reqMap.put("platform", "3");
        reqMap.put("deviceNo", "");
        reqMap.put("version", "1.0.0");
        reqMap.put("query", content);

        String body = JSON.toJSONString(reqMap);
        log.info("会诊结论目标解析参数 >>>>>>：{}", body);

        String result = loopPostHttp(analysisUrl, body, 0);
        if (StrUtil.isBlank(result)) {
            return null;
        }

        int num = 0;
        while (result.contains("ping") && num < 5) {
            num++;
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e1) {
                e1.printStackTrace();
            }
        }

        log.info("会诊结论目标解析结果 <<<< {}", result);

        HAnalysisDataModel dataModel = new HAnalysisDataModel();
        dataModel.setSourceData(result);

        if (StrUtil.isNotBlank(result)) {
            try {
                JSONObject jsonObject = JSON.parseObject(result.replace("data:", ""));
                String text = jsonObject.getString("text");

                if (StrUtil.isNotBlank(text)) {
                    text = text.replace("```json", "").replace("```", "");

                    Map<String, String> map = JSONObject.parseObject(text, new TypeReference<Map<String, String>>(){});
                    
                    dataModel.setIsFollowUp("N");
                    String isFollowUp = map.get("是否随访");
                    if ("是".equals(isFollowUp)) {
                        dataModel.setIsFollowUp("Y");
                    }

                    dataModel.setFollowUpDate(map.get("随访时间"));
                    dataModel.setFollowUpDateUnit(map.get("随访时间单位"));

                    dataModel.setIsChangeDept("N");
                    String isChangeDept = map.get("是否转科室治疗");
                    if ("是".equals(isChangeDept)) {
                        dataModel.setIsChangeDept("Y");
                    }

                    dataModel.setTreatmentMethods(map.get("治疗的方式"));
                    dataModel.setCheckContent(map.get("建议完善检查内容"));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return dataModel;
    }

    // 调用失败则暂停5秒重新请求，最多重调5次
    private static String loopPostHttp(String url, String body, Integer loopNum) {
        try {
            String result = HttpRequest.post(url)
                .body(body)
                .execute().body();
    
            return result;
        } catch (Exception e) {
            log.info("POST调用异常: {} ", url);
            e.printStackTrace();
            
            // 最多循环5次
            if (loopNum < 5) {
                // 先暂停
                log.info("休眠5000ms");
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                // 出现异常再次调用
                return loopPostHttp(url, body, loopNum + 1);
            }
        }

        return null;
    }

    public static void main(String[] args) {
        // getAnalysisData("病史敬悉，简要病史：患者周爱侠，女，58岁，因反复胸闷1周入院；患者时有咳嗽咳痰，痰中带鲜血，近3月偶发，间断出现，每次量不多，咳痰后喉部疼痛不适，平素伴有口干、口苦，偶有反酸嗳气；无鼻腔出血，大便颜色正常。查体：咽部粘膜光滑，双侧扁桃体不大，无充血溃烂。辅助检查：胸部CT提示：两肺少许炎症、右肺上叶后段结节（低危），与前片（2022-08-12）比较无明显变化，随诊；左肺上叶钙化灶。考虑诊断：反流性咽喉炎建议：1.同意贵科诊治2.完善电子鼻咽喉镜检查排除鼻咽部及咽喉部病变等，必要时消化科会诊，行胃镜检查排除消化道出血，3.可予以PPI抑酸保胃、金嗓利咽胶囊等药物治疗；4.我科随访，1周随访一次。");
        // getAnalysisData("病史敬悉，简要病史：患者马再荣，男，76岁，因反复咳痰喘6年余，再发加重伴双下肢无力3天入院，体格检查：神清，精神一般，营养与发育中等，呼吸稍促，轻度喘息貌，步入病房，查体合作，对答切题，全身皮肤粘膜无黄染，浅表淋巴结未触及肿大，头颅无畸形，双瞳孔等大等圆，直径3.0ｍｍ，对光反射灵敏，耳鼻无异常分泌物，口唇无紫绀，咽无充血，颈软，气管居中，甲状腺无肿大，桶状胸，触诊语颤减弱，叩诊过清音，双肺呼吸音低，可闻及少许干湿罗音及哮鸣音，心前区无隆起，心尖搏动无弥散，心脏相对浊音界无扩大，心律齐，各瓣膜区未闻及明显杂音，心音剑突下响；腹软，无压痛、反跳痛，肝脾肋下未触及，叩诊呈鼓音，肠鸣音无亢进，约3-5次/分；脊柱及四肢无畸形，未见杵状指，活动自如，双下肢无水肿，生理反射存在，病理反射未引出，双侧巴士征阴性；布、克氏征（-）；辅助检查：2024-09-26128层胸部HRCT（薄层）慢性支气管炎伴少许感染、肺气肿、肺大泡形成纵隔多发淋巴结显示冠脉钙化；心包积液两侧胸膜局限性增厚肝脏密度不均匀减低，建议增强检查。腹部CT提示肝脏多发转移瘤。考虑诊断：肝转移性癌，慢性支气管炎急性发作肺气肿胆囊结石陈旧性脑梗死高血压病3级（极高危）肺大疱三尖瓣返流（轻中度）建议：1.同意在本科室继续诊治2.目前不需要做检验检查；3.建议MDT会诊；4.我科不随访。");
        // getAnalysisData("病史敬悉，简要病史：患者周宗树，男，70岁，因间断咳痰喘9月余，再发十余天入院。患者四肢水肿、胸闷、气喘，入院后查心电图示：“1.窦性心律2.ST-T改变”，BNP示：“1614pg/ml”，考虑心衰考虑诊断：1.肺部感染2.肝硬化失代偿期3.酒精性肝炎4.腹腔积液5.2型糖尿病6.原发性高血压7.左肺占位（上叶）8.右肺结节（中叶）9.脑梗死恢复期10.髋关节置换术后（双侧股骨头）11.心功能衰竭建议：1.同意在贵科治疗2.改善心功能，肾功能治疗；3.给予xxx治疗等（包括药物、手术/操作等）/同意贵科诊治；4.我科不随访。");
        // getAnalysisData("病史敬悉，简要病史：患者系脑干出血术后两月余；近期反复发热。考虑诊断：脑干出血术后，发热待查建议：1.同意贵科诊治；2.建议完善腰椎穿刺化验脑脊液排除颅内感染；3.予以抗感染对症治疗，若患者明确颅内感染，可留置腰大池引流；4.我科不随访。");
        getAnalysisData("病史敬悉，简要病史：双侧耳鸣听力下降10天，无耳流水；耳鸣呈持续性嗡嗡声；无视物旋转。查体：双侧外耳道畅，鼓膜完整无充血，光锥存在考虑诊断：感音神经性聋建议：1.完善听力检查2.必要时转入进一步治疗；3.-；4.我科不随访。");
    }


    

}


