package com.xxh.cloud.healthy.mdt.framework.modules.async;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant.ConsultationTargetType;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.HisConsultationAdviceInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.HisConsultationAdviceInfoService;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
@Async("consultationTargetExecutor")
public class ConsultationAsyncBizServiceImpl implements ConsultationAsyncBizService {

    private static final String analysisUrl = "/consultation/advice/getAnalysisData";

    @Autowired
    private HisConsultationAdviceInfoService hisConsultationAdviceInfoService;
    
    @Override
    public void asyncAnalysisData(String host, HisConsultationAdviceInfo entity) {
        HisConsultationAdviceInfo hisConsultationAdviceInfo = new HisConsultationAdviceInfo();
        hisConsultationAdviceInfo.setId(entity.getId());
        hisConsultationAdviceInfo.setTargetType(ConsultationTargetType.NONE.getCode());

        try {
            String result = loopPostHttp(host + analysisUrl, entity.getSubmitContent(), 0);

            if (StrUtil.isNotBlank(result)) {
                Result<AnalysisDataModel> sendResult = JSONObject.parseObject(result, new TypeReference<Result<AnalysisDataModel>>(){});
                hisConsultationAdviceInfo.setTarget(sendResult.getMsg());
                
                AnalysisDataModel analysisDataModel = sendResult.getData();
                if (null != analysisDataModel) {
                    if (YN.YES.getCode().equals(analysisDataModel.getIsFollowUp())) {
                        hisConsultationAdviceInfo.setTargetType(ConsultationTargetType.Follow_Up.getCode());
                    } else if (YN.YES.getCode().equals(analysisDataModel.getIsChangeDept())) {
                        hisConsultationAdviceInfo.setTargetType(ConsultationTargetType.Change_Dept.getCode());
                    } else {
                        hisConsultationAdviceInfo.setTargetType(ConsultationTargetType.NONE.getCode());
                    }
                    hisConsultationAdviceInfo.setTarget(analysisDataModel.getSourceData());
                }
            }
        } catch (Exception e) {
            log.error("解析会诊结论目标失败: message={}", e.getMessage());
            e.printStackTrace();
        }

        hisConsultationAdviceInfoService.updateById(hisConsultationAdviceInfo);
    }
    
    private static String loopPostHttp(String url, String body, Integer loopNum) {
        Map<String, String> mapContent = new HashMap<>();
        mapContent.put("content", body);
        try {
            String result = HttpRequest.post(url)
                .body(JSON.toJSONString(mapContent))
                .execute().body();
    
            log.info("发送结果：{}", result);
            return result;
        } catch (Exception e) {
            log.info("POST调用异常: {} ", url);
            e.printStackTrace();
            
            // 最多循环3次
            if (loopNum < 3) {
                // 先暂停
                log.info("休眠3000ms");
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                // 出现异常再次调用
                return loopPostHttp(url, body, loopNum + 1);
            }
        }

        return null;
    }

    public static void main(String[] args) {
        HisConsultationAdviceInfo hisConsultationAdviceInfo = new HisConsultationAdviceInfo();
        hisConsultationAdviceInfo.setSubmitContent("病史敬悉，简要病史：患者周爱侠，女，58岁，因反复胸闷1周入院；患者时有咳嗽咳痰，痰中带鲜血，近3月偶发，间断出现，每次量不多，咳痰后喉部疼痛不适，平素伴有口干、口苦，偶有反酸嗳气；无鼻腔出血，大便颜色正常。查体：咽部粘膜光滑，双侧扁桃体不大，无充血溃烂。辅助检查：胸部CT提示：两肺少许炎症、右肺上叶后段结节（低危），与前片（2022-08-12）比较无明显变化，随诊；左肺上叶钙化灶。考虑诊断：反流性咽喉炎建议：1.同意贵科诊治2.完善电子鼻咽喉镜检查排除鼻咽部及咽喉部病变等，必要时消化科会诊，行胃镜检查排除消化道出血，3.可予以PPI抑酸保胃、金嗓利咽胶囊等药物治疗；4.我科随访，1周随访一次。");


        ConsultationAsyncBizServiceImpl impl = new ConsultationAsyncBizServiceImpl();

        impl.asyncAnalysisData("localhost:8081", hisConsultationAdviceInfo);

    }
}
