package com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.entity.BaseEntity;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * 发起会诊申请信息表
 */
@Getter
@Setter
public class ConsultationLaunchApplyVO extends BaseEntity {
    
    /**
     * id
     */
     @ApiModelProperty(value = "id")
      private String id;

    /**
     * 会诊类型
     */
     @ApiModelProperty(value = "会诊类型")
      private String consultationType;

    /**
     * 患者编号
     */
     @ApiModelProperty(value = "患者编号")
      private String patientId;

    /**
     * 住院号
     */
     @ApiModelProperty(value = "住院号")
      private String inpatientId;

    /**
     * 门诊号
     */
     @ApiModelProperty(value = "门诊号")
      private String visitId;

    /**
     * 相关疾病ID
     */
     @ApiModelProperty(value = "相关疾病ID")
      private String diseaseId;

    /**
     * 申请负责医生ID
     */
     @ApiModelProperty(value = "申请医生ID")
      private String applyDoctorId;

    /**
     * 发起会诊医生ID
     */
     @ApiModelProperty(value = "发起会诊医生ID")
      private String launchDoctorId;

    /**
     * 申请发起会诊内容
     */
     @ApiModelProperty(value = "申请发起会诊内容")
      private String content;

    /**
     * 实际发起申请会诊提醒的时间
     */
     @ApiModelProperty(value = "实际发起申请会诊提醒的时间")
      private Date applyTime;

    /**
     * 实际发起的会诊id
     */
     @ApiModelProperty(value = "实际发起的会诊id")
      private String consultationId;

    /**
     * 实际发起会诊的时间
     */
     @ApiModelProperty(value = "实际发起会诊的时间")
      private Date submitTime;

    /**
     * 实际会诊完成的时间
     */
     @ApiModelProperty(value = "实际会诊完成的时间")
      private Date finishTime;

    /**
     * 状态
     */
     @ApiModelProperty(value = "状态")
      private String status;

    /**
     * 管理id
     */
     @ApiModelProperty(value = "管理id")
      private String manageId;

    /**
     * 申请类型[ConsultationLaunchApplyConstant.Type]
     */
     @ApiModelProperty(value = "申请类型[ConsultationLaunchApplyConstant.Type]")
      private String type;

    /**
     * 会诊标签id
     */
     @ApiModelProperty(value = "会诊标签id")
      private String consultationLabelId;

    /**
     * 申请会诊医生信息
     */
     @ApiModelProperty(value = "申请会诊医生信息")
      private AdminUserBO applyDoctorInfo;

    /**
     * 申请会诊医生部门信息
     */
     @ApiModelProperty(value = "申请会诊医生部门信息")
      private DeptBO applyDeptInfo;

    /**
     * 发起会诊医生信息
     */
     @ApiModelProperty(value = "发起会诊医生信息")
      private AdminUserBO launchDoctorInfo;

    /**
     * 发起会诊医生部门信息
     */
     @ApiModelProperty(value = "发起会诊医生部门信息")
      private DeptBO launchDeptInfo;

    /**
     * 患者信息
     */
     @ApiModelProperty(value = "患者信息")
      private PatientClientBO patientInfo;

    /**
     * 病种信息
     */
     @ApiModelProperty(value = "病种信息")
      private StandardDiseaseBO diseaseInfo;

    /**
     * 会诊标签信息
     */
     @ApiModelProperty(value = "会诊标签信息")
      private LabelBO consultationLabelInfo;

    /**
     * 会诊信息
     */
     @ApiModelProperty(value = "会诊信息")
      private ConsultationVO consultationInfo;


    @ApiModelProperty(value = "参加会诊部门")
    private List<String> joinDept;

    /**
     * 会诊科室信息
     */
     @ApiModelProperty(value = "会诊科室信息")
      private List<DeptBO> consultationDeptInfo;

    /**
     * 会诊医生信息
     */
     @ApiModelProperty(value = "会诊医生信息")
      private List<ConsultationDoctorVO> consultationDoctorList;

    @ApiModelProperty(value = "是否需要重发")
    private String resendEnable;

    @ApiModelProperty(value = "重发标识")
    private String resendFlg;

    @ApiModelProperty(value = "重发次数")
    private Integer resendCount;

    @ApiModelProperty(value = "重发时间")
    private Date resendDate;

    @ApiModelProperty(value = "发送日志")
    private List<ConsultationLaunchApplyLogVO> applyLogs;

    @ApiModelProperty(value = "入住时间")
    private Date inpatientDate;

    @ApiModelProperty(value = "是否出院")
    private String isOut;

    @ApiModelProperty(value = "出院时间")
    private Date outDate;

    @ApiModelProperty(value = "实际状态")
    private String realStatus;

    // 申诉内容
    @ApiModelProperty(value = "申诉类型")
    private String examineType;

    @ApiModelProperty(value = "申诉描述")
    private String examineState;

    @ApiModelProperty(value = "申诉描述")
    private String examineDescribe;

    @ApiModelProperty(value = "申诉描述")
    private Date examineDate;

    @ApiModelProperty(value = "审批内容")
    private String examineAuditInfo;

    @ApiModelProperty(value = "审批人")
    private String examineAuditUser;

    @ApiModelProperty(value = "审批时间")
    private Date examineAuditDate;


    public void resendEnable(){
        if (ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode().equals(this.status)
                || ConsultationLaunchApplyConstant.ApplyStatus.START.getCode().equals(this.status)
                || ConsultationLaunchApplyConstant.ApplyStatus.START_EXPIRE.getCode().equals(this.status)
                || ConsultationLaunchApplyConstant.ApplyStatus.EXPIRE.getCode().equals(this.status)
                || ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDOING.getCode().equals(this.status)
                || ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode().equals(this.status)) {
            this.resendEnable = YN.YES.getCode();
        } else if (ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(this.status)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(this.submitTime);
            calendar.add(Calendar.DATE, 1);
            Date submitAdd1 = calendar.getTime();
            if (submitAdd1.compareTo(new Date()) > 0) {
                this.resendEnable = YN.NO.getCode();
            } else {
                this.resendEnable = YN.YES.getCode();
            }
        } else {
            this.resendEnable = YN.NO.getCode();
        }
    }
}
