package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationAdviceInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationAdviceInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationAdviceAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationAdviceQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorGetDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorUpdateStatusDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationAdviceVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationDoctorVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationAdviceBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationDoctorBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class ConsultationAdviceBizServiceImpl implements ConsultationAdviceBizService {

    @Autowired
    private ConsultationAdviceInfoService consultationAdviceInfoService;
    @Autowired
    private ConsultationDoctorBizService consultationDoctorBizService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(ConsultationAdviceAddDTO params) {

        // 查询是否填写会诊意见
        ConsultationAdviceInfo existEntity = consultationAdviceInfoService.getOne(new QueryWrapper<ConsultationAdviceInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("consultation_id", params.getConsultationId())
            .eq("doctor_id", params.getDoctorId())
        );

        // 更新
        if (null != existEntity) {
            existEntity.setAdvice(params.getAdvice());
            consultationAdviceInfoService.updateById(existEntity);
            return;
        }

        // 新增
        ConsultationAdviceInfo entity = new ConsultationAdviceInfo();
        entity.setId(ConsultationHelper.getConsultationSignId());
        entity.setConsultationId(params.getConsultationId());
        entity.setDoctorId(params.getDoctorId());
        entity.setAdvice(params.getAdvice());
        entity.setRowsStatus(YN.YES.getCode());
        consultationAdviceInfoService.save(entity);

        // 更新会诊医生状态
        ConsultationDoctorUpdateStatusDTO consultationDoctorUpdateStatusDTO = new ConsultationDoctorUpdateStatusDTO();
        consultationDoctorUpdateStatusDTO.setConsultationId(entity.getConsultationId());
        consultationDoctorUpdateStatusDTO.setDoctorId(entity.getDoctorId());
        consultationDoctorUpdateStatusDTO.setStatus(ConsultationConstant.DoctorStatus.ADVICE.getCode());
        consultationDoctorBizService.update(consultationDoctorUpdateStatusDTO);
    }

    @Override
    public List<ConsultationAdviceVO> list(ConsultationAdviceQueryDTO params) {
        List<ConsultationAdviceInfo> list = consultationAdviceInfoService.list(new QueryWrapper<ConsultationAdviceInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> doctorIds = new ArrayList<>();
        list.forEach(i -> doctorIds.add(i.getDoctorId()));

        // 查询会诊医生信息
        ConsultationDoctorQueryDTO doctorQueryDTO = new ConsultationDoctorQueryDTO();
        doctorQueryDTO.setConsultationId(params.getConsultationId());
        doctorQueryDTO.setDoctorIds(doctorIds);
        List<ConsultationDoctorVO> doctorList = consultationDoctorBizService.list(doctorQueryDTO);
        Map<String, ConsultationDoctorVO> doctorMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(doctorList)) {
            doctorList.forEach(i -> doctorMap.put(i.getDoctorId(), i));
        }

        return list.stream().map(i -> {
            ConsultationAdviceVO vo = ConsultationHelper.getConsultationAdviceVO(i);

            ConsultationDoctorVO doctorVO = doctorMap.get(vo.getDoctorId());
            if (null != doctorVO) {
                vo.setDeptCode(doctorVO.getDeptCode());
                vo.setDeptName(doctorVO.getDeptName());
                vo.setDoctorName(doctorVO.getDoctorName());
                vo.setDutyCode(doctorVO.getDutyCode());
                vo.setDutyName(doctorVO.getDutyName());
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public ConsultationAdviceVO detail(ConsultationAdviceQueryDTO params) {
        // 会诊医生只能查看自己的会诊意见（参数：consultation_id、docktor_id）；会诊发起人可以查看所有人的会诊意见（参数：consultation_id、id）
        
        ConsultationAdviceInfo entity = consultationAdviceInfoService.getOne(new QueryWrapper<ConsultationAdviceInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
        );

        if (null == entity) {
            return null;
        }

        ConsultationAdviceVO vo = ConsultationHelper.getConsultationAdviceVO(entity);

        ConsultationDoctorGetDTO doctorGetDTO = new ConsultationDoctorGetDTO();
        doctorGetDTO.setConsultationId(entity.getConsultationId());
        doctorGetDTO.setDoctorId(entity.getDoctorId());
        ConsultationDoctorVO doctorVO = consultationDoctorBizService.detail(doctorGetDTO);
        if (null != doctorVO) {
            vo.setDeptCode(doctorVO.getDeptCode());
            vo.setDeptName(doctorVO.getDeptName());
            vo.setDoctorName(doctorVO.getDoctorName());
            vo.setDutyCode(doctorVO.getDutyCode());
            vo.setDutyName(doctorVO.getDutyName());
        }

        return vo;
    }

}
