package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationDoctorInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationDoctorInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorAlternateDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorGetDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorUpdateStatusDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationDoctorVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationDoctorBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class ConsultationDoctorBizServiceImpl implements ConsultationDoctorBizService {

    @Autowired
    private ConsultationDoctorInfoService consultationDoctorInfoService;
    @Autowired
    private ConsultationMapper consultationMapper;

    @Override
    public ConsultationDoctorVO add(ConsultationDoctorAddDTO params) {
        ValidatorUtils.validateEntity(params);

        ConsultationDoctorInfo existEntity = consultationDoctorInfoService.getOne(new QueryWrapper<ConsultationDoctorInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("CONCAT(consultation_id, '-', doctor_id)", params.getConsultationId() + "-" + params.getDoctorId())
        );

        if (null != existEntity) {
            return null;
        }

        DateTime now = DateUtil.date();
        ConsultationDoctorInfo entity = new ConsultationDoctorInfo();
        entity.setId(ConsultationHelper.getConsultationDoctorId());
        entity.setConsultationId(params.getConsultationId());
        entity.setDeptCode(params.getDeptCode());
        entity.setDeptName(params.getDeptName());
        entity.setDoctorId(params.getDoctorId());
        entity.setDoctorName(params.getDoctorName());
        entity.setDutyCode(params.getDutyCode());
        entity.setDutyName(params.getDutyName());
        entity.setDoctorType(params.getDoctorType());
        entity.setTransferDoctorId(params.getTransferDoctorId());
        entity.setStatus(params.getStatus());
        entity.setRemark(params.getRemark());
        entity.setOpTime(now);
        entity.setRowsStatus(YN.YES.getCode());

        consultationDoctorInfoService.save(entity);

        return ConsultationHelper.getConsultationDoctorVO(entity);
    }

    @Override
    public void addBatch(List<ConsultationDoctorAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> {
            ValidatorUtils.validateEntity(i);
        });

        List<String> queryList = new ArrayList<>();
        params.forEach(i -> queryList.add(i.getConsultationId() + "-" + i.getDoctorId()));

        // 查询是否存在，存在则过滤掉
        List<ConsultationDoctorInfo> existList = consultationDoctorInfoService.list(new QueryWrapper<ConsultationDoctorInfo>()
            .in("CONCAT(consultation_id, '-', doctor_id)", queryList)
        );
        Map<String, ConsultationDoctorInfo> existMap = new HashMap<>();
        existList.forEach(i -> existMap.put(i.getConsultationId() + "-" + i.getDoctorId(), i));

        DateTime now = DateUtil.date();
        List<ConsultationDoctorInfo> entityList = new ArrayList<>();
        params.forEach(i -> {
            ConsultationDoctorInfo entity = existMap.get(i.getConsultationId() + "-" + i.getDoctorId());
            if (null != entity) {
                return;
            }

            entity = new ConsultationDoctorInfo();
            entity.setId(ConsultationHelper.getConsultationDoctorId());
            entity.setConsultationId(i.getConsultationId());
            entity.setDeptCode(i.getDeptCode());
            entity.setDeptName(i.getDeptName());
            entity.setDoctorId(i.getDoctorId());
            entity.setDoctorName(i.getDoctorName());
            entity.setDutyCode(i.getDutyCode());
            entity.setDutyName(i.getDutyName());
            entity.setDoctorType(i.getDoctorType());
            entity.setTransferDoctorId(i.getTransferDoctorId());
            entity.setStatus(i.getStatus());
            entity.setRemark(i.getRemark());
            entity.setOpTime(now);
            entity.setRowsStatus(YN.YES.getCode());

            entityList.add(entity);
        });

        consultationDoctorInfoService.saveBatch(entityList);
    }

    @Override
    public void update(ConsultationDoctorUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);

        ConsultationDoctorInfo entity = consultationDoctorInfoService.getOne(new QueryWrapper<ConsultationDoctorInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("consultation_id", params.getConsultationId())
            .eq("doctor_id", params.getDoctorId())
        );
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus()) || !entity.getDoctorId().equals(params.getDoctorId())) {
            throw new BusinessException("未查询到就诊信息");
        }

        boolean flag = true;
        // // alternate -> todo
        // if (ConsultationConstant.DoctorStatus.TODO.getCode().equals(params.getStatus()) && !ConsultationConstant.DoctorStatus.ALTERNATE.getCode().equals(entity.getStatus())) {
        //     flag = false;
        // }
        // todo -> join / refuse
        if ((ConsultationConstant.DoctorStatus.JOIN.getCode().equals(params.getStatus()) || ConsultationConstant.DoctorStatus.REFUSE.getCode().equals(params.getStatus())) && !ConsultationConstant.DoctorStatus.TODO.getCode().equals(entity.getStatus())) {
            flag = false;
        }
        // join -> sign
        if (ConsultationConstant.DoctorStatus.JOIN.getCode().equals(entity.getStatus()) && !ConsultationConstant.DoctorStatus.SIGN.getCode().equals(params.getStatus())) {
            flag = false;
        }
        // sign -> advice
        if (ConsultationConstant.DoctorStatus.SIGN.getCode().equals(entity.getStatus()) && !ConsultationConstant.DoctorStatus.ADVICE.getCode().equals(params.getStatus())) {
            flag = false;
        }

        if (!flag) {
            throw new BusinessException("不支持的状态更新");
        }

        if (ConsultationConstant.DoctorStatus.JOIN.getCode().equals(params.getStatus()) || ConsultationConstant.DoctorStatus.REFUSE.getCode().equals(params.getStatus())) {
            entity.setOpTime(DateUtil.date()); // 只有join或refuse才更新opTime，此时间用于判断用户的join或refuse时间
        }
        entity.setStatus(params.getStatus());
        
        consultationDoctorInfoService.updateById(entity);
    }

    @Override
    public void updateBatch(List<ConsultationDoctorUpdateStatusDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> {
            queryParams.add(i.getConsultationId() + "-" + i.getDoctorId());
        });

        List<ConsultationDoctorInfo> entityList = consultationDoctorInfoService.list(new QueryWrapper<ConsultationDoctorInfo>()
            .eq("rows_status", YN.YES.getCode())
            .in("CONCAT(consultation_id, '-', doctor_id)", queryParams)
        );
        if (CollectionUtil.isEmpty(entityList)) {
            return;
        }
        Map<String, ConsultationDoctorInfo> entityMap = new HashMap<>();
        entityList.forEach(i -> entityMap.put(i.getConsultationId() + "-" + i.getDoctorId(), i));

        List<ConsultationDoctorInfo> list = new ArrayList<>();
        params.forEach(i -> {
            ConsultationDoctorInfo entity = entityMap.get(i.getConsultationId() + "-" + i.getDoctorId());
            
            if (ConsultationConstant.DoctorStatus.JOIN.getCode().equals(i.getStatus()) || ConsultationConstant.DoctorStatus.REFUSE.getCode().equals(i.getStatus())) {
                entity.setOpTime(DateUtil.date()); // 只有join或refuse才更新opTime，此时间用于判断用户的join或refuse时间
            }

            entity.setStatus(i.getStatus());

            list.add(entity);
        });

        consultationDoctorInfoService.updateBatchById(list);
    }

    @Override
    public List<ConsultationDoctorVO> list(ConsultationDoctorQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ConsultationDoctorInfo> list = consultationDoctorInfoService.list(new QueryWrapper<ConsultationDoctorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
                .in(CollectionUtil.isNotEmpty(params.getConsultationIds()), "consultation_id", params.getConsultationIds())
                .eq(StrUtil.isNotBlank(params.getDoctorType()), "doctor_type", params.getDoctorType())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .orderByDesc("op_time")
        );
    
        return list.stream().map(entity ->  ConsultationHelper.getConsultationDoctorVO(entity)).collect(Collectors.toList());
    }
    
    @Override
    public ConsultationDoctorVO detail(ConsultationDoctorGetDTO params) {
        ConsultationDoctorInfo entity = consultationDoctorInfoService.getOne(new QueryWrapper<ConsultationDoctorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
        );

        if (null == entity) {
            return null;
        }

        return ConsultationHelper.getConsultationDoctorVO(entity);
    }

    @Override
    public ConsultationDoctorVO alternate(ConsultationDoctorAlternateDTO params) {
        ValidatorUtils.validateEntity(params);
        // 拒绝前信息
        ConsultationDoctorInfo refuseEntity = consultationDoctorInfoService.getOne(new QueryWrapper<ConsultationDoctorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("status", ConsultationConstant.DoctorStatus.TODO.getCode())
                .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .last(" limit 1 ")
        );
        if (null == refuseEntity) {
            return null;
        }

        // 拒绝
        refuseEntity.setStatus(ConsultationConstant.DoctorStatus.REFUSE.getCode());
        consultationDoctorInfoService.updateById(refuseEntity);

        // 查询拒绝医生的学科编号
        Map<String, String> doctor = consultationMapper.subjectDoctor(new QueryWrapper<Void>()
            .eq("a.rows_status", YN.YES.getCode())
            .eq("a.id", refuseEntity.getDoctorId())
        );
        if (null == doctor || StrUtil.isBlank(doctor.get("subjectCode"))) {
            return null;
        }

        // 候补会诊医生
        // 根据学科（dutyCode）查询sys_user表获取可以替补的医生，需要排除已经在该会诊的医生，一个医生可以同时参加多个会诊
        Map<String, String> doctorMap = consultationMapper.subjectDoctor(new QueryWrapper<Void>()
            .eq("a.rows_status", YN.YES.getCode())
            // .eq("dept_id", refuseEntity.getDeptCode())
            .eq("a.subject_code", doctor.get("subjectCode"))
            .ne("a.id", refuseEntity.getDoctorId())
            .orderByDesc("a.subject_order")
            .last(" limit 1 ")
        );

        if (CollectionUtil.isEmpty(doctorMap)) {
            return null;
        }

        String doctorId = doctorMap.get("doctorId");
        String doctorName = doctorMap.get("doctorName");
        String deptId = doctorMap.get("deptId");
        String deptName = doctorMap.get("deptName");

        ConsultationDoctorAddDTO consultationDoctorAddDTO = new ConsultationDoctorAddDTO();
        consultationDoctorAddDTO.setConsultationId(refuseEntity.getConsultationId());
        consultationDoctorAddDTO.setDeptCode(deptId);
        consultationDoctorAddDTO.setDeptName(deptName);
        consultationDoctorAddDTO.setDoctorId(doctorId);
        consultationDoctorAddDTO.setDoctorName(doctorName);
        consultationDoctorAddDTO.setDutyCode(refuseEntity.getDutyCode());
        consultationDoctorAddDTO.setDutyName(refuseEntity.getDutyName());
        consultationDoctorAddDTO.setDoctorType(ConsultationConstant.DoctorType.ALTERNATE.getCode());
        consultationDoctorAddDTO.setStatus(ConsultationConstant.DoctorStatus.TODO.getCode());
        if (ConsultationConstant.DoctorType.APPOINT.getCode().equals(refuseEntity.getDoctorType())) {
            consultationDoctorAddDTO.setTransferDoctorId(refuseEntity.getDoctorId());
        } else {
            consultationDoctorAddDTO.setTransferDoctorId(refuseEntity.getTransferDoctorId());
        }

        return this.add(consultationDoctorAddDTO);
    }

    @Override
    public ConsultationDoctorVO appointAlternate(ConsultationDoctorAlternateDTO params) {
        ValidatorUtils.validateEntity(params);
        if (StrUtil.isBlank(params.getAppointDoctorId())) {
            throw new BusinessException("未指定替补会诊医生");
        }

        // 拒绝前信息
        ConsultationDoctorInfo refuseEntity = consultationDoctorInfoService.getOne(new QueryWrapper<ConsultationDoctorInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("status", ConsultationConstant.DoctorStatus.TODO.getCode())
                .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .last(" limit 1 ")
        );
        if (null == refuseEntity) {
            return null;
        }

        // 拒绝
        refuseEntity.setStatus(ConsultationConstant.DoctorStatus.REFUSE.getCode());
        consultationDoctorInfoService.updateById(refuseEntity);

        // 查询指定的候补医生
        Map<String, String> doctorMap = consultationMapper.subjectDoctor(new QueryWrapper<Void>()
            .eq("a.rows_status", YN.YES.getCode())
            .eq("a.id", params.getDoctorId())
        );

        if (CollectionUtil.isEmpty(doctorMap)) {
            return null;
        }

        String doctorId = doctorMap.get("doctorId");
        String doctorName = doctorMap.get("doctorName");
        String deptId = doctorMap.get("deptId");
        String deptName = doctorMap.get("deptName");

        ConsultationDoctorAddDTO consultationDoctorAddDTO = new ConsultationDoctorAddDTO();
        consultationDoctorAddDTO.setConsultationId(refuseEntity.getConsultationId());
        consultationDoctorAddDTO.setDeptCode(deptId);
        consultationDoctorAddDTO.setDeptName(deptName);
        consultationDoctorAddDTO.setDoctorId(doctorId);
        consultationDoctorAddDTO.setDoctorName(doctorName);
        consultationDoctorAddDTO.setDutyCode(refuseEntity.getDutyCode());
        consultationDoctorAddDTO.setDutyName(refuseEntity.getDutyName());
        consultationDoctorAddDTO.setDoctorType(ConsultationConstant.DoctorType.ALTERNATE.getCode());
        consultationDoctorAddDTO.setStatus(ConsultationConstant.DoctorStatus.TODO.getCode());
        if (ConsultationConstant.DoctorType.APPOINT.getCode().equals(refuseEntity.getDoctorType())) {
            consultationDoctorAddDTO.setTransferDoctorId(refuseEntity.getDoctorId());
        } else {
            consultationDoctorAddDTO.setTransferDoctorId(refuseEntity.getTransferDoctorId());
        }

        return this.add(consultationDoctorAddDTO);
    }

    @Override
    public List<ConsultationDoctorVO> trendsList(ConsultationDoctorQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ConsultationDoctorInfo> list = consultationDoctorInfoService.list(new QueryWrapper<ConsultationDoctorInfo>()
                .eq("rows_status", YN.YES.getCode())
                // .ne("doctor_type", ConsultationConstant.DoctorStatus.INITIATION.getCode())
                .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
                // .orderByDesc("op_time")
        );

        return list.stream().map(entity -> ConsultationHelper.getConsultationDoctorVO(entity)).collect(Collectors.toList());
    }

}
