package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientChangeClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.framework.modules.biz.mapper.DwdPatientDataBizMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationDwdBizMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationDwdBizService;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

@DS("wd")
@Slf4j
@Service
public class ConsultationDwdBizServiceImpl implements ConsultationDwdBizService {

    @Autowired
    private ConsultationDwdBizMapper consultationDwdBizMapper;
    @Autowired
    private DwdPatientDataBizMapper dwdPatientDataBizMapper;
    
    @Override
    public String getPatientIdByVisitId(String visitId) {
        return consultationDwdBizMapper.getPatientIdByVisitId(new QueryWrapper<Void>()
            .eq("id", visitId)
        );
    }

    @Override
    public Map<String, String> mapPatientIdByVisitId(List<String> visitIds) {
        Map<String, String> map = new HashMap<>();
        if (CollectionUtil.isEmpty(visitIds)) {
            return map;
        }

        List<Map<String, String>> list = consultationDwdBizMapper.listPatientIdByVisitId(new QueryWrapper<Void>()
            .in("id", visitIds)
        );

        if (CollectionUtil.isEmpty(list)) {
            return map;
        }

        list.forEach(i -> map.put(i.get("id"), i.get("patientId")));

        log.info("就诊id查询患者id >>>>>> {}", JSON.toJSONString(map));
        return map;
    }


    @Override
    public String getPatientIdByInpatientId(String inpatientId) {
        return consultationDwdBizMapper.getPatientIdByInpatientId(new QueryWrapper<Void>()
            .eq("id", inpatientId)
        );
    }

    @Override
    public Map<String, String> mapPatientIdByInpatientId(List<String> inpatientIds) {
        Map<String, String> map = new HashMap<>();
        if (CollectionUtil.isEmpty(inpatientIds)) {
            return map;
        }

        List<Map<String, String>> list = consultationDwdBizMapper.listPatientIdByInpatientId(new QueryWrapper<Void>()
            .in("id", inpatientIds)
        );

        if (CollectionUtil.isEmpty(list)) {
            return map;
        }

        list.forEach(i -> map.put(i.get("id"), i.get("patientId")));

        log.info("住院id查询患者id >>>>>> {}", JSON.toJSONString(map));
        return map;
    }

    @Override
    public List<PatientInpatientRecordClientBO> listPatientInpatientChangeLastDateInfo(String patientId) {
        return dwdPatientDataBizMapper.listPatientInpatientInfo(new QueryWrapper<PatientInpatientRecordClientBO>()
                .eq("patient_id", patientId)
                .orderByDesc("inpatient_date")
                .last("limit 1"));
    }


}
