package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.helper.DesensitizedHelper;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.UserTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DoctorDwdPatientDataBizService;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant.ConsultationType;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyLog;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyTimeoutStat;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyLogService;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyTimeoutStatService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.*;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationNotifyInfoBizService;
import com.xxh.cloud.healthy.mdt.third.bean.WecomSendTextcardReq;
import com.xxh.cloud.healthy.mdt.third.bean.WecomSendTextcardResp;
import com.xxh.cloud.healthy.mdt.third.utils.MdtSendTextCardHttpUtil;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.PushWeixinLogAddDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.PushWeixinBizService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRemindInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRemindInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ConsultationLaunchApplyBizServiceImpl implements ConsultationLaunchApplyBizService {

    @Autowired
    private ConsultationLaunchApplyInfoService consultationLaunchApplyInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private DoctorRemindInfoService doctorRemindInfoService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private ConsultationBizService consultationBizService;
    @Autowired
    private ConsultationLaunchApplyLogService consultationLaunchApplyLogService;
    @Autowired
    private ConsultationNotifyInfoBizService consultationNotifyInfoBizService;
    @Autowired
    private DoctorDwdPatientDataBizService doctorDwdPatientDataBizService;
    @Autowired
    private ConsultationLaunchApplyTimeoutStatService consultationLaunchApplyTimeoutStatService;
    @Autowired
    private ConsultationMapper consultationMapper;
    @Autowired
    private PushWeixinBizService pushWeixinBizService;

    @Value("${xx.stat.con-apply.stat-time: 18}")
    private String statTime;

    @Override
    public PageData<ConsultationLaunchApplyVO> page(ConsultationLaunchApplyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 若按照患者名称筛选则先查询患者信息
        Set<String> patientIds = new HashSet<>();
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new PageData<>(null, 0);
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }

        QueryWrapper<ConsultationLaunchApplyInfo> queryWrapper = new QueryWrapper<ConsultationLaunchApplyInfo>();
        if (StrUtil.isNotBlank(params.getCustStatus())) {
            switch (params.getCustStatus()) {
                case "1":
                    queryWrapper.isNull("submit_time");
                    break;
                case "2":
                    queryWrapper.isNotNull("submit_time");
                    break;
                case "3":
                    queryWrapper.isNotNull("submit_time").isNotNull("finish_time");
                    break;
                case "4":
                    queryWrapper.isNotNull("submit_time").isNull("finish_time");
                    break;
                default:
                    break;
            }
        }
        
        IPage<ConsultationLaunchApplyInfo> page = consultationLaunchApplyInfoService.page(new Page<>(params.getPage(), params.getRows()), queryWrapper
                .eq("rows_status", YN.YES.getCode())
                .ne(YN.YES.getCode().equals(params.getManageFlag()), "status", ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getConsultationType()), "consultation_type", params.getConsultationType())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(patientIds), "patient_id", patientIds)
                .eq(StrUtil.isNotBlank(params.getApplyDoctorId()), "apply_doctor_id", params.getApplyDoctorId())
                .eq(StrUtil.isNotBlank(params.getLaunchDoctorId()), "launch_doctor_id", params.getLaunchDoctorId())
                .inSql(StrUtil.isNotBlank(params.getLaunchDept()), "launch_doctor_id",
                        StrUtil.format("select id from sys_user where dept_id = '{}'", params.getLaunchDept()))
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getManageId()), "manage_id", params.getManageId())
                .in(CollUtil.isNotEmpty(params.getManageIds()), "manage_id", params.getManageIds())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .ge(null != params.getStartDate(), "apply_time", params.getStartDate())
                .lt(null != params.getEndDate(), "apply_time", params.getEndDate())
                .eq(YN.YES.getCode().equals(params.getResendFlg()), "resend_flg", YN.YES.getCode())
                .and(YN.NO.getCode().equals(params.getResendFlg()),
                        i->i.ne("resend_flg", YN.YES.getCode()).or().isNull("resend_flg"))
                .eq(StrUtil.isNotBlank(params.getExamineType()), "examine_type", params.getExamineType())
                .eq(StrUtil.isNotBlank(params.getExamineState()), "examine_state", params.getExamineState())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询需要在海泰发起mdt会诊的医生的信息
        
        Set<String> doctorIds = new HashSet<>();
        // Set<String> patientIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        Set<String> labelIds = new HashSet<>();
        Set<String> consultationIds = new HashSet<>();
        Set<String> applyIds = new HashSet<>();
        Set<String> inpatientIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            applyIds.add(i.getId());
            if (StrUtil.isNotBlank(i.getLaunchDoctorId())) {
                doctorIds.add(i.getLaunchDoctorId());
            }
            if (StrUtil.isNotBlank(i.getApplyDoctorId())) {
                doctorIds.add(i.getApplyDoctorId());
            }
            if (StrUtil.isNotBlank(i.getResendOpUserId())) {
                doctorIds.add(i.getResendOpUserId());
            }
            if (StrUtil.isNotBlank(i.getPatientId())) {
                patientIds.add(i.getPatientId());
            }
            if (StrUtil.isNotBlank(i.getDiseaseId())) {
                diseaseIds.add(i.getDiseaseId());
            }
            if (StrUtil.isNotBlank(i.getConsultationLabelId())) {
                labelIds.add(i.getConsultationLabelId());
            }
            if (StrUtil.isNotBlank(i.getConsultationId())) {
                consultationIds.add(i.getConsultationId());
            }
            if (StrUtil.isNotBlank(i.getInpatientId())) {
                inpatientIds.add(i.getInpatientId());
            }
        });
        
        // 查询需要在海泰发起mdt会诊的医生的信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        // 患者信息
        // Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isEmpty(patientMap) && CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            if (CollUtil.isNotEmpty(diseaseList)) {
                diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            }
        }

        // 标签
        Map<String, LabelBO> labelMap = labelClientService.getLabelMapByIds(labelIds);

        // 会诊信息
        Map<String, ConsultationVO> consultationMap = new HashMap<>();
        if (CollUtil.isNotEmpty(consultationIds)) {
            ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
            // consultationQueryDTO.setApplyType(ConsultationType.ORDINARY.getCode());
            consultationQueryDTO.setIds(new ArrayList<>(consultationIds));
            List<ConsultationVO> consultationList = consultationBizService.simpleList(consultationQueryDTO);
            if (CollUtil.isNotEmpty(consultationList)) {
                consultationList.forEach(i -> consultationMap.put(i.getId(), i));
            }
        }

        // 添加日志
        Map<String, List<ConsultationLaunchApplyLogVO>> logMap = applyLogMap(applyIds);
        // 查询入院出院记录
        PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
        patientLastDataQueryDTO.setPatientIds(new ArrayList<>(patientIds));
        patientLastDataQueryDTO.setVisitIds(new ArrayList<>(inpatientIds));
        List<PatientInpatientRecordClientBO> inpRecordList = doctorDwdPatientDataBizService.listPatientInpatientLastDateAndOut(patientLastDataQueryDTO);
        Map<String, PatientInpatientRecordClientBO> inpRecordMap = new HashMap<>();
        if (CollUtil.isNotEmpty(inpRecordList)) {
            inpRecordList.forEach(i -> inpRecordMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyVO> data = new PageConverter<ConsultationLaunchApplyInfo, ConsultationLaunchApplyVO>() {
            @Override
            public ConsultationLaunchApplyVO converter(ConsultationLaunchApplyInfo entity) {
                ConsultationLaunchApplyVO vo = ConsultationHelper.getConsultationLaunchApplyVO(entity);

                AdminUserBO applyDoctor = doctorMap.get(vo.getApplyDoctorId());
                if (null != applyDoctor) {
                    vo.setApplyDoctorInfo(applyDoctor);
                    vo.setApplyDeptInfo(deptMap.get(applyDoctor.getDeptId()));
                }

                AdminUserBO launchDoctor = doctorMap.get(vo.getLaunchDoctorId());
                if (null != launchDoctor) {
                    vo.setLaunchDoctorInfo(launchDoctor);
                    vo.setLaunchDeptInfo(deptMap.get(launchDoctor.getDeptId()));
                }

                vo.setPatientInfo(patientMap.get(vo.getPatientId()));
                vo.setDiseaseInfo(diseaseMap.get(vo.getDiseaseId()));
                
                LabelBO label = labelMap.get(vo.getConsultationLabelId());
                if (null != label) {
                    vo.setConsultationLabelInfo(label);
                }

                if (StrUtil.isNotBlank(vo.getConsultationId()) && null != consultationMap.get(vo.getConsultationId())) {
                    vo.setConsultationInfo(consultationMap.get(vo.getConsultationId()));
                    vo.setConsultationDoctorList(consultationMap.get(vo.getConsultationId()).getConsultationDoctorList());
                }
                if (logMap.containsKey(vo.getId())) {
                    vo.setApplyLogs(logMap.get(vo.getId()));
                }
                if (inpRecordMap.containsKey(vo.getInpatientId())) {
                    PatientInpatientRecordClientBO bo = inpRecordMap.get(vo.getInpatientId());
                    vo.setInpatientDate(bo.getInpatientDate());
                    vo.setIsOut(bo.getIsOut());
                    vo.setOutDate(bo.getOutDate());
                }

                vo.setRealStatus("未发起会诊");
                if (null != vo.getSubmitTime() && null == vo.getFinishTime()) {
                    vo.setRealStatus("未完成会诊");
                } else if (null != vo.getSubmitTime() && null != vo.getFinishTime()) {
                    vo.setRealStatus("已完成会诊");
                }
    
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public ConsultationLaunchApplyStatisticVO statistic(ConsultationLaunchApplyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        // 若按照患者名称筛选则先查询患者信息
        Set<String> patientIds = new HashSet<>();
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (StrUtil.isNotBlank(params.getPatientName())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientName(params.getPatientName());
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollectionUtil.isEmpty(patientList)) {
                return new ConsultationLaunchApplyStatisticVO();
            }
            patientList.forEach(i -> {
                patientIds.add(i.getPatientId());
                patientMap.put(i.getPatientId(), i);
            });
        }

        QueryWrapper<ConsultationLaunchApplyInfo> queryWrapper = new QueryWrapper<ConsultationLaunchApplyInfo>();
        if (StrUtil.isNotBlank(params.getCustStatus())) {
            switch (params.getCustStatus()) {
                case "1":
                    queryWrapper.isNull("submit_time");
                    break;
                case "2":
                    queryWrapper.isNotNull("submit_time");
                    break;
                case "3":
                    queryWrapper.isNotNull("submit_time").isNotNull("finish_time");
                    break;
                case "4":
                    queryWrapper.isNotNull("submit_time").isNull("finish_time");
                    break;
                default:
                    break;
            }
        }
        
        ConsultationLaunchApplyStatisticVO statusStatistic = consultationMapper.consultationApplyStatusStatistic(queryWrapper
                .eq("rows_status", YN.YES.getCode())
                .ne(YN.YES.getCode().equals(params.getManageFlag()), "status", ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getConsultationType()), "consultation_type", params.getConsultationType())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(patientIds), "patient_id", patientIds)
                .eq(StrUtil.isNotBlank(params.getApplyDoctorId()), "apply_doctor_id", params.getApplyDoctorId())
                .eq(StrUtil.isNotBlank(params.getLaunchDoctorId()), "launch_doctor_id", params.getLaunchDoctorId())
                .inSql(StrUtil.isNotBlank(params.getLaunchDept()), "launch_doctor_id",
                        StrUtil.format("select id from sys_user where dept_id = '{}'", params.getLaunchDept()))
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getManageId()), "manage_id", params.getManageId())
                .in(CollUtil.isNotEmpty(params.getManageIds()), "manage_id", params.getManageIds())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .ge(null != params.getStartDate(), "apply_time", params.getStartDate())
                .lt(null != params.getEndDate(), "apply_time", params.getEndDate())
                .eq(YN.YES.getCode().equals(params.getResendFlg()), "resend_flg", YN.YES.getCode())
                .and(YN.NO.getCode().equals(params.getResendFlg()),
                        i->i.ne("resend_flg", YN.YES.getCode()).or().isNull("resend_flg"))
                .eq(StrUtil.isNotBlank(params.getExamineType()), "examine_type", params.getExamineType())
                .eq(StrUtil.isNotBlank(params.getExamineState()), "examine_state", params.getExamineState())
                .orderByDesc("update_date")
        );

        return statusStatistic;
    }

    @Override
    public List<ConsultationLaunchApplyVO> list(ConsultationLaunchApplyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        Set<String> searchDoctorIds = new HashSet<>();
        if (StrUtil.isNotEmpty(params.getDeptRelationId())) {
            List<String> strList = consultationMapper.doctorIdsByRelationDept(new QueryWrapper<Void>()
                    .eq("t2.group_id", params.getDeptRelationId()));
            if (CollUtil.isNotEmpty(strList)) {
                searchDoctorIds.addAll(strList);
            } else {
                searchDoctorIds.add("dummy"); // 查询为空
            }
        }

        List<ConsultationLaunchApplyInfo> list = consultationLaunchApplyInfoService.list(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getConsultationType()), "consultation_type", params.getConsultationType())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getApplyDoctorId()), "apply_doctor_id", params.getApplyDoctorId())
                .eq(StrUtil.isNotBlank(params.getLaunchDoctorId()), "launch_doctor_id", params.getLaunchDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .in(CollUtil.isNotEmpty(params.getStatuses()), "status", params.getStatuses())
                .eq(StrUtil.isNotBlank(params.getManageId()), "manage_id", params.getManageId())
                .in(CollUtil.isNotEmpty(params.getManageIds()), "manage_id", params.getManageIds())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .ge(null != params.getStartDate(), "apply_time", params.getStartDate())
                .lt(null != params.getEndDate(), "apply_time", params.getEndDate())
                .ge(null != params.getApplyStartDate(), "apply_time", params.getApplyStartDate())
                .lt(null != params.getApplyEndDate(), "apply_time", params.getApplyEndDate())
                .ge(null != params.getSubmitStartDate(), "submit_time", params.getSubmitStartDate())
                .lt(null != params.getSubmitEndDate(), "submit_time", params.getSubmitEndDate())
                .in(CollUtil.isNotEmpty(searchDoctorIds), "apply_doctor_id", searchDoctorIds)
                .eq(YN.YES.getCode().equals(params.getResendFlg()), "resend_flg", YN.YES.getCode())
                .and(YN.NO.getCode().equals(params.getResendFlg()),
                        i->i.ne("resend_flg", YN.YES.getCode()).or().isNull("resend_flg"))
                .eq(StrUtil.isNotBlank(params.getExamineType()), "examine_type", params.getExamineType())
                .eq(StrUtil.isNotBlank(params.getExamineState()), "examine_state", params.getExamineState())
                .orderByDesc("update_date")
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Set<String> doctorIds = new HashSet<>();
        Set<String> patientIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        Set<String> labelIds = new HashSet<>();
        Set<String> consultationIds = new HashSet<>();
        Set<String> applyIds = new HashSet<>();
        list.forEach(i -> {
            applyIds.add(i.getId());
            if (StrUtil.isNotBlank(i.getLaunchDoctorId())) {
                doctorIds.add(i.getLaunchDoctorId());
            }
            if (StrUtil.isNotBlank(i.getApplyDoctorId())) {
                doctorIds.add(i.getApplyDoctorId());
            }
            if (StrUtil.isNotBlank(i.getResendOpUserId())) {
                doctorIds.add(i.getResendOpUserId());
            }
            if (StrUtil.isNotBlank(i.getPatientId())) {
                patientIds.add(i.getPatientId());
            }
            if (StrUtil.isNotBlank(i.getDiseaseId())) {
                diseaseIds.add(i.getDiseaseId());
            }
            if (StrUtil.isNotBlank(i.getConsultationLabelId())) {
                labelIds.add(i.getConsultationLabelId());
            }
            if (StrUtil.isNotBlank(i.getConsultationId())) {
                consultationIds.add(i.getConsultationId());
            }
        });
        
        // 查询需要在海泰发起mdt会诊的医生的信息
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                List<String> deptIds = new ArrayList<>();
                doctorList.forEach(i -> {
                    doctorMap.put(i.getId(), i);
                    deptIds.add(i.getDeptId());
                });
                
                if (CollUtil.isNotEmpty(deptIds)) {
                    List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
                    if (CollUtil.isNotEmpty(deptList)) {
                        deptList.forEach(i -> deptMap.put(i.getId(), i));
                    }
                }
            }
        }

        // 患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            if (CollUtil.isNotEmpty(diseaseList)) {
                diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            }
        }

        // 标签
        Map<String, LabelBO> labelMap = labelClientService.getLabelMapByIds(labelIds);

        // 会诊信息
        Map<String, ConsultationVO> consultationMap = new HashMap<>();
        if (CollUtil.isNotEmpty(consultationIds)) {
            ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
            consultationQueryDTO.setApplyType(ConsultationType.ORDINARY.getCode());
            consultationQueryDTO.setIds(new ArrayList<>(consultationIds));
            List<ConsultationVO> consultationList = consultationBizService.simpleList(consultationQueryDTO);
            if (CollUtil.isNotEmpty(consultationList)) {
                consultationList.forEach(i -> consultationMap.put(i.getId(), i));
            }
        }
        // 添加日志
        Map<String, List<ConsultationLaunchApplyLogVO>> logMap = applyLogMap(applyIds);

        return list.stream().map(i -> {
            ConsultationLaunchApplyVO vo = ConsultationHelper.getConsultationLaunchApplyVO(i);
            
            AdminUserBO applyDoctor = doctorMap.get(vo.getApplyDoctorId());
            if (null != applyDoctor) {
                vo.setApplyDoctorInfo(applyDoctor);
                vo.setApplyDeptInfo(deptMap.get(applyDoctor.getDeptId()));
            }

            AdminUserBO launchDoctor = doctorMap.get(vo.getLaunchDoctorId());
            if (null != launchDoctor) {
                vo.setLaunchDoctorInfo(launchDoctor);
                vo.setLaunchDeptInfo(deptMap.get(launchDoctor.getDeptId()));
            }

            vo.setPatientInfo(patientMap.get(vo.getPatientId()));
            vo.setDiseaseInfo(diseaseMap.get(vo.getDiseaseId()));

            vo.setConsultationLabelInfo(labelMap.get(i.getConsultationLabelId()));

            if (StrUtil.isNotBlank(vo.getConsultationId()) && null != consultationMap.get(vo.getConsultationId())) {
                vo.setConsultationInfo(consultationMap.get(vo.getConsultationId()));
                vo.setConsultationDoctorList(consultationMap.get(vo.getConsultationId()).getConsultationDoctorList());
            }
            if (logMap.containsKey(vo.getId())) {
                vo.setApplyLogs(logMap.get(vo.getId()));
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public List<ConsultationLaunchApplyVO> simpleList(ConsultationLaunchApplyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<ConsultationLaunchApplyInfo> list = consultationLaunchApplyInfoService.list(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getConsultationType()), "consultation_type", params.getConsultationType())
                .eq(StrUtil.isNotBlank(params.getInpatientId()), "inpatient_id", params.getInpatientId())
                .in(CollUtil.isNotEmpty(params.getInpatientIds()), "inpatient_id", params.getInpatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getApplyDoctorId()), "apply_doctor_id", params.getApplyDoctorId())
                .eq(StrUtil.isNotBlank(params.getLaunchDoctorId()), "launch_doctor_id", params.getLaunchDoctorId())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .eq(StrUtil.isNotBlank(params.getManageId()), "manage_id", params.getManageId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .in(CollUtil.isNotEmpty(params.getManageIds()), "manage_id", params.getManageIds())
                .ge(null != params.getStartDate(), "apply_time", params.getStartDate())
                .lt(null != params.getEndDate(), "apply_time", params.getEndDate())
                .orderByDesc("update_date")
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(i -> {
            ConsultationLaunchApplyVO vo = ConsultationHelper.getConsultationLaunchApplyVO(i);
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public ConsultationLaunchApplyVO detail(ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        ConsultationLaunchApplyInfo entity = consultationLaunchApplyInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            return null;
        }
        
        ConsultationLaunchApplyVO vo = ConsultationHelper.getConsultationLaunchApplyVO(entity);

        if (StrUtil.isNotBlank(vo.getApplyDoctorId())) {
            AdminUserBO user = adminUserClientService.getUserById(vo.getApplyDoctorId());
            vo.setApplyDoctorInfo(user);
            if (null != user && StrUtil.isNotBlank(user.getDeptId())) {
                vo.setApplyDeptInfo(deptClientService.getDeptById(user.getDeptId()));
            }
        }
        if (StrUtil.isNotBlank(vo.getLaunchDoctorId())) {
            AdminUserBO user = adminUserClientService.getUserById(vo.getLaunchDoctorId());
            vo.setLaunchDoctorInfo(user);
            if (null != user && StrUtil.isNotBlank(user.getDeptId())) {
                vo.setLaunchDeptInfo(deptClientService.getDeptById(user.getDeptId()));
            }
        }
        if (StrUtil.isNotBlank(vo.getPatientId())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientId(vo.getPatientId());
            vo.setPatientInfo(patientClientService.getOneSimple(patientQueryDTO));
        }
        if (StrUtil.isNotBlank(vo.getDiseaseId())) {
            vo.setDiseaseInfo(standardDiseaseClientService.getById(vo.getDiseaseId()));
        }

        // 查询发送日志
        List<ConsultationLaunchApplyLog> logList = consultationLaunchApplyLogService.list(new QueryWrapper<ConsultationLaunchApplyLog>()
                .eq("apply_id", params.getId()));
        if (CollUtil.isNotEmpty(logList)) {
            Set<String> opUserIds=new HashSet<>();
            logList.forEach(item-> opUserIds.add(item.getOpUserId()));
            Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(opUserIds);
            List<ConsultationLaunchApplyLogVO> applyLogs = logList.stream().map(
                    item-> ConsultationHelper.getConsultationLaunchApplyLogVO(item, userMap)).collect(Collectors.toList());
            vo.setApplyLogs(applyLogs);
        }
        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ConsultationLaunchApplyVO add(ConsultationLaunchApplyAddDTO params) {
        ValidatorUtils.validateEntity(params);
        // 校验病种重复标签
        List<Map<String, String>> patientMapList = consultationMapper.standardLabelList(new QueryWrapper<Void>()
                .eq("id", params.getConsultationLabelId()));
        List<String> applyDeptIds = new ArrayList<>();
        if (CollUtil.isNotEmpty(patientMapList)) {
            applyDeptIds.addAll(StrJoinSplitUtils.split2List(patientMapList.get(0).get("consultation_dept_ids")));
        }

        if (repeatApply(params.getPatientId(), params.getConsultationLabelId(), applyDeptIds)) {
            return null;
        }

        // 保存发起会诊申请信息
        ConsultationLaunchApplyInfo entity = new ConsultationLaunchApplyInfo();
        entity.setId(ConsultationHelper.getConsultationLaunchApplyId());

        entity.setConsultationType(params.getConsultationType());
        entity.setPatientId(params.getPatientId());
        entity.setInpatientId(params.getInpatientId());
        entity.setVisitId(params.getVisitId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setApplyDoctorId(params.getApplyDoctorId());
        entity.setLaunchDoctorId(params.getLaunchDoctorId());
        entity.setContent(params.getContent());
        entity.setManageId(params.getManageId());
        entity.setType(params.getType());

        entity.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode());
        if (CollUtil.isNotEmpty(params.getJoinDept())) {
            entity.setJoinDept(StrJoinSplitUtils.join(params.getJoinDept()));
        } else {
            entity.setJoinDept(StrJoinSplitUtils.join(applyDeptIds));
        }
        entity.setConsultationLabelId(params.getConsultationLabelId());
        entity.setRowsStatus(YN.YES.getCode());
        consultationLaunchApplyInfoService.save(entity);

        // 发送会诊消息通知
        consultationNotifyInfoBizService.addNotify(entity);

        return ConsultationHelper.getConsultationLaunchApplyVO(entity);
    }

    @Override
    public void update(ConsultationLaunchApplyUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        
        // 保存发起会诊申请信息
        ConsultationLaunchApplyInfo entity = new ConsultationLaunchApplyInfo();
        entity.setId(params.getId());
        entity.setConsultationType(params.getConsultationType());
        entity.setPatientId(params.getPatientId());
        entity.setInpatientId(params.getInpatientId());
        entity.setVisitId(params.getVisitId());
        entity.setDiseaseId(params.getDiseaseId());
        entity.setApplyDoctorId(params.getApplyDoctorId());
        entity.setLaunchDoctorId(params.getLaunchDoctorId());
        entity.setContent(params.getContent());

        consultationLaunchApplyInfoService.updateById(entity);
        // 更新会诊消息通知记录
        // consultationNotifyInfoBizService.addNotify(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void batchUpdateStatus(List<ConsultationLaunchApplyUpdateStatusDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        params.forEach(i -> ValidatorUtils.validateEntity(i));

        List<String> ids = new ArrayList<>();
        Map<String, ConsultationLaunchApplyUpdateStatusDTO> paramsMap = new HashMap<>();
        params.forEach(i -> {
            ids.add(i.getId());
            paramsMap.put(i.getId(), i);
        });

        List<ConsultationLaunchApplyInfo> entityList = consultationLaunchApplyInfoService.listByIds(ids);
        if (CollUtil.isEmpty(entityList)) {
            return;
        }

        List<ConsultationLaunchApplyInfo> list = new ArrayList<>();
        entityList.forEach(i -> {
            if (YN.YES.getCode().equals(i.getRowsStatus())) {
                ConsultationLaunchApplyUpdateStatusDTO param = paramsMap.get(i.getId());
                if (null != param) {
                    i.setConsultationId(param.getConsultationId());
                    i.setStatus(param.getStatus());
                    i.setSubmitTime(param.getSubmitTime());
                    i.setFinishTime(param.getFinishTime());
                    list.add(i);
                }
            }
        });

        if (CollUtil.isNotEmpty(list)) {
            consultationLaunchApplyInfoService.updateBatchById(list);
            // 更新会诊消息通知记录
            consultationNotifyInfoBizService.updateNotify(list);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void launch(ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        // 查询申请
        ConsultationLaunchApplyGetDTO getDTO = new ConsultationLaunchApplyGetDTO();
        getDTO.setId(params.getId());
        ConsultationLaunchApplyVO vo = this.detail(getDTO);

        if (null == vo) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if (ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(vo.getStatus())) {
            throw new BusinessException("会诊已完成");
        }
        if (ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode().equals(vo.getStatus())) {
            throw new BusinessException("发起会诊申请已取消");
        }

        // 消息发送日志
        ConsultationLaunchApplyLog applyLog = new ConsultationLaunchApplyLog();
        applyLog.setId(IdUtil.getSnowflake(1,8).nextIdStr());
        applyLog.setApplyId(vo.getId());
        applyLog.setApplyStatus(vo.getStatus());
        if (YN.YES.getCode().equals(params.getResendFlg())) { // 重发申请时的校验
            if (ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(vo.getStatus())) { // 会诊进行中超过一天
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(vo.getSubmitTime());
                calendar.add(Calendar.DATE, 1);
                Date nextDay = calendar.getTime();
                if (nextDay.compareTo(new Date()) > 0) {
                    throw new BusinessException("会诊进行中");
                }
            }
        } else {
            if (ConsultationLaunchApplyConstant.ApplyStatus.EXPIRE.getCode().equals(vo.getStatus())) {
                throw new BusinessException("会诊过期");
            }
            if (ConsultationLaunchApplyConstant.ApplyStatus.START.getCode().equals(vo.getStatus())) {
                throw new BusinessException("已发起会诊提醒");
            }
            if (ConsultationLaunchApplyConstant.ApplyStatus.START_EXPIRE.getCode().equals(vo.getStatus())) {
                throw new BusinessException("提醒过期未发起会诊");
            }
            if (ConsultationLaunchApplyConstant.ApplyStatus.DOING.getCode().equals(vo.getStatus())) {
                throw new BusinessException("会诊进行中");
            }
        }

        // 更新申请状态
        if (ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode().equals(vo.getStatus())) {
            consultationLaunchApplyInfoService.update(new UpdateWrapper<ConsultationLaunchApplyInfo>()
                    .set("status", ConsultationLaunchApplyConstant.ApplyStatus.START.getCode())
                    .set("apply_time", DateUtil.date())
                    .eq("id", vo.getId())
            );
        }

        // 已发起过还能发起，即多次提醒
        // 发起申请提醒(保存到医生提醒表中对就诊医生/住院医生发起提醒)
        
        Map<String, String> contentMap = new HashMap<>();
        contentMap.put("applyDeptName", vo.getApplyDeptInfo().getName());
        contentMap.put("applyDoctorName", vo.getApplyDoctorInfo().getRealName());
        contentMap.put("patientName", vo.getPatientInfo().getPatientName());
        contentMap.put("content", vo.getContent());

        DoctorRemindInfo remindDoctor = new DoctorRemindInfo();
        remindDoctor.setId(DoctorPushTaskHelper.getDoctorRemindId());
        remindDoctor.setDoctorId(vo.getLaunchDoctorId());
        remindDoctor.setBizId(vo.getId());
        remindDoctor.setBizOneType(PushTaskConstant.OpType.ApplyMDT.getCode()); // 会诊
        remindDoctor.setBizTwoType(PushTaskConstant.OpType.ApplyMDT.getCode()); // 会诊
        remindDoctor.setTitle("发起会诊消息提醒");
        remindDoctor.setContent(JSON.toJSONString(contentMap));
        remindDoctor.setRemark("发起会诊消息提醒"); // TODO FIXME: 使用html或markdown进行消息的配置
        remindDoctor.setPushDate(DateUtil.date());
        remindDoctor.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());
        remindDoctor.setRowsStatus(YN.YES.getCode());
        doctorRemindInfoService.save(remindDoctor);

        // add by li 发送邀请信息
        consultationNotifyInfoBizService.addDoctorRemind(vo.getId(), vo.getApplyDeptInfo().getName(),
                vo.getApplyDoctorInfo().getRealName(),vo.getPatientInfo().getPatientName());

        if (YN.YES.getCode().equals(params.getResendFlg())) { // 重发申请时保存发送日志
            applyLog.setApplyLogType(ConsultationLaunchApplyConstant.ApplyLogType.RESEND.getCode());
            applyLog.setOpUserId(params.getOpUserId());
            applyLog.setOpDate(new Date());
            consultationLaunchApplyLogService.save(applyLog);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveAndLaunch(ConsultationLaunchApplyAddDTO params) {
        ValidatorUtils.validateEntity(params);
        
        // 保存会诊
        ConsultationLaunchApplyVO vo = this.add(params);
        if (null == vo) {
            return;
        }
        // 发起申请提醒(保存到医生提醒表中对就诊医生/住院医生发起提醒)
        ConsultationLaunchApplyGetDTO getDTO = new ConsultationLaunchApplyGetDTO();
        getDTO.setId(vo.getId());
        getDTO.setOpUserId(params.getOpUserId());

        this.launch(getDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void resendLaunch(ConsultationLaunchApplyGetDTO params) {
        ValidatorUtils.validateEntity(params);

        // 保存发起会诊申请信息
        ConsultationLaunchApplyInfo entity = consultationLaunchApplyInfoService.getById(params.getId());
        if (entity == null) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setResendFlg(YN.YES.getCode());
        entity.setResendDate(new Date());
        entity.setResendCount(entity.getResendCount() == null ? 1 : entity.getResendCount() + 1);
        entity.setResendOpUserId(params.getOpUserId());
        consultationLaunchApplyInfoService.updateById(entity);

        // 发起申请提醒(保存到医生提醒表中对就诊医生/住院医生发起提醒)
        ConsultationLaunchApplyGetDTO getDTO = new ConsultationLaunchApplyGetDTO();
        getDTO.setId(entity.getId());
        getDTO.setResendFlg(YN.YES.getCode());
        getDTO.setOpUserId(params.getOpUserId());
        this.launch(getDTO);

    }

    @Override
    public List<ConsultationLaunchApplyVO> listUndoing48H(String searchDate) {
        Date now = DateUtil.parseDateTime(StrUtil.format(searchDate + " {}:00:00", statTime));
        Date threeDayAgo = DateUtil.offsetDay(now, -3);
        Date twoDayAgo = DateUtil.offsetDay(now, -2);
        ConsultationLaunchApplyQueryDTO params = new ConsultationLaunchApplyQueryDTO();
        params.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDOING.getCode());
        params.setApplyStartDate(threeDayAgo);
        params.setApplyEndDate(twoDayAgo);
        List<ConsultationLaunchApplyVO> list = this.list(params);
        desensitizedData(list); // 数据脱敏
        return list;
    }

    @Override
    public List<ConsultationLaunchApplyVO> listUndone48H(String searchDate) {
        Date now = DateUtil.parseDateTime(StrUtil.format(searchDate + " {}:00:00", statTime));
        Date threeDayAgo = DateUtil.offsetDay(now, -3);
        Date twoDayAgo = DateUtil.offsetDay(now, -2);
        ConsultationLaunchApplyQueryDTO params = new ConsultationLaunchApplyQueryDTO();
        params.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode());
        params.setApplyStartDate(threeDayAgo);
        params.setApplyEndDate(twoDayAgo);
        List<ConsultationLaunchApplyVO> list = this.list(params);
        desensitizedData(list); // 数据脱敏
        return list;

    }

    @Override
    public List<ConsultationLaunchApplyVO> listUndone24H(String searchDate) {
        Date now = DateUtil.parseDateTime(StrUtil.format(searchDate + " {}:00:00", statTime));
        Date twoDayAgo = DateUtil.offsetDay(now, -2);
        Date oneDayAgo = DateUtil.offsetDay(now, -1);
        ConsultationLaunchApplyQueryDTO params = new ConsultationLaunchApplyQueryDTO();
        params.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode());
        params.setSubmitStartDate(twoDayAgo);
        params.setSubmitEndDate(oneDayAgo);
        List<ConsultationLaunchApplyVO> list = this.list(params);
        desensitizedData(list); // 数据脱敏
        return list;
    }

    @Override
    public List<ConsultationStatusStatDeptVO> statByDeptRelation(String searchDate) {
        // 实际日期未2天前
        Date now = DateUtil.parseDateTime(StrUtil.format(searchDate + " {}:00:00", statTime));
        Date threeDayAgo = DateUtil.offsetDay(now, -3);
        Date twoDayAgo = DateUtil.offsetDay(now, -2);
        List<ConsultationStatusStatDeptVO> list = consultationMapper.groupByRelationDept(new QueryWrapper<Void>()
                .ge("apply_time", threeDayAgo)
                .lt("apply_time", twoDayAgo)
        );
        return list;
    }


    @Override
    public List<ConsultationLaunchApplyVO> listApply48H(ConsultationLaunchApplyListDTO params) {
        // 实际日期未2天前
        Date now = DateUtil.parseDateTime(StrUtil.format(params.getSearchDate() + " {}:00:00", statTime));
        Date threeDayAgo = DateUtil.offsetDay(now, -3);
        Date twoDayAgo = DateUtil.offsetDay(now, -2);
        ConsultationLaunchApplyQueryDTO queryDTO = new ConsultationLaunchApplyQueryDTO();
        queryDTO.setApplyStartDate(threeDayAgo);
        queryDTO.setApplyEndDate(twoDayAgo);
        queryDTO.setDeptRelationId(params.getDeptRelationId());
        queryDTO.setStatus(params.getStatus());
        queryDTO.setStatuses(params.getStatuses());
        List<ConsultationLaunchApplyVO> list = this.list(queryDTO);
        desensitizedData(list); // 数据脱敏
        return list;
    }

    @Override
    public List<ConsultationApplyTimeOutStat> searchTimeoutStat(Integer days) {
        List<ConsultationLaunchApplyTimeoutStat> list = consultationLaunchApplyTimeoutStatService.list(new QueryWrapper<ConsultationLaunchApplyTimeoutStat>()
                .orderByDesc("stat_date")
                .last(" limit " + days));
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(ConsultationApplyTimeOutStat::new).collect(Collectors.toList());
    }

    @Override
    public void createExamine(ConsultationApplyExamineCreateDTO params) {
        ConsultationLaunchApplyInfo applyInfo = consultationLaunchApplyInfoService.getById(params.getApplyId());
        if (applyInfo == null) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if (!applyInfo.getLaunchDoctorId().equals(params.getOpUserId())) {
            throw new BusinessException("需要申请医生发起申诉");
        }
        if (StrUtil.isNotEmpty(applyInfo.getExamineType())) {
            throw new BusinessException("会诊申请已经发起过申诉");
        }
        applyInfo.setExamineType(params.getExamineType());
        applyInfo.setExamineState(ConsultationLaunchApplyConstant.ExamineState.INIT.getCode());
        applyInfo.setExamineDate(new Date());
        applyInfo.setExamineDescribe(params.getExamineDescribe());
        consultationLaunchApplyInfoService.updateById(applyInfo);
    }

    @Override
    public void auditExamine(ConsultationApplyExamineAuditDTO params) {
        ConsultationLaunchApplyInfo applyInfo = consultationLaunchApplyInfoService.getById(params.getApplyId());
        if (applyInfo == null) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if (!ConsultationLaunchApplyConstant.ExamineState.INIT.getCode().equals(applyInfo.getExamineState())) {
            throw new BusinessException("当前申诉状态不能审核");
        }
        applyInfo.setExamineState(params.getExamineState());
        applyInfo.setExamineAuditInfo(params.getExamineAuditInfo());
        applyInfo.setExamineAuditUser(params.getOpUserId());
        applyInfo.setExamineAuditDate(new Date());
        if (ConsultationLaunchApplyConstant.ExamineType.CANCEL.getCode().equals(applyInfo.getExamineType()) &&
                ConsultationLaunchApplyConstant.ExamineState.PASS.getCode().equals(params.getExamineState())) {
//            if (!ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDOING.getCode().equals(applyInfo.getStatus()) &&
//                    !ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode().equals(applyInfo.getStatus()) &&
//                    !ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(applyInfo.getStatus())) {
//                // 取消会诊申请,超时,进行中,已完成的不更改状态
//                applyInfo.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode());
//            }
            // 申诉通过，会诊取消
            applyInfo.setStatus(ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode());
        }
        consultationLaunchApplyInfoService.updateById(applyInfo);

        // 通知申请人员
        String title = "会诊申诉{}";
        String description = "审批描述:{}";

        try {
            AdminUserBO doctor = adminUserClientService.getUserById(applyInfo.getLaunchDoctorId());
            WecomSendTextcardResp resp = null;
            if (ConsultationLaunchApplyConstant.ExamineState.PASS.getCode().equals(params.getExamineState())) {
                WecomSendTextcardReq textcardReq = new WecomSendTextcardReq(doctor.getUserName(),
                        StrUtil.format(title, "已通过"),
                        StrUtil.format(description, applyInfo.getExamineAuditInfo()),
                        "https://slife.bydsfy.com/doctorwx/#/consultation");
                resp = MdtSendTextCardHttpUtil.send(textcardReq);
            } else if (ConsultationLaunchApplyConstant.ExamineState.REFUSE.getCode().equals(params.getExamineState())) {
                WecomSendTextcardReq textcardReq = new WecomSendTextcardReq(doctor.getUserName(),
                        StrUtil.format(title, "被拒绝"),
                        StrUtil.format(description, applyInfo.getExamineAuditInfo()),
                        "https://slife.bydsfy.com/doctorwx/#/consultation");
                resp = MdtSendTextCardHttpUtil.send(textcardReq);
            }
            if (resp != null) {
                // 保存推送日志
                PushWeixinLogAddDTO pushWxLog = new PushWeixinLogAddDTO();
                pushWxLog.setUserType(UserTypeEnum.Doctor.getCode());
                pushWxLog.setUserId(applyInfo.getLaunchDoctorId());
                pushWxLog.setBizId(applyInfo.getId());
                pushWxLog.setRemark("会诊申诉审核结果");
                if ("200".equals(resp.getCode())) {
                    pushWxLog.setSendStatus(DoctorTaskConstant.PushStatus.Success.getCode());
                    pushWxLog.setErrorCode(DoctorTaskConstant.PushStatus.Success.getCode());
                } else {
                    pushWxLog.setSendStatus(DoctorTaskConstant.PushStatus.Fail.getCode());
                    // 记录报错内容
                    pushWxLog.setErrorCode(resp.getCode());
                    pushWxLog.setErrorMsg(resp.getMessage());
                }
                pushWeixinBizService.addLog(pushWxLog);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    // 脱敏患者信息
    private void desensitizedData(List<ConsultationLaunchApplyVO> list) {
        if (CollUtil.isNotEmpty(list)) {
            list.forEach(vo->{
                if (null != vo.getPatientInfo()) {
                    PatientClientBO pBo = vo.getPatientInfo();
                    pBo.setPatientName(DesensitizedHelper.realName(pBo.getPatientName()));
                    pBo.setPhone(DesensitizedHelper.mobilePhone(pBo.getPhone()));
                    pBo.setSparePhone(DesensitizedHelper.mobilePhone(pBo.getSparePhone()));
                }

            });

        }
    }


    private Map<String, List<ConsultationLaunchApplyLogVO>> applyLogMap(Set<String> applyIds) {
        Map<String, List<ConsultationLaunchApplyLogVO>> map = new HashMap<>();
        List<ConsultationLaunchApplyLog> logList = consultationLaunchApplyLogService.list(new QueryWrapper<ConsultationLaunchApplyLog>().in("apply_id", applyIds));
        if (CollUtil.isNotEmpty(logList)) {
            Set<String> opUserIds=new HashSet<>();
            logList.forEach(item-> opUserIds.add(item.getOpUserId()));
            Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(opUserIds);
            List<ConsultationLaunchApplyLogVO> applyLogs = logList.stream().map(
                    item-> ConsultationHelper.getConsultationLaunchApplyLogVO(item, userMap)).collect(Collectors.toList());
            for (ConsultationLaunchApplyLogVO vo : applyLogs) {
                if (map.containsKey(vo.getApplyId())) {
                    List<ConsultationLaunchApplyLogVO> list = map.get(vo.getApplyId());
                    list.add(vo);
                    map.put(vo.getApplyId(), list);
                } else {
                    List<ConsultationLaunchApplyLogVO> list = new ArrayList<>();
                    list.add(vo);
                    map.put(vo.getApplyId(), list);
                }
            }
        }
        return map;
    }


    // 校验同一患者，住院号，同一种病种的标签发起过会诊，  相同标签的校验
    /**
     * @param patientId   患者id
     * @param applyConsultationLabelId 会诊标签
     * @param applyDeptIds 参加会诊的部门ids
     *
     */
    private boolean repeatApply(String patientId, String applyConsultationLabelId, List<String> applyDeptIds){
        Set<String> consultationLabelIds = new HashSet<>();
        // 手动发起时,校验一天是否已存在同种标签
        List<ConsultationLaunchApplyInfo> applyInfoList = consultationLaunchApplyInfoService.list(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("patient_id", patientId)
                .ge("apply_time",  DateUtil.offsetDay(new Date(), -1))

        );
        if (CollUtil.isNotEmpty(applyInfoList)) {
            applyInfoList.forEach(item->{
                if (StrUtil.isNotEmpty(item.getConsultationLabelId())) {
                    consultationLabelIds.add(item.getConsultationLabelId());
                }
            });
        }
        if (CollUtil.isEmpty(consultationLabelIds)) {
            // 没有同病种的标签
            return false;
        }

        List<Map<String, String>> patientMapList = consultationMapper.standardLabelList(new QueryWrapper<Void>()
                .in("id", consultationLabelIds)
        );
        for (Map<String, String> patientMap : patientMapList) {
            List<String> deptIds = StrJoinSplitUtils.split2List(patientMap.get("consultation_dept_ids"));
            List<String> intersection = applyDeptIds.stream().filter(deptIds::contains).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(intersection)) {
                String consultationLabelId = patientMap.get("id");
                List<ConsultationLaunchApplyInfo> applyInfoList1 = consultationLaunchApplyInfoService.list(new QueryWrapper<ConsultationLaunchApplyInfo>()
                        .eq("patient_id", patientId)
                        .ge("apply_time",  DateUtil.offsetDay(new Date(), -1))
                        .eq("consultation_label_id",consultationLabelId)
                );
                applyInfoList1.forEach(apply->{
                    apply.setExtend1(StrUtil.isEmpty(apply.getExtend1()) ?
                            applyConsultationLabelId : apply.getExtend1() + "," + applyConsultationLabelId);
                });
                consultationLaunchApplyInfoService.updateBatchById(applyInfoList1);
                // 部门存在交集，校验为同病种
                return true;
            }
        }
        return false;
    }
}
