package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyChangeStatInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyChangeStatInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyChangeStatQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyChangeStatSaveDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyChangeStatDetailVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyChangeStatVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyChangeStatBizService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class ConsultationLaunchApplyChangeStatBizServiceImpl implements ConsultationLaunchApplyChangeStatBizService {
    
    @Autowired
    private ConsultationLaunchApplyChangeStatInfoService consultationLaunchApplyChangeStatInfoService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private ConsultationMapper consultationMapper;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private ConsultationBizService consultationBizService;

    @Override
    public PageData<ConsultationLaunchApplyChangeStatVO> query(ConsultationLaunchApplyChangeStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        IPage<ConsultationLaunchApplyChangeStatInfo> page = consultationLaunchApplyChangeStatInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationLaunchApplyChangeStatInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
            .eq(StrUtil.isNotBlank(params.getDataTime()), "data_time", params.getDataTime())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "data_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "data_time", params.getEndDate())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> deptIds = new ArrayList<>();
        List<String> diseaseIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            deptIds.add(i.getDeptId());
            if (StrUtil.isNotBlank(i.getDiseaseId())) {
                diseaseIds.add(i.getDiseaseId());
            }
        });
        
        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyChangeStatVO> data= new PageConverter<ConsultationLaunchApplyChangeStatInfo, ConsultationLaunchApplyChangeStatVO>() {
            @Override
            public ConsultationLaunchApplyChangeStatVO converter(ConsultationLaunchApplyChangeStatInfo entity) {
                ConsultationLaunchApplyChangeStatVO vo = ConsultationHelper.getConsultationLaunchApplyChangeStatVO(entity);

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                if (StrUtil.isNotBlank(vo.getDiseaseId())) {
                    StandardDiseaseBO disease = diseaseMap.get(vo.getDiseaseId());
                    if (null != disease) {
                        vo.setDiseaseName(disease.getName());
                    }
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public PageData<ConsultationLaunchApplyChangeStatVO> queryByDept(ConsultationLaunchApplyChangeStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        IPage<ConsultationLaunchApplyChangeStatInfo> page = consultationLaunchApplyChangeStatInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationLaunchApplyChangeStatInfo>()
            .select("dept_id deptId", "disease_id diseaseId", "SUM(finish_num) finishNum", "SUM(change_num) changeNum", "SUM(change_now_num) changeNowNum",
                    "SUM(out_change_num) outChangeNum", "SUM(therapy_change_num) therapyChangeNum", "SUM(real_change_num) realChangeNum"
            )
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
            .eq(StrUtil.isNotBlank(params.getDataTime()), "data_time", params.getDataTime())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "data_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "data_time", params.getEndDate())
            .groupBy("dept_id", "disease_id")
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> deptIds = new ArrayList<>();
        List<String> diseaseIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            deptIds.add(i.getDeptId());
            if (StrUtil.isNotBlank(i.getDiseaseId())) {
                diseaseIds.add(i.getDiseaseId());
            }
        });
        
        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyChangeStatVO> data= new PageConverter<ConsultationLaunchApplyChangeStatInfo, ConsultationLaunchApplyChangeStatVO>() {
            @Override
            public ConsultationLaunchApplyChangeStatVO converter(ConsultationLaunchApplyChangeStatInfo entity) {
                ConsultationLaunchApplyChangeStatVO vo = ConsultationHelper.getConsultationLaunchApplyChangeStatVO(entity);

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                if (StrUtil.isNotBlank(vo.getDiseaseId())) {
                    StandardDiseaseBO disease = diseaseMap.get(vo.getDiseaseId());
                    if (null != disease) {
                        vo.setDiseaseName(disease.getName());
                    }
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public PageData<ConsultationLaunchApplyChangeStatDetailVO> queryDetailByDept(ConsultationLaunchApplyChangeStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        IPage<ConsultationLaunchApplyChangeStatDetailVO> page = consultationMapper.consultationChangeStatisticPage(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationLaunchApplyChangeStatInfo>()
            .ne("a.status", ConsultationLaunchApplyConstant.ApplyStatus.TODO.getCode())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "b.mdt_dept_code", params.getDeptId())
            .eq(StrUtil.isNotBlank(params.getDiseaseId()), "a.disease_id", params.getDiseaseId())
            .eq(StrUtil.isNotBlank(params.getDataTime()), "a.apply_time", params.getDataTime())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "a.apply_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "a.apply_time", params.getEndDate())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> deptIds = new ArrayList<>();
        List<String> consultationIds = new ArrayList<>();
        List<String> diseaseIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            deptIds.add(i.getDeptId());
            if (StrUtil.isNotBlank(i.getConsultationId())) {
                consultationIds.add(i.getConsultationId());
            }
            if (StrUtil.isNotBlank(i.getDiseaseId())) {
                diseaseIds.add(i.getDiseaseId());
            }
        });
        
        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
        }

        // 会诊信息
        Map<String, ConsultationVO> consultationMap = new HashMap<>();
        if (CollUtil.isNotEmpty(consultationIds)) {
            ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
            consultationQueryDTO.setIds(consultationIds);
            List<ConsultationVO> consultationList = consultationBizService.simpleList(consultationQueryDTO);
            consultationList.forEach(i -> consultationMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyChangeStatDetailVO> data= new PageConverter<ConsultationLaunchApplyChangeStatDetailVO, ConsultationLaunchApplyChangeStatDetailVO>() {
            @Override
            public ConsultationLaunchApplyChangeStatDetailVO converter(ConsultationLaunchApplyChangeStatDetailVO vo) {
                
                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                if (StrUtil.isNotBlank(vo.getDiseaseId())) {
                    StandardDiseaseBO disease = diseaseMap.get(vo.getDiseaseId());
                    if (null != disease) {
                        vo.setDiseaseName(disease.getName());
                    }
                }

                if (StrUtil.isNotBlank(vo.getConsultationId())) {
                    ConsultationVO consultation = consultationMap.get(vo.getConsultationId());
                    if (null != consultation) {
                        vo.setConsultationInfo(consultation);
                        
                        vo.setName(consultation.getName());
                        vo.setRemark(consultation.getRemark());
                        vo.setPatientId(consultation.getPatientId());
                        vo.setInpatientId(consultation.getInpatientId());
                        vo.setVisitId(consultation.getVisitId());
                        vo.setDiseaseId(consultation.getDiseaseId());
                        vo.setApplyTime(consultation.getApplyTime());
                        vo.setApplyDoctorId(consultation.getApplyDoctorId());
                        vo.setApplyDoctorName(consultation.getApplyDoctorName());
                        vo.setApplyDeptId(consultation.getApplyDeptId());
                        vo.setApplyDeptName(consultation.getApplyDeptName());
                        vo.setSubmitDoctorId(consultation.getSubmitDoctorId());
                        vo.setSubmitDoctorName(consultation.getSubmitDoctorName());
                        vo.setSubmitDeptId(consultation.getSubmitDeptId());
                        vo.setSubmitDeptName(consultation.getSubmitDeptName());
                        vo.setWardCode(consultation.getWardCode());
                        vo.setWardName(consultation.getWardName());
                        vo.setOverdueTime(consultation.getOverdueTime());
                        vo.setConsultationTime(consultation.getConsultationTime());
                        vo.setConsultationVenue(consultation.getConsultationVenue());
                        vo.setSummary(consultation.getSummary());
                        vo.setSummaryTime(consultation.getSummaryTime());
                        vo.setSummaryUserId(consultation.getSummaryUserId());
                        vo.setImages(consultation.getImages());
                        vo.setStatus(consultation.getStatus());
                        vo.setApplyType(consultation.getApplyType());
                        vo.setConsultationDoctorList(consultation.getConsultationDoctorList());
                        vo.setHisConsultationAdviceList(consultation.getHisConsultationAdviceList());
                        vo.setJoinDoctorId(consultation.getJoinDoctorId());
                        vo.setJoinDoctorType(consultation.getJoinDoctorType());
                        vo.setTransferDoctorId(consultation.getTransferDoctorId());
                        vo.setJoinStatus(consultation.getJoinStatus());
                        vo.setPatientName(consultation.getPatientName());
                        vo.setPatientPhone(consultation.getPatientPhone());
                        vo.setPatientSex(consultation.getPatientSex());
                        vo.setPatientAge(consultation.getPatientAge());
                    }
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }
    
    @Override
    public void saveBatch(List<ConsultationLaunchApplyChangeStatSaveDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        // 存在的数据更新
        List<String> existParams = new ArrayList<>();
        params.forEach(i -> existParams.add(i.getDataTime() + "_" + i.getDeptId()));

        Map<String, List<ConsultationLaunchApplyChangeStatInfo>> existMap = new HashMap<>();
        List<ConsultationLaunchApplyChangeStatInfo> existList = consultationLaunchApplyChangeStatInfoService.list(new QueryWrapper<ConsultationLaunchApplyChangeStatInfo>()
            .in("CONCAT(data_time, '_', dept_id)", existParams)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> {
                List<ConsultationLaunchApplyChangeStatInfo> l = existMap.get(i.getDataTime() + "_" + i.getDeptId());
                if (null == l) {
                    l = new ArrayList<>();
                    existMap.put(i.getDataTime() + "_" + i.getDeptId(), l);
                }
                l.add(i);
            });
        }
        
        List<ConsultationLaunchApplyChangeStatInfo> saveList = new ArrayList<>();
        List<ConsultationLaunchApplyChangeStatInfo> updateList = new ArrayList<>();
        params.forEach(i -> {
            ConsultationLaunchApplyChangeStatInfo stat = null;

            List<ConsultationLaunchApplyChangeStatInfo> statList = existMap.get(i.getDataTime() + "_" + i.getDeptId());
            if (CollUtil.isNotEmpty(statList)) {
                for (int j = 0; j < statList.size(); j++) {
                    ConsultationLaunchApplyChangeStatInfo item = statList.get(j);
                    if ((StrUtil.isNotBlank(i.getDiseaseId()) && i.getDiseaseId().equals(item.getDiseaseId())) || (StrUtil.isBlank(item.getDiseaseId()) && StrUtil.isBlank(i.getDiseaseId()))) {
                        stat = item;
                    }
                }
                if (null == stat) {
                    stat = new ConsultationLaunchApplyChangeStatInfo();
                    stat.setId(StrUtil.format("CLACS{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                    saveList.add(stat);
                }
                updateList.add(stat);
            } else {
                stat = new ConsultationLaunchApplyChangeStatInfo();
                stat.setId(StrUtil.format("CLACS{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                saveList.add(stat);
            }

            stat.setDataTime(i.getDataTime());
            stat.setDeptId(i.getDeptId());
            stat.setDiseaseId(i.getDiseaseId());
            stat.setFinishNum(i.getFinishNum());
            stat.setChangeNum(i.getChangeNum());
            stat.setChangeNowNum(i.getChangeNowNum());
            stat.setOutChangeNum(i.getOutChangeNum());
            stat.setTherapyChangeNum(i.getTherapyChangeNum());
            stat.setRealChangeNum(i.getRealChangeNum());
            stat.setDataUpdateDate(DateUtil.date());
        });

        if (CollUtil.isNotEmpty(saveList)) {
            consultationLaunchApplyChangeStatInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            consultationLaunchApplyChangeStatInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public ConsultationLaunchApplyChangeStatVO statTotal(ConsultationLaunchApplyChangeStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        ConsultationLaunchApplyChangeStatInfo entity = consultationLaunchApplyChangeStatInfoService.getOne(new QueryWrapper<ConsultationLaunchApplyChangeStatInfo>()
            .select("SUM(finish_num) finishNum", "SUM(change_num) changeNum", "SUM(change_now_num) changeNowNum",
                    "SUM(out_change_num) outChangeNum", "SUM(therapy_change_num) therapyChangeNum", "SUM(real_change_num) realChangeNum"
            )
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
            .eq(StrUtil.isNotBlank(params.getDataTime()), "data_time", params.getDataTime())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "data_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "data_time", params.getEndDate())
        );

        if (null == entity) {
            return null;
        }

        return ConsultationHelper.getConsultationLaunchApplyChangeStatVO(entity);
    }

}
