package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyStatInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyStatInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyStatQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationLaunchApplyStatSaveDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationLaunchApplyStatVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationLaunchApplyStatBizService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class ConsultationLaunchApplyStatBizServiceImpl implements ConsultationLaunchApplyStatBizService {
    
    @Autowired
    private ConsultationLaunchApplyStatInfoService consultationLaunchApplyStatInfoService;
    @Autowired
    private DeptClientService deptClientService;
    
    @Override
    public PageData<ConsultationLaunchApplyStatVO> query(ConsultationLaunchApplyStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        IPage<ConsultationLaunchApplyStatInfo> page = consultationLaunchApplyStatInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationLaunchApplyStatInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .ge(null != params.getStartDate(), "data_time", params.getStartDate())
            .lt(null != params.getEndDate(), "data_time", params.getEndDate())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> deptIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            deptIds.add(i.getDeptId());
        });
        
        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyStatVO> data= new PageConverter<ConsultationLaunchApplyStatInfo, ConsultationLaunchApplyStatVO>() {
            @Override
            public ConsultationLaunchApplyStatVO converter(ConsultationLaunchApplyStatInfo entity) {
                ConsultationLaunchApplyStatVO vo = ConsultationHelper.getConsultationLaunchApplyStatVO(entity);

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public PageData<ConsultationLaunchApplyStatVO> queryByDept(ConsultationLaunchApplyStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        IPage<ConsultationLaunchApplyStatInfo> page = consultationLaunchApplyStatInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationLaunchApplyStatInfo>()
            .select("dept_id deptId", "SUM(start_num) startNum", "SUM(apply_num) applyNum", "SUM(not_apply_num) notApplyNum", 
                    "SUM(examine_num) examineNum", "SUM(finish_num) finishNum", "SUM(change_num) changeNum", "SUM(change_now_num) changeNowNum",
                    "SUM(out_change_num) outChangeNum", "SUM(therapy_change_num) therapyChangeNum", "SUM(real_change_num) realChangeNum"
            )
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "data_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "data_time", params.getEndDate())
            .groupBy("dept_id")
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> deptIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            deptIds.add(i.getDeptId());
        });
        
        // 查询科室信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            deptList.forEach(i -> deptMap.put(i.getId(), i));
        }

        PageData<ConsultationLaunchApplyStatVO> data= new PageConverter<ConsultationLaunchApplyStatInfo, ConsultationLaunchApplyStatVO>() {
            @Override
            public ConsultationLaunchApplyStatVO converter(ConsultationLaunchApplyStatInfo entity) {
                ConsultationLaunchApplyStatVO vo = ConsultationHelper.getConsultationLaunchApplyStatVO(entity);

                DeptBO dept = deptMap.get(vo.getDeptId());
                if (null != dept) {
                    vo.setDeptName(dept.getName());
                }

                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public void saveBatch(List<ConsultationLaunchApplyStatSaveDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        // 存在的数据更新
        List<String> existParams = new ArrayList<>();
        params.forEach(i -> existParams.add(i.getDataTime() + "_" + i.getDeptId()));

        Map<String, ConsultationLaunchApplyStatInfo> existMap = new HashMap<>();
        List<ConsultationLaunchApplyStatInfo> existList = consultationLaunchApplyStatInfoService.list(new QueryWrapper<ConsultationLaunchApplyStatInfo>()
            .in("CONCAT(data_time, '_', dept_id)", existParams)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime() + "_" + i.getDeptId(), i));
        }
        
        List<ConsultationLaunchApplyStatInfo> saveList = new ArrayList<>();
        List<ConsultationLaunchApplyStatInfo> updateList = new ArrayList<>();
        params.forEach(i -> {
            ConsultationLaunchApplyStatInfo stat = existMap.get(i.getDataTime() + "_" + i.getDeptId());
            if (null != stat) {
                updateList.add(stat);
            } else {
                stat = new ConsultationLaunchApplyStatInfo();
                stat.setId(StrUtil.format("CLAS{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                saveList.add(stat);
            }

            stat.setDataTime(i.getDataTime());
            stat.setDeptId(i.getDeptId());
            stat.setStartNum(i.getStartNum());
            stat.setApplyNum(i.getApplyNum());
            stat.setNotApplyNum(i.getNotApplyNum());
            stat.setExamineNum(i.getExamineNum());
            stat.setFinishNum(i.getFinishNum());
            stat.setChangeNum(i.getChangeNum());
            stat.setChangeNowNum(i.getChangeNowNum());
            stat.setOutChangeNum(i.getOutChangeNum());
            stat.setTherapyChangeNum(i.getTherapyChangeNum());
            stat.setRealChangeNum(i.getRealChangeNum());
            stat.setDataUpdateDate(DateUtil.date());
        });

        if (CollUtil.isNotEmpty(saveList)) {
            consultationLaunchApplyStatInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            consultationLaunchApplyStatInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public ConsultationLaunchApplyStatVO statTotal(ConsultationLaunchApplyStatQueryDTO params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            params.setEndDate(DateUtil.offsetDay(DateUtil.parseDate(params.getEndDate()), 1).toDateStr());
        }

        ConsultationLaunchApplyStatInfo entity = consultationLaunchApplyStatInfoService.getOne(new QueryWrapper<ConsultationLaunchApplyStatInfo>()
            .select("SUM(start_num) startNum", "SUM(apply_num) applyNum", "SUM(not_apply_num) notApplyNum", 
                    "SUM(examine_num) examineNum", "SUM(finish_num) finishNum", "SUM(change_num) changeNum", "SUM(change_now_num) changeNowNum",
                    "SUM(out_change_num) outChangeNum", "SUM(therapy_change_num) therapyChangeNum", "SUM(real_change_num) realChangeNum"
            )
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
            .ge(StrUtil.isNotBlank(params.getStartDate()), "data_time", params.getStartDate())
            .lt(StrUtil.isNotBlank(params.getEndDate()), "data_time", params.getEndDate())
        );

        if (null == entity) {
            return null;
        }


        return ConsultationHelper.getConsultationLaunchApplyStatVO(entity);
    }

}
