package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationNotifyConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationNotifyInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationNotifyInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationNotifyQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationNotifyVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationNotifyConvert;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationNotifyInfoBizService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRemindInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRemindInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ConsultationNotifyInfoBizServiceImpl implements ConsultationNotifyInfoBizService {

    @Autowired
    private ConsultationNotifyInfoService consultationNotifyInfoService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private DoctorRemindInfoService doctorRemindInfoService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private ConsultationLaunchApplyInfoService consultationLaunchApplyInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DeptClientService deptClientService;


    @Override
    public void addNotify(ConsultationLaunchApplyInfo applyInfo) {
        List<ConsultationNotifyInfo> entityList = new ArrayList<>();
        // 发起会诊通知
        ConsultationNotifyInfo inviteNotify = new ConsultationNotifyInfo();
        inviteNotify.setId(ConsultationHelper.getConsultationNotifyId());
        inviteNotify.setApplyId(applyInfo.getId());
        inviteNotify.setDoctorId(applyInfo.getLaunchDoctorId());
        inviteNotify.setDiseaseId(applyInfo.getDiseaseId());
        inviteNotify.setConsultationLabelId(applyInfo.getConsultationLabelId());
        inviteNotify.setPatientId(applyInfo.getPatientId());
        inviteNotify.setNotifyInfo(applyInfo.getContent());
        inviteNotify.setNotifyType(ConsultationNotifyConstant.NotifyType.INVITE.getCode());
        inviteNotify.setNotifyStatus(ConsultationNotifyConstant.NotifyStatus.TODO.getCode());
        inviteNotify.setRowsStatus(YN.YES.getCode());
        entityList.add(inviteNotify);

        // 查询会诊标签
        LabelQueryPageDTO dto = new LabelQueryPageDTO();
        dto.setConsultationRemindStatus(YN.YES.getCode());
        if (StrUtil.isNotEmpty(applyInfo.getConsultationLabelId())) {
            dto.setId(applyInfo.getConsultationLabelId());
        } else if (StrUtil.isNotEmpty(applyInfo.getDiseaseId())) {
            dto.setDiseaseId(applyInfo.getDiseaseId());
        } else {
            log.info("ConsultationNotifyInfoBizServiceImpl.addNotify:未配置疾病>>>>>> applyId{}", applyInfo.getId());
            return;
        }
        List<LabelBO> labelBOList = labelClientService.queryList(dto);

        // 参加会诊通知
        if (CollUtil.isNotEmpty(labelBOList)) {
            for (LabelBO bo : labelBOList) {
                if (StrUtil.isNotEmpty(bo.getConsultationManager())) {
                    ConsultationNotifyInfo joinNotify = new ConsultationNotifyInfo();
                    joinNotify.setId(ConsultationHelper.getConsultationNotifyId());
                    joinNotify.setApplyId(applyInfo.getId());
                    joinNotify.setDoctorId(bo.getConsultationManager());
                    joinNotify.setDiseaseId(applyInfo.getDiseaseId());
                    joinNotify.setPatientId(applyInfo.getPatientId());
                    joinNotify.setNotifyInfo(bo.getConsultationMessage());
                    joinNotify.setNotifyType(ConsultationNotifyConstant.NotifyType.JOIN.getCode());
                    joinNotify.setNotifyStatus(ConsultationNotifyConstant.NotifyStatus.TODO.getCode());
                    joinNotify.setRowsStatus(YN.YES.getCode());
                    entityList.add(joinNotify);
                }
            }
        }

        consultationNotifyInfoService.saveBatch(entityList);
    }

    @Override
    public void addNotify(List<ConsultationLaunchApplyInfo> applyInfoList) {
        if (CollUtil.isEmpty(applyInfoList)) {
          return;
        }
        for (ConsultationLaunchApplyInfo applyInfo : applyInfoList) {
            addNotify(applyInfo);
        }
    }

    @Override
    public void updateNotify(ConsultationLaunchApplyInfo applyInfo) {
        List<ConsultationNotifyInfo> list = consultationNotifyInfoService.list(new QueryWrapper<ConsultationNotifyInfo>()
                .eq("apply_id", applyInfo.getId()));
        if (CollUtil.isEmpty(list)) {
            return;
        }
        for (ConsultationNotifyInfo info : list) {
            info.setConsultationId(applyInfo.getConsultationId());
            info.setConsultationStatus(applyInfo.getStatus());
            if (ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode().equals(applyInfo.getStatus())) {
                info.setNotifyStatus(ConsultationNotifyConstant.NotifyStatus.DONE.getCode());
            } else if (null != applyInfo.getFinishTime()) {
                info.setNotifyStatus(ConsultationNotifyConstant.NotifyStatus.DONE.getCode());
            }
        }
        consultationNotifyInfoService.updateBatchById(list);
    }

    @Override
    public void updateNotify(List<ConsultationLaunchApplyInfo> applyInfoList) {
        if (CollUtil.isEmpty(applyInfoList)) {
            return;
        }
        for (ConsultationLaunchApplyInfo applyInfo : applyInfoList) {
            updateNotify(applyInfo);
        }
    }

    @Override
    public void addDoctorRemind(String applyId, String applyDeptName, String applyDoctorName, String patientName) {
        // 添加参与医生的发送任务,只添加了参与会诊的发送,发起会诊的发送在launch()种已经添加
        List<ConsultationNotifyInfo> notifyInfos = consultationNotifyInfoService.list(new QueryWrapper<ConsultationNotifyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("apply_id", applyId)
                .eq("notify_type", ConsultationNotifyConstant.NotifyType.JOIN.getCode())
        );

        if (CollUtil.isEmpty(notifyInfos)) {
            return;
        }
        for (ConsultationNotifyInfo notify : notifyInfos) {
            Map<String, String> contentMap = new HashMap<>();
            contentMap.put("applyDeptName", applyDeptName);
            contentMap.put("applyDoctorName", applyDoctorName);
            contentMap.put("patientName", patientName);
            contentMap.put("content", notify.getNotifyInfo());

            DoctorRemindInfo remindDoctor = new DoctorRemindInfo();
            remindDoctor.setId(DoctorPushTaskHelper.getDoctorRemindId());
            remindDoctor.setDoctorId(notify.getDoctorId());
            remindDoctor.setBizId(notify.getApplyId());
            remindDoctor.setBizOneType(PushTaskConstant.OpType.JoinMDT.getCode()); // 会诊
            remindDoctor.setBizTwoType(PushTaskConstant.OpType.JoinMDT.getCode()); // 会诊
            remindDoctor.setTitle("发现目标患者");
            remindDoctor.setContent(JSON.toJSONString(contentMap));
            remindDoctor.setRemark("发现目标患者");
            remindDoctor.setPushDate(DateUtil.date());
            remindDoctor.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());
            remindDoctor.setRowsStatus(YN.YES.getCode());
            doctorRemindInfoService.save(remindDoctor);
        }

    }

    @Override
    public PageData<ConsultationNotifyVO> query(ConsultationNotifyQueryDTO params) {
        IPage<ConsultationNotifyInfo> page = consultationNotifyInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<ConsultationNotifyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getNotifyType()), "notify_type", params.getNotifyType())
                .eq(StrUtil.isNotBlank(params.getNotifyStatus()), "notify_status", params.getNotifyStatus())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .le(null != params.getEndDate(), "create_date", params.getEndDate())
                .and(StrUtil.isNotEmpty(params.getApplyDoctorId()), i -> i
                        .inSql("apply_id", StrUtil.format("select id from mdt_consultation_launch_apply_info where apply_doctor_id = '{}'", params.getApplyDoctorId()))
                        .eq("notify_type", ConsultationNotifyConstant.NotifyType.INVITE.getCode()))
                .orderByDesc("create_date")
        );
        Set<String> patientId = new HashSet<>();
        Set<String> applyId = new HashSet<>();
        Set<String> applyDoctorId = new HashSet<>();
        Set<String> applyDeptId = new HashSet<>();
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getPatientId())){
                patientId.add(item.getPatientId());
            }
            if(StrUtil.isNotEmpty(item.getApplyId())){
                applyId.add(item.getApplyId());
            }
        });
        // 查询关联的患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientId)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientId));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }
        // 查询关联的申请信息
        Map<String, ConsultationLaunchApplyInfo> applyMap = new HashMap<>();
        Map<String, AdminUserBO> applyDoctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(applyId)) {
            List<ConsultationLaunchApplyInfo> applyList = consultationLaunchApplyInfoService.listByIds(applyId);
            applyList.forEach(apply->{
                applyMap.put(apply.getId(), apply);
                applyDoctorId.add(apply.getApplyDoctorId());
            });
            List<AdminUserBO> applyDoctorList = adminUserClientService.getUserListByIds(new ArrayList<>(applyDoctorId));
            if (CollUtil.isNotEmpty(applyDoctorList)) {
                applyDoctorList.forEach(doctor->{
                    applyDoctorMap.put(doctor.getId(), doctor);
                    applyDeptId.add(doctor.getDeptId());
                });
            }
            if (CollUtil.isNotEmpty(applyDeptId)) {
                deptMap.putAll(deptClientService.getDeptMapByIds(applyDeptId));
            }
        }

        PageData<ConsultationNotifyVO> data = new PageConverter<ConsultationNotifyInfo, ConsultationNotifyVO>() {
            @Override
            public ConsultationNotifyVO converter(ConsultationNotifyInfo entity) {
                return ConsultationNotifyConvert.getConsultationNotifyVO(entity, patientMap, applyMap, applyDoctorMap, deptMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<ConsultationNotifyVO> list(ConsultationNotifyQueryDTO params) {
        List<ConsultationNotifyInfo> list = consultationNotifyInfoService.list(new QueryWrapper<ConsultationNotifyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getDiseaseId()), "disease_id", params.getDiseaseId())
                .eq(StrUtil.isNotBlank(params.getNotifyType()), "notify_type", params.getNotifyType())
                .eq(StrUtil.isNotBlank(params.getNotifyStatus()), "notify_status", params.getNotifyStatus())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .le(null != params.getEndDate(), "create_date", params.getEndDate())
                .orderByDesc("create_date"));
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        Set<String> patientId = new HashSet<>();
        Set<String> applyId = new HashSet<>();
        Set<String> applyDoctorId = new HashSet<>();
        Set<String> applyDeptId = new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getPatientId())){
                patientId.add(item.getPatientId());
            }
            if(StrUtil.isNotEmpty(item.getApplyId())){
                applyId.add(item.getApplyId());
            }
        });
        // 查询关联的患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientId)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientId));
            List<PatientClientBO> patientList = patientClientService.queryPatientList(patientQueryDTO);
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }
        // 查询关联的申请信息
        Map<String, ConsultationLaunchApplyInfo> applyMap = new HashMap<>();
        Map<String, AdminUserBO> applyDoctorMap = new HashMap<>();
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(applyId)) {
            List<ConsultationLaunchApplyInfo> applyList = consultationLaunchApplyInfoService.listByIds(applyId);
            applyList.forEach(apply->{
                applyMap.put(apply.getId(), apply);
                applyDoctorId.add(apply.getApplyDoctorId());
            });
            List<AdminUserBO> applyDoctorList = adminUserClientService.getUserListByIds(new ArrayList<>(applyDoctorId));
            if (CollUtil.isNotEmpty(applyDoctorList)) {
                applyDoctorList.forEach(doctor->{
                    applyDoctorMap.put(doctor.getId(), doctor);
                    applyDeptId.add(doctor.getDeptId());
                });
            }
            if (CollUtil.isNotEmpty(applyDeptId)) {
                deptMap.putAll(deptClientService.getDeptMapByIds(applyDeptId));
            }
        }
        return list.stream().map(item->ConsultationNotifyConvert.getConsultationNotifyVO(item,
                patientMap, applyMap, applyDoctorMap, deptMap)).collect(Collectors.toList());
    }
}
