package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationSignInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationSignInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationSignAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationSignQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationSignVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationSignBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class ConsultationSignBizServiceImpl implements ConsultationSignBizService {

    @Autowired
    private ConsultationSignInfoService consultationSignInfoService;

    @Override
    public void add(ConsultationSignAddDTO params) {
        // 查询是否签到
        ConsultationSignInfo existEntity = consultationSignInfoService.getOne(new QueryWrapper<ConsultationSignInfo>()
            .eq("consultation_id", params.getConsultationId())
            .eq("doctor_id", params.getDoctorId())
        );

        if (null != existEntity) {
            return;
        }

        // 签到
        ConsultationSignInfo entity = new ConsultationSignInfo();
        entity.setId(ConsultationHelper.getConsultationSignId());
        entity.setConsultationId(params.getConsultationId());
        entity.setDoctorId(params.getDoctorId());
        if (null != params.getSignTime()) {
            entity.setSignTime(params.getSignTime());
        } else {
            entity.setSignTime(DateUtil.date());
        }
        entity.setSignWay(params.getSignWay());
        entity.setSignLatitude(params.getSignLatitude());
        entity.setSignLongitude(params.getSignLongitude());
        entity.setSignIp(params.getSignIp());
        consultationSignInfoService.save(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<ConsultationSignAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        // 查询是否签到，签到的过滤
        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> queryParams.add(i.getConsultationId() + "-" + i.getDoctorId()));
        Map<String, ConsultationSignInfo> existMap = new HashMap<>();
        List<ConsultationSignInfo> existList = consultationSignInfoService.list(new QueryWrapper<ConsultationSignInfo>()
            .eq("CONCAT(consultation_id, '-', doctor_id)", queryParams)
        );
        if (CollectionUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getConsultationId() + "-" + i.getDoctorId(), i));
        }
    
        List<ConsultationSignInfo> list = new ArrayList<>();
        params.forEach(i -> {
            ConsultationSignInfo exist = existMap.get(i.getConsultationId() + "-" + i.getDoctorId());
            if (null != exist) {
                return;
            }

            ConsultationSignInfo entity = new ConsultationSignInfo();
            entity.setId(ConsultationHelper.getConsultationSignId());
            entity.setConsultationId(i.getConsultationId());
            entity.setDoctorId(i.getDoctorId());
            if (null != i.getSignTime()) {
                entity.setSignTime(i.getSignTime());
            } else {
                entity.setSignTime(DateUtil.date());
            }
            entity.setSignWay(i.getSignWay());
            entity.setSignLatitude(i.getSignLatitude());
            entity.setSignLongitude(i.getSignLongitude());
            entity.setSignIp(i.getSignIp());

            list.add(entity);
        });

        if (CollectionUtil.isNotEmpty(list)) {
            consultationSignInfoService.saveBatch(list);
        }
    }

    @Override
    public void remove(ConsultationSignAddDTO params) {
        consultationSignInfoService.remove(new QueryWrapper<ConsultationSignInfo>()
            .eq("consultation_id", params.getConsultationId())
            .eq("doctor_id", params.getDoctorId())
        );
    }

    @Override
    public List<ConsultationSignVO> list(ConsultationSignQueryDTO params) {
        List<ConsultationSignInfo> list = consultationSignInfoService.list(new QueryWrapper<ConsultationSignInfo>()
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
            .eq(StrUtil.isNotBlank(params.getSignWay()), "sign_way", params.getSignWay())
            .orderByAsc("id")
        );
        return list.stream().map(ConsultationHelper::getConsultationSignVO).collect(Collectors.toList());
    }

}
