package com.xxh.cloud.healthy.mdt.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.HisConsultationAdviceInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.HisConsultationAdviceInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.HisConsultationAdviceSyncDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.HisConsultationAdviceVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.HisConsultationAdviceBizService;
import com.xxh.cloud.healthy.mdt.third.bean.MdtSyncDataResp;
import com.xxh.cloud.healthy.mdt.third.bean.SlifeMdtApplyDetail;
import com.xxh.cloud.healthy.mdt.third.utils.MdtDetailHttpUtil;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class HisConsultationAdviceBizServiceImpl implements HisConsultationAdviceBizService {

    @Autowired
    private HisConsultationAdviceInfoService hisConsultationAdviceInfoService;

    @Override
    public void add(HisConsultationAdviceAddDTO params) {
        HisConsultationAdviceInfo entity = new HisConsultationAdviceInfo();
        // entity.setId(ConsultationHelper.getHisConsultationAdviceId());
        entity.setId(params.getId());
        entity.setConsultationId(params.getConsultationId());
        entity.setInpatientId(params.getInpatientId());
        entity.setVisitId(params.getVisitId());
        entity.setMdtDeptCode(params.getMdtDeptCode());
        entity.setMdtDeptName(params.getMdtDeptName());
        entity.setDeptCode(params.getDeptCode());
        entity.setDeptName(params.getDeptName());
        entity.setWardCode(params.getWardCode());
        entity.setWardName(params.getWardName());
        entity.setSubmitContent(params.getConsultationId());
        entity.setSubmitXmlContent(params.getSubmitXmlContent());
        entity.setSubmitDoctorId(params.getSubmitDoctorId());
        entity.setSubmitDoctorName(params.getSubmitDoctorName());
        entity.setSubmitTime(params.getSubmitTime());
        entity.setStatus(params.getStatus());
        
        HisConsultationAdviceInfo existEntity = hisConsultationAdviceInfoService.getById(params.getId());
        if (null == existEntity) {
            hisConsultationAdviceInfoService.save(entity);
        } else {
            hisConsultationAdviceInfoService.updateById(entity);
        }

    }

    // TODO 会诊结论更改可能结论id也会变，所以结论id变的数据要新增并删除旧结论
    @Override
    public void addBatch(List<HisConsultationAdviceAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        List<String> ids = new ArrayList<>();
        params.forEach(i -> ids.add(i.getId()));

        // 查询是否存在，存在则更新
        List<HisConsultationAdviceInfo> existList = hisConsultationAdviceInfoService.listByIds(ids);
        Map<String, HisConsultationAdviceInfo> existMap = new HashMap<>();
        existList.forEach(i -> existMap.put(i.getId(), i));

        List<HisConsultationAdviceInfo> updateEntity = new ArrayList<>();
        List<HisConsultationAdviceInfo> addEntity = new ArrayList<>();
        params.forEach(i -> {
            HisConsultationAdviceInfo entity = existMap.get(i.getId());
            if (null == entity) {
                entity = new HisConsultationAdviceInfo();
                entity.setId(i.getId());

                addEntity.add(entity);
            } else {
                updateEntity.add(entity);
            }
            
            entity.setConsultationId(i.getConsultationId());
            entity.setInpatientId(i.getInpatientId());
            entity.setVisitId(i.getVisitId());
            entity.setMdtDeptCode(i.getMdtDeptCode());
            entity.setMdtDeptName(i.getMdtDeptName());
            entity.setDeptCode(i.getDeptCode());
            entity.setDeptName(i.getDeptName());
            entity.setWardCode(i.getWardCode());
            entity.setWardName(i.getWardName());
            entity.setSubmitContent(i.getConsultationId());
            entity.setSubmitDoctorId(i.getSubmitDoctorId());
            entity.setSubmitDoctorName(i.getSubmitDoctorName());
            entity.setSubmitContent(i.getSubmitContent());
            entity.setSubmitXmlContent(i.getSubmitXmlContent());
            entity.setSubmitTime(i.getSubmitTime());
            entity.setStatus(i.getStatus());
            // 随访时间
            entity.setFollowup(i.getFollowup());
        });

        // add or update
        if (CollectionUtil.isNotEmpty(addEntity)) {
            hisConsultationAdviceInfoService.saveBatch(addEntity);
        }
        if (CollectionUtil.isNotEmpty(updateEntity)) {
            hisConsultationAdviceInfoService.updateBatchById(updateEntity);
        }
    }

    @Override
    public PageData<HisConsultationAdviceVO> page(HisConsultationAdviceQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<HisConsultationAdviceInfo> page = hisConsultationAdviceInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<HisConsultationAdviceInfo>()
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
            .in(CollectionUtil.isNotEmpty(params.getConsultationIds()), "consultation_id", params.getConsultationIds())
            .eq(StrUtil.isNotBlank(params.getInpatientId()), "inpatient_id", params.getInpatientId())
            .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
            .eq(StrUtil.isNotBlank(params.getMdtDeptCode()), "mdt_dept_code", params.getMdtDeptCode())
            .like(StrUtil.isNotBlank(params.getMdtDeptName()), "mdt_dept_name", params.getMdtDeptName())
            .eq(StrUtil.isNotBlank(params.getDeptCode()), "dept_code", params.getDeptCode())
            .like(StrUtil.isNotBlank(params.getDeptName()), "dept_name", params.getDeptName())
            .eq(StrUtil.isNotBlank(params.getWardCode()), "ward_code", params.getWardCode())
            .like(StrUtil.isNotBlank(params.getWardName()), "ward_name", params.getWardName())
            .eq(StrUtil.isNotBlank(params.getSubmitDoctorId()), "submit_doctor_id", params.getSubmitDoctorId())
            .like(StrUtil.isNotBlank(params.getSubmitDoctorName()), "submit_doctor_name", params.getSubmitDoctorName())
            .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
            .ge(null != params.getStartDate(), "submit_time", params.getStartDate())
            .lt(null != params.getEndDate(), "submit_time", params.getEndDate())
        );

        PageData<HisConsultationAdviceVO> data = new PageConverter<HisConsultationAdviceInfo, HisConsultationAdviceVO>() {
            @Override
            public HisConsultationAdviceVO converter(HisConsultationAdviceInfo entity) {
                HisConsultationAdviceVO vo = ConsultationHelper.getHisConsultationAdviceVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<HisConsultationAdviceVO> list(HisConsultationAdviceQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<HisConsultationAdviceInfo> list = hisConsultationAdviceInfoService.list(new QueryWrapper<HisConsultationAdviceInfo>()
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
            .in(CollectionUtil.isNotEmpty(params.getConsultationIds()), "consultation_id", params.getConsultationIds())
            .eq(StrUtil.isNotBlank(params.getInpatientId()), "inpatient_id", params.getInpatientId())
            .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
            .eq(StrUtil.isNotBlank(params.getMdtDeptCode()), "mdt_dept_code", params.getMdtDeptCode())
            .like(StrUtil.isNotBlank(params.getMdtDeptName()), "mdt_dept_name", params.getMdtDeptName())
            .eq(StrUtil.isNotBlank(params.getDeptCode()), "dept_code", params.getDeptCode())
            .like(StrUtil.isNotBlank(params.getDeptName()), "dept_name", params.getDeptName())
            .eq(StrUtil.isNotBlank(params.getWardCode()), "ward_code", params.getWardCode())
            .like(StrUtil.isNotBlank(params.getWardName()), "ward_name", params.getWardName())
            .eq(StrUtil.isNotBlank(params.getSubmitDoctorId()), "submit_doctor_id", params.getSubmitDoctorId())
            .like(StrUtil.isNotBlank(params.getSubmitDoctorName()), "submit_doctor_name", params.getSubmitDoctorName())
            .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
            .ge(null != params.getStartDate(), "submit_time", params.getStartDate())
            .lt(null != params.getEndDate(), "submit_time", params.getEndDate())
        );

        return list.stream().map(ConsultationHelper::getHisConsultationAdviceVO).collect(Collectors.toList());
    }

    @Override
    public HisConsultationAdviceVO detail(HisConsultationAdviceQueryDTO params) {
        // 会诊医生只能查看自己的会诊意见（参数：consultation_id、docktor_id）；会诊发起人可以查看所有人的会诊意见（参数：consultation_id、id）
        
        HisConsultationAdviceInfo entity = hisConsultationAdviceInfoService.getOne(new QueryWrapper<HisConsultationAdviceInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getConsultationId()), "consultation_id", params.getConsultationId())
        );

        return ConsultationHelper.getHisConsultationAdviceVO(entity);
    }

    @Override
    public void syncConsultationAdvice(HisConsultationAdviceSyncDTO params) {
        ValidatorUtils.validateEntity(params);

        // 同步海泰会诊结论
        MdtSyncDataResp<SlifeMdtApplyDetail> resultResp = null;
        if (null != params.getStartDate() && null != params.getEndDate()) {
            String startTime = DateUtil.format(params.getStartDate(), "yyyy-MM-dd HH:mm:ss");
            String endTime = DateUtil.format(params.getEndDate(), "yyyy-MM-dd HH:mm:ss");

            resultResp = MdtDetailHttpUtil.getData(startTime, endTime, null);
        } else if (CollectionUtil.isNotEmpty(params.getConsultationIds())) {
            resultResp = MdtDetailHttpUtil.getData(null, null, params.getConsultationIds());
        } else {
            return;
        }
        
        List<SlifeMdtApplyDetail> detailList = resultResp.getData();
        if (CollectionUtil.isEmpty(detailList)) {
            return;
        }

        List<HisConsultationAdviceAddDTO> addList = new ArrayList<>();
        detailList.forEach(i -> {
            HisConsultationAdviceAddDTO addDTO = new HisConsultationAdviceAddDTO();
            addDTO.setId(i.getApplyDetailId());
            addDTO.setConsultationId(i.getApplyId());
            addDTO.setInpatientId(i.getIpid());
            addDTO.setVisitId(i.getPid());
            addDTO.setMdtDeptCode(i.getMdtDeptCode());
            addDTO.setMdtDeptName(i.getMdtDeptName());
            addDTO.setDeptCode(i.getDeptCode());
            addDTO.setDeptName(i.getDeptName());
            addDTO.setWardCode(i.getWardCode());
            addDTO.setWardName(i.getWardName());
            addDTO.setSubmitDoctorId(i.getSubmitDocCode());
            addDTO.setSubmitDoctorName(i.getSubmitDocName());
            addDTO.setSubmitContent(i.getMdtContent());
            addDTO.setSubmitXmlContent(i.getXmlContent());
            addDTO.setSubmitTime(i.getSubmitTime());
            addDTO.setStatus(i.getStatus() + "");

            // 随访时间
            if (null != i.getVisitObject()) {
                addDTO.setFollowup(i.getVisitObject().getVisitNum() + i.getVisitObject().getVisitUnit());
            }

            addList.add(addDTO);
        });

        // 保存
        this.addBatch(addList);
    }

}
