package com.xxh.cloud.healthy.mdt.framework.modules.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.xxh.cloud.healthy.mdt.api.ConsultationClientService;
import com.xxh.cloud.healthy.mdt.api.bean.bo.ConsultationBO;
import com.xxh.cloud.healthy.mdt.api.bean.dto.ConsultationQueryClientDTO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant.ConsultationType;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationQueryDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.vo.ConsultationVO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.helper.ConsultationHelper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;

import cn.hutool.core.collection.CollUtil;

@Service
public class ConsultationClientServiceImpl implements ConsultationClientService {

    @Autowired
    private ConsultationBizService consultationBizService;

    @Override
    public Map<String, ConsultationBO> lastMdtSimpleMap(List<String> patientIds) {
        ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
        consultationQueryDTO.setPatientIds(patientIds);
        consultationQueryDTO.setApplyType(ConsultationType.MDT.getCode());
        List<ConsultationVO> list = consultationBizService.lastSimpleList(consultationQueryDTO);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }

        Map<String, ConsultationBO> map = new HashMap<>();
        list.forEach(i -> {
            map.put(i.getPatientId(), ConsultationHelper.getConsultationBO(i));
        });

        return map;
    }

    @Override
    public Map<String, ConsultationBO> lastOrdinarySimpleMap(List<String> patientIds) {
        ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
        consultationQueryDTO.setPatientIds(patientIds);
        consultationQueryDTO.setApplyType(ConsultationType.ORDINARY.getCode());
        List<ConsultationVO> list = consultationBizService.lastSimpleList(consultationQueryDTO);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }

        Map<String, ConsultationBO> map = new HashMap<>();
        list.forEach(i -> {
            map.put(i.getPatientId(), ConsultationHelper.getConsultationBO(i));
        });

        return map;
    }

    @Override
    public List<ConsultationBO> list(ConsultationQueryClientDTO params) {
        ConsultationQueryDTO consultationQueryDTO = new ConsultationQueryDTO();
        consultationQueryDTO.setId(params.getId());
        consultationQueryDTO.setIds(params.getIds());
        consultationQueryDTO.setPatientId(params.getPatientId());
        consultationQueryDTO.setPatientIds(params.getPatientIds());
        consultationQueryDTO.setStartDate(params.getStartDate());
        consultationQueryDTO.setEndDate(params.getEndDate());
        consultationQueryDTO.setApplyType(params.getApplyType());
        List<ConsultationVO> voList = consultationBizService.simpleList(consultationQueryDTO);

        if (CollUtil.isEmpty(voList)) {
            return new ArrayList<>(0);
        }

        return voList.stream().map(ConsultationHelper::getConsultationBO).collect(Collectors.toList());
    }
    
}
