package com.xxh.cloud.healthy.mdt.framework.modules.task.schedule;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationApplyTaskBizService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationBizTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class ConsultationTask {
    
    @Autowired
    private ConsultationBizTaskService consultationBizTaskService;
    @Autowired
    private ConsultationApplyTaskBizService consultationApplyTaskBizService;
    @Value("${xx.stat.con-apply.stat-time: 18}")
    private String statTime;


    // 定时任务处理过期会诊数据
    @XxlJob("mdtExpireTask")
    public void mdtExpireTask() throws Exception {
        log.info("定时任务处理过期会诊数据开始 >>>>>> ");
        XxlJobHelper.log("定时任务处理过期会诊数据开始 >>>>>> ");

        consultationBizTaskService.consultationExpireTask();

        XxlJobHelper.log("定时任务处理过期会诊数据结束 <<<<<< ");
        log.info("定时任务处理过期会诊数据结束 <<<<<< ");
    }

    // 定时任务同步海泰会诊申请数据
    // 跑历史mdt需要修改basic_params中的MDT_Apply时间
    @XxlJob("syncHisMdtTask")
    public void syncHisMdtTask() throws Exception {
        log.info("定时任务同步海泰会诊申请数据开始 >>>>>> ");
        XxlJobHelper.log("定时任务同步海泰会诊申请数据开始 >>>>>> ");

        consultationBizTaskService.syncHisConsultationApplyTask();

        XxlJobHelper.log("定时任务同步海泰会诊申请数据结束 <<<<<< ");
        log.info("定时任务同步海泰会诊申请数据结束 <<<<<< ");
    }

    // 通过已同步的会诊同步海泰会诊结论数据
    @XxlJob("syncHisMdtResultTask")
    public void syncHisMdtResultTask() throws Exception {
        log.info("同步海泰会诊结论数据开始 >>>>>> ");
        XxlJobHelper.log("同步海泰会诊结论数据开始 >>>>>> ");

        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetDay(now, -2); // 默认两天
        Date endTime = DateUtil.date();

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.parseDateTime(end);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        consultationBizTaskService.syncHisConsultationResultTask(startTime, endTime);

        XxlJobHelper.log("同步海泰会诊结论数据结束 <<<<<< ");
        log.info("同步海泰会诊结论数据结束 <<<<<< ");
    }
    
    // 通过时间同步海泰会诊结论数据
    @XxlJob("syncHisMdtResultByTimeTask")
    public void syncHisMdtResultByTimeTask() throws Exception {
        log.info("通过时间同步海泰会诊结论数据开始 >>>>>> ");
        XxlJobHelper.log("通过时间同步海泰会诊结论数据开始 >>>>>> ");

        // 2024-07-18 12:00:21,2024-07-19 12:00:21
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();

        String startTime = DateUtil.offsetHour(now, -1).toString(); // 默认同步近一小时的结论
        String endTime = now.toString();
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startTime = jobParamArray[0].trim();
            if (jobParamArray.length == 2) {
                endTime = jobParamArray[1].trim();
            }
        }

        consultationBizTaskService.syncHisConsultationResultByTimeTask(startTime, endTime);

        XxlJobHelper.log("通过时间同步海泰会诊结论数据结束 <<<<<< ");
        log.info("通过时间同步海泰会诊结论数据结束 <<<<<< ");
    }

    // zs20240909 医生参加会诊提醒
    @XxlJob("mdtDoctorJoinRemindTask")
    public void mdtDoctorJoinRemindTask() throws Exception {
        log.info("医生参加会诊提醒开始 >>>>>> ");
        XxlJobHelper.log("医生参加会诊提醒开始 >>>>>> ");

        // 2024-07-18 12:00:21,2024-07-19 12:00:21
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();

        String startTime = DateUtil.offsetHour(now, -1).toString(); // 默认提醒近一小时的提醒
        String endTime = now.toString();
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startTime = jobParamArray[0].trim();
            if (jobParamArray.length == 2) {
                endTime = jobParamArray[1].trim();
            }
        }

        consultationBizTaskService.mdtDoctorJoinRemindTask(startTime, endTime);

        XxlJobHelper.log("医生参加会诊提醒结束 <<<<<< ");
        log.info("医生参加会诊提醒结束 <<<<<< ");
    }

    // 定时任务判断是否有医生超时未接收邀请
    @XxlJob("mdtJoinTimeoutTask")
    public void mdtJoinTimeoutTask() throws Exception {
        String jobParam = XxlJobHelper.getJobParam();

        int timeout = 60; // 60分钟
        if (StrUtil.isNotBlank(jobParam)) {
            timeout = Integer.parseInt(jobParam);
        }

        log.info("定时任务判断是否有医生超时未接收邀请开始 >>>>>> 超时时间：{}", timeout);
        XxlJobHelper.log("定时任务判断是否有医生超时未接收邀请开始 >>>>>> 超时时间：{}", timeout);

        consultationBizTaskService.consultationJoinTaskJobHandler(timeout);

        XxlJobHelper.log("定时任务判断是否有医生超时未接收邀请结束 <<<<<< ");
        log.info("定时任务判断是否有医生超时未接收邀请结束 <<<<<< ");
    }

    // 定时任务处理申请发起会诊申请是否已经发起(该申请是否跟海泰发起的会诊匹配，匹配更新状态)
    @XxlJob("consultationLaunchApplyTask")
    public void consultationLaunchApplyTask() throws Exception {
        log.info("定时任务处理申请发起会诊申请是否已经发起开始 >>>>>> ");
        XxlJobHelper.log("定时任务处理申请发起会诊申请是否已经发起开始 >>>>>> ");

        // 格式：
        // 1) 24,10
        // 2) 24,2024-10-01,2024-10-30
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();
        
        int timeout = 24; // 24小时
        // int syncTime = 10; // 10天

        Date startDate = DateUtil.offsetDay(now, -10); // 默认处理10天数据
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            timeout = Integer.parseInt(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                int syncDayNum = Integer.parseInt(jobParamArray[1].trim());
                startDate = DateUtil.offsetDay(now, -syncDayNum);
            } else if (jobParamArray.length == 3) {
                String start = jobParamArray[1].trim();
                String end = jobParamArray[2].trim();
                startDate = DateUtil.parseDate(start);
                endDate = DateUtil.parseDate(end);
            }
        }

        consultationBizTaskService.consultationLaunchApplyTask(timeout, startDate, endDate);

        XxlJobHelper.log("定时任务处理申请发起会诊申请是否已经发起结束 <<<<<< ");
        log.info("定时任务处理申请发起会诊申请是否已经发起结束 <<<<<< ");
    }

    // 定时任务处理申请发起会诊申请是否已经完成(该申请是否跟海泰发起的会诊结论匹配，匹配更新状态)
    @XxlJob("consultationLaunchApplyResultTask")
    public void consultationLaunchApplyResultTask() throws Exception {
        log.info("定时任务处理已申请发起的会诊是否已经有会诊结论开始 >>>>>> ");
        XxlJobHelper.log("定时任务处理已申请发起的会诊是否已经有会诊结论开始 >>>>>> ");

        // 格式：
        // 1) 24,10
        // 2) 24,2024-10-01,2024-10-30
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();
        
        int timeout = 24; // 24小时
        // int syncTime = 10; // 10天

        Date startDate = DateUtil.offsetDay(now, -10); // 默认处理10天数据
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            timeout = Integer.parseInt(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                int syncDayNum = Integer.parseInt(jobParamArray[1].trim());
                startDate = DateUtil.offsetDay(now, -syncDayNum);
            } else if (jobParamArray.length == 3) {
                String start = jobParamArray[1].trim();
                String end = jobParamArray[2].trim();
                startDate = DateUtil.parseDate(start);
                endDate = DateUtil.parseDate(end);
            }
        }

        consultationBizTaskService.consultationLaunchApplyResultTask(timeout, startDate, endDate);

        XxlJobHelper.log("定时任务处理已申请发起的会诊是否已经有会诊结论结束 <<<<<< ");
        log.info("定时任务处理已申请发起的会诊是否已经有会诊结论结束 <<<<<< ");
    }

    // 定时任务扫描需要发起会诊的标签患者
    @XxlJob("labelConsultationLaunchApplyResultTask")
    public void labelConsultationLaunchApplyResultTask() throws Exception {
        log.info("定时任务自动发起会诊提醒开始 >>>>>> ");
        XxlJobHelper.log("定时任务自动发起会诊提醒开始 >>>>>> ");

        // 指定的小时
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();
        Date startTime = DateUtil.parseDate(DateUtil.date().toDateStr());
        Date endTime = now;
        
        if (StrUtil.isNotBlank(jobParam)) {
            Integer timeDuring = Integer.valueOf(jobParam);
            startTime = DateUtil.offsetHour(now, -timeDuring);
        }

        consultationBizTaskService.labelConsultationLaunchApplyResultTask(startTime, endTime);

        XxlJobHelper.log("定时任务自动发起会诊提醒结束 <<<<<< ");
        log.info("定时任务自动发起会诊提醒结束 <<<<<< ");
    }

    @XxlJob("createHtTask")
    public void createHtTask() throws Exception {
        log.info("定时任务创建会诊消息开始 >>>>>> ");
        XxlJobHelper.log("定时任务创建会诊消息开始 >>>>>> ");
        String jobParam = XxlJobHelper.getJobParam();
        int offset = -2;
        if (StrUtil.isNotBlank(jobParam)) {
            offset = -Integer.parseInt(jobParam);
        }
        Date now = new Date();
        Date start = DateUtil.offsetHour(now, offset);
        consultationApplyTaskBizService.createHtTask(start, now);
        XxlJobHelper.log("定时任务创建会诊消息结束 <<<<<< ");
        log.info("定时任务创建会诊消息结束 <<<<<< ");
    }


    @XxlJob("consultationApplyTimeoutRemind")
    public void consultationApplyTimeoutRemind() throws Exception {
        // 48小时未发起会诊统计数量通知盛院长 统计时间每天18:00
        log.info("会诊申请超时48小时提醒开始 >>>>>> ");
        XxlJobHelper.log("会诊申请超时48小时提醒开始 >>>>>> ");
        String jobParam = XxlJobHelper.getJobParam();
        String dateTime;
        String sendFlg;
        if (StrUtil.isNotEmpty(jobParam)) {
            List<String> params = StrJoinSplitUtils.split2List(jobParam);
            dateTime = params.get(0);
            sendFlg = params.get(1);
        } else {
            dateTime = StrUtil.format(DateUtil.formatDate(new Date()) + " {}:00:00", statTime);
            sendFlg = YN.YES.getCode();
        }
        consultationApplyTaskBizService.consultationApplyTimeOutStat(dateTime, sendFlg);
        XxlJobHelper.log("会诊申请超时48小时提醒结束 <<<<<< ");
        log.info("会诊申请超时48小时提醒结束 <<<<<< ");
    }


    // 扫描会诊结论判断会诊是否有会诊目标并记录
    @XxlJob("consultationAdviceTargetTask")
    public void consultationAdviceTargetTask() throws Exception {
        log.info("定时任务处理会诊目标开始 >>>>>> ");
        XxlJobHelper.log("定时任务处理会诊目标开始 >>>>>> ");

        // 2024-07-18 12:00:21,2024-07-19 12:00:21
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();

        Date startTime = DateUtil.offsetHour(now, -1); // 默认查询近一小时的会诊结论
        Date endTime = now;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startTime = DateUtil.parseDateTime(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endTime = DateUtil.parseDateTime(jobParamArray[1].trim());
            }
        }

        consultationBizTaskService.consultationAdviceTargetTask(startTime, endTime);

        XxlJobHelper.log("定时任务处理会诊目标结束 <<<<<< ");
        log.info("定时任务处理会诊目标结束 <<<<<< ");
    }

    // 扫描有会诊目标（只针对转科目标）的会诊并判断是否完成
    @XxlJob("consultationAdviceTargetFinishTask")
    public void consultationAdviceTargetFinishTask() throws Exception {
        log.info("定时任务处理会诊目标是否完成开始 >>>>>> ");
        XxlJobHelper.log("定时任务处理会诊目标是否完成开始 >>>>>> ");

        // 2024-07-18 12:00:21,2024-07-19 12:00:21
        String jobParam = XxlJobHelper.getJobParam();

        Date now = DateUtil.date();

        Date startTime = DateUtil.offsetDay(now, -10); // 默认查询10天的数据
        Date endTime = now;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startTime = DateUtil.parseDateTime(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endTime = DateUtil.parseDateTime(jobParamArray[1].trim());
            }
        }

        consultationBizTaskService.consultationAdviceTargetFinishTask(startTime, endTime);

        XxlJobHelper.log("定时任务处理会诊目标是否完成结束 <<<<<< ");
        log.info("定时任务处理会诊目标是否完成结束 <<<<<< ");
    }

    // 会诊申请统计
    @XxlJob("consultationStatTask")
    public void consultationStatTask() throws Exception {
        log.info("会诊统计开始 >>>>>> ");
        XxlJobHelper.log("会诊统计开始 >>>>>> ");

        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        consultationBizTaskService.consultationStatTask(startDate, endDate);

        XxlJobHelper.log("会诊统计结束 <<<<<< ");
        log.info("会诊统计结束 <<<<<< ");
    }
    
    // 会诊申请转科统计
    @XxlJob("consultationChangeStatTask")
    public void consultationChangeStatTask() throws Exception {
        log.info("会诊转科统计开始 >>>>>> ");
        XxlJobHelper.log("会诊转科统计开始 >>>>>> ");

        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        consultationBizTaskService.consultationChangeStatTask(startDate, endDate);

        XxlJobHelper.log("会诊转科统计结束 <<<<<< ");
        log.info("会诊转科统计结束 <<<<<< ");
    }
}
