package com.xxh.cloud.healthy.mdt.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyInfo;
import com.xxh.cloud.healthy.mdt.framework.modules.base.entity.ConsultationLaunchApplyTimeoutStat;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyInfoService;
import com.xxh.cloud.healthy.mdt.framework.modules.base.service.ConsultationLaunchApplyTimeoutStatService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.mapper.ConsultationMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationDwdBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.cache.ConsultationLaunchApplyCacheService;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationApplyTaskBizService;
import com.xxh.cloud.healthy.mdt.third.bean.MdtHtTaskReq;
import com.xxh.cloud.healthy.mdt.third.bean.MdtHtTaskResp;
import com.xxh.cloud.healthy.mdt.third.bean.WecomSendTextcardReq;
import com.xxh.cloud.healthy.mdt.third.utils.MdtHtTaskApiHelper;
import com.xxh.cloud.healthy.mdt.third.utils.MdtSendTextCardHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class ConsultationApplyTaskBizServiceImpl implements ConsultationApplyTaskBizService {

    @Autowired
    private ConsultationLaunchApplyInfoService consultationLaunchApplyInfoService;
    @Autowired
    private ConsultationLaunchApplyTimeoutStatService consultationLaunchApplyTimeoutStatService;
    @Autowired
    private ConsultationDwdBizService consultationDwdBizService;
    @Autowired
    private ConsultationLaunchApplyCacheService consultationLaunchApplyCacheService;
    @Resource
    private ConsultationMapper consultationMapper;

    @Value("${xx.oa.wecom.toUser:}")
    private String toUser;
    @Value("${xx.oa.wecom.title:}")
    private String title;
    @Value("${xx.oa.wecom.description:}")
    private String description;
    @Value("${xx.oa.wecom.url:}")
    private String url;

    @Override
    public String consultationApplyTimeOutStat(String dateTime, String sendFlg) {
        Date dt = DateUtil.parseDateTime(dateTime);
        Date threeDayAgo = DateUtil.offsetDay(dt, -3);
        Date twoDayAgo = DateUtil.offsetDay(dt, -2);
        Date oneDayAgo = DateUtil.offsetDay(dt, -1);

        // 查询原有的数据
        String statDate = DateUtil.formatDate(dt);
        ConsultationLaunchApplyTimeoutStat stat =
                consultationLaunchApplyTimeoutStatService.getOne(new QueryWrapper<ConsultationLaunchApplyTimeoutStat>().eq("stat_date", statDate));
        if (null == stat) {
            stat = new ConsultationLaunchApplyTimeoutStat();
            stat.setId(IdUtil.getSnowflake(5,10).nextIdStr());
            stat.setStatDate(DateUtil.formatDate(dt));
        }

        // 24小时内申请数量
        long applyTotal = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .lt("apply_time", dt)
                .ge("apply_time", oneDayAgo)
        );
        stat.setApplyTotal((int) applyTotal);

        // 48小时前(2天前)的数据
        long applyTotal48 = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .lt("apply_time", twoDayAgo)
                .ge("apply_time", threeDayAgo)
        );
        stat.setStatDate48(DateUtil.formatDate(twoDayAgo));
        stat.setApplyTotal48((int) applyTotal48);

        // 超过48小时未发起会诊
        long undoing = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                .isNull("submit_time") // submit_time = null 未发起
                .lt("apply_time", twoDayAgo)
                .ge("apply_time", threeDayAgo)
        );
        stat.setUndoing((int) undoing);
        // 更新状态 超时未发起 START -> TIMEOUT_UNDOING
        if (undoing > 0) {
            consultationLaunchApplyInfoService.update(new UpdateWrapper<ConsultationLaunchApplyInfo>()
                    .set("status", ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDOING.getCode())
                    .eq("rows_status", YN.YES.getCode())
                    .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                    .isNull("submit_time") // submit_time = null 未发起 finishTime
                    .lt("apply_time", twoDayAgo)
                    .ge("apply_time", threeDayAgo));
        }
        // 超过48小时发起后未完成的
        long undone48 = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                .isNotNull("submit_time")
                .isNull("finish_time") // submit_time = null and finish_time != null 发起未完成
                .lt("apply_time", twoDayAgo)
                .ge("apply_time", threeDayAgo)
        );
        stat.setUndone48((int) undone48);
        // 更新状态 超时未发起 START -> TIMEOUT_UNDOING
        if (undone48 > 0) {
            consultationLaunchApplyInfoService.update(new UpdateWrapper<ConsultationLaunchApplyInfo>()
                    .set("status", ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode())
                    .eq("rows_status", YN.YES.getCode())
                    .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                    .isNotNull("submit_time")
                    .isNull("finish_time") // submit_time = null and finish_time != null 发起未完成
                    .lt("apply_time", twoDayAgo)
                    .ge("apply_time", threeDayAgo));
        }
        // 发起后24小时未完成的会诊
        long undone = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                .isNull("finish_time") // finish_time = null 未完成
                .lt("submit_time", oneDayAgo)
                .ge("submit_time", twoDayAgo)
        );
        stat.setUndone((int) undone);
        // 更新状态 超时未发起 DOING -> TIMEOUT_UNDONE
        if (undoing > 0) {
            consultationLaunchApplyInfoService.update(new UpdateWrapper<ConsultationLaunchApplyInfo>()
                    .set("status", ConsultationLaunchApplyConstant.ApplyStatus.TIMEOUT_UNDONE.getCode())
                    .eq("rows_status", YN.YES.getCode())
                    .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                    .isNull("finish_time") // finish_time = null 未完成
                    .lt("submit_time", oneDayAgo)
                    .ge("submit_time", twoDayAgo));
        }

        // 24小时内完成
        long done = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("status", ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode())
                .lt("apply_time", dt)
                .ge("apply_time", oneDayAgo)
        );
        stat.setDone((int) done);

        // 超过48小时发起督办的数量
        long resendCount48 = consultationLaunchApplyInfoService.count(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("resend_flg", YN.YES.getCode())
                .lt("apply_time", twoDayAgo)
                .ge("apply_time", threeDayAgo)
        );
        stat.setResendCount48((int) resendCount48);
        // 保存统计数据
        consultationLaunchApplyTimeoutStatService.saveOrUpdate(stat);

        // 通知OA系统
        String key = consultationLaunchApplyCacheService.getUrlKey();
        WecomSendTextcardReq req = new WecomSendTextcardReq(StrJoinSplitUtils.split2List(toUser),
                StrUtil.format(title, stat.getStatDate()),
                StrUtil.format(description, stat.getUndoing(), stat.getUndone()),
                StrUtil.format(url, key));
        if (YN.YES.getCode().equals(sendFlg)) {
            MdtSendTextCardHttpUtil.send(req);
        } else {
            log.info(StrUtil.format("推送OA参数:{}", JSONObject.toJSONString(req)));
        }
        return req.getUrl();
    }

    @Override
    public void createHtTask(Date startDate, Date endDate) {
        List<ConsultationLaunchApplyInfo> applyList = consultationLaunchApplyInfoService.list(new QueryWrapper<ConsultationLaunchApplyInfo>()
                .eq("rows_status", YN.YES.getCode())
//                .ne("status", ConsultationLaunchApplyConstant.ApplyStatus.CANCEL.getCode())
                .and(i->i.isNull("ht_task_state").or().eq("ht_task_state", ""))
                .ge(null != startDate, "apply_time", startDate)
                .lt(null != endDate, "apply_time", endDate)
                .orderByDesc("update_date")
        );
        if (CollUtil.isEmpty(applyList)) {
            return;
        }
        List<ConsultationLaunchApplyInfo> updateList = new ArrayList<>();
        for (ConsultationLaunchApplyInfo applyInfo : applyList) {
            if (StrUtil.isEmpty(applyInfo.getJoinDept())) {
                // 没有设置参加会诊部门
                applyInfo.setHtTaskState(YN.NO.getCode());
                applyInfo.setHtTaskResult(JSONObject.toJSONString(new MdtHtTaskResp("没有设置参加会诊部门")));
                updateList.add(applyInfo);
                continue;
            }
            List<String> joinDept = StrJoinSplitUtils.split2List(applyInfo.getJoinDept());
            List<PatientInpatientRecordClientBO> inpatientRecord =
                    consultationDwdBizService.listPatientInpatientChangeLastDateInfo(applyInfo.getPatientId());
            if (CollUtil.isEmpty(inpatientRecord)) {
                applyInfo.setHtTaskState(YN.NO.getCode());
                applyInfo.setHtTaskResult(JSONObject.toJSONString(new MdtHtTaskResp("患者没有住院号")));
                updateList.add(applyInfo);
                continue;
            }
            // 查询his的部门标签
            List<String> hisDeptIds = consultationMapper.getHisDeptIds(new QueryWrapper<Void>()
                    .in("id", joinDept)
                    .eq("extend4", "1") // 如果海泰部门未开通, 扩展4 会被设置成0
                    .eq("extend5", "1")); // 调用海泰的部门开通状态
            if (CollUtil.isEmpty(hisDeptIds)) {
                applyInfo.setHtTaskState(YN.NO.getCode());
                applyInfo.setHtTaskResult(JSONObject.toJSONString(new MdtHtTaskResp("没有关联到his的部门")));
                continue;
            }
            MdtHtTaskReq req = new MdtHtTaskReq(applyInfo.getId(), inpatientRecord.get(0).getId(), applyInfo.getLaunchDoctorId(),
                    StrJoinSplitUtils.join(hisDeptIds));
            MdtHtTaskResp resp = MdtHtTaskApiHelper.htTaskExternal(req);
            applyInfo.setHtTaskState(resp.getSuccess());
            applyInfo.setHtTaskResult(JSONObject.toJSONString(resp));
            updateList.add(applyInfo);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            consultationLaunchApplyInfoService.updateBatchById(updateList);
        }
    }
}
