package com.xxh.cloud.healthy.mdt.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.common.constant.PushWayEnum;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationConstant.ConsultationType;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.bean.dto.ConsultationDoctorAddDTO;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.biz.service.ConsultationDoctorBizService;
import com.xxh.cloud.healthy.mdt.framework.modules.sync.mapper.MdtSyncQueryMapper;
import com.xxh.cloud.healthy.mdt.framework.modules.task.service.ConsultationBizTaskHandlerService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRemindInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRemindInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class ConsultationBizTaskHandlerServiceImpl implements ConsultationBizTaskHandlerService {

    @Autowired
    private ConsultationBizService consultationBizService;
    @Autowired
    private MdtSyncQueryMapper mdtSyncQueryMapper;
    @Autowired
    private ConsultationDoctorBizService consultationDoctorBizService;
    @Autowired
    private DoctorRemindInfoService doctorRemindInfoService;

    private String MDT_Apply = "MDT_Apply";

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveConsultationAndDoctorAndSendMsg(List<ConsultationAddDTO> addList, String flagTime) {

        // 保存会诊信息
        if (CollectionUtil.isNotEmpty(addList)) {
            consultationBizService.addBatch(addList);

            List<ConsultationDoctorAddDTO> addDoctorList = new ArrayList<>();
            // List<DoctorRemindInfo> doctorRemindInfos = new ArrayList<>();
            // DateTime pushDate = DateUtil.date();
            addList.forEach(i -> {
                // TODO zs20240909 会诊申请时间早于当前时间2小时则不生成提醒
                if (CollectionUtil.isNotEmpty(i.getDoctorList())) {
                    addDoctorList.addAll(i.getDoctorList());
                    
                    // TODO zs20240909 同步时不要直接提醒，可以使用定时任务去提醒（mdtDoctorJoinRemindTask）
                    // if (ConsultationType.MDT.getCode().equals(i.getApplyType())) {
                    //     i.getDoctorList().forEach(j -> {
                    //         // 保存到医生提醒
                    //         // TODO 根据数据的完整性来进行提醒，若数据不完整，可以通过手动编辑再触发提醒
                    //         DoctorRemindInfo remindInfo = this.getDoctorRemindInfo(i, j, pushDate);
                    //         if (null != remindInfo) {
                    //             doctorRemindInfos.add(remindInfo);
                    //         }
                    //     });
                    // }
                }
            });

            // 保存会诊医生信息
            if (CollectionUtil.isNotEmpty(addDoctorList)) {
                consultationDoctorBizService.addBatch(addDoctorList);
            }
            // if (CollectionUtil.isNotEmpty(doctorRemindInfos)) {
            //     doctorRemindInfoService.saveBatch(doctorRemindInfos);
            // }
        }

        // 更新同步标记
        log.info("更新同步标记时间 >>>>>> flagTime={}", flagTime);
        mdtSyncQueryMapper.updateMdtSyncFlag(flagTime, MDT_Apply);


    }

    /**
     * 提醒 
     * 用于企业微信、短信推送
     */
    private DoctorRemindInfo getDoctorRemindInfo(ConsultationAddDTO consultation, ConsultationDoctorAddDTO consultationDoctor, DateTime pushDate) {
        if (null == consultation || StrUtil.isBlank(consultation.getName())) {
            return null;
        }
        
        // TODO 暂时没有时间地点
        // String title = StrUtil.format("【MDT会诊】拟{}，在{}举行{}，邀您参加，并准备相应的病历。如不能参加请安排科室其他人员参加，谢谢！", consultation.getConsultationTime(), consultation.getConsultationVenue(), consultation.getName());
        // String content = StrUtil.format("拟{}，在{}举行{}，邀您参加，并准备相应的病历。如不能参加请安排科室其他人员参加，谢谢！", consultation.getConsultationTime(), consultation.getConsultationVenue(), consultation.getName());
        // String title = StrUtil.format("【MDT会诊】邀您参加{}，请准备相应的病历。如不能参加请安排科室其他人员参加，谢谢！", consultation.getName());
        // String content = StrUtil.format("邀您参加{}，请准备相应的病历。如不能参加请安排科室其他人员参加，谢谢！", consultation.getName());
        // String title = StrUtil.isNotBlank(mdtTitle) ? StrUtil.format(mdtTitle, consultation.getName()) : consultation.getName();
        // String content = StrUtil.isNotBlank(mdtContent) ? StrUtil.format(mdtContent, consultation.getName()) : consultation.getName();

        Map<String, String> contentMap = new HashMap<>();
        contentMap.put("name", consultation.getName());
        contentMap.put("applyDeptName", consultation.getApplyDeptName());
        contentMap.put("applyDoctorName", consultation.getApplyDoctorName());

        DoctorRemindInfo doctorRemindInfo = new DoctorRemindInfo();
        doctorRemindInfo.setId(DoctorPushTaskHelper.getDoctorRemindId());
        doctorRemindInfo.setDoctorId(consultationDoctor.getDoctorId());
        doctorRemindInfo.setBizOneType(PushTaskConstant.OpType.MDT.getCode()); // 会诊
        doctorRemindInfo.setBizId(consultationDoctor.getConsultationId());
        doctorRemindInfo.setBizTwoType(PushTaskConstant.OpType.MDT.getCode()); // 会诊
        doctorRemindInfo.setTitle(consultation.getName());
        doctorRemindInfo.setContent(JSON.toJSONString(contentMap));
        doctorRemindInfo.setRemark("消息提醒");
        doctorRemindInfo.setPushDate(pushDate);
        doctorRemindInfo.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());
        doctorRemindInfo.setRowsStatus(YN.YES.getCode());
        doctorRemindInfo.setPushChannel(PushWayEnum.PCW_WX_WORK.getCode());

        return doctorRemindInfo;
    }

}
