package com.xxh.cloud.healthy.push.framework.modules.biz.mapper;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.client.api.bean.bos.CustomerUserBO;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.PatientSubscribeDataLog;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatScanQRCodeDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.vos.WechatScanQRCodeDayVO;

public interface WechatQRCodeDataBizMapper {
    /**
     * 根据二维码数据id统计扫码客户数量（去重）
     */
    @Select("  SELECT a.qrcode_id AS qrcodeId, COUNT(1) userNum " +
            "  FROM ( " + 
            "    SELECT DISTINCT qrcode_id, open_id " +
            "    FROM hd_patient_subscribe_data_log " +
            "    ${ew.customSqlSegment} " +
            "  ) a " +
            "  GROUP BY a.qrcode_id ")
    List<Map<String, Object>> scanCountByQrcodeId(@Param(Constants.WRAPPER) Wrapper<PatientSubscribeDataLog> wrapper);
    
    /**
     * 根据二维码数据id统计扫码注册客户数量（去重）
     */
    @Select("  SELECT a.qrcode_id AS qrcodeId, COUNT(1) userNum " +
            "  FROM ( " + 
            "    SELECT DISTINCT a.qrcode_id, b.id " +
            "    FROM hd_patient_subscribe_data_log a " +
            "    INNER JOIN cust_customer_user_info b ON b.source_one = 'scan_qrcode' AND b.source_two = a.qrcode_id " +
            "    ${ew.customSqlSegment} " +
            "  ) a " +
            "  GROUP BY a.qrcode_id ")
    List<Map<String, Object>> registerCountByQrcodeId(@Param(Constants.WRAPPER) Wrapper<PatientSubscribeDataLog> wrapper);
    
    /**
     * 根据二维码数据id统计扫码注册客户列表（去重）
     */
    @Select("  SELECT * " +
            "  FROM cust_customer_user_info " + 
            "  WHERE (source_two, id) IN " + 
            "  ( " +
            "    SELECT DISTINCT a.qrcode_id, b.id " +
            "    FROM hd_patient_subscribe_data_log a " +
            "    INNER JOIN cust_customer_user_info b ON b.source_one = 'scan_qrcode' AND b.source_two = a.qrcode_id " +
            "    ${ew.customSqlSegment} " +
            "  ) ")
    IPage<CustomerUserBO> registerListByQrcodeId(Page<PatientSubscribeDataLog> page, @Param(Constants.WRAPPER) Wrapper<PatientSubscribeDataLog> wrapper);
    
    /**
     * 根据二维码数据id统计扫码注册客户数量（去重）
     */
    @Select("  SELECT a.qrcode_id AS qrcodeId, COUNT(1) userNum " +
            "  FROM ( " + 
            "    SELECT DISTINCT a.qrcode_id, b.user_id " +
            "    FROM hd_patient_subscribe_data_log a " +
            "    INNER JOIN cust_customer_family_member_info b ON b.source_one = 'scan_qrcode' AND b.source_two = a.qrcode_id " +
            "    ${ew.customSqlSegment} " +
            "  ) a " +
            "  GROUP BY a.qrcode_id ")
    List<Map<String, Object>> bindFamilyCountByQrcodeId(@Param(Constants.WRAPPER) Wrapper<PatientSubscribeDataLog> wrapper);
    
    /**
     * 根据二维码数据id统计扫码注册客户数量（去重）
     */
    @Select("  SELECT * " +
            "  FROM cust_customer_user_info " + 
            "  WHERE (source_two, id) IN " +
            "  ( " +
            "    SELECT DISTINCT a.qrcode_id, b.user_id " +
            "    FROM hd_patient_subscribe_data_log a " +
            "    INNER JOIN cust_customer_family_member_info b ON b.source_one = 'scan_qrcode' AND b.source_two = a.qrcode_id " +
            "    ${ew.customSqlSegment} " +
            "  ) ")
    IPage<CustomerUserBO> bindFamilyListByQrcodeId(Page<PatientSubscribeDataLog> page, @Param(Constants.WRAPPER) Wrapper<PatientSubscribeDataLog> wrapper);
    
    /**
     * 按天与二维码统计扫码用户数量
     */
    @Select("  SELECT a.qrcode_id AS qrcodeId, a.biz_day AS bizDay, COUNT(1) userNum " +
            "  FROM ( " + 
            "    SELECT DISTINCT qrcode_id, open_id, DATE_FORMAT(create_date, '%Y-%m-%d') biz_day " +
            "    FROM hd_patient_subscribe_data_log " +
            "    ${ew.customSqlSegment} " +
            "  ) a " +
            "  GROUP BY a.qrcode_id, a.biz_day ")
    List<WechatScanQRCodeDayVO> listByQrcodeIdAndDay(@Param(Constants.WRAPPER) Wrapper<WechatScanQRCodeDTO> wrapper);
    
    
    /**
     * TODO 临时生辰二维码源数据
     */
    @Select("SELECT type, name, code FROM wx_promotion_code ")
    List<Map<String, String>> listWxQrCodeSourceData();
    
}
