package com.xxh.cloud.healthy.push.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.company.cloud.sdk.weixin.mp.WxMpMsgManageApi;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgDataModel;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgMiniProgramModel;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgModel;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.bean.ServPushConfigModel;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.PushWeixinLog;
import com.xxh.cloud.healthy.push.framework.modules.base.service.PushWeixinLogService;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.bos.PushWeixinStatusBO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.PushWeixinLogAddDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.SendWeixinDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.config.WechatConfig;
import com.xxh.cloud.healthy.push.framework.modules.biz.config.WechatTemplateConfig;
import com.xxh.cloud.healthy.push.framework.modules.biz.constant.PushWeixinConstant;
import com.xxh.cloud.healthy.push.framework.modules.biz.helper.PushWechatHelper;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.PushWeixinBizService;
import com.xxh.cloud.third.framework.modules.thirdWX.service.ThirdWeixinAccessTokenService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * zs20230924
 */
@Slf4j
@Service
public class PushWeixinBizServiceImpl implements PushWeixinBizService {

    @Autowired
    private ThirdWeixinAccessTokenService thirdWeixinAccessTokenService;
    @Autowired
    private WechatConfig wechatConfig;
    @Autowired
    private PushWeixinLogService pushWeixinLogService;
    @Autowired
    private ParamsClientService paramsClientService;

    // 古老代码已没用
    @Deprecated
    @Override
    public PushWeixinStatusBO sendWechatTemplateMsg(SendWeixinDTO params) {
        log.info("PushWeixinBizServiceImpl:sendWechatTemplateMsg:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        WechatTemplateConfig wechatTemplateConfig = wechatConfig.getPatient();
        if (PushWeixinConstant.UserType.DOCTOR.getCode().equals(params.getUserType())) {
            wechatTemplateConfig = wechatConfig.getDoctor();
        }

        if (null == wechatTemplateConfig) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }

        PushWeixinLogAddDTO logInfo = params.getLogInfo();

        logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Fail.getCode());
        // 获取token
        // zs20230926 从数据库中查询appid与secret
        WxApiConfig wxApiConfig = paramsClientService.getParamConfig(StrUtil.isNotBlank(params.getParamId())? params.getParamId() : wechatTemplateConfig.getParamId(), new WxApiConfig());

        ServPushConfigModel pushConfig = params.getPushConfig();

        SendTempMsgModel sendTempMsgModel = new SendTempMsgModel();
        sendTempMsgModel.setTouser(params.getOpenid());
        sendTempMsgModel.setTemplate_id(StrUtil.isNotBlank(params.getTempId())? params.getTempId() : wechatTemplateConfig.getTemplateId());
        
        if (StrUtil.isNotBlank(wechatTemplateConfig.getMiniAppid()) && StrUtil.isNotBlank(pushConfig.getUrl())) {
            SendTempMsgMiniProgramModel sendTempMsgMiniProgramModel = new SendTempMsgMiniProgramModel();
            sendTempMsgMiniProgramModel.setAppid(wechatTemplateConfig.getMiniAppid());
            sendTempMsgMiniProgramModel.setPagepath(pushConfig.getUrl());
            sendTempMsgModel.setMiniprogram(sendTempMsgMiniProgramModel);
        } else if (StrUtil.isNotBlank(pushConfig.getUrl())) {
            sendTempMsgModel.setUrl(pushConfig.getUrl());
        }

        // zs20231010
        Map<String, SendTempMsgDataModel> tempMsgMap = JSONObject.parseObject(this.msgStr(wechatTemplateConfig.getTemplate(), pushConfig), new TypeReference<Map<String, SendTempMsgDataModel>>() {});

        sendTempMsgModel.setData(tempMsgMap);

        // 发送模版消息
        try {
            log.info("PushWeixinBizServiceImpl:sendWechatTemplateMsg:wxApiConfig:>>>>>> {}", JSON.toJSONString(wxApiConfig));
            String accessToken = thirdWeixinAccessTokenService.getAccessToken(wxApiConfig);

            log.info("PushWeixinBizServiceImpl:sendWechatTemplateMsg:sendTempMsgModel:>>>>>> {}, {}", accessToken, JSON.toJSONString(sendTempMsgModel));
            SendTempMsgResp sendTempMsgResp = WxMpMsgManageApi.sendTempMsg(accessToken, sendTempMsgModel);

            logInfo.setErrorCode(sendTempMsgResp.getErrcode() + "");
            logInfo.setErrorMsg(sendTempMsgResp.getErrmsg());
            if (sendTempMsgResp.getErrcode() != 0) {
                logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Fail.getCode());
                logInfo.setSendResult("fail");
            } else {
                logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Success.getCode());
                logInfo.setSendResult("success");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        this.addLog(logInfo);

        PushWeixinStatusBO pushWeixinStatusBO = new PushWeixinStatusBO();
        pushWeixinStatusBO.setRemindId(logInfo.getRemindId());
        pushWeixinStatusBO.setSendStatus(logInfo.getSendStatus());


        log.info("PushWeixinBizServiceImpl:sendWechatTemplateMsg:pushWeixinStatusBO:<<<<<< {}", JSON.toJSONString(pushWeixinStatusBO));
        return pushWeixinStatusBO;
    }

    private String msgStr(String templateStr, ServPushConfigModel model) {
        return templateStr.replace("#title", null == model.getTitle() ? "" : model.getTitle())
                        .replace("#content", null == model.getContent() ? "" : model.getContent())
                        .replace("#date", null == model.getDate() ? "" : model.getDate())
                        .replace("#url", null == model.getUrl() ? "" : model.getUrl())
                        .replace("#remark", null == model.getRemark() ? "" : model.getRemark())
                        .replace("#doctor", null == model.getDoctor() ? "" : model.getDoctor());
    }

    // TODO
    private String urlStr(String templateStr, SendWeixinDTO model) {
        return templateStr.replace("#id", null == model.getMediaId() ? "" : model.getMediaId())
                        .replace("#type", null == model.getMediaType() ? "" : model.getMediaType())
                        .replace("#key", null == model.getSecureData() ? "" : model.getSecureData());
    }

    // 古老代码已没用
    @Deprecated
    @Override
    public List<PushWeixinStatusBO> sendBatchWechatTemplateMsg(List<SendWeixinDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return new ArrayList<>(0);
        }

        List<PushWeixinStatusBO> pushWeixinStatusBOs = new ArrayList<>();
        List<PushWeixinLogAddDTO> pushWeixinLogAddDTOs = new ArrayList<>(); 
        params.forEach(i -> {
            log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:SendWeixinDTO:>>>>>> {}", JSON.toJSONString(i));

            WechatTemplateConfig wechatTemplateConfig = wechatConfig.getPatient();
            if (PushWeixinConstant.UserType.DOCTOR.getCode().equals(i.getUserType())) {
                wechatTemplateConfig = wechatConfig.getDoctor();
            }

            log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:WechatTemplateConfig:>>>>>> {}", JSON.toJSONString(wechatTemplateConfig));
            if (null == wechatTemplateConfig) {
                throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
            }

            PushWeixinLogAddDTO logInfo = i.getLogInfo();
            log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:PushWeixinLogAddDTO:>>>>>> {}", JSON.toJSONString(logInfo));
            if (null == logInfo || null == i.getPushConfig()) {
                return;
            }

            if (StrUtil.isBlank(logInfo.getWxAppId())) {
                logInfo.setWxAppId(wechatTemplateConfig.getAppid());
            }

            logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Fail.getCode());
            logInfo.setSendResult("fail");

            if (StrUtil.isNotBlank(i.getOpenid())) {
                // 获取token
                // zs20230926 从数据库中查询appid与secret
                WxApiConfig wxApiConfig = paramsClientService.getParamConfig(StrUtil.isNotBlank(i.getParamId())? i.getParamId() : wechatTemplateConfig.getParamId(), new WxApiConfig());

                ServPushConfigModel pushConfig = i.getPushConfig();

                SendTempMsgModel sendTempMsgModel = new SendTempMsgModel();
                sendTempMsgModel.setTouser(i.getOpenid());
                sendTempMsgModel.setTemplate_id(StrUtil.isNotBlank(i.getTempId()) ? i.getTempId() : wechatTemplateConfig.getTemplateId());

                log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:ServPushConfigModel:>>>>>> {}", JSON.toJSONString(pushConfig));

                if (StrUtil.isNotBlank(wechatTemplateConfig.getMiniAppid()) && StrUtil.isNotBlank(pushConfig.getUrl())) {
                    SendTempMsgMiniProgramModel sendTempMsgMiniProgramModel = new SendTempMsgMiniProgramModel();
                    sendTempMsgMiniProgramModel.setAppid(wechatTemplateConfig.getMiniAppid());
                    sendTempMsgMiniProgramModel.setPagepath(pushConfig.getUrl());
                    sendTempMsgModel.setMiniprogram(sendTempMsgMiniProgramModel);
                } else if (StrUtil.isNotBlank(pushConfig.getUrl())) {
                    sendTempMsgModel.setUrl(pushConfig.getUrl());
                }

                Map<String, SendTempMsgDataModel> tempMsgMap = JSONObject.parseObject(this.msgStr(wechatTemplateConfig.getTemplate(), pushConfig), new TypeReference<Map<String, SendTempMsgDataModel>>() {});

                log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:tempMsgMap: >>>>>> {}", JSON.toJSONString(tempMsgMap));
                
                sendTempMsgModel.setData(tempMsgMap);

                // 微信跳转url
                if (StrUtil.isNotBlank(wechatTemplateConfig.getUrl()) && StrUtil.isBlank(wechatTemplateConfig.getMiniAppid())) {
                    sendTempMsgModel.setUrl(this.urlStr(wechatTemplateConfig.getUrl(), i));
                }
                log.info("1.PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:sendTempMsgModel: >>>>>> {}", JSON.toJSONString(sendTempMsgModel));

                // 发送模版消息
                try {
                    // TODO redis中token过期时间有问题，要小于token最大过期时间
                    log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:wxApiConfig:>>>>>> {}", JSON.toJSONString(wxApiConfig));
                    String accessToken = thirdWeixinAccessTokenService.getAccessToken(wxApiConfig);
                    log.info("2.PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:sendTempMsgModel:>>>>>> {}", JSON.toJSONString(sendTempMsgModel));
                    SendTempMsgResp sendTempMsgResp = WxMpMsgManageApi.sendTempMsg(accessToken, sendTempMsgModel);
                    log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:sendTempMsgResp:<<<<<< {}", JSON.toJSONString(sendTempMsgResp));

                    logInfo.setErrorCode(sendTempMsgResp.getErrcode() + "");
                    logInfo.setErrorMsg(sendTempMsgResp.getErrmsg());
                    if (sendTempMsgResp.getErrcode() != 0) {
                        logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Fail.getCode());
                        logInfo.setSendResult("fail");
                    } else {
                        logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Success.getCode());
                        logInfo.setSendResult("success");
                    }
                } catch (Exception e) {
                    log.info("exception >>>>>>>>>>>>>>>>>>>>>>  {}", e.getMessage());
                    e.printStackTrace();
                }

            } else {
                logInfo.setSendStatus(PushWeixinConstant.WechatPushStatus.Invalid.getCode());
                logInfo.setSendResult("invalid");
            }

            pushWeixinLogAddDTOs.add(logInfo);

            PushWeixinStatusBO pushWeixinStatusBO = new PushWeixinStatusBO();
            pushWeixinStatusBO.setRemindId(logInfo.getRemindId());
            pushWeixinStatusBO.setSendStatus(logInfo.getSendStatus());
            pushWeixinStatusBO.setErrorCode(logInfo.getErrorCode());
            pushWeixinStatusBO.setErrorMsg(logInfo.getErrorMsg());
            pushWeixinStatusBOs.add(pushWeixinStatusBO);
        });

        // 保存日志
        // 添加异常捕获，防止报错无法更新微信推送状态，导致下次还会推送
        if (CollectionUtil.isNotEmpty(pushWeixinLogAddDTOs)) {
            try {
                this.addBatchLog(pushWeixinLogAddDTOs);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        
        log.info("PushWeixinBizServiceImpl:sendBatchWechatTemplateMsg:pushWeixinStatusBOs:<<<<<< {}", JSON.toJSONString(pushWeixinStatusBOs));
        // 返回发送状态
        return pushWeixinStatusBOs;
    }

    @Override
    public void addLog(PushWeixinLogAddDTO params) {
        log.info("PushWeixinBizServiceImpl:addLog:params:>>>>>> {}", JSON.toJSONString(params));
        DateTime now = DateTime.now();

        PushWeixinLog pushWeixinLog = new PushWeixinLog();
        pushWeixinLog.setId(PushWechatHelper.getPushWeixinLogId());
        pushWeixinLog.setUserType(params.getUserType());
        pushWeixinLog.setUserId(params.getUserId());
        pushWeixinLog.setOpenId(params.getOpenId());
        pushWeixinLog.setWxAppId(params.getWxAppId());
        pushWeixinLog.setBizOneType(params.getBizOneType());
        pushWeixinLog.setBizTwoType(params.getBizTwoType());
        pushWeixinLog.setBizId(params.getBizId());
        pushWeixinLog.setTempId(params.getTempId());
        pushWeixinLog.setRemindId(params.getRemindId());
        pushWeixinLog.setSendStatus(params.getSendStatus());
        pushWeixinLog.setSendResult(params.getSendResult());
        pushWeixinLog.setFirstSendDt(now);
        pushWeixinLog.setLastSendDt(now);
        pushWeixinLog.setSendNum(1);
        pushWeixinLog.setRemark(params.getRemark());
        pushWeixinLog.setRowsStatus(YN.YES.getCode());

        pushWeixinLog.setErrorCode(params.getErrorCode());
        pushWeixinLog.setErrorMsg(params.getErrorMsg());

        pushWeixinLogService.save(pushWeixinLog);
    }

    @Override
    public void addBatchLog(List<PushWeixinLogAddDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        List<PushWeixinLog> list = new ArrayList<>();

        params.forEach(i -> {
            log.info("PushWeixinBizServiceImpl:addBatchLog:i:>>>>>> {}", JSON.toJSONString(i));
            DateTime now = DateTime.now();
            
            PushWeixinLog pushWeixinLog = new PushWeixinLog();
            pushWeixinLog.setId(PushWechatHelper.getPushWeixinLogId());
            pushWeixinLog.setUserType(i.getUserType());
            pushWeixinLog.setUserId(i.getUserId());
            pushWeixinLog.setOpenId(i.getOpenId());
            pushWeixinLog.setWxAppId(i.getWxAppId());
            pushWeixinLog.setBizOneType(i.getBizOneType());
            pushWeixinLog.setBizTwoType(i.getBizTwoType());
            pushWeixinLog.setBizId(i.getBizId());
            pushWeixinLog.setTempId(i.getTempId());
            pushWeixinLog.setRemindId(i.getRemindId());
            pushWeixinLog.setSendStatus(i.getSendStatus());
            pushWeixinLog.setSendResult(i.getSendResult());
            pushWeixinLog.setFirstSendDt(now);
            pushWeixinLog.setLastSendDt(now);
            pushWeixinLog.setSendNum(1);
            pushWeixinLog.setRemark(i.getRemark());
            pushWeixinLog.setRowsStatus(YN.YES.getCode());

            pushWeixinLog.setErrorCode(i.getErrorCode());
            pushWeixinLog.setErrorMsg(i.getErrorMsg());

            list.add(pushWeixinLog);
        });

        pushWeixinLogService.saveBatch(list);
    }

}
