package com.xxh.cloud.healthy.push.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.client.api.bean.bos.CustomerUserBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.PatientSubscribeDataLog;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.WechatQRCodeDataInfo;
import com.xxh.cloud.healthy.push.framework.modules.base.service.WechatQRCodeDataInfoService;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataAddDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataGetDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataQueryDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataUpdateDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeUserQueryDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatScanQRCodeDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.vos.WechatQRCodeDataVO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.vos.WechatScanQRCodeDayVO;
import com.xxh.cloud.healthy.push.framework.modules.biz.helper.PushWechatHelper;
import com.xxh.cloud.healthy.push.framework.modules.biz.mapper.WechatQRCodeDataBizMapper;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WechatQRCodeDataBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class WechatQRCodeDataBizServiceImpl implements WechatQRCodeDataBizService {

    @Autowired
    private WechatQRCodeDataInfoService wechatQRCodeDataService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private WechatQRCodeDataBizMapper wechatQRCodeDataBizMapper;
    
    @Override
    public void save(WechatQRCodeDataAddDTO params) {
        log.info("保存生成二维码日志>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        WechatQRCodeDataInfo entity = new WechatQRCodeDataInfo();

        if (StrUtil.isBlank(params.getId())) {
            entity.setId(PushWechatHelper.getQRCodeDataInfoId());
        } else {
            entity.setId(params.getId());
        }
        entity.setSceneType(params.getSceneType());
        entity.setUserId(params.getUserId());
        entity.setOwnerType(params.getOwnerType());
        entity.setOwnerCode(params.getOwnerCode());
        entity.setOwnerName(params.getOwnerName());
        entity.setQrType(params.getQrType());
        entity.setScene(params.getScene());
        entity.setTicket(params.getTicket());
        entity.setUrl(params.getUrl());
        entity.setExpireSeconds(params.getExpireSeconds());
        entity.setRowsStatus(YN.YES.getCode());

        DateTime now = DateUtil.date();
        if (null != params.getExpireSeconds() && params.getExpireSeconds() > 0) {
            entity.setExpireDate(DateUtil.offsetSecond(now, params.getExpireSeconds()));
        }
        entity.setCreateDate(now);

        wechatQRCodeDataService.save(entity);
    }

    @Override
    public PageData<WechatQRCodeDataVO> page(WechatQRCodeDataQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        Date now = DateUtil.date();
        IPage<WechatQRCodeDataInfo> entityPage = wechatQRCodeDataService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getSceneType()), "scene_type", params.getSceneType())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypes()), "scene_type", params.getSceneTypes())
                .eq(StrUtil.isNotBlank(params.getUserId()), "user_id", params.getUserId())
                .in(CollectionUtil.isNotEmpty(params.getUserIds()), "user_id", params.getUserIds())
                .eq(StrUtil.isNotBlank(params.getOwnerType()), "owner_type", params.getOwnerType())
                .eq(StrUtil.isNotBlank(params.getOwnerCode()), "owner_code", params.getOwnerCode())
                .like(StrUtil.isNotBlank(params.getOwnerName()), "owner_name", params.getOwnerName())
                .eq(StrUtil.isNotBlank(params.getQrType()), "qr_type", params.getQrType())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .ge(YN.NO.getCode().equals(params.getIsExpired()), "expire_date", now)
                .lt(YN.YES.getCode().equals(params.getIsExpired()), "expire_date", now)
                .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(entityPage.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> qrcodeIds = new ArrayList<>();
        Set<String> userIds = new HashSet<>();
        entityPage.getRecords().forEach(i -> {
            qrcodeIds.add(i.getId());
            if (StrUtil.isNotBlank(i.getUserId())) {
                userIds.add(i.getUserId());
            }
        });
        // 根据userid查询本系统用户信息
        Map<String, AdminUserBO> adminUserMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(userIds)) {
            Map<String, AdminUserBO> adminUserBOMap = adminUserClientService.getUserMapByIds(userIds);
            if (CollectionUtil.isNotEmpty(adminUserBOMap)) {
                adminUserMap.putAll(adminUserBOMap);
            }
        }

        // 根据id查询本二维码被扫过的openid列表（用于确认本二维码多少人扫过）
        Map<String, Long> scanCountMap = new HashMap<>();
        List<Map<String, Object>> scanCount = wechatQRCodeDataBizMapper.scanCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .in("qrcode_id", qrcodeIds)
        );
        if (CollectionUtil.isNotEmpty(scanCount)) {
            scanCount.forEach(i -> {
                String qrcodeId = String.valueOf(i.get("qrcodeId"));
                if (StrUtil.isNotBlank(qrcodeId)) {
                    Object userNumObj = i.get("userNum");
                    Long userNum = 0L;
                    if (null != userNumObj) {
                        userNum = Long.valueOf(String.valueOf(userNumObj));
                    }
                    scanCountMap.put(qrcodeId, userNum);
                }
            });
        }

        // 根据id查询客户注册信息列表（用于确认本二维码多少人扫码并注册过）
        Map<String, Long> registerCountMap = new HashMap<>();
        List<Map<String, Object>> registerCount = wechatQRCodeDataBizMapper.registerCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .in("qrcode_id", qrcodeIds)
        );
        if (CollectionUtil.isNotEmpty(registerCount)) {
            registerCount.forEach(i -> {
                String qrcodeId = String.valueOf(i.get("qrcodeId"));
                if (StrUtil.isNotBlank(qrcodeId)) {
                    Object userNumObj = i.get("userNum");
                    Long userNum = 0L;
                    if (null != userNumObj) {
                        userNum = Long.valueOf(String.valueOf(userNumObj));
                    }
                    registerCountMap.put(qrcodeId, userNum);
                }
            });
        }

        // 根据id查询家属关系表（用于确认本二维码多少人扫码并绑定过家属）
        Map<String, Long> bindFamilyCountMap = new HashMap<>();
        List<Map<String, Object>> bindFamilyCount = wechatQRCodeDataBizMapper.bindFamilyCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .in("qrcode_id", qrcodeIds)
        );
        if (CollectionUtil.isNotEmpty(bindFamilyCount)) {
            bindFamilyCount.forEach(i -> {
                String qrcodeId = String.valueOf(i.get("qrcodeId"));
                if (StrUtil.isNotBlank(qrcodeId)) {
                    Object userNumObj = i.get("userNum");
                    Long userNum = 0L;
                    if (null != userNumObj) {
                        userNum = Long.valueOf(String.valueOf(userNumObj));
                    }
                    bindFamilyCountMap.put(qrcodeId, userNum);
                }
            });
        }

        PageData<WechatQRCodeDataVO> data = new PageConverter<WechatQRCodeDataInfo, WechatQRCodeDataVO>() {
            @Override
            public WechatQRCodeDataVO converter(WechatQRCodeDataInfo entity) {
                WechatQRCodeDataVO vo = PushWechatHelper.getWechatQRCodeDataVO(entity);

                if (StrUtil.isNotBlank(vo.getUserId())) {
                    AdminUserBO adminUserBO = adminUserMap.get(vo.getUserId());
                    if (null != adminUserBO) {
                        vo.setUserName(adminUserBO.getUserName());
                        vo.setRealName(adminUserBO.getRealName());
                        vo.setMobile(adminUserBO.getMobile());
                        vo.setEmail(adminUserBO.getEmail());
                        vo.setDeptId(adminUserBO.getDeptId());
                    }
                }

                Long scanCount = scanCountMap.get(vo.getId());
                if (null != scanCount) {
                    vo.setScanCount(scanCount);
                }
                Long registerCount = registerCountMap.get(vo.getId());
                if (null != registerCount) {
                    vo.setRegisterCount(registerCount);
                }
                Long bindFamilyCount = bindFamilyCountMap.get(vo.getId());
                if (null != bindFamilyCount) {
                    vo.setBindFamilyCount(bindFamilyCount);
                }
                return vo;
            }
        }.getPageData(entityPage);
        return data;
    }

    @Override
    public List<WechatQRCodeDataVO> list(WechatQRCodeDataQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        Date now = DateUtil.date();
        List<WechatQRCodeDataInfo> list = wechatQRCodeDataService.list(new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getSceneType()), "scene_type", params.getSceneType())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypes()), "scene_type", params.getSceneTypes())
                .eq(StrUtil.isNotBlank(params.getUserId()), "user_id", params.getUserId())
                .in(CollectionUtil.isNotEmpty(params.getUserIds()), "user_id", params.getUserIds())
                .eq(StrUtil.isNotBlank(params.getOwnerType()), "owner_type", params.getOwnerType())
                .eq(StrUtil.isNotBlank(params.getOwnerCode()), "owner_code", params.getOwnerCode())
                .like(StrUtil.isNotBlank(params.getOwnerName()), "owner_name", params.getOwnerName())
                .eq(StrUtil.isNotBlank(params.getQrType()), "qr_type", params.getQrType())
                .ge(null != params.getStartDate(), "create_date", params.getStartDate())
                .lt(null != params.getEndDate(), "create_date", params.getEndDate())
                .ge(YN.NO.getCode().equals(params.getIsExpired()), "expire_date", now)
                .lt(YN.YES.getCode().equals(params.getIsExpired()), "expire_date", now)
                .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        // 根据userid查询本系统用户信息
        Set<String> userIds = new HashSet<>();
        list.forEach(i -> {
            if (StrUtil.isNotBlank(i.getUserId())) {
                userIds.add(i.getUserId());
            }
        });
        Map<String, AdminUserBO> adminUserMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(userIds)) {
            Map<String, AdminUserBO> adminUserBOMap = adminUserClientService.getUserMapByIds(userIds);
            if (CollectionUtil.isNotEmpty(adminUserBOMap)) {
                adminUserMap.putAll(adminUserBOMap);
            }
        }

        return list.stream().map(i -> {
            WechatQRCodeDataVO vo = PushWechatHelper.getWechatQRCodeDataVO(i);
            if (StrUtil.isNotBlank(vo.getUserId())) {
                AdminUserBO adminUserBO = adminUserMap.get(vo.getUserId());
                if (null != adminUserBO) {
                    vo.setUserName(adminUserBO.getUserName());
                    vo.setRealName(adminUserBO.getRealName());
                    vo.setMobile(adminUserBO.getMobile());
                    vo.setEmail(adminUserBO.getEmail());
                    vo.setDeptId(adminUserBO.getDeptId());
                }
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public WechatQRCodeDataVO detail(WechatQRCodeDataGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        WechatQRCodeDataInfo entity = wechatQRCodeDataService.getOne(new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("id", params.getId())
        );

        if (null == entity) {
            return null;
        }

        WechatQRCodeDataVO vo = PushWechatHelper.getWechatQRCodeDataVO(entity);

        // 根据userid查询本系统用户信息
        if (StrUtil.isNotBlank(vo.getUserId())) {
            AdminUserBO adminUserBO = adminUserClientService.getUserById(vo.getUserId());
            if (null != adminUserBO) {
                vo.setUserName(adminUserBO.getUserName());
                vo.setRealName(adminUserBO.getRealName());
                vo.setMobile(adminUserBO.getMobile());
                vo.setEmail(adminUserBO.getEmail());
                vo.setDeptId(adminUserBO.getDeptId());
            }
        }

        // 根据数据id查询本二维码被扫过的openid列表（用于确认本二维码多少人扫过）
        List<Map<String, Object>> scanCount = wechatQRCodeDataBizMapper.scanCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .eq("qrcode_id", vo.getId())
        );
        if (CollectionUtil.isNotEmpty(scanCount)) {
            Object obj = scanCount.get(0).get("userNum");
            if (null != obj) {
                Long userNum = Long.valueOf(String.valueOf(obj));
                vo.setScanCount(userNum);
            }
        }

        // 根据数据id查询客户注册信息列表（用于确认本二维码多少人扫码并注册过）
        List<Map<String, Object>> registerCount = wechatQRCodeDataBizMapper.registerCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .eq("qrcode_id", vo.getId())
        );
        if (CollectionUtil.isNotEmpty(registerCount)) {
            Object obj = registerCount.get(0).get("userNum");
            if (null != obj) {
                Long userNum = Long.valueOf(String.valueOf(obj));
                vo.setRegisterCount(userNum);
            }
        }

        // 根据数据id查询家属关系表（用于确认本二维码多少人扫码并绑定过家属）
        List<Map<String, Object>> bindFamilyCount = wechatQRCodeDataBizMapper.bindFamilyCountByQrcodeId(new QueryWrapper<PatientSubscribeDataLog>()
                .eq("qrcode_id", vo.getId())
        );
        if (CollectionUtil.isNotEmpty(bindFamilyCount)) {
            Object obj = bindFamilyCount.get(0).get("userNum");
            if (null != obj) {
                Long userNum = Long.valueOf(String.valueOf(obj));
                vo.setRegisterCount(userNum);
            }
        }

        return vo;
    }

    @Override
    public WechatQRCodeDataVO simpleDetail(WechatQRCodeDataGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        WechatQRCodeDataInfo entity = wechatQRCodeDataService.getOne(new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("id", params.getId())
        );

        if (null == entity) {
            return null;
        }

        return PushWechatHelper.getWechatQRCodeDataVO(entity);
    }

    @Override
    public WechatQRCodeDataVO update(WechatQRCodeDataUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        WechatQRCodeDataInfo entity = wechatQRCodeDataService.getOne(new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );
        if (null == entity) {
            throw new BusinessException("未查询到数据");
        }

        if (StrUtil.isNotBlank(params.getOwnerType())) {
            entity.setOwnerType(params.getOwnerType());
        }
        if (StrUtil.isNotBlank(params.getOwnerName())) {
            entity.setOwnerName(params.getOwnerName());
        }
        if (StrUtil.isNotBlank(params.getUserId())) {
            entity.setUserId(params.getUserId());
        }
        wechatQRCodeDataService.updateById(entity);

        return PushWechatHelper.getWechatQRCodeDataVO(entity);
    }

    @Override
    public void del(WechatQRCodeDataGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        WechatQRCodeDataInfo entity = wechatQRCodeDataService.getOne(new QueryWrapper<WechatQRCodeDataInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );

        if (null != entity) {
            wechatQRCodeDataService.update(new UpdateWrapper<WechatQRCodeDataInfo>()
                .set("rows_status", YN.NO.getCode())
                .eq("id", entity.getId())
            );
        }
    }
    
    @Override
    public PageData<CustomerUserBO> registerUserPage(WechatQRCodeUserQueryDTO params) {
        IPage<CustomerUserBO> page = wechatQRCodeDataBizMapper.registerListByQrcodeId(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientSubscribeDataLog>()
                .eq(StrUtil.isNotBlank(params.getQrcodeId()), "qrcode_id", params.getQrcodeId())
        );
        return new PageData<>(page.getRecords(), page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public PageData<CustomerUserBO> bindFamilyUserPage(WechatQRCodeUserQueryDTO params) {
        IPage<CustomerUserBO> page = wechatQRCodeDataBizMapper.bindFamilyListByQrcodeId(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientSubscribeDataLog>()
                .eq(StrUtil.isNotBlank(params.getQrcodeId()), "qrcode_id", params.getQrcodeId())
        );
        return new PageData<>(page.getRecords(), page.getTotal(), page.getSize(), page.getPages());
    }

    @Override
    public List<WechatScanQRCodeDayVO> listByQrcodeIdAndDay(WechatScanQRCodeDTO params) {
        log.info("1.按日与员工统计扫码人数参数>>>>>> {}", JSON.toJSONString(params));
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<WechatScanQRCodeDayVO> list = wechatQRCodeDataBizMapper.listByQrcodeIdAndDay(new QueryWrapper<WechatScanQRCodeDTO>()
                .eq(StrUtil.isNotBlank(params.getEvent()), "event", params.getEvent())
                .ge(null != params.getStartDate(), "scan_date", params.getStartDate())
                .lt(null != params.getEndDate(), "scan_date", params.getEndDate())
        );

        return list;
    }

}
