package com.xxh.cloud.healthy.push.framework.modules.task.schedule;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeCreateDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.constant.WeixinQRCodeConstant;
import com.xxh.cloud.healthy.push.framework.modules.biz.mapper.WechatQRCodeDataBizMapper;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WeixinBizService;
import com.xxh.cloud.healthy.push.framework.modules.task.service.PromotionBizService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
@RestController
public class QrCode {
    
    @Autowired
    private WeixinBizService weixinBizService;
    @Autowired
    private WechatQRCodeDataBizMapper wechatQRCodeDataBizMapper;
    @Autowired
    private PromotionBizService promotionBizService;

    // TODO 临时批量生成二维码
    // @XxlJob("wxQrcodeBatchSave")
    @PostMapping("/wxQrcodeBatchSave")
    public void wxQrcodeBatchSave() throws Exception {
        log.info("批量生成二维码开始 >>>>>> ");
        XxlJobHelper.log("批量生成二维码开始 >>>>>> ");

        // 查询推广人员信息
        List<Map<String, String>> dataList = wechatQRCodeDataBizMapper.listWxQrCodeSourceData();
        if (CollUtil.isNotEmpty(dataList)) {
            dataList.forEach(i -> {
                String type = i.get("type");
                String name = i.get("name");
                String code = i.get("code");
                
                if ("1".equals(type)) { // 医生
                    WechatQRCodeCreateDTO createDto = new WechatQRCodeCreateDTO();
                    createDto.setSceneType(WeixinQRCodeConstant.SceneType.Doctor.getCode());
                    createDto.setOwnerName(name);
                    createDto.setOwnerCode(code);
                    createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Doctor.getCode());
                    createDto.setQrType(WeixinQRCodeConstant.QrcodeType.QR_Str_Permanent.getCode());
                    weixinBizService.createQRCode(createDto);
                } else if ("2".equals(type)) { // 科室
                    WechatQRCodeCreateDTO createDto = new WechatQRCodeCreateDTO();
                    createDto.setSceneType(WeixinQRCodeConstant.SceneType.Dept.getCode());
                    createDto.setOwnerName(name);
                    createDto.setOwnerCode(code);
                    createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Dept.getCode());
                    createDto.setQrType(WeixinQRCodeConstant.QrcodeType.QR_Str_Permanent.getCode());
                    weixinBizService.createQRCode(createDto);
                } else if ("3".equals(type)) { // 诊疗组
                    WechatQRCodeCreateDTO createDto = new WechatQRCodeCreateDTO();
                    createDto.setSceneType(WeixinQRCodeConstant.SceneType.Diagnosis_Treatment_Team.getCode());
                    createDto.setOwnerName(name);
                    createDto.setOwnerCode(code);
                    createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Dept.getCode());
                    createDto.setQrType(WeixinQRCodeConstant.QrcodeType.QR_Str_Permanent.getCode());
                    weixinBizService.createQRCode(createDto);
                } else if ("4".equals(type)) { // 护理站
                    WechatQRCodeCreateDTO createDto = new WechatQRCodeCreateDTO();
                    createDto.setSceneType(WeixinQRCodeConstant.SceneType.Nursing_Station.getCode());
                    createDto.setOwnerName(name);
                    createDto.setOwnerCode(code);
                    createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Dept.getCode());
                    createDto.setQrType(WeixinQRCodeConstant.QrcodeType.QR_Str_Permanent.getCode());
                    weixinBizService.createQRCode(createDto);
                }
            });
        }

        XxlJobHelper.log("批量生成二维码结束 <<<<<< ");
        log.info("批量生成二维码结束 <<<<<< ");
    }

    // 同步互联网医院的推广码信息
    @XxlJob("syncPromotionCodeTask")
    public void syncPromotionCodeTask() throws Exception {
        log.info("同步互联网医院的推广码信息开始 >>>>>> ");
        XxlJobHelper.log("同步互联网医院的推广码信息开始 >>>>>> ");

        promotionBizService.syncPromotionCodeTaskExecute();

        XxlJobHelper.log("同步互联网医院的推广码信息结束 <<<<<< ");
        log.info("同步互联网医院的推广码信息结束 <<<<<< ");
    }

    // 生成推广码对应的微信二维码
    @XxlJob("createPromotionWxCodeTask")
    public void createPromotionWxCodeTask() throws Exception {
        log.info("生成推广码对应的微信二维码开始 >>>>>> ");
        XxlJobHelper.log("生成推广码对应的微信二维码开始 >>>>>> ");

        promotionBizService.createPromotionWxCodeTaskExecute();

        XxlJobHelper.log("生成推广码对应的微信二维码结束 <<<<<< ");
        log.info("生成推广码对应的微信二维码结束 <<<<<< ");
    }

}
