package com.xxh.cloud.healthy.push.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.HisPromotionInfo;
import com.xxh.cloud.healthy.push.framework.modules.base.service.HisPromotionInfoService;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeCreateDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.constant.WeixinQRCodeConstant;
import com.xxh.cloud.healthy.push.framework.modules.biz.mapper.PromotionBizMapper;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WeixinBizService;
import com.xxh.cloud.healthy.push.framework.modules.task.service.PromotionBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;

@Service
public class PromotionBizServiceImpl implements PromotionBizService {

    /***/
    private String domain = "http://172.17.100.39:5035";
    /***/
    private String appid = "14631378478533";
    /***/
    private String secret = "7ac0ecfb8c73859fcc9b6eec23c1b769";
    /***/
    private String path = "/api/patientRecord/extend_list";

    @Autowired
    private PromotionBizMapper promotionBizMapper;
    @Autowired
    private WeixinBizService weixinBizService;
    @Autowired
    private HisPromotionInfoService hisPromotionInfoService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncPromotionCodeTaskExecute() {
        // 同步互联网医院推广码信息
        List<HisPromotionInfo> promotionList = getPromotionResp();
        if (CollUtil.isEmpty(promotionList)) {
            return;
        }

        hisPromotionInfoService.saveOrUpdateBatch(promotionList);
    }

    private List<HisPromotionInfo> getPromotionResp() {
        Map<String, String> params = new HashMap<>();

        String result = null;
        try {
            String url = StrUtil.format("{}{}?appid={}&secret={}", this.domain, path, this.appid, this.secret);
            result = HttpUtil.post(url, JSON.toJSONString(params), 1000 * 120);

            JSONObject jsonObject = JSON.parseObject(result);
            String content = jsonObject.getString("result");
            return JSON.parseArray(content, HisPromotionInfo.class);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return new ArrayList<>(0);
    }

    @Override
    public void createPromotionWxCodeTaskExecute() {
        List<HisPromotionInfo> list = promotionBizMapper.listPromotionInfo();
        if (CollUtil.isEmpty(list)) {
            return;
        }

        list.forEach(i -> {
            WechatQRCodeCreateDTO createDto = new WechatQRCodeCreateDTO();
            createDto.setSceneType(WeixinQRCodeConstant.SceneType.Default.getCode());
            createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Guide.getCode());
            if ("1".equals(i.getExtendType())) {
                createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Doctor.getCode());
            } else if ("2".equals(i.getExtendType())) {
                createDto.setOwnerType(WeixinQRCodeConstant.OwnerType.Dept.getCode());
            }
            createDto.setOwnerName(i.getExtendName());
            createDto.setOwnerCode(i.getExtendCode());
            createDto.setQrType(WeixinQRCodeConstant.QrcodeType.QR_Str_Permanent.getCode());
            weixinBizService.createQRCode(createDto);
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncPromotion() {
        // 同步互联网医院推广码
        this.syncPromotionCodeTaskExecute();
        // 生成推广码对应的微信二维码
        this.createPromotionWxCodeTaskExecute();
    }

}
