package com.xxh.cloud.healthy.scale.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultInfoDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultQueryDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultSaveDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.vos.ScaleResultVO;
import com.xxh.cloud.healthy.scale.framework.modules.result.service.ScaleResultInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/scale/result")
@Api(tags = "应用端,量表结果管理")
public class ScaleResultAppController {


    @Autowired
    private ScaleResultInfoService scaleResultInfoService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;

    @PostMapping("/query")
    @ApiOperation(value = "查询我的量表结果", response = ScaleResultVO.class)
    public Result<PageData<ScaleResultVO>> query(@RequestBody ScaleResultQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        params.setCustUserId(loginUser.getId());
        PageData<ScaleResultVO> data= scaleResultInfoService.queryPage(params);
        return new Result<PageData<ScaleResultVO>>().ok(data);
    }

    @PostMapping("/info")
    @ApiOperation(value = "获取量表结果信息", response = ScaleResultVO.class)
    public Result<ScaleResultVO> info(@RequestBody ScaleResultInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleResultVO data= scaleResultInfoService.infoById(params.getId());
        return new Result<ScaleResultVO>().ok(data);
    }


    @PostMapping("/write")
    @ApiOperation(value = "填写量表", response = ScaleResultVO.class)
    public Result<ScaleResultVO> write(@RequestBody ScaleResultSaveDTO params) {
        params.setPatientId(null);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        params.setCustUserId(loginUser.getId());
        MemberFamilyBO memberFamilyBO = customerFamilyClientService.getChoose(loginUser.getId());
        if(memberFamilyBO!=null){
            params.setPatientId(memberFamilyBO.getPatHisNo());
        }

        ValidatorUtils.validateEntity(params);

        if(StrUtil.isEmpty(params.getPatientId())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"患者编号不能为空");
        }

        ScaleResultVO data= scaleResultInfoService.add(params);
        return new Result<ScaleResultVO>().ok(data);
    }



}
