package com.xxh.cloud.healthy.scale.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.scale.framework.modules.client.api.ScaleResultClientService;
import com.xxh.cloud.healthy.scale.framework.modules.client.api.bean.bos.ScaleResultBO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.enums.ScaleResultStatusEnum;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dos.ScaleResultInfo;
import com.xxh.cloud.healthy.scale.framework.modules.result.service.ScaleResultInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class ScaleResultClientServiceImpl implements ScaleResultClientService {


    @Autowired
    private ScaleResultInfoService scaleResultInfoService;

    private ScaleResultBO getScaleResultBO(ScaleResultInfo entity){
        ScaleResultBO data=new ScaleResultBO();
        data.setId(entity.getId());
        data.setCustUserId(entity.getCustUserId());
        data.setPatientId(entity.getPatientId());
        data.setScaleConfigId(entity.getScaleConfigId());
        data.setScaleConfigName(entity.getScaleConfigName());
        data.setStatus(entity.getStatus());
        data.setStartEvaluationDate(entity.getStartEvaluationDate());
        data.setEndEvaluationDate(entity.getEndEvaluationDate());
        if(StrUtil.isNotEmpty(entity.getScaleResult())){
            data.setScaleResult(JSON.parseObject(entity.getScaleResult(), Map.class));
        }
        return data;
    }

    @Override
    public List<ScaleResultBO> getScaleResultListByPid(String patientId) {
        List<ScaleResultBO> data=new ArrayList<>();
        if(StrUtil.isEmpty(patientId)){
            return data;
        }
        List<ScaleResultInfo> list= scaleResultInfoService.list(
                new QueryWrapper<ScaleResultInfo>()
                        .eq("patient_id",patientId)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("status", ScaleResultStatusEnum.SRS_Complete.getCode())
                        .orderByDesc("end_evaluation_date")
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(this.getScaleResultBO(item));
            });
        }
        return data;
    }

    @Override
    public List<ScaleResultBO> getScaleResultListByPids(List<String> patientIds) {
        List<ScaleResultBO> data=new ArrayList<>();
        if(CollUtil.isEmpty(patientIds)){
            return data;
        }
        List<ScaleResultInfo> list= scaleResultInfoService.list(
                new QueryWrapper<ScaleResultInfo>()
                        .in("patient_id",patientIds)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("status", ScaleResultStatusEnum.SRS_Complete.getCode())
                        .orderByDesc("end_evaluation_date")
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(this.getScaleResultBO(item));
            });
        }
        return data;
    }

    @Override
    public Map<String, List<ScaleResultBO>> getScaleResultMapByPids(List<String> patientIds) {
        Map<String, List<ScaleResultBO>> data=new HashMap<>();
        if(CollUtil.isEmpty(patientIds)){
            return data;
        }
        List<ScaleResultInfo> list= scaleResultInfoService.list(
                new QueryWrapper<ScaleResultInfo>()
                        .in("patient_id",patientIds)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("status", ScaleResultStatusEnum.SRS_Complete.getCode())
                        .orderByDesc("end_evaluation_date")
        );
        if(CollUtil.isEmpty(list)){
           return data;
        }

        list.forEach(item->{
            List<ScaleResultBO> srlist = data.get(item.getPatientId());
            if(srlist==null){
                srlist=new ArrayList<>();
                data.put(item.getPatientId(),srlist);
            }
            srlist.add(this.getScaleResultBO(item));
        });

        return data;
    }



}
