package com.xxh.cloud.healthy.scale.framework.modules.config.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.cache.helper.ScaleCacheKeyHelper;
import com.xxh.cloud.healthy.scale.framework.modules.cache.service.ScaleConfigCategoryCacheService;
import com.xxh.cloud.healthy.scale.framework.modules.config.biz.ScaleConfigBizService;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dos.ScaleConfigInfo;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dos.ScaleItemConfigInfo;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.*;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.enums.ScaleConfigItemTypeEnum;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigCategoryVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.helper.ScaleConfigConvert;
import com.xxh.cloud.healthy.scale.framework.modules.config.helper.ScaleConfigHelper;
import com.xxh.cloud.healthy.scale.framework.modules.config.service.ScaleConfigInfoService;
import com.xxh.cloud.healthy.scale.framework.modules.config.service.ScaleItemConfigInfoService;
import com.xxh.cloud.healthy.third.client.modules.api.HealthyThirdUserService;
import com.xxh.cloud.healthy.third.client.modules.api.entity.bos.HealthyUserBO;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
public class ScaleConfigBizServiceImpl implements ScaleConfigBizService {


    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private HealthyThirdUserService healthyThirdUserService;

    @Autowired
    private ScaleConfigCategoryCacheService scaleConfigCategoryCacheService;

    @Autowired
    private ScaleConfigInfoService scaleConfigInfoService;

    @Autowired
    private ScaleItemConfigInfoService scaleItemConfigInfoService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;


    @Override
    public PageData<ScaleConfigVO> query(ScaleConfigInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<ScaleConfigInfo> page = scaleConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ScaleConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> categoryIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                Set<String> disIds= StrJoinSplitUtils.split2Set(item.getDiseaseIds());
                diseaseIds.addAll(disIds);
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });

        Map<String, HealthyUserBO> userMap= healthyThirdUserService.getMapByIds(userIds);
        Map<String, ScaleConfigCategoryVO> categoryMap= scaleConfigCategoryCacheService.getMapByIds(categoryIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);


        PageData<ScaleConfigVO> data= new PageConverter<ScaleConfigInfo, ScaleConfigVO>() {
            @Override
            public ScaleConfigVO converter(ScaleConfigInfo entity) {
                return  ScaleConfigConvert.getScaleConfigVO(entity,userMap,categoryMap,diseaseMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public IPage<ScaleItemConfigInfo> queryScaleItemConfig(ScaleItemConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<ScaleItemConfigInfo> page= scaleItemConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ScaleItemConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getScaleConfigId()),"scale_config_id", params.getScaleConfigId())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .and(StrUtil.isNotEmpty(params.getKeywords()),
                                i->i.or().like("title",params.getKeywords())
                                        .or().like("sub_title",params.getKeywords())
                        )
                        .inSql("scale_config_id","SELECT id FROM h_scale_config_info where rows_status ='Y'")
                        .orderByDesc("scale_config_id","orders")
        );
        return page;
    }



    @Override
    public IPage<ScaleConfigInfo> queryScaleConfig(ScaleConfigInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<ScaleConfigInfo> page = scaleConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ScaleConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .isNotNull(StrUtil.isNotEmpty(params.getResultConfigNullFlag()),"result_config")
        );
        return page;
    }

    @Override
    public ScaleConfigVO add(ScaleConfigInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigInfo entity= ScaleConfigHelper.getScaleConfigInfo_Add(params);
        scaleConfigInfoService.save(entity);
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ScaleConfigVO add2Item(ScaleConfigInfo2ItemSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigInfo entity= ScaleConfigHelper.getScaleConfigInfo_Add(params);

        List<ScaleItemConfigInfo> itemList=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getItems())){
            params.getItems().forEach(item->{
                itemList.add(ScaleConfigHelper.getScaleItemConfigInfo_Add(entity.getId(),item));
            });
        }

        scaleConfigInfoService.save(entity);
        if(itemList.size()>0){
            scaleItemConfigInfoService.saveBatch(itemList);
        }
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Override
    public ScaleConfigVO edit(ScaleConfigInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigInfo entity= scaleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        ScaleConfigHelper.getScaleConfigInfo_Edit(entity,params);
        scaleConfigInfoService.updateById(entity);
        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigKey(entity.getId()));
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ScaleConfigVO edit2Item(ScaleConfigInfo2ItemUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigInfo entity= scaleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }

        ScaleConfigHelper.getScaleConfigInfo_Edit(entity,params);

        //items
        Map<String,ScaleItemConfigInfo> itemMap=new HashMap<>();
        List<ScaleItemConfigInfo> items= scaleItemConfigInfoService.list(
                new QueryWrapper<ScaleItemConfigInfo>()
                        .eq("scale_config_id",entity.getId())
                        .orderByDesc("orders")
        );
        if(CollUtil.isNotEmpty(items)){
            items.forEach(item->{
                itemMap.put(item.getId(),item);
            });
        }

        List<ScaleItemConfigInfo> saveItemList=new ArrayList<>();
        List<ScaleItemConfigInfo> updateItemList=new ArrayList<>();

        for(ScaleItemConfigItemOpDTO  paramItem :  params.getItems()){
            if(StrUtil.isNotEmpty(paramItem.getId())){
                ScaleItemConfigInfo itemEntity=itemMap.get(paramItem.getId());
                if(itemEntity!=null){
                    ScaleConfigHelper.getScaleItemConfigInfo_Edit(itemEntity,paramItem);
                    updateItemList.add(itemEntity);
                }else {
                    itemEntity= ScaleConfigHelper.getScaleItemConfigInfo_Add(entity.getId(),paramItem);
                    saveItemList.add(itemEntity);
                }
            }else {
                ScaleItemConfigInfo itemEntity= ScaleConfigHelper.getScaleItemConfigInfo_Add(entity.getId(),paramItem);
                saveItemList.add(itemEntity);
            }
        }

        scaleItemConfigInfoService.update(
                new UpdateWrapper<ScaleItemConfigInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .eq("scale_config_id",entity.getId())
        );

        scaleConfigInfoService.updateById(entity);

        if(saveItemList.size()>0){
            scaleItemConfigInfoService.saveBatch(saveItemList);
        }

        if(updateItemList.size()>0){
            scaleItemConfigInfoService.updateBatchById(updateItemList);
        }

        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigKey(entity.getId()));
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public ScaleConfigVO editItemList(ScaleConfigInfoItemUpdateDTO params) {
        ValidatorUtils.validateEntity(params);

        ScaleConfigInfo entity= scaleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }

        entity.setResultScriptContent(params.getResultScriptContent());
        if(CollUtil.isNotEmpty(params.getResultConfig())){
            entity.setResultConfig(JSON.toJSONString(params.getResultConfig()));
        }

        //items
        Map<String,ScaleItemConfigInfo> itemMap=new HashMap<>();
        List<ScaleItemConfigInfo> items= scaleItemConfigInfoService.list(
                new QueryWrapper<ScaleItemConfigInfo>()
                        .eq("scale_config_id",entity.getId())
                        .orderByDesc("orders")
        );
        if(CollUtil.isNotEmpty(items)){
            items.forEach(item->{
                itemMap.put(item.getId(),item);
            });
        }

        List<ScaleItemConfigInfo> saveItemList=new ArrayList<>();
        List<ScaleItemConfigInfo> updateItemList=new ArrayList<>();

        for(ScaleItemConfigItemOpDTO  paramItem :  params.getItems()){
            if(StrUtil.isNotEmpty(paramItem.getId())){
                ScaleItemConfigInfo itemEntity=itemMap.get(paramItem.getId());
                if(itemEntity!=null){
                    ScaleConfigHelper.getScaleItemConfigInfo_Edit(itemEntity,paramItem);
                    updateItemList.add(itemEntity);
                }else {
                    itemEntity= ScaleConfigHelper.getScaleItemConfigInfo_Add(entity.getId(),paramItem);
                    saveItemList.add(itemEntity);
                }
            }else {
                ScaleItemConfigInfo itemEntity= ScaleConfigHelper.getScaleItemConfigInfo_Add(entity.getId(),paramItem);
                saveItemList.add(itemEntity);
            }
        }

        scaleItemConfigInfoService.update(
                new UpdateWrapper<ScaleItemConfigInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .eq("scale_config_id",entity.getId())
        );

        scaleConfigInfoService.updateById(entity);

        if(saveItemList.size()>0){
            scaleItemConfigInfoService.saveBatch(saveItemList);
        }

        if(updateItemList.size()>0){
            scaleItemConfigInfoService.updateBatchById(updateItemList);
        }
        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigKey(entity.getId()));
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Override
    public ScaleConfigVO updateStatus(ScaleConfigInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigInfo entity= scaleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(!params.getStatus().equals(entity.getStatus())){
            entity.setStatus(params.getStatus());
            scaleConfigInfoService.updateById(entity);
            redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigKey(entity.getId()));
        }
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Override
    public ScaleConfigVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleConfigInfo entity= scaleConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ScaleConfigConvert.getScaleConfigVO(entity);
    }

    @Override
    public ScaleConfigVO infoFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleConfigInfo entity= scaleConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        List<ScaleItemConfigInfo> items= scaleItemConfigInfoService.list(
                new QueryWrapper<ScaleItemConfigInfo>()
                        .eq("scale_config_id",entity.getId())
                        .eq("rows_status",YN.YES.getCode())
                        .orderByDesc("orders")
        );
        return ScaleConfigConvert.getScaleConfigVO_Full(entity,items);
    }

//    @Override
//    public ScaleConfigVO info2NullFullById(String id) {
//        if(StrUtil.isEmpty(id)){
//            return null;
//        }
//        ScaleConfigInfo entity= scaleConfigInfoService.getById(id);
//        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
//            return null;
//        }
//        List<ScaleItemConfigInfo> items= scaleItemConfigInfoService.list(
//                new QueryWrapper<ScaleItemConfigInfo>()
//                        .eq("scale_config_id",entity.getId())
//                        .eq("rows_status",YN.YES.getCode())
//                        .orderByDesc("orders")
//        );
//        return ScaleConfigConvert.getScaleConfigVO_Full(entity,items);
//    }

//    @Override
//    public List<ScaleConfigVO> listFullByIds(Collection<String> ids) {
//        if(CollUtil.isEmpty(ids)){
//            return null;
//        }
//        List<ScaleConfigInfo> list= scaleConfigInfoService.list(
//                new QueryWrapper<ScaleConfigInfo>()
//                        .eq("rows_status", YN.YES.getCode())
//                        .in("id",ids)
//        );
//        if(CollUtil.isEmpty(list)){
//            return null;
//        }
//        List<String> scids=new ArrayList<>();
//        list.forEach(item->{
//            scids.add(item.getId());
//        });
//
//        List<ScaleItemConfigInfo> items= scaleItemConfigInfoService.list(
//                new QueryWrapper<ScaleItemConfigInfo>()
//                        .in("scale_config_id",scids)
//                        .eq("rows_status",YN.YES.getCode())
//                        .orderByDesc("orders")
//        );
//        List<ScaleConfigVO> data=new ArrayList<>();
//        list.forEach(item->{
//            data.add(ScaleConfigConvert.getScaleConfigVO_Full(item,items));
//        });
//        return data;
//    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleConfigInfo entity= scaleConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        scaleConfigInfoService.updateById(entity);

        scaleItemConfigInfoService.update(
                new UpdateWrapper<ScaleItemConfigInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .eq("scale_config_id",entity.getId())
                        .eq("rows_status",YN.YES.getCode())
        );
        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigKey(entity.getId()));

    }


}
