package com.xxh.cloud.healthy.scale.framework.modules.config.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.cache.helper.ScaleCacheKeyHelper;
import com.xxh.cloud.healthy.scale.framework.modules.config.biz.ScaleConfigCategoryBizService;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dos.ScaleConfigCategoryInfo;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategorySaveDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategoryTreeDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategoryUpdateDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigCategoryVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.helper.ScaleConfigCategoryHelper;
import com.xxh.cloud.healthy.scale.framework.modules.config.service.ScaleConfigCategoryInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ScaleConfigCategoryBizServiceImpl implements ScaleConfigCategoryBizService {


    @Autowired
    private ScaleConfigCategoryInfoService scaleConfigCategoryInfoService;


    @Autowired
    private RedisUtils redisUtils;

    @Override
    public List<ScaleConfigCategoryVO> tree(ScaleConfigCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ScaleConfigCategoryInfo> list= scaleConfigCategoryInfoService.list(
                new QueryWrapper<ScaleConfigCategoryInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .orderByDesc("pid","orders")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<ScaleConfigCategoryVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(ScaleConfigCategoryHelper.getScaleConfigCategoryVO(item));
        });
        List<ScaleConfigCategoryVO> data = TreeUtils.build(dataList);
        return data;
    }

    @Override
    public List<ScaleConfigCategoryVO> list(ScaleConfigCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ScaleConfigCategoryVO> data=new ArrayList<>();
        List<ScaleConfigCategoryInfo> list= scaleConfigCategoryInfoService.list(
                new QueryWrapper<ScaleConfigCategoryInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(ScaleConfigCategoryHelper.getScaleConfigCategoryVO(item));
            });
        }
        return data;
    }

    @Override
    public ScaleConfigCategoryVO save(ScaleConfigCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigCategoryInfo entity=new ScaleConfigCategoryInfo();
        entity.setId(ScaleConfigCategoryHelper.getScaleConfigCategoryId());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setIcon(params.getIcon());
        entity.setOrders(params.getOrders());
        if(StrUtil.isEmpty(params.getPid())){
            entity.setPid(Constant.RootParentNodeId);
        }else {
            entity.setPid(params.getPid());
        }

        entity.setRowsStatus(YN.YES.getCode());
        scaleConfigCategoryInfoService.save(entity);
        return ScaleConfigCategoryHelper.getScaleConfigCategoryVO(entity);
    }

    @Override
    public ScaleConfigCategoryVO update(ScaleConfigCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigCategoryInfo entity= scaleConfigCategoryInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setIcon(params.getIcon());
        entity.setOrders(params.getOrders());
        scaleConfigCategoryInfoService.updateById(entity);
        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigCategoryKey(entity.getId()));
        return ScaleConfigCategoryHelper.getScaleConfigCategoryVO(entity);
    }

    @Override
    public ScaleConfigCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleConfigCategoryInfo entity= scaleConfigCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ScaleConfigCategoryHelper.getScaleConfigCategoryVO(entity);
    }


    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }

        long childNum= scaleConfigCategoryInfoService.count(new QueryWrapper<ScaleConfigCategoryInfo>()
                .eq("pid",id)
                .eq("rows_status", YN.YES.getCode())
        );
        if(childNum>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"子分类，不能删除");
        }

        ScaleConfigCategoryInfo entity= scaleConfigCategoryInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        scaleConfigCategoryInfoService.updateById(entity);
        redisUtils.delete(ScaleCacheKeyHelper.getScaleConfigCategoryKey(entity.getId()));
    }


}
