package com.xxh.cloud.healthy.scale.framework.modules.config.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dos.ScaleConfigInfo;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dos.ScaleItemConfigInfo;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.enums.ScaleConfigItemTypeEnum;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.*;
import com.xxh.cloud.healthy.third.client.modules.api.entity.bos.HealthyUserBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;

import java.util.List;
import java.util.Map;

public class ScaleConfigConvert {



    public static ScaleItemConfigVO getScaleItemConfigVO(ScaleItemConfigInfo entity){
        ScaleItemConfigVO data=new ScaleItemConfigVO();
        BeanUtil.copyProperties(entity,data,"content");

        if(StrUtil.isNotEmpty(entity.getType()) && StrUtil.isNotEmpty(entity.getContent())){
            if(ScaleConfigItemTypeEnum.SCI_Input.getCode().equals(entity.getType())){
                data.setInputContent(JSON.parseObject(entity.getContent(), ScaleItemConfigContentInputVO.class));
            }else if(ScaleConfigItemTypeEnum.SCI_Date.getCode().equals(entity.getType())){
                data.setDateContent(JSON.parseObject(entity.getContent(),ScaleItemConfigContentDateVO.class));
            } else {
                data.setListContent(JSON.parseObject(entity.getContent(), ScaleItemConfigContentListVO.class));
            }
        }
        return data;
    }


    public static ScaleConfigVO getScaleConfigVO(ScaleConfigInfo entity){
        ScaleConfigVO data=new ScaleConfigVO();
        BeanUtil.copyProperties(entity,data,"diseaseIds","resultConfig");
        data.setDiseaseIds(StrJoinSplitUtils.split2List(entity.getDiseaseIds()));
        if(StrUtil.isNotEmpty(entity.getResultConfig())){
            data.setResultConfig(JSON.parseArray(entity.getResultConfig(),ScaleResultConfigVO.class));
        }
        return data;
    }

    public static ScaleConfigVO getScaleConfigVO(ScaleConfigInfo entity, Map<String, HealthyUserBO> userMap,
                                                 Map<String, ScaleConfigCategoryVO> categoryMap,
                                                 Map<String, StandardDiseaseBO> diseaseMap){

        ScaleConfigVO data=getScaleConfigVO(entity);

        if(MapUtil.isNotEmpty(userMap)){
            if(StrUtil.isNotEmpty(data.getCreator())){
                HealthyUserBO user= userMap.get(data.getCreator());
                if(user!=null){
                    data.setCreatorUserName(user.getUserName());
                    data.setCreatorNickName(user.getNickName());
                    data.setCreatorRealName(user.getRealName());
                }
            }
            if(StrUtil.isNotEmpty(data.getUpdater())){
                HealthyUserBO user= userMap.get(data.getUpdater());
                if(user!=null){
                    data.setUpdaterUserName(user.getUserName());
                    data.setUpdaterRealName(user.getRealName());
                    data.setUpdaterNickName(user.getNickName());
                }
            }
        }

        if(MapUtil.isNotEmpty(categoryMap)){
            if(StrUtil.isNotEmpty(data.getCategoryId())){
                ScaleConfigCategoryVO configCategory = categoryMap.get(data.getCategoryId());
                if(configCategory!=null){
                    data.setCategoryName(configCategory.getName());
                }
            }
        }

        if(MapUtil.isNotEmpty(diseaseMap) && CollUtil.isNotEmpty(data.getDiseaseIds())){
            data.getDiseaseIds().forEach(id->{
                StandardDiseaseBO diseaseBO = diseaseMap.get(id);
                if(diseaseBO!=null){
                    data.addDisease(diseaseBO.getId(),diseaseBO.getName());
                }
            });
        }

        return data;
    }


    public static ScaleConfigVO getScaleConfigVO_Full(ScaleConfigInfo entity, List<ScaleItemConfigInfo> items){
        ScaleConfigVO data=getScaleConfigVO(entity);
        if(CollUtil.isNotEmpty(items)){
            items.forEach(item->{
                if(data.getId().equals(item.getScaleConfigId())){
                    data.addItem(getScaleItemConfigVO(item));
                }
            });
        }
        return data;
    }







}
