package com.xxh.cloud.healthy.scale.framework.modules.result.helper;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.exception.BusinessException;
import jdk.nashorn.api.scripting.ScriptObjectMirror;

import javax.script.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class JavaScriptUtil {



    /** 单例的JavaScript解析引擎 */
    private static ScriptEngine javaScriptEngine;


    static {
        ScriptEngineManager manager = new ScriptEngineManager();
//        ScriptEngine scriptEngine = manager.getEngineByName("JavaScript");
        ScriptEngine scriptEngine = manager.getEngineByName("nashorn");
        if (scriptEngine == null) {
            throw new RuntimeException("获取JavaScript解析引擎失败");
        }
        javaScriptEngine = scriptEngine;
    }


    /**
     * 执行一段JavaScript代码
     *
     * @param script JavaScript的代码
     * @return JavaScript代码运行结果的值
     * @throws ScriptException JavaScript代码运行异常
     */
    public static Object execute(String script) throws ScriptException {
        return javaScriptEngine.eval(script);
    }


    /**
     * 运行一个JavaScript代码段,并获取指定变量名的值
     *
     * @param script        代码段
     * @param attributeName 已知的变量名
     * @return 指定变量名对应的值
     * @throws ScriptException JavaScript代码运行异常
     */
    public static Object executeForAttribute(String script, String attributeName) throws ScriptException {
        javaScriptEngine.eval(script);
        return javaScriptEngine.getContext().getAttribute(attributeName);
    }


    public static Map<String,Object> executeMethod(String script ,String methodName ,Map<String,Object> params) throws BusinessException {
        Object eval;
        try {
            javaScriptEngine.eval(script);
            Invocable inv = (Invocable)javaScriptEngine;
            eval = inv.invokeFunction(methodName, params);
        } catch (ScriptException e) {
            throw new BusinessException(9986,"执行脚本错误,"+e.getMessage());
        } catch (NoSuchMethodException e) {
            throw new BusinessException(9986,"执行脚本错误,"+e.getMessage());
        }

        if(eval==null){
            throw new BusinessException(9986,"无返回值");
        }

        Map<String,Object> resutlMap=new HashMap<>();
        if(eval instanceof  ScriptObjectMirror){
            ScriptObjectMirror obj = (ScriptObjectMirror)eval;
            for( Map.Entry<String,Object> m:obj.entrySet()){
                resutlMap.put(m.getKey(),m.getValue());
            }
        }else {
            throw new BusinessException(9987,"返回不是对象");
        }
        return resutlMap;
    }





    /**
     * 获取当前语句运行后第一个有值变量的值
     *
     * @param script 代码段
     * @return 第一个有值变量的值
     * @throws ScriptException JavaScript代码运行异常
     */
    public static Object executeForFirstAttribute(String script) throws ScriptException {

        //这里重新获取一个JavaScript解析引擎是为了避免代码中有其他调用工具类的地方的变量干扰
        //重新获取后,这个JavaScript解析引擎只执行了这次传入的代码,不会保存其他地方的变量
        //全局的解析器中会保存最大200个变量,JavaScript解析引擎本身最大保存100个变量
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine scriptEngine = manager.getEngineByName("js");
        if (scriptEngine == null) {
            throw new RuntimeException("获取JavaScript解析引擎失败");
        }

        scriptEngine.eval(script);
        ScriptContext context = scriptEngine.getContext();
        if (context == null) {
            return null;
        }

        Bindings bindings = context.getBindings(ScriptContext.ENGINE_SCOPE);
        if (bindings == null) {
            return null;
        }
        Set<Map.Entry<String, Object>> entrySet = bindings.entrySet();
        if (entrySet == null || entrySet.isEmpty()) {
            return null;
        }

        for (Map.Entry<String, Object> entry : entrySet) {
            if (entry.getValue() != null) {
                return entry.getValue();
            }
        }

        return null;
    }


    public static void main(String[] args) throws ScriptException, NoSuchMethodException {
//        Map<String,Object> params=new HashMap<>();
//        params.put("name","wjj");
//        params.put("age",35);
//
//
//        String script = "function scale(p){" +
//                " var obj={} ; " +
//                " obj.name=p.get('name');" +
//                " return obj;" +
//                "}";
//
//
//        System.out.println(script);
//
//
//        Map<String,Object> result= executeMethod(script,"scale",params);
//
//        System.out.println("=============");
//
//        result.forEach((k,v)->{
//            System.out.println(k+"  : "+v);
//        });



        Map<String,Object> params=new HashMap<>();
        params.put("1711687106818220032",18);


        String script = "function scale(p) {" +
                "  var item1 = p.get('1711687106818220032');" +
                "  var str ='';" +
                " if(item1 == 18){" +
                "    str= '小明'" +
                "  }" +
//                "  var obj = new Map();" +
//                "  obj.set(\"code123\", str); " +

                "  var obj = {};" +
                "  obj[\"code123\"]=str; " +
                "  return obj;" +
                " }";


        System.out.println(script);


        Map<String,Object> result= executeMethod(script,"scale",params);

        System.out.println(JSON.toJSONString(result));

    }



}
