package com.xxh.cloud.healthy.scale.framework.modules.result.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.common.mq.bean.MQRuleLabelBean;
import com.xxh.cloud.common.mq.constant.DataSceneType;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.scale.framework.modules.cache.service.ScaleConfigCacheService;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.enums.ScaleResultStatusEnum;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleItemConfigVO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dos.ScaleItemResultInfo;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dos.ScaleResultInfo;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultQueryDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultSaveDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.vos.ScaleResultVO;
import com.xxh.cloud.healthy.scale.framework.modules.result.helper.JavaScriptUtil;
import com.xxh.cloud.healthy.scale.framework.modules.result.helper.ScaleResultConvert;
import com.xxh.cloud.healthy.scale.framework.modules.result.helper.ScaleResultHelper;
import com.xxh.cloud.healthy.scale.framework.modules.result.mapper.ScaleResultInfoMapper;
import com.xxh.cloud.healthy.scale.framework.modules.result.service.ScaleItemResultInfoService;
import com.xxh.cloud.healthy.scale.framework.modules.result.service.ScaleResultInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.healthy.third.client.modules.api.HealthyThirdUserService;
import com.xxh.cloud.healthy.third.client.modules.api.entity.bos.HealthyUserBO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.script.ScriptException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 量表评估结果表 服务实现类
 * </p>
 */
@Service
public class ScaleResultInfoServiceImpl extends ServiceImpl<ScaleResultInfoMapper, ScaleResultInfo> implements ScaleResultInfoService {

    @Autowired
    private ScaleItemResultInfoService scaleItemResultInfoService;
    @Autowired
    private HealthyThirdUserService healthyThirdUserService;
    @Autowired
    private ScaleConfigCacheService scaleConfigCacheService;
    @Autowired
    private PatientClientService patientClientService;


    @Override
    public PageData<ScaleResultVO> queryPage(ScaleResultQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<ScaleResultInfo> page = this.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ScaleResultInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCustUserId()),"cust_user_id",params.getCustUserId())
                        .eq(StrUtil.isNotEmpty(params.getPatientId()),"patient_id",params.getPatientId())
                        .eq(StrUtil.isNotEmpty(params.getScaleConfigId()),"scale_config_id",params.getScaleConfigId())
                        .like(StrUtil.isNotEmpty(params.getScaleConfigName()),"scale_config_name",params.getScaleConfigName())
                        .orderByDesc("create_date")
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> patientIds=new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCustUserId())){
                userIds.add(item.getCustUserId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getPatientId())){
                patientIds.add(item.getPatientId());
            }
        });
        Map<String, HealthyUserBO> userMap= healthyThirdUserService.getMapByIds(userIds);
        // 查询患者列表
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientList)) {
            patientMap.putAll(patientList.stream().collect(Collectors.toMap(PatientClientBO::getPatientId, item -> item)));
        }


        PageData<ScaleResultVO> data= new PageConverter<ScaleResultInfo, ScaleResultVO>() {
            @Override
            public ScaleResultVO converter(ScaleResultInfo entity) {
                return  ScaleResultConvert.getScaleResultVO(entity,userMap,patientMap);
            }
        }.getPageData(page);
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ScaleResultVO add(ScaleResultSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO scaleConfig= scaleConfigCacheService.getById(params.getScaleConfigId());
        if(scaleConfig==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"量表不存在");
        }
        Map<String, ScaleItemConfigVO> scaleItemConfigMap= scaleConfig.getItemMap();

        ScaleResultInfo entity=new ScaleResultInfo();
        entity.setPatientId(params.getPatientId());
        entity.setId(ScaleResultHelper.getScaleResultId());
        entity.setCustUserId(params.getCustUserId());
        entity.setScaleConfigId(params.getScaleConfigId());
        entity.setScaleConfigName(scaleConfig.getName());
        entity.setScaleConfigType(scaleConfig.getType());
        entity.setTaskId(params.getTaskId());
        entity.setStatus(ScaleResultStatusEnum.SRS_Complete.getCode());
        entity.setStartEvaluationDate(new Date());
        entity.setEndEvaluationDate(new Date());
        entity.setRowsStatus(YN.YES.getCode());
        if(CollUtil.isNotEmpty(scaleConfig.getResultConfig())){
            entity.setScaleResultConfig(JSON.toJSONString(scaleConfig.getResultConfig()));
        }

        List<ScaleItemResultInfo> items=new ArrayList<>();

        Map<String,Object> jsParams=new HashMap<>();

        params.getItems().forEach(item->{
            ScaleItemConfigVO scaleItemConfig= scaleItemConfigMap.get(item.getScaleItemConfigId());
            if(scaleItemConfig==null){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"量表明细不存在");
            }
            ScaleItemResultInfo resultEntity=new ScaleItemResultInfo();
            resultEntity.setId(ScaleResultHelper.getScaleItemResultId());
            resultEntity.setScaleResultId(entity.getId());
            resultEntity.setScaleItemConfigId(item.getScaleItemConfigId());
            resultEntity.setResultVal(item.getResultVal());
            resultEntity.setScaleItemConfig(JSON.toJSONString(scaleItemConfig));
            items.add(resultEntity);

            jsParams.put(resultEntity.getScaleItemConfigId(),resultEntity.getResultVal());
        });

        //调用引擎解析，量表执行结果
        if(StrUtil.isNotEmpty(scaleConfig.getResultScriptContent())){
            String decodeScript = Base64.decodeStr(scaleConfig.getResultScriptContent());
            Map<String,Object> calculateVal= JavaScriptUtil.executeMethod(decodeScript,"scale",jsParams);
            if(MapUtil.isNotEmpty(calculateVal)){
                entity.setScaleResult(JSON.toJSONString(calculateVal));
            }
        }


        this.save(entity);
        scaleItemResultInfoService.saveBatch(items);

        //todo:推送到引擎
        if(StrUtil.isNotEmpty(entity.getPatientId())){
            List<MQRuleLabelBean> patientIds=new ArrayList<>();
            patientIds.add(new MQRuleLabelBean(entity.getPatientId(), DataSceneType.DS_Scale, DateUtil.formatDateTime(entity.getEndEvaluationDate()),entity.getId()));
            ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_Rule_Data_Save, JSON.toJSONString(patientIds),1);
        }

        return  ScaleResultConvert.getScaleResultVO_Full(entity,items,new HashMap<>(), new HashMap<>());
    }

    @Override
    public ScaleResultVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleResultInfo entity= this.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }


        Set<String> userIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCustUserId())){
            userIds.add(entity.getCustUserId());
        }
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, HealthyUserBO> userMap= healthyThirdUserService.getMapByIds(userIds);

        List<ScaleItemResultInfo> items= scaleItemResultInfoService.list(
                new QueryWrapper<ScaleItemResultInfo>()
                        .eq("scale_result_id",entity.getId())
        );

        // 查询患者的信息
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientId(entity.getPatientId());
        PatientClientBO patient = patientClientService.getOneSimple(patientQueryDTO);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (null != patient) {
            patientMap.put(patient.getPatientId(), patient);
        }

        return  ScaleResultConvert.getScaleResultVO_Full(entity,items,userMap,patientMap);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ScaleResultInfo entity= this.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        this.updateById(entity);
    }
}
