package com.xxh.cloud.healthy.scale.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.config.biz.ScaleConfigCategoryBizService;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategoryInfoDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategorySaveDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategoryTreeDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigCategoryUpdateDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigCategoryVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.service.ScaleConfigCategoryInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:healthy:scale:category")
@Log4j2
@RestController
@RequestMapping("/sys/healthy/scale/category")
@Api(tags = "运营端,量表分类管理")
public class ScaleCategoryManagerController {


    @Autowired
    private ScaleConfigCategoryBizService scaleConfigCategoryBizService;


    @SaCheckPermission("sys:healthy:scale:category:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = ScaleConfigCategoryVO.class)
    public Result<List<ScaleConfigCategoryVO>> query(@RequestBody ScaleConfigCategoryTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ScaleConfigCategoryVO> data= scaleConfigCategoryBizService.tree(params);
        return new Result<List<ScaleConfigCategoryVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:category:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = ScaleConfigCategoryVO.class)
    public Result<ScaleConfigCategoryVO> save(@RequestBody ScaleConfigCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigCategoryVO data= scaleConfigCategoryBizService.save(params);
        return new Result<ScaleConfigCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:healthy:scale:category:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = ScaleConfigCategoryVO.class)
    public Result<ScaleConfigCategoryVO> update(@RequestBody ScaleConfigCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigCategoryVO data=scaleConfigCategoryBizService.update(params);
        return new Result<ScaleConfigCategoryVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:category:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody ScaleConfigCategoryInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        scaleConfigCategoryBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthy:scale:category:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = ScaleConfigCategoryVO.class)
    public  Result<ScaleConfigCategoryVO>  info(@RequestBody ScaleConfigCategoryInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigCategoryVO data= scaleConfigCategoryBizService.infoById(params.getId());
        return new Result<ScaleConfigCategoryVO>().ok(data);
    }






}
