package com.xxh.cloud.healthy.scale.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.config.biz.ScaleConfigBizService;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.*;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigCategoryVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigVO;
import com.xxh.cloud.healthy.scale.framework.modules.config.service.ScaleConfigInfoService;
import com.xxh.cloud.healthy.scale.framework.modules.result.helper.JavaScriptUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

//@SaCheckPermission("sys:healthy:scale:config")
@Log4j2
@RestController
@RequestMapping("/sys/healthy/scale/config")
@Api(tags = "运营端,量表管理")
public class ScaleConfigManagerController {


    @Autowired
    private ScaleConfigBizService scaleConfigBizService;


    //ScaleConfigBizService

    @SaCheckPermission("sys:healthy:scale:config:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询量表", response = ScaleConfigVO.class)
    public Result<PageData<ScaleConfigVO>> query(@RequestBody ScaleConfigInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ScaleConfigVO> data= scaleConfigBizService.query(params);
        return new Result<PageData<ScaleConfigVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:config:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增量表", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> save(@RequestBody ScaleConfigInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data= scaleConfigBizService.add(params);
        return new Result<ScaleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:config:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改量表", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> update(@RequestBody ScaleConfigInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data= scaleConfigBizService.edit(params);
        return new Result<ScaleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:config:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改量表状态", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> updateStatus(@RequestBody ScaleConfigInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data= scaleConfigBizService.updateStatus(params);
        return new Result<ScaleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:config:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除量表", response = String.class)
    public Result<String> del(@RequestBody ScaleConfigInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        scaleConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


    @SaCheckPermission("sys:healthy:scale:config:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取量表", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> info(@RequestBody ScaleConfigInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data=  scaleConfigBizService.infoById(params.getId());
        return new Result<ScaleConfigVO>().ok(data);
    }



    @SaCheckPermission("sys:healthy:scale:config:infoFull")
    @PostMapping("/infoFull")
    @ApiOperation(value = "获取量表和明细", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> infoFull(@RequestBody ScaleConfigInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data=  scaleConfigBizService.infoFullById(params.getId());
        return new Result<ScaleConfigVO>().ok(data);
    }




    @SaCheckPermission("sys:healthy:scale:config:save2item")
    @PostMapping("/save2item")
    @ApiOperation(value = "新增量表和明细", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> save2item(@RequestBody ScaleConfigInfo2ItemSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data= scaleConfigBizService.add2Item(params);
        return new Result<ScaleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:config:update2item")
    @PostMapping("/update2item")
    @ApiOperation(value = "修改量表和明细", response = ScaleConfigVO.class)
    public Result<ScaleConfigVO> update2item(@RequestBody ScaleConfigInfo2ItemUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleConfigVO data= scaleConfigBizService.edit2Item(params);
        return new Result<ScaleConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:healthy:scale:config:updateItem")
    @PostMapping("/updateItem")
    @ApiOperation(value = "修改量表明细", response = String.class)
    public Result<String> updateItem(@RequestBody ScaleConfigInfoItemUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        scaleConfigBizService.editItemList(params);
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:healthy:scale:config:exeResultScript")
    @PostMapping("/exeResultScript")
    @ApiOperation(value = "测试量表脚本", response = Map.class)
    public Result<Map<String,Object>> exeResultScript(@RequestBody ScaleConfigExeResultScriptDTO params) {
        ValidatorUtils.validateEntity(params);

        String decodeScript = Base64.decodeStr(params.getResultScriptContent());

        System.out.println("+++++++++++++++++++++ 解析脚本");
        System.out.println(decodeScript);

        Map<String,Object> calculateVal= JavaScriptUtil.executeMethod(decodeScript,"scale",params.getParams());
        return new Result<Map<String,Object>>().ok(calculateVal);
    }


    @SaCheckPermission("sys:healthy:scale:config:exeResultScriptById")
    @PostMapping("/exeResultScriptById")
    @ApiOperation(value = "测试量表脚本(根据量表ID)", response = Map.class)
    public Result<Map<String,Object>> exeResultScriptById(@RequestBody ScaleConfigExeResultScriptByIdDTO params) {
        ValidatorUtils.validateEntity(params);

        ScaleConfigVO configVO= scaleConfigBizService.infoById(params.getConfigId());

        String decodeScript = Base64.decodeStr(configVO.getResultScriptContent());

        System.out.println("+++++++++++++++++++++ 解析脚本");
        System.out.println(decodeScript);

        Map<String,Object> calculateVal= JavaScriptUtil.executeMethod(decodeScript,"scale",params.getParams());
        return new Result<Map<String,Object>>().ok(calculateVal);
    }


}
