package com.xxh.cloud.healthy.scale.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigInfoGetDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.dto.ScaleConfigInfoQueryDTO;
import com.xxh.cloud.healthy.scale.framework.modules.config.entity.vos.ScaleConfigVO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultInfoDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultQueryDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.dto.ScaleResultSaveDTO;
import com.xxh.cloud.healthy.scale.framework.modules.result.entity.vos.ScaleResultVO;
import com.xxh.cloud.healthy.scale.framework.modules.result.service.ScaleResultInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("sys:healthy:scale:result")
@Log4j2
@RestController
@RequestMapping("/sys/healthy/scale/result")
@Api(tags = "运营端,量表结果管理")
public class ScaleResultManagerController {

    @Autowired
    private ScaleResultInfoService scaleResultInfoService;




    @SaCheckPermission("sys:healthy:scale:result:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询量表", response = ScaleResultVO.class)
    public Result<PageData<ScaleResultVO>> query(@RequestBody ScaleResultQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ScaleResultVO> data= scaleResultInfoService.queryPage(params);
        return new Result<PageData<ScaleResultVO>>().ok(data);
    }


    @SaCheckPermission("sys:healthy:scale:result:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除量表", response = String.class)
    public Result<String> del(@RequestBody ScaleResultInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        scaleResultInfoService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("/write")
    @ApiOperation(value = "填写量表", response = ScaleResultVO.class)
    public Result<ScaleResultVO> write(@RequestBody ScaleResultSaveDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        params.setCustUserId(loginUser.getId());
        ValidatorUtils.validateEntity(params);
        if(StrUtil.isEmpty(params.getPatientId())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"患者编号不能为空");
        }
        ScaleResultVO data= scaleResultInfoService.add(params);
        return new Result<ScaleResultVO>().ok(data);
    }



    @SaCheckPermission("sys:healthy:scale:result:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取量表", response = ScaleResultVO.class)
    public Result<ScaleResultVO> info(@RequestBody ScaleResultInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        ScaleResultVO data= scaleResultInfoService.infoById(params.getId());
        return new Result<ScaleResultVO>().ok(data);
    }



}
