package com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.LabelPatientDataInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelHistoryInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.LabelPatientDataInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.LabelPatientDataAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientActivityLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.LabelPatientActivityDataQueryDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.LabelPatientDataHistogramVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.LabelPatientDataVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientActivityLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.helper.PatientLabelHelper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.mapper.PatientLabelMapper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.LabelPatientDataBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class LabelPatientDataBizServiceImpl implements LabelPatientDataBizService {

    @Autowired
    private LabelPatientDataInfoService labelPatientDataInfoService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private PatientLabelMapper patientLabelMapper;
    @Autowired
    private PatientClientService patientClientService;

    @Override
    public PageData<LabelPatientDataVO> page(LabelQueryPageDTO params) {
        // 查询标签
        params.setStatus(YN.YES.getCode()); // 20231225 下架标签不需要查出来
        PageData<LabelBO> labelBOs = labelClientService.queryPage(params);

        // 整理数据
        PageData<LabelPatientDataVO> data = new PageConverter<LabelBO, LabelPatientDataVO>() {
            @Override
            public LabelPatientDataVO converter(LabelBO bo) {
                LabelPatientDataVO vo = new LabelPatientDataVO();
                vo.setLabelName(bo.getName());
                vo.setLabelCategoryId(bo.getCategoryId());
                vo.setDiseaseIds(bo.getDiseaseIds());
                vo.setLabelId(bo.getId());
                vo.setLabelStatus(bo.getStatus());
                vo.setLabelStatDimensionType(bo.getStatDimensionType());
                vo.setLabelBizType(bo.getBizType());
                vo.setLabelRemark(bo.getRemark());
                vo.setDiseaseList(bo.getDiseaseList());
                return vo;
            }
        }.getPageData(labelBOs.getList(), labelBOs.getTotal(), labelBOs.getSize(), labelBOs.getPages());

        // 查询统计数据放到标签信息中
        if (CollectionUtil.isNotEmpty(data.getList())) {
            List<String> labelIds = new ArrayList<>();
            data.getList().forEach(i -> labelIds.add(i.getLabelId()));

            List<LabelPatientDataInfo> dataList = labelPatientDataInfoService.list(new QueryWrapper<LabelPatientDataInfo>()
                    .in("label_id", labelIds)
            );

            Map<String, LabelPatientDataInfo> dataMap = new HashMap<>();
            dataList.forEach(i -> dataMap.put(i.getLabelId(), i));

            data.getList().forEach(i -> {
                LabelPatientDataInfo entity = dataMap.get(i.getLabelId());
                if (null != entity) {
                    i.setPatientNum(entity.getPatientNum());
                    i.setDataDt(entity.getDataDt());
                } else {
                    i.setPatientNum(0);
                }
            });
        }

        return data;
    }

    @Override
    public PageData<LabelPatientDataVO> page(PatientLabelQueryParam params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        // 查询标签
        LabelQueryPageDTO labelQueryPageDTO = new LabelQueryPageDTO();
        labelQueryPageDTO.setPage(params.getPage());
        labelQueryPageDTO.setRows(params.getRows());
        labelQueryPageDTO.setId(params.getLabelId());
        labelQueryPageDTO.setIds(params.getLabelIds());
        labelQueryPageDTO.setName(params.getLabelName());
        labelQueryPageDTO.setBizType(params.getBizType());
        labelQueryPageDTO.setDiseaseId(params.getDiseaseId());
        labelQueryPageDTO.setDiseaseIds(params.getDiseaseIds());
        labelQueryPageDTO.setDiseaseQueryMode(params.getDiseaseQueryMode());
        labelQueryPageDTO.setStatus(YN.YES.getCode()); // 20231225 下架标签不需要查出来
        labelQueryPageDTO.setAdminLabel(params.getAdminLabel());
        PageData<LabelBO> labelBOs = labelClientService.queryPage(labelQueryPageDTO);

        // 整理数据
        PageData<LabelPatientDataVO> data = new PageConverter<LabelBO, LabelPatientDataVO>() {
            @Override
            public LabelPatientDataVO converter(LabelBO bo) {
                LabelPatientDataVO vo = new LabelPatientDataVO();
                vo.setLabelName(bo.getName());
                vo.setLabelCategoryId(bo.getCategoryId());
                vo.setDiseaseIds(bo.getDiseaseIds());
                vo.setLabelId(bo.getId());
                vo.setLabelStatus(bo.getStatus());
                vo.setLabelStatDimensionType(bo.getStatDimensionType());
                vo.setLabelBizType(bo.getBizType());
                vo.setLabelRemark(bo.getRemark());
                vo.setDiseaseList(bo.getDiseaseList());
                return vo;
            }
        }.getPageData(labelBOs.getList(), labelBOs.getTotal(), labelBOs.getSize(), labelBOs.getPages());

        // 查询统计数据放到标签信息中
        if (CollectionUtil.isNotEmpty(data.getList())) {
            List<String> labelIds = new ArrayList<>();
            data.getList().forEach(i -> labelIds.add(i.getLabelId()));

            // 查询所有用户所有
            Map<String, List<LabelPatientDataHistogramVO>> dataMap = new HashMap<>();
            List<LabelPatientDataHistogramVO> dataList = patientLabelMapper.queryPatientLabelList(new QueryWrapper<PatientLabelInfo>()
                // .eq("a.op_type", OpTypeEnum.Create.getCode())
                .in("a.label_id", labelIds)
                .ge(null != params.getStartDate(), "a.data_dt", params.getStartDate())
                .lt(null != params.getEndDate(), "a.data_dt", params.getEndDate())
            );
            if (CollectionUtil.isNotEmpty(dataList)) {
                dataList.forEach(i -> {
                    List<LabelPatientDataHistogramVO> list = dataMap.get(i.getLabelId());
                    if (null == list) {
                        list = new ArrayList<>();
                        dataMap.put(i.getLabelId(), list);
                    }
                    list.add(i);
                });
            }

            data.getList().forEach(i -> {
                i.setPatientNum(0);

                List<LabelPatientDataHistogramVO> list = dataMap.get(i.getLabelId());
                if (CollectionUtil.isNotEmpty(list)) {
                    list.forEach(j -> {
                        if (null != j.getPatientNum()) {
                            i.setPatientNum(i.getPatientNum() + j.getPatientNum());
                        }
                    });
                }
            });
        }

        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveBatch(List<LabelPatientDataAddBatchDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        // 查询标签统计数据是否存在
        Map<String, LabelPatientDataInfo> labelMap = new HashMap<>(); // 已存在的数据
        List<String> labelIds = params.stream().map(LabelPatientDataAddBatchDTO::getLabelId).collect(Collectors.toList());
        List<LabelPatientDataInfo> existList = labelPatientDataInfoService.list(new QueryWrapper<LabelPatientDataInfo>()
                .in("label_id", labelIds)
        );
        if (CollectionUtil.isNotEmpty(existList)) {
            existList.forEach(i -> labelMap.put(i.getLabelId(), i));
        }

        List<LabelPatientDataInfo> saveList = new ArrayList<>(); // 新增的数据
        List<LabelPatientDataInfo> updateList = new ArrayList<>(); // 修改的数据
        params.forEach(i -> {
            LabelPatientDataInfo exist = labelMap.get(i.getLabelId());
            if (null != exist) { // 更新
                exist.setPatientNum(i.getPatientNum());
                exist.setDataDt(i.getDataDt());
                updateList.add(exist);
                return;
            }

            // 新增
            LabelPatientDataInfo entity = new LabelPatientDataInfo();
            entity.setId(PatientLabelHelper.getLabelPatientDataId());
            entity.setLabelId(i.getLabelId());
            entity.setPatientNum(i.getPatientNum());
            entity.setDataDt(i.getDataDt());
            saveList.add(entity);
        });

        if (CollectionUtil.isNotEmpty(saveList)) {
            labelPatientDataInfoService.saveBatch(saveList);
        }
        if (CollectionUtil.isNotEmpty(updateList)) {
            labelPatientDataInfoService.updateBatchById(updateList);
        }
    }



    @Override
    public PageData<LabelPatientDataVO> activityPage(LabelPatientActivityDataQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        IPage<LabelPatientDataVO> labelPatientCountPage = patientLabelMapper.queryHistoryLabelPatientCount(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientLabelHistoryInfo>()
                .eq("b.rows_status", YN.YES.getCode())
                .eq("b.biz_two_type", "Y")
                .eq("a.op_type", OpTypeEnum.Create.getCode())
                .ge(null != params.getStartDate(), "a.data_dt", params.getStartDate())
                .lt(null != params.getEndDate(), "a.data_dt", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getBizThreeType()), "b.biz_three_type", params.getBizThreeType())
        );

        if (CollectionUtil.isEmpty(labelPatientCountPage.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> labelIds = new ArrayList<>();
        labelPatientCountPage.getRecords().forEach(i -> labelIds.add(i.getLabelId()));

        List<LabelBO> labelList = labelClientService.getLabelListByIds(labelIds);
        Map<String, LabelBO> labelMap = new HashMap<>();
        labelList.forEach(i -> labelMap.put(i.getId(), i));

        labelPatientCountPage.getRecords().forEach(i -> {
            LabelBO labelBo = labelMap.get(i.getLabelId());
            if (null != labelBo) {
                i.setLabelName(labelBo.getName());
                i.setLabelCategoryId(labelBo.getCategoryId());
                i.setLabelStatus(labelBo.getStatus());
                i.setLabelStatDimensionType(labelBo.getStatDimensionType());
                i.setLabelBizType(labelBo.getBizType());
                i.setLabelRemark(labelBo.getRemark());
                i.setDiseaseIds(labelBo.getDiseaseIds());
                i.setDiseaseList(labelBo.getDiseaseList());
                i.setLabelBizTwoType(labelBo.getBizTwoType());
                i.setLabelBizThreeType(labelBo.getBizThreeType());
            }
        });

        return new PageData<>(labelPatientCountPage.getRecords(), labelPatientCountPage.getTotal(), labelPatientCountPage.getSize(), labelPatientCountPage.getPages());
    }

    @Override
    public PageData<PatientActivityLabelVO> activityPatientPage(PatientActivityLabelQueryParam params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        // 查询所有用户所有
        IPage<PatientLabelHistoryInfo> page = patientLabelMapper.queryPatientHistoryLabel(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientLabelHistoryInfo>()
            .eq("a.op_type", OpTypeEnum.Create.getCode())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
            .eq(StrUtil.isNotBlank(params.getLabelId()), "a.label_id", params.getLabelId())
            .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
            .ge(null != params.getStartDate(), "a.op_date", params.getStartDate())
            .lt(null != params.getEndDate(), "a.op_date", params.getEndDate())
        );
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询患者详情
        List<String> patientIds = new ArrayList<>();
        page.getRecords().forEach(i -> patientIds.add(i.getPatientId()));
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientIds(patientIds);
        List<PatientClientBO> patientBOs = patientClientService.simpleList(patientQueryDTO);
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        patientBOs.forEach(i -> patientMap.put(i.getPatientId(), i));


        List<PatientActivityLabelVO> voList = new ArrayList<>();
        page.getRecords().forEach(i -> {
            PatientActivityLabelVO vo = new PatientActivityLabelVO();
            vo.setPatientId(i.getPatientId());
            vo.setDataDt(i.getDataDt());
            if (null == vo.getDataDt()) {
                vo.setDataDt(i.getOpDate());
            }

            PatientClientBO patient = patientMap.get(i.getPatientId());
            if (patient != null) {
                vo.setPatientName(patient.getPatientName());
                vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                vo.setPhone(patient.getPhone());
                vo.setBirthday(patient.getBirthday());
                vo.setAge(patient.getAge());
            }

            voList.add(vo);
        });

        return new PageData<>(voList, page.getTotal(), page.getSize(), page.getPages());
    }

}
