package com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.data.client.api.TaskDeptPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.dto.DeptPatientAddDTO;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelHistoryInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.PatientLabelHistoryInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.PatientLabelInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDelDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDirectRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelSaveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.helper.PatientLabelHelper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.cache.service.PatientLabelCacheService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.dto.DeptQueryDTO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class PatientLabelBizServiceImpl implements PatientLabelBizService {

    @Autowired
    private PatientLabelInfoService patientLabelInfoService;
    @Autowired
    private PatientLabelHistoryInfoService patientLabelHistoryInfoService;
    @Autowired
    private PatientLabelCacheService patientLabelCacheService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private TaskDeptPatientClientService taskDeptPatientClientService;

    /**
     * zs:2023-09-18
     * 1. 已绑定的正常状态的标签不做操作，已过期的更新有效期
     * 
     * zs:20231019
     * 1. 绑定标签时判断标签上的病种与哪些部门的病种有交叉，那就把该患者与该部门进行临时绑定，直到有交叉的标签失效
     * 
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addLabel(PatientLabelAddDTO params) {
        log.info("PatientLabelBizServiceImpl.addLabel(PatientLabelAddDTO): >>>>>> patientId:{}, {}", params.getPatientId(), JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DateTime now = DateUtil.date();
        String validityValue = DateUtil.offsetDay(now, params.getValidityNum()).toString();

        // 判断是否有已绑定的标签，有则过滤不操作
        List<PatientLabelInfo> existList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(params.getLabelIds()), "label_id", params.getLabelIds())
        );
        if (CollectionUtil.isNotEmpty(existList)) {
            Set<String> exclusionPatientLabelIds = new HashSet<>(); // 需要过滤的患者标签id列表
            List<PatientLabelInfo> updatePatientLabelList = new ArrayList<>(); // 需要更新的患者标签列表

            String nowStr = DateUtil.now();

            existList.forEach(i -> {
                if (nowStr.compareTo(i.getValidityValue()) > 0) {
                    updatePatientLabelList.add(i);
                } else {
                    exclusionPatientLabelIds.add(i.getLabelId());
                }
            });

            if (CollectionUtil.isNotEmpty(exclusionPatientLabelIds)) {
                params.getLabelIds().removeIf(i -> exclusionPatientLabelIds.contains(i));
            }

            if (CollectionUtil.isNotEmpty(updatePatientLabelList)) {
                params.getLabelIds().removeIf(i -> updatePatientLabelList.stream().map(j -> j.getLabelId()).collect(Collectors.toList()).contains(i));
                // 更新过期的标签
                this.updatePatienLabel(params, updatePatientLabelList, now, validityValue);
            }
        }

        // 查询标签是否存在
        List<LabelBO> labelBOs = labelClientService.getLabelListByIds(params.getLabelIds());
        if (CollectionUtil.isEmpty(labelBOs)) {
            return;
        }
        Set<String> labelIds = new HashSet<>();
        labelBOs.forEach(i -> labelIds.add(i.getId()));

        List<PatientLabelInfo> labelList = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        labelIds.forEach(i -> {
            if (null == params.getDataDt()) {
                params.setDataDt(now);
            }

            PatientLabelInfo label = new PatientLabelInfo();
            label.setId(PatientLabelHelper.getPatientLabelId());
            label.setPatientId(params.getPatientId());
            label.setLabelId(i);
            label.setRemark(params.getRemark());
            label.setValidityNum(params.getValidityNum());
            label.setValidityValue(validityValue);
            label.setIsValidity(YN.YES.getCode());
            label.setCreator(params.getOpUser());
            label.setCreateDate(now);
            // zs20230920
            label.setSourceType(params.getSourceType());
            label.setSourceRelId(params.getSourceRelId());
            label.setSourceRelTwoId(params.getSourceRelTwoId());
            label.setDataDt(params.getDataDt());

            label.setExtend1(params.getExtend1());
            label.setExtend2(params.getExtend2());
            label.setExtend3(params.getExtend3());
            label.setExtend4(params.getExtend4());

            labelList.add(label);

            // 保存到历史表
            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(params.getPatientId());
            history.setLabelId(i);
            history.setIsValidity(YN.YES.getCode());
            history.setValidityNum(params.getValidityNum());
            history.setValidityValue(validityValue);
            history.setOpUser(params.getOpUser());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.Create.getCode());
            // zs20230920
            history.setSourceType(params.getSourceType());
            history.setSourceRelId(params.getSourceRelId());
            history.setSourceRelTwoId(params.getSourceRelTwoId());
            history.setDataDt(params.getDataDt());

            history.setExtend1(params.getExtend1());
            history.setExtend2(params.getExtend2());
            history.setExtend3(params.getExtend3());
            history.setExtend4(params.getExtend4());

            historyList.add(history);
        });

        // 保存患者标签
        patientLabelInfoService.saveBatch(labelList);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);

        // 删除缓存
        patientLabelCacheService.removeByPatientId(params.getPatientId());

        // zs20231020 根据标签的病种与部门的病种的对应绑定患者与部门
        // List<LabelBO> labelBOs = labelClientService.getLabelListByIds(params.getLabelIds());
        if (CollectionUtil.isNotEmpty(labelBOs)) {
            List<String> labelDiseaseIds = new ArrayList<>();
            labelBOs.forEach(i -> {
                labelDiseaseIds.addAll(i.getDiseaseIds());
            });
            List<DeptBO> deptBOs = deptClientService.getDeptList(new DeptQueryDTO());
            if (CollectionUtil.isNotEmpty(deptBOs)) {
                Set<String> deptIds = new HashSet<>();
                deptBOs.forEach(i -> {
                    List<String> deptDiseaseIds = i.getDiseaseIds();
                    if (CollectionUtil.isNotEmpty(deptDiseaseIds)) {
                        deptDiseaseIds.forEach(item -> {
                            if (labelDiseaseIds.contains(item)) {
                                deptIds.add(i.getId());
                            }
                        });
                    }
                });
                if (CollectionUtil.isNotEmpty(deptIds)) {
                    // 绑定部门与患者
                    List<DeptPatientAddDTO> deptPatientAddDTOs = new ArrayList<>();
                    deptIds.forEach(i -> {
                        DeptPatientAddDTO deptPatientAddDTO = new DeptPatientAddDTO();
                        deptPatientAddDTO.setDeptId(i);
                        deptPatientAddDTO.setPatientId(params.getPatientId());
                        deptPatientAddDTO.setType(DeptPatientTypeEnum.Label.getCode());
                        deptPatientAddDTOs.add(deptPatientAddDTO);
                    });
                    taskDeptPatientClientService.addBatch(deptPatientAddDTOs);
                }
            }
        }

    }

    private void updatePatienLabel(PatientLabelAddDTO params, List<PatientLabelInfo> patientLabelList, DateTime now, String validityValue) {
        log.info("PatientLabelBizServiceImpl.updatePatienLabel(PatientLabelAddDTO, List<PatientLabelInfo>, DateTime, String):>>>>>> {}, {}, {}, {}", JSON.toJSONString(params), JSON.toJSONString(patientLabelList), now.toString(), validityValue);

        List<PatientLabelInfo> labelList = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        patientLabelList.forEach(i -> {
            if (null == params.getDataDt()) {
                params.setDataDt(now);
            }

            PatientLabelInfo label = new PatientLabelInfo();
            label.setId(i.getId());
            label.setPatientId(i.getPatientId());
            label.setLabelId(i.getLabelId());
            label.setRemark(params.getRemark());
            label.setValidityNum(params.getValidityNum());
            label.setValidityValue(validityValue);
            label.setIsValidity(YN.YES.getCode());
            label.setCreator(params.getOpUser());
            label.setCreateDate(now);
            // zs20230920
            label.setSourceType(params.getSourceType());
            label.setSourceRelId(params.getSourceRelId());
            label.setSourceRelTwoId(params.getSourceRelTwoId());
            label.setDataDt(params.getDataDt());

            label.setExtend1(params.getExtend1());
            label.setExtend2(params.getExtend2());
            label.setExtend3(params.getExtend3());
            label.setExtend4(params.getExtend4());

            labelList.add(label);

            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(i.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(YN.YES.getCode());
            history.setValidityNum(params.getValidityNum());
            history.setValidityValue(validityValue);
            history.setOpUser(params.getOpUser());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.Update.getCode());
            // zs20230920
            history.setSourceType(params.getSourceType());
            history.setSourceRelId(params.getSourceRelId());
            history.setSourceRelTwoId(params.getSourceRelTwoId());
            history.setDataDt(params.getDataDt());

            history.setExtend1(params.getExtend1());
            history.setExtend2(params.getExtend2());
            history.setExtend3(params.getExtend3());
            history.setExtend4(params.getExtend4());

            historyList.add(history);
        });

        // 更新患者标签
        patientLabelInfoService.updateBatchById(labelList);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);
    }

    /**
     * zs:2023-09-19
     * 
     * 注意点：
     * 1. 已绑定的正常状态的标签不做操作，已过期的更新有效期
     * 
     * zs:20231020
     * 1. 绑定标签时判断标签上的病种与哪些部门的病种有交叉，那就把该患者与该部门进行临时绑定，直到有交叉的标签失效 TODO
     * 
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatchLabel(PatientLabelAddBatchDTO params) {
        log.info("PatientLabelBizServiceImpl:addBatchLabel:params:>>>>>> patientId:{}, {}", params.getPatientId(), JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DateTime now = DateUtil.date();

        Map<String, PatientLabelDTO> labelMap = new HashMap<>();
        params.getLabelList().forEach(i -> labelMap.put(i.getLabelId(), i));

        Map<String, LabelBO> labelBOMap = labelClientService.getLabelMapByIds(labelMap.keySet());
        if (CollectionUtil.isEmpty(labelBOMap)) {
            return;
        }

        // 判断是否有已绑定的标签，有则过滤不操作
        List<PatientLabelInfo> existList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(labelMap), "label_id", labelMap.keySet())
        );
        if (CollectionUtil.isNotEmpty(existList)) {
            Set<String> exclusionPatientLabelIds = new HashSet<>(); // 需要过滤的患者标签id列表
            List<PatientLabelInfo> updatePatientLabelList = new ArrayList<>(); // 需要更新的患者标签列表

            String nowStr = DateUtil.now();

            existList.forEach(i -> {
                // 过期的标签更新
                if (nowStr.compareTo(i.getValidityValue()) > 0) {
                    updatePatientLabelList.add(i);
                } else {
                    exclusionPatientLabelIds.add(i.getLabelId()); // 未过期的标签不操作
                }
            });

            if (CollectionUtil.isNotEmpty(exclusionPatientLabelIds)) {
                exclusionPatientLabelIds.forEach(i -> {
                    labelMap.remove(i);
                });
            }

            if (CollectionUtil.isNotEmpty(updatePatientLabelList)) {
                Map<String, PatientLabelDTO> updateLabelMap = new HashMap<>(); // 要更新的具体信息

                updatePatientLabelList.forEach(i -> {
                    updateLabelMap.put(i.getLabelId(), labelMap.get(i.getLabelId()));
                    labelMap.remove(i.getLabelId());
                });

                // 更新过期的标签
                this.updateBatchPatienLabel(updatePatientLabelList, now, updateLabelMap, OpTypeEnum.Update.getCode(), params.getOpUser());
            }
        }

        List<PatientLabelInfo> labelList = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        params.getLabelList().forEach(item -> {
            LabelBO labelBO = labelBOMap.get(item.getLabelId());
            if (null == labelBO) {
                return;
            }

            PatientLabelDTO i = labelMap.get(item.getLabelId());
            if (null == i) {
                return;
            }

            if (null == i.getDataDt()) {
                i.setDataDt(now);
            }

            String validityValue = DateUtil.offsetDay(now, i.getValidityNum()).toString();

            PatientLabelInfo label = new PatientLabelInfo();
            label.setId(PatientLabelHelper.getPatientLabelId());
            label.setPatientId(params.getPatientId());
            label.setLabelId(i.getLabelId());
            label.setRemark(i.getRemark());
            label.setValidityNum(i.getValidityNum());
            label.setValidityValue(validityValue);
            label.setIsValidity(YN.YES.getCode());
            label.setCreator(params.getOpUser());
            label.setCreateDate(now);

            label.setSourceType(i.getSourceType());
            label.setSourceRelId(i.getSourceRelId());
            label.setSourceRelTwoId(i.getSourceRelTwoId());
            label.setDataDt(i.getDataDt());

            label.setExtend1(i.getExtend1());
            label.setExtend2(i.getExtend2());
            label.setExtend3(i.getExtend3());
            label.setExtend4(i.getExtend4());

            labelList.add(label);

            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(params.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(YN.YES.getCode());
            history.setValidityNum(i.getValidityNum());
            history.setValidityValue(validityValue);
            history.setOpUser(params.getOpUser());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.Create.getCode());

            history.setSourceType(i.getSourceType());
            history.setSourceRelId(i.getSourceRelId());
            history.setSourceRelTwoId(i.getSourceRelTwoId());
            history.setDataDt(i.getDataDt());

            history.setExtend1(i.getExtend1());
            history.setExtend2(i.getExtend2());
            history.setExtend3(i.getExtend3());
            history.setExtend4(i.getExtend4());

            historyList.add(history);
        });

        if (CollectionUtil.isNotEmpty(labelList)) {
            // 保存患者标签
            patientLabelInfoService.saveBatch(labelList);
            // 保存患者标签操作历史
            patientLabelHistoryInfoService.saveBatch(historyList);
        }

        // 删除缓存
        patientLabelCacheService.removeByPatientId(params.getPatientId());

        // zs20231020 根据标签的病种与部门的病种的对应绑定患者与部门
        List<String> labelIds = new ArrayList<>();
        params.getLabelList().forEach(i -> labelIds.add(i.getLabelId()));
        List<LabelBO> labelBOs = labelClientService.getLabelListByIds(labelIds);
        if (CollectionUtil.isNotEmpty(labelBOs)) {
            List<String> labelDiseaseIds = new ArrayList<>();
            labelBOs.forEach(i -> {
                labelDiseaseIds.addAll(i.getDiseaseIds());
            });
            List<DeptBO> deptBOs = deptClientService.getDeptList(new DeptQueryDTO());
            if (CollectionUtil.isNotEmpty(deptBOs)) {
                Set<String> deptIds = new HashSet<>();
                deptBOs.forEach(i -> {
                    List<String> deptDiseaseIds = i.getDiseaseIds();
                    if (CollectionUtil.isNotEmpty(deptDiseaseIds)) {
                        deptDiseaseIds.forEach(item -> {
                            if (labelDiseaseIds.contains(item)) {
                                deptIds.add(i.getId());
                            }
                        });
                    }
                });
                if (CollectionUtil.isNotEmpty(deptIds)) {
                    // 绑定部门与患者
                    List<DeptPatientAddDTO> deptPatientAddDTOs = new ArrayList<>();
                    deptIds.forEach(i -> {
                        DeptPatientAddDTO deptPatientAddDTO = new DeptPatientAddDTO();
                        deptPatientAddDTO.setDeptId(i);
                        deptPatientAddDTO.setPatientId(params.getPatientId());
                        deptPatientAddDTO.setType(DeptPatientTypeEnum.Label.getCode());
                        deptPatientAddDTOs.add(deptPatientAddDTO);
                    });
                    taskDeptPatientClientService.addBatch(deptPatientAddDTOs);
                }
            }
        }

    }

    private void updateBatchPatienLabel(List<PatientLabelInfo> patientLabelList, DateTime now, Map<String, PatientLabelDTO> updateLabelMap, String opType, String opUser) {
        log.info("PatientLabelBizServiceImpl:updateBatchPatienLabel:patientLabelList:>>>>>> {}", JSON.toJSONString(patientLabelList));
        log.info("PatientLabelBizServiceImpl:updateBatchPatienLabel:now:>>>>>> {}", now.toString());
        
        List<PatientLabelInfo> labelList = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        patientLabelList.forEach(i -> {
            PatientLabelDTO patientLabelDTO = updateLabelMap.get(i.getLabelId());
            String validityValue = DateUtil.offsetDay(now, patientLabelDTO.getValidityNum()).toString();

            if (null == i.getDataDt()) {
                i.setDataDt(now);
            }

            PatientLabelInfo label = new PatientLabelInfo();
            label.setId(i.getId());
            label.setPatientId(i.getPatientId());
            label.setLabelId(i.getLabelId());
            label.setRemark(patientLabelDTO.getRemark());
            label.setValidityNum(patientLabelDTO.getValidityNum());
            label.setValidityValue(validityValue);
            label.setIsValidity(YN.YES.getCode());
            label.setCreator(opUser);
            label.setCreateDate(now);
            
            label.setSourceType(patientLabelDTO.getSourceType());
            label.setSourceRelId(patientLabelDTO.getSourceRelId());
            label.setSourceRelTwoId(patientLabelDTO.getSourceRelTwoId());
            label.setDataDt(i.getDataDt());

            label.setExtend1(i.getExtend1());
            label.setExtend2(i.getExtend2());
            label.setExtend3(i.getExtend3());
            label.setExtend4(i.getExtend4());

            labelList.add(label);

            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(i.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(YN.YES.getCode());
            history.setValidityNum(patientLabelDTO.getValidityNum());
            history.setValidityValue(validityValue);
            history.setOpUser(opUser);
            history.setOpDate(now);
            history.setOpType(opType);
            
            history.setSourceType(patientLabelDTO.getSourceType());
            history.setSourceRelId(patientLabelDTO.getSourceRelId());
            history.setSourceRelTwoId(patientLabelDTO.getSourceRelTwoId());
            history.setDataDt(i.getDataDt());

            history.setExtend1(i.getExtend1());
            history.setExtend2(i.getExtend2());
            history.setExtend3(i.getExtend3());
            history.setExtend4(i.getExtend4());

            historyList.add(history);
        });

        // 更新患者标签
        patientLabelInfoService.updateBatchById(labelList);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void removeBatchLabel(PatientLabelRemoveDTO params) {
        log.info("PatientLabelBizServiceImpl:removeBatchLabel:params:>>>>>> {}", JSON.toJSONString(params));

        DateTime now = DateUtil.date();

        // 查询标签列表，用于做历史记录
        List<PatientLabelInfo> patientLabelList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
                    .eq("patient_id", params.getPatientId())
                    .in("label_id", params.getLabelIds())
        );

        if (CollectionUtil.isEmpty(patientLabelList)) {
            return;
        }

        List<String> patientLabelIds = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        patientLabelList.forEach(i -> {
            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(i.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(YN.NO.getCode());
            history.setValidityNum(i.getValidityNum());
            history.setValidityValue(i.getValidityValue());
            history.setOpUser(params.getOpUser());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());
            
            history.setSourceType(i.getSourceType());
            history.setSourceRelId(i.getSourceRelId());
            history.setSourceRelTwoId(i.getSourceRelTwoId());
            history.setDataDt(i.getDataDt());

            history.setExtend1(i.getExtend1());
            history.setExtend2(i.getExtend2());
            history.setExtend3(i.getExtend3());
            history.setExtend4(i.getExtend4());

            historyList.add(history);

            patientLabelIds.add(i.getId());
        });

        // 删除患者标签
        patientLabelInfoService.removeBatchByIds(patientLabelIds);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);

        // 删除缓存
        patientLabelCacheService.removeByPatientId(params.getPatientId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveLabelBatch(List<PatientLabelSaveDTO> params) {
        log.info("PatientLabelBizServiceImpl:saveLabelBatch:params:>>>>>> {}", JSON.toJSONString(params));
        params.forEach(i -> {
            ValidatorUtils.validateEntity(params);
        });

        DateTime now = DateUtil.date();

        List<String> patientLabelIds = params.stream().map(i -> {
            return i.getPatientId() + "-" + i.getLabelId();
        }).collect(Collectors.toList());

        // 判断是否有已绑定的标签，有则过滤不操作
        Set<String> existPatientLabnelIds = new HashSet<>();
        if (CollectionUtil.isNotEmpty(patientLabelIds)) {
            List<PatientLabelInfo> existList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
                .in("CONCAT(patient_id, '-', label_id)", patientLabelIds)
            );
            existList.forEach(i -> {
                existPatientLabnelIds.add(i.getPatientId() + "-" + i.getLabelId());
            });
        }

        List<PatientLabelInfo> labelList = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        params.forEach(i -> {
            if (!existPatientLabnelIds.contains(i.getPatientId() + "-" + i.getLabelId())) {
                PatientLabelInfo label = new PatientLabelInfo();
                label.setId(PatientLabelHelper.getPatientLabelId());
                label.setPatientId(i.getPatientId());
                label.setLabelId(i.getLabelId());
                label.setRemark(i.getRemark());
                label.setValidityNum(i.getValidityNum());
                label.setIsValidity(YN.YES.getCode());
                label.setCreateDate(now);

                label.setSourceType(i.getSourceType());
                label.setSourceRelId(i.getSourceRelId());
                label.setSourceRelTwoId(i.getSourceRelTwoId());
                label.setDataDt(i.getDataDt());

                if (null != i.getDataDt()) {
                    label.setValidityValue(DateUtil.offsetDay(i.getDataDt(), i.getValidityNum()).toString());
                }
                
                label.setExtend1(i.getExtend1());
                label.setExtend2(i.getExtend2());
                label.setExtend3(i.getExtend3());
                label.setExtend4(i.getExtend4());

                labelList.add(label);

                PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
                history.setId(PatientLabelHelper.getPatientLabelHistoryId());
                history.setPatientId(i.getPatientId());
                history.setLabelId(i.getLabelId());
                history.setIsValidity(YN.YES.getCode());
                history.setValidityNum(i.getValidityNum());
                history.setOpDate(now);
                history.setOpType(OpTypeEnum.Create.getCode());

                history.setSourceType(i.getSourceType());
                history.setSourceRelId(i.getSourceRelId());
                history.setSourceRelTwoId(i.getSourceRelTwoId());
                history.setDataDt(i.getDataDt());

                if (null != i.getDataDt()) {
                    history.setValidityValue(label.getValidityValue());
                }
                
                history.setExtend1(i.getExtend1());
                history.setExtend2(i.getExtend2());
                history.setExtend3(i.getExtend3());
                history.setExtend4(i.getExtend4());

                historyList.add(history);
            }
        });

        // 保存患者标签
        patientLabelInfoService.saveBatch(labelList);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);

        // 删除缓存
        params.forEach(i -> {
            patientLabelCacheService.removeByPatientId(i.getPatientId());
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delLabelBatch(List<PatientLabelDelDTO> params) {
        log.info("PatientLabelBizServiceImpl:delLabelBatch:params:>>>>>> {}", JSON.toJSONString(params));

        DateTime now = DateUtil.date();

        // 查询标签列表，用于做历史记录
        List<String> patientLabelIdList = params.stream().map(i -> {
            return i.getPatientId() + "-" + i.getLabelId();
        }).collect(Collectors.toList());

        // 判断是否有已绑定的标签，有则过滤不操作
        if (CollectionUtil.isEmpty(patientLabelIdList)) {
            return;
        }

        List<PatientLabelInfo> existList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
            .in("CONCAT(patient_id, '-', label_id)", patientLabelIdList)
        );

        if (CollectionUtil.isEmpty(existList)) {
            return;
        }

        List<String> patientLabelIds = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        existList.forEach(i -> {
            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(i.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(i.getIsValidity());
            history.setValidityNum(i.getValidityNum());
            history.setValidityValue(i.getValidityValue());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());

            history.setSourceType(i.getSourceType());
            history.setSourceRelId(i.getSourceRelId());
            history.setSourceRelTwoId(i.getSourceRelTwoId());
            history.setDataDt(i.getDataDt());

            history.setExtend1(i.getExtend1());
            history.setExtend2(i.getExtend2());
            history.setExtend3(i.getExtend3());
            history.setExtend4(i.getExtend4());

            historyList.add(history);

            patientLabelIds.add(i.getId());
        });

        // 删除患者标签
        patientLabelInfoService.removeBatchByIds(patientLabelIds);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);

        // 删除缓存
        params.forEach(i -> {
            patientLabelCacheService.removeByPatientId(i.getPatientId());
        });
    }

    @Override
    public void removeBatchLabelByIds(List<String> ids) {
        if (CollectionUtil.isEmpty(ids)) {
            return;
        }
        
        List<PatientLabelInfo> existList = patientLabelInfoService.list(new QueryWrapper<PatientLabelInfo>()
            .in("id", ids)
        );
        if (CollectionUtil.isEmpty(existList)) {
            return;
        }

        DateTime now = DateUtil.date();
        List<String> patientLabelIds = new ArrayList<>();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        existList.forEach(i -> {
            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(i.getPatientId());
            history.setLabelId(i.getLabelId());
            history.setIsValidity(i.getIsValidity());
            history.setValidityNum(i.getValidityNum());
            history.setValidityValue(i.getValidityValue());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());

            history.setSourceType(i.getSourceType());
            history.setSourceRelId(i.getSourceRelId());
            history.setSourceRelTwoId(i.getSourceRelTwoId());
            history.setDataDt(i.getDataDt()); // 历史的数据时间就是标签绑定时的创建时间

            history.setExtend1(i.getExtend1());
            history.setExtend2(i.getExtend2());
            history.setExtend3(i.getExtend3());
            history.setExtend4(i.getExtend4());

            historyList.add(history);

            patientLabelIds.add(i.getId());
        });

        // 删除患者标签
        patientLabelInfoService.removeBatchByIds(patientLabelIds);
        // 保存患者标签操作历史
        patientLabelHistoryInfoService.saveBatch(historyList);

        // 删除缓存
        existList.forEach(i -> {
            patientLabelCacheService.removeByPatientId(i.getPatientId());
        });
    }

    @Override
    public void saveHistoryLabelBatch(List<PatientLabelSaveDTO> params) {
        
        Date now = DateUtil.date();
        List<PatientLabelHistoryInfo> historyList = new ArrayList<>();

        params.forEach(item -> {
            PatientLabelHistoryInfo history = new PatientLabelHistoryInfo();
            history.setId(PatientLabelHelper.getPatientLabelHistoryId());
            history.setPatientId(item.getPatientId());
            history.setLabelId(item.getLabelId());
            history.setIsValidity(YN.YES.getCode());
            history.setValidityNum(item.getValidityNum());
            history.setDataDt(item.getDataDt());
            if (null != item.getDataDt()) {
                history.setValidityValue(DateUtil.offsetDay(item.getDataDt(), item.getValidityNum()).toString());
            }
            history.setOpUser(item.getOpUser());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());

            history.setSourceType(item.getSourceType());
            history.setSourceRelId(item.getSourceRelId());
            history.setSourceRelTwoId(item.getSourceRelTwoId());

            history.setExtend1(item.getExtend1());
            history.setExtend2(item.getExtend2());
            history.setExtend3(item.getExtend3());
            history.setExtend4(item.getExtend4());

            historyList.add(history);
        });

        if (CollectionUtil.isNotEmpty(historyList)) {
            // 保存患者标签操作历史
            patientLabelHistoryInfoService.saveBatch(historyList);
        }
    }

    @Override
    public void directDelNowPatientLabelBatch(PatientLabelDirectRemoveDTO params) {
        ValidatorUtils.validateEntity(params);
        if (StrUtil.isBlank(params.getPatientId()) && StrUtil.isBlank(params.getLabelId()) && CollectionUtil.isEmpty(params.getPatientIds()) && CollectionUtil.isEmpty(params.getLabelIds())) {
            throw new BusinessException("请指定至少一个必须参数");
        }
        // 物理删除当前标签 不删除当前标签对应的历史标签
        patientLabelInfoService.remove(new QueryWrapper<PatientLabelInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getLabelId()), "label_id", params.getLabelId())
                .in(CollectionUtil.isNotEmpty(params.getLabelIds()), "label_id", params.getLabelIds())
                .eq(StrUtil.isNotBlank(params.getSourceType()), "source_type", params.getSourceType())
                .eq(StrUtil.isNotBlank(params.getSourceRelId()), "source_rel_id", params.getSourceRelId())
                .eq(StrUtil.isNotBlank(params.getSourceRelTwoId()), "source_rel_two_id", params.getSourceRelTwoId())
                .ge(null != params.getStartDate(), "data_dt", params.getStartDate())
                .le(null != params.getEndDate(), "data_dt", params.getEndDate())
        );
    }

    @Override
    public void directDelHistoryPatientLabelBatch(PatientLabelDirectRemoveDTO params) {
        ValidatorUtils.validateEntity(params);
        if (StrUtil.isBlank(params.getPatientId()) && StrUtil.isBlank(params.getLabelId()) && CollectionUtil.isEmpty(params.getPatientIds()) && CollectionUtil.isEmpty(params.getLabelIds())) {
            throw new BusinessException("请指定至少一个必须参数");
        }
        // 物理删除历史标签
        patientLabelHistoryInfoService.remove(new QueryWrapper<PatientLabelHistoryInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getLabelId()), "label_id", params.getLabelId())
                .in(CollectionUtil.isNotEmpty(params.getLabelIds()), "label_id", params.getLabelIds())
                .eq(StrUtil.isNotBlank(params.getSourceType()), "source_type", params.getSourceType())
                .eq(StrUtil.isNotBlank(params.getSourceRelId()), "source_rel_id", params.getSourceRelId())
                .eq(StrUtil.isNotBlank(params.getSourceRelTwoId()), "source_rel_two_id", params.getSourceRelTwoId())
                .ge(null != params.getStartDate(), "data_dt", params.getStartDate())
                .le(null != params.getEndDate(), "data_dt", params.getEndDate())
        );
    }

}
