package com.xxh.cloud.healthy.serve.label.framework.modules.cache.service.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelHistoryInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.PatientLabelHistoryInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.PatientLabelInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelHistoryVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.helper.PatientLabelHelper;
import com.xxh.cloud.healthy.serve.label.framework.modules.cache.helper.PatientLabelCacheKeysHelper;
import com.xxh.cloud.healthy.serve.label.framework.modules.cache.service.PatientLabelCacheService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * zs20230921
 */
@Slf4j
@Service
public class PatientLabelCacheServiceImpl implements PatientLabelCacheService {

    @Autowired
    protected RedisUtils redisUtils;
    @Autowired
    private PatientLabelInfoService patientLabelInfoService;
    @Autowired
    private PatientLabelHistoryInfoService patientLabelHistoryInfoService;
    @Autowired
    private LabelClientService labelClientService;

    @Override
    public List<PatientLabelVO> listByPatientId(String patientId) {
        String key = PatientLabelCacheKeysHelper.getPatientLabelKey(patientId);

        List<PatientLabelVO> data = null;

        Object obj = this.redisUtils.get(key);
        if (null != obj) {
            data = JSON.parseArray(obj.toString(), PatientLabelVO.class);
        }

        if (CollectionUtil.isEmpty(data)) {
            List<PatientLabelInfo> list = patientLabelInfoService.list(
                new QueryWrapper<PatientLabelInfo>().eq("patient_id", patientId)
            );
            data = list.stream().map(PatientLabelHelper::getPatientLabelVO).collect(Collectors.toList());

            if(CollUtil.isNotEmpty(data)){
                redisUtils.set(key, JSON.toJSONString(data), -1); // 存到redis
            }
        }
        
        if (CollectionUtil.isNotEmpty(data)) {
            List<String> labelIds = new ArrayList<>();
            data.forEach(i -> labelIds.add(i.getLabelId()));

            if (CollectionUtil.isNotEmpty(labelIds)) {
                Map<String, LabelBO> labelMap = labelClientService.getLabelMapByIds(labelIds);
                if (CollectionUtil.isNotEmpty(labelMap)) {
                    data.forEach(i -> i.setLabelBO(labelMap.get(i.getLabelId())));
                }
            }
        }

        return data;
    }

    @Override
    public Map<String, List<PatientLabelVO>> mapByPatientIds(List<String> patientIds) {
        log.info("1.参数>>>>>> {}", JSON.toJSONString(patientIds));
        Set<String> keys = patientIds.stream().map(PatientLabelCacheKeysHelper::getPatientLabelKey).collect(Collectors.toSet());

        Map<String, List<PatientLabelVO>> patientLabelMap = new HashMap<>();
        List<Object> objList = this.redisUtils.getBatchList(keys);
        log.info("2.缓存中获取>>>>>> {}", JSON.toJSONString(objList));
        if (CollUtil.isNotEmpty(objList)) {
            objList.forEach(i -> {
                if (null == i) {
                    return;
                }
                List<PatientLabelVO> voList = JSONArray.parseArray(i.toString(), PatientLabelVO.class);
                voList.forEach(vo -> {
                    List<PatientLabelVO> vos = patientLabelMap.get(vo.getPatientId());
                    if (null == vos) {
                        vos = new ArrayList<>();
                        patientLabelMap.put(vo.getPatientId(), vos);
                    }
                    vos.add(vo);
                });
            });
        }

        // 从数据库中查询
        if (CollUtil.isEmpty(patientLabelMap)) {
            List<PatientLabelInfo> plList = patientLabelInfoService.list(
                new QueryWrapper<PatientLabelInfo>().in("patient_id", patientIds)
            );
            if(CollUtil.isEmpty(plList)){
                return new HashMap<>();
            }
            plList.forEach(i -> {
                List<PatientLabelVO> list = patientLabelMap.get(i.getPatientId());
                if (null == list) {
                    list = new ArrayList<>();
                    patientLabelMap.put(i.getPatientId(), list);
                }

                list.add(PatientLabelHelper.getPatientLabelVO(i));
            });

            patientLabelMap.forEach((i,j) -> {
                redisUtils.set(i, JSON.toJSONString(j), -1);
            });
        }
        
        if (CollectionUtil.isNotEmpty(patientLabelMap)) {
            List<String> labelIds = new ArrayList<>();
            patientLabelMap.forEach((k, v) -> {
                v.forEach(i -> labelIds.add(i.getLabelId()));
            });

            if (CollectionUtil.isNotEmpty(labelIds)) {
                Map<String, LabelBO> labelMap = labelClientService.getLabelMapByIds(labelIds);
                if (CollectionUtil.isNotEmpty(labelMap)) {
                    patientLabelMap.forEach((k, v) -> {
                        v.forEach(i -> {
                            i.setLabelBO(labelMap.get(i.getLabelId()));
                        });
                    });
                }
            }
        }

        return patientLabelMap;
    }

    @Override
    public void removeByPatientId(String patientId) {
        String key = PatientLabelCacheKeysHelper.getPatientLabelKey(patientId);
        redisUtils.delete(key);
    }

    @Override
    public List<PatientLabelHistoryVO> historyListByPatientId(String patientId) {
        String key = PatientLabelCacheKeysHelper.getPatientLabelHistoryKey(patientId);

        Object obj = this.redisUtils.get(key);
        if (null != obj) {
            return JSON.parseArray(obj.toString(), PatientLabelHistoryVO.class);
        }

        List<PatientLabelHistoryInfo> list = patientLabelHistoryInfoService.list(
            new QueryWrapper<PatientLabelHistoryInfo>().eq("patient_id", patientId)
        );
        if(CollUtil.isNotEmpty(list)){
            redisUtils.set(key, JSON.toJSONString(list), -1);
        }

        return list.stream().map(PatientLabelHelper::getPatientLabelHistoryVO).collect(Collectors.toList());
    }

    @Override
    public Map<String, List<PatientLabelHistoryVO>> historyMapByPatientIds(List<String> patientIds) {
        Set<String> keys = patientIds.stream().map(PatientLabelCacheKeysHelper::getPatientLabelHistoryKey).collect(Collectors.toSet());

        List<Object> objList = this.redisUtils.getBatchList(keys);
        if (CollUtil.isNotEmpty(objList)) {
            Map<String, List<PatientLabelHistoryVO>> patientLabelHistoryMap = new HashMap<>();
            objList.forEach(i -> {
                PatientLabelHistoryVO vo = JSON.parseObject(i.toString(), PatientLabelHistoryVO.class);
                List<PatientLabelHistoryVO> list = patientLabelHistoryMap.get(vo.getPatientId());
                if (null == list) {
                    list = new ArrayList<>();
                    patientLabelHistoryMap.put(vo.getPatientId(), list);
                }
                list.add(vo);
            });
            return patientLabelHistoryMap;
        }

        List<PatientLabelHistoryInfo> plList = patientLabelHistoryInfoService.list(
            new QueryWrapper<PatientLabelHistoryInfo>().in("patient_id", patientIds)
        );
        if(CollUtil.isEmpty(plList)){
            return new HashMap<>();
        }
        Map<String, List<PatientLabelHistoryVO>> patientLabelHistoryMap = new HashMap<>();
        plList.forEach(i -> {
            List<PatientLabelHistoryVO> list = patientLabelHistoryMap.get(i.getPatientId());
            if (null == list) {
                list = new ArrayList<>();
                patientLabelHistoryMap.put(i.getPatientId(), list);
            }
            list.add(PatientLabelHelper.getPatientLabelHistoryVO(i));
        });

        patientLabelHistoryMap.forEach((i,j) -> {
            redisUtils.set(i, JSON.toJSONString(j), -1);
        });

        return patientLabelHistoryMap;
    }

    @Override
    public void removeHistoryByPatientId(String patientId) {
        String key = PatientLabelCacheKeysHelper.getPatientLabelHistoryKey(patientId);
        redisUtils.delete(key);
    }

}
