package com.xxh.cloud.healthy.serve.label.framework.modules.client.service;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelSaveDTO;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorDwdPatientBO;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.*;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDelDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDirectRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelSaveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelHistoryVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelQueryService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * zs20230918
 */
@Slf4j
@Service
public class PatientLabelClientServiceImpl implements PatientLabelClientService {

    @Autowired
    private PatientLabelBizService patientLabelBizService;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private PatientLabelQueryService patientLabelQueryService;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;

    @Override
    public void add(LabelAddDTO params) {
        
        // 判断患者是否已去世
        DoctorDwdPatientBO patientClientBO = dwdDataPatientClientService.getOnePatient(params.getPatientId());
        if (null == patientClientBO) {
            return;
        }
        if (YN.NO.getCode().equals(patientClientBO.getDeathStatus())) {
            log.info("患者已去世 >>>>>> {}", JSON.toJSONString(patientClientBO));
            return;
        }


        PatientLabelAddDTO patientLabelAddDTO = new PatientLabelAddDTO();
        patientLabelAddDTO.setPatientId(params.getPatientId());
        patientLabelAddDTO.setLabelIds(params.getLabelIds());
        patientLabelAddDTO.setValidityNum(params.getValidityNum());
        patientLabelAddDTO.setRemark(params.getRemark());
        patientLabelAddDTO.setOpUser(params.getOpUser());
        patientLabelAddDTO.setSourceRelId(params.getSourceRelId());
        patientLabelAddDTO.setSourceRelTwoId(params.getSourceRelTwoId());
        patientLabelAddDTO.setSourceType(params.getSourceType());
        if (null != params.getDataDt()) {
            patientLabelAddDTO.setDataDt(params.getDataDt());
        } else {
            patientLabelAddDTO.setDataDt(DateUtil.date());
        }

        patientLabelAddDTO.setExtend1(params.getExtend1());
        patientLabelAddDTO.setExtend2(params.getExtend2());
        patientLabelAddDTO.setExtend3(params.getExtend3());
        patientLabelAddDTO.setExtend4(params.getExtend4());

        patientLabelBizService.addLabel(patientLabelAddDTO);
    }

    @Override
    public void addBatch(LabelAddBatchDTO params) {
        if (null == params || CollectionUtil.isEmpty(params.getLabelList())) {
            return;
        }
        
        // 判断患者是否已去世
        DoctorDwdPatientBO patientClientBO = dwdDataPatientClientService.getOnePatient(params.getPatientId());
        if (null == patientClientBO) {
            return;
        }
        if (YN.NO.getCode().equals(patientClientBO.getDeathStatus())) {
            log.info("患者已去世 >>>>>> {}", JSON.toJSONString(patientClientBO));
            return;
        }

        PatientLabelAddBatchDTO patientLabelAddBatchDTO = new PatientLabelAddBatchDTO();
        patientLabelAddBatchDTO.setPatientId(params.getPatientId());
        patientLabelAddBatchDTO.setOpUser(params.getOpUser());

        Set<String> dataCheck = new HashSet<>(); // 用于需要保存的数据去重
        List<PatientLabelDTO> patientLabelDTOs = new ArrayList<>();
        params.getLabelList().forEach(i -> {
            if (dataCheck.contains(i.getLabelId())) {
                return;
            }

            PatientLabelDTO patientLabelDTO = new PatientLabelDTO();
            patientLabelDTO.setLabelId(i.getLabelId());
            patientLabelDTO.setValidityNum(i.getValidityNum());
            patientLabelDTO.setRemark(i.getRemark());
            patientLabelDTO.setSourceRelId(i.getSourceRelId());
            patientLabelDTO.setSourceRelTwoId(i.getSourceRelTwoId());
            patientLabelDTO.setSourceType(i.getSourceType());

            if (null != i.getDataDt()) {
                patientLabelDTO.setDataDt(i.getDataDt());
            } else {
                patientLabelDTO.setDataDt(DateUtil.date());
            }

            patientLabelDTO.setExtend1(i.getExtend1());
            patientLabelDTO.setExtend2(i.getExtend2());
            patientLabelDTO.setExtend3(i.getExtend3());
            patientLabelDTO.setExtend4(i.getExtend4());

            patientLabelDTOs.add(patientLabelDTO);

            dataCheck.add(i.getLabelId());
        });

        // List<PatientLabelDTO> patientLabelDTOs = params.getLabelList().stream().map(i -> {
        //     PatientLabelDTO patientLabelDTO = new PatientLabelDTO();
        //     patientLabelDTO.setLabelId(i.getLabelId());
        //     patientLabelDTO.setValidityNum(i.getValidityNum());
        //     patientLabelDTO.setRemark(i.getRemark());
        //     patientLabelDTO.setSourceRelId(i.getSourceRelId());
        //     patientLabelDTO.setSourceRelTwoId(i.getSourceRelTwoId());
        //     patientLabelDTO.setSourceType(i.getSourceType());

        //     if (null != i.getDataDt()) {
        //         patientLabelDTO.setDataDt(i.getDataDt());
        //     } else {
        //         patientLabelDTO.setDataDt(DateUtil.date());
        //     }

        //     patientLabelDTO.setExtend1(i.getExtend1());
        //     patientLabelDTO.setExtend2(i.getExtend2());
        //     patientLabelDTO.setExtend3(i.getExtend3());
        //     patientLabelDTO.setExtend4(i.getExtend4());

        //     return patientLabelDTO;
        // }).collect(Collectors.toList());

        patientLabelAddBatchDTO.setLabelList(patientLabelDTOs);

        patientLabelBizService.addBatchLabel(patientLabelAddBatchDTO);
    }

    @Override
    public void remove(LabelRemoveDTO params) {

        PatientLabelRemoveDTO patientLabelRemoveDTO = new PatientLabelRemoveDTO();
        patientLabelRemoveDTO.setPatientId(params.getPatientId());
        patientLabelRemoveDTO.setLabelIds(params.getLabelIds());
        patientLabelRemoveDTO.setOpUser(params.getOpUser());

        patientLabelBizService.removeBatchLabel(patientLabelRemoveDTO);
    }

    @Override
    public void addBatch2(List<LabelAddOneDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        
        // 判断患者是否已去世
        List<String> patientIds = new ArrayList<>();
        params.forEach(i -> patientIds.add(i.getPatientId()));
        List<DoctorDwdPatientBO> patientClientBOList = dwdDataPatientClientService.queryPatientList(patientIds);
        if (CollectionUtil.isEmpty(patientClientBOList)) {
            return;
        }
        Map<String, DoctorDwdPatientBO> patientMap = new HashMap<>();
        patientClientBOList.forEach(i -> patientMap.put(i.getPatientId(), i));

        Iterator<LabelAddOneDTO> it = params.iterator();
        while (it.hasNext()) {
            DoctorDwdPatientBO bo = patientMap.get(it.next().getPatientId());
            if (YN.NO.getCode().equals(bo.getDeathStatus())) {
                log.info("患者已去世 >>>>>> {}", JSON.toJSONString(bo));
                it.remove();
            }
        }

        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        //todo:
        log.info("1.住院流失标签>>>>>> params={}", JSON.toJSONString(params));

        // 1.查询标签
        List<String> patientHistoryLabelQueryParams = new ArrayList<>(); // 历史标签
        params.forEach(i -> patientHistoryLabelQueryParams.add(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType() + "-" + i.getSourceRelId() + "-" + DateUtil.formatDateTime(i.getDataDt())));
        log.info("2.查询标签>>>>>> patientHistoryLabelQueryParams={}", JSON.toJSONString(patientHistoryLabelQueryParams));

        // 查询历史标签，如果有则跳过，没有则判断当前标签
        PatientLabelQueryParam patientHistoryLabelQueryParam = new PatientLabelQueryParam();
        patientHistoryLabelQueryParam.setQueryParams(patientHistoryLabelQueryParams);
        List<PatientLabelHistoryVO> historyLabelList = patientLabelQueryService.historyList(patientHistoryLabelQueryParam);
        log.info("3.查询历史标签<<<<<< historyLabelList={}", JSON.toJSONString(historyLabelList));
        Map<String, PatientLabelHistoryVO> historyLabelMap = new HashMap<>(); // 存在的历史标签，直接return
        historyLabelList.forEach(i -> historyLabelMap.put(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType() + "-" + i.getSourceRelId() + "-" + DateUtil.formatDateTime(i.getDataDt()), i));
        // 过滤掉历史已存在的数据
        params.removeIf(i -> null != historyLabelMap.get(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType() + "-" + i.getSourceRelId() + "-" + DateUtil.formatDateTime(i.getDataDt())));

        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        // 查询当前标签，如果有则判断时间，时间相等则跳过，大于给定的时间，则直接插入数据到历史，小于给定的时间，则移除当前标签并打上新标签
        List<String> patientLabelQueryParams = new ArrayList<>(); // 当前标签
        params.forEach(i -> patientLabelQueryParams.add(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType()));
        log.info("4.查询标签>>>>>> patientLabelQueryParams={}", JSON.toJSONString(patientLabelQueryParams));
        
        PatientLabelQueryParam patientLabelQueryParam = new PatientLabelQueryParam();
        patientLabelQueryParam.setQueryParams(patientLabelQueryParams);
        List<PatientLabelVO> labelList = patientLabelQueryService.list(patientLabelQueryParam);
        log.info("5.查询当前标签<<<<<< labelList={}", JSON.toJSONString(labelList));
        Map<String, PatientLabelVO> labelMap = new HashMap<>(); // 存在当前标签标签，直接return
        labelList.forEach(i -> labelMap.put(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType(), i));
        
        List<PatientLabelSaveDTO> patientLabelSaveDTOs = new ArrayList<>(); // 新增标签
        List<String> patientLabelRemoveIds = new ArrayList<>(); // 要删除的标签
        List<PatientLabelSaveDTO> patientLableHistorySave = new ArrayList<>();
        // 判断标签是否存在
        params.forEach(i -> {
            PatientLabelVO patientLabelVO = labelMap.get(i.getPatientId() + "-" + i.getLabelId() + "-" + i.getSourceType());
            if (null == patientLabelVO) { // 不存在则新增
                PatientLabelSaveDTO patientLabelSaveDTO = new PatientLabelSaveDTO();
                patientLabelSaveDTO.setLabelId(i.getLabelId());
                patientLabelSaveDTO.setPatientId(i.getPatientId());
                patientLabelSaveDTO.setValidityNum(i.getValidityNum());
                patientLabelSaveDTO.setDataDt(i.getDataDt());
                patientLabelSaveDTO.setSourceType(i.getSourceType());
                patientLabelSaveDTO.setSourceRelId(i.getSourceRelId());
                patientLabelSaveDTO.setDataDt(i.getDataDt());

                patientLabelSaveDTOs.add(patientLabelSaveDTO);
            } else if (i.getDataDt().after(patientLabelVO.getDataDt())) { // 时间大于当前标签时间则新增，并移除旧标签
                PatientLabelSaveDTO patientLabelSaveDTO = new PatientLabelSaveDTO();
                patientLabelSaveDTO.setLabelId(i.getLabelId());
                patientLabelSaveDTO.setPatientId(i.getPatientId());
                patientLabelSaveDTO.setValidityNum(i.getValidityNum());
                patientLabelSaveDTO.setDataDt(i.getDataDt());
                patientLabelSaveDTO.setSourceType(i.getSourceType());
                patientLabelSaveDTO.setSourceRelId(i.getSourceRelId());
                patientLabelSaveDTO.setDataDt(i.getDataDt());

                patientLabelSaveDTOs.add(patientLabelSaveDTO);

                patientLabelRemoveIds.add(patientLabelVO.getId());
            } else if (i.getDataDt().before(patientLabelVO.getDataDt())) { // 时间小于当前标签时间则直接添加到历史
                PatientLabelSaveDTO patientLabelSaveDTO = new PatientLabelSaveDTO();
                patientLabelSaveDTO.setLabelId(i.getLabelId());
                patientLabelSaveDTO.setPatientId(i.getPatientId());
                patientLabelSaveDTO.setValidityNum(i.getValidityNum());
                patientLabelSaveDTO.setDataDt(i.getDataDt());
                patientLabelSaveDTO.setSourceType(i.getSourceType());
                patientLabelSaveDTO.setSourceRelId(i.getSourceRelId());
                patientLabelSaveDTO.setDataDt(i.getDataDt());

                patientLableHistorySave.add(patientLabelSaveDTO);
            }
        });
        
        if (CollectionUtil.isNotEmpty(patientLabelRemoveIds)) {
            patientLabelBizService.removeBatchLabelByIds(patientLabelRemoveIds);
        }
        if (CollectionUtil.isNotEmpty(patientLabelSaveDTOs)) {
            patientLabelBizService.saveLabelBatch(patientLabelSaveDTOs);
        }
        if (CollectionUtil.isNotEmpty(patientLableHistorySave)) {
            patientLabelBizService.saveHistoryLabelBatch(patientLableHistorySave);
        }
    }

    @Override
    public void createLabelAndBindPatientBatch(List<LabelAndPatientAddDTO> params) {
        // 批量创建标签
        // 标签去重
        // Map<String, LabelSaveDTO> labelMap = new HashMap<>();
        Set<String> labelSet = new HashSet<>();
        List<LabelSaveDTO> labelSaveDTOs = new ArrayList<>();
        params.forEach(i -> {
            if (!labelSet.contains(i.getLabelId())) {
                LabelSaveDTO labelSaveDTO = new LabelSaveDTO();
                labelSaveDTO.setId(i.getLabelId());
                labelSaveDTO.setName(i.getLabelName());
                labelSaveDTO.setBizType(i.getBizType());
                labelSaveDTO.setCategoryId(i.getLabelCategoryId());
                labelSaveDTO.setDiseaseIds(i.getLabelDiseaseIds());
                labelSaveDTO.setProjectCode(i.getProjectCode());
                labelSaveDTO.setRemark(i.getRemark());
                labelSaveDTO.setOrders(i.getOrders());
                labelSaveDTO.setStatDimensionType(i.getStatDimensionType());

                labelSet.add(i.getLabelId());
                labelSaveDTOs.add(labelSaveDTO);
            }
        });
        List<LabelBO> labelBOs = labelClientService.addBatch(labelSaveDTOs);

        Map<String, LabelBO> labelBOMap = labelBOs.stream().collect(Collectors.toMap(i -> i.getId(), i -> i));

        List<PatientLabelSaveDTO> patientLabelSaveDTOs = new ArrayList<>();
        params.forEach(i -> {
            LabelBO labelBO = labelBOMap.get(i.getLabelId());
            if (null != labelBO) {
                PatientLabelSaveDTO patientLabelSaveDTO = new PatientLabelSaveDTO();
                patientLabelSaveDTO.setLabelId(labelBO.getId());
                patientLabelSaveDTO.setPatientId(i.getPatientId());
                patientLabelSaveDTO.setValidityNum(i.getValidityNum());
                patientLabelSaveDTO.setOpUser(i.getOpUser());

                if (null != i.getDataDt()) {
                    patientLabelSaveDTO.setDataDt(i.getDataDt());
                } else {
                    patientLabelSaveDTO.setDataDt(DateUtil.date());
                }

                patientLabelSaveDTO.setExtend1(i.getExtend1());
                patientLabelSaveDTO.setExtend2(i.getExtend2());
                patientLabelSaveDTO.setExtend3(i.getExtend3());
                patientLabelSaveDTO.setExtend4(i.getExtend4());

                patientLabelSaveDTOs.add(patientLabelSaveDTO);
            }
        });
        // 标签绑定患者
        patientLabelBizService.saveLabelBatch(patientLabelSaveDTOs);
    }

    @Override
    public void patientRemoveLabelBatch(List<PatientRemoveLabelDTO> params) {
        // 标签解绑患者
        List<PatientLabelDelDTO> patientLabelSaveDTOs = new ArrayList<>();
        params.forEach(i -> {
            PatientLabelDelDTO patientLabelDelDTO = new PatientLabelDelDTO();
            patientLabelDelDTO.setPatientId(i.getPatientId());
            patientLabelDelDTO.setLabelId(i.getLabelId());
            patientLabelDelDTO.setOpUser(i.getOpUser());

            patientLabelSaveDTOs.add(patientLabelDelDTO);
        });
        patientLabelBizService.delLabelBatch(patientLabelSaveDTOs);
    }

    @Override
    public void directDelNowPatientLabelBatch(LabelDirectRemoveDTO params) {
        PatientLabelDirectRemoveDTO patientLabelRemoveDTO = new PatientLabelDirectRemoveDTO();
        patientLabelRemoveDTO.setPatientId(params.getPatientId());
        patientLabelRemoveDTO.setPatientIds(params.getPatientIds());
        patientLabelRemoveDTO.setLabelIds(params.getLabelIds());
        patientLabelRemoveDTO.setLabelIds(params.getLabelIds());
        patientLabelRemoveDTO.setSourceType(params.getSourceType());
        patientLabelRemoveDTO.setSourceRelId(params.getSourceRelId());
        patientLabelRemoveDTO.setSourceRelTwoId(params.getSourceRelTwoId());
        patientLabelRemoveDTO.setStartDate(params.getStartDate());
        patientLabelRemoveDTO.setEndDate(params.getEndDate());

        patientLabelBizService.directDelNowPatientLabelBatch(patientLabelRemoveDTO);
    }

    @Override
    public void directDelHistoryPatientLabelBatch(LabelDirectRemoveDTO params) {
        PatientLabelDirectRemoveDTO patientLabelRemoveDTO = new PatientLabelDirectRemoveDTO();
        patientLabelRemoveDTO.setPatientId(params.getPatientId());
        patientLabelRemoveDTO.setPatientIds(params.getPatientIds());
        patientLabelRemoveDTO.setLabelIds(params.getLabelIds());
        patientLabelRemoveDTO.setLabelIds(params.getLabelIds());
        patientLabelRemoveDTO.setSourceType(params.getSourceType());
        patientLabelRemoveDTO.setSourceRelId(params.getSourceRelId());
        patientLabelRemoveDTO.setSourceRelTwoId(params.getSourceRelTwoId());
        patientLabelRemoveDTO.setStartDate(params.getStartDate());
        patientLabelRemoveDTO.setEndDate(params.getEndDate());

        patientLabelBizService.directDelHistoryPatientLabelBatch(patientLabelRemoveDTO);
    }

}
