package com.xxh.cloud.healthy.serve.label.framework.modules.client.service;

import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelQueryClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelHistoryBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelQueryDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelHistoryVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.helper.PatientLabelHelper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelQueryService;
import com.xxh.cloud.healthy.serve.label.framework.modules.cache.service.PatientLabelCacheService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * zs20230918
 */
@Service
public class PatientLabelQueryClientServiceImpl implements PatientLabelQueryClientService {

    @Autowired
    private PatientLabelCacheService patientLabelCacheService;
    @Autowired
    private PatientLabelQueryService patientLabelQueryService;

    @Override
    public List<PatientLabelBO> getListByPatientId(String patientId) {
        List<PatientLabelVO> list = patientLabelCacheService.listByPatientId(patientId);
        return list.stream().map(PatientLabelHelper::getPatientLabelBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, List<PatientLabelBO>> getListByPatientIds(List<String> patientIds) {
        Map<String, List<PatientLabelVO>> patientLabelMap = patientLabelCacheService.mapByPatientIds(patientIds);
        Map<String, List<PatientLabelBO>> patientLabelBOMap = new HashMap<>();
        patientLabelMap.forEach((i, j) -> patientLabelBOMap.put(i, j.stream().map(PatientLabelHelper::getPatientLabelBO).collect(Collectors.toList())));
        return patientLabelBOMap;
    }

    @Override
    public List<PatientLabelBO> queryList(LabelQueryDTO params) {
        PatientLabelQueryParam patientLabelQueryParam = new PatientLabelQueryParam();
        patientLabelQueryParam.setPatientId(params.getPatientId());
        patientLabelQueryParam.setPatientIds(params.getPatientIds());
        patientLabelQueryParam.setDiseaseIds(params.getDiseaseIds());
        patientLabelQueryParam.setLabelIds(params.getLabelIds());
        patientLabelQueryParam.setLabelId(params.getLabelId());
        List<PatientLabelVO> list = patientLabelQueryService.list(patientLabelQueryParam);
        return list.stream().map(PatientLabelHelper::getPatientLabelBO).collect(Collectors.toList());
    }


    @Override
    public List<PatientLabelHistoryBO> getHistoryListByPatientId(String patientId) {
        PatientLabelQueryParam queryParam = new PatientLabelQueryParam();
        queryParam.setPatientId(patientId);
        List<PatientLabelHistoryVO> list = patientLabelQueryService.historyList(queryParam);
        return list.stream().map(PatientLabelHelper::getPatientLabelHistoryBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, List<PatientLabelHistoryBO>> getHistoryListByPatientIds(List<String> patientIds) {
        PatientLabelQueryParam queryParam = new PatientLabelQueryParam();
        queryParam.setPatientIds(patientIds);
        List<PatientLabelHistoryVO> patientLabelHistoryList = patientLabelQueryService.historyList(queryParam);
        Map<String, List<PatientLabelHistoryVO>> patientLabelHistoryMap = new HashMap<>();
        patientLabelHistoryList.forEach(i -> {
            List<PatientLabelHistoryVO> list = patientLabelHistoryMap.get(i.getPatientId());
            if (null == list) {
                list = new ArrayList<>();
                patientLabelHistoryMap.put(i.getPatientId(), list);
            }
            list.add(i);
        });

        Map<String, List<PatientLabelHistoryBO>> patientLabelHistoryBOMap = new HashMap<>();
        patientLabelHistoryMap.forEach((i, j) -> patientLabelHistoryBOMap.put(i, j.stream().map(PatientLabelHelper::getPatientLabelHistoryBO).collect(Collectors.toList())));
        return patientLabelHistoryBOMap;
    }

    @Override
    public List<PatientLabelHistoryBO> getHistoryListByLabelIds(List<String> labelIds) {
        PatientLabelQueryParam queryParam = new PatientLabelQueryParam();
        queryParam.setLabelIds(labelIds);
        List<PatientLabelHistoryVO> list = patientLabelQueryService.historyList(queryParam);
        return list.stream().map(PatientLabelHelper::getPatientLabelHistoryBO).collect(Collectors.toList());
    }

    @Override
    public List<PatientLabelHistoryBO> queryHistory(LabelQueryDTO params) {
        PatientLabelQueryParam queryParam = new PatientLabelQueryParam();
        queryParam.setDiseaseIds(params.getDiseaseIds());
        queryParam.setPatientId(params.getPatientId());
        queryParam.setPatientIds(params.getPatientIds());
        queryParam.setLabelId(params.getLabelId());
        queryParam.setLabelIds(params.getLabelIds());
        queryParam.setOpType(params.getOpType());
        queryParam.setStartDate(params.getStartDate());
        queryParam.setEndDate(params.getEndDate());
        List<PatientLabelHistoryVO> list = patientLabelQueryService.historyList(queryParam);
        return list.stream().map(PatientLabelHelper::getPatientLabelHistoryBO).collect(Collectors.toList());
    }



}
