package com.xxh.cloud.healthy.serve.label.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.healthy.data.client.api.TaskDeptLabelClientService;
import com.xxh.cloud.healthy.data.client.api.bean.dto.DeptLabelAddDTO;
import com.xxh.cloud.healthy.data.client.api.constant.DeptPatientTypeEnum;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.mapper.PatientLabelExpireMapper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelQueryService;
import com.xxh.cloud.healthy.serve.label.framework.modules.task.service.PatientLabelExpireBizService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.dto.DeptQueryDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Slf4j
@Service
public class PatientLabelExpireBizServiceImpl implements PatientLabelExpireBizService {

    @Autowired
    private PatientLabelQueryService patientLabelQueryService;
    @Autowired
    private PatientLabelBizService patientLabelBizService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private TaskDeptLabelClientService taskDeptLabelClientService;
    @Resource
    private PatientLabelExpireMapper patientLabelExpireMapper;

    @Override
    public void deptLabelRespect() {
        log.info("1.查询部门列表>>>>>>");
        List<DeptBO> deptList = deptClientService.getDeptList(new DeptQueryDTO());
        if (CollUtil.isEmpty(deptList)) {
            log.info("2.无数据>>>>>>");
            return;
        }
        log.info("3.清理原有数据>>>>>>");
        patientLabelExpireMapper.deleteDeptLabelRelationAll();
        for (DeptBO dept : deptList) {
            log.info("4.部门{}逐条处理>>>>>>", dept.getId());
            if (CollUtil.isEmpty(dept.getDiseaseIds())) {
                continue;
            }
            QueryWrapper<String> queryWrapper = new QueryWrapper<>();
            dept.getDiseaseIds().forEach(di->{
                queryWrapper.or().like("disease_ids", "," + di + ",");
            });
            List<String> labelIds = patientLabelExpireMapper.queryStandardLabel(queryWrapper);
            if (CollUtil.isEmpty(labelIds)) {
                continue;
            }
            List<DeptLabelAddDTO> addList = new ArrayList<>();
            labelIds.forEach(item-> addList.add(new DeptLabelAddDTO(dept.getId(), item)));
            taskDeptLabelClientService.addBatch(addList);
        }

        log.info("5.处理结束>>>>>>");
    }

    @Override
    public void patientLabelExpire() {
        // 扫描过期的标签
        log.info("1.扫描过期的标签>>>>>>");
        PatientLabelQueryParam patientLabelQueryParam = new PatientLabelQueryParam();
        patientLabelQueryParam.setIsExpire(true);
        List<PatientLabelVO> patientLabelList = patientLabelQueryService.list(patientLabelQueryParam);
        log.info("1.扫描过期的标签<<<<<< {}", JSON.toJSONString(patientLabelList));

        if (CollectionUtil.isEmpty(patientLabelList)) {
            return;
        }

        Set<String> ids = new HashSet<>();
        Set<String> patientIds = new HashSet<>();

        patientLabelList.forEach(i -> {
            ids.add(i.getId());
            patientIds.add(i.getPatientId());
        });
        
        // 删除过期标签并记录历史
        log.info("2.把过期的标签记录到历史表中>>>>>>");
        patientLabelBizService.removeBatchLabelByIds(new ArrayList<>(ids));

        for (String patientId : patientIds) {
            log.info("3.查询患者ID_{}的标签关联部门>>>>>>", patientId);
            List<String> dept = patientLabelExpireMapper.queryDeptByPatientLabel(new QueryWrapper<String>().eq("pl.patient_id", patientId));
            log.info("4.删除没有关联的部门患者关系,type=label>>>>>>");
            patientLabelExpireMapper.deleteDeptPatientRelationAll(new QueryWrapper<String>()
                    .eq("patient_id", patientId)
                    .eq("type", DeptPatientTypeEnum.Label.getCode())
                    .notIn(CollUtil.isNotEmpty(dept), "dept_id", dept)
            );
        }
        log.info("5.处理结束>>>>>>");

    }

    @Override
    public void deptPatientRespectByLabel() {
        // 拉取所有部门标签数据

        // 拉取所有打过标签的患者

        // 

        
    }

}
