package com.xxh.cloud.healthy.serve.label.framework.modules.task.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.entity.PatientLabelInfo;
import com.xxh.cloud.healthy.serve.label.framework.modules.base.service.PatientLabelInfoService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.LabelPatientDataAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientRecordDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelSimpleVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientRecordVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.mapper.PatientLabelMapper;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.DwdPatientRecordBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.LabelPatientDataBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.task.service.PatientLabelTaskBizService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorRemindClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.dto.DoctorRemindClientAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Slf4j
@Service
public class PatientLabelTaskBizServiceImpl implements PatientLabelTaskBizService {

    @Autowired
    private PatientLabelMapper patientLabelMapper;
    @Autowired
    private DwdPatientRecordBizService dwdPatientRecordBizService;
    @Autowired
    private LabelPatientDataBizService labelPatientDataBizService;
    @Autowired
    private PatientLabelInfoService patientLabelInfoService;
    @Autowired
    private DoctorRemindClientService doctorRemindClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public void labelPatientCountTaskExe() {
        Date now = DateUtil.date();

        List<LabelPatientDataAddBatchDTO> list = patientLabelMapper.labelPatientCount();
        list.forEach(i -> {
            i.setDataDt(now);
        });

        if (CollectionUtil.isNotEmpty(list)) {
            // 保存到数据库
            labelPatientDataBizService.saveBatch(list);
        }
    }

    @Override
    public void remindPatientLabel(int hours) {
        Date start = DateUtil.offsetHour(new Date(), -hours);

        List<PatientLabelSimpleVO> list = patientLabelMapper.selectPatientLabel(new QueryWrapper<Void>()
                .ge("p.create_date",start)
                .eq("l.label_remind_status", YN.YES.getCode())
                .isNull("p.extend4")
                .isNotNull("l.label_manager")
        );
        if (CollUtil.isEmpty(list)) {
            return;
        }

        List<DoctorRemindClientAddDTO> addRemindList = new ArrayList<>();
        Set<String> ids = new HashSet<>();
        for (PatientLabelSimpleVO vo : list) {
            // 查询关联的诊断信息
            PatientRecordDTO dto = new PatientRecordDTO();
            dto.setPatientId(vo.getPatientId());
            dto.setStartDate(DateUtil.offsetDay(DateUtil.date(), -5)); // 查询5天内的就诊，住院记录
            List<PatientRecordVO> patientRecords = dwdPatientRecordBizService.selectPatientReport(dto);

            DoctorRemindClientAddDTO remindDto = new DoctorRemindClientAddDTO();
            remindDto.setDoctorId(vo.getLabelManager());
            remindDto.setBizId(vo.getId());
            remindDto.setBizOneType(PushTaskConstant.OpType.Found.getCode()); // 发现患者
            remindDto.setBizTwoType(PushTaskConstant.OpType.Found.getCode()); // 发现患者
            remindDto.setTitle("发现目标患者");
            Map<String, String> contentMap = new HashMap<>();
            contentMap.put("patientId", vo.getPatientId());
            contentMap.put("patientName", vo.getPatientName());
            contentMap.put("labelName", vo.getLabelName());
            contentMap.put("labelRemark", vo.getLabelRemark());
            if (CollUtil.isNotEmpty(patientRecords)) {
                PatientRecordVO report = patientRecords.get(0);
                contentMap.put("departmentName", report.getDepartmentName());
                AdminUserBO doctor = adminUserClientService.getUserByUserName(report.getDoctorCode());
                if (null != doctor) {
                    contentMap.put("doctorName", doctor.getRealName());
                    contentMap.put("doctorPhone", StrUtil.isNotEmpty(doctor.getMobile()) ? doctor.getMobile() : "-");
                } else {
                    contentMap.put("doctorName", "-");
                    contentMap.put("doctorPhone", "-");
                }
            } else {
                contentMap.put("doctorName", "-");
                contentMap.put("departmentName", "-");
                contentMap.put("doctorPhone", "-");
            }

            remindDto.setContent(JSON.toJSONString(contentMap));
            remindDto.setRemark("发现目标患者");
            remindDto.setPushDate(DateUtil.date());
            remindDto.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());
//            addRemindList.add(remindDto);
            ids.add(vo.getId());
            // 推送消息 病种负责人, 运营助理,部门负责人
            if (StrUtil.isNotEmpty(vo.getLabelManager())) {
                DoctorRemindClientAddDTO remindDoctor = new DoctorRemindClientAddDTO();
                BeanUtil.copyProperties(remindDto, remindDoctor);
                remindDoctor.setDoctorId(vo.getLabelManager());
                addRemindList.add(remindDoctor);
            }
            if (StrUtil.isNotEmpty(vo.getLabelAssistant())) {
                DoctorRemindClientAddDTO remindAssistant = new DoctorRemindClientAddDTO();
                BeanUtil.copyProperties(remindDto, remindAssistant);
                remindAssistant.setDoctorId(vo.getLabelAssistant());
                addRemindList.add(remindAssistant);
            }
            if (StrUtil.isNotEmpty(vo.getLabelDeptManager())) {
                DoctorRemindClientAddDTO deptManage = new DoctorRemindClientAddDTO();
                BeanUtil.copyProperties(remindDto, deptManage);
                deptManage.setDoctorId(vo.getLabelDeptManager());
                addRemindList.add(deptManage);
            }
        }
        doctorRemindClientService.addbatch(addRemindList);

        // 更新字段
        patientLabelInfoService.update(new UpdateWrapper<PatientLabelInfo>()
                .set("extend4", YN.YES.getCode())
                .in("id", ids));

    }

}
