package com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.impl;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.data.client.api.constant.PatientDataOperatedTypeEnum;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeHistoryQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServiceDataDetailQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServiceDataQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientRemindQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServiceDataDetailVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServiceDataVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientRemindVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.mapper.PatientPushServeMapper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServiceManageService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientPushServiceManageServiceImpl implements PatientPushServiceManageService {

    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private PatientPushServeMapper patientPushServeMapper;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;

    @Override
    public PageData<PatientPushServiceDataVO> pushServicePage(PatientPushServiceDataQueryDTO params) {

        // 阅读状态
        Map<String, Map<String, Object>> patientPushServiceBrowseMap = new HashMap<>();
        if (YN.YES.getCode().equals(params.getIsBrowse())) {
            IPage<Map<String, Object>> browsPage = patientPushServeMapper.patientPushServiceBrowseMapPage(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<Void>()
                .eq("a.source_two_type", "patient_push_serve")
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
                .eq(StrUtil.isNotBlank(params.getPatientPushServeId()), "a.source_two_id", params.getPatientPushServeId())
                .in(CollUtil.isNotEmpty(params.getPatientPushServeIds()), "a.source_two_id", params.getPatientPushServeIds())
            );
            if (null == browsPage || CollUtil.isEmpty(browsPage.getRecords())) {
                return new PageData<>(null, 0);
            }
            List<String> browsPatientPushServiceIds = new ArrayList<>();
            browsPage.getRecords().forEach(i -> {
                Object patientPushServiceId = i.get("sourceTwoId");
                if (null != patientPushServiceId) {
                    browsPatientPushServiceIds.add(String.valueOf(patientPushServiceId));
                    patientPushServiceBrowseMap.put(String.valueOf(patientPushServiceId), i);
                }
            });
            params.setPatientPushServeIds(browsPatientPushServiceIds);
        }

        // 推送状态
        Map<String, List<PatientRemindVO>> remindMap = new HashMap<>();
        if (YN.YES.getCode().equals(params.getIsPush())) {
            PatientRemindQueryDTO remindQuery = new PatientRemindQueryDTO();
            remindQuery.setPage(params.getPage());
            remindQuery.setRows(params.getRows());
            remindQuery.setBizId(params.getPatientPushServeId());
            remindQuery.setBizIds(params.getPatientPushServeIds());
            remindQuery.setPushStatus(PatientPushServeConstant.RemindPushStatus.Success.getCode());
            PageData<PatientRemindVO> remindPage = patientPushServeQueryBizService.pagePatientRemind(remindQuery);
            if (null == remindPage || CollUtil.isEmpty(remindPage.getList())) {
                return new PageData<>(null, 0);
            }

            List<PatientRemindVO> remindList = remindPage.getList();

            Set<String> patientPushServeIds = new HashSet<>();
            remindList.forEach(i -> {
                List<PatientRemindVO> l = remindMap.get(i.getBizId());
                if (null == l) {
                    l = new ArrayList<>();
                    remindMap.put(i.getBizId(), l);
                }
                l.add(i);

                patientPushServeIds.add(i.getBizId());
            });

            if (CollUtil.isNotEmpty(params.getPatientPushServeIds())) {
                patientPushServeIds.addAll(params.getPatientPushServeIds());
            }
            params.setPatientPushServeIds(new ArrayList<>(patientPushServeIds));
        }

        // 查询患者历史推送服务
        PatientPushServeHistoryQueryParam pushServeHistoryQueryParam = new PatientPushServeHistoryQueryParam();
        pushServeHistoryQueryParam.setPage(params.getPage());
        pushServeHistoryQueryParam.setRows(params.getRows());
        pushServeHistoryQueryParam.setPushServeId(params.getPushServeId());
        pushServeHistoryQueryParam.setPushServeIds(params.getPushServeIds());
        pushServeHistoryQueryParam.setPatientPushServeId(params.getPatientPushServeId());
        pushServeHistoryQueryParam.setPatientPushServeIds(params.getPatientPushServeIds());
        pushServeHistoryQueryParam.setPushServeName(params.getPushServiceName());
        pushServeHistoryQueryParam.setPatientName(params.getPatientName());
        pushServeHistoryQueryParam.setPatientId(params.getPatientId());
        pushServeHistoryQueryParam.setStartDate(params.getStartDate());
        pushServeHistoryQueryParam.setEndDate(params.getEndDate());
        pushServeHistoryQueryParam.setOpType(OpTypeEnum.Create.getCode());
        PageData<PatientPushServeHistoryVO> hisPushServicePage = patientPushServeQueryBizService.pushServeHistoryPage(pushServeHistoryQueryParam);
        if (null == hisPushServicePage || CollUtil.isEmpty(hisPushServicePage.getList())) {
            return new PageData<>(null, 0);
        }

        Set<String> patientPushServiceIds = new HashSet<>();
        Set<String> patientIds = new HashSet<>();
        hisPushServicePage.getList().forEach(i -> {
            patientPushServiceIds.add(i.getPatientPushServeId());
            patientIds.add(i.getPatientId());
        });

        // 患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO taskPatientQueryDTO = new PatientQueryDTO();
            taskPatientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(taskPatientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        // 查询提醒数据
        // Map<String, List<PatientRemindVO>> remindMap = new HashMap<>();
        if (CollUtil.isEmpty(remindMap)) {
            PatientRemindQueryDTO remindQuery = new PatientRemindQueryDTO();
            remindQuery.setBizIds(new ArrayList<>(patientPushServiceIds));
            List<PatientRemindVO> remindList = patientPushServeQueryBizService.listPatientRemind(remindQuery);
            if (CollUtil.isNotEmpty(remindList)) {
                remindList.forEach(i -> {
                    List<PatientRemindVO> l = remindMap.get(i.getBizId());
                    if (null == l) {
                        l = new ArrayList<>();
                        remindMap.put(i.getBizId(), l);
                    }
                    l.add(i);
                });
            }
        }

        // 行为
        // 阅读
        if (CollUtil.isEmpty(patientPushServiceBrowseMap)) {
            List<Map<String, Object>> patientPushServiceBrowseList = patientPushServeMapper.patientPushServiceBrowseMap(new QueryWrapper<Void>()
                .eq("a.source_two_type", "patient_push_serve")
                .in("a.source_two_id", patientPushServiceIds)
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
            );
            if (CollUtil.isNotEmpty(patientPushServiceBrowseList)) {
                patientPushServiceBrowseList.forEach(i -> {
                    Object patientPushServiceId = i.get("sourceTwoId");
                    if (null != patientPushServiceId) {
                        patientPushServiceBrowseMap.put(String.valueOf(i.get("sourceTwoId")), i);
                    }
                });
            }
        }

        List<PatientPushServiceDataVO> list = new ArrayList<>();
        hisPushServicePage.getList().forEach(i -> {
            PatientPushServiceDataVO vo = new PatientPushServiceDataVO();
            vo.setPushSuccessCount(0);
            vo.setPushFailCount(0);
            vo.setPatientId(i.getPatientId());
            vo.setPushServiceName(i.getPushServeName());
            vo.setPushServiceTitle(i.getPushServeTitle());
            vo.setPushServiceId(i.getPushServeId());
            vo.setPatientPushServiceId(i.getPatientPushServeId());
            vo.setPushDate(i.getOpDate());
            // if (CollUtil.isNotEmpty(i.getSendDateList())) {
            //     vo.setPushDate(StrJoinSplitUtils.join((i.getSendDateList())));
            // }

            PushServeConfigBO config = i.getPushServeConfig();
            if (null != config) {
                vo.setPushTarget(config.getPushTarget());
            }

            PatientClientBO patient = patientMap.get(i.getPatientId());
            if (null != patient) {
                vo.setPatientName(patient.getPatientName());
                vo.setPatientBirthday(patient.getBirthday());
                vo.setPatientAge(patient.getAge());
                vo.setPatientGenderStandard(patient.getPatientGenderStandard());
                vo.setPatientPhone(patient.getPhone());
            }

            List<PatientRemindVO> rl = remindMap.get(i.getPatientPushServeId());
            if (CollUtil.isEmpty(rl)) {
                vo.setPushServiceStatus("none");
                vo.setPushServiceErrorMsg("患者未绑定家属信息");
            } else {
                int pushSuccessCount = 0;
                int pushFailCount = 0;
                Boolean boo = true;
                PatientRemindVO first = null;
                PatientRemindVO last = null;
                for (PatientRemindVO r : rl) {
                    if (PatientPushServeConstant.RemindPushStatus.Success.getCode().equals(r.getPushStatus())) {
                        vo.setPushServiceStatus("success");
                        pushSuccessCount++;
                        boo = false;
                    } else if (boo && PatientPushServeConstant.RemindPushStatus.Todo.getCode().equals(r.getPushStatus())) {
                        vo.setPushServiceStatus("todo");
                        boo = false;
                    } else {
                        pushFailCount++;
                    }

                    if (null == first || first.getPushDate().after(r.getPushDate())) {
                        first = r;
                    }
                    if (null == last || last.getPushDate().before(r.getPushDate())) {
                        last = r;
                    }
                }
                vo.setPushSuccessCount(pushSuccessCount);
                vo.setPushFailCount(pushFailCount);
                vo.setFirstSendDate(first.getPushDate());
                vo.setLastSendDate(last.getPushDate());
                if (boo) {
                    vo.setPushServiceStatus("fail");
                    vo.setPushServiceErrorCode(last.getErrorCode());
                    vo.setPushServiceErrorMsg(last.getErrorMsg());
                }
            }

            vo.setRead(YN.NO.getCode());
            Map<String, Object> patientPushServiceBrowse = patientPushServiceBrowseMap.get(i.getPatientPushServeId());
            if (null != patientPushServiceBrowse) {
                vo.setRead(YN.YES.getCode());

                String operatedDate = null;
                Object date = patientPushServiceBrowse.get("operatedDate");
                if (null != date) {
                    if (date instanceof LocalDateTime) {
                        LocalDateTime ldt = (LocalDateTime) date;
                        operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                    } else if (date instanceof LocalDate) {
                        LocalDate ldt = (LocalDate) date;
                        operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                    } else {
                        operatedDate = String.valueOf(date);
                    }
                }
                vo.setReadDate(operatedDate);
            }

            vo.setPushServeConfig(i.getPushServeConfig());

            list.add(vo);
        });

        return new PageData<>(list, hisPushServicePage.getTotal(), hisPushServicePage.getSize(), hisPushServicePage.getPages());
    }

    @Override
    public List<PatientPushServiceDataDetailVO> pushServiceDetailList(PatientPushServiceDataDetailQueryDTO params) {
        
        // 查询患者历史推送服务
        PatientPushServeHistoryQueryParam pushServeHistoryQueryParam = new PatientPushServeHistoryQueryParam();
        pushServeHistoryQueryParam.setPatientPushServeId(params.getPatientPushServiceId());
        pushServeHistoryQueryParam.setOpType(OpTypeEnum.Create.getCode());
        PatientPushServeHistoryVO hisPushService = patientPushServeQueryBizService.pushServeHistoryDetail(pushServeHistoryQueryParam);
        if (null == hisPushService) {
            return new ArrayList<>(0);
        }

        // 提醒
        PatientRemindQueryDTO remindQuery = new PatientRemindQueryDTO();
        remindQuery.setBizId(hisPushService.getPatientPushServeId());
        List<PatientRemindVO> remindList = patientPushServeQueryBizService.listPatientRemind(remindQuery);
        if (CollUtil.isEmpty(remindList)) {
            return new ArrayList<>(0);
        }

        List<String> userIds = new ArrayList<>();
        remindList.forEach(i -> userIds.add(i.getUserId()));
        MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
        memberFamilyListClientDTO.setPatHisNo(hisPushService.getPatientId());
        List<MemberFamilyBO> memberFamilyBOs = customerFamilyClientService.listByUser(memberFamilyListClientDTO);

        Map<String, MemberFamilyBO> memberFamilyMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(memberFamilyBOs)) {
            memberFamilyBOs.forEach(i -> memberFamilyMap.put(i.getUserId(), i));
        }

        // 行为
        // 阅读
        Map<String, Map<String, Object>> userBrowseMap = new HashMap<>();
        List<Map<String, Object>> userBrowseList = patientPushServeMapper.userBrowseMap(new QueryWrapper<Void>()
                .eq("a.source_two_type", "patient_push_serve")
                .eq("a.source_two_id", hisPushService.getPatientPushServeId())
                .in("user_id", userIds)
                .eq("a.operated_type", PatientDataOperatedTypeEnum.Browse.getCode())
        );
        if (CollUtil.isNotEmpty(userBrowseList)) {
            userBrowseList.forEach(i -> {
                Object userId = i.get("userId");
                if (null != userId) {
                    userBrowseMap.put(String.valueOf(i.get("userId")), i);
                }
            });
        }

        List<PatientPushServiceDataDetailVO> list = new ArrayList<>();
        remindList.forEach(i -> {
            PatientPushServiceDataDetailVO vo = new PatientPushServiceDataDetailVO();

            vo.setPushServiceName(hisPushService.getPushServeName());
            vo.setPushServiceTitle(hisPushService.getPushServeTitle());
            vo.setPatientPushServiceId(hisPushService.getPatientPushServeId());
            vo.setUserId(i.getUserId());
            vo.setPushServiceDate(i.getPushDate());
            vo.setPushServiceStatus(i.getPushStatus());
            vo.setPushServiceErrorCode(i.getErrorCode());
            vo.setPushServiceErrorMsg(i.getErrorMsg());

            MemberFamilyBO family = memberFamilyMap.get(i.getUserId());
            if (null != family) {
                vo.setFamilyRelation(family.getRelationName());
            }

            vo.setRead(YN.NO.getCode());
            Map<String, Object> userBrows = userBrowseMap.get(i.getUserId());
            if (CollUtil.isNotEmpty(userBrows)) {
                vo.setRead(YN.YES.getCode());

                String operatedDate = null;
                Object date = userBrows.get("operatedDate");
                if (null != date) {
                    if (date instanceof LocalDateTime) {
                        LocalDateTime ldt = (LocalDateTime) date;
                        operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                    } else if (date instanceof LocalDate) {
                        LocalDate ldt = (LocalDate) date;
                        operatedDate = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                    } else {
                        operatedDate = String.valueOf(date);
                    }
                }
                vo.setReadDate(operatedDate);
            }

            vo.setPushServeConfig(hisPushService.getPushServeConfig());

            list.add(vo);
        });

        return list;
    }
    
}
