package com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.entity.PatientReturnVisitInfo;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.service.PatientReturnVisitInfoService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientReturnVisitAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientReturnVisitQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientReturnVisitUpdateDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientReturnVisitVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.helper.PatientPushServeHelper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientReturnVisitBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientReturnVisitBizServiceImpl implements PatientReturnVisitBizService {

    @Autowired
    private PatientReturnVisitInfoService patientReturnVisitInfoService;

    @Override
    public void saveBatch(List<PatientReturnVisitAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        List<String> existQuery = new ArrayList<>();
        params.forEach(i -> existQuery.add(i.getSourceType() + '-' + i.getSourceId()));

        // 查询复诊是否存在, 存在的不操作
        Map<String, PatientReturnVisitInfo> existMap = new HashMap<>();
        List<PatientReturnVisitInfo> existList = patientReturnVisitInfoService.list(new QueryWrapper<PatientReturnVisitInfo>()
            .in("CONCAT(source_type,'-',source_id)", existQuery)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> {
                existMap.put(i.getSourceType() + '-' + i.getSourceId(), i);
            });
        }

        List<PatientReturnVisitInfo> list = new ArrayList<>();
        params.forEach(i -> {
            if (null != existMap.get(i.getSourceType() + '-' + i.getSourceId())) {
                return;
            }

            PatientReturnVisitInfo entity = new PatientReturnVisitInfo();
            entity.setId(PatientPushServeHelper.getPatientPushServeId());
            entity.setPatientId(i.getPatientId());
            entity.setVisitType(i.getVisitType());
            entity.setVisitId(i.getVisitId());
            entity.setSourceType(i.getSourceType());
            entity.setSourceId(i.getSourceId());
            entity.setSourceDeptId(i.getSourceDeptId());
            entity.setSourceTime(i.getSourceTime());
            if (CollUtil.isNotEmpty(i.getDeptIds())) {
                entity.setDeptIds(StrJoinSplitUtils.join(i.getDeptIds()));
            }
            entity.setDeptId(i.getDeptId());
            entity.setContent(i.getContent());
            entity.setVisitStartTime(i.getVisitStartTime());
            entity.setVisitEndTime(i.getVisitEndTime());
            entity.setRealTime(i.getRealTime());
            entity.setStatus(i.getStatus());
            entity.setOpType(i.getOpType());
            entity.setRowsStatus(YN.YES.getCode());

            list.add(entity);
        });

        patientReturnVisitInfoService.saveBatch(list);
    }

    @Override
    public List<PatientReturnVisitVO> list(PatientReturnVisitQueryDTO params) {
        
        List<PatientReturnVisitInfo> list = patientReturnVisitInfoService.list(new QueryWrapper<PatientReturnVisitInfo>()
            .eq(StrUtil.isNotBlank(params.getSourceType()), "source_type", params.getSourceType())
            .eq(StrUtil.isNotBlank(params.getSourceId()), "source_id", params.getSourceId())
            .in(CollUtil.isNotEmpty(params.getSourceIds()), "source_id", params.getSourceIds())
        );

        return list.stream().map(PatientPushServeHelper::getPatientReturnVisitVO).collect(Collectors.toList());
    }

    @Override
    public void updateBatch(List<PatientReturnVisitUpdateDTO> params) {
        List<PatientReturnVisitInfo> list = new ArrayList<>();
        params.forEach(i -> {
            PatientReturnVisitInfo entity = new PatientReturnVisitInfo();
            entity.setId(i.getId());
            entity.setVisitStartTime(i.getVisitStartTime());
            entity.setStatus(i.getStatus());

            list.add(entity);
        });

        patientReturnVisitInfoService.updateBatchById(list);
    }
    
}
