package com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.entity.PatientServeTest;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.service.PatientServeTestService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientServeTestQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientServeTestVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientServeTestBizService;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class PatientServeTestBizServiceImpl implements PatientServeTestBizService {

    @Autowired
    private PatientServeTestService patientServeTestService;
    @Autowired
    private PatientClientService patientClientService;

    @Override
    public PageData<PatientServeTestVO> query(PatientServeTestQueryParam params) {
        if (StrUtil.isNotBlank(params.getEndDate())) {
            String endDate = DateUtil.offsetDay(DateUtil.parse(params.getEndDate()), 1).toDateStr(); // 往后偏移一天
            params.setEndDate(endDate);
        }

        IPage<PatientServeTest> page = patientServeTestService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientServeTest>()
                .eq(StrUtil.isNotBlank(params.getPushServeId()), "push_serve_id", params.getPushServeId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .like(StrUtil.isNotBlank(params.getTitle()), "title", params.getTitle())
                .eq(StrUtil.isNotBlank(params.getPushType()), "push_type", params.getPushType())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "create_date", params.getStartDate())
                .lt(StrUtil.isNotBlank(params.getEndDate()), "create_date", params.getEndDate())
                .orderByDesc("create_date")
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        List<String> patientIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            patientIds.add(i.getPatientId());
        });

        // 患者信息
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(new ArrayList<>(patientIds));
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            if (CollUtil.isNotEmpty(patientList)) {
                patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
            }
        }

        PageData<PatientServeTestVO> data = new PageConverter<PatientServeTest, PatientServeTestVO>() {
            @Override
            public PatientServeTestVO converter(PatientServeTest entity) {
                PatientServeTestVO vo = new PatientServeTestVO();
                BeanUtil.copyProperties(entity, vo);

                PatientClientBO patientInfo = patientMap.get(entity.getPatientId());
                if (null != patientInfo) {
                    vo.setPatientName(patientInfo.getPatientName());
                    vo.setPatientAge(patientInfo.getAge());
                    vo.setPatientGenderStandard(patientInfo.getPatientGenderStandard());
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }
}
