package com.xxh.cloud.healthy.serve.pushServe.framework.modules.client.service;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.healthy.basic.common.bean.ServPushConfigModel;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.common.constant.PushServeTargetEnum;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.DoctorDwdPatientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientPushServeClientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientPushServeDateClientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientRemindBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientScheduleBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeDateQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeRemoveClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientRemindAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientRemindQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleCreateClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeDateQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeHistoryQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeRemoveDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientRemindAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientRemindQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleGetDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleUpdateDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeDateVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientRemindVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientScheduleVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.helper.PatientPushServeHelper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * zs20230920
 */
@Slf4j
@Service
public class PatientPushServeClientServiceImpl implements PatientPushServeClientService {

    @Autowired
    private PatientPushServeBizService patientPushServeBizService;
    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;

    @Override
    public PatientPushServeClientBO add(PatientPushServeAddClientDTO params) {

        // 判断患者是否已去世
        DoctorDwdPatientBO patientClientBO = dwdDataPatientClientService.getOnePatient(params.getPatientId());
        if (null == patientClientBO) {
            return null;
        }
        if (YN.NO.getCode().equals(patientClientBO.getDeathStatus())) {
            log.info("患者已去世 >>>>>> {}", JSON.toJSONString(patientClientBO));
            return null;
        }

        PatientPushServeAddDTO addDTO = new PatientPushServeAddDTO();
        addDTO.setPatientId(params.getPatientId());
        addDTO.setPushServeId(params.getPushServeId());
        addDTO.setBenchmarkDateValue(params.getBenchmarkDateValue());
        addDTO.setBenchmarkDateMap(params.getBenchmarkDateMap());
        addDTO.setOpUser(params.getOpUser());
        addDTO.setRelType(params.getRelType());
        addDTO.setRelOneId(params.getRelOneId());
        addDTO.setRelTwoId(params.getRelTwoId());

        PatientPushServeVO vo = patientPushServeBizService.add(addDTO);

        return PatientPushServeHelper.getPatientPushServeClientBO(vo);
    }

    @Override
    public void remove(PatientPushServeRemoveClientDTO params) {
        PatientPushServeRemoveDTO removeDTO = new PatientPushServeRemoveDTO();
        removeDTO.setId(params.getPatientId());
        removeDTO.setIds(params.getIds());
        removeDTO.setOpUser(params.getOpUser());

        patientPushServeBizService.remove(removeDTO);
    }

    @Override
    public PatientScheduleBO getPatientScheduleBO(String scheduleId) {
        PatientScheduleGetDTO patientScheduleGetDTO = new PatientScheduleGetDTO();
        patientScheduleGetDTO.setId(scheduleId);
        PatientScheduleVO vo = patientPushServeQueryBizService.patientScheduleDetail(patientScheduleGetDTO);
        return PatientPushServeHelper.getPatientScheduleBO(vo);
    }

    @Override
    public List<PatientScheduleBO> patientScheduleBOList(PatientScheduleQueryClientDTO params) {
        PatientScheduleQueryDTO patientScheduleQueryDTO = new PatientScheduleQueryDTO();
        patientScheduleQueryDTO.setId(params.getId());
        patientScheduleQueryDTO.setBizIdAndScheduleDate(params.getBizIdAndScheduleDate());
        patientScheduleQueryDTO.setStartDate(params.getStartDate());
        patientScheduleQueryDTO.setEndDate(params.getEndDate());
        patientScheduleQueryDTO.setQuerParams(params.getQuerParams());
        List<PatientScheduleVO> vo = patientPushServeQueryBizService.patientScheduleList(patientScheduleQueryDTO);

        return vo.stream().map(PatientPushServeHelper::getPatientScheduleBO).collect(Collectors.toList());
    }

    @Override
    public PatientScheduleBO updatePatientScheduleStatus(String scheduleId, String status) {
        PatientScheduleUpdateDTO patientScheduleUpdateDTO = new PatientScheduleUpdateDTO();
        patientScheduleUpdateDTO.setId(scheduleId);
        patientScheduleUpdateDTO.setStatus(status);
        PatientScheduleVO vo = patientPushServeBizService.updatePatientScheduleStatus(patientScheduleUpdateDTO);
        return PatientPushServeHelper.getPatientScheduleBO(vo);
    }

    @Override
    public PatientRemindBO getPatientRemindBO(String remindId) {
        PatientRemindQueryDTO patientRemindQueryDTO = new PatientRemindQueryDTO();
        patientRemindQueryDTO.setId(remindId);
        PatientRemindVO vo = patientPushServeQueryBizService.getPatientRemind(patientRemindQueryDTO);
        return PatientPushServeHelper.getPatientRemindBO(vo);
    }

    @Override
    public List<PatientRemindBO> patientRemindBOList(PatientRemindQueryClientDTO params) {
        PatientRemindQueryDTO patientRemindQueryDTO = new PatientRemindQueryDTO();
        patientRemindQueryDTO.setPatientId(params.getPatientId());
        patientRemindQueryDTO.setUserId(params.getUserId());
        patientRemindQueryDTO.setBizId(params.getBizId());
        patientRemindQueryDTO.setBizIds(params.getBizIds());
        patientRemindQueryDTO.setQuerParams(params.getQuerParams());
        patientRemindQueryDTO.setBizIdAndPushDate(params.getBizIdAndPushDate());
        List<PatientRemindVO> vo = patientPushServeQueryBizService.listPatientRemind(patientRemindQueryDTO);

        return vo.stream().map(PatientPushServeHelper::getPatientRemindBO).collect(Collectors.toList());
    }

    @Override
    public void createSchedule(PatientScheduleCreateClientDTO params) {
        log.info("用户生成日程>>>>>> {}", JSON.toJSONString(params));
        if (StrUtil.isBlank(params.getPatientId()) || StrUtil.isBlank(params.getUserId())) {
            log.info("用户生成日程参数错误<<<<<<");
            return;
        }

        // 查询患者未来要推送的服务
        // 查询当前及未来时间的服务
        PatientPushServeDateQueryParam patientPushServeDateQueryParam = new PatientPushServeDateQueryParam();
        patientPushServeDateQueryParam.setPatientId(params.getPatientId());
        patientPushServeDateQueryParam.setStartDate(DateUtil.date());
        List<PatientPushServeDateVO> pushServeDateList = patientPushServeQueryBizService.pushServeDateList(patientPushServeDateQueryParam);
        if (CollectionUtil.isEmpty(pushServeDateList)) {
            return;
        }
        Set<String> patientPushServeIdSet = new HashSet<>();
        pushServeDateList.forEach(i -> patientPushServeIdSet.add(i.getPatientPushServeId()));

        PatientPushServeHistoryQueryParam patientPushServeQueryParam = new PatientPushServeHistoryQueryParam();
        patientPushServeQueryParam.setPatientId(params.getPatientId());
        patientPushServeQueryParam.setPatientPushServeIds(new ArrayList<>(patientPushServeIdSet));
        patientPushServeQueryParam.setOpType(OpTypeEnum.Create.getCode());
        List<PatientPushServeHistoryVO> patientPushServeVOList = patientPushServeQueryBizService.pushServeHistoryList(patientPushServeQueryParam);
        if (CollectionUtil.isEmpty(patientPushServeVOList)) {
            log.info("未查询到患者推送服务信息<<<<<< ");
            return;
        }
        // 查询患者家属信息
        MemberFamilyBO family = this.getUserIdListByPatientId(params.getPatientId(), params.getUserId());
        if (null == family) {
            throw new BusinessException("未查询到患者对应的用户数据");
        }

        List<String> patientPushServeIds = new ArrayList<>();
        patientPushServeVOList.forEach(i -> patientPushServeIds.add(i.getPatientPushServeId()));

        // 查询要生成日程的用户是否已经生成过日程，生成过则跳过
        // 只查询当前和未来时间的日程，已过的日程不管
        PatientScheduleQueryDTO patientScheduleQueryDTO = new PatientScheduleQueryDTO();
        patientScheduleQueryDTO.setPatientId(params.getPatientId());
        patientScheduleQueryDTO.setUserId(params.getUserId());
        patientScheduleQueryDTO.setBizIds(patientPushServeIds);
        patientScheduleQueryDTO.setStartDate(DateUtil.now());
        List<PatientScheduleVO> scheduleVOs = patientPushServeQueryBizService.patientScheduleList(patientScheduleQueryDTO);

        // 理论上查出来的日程都为失效状态，但不信任
        // 两个map的key不会一样，因为同一个patientPushServeId下的日程不可能有失效状态也有正常状态，如有则排查日程状态更新逻辑
        Map<String, Boolean> existScheduleMap = new HashMap<>(); // 该用户已存在的日程
        Map<String, List<PatientScheduleVO>> beforeExistScheduleMap = new HashMap<>(); // 该用户已存在的之前授权的日程
        scheduleVOs.forEach(i -> {
            if (PatientPushServeConstant.ScheduleStatus.Invalid.getCode().equals(i.getStatus())) {
                List<PatientScheduleVO> schedules = beforeExistScheduleMap.get(i.getBizId());
                if (null == schedules) {
                    schedules = new ArrayList<>();
                    beforeExistScheduleMap.put(i.getBizId(), schedules);
                }
                schedules.add(i);
            } else {
                Boolean boo = existScheduleMap.get(i.getBizId());
                if (null == boo) {
                    existScheduleMap.put(i.getBizId(), true);
                }
            }
        });

        List<PatientScheduleAddDTO> patientScheduleAddDTOs = new ArrayList<>();
        List<PatientScheduleUpdateDTO> patientScheduleUpdateDTOs = new ArrayList<>();
        for(PatientPushServeHistoryVO patientPushServeVO : patientPushServeVOList) {
            // 用户已有正常日程则跳过
            Boolean boo = existScheduleMap.get(patientPushServeVO.getPatientPushServeId());
            if (null != boo && boo) {
                continue;
            }
            // 用户有之前授权的日程，则更新状态
            List<PatientScheduleVO> schedules = beforeExistScheduleMap.get(patientPushServeVO.getPatientPushServeId());
            if (CollectionUtil.isNotEmpty(schedules)) {
                PatientScheduleUpdateDTO patientScheduleUpdateDTO = new PatientScheduleUpdateDTO();
                patientScheduleUpdateDTO.setId(patientPushServeVO.getPatientPushServeId());
                patientScheduleUpdateDTO.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
                patientScheduleUpdateDTOs.add(patientScheduleUpdateDTO);
                continue;
            }

            if (null == patientPushServeVO.getPushServeConfig()) {
                log.info("未查询到患者推送服务配置信息<<<<<< ");
                continue;
            }
            // 处理要生成的日程的时间；时间从现在开始，过去的日程不生成
            List<String> sendDateList = patientPushServeVO.getSendDateList();
            if (CollectionUtil.isEmpty(sendDateList)) {
                log.info("没有要生成的日程1<<<<<<");
                return;
            }
            sendDateList.removeIf(i -> i.compareTo(DateUtil.now()) < 0);
            if (CollectionUtil.isEmpty(sendDateList)) {
                log.info("没有要生成的日程2<<<<<<");
                return;
            }
            // 生成日程信息
            this.saveSchedule(patientScheduleAddDTOs, patientPushServeVO, patientPushServeVO.getPushServeConfig(), sendDateList, family);
        }
        // 保存新日程
        if (CollectionUtil.isNotEmpty(patientScheduleAddDTOs)) {
            patientPushServeBizService.addPatientScheduleBatch(patientScheduleAddDTOs);
        }
        // 更新日程状态
        if (CollectionUtil.isNotEmpty(patientScheduleUpdateDTOs)) {
            patientPushServeBizService.updatePatientScheduleStatusBatch(patientScheduleUpdateDTOs);
        }
    }

    private void saveSchedule(List<PatientScheduleAddDTO> patientScheduleAddDTOs, PatientPushServeHistoryVO entity, PushServeConfigBO pushserveConfigBO, List<String> sendDateList,MemberFamilyBO family) {
        sendDateList.forEach(i -> {
            // 是否有权限
            if (YN.NO.getCode().equals(family.getIsAuth())) {
                return;
            }
            // 判断推送目标选择推送用户
            String pushTarget = pushserveConfigBO.getPushTarget();
            // 推送本人，如是家属则跳过
            if (PushServeTargetEnum.Self.getCode().equals(pushTarget) && !"1".equals(family.getRelationType())) {
                return;
            }
            // 推送家属，若是本人则跳过
            if (PushServeTargetEnum.Family.getCode().equals(pushTarget) && "1".equals(family.getRelationType())) {
                return;
            }

            PatientScheduleAddDTO patientScheduleAddDTO = new PatientScheduleAddDTO();
            patientScheduleAddDTO.setScheduleDate(i);
            patientScheduleAddDTO.setPatientId(entity.getPatientId());
            patientScheduleAddDTO.setUserId(family.getUserId());
            patientScheduleAddDTO.setBizId(entity.getPatientPushServeId());
            patientScheduleAddDTO.setBizOneType(ObjectTypeEnum.OT_OneType_PatientPushServe.getCode());
            // patientScheduleInfo.setBizTwoType(null);
            patientScheduleAddDTO.setTitle(entity.getPushServeTitle());
            patientScheduleAddDTO.setContent(pushserveConfigBO.getMediaId()); // mediaId为推送消息的id，这个id可以使消息跳转到具体的消息页
            patientScheduleAddDTO.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
            patientScheduleAddDTO.setScheduleType(pushserveConfigBO.getRemindWay());

            patientScheduleAddDTOs.add(patientScheduleAddDTO);
        });
    }

    private MemberFamilyBO getUserIdListByPatientId(String patientId, String userId) {
        MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
        memberFamilyListClientDTO.setUserId(userId);
        memberFamilyListClientDTO.setPatHisNo(patientId); // zs20231115 家属表中patientId来自海鹚和本平台的patientId不一致，一致的是patHisNos（就诊人门诊id）
        // memberFamilyListClientDTO.setMainPersonId(patientId); // zs20240318
        memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
        List<MemberFamilyBO> memberFamilyBOs = customerFamilyClientService.listByUser(memberFamilyListClientDTO);
        // 根据userId和patientId查询最多查询一条
        if (CollectionUtil.isEmpty(memberFamilyBOs)) {
            return null;
        }
        return memberFamilyBOs.get(0);
    }

    @Override
    public List<PatientPushServeClientBO> getPatientPushServeList(PatientPushServeQueryClientDTO params) {
        PatientPushServeHistoryQueryParam patientPushServeHistoryQueryParam = new PatientPushServeHistoryQueryParam();
        patientPushServeHistoryQueryParam.setPatientPushServeIds(params.getPatientPushServeIds());
        patientPushServeHistoryQueryParam.setPatientId(params.getPatientId());
        patientPushServeHistoryQueryParam.setPatientIds(params.getPatientIds());
        patientPushServeHistoryQueryParam.setStartDate(params.getStartDate());
        patientPushServeHistoryQueryParam.setEndDate(params.getEndDate());
        patientPushServeHistoryQueryParam.setOpType(params.getOpType());
        patientPushServeHistoryQueryParam.setPushServeId(params.getPushServeId());
        patientPushServeHistoryQueryParam.setPushServeIds(params.getPushServeIds());
        List<PatientPushServeHistoryVO> list = patientPushServeQueryBizService.pushServeHistoryList(patientPushServeHistoryQueryParam);
        return list.stream().map(PatientPushServeHelper::getPatientPushServeClientBO).collect(Collectors.toList());
    }

    @Override
    public List<PatientPushServeDateClientBO> pushServeDateList(PatientPushServeDateQueryClientDTO params) {
        
        PatientPushServeDateQueryParam dateQueryParam = new PatientPushServeDateQueryParam();
        dateQueryParam.setPatientId(params.getPatientId());
        dateQueryParam.setPatientIds(params.getPatientIds());
        dateQueryParam.setPatientPushServeId(params.getPatientPushServeId());
        dateQueryParam.setPatientPushServeIds(params.getPatientPushServeIds());
        dateQueryParam.setStartDate(params.getStartDate());
        dateQueryParam.setEndDate(params.getEndDate());
        List<PatientPushServeDateVO> patientPushServeDateVOs = patientPushServeQueryBizService.pushServeDateList(dateQueryParam);
        
        return patientPushServeDateVOs.stream().map(PatientPushServeHelper::getPatientPushServeDateClientBO).collect(Collectors.toList());
    }

    @Override
    public void addPatientScheduleBatch(List<PatientScheduleAddClientDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        List<PatientScheduleAddDTO> list = new ArrayList<>();
        params.forEach(i -> {
            PatientScheduleAddDTO patientScheduleAddDTO = new PatientScheduleAddDTO();
            patientScheduleAddDTO.setPatientId(i.getPatientId());
            patientScheduleAddDTO.setUserId(i.getUserId());
            patientScheduleAddDTO.setBizId(i.getBizId());
            patientScheduleAddDTO.setBizOneType(i.getBizOneType());
            patientScheduleAddDTO.setBizTwoType(i.getBizTwoType());
            patientScheduleAddDTO.setTitle(i.getTitle());
            patientScheduleAddDTO.setContent(i.getContent());
            patientScheduleAddDTO.setScheduleDate(i.getScheduleDate());
            patientScheduleAddDTO.setStatus(i.getStatus());
            patientScheduleAddDTO.setScheduleType(i.getScheduleType());

            list.add(patientScheduleAddDTO);
        });
        patientPushServeBizService.addPatientScheduleBatch(list);
    }

    @Override
    public List<PatientRemindBO> addPatientRemindBatch(List<PatientRemindAddClientDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return new ArrayList<>(0);
        }
        List<PatientRemindAddDTO> list = new ArrayList<>();
        params.forEach(i -> {
            PatientRemindAddDTO patientRemindAddDTO = new PatientRemindAddDTO();
            patientRemindAddDTO.setPatientId(i.getPatientId());
            patientRemindAddDTO.setUserId(i.getUserId());
            patientRemindAddDTO.setBizId(i.getBizId());
            patientRemindAddDTO.setBizOneType(i.getBizOneType());
            patientRemindAddDTO.setBizTwoType(i.getBizTwoType());
            patientRemindAddDTO.setTitle(i.getTitle());
            patientRemindAddDTO.setContent(i.getContent());
            patientRemindAddDTO.setPushDate(i.getPushDate());
            patientRemindAddDTO.setPushTempId(i.getPushTempId());
            patientRemindAddDTO.setPushChannel(i.getPushChannel());
            patientRemindAddDTO.setPushAccount(i.getPushAccount());
            patientRemindAddDTO.setPushStatus(i.getPushStatus());
            patientRemindAddDTO.setRemark(i.getRemark());

            if (null != i.getPushConfig()) {
                ServPushConfigModel configModel = new ServPushConfigModel();
                configModel.setTemplateId(i.getPushConfig().getTemplateId());
                configModel.setDate(i.getPushConfig().getDate());
                configModel.setDoctor(i.getPushConfig().getDoctor());
                configModel.setTitle(i.getPushConfig().getTitle());
                configModel.setContent(i.getPushConfig().getContent());
                configModel.setUrl(i.getPushConfig().getUrl());
                configModel.setRemark(i.getPushConfig().getRemark());

                patientRemindAddDTO.setPushConfig(configModel);
            }

            list.add(patientRemindAddDTO);
        });
        
        List<PatientRemindVO> remindList = patientPushServeBizService.addPatientRemindBatch(list);

        return remindList.stream().map(PatientPushServeHelper::getPatientRemindBO).collect(Collectors.toList());
    }

}
