package com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.schedule;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeDateQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeHistoryQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeDateVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.mapper.PatientPushServeMapper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.PatientPushServeDataService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class PatientPushServeDataTask {

    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    // @Autowired
    // private PatientScheduleInfoService patientScheduleInfoService;
    @Autowired
    private PatientPushServeDataService patientPushServeDataService;
    @Autowired
    private PatientPushServeMapper patientPushServeMapper;
    
    // 扫描患者推送服务历史表生成日程和提醒
    @XxlJob("createTestData")
    public void createTestData() throws Exception {
        log.info("扫描患者推送服务历史表生成日程和提醒开始 >>>>>> ");
        XxlJobHelper.log("扫描患者推送服务历史表生成日程和提醒开始 >>>>>> ");

        String limit = XxlJobHelper.getJobParam();
        if (StrUtil.isBlank(limit)) {
            limit = "100";
        }
        this.create(Integer.valueOf(limit));

        XxlJobHelper.log("扫描患者推送服务历史表生成日程和提醒结束 <<<<<< ");
        log.info("扫描患者推送服务历史表生成日程和提醒结束 <<<<<< ");
    }

    // @GetMapping("/createTestData")
    public void createTestDataController(@RequestParam(value = "limit", required = false) Integer limit) {
        if (null == limit) {
            limit = 100;
        }
        this.create(limit);
    }


    /**
     * 服务接受率：28%-60%
     * 行动率：1%-15%
     * 阅读时长：80-120秒
     */
    private void create(Integer limit) {

        // 查询未生成日程的历史服务信息
        List<String> patientPushServeHistoryIds = patientPushServeMapper.patientPushServeHistoryList(limit);
        if (CollectionUtil.isEmpty(patientPushServeHistoryIds)) {
            return;
        }

        // 查询所有历史服务信息
        PatientPushServeHistoryQueryParam historyQueryParam = new PatientPushServeHistoryQueryParam();
        // historyQueryParam.setOpType(OpTypeEnum.Create.getCode());
        historyQueryParam.setIds(patientPushServeHistoryIds);
        List<PatientPushServeHistoryVO> historyList = patientPushServeQueryBizService.pushServeHistoryList(historyQueryParam);
        if (CollectionUtil.isEmpty(historyList)) {
            return;
        }

        List<String> patientPushServeIds = new ArrayList<>();
        historyList.forEach(i -> patientPushServeIds.add(i.getPatientPushServeId()));

        // 查询历史服务对应的服务时间，并整理成key为服务id，value为服务时间列表的map形式
        PatientPushServeDateQueryParam dateQueryParam = new PatientPushServeDateQueryParam();
        dateQueryParam.setPatientPushServeIds(patientPushServeIds);
        List<PatientPushServeDateVO> patientPushServeDateList = patientPushServeQueryBizService.pushServeDateList(dateQueryParam);
        if (CollectionUtil.isEmpty(patientPushServeDateList)) {
            return;
        }

        Map<String, List<PatientPushServeDateVO>> patientPushServeDateMap = new HashMap<>();
        patientPushServeDateList.forEach(i -> {
            List<PatientPushServeDateVO> dateList = patientPushServeDateMap.get(i.getPatientPushServeId());
            if (null == dateList) {
                dateList = new ArrayList<>();
                patientPushServeDateMap.put(i.getPatientPushServeId(), dateList);
            }
            dateList.add(i);
        });

        // 循环服务获取服务时间并循环时间生成日程和提醒，根据生成的日程和提醒生成行为
        for (PatientPushServeHistoryVO history : historyList) {
            List<PatientPushServeDateVO> dateList = patientPushServeDateMap.get(history.getPatientPushServeId());
            if (CollectionUtil.isEmpty(dateList)) {
                continue;
            }

            log.info("1.根据患者历史服务生成数据 >>>>>>");
            // 生成日程、提醒、行为
            try {
                patientPushServeDataService.saveScheduleAndRemind(history, dateList);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }

    // private void createSchedule(PatientPushServeHistoryVO history, List<PatientPushServeDateVO> serveDateList) {
    //     // 查询要生成日程的用户是否已经生成过日程，生成过则跳过
    //     // 只查询当前和未来时间的日程，已过的日程不管

    //     // 查询是否存在日程，存在则跳过，不存在则生成
    //     List<PatientScheduleInfo> list = patientScheduleInfoService.list(new QueryWrapper<PatientScheduleInfo>()
    //             .eq("rows_status", YN.YES.getCode())
    //             .eq(StrUtil.isNotBlank(history.getPatientPushServeId()), "biz_id", history.getPatientPushServeId())
    //     );
    //     if (CollectionUtil.isNotEmpty(list)) {
    //         return;
    //     }

    //     // 生成日程信息
    //     patientPushServeDataService.saveScheduleAndRemind(history, serveDateList);
    // }

}
