package com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.schedule;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.PatientPushServeTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class PatientPushServeTask {

    @Autowired
    private PatientPushServeTaskService patientPushServeTaskService;
    
    // 保存推送服务数据
    @XxlJob("patientPushServeJobHandler")
    public void patientPushServeJobHandler() throws Exception {
        String cycle = XxlJobHelper.getJobParam();
        if (StrUtil.isBlank(cycle)) {
            cycle = "5";
        }

        log.info("保存推送服务数据开始 >>>>>> {}", cycle);
        XxlJobHelper.log("保存推送服务数据开始 >>>>>> " + cycle);

        patientPushServeTaskService.pushServeTask(Integer.parseInt(cycle));

        XxlJobHelper.log("保存推送服务数据结束 <<<<<< ");
        log.info("保存推送服务数据结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }


    
    // /sys/healthy/patient/pushService/remind/list
    // 扫描提醒表推送提醒
    @XxlJob("patientPushServeRemindJobHandler")
    public void patientPushServeRemindJobHandler() throws Exception {
        log.info("推送提醒开始 >>>>>> ");
        XxlJobHelper.log("推送提醒开始 >>>>>> ");

        // patientPushServeTaskService.pushServeTaskExecute();
        patientPushServeTaskService.remindTaskExecute();

        XxlJobHelper.log("推送提醒结束 <<<<<< ");
        log.info("推送提醒结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }

    // /sys/healthy/patient/pushService/remind/list
    // 扫描提醒表推送用药提醒
    @XxlJob("patientRecipeMedicineRemindJobHandler")
    public void patientRecipeMedicineRemindJobHandler() throws Exception {
        log.info("推送用药提醒开始 >>>>>> ");
        XxlJobHelper.log("推送用药提醒开始 >>>>>> ");

        patientPushServeTaskService.recipeMedicineRemindTaskExecute();

        XxlJobHelper.log("推送用药提醒结束 <<<<<< ");
        log.info("推送用药提醒结束 <<<<<< ");
    }

    

    // 扫描dwd当天的患者检查检验报告并发送提醒 // 互联网医院做
    @Deprecated
    @XxlJob("pushExamineAndLaboratoryReportJobHandler")
    public void pushExamineAndLaboratoryReportJobHandler() throws Exception {
        log.info("扫描dwd当天的患者检查检验报告并发送提醒开始 >>>>>> ");
        XxlJobHelper.log("扫描dwd当天的患者检查检验报告并发送提醒开始 >>>>>> ");

        patientPushServeTaskService.pushExamineAndLaboratoryReportTaskExecute();

        XxlJobHelper.log("扫描dwd当天的患者检查检验报告并发送提醒结束 <<<<<< ");
        log.info("扫描dwd当天的患者检查检验报告并发送提醒结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }

    // 扫描dwd当天的患者医嘱服药信息并发送提醒
    // @XxlJob("pushRecipeMedicineJobHandler")
    @Deprecated // 改为 recipeMedicineScheduleJobHandler
    public void pushRecipeMedicineJobHandler() throws Exception {
        log.info("扫描dwd当天的患者医嘱服药信息并发送提醒开始 >>>>>> ");
        XxlJobHelper.log("扫描dwd当天的患者医嘱服药信息并发送提醒开始 >>>>>> ");

        patientPushServeTaskService.pushRecipeMedicineTaskExecute();

        XxlJobHelper.log("扫描dwd当天的患者医嘱服药信息并发送提醒结束 <<<<<< ");
        log.info("扫描dwd当天的患者医嘱服药信息并发送提醒结束 <<<<<< ");
        TimeUnit.SECONDS.sleep(2);
    }

    // 保存推送服务日程数据
    @XxlJob("patientPushServeScheduleJob")
    public void patientPushServeScheduleJob() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("日程补刀开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("日程补刀开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetHour(now, -1); // 默认1小时
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        patientPushServeTaskService.patientPushServeScheduleJob(startTime, endTime);

        XxlJobHelper.log("日程补刀结束 <<<<<< ");
        log.info("日程补刀结束 <<<<<< ");
    }

}
