package com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.basic.pushserve.client.api.bean.bos.PushServeConfigBO;
import com.xxh.cloud.healthy.behavior.client.api.PatientBehaviorClientService;
import com.xxh.cloud.healthy.behavior.client.api.bean.dto.PatientBehaviorSaveClientDTO;
import com.xxh.cloud.healthy.behavior.client.api.constant.BehaviorSourceTypeConstant;
import com.xxh.cloud.healthy.behavior.client.api.constant.PatientOperatedTypeEnum;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.constant.WechatErrorCodeEnum;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.entity.PatientRemindInfo;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.entity.PatientScheduleInfo;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.service.PatientRemindInfoService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.base.service.PatientScheduleInfoService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeDateVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.helper.PatientPushServeHelper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.mapper.PatientPushServeMapper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.PatientPushServeDataService;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientPushServeDataServiceImpl implements PatientPushServeDataService {
    
    @Autowired
    private PatientScheduleInfoService patientScheduleInfoService;
    @Autowired
    private PatientRemindInfoService patientRemindInfoService;
    @Autowired
    private PatientBehaviorClientService patientBehaviorClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private PatientPushServeMapper patientPushServeMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveScheduleAndRemind(PatientPushServeHistoryVO history, List<PatientPushServeDateVO> sendDateList) {
        
        PushServeConfigBO pushserveConfigBO = history.getPushServeConfig();

        // 查询患者信息
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setPatientId(history.getPatientId());
        PatientClientBO patientBO = patientClientService.getOneSimple(patientQueryDTO);

        if (null != patientBO) {

            Long isExistFamilyMember = patientPushServeMapper.isExistFamilyMember(patientBO.getPatientId());
            if (null == isExistFamilyMember || isExistFamilyMember == 0) {
                MemberFamilyBO familyMember = new MemberFamilyBO(); // 要保存的家属信息
                familyMember.setId("d_" + StrUtil.format("FM{}", IdUtil.getSnowflake(4,2).nextIdStr()));
                familyMember.setUserId(patientBO.getPatientId());
                familyMember.setName(patientBO.getPatientName());
                familyMember.setMobile(patientBO.getPhone());
                familyMember.setGender(patientBO.getPatientGenderStandard());
                familyMember.setPatHisNo(patientBO.getPatientId()); // todo
                familyMember.setMainPersonId(patientBO.getPatientId());
    
                patientPushServeMapper.saveFamilyMemberInfo(familyMember);
            }

        }
        

        List<PatientScheduleInfo> patientScheduleInfoList = new ArrayList<>(); // 要保存的日程
        List<PatientRemindInfo> patientRemindInfoList = new ArrayList<>(); // 要保存的提醒

        List<PatientBehaviorSaveClientDTO> behaviorList = new ArrayList<>(); // 要保存的日程/提醒行为

        sendDateList.forEach(i -> {
            log.info("2.生成日程 >>>>>> ");
            // 日程：随机生成日程状态，0-5：待查看；6-9：已查看
            
            PatientScheduleInfo patientScheduleInfo = new PatientScheduleInfo();
            patientScheduleInfo.setId("d_" + PatientPushServeHelper.getPatientScheduleId());
            patientScheduleInfo.setScheduleDate(i.getSendDate());
            patientScheduleInfo.setPatientId(i.getPatientId());
            patientScheduleInfo.setUserId(i.getPatientId());
            patientScheduleInfo.setBizId(i.getPatientPushServeId());
            patientScheduleInfo.setBizOneType(ObjectTypeEnum.OT_OneType_PatientPushServe.getCode());
            // patientScheduleInfo.setBizTwoType(null);
            patientScheduleInfo.setTitle(history.getPushServeTitle());
            patientScheduleInfo.setContent(pushserveConfigBO.getMediaId()); // mediaId为推送消息的id，这个id可以使消息跳转到具体的消息页
            patientScheduleInfo.setScheduleType(pushserveConfigBO.getRemindWay());

            // 随机是否阅读：0-7：待查看；8-9：已查看
            Integer status = (int) (Math.random() * 10);
            if (status <= 7) {
                log.info("2.1.待查看日程 >>>>>>");
                patientScheduleInfo.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
            } else {
                log.info("2.1.已查看日程 >>>>>>");
                patientScheduleInfo.setStatus(PatientPushServeConstant.ScheduleStatus.DONE.getCode());
            }
            patientScheduleInfo.setRowsStatus(YN.YES.getCode());

            patientScheduleInfoList.add(patientScheduleInfo);


            // 日程行为：只有加入日程且状态为已查看的服务才能生成行为
            if (!RemindWayEnum.Remind.getCode().equals(pushserveConfigBO.getRemindWay()) && PatientPushServeConstant.ScheduleStatus.DONE.getCode().equals(patientScheduleInfo.getStatus())) {
                log.info("2.2.生成日程行为 >>>>>> ");
                this.saveScheduleBehavior(behaviorList, patientScheduleInfo, pushserveConfigBO.getId(), pushserveConfigBO.getMediaType(), pushserveConfigBO.getMediaId(), history.getOpDate());
            }
            
            // 提醒
            // 推送时间小于当前时间才生成提醒
            // 是否提醒：随机提醒状态，成功可以生成提醒行为
            if (!RemindWayEnum.NotRemind.getCode().equals(pushserveConfigBO.getRemindWay()) && i.getSendDate().before(DateUtil.date())) {
                log.info("3.生成提醒 >>>>>> ");

                PatientRemindInfo patientRemindInfo = new PatientRemindInfo();
                patientRemindInfo.setId("d_" + PatientPushServeHelper.getPatientRemindId());
                patientRemindInfo.setPatientId(i.getPatientId());
                patientRemindInfo.setBizId(i.getPatientPushServeId());
                patientRemindInfo.setBizOneType(ObjectTypeEnum.OT_OneType_PatientPushServe.getCode());
                // patientRemindInfo.setBizTwoType(i.getBizTwoType());
                patientRemindInfo.setTitle(history.getPushServeTitle());
                patientRemindInfo.setContent(pushserveConfigBO.getMediaId());
                patientRemindInfo.setPushChannel(pushserveConfigBO.getPushWay());
                // patientRemindInfo.setPushTempId(i.getPushTempId());
                // patientRemindInfo.setPushConfig(JSON.toJSONString(i.getPushConfig()));
                patientRemindInfo.setPushDate(i.getSendDate());
                // patientRemindInfo.setRemark(i.getRemark());
                patientRemindInfo.setRowsStatus(YN.YES.getCode());
                patientRemindInfo.setUserId(i.getPatientId());
                
                // patientRemindInfo.setPushAccount(i.getPushAccount());

                // 随机是否关注：0-4：未关注；5-7：成功；8-9：失败
                // Integer pushStatus = (int) (Math.random() * 10);
                // if (pushStatus <= 4) {
                //     log.info("3.1.未关注提醒 >>>>>>");
                //     patientRemindInfo.setPushStatus(PatientPushServeConstant.RemindPushStatus.NotFollow.getCode());
                //     patientRemindInfo.setErrorCode(WechatErrorCodeEnum.F43004.getCode());
                //     patientRemindInfo.setErrorMsg(WechatErrorCodeEnum.F43004.getMsg());
                // } else if (pushStatus >= 8) {
                //     log.info("3.1.发送失败提醒 >>>>>>");
                //     patientRemindInfo.setPushStatus(PatientPushServeConstant.RemindPushStatus.Fail.getCode());
                // } else {
                    log.info("3.1.发送成功提醒 >>>>>>");
                    patientRemindInfo.setPushStatus(PatientPushServeConstant.RemindPushStatus.Success.getCode());
                    patientRemindInfo.setErrorCode(WechatErrorCodeEnum.SUCCESS.getCode());
                    patientRemindInfo.setErrorMsg("ok");
                // }
                patientRemindInfo.setRowsStatus(YN.YES.getCode());

                patientRemindInfoList.add(patientRemindInfo);

                // 提醒行为
                if (PatientPushServeConstant.RemindPushStatus.Success.getCode().equals(patientRemindInfo.getPushStatus())) {
                    log.info("3.2.生成提醒行为 >>>>>> ");
                    this.saveRemindBehavior(behaviorList, patientRemindInfo, pushserveConfigBO.getId(), pushserveConfigBO.getMediaType(), pushserveConfigBO.getMediaId());
                }
                
            }

        });
        // 日程
        if (CollectionUtil.isNotEmpty(patientScheduleInfoList)) {
            log.info("4.保存日程 >>>>>> ");
            patientScheduleInfoService.saveBatch(patientScheduleInfoList);
        }
        // 提醒
        if (CollectionUtil.isNotEmpty(patientRemindInfoList)) {
            log.info("5.保存提醒 >>>>>> ");
            patientRemindInfoService.saveBatch(patientRemindInfoList);
        }
        // 行为
        if (CollectionUtil.isNotEmpty(behaviorList)) {
            log.info("6.保存行为 >>>>>> ");
            patientBehaviorClientService.simpleSaveBatch(behaviorList);
        }
    }

    private void saveScheduleBehavior(List<PatientBehaviorSaveClientDTO> behaviorList, PatientScheduleInfo patientScheduleInfo, String pushServeId, String mediaType, String mediaId, Date patientPushServeDate) {

        // 1. 用户是否阅读，若未阅读则也没有其他行为，若阅读则也会有阅读时长，阅读数据条数随机，阅读时长最多一条数据且随机
        // 2. 在阅读的基础上可以随机生成其他行为：0.感兴趣、1.已知晓、2.衍生阅读、3.挂号、4.在线问诊、5.就诊

        List<Date> browseDateList = new ArrayList<>();
        // 已阅读：阅读时间在日程时间的基础上随机增加时间
        // Integer browseNum = (int) (Math.random() * 2);
        Integer browseNum = 1;
        log.info("2.2.1.浏览行为 >>>>>> 次数：{}", browseNum);
        for (int i = 0; i < browseNum; i++) {
            Date createDate = DateUtil.offsetMinute(patientScheduleInfo.getScheduleDate(), (int) (Math.random() * 120) + 10);
            
            Date now = DateUtil.date();
            if (null != patientPushServeDate && patientScheduleInfo.getScheduleDate().after(now)) {
                long duration = DateUtil.between(patientPushServeDate, now, DateUnit.MINUTE);
                createDate = DateUtil.offsetMinute(patientPushServeDate, (int) (Math.random() * duration));
            }

            PatientBehaviorSaveClientDTO browse = new PatientBehaviorSaveClientDTO();
            browse.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            browse.setPatientId(patientScheduleInfo.getPatientId());
            browse.setUserId(patientScheduleInfo.getUserId());
            browse.setContentType(mediaType);
            browse.setContentId(mediaId);
            browse.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode());
            browse.setSourceOneId(patientScheduleInfo.getId());
            browse.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
            browse.setSourceTwoId(patientScheduleInfo.getBizId());
            browse.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
            browse.setSourceThreeId(pushServeId);
            browse.setOperatedType(PatientOperatedTypeEnum.Browse.getCode());
            browse.setCreateDate(createDate);

            behaviorList.add(browse);

            browseDateList.add(createDate);
        }

        // 阅读时长：阅读时长随机
        Integer browseDurationTime = (int) (Math.random() * 60) + 80; // 随机80-140秒
        log.info("2.2.2.浏览时长 >>>>>> {}", browseDurationTime);
        PatientBehaviorSaveClientDTO browseDuration = new PatientBehaviorSaveClientDTO();
        browseDuration.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
        browseDuration.setPatientId(patientScheduleInfo.getPatientId());
        browseDuration.setUserId(patientScheduleInfo.getUserId());
        browseDuration.setContentType(mediaType);
        browseDuration.setContentId(mediaId);
        browseDuration.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode());
        browseDuration.setSourceOneId(patientScheduleInfo.getId());
        browseDuration.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
        browseDuration.setSourceTwoId(patientScheduleInfo.getBizId());
        browseDuration.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
        browseDuration.setSourceThreeId(pushServeId);
        browseDuration.setOperatedType(PatientOperatedTypeEnum.Browse_Duration.getCode());
        browseDuration.setOperatedDuration(browseDurationTime);
        
        int browseDateIndex = (int) (Math.random() * browseDateList.size());
        browseDuration.setCreateDate(DateUtil.offsetMinute(browseDateList.get(browseDateIndex), (int) (Math.random() * 10) + 10));

        behaviorList.add(browseDuration);

        // 是否感兴趣：根据阅读时长是否大于等于30秒确定是否感兴趣 // TODO
        if (browseDurationTime >= 100) {
            log.info("2.2.3.感兴趣 >>>>>> ");
            PatientBehaviorSaveClientDTO interested = new PatientBehaviorSaveClientDTO();
            interested.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            interested.setPatientId(patientScheduleInfo.getPatientId());
            interested.setUserId(patientScheduleInfo.getUserId());
            interested.setContentType(mediaType);
            interested.setContentId(mediaId);
            interested.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode());
            interested.setSourceOneId(patientScheduleInfo.getId());
            interested.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
            interested.setSourceTwoId(patientScheduleInfo.getBizId());
            interested.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
            interested.setSourceThreeId(pushServeId);
            interested.setOperatedType(PatientOperatedTypeEnum.Interested.getCode());
            interested.setCreateDate(browseDuration.getCreateDate());

            behaviorList.add(interested);
        }

        // 是否已知晓：在阅读时间上随机增加时间
        Integer isKnew = (int) (Math.random() * 100);
        if (isKnew > 93) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("2.2.4.已知晓 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Knew.getCode(), 
                                loopNum, 
                                patientScheduleInfo.getPatientId(), 
                                patientScheduleInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode(), 
                                patientScheduleInfo.getId(), 
                                patientScheduleInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否衍生阅读：在阅读时间上随机增加时间
        Integer isFurtherReading = (int) (Math.random() * 100);
        if (isFurtherReading > 93) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("2.2.5.衍生阅读 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Further_Reading.getCode(), 
                                loopNum, 
                                patientScheduleInfo.getPatientId(), 
                                patientScheduleInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode(), 
                                patientScheduleInfo.getId(), 
                                patientScheduleInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否挂号：在阅读时间上随机增加时间
        Integer isRegister = (int) (Math.random() * 100);
        if (isRegister > 94) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("2.2.6.挂号 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Register.getCode(), 
                                loopNum, 
                                patientScheduleInfo.getPatientId(), 
                                patientScheduleInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode(), 
                                patientScheduleInfo.getId(), 
                                patientScheduleInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否在线问诊：在阅读时间上随机增加时间
        Integer isConsultationOnline = (int) (Math.random() * 100);
        if (isConsultationOnline > 95) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("2.2.7.在线问诊 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Consultation_Online.getCode(), 
                                loopNum, 
                                patientScheduleInfo.getPatientId(), 
                                patientScheduleInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode(), 
                                patientScheduleInfo.getId(), 
                                patientScheduleInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否就诊：在阅读时间上随机增加时间
        Integer isConsultation = (int) (Math.random() * 100);
        if (isConsultation > 96) {
            // Integer loopNum = (int) (Math.random() * 2) + 1;
            Integer loopNum = 1;
            log.info("2.2.8.就诊 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Consultation.getCode(), 
                                loopNum, 
                                patientScheduleInfo.getPatientId(), 
                                patientScheduleInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Schedule.getCode(), 
                                patientScheduleInfo.getId(), 
                                patientScheduleInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

    }
    
    private void saveRemindBehavior(List<PatientBehaviorSaveClientDTO> behaviorList, PatientRemindInfo patientRemindInfo, String pushServeId, String mediaType, String mediaId) {
        
        // 1. 用户是否阅读，若未阅读则也没有其他行为，若阅读则也会有阅读时长，阅读数据条数随机，阅读时长最多一条数据且随机
        // 2. 在阅读的基础上可以随机生成其他行为：0.感兴趣、1.已知晓、2.衍生阅读、3.挂号、4.在线问诊、5.就诊

        // 随机是否阅读：0-7：待查看；8-9：已查看
        Integer status = (int) (Math.random() * 10);
        if (status <= 7) {
            return;
        }

        List<Date> browseDateList = new ArrayList<>(); // 浏览时间列表
        // 已阅读：阅读时间在日程时间的基础上随机增加时间
        // Integer browseNum = (int) (Math.random() * 4) + 1;
        Integer browseNum = 1;
        log.info("3.2.1.浏览行为 >>>>>> 次数：{}", browseNum);
        for (int i = 0; i < browseNum; i++) {
            Date createDate = DateUtil.offsetMinute(patientRemindInfo.getPushDate(), (int) (Math.random() * 120) + 10);

            PatientBehaviorSaveClientDTO browse = new PatientBehaviorSaveClientDTO();
            browse.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            browse.setPatientId(patientRemindInfo.getPatientId());
            browse.setUserId(patientRemindInfo.getUserId());
            browse.setContentType(mediaType);
            browse.setContentId(mediaId);
            browse.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode());
            browse.setSourceOneId(patientRemindInfo.getId());
            browse.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
            browse.setSourceTwoId(patientRemindInfo.getBizId());
            browse.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
            browse.setSourceThreeId(pushServeId);
            browse.setOperatedType(PatientOperatedTypeEnum.Browse.getCode());
            browse.setCreateDate(createDate);

            behaviorList.add(browse);

            browseDateList.add(createDate);
        }

        // 阅读时长：阅读时长随机
        Integer browseDurationTime = (int) (Math.random() * 60) + 80; // 随机80-140秒
        log.info("3.2.2.阅读时长 >>>>>> {}", browseDurationTime);
        PatientBehaviorSaveClientDTO browseDuration = new PatientBehaviorSaveClientDTO();
        browseDuration.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
        browseDuration.setPatientId(patientRemindInfo.getPatientId());
        browseDuration.setUserId(patientRemindInfo.getUserId());
        browseDuration.setContentType(mediaType);
        browseDuration.setContentId(mediaId);
        browseDuration.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode());
        browseDuration.setSourceOneId(patientRemindInfo.getId());
        browseDuration.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
        browseDuration.setSourceTwoId(patientRemindInfo.getBizId());
        browseDuration.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
        browseDuration.setSourceThreeId(pushServeId);
        browseDuration.setOperatedType(PatientOperatedTypeEnum.Browse_Duration.getCode());
        browseDuration.setOperatedDuration(browseDurationTime);
        
        int browseDateIndex = (int) (Math.random() * browseDateList.size());
        browseDuration.setCreateDate(DateUtil.offsetMinute(browseDateList.get(browseDateIndex), (int) (Math.random() * 10) + 10));

        behaviorList.add(browseDuration);


        // 是否感兴趣：根据阅读时长是否大于等于30秒确定是否感兴趣 // TODO
        if (browseDurationTime >= 100) {
            log.info("3.2.3.感兴趣 >>>>>> ");
            PatientBehaviorSaveClientDTO interested = new PatientBehaviorSaveClientDTO();
            interested.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            interested.setPatientId(patientRemindInfo.getPatientId());
            interested.setUserId(patientRemindInfo.getUserId());
            interested.setContentType(mediaType);
            interested.setContentId(mediaId);
            interested.setSourceOneType(BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode());
            interested.setSourceOneId(patientRemindInfo.getId());
            interested.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
            interested.setSourceTwoId(patientRemindInfo.getBizId());
            interested.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
            interested.setSourceThreeId(pushServeId);
            interested.setOperatedType(PatientOperatedTypeEnum.Interested.getCode());
            interested.setCreateDate(browseDuration.getCreateDate());

            behaviorList.add(interested);
        }

        // 是否已知晓：在阅读时间上随机增加时间
        Integer isKnew = (int) (Math.random() * 100);
        if (isKnew > 93) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("3.2.4.已知晓 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Knew.getCode(), 
                                loopNum, 
                                patientRemindInfo.getPatientId(), 
                                patientRemindInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode(), 
                                patientRemindInfo.getId(), 
                                patientRemindInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }


        // 是否衍生阅读：在阅读时间上随机增加时间
        Integer isFurtherReading = (int) (Math.random() * 100);
        if (isFurtherReading > 93) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("3.2.5.衍生阅读 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Further_Reading.getCode(), 
                                loopNum, 
                                patientRemindInfo.getPatientId(), 
                                patientRemindInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode(), 
                                patientRemindInfo.getId(), 
                                patientRemindInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否挂号：在阅读时间上随机增加时间
        Integer isRegister = (int) (Math.random() * 100);
        if (isRegister > 94) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("3.2.6.挂号 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Register.getCode(), 
                                loopNum, 
                                patientRemindInfo.getPatientId(), 
                                patientRemindInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode(), 
                                patientRemindInfo.getId(), 
                                patientRemindInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否在线问诊：在阅读时间上随机增加时间
        Integer isConsultationOnline = (int) (Math.random() * 100);
        if (isConsultationOnline > 95) {
            // Integer loopNum = (int) (Math.random() * 4) + 1;
            Integer loopNum = 1;
            log.info("3.2.7.在线问诊 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Consultation_Online.getCode(), 
                                loopNum, 
                                patientRemindInfo.getPatientId(), 
                                patientRemindInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode(), 
                                patientRemindInfo.getId(), 
                                patientRemindInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

        // 是否就诊：在阅读时间上随机增加时间
        Integer isConsultation = (int) (Math.random() * 100);
        if (isConsultation > 96) {
            // Integer loopNum = (int) (Math.random() * 2) + 1;
            Integer loopNum = 1;
            log.info("3.2.8.就诊 >>>>>> 循环次数：{}", loopNum);
            this.createBehavior(behaviorList, 
                                PatientOperatedTypeEnum.Consultation.getCode(), 
                                loopNum, 
                                patientRemindInfo.getPatientId(), 
                                patientRemindInfo.getUserId(), 
                                BehaviorSourceTypeConstant.SourceOneTypeEnum.Remind.getCode(), 
                                patientRemindInfo.getId(), 
                                patientRemindInfo.getBizId(), 
                                browseDateList, 
                                pushServeId, 
                                mediaType, 
                                mediaId);
        }

    }

    // 生成行为
    private void createBehavior(List<PatientBehaviorSaveClientDTO> behaviorList, 
                            String operatedType,
                            int loopNum,
                            String patientId, 
                            String userId, 
                            String sourceOneType, 
                            String sourceOneId, 
                            String sourceTwoId, 
                            List<Date> browseDateList, 
                            String pushServeId, 
                            String mediaType, 
                            String mediaId) {
        for (int i = 0; i < loopNum; i++) {
            PatientBehaviorSaveClientDTO behavior = new PatientBehaviorSaveClientDTO();
            behavior.setId("d_" + StrUtil.format("PB{}", IdUtil.getSnowflake(5,6).nextIdStr()));
            behavior.setPatientId(patientId);
            behavior.setUserId(userId);
            behavior.setContentType(mediaType);
            behavior.setContentId(mediaId);
            behavior.setSourceOneType(sourceOneType);
            behavior.setSourceOneId(sourceOneId);
            behavior.setSourceTwoType(BehaviorSourceTypeConstant.SourceTwoTypeEnum.PatientPushServe.getCode());
            behavior.setSourceTwoId(sourceTwoId);
            behavior.setSourceThreeType(BehaviorSourceTypeConstant.SourceThreeTypeEnum.PushServeConfig.getCode());
            behavior.setSourceThreeId(pushServeId);
            behavior.setOperatedType(operatedType);

            int browseDateIndex = (int) (Math.random() * browseDateList.size());
            behavior.setCreateDate(DateUtil.offsetMinute(browseDateList.get(browseDateIndex), (int) (Math.random() * 10) + 10));

            behaviorList.add(behavior);
        }
    }

}
