package com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.client.api.bean.dto.MemberFamilyListClientDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.healthy.basic.common.bean.ServPushConfigModel;
import com.xxh.cloud.healthy.basic.common.constant.PushWayEnum;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientExamineReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientLaboratoryReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientRecipeMedicineClientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientPushServeConstant;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientRemindAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.mapper.PatientPushSyncFlagMapper;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.task.service.PatientPushServeTaskHandlerService;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class PatientPushServeTaskHandlerServiceImpl implements PatientPushServeTaskHandlerService {

    @Autowired
    private PatientPushSyncFlagMapper patientPushSyncFlagMapper;
    @Autowired
    private PatientPushServeBizService patientPushServeBizService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveExamineAndLaboratoryReportRemind(List<PatientExamineReportClientBO> examineReportList, List<PatientLaboratoryReportClientBO> laboraatoryReportList, String flagTime) {
          
        String date = DateUtil.parse(DateUtil.now(), "yyyy-MM-dd HH:mm").toString();

        this.saveExamineReport(examineReportList, date);

        this.saveLaboratoryReport(laboraatoryReportList, date);

        // 更新同步标记
        log.info("更新同步标记时间 >>>>>> flagTime={}", flagTime);
        patientPushSyncFlagMapper.updateSyncFlag(flagTime, "Examine_Laboratory_Report_Remind");

    }

    // 处理患者检查报告
    private void saveExamineReport(List<PatientExamineReportClientBO> examineReportList, String date) {
        if (CollectionUtil.isEmpty(examineReportList)) {
            return;
        }

        List<PatientScheduleAddDTO> scheduleList = new ArrayList<>();
        List<PatientRemindAddDTO> remindList = new ArrayList<>();

        List<String> patientIds = new ArrayList<>(); // 患者id，用户查询对应的用户信息（user_id, open_id），只查本人

        examineReportList.forEach(i -> {
            patientIds.add(i.getPatientId());

            PatientScheduleAddDTO schedule = new PatientScheduleAddDTO();
            schedule.setScheduleDate(date);
            schedule.setPatientId(i.getPatientId());
            // schedule.setUserId(); // 另外查询
            // schedule.setBizId();
            schedule.setBizOneType("examine_report"); // 检查报告类型
            // patientScheduleInfo.setBizTwoType(null);
            schedule.setTitle(i.getReportNameStandard());
            schedule.setContent(i.getReportDescriptionStandard()); // mediaId为推送消息的id，这个id可以使消息跳转到具体的消息页
            schedule.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
            schedule.setScheduleType(RemindWayEnum.Remind.getCode()); // 只提醒

            scheduleList.add(schedule);



            PatientRemindAddDTO remind = new PatientRemindAddDTO();
            remind.setPatientId(i.getPatientId());
            // remind.setUserId(user.getUserId()); // 另外查询
            // remind.setBizId();
            remind.setBizOneType("examine_report"); // 检查报告类型
            // patientRemindAddDTO.setBizTwoType(null);
            remind.setTitle(i.getReportNameStandard());
            remind.setContent(i.getReportDescriptionStandard());
            remind.setRemark("检查报告提醒");
            remind.setPushChannel(PushWayEnum.PCW_WX.getCode());
            // remind.setPushAccount();
            // remind.setPushTempId();
            remind.setPushDate(DateUtil.parseDateTime(date));
            remind.setPushStatus(PatientPushServeConstant.RemindPushStatus.Todo.getCode());

            ServPushConfigModel pushConfig = new ServPushConfigModel();
            pushConfig.setTitle(i.getReportNameStandard());
            pushConfig.setContent(i.getReportDescriptionStandard());
            pushConfig.setDate(date);
            pushConfig.setDoctor(i.getDoctorCode());
            remind.setPushConfig(pushConfig);
    
            remindList.add(remind);
            
        });

        if (CollectionUtil.isNotEmpty(patientIds)) {
            
            List<PatientScheduleAddDTO> schedules = new ArrayList<>();
            List<PatientRemindAddDTO> reminds = new ArrayList<>();

            Map<String, MemberFamilyBO> memberMap = this.getMemberMapByPatientIds(patientIds);
            scheduleList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    i.setUserId(member.getUserId());
                    schedules.add(i);
                }
            });
            remindList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    i.setUserId(member.getUserId());
                    i.setPushAccount(member.getUserOpenId());
                    reminds.add(i);
                }
            });

            // 保存到日程
            patientPushServeBizService.addPatientScheduleBatch(schedules);
            // 保存到提醒
            patientPushServeBizService.addPatientRemindBatch(reminds);
        }
    }

    // 处理患者检验报告
    private void saveLaboratoryReport(List<PatientLaboratoryReportClientBO> laboraatoryReportList, String date) {
        if (CollectionUtil.isEmpty(laboraatoryReportList)) {
            return;
        }
        
        List<PatientScheduleAddDTO> scheduleList = new ArrayList<>();
        List<PatientRemindAddDTO> remindList = new ArrayList<>();

        List<String> patientIds = new ArrayList<>(); // 患者id，用户查询对应的用户信息（user_id, open_id），只查本人

        laboraatoryReportList.forEach(i -> {
            patientIds.add(i.getPatientId());

            PatientScheduleAddDTO schedule = new PatientScheduleAddDTO();
            schedule.setScheduleDate(date);
            schedule.setPatientId(i.getPatientId());
            // schedule.setUserId(); // 另外查询
            // schedule.setBizId();
            schedule.setBizOneType("laboratory_report"); // 检验报告类型
            // patientScheduleInfo.setBizTwoType(null);
            schedule.setTitle(i.getReportName());
            schedule.setContent(i.getDescriptionStandard()); // mediaId为推送消息的id，这个id可以使消息跳转到具体的消息页
            schedule.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
            schedule.setScheduleType(RemindWayEnum.Remind.getCode()); // 只提醒

            scheduleList.add(schedule);



            PatientRemindAddDTO remind = new PatientRemindAddDTO();
            remind.setPatientId(i.getPatientId());
            // remind.setUserId(user.getUserId()); // 另外查询
            // remind.setBizId();
            remind.setBizOneType("laboratory_report"); // 检验报告类型
            // patientRemindAddDTO.setBizTwoType(null);
            remind.setTitle(i.getReportName());
            remind.setContent(i.getDescriptionStandard());
            remind.setRemark("检验报告提醒");
            remind.setPushChannel(PushWayEnum.PCW_WX.getCode());
            // remind.setPushAccount();
            // remind.setPushTempId();
            remind.setPushDate(DateUtil.parseDateTime(date));
            remind.setPushStatus(PatientPushServeConstant.RemindPushStatus.Todo.getCode());
    
            ServPushConfigModel pushConfig = new ServPushConfigModel();
            pushConfig.setTitle(i.getReportName());
            pushConfig.setContent(i.getDescriptionStandard());
            pushConfig.setDate(date);
            pushConfig.setDoctor(i.getDoctorCode());
            remind.setPushConfig(pushConfig);
    
            remindList.add(remind);
            
        });

        if (CollectionUtil.isNotEmpty(patientIds)) {
            
            List<PatientScheduleAddDTO> schedules = new ArrayList<>();
            List<PatientRemindAddDTO> reminds = new ArrayList<>();

            Map<String, MemberFamilyBO> memberMap = this.getMemberMapByPatientIds(patientIds);
            scheduleList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    i.setUserId(member.getUserId());
                    schedules.add(i);
                }
            });
            remindList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    i.setUserId(member.getUserId());
                    i.setPushAccount(member.getUserOpenId());
                    reminds.add(i);
                }
            });

            // 保存到日程
            patientPushServeBizService.addPatientScheduleBatch(schedules);
            // 保存到提醒
            patientPushServeBizService.addPatientRemindBatch(reminds);
        }
    }

    // TODO 暂时只推送给本人，后续需要区分推送给本人和家属
    private Map<String, MemberFamilyBO> getMemberMapByPatientIds(List<String> patientIds) {
        // 1:本人 2:配偶 3:父母 4:子女 5:他人
        MemberFamilyListClientDTO memberFamilyListClientDTO = new MemberFamilyListClientDTO();
        memberFamilyListClientDTO.setRelationType("1"); // TODO 只查询本人
        memberFamilyListClientDTO.setPatHisNos(patientIds); // zs20231115 家属表中patientId来自海鹚和本平台的patientId不一致，一致的是patHisNos（就诊人门诊id）
        // memberFamilyListClientDTO.setMainPersonId(patientId); // zs20240318
        memberFamilyListClientDTO.setIsAuth(YN.YES.getCode());
        List<MemberFamilyBO> memberFamilyBOs = customerFamilyClientService.listByUser(memberFamilyListClientDTO);

        Map<String, MemberFamilyBO> memberFamilyMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(memberFamilyBOs)) {
            memberFamilyBOs.forEach(i -> {
                memberFamilyMap.put(i.getPatHisNo(), i);
            });
        }
        return memberFamilyMap;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveRecipeMedicineSchedule(List<PatientRecipeMedicineClientBO> recipeMedicineList, String flagTime) {
          
        this.saveRecipeMedicine(recipeMedicineList);

        // 更新同步标记
        log.info("更新同步标记时间 >>>>>> flagTime={}", flagTime);
        patientPushSyncFlagMapper.updateSyncFlag(flagTime, "Recipe_Medicine_Remind");

    }

    // 处理患者服药
    private void saveRecipeMedicine(List<PatientRecipeMedicineClientBO> recipeMedicineList) {
        if (CollectionUtil.isEmpty(recipeMedicineList)) {
            return;
        }

        String date = DateUtil.parse(DateUtil.now(), "yyyy-MM-dd HH:mm").toString();

        Map<String, PatientScheduleAddDTO> scheduleMap = new HashMap<>();
        Map<String, PatientRemindAddDTO> remindMap = new HashMap<>();

        Set<String> patientIds = new HashSet<>(); // 患者id，用户查询对应的用户信息（user_id, open_id），只查本人

        for (PatientRecipeMedicineClientBO i : recipeMedicineList) {
            patientIds.add(i.getPatientId());

            // // TODO 根据具体的药物的服用频次，生成对应的日程，已有相同时间的日程则跳过
            // for (PatientRecipeMedicineItemClientBO j : i.getPatientRecipeMedicineItemList()) {

            //     // TODO 根据总药量/总服药天数和药物频次生成日程
            //     // 如：每天三次，服用一周，则生成21条日程，外层循环7天，内层循环3次；暂时每天只生成一次，即生成7条日程

                
            //     // 根据当前提醒时间计算下次提醒时间
            //     date = DateUtil.offsetDay(DateUtil.parseDate(date), 1).toDateStr() + " 07:00:00";
            // }

            // TODO 没有总药量/总天数，暂时只生成一次药物提醒
            PatientScheduleAddDTO schedule = scheduleMap.get(i.getPatientId() + "--" + date);
            if (null != schedule) {
                continue;
            }
            schedule = new PatientScheduleAddDTO();
            schedule.setScheduleDate(date);
            schedule.setPatientId(i.getPatientId());
            // schedule.setUserId(); // 另外查询
            schedule.setBizId(i.getId());
            schedule.setBizOneType("recipe_medicine");
            // patientScheduleInfo.setBizTwoType(null);
            schedule.setTitle("【服药提醒】请遵医嘱服药");
            schedule.setContent(i.getId()); // 医嘱服药id
            schedule.setStatus(PatientPushServeConstant.ScheduleStatus.TODO.getCode());
            schedule.setScheduleType(RemindWayEnum.ALL.getCode()); // 只提醒
    
            scheduleMap.put(i.getPatientId() + "--" + date, schedule);


            // PatientRemindAddDTO remind = remindMap.get(i.getPatientId() + "--" + date);
            // if (null != remind) {
            //     continue;
            // }
            PatientRemindAddDTO remind = new PatientRemindAddDTO();
            remind.setPatientId(i.getPatientId());
            // remind.setUserId(user.getUserId()); // 另外查询
            remind.setBizId(i.getId());
            remind.setBizOneType("recipe_medicine");
            // patientRemindAddDTO.setBizTwoType(null);
            remind.setTitle("【服药提醒】请遵医嘱服药");
            remind.setContent(i.getId());
            remind.setRemark("医嘱服药提醒");
            remind.setPushChannel(PushWayEnum.PCW_WX.getCode());
            // remind.setPushAccount();
            // remind.setPushTempId();
            remind.setPushDate(DateUtil.parseDateTime(date));
            remind.setPushStatus(PatientPushServeConstant.RemindPushStatus.Todo.getCode());
    
            ServPushConfigModel pushConfig = new ServPushConfigModel();
            pushConfig.setTitle("【服药提醒】请遵医嘱服药");
            pushConfig.setContent(i.getId());
            pushConfig.setDate(date);
            pushConfig.setDoctor(i.getDoctorCode());
            remind.setPushConfig(pushConfig);

    
            remindMap.put(i.getPatientId() + "--" + date, remind);

        }

        if (CollectionUtil.isNotEmpty(patientIds)) {
            // 生成的未指定家属的日程和提醒
            List<PatientScheduleAddDTO> scheduleList = new ArrayList<>();
            List<PatientRemindAddDTO> remindList = new ArrayList<>();
            scheduleMap.forEach((k, v) -> scheduleList.add(v));
            remindMap.forEach((k, v) -> remindList.add(v));

            // TODO 根据患者查询家属（暂时是本人）
            List<PatientScheduleAddDTO> schedules = new ArrayList<>();
            List<PatientRemindAddDTO> reminds = new ArrayList<>();
            
            Map<String, MemberFamilyBO> memberMap = this.getMemberMapByPatientIds(new ArrayList<>(patientIds));
            scheduleList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    // TODO 设置要发送给家属的家属id，若家属有多个那需要copy实体并分别设置家属id
                    PatientScheduleAddDTO schedule = new PatientScheduleAddDTO();
                    BeanUtil.copyProperties(i, schedule, false);
                    schedule.setUserId(member.getUserId());
                    schedules.add(schedule);
                }
            });
            remindList.forEach(i -> {
                MemberFamilyBO member = memberMap.get(i.getPatientId());
                if (member != null) {
                    // TODO 设置要发送给家属的家属id，若家属有多个那需要copy实体并分别设置家属id
                    PatientRemindAddDTO remind = new PatientRemindAddDTO();
                    BeanUtil.copyProperties(i, remind, false);
                    remind.setUserId(member.getUserId());
                    remind.setPushAccount(member.getUserOpenId());
                    reminds.add(remind);
                }
            });

            // 保存到日程
            patientPushServeBizService.addPatientScheduleBatch(schedules);
            // 保存到提醒
            patientPushServeBizService.addPatientRemindBatch(reminds);
        }
    }
}
