package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorFollowUpOneDayInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorFollowUpOneDayInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpOneDayAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpOneDayQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorFollowUpOneDayVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorFollowUpOneDayBizService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class DoctorFollowUpOneDayBizServiceImpl implements DoctorFollowUpOneDayBizService {
    
    @Autowired
    private DoctorFollowUpOneDayInfoService doctorFollowUpOneDayInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public PageData<DoctorFollowUpOneDayVO> query(DoctorFollowUpOneDayQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<DoctorFollowUpOneDayInfo> page = doctorFollowUpOneDayInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorFollowUpOneDayInfo>()
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .ge(null != params.getStartDate(), "biz_day", params.getStartDate())
                .le(null != params.getEndDate(), "biz_day", params.getEndDate())
                .orderByDesc("biz_day")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> doctorIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            doctorIds.add(i.getDoctorId());
        });

        // 医生信息
        Map<String, AdminUserBO> doctorMap = adminUserClientService.getUserMapByIds(doctorIds);

        PageData<DoctorFollowUpOneDayVO> data = new PageConverter<DoctorFollowUpOneDayInfo, DoctorFollowUpOneDayVO>() {
            @Override
            public DoctorFollowUpOneDayVO converter(DoctorFollowUpOneDayInfo entity) {
                DoctorFollowUpOneDayVO vo = new DoctorFollowUpOneDayVO();
                vo.setId(entity.getId());
                vo.setBizDay(entity.getBizDay());
                vo.setDoctorId(entity.getDoctorId());
                vo.setTotalNum(entity.getTotalNum());
                vo.setRealNum(entity.getRealNum());
                vo.setOpDate(entity.getOpDate());

                AdminUserBO doctor = doctorMap.get(entity.getDoctorId());
                if (null != doctor) {
                    vo.setDoctorName(doctor.getRealName());
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public void saveBatch(List<DoctorFollowUpOneDayAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        List<String> existParams = new ArrayList<>();
        params.forEach(i -> {
            existParams.add(i.getBizDay() + "_" + i.getDoctorId());
        });

        Map<String, DoctorFollowUpOneDayInfo> existMap = new HashMap<>();
        List<DoctorFollowUpOneDayInfo> existList = doctorFollowUpOneDayInfoService.list(new QueryWrapper<DoctorFollowUpOneDayInfo>()
            .in("CONCAT(biz_day,'_',doctor_id)", existParams)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> {
                existMap.put(i.getBizDay() + "_" + i.getDoctorId(), i);
            });
        }

        List<DoctorFollowUpOneDayInfo> saveList = new ArrayList<>();
        List<DoctorFollowUpOneDayInfo> updateList = new ArrayList<>();

        params.forEach(i -> {
            DoctorFollowUpOneDayInfo entity = existMap.get(i.getBizDay() + "_" + i.getDoctorId());

            if (null == entity) {
                entity = new DoctorFollowUpOneDayInfo();
                entity.setId(StrUtil.format("DFOD{}", IdUtil.getSnowflake(5,6).nextIdStr()));
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }

            entity.setBizDay(i.getBizDay());
            entity.setDoctorId(i.getDoctorId());
            entity.setTotalNum(i.getTotalNum());
            entity.setRealNum(i.getRealNum());
            entity.setOpDate(DateUtil.date());
        });
        
        if (CollUtil.isNotEmpty(saveList)) {
            doctorFollowUpOneDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            doctorFollowUpOneDayInfoService.updateBatchById(updateList);
        }
    }

}
