package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRelationInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRelationInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationAddBatchDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationGetByDoctorDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationGetDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRelationByDoctorVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRelationVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.mapper.DoctorRelationBizMapper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorRelationBizService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class DoctorRelationBizServiceImpl implements DoctorRelationBizService {

    @Autowired
    private DoctorRelationInfoService doctorRelationInfoService;
    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private DoctorRelationBizMapper doctorRelationBizMapper;

    @Override
    public PageData<DoctorRelationVO> page(DoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        IPage<DoctorRelationInfo> page = doctorRelationInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorRelationInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getRelationId()), "relation_id", params.getRelationId())
                .in(CollUtil.isNotEmpty(params.getRelationIds()), "relation_id", params.getRelationIds())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> doctorIds = new HashSet<>();
        Set<String> deptIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        page.getRecords().forEach(i -> {
            if (StrUtil.isNotBlank(i.getDoctorId())) {
                doctorIds.add(i.getDoctorId());
            }
            if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                deptIds.add(i.getRelationId());
            }
            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                diseaseIds.add(i.getRelationId());
            }
        });
        
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                doctorList.forEach(i -> doctorMap.put(i.getId(), i));
            }
        }

        // 部门信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            if (CollUtil.isNotEmpty(deptList)) {
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            if (CollUtil.isNotEmpty(diseaseList)) {
                diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            }
        }

        PageData<DoctorRelationVO> data = new PageConverter<DoctorRelationInfo, DoctorRelationVO>() {
            @Override
            public DoctorRelationVO converter(DoctorRelationInfo entity) {
                DoctorRelationVO vo = DoctorPushTaskHelper.getDoctorRelationVO(entity);

                vo.setDoctorInfo(doctorMap.get(vo.getDoctorId()));
                if (DoctorRelationConstant.Type.DEPT.getCode().equals(vo.getType())) {
                    vo.setDeptInfo(deptMap.get(vo.getRelationId()));
                }
                if (DoctorRelationConstant.Type.DISEASE.getCode().equals(vo.getType())) {
                    vo.setDiseaseInfo(diseaseMap.get(vo.getRelationId()));
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DoctorRelationVO> list(DoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        List<DoctorRelationInfo> list = doctorRelationInfoService.list(new QueryWrapper<DoctorRelationInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getRelationId()), "relation_id", params.getRelationId())
                .in(CollUtil.isNotEmpty(params.getRelationIds()), "relation_id", params.getRelationIds())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Set<String> doctorIds = new HashSet<>();
        Set<String> deptIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        list.forEach(i -> {
            if (StrUtil.isNotBlank(i.getDoctorId())) {
                doctorIds.add(i.getDoctorId());
            }
            if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                deptIds.add(i.getRelationId());
            }
            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                diseaseIds.add(i.getRelationId());
            }
        });
        
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                doctorList.forEach(i -> doctorMap.put(i.getId(), i));
            }
        }

        // 部门信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            if (CollUtil.isNotEmpty(deptList)) {
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            if (CollUtil.isNotEmpty(diseaseList)) {
                diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            }
        }

        return list.stream().map(i -> {
            DoctorRelationVO vo = DoctorPushTaskHelper.getDoctorRelationVO(i);

            vo.setDoctorInfo(doctorMap.get(vo.getDoctorId()));
            if (DoctorRelationConstant.Type.DEPT.getCode().equals(vo.getType())) {
                vo.setDeptInfo(deptMap.get(vo.getRelationId()));
            }
            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(vo.getType())) {
                vo.setDiseaseInfo(diseaseMap.get(vo.getRelationId()));
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public List<DoctorRelationVO> simpleList(DoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        
        List<DoctorRelationInfo> list = doctorRelationInfoService.list(new QueryWrapper<DoctorRelationInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getRelationId()), "relation_id", params.getRelationId())
                .in(CollUtil.isNotEmpty(params.getRelationIds()), "relation_id", params.getRelationIds())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .orderByDesc("update_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(DoctorPushTaskHelper::getDoctorRelationVO).collect(Collectors.toList());
    }

    @Override
    public DoctorRelationVO detail(DoctorRelationGetDTO params) {
        ValidatorUtils.validateEntity(params);
        
        DoctorRelationInfo entity = doctorRelationInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return null;
        }

        return DoctorPushTaskHelper.getDoctorRelationVO(entity);
    }

    @Override
    public void add(DoctorRelationAddDTO params) {

        DoctorRelationInfo entity = doctorRelationInfoService.getOne(new QueryWrapper<DoctorRelationInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq("type", params.getType())
            .eq("doctor_id", params.getDoctorId())
            .eq("relation_id", params.getRelationId())
        );
        if (null != entity) {
            return;
        }
        
        entity = new DoctorRelationInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorRelationId());
        entity.setType(params.getType());
        entity.setDoctorId(params.getDoctorId());
        entity.setRelationId(params.getRelationId());
        entity.setRemark(params.getRemark());
        entity.setStatus(YN.YES.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        doctorRelationInfoService.save(entity);
    }

    @Override
    public void update(DoctorRelationUpdateDTO params) {
        DoctorRelationInfo entity = doctorRelationInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return;
        }

        entity.setDoctorId(params.getDoctorId());
        entity.setType(params.getType());
        entity.setRelationId(params.getRelationId());
        entity.setStatus(params.getStatus());
        entity.setRemark(params.getRemark());

        doctorRelationInfoService.updateById(entity);
    }

    @Override
    public void del(DoctorRelationGetDTO params) {
        DoctorRelationInfo entity = doctorRelationInfoService.getById(params.getId());
        if (null == entity || !YN.YES.getCode().equals(entity.getRowsStatus())) {
            return;
        }

        entity.setStatus(YN.NO.getCode());
        entity.setRowsStatus(YN.NO.getCode());

        doctorRelationInfoService.updateById(entity);
    }

    @Override
    public PageData<DoctorRelationByDoctorVO> pageByDoctor(DoctorRelationQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<DoctorRelationByDoctorVO> page = doctorRelationBizMapper.pageByDoctor(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DoctorRelationInfo>()
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .in(CollUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
                .eq(StrUtil.isNotBlank(params.getRelationId()), "relation_id", params.getRelationId())
                .in(CollUtil.isNotEmpty(params.getRelationIds()), "relation_id", params.getRelationIds())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
        );
        
        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        Set<String> doctorIds = new HashSet<>();
        page.getRecords().forEach(i -> doctorIds.add(i.getDoctorId()));
        
        List<DoctorRelationInfo> list = doctorRelationInfoService.list(new QueryWrapper<DoctorRelationInfo>()
                .eq("rows_status", YN.YES.getCode())
                .in("doctor_id", doctorIds)
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getType()), "type", params.getType())
                .eq(StrUtil.isNotBlank(params.getRelationId()), "relation_id", params.getRelationId())
                .in(CollUtil.isNotEmpty(params.getRelationIds()), "relation_id", params.getRelationIds())
                .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
                .orderByDesc("update_date")
        );

        Map<String, List<DoctorRelationInfo>> map = new HashMap<>();
        list.forEach(i -> {
            List<DoctorRelationInfo> l = map.get(i.getDoctorId());
            if (null == l) {
                l = new ArrayList<>();
                map.put(i.getDoctorId(), l);
            }
            l.add(i);
        });

        Set<String> deptIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        list.forEach(i -> {
            if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                deptIds.add(i.getRelationId());
            }
            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                diseaseIds.add(i.getRelationId());
            }
        });
        
        Map<String, AdminUserBO> doctorMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorIds)) {
            List<AdminUserBO> doctorList = adminUserClientService.getUserListByIds(doctorIds);
            if (CollUtil.isNotEmpty(doctorList)) {
                doctorList.forEach(i -> doctorMap.put(i.getId(), i));
            }
        }

        // 部门信息
        Map<String, DeptBO> deptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            List<DeptBO> deptList = deptClientService.getDeptListByIds(deptIds);
            if (CollUtil.isNotEmpty(deptList)) {
                deptList.forEach(i -> deptMap.put(i.getId(), i));
            }
        }

        // 病种
        Map<String, StandardDiseaseBO> diseaseMap = new HashMap<>();
        if (CollUtil.isNotEmpty(diseaseIds)) {
            List<StandardDiseaseBO> diseaseList = standardDiseaseClientService.getList(diseaseIds);
            if (CollUtil.isNotEmpty(diseaseList)) {
                diseaseList.forEach(i -> diseaseMap.put(i.getId(), i));
            }
        }

        PageData<DoctorRelationByDoctorVO> data = new PageConverter<DoctorRelationByDoctorVO, DoctorRelationByDoctorVO>() {
            @Override
            public DoctorRelationByDoctorVO converter(DoctorRelationByDoctorVO vo) {

                vo.setDoctorInfo(doctorMap.get(vo.getDoctorId()));
                
                List<DoctorRelationInfo> l = map.get(vo.getDoctorId());

                List<String> deptIdList = new ArrayList<>();
                List<DeptBO> deptList = new ArrayList<>();
                
                List<String> diseaseIdList = new ArrayList<>();
                List<StandardDiseaseBO> diseaseList = new ArrayList<>();

                l.forEach(i -> {
                    if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType())) {
                        DeptBO dept = deptMap.get(i.getRelationId());
                        if (null != dept) {
                            deptIdList.add(dept.getId());
                            deptList.add(dept);
                        }
                    }
                    if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType())) {
                        StandardDiseaseBO disease = diseaseMap.get(i.getRelationId());
                        if (null != disease) {
                            diseaseIdList.add(disease.getId());
                            diseaseList.add(disease);
                        }
                    }
                });

                vo.setDeptIds(deptIdList);
                vo.setDeptInfoList(deptList);
                vo.setDiseaseIds(diseaseIdList);
                vo.setDiseaseInfoList(diseaseList);

                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public DoctorRelationByDoctorVO detailByDoctor(DoctorRelationGetByDoctorDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorRelationQueryDTO queryDTO = new DoctorRelationQueryDTO();
        queryDTO.setDoctorId(params.getDoctorId());
        List<DoctorRelationVO> list = this.list(queryDTO);

        if (CollUtil.isEmpty(list)) {
            return null;
        }

        Set<String> deptIds = new HashSet<>();
        Set<String> diseaseIds = new HashSet<>();
        list.forEach(i -> {
            if (DoctorRelationConstant.Type.DEPT.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                deptIds.add(i.getRelationId());
            }
            if (DoctorRelationConstant.Type.DISEASE.getCode().equals(i.getType()) && StrUtil.isNotBlank(i.getRelationId())) {
                diseaseIds.add(i.getRelationId());
            }
        });
        
        AdminUserBO doctor = adminUserClientService.getUserById(params.getDoctorId());

        // 部门信息
        List<DeptBO> deptList = null;
        if (CollUtil.isNotEmpty(deptIds)) {
            deptList = deptClientService.getDeptListByIds(deptIds);
        }

        // 病种
        List<StandardDiseaseBO> diseaseList = null;
        if (CollUtil.isNotEmpty(diseaseIds)) {
            diseaseList = standardDiseaseClientService.getList(diseaseIds);
        }

        DoctorRelationByDoctorVO vo = new DoctorRelationByDoctorVO();
        vo.setDoctorId(params.getDoctorId());
        vo.setDoctorInfo(doctor);
        vo.setDeptInfoList(deptList);
        vo.setDiseaseInfoList(diseaseList);

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveByDoctor(DoctorRelationAddBatchDTO params) {
        
        // 删除该医生下所有的数据
        doctorRelationInfoService.remove(new QueryWrapper<DoctorRelationInfo>()
            .eq("doctor_id", params.getDoctorId())
        );

        // 新增指定的数据
        List<DoctorRelationInfo> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(params.getDeptIds())) {
            params.getDeptIds().forEach(i -> {
                DoctorRelationInfo entity = new DoctorRelationInfo();
                entity.setId(DoctorPushTaskHelper.getDoctorRelationId());
                entity.setType(DoctorRelationConstant.Type.DEPT.getCode());
                entity.setDoctorId(params.getDoctorId());
                entity.setRelationId(i);
                entity.setStatus(YN.YES.getCode());
                entity.setRowsStatus(YN.YES.getCode());

                list.add(entity);
            });
        }
        if (CollUtil.isNotEmpty(params.getDiseaseIds())) {
            params.getDiseaseIds().forEach(i -> {
                DoctorRelationInfo entity = new DoctorRelationInfo();
                entity.setId(DoctorPushTaskHelper.getDoctorRelationId());
                entity.setType(DoctorRelationConstant.Type.DISEASE.getCode());
                entity.setDoctorId(params.getDoctorId());
                entity.setRelationId(i);
                entity.setStatus(YN.YES.getCode());
                entity.setRowsStatus(YN.YES.getCode());

                list.add(entity);
            });
        }
        
        doctorRelationInfoService.saveBatch(list);
    }

    @Override
    public void delByDoctor(DoctorRelationGetByDoctorDTO params) {
        // 删除该医生下所有的数据
        doctorRelationInfoService.remove(new QueryWrapper<DoctorRelationInfo>()
            .eq("doctor_id", params.getDoctorId())
        );
    }
    
}
