package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorFollwUpClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.bo.DoctorFollowUpBO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRemindConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRemindInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRemindInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRemindAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRemindQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRemindUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRemindVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorRemindBizService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorRemindBizServiceImpl implements DoctorRemindBizService {

    @Autowired
    private DoctorRemindInfoService doctorRemindInfoService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private DoctorFollwUpClientService doctorFollwUpClientService;
    
    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorRemindVO add(DoctorRemindAddDTO params) {
        log.info("DoctorRemindBizServiceImpl.add:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorRemindInfo entity = new DoctorRemindInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorRemindId());
        entity.setDoctorId(params.getDoctorId());
        entity.setBizOneType(params.getBizOneType());
        entity.setBizTwoType(params.getBizTwoType());
        entity.setBizId(params.getBizId());
        entity.setTitle(params.getTitle());
        entity.setContent(params.getContent());
        entity.setPushChannel(params.getPushChannel());
        entity.setPushTempId(params.getPushTempId());
        entity.setUrgentLevel(params.getUrgentLevel());
        entity.setPushDate(params.getPushDate());
        entity.setRemark(params.getRemark());
        entity.setPushConfig(params.getPushConfig());

        entity.setPushStatus(StrUtil.isNotBlank(params.getPushStatus()) ? params.getPushStatus() : YN.YES.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        doctorRemindInfoService.save(entity);

        return DoctorPushTaskHelper.getDoctorRemindVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<DoctorRemindAddDTO> params) {
        log.info("DoctorRemindBizServiceImpl.addBatch:params:>>>>>> {}", JSON.toJSONString(params));
        if (CollUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(params));

        List<DoctorRemindInfo> list = new ArrayList<>();

        params.forEach(i -> {
            DoctorRemindInfo entity = new DoctorRemindInfo();
            entity.setId(DoctorPushTaskHelper.getDoctorRemindId());
            entity.setDoctorId(i.getDoctorId());
            entity.setBizOneType(i.getBizOneType());
            entity.setBizTwoType(i.getBizTwoType());
            entity.setBizId(i.getBizId());
            entity.setTitle(i.getTitle());
            entity.setContent(i.getContent());
            entity.setPushChannel(i.getPushChannel());
            entity.setPushTempId(i.getPushTempId());
            entity.setUrgentLevel(i.getUrgentLevel());
            entity.setPushDate(i.getPushDate());
            entity.setRemark(i.getRemark());
            entity.setPushConfig(i.getPushConfig());

            entity.setPushStatus(StrUtil.isNotBlank(i.getPushStatus()) ? i.getPushStatus() : YN.YES.getCode());
            entity.setRowsStatus(YN.YES.getCode());

            list.add(entity);
        });

        doctorRemindInfoService.saveBatch(list);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateById(DoctorRemindUpdateDTO params) {
        log.info("DoctorRemindBizServiceImpl.update:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorRemindInfo entity = doctorRemindInfoService.getOne(new QueryWrapper<DoctorRemindInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        entity = new DoctorRemindInfo();
        entity.setId(params.getId());
        entity.setPushStatus(params.getPushStatus());
        entity.setPushDate(params.getPushDate());
        entity.setWxworkStatus(params.getWxworkStatus());
        entity.setSmsStatus(params.getSmsStatus());

        doctorRemindInfoService.updateById(entity);
    }

    @Override
    public PageData<DoctorRemindVO> page(DoctorRemindQueryDTO params) {
        IPage<DoctorRemindInfo> page = doctorRemindInfoService.page(new Page<DoctorRemindInfo>(params.getPage(), params.getRows()), new QueryWrapper<DoctorRemindInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
            .eq(StrUtil.isNotBlank(params.getBizOneType()), "biz_one_type", params.getBizOneType())
            .eq(StrUtil.isNotBlank(params.getBizTwoType()), "biz_two_type", params.getBizTwoType())
            .eq(StrUtil.isNotBlank(params.getPushStatus()), "push_status", params.getPushStatus())
            .ge(null != params.getStartDate(), "date", params.getStartDate())
            .le(null != params.getEndDate(), "date", params.getEndDate())
            .eq(StrUtil.isNotBlank(params.getUrgentLevel()), "urgent_level", params.getUrgentLevel())
            .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 根据业务分类及业务id查询对应数据信息
        List<String> patientIds = new ArrayList<>();
        List<String> followupIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            if (DoctorRemindConstant.BizOneType.Patient.getCode().equals(i.getBizOneType()) && StrUtil.isNotBlank(i.getBizId())) {
                patientIds.add(i.getBizId());
            }
            if (DoctorRemindConstant.BizOneType.FollowUp.getCode().equals(i.getBizOneType()) && StrUtil.isNotBlank(i.getBizId())) {
                followupIds.add(i.getBizId());
            }
        });
        // 患者
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(patientIds);
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
        }

        // 随访
        Map<String, DoctorFollowUpBO> followupMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(followupIds)) {
            List<DoctorFollowUpBO> followupList = doctorFollwUpClientService.listByIds(followupIds);
            followupList.forEach(i -> followupMap.put(i.getId(), i));
        }

        PageData<DoctorRemindVO> data = new PageConverter<DoctorRemindInfo, DoctorRemindVO>() {
            @Override
            public DoctorRemindVO converter(DoctorRemindInfo entity) {
                DoctorRemindVO vo = DoctorPushTaskHelper.getDoctorRemindVO(entity);
                if (DoctorRemindConstant.BizOneType.Patient.getCode().equals(vo.getBizOneType())) {
                    vo.setPatientInfo(patientMap.get(vo.getBizId()));
                }
                if (DoctorRemindConstant.BizOneType.FollowUp.getCode().equals(vo.getBizOneType())) {
                    vo.setFollowupInfo(followupMap.get(vo.getBizId()));
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public DoctorRemindVO getById(DoctorRemindQueryDTO params) {
        if (StrUtil.isBlank(params.getId())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        
        DoctorRemindInfo entity = doctorRemindInfoService.getOne(new QueryWrapper<DoctorRemindInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
        );
        if (null == entity) {
            return null;
        }

        DoctorRemindVO vo = DoctorPushTaskHelper.getDoctorRemindVO(entity);

        if (DoctorRemindConstant.BizOneType.Patient.getCode().equals(vo.getBizOneType()) && StrUtil.isNotBlank(vo.getBizId())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientId(vo.getBizId());
            PatientClientBO patient = patientClientService.getOneSimple(patientQueryDTO);
            vo.setPatientInfo(patient);
        }
        if (DoctorRemindConstant.BizOneType.FollowUp.getCode().equals(vo.getBizOneType()) && StrUtil.isNotBlank(vo.getBizId())) {
            DoctorFollowUpBO followup = doctorFollwUpClientService.getById(vo.getBizId());
            vo.setFollowupInfo(followup);
        }

        return vo;
    }

    @Override
    public List<DoctorRemindVO> list(DoctorRemindQueryDTO params) {
        List<DoctorRemindInfo> list = doctorRemindInfoService.list(new QueryWrapper<DoctorRemindInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
            .eq(StrUtil.isNotBlank(params.getBizOneType()), "biz_one_type", params.getBizOneType())
            .eq(StrUtil.isNotBlank(params.getBizTwoType()), "biz_two_type", params.getBizTwoType())
            .eq(StrUtil.isNotBlank(params.getPushStatus()), "status", params.getPushStatus())
            .ge(null != params.getStartDate(), "date", params.getStartDate())
            .le(null != params.getEndDate(), "date", params.getEndDate())
            .eq(StrUtil.isNotBlank(params.getUrgentLevel()), "urgent_level", params.getUrgentLevel())
            .orderByDesc("create_date")
        );

        return list.stream().map(DoctorPushTaskHelper::getDoctorRemindVO).collect(Collectors.toList());
    }
}
