package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.DoctorFollwUpClientService;
import com.xxh.cloud.healthy.serve.pushTask.client.api.bean.bo.DoctorFollowUpBO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorScheduleConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorScheduleInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorScheduleInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleRemoveDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorScheduleVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorScheduleBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorScheduleBizServiceImpl implements DoctorScheduleBizService {

    @Autowired
    private DoctorScheduleInfoService doctorScheduleInfoService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private DoctorFollwUpClientService doctorFollwUpClientService;
    
    @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorScheduleVO add(DoctorScheduleAddDTO params) {
        log.info("DoctorScheduleBizServiceImpl.add:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorScheduleInfo entity = new DoctorScheduleInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorScheduleId());
        entity.setDoctorId(params.getDoctorId());
        entity.setBizOneType(params.getBizOneType());
        entity.setBizTwoType(params.getBizTwoType());
        entity.setBizId(params.getBizId());
        entity.setTitle(params.getTitle());
        entity.setContent(params.getContent());
        entity.setUrgentLevel(params.getUrgentLevel());
        entity.setDate(params.getDate());

        entity.setStatus(StrUtil.isNotBlank(params.getStatus()) ? params.getStatus() : YN.YES.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        doctorScheduleInfoService.save(entity);

        return DoctorPushTaskHelper.getDoctorScheduleVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateById(DoctorScheduleUpdateDTO params) {
        log.info("DoctorScheduleBizServiceImpl.update:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorScheduleInfo entity = doctorScheduleInfoService.getOne(new QueryWrapper<DoctorScheduleInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        entity = new DoctorScheduleInfo();
        entity.setId(params.getId());
        entity.setDoctorId(params.getDoctorId());
        entity.setBizOneType(params.getBizOneType());
        entity.setBizTwoType(params.getBizTwoType());
        entity.setBizId(params.getBizId());
        entity.setTitle(params.getTitle());
        entity.setContent(params.getContent());
        entity.setStatus(params.getStatus());
        entity.setUrgentLevel(params.getUrgentLevel());
        entity.setDate(DateUtil.parseDateTime(params.getDate()));

        doctorScheduleInfoService.updateById(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(DoctorScheduleRemoveDTO params) {
        log.info("DoctorScheduleBizServiceImpl.remove:params:>>>>>> {}", JSON.toJSONString(params));

        doctorScheduleInfoService.update(new UpdateWrapper<DoctorScheduleInfo>()
            .set("rows_status", YN.NO.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
        );
    }

    @Override
    public PageData<DoctorScheduleVO> page(DoctorScheduleQueryDTO params) {
        IPage<DoctorScheduleInfo> page = doctorScheduleInfoService.page(new Page<DoctorScheduleInfo>(params.getPage(), params.getRows()), new QueryWrapper<DoctorScheduleInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
            .eq(StrUtil.isNotBlank(params.getBizOneType()), "biz_one_type", params.getBizOneType())
            .eq(StrUtil.isNotBlank(params.getBizTwoType()), "biz_two_type", params.getBizTwoType())
            .eq(StrUtil.isNotBlank(params.getBizId()), "biz_id", params.getBizId())
            .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
            .and(StrUtil.isNotBlank(params.getDate()), i -> {
                DateTime date = DateUtil.parseDate(params.getDate());
                String startDate = date.toDateStr();
                String endDate = DateUtil.offsetDay(date, 1).toDateStr();
                i.ge("date", startDate).lt("date", endDate);
            })
            .ge(StrUtil.isNotBlank(params.getStartDate()), "date", params.getStartDate())
            .le(StrUtil.isNotBlank(params.getEndDate()), "date", params.getEndDate())
            .eq(StrUtil.isNotBlank(params.getUrgentLevel()), "urgent_level", params.getUrgentLevel())
            .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 根据业务分类及业务id查询对应数据信息
        List<String> patientIds = new ArrayList<>();
        List<String> followupIds = new ArrayList<>();
        page.getRecords().forEach(i -> {
            if (DoctorScheduleConstant.BizOneType.Patient.getCode().equals(i.getBizOneType()) && StrUtil.isNotBlank(i.getBizId())) {
                patientIds.add(i.getBizId());
            }
            if (DoctorScheduleConstant.BizOneType.FollowUp.getCode().equals(i.getBizOneType()) && StrUtil.isNotBlank(i.getBizId())) {
                followupIds.add(i.getBizId());
            }
        });
        // 患者
        Map<String, PatientClientBO> patientMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(patientIds)) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientIds(patientIds);
            List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
            patientList.forEach(i -> patientMap.put(i.getPatientId(), i));
        }

        // 随访
        Map<String, DoctorFollowUpBO> followupMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(followupIds)) {
            List<DoctorFollowUpBO> followupList = doctorFollwUpClientService.listByIds(followupIds);
            followupList.forEach(i -> followupMap.put(i.getId(), i));
        }

        PageData<DoctorScheduleVO> data = new PageConverter<DoctorScheduleInfo, DoctorScheduleVO>() {
            @Override
            public DoctorScheduleVO converter(DoctorScheduleInfo entity) {
                DoctorScheduleVO vo = DoctorPushTaskHelper.getDoctorScheduleVO(entity);
                if (DoctorScheduleConstant.BizOneType.Patient.getCode().equals(vo.getBizOneType())) {
                    vo.setPatientInfo(patientMap.get(vo.getBizId()));
                }
                if (DoctorScheduleConstant.BizOneType.FollowUp.getCode().equals(vo.getBizOneType())) {
                    vo.setFollowupInfo(followupMap.get(vo.getBizId()));
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public DoctorScheduleVO getById(DoctorScheduleQueryDTO params) {
        if (StrUtil.isBlank(params.getId())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        
        DoctorScheduleInfo entity = doctorScheduleInfoService.getOne(new QueryWrapper<DoctorScheduleInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
        );
        if (null == entity) {
            return null;
        }

        DoctorScheduleVO vo = DoctorPushTaskHelper.getDoctorScheduleVO(entity);

        if (DoctorScheduleConstant.BizOneType.Patient.getCode().equals(vo.getBizOneType()) && StrUtil.isNotBlank(vo.getBizId())) {
            PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
            patientQueryDTO.setPatientId(vo.getBizId());
            PatientClientBO patient = patientClientService.getOneSimple(patientQueryDTO);
            vo.setPatientInfo(patient);
        }
        if (DoctorScheduleConstant.BizOneType.FollowUp.getCode().equals(vo.getBizOneType()) && StrUtil.isNotBlank(vo.getBizId())) {
            DoctorFollowUpBO followup = doctorFollwUpClientService.getById(vo.getBizId());
            vo.setFollowupInfo(followup);
        }

        return vo;
    }

    @Override
    public List<DoctorScheduleVO> list(DoctorScheduleQueryDTO params) {
        List<DoctorScheduleInfo> list = doctorScheduleInfoService.list(new QueryWrapper<DoctorScheduleInfo>()
            .eq("rows_status", YN.YES.getCode())
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
            .in(CollectionUtil.isNotEmpty(params.getDoctorIds()), "doctor_id", params.getDoctorIds())
            .eq(StrUtil.isNotBlank(params.getBizOneType()), "biz_one_type", params.getBizOneType())
            .eq(StrUtil.isNotBlank(params.getBizTwoType()), "biz_two_type", params.getBizTwoType())
            .eq(StrUtil.isNotBlank(params.getBizId()), "biz_id", params.getBizId())
            .eq(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus())
            .and(StrUtil.isNotBlank(params.getDate()), i -> {
                DateTime date = DateUtil.parseDate(params.getDate());
                String startDate = date.toDateStr();
                String endDate = DateUtil.offsetDay(date, 1).toDateStr();
                i.ge("date", startDate).lt("date", endDate);
            })
            .ge(StrUtil.isNotBlank(params.getStartDate()), "date", params.getStartDate())
            .le(StrUtil.isNotBlank(params.getEndDate()), "date", params.getEndDate())
            .eq(StrUtil.isNotBlank(params.getUrgentLevel()), "urgent_level", params.getUrgentLevel())
            .orderByDesc("create_date")
        );

        return list.stream().map(DoctorPushTaskHelper::getDoctorScheduleVO).collect(Collectors.toList());
    }
}
