package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.pushtask.client.api.PushTaskConfigClientService;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskConfigBO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorPushServeLogInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorRemindInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorTaskInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorPushServeLogInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorRemindInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorTaskInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.mapper.DoctorTaskQueryBizMapper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.mapper.PushTaskConfigRO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeHandlerBizService;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * zs20230924
 */
@Slf4j
@Service
public class DoctorTaskBizServiceImpl implements DoctorTaskBizService {

    @Autowired
    private DoctorTaskInfoService doctorTaskInfoService;
    @Autowired
    private DoctorRemindInfoService doctorRemindInfoService;
    @Autowired
    private PushTaskConfigClientService pushTaskConfigClientService;
    @Autowired
    private DoctorPushServeLogInfoService doctorPushServeLogInfoService;
    @Autowired
    private DoctorTaskExeHandlerBizService doctorTaskExeHandlerBizService;

    @Autowired
    private DoctorTaskQueryBizMapper doctorTaskQueryBizMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addTask(TaskDoctorInfoAddDTO params) {
        log.info("DoctorTaskBizServiceImpl:addTask:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        // TODO 同一个任务、同一医生、同一个患者、同一天只推送一次
        List<DoctorTaskInfo> existList = doctorTaskInfoService.list(new QueryWrapper<DoctorTaskInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("task_id", params.getTaskId())
                .eq("patient_id", params.getPatientId())
                .and(i -> {
                    params.getDoctorIds().forEach(j -> {
                        i.or().like("doctor_ids", "," + j + ",");
                    });
                })
                .ge("create_date", DateUtil.date().toDateStr())
        );
        if (CollectionUtil.isNotEmpty(existList)) {
            // 移除已经推送给医生的医生id
            existList.forEach(i -> {
                List<String> doctorIds = StrJoinSplitUtils.split2List(i.getDoctorIds());
                Iterator<String> it = params.getDoctorIds().iterator();
                while (it.hasNext()) {
                    String next = it.next();
                    if (doctorIds.contains(next)) {
                        it.remove();
                    }
                }
            });
        }
        if (CollectionUtil.isEmpty(params.getDoctorIds())) { // 移除了所有医生则就不需要生成任务
            return;
        }

        // 查询配置信息
        PushTaskConfigBO pushTaskConfigBo = pushTaskConfigClientService.getById(params.getTaskId());
        if (null == pushTaskConfigBo) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        DoctorTaskInfo entity = new DoctorTaskInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorPushTaskId());
        entity.setTaskId(params.getTaskId());
        entity.setTaskName(pushTaskConfigBo.getName());
        entity.setTaskType(pushTaskConfigBo.getType());
        entity.setRelType(params.getRelType());
        entity.setRelOneId(params.getRelOneId());
        entity.setRelTwoId(params.getRelTwoId());
        entity.setPatientId(params.getPatientId());
        entity.setPatientName(params.getPatientName());
        entity.setPatientSex(params.getPatientSex());
        entity.setUrgentLevel(pushTaskConfigBo.getUrgentLevel());
        entity.setStatus(DoctorTaskConstant.DoctorTaskStatus.Create.getCode());
        entity.setRowsStatus(YN.YES.getCode());

        String doctorIds = StrJoinSplitUtils.joinPrefix(params.getDoctorIds());
        entity.setDoctorIds(doctorIds);
        entity.setNotDoctorIds(doctorIds);
        log.info("DoctorTaskBizServiceImpl:addTask:entity:>>>>>> {}", JSON.toJSONString(entity));
        doctorTaskInfoService.save(entity);

        // 保存到执行表
        this.saveExe(params.getDoctorIds(), entity);

        // 保存到提醒
        if (YN.YES.getCode().equals(pushTaskConfigBo.getIsTaskPush())) {
            DateTime pushDate = DateUtil.date();
            List<DoctorRemindInfo> doctorRemindInfos = new ArrayList<>();
            params.getDoctorIds().forEach(i -> doctorRemindInfos.add(this.getPatientRemindAddDTO(entity, pushTaskConfigBo, i, pushDate)));
            if (CollectionUtil.isNotEmpty(doctorRemindInfos)) {
                doctorRemindInfoService.saveBatch(doctorRemindInfos);
            }
        }
    }

    @Override
    public void addTask2(TaskDoctor2AddDTO params) {
        //todo:

        List<PushTaskConfigRO> pushTaskConfigROList= doctorTaskQueryBizMapper.getPushTaskConfigList(
                new QueryWrapper<PushTaskConfigRO>()
                        //.eq("type","risk")
                        .eq("status",YN.YES.getCode())
                        .eq("rows_status",YN.YES.getCode())
                        .like("disease_ids",","+params.getDiseaseId()+",")
        );

        if(CollUtil.isEmpty(pushTaskConfigROList)){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"任务配置不存在");
        }
        PushTaskConfigRO pushTaskConfig= pushTaskConfigROList.get(0);


        DoctorTaskInfo entity = new DoctorTaskInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorPushTaskId());
        entity.setTaskId(pushTaskConfig.getId());
        entity.setTaskName(params.getTaskName());
        entity.setTaskType(pushTaskConfig.getType());

        entity.setPatientId(params.getPatientId());
        entity.setPatientName("测试患者");
        entity.setPatientSex("W");

        entity.setStatus(DoctorTaskConstant.DoctorTaskStatus.Create.getCode());
        entity.setRowsStatus(YN.YES.getCode());

        List<String> docIds=new ArrayList<>();
        docIds.add(params.getDoctorId());
        String doctorIds = StrJoinSplitUtils.joinPrefix(docIds);
        entity.setDoctorIds(doctorIds);
        entity.setNotDoctorIds(doctorIds);

        doctorTaskInfoService.save(entity);

        //,"yyyy-MM-dd HH:mm:ss"
        DateTime dtime=  DateUtil.parseDateTime(params.getAddTime());
        doctorTaskInfoService.update(new UpdateWrapper<DoctorTaskInfo>()
                .set("create_date",dtime)
                .set("update_date",dtime)
                .eq("id",entity.getId())
        );

    }

    @Override
    public void delTask2(String id) {
        DoctorTaskInfo entity =doctorTaskInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        doctorTaskInfoService.updateById(entity);
    }

    private void saveExe(List<String> doctorIds, DoctorTaskInfo entity) {
        doctorIds.forEach(i -> {
            // 医生任务执行信息
            DoctorPushServeAddDTO doctorPushServeAddDTO = new DoctorPushServeAddDTO();
            doctorPushServeAddDTO.setDoctorId(i);
            doctorPushServeAddDTO.setDoctorTaskId(entity.getId());
            doctorTaskExeHandlerBizService.doctorPushServeInitExe(doctorPushServeAddDTO);
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatchTask(List<TaskDoctorInfoAddDTO> params) {
        log.info("DoctorTaskBizServiceImpl:addBatchTask:params:>>>>>> {}", JSON.toJSONString(params));
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        DateTime pushDate = DateUtil.date();
        
        List<String> taskIds = params.stream().map(TaskDoctorInfoAddDTO::getTaskId).collect(Collectors.toList());
        Map<String, PushTaskConfigBO> taskConfigMap = pushTaskConfigClientService.getByIds(taskIds);

        List<DoctorTaskInfo> doctorTaskInfos = new ArrayList<>();
        List<DoctorRemindInfo> doctorRemindInfos = new ArrayList<>();
        params.forEach(item -> {
            // 配置信息
            PushTaskConfigBO pushTaskConfigBo = taskConfigMap.get(item.getTaskId());
            if (null == pushTaskConfigBo) {
                return;
            }

            DoctorTaskInfo entity = new DoctorTaskInfo();
            entity.setId(DoctorPushTaskHelper.getDoctorPushTaskId());
            entity.setTaskId(item.getTaskId());
            entity.setTaskName(pushTaskConfigBo.getName());
            entity.setTaskType(pushTaskConfigBo.getType());
            entity.setRelType(item.getRelType());
            entity.setRelOneId(item.getRelOneId());
            entity.setRelTwoId(item.getRelTwoId());
            entity.setPatientId(item.getPatientId());
            entity.setPatientName(item.getPatientName());
            entity.setPatientSex(item.getPatientSex());
            entity.setUrgentLevel(pushTaskConfigBo.getUrgentLevel());
            entity.setStatus(DoctorTaskConstant.DoctorTaskStatus.Create.getCode());
            entity.setRowsStatus(YN.YES.getCode());
            
            String doctorIds = StrJoinSplitUtils.join(item.getDoctorIds());
            entity.setDoctorIds(doctorIds);
            entity.setNotDoctorIds(doctorIds);
            
            doctorTaskInfos.add(entity);

            // 保存到提醒
            if (YN.YES.getCode().equals(pushTaskConfigBo.getIsTaskPush())) {
                item.getDoctorIds().forEach(i -> doctorRemindInfos.add(getPatientRemindAddDTO(entity, pushTaskConfigBo, i, pushDate)));
            }
        });
        doctorTaskInfoService.saveBatch(doctorTaskInfos);
        if (CollectionUtil.isNotEmpty(doctorRemindInfos)) {
            doctorRemindInfoService.saveBatch(doctorRemindInfos);
        }
    }

    /**
     * 提醒
     * @param
     * @param pushDate 推送时间
     * @return
     */
    private DoctorRemindInfo getPatientRemindAddDTO(DoctorTaskInfo doctorTaskInfo, PushTaskConfigBO config, String dockerId, DateTime pushDate) {
        DoctorRemindInfo doctorRemindInfo = new DoctorRemindInfo();
        doctorRemindInfo.setId(DoctorPushTaskHelper.getDoctorRemindId());
        doctorRemindInfo.setDoctorId(dockerId);
        doctorRemindInfo.setBizId(doctorTaskInfo.getId());
        doctorRemindInfo.setBizOneType(ObjectTypeEnum.OT_OneType_DoctorPushTask.getCode());
        if (PushTaskConstant.OpType.MDT.getCode().equals(config.getOpType())) { // 是否是mdt
            doctorRemindInfo.setBizOneType(PushTaskConstant.OpType.MDT.getCode());
            doctorRemindInfo.setBizTwoType(PushTaskConstant.OpType.MDT.getCode());
        }
        doctorRemindInfo.setTitle(config.getRemindTitle());
        doctorRemindInfo.setContent(config.getRemindDescribe()); // TODO FIXME: 使用html或markdown进行消息的配置
        doctorRemindInfo.setRemark("消息提醒");
        doctorRemindInfo.setPushChannel(config.getTaskPushWay());
        doctorRemindInfo.setPushTempId(config.getTaskPushConfig().getTemplateId());
        doctorRemindInfo.setPushConfig(JSON.toJSONString(config.getTaskPushConfig()));
        doctorRemindInfo.setUrgentLevel(config.getUrgentLevel());
        doctorRemindInfo.setPushDate(pushDate);
        doctorRemindInfo.setPushStatus(DoctorTaskConstant.PushStatus.Todo.getCode());
        doctorRemindInfo.setRowsStatus(YN.YES.getCode());

        return doctorRemindInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void cancelTask(TaskDoctorInfoCancelDTO params) {
        log.info("DoctorTaskBizServiceImpl:cancelTask:params:>>>>>> {}", JSON.toJSONString(params));

        List<DoctorTaskInfo> list = doctorTaskInfoService.list(new QueryWrapper<DoctorTaskInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .orderByDesc("create_date")
        );

        if (CollectionUtil.isEmpty(list)) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        List<String> ids = list.stream().map(DoctorTaskInfo::getId).collect(Collectors.toList());
        
        doctorTaskInfoService.update(new UpdateWrapper<DoctorTaskInfo>()
                // .set("rows_status", YN.NO.getCode())
                .set("status", DoctorTaskConstant.DoctorTaskStatus.Cancel.getCode())
                .in( "id", ids)
        );
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTask(TaskDoctorInfoUpdateDTO params) {
        log.info("DoctorTaskBizServiceImpl:updateTask:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorTaskInfo entity = doctorTaskInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        log.info("DoctorTaskBizServiceImpl:updateTask:entity:>>>>>> {}", JSON.toJSONString(entity));
        if (DoctorTaskConstant.DoctorTaskStatus.Done.getCode().equals(entity.getStatus()) 
                || DoctorTaskConstant.DoctorTaskStatus.Cancel.getCode().equals(entity.getStatus()) 
                || entity.getStatus().equals(params.getStatus())) {
            return;
        }
        
        // 处理已执行人，把当前执行人加入到已执行人中
        Set<String> yesDoctorIds = StrJoinSplitUtils.split2Set(entity.getYesDoctorIds());
        yesDoctorIds.addAll(params.getDoctorIds());

        // 处理未执行人，把当前执行人从未执行人中移除
        Set<String> notDoctorIds = StrJoinSplitUtils.split2Set(entity.getNotDoctorIds());
        List<String> notDoctorIdList = notDoctorIds.stream().filter(i -> !params.getDoctorIds().contains(i)).collect(Collectors.toList());

        log.info("DoctorTaskBizServiceImpl:updateTask:yesDoctorIds:>>>>>> {}", JSON.toJSONString(yesDoctorIds));
        log.info("DoctorTaskBizServiceImpl:updateTask:notDoctorIdList:>>>>>> {}", JSON.toJSONString(notDoctorIdList));

        // 更新已执行人与未执行人数据
        doctorTaskInfoService.update(new UpdateWrapper<DoctorTaskInfo>()
                .set("yes_doctor_ids", StrJoinSplitUtils.join(yesDoctorIds))
                .set("not_doctor_ids", CollectionUtil.isEmpty(notDoctorIdList) ? "" : StrJoinSplitUtils.join(notDoctorIdList))
                .set(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus()) // zs202311131910 状态为空则不更新状态
                .eq("id", params.getId())
        );
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void pushServeAdd(DoctorPushServeLogAddDTO params) {
        DoctorPushServeLogInfo entity = new DoctorPushServeLogInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorPushServeLogId());
        entity.setDoctorId(params.getDoctorId());
        entity.setPushTaskId(params.getPushTaskId());
        entity.setPatientId(params.getPatientId());
        entity.setPushServeId(params.getPushServeId());
        entity.setPatientPushServeId(params.getPatientPushServeId());
        entity.setType(params.getType());
        entity.setStatus(params.getStatus());
        doctorPushServeLogInfoService.save(entity);
    }

    @Override
    public void updateRemindStatusBatch(List<DoctorRemindUpdateDTO> params) {
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        List<String> ids = new ArrayList<>();
        Map<String, DoctorRemindUpdateDTO> map = new HashMap<>();
        params.forEach(i -> {
            ids.add(i.getId());
            map.put(i.getId(), i);
        });

        List<DoctorRemindInfo> list = doctorRemindInfoService.list(new QueryWrapper<DoctorRemindInfo>()
            .eq("rows_status", YN.YES.getCode())
            .in(CollectionUtil.isNotEmpty(ids), "id", ids)
        );

        list.forEach(i -> {
            DoctorRemindUpdateDTO dto = map.get(i.getId());
            if (null != dto) {
                i.setPushStatus(dto.getPushStatus());
                i.setPushDate(dto.getPushDate());
                i.setWxworkStatus(dto.getWxworkStatus());
                i.setSmsStatus(dto.getSmsStatus());
            }
        });

        if (CollectionUtil.isNotEmpty(list)) {
            doctorRemindInfoService.updateBatchById(list);
        }
    }

}
