package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskConfigBO;
import com.xxh.cloud.healthy.basic.pushtask.client.api.constant.PushTaskConstant;
import com.xxh.cloud.healthy.doctor.crm.client.api.DoctorPatientStatusMarkClientService;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorPatientStatusMarkBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.dto.DoctorPatientStatusMarkAddClientDTO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorPatientConstant;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorTaskExeInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorTaskExeInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorTaskExeDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorTaskGetDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorTaskManageDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskExeVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeHandlerBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskQueryBizService;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class DoctorTaskExeBizServiceImpl implements DoctorTaskExeBizService {

    @Autowired
    private DoctorTaskExeInfoService doctorTaskExeInfoService;
    @Autowired
    private DoctorTaskQueryBizService doctorTaskQueryBizService;
    @Autowired
    private DoctorTaskExeHandlerBizService doctorTaskExeHandlerBizService;
    @Autowired
    private DoctorPatientStatusMarkClientService doctorPatientStatusMarkClientService;

    //@Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorTaskExeVO add(DoctorTaskExeDTO params) {
        log.info("DoctorTaskExeBizServiceImpl:add:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorTaskExeInfo entity = new DoctorTaskExeInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorPushTaskId());
        entity.setDoctorId(params.getDoctorId());
        entity.setDoctorTaskId(params.getDoctorTaskId());
        entity.setRemark(params.getRemark());
        entity.setActionConfig(params.getActionConfig());
        entity.setStatus(params.getStatus());
        entity.setRowsStatus(YN.YES.getCode());
        log.info("DoctorTaskExeBizServiceImpl:add:entity:>>>>>> {}", JSON.toJSONString(entity));
        doctorTaskExeInfoService.save(entity);

        // 更新任务信息表的执行人信息
        // Set<String> dockerIds = new HashSet<>();
        // dockerIds.add(params.getDoctorId());

        // TaskDoctorInfoUpdateDTO taskInfo = new TaskDoctorInfoUpdateDTO();
        // taskInfo.setId(entity.getDoctorTaskId());
        // taskInfo.setDoctorIds(dockerIds);
        // log.info("DoctorTaskExeBizServiceImpl:add:taskInfo:>>>>>> {}", JSON.toJSONString(taskInfo));
        // doctorTaskBizService.updateTask(taskInfo);

        return DoctorPushTaskHelper.getDoctorTaskExeVO(entity);
    }

    @Override
    public void doctorPushServeAdd(DoctorPushServeAddDTO params) {
        log.info("1.执行医生任务推送患者服务>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        // 查询推送任务信息
        DoctorTaskGetDTO doctorTaskGetDTO = new DoctorTaskGetDTO();
        doctorTaskGetDTO.setId(params.getDoctorTaskId());
        doctorTaskGetDTO.setIsFinished(false); // 查询未完成的任务
        DoctorTaskVO doctorTaskVO = doctorTaskQueryBizService.getOne(doctorTaskGetDTO);
        log.info("2.查询医生任务详情>>>>>> {}", JSON.toJSONString(doctorTaskVO));
        if (null == doctorTaskVO || null == doctorTaskVO.getPushTaskConfig() || CollectionUtil.isEmpty(doctorTaskVO.getPushTaskConfig().getActionList())) {
            throw new BusinessException("未查询到推送任务信息");
        }

        // 判断任务执行状态
        if (DoctorTaskConstant.DoctorTaskStatus.Done.getCode().equals(doctorTaskVO.getStatus())) {
            throw new BusinessException("任务已结束");
        }
        if (DoctorTaskConstant.DoctorTaskStatus.Cancel.getCode().equals(doctorTaskVO.getStatus())) {
            throw new BusinessException("任务已取消");
        }
        
        // 通知引擎执行结果，获取任务状态
        // String status = this.getTaskStatusByEngine(doctorTaskVO, params.getDoctorId(), "success"); // 通知引擎执行结果，获取任务状态

        doctorTaskExeHandlerBizService.doctorPushServeAddExe(params, doctorTaskVO, null);
    }

    @Override
    public void doctorPushServeIgnore(DoctorPushServeAddDTO params) {
        ValidatorUtils.validateEntity(params);

        DoctorTaskGetDTO doctorTaskGetDTO = new DoctorTaskGetDTO();
        doctorTaskGetDTO.setId(params.getDoctorTaskId());
        doctorTaskGetDTO.setIsFinished(false); // 查询未完成的任务
        DoctorTaskVO doctorTaskVO = doctorTaskQueryBizService.getOne(doctorTaskGetDTO);
        if (null == doctorTaskVO) {
            throw new BusinessException("未查询到推送任务信息");
        }

        // 通知引擎执行结果，获取任务状态
        // String status = this.getTaskStatusByEngine(doctorTaskVO, params.getDoctorId(), "ignore"); // 通知引擎执行结果，获取任务状态

        doctorTaskExeHandlerBizService.doctorPushServeIgnoreExe(params, doctorTaskVO, null);
    }

    // zs20231126 不再调用引擎获取任务状态了
    // 调用引擎，获取任务执行状态
    // private String getTaskStatusByEngine(DoctorTaskVO doctorTaskVO, String doctorId, String handleStatus) {
    //     DoctorExeTaskDTO doctorExeTaskDTO = new DoctorExeTaskDTO();
    //     doctorExeTaskDTO.setNodeTaskId(doctorTaskVO.getRelOneId());
    //     doctorExeTaskDTO.setPatientId(doctorTaskVO.getRelTwoId());
    //     doctorExeTaskDTO.setOpUserId(doctorId);
    //     doctorExeTaskDTO.setHandleStatus(handleStatus);

    //     DoctorExeResultBO hDoctorExeResultBO = null;
    //     try {
    //         hDoctorExeResultBO = hdeFlowCallExeClientService.doctorExeTask(doctorExeTaskDTO);
    //     } catch (Exception e) {
    //         log.info("1.医生任务调用引擎异常<<<<<< {}", e.getMessage());
    //         // hdeFlowCallExeClientService.getDoctorTaskStatus(doctorTaskVO.getTaskId()); // 重新调用引擎逻辑
    //         e.printStackTrace();
    //         return DoctorTaskConstant.DoctorTaskStatus.Doing.getCode();
    //     }

    //     if (null != hDoctorExeResultBO) {
    //         if ("complete".equals(hDoctorExeResultBO.getNodeTaskStatus())) {
    //             return DoctorTaskConstant.DoctorTaskStatus.Done.getCode();
    //         } else if ("ing".equals(hDoctorExeResultBO.getNodeTaskStatus())) {
    //             return DoctorTaskConstant.DoctorTaskStatus.Doing.getCode();
    //         } else if ("kill".equals(hDoctorExeResultBO.getNodeTaskStatus())) {
    //             return DoctorTaskConstant.DoctorTaskStatus.Cancel.getCode();
    //         }
    //     }

    //     return DoctorTaskConstant.DoctorTaskStatus.Doing.getCode(); // FIXME 调用引擎，但是引擎依赖问题无法引入
    // }

    // 个案管理
    // @Transactional(rollbackFor = Exception.class)
    @Override
    public DoctorPatientStatusMarkBO doctorManageExe(DoctorTaskManageDTO params, String status) {
        ValidatorUtils.validateEntity(params);

        DoctorTaskGetDTO doctorTaskGetDTO = new DoctorTaskGetDTO();
        doctorTaskGetDTO.setId(params.getDoctorTaskId());
        doctorTaskGetDTO.setIsFinished(false); // 查询未完成的任务
        DoctorTaskVO doctorTaskVO = doctorTaskQueryBizService.getOne(doctorTaskGetDTO);
        if (null == doctorTaskVO) {
            throw new BusinessException("未查询到推送任务信息");
        }

        PushTaskConfigBO pushTaskConfigBO = doctorTaskVO.getPushTaskConfig();
        if (null == pushTaskConfigBO) {
            return null;
        }
        
        if (!PushTaskConstant.OpType.Manage.getCode().equals(pushTaskConfigBO.getOpType()) || CollectionUtil.isEmpty(pushTaskConfigBO.getDiseaseIds())) {
            return null;
        }
        
        // 更新推送服务状态
        if (CollectionUtil.isEmpty(doctorTaskVO.getYesDoctorIds()) || !doctorTaskVO.getYesDoctorIds().contains(params.getDoctorId())) {
            DoctorPushServeAddDTO addDTO = new DoctorPushServeAddDTO();
            addDTO.setDoctorId(params.getDoctorId());
            addDTO.setDoctorTaskId(params.getDoctorTaskId());
            addDTO.setDeptId(params.getDeptId());
            if (DoctorStatusMarkConstant.Status.Normal.getCode().equals(status)) {
                doctorTaskExeHandlerBizService.doctorPushServeAddExe(addDTO, doctorTaskVO, null);
            } else {
                doctorTaskExeHandlerBizService.doctorPushServeIgnoreExe(addDTO, doctorTaskVO, null);
            }
        }

        // 加入个案管理
        DoctorPatientStatusMarkAddClientDTO addClientDTO = new DoctorPatientStatusMarkAddClientDTO();
        addClientDTO.setDoctorId(params.getDoctorId());
        addClientDTO.setPatientId(doctorTaskVO.getPatientId());
        addClientDTO.setDiseaseId(pushTaskConfigBO.getDiseaseIds().get(0)); // 由于历史原因，任务病种为列表，现在是单个病种
        addClientDTO.setSourceType(DoctorPatientConstant.SourceType.Doctor_Task.getCode());
        addClientDTO.setSourceId(doctorTaskVO.getId());
        addClientDTO.setStatus(status);
        addClientDTO.setOpUserId(params.getDoctorId());
        addClientDTO.setTargetId(pushTaskConfigBO.getTargetId());
        addClientDTO.setType(pushTaskConfigBO.getManageType());
        addClientDTO.setAddTime(doctorTaskVO.getCreateDate());
        return doctorPatientStatusMarkClientService.add(addClientDTO);
    }
}
