package com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.PushWayEnum;
import com.xxh.cloud.healthy.basic.pushtask.client.api.bean.bos.PushTaskActionConfigBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientPushServeClientBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeAddClientDTO;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorTaskConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorPushServeLogInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorTaskExeInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorTaskInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorPushServeLogInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorTaskExeInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorTaskInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorConsultationIgnoreDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorConsultationInitiateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeLogAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorTaskExeDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.TaskDoctorInfoUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskExeVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.helper.DoctorPushTaskHelper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeHandlerBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorTaskExeHandlerBizServiceImpl implements DoctorTaskExeHandlerBizService {
    
    @Autowired
    private DoctorTaskExeInfoService doctorTaskExeInfoService;
    @Autowired
    private PatientPushServeClientService patientPushServeClientService;
    @Autowired
    private DoctorTaskInfoService doctorTaskInfoService;
    @Autowired
    private DoctorPushServeLogInfoService doctorPushServeLogInfoService;
    
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void doctorPushServeInitExe(DoctorPushServeAddDTO params) {
        // 医生任务执行信息
        DoctorTaskExeDTO doctorTaskExeDTO = new DoctorTaskExeDTO();
        doctorTaskExeDTO.setDoctorId(params.getDoctorId());
        doctorTaskExeDTO.setDoctorTaskId(params.getDoctorTaskId());
        doctorTaskExeDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.TODO.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeAdd:doctorPushServeInitExe:>>>>>> {}", JSON.toJSONString(doctorTaskExeDTO));
        this.add(doctorTaskExeDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void doctorPushServeAddExe(DoctorPushServeAddDTO params, DoctorTaskVO doctorTaskVO, String status) {
        if (null != params.getAction()) {
            for(PushTaskActionConfigBO item : doctorTaskVO.getPushTaskConfig().getActionList()) {
                // 微信
                if (params.getAction().equals(item.getType())) {
                    if (null != item.getDataConfig()) {
                        PatientPushServeAddClientDTO patientPushServeAddDTO = new PatientPushServeAddClientDTO();
                        patientPushServeAddDTO.setPushServeId(String.valueOf(item.getDataConfig().get("push_id")));
                        patientPushServeAddDTO.setPatientId(doctorTaskVO.getPatientId());
                        patientPushServeAddDTO.setBenchmarkDateValue(DateTime.now().toString());
                        patientPushServeAddDTO.setOpUser(params.getDoctorId());
                        // 患者推送服务
                        PatientPushServeClientBO patientPushServeBO = patientPushServeClientService.add(patientPushServeAddDTO);
                        if (null == patientPushServeBO) {
                            throw new BusinessException("患者推送服务失败");
                        }

                        // 记录日志
                        DoctorPushServeLogAddDTO doctorPushServeLogAddDTO = new DoctorPushServeLogAddDTO();
                        doctorPushServeLogAddDTO.setType(PushWayEnum.PCW_WX.getCode());
                        doctorPushServeLogAddDTO.setDeptId(params.getDeptId());
                        doctorPushServeLogAddDTO.setDoctorId(params.getDoctorId());
                        doctorPushServeLogAddDTO.setPushTaskId(params.getDoctorTaskId());
                        doctorPushServeLogAddDTO.setPatientId(doctorTaskVO.getPatientId());
                        doctorPushServeLogAddDTO.setPushServeId(patientPushServeBO.getPushServeId());
                        doctorPushServeLogAddDTO.setPatientPushServeId(patientPushServeBO.getId());
                        doctorPushServeLogAddDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Send.getCode());
                        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeAdd:doctorPushServeLogAddDTO:>>>>>> {}", JSON.toJSONString(doctorPushServeLogAddDTO));
                        this.pushServeAdd(doctorPushServeLogAddDTO);

                    }
                }
                // TODO 短信
                // TODO ai电话

            }
        }

        // 医生任务执行信息
        DoctorTaskExeDTO doctorTaskExeDTO = new DoctorTaskExeDTO();
        doctorTaskExeDTO.setDoctorId(params.getDoctorId());
        doctorTaskExeDTO.setDoctorTaskId(doctorTaskVO.getId());
        doctorTaskExeDTO.setRemark(params.getRemark());
        doctorTaskExeDTO.setActionConfig(params.getActionConfig());
        doctorTaskExeDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Send.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeAdd:doctorTaskExeDTO:>>>>>> {}", JSON.toJSONString(doctorTaskExeDTO));
        this.add(doctorTaskExeDTO);

        // 更新医生推送服务状态
        Set<String> doctorIds = new HashSet<>();
        doctorIds.add(params.getDoctorId());
        TaskDoctorInfoUpdateDTO taskDoctorInfoUpdateDTO = new TaskDoctorInfoUpdateDTO();
        taskDoctorInfoUpdateDTO.setId(params.getDoctorTaskId());
        taskDoctorInfoUpdateDTO.setDoctorIds(doctorIds);
        taskDoctorInfoUpdateDTO.setStatus(status);
        this.updateTask(taskDoctorInfoUpdateDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void doctorPushServeIgnoreExe(DoctorPushServeAddDTO params, DoctorTaskVO doctorTaskVO, String status) {

        // 医生任务执行信息
        DoctorTaskExeDTO doctorTaskExeDTO = new DoctorTaskExeDTO();
        doctorTaskExeDTO.setDoctorId(params.getDoctorId());
        doctorTaskExeDTO.setDoctorTaskId(params.getDoctorTaskId());
        doctorTaskExeDTO.setRemark(params.getRemark());
        doctorTaskExeDTO.setActionConfig(params.getActionConfig());
        doctorTaskExeDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Ignore.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeIgnore:doctorTaskExeDTO:>>>>>> {}", JSON.toJSONString(doctorTaskExeDTO));
        this.add(doctorTaskExeDTO);

        // 直接记录日志
        DoctorPushServeLogAddDTO doctorPushServeLogAddDTO = new DoctorPushServeLogAddDTO();
        doctorPushServeLogAddDTO.setDeptId(params.getDeptId());
        doctorPushServeLogAddDTO.setDoctorId(params.getDoctorId());
        doctorPushServeLogAddDTO.setPushTaskId(params.getDoctorTaskId());
        doctorPushServeLogAddDTO.setPatientId(doctorTaskVO.getPatientId());
        doctorPushServeLogAddDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Ignore.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeIgnore:doctorPushServeLogAddDTO:>>>>>> {}", JSON.toJSONString(doctorPushServeLogAddDTO));
        this.pushServeAdd(doctorPushServeLogAddDTO);

        // 更新医生推送服务状态
        Set<String> doctorIds = new HashSet<>();
        doctorIds.add(params.getDoctorId());
        TaskDoctorInfoUpdateDTO taskDoctorInfoUpdateDTO = new TaskDoctorInfoUpdateDTO();
        taskDoctorInfoUpdateDTO.setId(params.getDoctorTaskId());
        taskDoctorInfoUpdateDTO.setDoctorIds(doctorIds);
        taskDoctorInfoUpdateDTO.setStatus(status);
        log.info("DoctorTaskQueryBizServiceImpl:doctorPushServeIgnore:taskDoctorInfoUpdateDTO:>>>>>> {}", JSON.toJSONString(taskDoctorInfoUpdateDTO));
        this.updateTask(taskDoctorInfoUpdateDTO);
    }

    private DoctorTaskExeVO add(DoctorTaskExeDTO params) {
        log.info("DoctorTaskExeBizServiceImpl:add:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        // zs20240117 查询是否存在待执行的数据，存在则更新，不存在则新增
        DoctorTaskExeInfo entity = doctorTaskExeInfoService.getOne(new QueryWrapper<DoctorTaskExeInfo>()
            .eq("doctor_id", params.getDoctorId())
            .eq("doctor_task_id", params.getDoctorTaskId())
            .eq("status", DoctorTaskConstant.DoctorTaskExeStatus.TODO.getCode())
        );
        if (null != entity) {
            entity.setStatus(params.getStatus());
            entity.setRemark(params.getRemark());
            entity.setActionConfig(params.getActionConfig());
            entity.setStatus(params.getStatus());
            entity.setRowsStatus(YN.YES.getCode());
            doctorTaskExeInfoService.updateById(entity);
        } else {
            entity = new DoctorTaskExeInfo();
            entity.setId(DoctorPushTaskHelper.getDoctorPushTaskId());
            entity.setDoctorId(params.getDoctorId());
            entity.setDoctorTaskId(params.getDoctorTaskId());
            entity.setRemark(params.getRemark());
            entity.setActionConfig(params.getActionConfig());
            entity.setStatus(params.getStatus());
            entity.setRowsStatus(YN.YES.getCode());
            log.info("DoctorTaskExeBizServiceImpl:add:entity:>>>>>> {}", JSON.toJSONString(entity));
            doctorTaskExeInfoService.save(entity);
        }

        // 更新任务信息表的执行人信息
        // Set<String> dockerIds = new HashSet<>();
        // dockerIds.add(params.getDoctorId());

        // TaskDoctorInfoUpdateDTO taskInfo = new TaskDoctorInfoUpdateDTO();
        // taskInfo.setId(entity.getDoctorTaskId());
        // taskInfo.setDoctorIds(dockerIds);
        // log.info("DoctorTaskExeBizServiceImpl:add:taskInfo:>>>>>> {}", JSON.toJSONString(taskInfo));
        // doctorTaskBizService.updateTask(taskInfo);

        return DoctorPushTaskHelper.getDoctorTaskExeVO(entity);
    }

    private void updateTask(TaskDoctorInfoUpdateDTO params) {
        log.info("DoctorTaskBizServiceImpl:updateTask:params:>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        DoctorTaskInfo entity = doctorTaskInfoService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        log.info("DoctorTaskBizServiceImpl:updateTask:entity:>>>>>> {}", JSON.toJSONString(entity));
        if (DoctorTaskConstant.DoctorTaskStatus.Done.getCode().equals(entity.getStatus()) 
                || DoctorTaskConstant.DoctorTaskStatus.Cancel.getCode().equals(entity.getStatus()) 
                || entity.getStatus().equals(params.getStatus())) {
            return;
        }
        
        // 处理已执行人，把当前执行人加入到已执行人中
        Set<String> yesDoctorIds = StrJoinSplitUtils.split2Set(entity.getYesDoctorIds());
        yesDoctorIds.addAll(params.getDoctorIds());

        // 处理未执行人，把当前执行人从未执行人中移除
        Set<String> notDoctorIds = StrJoinSplitUtils.split2Set(entity.getNotDoctorIds());
        List<String> notDoctorIdList = notDoctorIds.stream().filter(i -> !params.getDoctorIds().contains(i)).collect(Collectors.toList());

        log.info("DoctorTaskBizServiceImpl:updateTask:yesDoctorIds:>>>>>> {}", JSON.toJSONString(yesDoctorIds));
        log.info("DoctorTaskBizServiceImpl:updateTask:notDoctorIdList:>>>>>> {}", JSON.toJSONString(notDoctorIdList));

        // zs202311261907 若没有传过来状态则本地判断状态，医生都执行完则执行结束，否则为执行中
        // TODO 若只需要一个医生执行完就完成整个任务可以在配置中加字段来判断，暂时可以通过其他医生忽略来实现不需要全部医生都执行就完成任务的操作
        if (StrUtil.isBlank(params.getStatus())) {
            params.setStatus(DoctorTaskConstant.DoctorTaskStatus.Doing.getCode());
            if (CollectionUtil.isEmpty(notDoctorIdList)) {
                params.setStatus(DoctorTaskConstant.DoctorTaskStatus.Done.getCode());
            }
        }

        // 更新已执行人与未执行人数据
        doctorTaskInfoService.update(new UpdateWrapper<DoctorTaskInfo>()
                .set("yes_doctor_ids", StrJoinSplitUtils.join(yesDoctorIds))
                .set("not_doctor_ids", CollectionUtil.isEmpty(notDoctorIdList) ? "" : StrJoinSplitUtils.join(notDoctorIdList))
                .set(StrUtil.isNotBlank(params.getStatus()), "status", params.getStatus()) // zs202311131910 状态为空则不更新状态
                .eq("id", params.getId())
        );
    }
    
    private void pushServeAdd(DoctorPushServeLogAddDTO params) {
        DoctorPushServeLogInfo entity = new DoctorPushServeLogInfo();
        entity.setId(DoctorPushTaskHelper.getDoctorPushServeLogId());
        entity.setDoctorId(params.getDoctorId());
        entity.setPushTaskId(params.getPushTaskId());
        entity.setPatientId(params.getPatientId());
        entity.setPushServeId(params.getPushServeId());
        entity.setPatientPushServeId(params.getPatientPushServeId());
        entity.setType(params.getType());
        entity.setStatus(params.getStatus());
        doctorPushServeLogInfoService.save(entity);
    }

    @Override
    public void consultationInitiate(DoctorConsultationInitiateDTO params) {
        // 医生任务执行信息
        DoctorTaskExeDTO doctorTaskExeDTO = new DoctorTaskExeDTO();
        doctorTaskExeDTO.setDoctorId(params.getDoctorId());
        doctorTaskExeDTO.setDoctorTaskId(params.getDoctorTaskId());
        doctorTaskExeDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Send.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:consultationInitiate:doctorTaskExeDTO:>>>>>> {}", JSON.toJSONString(doctorTaskExeDTO));
        this.add(doctorTaskExeDTO);

        // 更新医生推送服务状态
        Set<String> doctorIds = new HashSet<>();
        doctorIds.add(params.getDoctorId());
        TaskDoctorInfoUpdateDTO taskDoctorInfoUpdateDTO = new TaskDoctorInfoUpdateDTO();
        taskDoctorInfoUpdateDTO.setId(params.getDoctorTaskId());
        taskDoctorInfoUpdateDTO.setDoctorIds(doctorIds);
        // taskDoctorInfoUpdateDTO.setStatus(status);
        this.updateTask(taskDoctorInfoUpdateDTO);
    }

    @Override
    public void consultationIgnore(DoctorConsultationIgnoreDTO params) {
        
        // 医生任务执行信息
        DoctorTaskExeDTO doctorTaskExeDTO = new DoctorTaskExeDTO();
        doctorTaskExeDTO.setDoctorId(params.getDoctorId());
        doctorTaskExeDTO.setDoctorTaskId(params.getDoctorTaskId());
        doctorTaskExeDTO.setStatus(DoctorTaskConstant.DoctorTaskExeStatus.Ignore.getCode());
        log.info("DoctorTaskQueryBizServiceImpl:consultationIgnore:doctorTaskExeDTO:>>>>>> {}", JSON.toJSONString(doctorTaskExeDTO));
        this.add(doctorTaskExeDTO);

        // 更新医生推送服务状态
        Set<String> doctorIds = new HashSet<>();
        doctorIds.add(params.getDoctorId());
        TaskDoctorInfoUpdateDTO taskDoctorInfoUpdateDTO = new TaskDoctorInfoUpdateDTO();
        taskDoctorInfoUpdateDTO.setId(params.getDoctorTaskId());
        taskDoctorInfoUpdateDTO.setDoctorIds(doctorIds);
        // taskDoctorInfoUpdateDTO.setStatus(status);
        log.info("DoctorTaskQueryBizServiceImpl:consultationIgnore:taskDoctorInfoUpdateDTO:>>>>>> {}", JSON.toJSONString(taskDoctorInfoUpdateDTO));
        this.updateTask(taskDoctorInfoUpdateDTO);
    }

}
