package com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.schedule;

import java.util.Date;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.service.DoctorFollowUpBizTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class DoctorFollowUpTask {

    @Value("${xx.returnVisitDuring:15}")
    private Integer returnVisitDuring;

    @Autowired
    private DoctorFollowUpBizTaskService doctorFollowUpBizTaskService;
    
    // 扫描出院及出院小结信息，根据出院随访时间，自动给医生生成随访信息
    @XxlJob("doctorFollowUpJobHandler")
    public void doctorFollowUpJobHandler() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("自动随访生成开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("自动随访生成开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetDay(now, -2); // 默认两天
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        doctorFollowUpBizTaskService.createFollowUpTaskExecute(startTime, endTime);

        XxlJobHelper.log("自动随访生成结束 <<<<<< ");
        log.info("自动随访生成结束 <<<<<< ");
    }

    // 扫描有随访标记的标签的住院会诊的患者，根据会诊的随访时间进行随访
    // @Deprecated // doctorConsultationFollowUpTask
    @XxlJob("doctorConsultationFollowUpJobHandler")
    public void doctorConsultationFollowUpJobHandler() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("住院会诊患者自动随访生成开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("住院会诊患者自动随访生成开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = null; // 默认查询所有未生成过随访数据的会诊，传时间则不判断是否已经生成过随访（跑历史）
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        doctorFollowUpBizTaskService.createConsultationFollowUpTaskExecute(startTime, endTime);

        XxlJobHelper.log("住院会诊患者自动随访生成结束 <<<<<< ");
        log.info("住院会诊患者自动随访生成结束 <<<<<< ");
    }

    // 扫描有随访标记的标签的会诊的出院患者，根据会诊的随访时间进行随访
    // @Deprecated // doctorConsultationFollowUpTask
    @XxlJob("doctorConsultationOutFollowUpJobHandler")
    public void doctorConsultationOutFollowUpJobHandler() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("住院会诊患者出院自动随访生成开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("住院会诊患者出院自动随访生成开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = null; // 默认查询所有未生成过随访数据的会诊，传时间则不判断是否已经生成过随访（跑历史）
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        doctorFollowUpBizTaskService.createConsultationOutFollowUpTaskExecute(startTime, endTime);

        XxlJobHelper.log("住院会诊患者出院自动随访生成结束 <<<<<< ");
        log.info("住院会诊患者出院自动随访生成结束 <<<<<< ");
    }

    // 扫描有随访时间的会诊进行随访（有随访标记的标签的会诊也会处理）
    @XxlJob("doctorConsultationFollowUpTask")
    public void doctorConsultationFollowUpTask() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"dayNum\": \"2\",\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("会诊患者自动随访生成开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("会诊患者自动随访生成开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetDay(now, - 2); // 默认查询所有未生成过随访数据的会诊，传时间则不判断是否已经生成过随访（跑历史）
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String dayNum = jobParamMap.get("dayNum");
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(dayNum)) {
                    startTime = DateUtil.offsetDay(now, - Integer.valueOf(dayNum));
                } else {
                    if (StrUtil.isNotBlank(start)) {
                        startTime = DateUtil.parseDateTime(start);
                        if (StrUtil.isNotBlank(end)) {
                            endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                        }
                    }
                }
            }
        }

        doctorFollowUpBizTaskService.createConsultationFollowUpTaskExe(startTime, endTime);

        XxlJobHelper.log("会诊患者自动随访生成结束 <<<<<< ");
        log.info("会诊患者自动随访生成结束 <<<<<< ");
    }

    // 随访过期处理
    @XxlJob("doctorFollowUpTimeout")
    public void doctorFollowUpTimeout() throws Exception {
        log.info("随访过期处理开始 >>>>>> ");
        XxlJobHelper.log("随访过期处理开始 >>>>>> ");

        doctorFollowUpBizTaskService.doctorFollowUpTimeout();

        XxlJobHelper.log("随访过期处理结束 <<<<<< ");
        log.info("随访过期处理结束 <<<<<< ");
    }

    // 扫描待随访的随访数据并生成相应的复诊
    @XxlJob("doctorFollowUpReturnVisitTask")
    public void doctorFollowUpReturnVisitTask() throws Exception {
        // dayNum与startTime、endTime二选一生效
        // String jobParam = "{\"startTime\": \"2024-07-10 10:10:10\",\"endTime\": \"2024-07-12 10:10:10\"}";
        String jobParam = XxlJobHelper.getJobParam();

        log.info("自动随访复诊生成开始 >>>>>> {}", jobParam);
        XxlJobHelper.log("自动随访复诊生成开始 >>>>>> {}", jobParam);

        Date now = DateUtil.date();
        Date startTime = DateUtil.offsetDay(now, -returnVisitDuring); // 默认
        Date endTime = null;

        if (StrUtil.isNotBlank(jobParam)) {
            Map<String, String> jobParamMap = JSONObject.parseObject(jobParam, new TypeReference<Map<String, String>>() {});
            if (CollUtil.isNotEmpty(jobParamMap)) {
                String start = jobParamMap.get("startTime");
                String end = jobParamMap.get("endTime");

                if (StrUtil.isNotBlank(start)) {
                    startTime = DateUtil.parseDateTime(start);
                    if (StrUtil.isNotBlank(end)) {
                        endTime = DateUtil.offsetDay(DateUtil.parseDateTime(end), 1);  // 不能单独设置结束时间
                    }
                }
            }
        }

        doctorFollowUpBizTaskService.createFollowUpReturnVisitTaskExecute(startTime, endTime, returnVisitDuring);

        XxlJobHelper.log("自动随访复诊生成结束 <<<<<< ");
        log.info("自动随访复诊生成结束 <<<<<< ");
    }

    // 扫描随访计划数据统计
    @XxlJob("doctorFollowUpOneDayStatTask")
    public void doctorFollowUpOneDayStatTask() throws Exception {
        
        // 2024-11-01,2024-11-10
        String jobParam = XxlJobHelper.getJobParam();

        Date startDate = null;
        Date endDate = null;
        if (StrUtil.isNotBlank(jobParam)) {
            String[] jobParamArray = jobParam.split(",");
            startDate = DateUtil.parseDate(jobParamArray[0].trim());
            if (jobParamArray.length == 2) {
                endDate = DateUtil.parseDate(jobParamArray[1].trim());
            }
        }

        doctorFollowUpBizTaskService.doctorFollowUpOneDayStatTask(startDate, endDate);

        XxlJobHelper.log("自动随访复诊生成结束 <<<<<< ");
        log.info("自动随访复诊生成结束 <<<<<< ");
    }

}
