package com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.schedule;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.service.DoctorTaskBizTaskService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import lombok.extern.slf4j.Slf4j;

import java.util.Date;

@Slf4j
@Component
public class DoctorPushTaskTask {

    @Autowired
    private DoctorTaskBizTaskService doctorTaskBizTaskService;
    @Value("${xx.remind.dateStart: 08}")
    private String remindDateStart;
    @Value("${xx.remind.dateEnd: 17}")
    private String remindDateEnd;

    // 扫描提醒表推送提醒
    @XxlJob("doctorPushTaskJobHandler")
    public void doctorPushTaskJobHandler() throws Exception {
        log.info("推送提醒开始 >>>>>> ");
        XxlJobHelper.log("推送提醒开始 >>>>>> ");

        if (!noRemindCheck()) {
            log.info("非推送时间段, 推送提醒结束 <<<<<<");
            XxlJobHelper.log("非推送时间段, 推送提醒结束 <<<<<<");
            return;
        }
        doctorTaskBizTaskService.pushDoctorTaskExecute();

        XxlJobHelper.log("推送提醒结束 <<<<<< ");
        log.info("推送提醒结束 <<<<<< ");
    }
    
    // 扫描提醒表企业微信推送mdt提醒
    @XxlJob("doctorPushMDTJobHandler")
    public void doctorPushMDTJobHandler() throws Exception {
        log.info("推送mdt提醒开始 >>>>>> ");
        XxlJobHelper.log("推送mdt提醒开始 >>>>>> ");

        if (!noRemindCheck()) {
            log.info("非推送时间段, 推送mdt提醒结束 <<<<<<");
            XxlJobHelper.log("非推送时间段, 推送mdt提醒结束 <<<<<<");
            return;
        }
        doctorTaskBizTaskService.pushDoctorMDTExecute();

        XxlJobHelper.log("推送mdt提醒结束 <<<<<< ");
        log.info("推送mdt提醒结束 <<<<<< ");
    }

    // 扫描提醒表推送发起会诊申请提醒
    @XxlJob("doctorPushConsultaionLaunchApplyJobHandler")
    public void doctorPushConsultaionLaunchApplyJobHandler() throws Exception {
        log.info("推送发起会诊申请提醒开始 >>>>>> ");
        XxlJobHelper.log("推送发起会诊申请提醒开始 >>>>>> ");

        if (!noRemindCheck()) {
            log.info("非推送时间段, 推送发起会诊申请提醒结束 <<<<<<");
            XxlJobHelper.log("非推送时间段, 推送发起会诊申请提醒结束 <<<<<<");
            return;
        }

        doctorTaskBizTaskService.pushDoctorConsultationLaunchApplyExecute();

        doctorTaskBizTaskService.pushDoctorConsultationJoinExecute(); // 参加会诊提醒

        XxlJobHelper.log("推送发起会诊申请提醒结束 <<<<<< ");
        log.info("推送发起会诊申请提醒结束 <<<<<< ");
    }

    // 推送标签的病种负责人
    @XxlJob("pushDoctorPatientLabelRemindJobHandler")
    public void pushDoctorPatientLabelRemindJobHandler() throws Exception {
        log.info("推送标签的病种负责人开始 >>>>>> ");
        XxlJobHelper.log("推送标签的病种负责人开始 >>>>>> ");

        if (!noRemindCheck()) {
            log.info("非推送时间段, 推送标签的病种负责人结束 >>>>>> ");
            XxlJobHelper.log("非推送时间段, 推送标签的病种负责人结束 <<<<<<");
            return;
        }

        doctorTaskBizTaskService.pushDoctorPatientLabelRemind(); // 通知标签的病种负责人

        XxlJobHelper.log("推送标签的病种负责人结束 <<<<<< ");
        log.info("推送标签的病种负责人结束 <<<<<< ");
    }

    // 扫描提醒表推送随访提醒
    @XxlJob("doctorPushFollowUpJobHandler")
    public void doctorPushFollowUpJobHandler() throws Exception {
        log.info("推送随访提醒开始 >>>>>> ");
        XxlJobHelper.log("推送随访提醒开始 >>>>>> ");

        if (!noRemindCheck()) {
            log.info("非推送时间段, 推送随访提醒结束 <<<<<<");
            XxlJobHelper.log("非推送时间段, 推送随访提醒结束 <<<<<<");
            return;
        }

        doctorTaskBizTaskService.doctorPushFollowUpJobHandler();

        XxlJobHelper.log("推送随访提醒结束 <<<<<< ");
        log.info("推送随访提醒结束 <<<<<< ");
    }

    // 扫描提醒表推送关注用户就诊提醒
    @XxlJob("doctorPushFollowJobHandler")
    public void doctorPushFollowJobHandler() throws Exception {
        log.info("推送关注用户就诊提醒开始 >>>>>> ");
        XxlJobHelper.log("推送关注用户就诊提醒开始 >>>>>> ");

        doctorTaskBizTaskService.doctorPushFollowJobHandler();

        XxlJobHelper.log("推送关注用户就诊提醒结束 <<<<<< ");
        log.info("推送关注用户就诊提醒结束 <<<<<< ");
    }

    // 免打扰校验
    private boolean noRemindCheck(){
        Date now = new Date();
        Date start = DateUtil.parse(StrUtil.format(DateUtil.formatDate(now) + " {}:00:00",remindDateStart));
        Date end = DateUtil.parse(StrUtil.format(DateUtil.formatDate(now) + " {}:00:00",remindDateEnd));
        if (now.compareTo(start) > 0 && now.compareTo(end) < 0) {
            return true;
        } else {
            return false;
        }
    }


}
