package com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.common.util.FollowUpRegexUtil;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.constant.LabelConsultationFollowUpBaseDateTypeEnum;
import com.xxh.cloud.healthy.data.client.api.DwdDataPatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDiagnosisReportClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeContentClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientDiagnosisDataClientDTO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientLastDataQueryDTO;
import com.xxh.cloud.healthy.mdt.api.constant.ConsultationLaunchApplyConstant;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientReturnVisitClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.bo.PatientReturnVisitBO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientReturnVisitAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientReturnVisitQueryClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientReturnVisitUpdateClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.constant.PatientReturnVisitConstant;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorFollowUpConstant;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorRelationConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.entity.DoctorFollowUpInfo;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.base.service.DoctorFollowUpInfoService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpOneDayAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.ConsultationApplyAdviceVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRelationVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.mapper.DoctorFollowUpQueryBizMapper;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorRelationBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorFollowUpBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorFollowUpOneDayBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.service.DoctorFollowUpBizTaskService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class DoctorFollowUpBizTaskServiceImpl implements DoctorFollowUpBizTaskService {

    @Autowired
    private DwdDataPatientClientService dwdDataPatientClientService;
    @Autowired
    private DoctorFollowUpBizService doctorFollowUpBizService;
    @Autowired
    private DoctorRelationBizService doctorRelationBizService;
    @Autowired
    private DoctorFollowUpQueryBizMapper doctorFollowUpQueryBizMapper;
    @Autowired
    private LabelClientService labelClientService;
    @Autowired
    private DoctorFollowUpInfoService doctorFollowUpInfoService;
    @Autowired
    private PatientReturnVisitClientService patientReturnVisitClientService;
    @Autowired
    private DoctorFollowUpOneDayBizService doctorFollowUpOneDayBizService;

    @Override
    public void createFollowUpTaskExecute(Date startTime, Date endTime) {

        log.info("1.查询一段时间内的患者最新出院信息");
        // 查询一段时间内的患者最新有随访时间的出院信息
        PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
        patientLastDataQueryDTO.setStartDate(startTime);
        if (null != endTime) {
            patientLastDataQueryDTO.setEndDate(endTime);
        }
        List<PatientDischargeRecordClientBO> patientLastDataList = dwdDataPatientClientService.listPatientOutpatientLastDateWitiSFSJInfo(patientLastDataQueryDTO);

        if (CollUtil.isEmpty(patientLastDataList)) {
            return;
        }

        List<String> deptIds = new ArrayList<>();
        List<String> visitIds = new ArrayList<>();
        patientLastDataList.forEach(i -> {
            deptIds.add(i.getOutDepartmentCodeStandard());
            visitIds.add(i.getVisitId());
        });

        log.info("2.查询诊断");
        // 诊断
        Map<String, Set<String>> diagnosisReportMap = new HashMap<>(); // key:visitId  value:diseaseIds
        if (CollUtil.isNotEmpty(visitIds)) {
            PatientDiagnosisDataClientDTO patientDiagnosisDataClientDTO = new PatientDiagnosisDataClientDTO();
            patientDiagnosisDataClientDTO.setVisitIds(visitIds);
            // patientDiagnosisDataClientDTO.setIndex("1");
            patientDiagnosisDataClientDTO.setSceneType("cyzd");
            List<PatientDiagnosisReportClientBO> diagnosisReportList = dwdDataPatientClientService.listPatientDiagnosisReportInfo(patientDiagnosisDataClientDTO);
            if (CollUtil.isNotEmpty(diagnosisReportList)) {
                diagnosisReportList.forEach(i -> {
                    Set<String> diagnosisReports = diagnosisReportMap.get(i.getVisitId());
                    if (null == diagnosisReports) {
                        diagnosisReports = new HashSet<>();
                        diagnosisReportMap.put(i.getVisitId(), diagnosisReports);
                    }
                    diagnosisReports.add(i.getDiagnosisCodeStandard());
                });

            }
        }

        log.info("3.查询需要随访的运营助理");
        // 查询需要随访的运营助理
        DoctorRelationQueryDTO doctorRelationQueryDTO = new DoctorRelationQueryDTO();
        doctorRelationQueryDTO.setType(DoctorRelationConstant.Type.DEPT.getCode());
        doctorRelationQueryDTO.setRelationIds(deptIds);
        List<DoctorRelationVO> doctorRelationVOs = doctorRelationBizService.simpleList(doctorRelationQueryDTO);
        if (CollUtil.isEmpty(doctorRelationVOs)) {
            return;
        }

        Map<String, Set<String>> deptRelationMap = new HashMap<>();  // key:deptId  value:doctorIds
        Set<String> doctorSet = new HashSet<>(); // 查询的运营助理所管理病种的参数
        doctorRelationVOs.forEach(i -> {
            Set<String> doctors = deptRelationMap.get(i.getRelationId());
            if (null == doctors) {
                doctors = new HashSet<>();
                deptRelationMap.put(i.getRelationId(), doctors);
            }
            doctors.add(i.getDoctorId());

            doctorSet.addAll(doctors);
        });

        Map<String, Set<String>> diseaseRelationMap = new HashMap<>(); // key:doctorId  value:diseaseIds
        log.info("4.查询需要随访的运营助理所管理的病种");
        DoctorRelationQueryDTO doctorDiseaseQueryDTO = new DoctorRelationQueryDTO();
        doctorDiseaseQueryDTO.setType(DoctorRelationConstant.Type.DISEASE.getCode());
        doctorDiseaseQueryDTO.setDoctorIds(new ArrayList<>(doctorSet));
        List<DoctorRelationVO> doctorDiseaseVOs = doctorRelationBizService.simpleList(doctorDiseaseQueryDTO);
        if (CollUtil.isNotEmpty(doctorDiseaseVOs)) {
            doctorDiseaseVOs.forEach(i -> {
                Set<String> diseases = diseaseRelationMap.get(i.getDoctorId());
                if (null == diseases) {
                    diseases = new HashSet<>();
                    diseaseRelationMap.put(i.getDoctorId(), diseases);
                }
                diseases.add(i.getRelationId());
            });
        }


        log.info("5.生成随访信息并设置提醒");
        // 生成随访信息并设置提醒
        List<DoctorFollowUpAddDTO> followUpSaveList = new ArrayList<>();
        List<DoctorFollowUpAddDTO> followUpFailSaveList = new ArrayList<>(); // 未解析出随访时间的数据
        patientLastDataList.forEach(i -> {
            Set<String> doctorIds = deptRelationMap.get(i.getOutDepartmentCodeStandard());
            if (CollUtil.isEmpty(doctorIds)) { // 没设置科室的运营助理不会生成随访
                return;
            }

            // 设置了病种的运营助理需要判断病种是否满足
            Set<String> diagnosisDiseaseIds =  diagnosisReportMap.get(i.getVisitId());

            PatientDischargeContentClientBO content = i.getContent();
            if (null != content) {
                for (String doctorId : doctorIds) {
                    Boolean flag = false;
                    Set<String> doctorDiseases = diseaseRelationMap.get(doctorId);
                    if (CollUtil.isNotEmpty(doctorDiseases)) {
                        if (CollUtil.isNotEmpty(diagnosisDiseaseIds)) {
                            for (String cd : diagnosisDiseaseIds) {
                                if (doctorDiseases.contains(cd)) {
                                    flag = true;
                                    break;
                                }
                            }
                        }
                    } else { // 没病种则直接匹配科室
                        flag = true;
                    }

                    if (!flag) {
                        continue;
                    }

                    if (StrUtil.isNotBlank(content.getFollowupDateStandard())) {
                        DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                        followUpAddDTO.setType(DoctorFollowUpConstant.Type.CY.getCode());
                        followUpAddDTO.setPatientId(i.getPatientId());
                        // followUpAddDTO.setInpatientId(i.getVisitId());
                        followUpAddDTO.setVisitId(i.getVisitId());
                        followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(diagnosisDiseaseIds));
                        followUpAddDTO.setDoctorId(doctorId); // 运营助理
                        // followUpAddDTO.setContent(content.getContentStandard()); // 出院小结，太大不保存
                        followUpAddDTO.setRemark(content.getFollowupDateRaw());
                        followUpAddDTO.setFollowDate(DateUtil.parseDateTime(content.getFollowupDateStandard()));
                        followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                        followUpAddDTO.setSourceDeptId(i.getOutDepartmentCodeStandard());
                        followUpAddDTO.setSourceDate(i.getLeaveDate());
                        
                        followUpSaveList.add(followUpAddDTO);
                    } else {
                        DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                        followUpAddDTO.setType(DoctorFollowUpConstant.Type.CY.getCode());
                        followUpAddDTO.setPatientId(i.getPatientId());
                        // followUpAddDTO.setInpatientId(i.getVisitId());
                        followUpAddDTO.setVisitId(i.getVisitId());
                        followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(diagnosisDiseaseIds));
                        followUpAddDTO.setDoctorId(doctorId); // 运营助理
                        // followUpAddDTO.setContent(content.getContentStandard()); // 出院小结，太大不保存
                        followUpAddDTO.setRemark(content.getFollowupDateRaw());
                        // followUpAddDTO.setFollowDate(null);  // 没有解析出随访时间
                        followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                        followUpAddDTO.setSourceDeptId(i.getOutDepartmentCodeStandard());
                        followUpAddDTO.setSourceDate(i.getLeaveDate());
                        
                        followUpFailSaveList.add(followUpAddDTO);
                    }
                }
            }
        });

        if (CollUtil.isNotEmpty(followUpSaveList)) {
            log.info("5.保存随访及提醒信息 >>>>>> {}", followUpSaveList.size());
            doctorFollowUpBizService.saveAndLaunchBatch(followUpSaveList);
        }
        if (CollUtil.isNotEmpty(followUpFailSaveList)) {
            log.info("6.保存未解析出随访时间的随访信息 >>>>>> {}", followUpFailSaveList.size());
            doctorFollowUpBizService.addBatch(followUpFailSaveList);
        }
    }

    @Override
    public void createConsultationFollowUpTaskExecute(Date startTime, Date endTime) {

        // 查询已完成会诊的会诊申请信息
        log.info("1.查询已完成会诊的会诊申请信息");
        List<ConsultationApplyAdviceVO> consultationMapList = doctorFollowUpQueryBizMapper.followupHzList(new QueryWrapper<ConsultationApplyAdviceVO>()
            .eq("a.type", ConsultationLaunchApplyConstant.Type.Task.getCode())
            .isNotNull("a.consultation_id")
            // .eq("a.status", ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode())
            .isNotNull("a.finish_time")
            .ge(null != startTime, "a.apply_time", startTime)
            .lt(null != endTime, "a.apply_time", endTime)
            .isNotNull("b.followup")
            .and(i -> i.isNull("c.consultation_follow_up_base_date_type").or().eq("c.consultation_follow_up_base_date_type", "").or().eq("c.consultation_follow_up_base_date_type", LabelConsultationFollowUpBaseDateTypeEnum.HZ.getCode()))
            .isNull(null == startTime, "d.consultation_id") // 默认只执行没有随访过的会诊，传时间则表示重新执行所有数据，不用关心是否重复，随访逻辑有验重
        );
        if (CollUtil.isEmpty(consultationMapList)) {
            return;
        }

        // 整理出必要的查询条件
        // List<String> consultationIds = new ArrayList<>(); // 会诊id  用于查会诊
        List<String> labelIds = new ArrayList<>(); // 标签id  用于查标签
        List<String> deptIds = new ArrayList<>(); // 会诊结论科室 用于查询随访医生（科室相关）
        Map<String, String> consultationLabelMap = new HashMap<>(); // 会诊标签关联  用于会诊与病种关联
        consultationMapList.forEach(i -> {
            String consultationId = i.getConsultationId();
            String consultationLabelId = i.getConsultationLabelId();
            // consultationIds.add(consultationId);
            labelIds.add(consultationLabelId);
            deptIds.add(i.getMdtDeptCode());
            consultationLabelMap.put(consultationId, consultationLabelId);
        });

        // if (CollUtil.isEmpty(consultationLabelMap)) {
        //     return;
        // }
        
        // log.info("2.查询一段时间内的患者最新可随访的会诊信息");
        // ConsultationQueryClientDTO consultationQueryClientDTO = new ConsultationQueryClientDTO();
        // consultationQueryClientDTO.setIds(consultationIds);
        // List<ConsultationBO> consultationList = consultationClientService.list(consultationQueryClientDTO);
        // if (CollUtil.isEmpty(consultationList)) {
        //     return;
        // }

        log.info("3.查询科室对应的需要随访的运营助理");
        // 查询需要随访的运营助理
        DoctorRelationQueryDTO doctorDeptQueryDTO = new DoctorRelationQueryDTO();
        doctorDeptQueryDTO.setRelationIds(deptIds);
        doctorDeptQueryDTO.setType(DoctorRelationConstant.Type.DEPT.getCode());
        List<DoctorRelationVO> doctorDeptVOs = doctorRelationBizService.simpleList(doctorDeptQueryDTO);
        if (CollUtil.isEmpty(doctorDeptVOs)) {
            return;
        }
        Map<String, Set<String>> deptRelationMap = new HashMap<>(); // key:deptId value:doctorIds 科室-运营助理关联
        Set<String> doctorSet = new HashSet<>(); // 查询的运营助理所管理病种的参数
        doctorDeptVOs.forEach(i -> {
            Set<String> d = deptRelationMap.get(i.getRelationId());
            if (null == d) {
                d = new HashSet<>();
                deptRelationMap.put(i.getRelationId(), d);
            }
            d.add(i.getDoctorId());

            doctorSet.addAll(d);
        });

        log.info("4.查询可随访会诊标签病种信息");
        Map<String, List<String>> labelDiseaseMap = new HashMap<>(); // key:labelId value:diseaseIds 标签-病种关联  用于会诊与病种关联
        if (CollUtil.isNotEmpty(labelIds)) {
            List<LabelBO> labelList = labelClientService.getLabelListByIds(labelIds);
            // 整理出会诊标签病种信息
            if (CollUtil.isNotEmpty(labelList)) {
                labelList.forEach(i -> {
                    labelDiseaseMap.put(i.getId(), i.getDiseaseIds());
                });
            }
        }
        // 整理出会诊病种信息
        Map<String, Set<String>> consultationDiseaseMap = new HashMap<>(); // key:consultaionId value:diseaseIds 会诊病种关联
        consultationLabelMap.forEach((k, v) -> {
            List<String> diseases = labelDiseaseMap.get(v);
            if (CollUtil.isNotEmpty(diseases)) {
                Set<String> l = consultationDiseaseMap.get(k);
                if (null == l) {
                    l = new HashSet<>();
                    consultationDiseaseMap.put(k, l);
                }
                l.addAll(diseases);
            }
        });

        log.info("5.查询病种对应的需要随访的运营助理");
        //根据科室对应的运营助理，再查询对应病种的运营助理
        Map<String, Set<String>> diseaseRelationMap = new HashMap<>(); // key:doctorId  value:diseaseIds
        if (CollUtil.isNotEmpty(doctorSet)) {
            DoctorRelationQueryDTO doctorDiseaseQueryDTO = new DoctorRelationQueryDTO();
            doctorDiseaseQueryDTO.setDoctorIds(new ArrayList<>(doctorSet));
            doctorDiseaseQueryDTO.setType(DoctorRelationConstant.Type.DISEASE.getCode());
            List<DoctorRelationVO> doctorDiseaseVOs = doctorRelationBizService.simpleList(doctorDiseaseQueryDTO);
            if (CollUtil.isNotEmpty(doctorDiseaseVOs)) {
                doctorDiseaseVOs.forEach(i -> {
                    Set<String> d = diseaseRelationMap.get(i.getDoctorId());
                    if (null == d) {
                        d = new HashSet<>();
                        diseaseRelationMap.put(i.getDoctorId(), d);
                    }
                    d.add(i.getRelationId());
                });
            }
        }

        log.info("6.生成随访信息并设置提醒");
        // 生成随访信息并设置提醒
        List<DoctorFollowUpAddDTO> followUpSaveList = new ArrayList<>();
        List<DoctorFollowUpAddDTO> followUpFailSaveList = new ArrayList<>(); // 未解析出随访时间的数据
        consultationMapList.forEach(i -> {

            // 处理随访时间，循环会诊结论获取其中一个有会诊随访时间的数据并解析
            String followUpTimeRaw = i.getFollowup();
            Date baseDate = i.getSubmitTime();
            String hisAdviceDoctorDeptId = i.getMdtDeptCode(); // 提交会诊结论医生科室
            String followupContent = i.getSubmitContent();

            // 原始随访时间为空说明不是需要随访的会诊
            if (StrUtil.isBlank(followUpTimeRaw)) {
                return;
            }

            String followUpTime = null == followUpTimeRaw ? null : FollowUpRegexUtil.getFollowupDateStandard(baseDate, followUpTimeRaw);

            log.info("followUpTime >>>> {}", followUpTime);
            Set<String> doctorIds = deptRelationMap.get(hisAdviceDoctorDeptId);
            if (CollUtil.isEmpty(doctorIds)) {
                return;
            }

            Set<String> consultationDiseases = consultationDiseaseMap.get(i.getConsultationId());

            for (String doctorId : doctorIds) {
                Boolean flag = false;
                Set<String> doctorDiseases = diseaseRelationMap.get(doctorId);
                if (CollUtil.isNotEmpty(doctorDiseases)) {
                    if (CollUtil.isNotEmpty(consultationDiseases)) {
                        for (String cd : consultationDiseases) {
                            if (doctorDiseases.contains(cd)) {
                                flag = true;
                                break;
                            }
                        }
                    }
                } else { // 没病种则直接匹配科室
                    flag = true;
                }

                if (!flag) {
                    continue;
                }

                if (StrUtil.isNotBlank(followUpTime)) {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(DoctorFollowUpConstant.Type.HZ.getCode());
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(consultationDiseases));
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark("会诊后：" + followUpTimeRaw);
                    followUpAddDTO.setFollowDate(DateUtil.offsetHour(DateUtil.parseDate(followUpTime), 9));
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpSaveList.add(followUpAddDTO);
                } else {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(DoctorFollowUpConstant.Type.HZ.getCode());
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(consultationDiseases));
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark("会诊后：" + followUpTimeRaw);
                    // followUpAddDTO.setFollowDate(null);  // 没有解析出随访时间
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpFailSaveList.add(followUpAddDTO);
                }
            }
        });

        if (CollUtil.isNotEmpty(followUpSaveList)) {
            log.info("5.保存随访及提醒信息 >>>>>> {}", JSON.toJSONString(followUpSaveList));
            doctorFollowUpBizService.saveAndLaunchBatch(followUpSaveList);
        }
        if (CollUtil.isNotEmpty(followUpFailSaveList)) {
            log.info("6.保存未解析出随访时间的随访信息 >>>>>> {}", JSON.toJSONString(followUpFailSaveList));
            doctorFollowUpBizService.addBatch(followUpFailSaveList);
        }
    }

    @Override
    public void createConsultationOutFollowUpTaskExecute(Date startTime, Date endTime) {

        // 查询已完成会诊的会诊申请信息
        log.info("1.查询已完成会诊且是出院随访标签标记的会诊申请信息");
        List<ConsultationApplyAdviceVO> consultationMapList = doctorFollowUpQueryBizMapper.followupHzList(new QueryWrapper<ConsultationApplyAdviceVO>()
            .eq("a.type", ConsultationLaunchApplyConstant.Type.Task.getCode())
            .isNotNull("a.consultation_id")
            // .eq("a.status", ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode())
            .isNotNull("a.finish_time")
            .ge(null != startTime, "a.apply_time", startTime)
            .lt(null != endTime, "a.apply_time", endTime)
            .isNotNull("b.followup")
            .eq("c.consultation_follow_up_base_date_type", LabelConsultationFollowUpBaseDateTypeEnum.CY.getCode())
            .isNull(null == startTime, "d.consultation_id") // 默认只执行没有随访过的会诊，传时间则表示重新执行所有数据，不用关心是否重复，随访逻辑有验重
        );
        if (CollUtil.isEmpty(consultationMapList)) {
            return;
        }
        
        List<String> patientIds = new ArrayList<>(); // 患者编号
        List<String> visitIds = new ArrayList<>(); // 住院编号
        consultationMapList.forEach(i -> {
            patientIds.add(i.getPatientId());
            visitIds.add(i.getVisitId());
        });

        log.info("2.过滤出已经出院的患者");
        PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
        patientLastDataQueryDTO.setPatientIds(patientIds);
        patientLastDataQueryDTO.setVisitIds(visitIds);
        List<PatientInpatientRecordClientBO> inpatientRecordList = dwdDataPatientClientService.listPatientInpatientLastDateAndOut(patientLastDataQueryDTO);
        if (CollUtil.isEmpty(inpatientRecordList)) {
            return;
        }
        Map<String, PatientInpatientRecordClientBO> inpatientRecordMap = new HashMap<>();
        inpatientRecordList.forEach(i -> inpatientRecordMap.put(i.getVisitId(), i));

        // 整理出必要的查询条件
        log.info("3.整理出必要的查询条件");
        List<ConsultationApplyAdviceVO> consultationMapListResult = new ArrayList<>();
        // List<String> consultationIds = new ArrayList<>(); // 会诊id  用于查会诊
        List<String> labelIds = new ArrayList<>(); // 标签id  用于查标签
        Map<String, String> consultationLabelMap = new HashMap<>(); // 会诊标签关联  用于会诊与病种关联
        List<String> deptIds = new ArrayList<>(); // 会诊结论科室 用于查询随访医生（科室相关）
        Map<String, Date> consultationAndLeaveDateMap = new HashMap<>();
        consultationMapList.forEach(i -> {
            if (i.getPatientId().startsWith("TB")) {
                return;
            }
            // 只处理出院的数据
            PatientInpatientRecordClientBO bo = inpatientRecordMap.get(i.getVisitId());
            if (null == bo || !YN.YES.getCode().equals(bo.getIsOut())) {
                return;
            }

            consultationMapListResult.add(i);

            deptIds.add(i.getMdtDeptCode());

            String consultationId = i.getConsultationId();
            String consultationLabelId = i.getConsultationLabelId();
            // consultationIds.add(consultationId);
            labelIds.add(consultationLabelId);
            consultationLabelMap.put(consultationId, consultationLabelId);
            consultationAndLeaveDateMap.put(consultationId, bo.getOutDate());
        });

        if (CollUtil.isEmpty(consultationMapListResult)) {
            return;
        }
        
        // log.info("4.查询患者最新可随访的会诊信息");
        // ConsultationQueryClientDTO consultationQueryClientDTO = new ConsultationQueryClientDTO();
        // consultationQueryClientDTO.setIds(consultationIds);
        // List<ConsultationBO> consultationList = consultationClientService.list(consultationQueryClientDTO);
        // if (CollUtil.isEmpty(consultationList)) {
        //     return;
        // }

        log.info("5.查询科室对应的需要随访的运营助理");
        // 查询需要随访的运营助理
        DoctorRelationQueryDTO doctorDeptQueryDTO = new DoctorRelationQueryDTO();
        doctorDeptQueryDTO.setRelationIds(deptIds);
        doctorDeptQueryDTO.setType(DoctorRelationConstant.Type.DEPT.getCode());
        List<DoctorRelationVO> doctorDeptVOs = doctorRelationBizService.simpleList(doctorDeptQueryDTO);
        if (CollUtil.isEmpty(doctorDeptVOs)) {
            return;
        }
        Map<String, Set<String>> deptRelationMap = new HashMap<>(); // 科室-运营助理关联
        Set<String> doctorSet = new HashSet<>(); // 查询的运营助理所管理病种的参数
        doctorDeptVOs.forEach(i -> {
            Set<String> d = deptRelationMap.get(i.getRelationId());
            if (null == d) {
                d = new HashSet<>();
                deptRelationMap.put(i.getRelationId(), d);
            }
            d.add(i.getDoctorId());

            doctorSet.addAll(d);
        });

        log.info("6.查询可随访会诊标签病种信息");
        Map<String, List<String>> labelDiseaseMap = new HashMap<>(); // 标签-病种关联  用于会诊与病种关联
        if (CollUtil.isNotEmpty(labelIds)) {
            List<LabelBO> labelList = labelClientService.getLabelListByIds(labelIds);
            // 整理出会诊标签病种信息
            if (CollUtil.isNotEmpty(labelList)) {
                labelList.forEach(i -> {
                    labelDiseaseMap.put(i.getId(), i.getDiseaseIds());
                });
            }
        }
        // 整理出会诊病种信息
        Map<String, Set<String>> consultationDiseaseMap = new HashMap<>(); // 会诊病种关联
        consultationLabelMap.forEach((k, v) -> {
            List<String> diseases = labelDiseaseMap.get(v);
            if (CollUtil.isNotEmpty(diseases)) {
                Set<String> l = consultationDiseaseMap.get(k);
                if (null == l) {
                    l = new HashSet<>();
                    consultationDiseaseMap.put(k, l);
                }
                l.addAll(diseases);
            }
        });

        log.info("7.查询病种对应的需要随访的运营助理");
        //根据科室对应的运营助理，再查询对应病种的运营助理
        Map<String, Set<String>> diseaseRelationMap = new HashMap<>();
        if (CollUtil.isNotEmpty(doctorSet)) {
            DoctorRelationQueryDTO doctorDiseaseQueryDTO = new DoctorRelationQueryDTO();
            doctorDiseaseQueryDTO.setDoctorIds(new ArrayList<>(doctorSet));
            doctorDiseaseQueryDTO.setType(DoctorRelationConstant.Type.DISEASE.getCode());
            List<DoctorRelationVO> doctorDiseaseVOs = doctorRelationBizService.simpleList(doctorDiseaseQueryDTO);
            if (CollUtil.isNotEmpty(doctorDiseaseVOs)) {
                doctorDiseaseVOs.forEach(i -> {
                    Set<String> d = diseaseRelationMap.get(i.getDoctorId());
                    if (null == d) {
                        d = new HashSet<>();
                        diseaseRelationMap.put(i.getDoctorId(), d);
                    }
                    d.add(i.getRelationId());
                });
            }
        }

        log.info("8.生成随访信息并设置提醒");
        // 生成随访信息并设置提醒
        Set<String> dataCheck = new HashSet<>(); // 用于需要保存的数据去重

        List<DoctorFollowUpAddDTO> followUpSaveList = new ArrayList<>();
        List<DoctorFollowUpAddDTO> followUpFailSaveList = new ArrayList<>(); // 未解析出随访时间的数据
        consultationMapListResult.forEach(i -> {

            // 处理随访时间，循环会诊结论获取其中一个有会诊随访时间的数据并解析
            String followUpTimeRaw = i.getFollowup();
            Date baseDate = consultationAndLeaveDateMap.get(i.getConsultationId());
            String hisAdviceDoctorDeptId = i.getMdtDeptCode(); // 提交会诊结论医生科室
            String followupContent = i.getSubmitContent();

            // 原始随访时间为空说明不是需要随访的会诊
            if (StrUtil.isBlank(followUpTimeRaw)) {
                return;
            }

            String followUpTime = null == followUpTimeRaw ? null : FollowUpRegexUtil.getFollowupDateStandard(baseDate, followUpTimeRaw);
            log.info("followUpTime <<<<<< {}", followUpTime);

            Set<String> doctorIds = deptRelationMap.get(hisAdviceDoctorDeptId);
            log.info("会诊科室对应运营助理 <<<<<< {}", JSON.toJSONString(doctorIds));
            if (CollUtil.isEmpty(doctorIds)) {
                return;
            }

            Set<String> consultationDiseases = consultationDiseaseMap.get(i.getConsultationId());
            log.info("会诊科室对应病种（会诊标签病种） <<<<<< {}", JSON.toJSONString(consultationDiseases));

            for (String doctorId : doctorIds) {
                if (dataCheck.contains(i.getPatientId() + "-" + doctorId + "-" + i.getInpatientId())) {
                    continue;
                }

                Boolean flag = false;
                Set<String> doctorDiseases = diseaseRelationMap.get(doctorId);
                log.info("运营助理管理病种 <<<<<< {}", JSON.toJSONString(doctorDiseases));
                if (CollUtil.isNotEmpty(doctorDiseases)) {
                    if (CollUtil.isNotEmpty(consultationDiseases)) {
                        for (String cd : consultationDiseases) {
                            if (doctorDiseases.contains(cd)) {
                                flag = true;
                                break;
                            }
                        }
                    }
                } else { // 没病种则直接匹配科室
                    flag = true;
                }

                if (!flag) {
                    continue;
                }

                if (StrUtil.isNotBlank(followUpTime)) {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(DoctorFollowUpConstant.Type.HZCY.getCode());
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(consultationDiseases));
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark("出院后：" + followUpTimeRaw);
                    followUpAddDTO.setFollowDate(DateUtil.offsetHour(DateUtil.parseDate(followUpTime), 9));
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpSaveList.add(followUpAddDTO);
                } else {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(DoctorFollowUpConstant.Type.HZCY.getCode());
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(StrJoinSplitUtils.join(consultationDiseases));
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark("出院后：" + followUpTimeRaw);
                    // followUpAddDTO.setFollowDate(null);  // 没有解析出随访时间
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpFailSaveList.add(followUpAddDTO);
                }

                dataCheck.add(i.getPatientId() + "-" + doctorId + "-" + i.getInpatientId());
            }
        });

        if (CollUtil.isNotEmpty(followUpSaveList)) {
            log.info("9.保存随访及提醒信息 >>>>>> {}", JSON.toJSONString(followUpSaveList));
            doctorFollowUpBizService.saveAndLaunchBatch(followUpSaveList);
        }
        if (CollUtil.isNotEmpty(followUpFailSaveList)) {
            log.info("10.保存未解析出随访时间的随访信息 >>>>>> {}", JSON.toJSONString(followUpFailSaveList));
            doctorFollowUpBizService.addBatch(followUpFailSaveList);
        }
    }

    @Override
    public void createConsultationFollowUpTaskExe(Date startTime, Date endTime) {
        // 查询已完成会诊的会诊及申请信息
        log.info("1.查询已完成会诊的会诊申请信息");
        List<ConsultationApplyAdviceVO> consultationMapList = doctorFollowUpQueryBizMapper.consultationFollowupList(
            // new QueryWrapper<ConsultationApplyAdviceVO>()
            //     .eq("a.type", ConsultationLaunchApplyConstant.Type.Task.getCode())
            //     .isNotNull("a.consultation_id")
            //     // .eq("a.status", ConsultationLaunchApplyConstant.ApplyStatus.DONE.getCode())
            //     .isNotNull("a.finish_time")
            //     .and(i -> i.isNull("b.consultation_follow_up_base_date_type").or().eq("b.consultation_follow_up_base_date_type", LabelConsultationFollowUpBaseDateTypeEnum.HZ.getCode()))
            // , 
            new QueryWrapper<ConsultationApplyAdviceVO>()
                .isNotNull("a.followup")
                .ne("a.followup", "")
                .isNull("c.consultation_id")
                .isNull("d.consultation_id")
                .ge(null != startTime, "a.submit_time", startTime)
                .lt(null != endTime, "a.submit_time", endTime)
        );
        if (CollUtil.isEmpty(consultationMapList)) {
            return;
        }

        // 整理出必要的查询条件
        // List<String> consultationIds = new ArrayList<>(); // 会诊id  用于查会诊
        List<String> labelIds = new ArrayList<>(); // 标签id  用于查标签
        List<String> deptIds = new ArrayList<>(); // 会诊结论科室 用于查询随访医生（科室相关）
        Map<String, String> consultationLabelMap = new HashMap<>(); // 会诊标签关联  用于会诊与病种关联
        List<ConsultationApplyAdviceVO> followBaseDateConsultationList = new ArrayList<>();
        consultationMapList.forEach(i -> {
            deptIds.add(i.getMdtDeptCode());

            String consultationId = i.getConsultationId();
            // consultationIds.add(consultationId);
            
            String consultationLabelId = i.getConsultationLabelId();
            if (StrUtil.isNotBlank(consultationLabelId)) {
                labelIds.add(consultationLabelId);
                consultationLabelMap.put(consultationId, consultationLabelId);
            } else { // 没有会诊标签则自动根据会诊结论确定随访的基准时间
                followBaseDateConsultationList.add(i);
            }
        });
        
        if (CollUtil.isEmpty(deptIds)) {
            return;
        }
        
        // log.info("2.查询一段时间内的患者最新可随访的会诊信息");
        // ConsultationQueryClientDTO consultationQueryClientDTO = new ConsultationQueryClientDTO();
        // consultationQueryClientDTO.setIds(consultationIds);
        // List<ConsultationBO> consultationList = consultationClientService.list(consultationQueryClientDTO);
        // if (CollUtil.isEmpty(consultationList)) {
        //     return;
        // }

        log.info("3.查询科室对应的需要随访的运营助理");
        // 查询需要随访的运营助理
        DoctorRelationQueryDTO doctorDeptQueryDTO = new DoctorRelationQueryDTO();
        doctorDeptQueryDTO.setRelationIds(deptIds);
        doctorDeptQueryDTO.setType(DoctorRelationConstant.Type.DEPT.getCode());
        List<DoctorRelationVO> doctorDeptVOs = doctorRelationBizService.simpleList(doctorDeptQueryDTO);
        if (CollUtil.isEmpty(doctorDeptVOs)) {
            return;
        }
        Map<String, Set<String>> deptRelationMap = new HashMap<>(); // key:deptId value:doctorIds 科室-运营助理关联
        Set<String> doctorSet = new HashSet<>(); // 查询的运营助理所管理病种的参数
        doctorDeptVOs.forEach(i -> {
            Set<String> d = deptRelationMap.get(i.getRelationId());
            if (null == d) {
                d = new HashSet<>();
                deptRelationMap.put(i.getRelationId(), d);
            }
            d.add(i.getDoctorId());

            doctorSet.addAll(d);
        });

        log.info("4.查询可随访会诊标签病种信息");
        Map<String, List<String>> labelDiseaseMap = new HashMap<>(); // key:labelId value:diseaseIds 标签-病种关联  用于会诊与病种关联
        if (CollUtil.isNotEmpty(labelIds)) {
            List<LabelBO> labelList = labelClientService.getLabelListByIds(labelIds);
            // 整理出会诊标签病种信息
            if (CollUtil.isNotEmpty(labelList)) {
                labelList.forEach(i -> {
                    labelDiseaseMap.put(i.getId(), i.getDiseaseIds());
                });
            }
        }
        // 整理出会诊病种信息
        Map<String, Set<String>> consultationDiseaseMap = new HashMap<>(); // key:consultaionId value:diseaseIds 会诊病种关联
        consultationLabelMap.forEach((k, v) -> {
            List<String> diseases = labelDiseaseMap.get(v);
            if (CollUtil.isNotEmpty(diseases)) {
                Set<String> l = consultationDiseaseMap.get(k);
                if (null == l) {
                    l = new HashSet<>();
                    consultationDiseaseMap.put(k, l);
                }
                l.addAll(diseases);
            }
        });

        log.info("5.查询病种对应的需要随访的运营助理");
        //根据科室对应的运营助理，再查询对应病种的运营助理
        Map<String, Set<String>> diseaseRelationMap = new HashMap<>(); // key:doctorId  value:diseaseIds
        if (CollUtil.isNotEmpty(doctorSet)) {
            DoctorRelationQueryDTO doctorDiseaseQueryDTO = new DoctorRelationQueryDTO();
            doctorDiseaseQueryDTO.setDoctorIds(new ArrayList<>(doctorSet));
            doctorDiseaseQueryDTO.setType(DoctorRelationConstant.Type.DISEASE.getCode());
            List<DoctorRelationVO> doctorDiseaseVOs = doctorRelationBizService.simpleList(doctorDiseaseQueryDTO);
            if (CollUtil.isNotEmpty(doctorDiseaseVOs)) {
                doctorDiseaseVOs.forEach(i -> {
                    Set<String> d = diseaseRelationMap.get(i.getDoctorId());
                    if (null == d) {
                        d = new HashSet<>();
                        diseaseRelationMap.put(i.getDoctorId(), d);
                    }
                    d.add(i.getRelationId());
                });
            }
        }

        log.info("处理未被会诊标签抓取的患者的会诊随访基准时间类型");
        if (CollUtil.isNotEmpty(followBaseDateConsultationList)) {
            followBaseDateConsultationList.forEach(i -> {
                String consultationFollowUpBaseDateType = FollowUpRegexUtil.getFollowUpBaseDate(i.getSubmitContent());
                if (StrUtil.isNotBlank(consultationFollowUpBaseDateType)) {
                    i.setConsultationFollowUpBaseDateType(consultationFollowUpBaseDateType);
                }
            });
        }

        log.info("6.过滤出已经出院的患者");
        List<String> patientIds = new ArrayList<>();
        List<String> visitIds = new ArrayList<>();
        consultationMapList.forEach(i -> {
            if (LabelConsultationFollowUpBaseDateTypeEnum.CY.getCode().equals(i.getConsultationFollowUpBaseDateType())) {
                patientIds.add(i.getPatientId());
                visitIds.add(i.getVisitId());
            }
        });
        Map<String, PatientInpatientRecordClientBO> inpatientRecordMap = new HashMap<>();
        if (CollUtil.isNotEmpty(patientIds) && CollUtil.isNotEmpty(visitIds)) {
            PatientLastDataQueryDTO patientLastDataQueryDTO = new PatientLastDataQueryDTO();
            patientLastDataQueryDTO.setPatientIds(patientIds);
            patientLastDataQueryDTO.setVisitIds(visitIds);
            List<PatientInpatientRecordClientBO> inpatientRecordList = dwdDataPatientClientService.listPatientInpatientLastDateAndOut(patientLastDataQueryDTO);
            if (CollUtil.isEmpty(inpatientRecordList)) {
                return;
            }
            inpatientRecordList.forEach(i -> {
                if (null != i && YN.YES.getCode().equals(i.getIsOut())) {
                    inpatientRecordMap.put(i.getVisitId(), i);
                }
            });
        }

        log.info("7.生成随访信息并设置提醒");
        // 生成随访信息并设置提醒
        List<DoctorFollowUpAddDTO> followUpSaveList = new ArrayList<>();
        List<DoctorFollowUpAddDTO> followUpFailSaveList = new ArrayList<>(); // 未解析出随访时间的数据
        consultationMapList.forEach(i -> {
            if (i.getPatientId().startsWith("TB")) {
                return;
            }

            String type = DoctorFollowUpConstant.Type.HZ.getCode();
            String remark = "（直接）会诊后：";
            Date baseDate = i.getSubmitTime();
            if (LabelConsultationFollowUpBaseDateTypeEnum.HZ.getCode().equals(i.getConsultationFollowUpBaseDateType())) {
                type = DoctorFollowUpConstant.Type.HZ.getCode();
                remark = "（标签）会诊后：";
            } else if (LabelConsultationFollowUpBaseDateTypeEnum.CY.getCode().equals(i.getConsultationFollowUpBaseDateType())) {
                PatientInpatientRecordClientBO bo = inpatientRecordMap.get(i.getVisitId());
                if (null == bo) {
                    return;
                }
                baseDate = bo.getOutDate();
                type = DoctorFollowUpConstant.Type.HZCY.getCode();
                remark = "（标签）会诊出院后：";
            }
            
            // 处理随访时间，循环会诊结论获取其中一个有会诊随访时间的数据并解析
            String followUpTimeRaw = i.getFollowup();
            String hisAdviceDoctorDeptId = i.getMdtDeptCode(); // 提交会诊结论医生科室
            String followupContent = i.getSubmitContent();
            

            // 原始随访时间为空说明不是需要随访的会诊
            if (StrUtil.isBlank(followUpTimeRaw)) {
                return;
            }

            String followUpTime = FollowUpRegexUtil.getFollowupDateStandard(baseDate, followUpTimeRaw);

            log.info("followUpTime >>>> {}", followUpTime);
            Set<String> doctorIds = deptRelationMap.get(hisAdviceDoctorDeptId);
            if (CollUtil.isEmpty(doctorIds)) {
                return;
            }

            Set<String> consultationDiseases = consultationDiseaseMap.get(i.getConsultationId());

            for (String doctorId : doctorIds) {
                Boolean flag = false;
                Set<String> doctorDiseases = diseaseRelationMap.get(doctorId);
                if (CollUtil.isNotEmpty(doctorDiseases)) {
                    if (CollUtil.isNotEmpty(consultationDiseases)) { // 为空说明找不到会诊对应的病种，而运营助理有配置病种则不匹配
                        for (String cd : consultationDiseases) {
                            if (doctorDiseases.contains(cd)) {
                                flag = true;
                                break;
                            }
                        }
                    }
                } else { // 没病种则直接匹配科室
                    flag = true;
                }

                if (!flag) {
                    continue;
                }

                String diseases = CollUtil.isEmpty(consultationDiseases) ? "" : StrJoinSplitUtils.join(consultationDiseases); 

                if (StrUtil.isNotBlank(followUpTime)) {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(type);
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(diseases);
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark(remark + followUpTimeRaw);
                    followUpAddDTO.setFollowDate(DateUtil.offsetHour(DateUtil.parseDate(followUpTime), 9));
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpSaveList.add(followUpAddDTO);
                } else {
                    DoctorFollowUpAddDTO followUpAddDTO = new DoctorFollowUpAddDTO();
                    followUpAddDTO.setType(type);
                    followUpAddDTO.setPatientId(i.getPatientId());
                    // followUpAddDTO.setInpatientId(i.getInpatientId());
                    followUpAddDTO.setVisitId(i.getInpatientId());
                    followUpAddDTO.setDiseaseId(diseases);
                    followUpAddDTO.setDoctorId(doctorId); // 运营助理
                    followUpAddDTO.setRemark(remark + followUpTimeRaw);
                    // followUpAddDTO.setFollowDate(null);  // 没有解析出随访时间
                    followUpAddDTO.setOpType(DoctorFollowUpConstant.OpType.TASK.getCode());
                    followUpAddDTO.setConsultationId(i.getConsultationId());
                    followUpAddDTO.setContent(followupContent);
                    followUpAddDTO.setSourceDeptId(hisAdviceDoctorDeptId);
                    followUpAddDTO.setSourceDate(baseDate);
                    
                    followUpFailSaveList.add(followUpAddDTO);
                }
            }
        });

        if (CollUtil.isNotEmpty(followUpSaveList)) {
            log.info("8.保存随访及提醒信息 >>>>>> {}", JSON.toJSONString(followUpSaveList));
            doctorFollowUpBizService.saveAndLaunchBatch(followUpSaveList);
        }
        if (CollUtil.isNotEmpty(followUpFailSaveList)) {
            log.info("9.保存未解析出随访时间的随访信息 >>>>>> {}", JSON.toJSONString(followUpFailSaveList));
            doctorFollowUpBizService.addBatch(followUpFailSaveList);
        }
    }

    @Override
    public void doctorFollowUpTimeout() {
        doctorFollowUpBizService.timeout();
    }

    @Override
    public void createFollowUpReturnVisitTaskExecute(Date startTime, Date endTime, Integer returnVisitDuring) {

        List<DoctorFollowUpInfo> followupList = doctorFollowUpInfoService.list(new QueryWrapper<DoctorFollowUpInfo>()
                .eq("rows_status", YN.YES.getCode())
                .eq("op_type", DoctorFollowUpConstant.OpType.TASK.getCode())
                .in("status", DoctorFollowUpConstant.Status.DOING.getCode(), DoctorFollowUpConstant.Status.DONE.getCode())
                .ge("follow_time", startTime)
                .lt(null != endTime, "follow_time", endTime)
        );
        if (CollUtil.isEmpty(followupList)) {
            return;
        }

        List<String> followIds = new ArrayList<>();
        followupList.forEach(i -> followIds.add(i.getId()));

        // 查询医生待随访的数据
        // List<DoctorFollowUpVO> followupList = doctorFollowUpQueryBizMapper.followupListWithoutReturnVisit(new QueryWrapper<DoctorFollowUpVO>()
        //     .in("a.status", DoctorFollowUpConstant.Status.DOING.getCode(), DoctorFollowUpConstant.Status.DONE.getCode())
        //     .ge("a.follow_time", startTime)
        //     .lt(null != endTime, "a.follow_time", endTime)
        //     .isNull("b.id")
        // );

        Map<String, PatientReturnVisitBO> returnVisitMap = new HashMap<>();
        PatientReturnVisitQueryClientDTO patientReturnVisitQueryClientDTO = new PatientReturnVisitQueryClientDTO();
        patientReturnVisitQueryClientDTO.setSourceType(PatientReturnVisitConstant.SourceType.Follow_Up.getCode());
        patientReturnVisitQueryClientDTO.setSourceIds(followIds);
        List<PatientReturnVisitBO> returnVisitList = patientReturnVisitClientService.list(patientReturnVisitQueryClientDTO);
        if (CollUtil.isNotEmpty(returnVisitList)) {
            returnVisitList.forEach(i -> returnVisitMap.put(i.getSourceId(), i));
        }

        List<String> deptIds = new ArrayList<>();
        followupList.forEach(i -> {
            if (null == returnVisitMap.get(i.getId())) { // 只使用没有生成过复诊信息的随访科室
                deptIds.add(i.getSourceDeptId());
            }
        });
        
        // 复诊科室组
        Map<String, List<String>> relationDeptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptIds)) {
            List<Map<String, String>> relationDeptList = doctorFollowUpQueryBizMapper.relationDeptList(new QueryWrapper<Void>()
                .in("b.dept_id", deptIds)
            );

            if (CollUtil.isNotEmpty(relationDeptList)) {
                relationDeptList.forEach(i -> {
                    String deptId = i.get("deptId");
                    String relationDeptId = i.get("relationDeptId");

                    List<String> l = relationDeptMap.get(deptId);
                    if (null == l) {
                        l = new ArrayList<>();
                        relationDeptMap.put(deptId, l);
                    }
                    l.add(relationDeptId);
                });
            }
        }

        // 生成复诊，已生成的复诊根据随访状态更新复诊信息
        List<PatientReturnVisitAddClientDTO> returnVisitAddList = new ArrayList<>();
        List<PatientReturnVisitUpdateClientDTO> returnVisitUpdateList = new ArrayList<>();
        followupList.forEach(i -> {
            // 已有的更新或跳过
            PatientReturnVisitBO returnVisit = returnVisitMap.get(i.getId());
            if (null != returnVisit) {
                // 待随访的复诊，若对应的随访已经完成则此复诊更新为已随访
                if (PatientReturnVisitConstant.Status.Not_Follow_Up.getCode().equals(returnVisit.getStatus()) && DoctorFollowUpConstant.Status.DONE.getCode().equals(i.getStatus())) {
                    PatientReturnVisitUpdateClientDTO returnVisitUpdateClientDTO = new PatientReturnVisitUpdateClientDTO();
                    returnVisitUpdateClientDTO.setId(returnVisit.getId());
                    returnVisitUpdateClientDTO.setVisitStartTime(i.getRealTime());
                    returnVisitUpdateClientDTO.setStatus(PatientReturnVisitConstant.Status.Follow_Up.getCode());
   
                    returnVisitUpdateList.add(returnVisitUpdateClientDTO);
                }

                return;
            }

            // 没有随访时间则无法生成对应的复诊时间，所以直接过滤掉
            if (null == i.getFollowTime()) {
                return;
            }

            PatientReturnVisitAddClientDTO returnVisitAddDTO = new PatientReturnVisitAddClientDTO();
            returnVisitAddDTO.setPatientId(i.getPatientId());
            returnVisitAddDTO.setVisitId(i.getVisitId());
            if (DoctorFollowUpConstant.Type.CY.getCode().equals(i.getType())) {
                returnVisitAddDTO.setVisitType(PatientReturnVisitConstant.Type.CY.getCode());
            } else if (DoctorFollowUpConstant.Type.HZ.getCode().equals(i.getType()) || DoctorFollowUpConstant.Type.HZCY.getCode().equals(i.getType())) {
                returnVisitAddDTO.setVisitType(PatientReturnVisitConstant.Type.HZ.getCode());
            }
            returnVisitAddDTO.setSourceType(PatientReturnVisitConstant.SourceType.Follow_Up.getCode());
            returnVisitAddDTO.setSourceId(i.getId());
            if (null != i.getSourceDate()) {
                returnVisitAddDTO.setSourceTime(DateUtil.formatDateTime(i.getSourceDate()));
            }
            if (StrUtil.isNotBlank(i.getSourceDeptId())) {
                returnVisitAddDTO.setSourceDeptId(i.getSourceDeptId());
                List<String> returnVisitDeptIds = relationDeptMap.get(i.getSourceDeptId());
                if (CollUtil.isNotEmpty(returnVisitUpdateList)) {
                    returnVisitAddDTO.setDeptIds(returnVisitDeptIds);
                } else {
                    returnVisitAddDTO.setDeptIds(Arrays.asList(i.getSourceDeptId()));
                }
            }
            returnVisitAddDTO.setContent(i.getRemark());
            returnVisitAddDTO.setStatus(PatientReturnVisitConstant.Status.Not_Follow_Up.getCode());
            returnVisitAddDTO.setOpType(PatientReturnVisitConstant.OpType.Task.getCode());

            // 默认的复诊开始时间为设置的随访时间
            if (null != i.getFollowTime()) {
                returnVisitAddDTO.setVisitStartTime(i.getFollowTime());
                returnVisitAddDTO.setVisitEndTime(DateUtil.offsetDay(i.getFollowTime(), returnVisitDuring));
            }
            // 若已经随访则复诊的开始时间为实际的随访时间，
            if (DoctorFollowUpConstant.Status.DONE.getCode().equals(i.getStatus())) {
                returnVisitAddDTO.setVisitStartTime(i.getRealTime());
                if (null == returnVisitAddDTO.getVisitEndTime()) {
                    returnVisitAddDTO.setVisitEndTime(DateUtil.offsetDay(i.getRealTime(), returnVisitDuring));
                }
                returnVisitAddDTO.setStatus(PatientReturnVisitConstant.Status.Follow_Up.getCode());
            }

            returnVisitAddList.add(returnVisitAddDTO);
        });
        
        if (CollUtil.isNotEmpty(returnVisitAddList)) {
            log.info("新增复诊 >>>>>> ");
            patientReturnVisitClientService.saveBatch(returnVisitAddList);
        }
        if (CollUtil.isNotEmpty(returnVisitUpdateList)) {
            log.info("更新复诊状态 >>>>>> ");
            patientReturnVisitClientService.updateBatch(returnVisitUpdateList);
        }
    }

    @Override
    public void doctorFollowUpOneDayStatTask(Date startDate, Date endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.date().toDateStr());
        if (null == startDate) {
            startDate = now;
        }
        if (null == endDate || endDate.after(now)) {
            endDate = now;
        }
        
        while (!startDate.after(endDate)) {
            String bizDay = DateUtil.formatDate(startDate);
            Date queryEndDate = DateUtil.offsetDay(startDate, 1);

            List<Map<String, Object>> statList = doctorFollowUpInfoService.listMaps(new QueryWrapper<DoctorFollowUpInfo>()
                .select("doctor_id as doctorId", "COUNT(1) as totalNum", "COUNT(IF(status = 'done',1,NULL)) as realNum")
                .eq("rows_status", YN.YES.getCode())
                .ge("follow_time", startDate)
                .lt("follow_time", queryEndDate)
                .groupBy("doctor_id")
            );

            List<DoctorFollowUpOneDayAddDTO> saveList = new ArrayList<>();
            if (CollUtil.isNotEmpty(statList)) {
                statList.forEach(i -> {
                    String doctorId = String.valueOf(i.get("doctorId"));
                    Integer totalNum = Integer.valueOf(String.valueOf(i.get("totalNum")));
                    Integer realNum = Integer.valueOf(String.valueOf(i.get("realNum")));

                    DoctorFollowUpOneDayAddDTO entity = new DoctorFollowUpOneDayAddDTO();
                    entity.setBizDay(bizDay);
                    entity.setDoctorId(doctorId);
                    entity.setTotalNum(totalNum);
                    entity.setRealNum(realNum);
                    
                    saveList.add(entity);
                });
            }

            if (CollUtil.isNotEmpty(saveList)) {
                doctorFollowUpOneDayBizService.saveBatch(saveList);
            }

            startDate = DateUtil.offsetDay(startDate, 1);
        }
    }

}
