package com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.basic.risk.client.api.RiskConfigClientService;
import com.xxh.cloud.healthy.basic.risk.client.api.bean.bo.RiskConfigBO;
import com.xxh.cloud.healthy.basic.risk.client.api.bean.dto.RiskConfigQueryClientDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.entity.PatientRiskHistoryInfo;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.entity.PatientRiskInfo;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.service.PatientRiskHistoryInfoService;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.service.PatientRiskInfoService;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskAddBatchDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskAddDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskDelBatchDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskDelDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.vo.PatientRiskVO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.helper.PatientRiskHelper;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.PatientRiskBizService;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.PatientRiskQueryService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class PatientRiskBizServiceImpl implements PatientRiskBizService {

    @Autowired
    private PatientRiskInfoService patientRiskInfoService;
    @Autowired
    private PatientRiskHistoryInfoService patientRiskHistoryInfoService;
    @Autowired
    private RiskConfigClientService riskConfigClientService;
    @Autowired
    private PatientRiskQueryService patientRiskQueryService;
    
    @Transactional(rollbackFor = Exception.class)
    @Override
    public PatientRiskVO add(PatientRiskAddDTO params) {
        log.info("PatientRiskBizServiceImpl.add(PatientRiskAddDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        RiskConfigBO bo = riskConfigClientService.getById(params.getRiskId());
        if (null == bo) {
            throw new BusinessException("风险配置信息不存在");
        }

        Date now = DateUtil.date();
        
        // 患者风险
        PatientRiskInfo entity = new PatientRiskInfo();
        entity.setId(PatientRiskHelper.getPatientRiskInfoId());
        entity.setPatientId(params.getPatientId());
        entity.setRiskId(params.getRiskId());
        entity.setRiskName(bo.getName());
        entity.setRiskLevel(bo.getLevel());
        entity.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
        entity.setRiskRelId(bo.getRelId());
        entity.setRiskRelType(bo.getRelType());
        entity.setRiskSysId(bo.getSysId());
        entity.setRiskType(bo.getType());
        entity.setSourceRelId(params.getSourceRelId());
        entity.setSourceType(params.getSourceType());
        entity.setDataDt(params.getDataDt());
        entity.setCreateDate(now);
        entity.setCreator(params.getOpUser());
        
        // 患者风险历史
        PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
        history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
        history.setPatientId(params.getPatientId());
        history.setRiskId(params.getRiskId());
        history.setRiskName(bo.getName());
        history.setRiskLevel(bo.getLevel());
        history.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
        history.setRiskRelId(bo.getRelId());
        history.setRiskRelType(bo.getRelType());
        history.setRiskSysId(bo.getSysId());
        history.setRiskType(bo.getType());
        history.setSourceRelId(params.getSourceRelId());
        history.setSourceType(params.getSourceType());
        history.setDataDt(params.getDataDt());
        history.setOpUser(params.getOpUser());
        history.setOpDate(now);
        history.setOpType(OpTypeEnum.Create.getCode());

        // 患者风险
        patientRiskInfoService.save(entity);
        // 患者风险历史
        patientRiskHistoryInfoService.save(history);

        return PatientRiskHelper.getPatientRiskVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<PatientRiskVO> addBatch(List<PatientRiskAddDTO> params) {
        log.info("PatientRiskBizServiceImpl.addBatch(List<PatientRiskAddDTO>): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        if (CollectionUtil.isEmpty(params)) {
            return new ArrayList<>(0);
        }
        
        Set<String> riskIds = new HashSet<>();
        Set<String> patientIds = new HashSet<>();
        
        params.forEach(i -> {
            riskIds.add(i.getRiskId());
            patientIds.add(i.getPatientId());
        });

        // 查询风险信息
        RiskConfigQueryClientDTO riskConfigQueryClientDTO = new RiskConfigQueryClientDTO();
        riskConfigQueryClientDTO.setIds(new ArrayList<>(riskIds));
        List<RiskConfigBO> riskConfigBOs = riskConfigClientService.list(riskConfigQueryClientDTO);
        if (CollectionUtil.isEmpty(riskConfigBOs)) {
            return new ArrayList<>(0);
        }
        Map<String, RiskConfigBO> riskConfigBOMap = riskConfigBOs.stream().collect(Collectors.toMap(RiskConfigBO::getId, i -> i));

        // 查询患者的风险
        PatientRiskQueryDTO patientRiskQueryDTO = new PatientRiskQueryDTO();
        patientRiskQueryDTO.setPatientIds(new ArrayList<>(patientIds));
        patientRiskQueryDTO.setRiskIds(new ArrayList<>(riskIds));
        List<PatientRiskVO> existPatientRiskVOs = patientRiskQueryService.list(patientRiskQueryDTO);
        // Map<String, PatientRiskVO> existPatientRiskVOMap = existPatientRiskVOs.stream().collect(Collectors.toMap(i -> i.getPatientId() + "-" + i.getRiskId(), i -> i));
        Map<String, PatientRiskVO> existPatientRiskVOMap = new HashMap<>();
        existPatientRiskVOs.forEach(i -> existPatientRiskVOMap.put(i.getPatientId() + "-" + i.getRiskId(), i));

        Date now = DateUtil.date();
        
        List<PatientRiskInfo> entityList = new ArrayList<>();
        List<PatientRiskHistoryInfo> historyList = new ArrayList<>();
        params.forEach(i -> {
            // 过滤掉已有的风险
            PatientRiskVO vo = existPatientRiskVOMap.get(i.getPatientId() + "-" + i.getRiskId());
            if (null != vo) {
                return;
            }

            RiskConfigBO bo = riskConfigBOMap.get(i.getRiskId());
            if (null != bo) {
                // 患者风险
                PatientRiskInfo entity = new PatientRiskInfo();
                entity.setId(PatientRiskHelper.getPatientRiskInfoId());
                entity.setPatientId(i.getPatientId());
                entity.setRiskId(i.getRiskId());
                entity.setRiskName(bo.getName());
                entity.setRiskLevel(bo.getLevel());
                entity.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
                entity.setRiskRelId(bo.getRelId());
                entity.setRiskRelType(bo.getRelType());
                entity.setRiskSysId(bo.getSysId());
                entity.setRiskType(bo.getType());
                entity.setSourceRelId(i.getSourceRelId());
                entity.setSourceType(i.getSourceType());
                entity.setRemark(i.getRemark());
                entity.setDataDt(i.getDataDt());
                entity.setCreateDate(now);
                entity.setCreator(i.getOpUser());

                entityList.add(entity);

                // 患者风险历史
                PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
                history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
                history.setPatientId(i.getPatientId());
                history.setRiskId(i.getRiskId());
                history.setRiskName(bo.getName());
                history.setRiskLevel(bo.getLevel());
                history.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
                history.setRiskRelId(bo.getRelId());
                history.setRiskRelType(bo.getRelType());
                history.setRiskSysId(bo.getSysId());
                history.setRiskType(bo.getType());
                history.setSourceRelId(i.getSourceRelId());
                history.setSourceType(i.getSourceType());
                history.setDataDt(i.getDataDt());
                history.setOpUser(i.getOpUser());
                history.setOpDate(now);
                history.setOpType(OpTypeEnum.Create.getCode());

                historyList.add(history);
            }
        });
        
        if (CollectionUtil.isNotEmpty(entityList)) {
            // 患者风险
            patientRiskInfoService.saveBatch(entityList);
            // 患者风险历史
            patientRiskHistoryInfoService.saveBatch(historyList);
        }

        return entityList.stream().map(PatientRiskHelper::getPatientRiskVO).collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<PatientRiskVO> addBatch(PatientRiskAddBatchDTO params) {
        log.info("PatientRiskBizServiceImpl.addBatch(PatientRiskAddBatchDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);

        // 查询患者的风险
        PatientRiskQueryDTO patientRiskQueryDTO = new PatientRiskQueryDTO();
        patientRiskQueryDTO.setPatientId(params.getPatientId());
        patientRiskQueryDTO.setRiskIds(params.getRiskIds());
        List<PatientRiskVO> existPatientRiskVOs = patientRiskQueryService.list(patientRiskQueryDTO);
        Map<String, PatientRiskVO> existPatientRiskVOMap = new HashMap<>();
        existPatientRiskVOs.forEach(i -> {
            existPatientRiskVOMap.put(i.getRiskId(), i);
        });

        // 查询风险信息
        RiskConfigQueryClientDTO riskConfigQueryClientDTO = new RiskConfigQueryClientDTO();
        riskConfigQueryClientDTO.setIds(params.getRiskIds());
        List<RiskConfigBO> riskConfigBOs = riskConfigClientService.list(riskConfigQueryClientDTO);
        if (CollectionUtil.isEmpty(riskConfigBOs)) {
            return new ArrayList<>(0);
        }
        Map<String, RiskConfigBO> riskConfigBOMap = riskConfigBOs.stream().collect(Collectors.toMap(RiskConfigBO::getId, i -> i));

        Date now = DateUtil.date();
        
        List<PatientRiskInfo> entityList = new ArrayList<>();
        List<PatientRiskHistoryInfo> historyList = new ArrayList<>();
        params.getRiskIds().forEach(i -> {
            // 过滤掉已有的风险
            PatientRiskVO vo = existPatientRiskVOMap.get(i);
            if (null != vo) {
                return;
            }

            RiskConfigBO bo = riskConfigBOMap.get(i);
            if (null != bo) {
                // 患者风险
                PatientRiskInfo entity = new PatientRiskInfo();
                entity.setId(PatientRiskHelper.getPatientRiskInfoId());
                entity.setPatientId(params.getPatientId());
                entity.setRiskId(i);
                entity.setRiskName(bo.getName());
                entity.setRiskLevel(bo.getLevel());
                entity.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
                entity.setRiskRelId(bo.getRelId());
                entity.setRiskRelType(bo.getRelType());
                entity.setRiskSysId(bo.getSysId());
                entity.setRiskType(bo.getType());
                entity.setSourceRelId(params.getSourceRelId());
                entity.setSourceType(params.getSourceType());
                entity.setRemark(params.getRemark());
                entity.setDataDt(params.getDataDt());
                entity.setCreateDate(now);
                entity.setCreator(params.getOpUser());

                entityList.add(entity);

                // 患者风险历史
                PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
                history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
                history.setPatientId(params.getPatientId());
                history.setRiskId(i);
                history.setRiskName(bo.getName());
                history.setRiskLevel(bo.getLevel());
                history.setRiskDiseaseIds(StrJoinSplitUtils.joinPrefix(bo.getDiseaseIds()));
                history.setRiskRelId(bo.getRelId());
                history.setRiskRelType(bo.getRelType());
                history.setRiskSysId(bo.getSysId());
                history.setRiskType(bo.getType());
                history.setSourceRelId(params.getSourceRelId());
                history.setSourceType(params.getSourceType());
                history.setDataDt(params.getDataDt());
                history.setOpUser(params.getOpUser());
                history.setOpDate(now);
                history.setOpType(OpTypeEnum.Create.getCode());

                historyList.add(history);
            }
        });
        
        if (CollectionUtil.isNotEmpty(entityList)) {
            // 患者风险
            patientRiskInfoService.saveBatch(entityList);
            // 患者风险历史
            patientRiskHistoryInfoService.saveBatch(historyList);
        }

        return entityList.stream().map(PatientRiskHelper::getPatientRiskVO).collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void remove(PatientRiskDelDTO params) {
        log.info("PatientRiskBizServiceImpl.remove(PatientRiskDelDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        
        DateTime now = DateUtil.date();

        PatientRiskInfo entity = patientRiskInfoService.getOne(new QueryWrapper<PatientRiskInfo>()
                .eq("patient_id", params.getPatientId())
                .eq("risk_id", params.getRiskId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
        history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
        history.setPatientId(entity.getPatientId());
        history.setRiskId(entity.getRiskId());
        history.setSourceRelId(entity.getSourceRelId());
        history.setSourceType(entity.getSourceType());
        history.setDataDt(entity.getDataDt());
        history.setOpDate(now);
        history.setOpType(OpTypeEnum.DEL.getCode());

        // 删除患者风险
        patientRiskInfoService.removeById(entity.getId());
        // 保存患者风险操作历史
        patientRiskHistoryInfoService.save(history);

    }

    @Override
    public void removeBatch(List<PatientRiskDelDTO> params) {
        log.info("PatientRiskBizServiceImpl.removeBatch(List<PatientRiskDelDTO>):>>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        
        DateTime now = DateUtil.date();

        // 查询患者风险列表，用于做历史记录
        List<String> patientRiskIdList = params.stream().map(i -> {
            return i.getPatientId() + "-" + i.getRiskId();
        }).collect(Collectors.toList());

        if (CollectionUtil.isEmpty(patientRiskIdList)) {
            return;
        }

        List<PatientRiskInfo> existList = patientRiskInfoService.list(new QueryWrapper<PatientRiskInfo>()
            .in("CONCAT(patient_id, '-', risk_id)", patientRiskIdList)
        );

        if (CollectionUtil.isEmpty(existList)) {
            return;
        }

        List<String> patientRiskIds = new ArrayList<>();
        List<PatientRiskHistoryInfo> historyList = new ArrayList<>();

        existList.forEach(i -> {
            PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
            history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
            history.setPatientId(i.getPatientId());
            history.setRiskId(i.getRiskId());
            history.setSourceRelId(i.getSourceRelId());
            history.setSourceType(i.getSourceType());
            history.setDataDt(i.getDataDt());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());

            historyList.add(history);

            patientRiskIds.add(i.getId());
        });

        // 删除患者风险
        patientRiskInfoService.removeBatchByIds(patientRiskIds);
        // 保存患者风险操作历史
        patientRiskHistoryInfoService.saveBatch(historyList);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void removeBatch(PatientRiskDelBatchDTO params) {
        log.info("PatientRiskBizServiceImpl.removeBatch(PatientRiskDelBatchDTO): >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        
        DateTime now = DateUtil.date();

        // 查询患者风险列表，用于做历史记录
        List<String> patientRiskIdList = params.getRiskIds().stream().map(i -> {
            return params.getPatientId() + "-" + i;
        }).collect(Collectors.toList());

        if (CollectionUtil.isEmpty(patientRiskIdList)) {
            return;
        }

        List<PatientRiskInfo> existList = patientRiskInfoService.list(new QueryWrapper<PatientRiskInfo>()
            .in("CONCAT(patient_id, '-', risk_id)", patientRiskIdList)
        );

        if (CollectionUtil.isEmpty(existList)) {
            return;
        }

        List<String> patientRiskIds = new ArrayList<>();
        List<PatientRiskHistoryInfo> historyList = new ArrayList<>();

        existList.forEach(i -> {
            PatientRiskHistoryInfo history = new PatientRiskHistoryInfo();
            history.setId(PatientRiskHelper.getPatientRiskHistoryInfoId());
            history.setPatientId(i.getPatientId());
            history.setRiskId(i.getRiskId());
            history.setOpDate(now);
            history.setOpType(OpTypeEnum.DEL.getCode());

            history.setSourceRelId(params.getSourceRelId());
            history.setSourceType(params.getSourceType());
            history.setDataDt(params.getDataDt());

            historyList.add(history);

            patientRiskIds.add(i.getId());
        });

        // 删除患者风险
        patientRiskInfoService.removeBatchByIds(patientRiskIds);
        // 保存患者风险操作历史
        patientRiskHistoryInfoService.saveBatch(historyList);
    }
}
