package com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.healthy.basic.risk.client.api.RiskConfigClientService;
import com.xxh.cloud.healthy.basic.risk.client.api.bean.bo.RiskConfigBO;
import com.xxh.cloud.healthy.basic.risk.client.api.bean.dto.RiskConfigQueryClientDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.entity.PatientRiskHistoryInfo;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.entity.PatientRiskInfo;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.service.PatientRiskHistoryInfoService;
import com.xxh.cloud.healthy.serve.risk.framework.modules.base.service.PatientRiskInfoService;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskGetDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskHistoryGetDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskHistoryQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.vo.PatientRiskHistoryVO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.vo.PatientRiskVO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.helper.PatientRiskHelper;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.PatientRiskQueryService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.StandardDiseaseSystemClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseSystemBO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class PatientRiskQueryServiceImpl implements PatientRiskQueryService {

    @Autowired
    private PatientRiskInfoService patientRiskInfoService;
    @Autowired
    private PatientRiskHistoryInfoService patientRiskHistoryInfoService;
    
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    @Autowired
    private StandardDiseaseSystemClientService standardDiseaseSystemClientService;

    @Autowired
    private RiskConfigClientService riskConfigClientService;

    @Override
    public PageData<PatientRiskVO> page(PatientRiskQueryDTO params) {
        log.info("PatientRiskQueryServiceImpl:page:params:>>>>>> {}", JSON.toJSONString(params));

        IPage<PatientRiskInfo> page = patientRiskInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientRiskInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .eq(StrUtil.isNotBlank(params.getRiskId()), "risk_id", params.getRiskId())
            .in(CollectionUtil.isNotEmpty(params.getRiskIds()), "risk_id", params.getRiskIds())
            .eq(StrUtil.isNotBlank(params.getRiskSysId()), "risk_sys_id", params.getRiskSysId())
            .eq(StrUtil.isNotBlank(params.getRiskType()), "risk_type", params.getRiskType())
            .eq(StrUtil.isNotBlank(params.getRiskLevel()), "risk_level", params.getRiskLevel())
            .eq(StrUtil.isNotBlank(params.getRiskRelType()), "risk_rel_type", params.getRiskRelType())
            .eq(StrUtil.isNotBlank(params.getRiskRelId()), "risk_rel_id", params.getRiskRelId())
            .eq(StrUtil.isNotBlank(params.getSourceRelId()), "source_rel_id", params.getSourceRelId())
            .eq(StrUtil.isNotBlank(params.getSourceType()), "source_type", params.getSourceType())
            .orderByDesc("data_dt")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return null;
        }

        Set<String> diseaseIds=new HashSet<>();
        Set<String> sysIds=new HashSet<>();
        page.getRecords().forEach(item -> {
            if(StrUtil.isNotEmpty(item.getRiskDiseaseIds())){
                Set<String> disIds = StrJoinSplitUtils.split2Set(item.getRiskDiseaseIds());
                diseaseIds.addAll(disIds);
            }
            if (StrUtil.isNotBlank(item.getRiskSysId())) {
                sysIds.add(item.getRiskSysId());
            }
        });
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        Map<String,StandardDiseaseSystemBO> sysMap = standardDiseaseSystemClientService.getMap(sysIds);


        PageData<PatientRiskVO> data = new PageConverter<PatientRiskInfo, PatientRiskVO>() {
            @Override
            public PatientRiskVO converter(PatientRiskInfo entity) {
                PatientRiskVO vo = PatientRiskHelper.getPatientRiskVO(entity);
                if (CollectionUtil.isNotEmpty(vo.getRiskDiseaseIds())) {
                    List<StandardDiseaseBO> bos = new ArrayList<>();
                    vo.getRiskDiseaseIds().forEach(id -> {
                        StandardDiseaseBO disease = diseaseMap.get(id);
                        if(disease != null){
                            bos.add(disease);
                        }
                    });
                    vo.setRiskDiseaseList(bos);
                }
                if (StrUtil.isNotBlank(vo.getRiskSysId())) {
                    vo.setRiskSys(sysMap.get(vo.getRiskSysId()));
                }
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PatientRiskVO> list(PatientRiskQueryDTO params) {
        log.info("PatientRiskQueryServiceImpl:list:params:>>>>>> {}", JSON.toJSONString(params));
        
        List<PatientRiskInfo> list = patientRiskInfoService.list(new QueryWrapper<PatientRiskInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .eq(StrUtil.isNotBlank(params.getRiskId()), "risk_id", params.getRiskId())
            .in(CollectionUtil.isNotEmpty(params.getRiskIds()), "risk_id", params.getRiskIds())
            .eq(StrUtil.isNotBlank(params.getRiskSysId()), "risk_sys_id", params.getRiskSysId())
            .eq(StrUtil.isNotBlank(params.getRiskType()), "risk_type", params.getRiskType())
            .eq(StrUtil.isNotBlank(params.getRiskLevel()), "risk_level", params.getRiskLevel())
            .eq(StrUtil.isNotBlank(params.getRiskRelType()), "risk_rel_type", params.getRiskRelType())
            .eq(StrUtil.isNotBlank(params.getRiskRelId()), "risk_rel_id", params.getRiskRelId())
            .eq(StrUtil.isNotBlank(params.getSourceRelId()), "source_rel_id", params.getSourceRelId())
            .eq(StrUtil.isNotBlank(params.getSourceType()), "source_type", params.getSourceType())
            .orderByDesc("data_dt")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        Set<String> diseaseIds=new HashSet<>();
        Set<String> sysIds=new HashSet<>();
        List<String> riskIds=new ArrayList<>();
        list.forEach(item -> {
            if(StrUtil.isNotEmpty(item.getRiskDiseaseIds())){
                Set<String> disIds = StrJoinSplitUtils.split2Set(item.getRiskDiseaseIds());
                diseaseIds.addAll(disIds);
            }
            if (StrUtil.isNotBlank(item.getRiskSysId())) {
                sysIds.add(item.getRiskSysId());
            }
            riskIds.add(item.getRiskId());
        });
        Map<String, StandardDiseaseBO> diseaseMap = standardDiseaseClientService.getMap(diseaseIds);
        Map<String,StandardDiseaseSystemBO> sysMap = standardDiseaseSystemClientService.getMap(sysIds);

        // zs20240325 返回风险描述
        Map<String, RiskConfigBO> riskMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(riskIds)) {
            RiskConfigQueryClientDTO riskConfigQueryClientDTO = new RiskConfigQueryClientDTO();
            riskConfigQueryClientDTO.setIds(riskIds);
            List<RiskConfigBO> riskConfigBOList = riskConfigClientService.list(riskConfigQueryClientDTO);
            riskConfigBOList.forEach(i -> riskMap.put(i.getId(), i));
        }

        return list.stream().map(i -> {
            PatientRiskVO vo = PatientRiskHelper.getPatientRiskVO(i);
            if (CollectionUtil.isNotEmpty(vo.getRiskDiseaseIds())) {
                List<StandardDiseaseBO> bos = new ArrayList<>();
                vo.getRiskDiseaseIds().forEach(id -> {
                    StandardDiseaseBO disease = diseaseMap.get(id);
                    if(disease != null){
                        bos.add(disease);
                    }
                });
                vo.setRiskDiseaseList(bos);
            }
            if (StrUtil.isNotBlank(i.getRiskSysId())) {
                vo.setRiskSys(sysMap.get(i.getRiskSysId()));
            }
            
            RiskConfigBO risk = riskMap.get(i.getRiskId());
            if (null != risk) {
                vo.setRemark(risk.getRemark());
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public PatientRiskVO info(PatientRiskGetDTO params) {
        log.info("PatientRiskQueryServiceImpl:info:params:>>>>>> {}", JSON.toJSONString(params));

        PatientRiskInfo entity = patientRiskInfoService.getOne(new QueryWrapper<PatientRiskInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );

        if(entity == null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        return PatientRiskHelper.getPatientRiskVO(entity);
    }

    @Override
    public PageData<PatientRiskHistoryVO> historyPage(PatientRiskHistoryQueryDTO params) {
        log.info("PatientRiskQueryServiceImpl:historyPage:params:>>>>>> {}", JSON.toJSONString(params));
        
        IPage<PatientRiskHistoryInfo> page = patientRiskHistoryInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<PatientRiskHistoryInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .eq(StrUtil.isNotBlank(params.getRiskId()), "risk_id", params.getRiskId())
            .in(CollectionUtil.isNotEmpty(params.getRiskIds()), "risk_id", params.getRiskIds())
            .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
            .orderByDesc("data_dt")
        );

        PageData<PatientRiskHistoryVO> data = new PageConverter<PatientRiskHistoryInfo, PatientRiskHistoryVO>() {
            @Override
            public PatientRiskHistoryVO converter(PatientRiskHistoryInfo entity) {
                return PatientRiskHelper.getPatientRiskHistoryVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PatientRiskHistoryVO> historyList(PatientRiskHistoryQueryDTO params) {
        log.info("PatientRiskQueryServiceImpl:historyList:params:>>>>>> {}", JSON.toJSONString(params));
        
        List<PatientRiskHistoryInfo> list = patientRiskHistoryInfoService.list(new QueryWrapper<PatientRiskHistoryInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .eq(StrUtil.isNotBlank(params.getRiskId()), "risk_id", params.getRiskId())
            .in(CollectionUtil.isNotEmpty(params.getRiskIds()), "risk_id", params.getRiskIds())
            .eq(StrUtil.isNotBlank(params.getOpType()), "op_type", params.getOpType())
            .orderByDesc("data_dt")
        );

        return list.stream().map(PatientRiskHelper::getPatientRiskHistoryVO).collect(Collectors.toList());
    }

    @Override
    public PatientRiskHistoryVO historyInfo(PatientRiskHistoryGetDTO params) {
        log.info("PatientRiskQueryServiceImpl:historyInfo:params:>>>>>> {}", JSON.toJSONString(params));
        
        PatientRiskHistoryInfo entity = patientRiskHistoryInfoService.getOne(new QueryWrapper<PatientRiskHistoryInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
        );

        if(entity == null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        return PatientRiskHelper.getPatientRiskHistoryVO(entity);
    }
}
