package com.xxh.cloud.healthy.service.com.app.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeHistoryQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientRemindQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientRemindVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/patient/pushService")
@Api(tags = "患者推送服务")
public class PatientPushServeAppController {
    
    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private VisitVerificationClientService visitVerificationClientService;

    private String getMainPersonId(String userId) {
        MemberFamilyBO memberFamilyBO = customerFamilyClientService.getChoose(userId);
        // if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getPatHisNo())) {
        //     return null;
        // }
        // return memberFamilyBO.getPatHisNo();
        if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getMainPersonId())) {
            return null;
        }
        
        // 黑名单验证
        // VisitVerificationSearchDTO visitVerificationSearchDTO = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), memberFamilyBO.getPatHisNo());
        Map<String, List<VisitVerificationBO>> verificationMap = visitVerificationClientService.getVisitVerificationByPatient(Arrays.asList(memberFamilyBO.getPatHisNo()));
        if (CollUtil.isNotEmpty(verificationMap) && isBlack(verificationMap.get(memberFamilyBO.getPatHisNo()))) {
            return null;
        }

        return memberFamilyBO.getMainPersonId();
    }
    
    private Boolean isBlack(List<VisitVerificationBO> verificationList) {
        if (CollUtil.isNotEmpty(verificationList)) {
            for (VisitVerificationBO bo : verificationList) {
                if (BasicVisitConstant.VerType.BLACK_LIST.getCode().equals(bo.getVerType())) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 分页查询患者的推送服务列表
     */
    @PostMapping("/page")
    @ApiOperation(value = "患者推送服务分页", response = PatientPushServeHistoryVO.class)
    public Result<PageData<PatientPushServeHistoryVO>> pushServePage(@RequestBody PatientPushServeHistoryQueryParam params) {
        log.info("/app/healthy/patient/pushService/page >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<PageData<PatientPushServeHistoryVO>>().success();
        }

        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<PageData<PatientPushServeHistoryVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        params.setOpType(OpTypeEnum.Create.getCode());
        PageData<PatientPushServeHistoryVO> data = patientPushServeQueryBizService.pushServeHistoryPage(params);
        return new Result<PageData<PatientPushServeHistoryVO>>().ok(data);
    }
    
    /**
     * 查询患者的推送服务列表
     */
    @PostMapping("/list")
    @ApiOperation(value = "患者推送服务列表", response = PatientPushServeVO.class)
    public Result<List<PatientPushServeHistoryVO>> pushServeList(@RequestBody PatientPushServeHistoryQueryParam params) {
        log.info("/app/healthy/patient/pushService/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<List<PatientPushServeHistoryVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<List<PatientPushServeHistoryVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        params.setOpType(OpTypeEnum.Create.getCode());
        List<PatientPushServeHistoryVO> data = patientPushServeQueryBizService.pushServeHistoryList(params);
        return new Result<List<PatientPushServeHistoryVO>>().ok(data);
    }

    @PostMapping("/remind/detail")
    @ApiOperation(value = "服务推送提醒详情", response = PatientRemindVO.class)
    public Result<PatientRemindVO> pushServePage(@RequestBody PatientRemindQueryDTO params) {
        log.info("/app/healthy/patient/pushService/remind/detail >>> {}", JSON.toJSONString(params));
        PatientRemindVO data = patientPushServeQueryBizService.getPatientRemind(params);
        return new Result<PatientRemindVO>().ok(data);
    }
    
}
