package com.xxh.cloud.healthy.service.com.app.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskHistoryQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.dto.PatientRiskQueryDTO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.vo.PatientRiskHistoryVO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.bean.vo.PatientRiskVO;
import com.xxh.cloud.healthy.serve.risk.framework.modules.biz.service.PatientRiskQueryService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/patient/risk")
@Api(tags = "患者风险查看")
public class PatientRiskManagerController {

    @Autowired
    private PatientRiskQueryService patientRiskQueryService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private VisitVerificationClientService visitVerificationClientService;
    
    private String getMainPersonId(String userId) {
        MemberFamilyBO memberFamilyBO = customerFamilyClientService.getChoose(userId);
        // if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getPatHisNo())) {
        //     return null;
        // }
        // return memberFamilyBO.getPatHisNo();
        if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getMainPersonId())) {
            return null;
        }
        
        // 黑名单验证
        // VisitVerificationSearchDTO visitVerificationSearchDTO = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), memberFamilyBO.getPatHisNo());
        Map<String, List<VisitVerificationBO>> verificationMap = visitVerificationClientService.getVisitVerificationByPatient(Arrays.asList(memberFamilyBO.getPatHisNo()));
        if (CollUtil.isNotEmpty(verificationMap) && isBlack(verificationMap.get(memberFamilyBO.getPatHisNo()))) {
            return null;
        }

        return memberFamilyBO.getMainPersonId();
    }
    
    private Boolean isBlack(List<VisitVerificationBO> verificationList) {
        if (CollUtil.isNotEmpty(verificationList)) {
            for (VisitVerificationBO bo : verificationList) {
                if (BasicVisitConstant.VerType.BLACK_LIST.getCode().equals(bo.getVerType())) {
                    return true;
                }
            }
        }
        return false;
    }

    @PostMapping("/list")
    @ApiOperation(value = "查询", response = PatientRiskVO.class)
    public Result<List<PatientRiskVO>> query(@RequestBody PatientRiskQueryDTO params) {
        log.info("/sys/healthy/patient/risk/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<List<PatientRiskVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<List<PatientRiskVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        ValidatorUtils.validateEntity(params);
        List<PatientRiskVO> data= patientRiskQueryService.list(params);
        return new Result<List<PatientRiskVO>>().ok(data);
    }

    @PostMapping("/historyList")
    @ApiOperation(value = "历史查询", response = PatientRiskHistoryVO.class)
    public Result<List<PatientRiskHistoryVO>> historyList(@RequestBody PatientRiskHistoryQueryDTO params) {
        log.info("/sys/healthy/patient/risk/historyList >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<List<PatientRiskHistoryVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<List<PatientRiskHistoryVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        ValidatorUtils.validateEntity(params);
        List<PatientRiskHistoryVO> data= patientRiskQueryService.historyList(params);
        return new Result<List<PatientRiskHistoryVO>>().ok(data);
    }

}
