package com.xxh.cloud.healthy.service.com.app.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.customer.client.api.CustomerFamilyClientService;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.basic.common.constant.RemindWayEnum;
import com.xxh.cloud.healthy.data.client.api.PatientClientService;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.dto.PatientQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleGetDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientScheduleVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/app/healthy/patient/schedule")
@Api(tags = "患者日程")
public class PatientScheduleAppController {
    
    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private CustomerFamilyClientService customerFamilyClientService;
    @Autowired
    private PatientClientService patientClientService;
    @Autowired
    private VisitVerificationClientService visitVerificationClientService;

    private String getMainPersonId(String userId) {
        MemberFamilyBO memberFamilyBO = customerFamilyClientService.getChoose(userId);
        // if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getPatHisNo())) {
        //     return null;
        // }
        // return memberFamilyBO.getPatHisNo();
        if (null == memberFamilyBO || StrUtil.isEmpty(memberFamilyBO.getMainPersonId())) {
            return null;
        }
        
        // 黑名单验证
        // VisitVerificationSearchDTO visitVerificationSearchDTO = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), memberFamilyBO.getPatHisNo());
        Map<String, List<VisitVerificationBO>> verificationMap = visitVerificationClientService.getVisitVerificationByPatient(Arrays.asList(memberFamilyBO.getPatHisNo()));
        if (CollUtil.isNotEmpty(verificationMap) && isBlack(verificationMap.get(memberFamilyBO.getPatHisNo()))) {
            return null;
        }

        return memberFamilyBO.getMainPersonId();
    }
    
    private Boolean isBlack(List<VisitVerificationBO> verificationList) {
        if (CollUtil.isNotEmpty(verificationList)) {
            for (VisitVerificationBO bo : verificationList) {
                if (BasicVisitConstant.VerType.BLACK_LIST.getCode().equals(bo.getVerType())) {
                    return true;
                }
            }
        }
        return false;
    }

    @PostMapping("/page")
    @ApiOperation(value = "分页", response = PatientScheduleVO.class)
    public Result<PageData<PatientScheduleVO>> patientSchedulePage(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/app/healthy/patient/schedule/page >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<PageData<PatientScheduleVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<PageData<PatientScheduleVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        params.setUserId(user.getId());
        
        // 只能看当前时间及之后的日程
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getStartDate()) || now.compareTo(params.getStartDate()) > 0) {
            params.setStartDate(now);
        }

        params.setScheduleTypes(Arrays.asList(RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode()));
        PageData<PatientScheduleVO> data = patientPushServeQueryBizService.patientSchedulePage(params);
        return new Result<PageData<PatientScheduleVO>>().ok(data);
    }
    
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = PatientScheduleVO.class)
    public Result<List<PatientScheduleVO>> patientScheduleList(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/app/healthy/patient/schedule/list >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<List<PatientScheduleVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<List<PatientScheduleVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        params.setUserId(user.getId());

        // 只能看当前时间及之后的日程
        String now = DateUtil.date().toDateStr();
        if (StrUtil.isBlank(params.getStartDate()) || now.compareTo(params.getStartDate()) > 0) {
            params.setStartDate(now);
        }

        params.setScheduleTypes(Arrays.asList(RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode()));
        List<PatientScheduleVO> data = patientPushServeQueryBizService.patientScheduleList(params);
        return new Result<List<PatientScheduleVO>>().ok(data);
    }

    /**
     * 患者的推送服务详情
     */
    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = PatientScheduleVO.class)
    public Result<PatientScheduleVO> patientScheduleDetail(@RequestBody PatientScheduleGetDTO params) {
        log.info("/app/healthy/patient/schedule/detail >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<PatientScheduleVO>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<PatientScheduleVO>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setPatientIds(patientIds);
        params.setUserId(user.getId());

        PatientScheduleVO vo = patientPushServeQueryBizService.patientScheduleDetail(params);
        return new Result<PatientScheduleVO>().ok(vo);
    }

    @PostMapping("/list/drug")
    @ApiOperation(value = "服药列表", response = PatientScheduleVO.class)
    public Result<List<PatientScheduleVO>> patientDrugScheduleList(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/app/healthy/patient/schedule/list >>> {}", JSON.toJSONString(params));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        String mainPersonId = this.getMainPersonId(user.getId());
        if (null == mainPersonId) {
            return new Result<List<PatientScheduleVO>>().success();
        }
        
        PatientQueryDTO patientQueryDTO = new PatientQueryDTO();
        patientQueryDTO.setMainPersonId(mainPersonId);
        List<PatientClientBO> patientList = patientClientService.simpleList(patientQueryDTO);
        if (CollectionUtil.isEmpty(patientList)) {
            return new Result<List<PatientScheduleVO>>().success();
        }
        List<String> patientIds = patientList.stream().map(PatientClientBO::getPatientId).collect(Collectors.toList());

        params.setBizOneType("recipe_medicine");
        params.setPatientIds(patientIds);
        params.setUserId(user.getId());

        // params.setScheduleTypes(Arrays.asList(RemindWayEnum.ALL.getCode(), RemindWayEnum.NotRemind.getCode()));
        List<PatientScheduleVO> data = patientPushServeQueryBizService.patientScheduleList(params);
        return new Result<List<PatientScheduleVO>>().ok(data);
    }

}
