package com.xxh.cloud.healthy.serv.framework.modules.userDisease.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.entity.dos.UserDiseaseInfo;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.entity.dos.UserDiseaseLogInfo;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.entity.dto.*;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.entity.vos.UserDiseaseVO;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.helper.UserDiseaseHelper;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.mapper.UserDiseaseInfoMapper;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.service.UserDiseaseInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.healthy.serv.framework.modules.userDisease.service.UserDiseaseLogInfoService;
import com.xxh.cloud.healthy.third.client.modules.api.entity.bos.HealthyUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 用户疾病信息表 服务实现类
 * </p>
 */
@Service
public class UserDiseaseInfoServiceImpl extends ServiceImpl<UserDiseaseInfoMapper, UserDiseaseInfo> implements UserDiseaseInfoService {


    @Autowired
    private UserDiseaseLogInfoService userDiseaseLogInfoService;


    @Override
    public PageData<UserDiseaseVO> query(UserDiseaseInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<UserDiseaseInfo> page = this.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<UserDiseaseInfo>()
                        .eq(StrUtil.isNotEmpty(params.getCustUserId()),"cust_user_id",params.getCustUserId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getRelIdType()),"rel_id_type",params.getRelIdType())
                        .eq(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())
                        .orderByDesc("cust_user_id","create_date")
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }
        Set<String> custUserIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCustUserId())){
                custUserIds.add(item.getCustUserId());
            }
            if(StrUtil.isNotEmpty(item.getDiseaseId())){
                diseaseIds.add(item.getDiseaseId());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                custUserIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                custUserIds.add(item.getUpdater());
            }
        });


//        Map<String, HealthyUserBO> userMap= healthyThirdUserService.getMapByIds(userIds);
//        PageData<ScaleResultVO> data= new PageConverter<ScaleResultInfo, ScaleResultVO>() {
//            @Override
//            public ScaleResultVO converter(ScaleResultInfo entity) {
//                return  ScaleResultConvert.getScaleResultVO(entity,userMap);
//            }
//        }.getPageData(page);
//

        return null;
    }

    @Override
    public List<UserDiseaseVO> list(UserDiseaseInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<UserDiseaseInfo> list= this.list(
                new QueryWrapper<UserDiseaseInfo>()
                        .eq(StrUtil.isNotEmpty(params.getCustUserId()),"cust_user_id",params.getCustUserId())
                        .eq(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_id",params.getDiseaseId())
                        .eq(StrUtil.isNotEmpty(params.getRelIdType()),"rel_id_type",params.getRelIdType())
                        .eq(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())
                        .orderByDesc("cust_user_id","create_date")
        );
        return null;
    }

    private Map<String,UserDiseaseInfo> getUserDiseaseMap(String userId){
        Map<String,UserDiseaseInfo> data=new HashMap<>();
        List<UserDiseaseInfo> list= this.list(
                new QueryWrapper<UserDiseaseInfo>()
                        .eq("cust_user_id",userId)
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(item.getDiseaseId(),item);
            });
        }
        return data;
    }

    @Override
    public void operateUserDisease(UserDiseaseInfoOperateDTO params) {
        ValidatorUtils.validateEntity(params);
        Map<String,UserDiseaseInfo> userDiseaseMap= this.getUserDiseaseMap(params.getCustUserId());

        List<UserDiseaseInfo> saveUserDiseaseList=new ArrayList<>();
        List<UserDiseaseInfo> updateUserDiseaseList=new ArrayList<>();
        Set<String> updateDiseaseIds= userDiseaseMap.keySet();
        for(UserDiseaseInfoItemDTO item:params.getItems()){
            UserDiseaseInfo userDisease =  userDiseaseMap.get(item.getDiseaseId());
            if(userDisease!=null){
                saveUserDiseaseList.add(UserDiseaseHelper.getUserDiseaseInfo(params.getCustUserId(),item));
            }else {
                updateDiseaseIds.add(item.getDiseaseId());
                updateUserDiseaseList.add(UserDiseaseHelper.getUserDiseaseInfo_edit(userDisease,item));
                userDiseaseMap.remove(item.getDiseaseId());
            }
        }

        //del
        Set<String> delDiseaseIds= userDiseaseMap.keySet();
        if(CollUtil.isNotEmpty(delDiseaseIds)){
            userDiseaseLogInfoService.update(
                    new UpdateWrapper<UserDiseaseLogInfo>()
                            .set("remove_op_date",new Date())
                            .set("remove_op_user",params.getOpUserId())
                            .eq("user_id",params.getCustUserId())
                            .in("disease_id",delDiseaseIds)
                            .isNull("remove_op_date")

            );
            this.remove(
                    new QueryWrapper<UserDiseaseInfo>()
                            .eq("cust_user_id",params.getCustUserId())
                            .in("disease_id",delDiseaseIds)
            );
        }

        //save
        if(saveUserDiseaseList.size()>0){
            List<UserDiseaseLogInfo> saveUserDiseaseLogList=new ArrayList<>();
            saveUserDiseaseList.forEach(item->{
                saveUserDiseaseLogList.add(UserDiseaseHelper.getUserDiseaseLogInfo(item));
            });
            userDiseaseLogInfoService.saveBatch(saveUserDiseaseLogList);
            this.saveBatch(saveUserDiseaseList);
        }

        //update
        if(updateUserDiseaseList.size()>0){
//            List<UserDiseaseLogInfo> userDiseaseLogList= userDiseaseLogInfoService.list(
//                    new QueryWrapper<UserDiseaseLogInfo>()
//                            .eq("user_id",params.getCustUserId())
//                            .in("disease_id",updateDiseaseIds)
//                            .isNull("remove_op_date")
//
//            );
            this.updateBatchById(updateUserDiseaseList);
        }

    }

    @Override
    public void removeUserDisease(UserDiseaseInfoRemoveDTO params) {
        ValidatorUtils.validateEntity(params);
        Map<String,UserDiseaseInfo> userDiseaseMap= this.getUserDiseaseMap(params.getCustUserId());
        Set<String> ids=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        for(String diseaseId:params.getDiseaseIds()){
            UserDiseaseInfo userDisease = userDiseaseMap.get(diseaseId);
            if(userDisease==null){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"数据不存在");
            }
            ids.add(userDisease.getId());
            diseaseIds.add(userDisease.getDiseaseId());
        }
        userDiseaseLogInfoService.update(
                new UpdateWrapper<UserDiseaseLogInfo>()
                        .set("remove_op_date",new Date())
                        .set("remove_op_user",params.getOpUserId())
                        .set("remove_remark",params.getRemark())
                        .eq("user_id",params.getCustUserId())
                        .in("disease_id",diseaseIds)
                        .isNull("remove_op_date")

        );
        this.removeBatchByIds(ids);
    }


    @Override
    public void addUserDisease(UserDiseaseInfoAddDTO params) {
        ValidatorUtils.validateEntity(params);
        Map<String,UserDiseaseInfo> userDiseaseMap= this.getUserDiseaseMap(params.getCustUserId());
        List<UserDiseaseInfo> saveList=new ArrayList<>();
        List<UserDiseaseLogInfo> logSaveList=new ArrayList<>();

        for(UserDiseaseInfoItemDTO item:params.getItems()){
            UserDiseaseInfo userDisease = userDiseaseMap.get(item.getDiseaseId());
            if(userDisease!=null){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"数据存在");
            }
            UserDiseaseInfo userDiseaseEntity=UserDiseaseHelper.getUserDiseaseInfo(params.getCustUserId(),item);
            saveList.add(userDiseaseEntity);
            logSaveList.add(UserDiseaseHelper.getUserDiseaseLogInfo(userDiseaseEntity));
        }

    }
}
