package com.xxh.cloud.healthy.service.com.manager.controller;

import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientDiversionOneDayQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageDetailQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.PatientManageOneDayQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientDiversionOneDayVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientManageDetailVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.PatientManageOneDayVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientDiversionOneDayBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageDetailBizService;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.PatientManageOneDayBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpOneDayQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorFollowUpOneDayVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorFollowUpOneDayBizService;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/stat")
@Api(tags = "运营助理工作统计")
public class AssistantStatController {

    @Autowired
    private PatientManageOneDayBizService patientManageOneDayBizService;
    @Autowired
    private PatientDiversionOneDayBizService patientDiversionOneDayBizService;
    @Autowired
    private DoctorFollowUpOneDayBizService doctorFollowUpOneDayBizService;
    @Autowired
    private PatientManageDetailBizService patientManageDetailBizService;
    
    @PostMapping("/manage/query")
    @ApiOperation(value = "患者管理日统计分页", response = PatientManageOneDayVO.class)
    public Result<PageData<PatientManageOneDayVO>> query(@RequestBody PatientManageOneDayQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<PatientManageOneDayVO> data = patientManageOneDayBizService.query(params);
        return new Result<PageData<PatientManageOneDayVO>>().ok(data);
    }

    @PostMapping("/manage/query/detail")
    @ApiOperation(value = "患者管理日统计详情", response = PatientManageDetailVO.class)
    public Result<PageData<PatientManageDetailVO>> queryDeatil(@RequestBody PatientManageDetailQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<PatientManageDetailVO> data = patientManageDetailBizService.query(params);
        return new Result<PageData<PatientManageDetailVO>>().ok(data);
    }

    @PostMapping("/diversion/query")
    @ApiOperation(value = "患者导流日统计分页", response = PatientDiversionOneDayVO.class)
    public Result<PageData<PatientDiversionOneDayVO>> diversionQuery(@RequestBody PatientDiversionOneDayQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<PatientDiversionOneDayVO> data = patientDiversionOneDayBizService.query(params);
        return new Result<PageData<PatientDiversionOneDayVO>>().ok(data);
    }

    @PostMapping("/diversion/query/detail")
    @ApiOperation(value = "患者管理日统计详情", response = PatientManageDetailVO.class)
    public Result<List<PatientManageDetailVO>> diversionQueryDeatil(@RequestBody PatientManageDetailQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        List<PatientManageDetailVO> data = patientManageDetailBizService.diversionQuery(params);
        return new Result<List<PatientManageDetailVO>>().ok(data);
    }

    @PostMapping("/followup/query")
    @ApiOperation(value = "医生随访日统计分页", response = DoctorFollowUpOneDayVO.class)
    public Result<PageData<DoctorFollowUpOneDayVO>> query(@RequestBody DoctorFollowUpOneDayQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<DoctorFollowUpOneDayVO> data = doctorFollowUpOneDayBizService.query(params);
        return new Result<PageData<DoctorFollowUpOneDayVO>>().ok(data);
    }

}
