package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import com.xxh.cloud.healthy.serve.label.framework.modules.task.service.PatientLabelTaskBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.task.service.DoctorTaskBizTaskService;
import com.xxh.cloud.healthy.service.com.manager.bean.StandardDiseaseBOVO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/dept/manager")
@Api(tags = "部门管理")
public class DoctorDeptManagerController {
    
    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;
    
    /**
     * 病种列表
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/disease/list")
    @ApiOperation(value = "病种列表", response = StandardDiseaseBOVO.class)
    public Result<List<StandardDiseaseBOVO>> list(@RequestBody Collection<String> ids) {
        log.info("/sys/healthy/dept/manager/disease/list >>> {}", JSON.toJSONString(ids));
        LoginUser user = AuthContextUtils.getCurrentLoginUser();

        // // 管理员查询所有病种数据 TODO 病种太多，暂时不能这么查询
        // if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
        //     ids = null;
        // } else if (CollectionUtil.isEmpty(ids)) {
        //     return new Result<List<StandardDiseaseBO>>().success();
        // }

        List<StandardDiseaseBO> list = standardDiseaseClientService.getList(ids);

        List<StandardDiseaseBOVO> result = new ArrayList<>();
        if (CollUtil.isNotEmpty(list)) {
            list.forEach(i -> {
                StandardDiseaseBOVO vo = new StandardDiseaseBOVO();
                BeanUtil.copyProperties(i, vo);
                vo.setIndex(100);
                if ("肺结节".equals(i.getName())) {
                    vo.setIndex(1);
                } else if ("心房颤动".equals(i.getName())) {
                    vo.setIndex(2);
                } else if ("胆石症".equals(i.getName())) {
                    vo.setIndex(3);
                } else if ("泌尿系结石".equals(i.getName())) {
                    vo.setIndex(4);
                } else if ("甲状腺结节".equals(i.getName())) {
                    vo.setIndex(5);
                } else if ("乳腺结节".equals(i.getName())) {
                    vo.setIndex(6);
                } else if ("前列腺增生".equals(i.getName())) {
                    vo.setIndex(7);
                } else if ("颅内动脉瘤".equals(i.getName())) {
                    vo.setIndex(8);
                } else if ("二尖瓣关闭不全".equals(i.getName())) {
                    vo.setIndex(9);
                } else if ("中央型房间隔缺损（卵圆孔型）".equals(i.getName())) {
                    vo.setIndex(10);
                } else if ("龋齿".equals(i.getName())) {
                    vo.setIndex(11);
                } else if ("牙列不齐".equals(i.getName())) {
                    vo.setIndex(12);
                } else if ("特殊检查宣教".equals(i.getName())) {
                    vo.setIndex(13);
                }

                result.add(vo);
            });
        }

        Comparator<StandardDiseaseBOVO> dataComparator = Comparator.comparing(StandardDiseaseBOVO::getIndex);
        result.sort(dataComparator);

        return new Result<List<StandardDiseaseBOVO>>().ok(result);
    }


//    测试代码
//    @Autowired
//    private DoctorTaskBizTaskService doctorTaskBizTaskService;
//    @Autowired
//    private PatientLabelTaskBizService patientLabelTaskBizService;
//
//    @PostMapping("/pushPatientLabelRemind")
//    @ApiOperation(value = "发送提醒负责人消息", response = StandardDiseaseBO.class)
//    public Result<List<StandardDiseaseBO>> pushPatientLabelRemind(@RequestBody Map<String, String> params) {
//        doctorTaskBizTaskService.pushDoctorConsultationPatientLabelRemind();
//        return new Result<List<StandardDiseaseBO>>().success();
//    }
//
//    @PostMapping("/remindPatientLabel")
//    @ApiOperation(value = "提醒标签病种负责人", response = StandardDiseaseBO.class)
//    public Result<List<StandardDiseaseBO>> remindPatientLabel(@RequestBody Map<String, String> params) {
//        patientLabelTaskBizService.remindPatientLabel(24);
//        return new Result<List<StandardDiseaseBO>>().success();
//    }
}
