package com.xxh.cloud.healthy.service.com.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serve.pushTask.client.api.constant.DoctorFollowUpConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpFollowDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpGetDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorFollowUpUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorFollowUpVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorFollowUpBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/followup")
@Api(tags = "医生随访管理")
public class DoctorFollowUpManagerController {
    
    @Autowired
    private DoctorFollowUpBizService doctorFollowUpBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DoctorFollowUpVO.class)
    public Result<PageData<DoctorFollowUpVO>> query(@RequestBody DoctorFollowUpQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorFollowUpVO> data = doctorFollowUpBizService.page(params);
        return new Result<PageData<DoctorFollowUpVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorFollowUpVO.class)
    public Result<DoctorFollowUpVO> detail(@RequestBody DoctorFollowUpGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorFollowUpVO vo = doctorFollowUpBizService.detail(params);
        return new Result<DoctorFollowUpVO>().ok(vo);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存", response = DoctorFollowUpVO.class)
    public Result<DoctorFollowUpVO> add(@RequestBody DoctorFollowUpAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        if (StrUtil.isBlank(params.getType())) {
            params.setType(DoctorFollowUpConstant.Type.Normal.getCode()); // 默认普通随防
        }
        params.setOpType(DoctorFollowUpConstant.OpType.MANUAL.getCode());
        DoctorFollowUpVO vo = doctorFollowUpBizService.add(params);
        return new Result<DoctorFollowUpVO>().ok(vo);
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody DoctorFollowUpUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorFollowUpBizService.update(params);
        return new Result<Void>().success();
    }

    @PostMapping("/launch")
    @ApiOperation(value = "发起提醒", response = Void.class)
    public Result<Void> launch(@RequestBody DoctorFollowUpGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorFollowUpBizService.launch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/saveAndLaunch")
    @ApiOperation(value = "保存并发起提醒", response = Void.class)
    public Result<Void> launch(@RequestBody DoctorFollowUpAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        if (StrUtil.isBlank(params.getType())) {
            params.setType(DoctorFollowUpConstant.Type.Normal.getCode()); // 默认普通随防
        }
        params.setOpType(DoctorFollowUpConstant.OpType.MANUAL.getCode());
        doctorFollowUpBizService.saveAndLaunch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/follow")
    @ApiOperation(value = "随访记录", response = Void.class)
    public Result<Void> follow(@RequestBody DoctorFollowUpFollowDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorFollowUpBizService.follow(params);
        return new Result<Void>().success();
    }

    @PostMapping("/del")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> del(@RequestBody DoctorFollowUpGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        doctorFollowUpBizService.del(params);
        return new Result<Void>().success();
    }

}
