package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.dto.DeptPatientRelationQueryDTO;
import com.xxh.cloud.healthy.data.framework.modules.biz.bean.vos.TaskDeptPatientRelationVO;
import com.xxh.cloud.healthy.data.framework.modules.biz.service.DeptPatientRelationBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientScheduleQueryDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientScheduleVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeLogGetDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorPushServeLogQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorPushServeLogVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/dept/doctor/patient")
@Api(tags = "部门患者管理")
public class DoctorPatientManagerController {
    
    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private DeptPatientRelationBizService deptPatientRelationBizService;
    @Autowired
    private DoctorTaskQueryBizService doctorTaskQueryBizService;
    @Autowired
    private DoctorTaskExeBizService doctorTaskExeBizService;

    /**
     * 创建患者的推送服务
     */
    // @SaCheckPermission("sys:healthyserve:patientPushServe:add")
    @PostMapping("/pushServe/add")
    @ApiOperation(value = "给患者推送服务", response = Void.class)
    public Result<Void> pushServeAdd(@RequestBody DoctorPushServeAddDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/pushServe/add >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        doctorTaskExeBizService.doctorPushServeAdd(params);
        return new Result<Void>().success();
    }
    
    /**
     * 忽略患者的推送服务
     */
    // @SaCheckPermission("sys:healthyserve:patientPushServe:add")
    @PostMapping("/pushServe/ignore")
    @ApiOperation(value = "忽略给患者推送服务", response = Void.class)
    public Result<Void> pushServeIgnore(@RequestBody DoctorPushServeAddDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/pushServe/ignore >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        params.setAction("ignore");
        doctorTaskExeBizService.doctorPushServeIgnore(params);
        return new Result<Void>().success();
    }
    
    /**
     * 查询患者日程
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/schedule/all/page")
    @ApiOperation(value = "分页查询所有患者日程", response = PatientScheduleVO.class)
    public Result<PageData<PatientScheduleVO>> allPatientSchedulePage(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/schedule/all/page >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDeptId(user.getDeptId());
        // DeptPatientRelationQueryDTO deptPatientRelationQueryDTO = new DeptPatientRelationQueryDTO();
        // deptPatientRelationQueryDTO.setDeptId(params.getDeptId());
        // deptPatientRelationQueryDTO.setPatientId(params.getPatientId());
        // deptPatientRelationQueryDTO.setPatientIds(params.getPatientIds());
        // List<TaskDeptPatientRelationVO> deptPatientList = deptPatientRelationBizService.query(deptPatientRelationQueryDTO);
        // if (CollectionUtil.isEmpty(deptPatientList)) {
        //     return new Result<PageData<PatientScheduleVO>>().success();
        // }
        // List<String> patientIds = deptPatientList.stream().map(TaskDeptPatientRelationVO::getPatientId).collect(Collectors.toList());
        // params.setPatientIds(patientIds);

        PageData<PatientScheduleVO> vo = patientPushServeQueryBizService.patientSchedulePage(params);
        return new Result<PageData<PatientScheduleVO>>().ok(vo);
    }

    /**
     * 查询患者日程
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/schedule/page")
    @ApiOperation(value = "分页查询患者日程", response = PatientScheduleVO.class)
    public Result<PageData<PatientScheduleVO>> patientSchedulePage(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/schedule/page >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());

        DeptPatientRelationQueryDTO deptPatientRelationQueryDTO = new DeptPatientRelationQueryDTO();
        deptPatientRelationQueryDTO.setDeptId(params.getDeptId());
        deptPatientRelationQueryDTO.setPatientId(params.getPatientId());
        deptPatientRelationQueryDTO.setPatientIds(params.getPatientIds());
        List<TaskDeptPatientRelationVO> deptPatientList = deptPatientRelationBizService.query(deptPatientRelationQueryDTO);
        if (CollectionUtil.isEmpty(deptPatientList)) {
            return new Result<PageData<PatientScheduleVO>>().success();
        }
        List<String> patientIds = deptPatientList.stream().map(TaskDeptPatientRelationVO::getPatientId).collect(Collectors.toList());
        params.setPatientIds(patientIds);

        PageData<PatientScheduleVO> vo = patientPushServeQueryBizService.patientSchedulePage(params);
        return new Result<PageData<PatientScheduleVO>>().ok(vo);
    }

    /**
     * 查询患者日程
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/schedule/list")
    @ApiOperation(value = "患者日程列表", response = PatientScheduleVO.class)
    public Result<List<PatientScheduleVO>> list(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/schedule/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());

        DeptPatientRelationQueryDTO deptPatientRelationQueryDTO = new DeptPatientRelationQueryDTO();
        deptPatientRelationQueryDTO.setDeptId(params.getDeptId());
        deptPatientRelationQueryDTO.setPatientId(params.getPatientId());
        deptPatientRelationQueryDTO.setPatientIds(params.getPatientIds());
        List<TaskDeptPatientRelationVO> deptPatientList = deptPatientRelationBizService.query(deptPatientRelationQueryDTO);
        if (CollectionUtil.isEmpty(deptPatientList)) {
            return new Result<List<PatientScheduleVO>>().success();
        }
        List<String> patientIds = deptPatientList.stream().map(TaskDeptPatientRelationVO::getPatientId).collect(Collectors.toList());
        params.setPatientIds(patientIds);

        List<PatientScheduleVO> vo = patientPushServeQueryBizService.patientScheduleList(params);
        return new Result<List<PatientScheduleVO>>().ok(vo);
    }

    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/schedule/patient/page")
    @ApiOperation(value = "分页查询患者日程", response = PatientScheduleVO.class)
    public Result<PageData<PatientScheduleVO>> patientSchedulePatientPage(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/schedule/patient/page >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());

        // DeptPatientRelationQueryDTO deptPatientRelationQueryDTO = new DeptPatientRelationQueryDTO();
        // deptPatientRelationQueryDTO.setDeptId(params.getDeptId());
        // deptPatientRelationQueryDTO.setPatientId(params.getPatientId());
        // deptPatientRelationQueryDTO.setPatientIds(params.getPatientIds());
        // List<TaskDeptPatientRelationVO> deptPatientList = deptPatientRelationBizService.query(deptPatientRelationQueryDTO);
        // if (CollectionUtil.isEmpty(deptPatientList)) {
        //     return new Result<PageData<PatientScheduleVO>>().success();
        // }
        // List<String> patientIds = deptPatientList.stream().map(TaskDeptPatientRelationVO::getPatientId).collect(Collectors.toList());
        // params.setPatientIds(patientIds);

        PageData<PatientScheduleVO> vo = patientPushServeQueryBizService.patientSchedulePatientPage(params);
        return new Result<PageData<PatientScheduleVO>>().ok(vo);
    }

    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/schedule/byservice/page")
    @ApiOperation(value = "分页查询（动态生成）患者服务推送日程（非家属日程）", response = PatientScheduleVO.class)
    public Result<PageData<PatientScheduleVO>> patientScheduleByServicePage(@RequestBody PatientScheduleQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/schedule/byservice/page >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());

        // DeptPatientRelationQueryDTO deptPatientRelationQueryDTO = new DeptPatientRelationQueryDTO();
        // deptPatientRelationQueryDTO.setDeptId(params.getDeptId());
        // deptPatientRelationQueryDTO.setPatientId(params.getPatientId());
        // deptPatientRelationQueryDTO.setPatientIds(params.getPatientIds());
        // List<TaskDeptPatientRelationVO> deptPatientList = deptPatientRelationBizService.query(deptPatientRelationQueryDTO);
        // if (CollectionUtil.isEmpty(deptPatientList)) {
        //     return new Result<PageData<PatientScheduleVO>>().success();
        // }
        // List<String> patientIds = deptPatientList.stream().map(TaskDeptPatientRelationVO::getPatientId).collect(Collectors.toList());
        // params.setPatientIds(patientIds);
        // params.setPatientId(null);

        PageData<PatientScheduleVO> vo = patientPushServeQueryBizService.patientSchedulePatientByServicePage(params);
        return new Result<PageData<PatientScheduleVO>>().ok(vo);
    }

    /**
     * 查询医生推送患者服务日志
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/pushServe/log/query")
    @ApiOperation(value = "分页查询医生推送患者服务日志", response = DoctorPushServeLogVO.class)
    public Result<PageData<DoctorPushServeLogVO>> query(@RequestBody DoctorPushServeLogQueryDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/pushServe/log/query >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        PageData<DoctorPushServeLogVO> data = doctorTaskQueryBizService.queryPushServe(params);
        return new Result<PageData<DoctorPushServeLogVO>>().ok(data);
    }

    /**
     * 查询医生推送患者服务日志
     */
    // @SaCheckPermission("sys:healthyserve:pushTaskExe:add")
    @PostMapping("/pushServe/log/detail")
    @ApiOperation(value = "查询医生推送患者服务日志详情", response = DoctorPushServeLogVO.class)
    public Result<DoctorPushServeLogVO> detail(@RequestBody DoctorPushServeLogGetDTO params) {
        log.info("/sys/healthy/dept/doctor/patient/pushServe/log/detail >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        DoctorPushServeLogVO vo = doctorTaskQueryBizService.pushServeDetail(params);
        return new Result<DoctorPushServeLogVO>().ok(vo);
    }

}
