package com.xxh.cloud.healthy.service.com.manager.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationAddBatchDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationGetByDoctorDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationGetDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorRelationUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRelationByDoctorVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRelationVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorRelationBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/doctor/relation")
@Api(tags = "医生关系管理")
public class DoctorRelationManagerController {
    
    @Autowired
    private DoctorRelationBizService doctorRelationBizService;

    @PostMapping("/query")
    @ApiOperation(value = "分页", response = DoctorRelationVO.class)
    public Result<PageData<DoctorRelationVO>> query(@RequestBody DoctorRelationQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DoctorRelationVO> data = doctorRelationBizService.page(params);
        return new Result<PageData<DoctorRelationVO>>().ok(data);
    }

    @PostMapping("/detail")
    @ApiOperation(value = "详情", response = DoctorRelationVO.class)
    public Result<DoctorRelationVO> detail(@RequestBody DoctorRelationGetDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        DoctorRelationVO data = doctorRelationBizService.detail(params);
        return new Result<DoctorRelationVO>().ok(data);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存", response = Void.class)
    public Result<Void> add(@RequestBody DoctorRelationAddDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorRelationBizService.add(params);
        return new Result<Void>().success();
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> update(@RequestBody DoctorRelationUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorRelationBizService.update(params);
        return new Result<Void>().success();
    }

    @PostMapping("/del")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> del(@RequestBody DoctorRelationGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorRelationBizService.del(params);
        return new Result<Void>().success();
    }

    @PostMapping("/queryByDoctor")
    @ApiOperation(value = "按医生分页", response = DoctorRelationByDoctorVO.class)
    public Result<PageData<DoctorRelationByDoctorVO>> queryByDoctor(@RequestBody DoctorRelationQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        PageData<DoctorRelationByDoctorVO> data = doctorRelationBizService.pageByDoctor(params);
        return new Result<PageData<DoctorRelationByDoctorVO>>().ok(data);
    }

    @PostMapping("/detailByDoctor")
    @ApiOperation(value = "按医生查详情", response = DoctorRelationByDoctorVO.class)
    public Result<DoctorRelationByDoctorVO> detailByDoctor(@RequestBody DoctorRelationGetByDoctorDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        if (StrUtil.isBlank(params.getDoctorId())) {
            return new Result<DoctorRelationByDoctorVO>().success();
        }
        DoctorRelationByDoctorVO data = doctorRelationBizService.detailByDoctor(params);
        return new Result<DoctorRelationByDoctorVO>().ok(data);
    }

    @PostMapping("/saveByDoctor")
    @ApiOperation(value = "按医生保存", response = Void.class)
    public Result<Void> saveByDoctor(@RequestBody DoctorRelationAddBatchDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorRelationBizService.saveByDoctor(params);
        return new Result<Void>().success();
    }

    @PostMapping("/delByDoctor")
    @ApiOperation(value = "按医生删除", response = Void.class)
    public Result<Void> delByDoctor(@RequestBody DoctorRelationGetByDoctorDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorRelationBizService.delByDoctor(params);
        return new Result<Void>().success();
    }

}
