package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleAddDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleQueryDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleRemoveDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.DoctorScheduleUpdateDTO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorScheduleVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskExeVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorScheduleBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/schedule")
@Api(tags = "医生日程管理")
public class DoctorScheduleManagerController {
    
    @Autowired
    private DoctorScheduleBizService doctorScheduleBizService;
    
    // @SaCheckPermission("sys:healthyserve:doctorSchedule:query")
    @PostMapping("/query")
    @ApiOperation(value = "医生日程分页", response = DoctorScheduleVO.class)
    public Result<PageData<DoctorScheduleVO>> query(@RequestBody DoctorScheduleQueryDTO params) {
        log.info("/sys/healthy/doctor/schedule/query >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorScheduleVO> data = doctorScheduleBizService.page(params);
        return new Result<PageData<DoctorScheduleVO>>().ok(data);
    }
    
    // @SaCheckPermission("sys:healthyserve:doctorSchedule:list")
    @PostMapping("/list")
    @ApiOperation(value = "医生日程", response = DoctorScheduleVO.class)
    public Result<List<DoctorScheduleVO>> list(@RequestBody DoctorScheduleQueryDTO params) {
        log.info("/sys/healthy/doctor/schedule/list >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        List<DoctorScheduleVO> data = doctorScheduleBizService.list(params);
        return new Result<List<DoctorScheduleVO>>().ok(data);
    }
    
    // @SaCheckPermission("sys:healthyserve:doctorSchedule:get")
    @PostMapping("/getById")
    @ApiOperation(value = "详情", response = DoctorTaskExeVO.class)
    public Result<DoctorScheduleVO> getById(@RequestBody DoctorScheduleQueryDTO params) {
        log.info("/sys/healthy/doctor/schedule/getById >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorScheduleVO vo = doctorScheduleBizService.getById(params);
        return new Result<DoctorScheduleVO>().ok(vo);
    }

    // @SaCheckPermission("sys:healthyserve:doctorSchedule:add")
    @PostMapping("/add")
    @ApiOperation(value = "新建", response = DoctorTaskExeVO.class)
    public Result<DoctorScheduleVO> add(@RequestBody DoctorScheduleAddDTO params) {
        log.info("/sys/healthy/doctor/schedule/add >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        ValidatorUtils.validateEntity(params);
        DoctorScheduleVO vo = doctorScheduleBizService.add(params);
        return new Result<DoctorScheduleVO>().ok(vo);
    }

    // @SaCheckPermission("sys:healthyserve:doctorSchedule:update")
    @PostMapping("/updateById")
    @ApiOperation(value = "更新", response = Void.class)
    public Result<Void> updateById(@RequestBody DoctorScheduleUpdateDTO params) {
        log.info("/sys/healthy/doctor/schedule/updateById >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        ValidatorUtils.validateEntity(params);
        doctorScheduleBizService.updateById(params);
        return new Result<Void>().success();
    }

    // @SaCheckPermission("sys:healthyserve:doctorSchedule:remove")
    @PostMapping("/remove")
    @ApiOperation(value = "删除", response = Void.class)
    public Result<Void> remove(@RequestBody DoctorScheduleRemoveDTO params) {
        log.info("/sys/healthy/doctor/schedule/remove >>> {}", JSON.toJSONString(params));
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        ValidatorUtils.validateEntity(params);
        doctorScheduleBizService.remove(params);
        return new Result<Void>().success();
    }

}
