package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskBizService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorPatientStatusMarkBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.constant.DoctorStatusMarkConstant;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRemindCountVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorRemindVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskCountVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskDateVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.DoctorTaskVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.bean.vos.ManageDoctorTaskVO;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskExeHandlerBizService;
import com.xxh.cloud.healthy.serve.pushTask.framework.modules.biz.service.DoctorTaskQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/doctor/pushTask")
@Api(tags = "医生任务管理")
public class DoctorTaskManagerController {
    
    @Autowired
    private DoctorTaskQueryBizService doctorTaskQueryBizService;
    @Autowired
    private DoctorTaskExeBizService doctorTaskExeBizService;
    @Autowired
    private DoctorTaskExeHandlerBizService doctorTaskExeHandlerBizService;

    @Autowired
    private DoctorTaskBizService doctorTaskBizService;


    @PostMapping("/add2")
    @ApiOperation(value = "添加AI建议", response = DoctorTaskVO.class)
    public Result<String> add2(@RequestBody TaskDoctor2AddDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
         doctorTaskBizService.addTask2(params);
        return new Result<String>().ok("ok");
    }

    @PostMapping("/del2")
    @ApiOperation(value = "删除AI建议", response = DoctorTaskVO.class)
    public Result<String> del2(@RequestBody DoctorTaskGetDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        doctorTaskBizService.delTask2(params.getId());
        return new Result<String>().ok("ok");
    }




    
    // @SaCheckPermission("sys:healthyserve:doctorTask:manageQuery")
    @PostMapping("/manageQuery")
    @ApiOperation(value = "分页查询医生任务（个案）", response = ManageDoctorTaskVO.class)
    public Result<PageData<ManageDoctorTaskVO>> manageQuery(@RequestBody DoctorTaskQueryDTO params) {
        PageData<ManageDoctorTaskVO> data = doctorTaskQueryBizService.queryByDoctor(params);
        return new Result<PageData<ManageDoctorTaskVO>>().ok(data);
    }
    
    // @SaCheckPermission("sys:healthyserve:doctorTask:query")
    @PostMapping("/query")
    @ApiOperation(value = "医生任务分页", response = DoctorTaskVO.class)
    public Result<PageData<DoctorTaskVO>> query(@RequestBody DoctorTaskQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorTaskVO> data = doctorTaskQueryBizService.query(params);
        return new Result<PageData<DoctorTaskVO>>().ok(data);
    }
    
    // @SaCheckPermission("sys:healthyserve:doctorTask:query")
    @PostMapping("/query/bydisease/all")
    @ApiOperation(value = "医生任务分页(通过病种)", response = DoctorTaskVO.class)
    public Result<PageData<DoctorTaskVO>> queryAll(@RequestBody DoctorTaskQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        // params.setDoctorId(user.getId());
        PageData<DoctorTaskVO> data = doctorTaskQueryBizService.queryByDisease(params);
        return new Result<PageData<DoctorTaskVO>>().ok(data);
    }
    
    // @SaCheckPermission("sys:healthyserve:doctorTask:queryByDate")
    @PostMapping("/queryByDate")
    @ApiOperation(value = "医生任务按日期分页", response = DoctorTaskDateVO.class)
    public Result<PageData<DoctorTaskDateVO>> queryByDate(@RequestBody DoctorTaskQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<DoctorTaskDateVO> data = doctorTaskQueryBizService.queryByDate(params);
        return new Result<PageData<DoctorTaskDateVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:doctorTask:detail")
    @PostMapping("/detail")
    @ApiOperation(value = "医生任务详情", response = DoctorTaskVO.class)
    public Result<DoctorTaskVO> detail(@RequestBody DoctorTaskGetDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorTaskVO data = doctorTaskQueryBizService.detail(params);
        return new Result<DoctorTaskVO>().ok(data);
    }

    @PostMapping("/getTaskCount")
    @ApiOperation(value = "我的任务：统计待处理任务/总任务的数量", response = DoctorTaskCountVO.class)
    public Result<DoctorTaskCountVO> getTaskCount(@RequestBody DoctorTaskQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorTaskCountVO data = doctorTaskQueryBizService.getTaskCount(params);
        return new Result<DoctorTaskCountVO>().ok(data);
    }

    @PostMapping("/listDoctorRemind")
    @ApiOperation(value = "医生提醒列表", response = DoctorRemindVO.class)
    public Result<List<DoctorRemindVO>> listDoctorRemind(@RequestBody DoctorRemindQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        List<DoctorRemindVO> data = doctorTaskQueryBizService.listDoctorRemind(params);
        return new Result<List<DoctorRemindVO>>().ok(data);
    }

    @PostMapping("/getRemindCountByUrgentLevel")
    @ApiOperation(value = "智慧提醒：统计紧急/重要/普通程度的提醒数量", response = DoctorRemindCountVO.class)
    public Result<DoctorRemindCountVO> getRemindCountByUrgentLevel(@RequestBody DoctorRemindQueryDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        DoctorRemindCountVO data = doctorTaskQueryBizService.getRemindCountByUrgentLevel(params);
        return new Result<DoctorRemindCountVO>().ok(data);
    }


    @PostMapping("/manage/add")
    @ApiOperation(value = "添加个案管理", response = DoctorPatientStatusMarkBO.class)
    public Result<DoctorPatientStatusMarkBO> pushServeAdd(@RequestBody DoctorTaskManageDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        params.setDeptId(user.getDeptId());
        DoctorPatientStatusMarkBO data = doctorTaskExeBizService.doctorManageExe(params, DoctorStatusMarkConstant.Status.Normal.getCode());
        return new Result<DoctorPatientStatusMarkBO>().ok(data);
    }
    
    @PostMapping("/manage/ignore")
    @ApiOperation(value = "忽略个案管理", response = DoctorPatientStatusMarkBO.class)
    public Result<DoctorPatientStatusMarkBO> pushServeIgnore(@RequestBody DoctorTaskManageDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        params.setDeptId(user.getDeptId());
        DoctorPatientStatusMarkBO data = doctorTaskExeBizService.doctorManageExe(params, DoctorStatusMarkConstant.Status.Ignore.getCode());
        return new Result<DoctorPatientStatusMarkBO>().ok(data);
    }
    
    @PostMapping("/consultation/initiate")
    @ApiOperation(value = "发起会诊", response = Void.class)
    public Result<Void> pushServeAdd(@RequestBody DoctorConsultationInitiateDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        doctorTaskExeHandlerBizService.consultationInitiate(params);
        return new Result<Void>().success();
    }
    
    @PostMapping("/consultation/ignore")
    @ApiOperation(value = "忽略会诊", response = Void.class)
    public Result<Void> pushServeIgnore(@RequestBody DoctorConsultationIgnoreDTO params) {
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDeptId(user.getDeptId());
        params.setDoctorId(user.getId());
        doctorTaskExeHandlerBizService.consultationIgnore(params);
        return new Result<Void>().success();
    }
    
}
