package com.xxh.cloud.healthy.service.com.manager.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.healthy.basic.label.client.api.bean.dto.LabelQueryPageDTO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientDischargeRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientInpatientRecordClientBO;
import com.xxh.cloud.healthy.data.client.api.bean.bo.PatientVisitRecordClientBO;
import com.xxh.cloud.healthy.doctor.crm.client.api.bean.bo.DoctorPatientRemarkBO;
import com.xxh.cloud.healthy.mdt.api.bean.bo.ConsultationBO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.LabelPatientDataHistogramDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientActivityLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.LabelPatientActivityDataQueryDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientByLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelAddDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelDirectRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelQueryParam;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.dto.PatientLabelRemoveDTO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.LabelPatientDataHistogramVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.LabelPatientDataVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientActivityLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelExportVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelHistoryVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.bean.vos.PatientLabelVO;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.LabelPatientDataBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelBizService;
import com.xxh.cloud.healthy.serve.label.framework.modules.biz.service.PatientLabelQueryService;
import com.xxh.cloud.healthy.serve.label.framework.modules.task.service.PatientLabelTaskBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

/**
 * 患者打标签管理
 */
@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/patient/label")
@Api(tags = "患者标签管理")
public class PatientLabelManagerController {

    @Autowired
    private PatientLabelBizService patientLabelBizService;
    @Autowired
    private PatientLabelQueryService patientLabelQueryService;
    @Autowired
    private LabelPatientDataBizService labelPatientDataBizService;
    @Autowired
    private PatientLabelTaskBizService patientLabelTaskBizService;
    
    /**
     * 查询患者的标签
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表查询", response = PatientLabelVO.class)
    public Result<List<PatientLabelVO>> list(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/list >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        List<PatientLabelVO> data= patientLabelQueryService.list(params);
        return new Result<List<PatientLabelVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:patientLabel:historyList")
    @PostMapping("/historyList")
    @ApiOperation(value = "查询", response = PatientLabelHistoryVO.class)
    public Result<List<PatientLabelHistoryVO>> historyList(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/historyList >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        List<PatientLabelHistoryVO> data= patientLabelQueryService.historyList(params);
        return new Result<List<PatientLabelHistoryVO>>().ok(data);
    }

    // @SaCheckPermission("sys:healthyserve:patientLabel:bind")
    @PostMapping("/bind")
    @ApiOperation(value = "绑定标签", response = Void.class)
    public Result<Void> bind(@RequestBody PatientLabelAddDTO params) {
        log.info("/sys/healthy/patient/label/bind >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        patientLabelBizService.addLabel(params);
        return new Result<Void>().success();
    }

    // @SaCheckPermission("sys:healthyserve:patientLabel:batchBind")
    @PostMapping("/batchBind")
    @ApiOperation(value = "批量绑定标签", response = Void.class)
    public Result<Void> batchBind(@RequestBody PatientLabelAddBatchDTO params) {
        log.info("/sys/healthy/patient/label/batchBind >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        patientLabelBizService.addBatchLabel(params);
        return new Result<Void>().success();
    }

    // @SaCheckPermission("sys:healthyserve:patientLabel:unbind")
    @PostMapping("/unbind")
    @ApiOperation(value = "解绑标签", response = Void.class)
    public Result<Void> unbind(@RequestBody PatientLabelRemoveDTO params) {
        log.info("/sys/healthy/patient/label/unbind >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        patientLabelBizService.removeBatchLabel(params);
        return new Result<Void>().success();
    }


    // 患者当前绑定的标签列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/patientLabel")
    @ApiOperation(value = "患者当前绑定的标签列表", response = LabelBO.class)
    public Result<PageData<LabelBO>> patientLabelPage(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/patientLabel >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        PageData<LabelBO> data= patientLabelQueryService.labelPageByPatient(params);
        return new Result<PageData<LabelBO>>().ok(data);
    }

    // 标签绑定的当前患者列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/labelPatient")
    @ApiOperation(value = "标签绑定的当前患者列表", response = PatientLabelVO.class)
    public Result<PageData<PatientLabelVO>> labelPatientPage(@RequestBody PatientByLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/labelPatient >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<PatientLabelVO> data= patientLabelQueryService.patientPageByLabel(params);
        return new Result<PageData<PatientLabelVO>>().ok(data);
    }

    // 标签绑定的当前患者列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @GetMapping("/labelPatient/export")
    @ApiOperation(value = "标签绑定的当前患者列表", response = Void.class)
    public void labelPatientPageExport(HttpServletResponse response, PatientByLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/labelPatient/export >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<PatientLabelVO> data= patientLabelQueryService.patientPageByLabel(params);

        List<PatientLabelExportVO> exportList = new ArrayList<>();
        data.getList().forEach(i -> {
            PatientClientBO patient = i.getPatientInfo();
            if (null != patient) {
                PatientLabelExportVO exportVO = new PatientLabelExportVO();
                // 关注
                exportVO.setFollowStatus("Y".equals(i.getFollowStatus()) ? "已关注" : "");

                // 基本信息
                exportVO.setPatientName(patient.getPatientName());
                exportVO.setPatientAge( patient.getAge());
                exportVO.setDeathStatus("N".equals(patient.getDeathStatus()) ? "已死亡" : "");
                exportVO.setPatientGender("未知");
                if ("W".equals(patient.getPatientGenderStandard())) {
                    exportVO.setPatientGender("女");
                } else if ("M".equals(patient.getPatientGenderStandard())) {
                    exportVO.setPatientGender("男");
                }
                exportVO.setPatientId(patient.getPatientId());

                // 备注
                DoctorPatientRemarkBO remark = i.getDoctorPatientRemarkLast();
                if (null != remark) {
                    exportVO.setRemark(remark.getRemark());
                }

                // 发现时间
                exportVO.setDataDt(DateUtil.format(i.getDataDt(), "yyyy-MM-dd HH:mm:ss"));

                // 最后就诊
                PatientVisitRecordClientBO visit = i.getVisitInfo();
                if (null != visit) {
                    exportVO.setLastVisitDept(visit.getDepartmentNameRaw());
                    exportVO.setLastVisitDoctor(visit.getDoctorName());
                    exportVO.setLastVisitDate(DateUtil.format(visit.getVisitDate(), "yyyy-MM-dd HH:mm:ss"));
                }

                // 最后住院
                PatientInpatientRecordClientBO inpatient = i.getInpatientInfo();
                if (null != inpatient) {
                    exportVO.setLastInpatientDept(inpatient.getAdmitDepartmentNameRaw());
                    exportVO.setLastInpatientDoctor(inpatient.getDoctorName());
                    exportVO.setLastInpatientDate(DateUtil.format(inpatient.getInpatientDate(), "yyyy-MM-dd HH:mm:ss"));
                }

                // 最后出院
                PatientDischargeRecordClientBO outpatient = i.getOutpatientInfo();
                if (null != outpatient) {
                    exportVO.setLastDischargeDept(outpatient.getAdmitDepartmentNameRaw());
                    exportVO.setLastDischargeDate(DateUtil.format(outpatient.getLeaveDate(), "yyyy-MM-dd HH:mm:ss"));
                }

                // 会诊
                ConsultationBO consultation = i.getMdtConsultationInfo();
                if (null != consultation) {
                    exportVO.setMdtApplyDoctor(consultation.getApplyDoctorName());
                    exportVO.setMdtApplyDate(DateUtil.format(consultation.getApplyTime(), "yyyy-MM-dd HH:mm:ss"));
                }

                exportList.add(exportVO);
            }
        });

        if (CollUtil.isEmpty(exportList)) {
            return;
        }

        // 导出为excel  .xlsx
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("followStatus", "是否关注");
        writer.addHeaderAlias("patientName", "患者姓名");
        writer.addHeaderAlias("patientAge", "患者年龄");
        writer.addHeaderAlias("deathStatus", "是否死亡");
        writer.addHeaderAlias("patientGender", "患者性别");
        writer.addHeaderAlias("patientId", "患者编号");
        writer.addHeaderAlias("remark", "备注");
        writer.addHeaderAlias("dataDt", "发现时间");
        writer.addHeaderAlias("lastVisitDept", "最后就诊科室");
        writer.addHeaderAlias("lastVisitDoctor", "最后就诊医生");
        writer.addHeaderAlias("lastVisitDate", "最后就诊时间");
        writer.addHeaderAlias("lastInpatientDept", "最后住院科室");
        writer.addHeaderAlias("lastInpatientDoctor", "最后住院医生");
        writer.addHeaderAlias("lastInpatientDate", "最后住院时间");
        writer.addHeaderAlias("lastDischargeDept", "最后出院科室");
        writer.addHeaderAlias("lastDischargeDate", "最后出院时间");
        writer.addHeaderAlias("mdtApplyDoctor", "MDT发起人");
        writer.addHeaderAlias("mdtApplyDate", "MDT申请时间");

        writer.write(exportList, true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
        response.setHeader("Content-Disposition","attachment;filename=label-patient.xlsx"); 
        ServletOutputStream out;
        try {
            out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        } catch (IOException e) {
            log.error("导出Excel异常：{}", e.getMessage());
            e.printStackTrace();
        }
    }

    // 患者历史绑定的标签列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/patientHistoryLabel")
    @ApiOperation(value = "患者历史绑定的标签列表(去重)", response = LabelBO.class)
    public Result<PageData<LabelBO>> patientHistoryLabelPage(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/patientHistoryLabel >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        PageData<LabelBO> data= patientLabelQueryService.historyLabelPageByPatient(params);
        return new Result<PageData<LabelBO>>().ok(data);
    }

    // 患者历史绑定的标签列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/patientHistoryLabel/list")
    @ApiOperation(value = "患者历史绑定的标签列表（不去重）", response = PatientLabelHistoryVO.class)
    public Result<List<PatientLabelHistoryVO>> patientHistoryLabelList(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/patientHistoryLabel >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        List<PatientLabelHistoryVO> data= patientLabelQueryService.historyLabelPageByPatientList(params);
        return new Result<List<PatientLabelHistoryVO>>().ok(data);
    }

    // 标签绑定过的历史患者列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/labelHistoryPatient")
    @ApiOperation(value = "标签绑定过的历史患者列表", response = PatientClientBO.class)
    public Result<PageData<PatientClientBO>> labelHistoryPatientPage(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/labelHistoryPatient >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        PageData<PatientClientBO> data= patientLabelQueryService.historyPatientPageByLabel(params);
        return new Result<PageData<PatientClientBO>>().ok(data);
    }

    // 标签绑定的当前患者列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/labelPatient/history")
    @ApiOperation(value = "标签绑定的历史患者列表", response = PatientLabelHistoryVO.class)
    public Result<PageData<PatientLabelHistoryVO>> labelPatientHistoryPage(@RequestBody PatientByLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/labelPatient >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUser();
        params.setDoctorId(user.getId());
        PageData<PatientLabelHistoryVO> data= patientLabelQueryService.patientPageByHistoryLabel(params);
        return new Result<PageData<PatientLabelHistoryVO>>().ok(data);
    }

    // 标签患者统计数据列表
    @PostMapping("/dataList")
    @ApiOperation(value = "标签患者统计数据列表", response = LabelPatientDataVO.class)
    public Result<PageData<LabelPatientDataVO>> labelPatientDataPage(@RequestBody LabelQueryPageDTO params) {
        log.info("/sys/healthy/patient/label/dataList >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setDiseaseIds(null);
        } else if (CollectionUtil.isEmpty(params.getDiseaseIds())) {
            return new Result<PageData<LabelPatientDataVO>>().success();
        }
        PageData<LabelPatientDataVO> data= labelPatientDataBizService.page(params);
        return new Result<PageData<LabelPatientDataVO>>().ok(data);
    }

    // 标签患者统计数据列表
    @PostMapping("/dataList/search")
    @ApiOperation(value = "标签患者统计数据列表(查询条件)", response = LabelPatientDataVO.class)
    public Result<PageData<LabelPatientDataVO>> labelPatientDataSearchPage(@RequestBody PatientLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/dataList/search >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        LoginUser user = AuthContextUtils.getCurrentLoginUser();
        // 管理员查询所有病种数据
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            params.setDiseaseIds(null);
        } else if (CollectionUtil.isEmpty(params.getDiseaseIds())) {
            return new Result<PageData<LabelPatientDataVO>>().success();
        }
        PageData<LabelPatientDataVO> data= labelPatientDataBizService.page(params);
        return new Result<PageData<LabelPatientDataVO>>().ok(data);
    }

    // 标签患者统计数据刷新
    @PostMapping("/dataList/dataRefresh")
    @ApiOperation(value = "标签患者统计数据刷新", response = Void.class)
    public Result<Void> labelPatientDataRefresh() {
        log.info("/sys/healthy/patient/label/dataList/dataRefresh >>> ");
        patientLabelTaskBizService.labelPatientCountTaskExe();
        return new Result<Void>().success();
    }

    // 标签用户柱状图
    @PostMapping("/dataList/histogram")
    @ApiOperation(value = "标签患者统计数据刷新", response = LabelPatientDataHistogramVO.class)
    public Result<List<LabelPatientDataHistogramVO>> labelPatientDataList(@RequestBody LabelPatientDataHistogramDTO params) {
        List<LabelPatientDataHistogramVO> list = patientLabelQueryService.historyHistogramList(params);
        return new Result<List<LabelPatientDataHistogramVO>>().ok(list);
    }




    // 活动数据
    
    // 活动标签患者统计数据列表
    @PostMapping("/dataList/activity")
    @ApiOperation(value = "活动标签患者统计数据列表", response = LabelPatientDataVO.class)
    public Result<PageData<LabelPatientDataVO>> activityLabelPatientDataPage(@RequestBody LabelPatientActivityDataQueryDTO params) {
        log.info("/sys/healthy/patient/label/dataList/activity >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        PageData<LabelPatientDataVO> data= labelPatientDataBizService.activityPage(params);
        return new Result<PageData<LabelPatientDataVO>>().ok(data);
    }

    // 活动标签绑定过的历史患者列表
    // @SaCheckPermission("sys:healthyserve:patientLabel:list")
    @PostMapping("/labelHistoryPatient/activity")
    @ApiOperation(value = "标签绑定过的历史患者列表", response = PatientClientBO.class)
    public Result<PageData<PatientActivityLabelVO>> activityLabelHistoryPatientPage(@RequestBody PatientActivityLabelQueryParam params) {
        log.info("/sys/healthy/patient/label/labelHistoryPatient/activity >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        PageData<PatientActivityLabelVO> data= labelPatientDataBizService.activityPatientPage(params);
        return new Result<PageData<PatientActivityLabelVO>>().ok(data);
    }




    // 直接物理删除患者标签

    @PostMapping("/directDelNowPatientLabelBatch")
    @ApiOperation(value = "物理删除用户当前标签", response = Void.class)
    public Result<Void> directDelNowPatientLabelBatch(@RequestBody PatientLabelDirectRemoveDTO params) {
        log.info("/sys/healthy/patient/label/directDelNowPatientLabelBatch >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        patientLabelBizService.directDelNowPatientLabelBatch(params);
        return new Result<Void>().success();
    }

    @PostMapping("/directDelHistoryPatientLabelBatch")
    @ApiOperation(value = "物理删除用户历史标签", response = Void.class)
    public Result<Void> directDelHistoryPatientLabelBatch(@RequestBody PatientLabelDirectRemoveDTO params) {
        log.info("/sys/healthy/patient/label/directDelHistoryPatientLabelBatch >>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        patientLabelBizService.directDelHistoryPatientLabelBatch(params);
        return new Result<Void>().success();
    }

}
