package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.healthy.basic.common.constant.OpTypeEnum;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeConfigSnapshotQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeHistoryQueryParam;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.dto.PatientPushServeRemoveDTO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeConfigSnapshotVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeHistoryVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.bean.vos.PatientPushServeVO;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeBizService;
import com.xxh.cloud.healthy.serve.pushServe.framework.modules.biz.service.PatientPushServeQueryBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/healthy/patient/pushService")
@Api(tags = "患者推送服务管理")
public class PatientPushServeManagerController {
    
    @Autowired
    private PatientPushServeQueryBizService patientPushServeQueryBizService;
    @Autowired
    private PatientPushServeBizService patientPushServeBizService;
    
    /**
     * 一般通过引擎触发推送服务：PatientPushServeClientService.add()
     * 本接口可直接推送服务
     */
    @PostMapping("/add")
    @ApiOperation(value = "直接给患者推送服务", response = PatientPushServeVO.class)
    public Result<PatientPushServeVO> pushServeAdd(@RequestBody PatientPushServeAddDTO params) {
        log.info("/sys/healthy/patient/pushService/add >>> {}", JSON.toJSONString(params));
        PatientPushServeVO data = patientPushServeBizService.add(params);
        return new Result<PatientPushServeVO>().ok(data);
    }
    
    @PostMapping("/addBatch")
    @ApiOperation(value = "直接批量给患者推送服务", response = Void.class)
    public Result<Void> pushServeAddBatch(@RequestBody PatientPushServeAddDTO params) {
        log.info("/sys/healthy/patient/pushService/addBatch >>> {}", JSON.toJSONString(params));
        if (CollUtil.isEmpty(params.getPushServeIds())) {
            throw new BusinessException("推送服务ID列表不能为空");
        }
        params.getPushServeIds().forEach(i -> {
            params.setPushServeId(i);
            patientPushServeBizService.add(params);
        });
        return new Result<Void>().success();
    }
    
    @PostMapping("/remove")
    @ApiOperation(value = "删除直接给患者推送的服务", response = Void.class)
    public Result<Void> pushServeDel(@RequestBody PatientPushServeRemoveDTO params) {
        log.info("/sys/healthy/patient/pushService/remove >>> {}", JSON.toJSONString(params));
        patientPushServeBizService.remove(params);
        return new Result<Void>().success();
    }
    
    /**
     * 分页查询患者的推送服务列表
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:patientPushServe:page")
    @PostMapping("/page")
    @ApiOperation(value = "患者推送服务分页", response = PatientPushServeHistoryVO.class)
    public Result<PageData<PatientPushServeHistoryVO>> pushServePage(@RequestBody PatientPushServeHistoryQueryParam params) {
        log.info("/sys/healthy/patient/pushService/page >>> {}", JSON.toJSONString(params));
        params.setOpType(OpTypeEnum.Create.getCode());
        PageData<PatientPushServeHistoryVO> data = patientPushServeQueryBizService.pushServeHistoryPage(params);
        return new Result<PageData<PatientPushServeHistoryVO>>().ok(data);
    }
    
    /**
     * 查询患者的推送服务列表
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:patientPushServe:list")
    @PostMapping("/list")
    @ApiOperation(value = "患者推送服务列表", response = PatientPushServeHistoryVO.class)
    public Result<List<PatientPushServeHistoryVO>> pushServeList(@RequestBody PatientPushServeHistoryQueryParam params) {
        log.info("/sys/healthy/patient/pushService/list >>> {}", JSON.toJSONString(params));
        List<PatientPushServeHistoryVO> data = patientPushServeQueryBizService.pushServeHistoryList(params);
        return new Result<List<PatientPushServeHistoryVO>>().ok(data);
    }

    /**
     * 查询患者的推送服务快照列表
     * @param params
     * @return
     */
    // @SaCheckPermission("sys:healthyserve:patientPushServeSnapshot:list")
    @PostMapping("/serveConfigSnapshotList")
    @ApiOperation(value = "推送服务快照", response = PatientPushServeConfigSnapshotVO.class)
    public Result<List<PatientPushServeConfigSnapshotVO>> serveConfigSnapshotList(@RequestBody PatientPushServeConfigSnapshotQueryParam params) {
        log.info("/sys/healthy/patient/pushService/serveConfigSnapshotList >>> {}", JSON.toJSONString(params));
        List<PatientPushServeConfigSnapshotVO> data = patientPushServeQueryBizService.listServeConfigSnapshot(params);
        return new Result<List<PatientPushServeConfigSnapshotVO>>().ok(data);
    }

}
