package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.ReceiveEventFullModel;
import com.xxh.cloud.framework.common.entity.BasePageDTO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.WeixinUserInfo;
import com.xxh.cloud.healthy.push.framework.modules.base.entity.WeixinUserPatientBindInfo;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WeixinUserSyncDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WeixinBizService;

import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RestController
@RequestMapping("/sys/healthy/wechat")
public class WeixinManagerController {
    
    @Autowired
    private WeixinBizService weixinBizService;

    /**
     * 此接口对外模块调用(health-frontend-java)
     */
    @PostMapping(value = "/reply")
    @ApiOperation(value = "微信事件回复", response = String.class)
    public Result<String> reply(@RequestBody ReceiveEventFullModel params) {
        log.info("/sys/healthy/wechat/reply >>> {}", JSON.toJSONString(params));
        return new Result<String>().ok(weixinBizService.passiveMsg(params));
    }

    /**
     * 此接口对外模块调用(health-frontend-java)
     */
    @PostMapping(value = "/saveWechatUserInfoBatch")
    @ApiOperation(value = "批量保存微信公众号关注用户信息", response = Void.class)
    public Result<Void> saveWechatUserInfoBatch(@RequestBody List<WeixinUserInfo> params) {
        log.info("/sys/healthy/wechat/saveWechatUserInfoBatch >>> {}", JSON.toJSONString(params));
        weixinBizService.saveWechatUserInfoBatch(params);
        return new Result<Void>().success();
    }

    /**
     * 此接口对外模块调用(health-frontend-java)
     */
    @PostMapping(value = "/queryWechatUserInfo")
    @ApiOperation(value = "获取微信用户信息", response = WeixinUserInfo.class)
    public List<WeixinUserInfo> queryWechatUserInfo(@RequestBody BasePageDTO params) {
        log.info("/sys/healthy/wechat/queryWechatUserInfo >>> {}", JSON.toJSONString(params));
        List<WeixinUserInfo> list = weixinBizService.queryWechatUserInfo(params);
        return list;
    }

    /**
     * 此接口对外模块调用(health-frontend-java)
     */
    @Deprecated
    @PostMapping(value = "/saveUserInfoBatch")
    @ApiOperation(value = "批量保存海鹚用户信息", response = Void.class)
    public Result<Void> saveUserInfoBatch(@RequestBody List<WeixinUserPatientBindInfo> params) {
        log.info("/sys/healthy/wechat/saveUserInfoBatch >>>");
        weixinBizService.saveUserInfoBatch(params);
        return new Result<Void>().success();
    }

    /**
     * 此接口对外模块调用(health-frontend-java)
     */
    @PostMapping(value = "/saveUserInfoBatch2")
    @ApiOperation(value = "批量保存海鹚用户信息", response = Void.class)
    public Result<Void> saveUserInfoBatch2(@RequestBody WeixinUserSyncDTO params) {
        log.info("/sys/healthy/wechat/saveUserInfoBatch2 >>>");
        weixinBizService.saveUserInfoBatch2(params);
        return new Result<Void>().success();
    }

}
