package com.xxh.cloud.healthy.service.com.manager.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.company.cloud.sdk.weixin.mp.model.QRcodeWithParamsResp;
import com.xxh.cloud.customer.client.api.bean.bos.CustomerUserBO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeCreateDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataGetDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataQueryDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeDataUpdateDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatQRCodeUserQueryDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.dto.WechatScanQRCodeDTO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.vos.WechatQRCodeDataVO;
import com.xxh.cloud.healthy.push.framework.modules.biz.bean.vos.WechatScanQRCodeDayVO;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WechatQRCodeDataBizService;
import com.xxh.cloud.healthy.push.framework.modules.biz.service.WeixinBizService;
import com.xxh.cloud.healthy.push.framework.modules.task.service.PromotionBizService;

import cn.dev33.satoken.annotation.SaCheckLogin;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@SaCheckLogin
@Slf4j
@RestController
@RequestMapping("/sys/healthy/wechat")
public class WeixinQRCodeManagerController {
    
    @Autowired
    private WeixinBizService weixinBizService;
    @Autowired
    private WechatQRCodeDataBizService wechatQRCodeDataBizService;
    @Autowired
    private PromotionBizService promotionBizService;

    @PostMapping(value = "/qrcode/create")
    @ApiOperation(value = "创建带参数二维码", response = QRcodeWithParamsResp.class)
    public Result<QRcodeWithParamsResp> createQRCode(@RequestBody WechatQRCodeCreateDTO params) {
        return new Result<QRcodeWithParamsResp>().ok(weixinBizService.createQRCode(params));
    }

    @PostMapping(value = "/qrcode/page")
    @ApiOperation(value = "分页查询带参数二维码", response = WechatQRCodeDataVO.class)
    public Result<PageData<WechatQRCodeDataVO>> queryQRCode(@RequestBody WechatQRCodeDataQueryDTO params) {
        return new Result<PageData<WechatQRCodeDataVO>>().ok(wechatQRCodeDataBizService.page(params));
    }

    @PostMapping(value = "/qrcode/list")
    @ApiOperation(value = "列表查询带参数二维码", response = WechatQRCodeDataVO.class)
    public Result<List<WechatQRCodeDataVO>> listQRCode(@RequestBody WechatQRCodeDataQueryDTO params) {
        return new Result<List<WechatQRCodeDataVO>>().ok(wechatQRCodeDataBizService.list(params));
    }

    @PostMapping(value = "/qrcode/detail")
    @ApiOperation(value = "查询带参数二维码详情", response = WechatQRCodeDataVO.class)
    public Result<WechatQRCodeDataVO> detailQRCode(@RequestBody WechatQRCodeDataGetDTO params) {
        return new Result<WechatQRCodeDataVO>().ok(wechatQRCodeDataBizService.detail(params));
    }

    @PostMapping(value = "/qrcode/simpleDetail")
    @ApiOperation(value = "查询带参数二维码简单详情", response = WechatQRCodeDataVO.class)
    public Result<WechatQRCodeDataVO> simpleDetailQRCode(@RequestBody WechatQRCodeDataGetDTO params) {
        return new Result<WechatQRCodeDataVO>().ok(wechatQRCodeDataBizService.simpleDetail(params));
    }

    @PostMapping(value = "/qrcode/update")
    @ApiOperation(value = "更新带参数二维码", response = WechatQRCodeDataVO.class)
    public Result<WechatQRCodeDataVO> updateQRCode(@RequestBody WechatQRCodeDataUpdateDTO params) {
        return new Result<WechatQRCodeDataVO>().ok(wechatQRCodeDataBizService.update(params));
    }

    @PostMapping(value = "/qrcode/remove")
    @ApiOperation(value = "删除带参数二维码", response = WechatQRCodeDataVO.class)
    public Result<WechatQRCodeDataVO> removeQRCode(@RequestBody WechatQRCodeDataGetDTO params) {
        wechatQRCodeDataBizService.del(params);
        return new Result<WechatQRCodeDataVO>().success();
    }

    

    @PostMapping(value = "/qrcode/registerUserPage")
    @ApiOperation(value = "统计扫描二维码注册用户列表", response = CustomerUserBO.class)
    public Result<PageData<CustomerUserBO>> registerUserPage(@RequestBody WechatQRCodeUserQueryDTO params) {
        return new Result<PageData<CustomerUserBO>>().ok(wechatQRCodeDataBizService.registerUserPage(params));
    }

    @PostMapping(value = "/qrcode/bindFamilyUserPage")
    @ApiOperation(value = "统计扫描二维码绑定家属用户列表", response = CustomerUserBO.class)
    public Result<PageData<CustomerUserBO>> bindFamilyUserPage(@RequestBody WechatQRCodeUserQueryDTO params) {
        return new Result<PageData<CustomerUserBO>>().ok(wechatQRCodeDataBizService.bindFamilyUserPage(params));
    }

    // TODO 可能不满足需求
    @PostMapping("/listByQrcodeIdAndDay")
    @ApiOperation(value = "按日统计每个二维码扫码数量", response = WechatScanQRCodeDayVO.class)
    public Result<List<WechatScanQRCodeDayVO>> listByQrcodeIdAndDay(@RequestBody WechatScanQRCodeDTO params) {
        ValidatorUtils.validateEntity(params);
        // LoginUser user= AuthContextUtils.getCurrentLoginUser();
        List<WechatScanQRCodeDayVO> data = wechatQRCodeDataBizService.listByQrcodeIdAndDay(params);
        return new Result<List<WechatScanQRCodeDayVO>>().ok(data);
    }

    @PostMapping(value = "/sync/promotion")
    @ApiOperation(value = "同步互联网医院推广码并创建微信二维码", response = Void.class)
    public Result<Void> syncPromotion() {
        promotionBizService.syncPromotion();
        return new Result<Void>().success();
    }

}
